//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-26 00:47:41 sutner> 
//
// :File:      atom.cc
// :Purpose:   Create user atoms from data given as ULists.
//---------------------------------------------------------------------------

#include  "atom.h"
#include  "util.h"
#include  "useratom.h"

UList   create_atom( UList atn, UList data )
{
    if( strcmp( cvalue(atn), "CA" )==0 )
	return create_ca( data ); 
    if( strcmp( cvalue(atn), "SA" )==0 )
	return create_sa( data ); 
    if( strcmp( cvalue(atn), "DFA" )==0 )
	return create_dfa( data ); 

    return nil;
}

// awkward after switch to CA[w,k,r] in Mta

UList   create_ca( UList data )
{
    int  w=3, k=2;

    UList  R, W, K;
    data >> W;

    if( data ) { data >> K; w = ivalue(W); }
    
    if( data ) { data >> R; k = ivalue(K); }
    
    if( R.isInt() )
    {
	return  new CAAtom( CA( ivalue(R), w, k ) );
    }
    else			// better have isInteg()
    {
	return  new CAAtom( CA( Ivalue(R), w, k ) );
    }
}

UList   create_sa( UList data )
{
    UList  N, K, T;
    data >> N >> K >> T;

    TransitionVector  tr;
    urlist_to_transition(T,tr);     // should be checked !!! 
    return  new SAAtom( SA( ivalue(N), ivalue(K), tr ) );
}

UList   create_fa( UList data )
{
    UList   n,k,T,ini,fin;
    data >> n >> k >> T >> ini >> fin;

    TransitionVector  tr;
    IntSet I, F;
    urlist_to_transition(T,tr);     // should be checked !!! 
    urlist_to_intset( ini, ivalue(n), I, -1 );
    urlist_to_intset( fin, ivalue(n), F, -1 );
    FA  fa( ivalue(n), ivalue(k), tr, I, F );
    return   new FAAtom(fa);
}

UList   create_dfa( UList data )
{
    UList   N,K,T,ini,fin;
    data >> N >> K >> T >> ini >> fin;

    int n = ivalue(N);
    int k = ivalue(K);
    DFA  dfa( n, k );
    urlist_to_intarray( flatten(T), n*k, dfa.transitions(), -1 );
    dfa.initial() = ivalue(ini) - 1;
    urlist_to_intset( fin, n, dfa.final(), -1 );
    return   new DFAAtom(dfa);
}

UList   create_graph( UList data )
{
    UList ll, nv, ne, T, H;
    data >> nv >> ne >> T >> ll;

    int t = ivalue(T);
    assert( t==1 || t==4 );

    Graph  gr( ivalue(nv), ivalue(ne) );
   
    if( t==1 )
    {
	gr.numEdge() = 0;		// will be incremented during insertions
	UList  src, targ;
	while( ll )
	{
	    ll >> H;
	    H  >> src >> targ;
	    gr.insertEdge( ivalue(src)-1, ivalue(targ)-1 );
	}
    }
    else
    {
	int i=0;
	while( ll )
	{
	    ll >> H;
	    urlist_to_intlist( H, gr.adjacencyList(i), -1 );
	    i++;
	}
    }

    return new GraphAtom(gr);
}

//--------------------------------------------------------------------------------
