//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:17:41 sutner> 
//
// :File:      semimod.cc
// :Purpose:   testing templated semimodule generation
//---------------------------------------------------------------------------


#include  <cctype>
#include  "stl-wrap.h"
#include  "basic.h"
#include  "pairs.h"
#include  "intset.h"

#include  <iostream>
#include  <hash_map.h>
#include  <algorithm>

#include  "semimod.h"

//--------------------------------------------------------------------------------

typedef  vector<bool>   bvector;
typedef  Triple<size_t,bvector,string>  SVS;
typedef  hash_map<size_t,string,hash<size_t>,equal_to<size_t> >  WitTable;

//--------------------------------------------------------------------------------

struct   hash<bvector>
{
    size_t  operator()( const bvector& A  ) const
    {
	size_t h = 0;
	for( size_t i = 0; i < A.size(); ++i )
	    h = h*103 + A[i];
	return h;
    }
};

bvector  normalize( const bvector& A )
{
    bvector  B(A), AA(A);
    
    do
    {
	rotate( B.begin(), B.begin() + 1, B.end() );
	if( B < AA ) AA = B;
    }
    while( B != A );
	    
    return AA;
}

bool  is_normal( const bvector& A )
{
    bvector  B(A);
    
    do
    {
	rotate( B.begin(), B.begin() + 1, B.end() );
	if( B < A ) return 0;
    }
    while( B != A );
	    
    return 1;
}


struct  one_perm: public binary_function<bvector,size_t,bvector>
{
    int   qq;			// shift factor for b transitions
    
    one_perm( int q ): qq(q) {} 

    bvector  operator()( const bvector& A, size_t s ) const 
    {
	bvector  B(A);

	if( s == 0 )
	{
	    if( B[0] ) B[qq-1] = 1;
	    rotate( B.begin(), B.end() - 1, B.end() );
	}
	else
	{
	    B[0] = 0;
	    rotate( B.begin(), B.end() - qq, B.end() );
	}
	
	return  B;
    }
};




// pre:   sm is fully constructed, generator q0 is in position 0
// post:  wit[p] is the length-lex minimal witness for p in Q


template<class A, class B>
void   extract_witnesses( SemiModule<A,B>& sm, WitTable& wit )
{
    size_t  n = sm.Q.size();
    size_t  k = sm.kk;
    
    assert( sm.del.size() == n * k );

    IntSet  R(n);
    wit[0] = "";
    R.insert(0);
    
    size_t p, s, q;
    ulist::iterator  it = sm.del.begin();
    
    for( p = 0; p < n; ++p )
	for( s = 0; s < k; ++s )
	{
	    q = *it++;
	    if( !R.find(q) )
	    {
		wit[q] = wit[p] + char('a'+s);
		R.insert(q);
	    }
	}
}





size_t  cardinality( const bvector& X ) 
{
    return accumulate( X.begin(), X.end(), 0 );
}


struct comp_2
{
    bool  operator()( const SVS& X, const SVS& Y ) const
    {
	size_t cx = cardinality( X.second() ); 
	size_t cy = cardinality( Y.second() );
	if( cx < cy ) return 1;
	if( cx == cy ) return  X.third() < Y.third();
	return 0;
    }
};


void  display( const bvector& A )
{
    size_t n = A.size();
    cout << A[n-2] << " " << A[n-1] << " .";
    copy( A.begin(), A.end()-2, ostream_iterator<int>( cout, " " ) );
    cout << endl;
}
    
void  print_bvector( const bvector& A )
{
    for( size_t i = 0; i < A.size(); ++i )
	cout << ( A[i] ? 'x' : '.' );
    cout << endl;
}


main( int argc, char **argv )
{
    size_t n = atoi(argv[1]);
    size_t q = atoi(argv[2]);
    
    size_t i;
    
    bvector   A( n, false );
    A[0] = 1;
    
    one_perm  op( q );

    SemiModule<bvector,one_perm> SM( 2, A, op );

    size_t  m = SM.generate();
    cout << "*** " << n << "  " << q << " " << m << endl;

    vector<Pair<size_t,bvector> >  tab(m);
    tab.resize(0);

    SemiModule<bvector,one_perm>::iterator  it;
    foreach_item( it, SM.Q )
	tab.push_back( Pair<size_t,bvector>(it->second,it->first) );

//    sort( tab.begin(), tab.end() );
    

    WitTable  W;
    extract_witnesses( SM, W );

//     for( size_t i = 0; i < m; ++i )
// 	cout << tab[i].first() << "\t" << tab[i].second() << "\t" << W[tab[i].first()] << endl;

    vector<SVS>  ordtab(m);
    for( size_t i = 0; i < m; ++i )
	ordtab[i] = SVS( tab[i].first(), tab[i].second(), W[tab[i].first()]);

   sort( ordtab.begin(), ordtab.end(), comp_2() );
    
//     for( size_t i = 0; i < m; ++i )
// 	cout << ordtab[i].first() << "\t" << ordtab[i].second() << "\t" << ordtab[i].third() << endl;

   hash_map<int,int,hash<int>,equal_to<int> >  Cnt;

    cout << endl;
    for( size_t i = 0; i < m; ++i )
    {
	A = ordtab[i].second();
	if( is_normal( A ) )
	{
	    bvector  B( A );
	    int cnt = 0;
	    do
	    {
		rotate( B.begin(), B.begin() + 1, B.end() );
		if( SM.Q.find( B ) == SM.Q.end() )
		    cout << A << " -> "  << B << endl;
		cnt++;
	    }
	    while( B != A );
	    cout << setw(4) << cnt << "   ";
	    print_bvector( A );
	    Cnt[cnt]++;
	}
    }
    cout << endl << "orbits: " << endl;
    
    int  nn = 0;
    hash_map<int,int,hash<int>,equal_to<int> >::iterator  cit;
    foreach_item( cit, Cnt )
	{
	    cout << cit->first << "\t" << cit->second << endl;
	    nn += cit->second;
	}
   cout << "total: " << nn << endl;
}



