//--------------------------------------------------------------------------------
//
// Time-stamp: <2003-11-24 17:26:32 sutner> 
//
// File:     extoutput.cc
// Purpose:  output routines Automata external to MathLink
//
//--------------------------------------------------------------------------------

#include  "basic.h"
#include  "extoutput.h"

//extern "C" 
// {
#include  "mathlink.h"
//}

//---------------------------------------------------------------------------

// for DFAs only, tr has size==nst*nsy
void  ML_write_trans( const uvector& tr, size_t nst, size_t nsy, int off )
{
   extern MLINK stdlink;
   
//    int  *tmp= new int[tr.size()];
   ivector  tmp(tr.size());
   size_t i;
   
   for( i=0; i<tr.size(); i++ )
      tmp[i] = tr[i] + off;
   
   MLPutFunction( stdlink, "List", nsy );
   for( i=0; i<nsy; i++ )
      MLPutIntegerList( stdlink, 
			  reinterpret_cast<int*>(&(*tmp.begin())+i*nst), nst );
//			tmp + i*nst, nst );
//   delete [] tmp;
}
   


void  ML_write( const TransV& A, int off )
{
    size_t  i;
   
    MLPutFunction( stdlink, "List", A.size() );
    for( i=0; i<A.size(); i++ )
    {
	MLPutFunction( stdlink, "List", 3 );
	MLPutInteger(stdlink, A[i].p + off );
	MLPutInteger(stdlink, A[i].s + off );
	MLPutInteger(stdlink, A[i].q + off );
    }
}

// long integers
void  ML_write( const Integer& nn )
{
    uvector  digs( nn.num_bits()/20 + 1 );
    integer_to_array( nn, digs, 1048576 );
    MLPutFunction( stdlink, "FromDigits", 2 );
    MLPutFunction( stdlink, "Reverse", 1 );
    MLPutIntegerList( stdlink, reinterpret_cast<int*>(&(*digs.begin())), digs.size() );
    MLPutInteger( stdlink, 1048576 );
}

// offset currently unimplemented, do we really need it???

void  ML_write( const ivector& A, int off )
{
    if( off==0 )
	MLPutIntegerList( stdlink, const_cast<int*>(&(*A.begin())), A.size() );
    else
    {
	ivector  AA(A);
	for( size_t i=0; i<A.size(); i++ )
	    AA[i] += off;
	MLPutIntegerList( stdlink, &(*AA.begin()), AA.size() );
    }
}

void  ML_write( const uvector& A, int off )
{
    if( off==0 )
	MLPutIntegerList( stdlink, 
			  reinterpret_cast<int*>(const_cast<size_t*>(&(*A.begin()))), A.size() );
    else
    {
	uvector  AA(A);
	for( size_t i=0; i<A.size(); i++ )
	    AA[i] += off;
	MLPutIntegerList( stdlink, reinterpret_cast<int*>(&(*AA.begin())), AA.size() );
    }
}

void  ML_write( const IntSet& A, int off )
{
    MLPutFunction( stdlink, "List", A.size() );
    IntSet::const_iterator  itr;
   
    foreach_item(itr,A)
	MLPutInteger( stdlink, *itr + off );
}

void  ML_write( const ilist& A, int off )
{
    MLPutFunction( stdlink, "List", A.size() );
    ilist::const_iterator  itr;

    foreach_item(itr,A)
	MLPutInteger( stdlink, *itr + off );
}

void  ML_write( const ulist& A, int off )
{
    MLPutFunction( stdlink, "List", A.size() );
    ulist::const_iterator  itr;

    foreach_item(itr,A)
	MLPutInteger( stdlink, int(*itr + off) );
}

void  ML_write( const vector<string>& wit )
{
    MLPutFunction( stdlink, "List", wit.size() );
    for( size_t i=0; i<wit.size(); i++ )
	MLPutString( stdlink, wit[i].c_str() );
}

void  ML_write( const list<Pair<string,string> >& equ )
{
    MLPutFunction( stdlink, "List", 2*equ.size() );
    list<Equation>::const_iterator itr;
    foreach_item( itr, equ )
	{
	    MLPutString( stdlink, (*itr).first().c_str() );
	    MLPutString( stdlink, (*itr).second().c_str() );
	}
}

//---------------------------------------------------------------------------
// D-classes are sent as side-effect for efficiency
// should probably check for composition

UList   ML_write_dclasses( UList L )
{
    assert( L.atom_type() == SGA ); 

    const  SGroup  &sg = sgvalue( L );

    ML_write_dc_array( sg.dclasses() );

    extern int  ML_needs_token;
    ML_needs_token = 0;

    return nil;
}      
      
void  ML_write_dc_array( const vector<DClass>& dcl )  
{
    int i, ndcl = dcl.size();
    MLPutFunction( stdlink, "List", ndcl );
    for( i=0; i<ndcl; i++ )
	ML_write_dc( dcl[i] );
}


void  ML_write_dc( const DClass& dc )
{
    int  ww = dc.width();
    int  ks = dc.kSize();
    int  is = dc.iSize();
    int  hs = dc.hSize();

    MLPutFunction( stdlink, "DCB", ks );
    for( int k=0; k<ks; k++ )
    {
	MLPutFunction( stdlink, "List", is );
	for( int i=0; i<is; i++ )
	{
	    MLPutFunction( stdlink, "List", hs );
	    for( int h=0; h<hs; h++ )
	    {
		MLPutFunction( stdlink, "T", ww );
		for( int j=ww-1; j>=0; j-- )
		    MLPutInteger( stdlink, dc.get( k,i,h,j ) + 1 );
	    }
	}
    }
}


void  ML_write_sg( const uvector& sg, int onefl, int w )
{
    MLPutFunction( stdlink, "List", 3 );
    MLPutInteger( stdlink, w );
    MLPutInteger( stdlink, onefl );
    ML_write( sg, 1 );
}

   
//---------------------------------------------------------------------------
// writing UList's to the link 

void   ML_write_urlist( UList LL )
{
    int  i,len,nv,nst,nsy;

    if( LL.atom_type() == NIL )
    {
	MLPutFunction( stdlink, "List", 0 );
	return;
    }
	

    if( LL.atom_type() == CONS )
    {				// presumably a list, not something like (a.b)
	len = length_int( LL );
	MLPutFunction( stdlink, "List", len );
	while( LL )
	{
	    ML_write_urlist( *LL );
	    ++LL;
	}
	return;
    }
    
    if( LL.atom_type() == INT )			// a single integer
    {	
	MLPutInteger( stdlink, ivalue(LL) );
	return;
    }
    
    if( LL.atom_type() == IARR )			// array of integers
    {
	ivector  &iv = iavalue( LL );
	MLPutIntegerList( stdlink, &(*iv.begin()), iv.size() );
	return;
    }
    
    if( LL.atom_type() == STR )
    {
	MLPutString( stdlink, svalue(LL) );
	return;
    }
    
    if( LL.atom_type() == SAA )
    {
	const  SA  &sa = savalue( LL );
	MLPutFunction( stdlink, "SA", 5 );
	MLPutInteger( stdlink, sa.numStates() );
	MLPutInteger( stdlink, sa.numSymbols() );
	ML_write( sa.transitions(), 1 );
	MLPutFunction( stdlink, "List", 0 );
	MLPutFunction( stdlink, "List", 0 );
	return; 
    }
    
    if( LL.atom_type() == FAA )
    {
	const  FA  &fa = favalue( LL );
	MLPutFunction( stdlink, "FA", 7 );
	MLPutInteger( stdlink, fa.numStates() );
	MLPutInteger( stdlink, fa.numSymbols() );
	ML_write( fa.transitions(), 1 );
	ML_write( fa.initial(), 1 );
	ML_write( fa.final(), 1 );
	MLPutFunction( stdlink, "List", 0 );
	MLPutFunction( stdlink, "List", 0 );
	return; 
    }
    
    if( LL.atom_type() == DFAA )
    {
	const  DFA  &dfa = dfavalue( LL );
	nst = dfa.numStates();
	nsy = dfa.numSymbols();
	MLPutFunction( stdlink, "DFA", 7 );
	MLPutInteger( stdlink, nst );
	MLPutInteger( stdlink, nsy );
	ML_write_trans( dfa.transitions(), nst, nsy, 1 );
	MLPutInteger( stdlink, dfa.initial() + 1 );
	ML_write( dfa.final(), 1 );
	MLPutFunction( stdlink, "List", 0 );
	MLPutFunction( stdlink, "List", 0 );
	return; 
    }
    
    if( LL.atom_type() == GRA )
    {
	const  Graph  &gr = grvalue( LL );
	nv = gr.numVertex();
	MLPutFunction( stdlink, "Graph", 3 );
	MLPutInteger( stdlink, nv );
	MLPutInteger( stdlink, 0 );
//      MLPutInteger( stdlink, gr.numEdge() );
	MLPutFunction( stdlink, "List", nv );
	for( i=0; i<nv; i++ )
	    ML_write( gr[i], 1 );
	return;
    }
    

    if( LL.atom_type() == CAA )
    {
	const  CA  &ca = cavalue( LL );
	MLPutFunction( stdlink, "CA", 3 );
	ML_write( ca.rule() );
	MLPutInteger( stdlink, ca.width() );
	MLPutInteger( stdlink, ca.numSymbols() );
	return; 
    }
    
    
    if( LL.atom_type() == SGA )
    {
	const  SGroup  &sg = sgvalue( LL );
	ML_write_sg( sg.semigroup(), sg.oneflag(), sg.width() );
	return; 
    }
    
    MLPutSymbol( stdlink, "$Failed" );
}

//--------------------------------------------------------------------------------








