//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 14:56:43 sutner> 
//
// :File:      functoid.cc
// :Purpose:   function atoms
//             Function:      rich handle to FuncOb, derived from Atom
//             FuncOb:        base class for actual function objects
//             NFuncOb:       nullary
//             UFuncOb:       unary, may be wrapped
//             BFuncOb:       binary, may be wrapped
// 
//---------------------------------------------------------------------------

#include  "basic.h"
#include  "functoid.h"
#include  "urlist.h"
#include  <string>

class SymTable;

//--------------------------------------------------------------------------------
// function atoms

Function::Function(): fp(NULL), ari(-1), fname(NULL) {}

Function::Function( NFuncOb* pp, const char *nm ): fp(pp), ari(0), fname(NULL)
{
    write_string( fname, nm );
}

Function::Function( UFuncOb* pp, const char *nm ): fp(pp), ari(1), fname(NULL)
{
    write_string( fname, nm );
}

Function::Function( BFuncOb* pp, const char *nm ): fp(pp), ari(2), fname(NULL)
{
    write_string( fname, nm );
}

Function::Function( WUFuncOb* pp, const char *nm ): fp(pp), ari(1), fname(NULL)
{
    write_string( fname, nm );
}

Function::Function( WBFuncOb* pp, const char *nm ): fp(pp), ari(2), fname(NULL)
{
    write_string( fname, nm );
}

Function::Function( TFuncOb* pp, const char *nm ): fp(pp), ari(3), fname(NULL)
{
    write_string( fname, nm );
}



Function::Function( const Function& ff ): fp( ff.fp ), ari(ff.ari), fname(NULL)
{
    fp->rcnt++;
    write_string( fname, ff.fname );
}

Function&  Function::operator=( const Function& src )
{
    if( this != &src )
    {
	src.fp->rcnt++;
	if( --fp->rcnt == 0 ) delete fp; 
	write_string( fname, src.fname );
	fp = src.fp;
    }
    return *this;
}
    
Function::~Function() 
{ 
    if( --fp->rcnt == 0 ) delete fp; 
    delete fname; 
}


//  Local Variables: ***
//  eval: (setq case-fold-search nil) ***
//  eval: (setq case-replace nil) ***
//  End: ***

