//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:20:46 sutner> 
//
// :File:      globop.cc
// :Purpose:   Standard functions for aut calculator.
// 
//---------------------------------------------------------------------------

#include  "basic.h"
#include  "globop.h"
#include  "urlist.h"
#include  "listutil.h"
#include  "parser.h"
#include  "symtab.h"
#include  "ioatom.h"

#define  YYSTYPE  UList
#include  "bparser.h"

//--------------------------------------------------------------------------------
// helper functions
// move some of these to util.cc and listutil.cc 


extern SymTable  *_glob_op_tab_;

UList   print_globals() { _glob_op_tab_->print(cout); return nil; }

UList   equal_ul( UList x, UList y ) { return new_int( x == y ); }
UList   less_ul( UList x, UList y ) { return new_int( x < y ); }

int   and_ii( int x, int y ) { return x && y; }
int   greater_ii( int x, int y ) { return x > y; }
int   not_i( int x ) { return !x; }
int   or_ii( int x, int y ) { return x || y; }
int   positive_i( int x ) { return x > 0; }

int   add_ii( int x, int y ) { return x + y; }
int   dec_i( int x ) { return x - 1; }
int   div_ii( int x, int y ) { return x / y; }
int   inc_i( int x ) { return x + 1; }
int   mult_ii( int x, int y ) { return x * y; }
int   sub_ii( int x, int y ) { return x - y; }

//--------------------------------------------------------------------------------
//  file read
//  integer ops
//  list ops
//  creating user urelements
//  CA, FSM, Graph, SGroup operations

void   load_standard_operators( SymTable& stab )
{
    stab.assign( print_globals,    "help" );
    stab.assign( read_atom,        "read" );
    stab.assign( write_atom,       "write" );

    stab.assign( positive_i,       "positive" );
    stab.assign( inc_i,            "inc" );
    stab.assign( dec_i,            "dec" );
    stab.assign( not_i,            "not" );
    stab.assign( greater_ii,       "greater" );
    stab.assign( add_ii,           "add" );
    stab.assign( sub_ii,           "sub" );
    stab.assign( mult_ii,          "mult" );
    stab.assign( div_ii,           "div" );
    stab.assign( and_ii,           "and" );
    stab.assign( or_ii,            "or" );
			           
    stab.assign( less_ul,          "less" );
    stab.assign( equal_ul,         "equal" );
    stab.assign( Head,             "H" );
    stab.assign( Tail,             "T" );
    stab.assign( flatone,          "flatone" );
    stab.assign( flatten,          "flat" );
    stab.assign( length,           "length" );
    stab.assign( size,             "size" );
    stab.assign( range,            "range" );
    stab.assign( reverse,          "rev" );
    stab.assign( unique_ul,        "unique" );
    stab.assign( mergesort,        "sort" );
    stab.assign( print_ul,         "print" );
    stab.assign( prepend,          "pre" );
    stab.assign( append,           "app" );
    stab.assign( at,               "at" );
    stab.assign( cartprod,         "cart" );
    stab.assign( join,             "join" );
    stab.assign( map_ul,           "map" );
    stab.assign( substitute,       "subst" );
    stab.assign( partition_ul,     "part" );
    stab.assign( powerset,         "powerset" );

    stab.assign( create_ca,        "CA" );
    stab.assign( create_dfa,       "DFA" );
    stab.assign( create_fa,        "FA" );
    stab.assign( create_sa,        "SA" );
    stab.assign( create_graph,     "Graph" );

    stab.assign( CAanalyze,        "analyze" );
    stab.assign( CAcompose,        "compose" );
    stab.assign( CAorbit,          "orbit" );
    stab.assign( CAtoSA,           "tosa" );
    stab.assign( CAtoWelch,        "towelch" );
    stab.assign( balanced,         "bal" );

    stab.assign( FSMStates,        "states" );
    stab.assign( FSMtoGraph,       "tograph" );
    stab.assign( accessibleFSM,    "acc" );
    stab.assign( reverseSA,        "revsa" );
    stab.assign( epsElimFA,        "epselim" );
    stab.assign( SAtoFA,           "tofa" );
    stab.assign( toDFA,            "todfa" );
    stab.assign( kernelSA,         "kernel" );
    stab.assign( toDFASize,        "todfasize" );
    stab.assign( productSA,        "prodsa" );
    stab.assign( productDFA,       "proddfa" );
    stab.assign( quotientDFA,      "quotdfa" );
    stab.assign( squareSA,         "squaresa" );
    stab.assign( minimize,         "minimize" );
    stab.assign( DFAtoLanguage,    "lang" );
    stab.assign( Fischer,          "fischer" );
    stab.assign( synchFischer,     "synch" );
    stab.assign( delta,            "delta" );
    stab.assign( Delta,            "Delta" );

    stab.assign( longestPath,      "longpath" );
    stab.assign( strongComponents, "scc" );

    stab.assign( toGeneratorsDFA,  "togen" );
    stab.assign( generateSG,       "generate" );
    stab.assign( aperiodicSG,      "aperiodic" );
    stab.assign( printDClassesSG,  "printdcl" );
    stab.assign( printEquationsSG, "printeq" );
    stab.assign( printWitnessesSG, "printwit" );
    stab.assign( printElementsSG,  "printsg" );
    stab.assign( equationListSG,   "eqlist" );
    stab.assign( witnessListSG,    "witlist" );
    stab.assign( dClassListSG,     "dclist" );

    stab.assign( piMatrix,         "pimatrix" );

}

void    load_from_global( TokenHTab& ht )
{
    extern  SymTable *_glob_op_tab_;
    SymTable::iterator  it;
    
    for( it = _glob_op_tab_->begin(); it != _glob_op_tab_->end(); ++it )
    {
	int k = arity( it->second ); //  UList : Function 
	ht[it->first.c_str()] = Token( OPER, k );
    }
}

//--------------------------------------------------------------------------------
