//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:37:20 sutner> 
//
// :File:      ca.h
// :Purpose:   CA class.
//             Conversions to  SA  and  Graph
//             
//---------------------------------------------------------------------------



#ifndef _CELLAUT_H_
#define _CELLAUT_H_

#include  "integer.h"

#include  "basic.h"
#include  "graph.h"
#include  "fsm.h"



//------------------------------------------------------------------------------
//  cellular automaton base class
//------------------------------------------------------------------------------

class  SA;
class  Graph;

class CA
{
    size_t       _wdth;		// width of CA
    size_t       _nsym;		// alphabet 0, ..., _nsym-1
    Integer      _loc;		// local map, base _nsym integer 


public:

    CA(): _wdth(0), _nsym(0), _loc(0) {}
    CA( size_t r, size_t w=3, size_t k=2): _wdth(w), _nsym(k), _loc((long int)r) {}
    CA( Integer r, size_t w=3, size_t k=2 ): _wdth(w), _nsym(k), _loc(r) {}
    CA( const CA& src ): _wdth(src._wdth), _nsym(src._nsym), _loc(src._loc) {}

    size_t&       width()          { return _wdth; }
    size_t&       numSymbols()     { return _nsym; }
    Integer&      rule()           { return _loc; } 

    const size_t  width() const         { return _wdth; }
    const size_t  numSymbols() const    { return _nsym; }
    const Integer rule() const  { return _loc; } 

    void          makeLabels( uvector& lab )  const;

    void  set( const Integer& r, const size_t w, const size_t k )
          {_wdth=w; _nsym=k; _loc=r; }

    void  orbit( const uvector& pat, int t, int cyc, uvector& orb ) const;
    void  toSA( SA& sa ) const;
    void  toGraph( Graph& gr ) const;
    int   analyze() const;
    int   balancedQ( int b=1 ) const;
    void  compose( const CA& c2, CA& cc ) const;

    friend  ostream&  operator<<( ostream& out, const CA& ca );
    
};  // end class CA

//------------------------------------------------------------------------------

#endif



