//--------------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:39:52 sutner> 
//
//  :File:      intset.h
//  :Purpose:   set of integers
//              -- bitvector for O(1) membership
//              -- stack for O(|S|) traversal
//              Note: elements appear on stack in order of insertion.
//              Should probably be subclassed from stack and bitvector.
//             
//--------------------------------------------------------------------------------

#ifndef _INTSET_H
#define _INTSET_H

#include  <list>
#include  <algorithm>
#include  "basic.h"
#include  "bitvect.h"
#include  "stl-wrap.h"

//----------------------------------------------------------------------
// defined herein

class IntSet;

//----------------------------------------------------------------------

class IntSet
{
    uvector      il;		// stack for traversal
    BitVector    bv;		// bitvector, packed 

public:
    typedef  uvector::iterator        iterator;
    typedef  uvector::const_iterator   const_iterator;

    IntSet(){}
    explicit  IntSet( size_t n ): bv(n,0) {}

    ~IntSet();
    
    size_t            capacity()  const   { return bv.size(); }
    const BitVector&  bitvector() const   { return bv; }

    iterator  begin()              { return il.begin(); }
    iterator  end()                { return il.end(); }
    const_iterator  begin() const  { return il.begin(); }
    const_iterator  end()   const  { return il.end(); }

    bool         empty() const     { return il.empty(); }     
    size_t       size()  const     { return il.size(); } 
    size_t       top()   const     { return il.back(); }
    size_t       pop();    
    void         sort()            { ::sort( il.begin(), il.end() ); }

    bool  find( size_t i ) const   { return bv.getbit(i); }
    void  insert( size_t i );
 
    // destroy and re-initialize
    void  initialize( size_t nn );

    // delete all elements, don't free bitvector
    void  sclear();
   
    // assignment, equality 
    IntSet&  operator=( const IntSet& src );
    bool     operator==( const IntSet& src ) const;

    void   print( ostream& os, int off=0 ) const;

    friend ostream& operator<<(ostream& out, const IntSet& st )
    { return out << st.il; }
 
};

inline  size_t  IntSet::pop() 
{ 
    size_t res = il.back(); 
    il.pop_back(); 
    bv.setbit(res,0); 
    return res; 
}


inline  void  IntSet::insert( size_t i )
{
    if( !bv.getbit(i) )
    {
	bv.setbit(i,1);
	il.push_back(i);
    }
}

//--------------------------------------------------------------------------------
#endif


