//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:42:32 sutner> 
//
//  :File:      parser.h
//  :Purpose:   automata parser
//              should be dumped and rewritten from scratch
//              
//---------------------------------------------------------------------------


#ifndef  _PARSER_H_
#define  _PARSER_H_

#include "basic.h"

#include  <iostream>
#include  <fstream.h>
#include  <string>
#include  <hash_map.h>
#include  <list>

#include  "urlist.h"
#include  "listutil.h"
#include  "util.h"
#include  "makeatom.h"
#include  "ptree.h"

#define  YYSTYPE  UList
#include  "bparser.h"

//--------------------------------------------------------------------------------
// only used with \0 terminated strings, so this works fine

struct eqchars: public binary_function<char*,char*,bool>
{
    eqchars() {}
    bool operator()(const char* s1, const char* s2) const
    {
	return strcmp(s1, s2) == 0;
    }
};

struct  Token
{
    int    tok;
    int    ari;
    
    Token(): tok(NONET), ari(-1) {}
    Token( int t, int k=-1 ): tok(t), ari(k) {}
};

inline  bool  operator==( const Token& tt, int t )
{
    return tt.tok == t;
}

inline  bool  operator!=( const Token& tt, int t )
{
    return tt.tok != t;
}

typedef  hash_map< const char*, Token, hash<const char*>, eqchars >  TokenHTab;

//--------------------------------------------------------------------------------

const  int BLEN = 16384;	// should probably be dynamic

class Parser 
{
public:
    enum  PrntT { AUTO, LOUD, QUIET, SILENT };

				// provides environment, runs parse() on all streams
    void    run();
    void    evaluate( const char *expr );
    UList   parse( SymTable& env );
    int     get_next_token( UList*& );

    bool    do_print()            { return pflag; }
    void    set_print( bool b=1 ) { pflag = b; }
    bool    is_quiet() const      { return loud != LOUD; }
    bool    is_silent() const     { return loud == SILENT; }
    void    set_loud( PrntT t )   { loud = t; }
    void    set_loud_0( PrntT t ) { loud_0 = t; }
    void    reset_loud()          { loud = loud_0; }
    bool    is_go()    const      { return goflag; }
    void    print_table();
    list<string>&  streams()	  { return strmtab; }
    bool    attach( const char *fname, const char *buff=0 );
    void    detach();
    const char*  stream_name() const { return strmnm; }
    
    UList   value() const         { return c_val; } 
    UList   set_value( UList v )  { return c_val = v; } 

    void    in_prompt()  const;
    void    out_prompt( UList x ) const;
    
    Parser();
    ~Parser();

private:

    Parser(const Parser&);
    Parser& operator=(const Parser&);

    PrntT  loud;		// loudness level
    PrntT  loud_0;		// base loudness level
    bool   pflag;		// print current expression
    bool   goflag;		// continue processing

    TokenHTab   toktab;		// table for reserved words
    Token       token;
    
    UList       c_val;		// value of current parse
    istream    *strmp;		// input stream for parser
    char       *strmnm;		// name of input file/cin
    list<string>  strmtab;	// list of stream names
    
    UList     read_int();
    UList     read_string();
    UList     read_ilist();

    // helpers
    int    look_ahead( int, int, int );
    
    static  void  initialize();
    static  void  finalize();
    static  int   parsercnt;
};


// for reentrant parser: pass paramters to yyparse, yylex

struct ParserControl
{
    Parser    *pap;		/* the parser calling yyparse()  */
    SymTable  *env;		/* symbol table for evaluation   */

    ParserControl( Parser *pp, SymTable *sp ): pap(pp), env(sp) {}
};
     

//--------------------------------------------------------------------------------
// error class for exceptions, unimplemented

class  ParseError
{
    char  *mssg1;
    char  *mssg2;
    
public:

    ParseError( char *s1=NULL, char *s2=NULL ): mssg1(s1), mssg2(s2) {}

    void  print() 
    { 
	cout << "Error: ";
	cout << (char*)mssg1 << " ";
	cout << (char*)mssg2 << endl; 
    }
};

//--------------------------------------------------------------------------------
#endif


//  Local Variables: ***
//  eval: (setq case-fold-search nil) ***
//  eval: (setq case-replace nil) ***
//  End: ***
