//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 19:01:32 sutner> 
//
// :File:      semimod.h
// :Purpose:   templated class for semimodules
//             generated by  op :  Q x (kk) --> Q
//             
//---------------------------------------------------------------------------

#ifndef  _SEMIMOD_H_
#define  _SEMIMOD_H_


#include  "stl-wrap.h"
#include  "basic.h"
#include  "pairs.h"
#include  <cctype>
#include  <iostream>
#include  <hash_map.h>
#include  <algorithm>
#include  <iterator> 

//--------------------------------------------------------------------------------
// pre:  type T must support  hash<T>  and  equal_to<T>
//
// note:  bound in generate() currently not implemented
//        hash_map seems to give better performance than map
//        
// ??? should  del be  uvector
// 

template<class T, class Oper>
struct   SemiModule
{
    typedef  hash_map<T,size_t,hash<T>,equal_to<T> >   elset;
    typedef  typename elset::iterator                    iterator;
    typedef  list<iterator>                     ellist;
    
    size_t         kk;		// number of operations
    elset          Q;		// hash table, (t,number) 
    ellist         L;		// elementlist for traversal (iterators to Q)
    T              q0;		// generator
    Oper           op;		// the operation   op :  Q x (kk) --> Q
    ulist          del;		// transition function 

    SemiModule( size_t k, const T& t, const Oper& F );
    
    size_t   generate();
};


template<class T, class Oper>  
SemiModule<T,Oper>::SemiModule( size_t k, const T& t, const Oper& F ): 
    kk(k) , q0(t), op(F) 
{
    pair<iterator,bool> pp = Q.insert(pair<T,size_t>(t,0));
    L.push_back(pp.first);
}

template<class T, class Oper>  
size_t  SemiModule<T,Oper>::generate()
{
    typename ellist::const_iterator  it;
    size_t  pp = 0;
    
    for( it = L.begin(); it != L.end(); ++it )
    {
	for( size_t i = 0; i < kk; ++i )
	{
	    T  z = op( (**it).first, i );
	    iterator  qit = Q.find( z );
	    if( qit == Q.end() )          // actual insertion
	    {
		L.push_back( Q.insert( pair<T,size_t>( z, ++pp ) ).first );
		del.push_back( pp );
	    }
	    else
		del.push_back( qit->second );
	}
    }
    
    return  L.size();
}



//--------------------------------------------------------------------------------
#endif

