//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:00:21 sutner> 
//
// File:     useratom.h
// Purpose:  
//---------------------------------------------------------------------------

#ifndef  _USERAT_H_
#define  _USERAT_H_

#include  "basic.h"

#include  <vector>
#include  <list>
#include  "integer.h"
#include  "urlist.h"
#include  "ca.h"
#include  "fsm.h"
#include  "sgroup.h"
#include  "graph.h"

//------------------------------------------------------------------------------
// classes defined herein
//------------------------------------------------------------------------------

class IntegerAtom;		  // gmp integers
class IntArrAtom;		  // arrays of plain integers
class CAAtom;			  // cellular automaton
class SAAtom;			  // semiautomaton
class FAAtom;			  // automaton
class DFAAtom;			  // dfa
class SGAtom;			  // semigroup 
class GraphAtom;		  // digraphs

template<class T> class TAtom;	  // templated user atoms, use with caution


//--------------------------------------------------------------------------------
// general layout for user atom

// struct  XXXAtom: public RefAtom
// {
//     XXX  dat;

//     XXXAtom( const XXX& );
//     ~XXXAtom();

//     XXXAtom        *destroy();
//     const XXXAtom  *clone() const;
//     AType  atom_type() const;
    
//     void        print( ostream& ) const;
//     void        sub_print( ostream& ) const;
//     XXX&        value();
//     const XXX&  value() const; 
//     UList       eval( SymTable& );
// };



//--------------------------------------------------------------------------------
// User Atoms, use reference counting 
//--------------------------------------------------------------------------------


struct  IntegerAtom: public RefAtom
{
    Integer  dat;

    IntegerAtom();
    IntegerAtom( long int x=0 );
    IntegerAtom( const char *digs );
    IntegerAtom( const Integer&  );
    IntegerAtom( const IntegerAtom& src );
    ~IntegerAtom();

    const IntegerAtom  *clone() const;
    AType  atom_type() const;
    
    void           print( ostream& out ) const;
    void           sub_print( ostream& ) const;
    Integer&       value();
    const Integer& value() const; 
    UList          eval( SymTable& );
};

struct  IntArrAtom: public RefAtom
{
    ivector  dat;
    
    IntArrAtom();
    IntArrAtom( const ivector& );
    IntArrAtom( const uvector& );
    ~IntArrAtom();

    const IntArrAtom  *clone() const;
    AType  atom_type() const;
    void               print( ostream& ) const;
    void               sub_print( ostream& ) const;
    ivector&           value();
    const ivector&     value() const;
    UList              eval( SymTable& );
};

struct  SAAtom: public RefAtom
{
    SA  dat;

    SAAtom();
    SAAtom( int, int );
    SAAtom( const SA& );
    ~SAAtom();

    const SAAtom  *clone() const;
    AType  atom_type() const;
    
    void       print( ostream& ) const;
    void       sub_print( ostream& ) const;
    SA&        value();
    const SA&  value() const; 
    UList      eval( SymTable& );
};

struct  FAAtom: public RefAtom
{
    FA  dat;

    FAAtom();
    FAAtom( int, int );
    FAAtom( const FA& );
    ~FAAtom();

    const FAAtom  *clone() const;
    AType  atom_type() const;
    
    void       print( ostream& ) const;
    void       sub_print( ostream& ) const;
    FA&        value();
    const FA&  value() const; 
    UList       eval( SymTable& );
};

struct  DFAAtom: public RefAtom
{
    DFA  dat;

    DFAAtom();
    DFAAtom( int, int );
    DFAAtom( const DFA& );
    ~DFAAtom();

    const DFAAtom  *clone() const;
    AType  atom_type() const;
    
    void       print( ostream& ) const;
    void       sub_print( ostream& ) const;
    DFA&        value();
    const DFA&  value() const; 
    UList       eval( SymTable& );
};

struct CAAtom: public RefAtom
{
    CA       dat;

    CAAtom();
    CAAtom( const CA& );
    ~CAAtom();

    const  CAAtom  *clone() const;
    AType  atom_type() const;
    void             print( ostream& out ) const;
    void             sub_print( ostream& out ) const;
    CA&              value();
    const CA&        value() const;
    UList           eval( SymTable& );
};

struct SGAtom: public RefAtom
{
    SGroup       dat;

    SGAtom();
    SGAtom( const SGroup& );
    ~SGAtom();

    const  SGAtom  *clone() const;
    AType  atom_type() const;
    void             print( ostream& ) const;
    void             sub_print( ostream& ) const;
    SGroup&              value();
    const SGroup&        value() const;
    UList           eval( SymTable& );
};


struct GraphAtom: public RefAtom
{
    Graph       dat;

    GraphAtom();
    GraphAtom( const Graph& );
    GraphAtom( int n, int e );
    ~GraphAtom();

    const  GraphAtom  *clone() const;
    AType  atom_type() const;
    void             print( ostream& ) const;
    void             sub_print( ostream& ) const;
    Graph&              value();
    const Graph&        value() const;
    UList           eval( SymTable& );
};




//--------------------------------------------------------------------------------
// utilities


#ifdef URLBUG

inline  Integer&  Ivalue( CRef a ) 
{ 
    IntegerAtom* p = dynamic_cast<IntegerAtom*>(a.ap);
    if( p ) return p->value(); 
    Error( "IntegerAtom: wrong atom type" );
    exit(1);
}
inline  ivector&  iavalue( CRef a ) 
{ 
    IntArrAtom* p = dynamic_cast<IntArrAtom*>(a.ap);
    if( p ) return p->value(); 
    Error( "IntArrAtom: wrong atom type" );
    exit(1);
}
inline  SA&  savalue( CRef a ) 
{ 
    SAAtom* p = dynamic_cast<SAAtom*>(a.ap);
    if( p ) return p->value(); 
    Error( "SAAtom: wrong atom type" );
    exit(1);
}
inline  FA&  favalue( CRef a ) 
{ 
    FAAtom* p = dynamic_cast<FAAtom*>(a.ap);
    if( p ) return p->value(); 
    Error( "FAAtom: wrong atom type" );
    exit(1);
}
inline  DFA&  dfavalue( CRef a ) 
{ 
    DFAAtom* p = dynamic_cast<DFAAtom*>(a.ap);
    if( p ) return p->value(); 
    Error( "DFAAtom: wrong atom type" );
    exit(1);
}
inline  CA&  cavalue( CRef a ) 
{ 
    CAAtom* p = dynamic_cast<CAAtom*>(a.ap);
    if( p ) return p->value(); 
    Error( "CAAtom: wrong atom type" );
    exit(1);
}
inline  SGroup&  sgvalue( CRef a ) 
{ 
    SGAtom* p = dynamic_cast<SGAtom*>(a.ap);
    if( p ) return p->value(); 
    Error( "SGAtom: wrong atom type" );
    exit(1);
}
inline  Graph&  grvalue( CRef a ) 
{ 
    GraphAtom* p = dynamic_cast<GraphAtom*>(a.ap);
    if( p ) return p->value(); 
    Error( "GraphAtom: wrong atom type" );
    exit(1);
}

#else

inline  Integer&  Ivalue( CRef a )  { return ((IntegerAtom*)(a.ap))->value(); } 
inline  IntArr&   iavalue( CRef a ) { return ((IntArrAtom*)(a.ap))->value(); } 
inline  SA&   savalue( CRef a )     { return ((SAAtom*)(a.ap))->value(); } 
inline  FA&   favalue( CRef a )     { return ((FAAtom*)(a.ap))->value(); } 
inline  DFA&  dfavalue( CRef a )    { return ((DFAAtom*)(a.ap))->value(); } 
inline  CA&   cavalue( CRef a )     { return ((CAAtom*)(a.ap))->value(); } 
inline  Graph&  grvalue( CRef a )   { return ((GraphAtom*)(a.ap))->value(); } 
inline  SGroup&  sgvalue( CRef a )  { return ((SGAtom*)(a.ap))->value(); } 

#endif


//--------------------------------------------------------------------------------
#endif

