//------------------------------------------------------------------------------
//
// Time-stamp: <2003-11-24 18:01:05 sutner>
// 
// :File:     util.h 
// :Purpose:  conversion utilities
//            in particular working on complicated atoms
//            see listutil.h for list oriented procedures
//------------------------------------------------------------------------------

#ifndef _UTIL_H
#define _UTIL_H

//----------------------------------------------------------------------

#include  <list>

#include  "basic.h"
#include  "bitvect.h"
#include  "ca.h"
#include  "fsm.h"
#include  "graph.h"
#include  "urlist.h"
#include  "sgroup.h"

//------------------------------------------------------------------------------
// add user functions here
// all MUST be of type  UList --> UList or  UList x UList --> UList 
//------------------------------------------------------------------------------

				// finite state machines
UList  FSMtoGraph( UList );

UList  toDFASize( UList );
UList  minimizeSA( UList );
UList  epsElimFA( UList );
UList  epsElimSA( UList );
UList  SAtoFA( UList );
UList  reverseFA( UList );
UList  reverseSA( UList );
UList  squareSA( UList );
UList  FSMStates( UList );
UList  Fischer( UList L );
UList  synchFischer( UList L );

				// cellular automata
UList  CAtoSA( UList );
UList  CAanalyze( UList );
UList  CAorbit( UList );
UList  CAcompose( UList, UList );
				// semigroups
UList  toGeneratorsDFA( UList );
UList  equationListSG( UList );
UList  witnessListSG( UList );
UList  dClassListSG( UList );
UList  printElementsSG( UList );
UList  printWitnessesSG( UList );
UList  printEquationsSG( UList );
UList  printDClassesSG( UList );
UList  aperiodicSG( UList );
				// graphs 
UList  BuildGraph( UList );
UList  strongComponents( UList, UList );
UList  longestPath( UList );

				// binary functions 
UList  minimize( UList, UList );
UList  accessibleFSM( UList, UList );
UList  toDFA( UList, UList );
UList  DFAtoLanguage( UList, UList );
UList  balanced( UList, UList );
UList  generateSG( UList, UList );
UList  CAtoWelch( UList, UList );
UList  productSA( UList, UList );

				// ternary functions 
UList  kernelSA( UList, UList, UList );
UList  quotientDFA( UList, UList, UList );
UList  productDFA( UList, UList, UList );

UList  delta( UList, UList, UList );
UList  Delta( UList, UList, UList );


				// general
UList  size( UList L );

//----------------------------------------------------------------------
// helper functions 
// conversion UList <--> standard data structures
//----------------------------------------------------------------------

void    ulist_to_intarray(  UList L, uvector& arr, int off=0 );
void    ulist_to_intset(  UList L, int msize, IntSet& set, int off=0 );
void    ulist_to_intlist( UList L, ulist& il, int off=0 ); 
void    ulist_to_ilist( UList L, ilist& il, int off=0 );
bool    ulist_to_transition( UList L, TransV& tr ); 

UList   intlist_to_ulist( const ilist& il, int off=0 ); 
UList   intarray_to_ulist( const uvector& arr, int off=0 );
UList   intset_to_ulist( const IntSet& set, int off=0 );

UList   scc_to_ulist( const ivector& cmp, const ivector& ntr, int ntriv );

UList   order_select_ulist( UList st, IntSet& reach, int n );
UList   partition_ulist( UList st, ivector& partind, int n, int npart );

UList   bitvector_to_ulist( const BitVector& bv, int off=0 );
UList   bitmatrix_to_ulist( const BitMatrix& bm, int off=0 );

//----------------------------------------------------------------------
// helper functions 
// conversion between standard data structures
//----------------------------------------------------------------------

void  integer_to_array( const Integer& n, uvector& A, const size_t k=2 );
void  array_to_integer( const uvector& A, Integer& n, const size_t k=2 );

//----------------------------------------------------------------------
#endif  // _UTIL_H





