//--------------------------------------------------------------------------------
//
// Time-stamp: <2003-11-24 14:55:32 sutner>
// 
// File:     intset.cc
// Purpose:  implementation file for bitvector/stack sets of integers. 
//
//--------------------------------------------------------------------------------

#include  "stl-wrap.h"
#include  "bitvect.h"
#include  "intset.h"

//----------------------------------------------------------------------
// Implementation
//----------------------------------------------------------------------


IntSet::~IntSet() {}      
   
 

void  IntSet::initialize( size_t nn )
{ 
    il.erase( il.begin(), il.end() );
    bv.initialize(nn,0);
} 

// delete all elements, don't free bitvector
void  IntSet::sclear()
{
    while( !il.empty() )
    { 
	size_t res = il.back(); 
	il.pop_back(); 
	bv.setbit(res,0); 
    }
}
   
   
// assignment, equality 
IntSet&  IntSet::operator=( const IntSet& src )
{
    if( &src==this )
	return *this;

    // get rid of previous contents, then copy
    if( capacity()==src.capacity() )
	sclear();
    else
	initialize( src.capacity() );  

    IntSet::const_iterator  it;
    foreach_item( it, src )
	insert(*it);

    return *this;
}


bool      IntSet::operator==( const IntSet& src ) const
{
    if( &src==this ) return 1;
      
    // get out if cardinality or universe differs
    // maybe counter-intuitive 
    if( size() != src.size() || capacity() != src.capacity() )
	return 0;

    IntSet::const_iterator it;
    
    foreach_item( it, src )
	if( !find(*it) ) return 0;

    return 1;
}



void   IntSet::print( ostream& os, int off ) const
{
    int  ww = ( capacity() > 999 ) ? 8 : 4;
    uvector::const_iterator  it;
    
    for( it = il.begin(); it != il.end(); ++it )
	os << setw(ww) << (*it + off);
    os << endl;
}

//----------------------------------------------------------------------




