//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:21:12 sutner> 
//
// :File:      listutil.cc
// :Purpose:   utility function for UList
// 
//---------------------------------------------------------------------------

#include  "listutil.h"
#include  <vector>

//--------------------------------------------------------------------------------

// needs to be beefed up
UList print_ul( UList L) 
{  
    L.print( cout );
    cout << endl;
    return nil;
}

int  length_int( UList L )
{
    int n = 0;
    while( L )
    {
	++n; ++L;
    }
    return n;
}

UList  length( UList L )
{
    return  new_int(length_int(L));
}

UList  reverse( UList L )
{
    UList res;
    while( L )
    {
	res += *L;
	++L;
    }
    return res;
}


UList  join( UList L, UList K )
{
    if( !L ) return K;
    UList h; 
    L >> h; 
    return h -= join( L, K );
}

UList  append( UList L, UList a )
{
    return join( L, a -= nil );
}

UList  prepend( UList L, UList a )
{
    return  L += a;
}

//--------------------------------------------------------------------------------
// Cartesian product
// first, using only ULists

// UList cartprod( UList A, UList B )
// {
//     if( !A || !B ) return nil;
//     UList  AA = reverse(A);
//     UList  BB = reverse(B);
//     UList  res, a, b;

//     while( AA )
//     {
// 	AA >> a;
// 	UList BBB = BB;
// 	while( BBB )
// 	{
// 	    BBB >> b;
// 	    res += (a -= b -= nil);
// 	}
//     }
//     return res;
// }


// and then a faster hack: using vector<UList> 

UList cartprod( UList A, UList B )
{
    if( !A || !B ) return nil;

    vector<UList>  AA(100);
    vector<UList>  BB(100);
    AA.resize(0);
    BB.resize(0);

    UList  res, x;

    while( A )
    {
	A >> x;
	AA.push_back( x );
    }
    
    while( B )
    {
	B >> x;
	BB.push_back( x );
    }
    
    for( int i = AA.size()-1; i >= 0; --i )
	for( int j = BB.size()-1; j >=0; --j )
	    res += (AA[i] -= BB[j] -= nil);

    return res;
}




UList cartprod( UList L )
{
    UList L1, L2;
    L >> L1 >> L2;
    return cartprod(L1, L2);
}

// L must be rectangular matrix
UList  transpose( UList L )
{
    vector<UList> row;
    UList  T, x;
    
    while( L )
    {
	L >> x;
	row.push_back(x);
    }
    int nr = row.size();
    int nc = length_int( row[0] );
    int i, j;
    
    for( i = 0; i < nc; ++i )
    {
	UList  r;
	for( j = nr-1; j >= 0; --j )
	{
	    row[j] >> x;
	    r += x;
	}
	T += r;
    }
    return reverse(T);
}

//--------------------------------------------------------------------------------

static void sub_flatten( UList L, UList& res )
{
    if( L.atom_type() == CONS ) 
    { 
        UList h;
        L >> h;
        if( L ) sub_flatten( L, res );
        if( h ) sub_flatten( h, res );
        return;
    }
    res += L;
}

UList flatten( UList L )
{
    UList res;
    sub_flatten(L, res);
    return res;
}


UList flatone( UList L )
{
    UList res, h, x; 
    while( L )
    {
	L >> h;
	if(  h.atom_type() == CONS )
	    while( h )
	    {
		h >> x;
		res += x;
	    }
	else
	    if( h ) res += h;
    }
    return reverse( res );
}

//----------------------------------------------------------------------


UList  range_i( int lo, int hi )
{
    UList  res;
    for( int i=hi; i>=lo; i-- )
        res += new_int(i);
    return res;
}

UList  range_i( int hi ) { return  range_i( 1, hi ); }

// L = { hi }  or  L = { lo, hi }
UList  range( UList L )
{
    UList  lo, hi;
    int  ilo=1, ihi;
    if( L.atom_type() == CONS )
    {
	L >> lo >> hi;

	if( !hi )
	    ihi = ivalue( lo );
	else 
	{
	    ihi = ivalue( hi );
	    ilo = ivalue( lo );
	}
    }
    else
	ihi = ivalue( L );
	

    return range_i( ilo, ihi );
}

//--------------------------------------------------------------------------------
// indexing 0,1,..,n-1, just like in an array

UList  at( UList L, UList P )
{
    if( !L ) return nil;

    if(  P.atom_type() == INT )
    {
	int  p = ivalue( P );
	while( p-- > 0 ) ++L;
	return *L;
    }
				// P is a list of positions
    UList res, LL = L;
    int  q = 0, p;		// q: current position in L, p in P
    while( P )
    {
	p = ivalue( *P ); ++P;
	if( p >= q )
	{
	    for( int i = 0; i < p - q; ++i ) ++L;
	}
	else
	{
	    L = LL;
	    for( int i = 0; i < p; ++i ) ++L;
	}
	res += *L;
	q = p;
    }
	    
    return reverse(res);
}

//--------------------------------------------------------------------------------
//  substitution of integers in ULists
//  
//  L: list, possibly nested with atoms in [0,n-1]
//  S: substitution list of size m >= n
//  result:  replace atom i by S_i

 
static UList  subst( UList L, vector<UList>& ptr )
{
    if( !L ) return nil;

    if(  L.atom_type() == CONS )
    { 
	UList h;
	L >> h;
	return subst( h, ptr ) -= subst( L, ptr );
    }
    
				// better be an int in [0,m-1]
    return  ptr[ivalue(L)];
}


UList  substitute( UList L, UList S )
{
    int i = 0, n = length_int(S);
    UList  h;
    vector<UList>  ptr(n);
  
    while( S )
    {
	S >> h; 
	ptr[i++] = h;
    }

    return  subst( L, ptr );
}

UList  substitute_int( UList L, int n )
{
    return substitute( L, range_i(1,n) );
}


//--------------------------------------------------------------------------------
// is this stuff really needed ? 

UList  erase( UList a, UList L )
{
    UList res, h;
    while( L )
    {
	L >> h;
	if( !(h == a) )
	    res += h;
    }
    return reverse(res);
}

UList  partition_ul( UList L, UList K )
{
    int k = ivalue(K);
    UList P, b, x;
    while( L )
    {
	b = nil;
	for( int i = 0; i < k; ++i )
	{
	    L >> x;
	    b += x;
	}
	P += reverse(b);
    }
    return reverse(P);
}

UList powerset( UList L )
{
    if( !L ) return nil -= nil;
    
    UList P, PP, a, x;
    L >> a;
    P = powerset(L);
    PP = reverse(P);
    while( P )
    {
	P >> x;
	x += a;
	PP += x;
    }
    return reverse(PP);
}

static  UList  merge( UList A, UList B )
{
    UList res, x;

    while( A && B )
    {
	if( *A < *B )
	    A >> x;
	else
	    B >> x;

	res += x;
    }
	
    while( A )
    {
	A >> x;
	res += x;
    }
    while( B )
    {
	B >> x;
	res += x;
    }
    
    return   reverse(res);
}

UList  mergesort( UList L )
{
    if( !L || !L.tail() ) return L;

    UList A, B, x;
    while( L )
    {
	L >> x;
	A += x;
	if( !L ) break;
	L >> x;
	B += x;
    }

    A = mergesort(A);
    B = mergesort(B);
    return  merge(A,B);
}

UList  unique_ul( UList L )
{
    if( !L ) return nil;
    UList  res, x, y;
    L >> x;
    res += x;
    while( L )
    {
	L >> y;
	if( !(x == y) )
	{
	    x = y;
	    res += x;
	}
    }
    return reverse(res);
}

UList  map_ul( UList F, UList L )
{
    UList x, res;
    while( L )
    {
	L >> x;
	res += F(x);
    }
    return reverse(res);
}


//--------------------------------------------------------------------------------

//  Local Variables: ***
//  eval: (setq case-fold-search nil) ***
//  eval: (setq case-replace nil) ***
//  End: ***
