//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:23:06 sutner> 
//
// :File:      semimod.cc
// :Purpose:   testing templated semimodule generation
//---------------------------------------------------------------------------


#include  <cctype>
#include  <iostream>
#include  <hash_map.h>
#include  <hash_set.h>
#include  <algorithm>

#include  "stl-wrap.h"
#include  "basic.h"
#include  "pairs.h"
#include  "intset.h"
#include  "semimod.h"

//--------------------------------------------------------------------------------

typedef  vector<bool>   bvector;
typedef  Pair<bvector,size_t>  Pattern;
typedef  Triple<size_t,bvector,string>  SVS;
typedef  hash_map<size_t,string,hash<size_t>,equal_to<size_t> >  WitTable;

//--------------------------------------------------------------------------------

struct   hash<bvector>
{
    size_t  operator()( const bvector& A  ) const
    {
	size_t h = 0;
	for( size_t i = 0; i < A.size(); ++i )
	    h = h*103 + A[i];
	return h;
    }
};

struct   hash<Pattern>
{
    size_t  operator()( const Pattern& A  ) const
    {
	const bvector  &AA = A.first();
	size_t h = A.second();
	for( size_t i = 0; i < AA.size(); ++i )
	    h = h*103 + AA[i];
	return h;
    }
};



struct  one_perm: public binary_function<Pattern,size_t,Pattern>
{
    size_t   nn, qq;			// shift factor for b transitions
    
    one_perm( size_t n, size_t q ): nn(n), qq(q) {} 

    void  dec( size_t& p ) const
    {
	if( p == 0 )
	    p = nn - 1;
	else
	    p--;
    }
    
    Pattern  operator()( const Pattern& A, size_t s ) const 
    {
	bvector  B( A.first() );
	size_t   p = A.second();
	if( s == 0 )
	{
	    if( B[p] )
	    { 
		size_t pp = p + qq-1;
		if( pp >= nn ) pp -= nn;
		B[pp] = 1;
	    }
	    dec( p );
	}
	else
	{
	    B[p] = 0;
	    dec( p );
	    dec( p );
	}

	return  Pattern( B, p );
    }
};

void  print_bvector( const bvector& A )
{
    for( size_t i = 0; i < A.size(); ++i )
	cout << ( A[i] ? 'x' : '.' );
}


main( int argc, char **argv )
{
    size_t n = atoi(argv[1]);
    size_t q = atoi(argv[2]);
    
    bvector   A( n, true );
    
    one_perm  op( n, q );

    SemiModule<Pattern,one_perm> SM( 2, Pattern(A,0), op );

    size_t  m = SM.generate();
    cout << "*** " << n << "  " << q << " " << m << endl;

//     SemiModule<Pattern,one_perm>::iterator  it;
//     foreach_item( it, SM.Q )
// 	{
// 	    cout << setw(4) << it->second << "   ";
// 	    print_bvector(it->first.first());
// 	    cout << setw(8) << it->first.second() << endl;
// 	}

    hash_set<bvector,hash<bvector>,equal_to<bvector> >  G;
    
    
    SemiModule<Pattern,one_perm>::iterator  it;
    foreach_item( it, SM.Q )
	G.insert( it->first.first() );
    
    cout << "*** " << G.size() << endl;
}



