//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:23:50 sutner> 
//
// :File:      parser.cc
// :Purpose:   
//---------------------------------------------------------------------------

#include  <iostream>
#include  <fstream>
#include  <strstream>
#include  <stack>

#include  "basic.h"
#include  "parser.h"
#include  "ptree.h"
#include  "symtab.h"
#include  "urlist.h"
#include  "useratom.h"
#include  "globop.h"

#define  YYSTYPE  UList
#include  "bparser.h"

//--------------------------------------------------------------------------------
// the reentrant parser

extern  int yyparse( void* );
static  char   symbuff[128];

//--------------------------------------------------------------------------------
// initialization and stream handling
//--------------------------------------------------------------------------------

int         Parser::parsercnt = 0;
Parser     *_einmal_;
SymTable   *_glob_op_tab_;


void  Parser::initialize() 
{
//    cerr << "*** Parser::initialize " << endl;
    _glob_op_tab_ = new SymTable(NULL);
    load_standard_operators( *_glob_op_tab_ );
    _einmal_ = new Parser;
}

void  Parser::finalize() 
{
    delete  _glob_op_tab_;
    delete  _einmal_;
}

//--------------------------------------------------------------------------------

// keep this in synch with operations in globOpTab for better performance
// really should be automatic
Parser::Parser(): 
    loud(LOUD), loud_0(LOUD), pflag(1), goflag(1), strmp(NULL), strmnm(NULL)
{
    if( !parsercnt++ ) Parser::initialize();

    toktab["if"]    = Token(IFT);
    toktab["fi"]    = Token(FIT);
    toktab["else"]  = Token(ELSET);
    toktab["while"] = Token(WHILET);
    toktab["whileread"] = Token(WHLRDT);
    toktab["defun"] = Token(DEFUNT);
    toktab["quit"]  = Token(STOPT);

    load_from_global( toktab );
}

Parser::~Parser() 
{
    if( !--parsercnt ) Parser::finalize();
}

    
void   Parser::detach()		// close last input file 
{
    if( strmp && *strmp != cin ) delete strmp;
}


bool   Parser::attach( const char *fname, const char *strbuff )
{
    detach();
  
    goflag = 1;

    if( !fname || strcmp(fname,"-")==0 || strcmp(fname,"cin")==0 )  // stdin
    {
	reset_loud();
	strmp  = ::new istream(cin.rdbuf());
	write_string( strmnm, "cin" );
	return 1;
    }
				// MathLink, expression eval, SILENT
    if( strcmp(fname,"string")==0 )        
    {
	set_loud( SILENT );
	strmp  = ::new istrstream(strbuff);
	write_string( strmnm, "string" );
	strmp->clear();
	return 0;
    }
				// ordinary file, always QUIET ???
    strmp = ::new ifstream(fname);
    write_string( strmnm, fname );
    if( !strmp )
    {
	cerr << "attach: cannot open input stream " << fname << endl;
	return 0;
    }
    strmp->clear();
    set_loud( QUIET );
    return  1;
}

//--------------------------------------------------------------------------------
//  scanner
//--------------------------------------------------------------------------------

//  special chars:  \n  %  (  )  *  ,  .  ;  <  =  >  [  ]  {  }

static const char lex_translate[] = {  
//            0     1     2     3     4     5     6     7     8     9  
/* 00 */      0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
/* 10 */   '\n',    0,    0,    0,    0,    0,    0,    0,    0,    0,
/* 20 */      0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
/* 30 */      0,    0,    0,    0,    0,    0,    0,  '%',    0,    0,
/* 40 */    '(',  ')',  '*',    0,  ',',    0,  '.',    0,    0,    0,
/* 50 */      0,    0,    0,    0,    0,    0,    0,    0,    0,  ';',
/* 60 */    '<',  '=',  '>',    0,    0,    0,    0,    0,    0,    0,
/* 70 */      0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
/* 80 */      0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
/* 90 */      0,  '[',    0,  ']',    0,    0,    0,    0,    0,    0,
/*100 */      0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
/*110 */      0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
/*120 */      0,    0,    0,  '{',    0,  '}',    0,    0,    0,    0,
/*130 */      0,    0,    0,    0,    0,    0,    0,    0,    0,    0
};

inline bool  is_special( int  cc ) { return (cc < 128) ? lex_translate[cc] : 0; }

//--------------------------------------------------------------------------------
// look_ahead currently only for '=='

int  Parser::look_ahead( int next, int yes, int no )

{
    int c = strmp->get();
    if( c == next ) return yes;
    strmp->unget();
    return no;
}


int  Parser::get_next_token( UList*& yylv )
{
    int   c;
    
    while( (c=strmp->get()) && ( c == ' ' || c == '\t' ) ); 

    if( c == '#' )
    {
	while( (c=strmp->get()) && c != '\n' );
	return  c;
    }
 
				// should depend on parse mode
    if( c == EOF ) return 0;

    if( is_special( c ) )
	switch( c )
	{
	case '=':  return look_ahead( '=', EQT, '=' );
	default:   return  c;
	}
    
    char  *q = symbuff;		// write into symbuff

    if( isdigit(c) || c == '-' )		// an integer
    {
	*q++ = c;		// copy minus or first digit
	while( (c=strmp->get()) && isdigit(c) )
	    *q++ = c;
	*q = '\0';
	strmp->unget();
	*yylv = read_int();
	return   INTT;
    }
    
    if( c == '\"' )		// a quoted string
    {
	while( (c=strmp->get()) && c != '\"' )
	    *q++ = c;
	*q = '\0';
	*yylv = new StringAtom( symbuff );
	return   STRT;
    }

				// a plain string
    *q++ = c;
    while( (c=strmp->get()) && isalnum(c) )
	*q++ = c;
    *q = '\0';
    strmp->unget();

    if( strcmp(symbuff,"quit") == 0 ) return 0;
    
				// unchecked UList<int>
    if( strcmp(symbuff,"IL") == 0 )
    {
	*yylv = read_ilist();
	return  LSTT;
    }
    
    TokenHTab::iterator  it;	// check for reserved word
    it = toktab.find(symbuff);
    if( it==toktab.end() )
    {
	*yylv = new StringAtom( symbuff );
	return  VART;
    }
    else
    {
	*yylv = new StringAtom( symbuff );
	return  it->second.tok;
    }

}

void  Parser::print_table()
{
    TokenHTab::iterator  it;
    for( it=toktab.begin(); it!=toktab.end(); ++it )
    {
	cerr << it->first << "\t" << (int)(it->second.tok); 
	cerr << "  " << (int)(it->second.ari) << endl;
    }
    cerr << endl;
}


//--------------------------------------------------------------------------------
//  parser interface
//--------------------------------------------------------------------------------

//  pre:   streams() has been filled
//         no environment needed
//  post:  will parse all streams

void   Parser::run()
{
    SymTable   env;
    while( !streams().empty() )
    {
	attach( streams().front().c_str() );
	streams().pop_front();
	parse( env );
    }
}


// parse current stream

UList  Parser::parse( SymTable& env )
{
    ParserControl   pc(this,&env);
    in_prompt();
    yyparse( (void*)&pc );	// reentrant
    return  c_val;
}


void    Parser::evaluate( const char *expr )
{
	SymTable  env;
	attach( "string", expr );
	parse( env );
}


void    Parser::in_prompt()  const 
{ 
    if( !is_quiet() ) cout << "\n> " << flush; 
}

void    Parser::out_prompt( UList x ) const
{ 
    if( is_silent() ) return;
    if( !is_quiet() ) cout << "  ";
    cout << x << endl;
}

//--------------------------------------------------------------------------------
// special input
//--------------------------------------------------------------------------------

// read a potentially long integer from symbuff
// test for small ints crude, does not permit negative gmp numbers

UList  Parser::read_int()
{
    PARSEMSSG("read_int");

    if( strlen(symbuff) >= 10  )
	return  new IntegerAtom( symbuff );

    if( symbuff[0] == '-' )
	return  new_int( -atoi( symbuff+1 ) );
    
    return  new_int( atoi(symbuff) );
}

// process an UList<int>
// performs no error checking, input must be properly formed
// scans  '[' list ']'

UList   Parser::read_ilist()
{
    stack<UList,vector<UList> >  ilistStack;
    UList tmp;
    ilistStack.push( nil );

    int  level = 0;
    char c;
    while( (c=strmp->get()) && ( c == ' ' || c == '\t' || c == '\n' ) ); 
    assert( c == '[' );
    
    do
    {
	while( (c=strmp->get()) && ( c == ' ' || c == '\t' || c == '\n' ) ); 

        switch( c )
        {
        case ',': 
            break;
                
        case  '(':
            ilistStack.push( nil );
            ++level;
            break;      

        case  ')':
            tmp = reverse( ilistStack.top() );
            ilistStack.pop();
            ilistStack.top() += tmp;
            --level;
            break;
      
        default:                // must be a plain unsigned integer
	    char  *bp = symbuff;
	    *bp++ = c;
	    while( (c=strmp->get()) && isdigit(c) )
		*bp++ = c;
	    *bp = '\0';
	    strmp->unget();
            ilistStack.top() += new_int( atoi(symbuff) ); 
            break;      
        }
    } while ( level>0 );
    
    while( (c=strmp->get()) && ( c == ' ' || c == '\t' || c == '\n' ) ); 
    assert( c == ']' );

    return  ilistStack.top().head();
}
    
//--------------------------------------------------------------------------------
// pathetic

UList  error(char *a, char *b=NULL)
{
    cerr << "error: " << a << " " << b << endl;
    return nil;
}

//--------------------------------------------------------------------------------


//  Local Variables: ***
//  eval: (setq case-fold-search nil) ***
//  eval: (setq case-replace nil) ***
//  End: ***
