//--------------------------------------------------------------------------------
//
// Time-stamp: <2003-11-24 18:29:02 sutner>
// 
// File:     recode.c
// Purpose:  Implements BEFAs and conversion of rational expressions to 
//           finite state machines.
//           
//--------------------------------------------------------------------------------

#include  "befa.h"


//--------------------------------------------------------------------------------

BEFA::~BEFA(){}


BEFA::BEFA( const BEFA& src ): _nstat(src._nstat), _tran(src._tran) {}


void  BEFA::add_transition( int p,int s,int q )
{
    _tran.push_back( p );
    _tran.push_back( s );
    _tran.push_back( q );
}

BEFA::BEFA( char *ww, char off )
{
    int ll = strlen(ww);
    for( int i=0; i<ll; i++ )
	add_transition( i, int(*ww++) - off, i+1 );
    _nstat = ll + 1;
}

BEFA::BEFA( int ): _nstat(0)	// currently only r=-1 used:  eps machine
{
    add_transition( 0, -1, 1 );
}


BEFA&  BEFA::operator+=( const BEFA& M )
{
    int n1 = numStates();
    int n2 = M.numStates();
    int t2 = M.numTransitions();

    int i, j;
    for( i=0, j=0; i<t2; i++, j+=3 )		// copy and shift second machine
	add_transition( M._tran[j] + n1, M._tran[j+1], M._tran[j+2] + n1 );
    
				// add two eps transitions
    add_transition( 0, -1, n1 );
    add_transition( n1-1, -1, n1+n2-1 );
    _nstat = n1 + n2;

    return  *this;
}

BEFA&  BEFA::operator*=( const BEFA& M )
{
    int n1 = numStates() - 1;	// note the -1
    int t2 = M.numTransitions();
   
    int i, j;
    for( i=0, j=0; i<t2; i++, j+=3 )		// copy and shift second machine
	add_transition( M._tran[j] + n1, M._tran[j+1], M._tran[j+2] + n1 );

    return  *this;
}


BEFA&   BEFA::star() 
{
    int n1 = numStates();
    int t1 = numTransitions();
   
    int i, j;

    for( i=0, j=0; i<t1; i++, j++ ) // shift machine
    {
	_tran[j]++;
	j += 2;
	_tran[j]++;
    }
				// add three eps transitions
    add_transition( 0, -1, 1 );
    add_transition( 1, -1, n1+1 );
    add_transition( n1, -1, 1 );
    _nstat = n1 + 2;

    return  *this;
}

// int   BEFA::convert( FA *fa ) const 
// {
//     int p,s,q, k=0;
//     int n = numStates();   

//     fa->set( n, 0, EPS_PROP, nil );

//     ivector::const_iterator  itr = _tran.begin();
   
//     fa->transitions().reserve( numTransitions() );
//     fa->transitions().resize( 0 );	                  // enable push_back 
   
//     for( int i=0; i<numTransitions(); i++ )        // copy transitions 
//     {
// 	p = *itr++;
// 	s = *itr++;  k = (s>k)?s:k;
// 	q = *itr++;
// 	fa->transitionsInsert(p,s,q);
//     }
   
//     fa->numSymbols() = k+1;   
//     fa->initial().initialize(n);
//     fa->final().initialize(n);
//     fa->initial().enter(0);
//     fa->final().enter(n-1);

//     return n;
// }


BEFA&  BEFA::operator=( const BEFA& src )
{
    if( &src != this )
    {
	_nstat = src._nstat;
	transitions() = src.transitions();
    }
    return *this;
}


// void  BEFA::Print() const 
// {
//    cout << numStates() << "  " << numTransitions() << endl;
//    cout << transitions() << endl;
// }

BEFA  operator+( const BEFA& A, const BEFA& B )
{
    BEFA  M = A;
    M += B;
    return M;
}

BEFA  operator*( const BEFA& A, const BEFA& B )
{
    BEFA  M = A;
    M *= B;
    return M;
}


ostream& operator<<( ostream& out, const BEFA& M )
{
    out << "#states: " << M._nstat << endl;
    copy( M._tran.begin(), M._tran.end(), ostream_iterator<int>(out," "));
    return out;
}

//----------------------------------------------------------------------
//eof


