//--------------------------------------------------------------------------------
//
// Time-stamp: <2003-11-24 18:30:21 sutner>
// 
// File:     smodule.cc
// Purpose:  Generating a semi module in pow(Q).
//           Used in power automaton construction.
//--------------------------------------------------------------------------------

#include  <iostream>
#include  <iomanip>

#include  "basic.h"
#include  "sgroup.h"

//--------------------------------------------------------------------------------

PSemiModuleGen::~PSemiModuleGen() {}

void    PSemiModuleGen::insert( const IntSet& st )
{
    bTrieP      leaf;
    btrie.probe_branch( st, leaf );
    el.push_back( leaf );
}

void  PSemiModuleGen::current_set( iterator itr, IntSet& st )
{
    st.sclear();
    int   ii = st.capacity() - 1;
    bTrieP btn = *itr; 
    while( btn->parent() )
    {
	if( btn->whichSon() )
	    st.insert(ii);
	ii--;
	btn = btn->parent();
    }
}


int    PSemiModuleGen::generate()
{
    int         s;
    IntSet      st(n);
    IntSet      nst(n);
    bTrieP      leaf;

    PSemiModuleGen::iterator  itr;
    ilist::iterator    it;
    IntSet::iterator   sit;
    
    // compute semimodule
    foreach_item( itr, el )
	{
	    current_set( itr, st );
    
	    for( s = 0; s < k; s++ )
	    {
		// compute next state set: nst = op(st,s)
		nst.sclear();
		foreach_item( sit, st )	// for each element p = sit() of st
		    foreach_item( it, op[ s*n + *sit ] )
		    nst.insert( *it );

		bool newset = !btrie.probe_branch( nst, leaf );

		if( newset )
		{
		    el.push_back( leaf );
		    del.push_back( size()-1 );
		}
		else
		    del.push_back( leaf->content() );
	    }
	}

    return  size();
}


//----------------------------------------------------------------------

