//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:30:44 sutner> 
//
// :File:      trace.cc
// :Purpose:   implementation for trace class
//---------------------------------------------------------------------------

#include "trace.h"


Trace::Trace( char *s ): n(n_cnt++), msg(NULL)
{
    if( s )
    {
	msg = new char[strlen(s)+1];
	strcpy( msg, s );
    }
    
    cu_cnt++; 
    t_cnt++; 
    sc_cnt++;

    if( fl_tr )
    {
	cerr << "--- constructor:  " << setw(3) << n << "  ";
	if( msg ) cerr << msg;
	cerr << endl;
    }

}
    

Trace::Trace( const Trace& src ): n(src.n), msg(NULL)
{
    if( src.msg )
    {
	msg = new char[strlen(src.msg)+1];
	strcpy( msg, src.msg );
    }
    
    cu_cnt++; 
    t_cnt++; 
    cc_cnt++;

    if( fl_tr )
    {
	cerr << "--- copy constr:  " << n << "  ";
	if( msg ) cerr << msg;
	cerr << endl;
    }
    
}
    

Trace::~Trace()
{ 
    cu_cnt--; 
    d_cnt++;

    if( fl_tr )
    {
	cerr << "--- destructor:   " << setw(3) << n << "  ";
	if( msg ) cerr << msg;
	cerr << endl;
    }
}

void  TraceClear()
{
    Trace::cu_cnt = 0; 
    Trace::t_cnt = 0; 
    Trace::sc_cnt = 0; 
    Trace::cc_cnt = 0; 
    Trace::as_cnt = 0; 
    Trace::d_cnt = 0; 
    Trace::n_cnt = 0; 
}

void  FullTraceOn()
{
    Trace::fl_tr = 1;
}

void  FullTraceOff()
{
    Trace::fl_tr = 0;
}

Trace&  Trace::operator=( const Trace& src )
{
    if( this==&src )
	return *this;
    as_cnt++;

    if( fl_tr )
    {
	cerr << "--- assignment:  " << n << "  ";
	if( msg ) cerr << msg;
	cerr << "  " << cu_cnt << endl; 
    }
    

    n = src.n;
    return *this;
}
    

void  TraceCensus( bool fc )
{ 
    if( fc || Trace::t_cnt )
    {
	cout << "--- census:       cu     tot      sc      ";
	cout << "cc      de      as\n            "; 
	cout << setw(8) << Trace::cu_cnt; 
	cout << setw(8) << Trace::t_cnt; 
	cout << setw(8) << Trace::sc_cnt; 
	cout << setw(8) << Trace::cc_cnt; 
	cout << setw(8) << Trace::d_cnt; 
	cout << setw(8) << Trace::as_cnt << endl; 
    }
}

ostream& operator<<( ostream& out, const Trace& x )
{ 
    out << "--- Trace: " << x.n; 
    if( x.msg ) 
	out << "  "  << x.msg;
    return out;
}



//----------------------------------------------------------------------
// need to allocate storage for all the static ints

int   Trace::cu_cnt = 0; 
int   Trace::t_cnt = 0; 
int   Trace::sc_cnt = 0; 
int   Trace::cc_cnt = 0; 
int   Trace::as_cnt = 0; 
int   Trace::d_cnt = 0; 
int   Trace::n_cnt = 0; 
bool  Trace::fl_tr = 0;


