//--------------------------------------------------------------------------------
//
// Time-stamp: <2003-11-24 18:31:25 sutner>
// 
// File:     trees.cc
// Purpose:  implementation file for binary and n-ary tries. 
//
//--------------------------------------------------------------------------------

#include  "basic.h"
#include  "stl-wrap.h"
#include  "pairs.h"
#include  "intset.h"
#include  "trees.h"

//----------------------------------------------------------------------
//  binary tries 
//----------------------------------------------------------------------

bTrieNode::~bTrieNode()			  // remove whole subtree
{
    delete lptr; 
    delete rptr; 
}

bTrie::~bTrie() { delete root; }


				// use aux for levels
bTrieP  bTrieNode::insert( int lr )
{
    if( lr )			// attach as right son
	return rptr = new bTrieNode( this, lr, aux+1 );
    else
	return lptr = new bTrieNode( this, lr, aux+1 );
}
   
// traverse from leaf to root, place 0/1 info into integer set

void   retrieve_bbranch( bTrieP btn, IntSet& st )
{
    int   i = st.capacity() - 1;
    while( btn->parent() )
    {
	if( btn->whichSon() )
	{
	    st.insert(i);
#ifdef DERRBUG
	    cout << "retrieve: " << i << "  " << st;
#endif
	}
	
	i--;
	btn = btn->parent();
    }
}

// if branch exists, return 1, owise insert it, return 0 
// pointer to leaf stored in second argument
// note: leaves store branch number, numbered 0 <= i < size()

int   bTrie::probe_branch( const IntSet& iS, bTrieP& leaf )
{
   size_t i = 0, found = 1;
   bTrieNode  *curr;
   
   curr = root;			  // walk from root along branch
   while( found && i<dpth )
   {
      if( iS.find(i) )	          // elm. i is in iS
      {
	 if (curr->rptr)
	   {curr = curr->rptr;i++;} 
	 else
	   found = 0;
      }
      else				  // elm. i not in iS
      {
	 if (curr->lptr)
	   {curr = curr->lptr;i++;} 
	 else
	   found = 0;
      }
   }

   if (found)			// branch exists
   {
      leaf = curr;
      return 1;
   }
   
   
   while ( i<dpth )			  // insert branch
     curr = curr->insert( iS.find(i++) );
   
				          // increment branch counter
   curr->aux = numbr++;			  // store branch number in leaf
   leaf = curr;
   return 0;
}

// same, but for *int type bit-vectors instead of IntSet

int   bTrie::probe_branch( int *ip, bTrieP& leaf )
{
   size_t i = 0, found = 1;
   bTrieNode  *curr;
   
   curr = root;			  // walk from root along branch
   while( found && i<dpth )
   {
      if( ip[i] )	          // elm. i is in iS
      {
	 if (curr->rptr != NULL)
	   {curr = curr->rptr;i++;} 
	 else
	   found = 0;
      }
      else				  // elm. i not in iS
      {
	 if (curr->lptr != NULL)
	   {curr = curr->lptr;i++;} 
	 else
	   found = 0;
      }
   }
   

   if (found)			// branch exists
   {
      leaf = curr;
      return 1;
   }
   
   
   while ( i<dpth )			  // insert branch
     curr = curr->insert( ip[i++] );
   
				          // increment branch counter
   curr->aux = numbr++;			  // store branch number in leaf
   leaf = curr;
   return 0;
}



//----------------------------------------------------------------------
// n-ary tries
//----------------------------------------------------------------------


nTrieNode::~nTrieNode()			  // remove whole subtree
{
    for( int i=0; i<width(); i++ )
	delete sons[i];
}

nTrie::~nTrie() { delete root; }


// static void retrieve_branch_kerim( const nTrieP& leaf, uvector& tt, IntSet& ss )
// {
//    assert( tt.size()==ss.capacity() );
   
//    ss.sclear();
//    nTrieP tp = leaf;

//    for( int i=ss.capacity()-1; i>=0; i-- )
//    {
//       if( tp->whichSon() )
// 	ss.insert(i);
//       tp = tp->parent();
//    }

//    retrieve_branch( tp, tt );

//    return;
// }


 				// return leaf--->tt.size() levels down
void   retrieve_branch( const nTrieP& leaf, uvector& tt )
{
   nTrieP  ptr = leaf;
   int n = tt.size();
   
   for( int i=n-1; i>=0; i-- )
   {
      tt[i] = ptr->whichSon();
      ptr   = ptr->parent();
   }
}

				// write branch into array
void  retrieve_branch( const nTrieP& leaf, uvector::iterator& it, int n )
{
   nTrieP  ptr = leaf;
   
   for( int i=0; i<n; i++ )
   {
      *it++ = ptr->whichSon();
      ptr   = ptr->parent();
   }
}

		// same, but write block in reverse order into large array
void   retrieve_branch_reverse( const nTrieP& leaf, uvector::iterator& it, int n )
{
   nTrieP  ptr = leaf;

   it += n;			// position iterator
   for( int i=0; i<n; i++ )
   {
      *it-- = ptr->whichSon();
      ptr   = ptr->parent();
   }
   it += n;			// reposition iterator
}


//----------------------------------------------------------------------

				// return 1 if tt is already there,
				// otherwise insert and return 0

bool   nTrie::probe_branch( uvector::iterator tt, nTrieP& leaf )
{
    size_t  i = 0; 
    bool    found = 1;
    nTrieP  next = NULL, curr = root;
   
    // walk from root along branch
    while( found && i < dpth )
	if( (next = (*curr)[*tt]) )	          // node exists
	{	
	    curr = next;
	    ++i;
	    ++tt;
	}
	else
	    found = 0;
	
    if (found)			  // branch exists
    {
	leaf = next;
	return 1;   
    }
   
   
    while ( i < dpth )			  // insert branch
    {   
	curr = curr->insert( *tt++ );
	i++;
    }   
    // increment branch counter
    curr->aux = numbr++;			  // store branch number in leaf 
    leaf = curr;
    return 0;
}

//----------------------------------------------------------------------
