//---------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:32:25 sutner> 
//
//  :File:      urlist.cc
//  :Purpose:   implementation file for UList
//---------------------------------------------------------------------------

#include  <alloc.h>
#include  <algorithm>
#include  "urlist.h"
#include  "functoid.h"
#include  "fsm.h"
#include  "useratom.h"

//--------------------------------------------------------------------------------
// initialization

int   UListInitializer::ulinitcnt = 0;
bool  UListInitializer::showtime = 0;
static  UListInitializer  _ulinit_;

NilAtom   *_nilatomptr_;


//--------------------------------------------------------------------------------

void  UList::initialize() { _nilatomptr_ = new NilAtom; }
void  UList::finalize()   { delete _nilatomptr_; }

//inlining these seems to make no difference
 

// only makes sense for int,string-lists    
bool  UList::operator==( CRef rhs )
{
    if( ap == rhs.ap )
	return 1; 

    if( atom_type() != rhs.atom_type() )
	return 0;
	
    if( atom_type() == CONS )
	return  ( head() == rhs.head() && tail() == rhs.tail() );

    if( atom_type() == INT )
	return ivalue(*this) == ivalue(rhs);
    
    if( atom_type() == STR )
	return svalue(*this) == svalue(rhs);
	
    if( atom_type() == DFAA )	// for isomorphism testing
	return dfavalue(*this) == dfavalue(rhs);
	
    return 0;			// means nothing
}

// length-lex type ordering, only makes sense for int,string-lists    
bool  UList::operator<( CRef rhs )
{
    if( !ap ) return rhs.ap;

    if( atom_type() != rhs.atom_type() )
	return atom_type() < rhs.atom_type();
    
    if( atom_type() == CONS )
    {
	UList  L = *this;
	UList  R = rhs;
	UList  l, r;
	int    d = 0;
	
	while( L ) 
	{
	    if( !R ) return 0;
	    L >> l;
	    R >> r;
	    if( !d )
	    {
		if( l < r ) d = -1;
		if( r < l ) d =  1;
	    }
	}
	if( R )
	    return 1;
	else
	    return (d == -1);
    }
    
    if( atom_type() == INT )
	return ivalue(*this) < ivalue(rhs);

    if( atom_type() == STR )
    {
	string s1(svalue(*this));
	string s2(svalue(rhs));
	if( s1.size() < s2.size() ) return 1;
	if( s1.size() == s2.size() ) 
	    return s1 < s2; 
    }
    
    return 0;			// means nothing
}

ostream&  operator<<( ostream& os, CRef L ) 
{ 
    L.print(os); 
    return os;
}


void  ConsAtom::print( ostream& os ) const 
{
    os << "(";
    car.print(os);
    cdr.sub_print(os);
    os << ")";
}

void  ConsAtom::sub_print( ostream& os ) const 
{
    os << ", ";
    car.print(os);
    cdr.sub_print(os);
}




//-------------------------------------------------------------------------------- 

StringAtom::StringAtom( const char* ss ): str(NULL)
{
    write_string( str, ss );
}
StringAtom::StringAtom( const string& ss ): str(NULL)
{
    write_string( str, ss.c_str() );
}
StringAtom::~StringAtom() { delete str; }
AType         StringAtom::atom_type() const { return STR; }
char  *StringAtom::value() { return str; }
const char *StringAtom::value() const  { return str; }
const StringAtom  *StringAtom::clone() const { return new StringAtom(str); }
void  StringAtom::print( ostream& os ) const { os << str; }
void  StringAtom::sub_print( ostream& os ) const { os << ", " << str; }

UList StringAtom::eval( SymTable& ) { return copy(); }

//--------------------------------------------------------------------------------

ostream&  operator<<( ostream& os, AType tt )
{
    char *names[] = {"NIL","FUNC","CONS","STRING","INT"};
    os << names[tt];
    return os;
}

int refcount( CRef x )
{
    RefAtom  *rp = dynamic_cast<RefAtom*>(x.ap);
    return  ( rp ) ?  rp->rc : -1;
}

int arity( CRef x )
{
    Function   *fp = dynamic_cast<Function*>(x.ap);
    return  ( fp ) ?  fp->arity() : -1;
}


//--------------------------------------------------------------------------------


