//--------------------------------------------------------------------------------
//
//  Time-stamp: <2003-11-24 18:33:01 sutner> 
//
//  :File:     useratom.cc
// 
//  :Purpose:  implementation for user-defined atoms, all ref counted
//             kill templated version ???
//--------------------------------------------------------------------------------

#include  "basic.h"
#include  "urlist.h"
#include  "useratom.h"
#include  "listutil.h"

//--------------------------------------------------------------------------------
// User atoms
//--------------------------------------------------------------------------------

//--------------------------------------------------------------------------------
// general layout for user atom, assume:  XXX  dat;
// constructor: may need special
// destructor:  only needed for special deallocation
// destroy():   only needed for special deallocation
// 
//     XXXAtom::XXXAtom() {}
//     XXXAtom::XXXAtom( const XXX& src ): dat(src.dat) {}
//     XXXAtom::~XXXAtom() {}
//     XXXAtom        *XXXAtom::destroy();
//     const XXXAtom  *XXXAtom::clone() const { return new XXXAtom(dat); }
//     AType  XXXAtom::atom_type() const { return ???; }
//     
//     void        XXXAtom::print( ostream& os ) const { os << dat; }
//     void        XXXAtom::sub_print( ostream& os ) const { os << dat; }
//     XXX&        XXXAtom::value() { return dat; }
//     const XXX&  XXXAtom::value() const  { return dat; }


IntArrAtom::IntArrAtom() {}
IntArrAtom::IntArrAtom( const ivector& src ): dat(src) {}
IntArrAtom::IntArrAtom( const uvector& src ): dat(src.size()) 
{
    for( size_t  i = 0; i < src.size(); ++i )
	dat[i] = int( src[i] );
}
IntArrAtom::~IntArrAtom() {}
// IntArrAtom        *IntArrAtom::destroy();
const IntArrAtom  *IntArrAtom::clone() const { return new IntArrAtom(dat); }
AType  IntArrAtom::atom_type() const { return IARR; }
void        IntArrAtom::print( ostream& os ) const { os << dat; }
void        IntArrAtom::sub_print( ostream& os ) const { os << dat; }
ivector&        IntArrAtom::value() { return dat; }
const ivector&  IntArrAtom::value() const  { return dat; }
UList       IntArrAtom::eval( SymTable& ) { return clone(); }

IntegerAtom::IntegerAtom() {}
IntegerAtom::IntegerAtom( const char *st ): dat(st) {}
IntegerAtom::IntegerAtom( long int src ): dat(src) {}
IntegerAtom::IntegerAtom( const Integer& src ): dat(src) {}
IntegerAtom::IntegerAtom( const IntegerAtom& src ): dat(src.dat) {}
IntegerAtom::~IntegerAtom() {}
//IntegerAtom        *IntegerAtom::destroy();
const IntegerAtom  *IntegerAtom::clone() const { return new IntegerAtom(dat); }
AType  IntegerAtom::atom_type() const { return INTL; }
void        IntegerAtom::print( ostream& os ) const { os << dat; }
void        IntegerAtom::sub_print( ostream& os ) const { os << dat; }
Integer&        IntegerAtom::value() { return dat; }
const Integer&  IntegerAtom::value() const  { return dat; }
UList       IntegerAtom::eval( SymTable& ) { return clone(); }

SAAtom::SAAtom() {}
SAAtom::SAAtom( int n, int k ): dat(n,k) {}
SAAtom::SAAtom( const SA& src ): dat(src) {}
SAAtom::~SAAtom() {}
// SAAtom        *SAAtom::destroy();
const SAAtom  *SAAtom::clone() const { return new SAAtom(dat); }
AType  SAAtom::atom_type() const { return SAA; }
void        SAAtom::print( ostream& os ) const { os << dat; }
void        SAAtom::sub_print( ostream& os ) const { os << dat; }
SA&        SAAtom::value() { return dat; }
const SA&  SAAtom::value() const  { return dat; }
UList       SAAtom::eval( SymTable& ) { return copy(); }

FAAtom::FAAtom() {}
FAAtom::FAAtom( int n, int k ): dat(n,k) {}
FAAtom::FAAtom( const FA& src ): dat(src) {}
FAAtom::~FAAtom() {}
// FAAtom        *FAAtom::destroy();
const FAAtom  *FAAtom::clone() const { return new FAAtom(dat); }
AType  FAAtom::atom_type() const { return FAA; }
void        FAAtom::print( ostream& os ) const { os << dat; }
void        FAAtom::sub_print( ostream& os ) const { os << dat; }
FA&        FAAtom::value() { return dat; }
const FA&  FAAtom::value() const  { return dat; }
UList       FAAtom::eval( SymTable& ) { return copy(); }

DFAAtom::DFAAtom() {}
DFAAtom::DFAAtom( int n, int k ): dat(n,k) {}
DFAAtom::DFAAtom( const DFA& src ): dat(src) {}
DFAAtom::~DFAAtom() {}
// DFAAtom        *DFAAtom::destroy();
const DFAAtom  *DFAAtom::clone() const { return new DFAAtom(dat); }
AType  DFAAtom::atom_type() const { return DFAA; }
void        DFAAtom::print( ostream& os ) const { os << dat; }
void        DFAAtom::sub_print( ostream& os ) const { os << dat; }
DFA&        DFAAtom::value() { return dat; }
const DFA&  DFAAtom::value() const  { return dat; }
UList       DFAAtom::eval( SymTable& ) { return copy(); }

CAAtom::CAAtom() {}
CAAtom::CAAtom( const CA& src ): dat(src) {}
CAAtom::~CAAtom() {}
// CAAtom        *CAAtom::destroy();
const CAAtom  *CAAtom::clone() const { return new CAAtom(dat); }
AType  CAAtom::atom_type() const { return CAA; }
void        CAAtom::print( ostream& os ) const { os << dat; }
void        CAAtom::sub_print( ostream& os ) const { os << dat; }
CA&        CAAtom::value() { return dat; }
const CA&  CAAtom::value() const  { return dat; }
UList       CAAtom::eval( SymTable& ) { return copy(); }

SGAtom::SGAtom() {}
SGAtom::SGAtom( const SGroup& src ): dat(src) {}
SGAtom::~SGAtom() {}
// SGAtom        *SGAtom::destroy();
const SGAtom  *SGAtom::clone() const { return new SGAtom(dat); }
AType  SGAtom::atom_type() const { return SGA; }
void        SGAtom::print( ostream& os ) const { os << dat; }
void        SGAtom::sub_print( ostream& os ) const { os << dat; }
SGroup&        SGAtom::value() { return dat; }
const SGroup&  SGAtom::value() const  { return dat; }
UList       SGAtom::eval( SymTable& ) { return copy(); }

GraphAtom::GraphAtom(): dat() {}
GraphAtom::GraphAtom( const Graph& src ): dat(src) {}
GraphAtom::GraphAtom( int n, int e ): dat(n,e) {}
GraphAtom::~GraphAtom() {}
// GraphAtom        *GraphAtom::destroy();
const GraphAtom  *GraphAtom::clone() const { return new GraphAtom(dat); }
AType  GraphAtom::atom_type() const { return GRA; }
void        GraphAtom::print( ostream& os ) const { os << dat; }
void        GraphAtom::sub_print( ostream& os ) const { os << dat; }
Graph&        GraphAtom::value() { return dat; }
const Graph&  GraphAtom::value() const  { return dat; }
UList       GraphAtom::eval( SymTable& ) { return copy(); }

//--------------------------------------------------------------------------------
// eof





