(* ------------------------------------------------------------------------- *)
(* 
*  Time-stamp: <2003-12-11 19:31:00 sutner>
*
*  File:     cellaut.m
*  Purpose:  some cellular automata macros 
*  Author:   K. Sutner, sutner@cs.cmu.edu
*
*)


(* ------------------------------------------------------------------------- *)
(* basic CAs *)
(* ------------------------------------------------------------------------- *)

ClearAll[RightShiftCA, LeftShiftCA,IdentityCA]

RightShiftCA[k_] := FunctionToCA[ Function[{y, z}, z], 2, -k];

LeftShiftCA[k_] := FunctionToCA[ Function[{y, z}, y], 2, -k];

IdentityCA[ w_Integer, k_Integer:-2 ] := 
With[ {kk = Abs[k],
       wl = Ceiling[(w - 1)/2],
       wr = Floor[(w - 1)/2] },
       CA[w, k, FromDigits[ Flatten@Table[i,{kk^wr},{i,kk,1,-1},{kk^wl}]-1, kk]]  
];

(* ------------------------------------------------------------------------- *)
(* combining CAs *)
(* ------------------------------------------------------------------------- *)

ClearAll[CombineCA]

CombineCA[ C1_CA, C2_CA] := 
Module[ {f1, f2, ff, k1, k2},
        k1 = AlphabetSize[C1];
        k2 = AlphabetSize[C2];
       	LocalMapCA[ f1, C1 ];
       	LocalMapCA[ f2, C2 ];
       	ff[xx__] := k1  f2 @@ Quotient[{xx}, k1] +  f1 @@ Mod[{xx}, k1 ];
       	FunctionToCA[ff, WidthCA[C1], - k1 k2]
]   /; WidthCA[C1] == WidthCA[C2];

CombineCA[C1_CA, C2_CA, gg_ ] :=
Module[{f1, f2, k, w},
        k = AlphabetSize[C1];
        w = WidthCA[C1];
        LocalMapCA[f1, C1];
        LocalMapCA[f2, C2];
        FunctionToCA[  gg[f1[##], f2[##]] &, WidthCA[C1], -k ]
        
] /; AlphabetSize[C1] == AlphabetSize[C2] && WidthCA[C1] == WidthCA[C2];

ClearAll[ToFredkinCA]

ToFredkinCA[ C_CA ] := 
Module[ {f, ff, k = AlphabetSize[C], w = WidthCA[C]},
    	LocalMapCA[ f, C ];
    	ff[xx__] := With[ { zz = Mod[{xx}, k ], ci = Ceiling[w/2] },
			  k zz[[ci]] + Mod[ f @@ zz + Quotient[{xx}[[ci]], k], k] ];
    	FunctionToCA[ff, w, -k^2]
];

(* ------------------------------------------------------------------------- *)

ecasurj = {30, 45, 75, 86, 89, 101, 106, 120, 135, 149, 153, 154, 
165, 166, 169, 180, 210, 225};

ecaopen = {60, 90, 102, 105, 150, 153, 165, 195};

ecainj = {15, 51, 85, 170, 204, 240};

Null

(* ------------------------------------------------------------------------- *)
(* eof *)
