(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    808462,      34252]*)
(*NotebookOutlinePosition[   1254103,      45899]*)
(*  CellTagsIndexPosition[   1237037,      45398]*)
(*WindowFrame->Normal*)



Notebook[{
Cell["$alphabet", "ObjectName",
  CellTags->"$alphabet"],

Cell["SysVariable", "Highlight",
  CellTags->"$alphabet"],

Cell[TextData[{
  StyleBox["$alphabet", "MR"],
  "  is the default alphabet used in commands that require an alphabet as \
part of their input. "
}], "Usage",
  CellTags->"$alphabet"],

Cell[TextData[{
  "The default value is 2, which represents standard alphabet ",
  Cell[BoxData[
      \(TraditionalForm\`{a, b}\)]],
  ". "
}], "Notes",
  CellTags->"$alphabet"],

Cell["\<\
Can be changed by user, but note that sizes of languages and \
machines may increase drastically.\
\>", "Notes",
  CellTags->"$alphabet"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$alphabet"],

Cell["We can change the default locally.", "ExampleText",
  CellTags->"$alphabet"],

Cell["Words[-2]", "Input",
  CellLabel->"In[57]:=",
  CellTags->"$alphabet"],

Cell[BoxData[
    \(Block[{$alphabet = 3}, Words[\(-2\)]]\)], "Input",
  CellLabel->"In[58]:=",
  CellTags->"$alphabet"],

Cell[BoxData[
    \(Block[{$alphabet = {"\<x\>", "\<y\>"}}, Words[\(-2\)]]\)], "Input",
  CellLabel->"In[59]:=",
  CellTags->"$alphabet"],

Cell["AlgebraicForm", "ObjectName",
  CellTags->"AlgebraicForm"],

Cell["Function", "Highlight",
  CellTags->"AlgebraicForm"],

Cell[TextData[{
  StyleBox["AlgebraicForm[w]", "MR"],
  "  writes word ",
  StyleBox["w", "MR"],
  " in algebraic form using exponents for runs of a symbol."
}], "Usage",
  CellTags->"AlgebraicForm"],

Cell[TextData[{
  " See also ",
  ButtonBox["PrintEquations",
    ButtonData:>"PrintEquations",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"AlgebraicForm"],

Cell["Further Examples", "ExampleSection",
  CellTags->"AlgebraicForm"],

Cell["\<\
On occasion it is preferable to write words in algebraic \
notation:\
\>", "ExampleText",
  CellTags->"AlgebraicForm"],

Cell[BoxData[
    \(AlgebraicForm[aaabbbbbbbcccccc]\)], "Input",
  CellLabel->"In[178]:=",
  CellTags->"AlgebraicForm"],

Cell[BoxData[
    \(AlgebraicForm[\[Epsilon]]\)], "Input",
  CellLabel->"In[179]:=",
  CellTags->"AlgebraicForm"],

Cell["Alphabet", "ObjectName",
  CellTags->"Alphabet"],

Cell["Function", "Highlight",
  CellTags->"Alphabet"],

Cell[TextData[{
  StyleBox["Alphabet[M]", "MR"],
  "  returns the proper alphabet of machine ",
  StyleBox["M", "MR"],
  "."
}], "Usage",
  CellTags->"Alphabet"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "True"},
          {"Normalize", "\[Rule]", "0", "1"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"Alphabet"],

Cell[TextData[{
  "Option  ",
  StyleBox["Full->True", "MR"],
  "  returns a pair  ",
  StyleBox["{k,K}", "MR"],
  "  where ",
  StyleBox["k", "MR"],
  " is the size of the alphabet ",
  StyleBox["K", "MR"],
  "."
}], "Notes",
  CellTags->"Alphabet"],

Cell[TextData[{
  "Option  ",
  StyleBox["Normalize->1", "MR"],
  "  causes a pair  ",
  StyleBox["{k,K}", "MR"],
  "  where ",
  StyleBox["k", "MR"],
  " is the size of the alphabet ",
  StyleBox["K", "MR"],
  " suitable for constructing a new machine."
}], "Notes",
  CellTags->"Alphabet"],

Cell[TextData[{
  "Has attribute   ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"Alphabet"],

Cell[TextData[{
  StyleBox["M", "MR"],
  "  is a finite state machine or a cellular automaton."
}], "Notes",
  CellTags->"Alphabet"],

Cell["Also applies to words and lists of words.", "Notes",
  CellTags->"Alphabet"],

Cell[TextData[{
  " See also ",
  ButtonBox["AlphabetQ",
    ButtonData:>"AlphabetQ",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["AlphabetSize",
    ButtonData:>"AlphabetSize",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["MakeAlphabet",
    ButtonData:>"MakeAlphabet",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Alphabet"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Alphabet"],

Cell["\<\
The underlying alphabet of a machine. Note that we use the \
abbreviated specification.\
\>", "ExampleText",
  CellTags->"Alphabet"],

Cell[BoxData[{
    \(m\  = \ IthSymbolFA[a, \(-5\), 3]\), "\n", 
    \(Alphabet[m]\)}], "Input",
  CellLabel->"In[658]:=",
  CellTags->"Alphabet"],

Cell["This time the alphabet is specified explicitely.", "ExampleText",
  CellTags->"Alphabet"],

Cell[BoxData[{
    \(m\  = \ IthSymbolFA["\<r\>", \(-5\), {"\<r\>", "\<s\>"}]\), "\n", 
    \(Alphabet[m, Full \[Rule] True]\)}], "Input",
  CellLabel->"In[662]:=",
  CellTags->"Alphabet"],

Cell["The alphabet of a word.", "ExampleText",
  CellTags->"Alphabet"],

Cell["Alphabet[\"hello\"]", "Input",
  CellLabel->"In[664]:=",
  CellTags->"Alphabet"],

Cell["Alphabetic", "ObjectName",
  CellTags->"Alphabetic"],

Cell["Option", "Highlight",
  CellTags->"Alphabetic"],

Cell[TextData[{
  StyleBox["Alphabetic", "MR"],
  "  is an option for ",
  StyleBox["FunctionToSA", "MR"],
  " and ",
  StyleBox["FunctionToDFA", "MR"],
  "."
}], "Usage",
  CellTags->"Alphabetic"],

Cell[TextData[{
  "Option ",
  StyleBox["Alphabetic->False", "MR"],
  StyleBox[" uses index notation for letters of the alphabet rather than \
characters.", "Text"]
}], "Notes",
  CellTags->"Alphabetic"],

Cell[TextData[{
  "See also ",
  ButtonBox["FunctionToSA",
    ButtonData:>"FunctionToSA",
    ButtonStyle->"AddOnsLink"],
  " , ",
  ButtonBox["FunctionToDFA",
    ButtonData:>"FunctionToDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Alphabetic"],

Cell["AlphabetQ", "ObjectName",
  CellTags->"AlphabetQ"],

Cell["Function", "Highlight",
  CellTags->"AlphabetQ"],

Cell[TextData[{
  StyleBox["AlphabetQ[K]", "MR"],
  "  tests whether ",
  StyleBox["K", "MR"],
  " represents an alphabet."
}], "Usage",
  CellTags->"AlphabetQ"],

Cell[TextData[{
  StyleBox["K", "MR"],
  "  can be a non-zero integer or a list of symbols or strings."
}], "Notes",
  CellTags->"AlphabetQ"],

Cell["\<\
Lists must either contain strings or symbols (make sure to clear \
the symbols before using them as letters).\
\>", "Notes",
  CellTags->"AlphabetQ"],

Cell[TextData[{
  "Positive integers represent lower-case letter alphabets starting at ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  ".  E.g., 3 represents ",
  Cell[BoxData[
      \(TraditionalForm\`{a, b, c}\)]],
  "."
}], "Notes",
  CellTags->"AlphabetQ"],

Cell[TextData[{
  "Negative integers represent digit alphabets starting at ",
  Cell[BoxData[
      \(TraditionalForm\`0\)]],
  ".  E.g., ",
  Cell[BoxData[
      \(TraditionalForm\`\(-2\)\)]],
  " represents ",
  Cell[BoxData[
      \(TraditionalForm\`{0, 1}\)]],
  "."
}], "Notes",
  CellTags->"AlphabetQ"],

Cell[TextData[{
  " See also ",
  ButtonBox["Alphabet",
    ButtonData:>"Alphabet",
    ButtonStyle->"AddOnsLink"],
  " , ",
  ButtonBox["AlphabetSize",
    ButtonData:>"AlphabetSize",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["MakeAlphabet",
    ButtonData:>"MakeAlphabet",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"AlphabetQ"],

Cell["Further Examples", "ExampleSection",
  CellTags->"AlphabetQ"],

Cell["Abbreviated alphabets.", "ExampleText",
  CellTags->"AlphabetQ"],

Cell[BoxData[{
    \(AlphabetQ[5]\), "\n", 
    \(AlphabetQ[\(-2\)]\)}], "Input",
  CellLabel->"In[1379]:=",
  CellTags->"AlphabetQ"],

Cell["This time the alphabet is specified explicitely.", "ExampleText",
  CellTags->"AlphabetQ"],

Cell[BoxData[{
    \(AlphabetQ[{a, b}]\), "\n", 
    \(AlphabetQ[{"\<a\>", "\<b\>"}]\)}], "Input",
  CellLabel->"In[1381]:=",
  CellTags->"AlphabetQ"],

Cell["Some non-alphabets.", "ExampleText",
  CellTags->"AlphabetQ"],

Cell["\<\
AlphabetQ[0]
AlphabetQ[{1,\"2\"}]\
\>", "Input",
  CellLabel->"In[1383]:=",
  CellTags->"AlphabetQ"],

Cell["AlphabetSize", "ObjectName",
  CellTags->"AlphabetSize"],

Cell["Function", "Highlight",
  CellTags->"AlphabetSize"],

Cell[TextData[{
  StyleBox["AlphabetSize[K]", "MR"],
  "  returns the size of the alphabet  ",
  StyleBox["K", "MR"],
  ". "
}], "Usage",
  CellTags->"AlphabetSize"],

Cell["Also applicable to machines.", "Notes",
  CellTags->"AlphabetSize"],

Cell[TextData[{
  "See also ",
  ButtonBox["Alphabet",
    ButtonData:>"Alphabet",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["AlphabetQ",
    ButtonData:>"AlphabetQ",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["MakeAlphabet",
    ButtonData:>"MakeAlphabet",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"AlphabetSize"],

Cell["Further Examples", "ExampleSection",
  CellTags->"AlphabetSize"],

Cell["The size of the alphabet of a machine.", "ExampleText",
  CellTags->"AlphabetSize"],

Cell[TextData[{
  "m = IthSymbolFA[a,-5,{a,b,c}];",
  "\n",
  Cell[BoxData[
      \(AlphabetSize[m]\)], "Input",
    CellLabel->"In[127]:="]
}], "Input",
  CellLabel->"In[130]:=",
  CellTags->"AlphabetSize"],

Cell["Some canonical alphabets.", "ExampleText",
  CellTags->"AlphabetSize"],

Cell[BoxData[
    \(AlphabetSize[{a, b, c}]\)], "Input",
  CellLabel->"In[132]:=",
  CellTags->"AlphabetSize"],

Cell[BoxData[
    \(AlphabetSize[\(-5\)]\)], "Input",
  CellLabel->"In[133]:=",
  CellTags->"AlphabetSize"],

Cell["Concatenate", "ObjectName",
  CellTags->"Concatenate"],

Cell["Function", "Highlight",
  CellTags->"Concatenate"],

Cell[TextData[{
  StyleBox["Concatenate[u,v]", "MR"],
  "  concatenates words ",
  StyleBox["u", "MR"],
  " and ",
  StyleBox["v", "MR"],
  ". "
}], "Usage",
  CellTags->"Concatenate"],

Cell[TextData[{
  StyleBox["Concatenate", "MR"],
  " is polyadic. "
}], "Notes",
  CellTags->"Concatenate"],

Cell[TextData[{
  StyleBox["Concatenate", "MR"],
  " behaves properly with respect to epsilon and symbols, but always returns \
a string."
}], "Notes",
  CellTags->"Concatenate"],

Cell[TextData[{
  StyleBox["Concatenate", "MR"],
  " distributes over lists of words."
}], "Notes",
  CellTags->"Concatenate"],

Cell[TextData[{
  "See also ",
  ButtonBox["Eps",
    ButtonData:>"Eps",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["WordQ",
    ButtonData:>"WordQ",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Concatenate"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Concatenate"],

Cell["\<\
Concatenation of words. Note that the words are entered as symbols, \
not strings. The result is returned in string form.\
\>", "ExampleText",
  CellTags->"Concatenate"],

Cell[BoxData[{
    \(Concatenate[aa, bb]\), "\n", 
    \(FullForm[%]\)}], "Input",
  CellLabel->"In[221]:=",
  CellTags->"Concatenate"],

Cell["\<\
Concatenation behaves properly with respect to \[Epsilon] and \
lists.\
\>", "ExampleText",
  CellTags->"Concatenate"],

Cell[BoxData[
    \(Concatenate[aa, \[Epsilon]]\)], "Input",
  CellLabel->"In[223]:=",
  CellTags->"Concatenate"],

Cell[BoxData[
    \(Concatenate[aa, {a, b, c}]\)], "Input",
  CellLabel->"In[224]:=",
  CellTags->"Concatenate"],

Cell[BoxData[
    \(Concatenate[{a, b, c}, {a, b, c}, {a, b, c}]\)], "Input",
  CellLabel->"In[225]:=",
  CellTags->"Concatenate"],

Cell["Eps", "ObjectName",
  CellTags->"Eps"],

Cell["Constructor", "Highlight",
  CellTags->"Eps"],

Cell[TextData[{
  StyleBox["Eps", "MR"],
  "  is used to denote the empty string. "
}], "Usage",
  CellTags->"Eps"],

Cell[TextData[{
  "As a regular expression it denotes the singleton set ",
  Cell[BoxData[
      \(TraditionalForm\`{\[Epsilon]}\)]],
  "."
}], "Notes",
  CellTags->"Eps"],

Cell[TextData[{
  "The system function  ",
  ButtonBox["ToString",
    ButtonStyle->"RefGuideLink"],
  "  is redefined to treat ",
  StyleBox["Eps", "MR"],
  " and  ",
  StyleBox["\[Epsilon]", "MR"],
  "  properly:   ",
  StyleBox["ToString[Eps] \[Rule] \"\".", "MR"]
}], "Notes",
  CellTags->"Eps"],

Cell[TextData[{
  "The empty word can be represented as ",
  StyleBox["Eps", "MR"],
  ", ",
  StyleBox["\"Eps\"", "MR"],
  ", ",
  StyleBox["\"\"", "MR"],
  ", ",
  StyleBox["\[Epsilon]", "MR"],
  ".  For obvious reasons the third form is discouraged."
}], "Notes",
  CellTags->"Eps"],

Cell[TextData[{
  "See ",
  ButtonBox["ToEps",
    ButtonData:>"ToEps",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Eps"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Eps"],

Cell[TextData[{
  "The empty word is denoted by the symbol ",
  StyleBox["Eps", "MR"],
  " which prints as ",
  StyleBox["\[Epsilon]", "MR"],
  "  in standard formatting mode."
}], "ExampleText",
  CellTags->"Eps"],

Cell[BoxData[
    \({Eps, "\<Eps\>", "\<\>"}\)], "Input",
  CellLabel->"In[267]:=",
  CellTags->"Eps"],

Cell[TextData[{
  "The built-in ",
  ButtonBox["ToString",
    ButtonStyle->"RefGuideLink"],
  " is modified to be compatible with ",
  StyleBox["Eps", "MR"],
  ".  To make the effect visible we have to temporarily suspend formatting."
}], "ExampleText",
  CellTags->"Eps"],

Cell[BoxData[{
    \(FormatAutomata[None]\), "\n", 
    \(ToString[Eps]\), "\n", 
    \(%\  // \ FullForm\)}], "Input",
  CellLabel->"In[262]:=",
  CellTags->"Eps"],

Cell[BoxData[
    \(FormatAutomata[Automatic]\)], "Input",
  CellLabel->"In[265]:=",
  CellTags->"Eps"],

Cell["FromEps", "ObjectName",
  CellTags->"FromEps"],

Cell["Function", "Highlight",
  CellTags->"FromEps"],

Cell[TextData[{
  StyleBox["FromEps[exp]", "MR"],
  "  replaces all occurences of ",
  StyleBox["Eps", "MR"],
  " (as symbol and as string) in ",
  StyleBox["exp", "MR"],
  " by the empty string."
}], "Usage",
  CellTags->"FromEps"],

Cell["Used internally as a clean up before further processing.", "Notes",
  CellTags->"FromEps"],

Cell[TextData[{
  "See also ",
  ButtonBox["Eps",
    ButtonData:>"Eps",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToEps",
    ButtonData:>"ToEps",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"FromEps"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FromEps"],

Cell["All versions of epsilon are recognized. ", "ExampleText",
  CellTags->"FromEps"],

Cell[BoxData[{
    \(FormatAutomata[None]\), "\n", 
    \(\({Eps, "\<Eps\>", "\<\>", \[Epsilon]}\  // \ FromEps\)\ \  // \ 
      FullForm\), "\n", 
    \(FormatAutomata[Automatic]\)}], "Input",
  CellLabel->"In[272]:=",
  CellTags->"FromEps"],

Cell["FromFrontier", "ObjectName",
  CellTags->"FromFrontier"],

Cell["Function", "Highlight",
  CellTags->"FromFrontier"],

Cell[TextData[{
  StyleBox["FromFrontier[W]", "MR"],
  "  computes the set of all proper prefixes of the finite language ",
  StyleBox["W", "MR"],
  ". "
}], "Usage",
  CellTags->"FromFrontier"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"FromFrontier"],

Cell[TextData[{
  "Option  ",
  StyleBox["Full->True", "MR"],
  " returns a list of lists of words, one for each word in ",
  Cell[BoxData[
      \(TraditionalForm\`W\)]],
  "."
}], "Notes",
  CellTags->"FromFrontier"],

Cell[TextData[{
  "On frontiers, this operation is the inverse to ",
  ButtonBox["ToFrontier",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"FromFrontier"],

Cell[TextData[{
  "If ",
  StyleBox["W", "MR"],
  " is only prefix but not right closed, the corresponding set   ",
  Cell[BoxData[
      \(TraditionalForm\`\(K\^*\) - W\ \(K\^*\)\)]],
  "   is infinite."
}], "Notes",
  CellTags->"FromFrontier"],

Cell[TextData[{
  "See also  ",
  ButtonBox["ToFrontier",
    ButtonData:>"ToFrontier",
    ButtonStyle->"AddOnsLink"],
  "  and ",
  ButtonBox["FrontierQ",
    ButtonData:>"FrontierQ",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"FromFrontier"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FromFrontier"],

Cell["A small frontier language and its prefixes.", "ExampleText",
  CellTags->"FromFrontier"],

Cell[BoxData[{
    \(\(W\  = \ {aa, bb, aba, abb, baa, baba, babb};\)\), "\n", 
    \(FromFrontier[W]\)}], "Input",
  CellLabel->"In[1]:=",
  CellTags->"FromFrontier"],

Cell["WordSort[Union[Flatten[%]]]", "Input",
  CellLabel->"In[3]:=",
  CellTags->"FromFrontier"],

Cell["The unflattened, non-uniqued output.", "ExampleText",
  CellTags->"FromFrontier"],

Cell[BoxData[
    \(FromFrontier[W, Full \[Rule] True]\)], "Input",
  CellLabel->"In[4]:=",
  CellTags->"FromFrontier"],

Cell["FrontierQ", "ObjectName",
  CellTags->"FrontierQ"],

Cell["Function", "Highlight",
  CellTags->"FrontierQ"],

Cell[TextData[{
  StyleBox["FrontierQ[L,K]", "MR"],
  "  tests whether the finite language ",
  StyleBox["L", "MR"],
  " is a frontier over alphabet ",
  StyleBox["K", "MR"],
  "."
}], "Usage",
  CellTags->"FrontierQ"],

Cell[TextData[{
  "Note that ",
  StyleBox["L", "MR"],
  " is supposed to be prefix, the algorithm does not check."
}], "Notes",
  CellTags->"FrontierQ"],

Cell[TextData[{
  "See also  ",
  ButtonBox["FromFrontier",
    ButtonData:>"FromFrontier",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToFrontier",
    ButtonData:>"ToFrontier",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["FrontierToDFA",
    ButtonData:>"FrontierToDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"FrontierQ"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FrontierQ"],

Cell["A small frontier language.", "ExampleText",
  CellTags->"FrontierQ"],

Cell[BoxData[
    \(FrontierQ[{aa, bb, aba, abb, \ baa, bab}, 2]\)], "Input",
  CellLabel->"In[280]:=",
  CellTags->"FrontierQ"],

Cell["Note that the underlying alphabet matters.", "ExampleText",
  CellTags->"FrontierQ"],

Cell[BoxData[
    \(FrontierQ[{aa, bb, aba, abb, \ baa, bab}, 3]\)], "Input",
  CellLabel->"In[281]:=",
  CellTags->"FrontierQ"],

Cell["MakeAlphabet", "ObjectName",
  CellTags->"MakeAlphabet"],

Cell["Function", "Highlight",
  CellTags->"MakeAlphabet"],

Cell[TextData[{
  StyleBox["MakeAlphabet[KK]", "MR"],
  "  returns  ",
  StyleBox["K", "MR"],
  "  where  ",
  StyleBox["K", "MR"],
  "  is the alphabet determined by the raw alphabet  ",
  StyleBox["KK", "MR"],
  ". "
}], "Usage",
  CellTags->"MakeAlphabet"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"MakeAlphabet"],

Cell[TextData[{
  "Option  ",
  StyleBox["Full->True", "MR"],
  "  causes a pair  ",
  StyleBox["{k,K}", "MR"],
  "  to be returned where ",
  StyleBox["k", "MR"],
  " is the size of ",
  StyleBox["K", "MR"],
  "."
}], "Notes",
  CellTags->"MakeAlphabet"],

Cell[TextData[{
  "The raw alphabet ",
  StyleBox["KK", "MR"],
  " may be an integer, a list of symbols or a list of characters."
}], "Notes",
  CellTags->"MakeAlphabet"],

Cell[TextData[{
  "See also  ",
  ButtonBox["Alphabet",
    ButtonData:>"Alphabet",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["AlphabetSize",
    ButtonData:>"AlphabetSize",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"MakeAlphabet"],

Cell["Further Examples", "ExampleSection",
  CellTags->"MakeAlphabet"],

Cell["Constructing an alphabet from symbols.", "ExampleText",
  CellTags->"MakeAlphabet"],

Cell["\<\
Clear[c,d]
MakeAlphabet[{c,d}]
% // FullForm\
\>", "Input",
  CellLabel->"In[83]:=",
  CellTags->"MakeAlphabet"],

Cell[BoxData[
    \(MakeAlphabet[{c, d}, Full \[Rule] True]\)], "Input",
  CellLabel->"In[86]:=",
  CellTags->"MakeAlphabet"],

Cell["A digit alphabet.", "ExampleText",
  CellTags->"MakeAlphabet"],

Cell["\<\
MakeAlphabet[-5]
% // FullForm\
\>", "Input",
  CellLabel->"In[87]:=",
  CellTags->"MakeAlphabet"],

Cell["MakeWord", "ObjectName",
  CellTags->"MakeWord"],

Cell["Function", "Highlight",
  CellTags->"MakeWord"],

Cell[TextData[{
  StyleBox["MakeWord[a,i,b,j,...]", "MR"],
  "  generates the word starting with ",
  StyleBox["i", "MR"],
  " repetitions of letter ",
  StyleBox["a", "MR"],
  ", ",
  StyleBox["j", "MR"],
  " repetitions of letter ",
  StyleBox["b", "MR"],
  ", and so on."
}], "Usage",
  CellTags->"MakeWord"],

Cell["Always returns a string.", "Notes",
  CellTags->"MakeWord"],

Cell["Mildly useful to generate long words.", "Notes",
  CellTags->"MakeWord"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordPower",
    ButtonData:>"WordPower",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"MakeWord"],

Cell["Further Examples", "ExampleSection",
  CellTags->"MakeWord"],

Cell["Producing long words.", "ExampleText",
  CellTags->"MakeWord"],

Cell["\<\
MakeWord[ab,5,c,17]
% // AlgebraicForm\
\>", "Input",
  CellLabel->"In[310]:=",
  CellTags->"MakeWord"],

Cell[BoxData[
    \(MakeWord[]\)], "Input",
  CellLabel->"In[312]:=",
  CellTags->"MakeWord"],

Cell["ParikhVector", "ObjectName",
  CellTags->"ParikhVector"],

Cell["Function", "Highlight",
  CellTags->"ParikhVector"],

Cell[TextData[{
  StyleBox["ParikhVector[w,K]", "MR"],
  "  computes the Parihk vector of word ",
  StyleBox["w", "MR"],
  " over the ordered alphabet ",
  StyleBox["K", "MR"],
  "."
}], "Usage",
  CellTags->"ParikhVector"],

Cell[TextData[{
  "Uses the default alphabet  ",
  ButtonBox["$alphabet",
    ButtonStyle->"AddOnsLink"],
  "  (rather than the alphabet of ",
  StyleBox["w", "MR"],
  ") if ",
  StyleBox["K", "MR"],
  " is not specified.  "
}], "Notes",
  CellTags->"ParikhVector"],

Cell["Also works for lists of words.", "Notes",
  CellTags->"ParikhVector"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ParikhVector"],

Cell["\<\
The Parikh vectors of all words of length 4 over the standard \
two-letter alphabet.\
\>", "ExampleText",
  CellTags->"ParikhVector"],

Cell["ParikhVector[ Words[4], 2 ]", "Input",
  CellLabel->"In[313]:=",
  CellTags->"ParikhVector"],

Cell["\<\
Using the standard default alphabet as opposed to the alphabet of \
the word.\
\>", "ExampleText",
  CellTags->"ParikhVector"],

Cell[BoxData[{
    \(ParikhVector[\ ababacccd\ ]\), "\n", 
    \(With[\ {w\  = \ ababacccd}, \ 
      ParikhVector[\ w, \ Alphabet[w]]]\)}], "Input",
  CellLabel->"In[317]:=",
  CellTags->"ParikhVector"],

Cell["RandomWord", "ObjectName",
  CellTags->"RandomWord"],

Cell["Function", "Highlight",
  CellTags->"RandomWord"],

Cell[TextData[{
  StyleBox["RandomWord[n,K]", "MR"],
  "  produces a random word over alphabet K of length n. "
}], "Usage",
  CellTags->"RandomWord"],

Cell["Symbols are chosen uniformly from the underlying alphabet.", "Notes",
  CellTags->"RandomWord"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToWord",
    ButtonData:>"ToWord",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"RandomWord"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RandomWord"],

Cell["Random words.", "ExampleText",
  CellTags->"RandomWord"],

Cell["RandomWord[20,2]", "Input",
  CellLabel->"In[114]:=",
  CellTags->"RandomWord"],

Cell["RandomWord[20,-3]", "Input",
  CellLabel->"In[115]:=",
  CellTags->"RandomWord"],

Cell["ToBlocks", "ObjectName",
  CellTags->"ToBlocks"],

Cell["Function", "Highlight",
  CellTags->"ToBlocks"],

Cell[TextData[{
  StyleBox["ToBlocks[L,w]", "MR"],
  "  partitions a list ",
  StyleBox["L", "MR"],
  " into a list of overlapping blocks of width ",
  StyleBox["w", "MR"],
  " (with offset 1). "
}], "Usage",
  CellTags->"ToBlocks"],

Cell[TextData[{
  "Options\n",
  Cell[BoxData[
      FormBox[GridBox[{
            {
              StyleBox["Boundary",
                "MR"], 
              StyleBox["\[Rule]",
                "MR"], 
              StyleBox["Cyclic",
                "MR"], 
              StyleBox[\(Fixed, pelm, None\),
                "MR"]}
            },
          ColumnWidths->Automatic,
          ColumnAlignments->{Left}], TraditionalForm]]]
}], "Notes",
  CellTags->"ToBlocks"],

Cell[TextData[{
  "Option ",
  StyleBox["Boundary->pelm", "MR"],
  "  uses a padding element. "
}], "Notes",
  CellTags->"ToBlocks"],

Cell[TextData[{
  "Has default ",
  StyleBox["Boundary->Cyclic", "MR"],
  ", alternatively ",
  StyleBox["Fixed", "MR"],
  ", a padding element ",
  StyleBox["pelm", "MR"],
  ", or ",
  StyleBox["None", "MR"],
  ". "
}], "Notes",
  CellTags->"ToBlocks"],

Cell[TextData[{
  "Also works when  ",
  StyleBox["L", "MR"],
  "  is a word rather than a list, returns a list of words.  "
}], "Notes",
  CellTags->"ToBlocks"],

Cell[TextData[{
  StyleBox["ToBlocks", "MR"],
  " is essentially just a wrapper for ",
  ButtonBox["Partition",
    ButtonStyle->"RefGuideLink"],
  "."
}], "Notes",
  CellTags->"ToBlocks"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToBlocks"],

Cell["Breaking a list into blocks.", "ExampleText",
  CellTags->"ToBlocks"],

Cell[BoxData[
    \(ToBlocks[Range[5], 3]\)], "Input",
  CellLabel->"In[69]:=",
  CellTags->"ToBlocks"],

Cell[BoxData[
    \(ToBlocks[Range[5], 3, Boundary \[Rule] Fixed]\)], "Input",
  CellLabel->"In[70]:=",
  CellTags->"ToBlocks"],

Cell["Also works on words.", "ExampleText",
  CellTags->"ToBlocks"],

Cell[BoxData[
    \(ToBlocks["\<abcde\>", 3, Boundary \[Rule] "\<*\>"]\)], "Input",
  CellLabel->"In[71]:=",
  CellTags->"ToBlocks"],

Cell["ToEps", "ObjectName",
  CellTags->"ToEps"],

Cell["Function", "Highlight",
  CellTags->"ToEps"],

Cell[TextData[{
  StyleBox["ToEps[x]", "MR"],
  "  replaces all occurences of the empty string in ",
  StyleBox["x", "MR"],
  " by ",
  StyleBox["Eps", "MR"],
  " (as a string)."
}], "Usage",
  CellTags->"ToEps"],

Cell["Used before printing to remove the internal form of epsilon.", "Notes",
  CellTags->"ToEps"],

Cell[TextData[{
  "See also ",
  ButtonBox["FromEps",
    ButtonData:>"FromEps",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToEps"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToEps"],

Cell["\<\
Note that the empty string prints as \[Epsilon] in standard \
formatting.\
\>", "ExampleText",
  CellTags->"ToEps"],

Cell[BoxData[
    \("\<\>"\  // \ ToEps\)], "Input",
  CellLabel->"In[39]:=",
  CellTags->"ToEps"],

Cell["ToFrontier", "ObjectName",
  CellTags->"ToFrontier"],

Cell["Function", "Highlight",
  CellTags->"ToFrontier"],

Cell[TextData[{
  StyleBox["ToFrontier[W,K]", "MR"],
  "  computes the prefix set corresponding to the finite language ",
  StyleBox["W", "MR"],
  " over alphabet ",
  StyleBox["K", "MR"],
  ". "
}], "Usage",
  CellTags->"ToFrontier"],

Cell[TextData[{
  "The algorithm actually computes ",
  Cell[BoxData[
      \(TraditionalForm\`W\ K\  - \ W\)]],
  ". "
}], "Notes",
  CellTags->"ToFrontier"],

Cell[TextData[{
  "On prefix-closed sets, this operation is the inverse to ",
  StyleBox["FromFrontier", "MR"],
  "."
}], "Notes",
  CellTags->"ToFrontier"],

Cell[TextData[{
  "See also  ",
  ButtonBox["FromFrontier",
    ButtonData:>"FromFrontier",
    ButtonStyle->"AddOnsLink"],
  "  and ",
  ButtonBox["FrontierQ",
    ButtonData:>"FrontierQ",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToFrontier"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToFrontier"],

Cell["A small frontier language and its prefixes.", "ExampleText",
  CellTags->"ToFrontier"],

Cell[BoxData[
    \(\(\(\(W\  = \ {aa, bb, baa, bab, aba, abb};\)\n
      WordSort[FromFrontier[W]]\)\(]\)\)\)], "Input",
  CellLabel->"In[178]:=",
  CellTags->"ToFrontier"],

Cell["ToFrontier[%]", "Input",
  CellLabel->"In[180]:=",
  CellTags->"ToFrontier"],

Cell["ToIndex", "ObjectName",
  CellTags->"ToIndex"],

Cell["Function", "Highlight",
  CellTags->"ToIndex"],

Cell[TextData[{
  StyleBox["ToIndex[w,K]", "MR"],
  "  converts a word ",
  StyleBox["w", "MR"],
  " over a sorted alphabet ",
  StyleBox["K", "MR"],
  " to positional notation."
}], "Usage",
  CellTags->"ToIndex"],

Cell["Internally most algorithms use positional notation.", "Notes",
  CellTags->"ToIndex"],

Cell["Also works on symbols and treats epsilon properly.", "Notes",
  CellTags->"ToIndex"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToWord",
    ButtonData:>"ToWord",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToIndex"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToIndex"],

Cell["Replacing symbols by their index.", "ExampleText",
  CellTags->"ToIndex"],

Cell["ToIndex[ abcde, 5 ]", "Input",
  CellLabel->"In[152]:=",
  CellTags->"ToIndex"],

Cell[BoxData[
    \(ToWord[\ %, \ 5\ ]\)], "Input",
  CellLabel->"In[108]:=",
  CellTags->"ToIndex"],

Cell["ToWord", "ObjectName",
  CellTags->"ToWord"],

Cell["Function", "Highlight",
  CellTags->"ToWord"],

Cell[TextData[{
  StyleBox["ToWord[L,K]", "MR"],
  "  converts a list of indexes to the corresponding word over sorted \
alphabet ",
  StyleBox["K", "MR"],
  "."
}], "Usage",
  CellTags->"ToWord"],

Cell[TextData[{
  "Default alphabet is ",
  ButtonBox["$alphabet",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"ToWord"],

Cell["\<\
A 0 entry is understood to represent the empty word \
\[Epsilon].\
\>", "Notes",
  CellTags->"ToWord"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToIndex",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToWord"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToWord"],

Cell["Translating positional lists into words.", "ExampleText",
  CellTags->"ToWord"],

Cell[" ToWord[Range[5],5]", "Input",
  CellLabel->"In[57]:=",
  CellTags->"ToWord"],

Cell[" ToWord[{1,2,1,2},{\"r\",\"s\"}]", "Input",
  CellLabel->"In[59]:=",
  CellTags->"ToWord"],

Cell["And translating back.", "ExampleText",
  CellTags->"ToWord"],

Cell[BoxData[
    \(ToIndex[%, {"\<r\>", "\<s\>"}]\)], "Input",
  CellLabel->"In[60]:=",
  CellTags->"ToWord"],

Cell["WordBinomial", "ObjectName",
  CellTags->"WordBinomial"],

Cell["Function", "Highlight",
  CellTags->"WordBinomial"],

Cell[TextData[{
  StyleBox["WordBinomial[u,v]", "MR"],
  "  generalizes binomial coefficients to words: counts the number of \
subwords ",
  StyleBox["v", "MR"],
  " in ",
  StyleBox["u", "MR"],
  ". "
}], "Usage",
  CellTags->"WordBinomial"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"DoTrace", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"WordBinomial"],

Cell[TextData[{
  "Option ",
  StyleBox["DoTrace->True", "MR"],
  " shows values for all prefixes of ",
  StyleBox["u", "MR"],
  " and ",
  StyleBox["v", "MR"],
  "."
}], "Notes",
  CellTags->"WordBinomial"],

Cell["\<\
Note that subword here means any subsequence, the letters need not \
be contiguous as in a factor.\
\>", "Notes",
  CellTags->"WordBinomial"],

Cell[TextData[{
  "See also ",
  ButtonBox["LongestCommonSubsequence",
    ButtonData:>"LongestCommonSubsequence",
    ButtonStyle->"AddOnsLink"],
  ". "
}], "Notes",
  CellTags->"WordBinomial"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordBinomial"],

Cell[TextData[{
  StyleBox["WordBinomial", "MR"],
  " properly generalizes ordinary binomial coefficients."
}], "ExampleText",
  CellTags->"WordBinomial"],

Cell[BoxData[{
    \(WordBinomial[aaaaa, aa]\), "\n", 
    \(Binomial[5, 2]\)}], "Input",
  CellLabel->"In[74]:=",
  CellTags->"WordBinomial"],

Cell["The trace shows how the count is being computed.", "ExampleText",
  CellTags->"WordBinomial"],

Cell[BoxData[{
    \(WordBinomial[ababab, abb]\), "\n", 
    \(WordBinomial[ababab, abb, DoTrace \[Rule] True]\  // \ 
      TableForm\)}], "Input",
  CellLabel->"In[771]:=",
  CellTags->"WordBinomial"],

Cell["WordConjugateQ", "ObjectName",
  CellTags->"WordConjugateQ"],

Cell["Function", "Highlight",
  CellTags->"WordConjugateQ"],

Cell[TextData[{
  StyleBox["WordConjugateQ[u,v]", "MR"],
  "  tests whether words ",
  StyleBox["u", "MR"],
  " and ",
  StyleBox["v", "MR"],
  " are conjugates."
}], "Usage",
  CellTags->"WordConjugateQ"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"WordConjugateQ"],

Cell[TextData[{
  "Uses ",
  ButtonBox["StringPosition",
    ButtonStyle->"RefGuideLink"],
  " to determine conjugacy and is therefore linear."
}], "Notes",
  CellTags->"WordConjugateQ"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordRotate",
    ButtonData:>"WordRotate",
    ButtonStyle->"AddOnsLink"],
  ". "
}], "Notes",
  CellTags->"WordConjugateQ"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordConjugateQ"],

Cell["Conjugates are obtained by rotation.", "ExampleText",
  CellTags->"WordConjugateQ"],

Cell["WordConjugateQ[ aaab, baaa ]", "Input",
  CellLabel->"In[95]:=",
  CellTags->"WordConjugateQ"],

Cell["WordConjugateQ[ aaab, baab ]", "Input",
  CellLabel->"In[96]:=",
  CellTags->"WordConjugateQ"],

Cell["Select[ Words[4], WordConjugateQ[ aabb, # ]& ]", "Input",
  CellLabel->"In[65]:=",
  CellTags->"WordConjugateQ"],

Cell["WordConjugates", "ObjectName",
  CellTags->"WordConjugates"],

Cell["Function", "Highlight",
  CellTags->"WordConjugates"],

Cell[TextData[{
  StyleBox["WordConjugates[w]", "MR"],
  "  generates all conjugates of a word ",
  StyleBox["w", "MR"],
  ", in lexicographic order."
}], "Usage",
  CellTags->"WordConjugates"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"WordConjugates"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordConjugateQ",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WordConjugates"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordConjugates"],

Cell[TextData[{
  "Conjugates are obtained by rotation. The number of conjugates depends on \
",
  Cell[BoxData[
      \(TraditionalForm\`w\)]],
  ", not just ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(|\)\(w\)\(|\)\)\)]],
  ". "
}], "ExampleText",
  CellTags->"WordConjugates"],

Cell["WordConjugates[ aaaab ]", "Input",
  CellLabel->"In[271]:=",
  CellTags->"WordConjugates"],

Cell["WordConjugates[ abab ]", "Input",
  CellLabel->"In[272]:=",
  CellTags->"WordConjugates"],

Cell["WordFactorizations", "ObjectName",
  CellTags->"WordFactorizations"],

Cell["Function", "Highlight",
  CellTags->"WordFactorizations"],

Cell[TextData[{
  StyleBox["WordFactorizations[w]", "MR"],
  "  generates all factorizations ",
  StyleBox["uv", "MR"],
  " of a word ",
  StyleBox["w.", "MR"],
  " "
}], "Usage",
  CellTags->"WordFactorizations"],

Cell[TextData[{
  "Returns a list of pairs ",
  StyleBox["{u,v}", "MR"],
  " such that ",
  Cell[BoxData[
      \(TraditionalForm\`w\  = \ u\[VeryThinSpace]v\)]],
  "."
}], "Notes",
  CellTags->"WordFactorizations"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordFactors",
    ButtonStyle->"AddOnsLink"],
  "  and  ",
  ButtonBox["WordFactorQ",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WordFactorizations"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordFactorizations"],

Cell[TextData[{
  "The output is a list of pairs and has length ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(|\)\(w\)\(|\)\(\ \)\(+\ 1\)\)\)]],
  "."
}], "ExampleText",
  CellTags->"WordFactorizations"],

Cell[" WordFactorizations[abcabc]", "Input",
  CellLabel->"In[70]:=",
  CellTags->"WordFactorizations"],

Cell[BoxData[
    \(Concatenate\  /@ \ %\  // \ Union\)], "Input",
  CellLabel->"In[71]:=",
  CellTags->"WordFactorizations"],

Cell["WordFactorQ", "ObjectName",
  CellTags->"WordFactorQ"],

Cell["Function", "Highlight",
  CellTags->"WordFactorQ"],

Cell[TextData[{
  StyleBox["WordFactorQ[x,y]", "MR"],
  "  tests whether word ",
  StyleBox["x", "MR"],
  " is a factor (or infix) of word ",
  StyleBox["y", "MR"],
  "."
}], "Usage",
  CellTags->"WordFactorQ"],

Cell["\<\
Also works on symbols and treats  \[Epsilon]  properly.\
\>", \
"Notes",
  CellTags->"WordFactorQ"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordQ",
    ButtonData:>"WordQ",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["WordPrefixQ",
    ButtonData:>"WordPrefixQ",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["WordSuffixQ",
    ButtonData:>"WordSuffixQ",
    ButtonStyle->"AddOnsLink"],
  "  and  ",
  ButtonBox["WordBinomial",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WordFactorQ"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordFactorQ"],

Cell["\<\
Testing whether a word is a factor of another. Note that only \
contiguous occurrences count.\
\>", "ExampleText",
  CellTags->"WordFactorQ"],

Cell[BoxData[
    \(WordFactorQ[ab, abc]\)], "Input",
  CellLabel->"In[79]:=",
  CellTags->"WordFactorQ"],

Cell[BoxData[
    \(WordFactorQ[ac, abc]\)], "Input",
  CellLabel->"In[80]:=",
  CellTags->"WordFactorQ"],

Cell[BoxData[
    \(Select[\ Words[\(-5\)], \ WordFactorQ[aaa, #] &\ ]\)], "Input",
  CellLabel->"In[81]:=",
  CellTags->"WordFactorQ"],

Cell["WordFactors", "ObjectName",
  CellTags->"WordFactors"],

Cell["Function", "Highlight",
  CellTags->"WordFactors"],

Cell[TextData[{
  StyleBox["WordFactors[w,opts]", "MR"],
  "  generates a list of all factors of word ",
  StyleBox["w", "MR"],
  ", in length-lex order. "
}], "Usage",
  CellTags->"WordFactors"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "True", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"WordFactors"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->False", "MR"],
  "  omits ",
  StyleBox["w", "MR"],
  " from the list of factors."
}], "Notes",
  CellTags->"WordFactors"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"WordFactors"],

Cell[TextData[{
  StyleBox["WordFactors[w,k]", "MR"],
  "  generates a list of all factors of word ",
  StyleBox["w", "MR"],
  " of length ",
  StyleBox["k", "MR"],
  "."
}], "Notes",
  CellTags->"WordFactors"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordFactorizations",
    ButtonData:>"WordFactorizations",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WordPrefixes",
    ButtonData:>"WordPrefixes",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["WordSuffixes",
    ButtonData:>"WordSuffixes",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["WordToFA",
    ButtonData:>"WordToFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WordFactors"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordFactors"],

Cell[TextData[{
  "The length of the output list depends on ",
  Cell[BoxData[
      \(TraditionalForm\`w\)]],
  ". "
}], "ExampleText",
  CellTags->"WordFactors"],

Cell[" WordFactors[aaaaaa]", "Input",
  CellLabel->"In[105]:=",
  CellTags->"WordFactors"],

Cell[" WordFactors[abcdef]", "Input",
  CellLabel->"In[106]:=",
  CellTags->"WordFactors"],

Cell["All factors of a given length.", "ExampleText",
  CellTags->"WordFactors"],

Cell[" WordFactors[ababab,3]", "Input",
  CellLabel->"In[111]:=",
  CellTags->"WordFactors"],

Cell["WordFusion", "ObjectName",
  CellTags->"WordFusion"],

Cell["Function", "Highlight",
  CellTags->"WordFusion"],

Cell[TextData[{
  StyleBox["WordFusion[u,v,r]", "MR"],
  "  returns the ",
  StyleBox["r", "MR"],
  "-fusion of words ",
  StyleBox["u", "MR"],
  " and ",
  StyleBox["v", "MR"],
  ". "
}], "Usage",
  CellTags->"WordFusion"],

Cell[TextData[{
  "Returns ",
  StyleBox["Null", "MR"],
  " if the fusion of the two input words is undefined."
}], "Notes",
  CellTags->"WordFusion"],

Cell[TextData[{
  "The input must have length at least ",
  StyleBox["r", "MR"],
  " (default ",
  Cell[BoxData[
      \(TraditionalForm\`r = 1\)]],
  "). "
}], "Notes",
  CellTags->"WordFusion"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordFusion"],

Cell[TextData[{
  "Fusion overlaps a suffix of ",
  Cell[BoxData[
      \(TraditionalForm\`u\)]],
  " and a prefix of ",
  Cell[BoxData[
      \(TraditionalForm\`v\)]],
  ". "
}], "ExampleText",
  CellTags->"WordFusion"],

Cell["WordFusion[ aabb, bbcc, 2 ]", "Input",
  CellLabel->"In[335]:=",
  CellTags->"WordFusion"],

Cell["WordFusion[ aabb, ccbb, 2 ]", "Input",
  CellLabel->"In[334]:=",
  CellTags->"WordFusion"],

Cell["WordLength", "ObjectName",
  CellTags->"WordLength"],

Cell["Function", "Highlight",
  CellTags->"WordLength"],

Cell[TextData[{
  StyleBox["WordLength[w]", "MR"],
  "  returns the length of word ",
  StyleBox["w. ", "MR"]
}], "Usage",
  CellTags->"WordLength"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"WordLength"],

Cell[TextData[{
  StyleBox["WordLength", "MR"],
  " behaves very much like ",
  StyleBox["StringLength", "MR"],
  " but accepts symbols as input and treats the empty word \[Epsilon] \
properly."
}], "Notes",
  CellTags->"WordLength"],

Cell[TextData[{
  StyleBox["WordLength[w,a]", "MR"],
  " returns the number of occurrences of letter ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  "  in word ",
  Cell[BoxData[
      \(TraditionalForm\`w\)]],
  "."
}], "Notes",
  CellTags->"WordLength"],

Cell[TextData[{
  "See also ",
  ButtonBox["ParikhVector",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WordLength"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordLength"],

Cell[TextData[{
  StyleBox["WordLength", "MR"],
  " deals properly with epsilon."
}], "ExampleText",
  CellTags->"WordLength"],

Cell["\<\
Words[-3]
WordLength[%]\
\>", "Input",
  CellLabel->"In[87]:=",
  CellTags->"WordLength"],

Cell["\<\
Words[-3]
WordLength[%,a]\
\>", "Input",
  CellLabel->"In[89]:=",
  CellTags->"WordLength"],

Cell["WordOrder", "ObjectName",
  CellTags->"WordOrder"],

Cell["Function", "Highlight",
  CellTags->"WordOrder"],

Cell[TextData[{
  StyleBox["WordOrder[u,v]", "MR"],
  "  returns -1, 0, +1 depending on the position of ",
  StyleBox["u", "MR"],
  " and ",
  StyleBox["v", "MR"],
  " in the length-lex order"
}], "Usage",
  CellTags->"WordOrder"],

Cell[TextData[{
  "This order is used in many algorithms and in particular in the semimodule \
generation algorithm, see ",
  ButtonBox["GenerateCSM",
    ButtonData:>"GenerateCSM",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WordOrder"],

Cell[TextData[{
  " ",
  StyleBox["u", "MR"],
  " is less than ",
  StyleBox["v", "MR"],
  " in the length-lex order if the length of ",
  StyleBox["u", "MR"],
  " is less than the length of ",
  StyleBox["v", "MR"],
  ", or if ",
  StyleBox["u", "MR"],
  " and ",
  StyleBox["v", "MR"],
  " have the same length but ",
  StyleBox["u", "MR"],
  " precedes ",
  StyleBox["v", "MR"],
  " lexicographically. "
}], "Notes",
  CellTags->"WordOrder"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordSort",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WordOrder"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordOrder"],

Cell["All comparisons between words of length at most 2.", "ExampleText",
  CellTags->"WordOrder"],

Cell["W = Words[-2]", "Input",
  CellLabel->"In[91]:=",
  CellTags->"WordOrder"],

Cell[BoxData[
    \(\(\(Apply[\ WordOrder, \ 
      CartesianProduct[W, W], {1}]\)\(\ \)\)\)], "Input",
  CellLabel->"In[92]:=",
  CellTags->"WordOrder"],

Cell["WordPeriod", "ObjectName",
  CellTags->"WordPeriod"],

Cell["Function", "Highlight",
  CellTags->"WordPeriod"],

Cell[TextData[{
  StyleBox["WordPeriod[w]", "MR"],
  "  returns the minimal period of a word ",
  StyleBox["w", "MR"],
  "."
}], "Usage",
  CellTags->"WordPeriod"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"WordPeriod"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"WordPeriod"],

Cell[TextData[{
  "Option  ",
  StyleBox["Full->True", "MR"],
  "  returns all periods of ",
  Cell[BoxData[
      \(TraditionalForm\`w\)]],
  ", rather than just the mimial one."
}], "Notes",
  CellTags->"WordPeriod"],

Cell[TextData[{
  "Determines the least positive integer ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(p\)\(\ \)\(\[LessEqual]\)\)\  | w | \)]],
  " such that ",
  Cell[BoxData[
      \(TraditionalForm\`w\  = \ \((x\[VeryThinSpace]y)\)\^\(\(\ \)\(k\)\)\ x\
\)]],
  "  where ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(|\)\(x\)\(|\)\) = p\)]],
  "."
}], "Notes",
  CellTags->"WordPeriod"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordPrimitiveQ",
    ButtonData:>"WordPrimitiveQ",
    ButtonStyle->"AddOnsLink"],
  " , ",
  ButtonBox["WordRoot",
    ButtonData:>"WordRoot",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["WordPower",
    ButtonData:>"WordPower",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WordPeriod"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordPeriod"],

Cell["\<\
A frequency table for the periods of all words of length 8 over a \
two-letter alphabet.\
\>", "ExampleText",
  CellTags->"WordPeriod"],

Cell[BoxData[{
    \(\(L\  = \ Pairs[\ Words[8], WordPeriod[\ Words[8]]];\)\), "\n", 
    \(\(Last\  /@ \ L\  // \ Frequencies\)\  // \ TableForm\)}], "Input",
  CellLabel->"In[575]:=",
  CellTags->"WordPeriod"],

Cell["\<\
Here are all words of period 4. Their prefixes of length 5 are duly \
primitive.\
\>", "ExampleText",
  CellTags->"WordPeriod"],

Cell[BoxData[
    \(First\  /@ \ Cases[\ L, \ {_, 5}]\)], "Input",
  CellLabel->"In[577]:=",
  CellTags->"WordPeriod"],

Cell[BoxData[
    \(WordPrimitiveQ[\(StringTake[#, 5] &\)\  /@ \ %]\)], "Input",
  CellLabel->"In[578]:=",
  CellTags->"WordPeriod"],

Cell["All periods of a word.", "ExampleText",
  CellTags->"WordPeriod"],

Cell[BoxData[
    \(WordPeriod[\ abaababa, \ Full \[Rule] True\ ]\)], "Input",
  CellLabel->"In[596]:=",
  CellTags->"WordPeriod"],

Cell["WordPower", "ObjectName",
  CellTags->"WordPower"],

Cell["Function", "Highlight",
  CellTags->"WordPower"],

Cell[TextData[{
  StyleBox["WordPower[w,r]", "MR"],
  "  returns word ",
  StyleBox["w", "MR"],
  " repeated ",
  StyleBox["r", "MR"],
  " times."
}], "Usage",
  CellTags->"WordPower"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"WordPower"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordPrimitiveQ",
    ButtonData:>"WordPrimitiveQ",
    ButtonStyle->"AddOnsLink"],
  " ,  ",
  ButtonBox["WordPrimitiveCount",
    ButtonData:>"WordPrimitiveCount",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WordPower"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordPower"],

Cell["\<\
The words that fail to be powers of shorter words are primitive. \
\
\>", "ExampleText",
  CellTags->"WordPower"],

Cell[BoxData[
    \(W = Complement[Words[8], Select[Words[8], WordPrimitiveQ]]\)], "Input",
  CellLabel->"In[118]:=",
  CellTags->"WordPower"],

Cell[BoxData[
    \(W1 = WordPower[Words[2], 4]\)], "Input",
  CellLabel->"In[119]:=",
  CellTags->"WordPower"],

Cell[BoxData[
    \(W2 = WordPower[Words[4], 2]\)], "Input",
  CellLabel->"In[120]:=",
  CellTags->"WordPower"],

Cell[BoxData[
    \(Union[W1, W2] === \ W\)], "Input",
  CellLabel->"In[121]:=",
  CellTags->"WordPower"],

Cell["WordPrefixQ", "ObjectName",
  CellTags->"WordPrefixQ"],

Cell["Function", "Highlight",
  CellTags->"WordPrefixQ"],

Cell[TextData[{
  StyleBox["WordPrefixQ[x,y]", "MR"],
  "  tests whether word ",
  StyleBox["x", "MR"],
  " is a prefix of word ",
  StyleBox["y", "MR"],
  "."
}], "Usage",
  CellTags->"WordPrefixQ"],

Cell["\<\
Also works on symbols and treats  \[Epsilon]  properly.\
\>", \
"Notes",
  CellTags->"WordPrefixQ"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordQ",
    ButtonData:>"WordQ",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["WordSuffixQ",
    ButtonData:>"WordSuffixQ",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["WordFactorQ",
    ButtonData:>"WordFactorQ",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WordPrefixQ"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordPrefixQ"],

Cell["Some simple prefix queries.", "ExampleText",
  CellTags->"WordPrefixQ"],

Cell[BoxData[
    \(WordPrefixQ[a, abc]\)], "Input",
  CellLabel->"In[414]:=",
  CellTags->"WordPrefixQ"],

Cell[BoxData[
    \(WordPrefixQ[b, abc]\)], "Input",
  CellLabel->"In[415]:=",
  CellTags->"WordPrefixQ"],

Cell[BoxData[
    \(WordPrefixQ[\[Epsilon], abc]\)], "Input",
  CellLabel->"In[416]:=",
  CellTags->"WordPrefixQ"],

Cell[BoxData[
    \(Select[\ Words[\(-5\)], \ WordPrefixQ[aaa, #] &\ ]\)], "Input",
  CellLabel->"In[418]:=",
  CellTags->"WordPrefixQ"],

Cell["WordPrefixes", "ObjectName",
  CellTags->"WordPrefixes"],

Cell["Function", "Highlight",
  CellTags->"WordPrefixes"],

Cell[TextData[{
  StyleBox["WordPrefixes[w]", "MR"],
  "  generates all prefixes of word  ",
  StyleBox["w", "MR"],
  ", in length order. "
}], "Usage",
  CellTags->"WordPrefixes"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "True", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"WordPrefixes"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->False", "MR"],
  "  omits ",
  StyleBox["w", "MR"],
  " from the list of prefixes."
}], "Notes",
  CellTags->"WordPrefixes"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"WordPrefixes"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordSuffixes",
    ButtonData:>"WordSuffixes",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["WordFactors",
    ButtonData:>"WordFactors",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["WordToFA",
    ButtonData:>"WordToFA",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["FromFrontier",
    ButtonData:>"FromFrontier",
    ButtonStyle->"AddOnsLink"],
  ". "
}], "Notes",
  CellTags->"WordPrefixes"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordPrefixes"],

Cell[TextData[{
  "The output list has length ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(|\)\(w\)\(|\)\(+1\)\)\)]],
  "."
}], "ExampleText",
  CellTags->"WordPrefixes"],

Cell[" WordPrefixes[abcabc]", "Input",
  CellLabel->"In[420]:=",
  CellTags->"WordPrefixes"],

Cell[" WordPrefixes[{abc,cba}]", "Input",
  CellLabel->"In[421]:=",
  CellTags->"WordPrefixes"],

Cell["All  proper prefixes.", "ExampleText",
  CellTags->"WordPrefixes"],

Cell[BoxData[
    \(WordPrefixes[aaab, Full \[Rule] False]\)], "Input",
  CellLabel->"In[422]:=",
  CellTags->"WordPrefixes"],

Cell["WordPrimitiveCount", "ObjectName",
  CellTags->"WordPrimitiveCount"],

Cell["Function", "Highlight",
  CellTags->"WordPrimitiveCount"],

Cell[TextData[{
  StyleBox["WordPrimitiveCount[n,k]", "MR"],
  "  returns the number of primitive words of length ",
  StyleBox["n", "MR"],
  " over a ",
  StyleBox["k", "MR"],
  " symbol alphabet. "
}], "Usage",
  CellTags->"WordPrimitiveCount"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"WordPrimitiveCount"],

Cell[TextData[{
  "Uses the built-in ",
  ButtonBox["MoebiusMu",
    ButtonStyle->"RefGuideLink"],
  " function."
}], "Notes",
  CellTags->"WordPrimitiveCount"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordPrimitiveQ",
    ButtonData:>"WordPrimitiveQ",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WordRoot",
    ButtonData:>"WordRoot",
    ButtonStyle->"AddOnsLink"],
  "  and  ",
  ButtonBox["WordPower",
    ButtonData:>"WordPower",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WordPrimitiveCount"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordPrimitiveCount"],

Cell["Most words are primitive.", "ExampleText",
  CellTags->"WordPrimitiveCount"],

Cell["Table[{n,2^n,WordPrimitiveCount[n,2]},{n,10}] // TableForm", "Input",
  CellLabel->"In[132]:=",
  CellTags->"WordPrimitiveCount"],

Cell["WordPrimitiveQ", "ObjectName",
  CellTags->"WordPrimitiveQ"],

Cell["Function", "Highlight",
  CellTags->"WordPrimitiveQ"],

Cell[TextData[{
  StyleBox["WordPrimitiveQ[w]", "MR"],
  "  tests whether word ",
  StyleBox["w", "MR"],
  " is primitive, i.e, not the power of a shorter word. "
}], "Usage",
  CellTags->"WordPrimitiveQ"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"WordPrimitiveQ"],

Cell[TextData[{
  "Uses built-in ",
  ButtonBox["StringPosition",
    ButtonStyle->"RefGuideLink"],
  "  and is linear time."
}], "Notes",
  CellTags->"WordPrimitiveQ"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordPrimitiveCount",
    ButtonData:>"WordPrimitiveCount",
    ButtonStyle->"AddOnsLink"],
  " , ",
  ButtonBox["WordRoot",
    ButtonData:>"WordRoot",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["WordPower",
    ButtonData:>"WordPower",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WordPrimitiveQ"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordPrimitiveQ"],

Cell["\<\
Only 16 words of length 8 (over a 2-symbol alphabet) fail to be \
primitive.\
\>", "ExampleText",
  CellTags->"WordPrimitiveQ"],

Cell[BoxData[{
    \(\(L\  = \ Select[Words[8], WordPrimitiveQ\ ];\)\), "\n", 
    \(L // \ Length\)}], "Input",
  CellLabel->"In[612]:=",
  CellTags->"WordPrimitiveQ"],

Cell["All the conjugates of a primitive word are again primitive.", \
"ExampleText",
  CellTags->"WordPrimitiveQ"],

Cell[BoxData[
    \(WordRotate[L[\([111]\)], Range[8]]\  // \ WordPrimitiveQ\)], "Input",
  CellLabel->"In[614]:=",
  CellTags->"WordPrimitiveQ"],

Cell["\<\
Only 2 words of prime length 11 over a 2-symbol alphabet fail to be \
primitive.\
\>", "ExampleText",
  CellTags->"WordPrimitiveQ"],

Cell[BoxData[
    \(Select[Words[11], WordPrimitiveQ] // \ Length\)], "Input",
  CellLabel->"In[615]:=",
  CellTags->"WordPrimitiveQ"],

Cell["WordQ", "ObjectName",
  CellTags->"WordQ"],

Cell["Function", "Highlight",
  CellTags->"WordQ"],

Cell[TextData[{
  StyleBox["WordQ[w]", "MR"],
  "  tests whether w is a word: w may be a symbol or a string."
}], "Usage",
  CellTags->"WordQ"],

Cell["Also treats epsilon properly.", "Notes",
  CellTags->"WordQ"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"WordQ"],

Cell[TextData[{
  "See also  ",
  ButtonBox["WordReduce",
    ButtonData:>"WordReduce",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WordBinomial",
    ButtonData:>"WordBinomial",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WordConjugateQ",
    ButtonData:>"WordConjugateQ",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WordConjugates",
    ButtonData:>"WordConjugates",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WordFactorizations",
    ButtonData:>"WordFactorizations",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WordFactors",
    ButtonData:>"WordFactors",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WordFusion",
    ButtonData:>"WordFusion",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WordLength",
    ButtonData:>"WordLength",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WordMap",
    ButtonData:>"WordMap",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WordOrder",
    ButtonData:>"WordOrder",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WordPower",
    ButtonData:>"WordPower",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WordPrefixes",
    ButtonData:>"WordPrefixes",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WordReverse",
    ButtonData:>"WordReverse",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WordRoot",
    ButtonData:>"WordRoot",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WordRotate",
    ButtonData:>"WordRotate",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WordShuffle",
    ButtonData:>"WordShuffle",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WordSort",
    ButtonData:>"WordSort",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WordSuffixes",
    ButtonData:>"WordSuffixes",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["Words",
    ButtonData:>"Words",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"WordQ"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordQ"],

Cell["Words as opposed to strings.", "ExampleText",
  CellTags->"WordQ"],

Cell[BoxData[
    \(WordQ[{abc, "\<abc\>", Eps, \[Epsilon], 1234, {a}}]\)], "Input",
  CellLabel->"In[375]:=",
  CellTags->"WordQ"],

Cell["WordReverse", "ObjectName",
  CellTags->"WordReverse"],

Cell["Function", "Highlight",
  CellTags->"WordReverse"],

Cell[TextData[{
  StyleBox["WordReverse[w]", "MR"],
  "  reverses the word ",
  StyleBox["w", "MR"],
  " like ",
  StyleBox["StringReverse", "MR"],
  " but accepts symbols as input and treats the empty word properly."
}], "Usage",
  CellTags->"WordReverse"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"WordReverse"],

Cell["Always returns a string as output.", "Notes",
  CellTags->"WordReverse"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordQ",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["WordRotate",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WordReverse"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordReverse"],

Cell["Flipping words, even when they are given as symbols. ", "ExampleText",
  CellTags->"WordReverse"],

Cell["WordReverse[ aaab ]", "Input",
  CellLabel->"In[112]:=",
  CellTags->"WordReverse"],

Cell["The output is a string.", "ExampleText",
  CellTags->"WordReverse"],

Cell[BoxData[
    \(WordReverse[aaab] // \ FullForm\)], "Input",
  CellLabel->"In[117]:=",
  CellTags->"WordReverse"],

Cell["The operation is an involution.", "ExampleText",
  CellTags->"WordReverse"],

Cell[BoxData[
    \(Words[3]\  === \ WordReverse[WordReverse[Words[3]]]\)], "Input",
  CellLabel->"In[116]:=",
  CellTags->"WordReverse"],

Cell["WordRoot", "ObjectName",
  CellTags->"WordRoot"],

Cell["Function", "Highlight",
  CellTags->"WordRoot"],

Cell[TextData[{
  StyleBox["WordRoot[w]", "MR"],
  "  computes the root of a word ",
  StyleBox["w", "MR"],
  ", i.e., the shortest word ",
  StyleBox["u", "MR"],
  " such that ",
  StyleBox["w", "MR"],
  " is a power of ",
  StyleBox["u", "MR"],
  "."
}], "Usage",
  CellTags->"WordRoot"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"WordRoot"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordPrimitiveQ",
    ButtonData:>"WordPrimitiveQ",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["WordPrimitiveCount",
    ButtonData:>"WordPrimitiveCount",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["WordPower",
    ButtonData:>"WordPower",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WordRoot"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordRoot"],

Cell["All roots of non-primitive words of length 8.", "ExampleText",
  CellTags->"WordRoot"],

Cell["W = Select[Words[8],!WordPrimitiveQ[#]&]", "Input",
  CellLabel->"In[135]:=",
  CellTags->"WordRoot"],

Cell[BoxData[
    \(WordRoot[W]\)], "Input",
  CellLabel->"In[136]:=",
  CellTags->"WordRoot"],

Cell["WordRotate", "ObjectName",
  CellTags->"WordRotate"],

Cell["Function", "Highlight",
  CellTags->"WordRotate"],

Cell[TextData[{
  StyleBox["WordRotate[w,n]", "MR"],
  "  rotates the word ",
  StyleBox["w", "MR"],
  " by ",
  StyleBox["n", "MR"],
  " places to the left.  "
}], "Usage",
  CellTags->"WordRotate"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"WordRotate"],

Cell[TextData[{
  StyleBox["n", "MR"],
  " has default 1."
}], "Notes",
  CellTags->"WordRotate"],

Cell["Treats epsilon properly. ", "Notes",
  CellTags->"WordRotate"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordConjugateQ",
    ButtonData:>"WordConjugateQ",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["WordConjugates",
    ButtonData:>"WordConjugates",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WordRotate"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordRotate"],

Cell["All rotations of a word.", "ExampleText",
  CellTags->"WordRotate"],

Cell["Table[ WordRotate[aaaab,n],{n,0,4}]", "Input",
  CellLabel->"In[142]:=",
  CellTags->"WordRotate"],

Cell[BoxData[
    \(WordConjugateQ[aaaab, %]\)], "Input",
  CellLabel->"In[143]:=",
  CellTags->"WordRotate"],

Cell["Words", "ObjectName",
  CellTags->"Words"],

Cell["Function", "Highlight",
  CellTags->"Words"],

Cell[TextData[{
  StyleBox["Words[r,K]", "MR"],
  "  produces a list of words of length ",
  StyleBox["r", "MR"],
  " over alphabet ",
  StyleBox["K", "MR"],
  ". "
}], "Usage",
  CellTags->"Words"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "True", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"Words"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->False", "MR"],
  "  omits ",
  Cell[BoxData[
      \(TraditionalForm\`\[Epsilon]\)]],
  " from the output (for negative ",
  Cell[BoxData[
      \(TraditionalForm\`r\)]],
  " only)."
}], "Notes",
  CellTags->"Words"],

Cell[TextData[{
  "For negative ",
  StyleBox["r", "MR"],
  " all words up to length ",
  StyleBox["-r", "MR"],
  " are generated. "
}], "Notes",
  CellTags->"Words"],

Cell["One symbol alphabets produce a flattened list.", "Notes",
  CellTags->"Words"],

Cell[TextData[{
  "The defaultvalue for  ",
  StyleBox["K", "MR"],
  "  is ",
  ButtonBox["$alphabet",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Words"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordQ",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Words"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Words"],

Cell["Default alphabet and user selected alphabets.", "ExampleText",
  CellTags->"Words"],

Cell[" Words[3]", "Input",
  CellLabel->"In[118]:=",
  CellTags->"Words"],

Cell[BoxData[{
    \(Words[\(-3\)]\), "\n", 
    \(Words[\(-3\), \ Full \[Rule] False]\)}], "Input",
  CellLabel->"In[119]:=",
  CellTags->"Words"],

Cell[BoxData[
    \(Words[3, 3]\)], "Input",
  CellLabel->"In[121]:=",
  CellTags->"Words"],

Cell[BoxData[
    \(Words[3, {"\<x\>", "\<y\>"}]\)], "Input",
  CellLabel->"In[122]:=",
  CellTags->"Words"],

Cell[TextData[{
  "Note that the size of the output grows exponentially (except for \
one-letter alphabets), so ",
  StyleBox["Words", "MR"],
  " is to be used with caution."
}], "ExampleText",
  CellTags->"Words"],

Cell[BoxData[
    \(Table[\ Length[Words[i]], {i, 10}]\)], "Input",
  CellLabel->"In[123]:=",
  CellTags->"Words"],

Cell["WordShuffle", "ObjectName",
  CellTags->"WordShuffle"],

Cell["Function", "Highlight",
  CellTags->"WordShuffle"],

Cell[TextData[{
  StyleBox["WordShuffle[u,v]", "MR"],
  "  generates the shuffle language of words ",
  StyleBox["u", "MR"],
  " and ",
  StyleBox["v", "MR"],
  "."
}], "Usage",
  CellTags->"WordShuffle"],

Cell["Also accepts lists of words as input.", "Notes",
  CellTags->"WordShuffle"],

Cell[TextData[{
  "See also  ",
  ButtonBox["ShuffleProductFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WordShuffle"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordShuffle"],

Cell["\<\
The size of the output depends on the input words, not just their \
lengths. Note that repetitions are eliminated.\
\>", "ExampleText",
  CellTags->"WordShuffle"],

Cell["WordShuffle[ aaa, aaa ]", "Input",
  CellLabel->"In[99]:=",
  CellTags->"WordShuffle"],

Cell["WordShuffle[ aaa, bbb ]", "Input",
  CellLabel->"In[100]:=",
  CellTags->"WordShuffle"],

Cell[TextData[{
  StyleBox["WordShuffle", "MR"],
  " behaves reasonably when the input consists of lists of words."
}], "ExampleText",
  CellTags->"WordShuffle"],

Cell["WordShuffle[ aa, {bb,cc} ]", "Input",
  CellLabel->"In[101]:=",
  CellTags->"WordShuffle"],

Cell["WordShuffle[ {a,aa}, {bb,cc} ]", "Input",
  CellLabel->"In[102]:=",
  CellTags->"WordShuffle"],

Cell["\<\
The size of the shuffle product of two words is fairly \
erratic.\
\>", "ExampleText",
  CellTags->"WordShuffle"],

Cell[BoxData[
    \(\(Length[WordShuffle[aabb, #]] &\)\  /@ \ \ Words[4]\)], "Input",
  CellLabel->"In[130]:=",
  CellTags->"WordShuffle"],

Cell["WordSort", "ObjectName",
  CellTags->"WordSort"],

Cell["Function", "Highlight",
  CellTags->"WordSort"],

Cell[TextData[{
  StyleBox["WordSort[L]", "MR"],
  "  sorts a list ",
  StyleBox["L", "MR"],
  " of words in length-lex order: first by length, then lexicographically. "
}], "Usage",
  CellTags->"WordSort"],

Cell[TextData[{
  "For example, on alphabet ",
  Cell[BoxData[
      \(TraditionalForm\`{a, b}\)]],
  " the order is ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\[Epsilon]\)\(,\)\(\ \)\(a\)\(,\)\(\ \
\)\(b\)\(,\)\(\ \)\(a\[VeryThinSpace]a\)\(,\)\(\ \
\)\(a\[VeryThinSpace]b\)\(,\)\(\ \)\(b\[VeryThinSpace]a\)\(,\)\(\ \)\(b\
\[VeryThinSpace]b\)\(,\)\(\ \)\(a\[VeryThinSpace]a\[VeryThinSpace]a\)\(,\)\(\ \
\)\(a\[VeryThinSpace]a\[VeryThinSpace]b\)\(\ \)\(,\)\(\[Ellipsis]\)\(\ \
\)\)\)]]
}], "Notes",
  CellTags->"WordSort"],

Cell["\<\
For inductive methods this order is preferable over the standard \
lexicographic order.\
\>", "Notes",
  CellTags->"WordSort"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordOrder",
    ButtonData:>"WordOrder",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WordSort"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordSort"],

Cell["\<\
The built-in sorting algorithm produces undesirable results when \
applied to sets of words.\
\>", "ExampleText",
  CellTags->"WordSort"],

Cell[" Sort[ Words[-3]]", "Input",
  CellLabel->"In[146]:=",
  CellTags->"WordSort"],

Cell[BoxData[
    \(WordSort[%]\)], "Input",
  CellLabel->"In[147]:=",
  CellTags->"WordSort"],

Cell["WordSuffixQ", "ObjectName",
  CellTags->"WordSuffixQ"],

Cell["Function", "Highlight",
  CellTags->"WordSuffixQ"],

Cell[TextData[{
  StyleBox["WordSuffixQ[x,y]", "MR"],
  "  tests whether word ",
  StyleBox["x", "MR"],
  " is a suffix of word ",
  StyleBox["y", "MR"],
  "."
}], "Usage",
  CellTags->"WordSuffixQ"],

Cell["\<\
Also works on symbols and treats  \[Epsilon]  properly.\
\>", \
"Notes",
  CellTags->"WordSuffixQ"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordQ",
    ButtonData:>"WordQ",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["WordPrefixQ",
    ButtonData:>"WordPrefixQ",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["WordFactorQ",
    ButtonData:>"WordFactorQ",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WordSuffixQ"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordSuffixQ"],

Cell["Some simple suffix queries.", "ExampleText",
  CellTags->"WordSuffixQ"],

Cell[BoxData[
    \(WordSuffixQ[c, abc]\)], "Input",
  CellLabel->"In[138]:=",
  CellTags->"WordSuffixQ"],

Cell[BoxData[
    \(WordSuffixQ[b, abc]\)], "Input",
  CellLabel->"In[139]:=",
  CellTags->"WordSuffixQ"],

Cell[BoxData[
    \(WordSuffixQ[\[Epsilon], abc]\)], "Input",
  CellLabel->"In[140]:=",
  CellTags->"WordSuffixQ"],

Cell[BoxData[
    \(Select[\ Words[\(-5\)], \ WordSuffixQ[aaa, #] &\ ]\)], "Input",
  CellLabel->"In[141]:=",
  CellTags->"WordSuffixQ"],

Cell["WordSuffixes", "ObjectName",
  CellTags->"WordSuffixes"],

Cell["Function", "Highlight",
  CellTags->"WordSuffixes"],

Cell[TextData[{
  StyleBox["WordSuffixes[w]", "MR"],
  "  generates all suffixes of word ",
  StyleBox["w", "MR"],
  ", in length order. "
}], "Usage",
  CellTags->"WordSuffixes"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "True", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"WordSuffixes"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->False", "MR"],
  "  omits ",
  StyleBox["w", "MR"],
  " from the list of suffixes."
}], "Notes",
  CellTags->"WordSuffixes"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"WordSuffixes"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordPrefixes",
    ButtonData:>"WordPrefixes",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WordFactors",
    ButtonData:>"WordFactors",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["WordToFA",
    ButtonData:>"WordToFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WordSuffixes"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordSuffixes"],

Cell[TextData[{
  "The output list has length ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(|\)\(w\)\(|\)\(+1\)\)\)]],
  "."
}], "ExampleText",
  CellTags->"WordSuffixes"],

Cell[" WordSuffixes[abcabc]", "Input",
  CellLabel->"In[433]:=",
  CellTags->"WordSuffixes"],

Cell[" WordSuffixes[{abc,cba}]", "Input",
  CellLabel->"In[434]:=",
  CellTags->"WordSuffixes"],

Cell["All  proper suffixes.", "ExampleText",
  CellTags->"WordSuffixes"],

Cell[BoxData[
    \(WordSuffixes[baaa, Full \[Rule] False]\)], "Input",
  CellLabel->"In[447]:=",
  CellTags->"WordSuffixes"],

Cell["$bottom", "ObjectName",
  CellTags->"$bottom"],

Cell["SysVariable", "Highlight",
  CellTags->"$bottom"],

Cell[TextData[{
  StyleBox["$bottom", "MR"],
  "   is the default object used for the trap state added to machines by ",
  StyleBox["CompleteFA", "MR"],
  ". "
}], "Usage",
  CellTags->"$bottom"],

Cell["Default value is -1.", "Notes",
  CellTags->"$bottom"],

Cell["\<\
Is only used when the input machine has a structured state set. \
\
\>", "Notes",
  CellTags->"$bottom"],

Cell[TextData[{
  "See also ",
  ButtonBox["CompleteFA",
    ButtonData:>"CompleteFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"$bottom"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$bottom"],

Cell["A machine with a structured state set and its completion.", \
"ExampleText",
  CellTags->"$bottom"],

Cell[BoxData[{
    \(\(m\  = \ ToFA@BuildDFA[aab, aac, 3, Normalize \[Rule] 1];\)\), "\n", 
    \(mm\  = \ 
      ToFactorFA[m, Type \[Rule] \ "\<Prefix\>", 
        Normalize \[Rule] 2]\)}], "Input",
  CellLabel->"In[614]:=",
  CellTags->"$bottom"],

Cell[BoxData[{
    \(mc\  = \ CompleteFA[mm]\), "\n", 
    \(mm\  // \ States\), "\n", 
    \(mc\  // \ States\)}], "Input",
  CellLabel->"In[621]:=",
  CellTags->"$bottom"],

Cell["The name of the added sink state can be chosen by the user.", \
"ExampleText",
  CellTags->"$bottom"],

Cell[BoxData[{
    \(\(Block[{\ $bottom\  = \ "\<\[UpTee]\>"}, CompleteFA[mm]];\)\), "\n", 
    \(States[%]\)}], "Input",
  CellLabel->"In[624]:=",
  CellTags->"$bottom"],

Cell["$defaultnormalize", "ObjectName",
  CellTags->"$defaultnormalize"],

Cell["SysVariable", "Highlight",
  CellTags->"$defaultnormalize"],

Cell[TextData[{
  StyleBox["$defaultnormalize", "MR"],
  "  is the default value used for all commands with option ",
  StyleBox["Normalize", "MR"],
  "."
}], "Usage",
  CellTags->"$defaultnormalize"],

Cell[TextData[{
  "The default setting for ",
  StyleBox["$defaultnormalize", "MR"],
  " is 0, meaning that all machines are normalized."
}], "Notes",
  CellTags->"$defaultnormalize"],

Cell[TextData[{
  "A normalized machine has as state set ",
  Cell[BoxData[
      \(TraditionalForm\`{1, 2, \[Ellipsis], n}\)]],
  "  where ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " is the size of the machine. Needless to say, this list is not stored in \
the machine."
}], "Notes",
  CellTags->"$defaultnormalize"],

Cell[TextData[{
  "Setting ",
  StyleBox["$defaultnormalize = 1", "MR"],
  " will produce state sets whose type determined by the operation performed, \
but the input machines are all treated as if their state sets were \
normalized. For example, ",
  StyleBox["ToDFA", "MR"],
  " will produce a subset of the power set of ",
  Cell[BoxData[
      \(TraditionalForm\`\([n]\)\)]],
  " as state set."
}], "Notes",
  CellTags->"$defaultnormalize"],

Cell[TextData[{
  "Setting ",
  StyleBox["$defaultnormalize = 2", "MR"],
  " ",
  "will not only preserve the state structure with respect to the last \
operation, but also the state sets of the input machines. "
}], "Notes",
  CellTags->"$defaultnormalize"],

Cell[TextData[{
  "Note that the memory requirement of non-normalized machines may be \
significantly larger than for their normalized counterparts. This true in \
particular for machines produced by command pipelines like ",
  StyleBox["ProductSA[m1,m2] // ToDFA // MinimizeFA", "MR"],
  "."
}], "Notes",
  CellTags->"$defaultnormalize"],

Cell[TextData[{
  "See also ",
  ButtonBox["Normalize",
    ButtonData:>"Normalize",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["NormalizeFA",
    ButtonData:>"NormalizeFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"$defaultnormalize"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$defaultnormalize"],

Cell["By default all machines are normalized.", "ExampleText",
  CellTags->"$defaultnormalize"],

Cell["ToDFA[ IthSymbolFA[b,-3] ] // States", "Input",
  CellLabel->"In[150]:=",
  CellTags->"$defaultnormalize"],

Cell["We can override locally.", "ExampleText",
  CellTags->"$defaultnormalize"],

Cell[BoxData[
    \(Block[{$defaultnormalize = 1}, 
      States[ToDFA[IthSymbolFA[b, \(-3\)]]]]\)], "Input",
  CellLabel->"In[152]:=",
  CellTags->"$defaultnormalize"],

Cell[BoxData[
    \(ToDFA[IthSymbolFA[b, \(-3\)], Normalize \[Rule] 1]\  // \ 
      States\)], "Input",
  CellLabel->"In[155]:=",
  CellTags->"$defaultnormalize"],

Cell["AcceptQFA", "ObjectName",
  CellTags->"AcceptQFA"],

Cell["Function", "Highlight",
  CellTags->"AcceptQFA"],

Cell[TextData[{
  StyleBox["AcceptQFA[M,w]", "MR"],
  "  tests whether machine ",
  StyleBox["M", "MR"],
  " accepts input ",
  StyleBox["w", "MR"],
  ". "
}], "Usage",
  CellTags->"AcceptQFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"DoTrace", "\[Rule]", "False", "-"},
          {"Multiplicity", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"AcceptQFA"],

Cell[TextData[{
  "Option ",
  StyleBox["DoTrace->True", "MR"],
  " produces a trace of the computation. "
}], "Notes",
  CellTags->"AcceptQFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Multiplicity->True", "MR"],
  " computes the multiplicity of ",
  Cell[BoxData[
      \(TraditionalForm\`w\)]],
  " with respect to ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  "."
}], "Notes",
  CellTags->"AcceptQFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["ComputationFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["PlotComputationFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["LanguageFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"AcceptQFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"AcceptQFA"],

Cell[TextData[{
  "A machine for all words containting exactly two ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  "'s."
}], "ExampleText",
  CellTags->"AcceptQFA"],

Cell["M = CounterDFA[a,2]", "Input",
  CellLabel->"In[142]:=",
  CellTags->"AcceptQFA"],

Cell[BoxData[
    \(AcceptQFA[M, babab]\)], "Input",
  CellLabel->"In[143]:=",
  CellTags->"AcceptQFA"],

Cell[BoxData[
    \(AcceptQFA[M, babab, DoTrace \[Rule] True]\)], "Input",
  CellLabel->"In[144]:=",
  CellTags->"AcceptQFA"],

Cell["All accepted words of length 4.", "ExampleText",
  CellTags->"AcceptQFA"],

Cell[BoxData[
    \(Select[\ Words[4], AcceptQFA[M, #] &\ ]\)], "Input",
  CellLabel->"In[145]:=",
  CellTags->"AcceptQFA"],

Cell["Of course, the last computation should be done thus:", "ExampleText",
  CellTags->"AcceptQFA"],

Cell[BoxData[
    \(LanguageFA[\ M, \ 4\ ]\)], "Input",
  CellLabel->"In[146]:=",
  CellTags->"AcceptQFA"],

Cell["AccessibleFA", "ObjectName",
  CellTags->"AccessibleFA"],

Cell["Function", "Highlight",
  CellTags->"AccessibleFA"],

Cell[TextData[{
  StyleBox["AccessibleFA[M]", "MR"],
  "  constructs the accessible part of a FSM ",
  StyleBox["M", "MR"],
  ". "
}], "Usage",
  CellTags->"AccessibleFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"},
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"AccessibleFA"],

Cell["The initial state of a DFA is renumbered to 1. ", "Notes",
  CellTags->"AccessibleFA"],

Cell["\<\
The operation is idempotent and can be used to check equivalence of \
DFAs. \
\>", "Notes",
  CellTags->"AccessibleFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["CoAccessibleFA",
    ButtonData:>"CoAccessibleFA",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["TrimFA",
    ButtonData:>"TrimFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"AccessibleFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"AccessibleFA"],

Cell["\<\
 A machine on 5 states, two of which fail to be accessible.  The \
accessible part is equivalent to the original machine.\
\>", "ExampleText",
  CellTags->"AccessibleFA"],

Cell[BoxData[
    \(m\  = \ 
      FA[\ 5, \ 2, \ {{1, 1, 3}, {3, 2, 4}, {2, 1, 1}, {2, 2, 
            5}}, {1}, {4}, {}, {a, b, c, d, e}]\)], "Input",
  CellLabel->"In[417]:=",
  CellTags->"AccessibleFA"],

Cell[BoxData[{
    \(ma\  = \ AccessibleFA[m]\), "\n", 
    \(EquivalentQFA[m, ma]\)}], "Input",
  CellLabel->"In[415]:=",
  CellTags->"AccessibleFA"],

Cell[" We can preserve the state set of the original machine.", "ExampleText",
  CellTags->"AccessibleFA"],

Cell[BoxData[
    \(AccessibleFA[m, Normalize \[Rule] 1]\  // \ States\)], "Input",
  CellLabel->"In[13]:=",
  CellTags->"AccessibleFA"],

Cell[BoxData[
    \(AccessibleFA[m, Normalize \[Rule] 2]\  // \ States\)], "Input",
  CellLabel->"In[14]:=",
  CellTags->"AccessibleFA"],

Cell["AddTransitionFA", "ObjectName",
  CellTags->"AddTransitionFA"],

Cell["Function", "Highlight",
  CellTags->"AddTransitionFA"],

Cell[TextData[{
  StyleBox["AddTransitionFA[M,p,w,q]", "MR"],
  "  adds the transition ",
  StyleBox["{p,w,q}", "MR"],
  " to ",
  StyleBox["M", "MR"],
  ". "
}], "Usage",
  CellTags->"AddTransitionFA"],

Cell[TextData[{
  "Here ",
  StyleBox["w", "MR"],
  " may be a word in which case a sequence of transitions is inserted."
}], "Notes",
  CellTags->"AddTransitionFA"],

Cell[TextData[{
  "Can also be used in the form ",
  StyleBox["AddTransitionFA[M,{p,w,q}]", "MR"],
  " or ",
  StyleBox["AddTransitionFA[M,L]", "MR"],
  " where ",
  StyleBox["L", "MR"],
  " is a list of transitions."
}], "Notes",
  CellTags->"AddTransitionFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["DeleteTransitionFA",
    ButtonData:>"DeleteTransitionFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SwitchTransitionFA",
    ButtonData:>"SwitchTransitionFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"AddTransitionFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"AddTransitionFA"],

Cell["A machine recognizing only the word aaa.", "ExampleText",
  CellTags->"AddTransitionFA"],

Cell[BoxData[{
    \(m\  = \ 
      FA[\ 4, \ 3, \ {{1, 1, 2}, {2, 1, 3}, {3, 1, 4}}, {1}, {4}]\), "\n", 
    \(LanguageFA[m, \(-5\)]\)}], "Input",
  CellLabel->"In[241]:=",
  CellTags->"AddTransitionFA"],

Cell[TextData[{
  "We add a back-transition labeled ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  ", specified in index notation."
}], "ExampleText",
  CellTags->"AddTransitionFA"],

Cell[BoxData[{
    \(m1 = AddTransitionFA[m, \ 4, 2, 2\ \ ]\), "\n", 
    \(LanguageFA[m1, \(-15\)\ ]\)}], "Input",
  CellLabel->"In[243]:=",
  CellTags->"AddTransitionFA"],

Cell[TextData[{
  "And another back-transition labeled ",
  Cell[BoxData[
      \(TraditionalForm\`c\[VeryThinSpace]c\)]],
  "."
}], "ExampleText",
  CellTags->"AddTransitionFA"],

Cell[BoxData[{
    \(\(m2 = AddTransitionFA[m1, \ 4, cc, 2\ \ ];\)\), "\n", 
    \(LanguageFA[m2, \(-15\)\ ]\)}], "Input",
  CellLabel->"In[247]:=",
  CellTags->"AddTransitionFA"],

Cell["AllDFA", "ObjectName",
  CellTags->"AllDFA"],

Cell["Function", "Highlight",
  CellTags->"AllDFA"],

Cell[TextData[{
  StyleBox["AllDFA[K]", "MR"],
  "  constructs a DFA that accepts all words over alphabet ",
  StyleBox["K", "MR"],
  "."
}], "Usage",
  CellTags->"AllDFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "True", "False"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"AllDFA"],

Cell[TextData[{
  "With option  ",
  StyleBox["Full\[Rule]False", "MR"],
  "  only non-empty words are accepted."
}], "Notes",
  CellTags->"AllDFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["LengthDFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["CounterDFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["StarFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"AllDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"AllDFA"],

Cell[TextData[{
  "A DFA accepting all non-empty words over the alphabet ",
  Cell[BoxData[
      \(TraditionalForm\`{a, b, c}\)]],
  ". "
}], "ExampleText",
  CellTags->"AllDFA"],

Cell[BoxData[
    \(AllDFA[{a, b, c}, Full \[Rule] False]\)], "Input",
  CellLabel->"In[438]:=",
  CellTags->"AllDFA"],

Cell[BoxData[
    \(LanguageFA[%, \(-3\)]\)], "Input",
  CellLabel->"In[121]:=",
  CellTags->"AllDFA"],

Cell["AmbiguousQFA", "ObjectName",
  CellTags->"AmbiguousQFA"],

Cell["Function", "Highlight",
  CellTags->"AmbiguousQFA"],

Cell[TextData[{
  StyleBox["AmbiguousQFA[M,opts]", "MR"],
  "  tests whether FSM ",
  StyleBox["M", "MR"],
  "  is ambiguous. "
}], "Usage",
  CellTags->"AmbiguousQFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"},
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"AmbiguousQFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  " in addition returns a witness for ambiguity (provided the input machine \
is ambiguous). "
}], "Notes",
  CellTags->"AmbiguousQFA"],

Cell["\<\
The algorithm uses a product automaton construction and is \
therefore quadratic.\
\>", "Notes",
  CellTags->"AmbiguousQFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["ProductSA",
    ButtonData:>"ProductSA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["DefiniteQFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DeterministicQFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"AmbiguousQFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"AmbiguousQFA"],

Cell["A trivially ambiguous machine.", "ExampleText",
  CellTags->"AmbiguousQFA"],

Cell[TextData[{
  "s = SA[4,2,{{1,1,2},{2,2,4},{1,1,3},{3,2,4}}];",
  "\n",
  Cell[BoxData[
      \(AmbiguousQFA[s]\)], "Input",
    CellLabel->"In[4]:="]
}], "Input",
  CellLabel->"In[2068]:=",
  CellTags->"AmbiguousQFA"],

Cell["An unambiguous machine based on a cellular automaton.", "ExampleText",
  CellTags->"AmbiguousQFA"],

Cell[BoxData[{
    \(\(s\  = \ ToSA[CA[90]];\)\), "\n", 
    \(AmbiguousQFA[s]\)}], "Input",
  CellLabel->"In[2070]:=",
  CellTags->"AmbiguousQFA"],

Cell["\<\
An ambiguous machine based on a cellular automaton and a witness \
for ambiguity. Note that the witness is not minimal.\
\>", "ExampleText",
  CellTags->"AmbiguousQFA"],

Cell[BoxData[{
    \(\(s\  = \ ToSA[CA[73]];\)\), "\n", 
    \(AmbiguousQFA[s, Full \[Rule] True]\)}], "Input",
  CellLabel->"In[2074]:=",
  CellTags->"AmbiguousQFA"],

Cell[BoxData[
    \(\(PlotComputationFA[\ s, \ "\<00000000\>"];\)\)], "Input",
  CellLabel->"In[2078]:=",
  CellTags->"AmbiguousQFA"],

Cell["BEFA", "ObjectName",
  CellTags->"BEFA"],

Cell["Constructor", "Highlight",
  CellTags->"BEFA"],

Cell[TextData[{
  StyleBox["BEFA[n,T]", "MR"],
  "  denotes a begin-exit automaton, a special type of nondeterminisitc \
automaton that has exactly one initial state (begin) and one final state \
(exit). "
}], "Usage",
  CellTags->"BEFA"],

Cell[TextData[{
  "No transitions end at the begin and no transitions start at the exit. By \
convention, 1 is the initial state (begin) and ",
  StyleBox["n", "MR"],
  " is the final state (exit) of  ",
  StyleBox["BEFA[n,T]", "MR"],
  "."
}], "Notes",
  CellTags->"BEFA"],

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`T\)]],
  " is a list of transitions ",
  Cell[BoxData[
      \(TraditionalForm\`{p, a, q}\)]],
  " where the states are in positional notation but the symbol is a \
character. "
}], "Notes",
  CellTags->"BEFA"],

Cell["Used only in the conversion of regular expressions to FAs.", "Notes",
  CellTags->"BEFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["RegularExpressionToBEFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"BEFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"BEFA"],

Cell[TextData[{
  "A BEFA for the language  ",
  Cell[BoxData[
      \(TraditionalForm\`a\ \(\((a + b)\)\^*\)\)]],
  ".  Note the \[Epsilon] transitions."
}], "ExampleText",
  CellTags->"BEFA"],

Cell[TextData[Cell[BoxData[
    \(RegularExpressionToBEFA[\ RET[a, RES[REP[a, b]]]]\)], "Input",
  CellLabel->"In[60]:="]], "Input",
  CellLabel->"In[147]:=",
  CellTags->"BEFA"],

Cell["It is easy to generate a trace of the conversion process.", \
"ExampleText",
  CellTags->"BEFA"],

Cell["\<\
Trace[RegularExpressionToBEFA[ RET[a,RES[REP[a,b]]]],BEFA[__]] // \
Flatten // Union // TableForm\
\>", "Input",
  CellLabel->"In[150]:=",
  CellTags->"BEFA"],

Cell["BEFAToFA", "ObjectName",
  CellTags->"BEFAToFA"],

Cell["Function", "Highlight",
  CellTags->"BEFAToFA"],

Cell[TextData[{
  StyleBox["BEFAToFA", "MR"],
  StyleBox["[M]", "MR"],
  "  converts BEFA ",
  StyleBox["M", "MR"],
  "  into an FA."
}], "Usage",
  CellTags->"BEFAToFA"],

Cell["Only useful in connecton with the finite power property.", "Notes",
  CellTags->"BEFAToFA"],

Cell[TextData[{
  "See ",
  ButtonBox["TransitionMatrixFA",
    ButtonData:>"TransitionMatrixFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Tropical",
    ButtonData:>"Tropical",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"BEFAToFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"BEFAToFA"],

Cell["A small BEFA and its corresponding FA. ", "ExampleText",
  CellTags->"BEFAToFA"],

Cell[BoxData[
    \(m\  = \ RegularExpressionToBEFA[\ REP[aa, bb]]\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->"BEFAToFA"],

Cell[BoxData[
    \(BEFAToFA[m, 2]\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->"BEFAToFA"],

Cell["BehaviorFA", "ObjectName",
  CellTags->"BehaviorFA"],

Cell["Function", "Highlight",
  CellTags->"BehaviorFA"],

Cell[TextData[{
  StyleBox["BehaviorFA[M,p]", "MR"],
  "  constructs the minimal DFA whose acceptance language is the behavior of \
state ",
  StyleBox["p", "MR"],
  " in automaton ",
  StyleBox["M", "MR"],
  "."
}], "Usage",
  CellTags->"BehaviorFA"],

Cell[TextData[{
  StyleBox["BehaviorFA[M,p,r]", "MR"],
  "  constructs part of the behavior of state ",
  StyleBox["p", "MR"],
  " in automaton ",
  StyleBox["M", "MR"],
  " , the acceptance language of the machine ",
  StyleBox[" BehaviorFA[M,p]", "MR"],
  ". Only words of length ",
  StyleBox["r", "MR"],
  "  are generated."
}], "Notes",
  CellTags->"BehaviorFA"],

Cell[TextData[{
  "For negative values of ",
  StyleBox["r", "MR"],
  " all words up to length ",
  StyleBox["Abs[r]", "MR"],
  "  are generated."
}], "Notes",
  CellTags->"BehaviorFA"],

Cell["\<\
Uses conversion to a deterministic machine for nondeterministic \
inputs; may have exponential time and space requirements.\
\>", "Notes",
  CellTags->"BehaviorFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["LanguageFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToDFA",
    ButtonStyle->"AddOnsLink"],
  " ."
}], "Notes",
  CellTags->"BehaviorFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"BehaviorFA"],

Cell[TextData[{
  "A machine accepting all words containing a number of ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  "'s congruent to 1 modulo 3."
}], "ExampleText",
  CellTags->"BehaviorFA"],

Cell[BoxData[
    \(m = CounterDFA[a, 1, 2, Modulus \[Rule] 3]\)], "Input",
  CellLabel->"In[151]:=",
  CellTags->"BehaviorFA"],

Cell["\<\
The behavior of the initial state 1 is just the language of the \
machine. \
\>", "ExampleText",
  CellTags->"BehaviorFA"],

Cell[BoxData[
    \(BehaviorFA[m, 1] === \ m\)], "Input",
  CellLabel->"In[152]:=",
  CellTags->"BehaviorFA"],

Cell[TextData[{
  "But state 2 has as behavior all words containing a multiple of 3 ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  "'s."
}], "ExampleText",
  CellTags->"BehaviorFA"],

Cell[BoxData[{
    \(BehaviorFA[m, 2]\), "\n", 
    \(LanguageFA[%, \(-4\)]\)}], "Input",
  CellLabel->"In[155]:=",
  CellTags->"BehaviorFA"],

Cell[BoxData[
    \(BehaviorFA[m, 2, \(-4\)]\)], "Input",
  CellLabel->"In[157]:=",
  CellTags->"BehaviorFA"],

Cell["Boundary", "ObjectName",
  CellTags->"Boundary"],

Cell["Option", "Highlight",
  CellTags->"Boundary"],

Cell[TextData[{
  StyleBox["Boundary", "MR"],
  "  is an option for ",
  StyleBox["ToBlocks", "MR"],
  ". "
}], "Usage",
  CellTags->"Boundary"],

Cell[TextData[{
  "Acceptable values are ",
  StyleBox["Cyclic", "MR"],
  ", ",
  StyleBox["Fixed", "MR"],
  " and ",
  StyleBox["None", "MR"],
  "."
}], "Notes",
  CellTags->"Boundary"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToBlocks",
    ButtonData:>"ToBlocks",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["EvolutionCA",
    ButtonData:>"EvolutionCA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["OrbitCA",
    ButtonData:>"OrbitCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Boundary"],

Cell["BuildBEFA", "ObjectName",
  CellTags->"BuildBEFA"],

Cell["Function", "Highlight",
  CellTags->"BuildBEFA"],

Cell[TextData[{
  StyleBox["BuildBEFA[w]", "MR"],
  "  constructs a BEFA (begin/exit automaton) that accepts exactly the word \
",
  StyleBox["w", "MR"],
  "."
}], "Usage",
  CellTags->"BuildBEFA"],

Cell[TextData[{
  "Returns a machine of type ",
  StyleBox["BEFA", "MR"],
  "."
}], "Notes",
  CellTags->"BuildBEFA"],

Cell["\<\
This is used in the Thompson style conversion of regular \
expressions to machines. \
\>", "Notes",
  CellTags->"BuildBEFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["BEFA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["RegularExpressionToBEFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"BuildBEFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"BuildBEFA"],

Cell[TextData[{
  "The initial state is implicitely assumed to be 1, and the final state ",
  StyleBox["n", "MR"],
  " where ",
  StyleBox["n", "MR"],
  " is the total number of states. "
}], "ExampleText",
  CellTags->"BuildBEFA"],

Cell["BuildBEFA[ abab ]", "Input",
  CellLabel->"In[158]:=",
  CellTags->"BuildBEFA"],

Cell["BuildDFA", "ObjectName",
  CellTags->"BuildDFA"],

Cell["Function", "Highlight",
  CellTags->"BuildDFA"],

Cell[TextData[{
  StyleBox["BuildDFA[w1,..,ws,K]", "MR"],
  "  constructs the minimal DFA for the finite language consisting of the \
given words, over alphabet ",
  StyleBox["K", "MR"],
  ". "
}], "Usage",
  CellTags->"BuildDFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"BuildDFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Normalize->1", "MR"],
  " causes the state set to be preserved in structured form: a set of sets of \
words, more precisely, all left quotients of ",
  StyleBox["L", "MR"],
  ". "
}], "Notes",
  CellTags->"BuildDFA"],

Cell["Also accepts a list of words as input.", "Notes",
  CellTags->"BuildDFA"],

Cell[TextData[{
  " See also ",
  ButtonBox["QuotientDFA",
    ButtonData:>"QuotientDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"BuildDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"BuildDFA"],

Cell["A language of cardinality 3.", "ExampleText",
  CellTags->"BuildDFA"],

Cell[BoxData[{
    \(m = BuildDFA[aa, bab, bbb, 2]\), "\n", 
    \(LanguageFA[m, \(-6\)]\)}], "Input",
  CellLabel->"In[159]:=",
  CellTags->"BuildDFA"],

Cell[TextData[{
  "All words of length 4, except for ",
  StyleBox["aaaa",
    FontSlant->"Italic"],
  "."
}], "ExampleText",
  CellTags->"BuildDFA"],

Cell[BoxData[{
    \(W = Rest[Words[4]]\), "\n", 
    \(m = \ BuildDFA[\ W, \ 2\ ]\), "\n", 
    \(LanguageFA[m, \(-6\)]\)}], "Input",
  CellLabel->"In[161]:=",
  CellTags->"BuildDFA"],

Cell["\<\
On occasion it is preferable to preserve the structured state \
set.\
\>", "ExampleText",
  CellTags->"BuildDFA"],

Cell[BoxData[
    \(BuildDFA[\ W, \ 2, Normalize \[Rule] 1\ ]\  // \ States\)], "Input",
  CellLabel->"In[166]:=",
  CellTags->"BuildDFA"],

Cell["BuildNNFA", "ObjectName",
  CellTags->"BuildNNFA"],

Cell["Function", "Highlight",
  CellTags->"BuildNNFA"],

Cell[TextData[{
  StyleBox["BuildNNFA[w]", "MR"],
  "  constructs a NNFA that accepts exactly the word ",
  StyleBox["w", "MR"],
  " (returns only the tail of the automaton)."
}], "Usage",
  CellTags->"BuildNNFA"],

Cell[TextData[{
  "Returns a machine of type ",
  StyleBox["NNFA", "MR"],
  "."
}], "Notes",
  CellTags->"BuildNNFA"],

Cell["\<\
Used only in the McNaugthon-Yamada conversion algorithm from \
regular expressions to machines.\
\>", "Notes",
  CellTags->"BuildNNFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["NNFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["RegularExpressionToNNFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"BuildNNFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"BuildNNFA"],

Cell["An NNFA for the word aab.", "ExampleText",
  CellTags->"BuildNNFA"],

Cell["BuildNNFA[aab]", "Input",
  CellLabel->"In[459]:=",
  CellTags->"BuildNNFA"],

Cell["BuildStarDFA", "ObjectName",
  CellTags->"BuildStarDFA"],

Cell["Function", "Highlight",
  CellTags->"BuildStarDFA"],

Cell[TextData[{
  StyleBox["BuildStarDFA[w]", "MR"],
  "  builds a DFA for the language of all words consisting of any number of \
repetitions of given word ",
  StyleBox["w", "MR"],
  "."
}], "Usage",
  CellTags->"BuildStarDFA"],

Cell["The initial and unique final state are both 1.", "Notes",
  CellTags->"BuildStarDFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["BuildDFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["StarFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"BuildStarDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"BuildStarDFA"],

Cell[TextData[{
  "A DFA accepting all words of the form ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          RowBox[{
            RowBox[{"(", 
              StyleBox["aabc",
                FontSlant->"Italic"], ")"}], " "}], "i"], TraditionalForm]]],
  ",  ",
  Cell[BoxData[
      \(TraditionalForm\`i\  \[GreaterEqual] \ 0\)]],
  "."
}], "ExampleText",
  CellTags->"BuildStarDFA"],

Cell[BoxData[
    \(m = BuildStarDFA[aabc, 3]\)], "Input",
  CellLabel->"In[169]:=",
  CellTags->"BuildStarDFA"],

Cell[BoxData[
    \(LanguageFA[m, \(-20\), \ SizeOnly \[Rule] True]\)], "Input",
  CellLabel->"In[170]:=",
  CellTags->"BuildStarDFA"],

Cell["\<\
The machine is (isomorphic to) the minimal DFA for the \
language.\
\>", "ExampleText",
  CellTags->"BuildStarDFA"],

Cell[BoxData[
    \(IsomorphicQDFA[\ MinimizeFA@StarFA[BuildDFA[aabc, 3]], m]\)], "Input",
  CellLabel->"In[173]:=",
  CellTags->"BuildStarDFA"],

Cell["CirculantSA", "ObjectName",
  CellTags->"CirculantSA"],

Cell["Function", "Highlight",
  CellTags->"CirculantSA"],

Cell[TextData[{
  StyleBox["CirculantSA[n,dd,lab,K]", "MR"],
  "  constructs a semiautomaton whose transition diagram is a circulant graph \
and whose transitions are given by the list ",
  StyleBox["lab", "MR"],
  "."
}], "Usage",
  CellTags->"CirculantSA"],

Cell[TextData[{
  "The underlying circulant graph is ",
  Cell[BoxData[
      \(TraditionalForm\`C(n; d)\)]],
  " where ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " denotes the number of nodes, and ",
  Cell[BoxData[
      \(TraditionalForm\`d\)]],
  " is a list of strides."
}], "Notes",
  CellTags->"CirculantSA"],

Cell[TextData[{
  "The strides are given by the integer list ",
  StyleBox["dd", "MR"],
  "."
}], "Notes",
  CellTags->"CirculantSA"],

Cell[TextData[{
  "The length of label list ",
  StyleBox["lab", "MR"],
  " must be  ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(n\)\(\ \)\(\[CenterDot]\)\) | d | \)]],
  ".   The labels are given in index notation."
}], "Notes",
  CellTags->"CirculantSA"],

Cell[TextData[{
  "See also ",
  ButtonBox["CirculantGraph",
    ButtonData:>"CirculantGraph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"CirculantSA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"CirculantSA"],

Cell[TextData[{
  "A circulant SA with stride list ",
  Cell[BoxData[
      \(TraditionalForm\`\((1, 2)\)\)]],
  " and exactly one ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  " transition. "
}], "ExampleText",
  CellTags->"CirculantSA"],

Cell[BoxData[
    \(m = CirculantSA[5, {1, 2}, Join[{1}, Table[2, {9}]]]\)], "Input",
  CellLabel->"In[174]:=",
  CellTags->"CirculantSA"],

Cell[BoxData[
    \(LanguageFA[m, 6]\)], "Input",
  CellLabel->"In[175]:=",
  CellTags->"CirculantSA"],

Cell["CoAccessibleFA", "ObjectName",
  CellTags->"CoAccessibleFA"],

Cell["Function", "Highlight",
  CellTags->"CoAccessibleFA"],

Cell[TextData[{
  StyleBox["CoAccessibleFA[M]", "MR"],
  "  returns the coaccessible part of a finite state machine ",
  StyleBox["M", "MR"],
  ". "
}], "Usage",
  CellTags->"CoAccessibleFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"},
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"CoAccessibleFA"],

Cell[TextData[{
  "Result will be an FA even if ",
  StyleBox["M", "MR"],
  " is a DFA and teh accessible part is complete."
}], "Notes",
  CellTags->"CoAccessibleFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["AccessibleFA",
    ButtonData:>"AccessibleFA",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["TrimFA",
    ButtonData:>"TrimFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"CoAccessibleFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"CoAccessibleFA"],

Cell["\<\
 A machine on 5 states, two of which fail to be coaccessible. \
\
\>", "ExampleText",
  CellTags->"CoAccessibleFA"],

Cell[BoxData[
    \(m\  = \ 
      FA[\ 5, \ 2, \ {{1, 1, 3}, {3, 2, 4}, {4, 1, 5}}, {1}, {4}, {}, {a, b, 
          c, d, e}]\)], "Input",
  CellLabel->"In[176]:=",
  CellTags->"CoAccessibleFA"],

Cell[BoxData[{
    \(CoAccessibleFA[m]\), "\n", 
    \(EquivalentQFA[m, %%]\)}], "Input",
  CellLabel->"In[177]:=",
  CellTags->"CoAccessibleFA"],

Cell[" We can preserve the state set of the original machine.", "ExampleText",
  CellTags->"CoAccessibleFA"],

Cell[BoxData[
    \(CoAccessibleFA[m, Normalize \[Rule] 1]\  // \ States\)], "Input",
  CellLabel->"In[179]:=",
  CellTags->"CoAccessibleFA"],

Cell[BoxData[
    \(CoAccessibleFA[m, Normalize \[Rule] 2]\  // \ States\)], "Input",
  CellLabel->"In[180]:=",
  CellTags->"CoAccessibleFA"],

Cell["ComplementFA", "ObjectName",
  CellTags->"ComplementFA"],

Cell["Function", "Highlight",
  CellTags->"ComplementFA"],

Cell[TextData[{
  StyleBox["ComplementFA[M]", "MR"],
  "  generates a DFA that accepts the complement of the acceptance language \
of machine ",
  StyleBox["M", "MR"],
  ". "
}], "Usage",
  CellTags->"ComplementFA"],

Cell[TextData[{
  "For a DFA ",
  StyleBox["M", "MR"],
  " this is accomplished by interchanging final and non-final states, but a \
nondeterministic machine is converted to a DFA first.  Hence, time and space \
requirements may be exponential."
}], "Notes",
  CellTags->"ComplementFA"],

Cell[TextData[{
  StyleBox["ComplementFA[M1,M2]", "MR"],
  " constructs a DFA for the language ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptCapitalL](M\_1)\  - \ \[ScriptCapitalL](
          M\_2)\)]],
  "."
}], "Notes",
  CellTags->"ComplementFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["UnionFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SumFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ProductFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["IntersectionFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ConcatenateFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["StarFA",
    ButtonStyle->"AddOnsLink"],
  "  and ",
  ButtonBox["ToDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ComplementFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ComplementFA"],

Cell["\<\
A DFA for all words of length other than 4 obtained from \
complementation.\
\>", "ExampleText",
  CellTags->"ComplementFA"],

Cell["m = BuildDFA[ Words[4], 2 ]", "Input",
  CellLabel->"In[181]:=",
  CellTags->"ComplementFA"],

Cell[BoxData[
    \(mm\  = \ ComplementFA[m]\)], "Input",
  CellLabel->"In[182]:=",
  CellTags->"ComplementFA"],

Cell[BoxData[
    \(LanguageFA[mm, \(-8\), SizeOnly \[Rule] True\ ]\)], "Input",
  CellLabel->"In[183]:=",
  CellTags->"ComplementFA"],

Cell["m5 = BuildDFA[ Words[-5], 2 ]", "Input",
  CellLabel->"In[184]:=",
  CellTags->"ComplementFA"],

Cell["A DFA for all words of length other than 4 and at most 5.", \
"ExampleText",
  CellTags->"ComplementFA"],

Cell[BoxData[
    \(mm\  = \ ComplementFA[\ m5, \ m\ ]\)], "Input",
  CellLabel->"In[185]:=",
  CellTags->"ComplementFA"],

Cell[BoxData[
    \(LanguageFA[mm, \(-8\), SizeOnly \[Rule] True\ ]\)], "Input",
  CellLabel->"In[186]:=",
  CellTags->"ComplementFA"],

Cell["CompleteFA", "ObjectName",
  CellTags->"CompleteFA"],

Cell["Function", "Highlight",
  CellTags->"CompleteFA"],

Cell[TextData[{
  StyleBox["CompleteFA[M]", "MR"],
  "  adds a sink to ",
  StyleBox["M", "MR"],
  " if necessary to produce a complete automaton."
}], "Usage",
  CellTags->"CompleteFA"],

Cell["\<\
Result is of type FA even if the machine is deterministic and thus \
a DFA.\
\>", "Notes",
  CellTags->"CompleteFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToDFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["CompleteQFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"CompleteFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"CompleteFA"],

Cell["An incomplete machine, and its complete counterpart. ", "ExampleText",
  CellTags->"CompleteFA"],

Cell["m = FA[2,2,{{1,1,2}},{1},{2}]", "Input",
  CellLabel->"In[190]:=",
  CellTags->"CompleteFA"],

Cell[BoxData[
    \(CompleteFA[m]\)], "Input",
  CellLabel->"In[191]:=",
  CellTags->"CompleteFA"],

Cell[BoxData[
    \(EquivalentQFA[%, %%]\)], "Input",
  CellLabel->"In[192]:=",
  CellTags->"CompleteFA"],

Cell["CompleteQFA", "ObjectName",
  CellTags->"CompleteQFA"],

Cell["Function", "Highlight",
  CellTags->"CompleteQFA"],

Cell[TextData[{
  StyleBox["CompleteQFA[M]", "MR"],
  "  tests whether the finite state machine ",
  StyleBox["M", "MR"],
  " is complete."
}], "Usage",
  CellTags->"CompleteQFA"],

Cell["\<\
 DFAs are trivially complete, for all other types a check is \
performed to see if all states are complete. \
\>", "Notes",
  CellTags->"CompleteQFA"],

Cell[" Note that the size of the alphabet is important here. ", "Notes",
  CellTags->"CompleteQFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToDFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["CompleteFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"CompleteQFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"CompleteQFA"],

Cell["An  complete machine over a 1-symbol alphabet.", "ExampleText",
  CellTags->"CompleteQFA"],

Cell["\<\
m = SA[2,1,{{1,1,2},{2,1,2}} ]
CompleteQFA[m]\
\>", "Input",
  CellLabel->"In[193]:=",
  CellTags->"CompleteQFA"],

Cell["But over a 2-symbol alphabet the same machine is incomplete.", \
"ExampleText",
  CellTags->"CompleteQFA"],

Cell["ReplacePart[m,2,2] // CompleteQFA", "Input",
  CellLabel->"In[195]:=",
  CellTags->"CompleteQFA"],

Cell["ComputationFA", "ObjectName",
  CellTags->"ComputationFA"],

Cell["Function", "Highlight",
  CellTags->"ComputationFA"],

Cell[TextData[{
  StyleBox["ComputationFA[M,w,opts]", "MR"],
  "  determines all computations of machine  ",
  StyleBox["M", "MR"],
  "  on an input word  ",
  StyleBox["w", "MR"],
  "  (starting at an initial state)."
}], "Usage",
  CellTags->"ComputationFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "True", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ComputationFA"],

Cell[TextData[{
  "Option  ",
  StyleBox["Full->False", "MR"],
  "  returns only the sequence of states, not the symbols."
}], "Notes",
  CellTags->"ComputationFA"],

Cell["\<\
A computation here is represented by a list whose entries alternate \
between states and symbols. The result is a collection of such lists.\
\>", \
"Notes",
  CellTags->"ComputationFA"],

Cell["\<\
Note that the number of computations may grow exponentially in the \
length of the input word.\
\>", "Notes",
  CellTags->"ComputationFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["ComputationGraphFA",
    ButtonData:>"ComputationGraphFA",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["PlotComputationFA",
    ButtonData:>"PlotComputationFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ComputationFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ComputationFA"],

Cell[TextData[{
  "A machine for the language of all words whose 4th symbol from the end is \
an ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  ". "
}], "ExampleText",
  CellTags->"ComputationFA"],

Cell[BoxData[
    \(m = IthSymbolFA[a, \(-4\)]\)], "Input",
  CellLabel->"In[197]:=",
  CellTags->"ComputationFA"],

Cell[TextData[{
  "All computations on input ",
  Cell[BoxData[
      FormBox[
        StyleBox["abbaaa",
          FontSlant->"Italic"], TraditionalForm]]],
  "."
}], "ExampleText",
  CellTags->"ComputationFA"],

Cell[BoxData[
    \(ComputationFA[\ m, \ abbaaa\ ]\)], "Input",
  CellLabel->"In[198]:=",
  CellTags->"ComputationFA"],

Cell["The same computations without the labels.", "ExampleText",
  CellTags->"ComputationFA"],

Cell[BoxData[
    \(ComputationFA[\ m, \ abbaaa, Full \[Rule] False\ ]\)], "Input",
  CellLabel->"In[201]:=",
  CellTags->"ComputationFA"],

Cell["The number of computations on all inputs of length 5.", "ExampleText",
  CellTags->"ComputationFA"],

Cell[BoxData[
    \(\(Length[ComputationFA[m, #]] &\)\  /@ \ Words[5]\)], "Input",
  CellLabel->"In[200]:=",
  CellTags->"ComputationFA"],

Cell["ComputationGraphFA", "ObjectName",
  CellTags->"ComputationGraphFA"],

Cell["Function", "Highlight",
  CellTags->"ComputationGraphFA"],

Cell[TextData[{
  StyleBox["ComputationGraphFA[M,w]", "MR"],
  "  produces the unfolding of the finite state machine ",
  StyleBox["M", "MR"],
  " with respect to input word ",
  StyleBox["w", "MR"],
  ". "
}], "Usage",
  CellTags->"ComputationGraphFA"],

Cell[TextData[{
  "Returns a ",
  StyleBox["Graph", "MR"],
  " object, is useful mostly for visualizations.  The vertex set of the graph \
has the form ",
  Cell[BoxData[
      \(TraditionalForm\`Q\ \[Cross]\ \([0, \(\(|\)\(w\)\(|\)\)]\)\)]],
  "."
}], "Notes",
  CellTags->"ComputationGraphFA"],

Cell[TextData[{
  "This command is essentially just a wrapper for ",
  ButtonBox["ToTDGraph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ComputationGraphFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToTDGraph",
    ButtonData:>"ToTDGraph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ComputationFA",
    ButtonData:>"ComputationFA",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["PlotComputationFA",
    ButtonData:>"PlotComputationFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ComputationGraphFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ComputationGraphFA"],

Cell[TextData[{
  "A machine for the language of all words whose 4th symbol from the end is \
an ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  ". "
}], "ExampleText",
  CellTags->"ComputationGraphFA"],

Cell[BoxData[
    \(m = IthSymbolFA[a, \(-4\)]\)], "Input",
  CellLabel->"In[202]:=",
  CellTags->"ComputationGraphFA"],

Cell[BoxData[
    \(\(\(ComputationGraphFA[m, aba, \ 
      Normalize \[Rule] 1\ ]\)\(\ \)\)\)], "Input",
  CellLabel->"In[205]:=",
  CellTags->"ComputationGraphFA"],

Cell["ConcatenateFA", "ObjectName",
  CellTags->"ConcatenateFA"],

Cell["Function", "Highlight",
  CellTags->"ConcatenateFA"],

Cell[TextData[{
  StyleBox["ConcatenateFA[M1,M2]", "MR"],
  "  generates a machine that accepts the concatenation of the acceptance \
languages of ",
  StyleBox["M1", "MR"],
  " and ",
  StyleBox["M2.", "MR"],
  " "
}], "Usage",
  CellTags->"ConcatenateFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"EpsElim", "\[Rule]", "True", "-"},
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ConcatenateFA"],

Cell[TextData[{
  "Option  ",
  StyleBox["EpsElim->False", "MR"],
  "  does not eliminate the \[Epsilon]-transitions introduced in the \
construction."
}], "Notes",
  CellTags->"ConcatenateFA"],

Cell[TextData[{
  "The acceptance language of the output machine is  ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[ScriptCapitalL](
          M\_1)\)\ \[CenterDot]\ \(\[ScriptCapitalL](M\_2)\)\)]],
  "."
}], "Notes",
  CellTags->"ConcatenateFA"],

Cell[TextData[{
  "The algorithm introduces \[Epsilon]-transitions from all states in ",
  Cell[BoxData[
      \(TraditionalForm\`F\_1\)]],
  " to all states in ",
  Cell[BoxData[
      \(TraditionalForm\`I\_2\)]],
  " and is thus quadratic. "
}], "Notes",
  CellTags->"ConcatenateFA"],

Cell[TextData[{
  StyleBox["ConcatenateFA", "MR"],
  " is polyadic."
}], "Notes",
  CellTags->"ConcatenateFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["UnionFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SumFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ProductFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["IntersectionFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ComplementFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["StarFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ConcatenateFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ConcatenateFA"],

Cell["All words of length 2 concatenated with themselves.", "ExampleText",
  CellTags->"ConcatenateFA"],

Cell[" m = BuildDFA[ Words[2], 2 ]", "Input",
  CellLabel->"In[187]:=",
  CellTags->"ConcatenateFA"],

Cell[BoxData[
    \(mm\  = \ ConcatenateFA[\ m, m]\)], "Input",
  CellLabel->"In[188]:=",
  CellTags->"ConcatenateFA"],

Cell[BoxData[
    \(LanguageFA[\ mm, \ \(-8\), \ SizeOnly \[Rule] True\ ]\)], "Input",
  CellLabel->"In[189]:=",
  CellTags->"ConcatenateFA"],

Cell["ConvertToFA", "ObjectName",
  CellTags->"ConvertToFA"],

Cell["Function", "Highlight",
  CellTags->"ConvertToFA"],

Cell[TextData[{
  StyleBox["ConvertToFA[H,Q,K,A,I,F]", "MR"],
  "  generates a machine with head ",
  StyleBox["H", "MR"],
  ", states ",
  StyleBox["Q", "MR"],
  ", alphabet ",
  StyleBox["K", "MR"],
  ", transitions ",
  StyleBox["A", "MR"],
  ", initial/final states ",
  StyleBox["I", "MR"],
  "/",
  StyleBox["F", "MR"],
  ". "
}], "Usage",
  CellTags->"ConvertToFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"},
          {"ToMatrix", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ConvertToFA"],

Cell[TextData[{
  "Option  ",
  StyleBox["ToMatrix->True", "MR"],
  "  causes a flat list of transitions to be converted into a matrix, only \
applies to construcation of a DFA."
}], "Notes",
  CellTags->"ConvertToFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["FunctionToDFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["GenerateDFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["FunctionToSA",
    ButtonStyle->"AddOnsLink"],
  ". "
}], "Notes",
  CellTags->"ConvertToFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ConvertToFA"],

Cell[TextData[{
  "On occasion it is more natural to write transitions in terms of actual \
states, not in positional notation. Here is a machine whose states are words \
",
  Cell[BoxData[
      FormBox[
        RowBox[{"{", 
          RowBox[{"\[Epsilon]", ",", "a", ",", 
            StyleBox["aa",
              FontSlant->"Italic"]}], "}"}], TraditionalForm]]],
  ": the machine counts the number of ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  "'s."
}], "ExampleText",
  CellTags->"ConvertToFA"],

Cell[BoxData[
    \(ConvertToFA[
      FA, {\[Epsilon], a, aa}, {a, 
        b}, {{\[Epsilon], a, a}, {a, a, aa}, {aa, a, 
          aa}}, {\[Epsilon]}, {aa}\ ]\)], "Input",
  CellLabel->"In[209]:=",
  AspectRatioFixed->True,
  CellTags->"ConvertToFA"],

Cell[BoxData[
    \(LanguageFA[%, \(-5\)]\)], "Input",
  CellLabel->"In[210]:=",
  CellTags->"ConvertToFA"],

Cell["A slightly more complicate example: a de Bruijn automaton.", \
"ExampleText",
  CellTags->"ConvertToFA"],

Cell[BoxData[{
    \(\(Q\  = \ Words[3];\)\), "\n", 
    \(tr\  = \ 
      Join[\ \ \ \({#, a, StringDrop[Concatenate[#, a], 1]} &\)\  /@ \ 
          Q, \ \[IndentingNewLine]\({#, b, 
              StringDrop[Concatenate[#, b], 1]} &\)\  /@ \ Q\ ]\)}], "Input",
  CellLabel->"In[214]:=",
  CellTags->"ConvertToFA"],

Cell[BoxData[
    \(ConvertToFA[\ SA, \ Q, \ {a, b}, \ tr\ ]\)], "Input",
  CellLabel->"In[218]:=",
  CellTags->"ConvertToFA"],

Cell["CounterDFA", "ObjectName",
  CellTags->"CounterDFA"],

Cell["Function", "Highlight",
  CellTags->"CounterDFA"],

Cell[TextData[{
  StyleBox["CounterDFA[a,r,K]", "MR"],
  "  constructs a DFA for the language of all words ",
  StyleBox["w", "MR"],
  " over alphabet ",
  StyleBox["K", "MR"],
  " that contain exactly ",
  StyleBox["r", "MR"],
  " many ",
  StyleBox["a", "MR"],
  "'s. "
}], "Usage",
  CellTags->"CounterDFA"],

Cell[TextData[{
  "Options\n",
  Cell[BoxData[
      FormBox[GridBox[{
            {
              StyleBox["Normalize",
                "MR"], "\[RuleDelayed]", 
              StyleBox["$defaultnormalize",
                "MR"], "-"},
            {
              StyleBox["Modulus",
                "MR"], "\[Rule]", 
              StyleBox["0",
                "MR"], "-"}
            },
          ColumnWidths->Automatic,
          ColumnAlignments->{Left}], TraditionalForm]]]
}], "Notes",
  CellTags->"CounterDFA"],

Cell[TextData[{
  "Parameter ",
  StyleBox["r", "MR"],
  " may also be a list of integers in which case a machine accepting words \
with any number of ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  "'s appearing on the list is generated."
}], "Notes",
  CellTags->"CounterDFA"],

Cell[TextData[{
  "If parameter ",
  StyleBox["r", "MR"],
  " is negative it corresponds to the list  ",
  Cell[BoxData[
      \(TraditionalForm\`{0, 1, \[Ellipsis], \(-r\)}\)]],
  ",  so all words with at most ",
  Cell[BoxData[
      \(TraditionalForm\`\(-r\)\)]],
  " many ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  "'s are accepted."
}], "Notes",
  CellTags->"CounterDFA"],

Cell[TextData[{
  "With option  ",
  "Modulus",
  StyleBox["->m", "MR"],
  "  the counting is done modulo ",
  Cell[BoxData[
      \(TraditionalForm\`m > 0\)]],
  "."
}], "Notes",
  CellTags->"CounterDFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["SubwordCounterDFA",
    ButtonData:>"SubwordCounterDFA",
    ButtonStyle->"AddOnsLink"],
  ". "
}], "Notes",
  CellTags->"CounterDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"CounterDFA"],

Cell[TextData[{
  "All words containing exactly 3 ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  "'s."
}], "ExampleText",
  CellTags->"CounterDFA"],

Cell[BoxData[{
    \(m = CounterDFA[b, 3]\), "\n", 
    \(LanguageFA[\ m, \ \(-5\)\ ]\)}], "Input",
  CellLabel->"In[18]:=",
  CellTags->"CounterDFA"],

Cell[TextData[{
  "All words containing between 2 and 4  ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  "'s."
}], "ExampleText",
  CellTags->"CounterDFA"],

Cell[BoxData[{
    \(m = CounterDFA[b, {2, 3, 4}]\), "\n", 
    \(LanguageFA[\ m, \ \(-5\)\ ]\)}], "Input",
  CellLabel->"In[106]:=",
  CellTags->"CounterDFA"],

Cell[TextData[{
  "All words containing a multiple of 3 ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  "'s."
}], "ExampleText",
  CellTags->"CounterDFA"],

Cell[BoxData[{
    \(m = CounterDFA[b, 0, 2, Modulus \[Rule] 3]\), "\n", 
    \(LanguageFA[\ m, \ \(-5\)\ ]\)}], "Input",
  CellLabel->"In[102]:=",
  CellTags->"CounterDFA"],

Cell["CoverQDFA", "ObjectName",
  CellTags->"CoverQDFA"],

Cell["Function", "Highlight",
  CellTags->"CoverQDFA"],

Cell[TextData[{
  StyleBox["CoverQDFA[M1,M2,opts]", "MR"],
  "  tests whether DFA ",
  StyleBox["M1", "MR"],
  " covers DFA ",
  StyleBox["M2", "MR"],
  "."
}], "Usage",
  CellTags->"CoverQDFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"DoTrace", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"CoverQDFA"],

Cell[TextData[{
  "Option ",
  StyleBox["DoTrace->True", "MR"],
  " causes the homomorphism to be displayed, otherwise the output is \
Boolean."
}], "Notes",
  CellTags->"CoverQDFA"],

Cell["The machines are assumed to be over the same alphabet. ", "Notes",
  CellTags->"CoverQDFA"],

Cell[TextData[{
  "The algorithm tries to construct a partial surjective homomorphism from \
DFA ",
  Cell[BoxData[
      \(TraditionalForm\`M\_1\)]],
  " to DFA ",
  Cell[BoxData[
      \(TraditionalForm\`M\_2\)]],
  "."
}], "Notes",
  CellTags->"CoverQDFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["IsomorphicQDFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["EquivalentQFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["HomomorphismFA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["MakeHomomorphism",
    ButtonData:>"MakeHomomorphism",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"CoverQDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"CoverQDFA"],

Cell["\<\
Every DFA for a fixed language covers the minimal DFA for that \
language.  \
\>", "ExampleText",
  CellTags->"CoverQDFA"],

Cell[BoxData[{
    \(m = IthSymbolFA[a, \(-3\), Full \[Rule] True]\), "\n", 
    \(mm = MinimizeFA[m]\)}], "Input",
  CellLabel->"In[219]:=",
  CellTags->"CoverQDFA"],

Cell[BoxData[
    \(CoverQDFA[m, mm]\)], "Input",
  CellLabel->"In[221]:=",
  CellTags->"CoverQDFA"],

Cell[BoxData[
    \(CoverQDFA[m, mm, DoTrace \[Rule] True]\)], "Input",
  CellLabel->"In[222]:=",
  CellTags->"CoverQDFA"],

Cell["DefiniteQFA", "ObjectName",
  CellTags->"DefiniteQFA"],

Cell["Function", "Highlight",
  CellTags->"DefiniteQFA"],

Cell[TextData[{
  StyleBox["DefiniteQFA[M]", "MR"],
  "  tests wether ",
  StyleBox["M", "MR"],
  "  is definite, i.e., whether for all sufficiently long words ",
  StyleBox["w", "MR"],
  " computations on ",
  StyleBox["w", "MR"],
  " always end at the same state, regardless of the starting state. \n"
}], "Usage",
  CellTags->"DefiniteQFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"},
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"DefiniteQFA"],

Cell[TextData[{
  "If option ",
  StyleBox["Full->True", "MR"],
  " is set, the definite-depth of the machine is computed. "
}], "Notes",
  CellTags->"DefiniteQFA"],

Cell["\<\
The algorithm computes the strongly connected components of the \
corresponding product machine and is thus quadratic.\
\>", "Notes",
  CellTags->"DefiniteQFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["ProductSA",
    ButtonData:>"ProductSA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["AmbiguousQFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DefiniteQFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DefiniteQFA"],

Cell["A definite automaton.", "ExampleText",
  CellTags->"DefiniteQFA"],

Cell[BoxData[{
    \(m = IthSymbolFA[a, \(-3\), {a, b}, Full \[Rule] True]\), "\n", 
    \(DefiniteQFA[m]\)}], "Input",
  CellLabel->"In[38]:=",
  AspectRatioFixed->True,
  CellTags->"DefiniteQFA"],

Cell["All words of length at least 3 lead to synchronization:", "ExampleText",
  CellTags->"DefiniteQFA"],

Cell[BoxData[
    \(DefiniteQFA[m, Full \[Rule] True]\)], "Input",
  CellLabel->"In[40]:=",
  CellTags->"DefiniteQFA"],

Cell[BoxData[
    \(TransitionFunctionFA[m, \[Delta]]\)], "Input",
  CellLabel->"In[42]:=",
  CellTags->"DefiniteQFA"],

Cell[BoxData[
    \(\(\[Delta][#, aba] &\)\  /@ \ Range[15]\)], "Input",
  CellLabel->"In[49]:=",
  CellTags->"DefiniteQFA"],

Cell[BoxData[
    \(\(\[Delta][#, aaa] &\)\  /@ \ Range[15]\)], "Input",
  CellLabel->"In[44]:=",
  CellTags->"DefiniteQFA"],

Cell["DegreesFA", "ObjectName",
  CellTags->"DegreesFA"],

Cell["Function", "Highlight",
  CellTags->"DegreesFA"],

Cell[TextData[{
  StyleBox["DegreesFA[M]", "MR"],
  "  returns a ",
  StyleBox["K", "MR"],
  " by ",
  StyleBox["Q", "MR"],
  " matrix of integers where entry ",
  StyleBox["(p,a)", "MR"],
  " is the number of transitions with source ",
  StyleBox["p", "MR"],
  " and label ",
  StyleBox["a", "MR"],
  "."
}], "Usage",
  CellTags->"DegreesFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Direction", "\[Rule]", "Forward", "-"},
          {"Full", "\[Rule]", "True", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"DegreesFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Direction->Backward", "MR"],
  "  constructs the corresponding matrix for the reverse of the automaton, \
i.e., it counts incoming transitions."
}], "Notes",
  CellTags->"DegreesFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->False", "MR"],
  "  returns a list of the out-degree of each state."
}], "Notes",
  CellTags->"DegreesFA"],

Cell["\<\
A 0 entry indicates an incomplete state, and an entry larger than 1 \
indicates a nondeterministic state.\
\>", "Notes",
  CellTags->"DegreesFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["DeterministicQFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToGraphFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DegreesFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DegreesFA"],

Cell["\<\
A machine that contains one nondeterministic state, and one \
incomplete state.\
\>", "ExampleText",
  CellTags->"DegreesFA"],

Cell["m = IthSymbolFA[b,-3,{a,b,c}]", "Input",
  CellLabel->"In[223]:=",
  CellTags->"DegreesFA"],

Cell[BoxData[
    \(DegreesFA[m]\  // \ MatrixForm\)], "Input",
  CellLabel->"In[224]:=",
  CellTags->"DegreesFA"],

Cell["DeleteStateFA", "ObjectName",
  CellTags->"DeleteStateFA"],

Cell["Function", "Highlight",
  CellTags->"DeleteStateFA"],

Cell[TextData[{
  StyleBox["DeleteStateFA[M,p]", "MR"],
  "  removes state ",
  StyleBox["p", "MR"],
  " from machine ",
  StyleBox["M", "MR"],
  "."
}], "Usage",
  CellTags->"DeleteStateFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[Rule]", "1", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"DeleteStateFA"],

Cell[TextData[{
  "Input ",
  Cell[BoxData[
      \(TraditionalForm\`p\)]],
  " can also be a list of states, all in positional notation."
}], "Notes",
  CellTags->"DeleteStateFA"],

Cell[TextData[{
  "Machine ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  " can be an FA, NFA or SA, but not a DFA (convert first)."
}], "Notes",
  CellTags->"DeleteStateFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["AddTransitionFA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["DeleteTransitionFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DeleteStateFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DeleteStateFA"],

Cell["\<\
A semiautomaton that accepts all words, and two mutilated versions.\
\
\>", "ExampleText",
  CellTags->"DeleteStateFA"],

Cell[BoxData[{
    \(\(s\  = \ 
        SA[4, 2, {{1, 1, 1}, {1, 2, 2}, {2, 1, 3}, {2, 2, 4}, {3, 2, 1}, {3, 
              1, 2}, {4, 2, 3}, {4, 1, 4}}];\)\), "\n", 
    \(s1\  = \ DeleteStateFA[s, 1]\), "\n", 
    \(s2\  = \ DeleteStateFA[s, 2]\)}], "Input",
  CellLabel->"In[226]:=",
  CellTags->"DeleteStateFA"],

Cell[BoxData[{
    \(LanguageFA[\ s, \(-8\), SizeOnly \[Rule] True\ \ ]\), "\n", 
    \(LanguageFA[\ s1, \(-8\), SizeOnly \[Rule] True\ \ ]\), "\n", 
    \(LanguageFA[\ s2, \(-8\), SizeOnly \[Rule] True\ \ ]\)}], "Input",
  CellLabel->"In[229]:=",
  CellTags->"DeleteStateFA"],

Cell["DeterministicQFA", "ObjectName",
  CellTags->"DeterministicQFA"],

Cell["Function", "Highlight",
  CellTags->"DeterministicQFA"],

Cell[TextData[{
  StyleBox["DeterministicQFA[M]", "MR"],
  "  tests whether machine ",
  StyleBox["M", "MR"],
  " is deterministic."
}], "Usage",
  CellTags->"DeterministicQFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "True"},
          {"Direction", "\[Rule]", "Forward", "Backward"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"DeterministicQFA"],

Cell[TextData[{
  "With option ",
  StyleBox["Full->True", "MR"],
  " the function tests whether ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  " is ",
  Cell[BoxData[
      \(TraditionalForm\`d\)]],
  "-deterministic, and returns ",
  Cell[BoxData[
      \(TraditionalForm\`d\)]],
  " or ",
  Cell[BoxData[
      \(TraditionalForm\`\[Infinity]\)]],
  " accordingly.  "
}], "Notes",
  CellTags->"DeterministicQFA"],

Cell[TextData[{
  "With option ",
  StyleBox["Direction->Backward", "MR"],
  " the function tests whether tests the reverse of the automaton."
}], "Notes",
  CellTags->"DeterministicQFA"],

Cell[TextData[{
  "A machine ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  " is ",
  Cell[BoxData[
      \(TraditionalForm\`d\)]],
  "-deterministic if all computations of length ",
  Cell[BoxData[
      \(TraditionalForm\`d + 1\)]],
  " with the same label and the same source agree on their second  state."
}], "Notes",
  CellTags->"DeterministicQFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["DegreesFA",
    ButtonData:>"DegreesFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["AmbiguousQFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DefiniteQFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DeterministicQFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DeterministicQFA"],

Cell["\<\
A deterministic and codeterministic semiautomaton based on an \
elementary cellular automaton.\
\>", "ExampleText",
  CellTags->"DeterministicQFA"],

Cell[BoxData[{
    \(s = ToSA[CA[90]]\), "\n", 
    \(DeterministicQFA[s]\), "\n", 
    \(DeterministicQFA[s, Direction \[Rule] Backward]\)}], "Input",
  CellLabel->"In[40]:=",
  CellTags->"DeterministicQFA"],

Cell["\<\
Another such semiautomaton that fails to be deterministic but that \
is 1-deterministic.\
\>", "ExampleText",
  CellTags->"DeterministicQFA"],

Cell[BoxData[{
    \(s = ToSA[CA[51]]\), "\n", 
    \(DeterministicQFA[s, Full \[Rule] True\ ]\)}], "Input",
  CellLabel->"In[47]:=",
  CellTags->"DeterministicQFA"],

Cell["\<\
Despite the nondeterminism, computations of length 2 cannot branch \
at the first step.\
\>", "ExampleText",
  CellTags->"DeterministicQFA"],

Cell[BoxData[{
    \(\(\(PlotComputationFA[s, #, Full \[Rule] False, 
            PlotStyle \[Rule] { .02,  .03}, \
\[IndentingNewLine]DisplayFunction \[Rule] Identity, 
            Frame \[Rule] True] &\)\  /@ \ Words[2, \(-2\)];\)\), "\n", 
    \(\(ShowArray[%];\)\)}], "Input",
  CellLabel->"In[45]:=",
  CellTags->"DeterministicQFA"],

Cell["DFA", "ObjectName",
  CellTags->"DFA"],

Cell["Constructor", "Highlight",
  CellTags->"DFA"],

Cell[TextData[{
  StyleBox["DFA[n,k,T,q0,F]", "MR"],
  "  denotes a deterministic complete finite automaton. "
}], "Usage",
  CellTags->"DFA"],

Cell[TextData[{
  StyleBox["n", "MR"],
  " is the number of states, ",
  StyleBox["k", "MR"],
  " the size of the alphabet, ",
  StyleBox["T", "MR"],
  " the transition matrix, ",
  StyleBox["q0", "MR"],
  " the initial state, and ",
  StyleBox["F", "MR"],
  " the set of final states. "
}], "Notes",
  CellTags->"DFA"],

Cell[TextData[{
  "For the sake of consistency, a DFA can also take the form ",
  StyleBox["DFA[Q,K,T,{q0},F]", "MR"],
  ", i.e., the initial state may also be given as a list, just as in all the \
other machines. "
}], "Notes",
  CellTags->"DFA"],

Cell[TextData[{
  "The entries in ",
  StyleBox["T", "MR"],
  ", ",
  StyleBox["q0", "MR"],
  " and ",
  StyleBox["F", "MR"],
  " are always in positional notation."
}], "Notes",
  CellTags->"DFA"],

Cell[TextData[{
  "The transition matrix ",
  StyleBox["T", "MR"],
  " has dimensions ",
  Cell[BoxData[
      \(TraditionalForm\`k\ \[Cross]\ n\)]],
  ". "
}], "Notes",
  CellTags->"DFA"],

Cell[TextData[{
  " Additional hidden fields may hold a structured state set ",
  StyleBox["Q", "MR"],
  ", an arbitrary list of objects of length ",
  StyleBox["n", "MR"],
  ", and the alphabet, a list of characters of length ",
  StyleBox["k", "MR"],
  ".  The display of these fields depends on the formatting level."
}], "Notes",
  CellTags->"DFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["FormatAutomata",
    ButtonData:>"FormatAutomata",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["FA",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["NFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DFA"],

Cell[TextData[{
  "A simple DFA.  The alphabet is assumed to be ",
  Cell[BoxData[
      \(TraditionalForm\`{a, b}\)]],
  " and the state set is assumed to be ",
  Cell[BoxData[
      \(TraditionalForm\`{1, 2, \[Ellipsis], 7}\)]],
  ".  Note how the missing state and alphabet fields are added automatically. \
 We turn off formatting to make this effect visible."
}], "ExampleText",
  CellTags->"DFA"],

Cell[BoxData[
    \(FormatAutomata[None]\)], "Input",
  CellLabel->"In[686]:=",
  CellTags->"DFA"],

Cell[BoxData[
    \(\(\(m\)\(\ \ \)\(=\)\(\ \)\(DFA[7, 
        2, {{2, 4, 5, 7, 5, 5, 5}, {3, 5, 6, 5, 5, 7, 5}}, 
        1, {7}]\)\(\ \)\)\)], "Input",
  CellLabel->"In[681]:=",
  CellTags->"DFA"],

Cell[BoxData[{
    \(States[m]\), "\n", 
    \(Alphabet[m]\)}], "Input",
  CellLabel->"In[682]:=",
  CellTags->"DFA"],

Cell[BoxData[
    \(FormatAutomata[Automatic]\)], "Input",
  CellLabel->"In[684]:=",
  CellTags->"DFA"],

Cell["DivisibilityDFA", "ObjectName",
  CellTags->"DivisibilityDFA"],

Cell["Function", "Highlight",
  CellTags->"DivisibilityDFA"],

Cell[TextData[{
  StyleBox["DivisibilityDFA[m,B,opts]", "MR"],
  "  produces the minimal DFA recognizing all numbers in base ",
  StyleBox["B", "MR"],
  " that are divisible by ",
  StyleBox["m", "MR"],
  ". "
}], "Usage",
  CellTags->"DivisibilityDFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"},
          {"SizeOnly", "\[Rule]", "False", "-"},
          {"Direction", "\[Rule]", "Forward", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"DivisibilityDFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  "  returns the canonical DFA (which fails to be minimal in general)."
}], "Notes",
  CellTags->"DivisibilityDFA"],

Cell[TextData[{
  "Option ",
  StyleBox["SizeOnly->True", "MR"],
  "  returns only the size of the minimal DFA."
}], "Notes",
  CellTags->"DivisibilityDFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Direction->Backward", "MR"],
  "  deals with numbers in reverse base ",
  Cell[BoxData[
      \(TraditionalForm\`B\)]],
  " notation (LSD first)."
}], "Notes",
  CellTags->"DivisibilityDFA"],

Cell[TextData[{
  "The canonical DFA in standard base ",
  Cell[BoxData[
      \(TraditionalForm\`B\)]],
  " notation has states ",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalZ]\_m\)]],
  " and is based on the action  ",
  Cell[BoxData[
      \(TraditionalForm\`p\ \[CenterDot]\ a\  = \ B\ p\  + \ a\ \ mod\ m\)]],
  ".   In reverse base ",
  Cell[BoxData[
      \(TraditionalForm\`B\)]],
  "  the states set is  ",
  Cell[BoxData[
      \(TraditionalForm\`{0, 1, \[Ellipsis], m - 1}\ \[Cross]\ P\)]],
  " where ",
  Cell[BoxData[
      \(TraditionalForm\`P\)]],
  " is the submonoid generated by ",
  Cell[BoxData[
      \(TraditionalForm\`B\)]],
  " in the multiplicative monoid ",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalZ]\_m\)]],
  ".  Thus the size of this machine may be close to ",
  Cell[BoxData[
      \(TraditionalForm\`m\^2\)]],
  ".   The action is given by  ",
  Cell[BoxData[
      \(TraditionalForm\`\((p, q)\)\ \[CenterDot]\ 
          a\  = \ \((\ p\ q\  + \ a, \ q\ B)\)\ mod\ m\)]],
  ". "
}], "Notes",
  CellTags->"DivisibilityDFA"],

Cell[TextData[{
  "The machines generated for standard base ",
  Cell[BoxData[
      \(TraditionalForm\`B\)]],
  " are always transitive, but for reverse base ",
  Cell[BoxData[
      \(TraditionalForm\`B\)]],
  " there may be a sink (in fact, there may be two trap states). "
}], "Notes",
  CellTags->"DivisibilityDFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["Solve",
    ButtonStyle->"RefGuideLink"],
  "  and  ",
  ButtonBox["FunctionToDFA",
    ButtonData:>"FunctionToDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DivisibilityDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DivisibilityDFA"],

Cell[TextData[{
  "An example of a divisibility tester in base ",
  Cell[BoxData[
      \(TraditionalForm\`B = 14\)]],
  " for modulus ",
  Cell[BoxData[
      \(TraditionalForm\`m = 100\)]],
  ".  First, the size of the minimal DFA."
}], "ExampleText",
  CellTags->"DivisibilityDFA"],

Cell[BoxData[{
    \(\(m\  = \ 100;\)\), "\n", 
    \(\(B\  = \ 14;\)\), "\n", 
    \(DivisibilityDFA[\ m, \ B\ , \ SizeOnly \[Rule] True\ ]\)}], "Input",
  CellLabel->"In[35]:=",
  CellTags->"DivisibilityDFA"],

Cell["Construct the actual minimal DFA and test some words.", "ExampleText",
  CellTags->"DivisibilityDFA"],

Cell[BoxData[
    \(M\  = \ DivisibilityDFA[\ m, \ B\ \ ]\)], "Input",
  CellLabel->"In[38]:=",
  CellTags->"DivisibilityDFA"],

Cell[BoxData[
    \(LanguageFA[M, 3]\)], "Input",
  CellLabel->"In[39]:=",
  CellTags->"DivisibilityDFA"],

Cell[BoxData[
    \(14^^61a\  // \ FullForm\)], "Input",
  CellLabel->"In[40]:=",
  CellTags->"DivisibilityDFA"],

Cell[TextData[{
  "The canonical DFA has ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  " states and covers ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  ". "
}], "ExampleText",
  CellTags->"DivisibilityDFA"],

Cell[BoxData[{
    \(MM\  = \ DivisibilityDFA[\ m, \ B\ , Full \[Rule] True\ ]\ \), "\n", 
    \(CoverQDFA[MM, M]\)}], "Input",
  CellLabel->"In[41]:=",
  CellTags->"DivisibilityDFA"],

Cell[TextData[{
  "In reverse base ",
  Cell[BoxData[
      \(TraditionalForm\`B\)]],
  " the minimal DFA has size 34."
}], "ExampleText",
  CellTags->"DivisibilityDFA"],

Cell[BoxData[
    \(DivisibilityDFA[\ m, \ B\ , \ SizeOnly \[Rule] True, \ 
      Direction \[Rule] Backward\ ]\)], "Input",
  CellLabel->"In[43]:=",
  CellTags->"DivisibilityDFA"],

Cell[TextData[{
  "We verify that the DFA is equivalent to the machine obtained from \
reversing the DFA for ordinary base ",
  Cell[BoxData[
      \(TraditionalForm\`B\)]],
  ". "
}], "ExampleText",
  CellTags->"DivisibilityDFA"],

Cell[BoxData[
    \(EquivalentQFA[\ ReverseFA@DivisibilityDFA[m, B], 
      DivisibilityDFA[m, B, Direction \[Rule] Backward]]\)], "Input",
  CellLabel->"In[44]:=",
  CellTags->"DivisibilityDFA"],

Cell["EdgeLU", "ObjectName",
  CellTags->"EdgeLU"],

Cell["Option", "Highlight",
  CellTags->"EdgeLU"],

Cell[TextData[{
  StyleBox["EdgeLU", "MR"],
  "  is an option for ",
  StyleBox["LocalUniversalFA", "MR"],
  "."
}], "Usage",
  CellTags->"EdgeLU"],

Cell["Determines the choice of edges in the de Bruijn automaton. ", "Notes",
  CellTags->"EdgeLU"],

Cell[TextData[{
  "See ",
  ButtonBox["LocalUniversalFA",
    ButtonData:>"LocalUniversalFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"EdgeLU"],

Cell["EmptyDFA", "ObjectName",
  CellTags->"EmptyDFA"],

Cell["Function", "Highlight",
  CellTags->"EmptyDFA"],

Cell[TextData[{
  StyleBox["EmptyDFA[K]", "MR"],
  "  constructs a DFA that accepts the empty language over alphabet ",
  StyleBox["K", "MR"],
  "."
}], "Usage",
  CellTags->"EmptyDFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["BuildDFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["EmptyQFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["FullQFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"EmptyDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"EmptyDFA"],

Cell["A DFA on a three-letter alphabet accepting no words at all.", \
"ExampleText",
  CellTags->"EmptyDFA"],

Cell["m = EmptyDFA[3]", "Input",
  CellLabel->"In[232]:=",
  CellTags->"EmptyDFA"],

Cell[BoxData[
    \(LanguageFA[m, \(-5\)]\)], "Input",
  CellLabel->"In[233]:=",
  CellTags->"EmptyDFA"],

Cell["EmptyQFA", "ObjectName",
  CellTags->"EmptyQFA"],

Cell["Function", "Highlight",
  CellTags->"EmptyQFA"],

Cell[TextData[{
  StyleBox["EmptyQFA[M]", "MR"],
  "  tests whether the acceptance language of ",
  StyleBox["M", "MR"],
  " is empty."
}], "Usage",
  CellTags->"EmptyQFA"],

Cell["\<\
The algorithm computes the accessible part of the automaton and is \
linear.\
\>", "Notes",
  CellTags->"EmptyQFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["FullQFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"EmptyQFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"EmptyQFA"],

Cell["Two midly complicated machines with empty intersection.", "ExampleText",
  CellTags->"EmptyQFA"],

Cell[BoxData[{
    \(\(m1 = \ 
        DFA[6, 2, {{2, 4, 3, 5, 6, 3}, {3, 3, 3, 3, 3, 3}}, 
          1, {6}];\)\), "\n", 
    \(\(m2\  = \ 
        DFA[9, 2, {{2, 4, 5, 6, 7, 8, 9, 8, 8}, {3, 5, 5, 7, 7, 9, 9, 8, 8}}, 
          1, {9}];\)\)}], "Input",
  CellLabel->"In[234]:=",
  CellTags->"EmptyQFA"],

Cell[BoxData[{
    \(EmptyQFA[m1]\), "\n", 
    \(EmptyQFA[m2]\), "\n", 
    \(EmptyQFA[IntersectionFA[m1, m2]]\)}], "Input",
  CellLabel->"In[236]:=",
  CellTags->"EmptyQFA"],

Cell["EpsElim", "ObjectName",
  CellTags->"EpsElim"],

Cell["Option", "Highlight",
  CellTags->"EpsElim"],

Cell[TextData[{
  StyleBox["EpsElim", "MR"],
  "  is an option for ",
  StyleBox["StarFA", "MR"],
  " and ",
  StyleBox["RegularExpressionToFA", "MR"],
  ". "
}], "Usage",
  CellTags->"EpsElim"],

Cell[TextData[{
  "Opition ",
  StyleBox["EpsElim->False", "MR"],
  " supresses epsilon elimination and returns an FA with epsilon moves. "
}], "Notes",
  CellTags->"EpsElim"],

Cell[TextData[{
  "See also  ",
  ButtonBox["StarFA",
    ButtonData:>"StarFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["RegularExpressionToFA",
    ButtonData:>"RegularExpressionToFA",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"EpsElim"],

Cell["EpsilonFreeQ", "ObjectName",
  CellTags->"EpsilonFreeQ"],

Cell["Function", "Highlight",
  CellTags->"EpsilonFreeQ"],

Cell[TextData[{
  StyleBox["EpsilonFreeQ[M]", "MR"],
  "  tests whether a machine is epsilon-free."
}], "Usage",
  CellTags->"EpsilonFreeQ"],

Cell[TextData[{
  "Machine ",
  StyleBox["M", "MR"],
  " can be a ",
  ButtonBox["DFA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["FA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SA",
    ButtonStyle->"AddOnsLink"],
  " or ",
  ButtonBox["BEFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"EpsilonFreeQ"],

Cell[TextData[{
  "See also ",
  ButtonBox["EpsilonEliminationFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"EpsilonFreeQ"],

Cell["Further Examples", "ExampleSection",
  CellTags->"EpsilonFreeQ"],

Cell["\<\
Conversion from regular expression usually produces epsilon \
transitions. \
\>", "ExampleText",
  CellTags->"EpsilonFreeQ"],

Cell["m = RegularExpressionToBEFA[ RES[REP[aa,bb]] ]", "Input",
  CellLabel->"In[226]:=",
  CellTags->"EpsilonFreeQ"],

Cell[BoxData[
    \(EpsilonFreeQ[m]\)], "Input",
  CellLabel->"In[227]:=",
  CellTags->"EpsilonFreeQ"],

Cell["EpsilonEliminationFA", "ObjectName",
  CellTags->"EpsilonEliminationFA"],

Cell["Function", "Highlight",
  CellTags->"EpsilonEliminationFA"],

Cell[TextData[{
  StyleBox["EpsilonEliminationFA[M]", "MR"],
  "  converts a machine M into an equivalent epsilon-free machine. "
}], "Usage",
  CellTags->"EpsilonEliminationFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"DoTrace", "\[Rule]", "False", "-"},
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"},
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"EpsilonEliminationFA"],

Cell[TextData[{
  "With option ",
  StyleBox["DoTrace->True", "MR"],
  " it returns ",
  Cell[BoxData[
      \(TraditionalForm\`{m, t\_\[Epsilon], t, t\_c}\)]],
  " where ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  " is the epsilon-free FA, ",
  Cell[BoxData[
      \(TraditionalForm\`t\_1\)]],
  " is a list of the epsilon transitions of ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`t\)]],
  " is a list of the non-epsilon transitions, and ",
  Cell[BoxData[
      \(TraditionalForm\`t\_c\)]],
  "represents the transitive, reflexive closure of ",
  Cell[BoxData[
      \(TraditionalForm\`t\_\[Epsilon]\)]],
  ". All are given as binary relations on the state set of ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  "."
}], "Notes",
  CellTags->"EpsilonEliminationFA"],

Cell["\<\
Epsilon elimination is used automatically in the Thompson style \
conversion algorithms from regular expressions to machines.\
\>", "Notes",
  CellTags->"EpsilonEliminationFA"],

Cell["\<\
The algorithm preserves the number of states but is in general \
quadratic due to the number of newly introduced transitions. \
\>", "Notes",
  CellTags->"EpsilonEliminationFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"EpsilonEliminationFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"EpsilonEliminationFA"],

Cell["\<\
A machine produced by conversion from regular expression, and its \
epsilon-free counterpart.\
\>", "ExampleText",
  CellTags->"EpsilonEliminationFA"],

Cell[BoxData[
    \(\(m = 
        FA[17, 2, {{2, 1, 3}, {3, 1, 5}, {4, 1, 4}, {5, 1, 4}, {6, 1, 4}, {2, 
              2, 4}, {3, 2, 4}, {4, 2, 4}, {5, 2, 6}, {6, 2, 4}, {1, 0, 
              2}, {6, 0, 1}, {1, 0, 7}, {8, 1, 9}, {9, 1, 9}, {10, 1, 
              9}, {11, 1, 12}, {12, 1, 9}, {8, 2, 10}, {9, 2, 9}, {10, 2, 
              11}, {11, 2, 9}, {12, 2, 9}, {7, 0, 8}, {12, 0, 7}, {7, 0, 
              13}, {13, 1, 14}, {14, 1, 14}, {15, 1, 14}, {16, 1, 14}, {17, 
              1, 14}, {13, 2, 15}, {14, 2, 14}, {15, 2, 16}, {16, 2, 
              17}, {17, 2, 14}}, {1}, {17}];\)\)], "Input",
  CellLabel->"In[570]:=",
  CellTags->"EpsilonEliminationFA"],

Cell[BoxData[
    \(\(mm\  = \ EpsilonEliminationFA[m];\)\)], "Input",
  CellLabel->"In[571]:=",
  CellTags->"EpsilonEliminationFA"],

Cell[BoxData[
    \(EquivalentQFA[m, mm]\)], "Input",
  CellLabel->"In[156]:=",
  CellTags->"EpsilonEliminationFA"],

Cell["The trace of the algorithm.", "ExampleText",
  CellTags->"EpsilonEliminationFA"],

Cell[BoxData[
    \(\({mm, te, t, tt}\  = \ 
        EpsilonEliminationFA[m, DoTrace \[Rule] True];\)\)], "Input",
  CellLabel->"In[160]:=",
  CellTags->"EpsilonEliminationFA"],

Cell[BoxData[
    \(te\)], "Input",
  CellLabel->"In[161]:=",
  CellTags->"EpsilonEliminationFA"],

Cell[BoxData[
    \(t\)], "Input",
  CellLabel->"In[162]:=",
  CellTags->"EpsilonEliminationFA"],

Cell[BoxData[
    \(tt\)], "Input",
  CellLabel->"In[163]:=",
  CellTags->"EpsilonEliminationFA"],

Cell["EquivalentQFA", "ObjectName",
  CellTags->"EquivalentQFA"],

Cell["Function", "Highlight",
  CellTags->"EquivalentQFA"],

Cell[TextData[{
  StyleBox["EquivalentQFA[M1,M2]", "MR"],
  "  tests whether automata M1 and M2 are equivalent. "
}], "Usage",
  CellTags->"EquivalentQFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"DoTrace", "\[Rule]", "False", "-"},
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"EquivalentQFA"],

Cell[TextData[{
  "Option ",
  StyleBox["DoTrace->True", "MR"],
  " returns a list ",
  Cell[BoxData[
      \(TraditionalForm\`\((n\_1, n\_2, F, R)\)\)]],
  "  where ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  " is the size of the first DFA, ",
  Cell[BoxData[
      \(TraditionalForm\`n\_2\)]],
  " the size of the second DFA, ",
  Cell[BoxData[
      \(TraditionalForm\`F\)]],
  " the disjoint union of the final states of both machines, and ",
  Cell[BoxData[
      \(TraditionalForm\`R\)]],
  " the partition on the joint state set induced by the algorithm. "
}], "Notes",
  CellTags->"EquivalentQFA"],

Cell["\<\
For deterministic machines the union/find algorithm is used. Thus, \
the time complexity should be nearly linear.
However, since union/find is currently only partially implemented (there is \
no ranked union nor path-compression) the actual complexity is unpredictable \
-- though it appears to be quite reasonable.\
\>", "Notes",
  CellTags->"EquivalentQFA"],

Cell["\<\
If the input is nondeterministic a conversion to a deterministic \
machine is performed, so the algorithm has potentially exponential space and \
time complexity.\
\>", "Notes",
  CellTags->"EquivalentQFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["EmptyQFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["FullQFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"EquivalentQFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"EquivalentQFA"],

Cell["Several equivalent machines.", "ExampleText",
  CellTags->"EquivalentQFA"],

Cell[BoxData[
    \(\(m0\  = \ 
        FA[4, 2, {{1, 1, 1}, {1, 1, 2}, {1, 2, 1}, {2, 1, 3}, {2, 2, 3}, {3, 
              1, 4}, {3, 2, 4}}, {1}, {4}];\)\)], "Input",
  CellLabel->"In[576]:=",
  CellTags->"EquivalentQFA"],

Cell[BoxData[
    \(\(m1\  = \ 
        DFA[15, 2, {{2, 4, 6, 8, 10, 12, 14, 8, 10, 12, 14, 8, 10, 12, 
              14}, {3, 5, 7, 9, 11, 13, 15, 9, 11, 13, 15, 9, 11, 13, 15}}, 
          1, {8, 9, 10, 11}];\)\)], "Input",
  CellLabel->"In[577]:=",
  CellTags->"EquivalentQFA"],

Cell[BoxData[
    \(\(m2\  = \ 
        DFA[8, 2, {{2, 3, 4, 4, 6, 3, 6, 2}, {1, 7, 5, 5, 8, 7, 8, 1}}, 
          1, {4, 5, 6, 8}];\)\)], "Input",
  CellLabel->"In[176]:=",
  CellTags->"EquivalentQFA"],

Cell[BoxData[
    \(EquivalentQFA[\ m1, \ m2\ ]\)], "Input",
  CellLabel->"In[177]:=",
  CellTags->"EquivalentQFA"],

Cell["This call requires deterministic simulation.", "ExampleText",
  CellTags->"EquivalentQFA"],

Cell[BoxData[
    \(EquivalentQFA[\ m0, \ m1\ ]\)], "Input",
  CellLabel->"In[180]:=",
  CellTags->"EquivalentQFA"],

Cell["\<\
A trace of the partition constructed by the algorithm. All blocks \
intersect either both final state sets, or a disjoint with both.\
\>", \
"ExampleText",
  CellTags->"EquivalentQFA"],

Cell[BoxData[
    \({n1, n2, F, R}\  = \ 
      EquivalentQFA[\ m1, \ m2, DoTrace \[Rule] True\ ]\)], "Input",
  CellLabel->"In[183]:=",
  CellTags->"EquivalentQFA"],

Cell[BoxData[
    \(\(Intersection[F, #] &\)\  /@ \ R\)], "Input",
  CellLabel->"In[184]:=",
  CellTags->"EquivalentQFA"],

Cell["FactorialQFA", "ObjectName",
  CellTags->"FactorialQFA"],

Cell["Function", "Highlight",
  CellTags->"FactorialQFA"],

Cell[TextData[{
  StyleBox["FactorialQFA[M,opts]", "MR"],
  "  tests whether the language accepted by FSM ",
  StyleBox["M", "MR"],
  " is factorial, i.e., closed under factors."
}], "Usage",
  CellTags->"FactorialQFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"},
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"FactorialQFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  "  returns in addition the semiautomaton constructed by the algorithm: the \
minimal automaton minus its sink state."
}], "Notes",
  CellTags->"FactorialQFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["TransitiveQFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"FactorialQFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FactorialQFA"],

Cell["Here is a small factorial DFA.", "ExampleText",
  CellTags->"FactorialQFA"],

Cell[BoxData[
    \(\(m = 
        DFA[6, 2, {{2, 2, 4, 4, 6, 6}, {1, 3, 1, 5, 2, 6}}, 
          1, {1, 2, 3, 4, 5}];\)\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->"FactorialQFA"],

Cell[BoxData[
    \(FactorialQFA[m]\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->"FactorialQFA"],

Cell["The semiautomaton used in the test.", "ExampleText",
  CellTags->"FactorialQFA"],

Cell[BoxData[
    \(FactorialQFA[m, Full \[Rule] True]\)], "Input",
  CellLabel->"In[3]:=",
  CellTags->"FactorialQFA"],

Cell["FA", "ObjectName",
  CellTags->"FA"],

Cell["Constructor", "Highlight",
  CellTags->"FA"],

Cell[TextData[{
  StyleBox["FA[n,k,T,I,F]", "MR"],
  "  denotes a general finite automaton. "
}], "Usage",
  CellTags->"FA"],

Cell[TextData[{
  StyleBox["n", "MR"],
  " is the number of states, ",
  StyleBox["k", "MR"],
  " the alphabet size, ",
  StyleBox["T", "MR"],
  " a list of transitions ",
  StyleBox["{p,a,q}.", "MR"]
}], "Notes",
  CellTags->"FA"],

Cell[TextData[{
  "The  initial states are given by ",
  StyleBox["I", "MR"],
  " and the final states by ",
  StyleBox["F", "MR"],
  "."
}], "Notes",
  CellTags->"FA"],

Cell[TextData[{
  StyleBox["The entries in ", "Text"],
  StyleBox["T", "MR"],
  StyleBox[", ", "Text"],
  StyleBox["I", "MR"],
  StyleBox[" and ", "Text"],
  StyleBox["F", "MR"],
  StyleBox["  are in positional notation.", "Text"]
}], "Notes",
  CellTags->"FA"],

Cell[TextData[{
  "There are hidden fields ",
  StyleBox["K", "MR"],
  " for the alphabet and ",
  StyleBox["Q", "MR"],
  " for a structured state set."
}], "Notes",
  CellTags->"FA"],

Cell[TextData[{
  "See also ",
  ButtonBox["FA",
    ButtonData:>"FA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["NFA",
    ButtonData:>"NFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DFA",
    ButtonData:>"DFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"FA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FA"],

Cell["A small FA.", "ExampleText",
  CellTags->"FA"],

Cell[BoxData[
    \(\(m = 
        FA[4, 2, {{1, 1, 1}, {1, 2, 1}, {1, 2, 2}, {2, 1, 3}, {2, 2, 3}, {3, 
              1, 4}, {3, 2, 4}}, {1}, {4}];\)\)], "Input",
  CellLabel->"In[239]:=",
  CellTags->"FA"],

Cell[BoxData[{
    \(Initial[m]\), "\n", 
    \(Final[m]\), "\n", 
    \(Alphabet[m]\)}], "Input",
  CellLabel->"In[240]:=",
  CellTags->"FA"],

Cell["FinalVectorFA", "ObjectName",
  CellTags->"FinalVectorFA"],

Cell["Function", "Highlight",
  CellTags->"FinalVectorFA"],

Cell[TextData[{
  StyleBox["FinalVectorFA[M]", "MR"],
  " returns the bitvector representing the  final states of M."
}], "Usage",
  CellTags->"FinalVectorFA"],

Cell["For efficiency reasons, 0/1-vectors are used. ", "Notes",
  CellTags->"FinalVectorFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["TransitionMatrixFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["InitialVectorFA",
    ButtonData:>"InitialVectorFA",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["Final",
    ButtonData:>"Final",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"FinalVectorFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FinalVectorFA"],

Cell["A random DFA and its final states bitvector.", "ExampleText",
  CellTags->"FinalVectorFA"],

Cell[BoxData[{
    \(\(m\  = \ 
        DFA[6, 2, {{2, 2, 4, 4, 6, 6}, {1, 3, 1, 5, 2, 6}}, 
          1, {1, 2, 3, 4, 5}];\)\), "\n", 
    \(FinalVectorFA[m]\)}], "Input",
  CellLabel->"In[243]:=",
  CellTags->"FinalVectorFA"],

Cell["Final", "ObjectName",
  CellTags->"Final"],

Cell["Function", "Highlight",
  CellTags->"Final"],

Cell[TextData[{
  StyleBox["Final[M]", "MR"],
  "  returns a list of the final states of finite state machine ",
  StyleBox["M", "MR"],
  " in positional notation."
}], "Usage",
  CellTags->"Final"],

Cell[TextData[{
  "See also ",
  ButtonBox["FinalVectorFA",
    ButtonData:>"FinalVectorFA",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["Initial",
    ButtonData:>"Initial",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Final"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Final"],

Cell["\<\
A somewhat complicated  FA for all words of length 2, and its \
initial states bitvector.\
\>", "ExampleText",
  CellTags->"Final"],

Cell[BoxData[
    \(\(m = 
        UnionFA @@ \(\((ToFA[BuildDFA[#1]] &)\) /@ Words[2]\);\)\)], "Input",
  CellLabel->"In[309]:=",
  CellTags->"Final"],

Cell[BoxData[
    \(Final[m]\)], "Input",
  CellLabel->"In[316]:=",
  CellTags->"Final"],

Cell["FiniteComplementSA", "ObjectName",
  CellTags->"FiniteComplementSA"],

Cell["Function", "Highlight",
  CellTags->"FiniteComplementSA"],

Cell[TextData[{
  StyleBox["FiniteComplementSA[W,K]", "MR"],
  "  returns the canonical semiautomaton that accepts the language of all \
words not containing any word in the finite language ",
  StyleBox["W", "MR"],
  " as a factor. "
}], "Usage",
  CellTags->"FiniteComplementSA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "1", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"FiniteComplementSA"],

Cell[TextData[{
  "Note that ",
  Cell[BoxData[
      \(TraditionalForm\`\(K\^*\)\  - \ \(K\^*\) W\ \(K\^*\)\)]],
  " is an ideal."
}], "Notes",
  CellTags->"FiniteComplementSA"],

Cell[TextData[{
  StyleBox["W", "MR"],
  " may also be a single word."
}], "Notes",
  CellTags->"FiniteComplementSA"],

Cell[TextData[{
  "See also ",
  ButtonBox["FactorialQFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"FiniteComplementSA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FiniteComplementSA"],

Cell[TextData[{
  "All words not containing the factors ",
  Cell[BoxData[
      FormBox[
        StyleBox["aa",
          FontSlant->"Italic"], TraditionalForm]]],
  " and ",
  Cell[BoxData[
      FormBox[
        StyleBox["bb",
          FontSlant->"Italic"], TraditionalForm]]],
  "."
}], "ExampleText",
  CellTags->"FiniteComplementSA"],

Cell[BoxData[
    \(m = FiniteComplementSA[{aa, bb}, 2]\)], "Input",
  CellLabel->"In[247]:=",
  CellTags->"FiniteComplementSA"],

Cell[BoxData[
    \(LanguageFA[m, \(-5\)]\)], "Input",
  CellLabel->"In[248]:=",
  CellTags->"FiniteComplementSA"],

Cell[BoxData[
    \(FactorialQFA[m]\)], "Input",
  CellLabel->"In[249]:=",
  CellTags->"FiniteComplementSA"],

Cell["FinLU", "ObjectName",
  CellTags->"FinLU"],

Cell["Option", "Highlight",
  CellTags->"FinLU"],

Cell[TextData[{
  StyleBox["FinLU", "MR"],
  "  is an option for ",
  StyleBox["LocalUniversalFA", "MR"],
  "."
}], "Usage",
  CellTags->"FinLU"],

Cell["Determines the choic of final states. ", "Notes",
  CellTags->"FinLU"],

Cell[TextData[{
  "See ",
  ButtonBox["LocalUniversalFA",
    ButtonData:>"LocalUniversalFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"FinLU"],

Cell["Fixed", "ObjectName",
  CellTags->"Fixed"],

Cell["Option", "Highlight",
  CellTags->"Fixed"],

Cell[TextData[{
  StyleBox["Fixed", "MR"],
  "  is an option for ",
  StyleBox["ToBlocks", "MR"],
  "."
}], "Usage",
  CellTags->"Fixed"],

Cell["Causes fixed boundary conditions to be used.", "Notes",
  CellTags->"Fixed"],

Cell[TextData[{
  "See ",
  ButtonBox["ToBlocks",
    ButtonData:>"ToBlocks",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Fixed"],

Cell["BoundLFA", "ObjectName",
  CellTags->"BoundLFA"],

Cell["Option", "Highlight",
  CellTags->"BoundLFA"],

Cell[TextData[{
  StyleBox["BoundLFA", "MR"],
  "  is an option for ",
  StyleBox["LanguageFA", "MR"],
  "."
}], "Usage",
  CellTags->"BoundLFA"],

Cell[" Default value is 50, limits the fixed point computation.", "Notes",
  CellTags->"BoundLFA"],

Cell[TextData[{
  " Only applies in mode ",
  StyleBox["Full->True", "MR"],
  ". "
}], "Notes",
  CellTags->"BoundLFA"],

Cell[TextData[{
  "See ",
  ButtonBox["LanguageFA",
    ButtonData:>"LanguageFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"BoundLFA"],

Cell["FrontierToDFA", "ObjectName",
  CellTags->"FrontierToDFA"],

Cell["Function", "Highlight",
  CellTags->"FrontierToDFA"],

Cell[TextData[{
  StyleBox["FrontierToDFA[g,F]", "MR"],
  "  converts frontier automaton given by ",
  StyleBox["g", "MR"],
  " and ",
  StyleBox["F", "MR"],
  "  into an ordinary DFA.  "
}], "Usage",
  CellTags->"FrontierToDFA"],

Cell[TextData[{
  "Here ",
  StyleBox["g", "MR"],
  " is a list of word pairs (the first components form a frontier language), \
",
  StyleBox["F", "MR"],
  " a list of words."
}], "Notes",
  CellTags->"FrontierToDFA"],

Cell[TextData[{
  " See also ",
  ButtonBox["ToFrontierDFA",
    ButtonData:>"ToFrontierDFA",
    ButtonStyle->"AddOnsLink"],
  "   and  ",
  ButtonBox["FrontierQ",
    ButtonData:>"FrontierQ",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"FrontierToDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FrontierToDFA"],

Cell["\<\
Constructing a frontier automaton. The word pairs are used as \
rewrite rules on the input.\
\>", "ExampleText",
  CellTags->"FrontierToDFA"],

Cell[BoxData[{
    \(\(\[CapitalGamma] = {{aa, \[Epsilon]}, {bb, \[Epsilon]}, {baa, 
            b}, {bab, a}, {aba, b}, {abb, a}};\)\), "\n", 
    \(FrontierQ[First /@ \[CapitalGamma], {a, b}]\)}], "Input",
  CellLabel->"In[626]:=",
  AspectRatioFixed->True,
  CellTags->"FrontierToDFA"],

Cell["\<\
Here is a table of these rewrite rules, and the corresponding \
DFA.\
\>", "ExampleText",
  CellTags->"FrontierToDFA"],

Cell[BoxData[
    \(PrintEquations[\[CapitalGamma]]\)], "Input",
  CellLabel->"In[91]:=",
  CellTags->"FrontierToDFA"],

Cell[BoxData[
    \(m = FrontierToDFA[\[CapitalGamma], {\[Epsilon]}]\)], "Input",
  CellLabel->"In[92]:=",
  AspectRatioFixed->True,
  CellTags->"FrontierToDFA"],

Cell["\<\
The states of the automaton correspond to the proper prefixes of \
the frontier associated with \[CapitalGamma].\
\>", "ExampleText",
  CellTags->"FrontierToDFA"],

Cell[BoxData[
    \(States[m]\)], "Input",
  CellLabel->"In[93]:=",
  CellTags->"FrontierToDFA"],

Cell["A few words accepted by the automaton.", "ExampleText",
  CellTags->"FrontierToDFA"],

Cell[BoxData[
    \(LanguageFA[\ m, \ \(-6\)\ ]\)], "Input",
  CellLabel->"In[99]:=",
  CellTags->"FrontierToDFA"],

Cell["We can check that the automaton works as expected.", "ExampleText",
  CellTags->"FrontierToDFA"],

Cell[BoxData[
    \(\(WordReduce[#, \[CapitalGamma]] &\)\  /@ \ Flatten[%]\)], "Input",
  CellLabel->"In[100]:=",
  CellTags->"FrontierToDFA"],

Cell["FSMQ", "ObjectName",
  CellTags->"FSMQ"],

Cell["Function", "Highlight",
  CellTags->"FSMQ"],

Cell[TextData[{
  StyleBox["FSMQ[M]", "MR"],
  "  tests whether ",
  StyleBox["M", "MR"],
  " is a finite state machine."
}], "Usage",
  CellTags->"FSMQ"],

Cell[TextData[{
  " Currently allowed headers for ",
  StyleBox["M", "MR"],
  ": DFA, NFA, FA, SA, NNFA and BEFA."
}], "Notes",
  CellTags->"FSMQ"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FSMQ"],

Cell["Should be used in functions defined on machines.", "ExampleText",
  CellTags->"FSMQ"],

Cell["f[M_?FSMQ] := ToDFA[ReverseFA[M]];", "Input",
  CellLabel->"In[2]:=",
  CellTags->"FSMQ"],

Cell[BoxData[
    \(\(\(f[\ IthSymbolFA[a, 3, 2]]\)\(\ \)\)\)], "Input",
  CellLabel->"In[3]:=",
  CellTags->"FSMQ"],

Cell[BoxData[
    \(f[\ Range[5]\ ]\)], "Input",
  CellLabel->"In[185]:=",
  CellTags->"FSMQ"],

Cell["FullQFA", "ObjectName",
  CellTags->"FullQFA"],

Cell["Function", "Highlight",
  CellTags->"FullQFA"],

Cell[TextData[{
  StyleBox["EmptyQFA[M]", "MR"],
  "  tests whether the acceptance language of ",
  StyleBox["M", "MR"],
  " is empty "
}], "Usage",
  CellTags->"FullQFA"],

Cell["\<\
The algorithm tests for emptyness of the complement and thus \
involves conversion to a DFA; time and space complexity may be exponential.\
\
\>", "Notes",
  CellTags->"FullQFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToDFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ComplementFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["EmptyQFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"FullQFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FullQFA"],

Cell[TextData[{
  "Two simple machines with empty intersection and union ",
  Cell[BoxData[
      \(TraditionalForm\`{a, b}\^\(\(\ \)\(*\)\)\)]],
  ". "
}], "ExampleText",
  CellTags->"FullQFA"],

Cell[BoxData[{
    \(\(m1\  = \ DFA[3, 2, {{1, 1, 3}, {2, 3, 3}}, 1, {1, 2}\ ];\)\), "\n", 
    \(\(m2\  = \ DFA[3, 2, {{1, 1, 3}, {2, 3, 3}}, 1, {3}];\)\)}], "Input",
  CellLabel->"In[591]:=",
  CellTags->"FullQFA"],

Cell[BoxData[{
    \(EmptyQFA[IntersectionFA[m1, m2]]\), "\n", 
    \(FullQFA[UnionFA[m1, m2]]\)}], "Input",
  CellLabel->"In[594]:=",
  CellTags->"FullQFA"],

Cell["FullTransitionMatrixFA", "ObjectName",
  CellTags->"FullTransitionMatrixFA"],

Cell["Function", "Highlight",
  CellTags->"FullTransitionMatrixFA"],

Cell[TextData[{
  StyleBox["FullTransitionMatrixFA[M]", "MR"],
  "  returns the transitions of a machine expressed as a matrix over a \
semiring  (integers, Boolean, or tropical)."
}], "Usage",
  CellTags->"FullTransitionMatrixFA"],

Cell[TextData[{
  "Options\n",
  Cell[BoxData[GridBox[{
          {"Type", "\[Rule]", "None", \(Boolean, Tropical\)},
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"FullTransitionMatrixFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Type->Boolean", "MR"],
  " returns a 0/1-matrix for efficiency reasons."
}], "Notes",
  CellTags->"FullTransitionMatrixFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  "  produces ordinary matrices, rather than ",
  ButtonBox["SparseArray",
    ButtonStyle->"RefGuideLink"],
  " objects as by default."
}], "Notes",
  CellTags->"FullTransitionMatrixFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["TransitionMatrixFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"FullTransitionMatrixFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FullTransitionMatrixFA"],

Cell["A DFA for a finite language and its transition matrix.", "ExampleText",
  CellTags->"FullTransitionMatrixFA"],

Cell["m = BuildDFA[aaa,abb,bbb,2]", "Input",
  CellLabel->"In[144]:=",
  CellTags->"FullTransitionMatrixFA"],

Cell[BoxData[{
    \(\(A\  = \ FullTransitionMatrixFA[m];\)\), "\n", 
    \(A\  // \ MatrixForm\)}], "Input",
  CellLabel->"In[145]:=",
  CellTags->"FullTransitionMatrixFA"],

Cell["\<\
The 3rd power of the matrix indicates that there are 3 accepted \
words.\
\>", "ExampleText",
  CellTags->"FullTransitionMatrixFA"],

Cell[BoxData[
    \(\(MatrixPower[A, 3]\)[\([1, 7]\)]\)], "Input",
  CellLabel->"In[147]:=",
  CellTags->"FullTransitionMatrixFA"],

Cell["FullTransitions", "ObjectName",
  CellTags->"FullTransitions"],

Cell["Function", "Highlight",
  CellTags->"FullTransitions"],

Cell[TextData[{
  StyleBox["FullTransitions[M]", "MR"],
  "  returns a list of the transitionsa finite state machine ",
  StyleBox["M", "MR"],
  "."
}], "Usage",
  CellTags->"FullTransitions"],

Cell[TextData[{
  "The entries in a labeled transition list are of the ",
  StyleBox["{p,a,q}", "MR"],
  " where ",
  StyleBox["p", "MR"],
  " and ",
  StyleBox["q", "MR"],
  " are states in positional notation, and ",
  StyleBox["a", "MR"],
  " is a symbol from the alphabet of the automaton (not in positional \
notation) or \[Epsilon]."
}], "Notes",
  CellTags->"FullTransitions"],

Cell[TextData[{
  "See also ",
  ButtonBox["Transitions",
    ButtonData:>"Transitions",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["FullTransitionMatrixFA",
    ButtonData:>"FullTransitionMatrixFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"FullTransitions"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FullTransitions"],

Cell["A DFA for a finite language and its transition table.", "ExampleText",
  CellTags->"FullTransitions"],

Cell["m = BuildDFA[aaa,abb,bbb,2]", "Input",
  CellLabel->"In[250]:=",
  CellTags->"FullTransitions"],

Cell[BoxData[
    \(FullTransitions[m]\  // \ TableForm\)], "Input",
  CellLabel->"In[251]:=",
  CellTags->"FullTransitions"],

Cell["FunctionToDFA", "ObjectName",
  CellTags->"FunctionToDFA"],

Cell["Function", "Highlight",
  CellTags->"FunctionToDFA"],

Cell[TextData[{
  StyleBox["FunctionToDFA[Q,K,del,q0,F,opts]", "MR"],
  "  returns the DFA determined by ",
  StyleBox["Q", "MR"],
  ", ",
  StyleBox["K", "MR"],
  ", ",
  StyleBox["del", "MR"],
  ", ",
  StyleBox["q0", "MR"],
  " and ",
  StyleBox["F", "MR"],
  ". "
}], "Usage",
  CellTags->"FunctionToDFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Alphabetic", "\[Rule]", "True", "-"},
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"FunctionToDFA"],

Cell[TextData[{
  "Here  ",
  Cell[BoxData[
      \(TraditionalForm\`\[Delta]\  : \ 
        Q\ \[Cross]\ K\ \[LongRightArrow]\ Q\)]],
  "  is a function that determines the transitions of the machine."
}], "Notes",
  CellTags->"FunctionToDFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Alphabetic->False", "MR"],
  " uses integers ",
  Cell[BoxData[
      \(TraditionalForm\`{1, 2, \[Ellipsis], k}\)]],
  "  as input to ",
  Cell[BoxData[
      \(TraditionalForm\`\[Delta]\)]],
  ", rather then letters of the alphabet. For digit alphabets the integers ",
  Cell[BoxData[
      \(TraditionalForm\`{0, 1, \[Ellipsis], k - 1}\)]],
  "  are supplied as input to the transition function."
}], "Notes",
  CellTags->"FunctionToDFA"],

Cell[TextData[{
  "Requires prior construction of the state set ",
  Cell[BoxData[
      \(TraditionalForm\`Q\)]],
  ", see ",
  ButtonBox["Closure",
    ButtonStyle->"AddOnsLink"],
  " for an alternative method that allows one to construct the accessible \
part directly."
}], "Notes",
  CellTags->"FunctionToDFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["FunctionToSA",
    ButtonData:>"FunctionToSA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["GenerateDFA",
    ButtonData:>"GenerateDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"FunctionToDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FunctionToDFA"],

Cell["A machine whose states are pairs of modular numbers. ", "ExampleText",
  CellTags->"FunctionToDFA"],

Cell[BoxData[
    \(Q\  = \ CartesianProduct[3, 5] - 1\)], "Input",
  CellLabel->"In[637]:=",
  CellTags->"FunctionToDFA"],

Cell[BoxData[{
    \(\(del[{p_, q_}, s_\ ] := {Mod[p + s + 1, 3], 
          Mod[q + 1, 5]};\)\), "\n", 
    \(m\  = \ 
      FunctionToDFA[Q, 2, del, 1, {1}, \ 
        Alphabetic \[Rule] False]\)}], "Input",
  CellLabel->"In[681]:=",
  CellTags->"FunctionToDFA"],

Cell[TextData[{
  "The language consists of all words of length a multiple of 5 such that the \
number of ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  "'s plus twice the number of ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  "'s is divisible by 3."
}], "ExampleText",
  CellTags->"FunctionToDFA"],

Cell[BoxData[
    \(LanguageFA[\ m, \ \(-5\)\ ]\)], "Input",
  CellLabel->"In[683]:=",
  CellTags->"FunctionToDFA"],

Cell["FunctionToSA", "ObjectName",
  CellTags->"FunctionToSA"],

Cell["Function", "Highlight",
  CellTags->"FunctionToSA"],

Cell[TextData[{
  StyleBox["FunctionToSA[Q,K,del]", "MR"],
  "  returns the semiautomaton determined by ",
  StyleBox["Q", "MR"],
  ",  ",
  StyleBox["K", "MR"],
  ",  and  ",
  StyleBox["del", "MR"],
  ". "
}], "Usage",
  CellTags->"FunctionToSA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Alphabetic", "\[Rule]", "True", "-"},
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"FunctionToSA"],

Cell[TextData[{
  "Here  ",
  Cell[BoxData[
      \(TraditionalForm\`\[Delta]\  : \ 
        Q\ \[Cross]\ K\ \[LongRightArrow]\ \(\[ScriptCapitalP](Q)\)\)]],
  "  is a function describing the transition behavior."
}], "Notes",
  CellTags->"FunctionToSA"],

Cell[TextData[{
  "Option ",
  StyleBox["Alphabetic->False", "MR"],
  " uses integers ",
  Cell[BoxData[
      \(TraditionalForm\`{1, 2, \[Ellipsis], k}\)]],
  "  as input to ",
  Cell[BoxData[
      \(TraditionalForm\`\[Delta]\)]],
  ", rather then letters of the alphabet."
}], "Notes",
  CellTags->"FunctionToSA"],

Cell[TextData[{
  "Requires prior construction of the state set ",
  Cell[BoxData[
      \(TraditionalForm\`Q\)]],
  ", see ",
  ButtonBox["Closure",
    ButtonStyle->"AddOnsLink"],
  " for an alternative method that allows one to construct the accessible \
part directly."
}], "Notes",
  CellTags->"FunctionToSA"],

Cell[TextData[{
  "See also ",
  ButtonBox["FunctionToDFA",
    ButtonData:>"FunctionToDFA",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["FunctionToCA",
    ButtonData:>"FunctionToCA",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"FunctionToSA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FunctionToSA"],

Cell["\<\
Constructing a semiautomaton with 4 states by specifying the \
transition function.\
\>", "ExampleText",
  CellTags->"FunctionToSA"],

Cell[BoxData[{
    \(\(Q\  = \ Range[0, 3];\)\), "\n", 
    \(del[0, 1]\  := \ {0, 1}\), "\n", 
    \(del[p_, 1]\  := \ {Mod[p + 1, 4]}\), "\n", 
    \(del[p_, 2]\  := \ {Mod[p + 2, 4]}\)}], "Input",
  CellLabel->"In[1]:=",
  CellTags->"FunctionToSA"],

Cell[BoxData[
    \(s\  = \ 
      FunctionToSA[\ Q, \ 2, \ del, \ 
        Alphabetic \[Rule] False\ \ ]\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->"FunctionToSA"],

Cell["Adding initial and final states.", "ExampleText",
  CellTags->"FunctionToSA"],

Cell[BoxData[
    \(ss = \ SetInitialFinalFA[s, 1, \ 1]\)], "Input",
  CellLabel->"In[8]:=",
  CellTags->"FunctionToSA"],

Cell[BoxData[
    \(LanguageFA[\ ss, \(-8\), SizeOnly \[Rule] True]\)], "Input",
  CellLabel->"In[9]:=",
  CellTags->"FunctionToSA"],

Cell["GenerateDFA", "ObjectName",
  CellTags->"GenerateDFA"],

Cell["Function", "Highlight",
  CellTags->"GenerateDFA"],

Cell[TextData[{
  StyleBox["GenerateDFA[q0,dot,K,fin]", "MR"],
  "  produces the cyclic semimodule generated by ",
  StyleBox["q0", "MR"],
  " and operation ",
  StyleBox["dot, ", "MR"],
  "as well as the corresponding accessible DFA.  "
}], "Usage",
  CellTags->"GenerateDFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"},
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"GenerateDFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  "  returns the pair  ",
  Cell[BoxData[
      \(TraditionalForm\`{M, W}\)]],
  "  where ",
  Cell[BoxData[
      \(TraditionalForm\`W\)]],
  " a set of witnesses for the elements of ",
  Cell[BoxData[
      \(TraditionalForm\`Q\)]],
  ", the structured state set, and ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  " the corresponding DFA whose final states are determined by predicate ",
  StyleBox["fin", "Program"],
  ". "
}], "Notes",
  CellTags->"GenerateDFA"],

Cell[TextData[{
  "The operation ",
  StyleBox["dot[p,s]", "Program"],
  " must return an element of the semimodule (where ",
  StyleBox["p", "Program"],
  " is an element of the semimodule and ",
  StyleBox["s", "Program"],
  " is a symbol in the alphabet). "
}], "Notes",
  CellTags->"GenerateDFA"],

Cell[TextData[{
  "Options are passed on to ",
  ButtonBox["GenerateCSM",
    ButtonStyle->"AddOnsLink"],
  ".  This is useful in particular for  ",
  ButtonBox["BoundCSM",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"GenerateDFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["GenerateCSM",
    ButtonData:>"GenerateCSM",
    ButtonStyle->"AddOnsLink"],
  "   and  ",
  ButtonBox["FunctionToDFA",
    ButtonData:>"FunctionToDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"GenerateDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"GenerateDFA"],

Cell[TextData[{
  "An operation based on modular arithmetic on ",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalZ]\_6\[Cross]\ \
\[DoubleStruckCapitalZ]\_3\)]],
  ". "
}], "ExampleText",
  CellTags->"GenerateDFA"],

Cell[BoxData[{
    \(\(dot[{p_, q_}, "\<a\>"] := {Mod[p + 2, 6], q};\)\), "\n", 
    \(\(dot[{p_, q_}, "\<b\>"] := {p, Mod[q + 1, 3]};\)\)}], "Input",
  CellLabel->"In[91]:=",
  CellTags->"GenerateDFA"],

Cell[BoxData[
    \({m, W} = 
      GenerateDFA[{0, 0}, dot, 2, # \[Equal] {0, 0} &, 
        Full \[Rule] True]\)], "Input",
  CellLabel->"In[93]:=",
  AspectRatioFixed->True,
  CellTags->"GenerateDFA"],

Cell[BoxData[
    \(GenerateDFA[{0, 0}, dot, 2, # \[Equal] {0, 0} &\ ]\  // \ 
      FullForm\)], "Input",
  CellLabel->"In[98]:=",
  AspectRatioFixed->True,
  CellTags->"GenerateDFA"],

Cell[TextData[{
  "The elements of the semimodule  ",
  Cell[BoxData[
      \(TraditionalForm\`Q\ \  \[Subset] \ \[DoubleStruckCapitalZ]\_6\[Cross]\
\ \[DoubleStruckCapitalZ]\_3\)]],
  " and their witnesses."
}], "ExampleText",
  CellTags->"GenerateDFA"],

Cell["{Q,W} // Thread // MatrixForm", "Input",
  CellLabel->"In[97]:=",
  AspectRatioFixed->True,
  CellTags->"GenerateDFA"],

Cell[BoxData[
    \(LanguageFA[m, \(-10\), SizeOnly \[Rule] True]\)], "Input",
  CellLabel->"In[100]:=",
  AspectRatioFixed->True,
  CellTags->"GenerateDFA"],

Cell["GenerateQuotients", "ObjectName",
  CellTags->"GenerateQuotients"],

Cell["Function", "Highlight",
  CellTags->"GenerateQuotients"],

Cell[TextData[{
  StyleBox["GenerateQuotients[L,K]", "MR"],
  "  computes all quotients of a finite language ",
  StyleBox["L", "MR"],
  "  over alphabet ",
  StyleBox["K", "MR"],
  " ."
}], "Usage",
  CellTags->"GenerateQuotients"],

Cell[TextData[{
  "The left quotients form the state set of the automaton produced by ",
  StyleBox["QuotientDFA", "MR"],
  "."
}], "Notes",
  CellTags->"GenerateQuotients"],

Cell[TextData[{
  "See also ",
  ButtonBox["BuildDFA",
    ButtonData:>"BuildDFA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["LeftQuotient",
    ButtonData:>"LeftQuotient",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["QuotientDFA",
    ButtonData:>"QuotientDFA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["QuotientMachineFA",
    ButtonData:>"QuotientMachineFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"GenerateQuotients"],

Cell["Further Examples", "ExampleSection",
  CellTags->"GenerateQuotients"],

Cell["\<\
The language of all words of length 3 other than aaa and all its \
left quotients.\
\>", "ExampleText",
  CellTags->"GenerateQuotients"],

Cell["W = Rest[Words[3]]", "Input",
  CellLabel->"In[155]:=",
  CellTags->"GenerateQuotients"],

Cell[BoxData[
    \(GenerateQuotients[W, 2]\)], "Input",
  CellLabel->"In[166]:=",
  CellTags->"GenerateQuotients"],

Cell[BoxData[
    \({{"\<aab\>", "\<aba\>", "\<abb\>", "\<baa\>", "\<bab\>", "\<bba\>", \
"\<bbb\>"}, {"\<ab\>", "\<ba\>", "\<bb\>"}, {"\<aa\>", "\<ab\>", "\<ba\>", \
"\<bb\>"}, {"\<b\>"}, {"\<a\>", "\<b\>"}, {}, {\[Epsilon]}}\)], "Input",
  CellLabel->"Out[166]=",
  CellTags->"GenerateQuotients"],

Cell["GrowthSeriesDFA", "ObjectName",
  CellTags->"GrowthSeriesDFA"],

Cell["Function", "Highlight",
  CellTags->"GrowthSeriesDFA"],

Cell[TextData[{
  StyleBox["GrowthSeriesDFA[M,x]", "MR"],
  "  returns the rational function in variable ",
  StyleBox["x", "MR"],
  " that represents the generating function for the growth sequence of DFA ",
  StyleBox["M", "MR"],
  "."
}], "Usage",
  CellTags->"GrowthSeriesDFA"],

Cell[TextData[{
  "Options\n",
  Cell[BoxData[
      FormBox[GridBox[{
            {
              StyleBox["Full",
                "MR"], "\[Rule]", 
              StyleBox["False",
                "MR"], "-"},
            {
              StyleBox["Method",
                "MR"], "\[Rule]", "-", 
              StyleBox["OneStepRowReduction",
                "MR"]}
            },
          ColumnWidths->Automatic,
          ColumnAlignments->{Left}], TraditionalForm]]]
}], "Notes",
  CellTags->"GrowthSeriesDFA"],

Cell[TextData[{
  "Option  ",
  StyleBox["Full->True", "MR"],
  "  causes the whole solution vector to be returned, along with the \
generating functionFinal."
}], "Notes",
  CellTags->"GrowthSeriesDFA"],

Cell[TextData[{
  "Option  ",
  StyleBox["Method->OneStepRowReduction", "MR"],
  "  can help to solve this system of equations (should be automatic, but is \
not)."
}], "Notes",
  CellTags->"GrowthSeriesDFA"],

Cell["\<\
Performing Taylor expansion on the generating function will produce \
a few terms of the growth function.\
\>", "Notes",
  CellTags->"GrowthSeriesDFA"],

Cell["\<\
One can use  to compute a closed form for the terms in the \
series.\
\>", "Notes",
  CellTags->"GrowthSeriesDFA"],

Cell[TextData[{
  "See ",
  ButtonBox["TransitionMatrixFA",
    ButtonData:>"TransitionMatrixFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Tropical",
    ButtonData:>"Tropical",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"GrowthSeriesDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"GrowthSeriesDFA"],

Cell["\<\
A small DFA and the generating function for the growth sequence of \
its acceptance language.\
\>", "ExampleText",
  CellTags->"GrowthSeriesDFA"],

Cell[BoxData[
    \(m\  = \ 
      DFA[8, 2, {{2, 2, 2, 5, 5, 7, 7, 2}, {3, 4, 3, 3, 6, 8, 7, 4}}, 
        1, {1, 2, 3, 4, 5, 6, 8}]\)], "Input",
  CellLabel->"In[252]:=",
  CellTags->"GrowthSeriesDFA"],

Cell[BoxData[{
    \(Clear[x]\), "\n", 
    \(gf\  = \ GrowthSeriesDFA[m, x]\)}], "Input",
  CellLabel->"In[258]:=",
  CellTags->"GrowthSeriesDFA"],

Cell[BoxData[
    \(Series[\ gf, \ {x, \ 0, 8}]\)], "Input",
  CellLabel->"In[260]:=",
  CellTags->"GrowthSeriesDFA"],

Cell["We can compare this to a few terms of the growth sequence.", \
"ExampleText",
  CellTags->"GrowthSeriesDFA"],

Cell[BoxData[
    \(LanguageFA[m, \(-8\), SizeOnly \[Rule] True]\)], "Input",
  CellLabel->"In[261]:=",
  CellTags->"GrowthSeriesDFA"],

Cell["Here is the whole solution vector of rational functions.", "ExampleText",
  CellTags->"GrowthSeriesDFA"],

Cell[BoxData[
    \(\(GrowthSeriesDFA[m, x, Full \[Rule] True]\  // \ Last\)\  // \ 
      Simplify\)], "Input",
  CellLabel->"In[263]:=",
  CellTags->"GrowthSeriesDFA"],

Cell["HomomorphismFA", "ObjectName",
  CellTags->"HomomorphismFA"],

Cell["Function", "Highlight",
  CellTags->"HomomorphismFA"],

Cell[TextData[{
  StyleBox["HomomorphismFA[M,h,K]", "MR"],
  "  generates an FA that accepts the image of the acceptance language of ",
  StyleBox["M", "MR"],
  " under homomorphism ",
  StyleBox["h", "MR"],
  ". "
}], "Usage",
  CellTags->"HomomorphismFA"],

Cell[TextData[{
  "The homomorphism is given as a list of words over the target alphabet ",
  StyleBox["K", "MR"],
  ". "
}], "Notes",
  CellTags->"HomomorphismFA"],

Cell[TextData[{
  "Alternatively, ",
  StyleBox["h", "MR"],
  " may be a homomorphism as defined by ",
  StyleBox["MakeHomomorphism", "MR"],
  "."
}], "Notes",
  CellTags->"HomomorphismFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["MakeHomomorphism",
    ButtonData:>"MakeHomomorphism",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["CoverQDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"HomomorphismFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"HomomorphismFA"],

Cell[TextData[{
  "Applying a homomorphism that erases letter ",
  Cell[BoxData[
      \(TraditionalForm\`c\)]],
  ", replaces ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  " by 0, and ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  " by 11."
}], "ExampleText",
  CellTags->"HomomorphismFA"],

Cell[BoxData[{
    \(m = IthSymbolFA[b, \(-3\), {a, b, c}]\), "\n", 
    \(mm = HomomorphismFA[m, {0, 11, \[Epsilon]}, \(-2\)]\)}], "Input",
  CellLabel->"In[706]:=",
  AspectRatioFixed->True,
  CellTags->"HomomorphismFA"],

Cell[BoxData[
    \(LanguageFA[mm, \(-5\)]\)], "Input",
  CellLabel->"In[68]:=",
  CellTags->"HomomorphismFA"],

Cell["Hopcroft", "ObjectName",
  CellTags->"Hopcroft"],

Cell["Option", "Highlight",
  CellTags->"Hopcroft"],

Cell[TextData[{
  StyleBox["Hopcroft", "MR"],
  "  is an option for ",
  StyleBox["StateMergeDFA", "MR"],
  "."
}], "Usage",
  CellTags->"Hopcroft"],

Cell[TextData[{
  "Causes Hopcroft's ",
  Cell[BoxData[
      \(TraditionalForm\`O(n\ lg\ n)\)]],
  " algorithm to be used. "
}], "Notes",
  CellTags->"Hopcroft"],

Cell["\<\
Note that in the internal implementation, the quadratic standard \
algorithm is actually faster.\
\>", "Notes",
  CellTags->"Hopcroft"],

Cell[TextData[{
  "See also  ",
  ButtonBox["StateMergeDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Hopcroft"],

Cell["HurwitzProduct", "ObjectName",
  CellTags->"HurwitzProduct"],

Cell["Function", "Highlight",
  CellTags->"HurwitzProduct"],

Cell[TextData[{
  StyleBox["HurwitzProduct[S1,S2]", "MR"],
  "  computes the Hurwitz product of two formal power series."
}], "Usage",
  CellTags->"HurwitzProduct"],

Cell["Corresponds to the shuffle operation.", "Notes",
  CellTags->"HurwitzProduct"],

Cell[TextData[{
  "See ",
  ButtonBox["ToSeries",
    ButtonData:>"ToSeries",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["ShuffleProductFA",
    ButtonData:>"ShuffleProductFA",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["WordShuffle",
    ButtonData:>"WordShuffle",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"HurwitzProduct"],

Cell["Further Examples", "ExampleSection",
  CellTags->"HurwitzProduct"],

Cell["Shuffling words produces rather complicated multiplicities.", \
"ExampleText",
  CellTags->"HurwitzProduct"],

Cell["\<\
L = {aa,ab,ba,bb};
WordShuffle[L,L]\
\>", "Input",
  CellLabel->"In[720]:=",
  AspectRatioFixed->True,
  CellTags->"HurwitzProduct"],

Cell["\<\
LL = L // LanguageToSeries;
H = HurwitzProduct[LL,LL]\
\>", "Input",
  CellLabel->"In[722]:=",
  AspectRatioFixed->True,
  CellTags->"HurwitzProduct"],

Cell[BoxData[
    \(LL\)], "Input",
  CellLabel->"In[724]:=",
  CellTags->"HurwitzProduct"],

Cell["H // ToSeries", "Input",
  CellLabel->"In[714]:=",
  AspectRatioFixed->True,
  CellTags->"HurwitzProduct"],

Cell[BoxData[
    TagBox[\("\<aaaa\>" + 2\ "\<aaab\>" + 4\ "\<aaba\>" + 3\ "\<aabb\>" + 
        4\ "\<abaa\>" + 5\ "\<abab\>" + 4\ "\<abba\>" + 2\ "\<abbb\>" + 
        2\ "\<baaa\>" + 4\ "\<baab\>" + 5\ "\<baba\>" + 4\ "\<babb\>" + 
        3\ "\<bbaa\>" + 4\ "\<bbab\>" + 2\ "\<bbba\>" + "\<bbbb\>"\),
      HoldForm]], "Input",
  CellLabel->"Out[714]=",
  CellTags->"HurwitzProduct"],

Cell[TextData[{
  "Since ",
  StyleBox["WordShuffle", "MR"],
  " eliminates duplicates we have to apply it pointwise to preserve \
multiplicities."
}], "ExampleText",
  CellTags->"HurwitzProduct"],

Cell[BoxData[{
    \(\(Apply[\ WordShuffle, \ CartesianProduct[L, L], {1}]\  // \ 
        Flatten;\)\), "\n", 
    \(\(%\  // \ Frequencies\)\  // \ TableForm\)}], "Input",
  CellLabel->"In[715]:=",
  CellTags->"HurwitzProduct"],

Cell["IdealBaseFA", "ObjectName",
  CellTags->"IdealBaseFA"],

Cell["Function", "Highlight",
  CellTags->"IdealBaseFA"],

Cell[TextData[{
  StyleBox["IdealBaseFA[M]", "MR"],
  "  constructs a DFA for the ideal base of the language of ",
  StyleBox["M", "MR"],
  "."
}], "Usage",
  CellTags->"IdealBaseFA"],

Cell[TextData[{
  "Does not check whether the language of ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  " really forms an ideal; use ",
  ButtonBox["IdealQFA",
    ButtonStyle->"AddOnsLink"],
  " to verify."
}], "Notes",
  CellTags->"IdealBaseFA"],

Cell[TextData[{
  StyleBox["IdealBaseFA", "MR"],
  " is just a wrapper that minimizes the output of  ",
  ButtonBox["ToFactorFA",
    ButtonData:>"ToFactorFA",
    ButtonStyle->"AddOnsLink"],
  " ."
}], "Notes",
  CellTags->"IdealBaseFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToFactorFA",
    ButtonData:>"ToFactorFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["IdealQFA",
    ButtonData:>"IdealQFA",
    ButtonStyle->"AddOnsLink"],
  " . "
}], "Notes",
  CellTags->"IdealBaseFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"IdealBaseFA"],

Cell["A machine whose acceptance language is an ideal.", "ExampleText",
  CellTags->"IdealBaseFA"],

Cell[BoxData[{
    \(m = ComplementFA[CounterDFA[a, \(-2\), 2]]\), "\n", 
    \(IdealQFA[m]\)}], "Input",
  CellLabel->"In[249]:=",
  CellTags->"IdealBaseFA"],

Cell[TextData[{
  "The ideal base here is the language of all words containing 3 ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  "'s, and starting and ending in ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  " (as a regular expression: ",
  Cell[BoxData[
      \(TraditionalForm\`a\ \(b\^*\) a\ \(b\^*\)\ a\)]],
  ")."
}], "ExampleText",
  CellTags->"IdealBaseFA"],

Cell[BoxData[{
    \(mm\  = \ IdealBaseFA[m]\), "\n", 
    \(LanguageFA[\ mm, \(-6\)\ ]\)}], "Input",
  CellLabel->"In[251]:=",
  CellTags->"IdealBaseFA"],

Cell["Here is a machine with a finite ideal base.", "ExampleText",
  CellTags->"IdealBaseFA"],

Cell[BoxData[{
    \(\(m\  = 
        DFA[5, 3, {{2, 2, 2, 4, 2}, {3, 4, 5, 4, 4}, {1, 4, 1, 4, 1}}, 
          1, {4}];\)\), "\n", 
    \(LanguageFA[IdealBaseFA[m], Full \[Rule] True]\)}], "Input",
  CellLabel->"In[69]:=",
  CellTags->"IdealBaseFA"],

Cell[TextData[{
  "We can verify the construction by building an ideal base for ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  " \"by hand\" (the algorithm in ",
  StyleBox["IdealBaseFA", "MR"],
  " uses a different method)."
}], "ExampleText",
  CellTags->"IdealBaseFA"],

Cell[BoxData[
    \(With[\ {one\  = \ LengthDFA[1, 3], \ 
          mc\  = \ \ ComplementFA[m]}, \[IndentingNewLine]IntersectionFA[\ 
          ConcatenateFA[mc, one], ConcatenateFA[one, mc], m\ ]\ ] // \ 
      MinimizeFA\)], "Input",
  CellLabel->"In[71]:=",
  CellTags->"IdealBaseFA"],

Cell[BoxData[
    \(LanguageFA[%, Full \[Rule] True\ ]\)], "Input",
  CellLabel->"In[72]:=",
  CellTags->"IdealBaseFA"],

Cell["IdealQFA", "ObjectName",
  CellTags->"IdealQFA"],

Cell["Function", "Highlight",
  CellTags->"IdealQFA"],

Cell[TextData[{
  StyleBox["IdealQFA[M]", "MR"],
  "  tests whether the language of ",
  StyleBox["M", "MR"],
  " forms an ideal. "
}], "Usage",
  CellTags->"IdealQFA"],

Cell[TextData[{
  "Use as test for ",
  StyleBox["IdeaBaseFA", "MR"],
  " to verify that M is suitable input."
}], "Notes",
  CellTags->"IdealQFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["IdealBaseFA",
    ButtonData:>"IdealBaseFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"IdealQFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"IdealQFA"],

Cell["A machine whose acceptance language is an ideal.", "ExampleText",
  CellTags->"IdealQFA"],

Cell[BoxData[
    \(m = ComplementFA[CounterDFA[a, \(-2\), 2]]\)], "Input",
  CellLabel->"In[71]:=",
  CellTags->"IdealQFA"],

Cell[BoxData[
    \(IdealQFA[m]\)], "Input",
  CellLabel->"In[72]:=",
  CellTags->"IdealQFA"],

Cell["Implementation", "ObjectName",
  CellTags->"Implementation"],

Cell["Option", "Highlight",
  CellTags->"Implementation"],

Cell[TextData[{
  StyleBox["Implementation", "MR"],
  " Implementation is an option for ",
  StyleBox["LanguageFA", "MR"],
  ". "
}], "Usage",
  CellTags->"Implementation"],

Cell["Allows selection of one of three algorithms.", "Notes",
  CellTags->"Implementation"],

Cell[TextData[{
  "See ",
  ButtonBox["LanguageFA",
    ButtonData:>"LanguageFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Implementation"],

Cell["InfiniteQFA", "ObjectName",
  CellTags->"InfiniteQFA"],

Cell["Function", "Highlight",
  CellTags->"InfiniteQFA"],

Cell[TextData[{
  StyleBox["InfiniteQFA[M]", "MR"],
  "  tests whether the acceptance language of ",
  StyleBox["M", "MR"],
  " is infinite."
}], "Usage",
  CellTags->"InfiniteQFA"],

Cell[TextData[{
  "Searches for cycles in the trim part of ",
  StyleBox["M", "MR"],
  "."
}], "Notes",
  CellTags->"InfiniteQFA"],

Cell[TextData[{
  "See ",
  ButtonBox["ToGraphFA",
    ButtonData:>"ToGraphFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"InfiniteQFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"InfiniteQFA"],

Cell["A machine with a finite language.", "ExampleText",
  CellTags->"InfiniteQFA"],

Cell["m = BuildDFA[ Words[-3,2], 3 ]", "Input",
  CellLabel->"In[258]:=",
  CellTags->"InfiniteQFA"],

Cell[BoxData[
    \(InfiniteQFA[m]\)], "Input",
  CellLabel->"In[259]:=",
  CellTags->"InfiniteQFA"],

Cell["But the Kleene star should have infinite language.", "ExampleText",
  CellTags->"InfiniteQFA"],

Cell["mm = StarFA[m]", "Input",
  CellLabel->"In[260]:=",
  CellTags->"InfiniteQFA"],

Cell[BoxData[
    \(InfiniteQFA[mm]\)], "Input",
  CellLabel->"In[261]:=",
  CellTags->"InfiniteQFA"],

Cell["IniLU", "ObjectName",
  CellTags->"IniLU"],

Cell["Option", "Highlight",
  CellTags->"IniLU"],

Cell[TextData[{
  StyleBox["IniLU", "MR"],
  "  is an option for ",
  StyleBox["LocalUniversalFA", "MR"],
  "."
}], "Usage",
  CellTags->"IniLU"],

Cell["Determines the choice of initial states. ", "Notes",
  CellTags->"IniLU"],

Cell[TextData[{
  "See ",
  ButtonBox["LocalUniversalFA",
    ButtonData:>"LocalUniversalFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"IniLU"],

Cell["InitialVectorFA", "ObjectName",
  CellTags->"InitialVectorFA"],

Cell["Function", "Highlight",
  CellTags->"InitialVectorFA"],

Cell[TextData[{
  StyleBox["InitialVectorFA[M]", "MR"],
  "   returns the bitvector representing the  initial states of M."
}], "Usage",
  CellTags->"InitialVectorFA"],

Cell["For efficiency reasons, 0/1-vectors are used. ", "Notes",
  CellTags->"InitialVectorFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["FinalVectorFA",
    ButtonData:>"FinalVectorFA",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["Initial",
    ButtonData:>"Initial",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"InitialVectorFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"InitialVectorFA"],

Cell["\<\
A somewhat complicated  FA for all words of length 2, and its \
initial states bitvector.\
\>", "ExampleText",
  CellTags->"InitialVectorFA"],

Cell[BoxData[
    \(\(m = 
        UnionFA @@ \(\((ToFA[BuildDFA[#1]] &)\) /@ Words[2]\);\)\)], "Input",
  CellLabel->"In[309]:=",
  CellTags->"InitialVectorFA"],

Cell[BoxData[
    \(InitialVectorFA[m]\)], "Input",
  CellLabel->"In[310]:=",
  CellTags->"InitialVectorFA"],

Cell["Initial", "ObjectName",
  CellTags->"Initial"],

Cell["Function", "Highlight",
  CellTags->"Initial"],

Cell[TextData[{
  StyleBox["Initial[M]", "MR"],
  "  returns a list of the initial states of finite state machine M in \
positional notation."
}], "Usage",
  CellTags->"Initial"],

Cell[TextData[{
  "See also ",
  ButtonBox["InitialVectorFA",
    ButtonData:>"InitialVectorFA",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["Final",
    ButtonData:>"Final",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Initial"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Initial"],

Cell["\<\
A somewhat complicated  FA for all words of length 2, and its \
initial states bitvector.\
\>", "ExampleText",
  CellTags->"Initial"],

Cell[BoxData[
    \(\(m = 
        UnionFA @@ \(\((ToFA[BuildDFA[#1]] &)\) /@ Words[2]\);\)\)], "Input",
  CellLabel->"In[318]:=",
  CellTags->"Initial"],

Cell[BoxData[
    \(Initial[m]\)], "Input",
  CellLabel->"In[319]:=",
  CellTags->"Initial"],

Cell["IntersectionFA", "ObjectName",
  CellTags->"IntersectionFA"],

Cell["Function", "Highlight",
  CellTags->"IntersectionFA"],

Cell[TextData[{
  StyleBox["IntersectionFA[M1,M2]", "MR"],
  "  generates a finite state machine that accepts the intersection of the \
acceptance languages of ",
  StyleBox["M1", "MR"],
  " and ",
  StyleBox["M2.", "MR"],
  " "
}], "Usage",
  CellTags->"IntersectionFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"},
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"IntersectionFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  " constructs full product machine."
}], "Notes",
  CellTags->"IntersectionFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Normalize->1", "MR"],
  " preserves state set  ",
  Cell[BoxData[
      \(TraditionalForm\`Q\_1\[Cross]Q\_2\)]],
  ". "
}], "Notes",
  CellTags->"IntersectionFA"],

Cell["Uses a product machine construction.", "Notes",
  CellTags->"IntersectionFA"],

Cell["Is polyadic.", "Notes",
  CellTags->"IntersectionFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["ProductFA",
    ButtonData:>"ProductFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SumFA",
    ButtonData:>"SumFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["UnionFA",
    ButtonData:>"UnionFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"IntersectionFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"IntersectionFA"],

Cell["\<\
Two machines for unary languages. The first accepts all words of \
length a multiple of 3, and the second all words of length a multiple of 4.\
\
\>", "ExampleText",
  CellTags->"IntersectionFA"],

Cell[BoxData[{
    \(m3 = CounterDFA[a, 0, 1, Modulus \[Rule] 3]\), "\n", 
    \(m4 = CounterDFA[a, 0, 1, Modulus \[Rule] 4]\)}], "Input",
  CellLabel->"In[142]:=",
  CellTags->"IntersectionFA"],

Cell["\<\
The intersection machine accepts all words of length a multiple of \
12.\
\>", "ExampleText",
  CellTags->"IntersectionFA"],

Cell[BoxData[{
    \(IntersectionFA[m3, m4]\), "\n", 
    \(LanguageFA[\ %, \ \(-25\), \ SizeOnly \[Rule] True\ ]\)}], "Input",
  CellLabel->"In[144]:=",
  CellTags->"IntersectionFA"],

Cell["IrreducibleFA", "ObjectName",
  CellTags->"IrreducibleFA"],

Cell["Function", "Highlight",
  CellTags->"IrreducibleFA"],

Cell[TextData[{
  StyleBox["IrreducibleFA[W,K]", "MR"],
  "  returns the canonical tree automaton that accepts all words not containg \
a factor ",
  StyleBox["w", "MR"],
  " from the the list ",
  StyleBox["W", "MR"],
  ". "
}], "Usage",
  CellTags->"IrreducibleFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"},
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"IrreducibleFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  " produces a machine that accepts all words of the form ",
  Cell[BoxData[
      \(TraditionalForm\`x\[VeryThinSpace]w\)]],
  " where ",
  Cell[BoxData[
      \(TraditionalForm\`w\)]],
  " is in ",
  StyleBox["W", "MR"],
  ". The latter machine is used in ",
  StyleBox["WordReduce", "MR"],
  "."
}], "Notes",
  CellTags->"IrreducibleFA"],

Cell[TextData[{
  "Assumes that no word in ",
  StyleBox["W", "MR"],
  " occurs as a factor of another."
}], "Notes",
  CellTags->"IrreducibleFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordReduce",
    ButtonData:>"WordReduce",
    ButtonStyle->"AddOnsLink"],
  ". "
}], "Notes",
  CellTags->"IrreducibleFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"IrreducibleFA"],

Cell[TextData[{
  "Excluding consecutive ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  "'s and consecutive ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  "'s. "
}], "ExampleText",
  CellTags->"IrreducibleFA"],

Cell["W = {aa,bb}", "Input",
  CellLabel->"In[51]:=",
  CellTags->"IrreducibleFA"],

Cell[BoxData[
    \(m\  = \ IrreducibleFA[W, 2]\)], "Input",
  CellLabel->"In[57]:=",
  CellTags->"IrreducibleFA"],

Cell[BoxData[
    \(LanguageFA[\ m, \ \(-5\)\ ]\)], "Input",
  CellLabel->"In[58]:=",
  CellTags->"IrreducibleFA"],

Cell["The census function is bounded in this case.", "ExampleText",
  CellTags->"IrreducibleFA"],

Cell[BoxData[
    \(LanguageFA[\ m, \ \(-10\), \ SizeOnly \[Rule] True\ \ ]\)], "Input",
  CellLabel->"In[59]:=",
  CellTags->"IrreducibleFA"],

Cell["IsomorphicQDFA", "ObjectName",
  CellTags->"IsomorphicQDFA"],

Cell["Function", "Highlight",
  CellTags->"IsomorphicQDFA"],

Cell[TextData[{
  StyleBox["IsomorphicQDFA[M1,M2]", "MR"],
  "  tests whether two DFAs M1 and M2 are isomorphic. "
}], "Usage",
  CellTags->"IsomorphicQDFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"DoTrace", "\[Rule]", "False", "-"},
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"IsomorphicQDFA"],

Cell[TextData[{
  StyleBox["IsomorphicQDFA", "MR"],
  " can be used to test equivalence of (minimal) DFAs."
}], "Notes",
  CellTags->"IsomorphicQDFA"],

Cell[TextData[{
  "Option ",
  StyleBox["DoTrace->True", "MR"],
  " causes the isomorphism to be displayed (if it exists). "
}], "Notes",
  CellTags->"IsomorphicQDFA"],

Cell["\<\
Note that only the accessible part of the machines is considered, \
so the isomorphism may be a partial map.\
\>", "Notes",
  CellTags->"IsomorphicQDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"IsomorphicQDFA"],

Cell["Two isomorphic machines.", "ExampleText",
  CellTags->"IsomorphicQDFA"],

Cell[BoxData[{
    \(\(m1\  = \ 
        DFA[8, 2, {{1, 3, 5, 7, 1, 3, 5, 7}, {2, 4, 6, 8, 2, 4, 6, 8}}, 
          1, {5, 6, 7, 8}];\)\), "\n", 
    \(\(m2\  = \ 
        DFA[8, 2, {{2, 4, 8, 6, 2, 6, 8, 4}, {3, 1, 7, 5, 3, 5, 7, 1}}, 
          6, {1, 4, 7, 8}];\)\)}], "Input",
  CellLabel->"In[20]:=",
  CellTags->"IsomorphicQDFA"],

Cell[BoxData[
    \(IsomorphicQDFA[m1, m2]\)], "Input",
  CellLabel->"In[27]:=",
  CellTags->"IsomorphicQDFA"],

Cell[BoxData[
    \(IsomorphicQDFA[m1, m2, DoTrace \[Rule] True]\)], "Input",
  CellLabel->"In[28]:=",
  CellTags->"IsomorphicQDFA"],

Cell["IthSymbolFA", "ObjectName",
  CellTags->"IthSymbolFA"],

Cell["Function", "Highlight",
  CellTags->"IthSymbolFA"],

Cell[TextData[{
  StyleBox["IthSymbolFA[a,i,K]", "MR"],
  "  constructs an FA for the language of all words whose whose i-th symbol \
is ",
  StyleBox["a", "MR"],
  " (over alphabet K)."
}], "Usage",
  CellTags->"IthSymbolFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"},
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"IthSymbolFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  "  causes a DFA to be generated, rather than a nondeterministic machine. \
Note that the DFA is not minimal."
}], "Notes",
  CellTags->"IthSymbolFA"],

Cell[TextData[{
  "Position parameter ",
  Cell[BoxData[
      \(TraditionalForm\`i\)]],
  " may be negative, in which case the ",
  Cell[BoxData[
      \(TraditionalForm\`i\)]],
  "th symbol from the end has to be ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  ". "
}], "Notes",
  CellTags->"IthSymbolFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["IthSymbolFA",
    ButtonData:>"IthSymbolFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"IthSymbolFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"IthSymbolFA"],

Cell[TextData[{
  "All words that have a ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  " in the third position from the end, over a two-letter alphabet."
}], "ExampleText",
  CellTags->"IthSymbolFA"],

Cell[BoxData[
    \(m = IthSymbolFA[b, \(-3\), 2]\)], "Input",
  CellLabel->"In[7]:=",
  CellTags->"IthSymbolFA"],

Cell[BoxData[
    \(LanguageFA[\ m, \ \(-5\)\ ]\)], "Input",
  CellLabel->"In[8]:=",
  CellTags->"IthSymbolFA"],

Cell["\<\
These machines blow up exponentially during deterministic \
simulation.\
\>", "ExampleText",
  CellTags->"IthSymbolFA"],

Cell[BoxData[
    \(Table[\ Size[ToDFA[IthSymbolFA[a, \(-i\), 2]]], {i, 10}]\)], "Input",
  CellLabel->"In[14]:=",
  CellTags->"IthSymbolFA"],

Cell[TextData[{
  "A DFA for all words that have an ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  " in the third position from the end."
}], "ExampleText",
  CellTags->"IthSymbolFA"],

Cell[BoxData[
    \(m = 
      IthSymbolFA[a, \(-3\), 2, Normalize \[Rule] 1, \ 
        Full \[Rule] True\ ]\)], "Input",
  CellLabel->"In[26]:=",
  CellTags->"IthSymbolFA"],

Cell[BoxData[
    \(LanguageFA[\ m, \ \(-5\)\ ]\)], "Input",
  CellLabel->"In[27]:=",
  CellTags->"IthSymbolFA"],

Cell["\<\
Note that this machine is not minimal.   It's a good exercise to \
determine states with the same behavior.\
\>", "ExampleText",
  CellTags->"IthSymbolFA"],

Cell[BoxData[{
    \(\(mm\  = \ MinimizeFA[m, Normalize \[Rule] 2];\)\), "\n", 
    \(mm\  // \ Size\), "\n", 
    \(States[mm]\)}], "Input",
  CellLabel->"In[28]:=",
  CellTags->"IthSymbolFA"],

Cell["JoinSA", "ObjectName",
  CellTags->"JoinSA"],

Cell["Function", "Highlight",
  CellTags->"JoinSA"],

Cell[TextData[{
  StyleBox["JoinSA[M1,M2,A]", "MR"],
  "  determines the join of two semiautomata according to overlap list ",
  StyleBox["A", "MR"],
  "."
}], "Usage",
  CellTags->"JoinSA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"JoinSA"],

Cell[TextData[{
  "The states that are to be identified are specified in the list of pairs ",
  StyleBox["A", "MR"],
  ". "
}], "Notes",
  CellTags->"JoinSA"],

Cell[TextData[{
  "See also ",
  ButtonBox["SumFA",
    ButtonData:>"SumFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"JoinSA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"JoinSA"],

Cell["\<\
Two semiautomata that count modulo two primes. Their diagrams \
consist of cycles of length 11 and 13, respectively.\
\>", "ExampleText",
  CellTags->"JoinSA"],

Cell[BoxData[{
    \(\(s1\  = \ 
        ToSA@CounterDFA[\ a, \ 0, \ 1, \ Modulus \[Rule] \ 5\ ];\)\), "\n", 
    \(\(s2\  = \ 
        ToSA@CounterDFA[\ a, \ 0, \ 1, \ 
            Modulus \[Rule] \ \ 11\ ];\)\)}], "Input",
  CellLabel->"In[10]:=",
  CellTags->"JoinSA"],

Cell["We glue the two machines together at one place.", "ExampleText",
  CellTags->"JoinSA"],

Cell[BoxData[
    \(s\  = \ JoinSA[\ s1, s2, \ {{1, 1}}\ ]\)], "Input",
  CellLabel->"In[12]:=",
  CellTags->"JoinSA"],

Cell["The corresponding minimal DFA is larger.", "ExampleText",
  CellTags->"JoinSA"],

Cell[BoxData[
    \(m\  = \ MinimizeFA[SetInitialFinalFA[s, 1, \ 1\ ]]\)], "Input",
  CellLabel->"In[13]:=",
  CellTags->"JoinSA"],

Cell[TextData[{
  "The last missing word is ",
  Cell[BoxData[
      \(TraditionalForm\`a\^39\)]],
  ": we have solved Frobenius'  M\[UDoubleDot]nzenproblem for coins of value \
5 and 11."
}], "ExampleText",
  CellTags->"JoinSA"],

Cell[BoxData[{
    \(LanguageFA[m, \(-50\), SizeOnly \[Rule] True\ ]\), "\n", 
    \(Position[%, 0] - 1\  // \ Flatten\ \), "\n", 
    \(\((5 - 1)\) \((11 - 1)\) - 1\)}], "Input",
  CellLabel->"In[16]:=",
  CellTags->"JoinSA"],

Cell["LanguageFA", "ObjectName",
  CellTags->"LanguageFA"],

Cell["Function", "Highlight",
  CellTags->"LanguageFA"],

Cell[TextData[{
  StyleBox["LanguageFA[M,n]", "MR"],
  "  displays all words of length ",
  StyleBox["n", "MR"],
  " accepted by machine ",
  StyleBox["M", "MR"],
  "."
}], "Usage",
  CellTags->"LanguageFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Multiplicity", "\[Rule]", "False", "-"},
          {"SizeOnly", "\[Rule]", "False", "-"},
          {"MakeTrim", "\[Rule]", "False", "-"},
          {"Full", "\[Rule]", "False", "-"},
          {"BoundLFA", "\[Rule]", "50", "-"},
          {"Implementation", "\[Rule]", "1", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"LanguageFA"],

Cell[TextData[{
  "For negative ",
  StyleBox["n", "MR"],
  " all accepted words up to length  ",
  StyleBox["-n", "MR"],
  "  are generated."
}], "Notes",
  CellTags->"LanguageFA"],

Cell[TextData[{
  "Option ",
  StyleBox["SizeOnly->True", "MR"],
  " generates only the first ",
  StyleBox["n", "MR"],
  " values of the growth function of the language. "
}], "Notes",
  CellTags->"LanguageFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Multiplicity->True", "MR"],
  " generates a multiset rather than a set for nondeterministic machines. "
}], "Notes",
  CellTags->"LanguageFA"],

Cell[TextData[{
  "Option ",
  StyleBox["MakeTrim->True", "MR"],
  " first transforms the automaton into a trim FA. "
}], "Notes",
  CellTags->"LanguageFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  " generates all words accepted by a FSM, but does not check first whether \
the acceptance language is finite. One can use ",
  StyleBox["BoundLFA", "MR"],
  " to control size of computation, default is ",
  StyleBox["BoundLFA->50", "MR"],
  "."
}], "Notes",
  CellTags->"LanguageFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Implementation->i", "MR"],
  " selects an algorithm (",
  Cell[BoxData[
      \(TraditionalForm\`i\  = \ 1, 2, 3\)]],
  " for deterministic machines and ",
  Cell[BoxData[
      \(TraditionalForm\`i\  = \ 1, 2\)]],
  " for nondeterministic ones, default is ",
  Cell[BoxData[
      \(TraditionalForm\`i = 1\)]],
  ")."
}], "Notes",
  CellTags->"LanguageFA"],

Cell[TextData[{
  "Use with caution, since in general the number of accepted words grows \
exponentially with ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  "."
}], "Notes",
  CellTags->"LanguageFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["AcceptQFA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["ComputationFA",
    ButtonStyle->"AddOnsLink"],
  "  and  ",
  ButtonBox["GrowthSeriesDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"LanguageFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"LanguageFA"],

Cell["\<\
The acceptance language of a DFA. This is an example of exponential \
growth.\
\>", "ExampleText",
  CellTags->"LanguageFA"],

Cell[BoxData[
    \(\(m = 
        DFA[8, 2, {{1, 3, 5, 7, 1, 3, 5, 7}, {2, 4, 6, 8, 2, 4, 6, 8}}, 
          1, {5, 6, 7, 8}];\)\)], "Input",
  CellLabel->"In[42]:=",
  CellTags->"LanguageFA"],

Cell[BoxData[
    \(LanguageFA[m, \(-5\)]\)], "Input",
  CellLabel->"In[43]:=",
  CellTags->"LanguageFA"],

Cell[BoxData[
    \(\(\(LanguageFA[m, \(-10\), \ 
      SizeOnly \[Rule] True\ ]\)\(\ \)\)\)], "Input",
  CellLabel->"In[44]:=",
  CellTags->"LanguageFA"],

Cell["The  same acceptance language, but this time using an  FA. ", \
"ExampleText",
  CellTags->"LanguageFA"],

Cell[BoxData[
    \(\(m\  = \ 
        FA[4, 2, {{1, 1, 1}, {1, 2, 1}, {1, 2, 2}, {2, 1, 3}, {2, 2, 3}, {3, 
              1, 4}, {3, 2, 4}}, {1}, {4}];\)\)], "Input",
  CellLabel->"In[45]:=",
  CellTags->"LanguageFA"],

Cell[BoxData[
    \(\(\(LanguageFA[m, \(-5\)]\n
    LanguageFA[m, \(-10\), \ SizeOnly \[Rule] True\ ]\)\(\ \)\)\)], "Input",
  CellLabel->"In[46]:=",
  CellTags->"LanguageFA"],

Cell["\<\
Every word in the language has exactly one accepting computation.\
\
\>", "ExampleText",
  CellTags->"LanguageFA"],

Cell[BoxData[
    \(LanguageFA[m, 5, Multiplicity \[Rule] True]\)], "Input",
  CellLabel->"In[48]:=",
  CellTags->"LanguageFA"],

Cell["\<\
Here is a semiautomaton with more interesting behavior when it \
comes to multiplicity.\
\>", "ExampleText",
  CellTags->"LanguageFA"],

Cell[BoxData[
    \(\(m\  = \ 
        SA[4, \(-2\), {{1, 1, 1}, {1, 2, 2}, {2, 1, 3}, {2, 2, 4}, {3, 2, 
              1}, {3, 1, 2}, {4, 2, 3}, {4, 1, 4}}];\)\)], "Input",
  CellLabel->"In[49]:=",
  CellTags->"LanguageFA"],

Cell[BoxData[
    \(LanguageFA[m, \(-8\), SizeOnly \[Rule] True]\)], "Input",
  CellLabel->"In[50]:=",
  CellTags->"LanguageFA"],

Cell[BoxData[
    \(LanguageFA[m, 4, Multiplicity \[Rule] True]\)], "Input",
  CellLabel->"In[51]:=",
  CellTags->"LanguageFA"],

Cell["Lastly, a DFA for a finite language.", "ExampleText",
  CellTags->"LanguageFA"],

Cell[BoxData[
    \(m\  = \ BuildDFA[\ Words[3]\ ]\)], "Input",
  CellLabel->"In[52]:=",
  CellTags->"LanguageFA"],

Cell[BoxData[
    \(LanguageFA[m, \(-6\), SizeOnly \[Rule] True]\)], "Input",
  CellLabel->"In[53]:=",
  CellTags->"LanguageFA"],

Cell[BoxData[
    \(LanguageFA[m, Full \[Rule] True]\)], "Input",
  CellLabel->"In[55]:=",
  CellTags->"LanguageFA"],

Cell["LanguageSort", "ObjectName",
  CellTags->"LanguageSort"],

Cell["Function", "Highlight",
  CellTags->"LanguageSort"],

Cell[TextData[{
  StyleBox["LanguageSort[LL]", "MR"],
  "  sorts a list ",
  StyleBox["LL ", "MR"],
  "of finite languages in natural order, also sorts the languages"
}], "Usage",
  CellTags->"LanguageSort"],

Cell[TextData[{
  " Uses ",
  StyleBox["WordSort", "MR"],
  " to sort the languages (length-lex order)."
}], "Notes",
  CellTags->"LanguageSort"],

Cell[TextData[{
  "See ",
  ButtonBox["SyntacticCongruence",
    ButtonData:>"SyntacticCongruence",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"LanguageSort"],

Cell["Further Examples", "ExampleSection",
  CellTags->"LanguageSort"],

Cell["\<\
A counting machine and part of the associated syntactic congruence \
relation. The classes are sorted naturally.\
\>", "ExampleText",
  CellTags->"LanguageSort"],

Cell[BoxData[
    \(\(m = 
        IntersectionFA[CounterDFA[a, 0, {a, b}, Modulus \[Rule] 2], 
          CounterDFA[b, 0, {a, b}, Modulus \[Rule] 2]];\)\)], "Input",
  CellLabel->"In[148]:=",
  AspectRatioFixed->True,
  CellTags->"LanguageSort"],

Cell[BoxData[{
    \(\(cong = LanguageSort[SyntacticCongruence[m, \(-4\)]];\)\), "\n", 
    \(ColumnForm[cong]\)}], "Input",
  CellLabel->"In[149]:=",
  AspectRatioFixed->True,
  CellTags->"LanguageSort"],

Cell["LanguageToSeries", "ObjectName",
  CellTags->"LanguageToSeries"],

Cell["Function", "Highlight",
  CellTags->"LanguageToSeries"],

Cell[TextData[{
  StyleBox["LanguageToSeries[L]", "MR"],
  "  adds multiplicities 1 to a language ",
  StyleBox["L", "MR"],
  ". "
}], "Usage",
  CellTags->"LanguageToSeries"],

Cell[TextData[{
  "Output can be printed as a formal power series using ",
  StyleBox["ToSeries", "MR"],
  ". "
}], "Notes",
  CellTags->"LanguageToSeries"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToSeries",
    ButtonData:>"ToSeries",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["HurwitzProduct",
    ButtonData:>"HurwitzProduct",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"LanguageToSeries"],

Cell["Further Examples", "ExampleSection",
  CellTags->"LanguageToSeries"],

Cell["All words of length 3.", "ExampleText",
  CellTags->"LanguageToSeries"],

Cell["Words[3]", "Input",
  CellLabel->"In[778]:=",
  CellTags->"LanguageToSeries"],

Cell[BoxData[
    \(LanguageToSeries[%]\)], "Input",
  CellLabel->"In[779]:=",
  CellTags->"LanguageToSeries"],

Cell[BoxData[
    \(ToSeries[%]\)], "Input",
  CellLabel->"In[142]:=",
  CellTags->"LanguageToSeries"],

Cell["LeftQuotient", "ObjectName",
  CellTags->"LeftQuotient"],

Cell["Function", "Highlight",
  CellTags->"LeftQuotient"],

Cell[TextData[{
  StyleBox["LeftQuotient[w,L]", "MR"],
  "  computes the left quotient of a finite language ",
  StyleBox["L", "MR"],
  " with respect to word ",
  StyleBox["w", "MR"],
  "."
}], "Usage",
  CellTags->"LeftQuotient"],

Cell[TextData[{
  "Returns ",
  Cell[BoxData[
      \(TraditionalForm\`{\ 
        x\  | \ w\[VeryThinSpace]x\  \[Element] \ L\ }\)]],
  ". "
}], "Notes",
  CellTags->"LeftQuotient"],

Cell[TextData[{
  "See also ",
  ButtonBox["BuildDFA",
    ButtonData:>"BuildDFA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["GenerateQuotients",
    ButtonData:>"GenerateQuotients",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["QuotientDFA",
    ButtonData:>"QuotientDFA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["QuotientMachineFA",
    ButtonData:>"QuotientMachineFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"LeftQuotient"],

Cell["Further Examples", "ExampleSection",
  CellTags->"LeftQuotient"],

Cell["\<\
The language of all words of length 3 other than aaa and some of \
its left quotients.\
\>", "ExampleText",
  CellTags->"LeftQuotient"],

Cell["W = Rest[Words[3]]", "Input",
  CellLabel->"In[155]:=",
  CellTags->"LeftQuotient"],

Cell[BoxData[
    \(LeftQuotient[a, W]\)], "Input",
  CellLabel->"In[167]:=",
  CellTags->"LeftQuotient"],

Cell[BoxData[
    \(LeftQuotient[aa, W]\)], "Input",
  CellLabel->"In[168]:=",
  CellTags->"LeftQuotient"],

Cell[BoxData[
    \(LeftQuotient[aaa, W]\)], "Input",
  CellLabel->"In[170]:=",
  CellTags->"LeftQuotient"],

Cell[BoxData[
    \(LeftQuotient[bb, W]\)], "Input",
  CellLabel->"In[169]:=",
  CellTags->"LeftQuotient"],

Cell["LengthDFA", "ObjectName",
  CellTags->"LengthDFA"],

Cell["Function", "Highlight",
  CellTags->"LengthDFA"],

Cell[TextData[{
  StyleBox["LengthDFA[n,K]", "MR"],
  "  constructs a DFA for the language of all words over alphabet ",
  StyleBox["K", "MR"],
  " of length ",
  StyleBox["n", "MR"],
  ". "
}], "Usage",
  CellTags->"LengthDFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"},
          {"Modulus", "\[Rule]", "0", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"LengthDFA"],

Cell[TextData[{
  "Option ",
  "Modulus",
  StyleBox["->m", "MR"],
  " uses counts modulo ",
  Cell[BoxData[
      \(TraditionalForm\`m\  > \ 0\)]],
  "."
}], "Notes",
  CellTags->"LengthDFA"],

Cell[TextData[{
  "For negative ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  ", all words of length at most ",
  Cell[BoxData[
      \(TraditionalForm\`\(-n\)\)]],
  " are accepted. "
}], "Notes",
  CellTags->"LengthDFA"],

Cell[TextData[{
  "Length parameter ",
  StyleBox["n", "MR"],
  " can also be replaced by a list of non-negative integers."
}], "Notes",
  CellTags->"LengthDFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["CounterDFA",
    ButtonData:>"CounterDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"LengthDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"LengthDFA"],

Cell["A machine for all words of length at most 4.", "ExampleText",
  CellTags->"LengthDFA"],

Cell[BoxData[
    \(\(\(\ \)\(m\  = \ LengthDFA[\ \(-4\), \ 2\ ]\)\)\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->"LengthDFA"],

Cell[BoxData[
    \(LanguageFA[\ m, \ \(-8\), \ SizeOnly \[Rule] True\ ]\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->"LengthDFA"],

Cell["We can explicitely select a number of lengths.", "ExampleText",
  CellTags->"LengthDFA"],

Cell[BoxData[
    \(\(\(\ \)\(\(m\  = \ LengthDFA[\ {2, 4}, \ 2\ ];\)\n
    LanguageFA[\ m, \ \(-8\), \ SizeOnly \[Rule] True\ ]\)\)\)], "Input",
  CellLabel->"In[3]:=",
  CellTags->"LengthDFA"],

Cell["Counting modulo 5.", "ExampleText",
  CellTags->"LengthDFA"],

Cell[BoxData[
    \(\(\(\ \)\(\(m\  = \ LengthDFA[\ \(-3\), \ 2, \ Modulus \[Rule] 5\ ];\)\n
    
    LanguageFA[\ m, \ \(-12\), \ SizeOnly \[Rule] True\ ]\)\)\)], "Input",
  CellLabel->"In[7]:=",
  CellTags->"LengthDFA"],

Cell["LocalQSA", "ObjectName",
  CellTags->"LocalQSA"],

Cell["Function", "Highlight",
  CellTags->"LocalQSA"],

Cell[TextData[{
  StyleBox["LocalQSA[M,opts]", "MR"],
  "  tests whether the semiautomaton  ",
  StyleBox["M", "MR"],
  "  is local."
}], "Usage",
  CellTags->"LocalQSA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"LocalQSA"],

Cell[TextData[{
  "Tests whether semiautomaton  ",
  StyleBox["M", "MR"],
  "  is local.  Assumes that  ",
  StyleBox["M", "MR"],
  "  is transitive or unambiguous and non-transient."
}], "Notes",
  CellTags->"LocalQSA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  "  also returns the diagonal and the strongly connected components of the \
product automaton used in the test."
}], "Notes",
  CellTags->"LocalQSA"],

Cell[TextData[{
  "See also ",
  ButtonBox["AmbiguousQFA",
    ButtonData:>"AmbiguousQFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ClassifyCA",
    ButtonData:>"ClassifyCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"LocalQSA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"LocalQSA"],

Cell["\<\
For the de Bruijn automata obtained from one-dimensional cellular \
automata locality is the same as reversibility. First, a CA with an open but \
not reversible global map.\
\>", "ExampleText",
  CellTags->"LocalQSA"],

Cell[BoxData[
    \(sa\  = CA[90]\  // \ ToSA\)], "Input",
  CellLabel->"In[1717]:=",
  CellTags->"LocalQSA"],

Cell[BoxData[{
    \(DeterministicQFA[sa]\), "\n", 
    \(CoDeterministicQFA[sa]\)}], "Input",
  CellLabel->"In[1718]:=",
  CellTags->"LocalQSA"],

Cell[BoxData[
    \(LocalQSA[sa]\)], "Input",
  CellLabel->"In[1720]:=",
  CellTags->"LocalQSA"],

Cell[BoxData[
    \(LocalQSA[sa, Full \[Rule] True]\)], "Input",
  CellLabel->"In[1721]:=",
  CellTags->"LocalQSA"],

Cell["A reversible CA.", "ExampleText",
  CellTags->"LocalQSA"],

Cell[BoxData[
    \(\(CA[51]\  // \ ToSA\)\  // \ LocalQSA\)], "Input",
  CellLabel->"In[1722]:=",
  CellTags->"LocalQSA"],

Cell["LocalUniversalFA", "ObjectName",
  CellTags->"LocalUniversalFA"],

Cell["Function", "Highlight",
  CellTags->"LocalUniversalFA"],

Cell[TextData[{
  StyleBox["LocalUniversalFA[n,k,opts]", "MR"],
  "  generates the ",
  StyleBox["n", "MR"],
  "-local-universal FA over a ",
  StyleBox["k", "MR"],
  " letter alphabet."
}], "Usage",
  CellTags->"LocalUniversalFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"},
          {"EdgeLU", "\[Rule]", \(Function[x, True]\), "-"},
          {"IniLU", "\[Rule]", \(Function[x, True]\), "-"},
          {"FinLU", "\[Rule]", \(Function[x, True]\), "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"LocalUniversalFA"],

Cell[TextData[{
  "Transitions, initial and final states are determined via options ",
  StyleBox["EdgeLU", "MR"],
  ", ",
  StyleBox["IniLU", "MR"],
  " and ",
  StyleBox["FinLU", "MR"],
  "; the defaults are: ",
  StyleBox["True&", "MR"],
  ". "
}], "Notes",
  CellTags->"LocalUniversalFA"],

Cell["Is based on a de Bruijn automaton.", "Notes",
  CellTags->"LocalUniversalFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["DeBruijnGraph",
    ButtonData:>"DeBruijnGraph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"LocalUniversalFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"LocalUniversalFA"],

Cell[TextData[{
  "The underlying state set here is ",
  Cell[BoxData[
      \(TraditionalForm\`\((a + b)\)\^\(\(\ \)\(3\)\)\)]],
  ". We select the initial state to be aaa, the final states to be ",
  Cell[BoxData[
      \(TraditionalForm\`b\ \((a + b)\)\^\(\(\ \)\(2\)\)\)]],
  ". "
}], "ExampleText",
  CellTags->"LocalUniversalFA"],

Cell[BoxData[{
    \(\(ini = #1 === "\<aaa\>" &;\)\), "\n", 
    \(\(fin = StringMatchQ[#1, "\<b*\>"] &;\)\), "\n", 
    \(m = 
      LocalUniversalFA[3, 2, IniLU \[Rule] ini, FinLU \[Rule] fin, 
        Normalize \[Rule] 1]\)}], "Input",
  CellLabel->"In[791]:=",
  CellTags->"LocalUniversalFA"],

Cell[BoxData[
    \(States[m]\)], "Input",
  CellLabel->"In[299]:=",
  CellTags->"LocalUniversalFA"],

Cell[TextData[{
  "The language is ",
  Cell[BoxData[
      \(TraditionalForm\`b\ \((a + b)\)\^\(\(\ \)\(2\)\)\)]],
  ". "
}], "ExampleText",
  CellTags->"LocalUniversalFA"],

Cell[BoxData[
    \(LanguageFA[\ m, \ \(-5\)\ ]\)], "Input",
  CellLabel->"In[300]:=",
  CellTags->"LocalUniversalFA"],

Cell["LoopAutomatonFA", "ObjectName",
  CellTags->"LoopAutomatonFA"],

Cell["Function", "Highlight",
  CellTags->"LoopAutomatonFA"],

Cell[TextData[{
  StyleBox["LoopAutomatonFA[M]", "MR"],
  "  converts DFA ",
  StyleBox["M", "MR"],
  " into a loop automaton. "
}], "Usage",
  CellTags->"LoopAutomatonFA"],

Cell["Only useful in connecton with the finite power property.", "Notes",
  CellTags->"LoopAutomatonFA"],

Cell[TextData[{
  "See ",
  ButtonBox["TransitionMatrixFA",
    ButtonData:>"TransitionMatrixFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Tropical",
    ButtonData:>"Tropical",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"LoopAutomatonFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"LoopAutomatonFA"],

Cell["\<\
A small DFA and its corresponding loop-automaton. The latter \
accepts the Kleene star of the language of the DFA.\
\>", "ExampleText",
  CellTags->"LoopAutomatonFA"],

Cell[BoxData[{
    \(\(m\  = \ DFA[3, 2, {{1, 3, 3}, {2, 2, 2}}, 1, {1, 2}];\)\), "\n", 
    \(mloop = LoopAutomatonFA[ToFA[m]]\)}], "Input",
  CellLabel->"In[80]:=",
  CellTags->"LoopAutomatonFA"],

Cell[BoxData[
    \(EquivalentQFA[StarFA[m], mloop]\)], "Input",
  AspectRatioFixed->True,
  CellTags->"LoopAutomatonFA"],

Cell[TextData[{
  "We assign the homomorphism from words to matrices over the tropical \
semiring induced by the loop-automaton to symbol ",
  StyleBox["f", "MR"],
  "."
}], "ExampleText",
  CellTags->"LoopAutomatonFA"],

Cell[BoxData[{
    \(\(TransitionMatrixFA[mloop, f, Type \[Rule] Tropical];\)\), "\n", 
    \(MatrixForm /@ {f[a], f[b]}\)}], "Input",
  CellLabel->"In[99]:=",
  CellTags->"LoopAutomatonFA"],

Cell["\<\
The values of the homomorphism on all words of length at most \
3.\
\>", "ExampleText",
  CellTags->"LoopAutomatonFA"],

Cell[BoxData[
    \(MatrixForm\  /@ \ Union[f /@ Words[\(-3\), {a, b}]]\)], "Input",
  CellLabel->"In[101]:=",
  AspectRatioFixed->True,
  CellTags->"LoopAutomatonFA"],

Cell["MakeHomomorphismDFA", "ObjectName",
  CellTags->"MakeHomomorphismDFA"],

Cell["Function", "Highlight",
  CellTags->"MakeHomomorphismDFA"],

Cell[TextData[{
  StyleBox["MakeHomomorphismDFA[h,M1,M2]", "MR"],
  "  attempts to construct a homomorphism from ",
  StyleBox["M1", "MR"],
  " to ",
  StyleBox["M2", "MR"],
  ", and assigns it to symbol ",
  StyleBox["h", "MR"],
  ". "
}], "Usage",
  CellTags->"MakeHomomorphismDFA"],

Cell[TextData[{
  "Symbol ",
  StyleBox["h", "MR"],
  " represents the homomorphism; has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"MakeHomomorphismDFA"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["HoldFirst", "MR"],
  "."
}], "Notes",
  CellTags->"MakeHomomorphismDFA"],

Cell[TextData[{
  "Returns ",
  StyleBox["True", "MR"],
  "/",
  StyleBox["False", "MR"],
  " indicating success or failure."
}], "Notes",
  CellTags->"MakeHomomorphismDFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["MakeHomomorphism",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["AssignFunction",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"MakeHomomorphismDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"MakeHomomorphismDFA"],

Cell["A DFA and the corresponding minimal DFA.", "ExampleText",
  CellTags->"MakeHomomorphismDFA"],

Cell[BoxData[{
    \(m = IthSymbolFA[a, \(-3\), Full \[Rule] True]\), "\n", 
    \(mm = MinimizeFA[m]\)}], "Input",
  CellLabel->"In[14]:=",
  CellTags->"MakeHomomorphismDFA"],

Cell["\<\
We construct the natural homomorphism, and test some of its \
properties. \
\>", "ExampleText",
  CellTags->"MakeHomomorphismDFA"],

Cell[BoxData[
    \(MakeHomomorphismDFA[\ h, m, \ mm\ \ \ ]\)], "Input",
  CellLabel->"In[16]:=",
  CellTags->"MakeHomomorphismDFA"],

Cell[BoxData[{
    \(TransitionFunctionFA[m, dm]\), "\n", 
    \(TransitionFunctionFA[mm, dmm]\), "\n", 
    \(\(h[dm[#, a]] == dmm[h[#], a] &\)\ \  /@ \ Range[15]\)}], "Input",
  CellLabel->"In[17]:=",
  CellTags->"MakeHomomorphismDFA"],

Cell["MakeHomomorphism", "ObjectName",
  CellTags->"MakeHomomorphism"],

Cell["Function", "Highlight",
  CellTags->"MakeHomomorphism"],

Cell[TextData[{
  StyleBox["MakeHomomorphism[f,A,B]", "MR"],
  "  assigns the monoid homomorphism on words given by ",
  StyleBox["A", "MR"],
  " and ",
  StyleBox["B", "MR"],
  " to symbol ",
  StyleBox["f", "MR"],
  ". "
}], "Usage",
  CellTags->"MakeHomomorphism"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Opts", "\[Rule]", \({}\), "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"MakeHomomorphism"],

Cell[TextData[{
  "Option  ",
  StyleBox["Opts->Listable", "MR"],
  "  attaches the attribute ",
  StyleBox["Listable", "MR"],
  " to the function."
}], "Notes",
  CellTags->"MakeHomomorphism"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["HoldFirst", "MR"],
  "."
}], "Notes",
  CellTags->"MakeHomomorphism"],

Cell[TextData[{
  StyleBox["A", "MR"],
  " must be an alphabet, ",
  StyleBox["B", "MR"],
  " a list of words of the same length as ",
  StyleBox["A", "MR"],
  "."
}], "Notes",
  CellTags->"MakeHomomorphism"],

Cell[TextData[{
  "See also ",
  ButtonBox["HomomorphismFA",
    ButtonData:>"HomomorphismFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"MakeHomomorphism"],

Cell["Further Examples", "ExampleSection",
  CellTags->"MakeHomomorphism"],

Cell["\<\
Iterating a homomorphism on a binary alphabet.  This is the \
Fibonacci homomorphism.\
\>", "ExampleText",
  CellTags->"MakeHomomorphism"],

Cell[" MakeHomomorphism[f,{0,1},{1,10}]", "Input",
  CellLabel->"In[69]:=",
  CellTags->"MakeHomomorphism"],

Cell[BoxData[{
    \(\(LS = NestList[\ f, "\<0\>", 9];\)\), "\n", 
    \(ColumnForm[LS]\)}], "Input",
  CellLabel->"In[70]:=",
  AspectRatioFixed->True,
  CellTags->"MakeHomomorphism"],

Cell["The lengths of these strings are the Fibonacci numbers:", "ExampleText",
  Evaluatable->False,
  CellTags->"MakeHomomorphism"],

Cell[BoxData[
    \(WordLength[LS]\)], "Input",
  CellLabel->"In[72]:=",
  CellTags->"MakeHomomorphism"],

Cell["MakeTrim", "ObjectName",
  CellTags->"MakeTrim"],

Cell["Option", "Highlight",
  CellTags->"MakeTrim"],

Cell[TextData[{
  StyleBox["MakeTrim", "MR"],
  " MakeTrim is an option for ",
  StyleBox["LanguageFA", "MR"],
  ". "
}], "Usage",
  CellTags->"MakeTrim"],

Cell["\<\
Causes the input automaton to be replaced with its trim version \
before the language is computed.\
\>", "Notes",
  CellTags->"MakeTrim"],

Cell[TextData[{
  "See also ",
  ButtonBox["LanguageFA",
    ButtonData:>"LanguageFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"MakeTrim"],

Cell["MatrixToTransitions", "ObjectName",
  CellTags->"MatrixToTransitions"],

Cell["Function", "Highlight",
  CellTags->"MatrixToTransitions"],

Cell[TextData[{
  StyleBox["MatrixToTransitions[M]", "MR"],
  "  converts a transition matrix of DFA or NFA  ",
  StyleBox["M", "MR"],
  "  into the corresponding list of transitions."
}], "Usage",
  CellTags->"MatrixToTransitions"],

Cell["Also works directly on the transition matrix.", "Notes",
  CellTags->"MatrixToTransitions"],

Cell[TextData[{
  "See also ",
  ButtonBox["TransitionMatrixFA",
    ButtonData:>"TransitionMatrixFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToMatrix",
    ButtonData:>"ToMatrix",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"MatrixToTransitions"],

Cell["Further Examples", "ExampleSection",
  CellTags->"MatrixToTransitions"],

Cell["\<\
Turning the transition matrix of a DFA into a transition \
list.\
\>", "ExampleText",
  CellTags->"MatrixToTransitions"],

Cell[BoxData[
    \(A = Transitions[CounterDFA[b, 0, 2, Modulus \[Rule] 3]]\)], "Input",
  CellLabel->"In[155]:=",
  CellTags->"MatrixToTransitions"],

Cell["MatrixToTransitions[A]", "Input",
  CellLabel->"In[156]:=",
  CellTags->"MatrixToTransitions"],

Cell["MinimalFischerFA", "ObjectName",
  CellTags->"MinimalFischerFA"],

Cell["Function", "Highlight",
  CellTags->"MinimalFischerFA"],

Cell[TextData[{
  StyleBox["MinimalFischerFA[M]", "MR"],
  "  returns the minimal Fischer automaton associated with semiautomaton ",
  StyleBox["M", "MR"],
  ". "
}], "Usage",
  CellTags->"MinimalFischerFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"},
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"},
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"MinimalFischerFA"],

Cell[TextData[{
  "Machine ",
  StyleBox["M", "MR"],
  " must accept a factorial, extensible and transitive (FET) language for \
this to work, ",
  StyleBox["MinimalFischerFA", "MR"],
  " does not check. "
}], "Notes",
  CellTags->"MinimalFischerFA"],

Cell[TextData[{
  "Options are ",
  StyleBox["Full->True", "MR"],
  " causes the minimal DFA of the language to be returned, in addition to the \
minimal Fischer automaton."
}], "Notes",
  CellTags->"MinimalFischerFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"MinimalFischerFA"],

Cell["Cellular automata are good source of FET languages.", "ExampleText",
  CellTags->"MinimalFischerFA"],

Cell["m = ToSA[ CA[91] ]", "Input",
  CellLabel->"In[27]:=",
  CellTags->"MinimalFischerFA"],

Cell[BoxData[
    \(mm\  = \ MinimalFischerFA[m]\)], "Input",
  CellLabel->"In[28]:=",
  CellTags->"MinimalFischerFA"],

Cell[BoxData[
    \(DeterministicQFA[mm]\)], "Input",
  CellLabel->"In[269]:=",
  CellTags->"MinimalFischerFA"],

Cell[BoxData[
    \(EquivalentQFA[\ m, mm\ ]\)], "Input",
  CellLabel->"In[271]:=",
  CellTags->"MinimalFischerFA"],

Cell["MinimizeDFA", "ObjectName",
  CellTags->"MinimizeDFA"],

Cell["Function", "Highlight",
  CellTags->"MinimizeDFA"],

Cell[TextData[{
  StyleBox["MinimizeDFA[M]", "MR"],
  "  computes the minimal automaton of a DFA ",
  StyleBox["M", "MR"],
  ". "
}], "Usage",
  CellTags->"MinimizeDFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"},
          {"DoTrace", "\[Rule]", "False", "-"},
          {"Hopcroft", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"MinimizeDFA"],

Cell[TextData[{
  "Unlike ",
  StyleBox["MinimizeFA", "MR"],
  ", this function does not handle nondeterministic machines and is useful \
mostly to produce traces of the minimization algorithm in conjunction with \
option ",
  StyleBox["DoTrace->True", "MR"],
  "."
}], "Notes",
  CellTags->"MinimizeDFA"],

Cell[TextData[{
  "Unlike ",
  StyleBox["MinimizeFA", "MR"],
  ", this function cannot use external code."
}], "Notes",
  CellTags->"MinimizeDFA"],

Cell[TextData[{
  "The standard algorithm has quadratic time complexity, and option ",
  StyleBox["Hopcroft->True", "MR"],
  " uses a ",
  Cell[BoxData[
      \(TraditionalForm\`O(n\ log\ n)\)]],
  " algorithm, at least in theory. "
}], "Notes",
  CellTags->"MinimizeDFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["MinimizeFA",
    ButtonData:>"MinimizeFA",
    ButtonStyle->"AddOnsLink"],
  "  and ",
  ButtonBox["StateMergeDFA",
    ButtonData:>"StateMergeDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"MinimizeDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"MinimizeDFA"],

Cell["Round up the usual suspects.", "ExampleText",
  CellTags->"MinimizeDFA"],

Cell[BoxData[
    \(m\  = \ IthSymbolFA[b, \(-3\), 2, Full \[Rule] True]\)], "Input",
  CellLabel->"In[198]:=",
  CellTags->"MinimizeDFA"],

Cell[BoxData[{
    \(\({mm, tr} = 
        MinimizeDFA[m, DoTrace \[Rule] True, 
          Normalize \[Rule] 1];\)\), "\n", 
    \(mm\), "\n", 
    \(States[mm]\), "\n", 
    \(TableForm[tr, TableSpacing \[Rule] {1, 1}]\)}], "Input",
  CellLabel->"In[213]:=",
  CellTags->"MinimizeDFA"],

Cell["MinimizeFA", "ObjectName",
  CellTags->"MinimizeFA"],

Cell["Function", "Highlight",
  CellTags->"MinimizeFA"],

Cell[TextData[{
  StyleBox["MinimizeFA[M,opts]", "MR"],
  "  computes the minimal automaton of a finite state machine ",
  StyleBox["M", "MR"],
  ", using the external code whenever possible. "
}], "Usage",
  CellTags->"MinimizeFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"DoTrace", "\[Rule]", "False", "-"},
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"},
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"MinimizeFA"],

Cell[TextData[{
  "If ",
  StyleBox["M", "MR"],
  " is nondeterministic, the power automaton is computed first, therefore the \
time and space complexity is potentially exponential. "
}], "Notes",
  CellTags->"MinimizeFA"],

Cell[TextData[{
  "Option ",
  StyleBox["DoTrace->True", "MR"],
  " generates a trace to the minimization process, but only for the internal \
version of the algorithms."
}], "Notes",
  CellTags->"MinimizeFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["MinimizeDFA,",
    ButtonData:>"MinimizeDFA,",
    ButtonStyle->"AddOnsLink"],
  " ",
  ButtonBox["CoverQDFA",
    ButtonData:>"CoverQDFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["EquivalentQFA",
    ButtonData:>"EquivalentQFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"MinimizeFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"MinimizeFA"],

Cell["\<\
Minimization can shrink DFAs, but for nondeterministic machines it \
may involve exponential blow-up.\
\>", "ExampleText",
  CellTags->"MinimizeFA"],

Cell["\<\
m = IthSymbolFA[b,-3,2,Full->True]
MinimizeFA[m]\
\>", "Input",
  CellLabel->"In[262]:=",
  CellTags->"MinimizeFA"],

Cell[BoxData[
    \(Table[\ Size[MinimizeFA[IthSymbolFA[b, \(-k\), 2]]], {k, 8}]\)], "Input",\

  CellLabel->"In[265]:=",
  CellTags->"MinimizeFA"],

Cell["NFA", "ObjectName",
  CellTags->"NFA"],

Cell["Constructor", "Highlight",
  CellTags->"NFA"],

Cell[TextData[{
  StyleBox["NFA[n,k,A,I,F]", "MR"],
  "  denotes a non-deterministic finite automaton. "
}], "Usage",
  CellTags->"NFA"],

Cell[TextData[{
  StyleBox["n", "MR"],
  " is the number of states, ",
  StyleBox["k", "MR"],
  " the alphabet size, ",
  StyleBox["A", "MR"],
  " a  ",
  Cell[BoxData[
      \(TraditionalForm\`k\ \[Cross]\ n\)]],
  " matrix of sets of states."
}], "Notes",
  CellTags->"NFA"],

Cell[TextData[{
  "The  initial states are given by ",
  StyleBox["I", "MR"],
  " and the final states by ",
  StyleBox["F", "MR"],
  "."
}], "Notes",
  CellTags->"NFA"],

Cell[TextData[{
  StyleBox["The entries in ", "Text"],
  StyleBox["A", "MR"],
  StyleBox[", ", "Text"],
  StyleBox["I", "MR"],
  StyleBox[" and ", "Text"],
  StyleBox["F", "MR"],
  StyleBox["  are in positional notation.", "Text"]
}], "Notes",
  CellTags->"NFA"],

Cell[TextData[{
  "There are hidden fields ",
  StyleBox["K", "MR"],
  " for the alphabet and ",
  StyleBox["Q", "MR"],
  " for a structured state set."
}], "Notes",
  CellTags->"NFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["FA",
    ButtonData:>"FA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SA",
    ButtonData:>"SA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DFA",
    ButtonData:>"DFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"NFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"NFA"],

Cell["A small NFA.", "ExampleText",
  CellTags->"NFA"],

Cell[BoxData[
    \(m\  = \ 
      NFA[4, 2, {{{1}, {3}, {4}, {}}, {{1, 
              2}, {3}, {4}, {}}}, {1}, {4}]\)], "Input",
  CellLabel->"In[89]:=",
  CellTags->"NFA"],

Cell[BoxData[
    \(Transitions[m]\  // \ MatrixForm\)], "Input",
  CellLabel->"In[91]:=",
  CellTags->"NFA"],

Cell["NNFAToFA", "ObjectName",
  CellTags->"NNFAToFA"],

Cell["Function", "Highlight",
  CellTags->"NNFAToFA"],

Cell[TextData[{
  StyleBox["NNFAToFA[M]", "MR"],
  "  converts the tail ",
  StyleBox["M", "MR"],
  " of an NNFA into the corresponding FA."
}], "Usage",
  CellTags->"NNFAToFA"],

Cell["\<\
Only used in connection with McNaughton-Yamada style conversion \
from regular expressions.\
\>", "Notes",
  CellTags->"NNFAToFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["RegularExpressionToNNFA",
    ButtonData:>"RegularExpressionToNNFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"NNFAToFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"NNFAToFA"],

Cell[TextData[{
  "A regular expression for ",
  Cell[BoxData[
      \(TraditionalForm\`\(\((a + b)\)\^\(\(\ \)\(*\)\)\) 
        b\ \((a + b)\)\^\(\(\ \)\(2\)\)\)]],
  " and the corresponding NNFA."
}], "ExampleText",
  CellTags->"NNFAToFA"],

Cell["re = RET[ RES[REP[a,b]],b,REP[a,b],REP[a,b]]", "Input",
  CellLabel->"In[41]:=",
  CellTags->"NNFAToFA"],

Cell[BoxData[
    \(RegularExpressionToNNFA[re]\)], "Input",
  CellLabel->"In[42]:=",
  CellTags->"NNFAToFA"],

Cell["Conversion to an FA.", "ExampleText",
  CellTags->"NNFAToFA"],

Cell["NNFAToFA[%]", "Input",
  CellLabel->"In[43]:=",
  CellTags->"NNFAToFA"],

Cell[BoxData[
    \(LanguageFA[%, \(-5\)]\)], "Input",
  CellLabel->"In[78]:=",
  CellTags->"NNFAToFA"],

Cell["NNFA", "ObjectName",
  CellTags->"NNFA"],

Cell["Constructor", "Highlight",
  CellTags->"NNFA"],

Cell[TextData[{
  StyleBox["NNFA[n,E,L,I,F,e]", "MR"],
  "  denotes the tail of a McNaughton-Yamada type finite automaton. "
}], "Usage",
  CellTags->"NNFA"],

Cell["\<\
Meaning of the fields: n: number of states (other than the virtual \
initial states), E: edges in the diagram, L: labels of the nodes, I: states \
at distance 1 from the initial state, F: final states, e: bit indicating \
whether Eps is accepted.\
\>", "Notes",
  CellTags->"NNFA"],

Cell[TextData[{
  "See ",
  ButtonBox["RegularExpressionToNNFA",
    ButtonData:>"RegularExpressionToNNFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["NNFAToFA",
    ButtonData:>"NNFAToFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"NNFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"NNFA"],

Cell["A small NNFA.", "ExampleText",
  CellTags->"NNFA"],

Cell["RegularExpressionToNNFA[ RES[REP[aa,bb]] ]", "Input",
  CellLabel->"In[84]:=",
  CellTags->"NNFA"],

Cell["Normalize", "ObjectName",
  CellTags->"Normalize"],

Cell["Option", "Highlight",
  CellTags->"Normalize"],

Cell[TextData[{
  StyleBox["Normalize", "MR"],
  " is an option for numerous commands in Automata."
}], "Usage",
  CellTags->"Normalize"],

Cell[TextData[{
  "Default value of ",
  StyleBox["Normalize", "MR"],
  " is ",
  StyleBox["$defaultnormalize", "MR"],
  StyleBox[", can be set globally with ", "Text"],
  StyleBox["NormalizeControl", "MR"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"Normalize"],

Cell[TextData[{
  " Normalize controls the structure of the state set of a machine or graph. \
For example, for two DFAs ",
  StyleBox["M1", "MR"],
  " and ",
  StyleBox["M2", "MR"],
  ", ",
  StyleBox["UnionFA[M1,M2,Normalize->0]", "MR"],
  " produces a product machine whose states are integers.  But  ",
  StyleBox["UnionFA[M1,M2,Normalize->1]", "MR"],
  " produces a product machine with states of the form ",
  StyleBox["{p,q}", "MR"],
  " where ",
  StyleBox["p", "MR"],
  " and ",
  StyleBox["q", "MR"],
  " are integers.  In ",
  StyleBox["UnionFA[M1,M2,Normalize->2]", "MR"],
  " the states are of the form ",
  StyleBox["{p,q}", "MR"],
  " where ",
  StyleBox["p", "MR"],
  " and ",
  StyleBox["q", "MR"],
  " are elements of the state sets of the input machines. "
}], "Notes",
  CellTags->"Normalize"],

Cell[TextData[{
  " Note that  ",
  StyleBox["Normalize->2", "MR"],
  "  can create rather large state sets."
}], "Notes",
  CellTags->"Normalize"],

Cell[TextData[{
  " See ",
  ButtonBox["NormalizeFA",
    ButtonData:>"NormalizeFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["NormalizeControl",
    ButtonData:>"NormalizeControl",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Normalize"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Normalize"],

Cell["Two small DFAs and their union.", "ExampleText",
  CellTags->"Normalize"],

Cell[BoxData[{
    \(\(m1\  = \ BuildDFA[aaa, ab, 2];\)\), "\n", 
    \(\(m2\  = \ BuildDFA[bbb, 2];\)\), "\n", 
    \(mm\  = \ UnionFA[m1, m2, Normalize \[Rule] 0]\), "\n", 
    \(LanguageFA[mm, \(-5\)]\)}], "Input",
  CellLabel->"In[137]:=",
  CellTags->"Normalize"],

Cell[BoxData[
    \(mm\  // \ States\)], "Input",
  CellLabel->"In[141]:=",
  CellTags->"Normalize"],

Cell["\<\
We can preserve the pair structure of the states in the new \
machine.\
\>", "ExampleText",
  CellTags->"Normalize"],

Cell[BoxData[
    \(UnionFA[m1, m2, Normalize \[Rule] 1]\  // \ States\)], "Input",
  CellLabel->"In[142]:=",
  CellTags->"Normalize"],

Cell["\<\
In this case, since the states of the input machines are \
unstructured, setting normalization to level 2 will produce the same \
output.\
\>", "ExampleText",
  CellTags->"Normalize"],

Cell[BoxData[
    \(UnionFA[m1, m2, Normalize \[Rule] 2]\  // \ States\)], "Input",
  CellLabel->"In[143]:=",
  CellTags->"Normalize"],

Cell["\<\
But if the input machines have structured states sets, we get more \
complicated output (pairs of finite sets of words in this case).\
\>", \
"ExampleText",
  CellTags->"Normalize"],

Cell[BoxData[{
    \(\(m1\  = \ BuildDFA[aaa, ab, 2, Normalize \[Rule] 1];\)\), "\n", 
    \(\(m2\  = \ BuildDFA[bbb, 2, Normalize \[Rule] 1];\)\), "\n", 
    \(\(UnionFA[m1, m2, Normalize \[Rule] 2]\  // \ States\)\  // \ 
      TableForm[#, TableDepth \[Rule] 1] &\)}], "Input",
  CellLabel->"In[144]:=",
  CellTags->"Normalize"],

Cell["NormalizeControl", "ObjectName",
  CellTags->"NormalizeControl"],

Cell["Function", "Highlight",
  CellTags->"NormalizeControl"],

Cell[TextData[{
  StyleBox["NormalizeControl[]", "MR"],
  "  turns off normalization for finite state machines. "
}], "Usage",
  CellTags->"NormalizeControl"],

Cell[TextData[{
  "Equivalent to setting  ",
  StyleBox["Normalize\[Rule]2 ", "MR"],
  "in all functions supporting this option.  "
}], "Notes",
  CellTags->"NormalizeControl"],

Cell[TextData[{
  "Normalization can be turned back on with ",
  StyleBox["NormalizeControl[0]", "MR"],
  "."
}], "Notes",
  CellTags->"NormalizeControl"],

Cell["\<\
Note that machines can become significantly larger as data \
structures when normalization is turned off. \
\>", "Notes",
  CellTags->"NormalizeControl"],

Cell["Further Examples", "ExampleSection",
  CellTags->"NormalizeControl"],

Cell["\<\
By default, normalization is turned on, but can be controlled in \
each single function call. \
\>", "ExampleText",
  CellTags->"NormalizeControl"],

Cell[BoxData[{
    \(\(ToDFA[IthSymbolFA[a, \(-3\)]];\)\), "\n", 
    \(States[%]\)}], "Input",
  CellLabel->"In[25]:=",
  CellTags->"NormalizeControl"],

Cell[BoxData[{
    \(\(ToDFA[IthSymbolFA[a, \(-3\)], Normalize \[Rule] 1];\)\), "\n", 
    \(States[%]\)}], "Input",
  CellLabel->"In[27]:=",
  CellTags->"NormalizeControl"],

Cell["\<\
Turning normalization permanently off is sometimes preferable. \
\
\>", "ExampleText",
  CellTags->"NormalizeControl"],

Cell[BoxData[{
    \(NormalizeControl[]\), "\n", 
    \(\(ToDFA[IthSymbolFA[a, \(-3\)]];\)\), "\n", 
    \(States[%]\)}], "Input",
  CellLabel->"In[29]:=",
  CellTags->"NormalizeControl"],

Cell["NormalizeFA", "ObjectName",
  CellTags->"NormalizeFA"],

Cell["Function", "Highlight",
  CellTags->"NormalizeFA"],

Cell[TextData[{
  StyleBox["NormalizeFA[M]", "MR"],
  "  normalizes finite state machine ",
  StyleBox["M", "MR"],
  "."
}], "Usage",
  CellTags->"NormalizeFA"],

Cell[TextData[{
  "The effect of normalization is that the state set is forced to be ",
  Cell[BoxData[
      \(TraditionalForm\`{1, 2, \[Ellipsis], n}\)]],
  " where ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " is the size of the machine. "
}], "Notes",
  CellTags->"NormalizeFA"],

Cell["\<\
This may significantly reduce the size of the expression \
representing the automaton. \
\>", "Notes",
  CellTags->"NormalizeFA"],

Cell[TextData[{
  "See ",
  ButtonBox["FormatAutomata",
    ButtonData:>"FormatAutomata",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"NormalizeFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"NormalizeFA"],

Cell["\<\
We build a machine with structured state set by deterministic \
simulation.\
\>", "ExampleText",
  CellTags->"NormalizeFA"],

Cell[BoxData[{
    \(m = ToDFA[IthSymbolFA[b, \(-3\), 2], Normalize \[Rule] 1]\), "\n", 
    \(States[m]\)}], "Input",
  CellLabel->"In[229]:=",
  CellTags->"NormalizeFA"],

Cell[BoxData[{
    \(mm\  = \ NormalizeFA[m]\), "\n", 
    \(States[mm]\)}], "Input",
  CellLabel->"In[231]:=",
  CellTags->"NormalizeFA"],

Cell["NumberOfDFA", "ObjectName",
  CellTags->"NumberOfDFA"],

Cell["Function", "Highlight",
  CellTags->"NumberOfDFA"],

Cell[TextData[{
  StyleBox["NumberOfDFA[n,k]", "MR"],
  "  computes the number of DFAs on ",
  StyleBox["n", "MR"],
  " states and ",
  StyleBox["k", "MR"],
  " symbols."
}], "Usage",
  CellTags->"NumberOfDFA"],

Cell[TextData[{
  StyleBox["NumberOfDFA", "MR"],
  " does not account for isomorphic machines (and is therefore not too \
useful)."
}], "Notes",
  CellTags->"NumberOfDFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["RandomDFA",
    ButtonData:>"RandomDFA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["RankToDFA",
    ButtonData:>"RankToDFA",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["RankOfDFA",
    ButtonData:>"RankOfDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"NumberOfDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"NumberOfDFA"],

Cell["The number of binary DFAs up to 8 states.", "ExampleText",
  CellTags->"NumberOfDFA"],

Cell["Table[ NumberOfDFA[n,2], {n,8} ] // TableForm", "Input",
  CellLabel->"In[151]:=",
  CellTags->"NumberOfDFA"],

Cell["NumTransitions", "ObjectName",
  CellTags->"NumTransitions"],

Cell["Function", "Highlight",
  CellTags->"NumTransitions"],

Cell[TextData[{
  StyleBox["NumTransitions[M]", "MR"],
  "  returns the number of transitions in machine ",
  StyleBox["M", "MR"],
  "."
}], "Usage",
  CellTags->"NumTransitions"],

Cell[TextData[{
  "See also ",
  ButtonBox["Size",
    ButtonData:>"Size",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"NumTransitions"],

Cell["Further Examples", "ExampleSection",
  CellTags->"NumTransitions"],

Cell["Extracting the number of transitions of a semiautomaton.", "ExampleText",
  CellTags->"NumTransitions"],

Cell[BoxData[
    \(\(m = 
        SA[4, 2, {{1, 1, 1}, {1, 2, 2}, {2, 2, 3}, {2, 2, 4}, {3, 1, 1}, {3, 
              2, 2}, {4, 2, 3}}];\)\)], "Input",
  CellLabel->"In[134]:=",
  CellTags->"NumTransitions"],

Cell[BoxData[
    \(NumTransitions[m]\)], "Input",
  CellLabel->"In[135]:=",
  CellTags->"NumTransitions"],

Cell["PermutationSA", "ObjectName",
  CellTags->"PermutationSA"],

Cell["Function", "Highlight",
  CellTags->"PermutationSA"],

Cell[TextData[{
  StyleBox["PermutationSA[perm,K]", "MR"],
  "  constructs a semiautomaton from the list of permutations ",
  StyleBox["perm", "MR"],
  " over alphabet K."
}], "Usage",
  CellTags->"PermutationSA"],

Cell[TextData[{
  "List ",
  StyleBox["perm", "MR"],
  " must have length the size of ",
  Cell[BoxData[
      \(TraditionalForm\`K\)]],
  ", and contain integer lists representing permutations, the command does \
not check."
}], "Notes",
  CellTags->"PermutationSA"],

Cell[TextData[{
  "See also  ",
  ButtonBox["FunctionToSA",
    ButtonStyle->"AddOnsLink"],
  "   and  ",
  ButtonBox["CirculantSA",
    ButtonStyle->"AddOnsLink"],
  ". "
}], "Notes",
  CellTags->"PermutationSA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"PermutationSA"],

Cell["Turning permutations into automata.", "ExampleText",
  CellTags->"PermutationSA"],

Cell["s = PermutationSA[{{2,3,4,1},{2,1,3,4}},2]", "Input",
  CellLabel->"In[88]:=",
  CellTags->"PermutationSA"],

Cell["\<\
The transformation semigroup here is the symmetric group on 4 \
points.\
\>", "ExampleText",
  CellTags->"PermutationSA"],

Cell[BoxData[{
    \(S\  = \ GenerateSG[ToGeneratorsFA[s]]\), "\n", 
    \(S // \ Length\)}], "Input",
  CellLabel->"In[93]:=",
  CellTags->"PermutationSA"],

Cell[BoxData[
    \(\(S\  // \ DClassDecomposition\)\  // \ \(Curry[DClassSize]\)[
        Full \[Rule] True]\)], "Input",
  CellLabel->"In[96]:=",
  CellTags->"PermutationSA"],

Cell[BoxData[""], "Input",
  CellTags->"PermutationSA"],

Cell["PermuteFA", "ObjectName",
  CellTags->"PermuteFA"],

Cell["Function", "Highlight",
  CellTags->"PermuteFA"],

Cell[TextData[{
  StyleBox["PermuteFA[M,perm]", "MR"],
  "  permutes the states of FSM ",
  StyleBox["M", "MR"],
  " according to permutation ",
  StyleBox["perm", "MR"],
  ". "
}], "Usage",
  CellTags->"PermuteFA"],

Cell[TextData[{
  "List ",
  StyleBox["perm", "MR"],
  " is required to be a permutation of length ",
  StyleBox["Size[M]", "MR"],
  StyleBox[", there is no check.", "Text"]
}], "Notes",
  CellTags->"PermuteFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"PermuteFA"],

Cell["Permuting a DFA will not change the acceptance language.", "ExampleText",
  CellTags->"PermuteFA"],

Cell[" m = CounterDFA[ b, 1, 2, Modulus->3 ]", "Input",
  CellLabel->"In[159]:=",
  CellTags->"PermuteFA"],

Cell[BoxData[
    \(mm\  = \ PermuteFA[\ m, \ {3, 2, 1}\ ]\)], "Input",
  CellLabel->"In[160]:=",
  CellTags->"PermuteFA"],

Cell[BoxData[
    \(IsomorphicQDFA[m, mm]\)], "Input",
  CellLabel->"In[161]:=",
  CellTags->"PermuteFA"],

Cell["PrintFA", "ObjectName",
  CellTags->"PrintFA"],

Cell["Function", "Highlight",
  CellTags->"PrintFA"],

Cell[TextData[{
  StyleBox["PrintFA[M]", "MR"],
  "  pretty prints a finite state machine ",
  StyleBox["M", "MR"],
  ". "
}], "Usage",
  CellTags->"PrintFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"},
          {"TransOnly", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"PrintFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  " tries to print non-normalized state sets. "
}], "Notes",
  CellTags->"PrintFA"],

Cell[TextData[{
  "Option ",
  StyleBox["TransOnly->True", "MR"],
  " only prints the transitions."
}], "Notes",
  CellTags->"PrintFA"],

Cell["This is useful only for small machines.", "Notes",
  CellTags->"PrintFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"PrintFA"],

Cell["Printing a DFA.", "ExampleText",
  CellTags->"PrintFA"],

Cell["m = CounterDFA[ b, 3, 3 ]", "Input",
  CellLabel->"In[220]:=",
  CellTags->"PrintFA"],

Cell[BoxData[
    \(PrintFA[m]\)], "Input",
  CellLabel->"In[222]:=",
  CellTags->"PrintFA"],

Cell["Printing an FA.", "ExampleText",
  CellTags->"PrintFA"],

Cell["m = IthSymbolFA[ b, -3, 3 ]", "Input",
  CellLabel->"In[223]:=",
  CellTags->"PrintFA"],

Cell[BoxData[
    \(PrintFA[m]\)], "Input",
  CellLabel->"In[224]:=",
  CellTags->"PrintFA"],

Cell["ProductFA", "ObjectName",
  CellTags->"ProductFA"],

Cell["Function", "Highlight",
  CellTags->"ProductFA"],

Cell[TextData[{
  StyleBox["ProductFA[M1,M2]", "MR"],
  "  computes the product automaton of ",
  StyleBox["M1", "MR"],
  " and ",
  StyleBox["M2", "MR"],
  ". "
}], "Usage",
  CellTags->"ProductFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"},
          {"Type", "\[Rule]", "Intersection", "Union"},
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ProductFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  "  causes the full product machine to be generated, rather than just the \
accessible part.  \nCurrently this feature is implemented for DFAs only."
}], "Notes",
  CellTags->"ProductFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Type->Union", "MR"],
  "  causes a machine for the union of the two regular languages to be \
returned. By default, a machine for intersection is produced."
}], "Notes",
  CellTags->"ProductFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["SumFA",
    ButtonData:>"SumFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["UnionFA",
    ButtonData:>"UnionFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["IntersectionFA",
    ButtonData:>"IntersectionFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ProductGraph",
    ButtonData:>"ProductGraph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ProductFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ProductFA"],

Cell["We construct the product of two counter automata.", "ExampleText",
  CellTags->"ProductFA"],

Cell["\<\
m1 = CounterDFA[ a, 2, 2 ]
m2 = CounterDFA[ b, 3, 2 ]\
\>", "Input",
  CellLabel->"In[76]:=",
  CellTags->"ProductFA"],

Cell[BoxData[
    \(m\  = \ ProductFA[\ m1, m2]\)], "Input",
  CellLabel->"In[82]:=",
  CellTags->"ProductFA"],

Cell[TextData[{
  "The product automaton accepts only words containing two ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  "'s and three ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  "'s."
}], "ExampleText",
  CellTags->"ProductFA"],

Cell[BoxData[
    \(LanguageFA[m, \(-7\)]\)], "Input",
  CellLabel->"In[83]:=",
  CellTags->"ProductFA"],

Cell["ProductTransitions", "ObjectName",
  CellTags->"ProductTransitions"],

Cell["Function", "Highlight",
  CellTags->"ProductTransitions"],

Cell[TextData[{
  StyleBox["ProductTransitions[T1,T2,k]", "MR"],
  "  computes the product of transition lists T1 and T2 over an alphabet of \
size k. "
}], "Usage",
  CellTags->"ProductTransitions"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "0", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ProductTransitions"],

Cell[TextData[{
  "The new transitions are of the form",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\ \)\((\((p, q)\), a, \((p', q')\))\)\)\)]],
  " where ",
  Cell[BoxData[
      \(TraditionalForm\`\((p, a, p')\)\)]],
  " is a transition in ",
  StyleBox["T1", "MR"],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\((q, a, q')\)\)]],
  " is a transition in ",
  StyleBox["T2", "MR"],
  ". "
}], "Notes",
  CellTags->"ProductTransitions"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->1", "MR"],
  " identifies ",
  Cell[BoxData[
      \(TraditionalForm\`\((p, q)\)\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\((q, p)\)\)]],
  "."
}], "Notes",
  CellTags->"ProductTransitions"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->2", "MR"],
  " identifies ",
  Cell[BoxData[
      \(TraditionalForm\`\((p, q)\)\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\((q, p)\)\)]],
  " and omits the diagonal."
}], "Notes",
  CellTags->"ProductTransitions"],

Cell[TextData[{
  "See also ",
  ButtonBox["ProductFA",
    ButtonData:>"ProductFA",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  " ",
  ButtonBox["SquareSA",
    ButtonData:>"SquareSA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ProductTransitions"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ProductTransitions"],

Cell["The transitions of a semiautomaton, and their product. ", "ExampleText",
  CellTags->"ProductTransitions"],

Cell[BoxData[
    \(\(t = {{1, 1, 1}, {1, 2, 2}, {2, 2, 3}, {2, 2, 4}, {3, 1, 1}, {3, 2, 
            2}, {4, 2, 3}, {4, 1, 4}};\)\)], "Input",
  CellLabel->"In[267]:=",
  CellTags->"ProductTransitions"],

Cell[BoxData[
    \(ProductTransitions[t, t, 2]\)], "Input",
  CellLabel->"In[268]:=",
  CellTags->"ProductTransitions"],

Cell[BoxData[
    \(ProductTransitions[t, t, 2, \ Full \[Rule] 1]\)], "Input",
  CellLabel->"In[269]:=",
  CellTags->"ProductTransitions"],

Cell[BoxData[
    \(ProductTransitions[t, t, 2, \ Full \[Rule] 2]\)], "Input",
  CellLabel->"In[270]:=",
  CellTags->"ProductTransitions"],

Cell["QuotientDFA", "ObjectName",
  CellTags->"QuotientDFA"],

Cell["Function", "Highlight",
  CellTags->"QuotientDFA"],

Cell[TextData[{
  StyleBox["QuotientDFA[L,K]", "MR"],
  "  computes the quotient automaton of a finite language ",
  StyleBox["L", "MR"],
  "  over alphabet ",
  StyleBox["K", "MR"],
  "."
}], "Usage",
  CellTags->"QuotientDFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"QuotientDFA"],

Cell["The resulting DFA is minimal. ", "Notes",
  CellTags->"QuotientDFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["BuildDFA",
    ButtonData:>"BuildDFA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["GenerateQuotients",
    ButtonData:>"GenerateQuotients",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["LeftQuotient",
    ButtonData:>"LeftQuotient",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["QuotientDFA",
    ButtonData:>"QuotientDFA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["QuotientMachineFA",
    ButtonData:>"QuotientMachineFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"QuotientDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"QuotientDFA"],

Cell["\<\
The language of all words of length 3 other than aaa, and the \
corresponding quotient automaton.\
\>", "ExampleText",
  CellTags->"QuotientDFA"],

Cell["W = Rest[Words[3]]", "Input",
  CellLabel->"In[116]:=",
  CellTags->"QuotientDFA"],

Cell[BoxData[
    \(m = QuotientDFA[W, 2, Normalize \[Rule] 1]\)], "Input",
  CellLabel->"In[118]:=",
  CellTags->"QuotientDFA"],

Cell[BoxData[
    \(States[m]\  // \ ColumnForm\)], "Input",
  CellLabel->"In[161]:=",
  CellTags->"QuotientDFA"],

Cell["The states are the left quotients of the original language.", \
"ExampleText",
  CellTags->"QuotientDFA"],

Cell[BoxData[{
    \(LeftQuotient[a, W]\), "\n", 
    \(LeftQuotient[aa, W]\)}], "Input",
  CellLabel->"In[163]:=",
  CellTags->"QuotientDFA"],

Cell["QuotientMachineFA", "ObjectName",
  CellTags->"QuotientMachineFA"],

Cell["Function", "Highlight",
  CellTags->"QuotientMachineFA"],

Cell[TextData[{
  StyleBox["QuotientMachineFA[M1,M2]", "MR"],
  "  computes an FA for the language of all words ",
  StyleBox["y", "MR"],
  " such that there is a word ",
  StyleBox["x", "MR"],
  " accepted by ",
  StyleBox["M1", "MR"],
  " such that ",
  StyleBox["M2", "MR"],
  " accepts ",
  StyleBox["xy", "MR"],
  "."
}], "Usage",
  CellTags->"QuotientMachineFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"QuotientMachineFA"],

Cell[TextData[{
  "The constructed machine accepts the left quotient language ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\[ScriptCapitalL](
              M\_1)\)\^\(\(\ \)\(-1\)\)\) \(\[ScriptCapitalL](M\_2)\)\)]],
  "."
}], "Notes",
  CellTags->"QuotientMachineFA"],

Cell["Uses a product machine construction and is thus quadratic.", "Notes",
  CellTags->"QuotientMachineFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["BuildDFA",
    ButtonData:>"BuildDFA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["GenerateQuotients",
    ButtonData:>"GenerateQuotients",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["LeftQuotient",
    ButtonData:>"LeftQuotient",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["QuotientDFA",
    ButtonData:>"QuotientDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"QuotientMachineFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"QuotientMachineFA"],

Cell[TextData[{
  "Machines for  ",
  Cell[BoxData[
      \(TraditionalForm\`\(a\^*\) b\ \(a\^*\)\)]],
  " and  ",
  Cell[BoxData[
      FormBox[
        RowBox[{\(a\^*\), "b", " ", 
          FormBox[\(\(a\^*\) \(b\)\(\ \)\),
            "TraditionalForm"], " ", \(a\^*\)}], TraditionalForm]]],
  ", and their left quotient."
}], "ExampleText",
  CellTags->"QuotientMachineFA"],

Cell[BoxData[
    \(MinimizeFA[\ 
      RegularExpressionToFA[RET[RES[a], b, RES[a], b, RES[a]], 2]]\)], "Input",\

  CellLabel->"In[182]:=",
  CellTags->"QuotientMachineFA"],

Cell[BoxData[
    \(\(m1 = DFA[3, 2, {{1, 2, 3}, {2, 3, 3}}, 1, {2}];\)\)], "Input",
  CellLabel->"In[190]:=",
  CellTags->"QuotientMachineFA"],

Cell[BoxData[
    \(\(m2\  = \ 
        DFA[4, 2, {{1, 2, 3, 4}, {2, 3, 4, 4}}, 1, {3}];\)\)], "Input",
  CellLabel->"In[191]:=",
  CellTags->"QuotientMachineFA"],

Cell["m = QuotientMachineFA[ m1, m2 ]", "Input",
  CellLabel->"In[192]:=",
  CellTags->"QuotientMachineFA"],

Cell[BoxData[
    \(LanguageFA[\ m, \ \(-5\)\ ]\)], "Input",
  CellLabel->"In[194]:=",
  CellTags->"QuotientMachineFA"],

Cell["The quotient machine is duly equivalent to m1. ", "ExampleText",
  CellTags->"QuotientMachineFA"],

Cell[BoxData[
    \(EquivalentQFA[\ m, \ m1\ ]\)], "Input",
  CellLabel->"In[195]:=",
  CellTags->"QuotientMachineFA"],

Cell["RandomDFA", "ObjectName",
  CellTags->"RandomDFA"],

Cell["Function", "Highlight",
  CellTags->"RandomDFA"],

Cell[TextData[{
  StyleBox["RandomDFA[n,k]", "MR"],
  "  produces a random DFA over alphabet K on n states. "
}], "Usage",
  CellTags->"RandomDFA"],

Cell["The machines are chosen according to a uniform distribution.", "Notes",
  CellTags->"RandomDFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["NumberOfDFA",
    ButtonData:>"NumberOfDFA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["RankToDFA",
    ButtonData:>"RankToDFA",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["RankOfDFA",
    ButtonData:>"RankOfDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"RandomDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RandomDFA"],

Cell["A random DFA on 5 states over a 3-letter alphabet. ", "ExampleText",
  CellTags->"RandomDFA"],

Cell["RandomDFA[5,3]", "Input",
  CellLabel->"In[153]:=",
  CellTags->"RandomDFA"],

Cell["RandomDFA[5,3]", "Input",
  CellLabel->"In[154]:=",
  CellTags->"RandomDFA"],

Cell["RankOfDFA", "ObjectName",
  CellTags->"RankOfDFA"],

Cell["Function", "Highlight",
  CellTags->"RankOfDFA"],

Cell[TextData[{
  StyleBox["RankOfDFA[M]", "MR"],
  "  computes the rank of the DFA ",
  StyleBox["M", "MR"],
  "  with respect to a canonical enumeration."
}], "Usage",
  CellTags->"RankOfDFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["RandomDFA",
    ButtonData:>"RandomDFA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["RankToDFA",
    ButtonData:>"RankToDFA",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["RankOfDFA",
    ButtonData:>"RankOfDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"RankOfDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RankOfDFA"],

Cell["The rank of a DFA.", "ExampleText",
  CellTags->"RankOfDFA"],

Cell[BoxData[
    \(m = IthSymbolFA[b, \(-2\), Full \[Rule] True]\)], "Input",
  CellLabel->"In[147]:=",
  CellTags->"RankOfDFA"],

Cell[BoxData[
    \(RankOfDFA[\ m\ ]\)], "Input",
  CellLabel->"In[148]:=",
  CellTags->"RankOfDFA"],

Cell[BoxData[
    \(RankToDFA[\ %, \ 7, \ 2\ ]\)], "Input",
  CellLabel->"In[149]:=",
  CellTags->"RankOfDFA"],

Cell["RankToDFA", "ObjectName",
  CellTags->"RankToDFA"],

Cell["Function", "Highlight",
  CellTags->"RankToDFA"],

Cell[TextData[{
  StyleBox["RankToDFA[r,n,k]", "MR"],
  "  computes the DFA of rank ",
  StyleBox["r", "MR"],
  " on ",
  StyleBox["n", "MR"],
  " states and ",
  StyleBox["k", "MR"],
  " symbols with respect to a canonical enumeration."
}], "Usage",
  CellTags->"RankToDFA"],

Cell["Is mildly useful to test algorithms operating on DFAs.", "Notes",
  CellTags->"RankToDFA"],

Cell[TextData[{
  "See also ",
  " ",
  ButtonBox["RandomDFA",
    ButtonData:>"RandomDFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  " ",
  ButtonBox["RankOfDFA",
    ButtonData:>"RankOfDFA",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["NumberOfDFA",
    ButtonData:>"NumberOfDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"RankToDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RankToDFA"],

Cell["A random DFA.", "ExampleText",
  CellTags->"RankToDFA"],

Cell["\<\
NumberOfDFA[5,2]
RankToDFA[ Random[Integer,{1,%}], 5, 2 ]\
\>", "Input",
  CellLabel->"In[148]:=",
  CellTags->"RankToDFA"],

Cell["DeleteTransitionFA", "ObjectName",
  CellTags->"DeleteTransitionFA"],

Cell["Function", "Highlight",
  CellTags->"DeleteTransitionFA"],

Cell[TextData[{
  StyleBox["DeleteTransitionFA", "MR"],
  StyleBox["[M,p,w,q]", "MR"],
  " removes the transition ",
  StyleBox["(p,w,q)", "MR"],
  "   from ",
  StyleBox["M", "MR"],
  ". Here ",
  StyleBox["w", "MR"],
  " may be a symbol or an integer. "
}], "Usage",
  CellTags->"DeleteTransitionFA"],

Cell[TextData[{
  "Can also be used in the form ",
  StyleBox["DeleteTransitionFA[M,{p,w,q}]", "MR"],
  " or ",
  StyleBox["DeleteTransitionFA", "MR"],
  StyleBox["[M,L]", "MR"],
  " where L is a list of transitions. "
}], "Notes",
  CellTags->"DeleteTransitionFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["AddTransitionFA",
    ButtonData:>"AddTransitionFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SwitchTransitionFA",
    ButtonData:>"SwitchTransitionFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DeleteTransitionFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DeleteTransitionFA"],

Cell["A semiautomaton that accepts all words.", "ExampleText",
  CellTags->"DeleteTransitionFA"],

Cell[BoxData[
    \(\(m = 
        SA[4, 2, {{1, 1, 1}, {1, 2, 2}, {2, 1, 3}, {2, 2, 4}, {3, 2, 1}, {3, 
              1, 2}, {4, 2, 3}, {4, 1, 4}}];\)\)], "Input",
  CellLabel->"In[158]:=",
  CellTags->"DeleteTransitionFA"],

Cell[BoxData[
    \(MinimizeFA[m]\)], "Input",
  CellLabel->"In[154]:=",
  CellTags->"DeleteTransitionFA"],

Cell["\<\
Removing one transition makes the acceptance language significantly \
more complicated.\
\>", "ExampleText",
  CellTags->"DeleteTransitionFA"],

Cell[BoxData[
    \(mm\  = \ DeleteTransitionFA[m, 1, 1, 1]\)], "Input",
  CellLabel->"In[159]:=",
  CellTags->"DeleteTransitionFA"],

Cell[BoxData[
    \(MinimizeFA[mm]\)], "Input",
  CellLabel->"In[72]:=",
  CellTags->"DeleteTransitionFA"],

Cell["ReverseFA", "ObjectName",
  CellTags->"ReverseFA"],

Cell["Function", "Highlight",
  CellTags->"ReverseFA"],

Cell[TextData[{
  StyleBox["ReverseFA[M]", "MR"],
  "  reverses a FSM ",
  StyleBox["M", "MR"],
  " by flipping transitions and interchanging initial and final states. "
}], "Usage",
  CellTags->"ReverseFA"],

Cell["\<\
The result is an FA or SA, even if the original machine is a DFA.\
\
\>", "Notes",
  CellTags->"ReverseFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ReverseFA"],

Cell[TextData[{
  "A DFA for all words whose 3rd symbol is a ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  " and its reversal."
}], "ExampleText",
  CellTags->"ReverseFA"],

Cell[BoxData[
    \(m = IthSymbolFA[b, 3, 2, Full \[Rule] True]\)], "Input",
  CellLabel->"In[162]:=",
  CellTags->"ReverseFA"],

Cell[BoxData[
    \(mm = ReverseFA[m]\)], "Input",
  CellLabel->"In[163]:=",
  CellTags->"ReverseFA"],

Cell[BoxData[
    \(LanguageFA[\ mm, \ \(-5\)\ ]\)], "Input",
  CellLabel->"In[200]:=",
  CellTags->"ReverseFA"],

Cell["SA", "ObjectName",
  CellTags->"SA"],

Cell["Constructor", "Highlight",
  CellTags->"SA"],

Cell[TextData[{
  StyleBox["SA[n,k,T]", "MR"],
  "  denotes a semiautomaton. "
}], "Usage",
  CellTags->"SA"],

Cell[TextData[{
  StyleBox["n", "MR"],
  " is the number of states, ",
  StyleBox["k", "MR"],
  " the alphabet size, ",
  StyleBox["T", "MR"],
  " a list of transitions ",
  StyleBox["{p,a,q}.", "MR"]
}], "Notes",
  CellTags->"SA"],

Cell[TextData[{
  StyleBox["The entries in ", "Text"],
  StyleBox["T", "MR"],
  StyleBox[" are always in positional notation.", "Text"]
}], "Notes",
  CellTags->"SA"],

Cell["All states are considered initial and final.", "Notes",
  CellTags->"SA"],

Cell[TextData[{
  "There are hidden fields ",
  StyleBox["K", "MR"],
  " for the alphabet and ",
  StyleBox["Q", "MR"],
  " for a structured state set."
}], "Notes",
  CellTags->"SA"],

Cell[TextData[{
  "See also ",
  ButtonBox["FA",
    ButtonData:>"FA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["NFA",
    ButtonData:>"NFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DFA",
    ButtonData:>"DFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SA"],

Cell["A small semiautomaton.", "ExampleText",
  CellTags->"SA"],

Cell[BoxData[
    \(\(m = 
        SA[4, \(-2\), {{1, 1, 1}, {1, 2, 2}, {2, 1, 3}, {2, 2, 4}, {3, 2, 
              1}, {3, 1, 2}, {4, 2, 3}, {4, 1, 4}}];\)\)], "Input",
  CellLabel->"In[70]:=",
  CellTags->"SA"],

Cell[BoxData[{
    \(Initial[m]\), "\n", 
    \(Final[m]\), "\n", 
    \(Alphabet[m]\)}], "Input",
  CellLabel->"In[71]:=",
  CellTags->"SA"],

Cell["SelectFA", "ObjectName",
  CellTags->"SelectFA"],

Cell["Function", "Highlight",
  CellTags->"SelectFA"],

Cell[TextData[{
  StyleBox["SelectFA[M,L]", "MR"],
  "  computes the subautomaton of ",
  StyleBox["M", "MR"],
  " induced by the states in ",
  StyleBox["L", "MR"],
  "."
}], "Usage",
  CellTags->"SelectFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[Rule]", "1", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"SelectFA"],

Cell["L is required to be a list in positional notation.", "Notes",
  CellTags->"SelectFA"],

Cell["\<\
The state set of the original machine is maintained by \
default.\
\>", "Notes",
  CellTags->"SelectFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SelectFA"],

Cell[TextData[{
  "A DFA for ",
  Cell[BoxData[
      \(TraditionalForm\`\(\((a + b)\)\^\(\(\ \)\(*\)\)\) 
        b\ \((a + b)\)\^\(\(\ \)\(2\)\)\)]],
  ".  We select the subautomaton with states 8 through 15."
}], "ExampleText",
  CellTags->"SelectFA"],

Cell[BoxData[
    \(\(m = 
        DFA[15, 2, {{2, 4, 6, 8, 10, 12, 14, 8, 10, 12, 14, 8, 10, 12, 
              14}, {3, 5, 7, 9, 11, 13, 15, 9, 11, 13, 15, 9, 11, 13, 15}}, 
          1, {12, 13, 14, 15}];\)\)], "Input",
  CellLabel->"In[1]:=",
  InitializationCell->True,
  CellTags->"SelectFA"],

Cell[BoxData[
    \(mm\  = \ 
      SetInitialFA[\ SelectFA[\ m, \ Range[8, 15]\ ], {1}]\)], "Input",
  CellLabel->"In[194]:=",
  InitializationCell->True,
  CellTags->"SelectFA"],

Cell[BoxData[
    \(States[mm]\)], "Input",
  CellLabel->"In[195]:=",
  InitializationCell->True,
  CellTags->"SelectFA"],

Cell["\<\
By choosing the proper initial state we magically obtain an \
equivalent automaton.\
\>", "ExampleText",
  CellTags->"SelectFA"],

Cell[BoxData[
    \(mm\  = \ SetInitialFA[\ mm, {1}]\)], "Input",
  CellLabel->"In[196]:=",
  InitializationCell->True,
  CellTags->"SelectFA"],

Cell[BoxData[
    \(EquivalentQFA[m, mm]\)], "Input",
  CellLabel->"In[197]:=",
  InitializationCell->True,
  CellTags->"SelectFA"],

Cell["SetAlphabet", "ObjectName",
  CellTags->"SetAlphabet"],

Cell["Function", "Highlight",
  CellTags->"SetAlphabet"],

Cell[TextData[{
  StyleBox["SetAlphabet[M,K]", "MR"],
  "  is a mutating operation that sets the alphabet of ",
  StyleBox["M", "MR"],
  " to ",
  StyleBox["K", "MR"],
  "."
}], "Usage",
  CellTags->"SetAlphabet"],

Cell["\<\
The new alphabet must have the same cardinality as the old \
alphabet.\
\>", "Notes",
  CellTags->"SetAlphabet"],

Cell[TextData[{
  "See also ",
  ButtonBox["SetStates",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SetAlphabet"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SetAlphabet"],

Cell[TextData[{
  "A DFA over the standard two-letter alphabet ",
  Cell[BoxData[
      FormBox[
        RowBox[{\({a, b}\), Cell[""]}], TraditionalForm]]],
  ". "
}], "ExampleText",
  CellTags->"SetAlphabet"],

Cell[BoxData[
    \(m = BuildDFA[\ aaa, bbb, \ 2\ ]\)], "Input",
  CellLabel->"In[20]:=",
  CellTags->"SetAlphabet"],

Cell[BoxData[
    \(Alphabet[m]\)], "Input",
  CellLabel->"In[21]:=",
  CellTags->"SetAlphabet"],

Cell[TextData[{
  "This switches the alphabet of ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  " to ",
  Cell[BoxData[
      \(TraditionalForm\`{c, d}\)]],
  "."
}], "ExampleText",
  CellTags->"SetAlphabet"],

Cell[BoxData[{
    \(\(SetAlphabet[m, {"\<c\>", "\<d\>"}];\)\), "\n", 
    \(Alphabet[m]\)}], "Input",
  CellLabel->"In[22]:=",
  CellTags->"SetAlphabet"],

Cell["The size of the new alphabet must match.", "ExampleText",
  CellTags->"SetAlphabet"],

Cell[BoxData[
    \(SetAlphabet[m, {"\<c\>"}]\)], "Input",
  CellLabel->"In[227]:=",
  CellTags->"SetAlphabet"],

Cell["SetFinalFA", "ObjectName",
  CellTags->"SetFinalFA"],

Cell["Function", "Highlight",
  CellTags->"SetFinalFA"],

Cell[TextData[{
  StyleBox["SetFinalFA[m,F]", "MR"],
  "  returns a FSM  ",
  StyleBox["M", "MR"],
  "  with final states changed to  ",
  StyleBox["F", "MR"],
  ". "
}], "Usage",
  CellTags->"SetFinalFA"],

Cell[TextData[{
  StyleBox["F", "MR"],
  " is required to be a list or integer in positional notation."
}], "Notes",
  CellTags->"SetFinalFA"],

Cell[TextData[{
  "Lists are automatically flattened, so output from ",
  ButtonBox["Position",
    ButtonStyle->"RefGuideLink"],
  " can be used."
}], "Notes",
  CellTags->"SetFinalFA"],

Cell[TextData[{
  "There may be type conversions, e.g., SAs are converted to FAs.  As a \
consequence,  ",
  StyleBox["SetFinalFA", "MR"],
  "  is not a mutating operation but returns a new FSM object."
}], "Notes",
  CellTags->"SetFinalFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["SetInitialFA",
    ButtonData:>"SetInitialFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SetInitialFinalFA",
    ButtonData:>"SetInitialFinalFA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["SetAlphabet",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SetStates",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SetFinalFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SetFinalFA"],

Cell[TextData[{
  "An automaton for ",
  Cell[BoxData[
      \(TraditionalForm\`L(
          b, \(-3\))\  = \ \(\((a + 
                  b)\)\^*\)\ b\ \((a + b)\)\^\(\(\ \)\(2\)\)\)]],
  " and some machines obtained by modifying the final states."
}], "ExampleText",
  CellTags->"SetFinalFA"],

Cell[BoxData[
    \(\(m = 
        FA[4, 2, {{1, 1, 1}, {1, 2, 1}, {1, 2, 2}, {2, 1, 3}, {2, 2, 
              3}, \[IndentingNewLine]{3, 1, 4}, {3, 2, 
              4}}, {1}, {4}];\)\)], "Input",
  CellLabel->"In[78]:=",
  CellTags->"SetFinalFA"],

Cell[BoxData[
    \(m0\  = \ SetFinalFA[m, 3\ ]\)], "Input",
  CellLabel->"In[79]:=",
  CellTags->"SetFinalFA"],

Cell[TextData[{
  "This machine accepts  ",
  Cell[BoxData[
      \(TraditionalForm\`L(
          b, \(-2\))\  = \ \(\((a + b)\)\^*\)\ b\ \((a + b)\)\)]],
  "."
}], "ExampleText",
  CellTags->"SetFinalFA"],

Cell[BoxData[
    \(LanguageFA[m0, \(-5\)]\)], "Input",
  CellLabel->"In[80]:=",
  CellTags->"SetFinalFA"],

Cell[TextData[{
  "And this one accepts  ",
  Cell[BoxData[
      \(TraditionalForm\`L(
          b, \(-2\))\  = \ \(\((a + b)\)\^*\)\ b\ \((a + 
              b)\) \((\ \[Epsilon]\  + \ a\  + \ b)\)\)]],
  "."
}], "ExampleText",
  CellTags->"SetFinalFA"],

Cell[BoxData[{
    \(\(m1\  = \ SetFinalFA[m, {3, 4}\ ];\)\), "\n", 
    \(LanguageFA[m1, \(-10\), \ SizeOnly \[Rule] True\ ]\)}], "Input",
  CellLabel->"In[87]:=",
  CellTags->"SetFinalFA"],

Cell["SetInitialFA", "ObjectName",
  CellTags->"SetInitialFA"],

Cell["Function", "Highlight",
  CellTags->"SetInitialFA"],

Cell[TextData[{
  StyleBox["SetInitialFinalFA[m,I]", "MR"],
  "  returns FSM ",
  StyleBox["M", "MR"],
  " with initial states changed to ",
  StyleBox["I", "MR"],
  "."
}], "Usage",
  CellTags->"SetInitialFA"],

Cell[TextData[{
  " ",
  StyleBox["I", "MR"],
  " is required to be a list or integer in positional notation."
}], "Notes",
  CellTags->"SetInitialFA"],

Cell[TextData[{
  "Lists are automatically flattened, so output from ",
  ButtonBox["Position",
    ButtonStyle->"RefGuideLink"],
  " can be used."
}], "Notes",
  CellTags->"SetInitialFA"],

Cell[TextData[{
  "There may be type conversions, e.g., SAs are converted to FAs.  As a \
consequence,  ",
  StyleBox["SetInitialFA", "MR"],
  "  is not a mutating operation but returns a new FSM object."
}], "Notes",
  CellTags->"SetInitialFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["SetInitialFinalFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SetFinalFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SetAlphabet",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SetStates",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SetInitialFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SetInitialFA"],

Cell[TextData[{
  "An automaton for ",
  Cell[BoxData[
      \(TraditionalForm\`\(\((a + 
                b)\)\^*\)\ b\ \((a + b)\)\^\(\(\ \)\(2\)\)\)]],
  " and some machines obtained by modifying the initial states."
}], "ExampleText",
  CellTags->"SetInitialFA"],

Cell[BoxData[
    \(\(m = 
        FA[4, 2, {{1, 1, 1}, {1, 2, 1}, {1, 2, 2}, {2, 1, 3}, {2, 2, 3}, {3, 
              1, 4}, {3, 2, 4}}, {1}, {4}];\)\)], "Input",
  CellLabel->"In[36]:=",
  CellTags->"SetInitialFA"],

Cell[BoxData[
    \(SetInitialFA[m, 2]\)], "Input",
  CellLabel->"In[26]:=",
  CellTags->"SetInitialFA"],

Cell[BoxData[
    \(LanguageFA[%, \(-5\)]\)], "Input",
  CellLabel->"In[27]:=",
  CellTags->"SetInitialFA"],

Cell[BoxData[{
    \(SetInitialFA[m, {2, 3, 4}]\), "\n", 
    \(LanguageFA[%, \(-5\)]\)}], "Input",
  CellLabel->"In[41]:=",
  CellTags->"SetInitialFA"],

Cell["SetInitialFinalFA", "ObjectName",
  CellTags->"SetInitialFinalFA"],

Cell["Function", "Highlight",
  CellTags->"SetInitialFinalFA"],

Cell[TextData[{
  StyleBox["SetInitialFinalFA[m,I,F]", "MR"],
  "  returns FSM ",
  StyleBox["M", "MR"],
  " with initial states changed to ",
  StyleBox["I", "MR"],
  " and final states changed to ",
  StyleBox["F", "MR"],
  ". "
}], "Usage",
  CellTags->"SetInitialFinalFA"],

Cell[TextData[{
  "Both ",
  StyleBox["I", "MR"],
  " and ",
  StyleBox["F", "MR"],
  " are required to be lists or integers in positional notation."
}], "Notes",
  CellTags->"SetInitialFinalFA"],

Cell[TextData[{
  "Lists are automatically flattened, so output from ",
  ButtonBox["Position",
    ButtonStyle->"RefGuideLink"],
  " can be used."
}], "Notes",
  CellTags->"SetInitialFinalFA"],

Cell[TextData[{
  "There may be type conversions, e.g., SAs are converted to FAs.  As a \
consequence,  ",
  StyleBox["SetInitialFinalFA", "MR"],
  "  is not a mutating operation but returns a new FSM object."
}], "Notes",
  CellTags->"SetInitialFinalFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["SetInitialFA",
    ButtonData:>"SetInitialFA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["SetFinalFA",
    ButtonData:>"SetFinalFA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["SetAlphabet",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SetStates",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SetInitialFinalFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SetInitialFinalFA"],

Cell["\<\
A semiautomaton and the machine obtained by selecting 1 to be the \
initial and final state.\
\>", "ExampleText",
  CellTags->"SetInitialFinalFA"],

Cell[BoxData[
    \(\(m = 
        SA[4, 2, {{1, 1, 1}, {1, 2, 2}, {2, 1, 3}, {2, 2, 4}, {3, 2, 1}, {3, 
              1, 2}, {4, 2, 3}, {4, 1, 4}}];\)\)], "Input",
  CellLabel->"In[43]:=",
  CellTags->"SetInitialFinalFA"],

Cell[BoxData[
    \(mm\  = \ SetInitialFinalFA[m, 1, 1\ ]\)], "Input",
  CellLabel->"In[46]:=",
  CellTags->"SetInitialFinalFA"],

Cell[BoxData[
    \(LanguageFA[mm, \(-5\)]\)], "Input",
  CellLabel->"In[47]:=",
  CellTags->"SetInitialFinalFA"],

Cell[TextData[{
  "A tedious construction of an automaton for all factors of ",
  Cell[BoxData[
      FormBox[
        StyleBox["aabbcc",
          FontSlant->"Italic"], TraditionalForm]]],
  "."
}], "ExampleText",
  CellTags->"SetInitialFinalFA"],

Cell[BoxData[{
    \(\(m\  = \ BuildDFA[\ aabbcc, \ 3\ ];\)\), "\n", 
    \(mm\  = \ DeleteStateFA[\ m, \ TrapStatesFA[m]\ ]\)}], "Input",
  CellLabel->"In[68]:=",
  CellTags->"SetInitialFinalFA"],

Cell[BoxData[{
    \(\(mmm\  = \ SetInitialFinalFA[\ mm, Range[7], Range[7]\ ];\)\), "\n", 
    \(LanguageFA[mmm, \(-8\)]\)}], "Input",
  CellLabel->"In[74]:=",
  CellTags->"SetInitialFinalFA"],

Cell["SetStates", "ObjectName",
  CellTags->"SetStates"],

Cell["Function", "Highlight",
  CellTags->"SetStates"],

Cell[TextData[{
  StyleBox["SetStates[M,Q]", "MR"],
  "  is a mutating operation that sets the states of ",
  StyleBox["M", "MR"],
  " to ",
  StyleBox["Q", "MR"],
  "."
}], "Usage",
  CellTags->"SetStates"],

Cell["\<\
The new state set must have the same cardinality as the old state \
set.\
\>", "Notes",
  CellTags->"SetStates"],

Cell[TextData[{
  "See also ",
  ButtonBox["SetAlphabet",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SetStates"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SetStates"],

Cell["\<\
A DFA whose natural state set consists of all left quotients of a \
finite set of words.\
\>", "ExampleText",
  CellTags->"SetStates"],

Cell[BoxData[
    \(m = BuildDFA[\ aaa, bbb, \ 2, \ Normalize \[Rule] \ 1\ ]\)], "Input",
  CellLabel->"In[15]:=",
  CellTags->"SetStates"],

Cell[BoxData[
    \(States[m]\)], "Input",
  CellLabel->"In[16]:=",
  CellTags->"SetStates"],

Cell["\<\
This switches the state set to one that records only the \
cardinalities of the original set.\
\>", "ExampleText",
  CellTags->"SetStates"],

Cell[BoxData[{
    \(\(SetStates[m, Length\  /@ \ States[m]];\)\), "\n", 
    \(States[m]\)}], "Input",
  CellLabel->"In[17]:=",
  CellTags->"SetStates"],

Cell["The lengths must match.", "ExampleText",
  CellTags->"SetStates"],

Cell[BoxData[
    \(SetStates[m, {1, 2, 3}]\)], "Input",
  CellLabel->"In[109]:=",
  CellTags->"SetStates"],

Cell["ShowAll", "ObjectName",
  CellTags->"ShowAll"],

Cell["Option", "Highlight",
  CellTags->"ShowAll"],

Cell[TextData[{
  StyleBox["ShowAll", "MR"],
  "   is an option for ",
  StyleBox["PlotComputationFA", "MR"],
  ", "
}], "Usage",
  CellTags->"ShowAll"],

Cell["Causes a display of all potential paths in the computation. ", "Notes",
  CellTags->"ShowAll"],

Cell[TextData[{
  "If set to ",
  StyleBox["False", "MR"],
  " only persistent paths are shown."
}], "Notes",
  CellTags->"ShowAll"],

Cell[TextData[{
  "See also ",
  ButtonBox["PlotComputationFA",
    ButtonData:>"PlotComputationFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ShowAll"],

Cell["ShowEdges", "ObjectName",
  CellTags->"ShowEdges"],

Cell["Option", "Highlight",
  CellTags->"ShowEdges"],

Cell[TextData[{
  StyleBox["ShowEdges", "MR"],
  "  is an option for ",
  StyleBox["PlotComputationFA", "MR"],
  ","
}], "Usage",
  CellTags->"ShowEdges"],

Cell["\<\
 Causes the edges of the computation graph rather than a graphics \
object to be returned.\
\>", "Notes",
  CellTags->"ShowEdges"],

Cell[TextData[{
  "See also ",
  ButtonBox["PlotComputationFA",
    ButtonData:>"PlotComputationFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ShowEdges"],

Cell["ShuffleProductFA", "ObjectName",
  CellTags->"ShuffleProductFA"],

Cell["Function", "Highlight",
  CellTags->"ShuffleProductFA"],

Cell[TextData[{
  StyleBox["ShuffleProductFA[M1,M2]", "MR"],
  "  generates a finite state machine that accepts the shuffle product of the \
acceptance languages of FSMs ",
  StyleBox["M1", "MR"],
  " and ",
  StyleBox["M2", "MR"],
  "."
}], "Usage",
  CellTags->"ShuffleProductFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ShuffleProductFA"],

Cell["Uses a product machine construction and is thus quadratic.", "Notes",
  CellTags->"ShuffleProductFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordShuffle",
    ButtonData:>"WordShuffle",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ShuffleProductFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ShuffleProductFA"],

Cell[TextData[{
  "Two machines for words consisting either of all ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  "'s or all ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  "'s.  Their shuffle product ought to be everything."
}], "ExampleText",
  CellTags->"ShuffleProductFA"],

Cell["\<\
 m1 = CounterDFA[ a, 0, 2 ]
m2 = CounterDFA[ b, 0, 2 ]\
\>", "Input",
  CellLabel->"In[24]:=",
  CellTags->"ShuffleProductFA"],

Cell[BoxData[
    \(mm\  = \ ShuffleProductFA[m1, m2]\)], "Input",
  CellLabel->"In[26]:=",
  CellTags->"ShuffleProductFA"],

Cell[BoxData[
    \(LanguageFA[mm, \(-8\), SizeOnly \[Rule] True\ ]\)], "Input",
  CellLabel->"In[155]:=",
  CellTags->"ShuffleProductFA"],

Cell["SinkQDFA", "ObjectName",
  CellTags->"SinkQDFA"],

Cell["Function", "Highlight",
  CellTags->"SinkQDFA"],

Cell[TextData[{
  StyleBox["SinkQDFA[M]", "MR"],
  " tests whether DFA  ",
  StyleBox["M", "MR"],
  "  is a sink automaton."
}], "Usage",
  CellTags->"SinkQDFA"],

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  " is considered a sink automaton if it contains a trap state that is the \
only non-final state in ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  ".  This is useful mostly for minimal DFAs."
}], "Notes",
  CellTags->"SinkQDFA"],

Cell[TextData[{
  "See also  ",
  ButtonBox["TrapStatesFA",
    ButtonData:>"TrapStatesFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SinkQDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SinkQDFA"],

Cell["A machine with two trap states but several non-final states.", \
"ExampleText",
  CellTags->"SinkQDFA"],

Cell[BoxData[{
    \(\(m\  = \ DFA[4, 2, {{2, 2, 3, 4}, {3, 1, 3, 4}}, 1, {2}];\)\), "\n", 
    \(TrapStatesFA[m]\)}], "Input",
  CellLabel->"In[133]:=",
  CellTags->"SinkQDFA"],

Cell[TextData[{
  "Hence, ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  " is not a sink automaton."
}], "ExampleText",
  CellTags->"SinkQDFA"],

Cell[BoxData[
    \(SinkQDFA[m]\)], "Input",
  CellLabel->"In[135]:=",
  CellTags->"SinkQDFA"],

Cell["\<\
Sink automata arise naturally in the context of one-dimensional \
cellular automata.\
\>", "ExampleText",
  CellTags->"SinkQDFA"],

Cell[BoxData[
    \(\(\(CA[110]\  // \ ToSA\)\  // \ MinimizeFA\)\  // \ 
      SinkQDFA\)], "Input",
  CellLabel->"In[136]:=",
  CellTags->"SinkQDFA"],

Cell["SizeOnly", "ObjectName",
  CellTags->"SizeOnly"],

Cell["Option", "Highlight",
  CellTags->"SizeOnly"],

Cell[TextData[{
  StyleBox["SizeOnly", "MR"],
  "  is an option in ",
  StyleBox["RegularExpressionToFA", "MR"]
}], "Usage",
  CellTags->"SizeOnly"],

Cell[TextData[{
  "Option ",
  StyleBox["SizeOnly->True", "MR"],
  " in  ",
  StyleBox["LanguageFA", "MR"],
  "  causes a list of the first ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " values of the growth function of the language to be returned."
}], "Notes",
  CellTags->"SizeOnly"],

Cell[TextData[{
  " Option ",
  StyleBox["SizeOnly->True", "MR"],
  " in ",
  StyleBox["RegularExpressionToFA", "MR"],
  " causes  only the size of the FA to be returned. "
}], "Notes",
  CellTags->"SizeOnly"],

Cell[TextData[{
  "See also ",
  ButtonBox["LanguageFA",
    ButtonData:>"LanguageFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["RegularExpressionToFA",
    ButtonData:>"RegularExpressionToFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SizeOnly"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SizeOnly"],

Cell["The usual suspect.", "ExampleText",
  CellTags->"SizeOnly"],

Cell[BoxData[
    \(\(m = 
        FA[4, 2, {{1, 1, 1}, {1, 2, 1}, {1, 2, 2}, {2, 1, 3}, {2, 2, 3}, {3, 
              1, 4}, {3, 2, 4}}, {1}, {4}];\)\)], "Input",
  CellLabel->"In[106]:=",
  CellTags->"SizeOnly"],

Cell[BoxData[
    \(LanguageFA[\ m, \ \(-10\), \ SizeOnly \[Rule] True\ ]\)], "Input",
  CellLabel->"In[107]:=",
  CellTags->"SizeOnly"],

Cell["Size", "ObjectName",
  CellTags->"Size"],

Cell["Function", "Highlight",
  CellTags->"Size"],

Cell[TextData[{
  StyleBox["Size[M]", "MR"],
  "  returns the number of states of machine M."
}], "Usage",
  CellTags->"Size"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"Size"],

Cell[TextData[{
  "Option  ",
  StyleBox["Full->True", "MR"],
  "  produces a listing of the number of states, the size of the alphabet, \
the number of transitions, the number of initial states and the number of \
final states."
}], "Notes",
  CellTags->"Size"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"Size"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Size"],

Cell["Extracting the number of states of a machine.", "ExampleText",
  CellTags->"Size"],

Cell[BoxData[{
    \(IthSymbolFA[a, \(-4\), 2, Full \[Rule] True]\), "\n", 
    \(%\  // \ Size\)}], "Input",
  CellLabel->"In[328]:=",
  CellTags->"Size"],

Cell["Extracting the number of states of a machine.", "ExampleText",
  CellTags->"Size"],

Cell[BoxData[
    \(Size[\ IthSymbolFA[a, \(-4\), 2], Full \[Rule] True]\)], "Input",
  CellLabel->"In[326]:=",
  CellTags->"Size"],

Cell["Some", "ObjectName",
  CellTags->"Some"],

Cell["Option", "Highlight",
  CellTags->"Some"],

Cell[TextData[{
  StyleBox["Some", "MR"],
  "  is an option for ",
  StyleBox["FormatAutomata", "MR"],
  "."
}], "Usage",
  CellTags->"Some"],

Cell[TextData[{
  "See ",
  ButtonBox["FormatAutomata",
    ButtonData:>"FormatAutomata",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Some"],

Cell["SquareSA", "ObjectName",
  CellTags->"SquareSA"],

Cell["Function", "Highlight",
  CellTags->"SquareSA"],

Cell[TextData[{
  StyleBox["SquareSA[M]", "MR"],
  "  computes the product semiautomaton of semiautomaton ",
  StyleBox["M", "MR"],
  " with itself. "
}], "Usage",
  CellTags->"SquareSA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "0", \(1, 2\)},
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", \(0, 1, 2\)}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"SquareSA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->0", "MR"],
  "  uses the full Cartesian product of the state set of ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  " with itself."
}], "Notes",
  CellTags->"SquareSA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->1", "MR"],
  "  generates only the part below and including the diagonal. "
}], "Notes",
  CellTags->"SquareSA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->2", "MR"],
  "  generates only the part strictly below the diagonal. "
}], "Notes",
  CellTags->"SquareSA"],

Cell[TextData[{
  "Option ",
  StyleBox["DoTrace->True", "MR"],
  " causes the positions of the diagonal elements ",
  Cell[BoxData[
      \(TraditionalForm\`{x, x}\)]],
  "  in the state set to be returned alongside with the automaton."
}], "Notes",
  CellTags->"SquareSA"],

Cell["Used in the classification algorithm for cellular automata.", "Notes",
  CellTags->"SquareSA"],

Cell[TextData[{
  "See also ",
  ButtonBox["ProductFA",
    ButtonData:>"ProductFA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["ClassifyCA",
    ButtonData:>"ClassifyCA",
    ButtonStyle->"AddOnsLink"],
  " . "
}], "Notes",
  CellTags->"SquareSA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SquareSA"],

Cell[TextData[{
  "A semiautomaton with ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " states and its square of size ",
  Cell[BoxData[
      \(TraditionalForm\`n\^2\)]],
  "."
}], "ExampleText",
  CellTags->"SquareSA"],

Cell[BoxData[
    \(\(m = 
        SA[4, 2, {{1, 1, 1}, {1, 2, 2}, {2, 2, 3}, {2, 2, 4}, {3, 1, 1}, {3, 
              2, 2}, {4, 2, 3}, {4, 1, 4}}];\)\)], "Input",
  CellLabel->"In[65]:=",
  CellTags->"SquareSA"],

Cell[BoxData[
    \(SquareSA[m]\)], "Input",
  CellLabel->"In[66]:=",
  CellTags->"SquareSA"],

Cell[TextData[{
  "We can reduce the size of the square to ",
  Cell[BoxData[
      \(TraditionalForm\`\(n(n + 1)\)/2\)]],
  "  and ",
  Cell[BoxData[
      \(TraditionalForm\`\(n(n - 1)\)/2\)]],
  ",  by omitting states of the form ",
  Cell[BoxData[
      \(TraditionalForm\`{x, y}\)]],
  " where ",
  Cell[BoxData[
      \(TraditionalForm\`x\  > \ y\)]],
  " or  ",
  Cell[BoxData[
      \(TraditionalForm\`x\  \[GreaterEqual] \ y\)]],
  ", respectively.  On occasion this can be used to speed up some \
algorithms."
}], "ExampleText",
  CellTags->"SquareSA"],

Cell[BoxData[{
    \(SquareSA[m, Full \[Rule] 1, Normalize \[Rule] 1\ ]\), "\n", 
    \(States[%]\)}], "Input",
  CellLabel->"In[67]:=",
  CellTags->"SquareSA"],

Cell[BoxData[
    \(SquareSA[m, Full \[Rule] 2]\)], "Input",
  CellLabel->"In[69]:=",
  CellTags->"SquareSA"],

Cell["StarFA", "ObjectName",
  CellTags->"StarFA"],

Cell["Function", "Highlight",
  CellTags->"StarFA"],

Cell[TextData[{
  StyleBox["StarFA[M]", "MR"],
  "  generates a FA that accepts the Kleene star of the acceptance languages \
of ",
  StyleBox["M", "MR"],
  ". "
}], "Usage",
  CellTags->"StarFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", \(0, 1, 2\)},
          {"EpsElim", "\[Rule]", "True", "False"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"StarFA"],

Cell[TextData[{
  "Option ",
  StyleBox["EpsElim->False", "MR"],
  " supresses epsilon elimination and returns a machine with epsilon moves."
}], "Notes",
  CellTags->"StarFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["UnionFA",
    ButtonData:>"UnionFA",
    ButtonStyle->"AddOnsLink"],
  " , ",
  ButtonBox["SumFA",
    ButtonData:>"SumFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ProductFA",
    ButtonData:>"ProductFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["IntersectionFA",
    ButtonData:>"IntersectionFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"StarFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"StarFA"],

Cell[TextData[{
  "Starring a machine for ",
  Cell[BoxData[
      \(TraditionalForm\`{a, b}\)]],
  " should produce a machine accepting all words."
}], "ExampleText",
  CellTags->"StarFA"],

Cell["m = BuildDFA[a,b,2]", "Input",
  CellLabel->"In[47]:=",
  CellTags->"StarFA"],

Cell[BoxData[
    \(mm\  = \ StarFA[m]\)], "Input",
  CellLabel->"In[48]:=",
  CellTags->"StarFA"],

Cell[BoxData[
    \(LanguageFA[\ mm, \ \(-8\), \ SizeOnly \[Rule] True\ ]\)], "Input",
  CellLabel->"In[148]:=",
  CellTags->"StarFA"],

Cell["StateMergeDFA", "ObjectName",
  CellTags->"StateMergeDFA"],

Cell["Function", "Highlight",
  CellTags->"StateMergeDFA"],

Cell[TextData[{
  StyleBox["StateMergeDFA[M,R]", "MR"],
  "  performs state-merging on a DFA ",
  StyleBox["M", "MR"],
  " according to the equivalence relation represented by list R. "
}], "Usage",
  CellTags->"StateMergeDFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"},
          {"DoTrace", "\[Rule]", "False", "-"},
          {"Hopcroft", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"StateMergeDFA"],

Cell[TextData[{
  StyleBox["StateMergeDFA", "MR"],
  " is used in all minimization algorithms in this package (but they differ \
in how the behavioral equivalence relation is computed)."
}], "Notes",
  CellTags->"StateMergeDFA"],

Cell[TextData[{
  "No tests are performed to check if ",
  StyleBox["R", "MR"],
  " is admissible with respect to ",
  StyleBox["M", "MR"],
  "."
}], "Notes",
  CellTags->"StateMergeDFA"],

Cell[TextData[{
  " ",
  StyleBox["R", "MR"],
  " is required to be represented as an index, as produced for example by ",
  StyleBox["MakeIndex", "MR"],
  ". "
}], "Notes",
  CellTags->"StateMergeDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"StateMergeDFA"],

Cell[TextData[{
  "A DFA for all words of the form ",
  Cell[BoxData[
      \(TraditionalForm\`\(\((a + b)\)\^*\) 
        b\ \((a + b)\)\^\(\(\ \)\(2\)\)\)]],
  " and an equivalence relation. "
}], "ExampleText",
  CellTags->"StateMergeDFA"],

Cell[BoxData[{
    \(\(m = IthSymbolFA[b, \(-3\), 2, Full \[Rule] True];\)\), "\n", 
    \(\(R\  = \ {1, 1, 2, 1, 2, 3, 4, 1, 2, 3, 4, 5, 6, 7, 8};\)\)}], "Input",\

  CellLabel->"In[330]:=",
  CellTags->"StateMergeDFA"],

Cell["The DFA obtained by merging the corresponding states.", "ExampleText",
  CellTags->"StateMergeDFA"],

Cell[BoxData[
    \(mm = StateMergeDFA[m, R]\)], "Input",
  CellLabel->"In[332]:=",
  CellTags->"StateMergeDFA"],

Cell[BoxData[
    \(LanguageFA[mm, \(-5\)]\)], "Input",
  CellLabel->"In[333]:=",
  CellTags->"StateMergeDFA"],

Cell["States", "ObjectName",
  CellTags->"States"],

Cell["Function", "Highlight",
  CellTags->"States"],

Cell[TextData[{
  StyleBox["States[M]", "MR"],
  "  returns the state set of finite state machine ",
  StyleBox["M", "MR"],
  " "
}], "Usage",
  CellTags->"States"],

Cell[TextData[{
  "Returns the structured state set if it exists, otherwise a list ",
  Cell[BoxData[
      \(TraditionalForm\`{1, 2, \[Ellipsis], n}\)]],
  ". "
}], "Notes",
  CellTags->"States"],

Cell[TextData[{
  "See also ",
  ButtonBox["Normalize",
    ButtonData:>"Normalize",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["NormalizeFA",
    ButtonData:>"NormalizeFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"States"],

Cell["Further Examples", "ExampleSection",
  CellTags->"States"],

Cell["\<\
Some constructors allow for structured state sets.  Here is the \
unstructured version:\
\>", "ExampleText",
  CellTags->"States"],

Cell[BoxData[{
    \(BuildDFA[abc, cba, 3]\), "\n", 
    \(States[%]\)}], "Input",
  CellLabel->"In[672]:=",
  CellTags->"States"],

Cell["An here the states as left quotients of the input language.", \
"ExampleText",
  CellTags->"States"],

Cell[BoxData[{
    \(BuildDFA[abc, cba, 3, Normalize \[Rule] 1]\), "\n", 
    \(States[%]\)}], "Input",
  CellLabel->"In[41]:=",
  CellTags->"States"],

Cell["SubsetQFA", "ObjectName",
  CellTags->"SubsetQFA"],

Cell["Function", "Highlight",
  CellTags->"SubsetQFA"],

Cell[TextData[{
  StyleBox["SubsetQFA[M1,M2]", "MR"],
  "  tests whether the acceptance language of ",
  StyleBox["M1", "MR"],
  " is a subset of the acceptance language of ",
  StyleBox["M2", "MR"],
  ". "
}], "Usage",
  CellTags->"SubsetQFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"SubsetQFA"],

Cell[TextData[{
  "Uses ",
  StyleBox["ToDFA", "MR"],
  " if input is non-deterministic and therefore has potentially exponential \
space and time requirement."
}], "Notes",
  CellTags->"SubsetQFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["EquivalentQFA",
    ButtonData:>"EquivalentQFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SubsetQFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SubsetQFA"],

Cell["\<\
Two machines whose acceptance depends on the length of input words.\
\
\>", "ExampleText",
  CellTags->"SubsetQFA"],

Cell[BoxData[{
    \(m1 = LengthDFA[{1, 5}, 2]\), "\n", 
    \(m2 = LengthDFA[{1, 3, 5}, 2]\)}], "Input",
  CellLabel->"In[66]:=",
  CellTags->"SubsetQFA"],

Cell[BoxData[
    \(SubsetQFA[\ m1, m2]\)], "Input",
  CellLabel->"In[68]:=",
  CellTags->"SubsetQFA"],

Cell[BoxData[
    \(SubsetQFA[\ m2, m1]\)], "Input",
  CellLabel->"In[145]:=",
  CellTags->"SubsetQFA"],

Cell["SubwordCounterDFA", "ObjectName",
  CellTags->"SubwordCounterDFA"],

Cell["Function", "Highlight",
  CellTags->"SubwordCounterDFA"],

Cell[TextData[{
  StyleBox["SubwordCounterDFA[w,p,K,opts]", "MR"],
  "  constructs a DFA that accepts all words ",
  StyleBox["x", "MR"],
  " such that  ",
  StyleBox["WordBinomial[x,w] == p", "MR"],
  "  over alphabet ",
  StyleBox["K", "MR"],
  ". "
}], "Usage",
  CellTags->"SubwordCounterDFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "1"},
          {"Modulus", "\[Rule]", "0", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"SubwordCounterDFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Normalize->1", "MR"],
  " preserves the states as length ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(|\)\(w\)\(|\)\)\)]],
  " vectors over ",
  Cell[BoxData[
      \(TraditionalForm\`{0, 1, \[Ellipsis], p - 1}\)]],
  " for counters, and as \nlength ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(|\)\(w\)\(|\)\)\)]],
  " vectors over ",
  Cell[BoxData[
      \(TraditionalForm\`{0, 1, \[Ellipsis], p, p + 1}\)]],
  " for modular counters."
}], "Notes",
  CellTags->"SubwordCounterDFA"],

Cell[TextData[{
  "Option ",
  "Modulus",
  StyleBox["->m", "MR"],
  "  counts modulo ",
  Cell[BoxData[
      \(TraditionalForm\`m > 0\)]],
  "."
}], "Notes",
  CellTags->"SubwordCounterDFA"],

Cell[TextData[{
  "The number of states in the DFA is bounded by ",
  Cell[BoxData[
      \(TraditionalForm\`p\^\(\(\ \)\(\(|\)\(w\)\(|\)\)\)\)]],
  " for counters and by  ",
  Cell[BoxData[
      \(TraditionalForm\`\((p + 2)\)\^\(\(|\)\(w\)\(|\)\)\)]],
  "  for modular counters. "
}], "Notes",
  CellTags->"SubwordCounterDFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["SubwordDFA",
    ButtonData:>"SubwordDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SubwordCounterDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SubwordCounterDFA"],

Cell["\<\
A subword counter automaton and a trace of a computation of the \
machine.\
\>", "ExampleText",
  CellTags->"SubwordCounterDFA"],

Cell[BoxData[{
    \(\(m = SubwordCounterDFA[abab, 3, 2, Normalize \[Rule] 1];\)\), "\n", 
    \(Size[m]\), "\n", 
    \(States[m]\)}], "Input",
  CellLabel->"In[774]:=",
  AspectRatioFixed->True,
  CellTags->"SubwordCounterDFA"],

Cell[BoxData[{
    \(\(w = bbaababbbb;\)\), "\n", 
    \(TableForm[AcceptQFA[m, w, DoTrace \[Rule] True]]\)}], "Input",
  CellLabel->"In[84]:=",
  AspectRatioFixed->True,
  CellTags->"SubwordCounterDFA"],

Cell[TextData[{
  "A machine for all words containing a multiple of 3 occurrences of ",
  Cell[BoxData[
      \(TraditionalForm\`a\[VeryThinSpace]b\)]],
  "."
}], "ExampleText",
  CellTags->"SubwordCounterDFA"],

Cell[BoxData[
    \(m = SubwordCounterDFA[ab, 0, \ 2, \ Modulus \[Rule] 3\ ]\)], "Input",
  CellLabel->"In[83]:=",
  CellTags->"SubwordCounterDFA"],

Cell[BoxData[
    \(LanguageFA[m, 5]\)], "Input",
  CellLabel->"In[74]:=",
  CellTags->"SubwordCounterDFA"],

Cell[BoxData[
    \(WordBinomial[%, ab]\)], "Input",
  CellLabel->"In[75]:=",
  CellTags->"SubwordCounterDFA"],

Cell["SubwordDFA", "ObjectName",
  CellTags->"SubwordDFA"],

Cell["Function", "Highlight",
  CellTags->"SubwordDFA"],

Cell[TextData[{
  StyleBox["SubwordDFA[w,K]", "MR"],
  "  constructs a DFA that accepts all words that contain subword ",
  StyleBox["w", "MR"],
  "  over alphabet ",
  StyleBox["K", "MR"],
  "."
}], "Usage",
  CellTags->"SubwordDFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "1"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"SubwordDFA"],

Cell[TextData[{
  "The structured state set consists of the prefixes of ",
  StyleBox["w", "MR"],
  "."
}], "Notes",
  CellTags->"SubwordDFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["SubwordCounterDFA",
    ButtonData:>"SubwordCounterDFA",
    ButtonStyle->"AddOnsLink"],
  " ."
}], "Notes",
  CellTags->"SubwordDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SubwordDFA"],

Cell["A DFA for all words containing the subword abb. ", "ExampleText",
  CellTags->"SubwordDFA"],

Cell[BoxData[
    \(m = SubwordDFA[abb, {a, b}, Normalize \[Rule] 1]\)], "Input",
  CellLabel->"In[80]:=",
  CellTags->"SubwordDFA"],

Cell[BoxData[
    \(States[m]\)], "Input",
  CellLabel->"In[93]:=",
  CellTags->"SubwordDFA"],

Cell[BoxData[
    \(LanguageFA[m, \(-5\)]\)], "Input",
  CellLabel->"In[89]:=",
  CellTags->"SubwordDFA"],

Cell[BoxData[
    \(WordBinomial[Flatten[%], abb]\)], "Input",
  CellLabel->"In[90]:=",
  CellTags->"SubwordDFA"],

Cell["SumFA", "ObjectName",
  CellTags->"SumFA"],

Cell["Function", "Highlight",
  CellTags->"SumFA"],

Cell[TextData[{
  StyleBox["SumFA[M1,M2]", "MR"],
  "  computes the disjoint sum of two machines ",
  StyleBox["M1", "MR"],
  " and ",
  StyleBox["M2", "MR"],
  ". "
}], "Usage",
  CellTags->"SumFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "1"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"SumFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Normalize->1", "MR"],
  " preserves the state set as a disjoint union."
}], "Notes",
  CellTags->"SumFA"],

Cell[TextData[{
  StyleBox["SumFA", "MR"],
  " is polyadic but note that the structured states are nested pairs; they \
are not flattened out to tuples."
}], "Notes",
  CellTags->"SumFA"],

Cell["\<\
Has linear time complexity even for DFAs (but produces a \
nondeterministic machine as output). \
\>", "Notes",
  CellTags->"SumFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["UnionFA",
    ButtonData:>"UnionFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ProductFA",
    ButtonData:>"ProductFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["IntersectionFA",
    ButtonData:>"IntersectionFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SumFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SumFA"],

Cell["\<\
The disjoint union construction produces an FA even if the input \
machines are both DFAs.\
\>", "ExampleText",
  CellTags->"SumFA"],

Cell["\<\
m1 = BuildDFA[aaa,2]
m2 = BuildDFA[bbb,2]
m = SumFA[ m1, m2 ]\
\>", "Input",
  CellLabel->"In[94]:=",
  CellTags->"SumFA"],

Cell[BoxData[
    \(LanguageFA[m, \(-5\)]\)], "Input",
  CellLabel->"In[62]:=",
  CellTags->"SumFA"],

Cell[TextData[{
  StyleBox["UnionFA", "MR"],
  " will produce a DFA for the union of the languages."
}], "ExampleText",
  CellTags->"SumFA"],

Cell[BoxData[
    \(mm\ \  = \ UnionFA[m1, m2]\)], "Input",
  CellLabel->"In[65]:=",
  CellTags->"SumFA"],

Cell[BoxData[
    \(EquivalentQFA[m, mm]\)], "Input",
  CellLabel->"In[66]:=",
  CellTags->"SumFA"],

Cell["SwitchTransitionFA", "ObjectName",
  CellTags->"SwitchTransitionFA"],

Cell["Function", "Highlight",
  CellTags->"SwitchTransitionFA"],

Cell[TextData[{
  StyleBox["SwitchTransitionFA[M,p,q]", "MR"],
  "  switches the transition ",
  StyleBox["{p,w,q}", "MR"],
  " in ",
  StyleBox["M", "MR"],
  " to ",
  StyleBox["{p,w+1,q}", "MR"],
  ". "
}], "Usage",
  CellTags->"SwitchTransitionFA"],

Cell[TextData[{
  "Here ",
  StyleBox["w+1", "MR"],
  " denotes the successor of ",
  StyleBox["w", "MR"],
  " in the alphabet of the machine. "
}], "Notes",
  CellTags->"SwitchTransitionFA"],

Cell[TextData[{
  "Can also be used in the form ",
  StyleBox["SwitchTransitionFA[M,{p,q}] ", "MR"],
  "or ",
  StyleBox["SwitchTransitionFA[M,L]", "MR"],
  " where L is a list of pairs of states. "
}], "Notes",
  CellTags->"SwitchTransitionFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["AddTransitionFA",
    ButtonData:>"AddTransitionFA",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["DeleteTransitionFA",
    ButtonData:>"DeleteTransitionFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SwitchTransitionFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SwitchTransitionFA"],

Cell["\<\
A permutation automaton and a 1-permutation automaton. The latter \
demonstrates exponential blow-up during deterministic simulation.\
\>", \
"ExampleText",
  CellTags->"SwitchTransitionFA"],

Cell[BoxData[
    \(\(m = 
        SA[4, 2, {{1, 1, 1}, {1, 2, 2}, {2, 1, 3}, {2, 2, 4}, {3, 2, 1}, {3, 
              1, 2}, {4, 2, 3}, {4, 1, 4}}];\)\)], "Input",
  CellLabel->"In[99]:=",
  CellTags->"SwitchTransitionFA"],

Cell[BoxData[
    \(m1\  = \ SwitchTransitionFA[m, 1, 1]\)], "Input",
  CellLabel->"In[100]:=",
  CellTags->"SwitchTransitionFA"],

Cell[BoxData[
    \(ToDFA[m1]\  // \ Size\)], "Input",
  CellLabel->"In[125]:=",
  CellTags->"SwitchTransitionFA"],

Cell["TheInitial", "ObjectName",
  CellTags->"TheInitial"],

Cell["Function", "Highlight",
  CellTags->"TheInitial"],

Cell[TextData[{
  StyleBox["TheInitial[M]", "MR"],
  "  returns the initial state of a DFA  ",
  StyleBox["M", "MR"],
  "  in positional notation."
}], "Usage",
  CellTags->"TheInitial"],

Cell[TextData[{
  "Unlike ",
  ButtonBox["Initial",
    ButtonData:>"Initial",
    ButtonStyle->"AddOnsLink"],
  ", this returns an integer."
}], "Notes",
  CellTags->"TheInitial"],

Cell["Further Examples", "ExampleSection",
  CellTags->"TheInitial"],

Cell["\<\
Most algorithms in the package produce DFAs whose initial state is \
1.\
\>", "ExampleText",
  CellTags->"TheInitial"],

Cell["m = CounterDFA[b,2,3,Modulus->3];", "Input",
  CellLabel->"In[706]:=",
  CellTags->"TheInitial"],

Cell[BoxData[{
    \(TheInitial[m]\), "\n", 
    \(Initial[m]\)}], "Input",
  CellLabel->"In[707]:=",
  CellTags->"TheInitial"],

Cell["Thompson", "ObjectName",
  CellTags->"Thompson"],

Cell["Option", "Highlight",
  CellTags->"Thompson"],

Cell[TextData[{
  StyleBox["Thompson", "MR"],
  "  is an option in ",
  StyleBox["RegularExpressionToFA", "MR"],
  ". "
}], "Usage",
  CellTags->"Thompson"],

Cell[TextData[{
  "By default, ",
  StyleBox["Thompson", "MR"],
  " has value ",
  StyleBox["False", "MR"],
  " and a McNaughton-Yamada type machine is used in the conversion."
}], "Notes",
  CellTags->"Thompson"],

Cell[TextData[{
  "See also ",
  ButtonBox["RegularExpressionToFA",
    ButtonData:>"RegularExpressionToFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Thompson"],

Cell["ToCharacters", "ObjectName",
  CellTags->"ToCharacters"],

Cell["Function", "Highlight",
  CellTags->"ToCharacters"],

Cell[TextData[{
  StyleBox["ToCharacters[w]", "MR"],
  "  converts symbols and strings to a list of atomic characters "
}], "Usage",
  CellTags->"ToCharacters"],

Cell[TextData[{
  "Similar to the built-in command ",
  StyleBox["Characters", "MR"],
  " (which works for strings only). "
}], "Notes",
  CellTags->"ToCharacters"],

Cell["Behaves properly with respect to epsilon.", "Notes",
  CellTags->"ToCharacters"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToCharacters"],

Cell["\<\
Can be directly applied to symbols, without stringifying \
first.\
\>", "ExampleText",
  CellTags->"ToCharacters"],

Cell["\<\
ToCharacters[aabb]
% // FullForm\
\>", "Input",
  CellLabel->"In[114]:=",
  CellTags->"ToCharacters"],

Cell[BoxData[
    \(ToCharacters[\[Epsilon]]\)], "Input",
  CellLabel->"In[118]:=",
  CellTags->"ToCharacters"],

Cell["ToDFA", "ObjectName",
  CellTags->"ToDFA"],

Cell["Function", "Highlight",
  CellTags->"ToDFA"],

Cell[TextData[{
  StyleBox["ToDFA[M]", "MR"],
  "  converts a finite state machine into an equivalent DFA. "
}], "Usage",
  CellTags->"ToDFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"},
          {"SizeOnly", "\[Rule]", "False", "-"},
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"},
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ToDFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Normalize->1", "MR"],
  " preserves states as elements of the power set of ",
  Cell[BoxData[
      \(TraditionalForm\`\([n]\)\)]],
  "."
}], "Notes",
  CellTags->"ToDFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Normalize->2", "MR"],
  " preserves states as elements of the power set of the state set of ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  ". "
}], "Notes",
  CellTags->"ToDFA"],

Cell["\<\
Note the in particular the last option may increase the size of the \
date structure representing the automaton considerably (internally as well as \
externally). \
\>", "Notes",
  CellTags->"ToDFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  " causes the whole power automaton of size ",
  Cell[BoxData[
      \(TraditionalForm\`2\^\(\(\ \)\(n\)\)\)]],
  " to be constructed. Only available in the internal implementation."
}], "Notes",
  CellTags->"ToDFA"],

Cell[TextData[{
  "Option ",
  StyleBox["SizeOnly->True", "MR"],
  " returns only the size of the power automaton. "
}], "Notes",
  CellTags->"ToDFA"],

Cell["\<\
The algorithm uses the Rabin-Scott subset construction and has \
potentially exponential time and space complexity.\
\>", "Notes",
  CellTags->"ToDFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToSA",
    ButtonData:>"ToSA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToFA",
    ButtonData:>"ToFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToNFA",
    ButtonData:>"ToNFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToDFA"],

Cell["\<\
A nondeterministic machine and the corresponding DFA. Note the \
exponential increase in size. \
\>", "ExampleText",
  CellTags->"ToDFA"],

Cell[BoxData[{
    \(m = IthSymbolFA[a, \(-9\)]\), "\n", 
    \(mm\  = \ ToDFA[\ m\ ]\)}], "Input",
  CellLabel->"In[133]:=",
  CellTags->"ToDFA"],

Cell[BoxData[
    \(EquivalentQFA[m, mm]\)], "Input",
  CellLabel->"In[176]:=",
  CellTags->"ToDFA"],

Cell[TextData[{
  "We can preserve the structure of the state set of the DFA.  In this case, \
the states are the elements of the power set of ",
  Cell[BoxData[
      \(TraditionalForm\`\([4]\)\)]],
  " that contain 1."
}], "ExampleText",
  CellTags->"ToDFA"],

Cell[BoxData[{
    \(m = IthSymbolFA[a, \(-3\)]\), "\n", 
    \(\(mm\  = \ ToDFA[\ m, \ Normalize \[Rule] 1\ ];\)\), "\n", 
    \(States[mm]\)}], "Input",
  CellLabel->"In[73]:=",
  CellTags->"ToDFA"],

Cell["ToEquationsDFA", "ObjectName",
  CellTags->"ToEquationsDFA"],

Cell["Function", "Highlight",
  CellTags->"ToEquationsDFA"],

Cell[TextData[{
  StyleBox["ToEquationsDFA[M]", "MR"],
  "  converts DFA ",
  StyleBox["M", "MR"],
  "  into a system of equations."
}], "Usage",
  CellTags->"ToEquationsDFA"],

Cell["\<\
The equations are to be interpreted over the language semiring with \
operations union, concatenation and Kleene star.\
\>", "Notes",
  CellTags->"ToEquationsDFA"],

Cell[TextData[{
  "See ",
  ButtonBox["ArdenSolve",
    ButtonData:>"ArdenSolve",
    ButtonStyle->"AddOnsLink"],
  "  and  ",
  ButtonBox["SubstituteRE",
    ButtonData:>"SubstituteRE",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToEquationsDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToEquationsDFA"],

Cell[TextData[{
  "A counter machine and the corresponding system of equations. Note that \
formatting of ",
  Cell[BoxData[
      \(TraditionalForm\`X\)]],
  " is not turned on by default."
}], "ExampleText",
  CellTags->"ToEquationsDFA"],

Cell[" m = CounterDFA[a,1,3,Modulus->3]", "Input",
  CellLabel->"In[168]:=",
  CellTags->"ToEquationsDFA"],

Cell[BoxData[{
    \(FormatX[]\), "\n", 
    \(ToEquationsDFA[m]\  // \ TableForm\)}], "Input",
  CellLabel->"In[170]:=",
  CellTags->"ToEquationsDFA"],

Cell["The system in algebraic form.", "ExampleText",
  CellTags->"ToEquationsDFA"],

Cell[BoxData[
    \(ToEquationsDFA[m]\  /. \ {REP \[Rule] Plus, 
          RET \[Rule] NonCommutativeMultiply} // \ TableForm\)], "Input",
  CellLabel->"In[172]:=",
  CellTags->"ToEquationsDFA"],

Cell["ToFA", "ObjectName",
  CellTags->"ToFA"],

Cell["Function", "Highlight",
  CellTags->"ToFA"],

Cell[TextData[{
  StyleBox["ToFA[M]", "MR"],
  "  converts a FSM  ",
  StyleBox["M", "MR"],
  "  to an equivalent FA. "
}], "Usage",
  CellTags->"ToFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ToFA"],

Cell[TextData[{
  "If ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  " is a semiautomaton all states will be initial as well as final. "
}], "Notes",
  CellTags->"ToFA"],

Cell["\<\
Also converts a cellular automaton to the corresponding de Bruijn \
automaton.\
\>", "Notes",
  CellTags->"ToFA"],

Cell[TextData[{
  "See also   ",
  ButtonBox["FA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToSA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToNFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToFA"],

Cell["Converting a DFA to an FA.", "ExampleText",
  CellTags->"ToFA"],

Cell[BoxData[
    \(\(m = 
        DFA[8, 2, {{1, 3, 5, 7, 1, 3, 5, 7}, {2, 4, 6, 8, 2, 4, 6, 8}}, 
          1, {5, 6, 7, 8}];\)\)], "Input",
  CellLabel->"In[92]:=",
  CellTags->"ToFA"],

Cell["ToFA[m]", "Input",
  CellLabel->"In[93]:=",
  CellTags->"ToFA"],

Cell[BoxData[
    \(DeterministicQFA[%]\)], "Input",
  CellLabel->"In[94]:=",
  CellTags->"ToFA"],

Cell["Converting a cellular automaton.", "ExampleText",
  CellTags->"ToFA"],

Cell[BoxData[
    \(ToFA[CA[110]]\)], "Input",
  CellLabel->"In[95]:=",
  CellTags->"ToFA"],

Cell["ToFactorFA", "ObjectName",
  CellTags->"ToFactorFA"],

Cell["Function", "Highlight",
  CellTags->"ToFactorFA"],

Cell[TextData[{
  StyleBox["ToFactorFA[M]", "MR"],
  "  converts a FSM ",
  StyleBox["M", "MR"],
  "  into a machine that accepts the language of all factors of words in the \
language of ",
  StyleBox["M", "MR"],
  "."
}], "Usage",
  CellTags->"ToFactorFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[Rule]", "0", "-"},
          {"Type", "\[Rule]", 
            StyleBox["\<\"\\\"Factor\\\"\"\>",
              "MR"], "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ToFactorFA"],

Cell[TextData[{
  "Option  ",
  StyleBox["Type", "MR"],
  "  controls the type of factor considered.   The possible values for ",
  StyleBox["Type", "MR"],
  " are:\n\t",
  StyleBox["\"Factor\"", "MR"],
  ",    ",
  StyleBox["\"NoFactor\"", "MR"],
  "\n\t",
  StyleBox["\"Prefix\", \"NoPrefix\"", "MR"],
  "\n\t",
  StyleBox["\"Suffix\", \"NoSuffix\"", "MR"],
  "\nNote that all option values are strings, not symbols."
}], "Notes",
  CellTags->"ToFactorFA"],

Cell[TextData[{
  "In the plain versions, a machine is constructed that accepts all words ",
  Cell[BoxData[
      \(TraditionalForm\`w\)]],
  " that are a factor (prefix, suffix) of some word in ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptCapitalL](M)\)]],
  "."
}], "Notes",
  CellTags->"ToFactorFA"],

Cell[TextData[{
  "In the exclusion versions, a machine is constructed that accepts all words \
 ",
  Cell[BoxData[
      \(TraditionalForm\`w\  \[Element] \ \[ScriptCapitalL](M)\)]],
  "  such than no proper factor (prefix, suffix) of ",
  Cell[BoxData[
      \(TraditionalForm\`w\)]],
  " lies in ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptCapitalL](M)\)]],
  "."
}], "Notes",
  CellTags->"ToFactorFA"],

Cell[TextData[{
  "Option  ",
  StyleBox["Type->\"NoFactor\"", "MR"],
  "  in particular constructs a machine for the minimal elements of ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptCapitalL](M)\)]],
  " with respect to the factor order,  see ",
  ButtonBox["WordFactorQ",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToFactorFA"],

Cell["\<\
The operations excluding factors of certain types may cause \
exponential blow-up in size. The others are linear.\
\>", "Notes",
  CellTags->"ToFactorFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordToFA",
    ButtonData:>"WordToFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["FrontierQ",
    ButtonData:>"FrontierQ",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["IdealBaseFA",
    ButtonData:>"IdealBaseFA",
    ButtonStyle->"AddOnsLink"],
  ". "
}], "Notes",
  CellTags->"ToFactorFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToFactorFA"],

Cell[TextData[{
  "Construct a machine accepting all words of the form  ",
  Cell[BoxData[
      \(TraditionalForm\`a\ \(b\^*\)\ \((a + c)\) \(b\^*\) c\)]],
  "  and corresponding machines for all prefixes, suffixes and factors of \
this language."
}], "ExampleText",
  CellTags->"ToFactorFA"],

Cell[BoxData[
    \(m\  = \ 
      RegularExpressionToFA[RET[\ a, RES[b], Plus[a, c], RES[b], c\ ], 
          3\ ]\  // \ MinimizeFA\)], "Input",
  CellLabel->"In[343]:=",
  CellTags->"ToFactorFA"],

Cell[BoxData[{
    \(mp\  = \ ToFactorFA[m, Type \[Rule] "\<Prefix\>"]\), "\n", 
    \(ms\  = \ ToFactorFA[m, Type \[Rule] "\<Suffix\>"]\), "\n", 
    \(mf\  = \ ToFactorFA[m, Type \[Rule] "\<Factor\>"]\)}], "Input",
  CellLabel->"In[344]:=",
  CellTags->"ToFactorFA"],

Cell["\<\
To see the effect of these operations generate some accepted words.\
\
\>", "ExampleText",
  CellTags->"ToFactorFA"],

Cell[BoxData[{
    \(LanguageFA[\ m, \ \(-4\)\ ]\), "\n", 
    \(LanguageFA[\ mp, \ \(-4\)\ ]\), "\n", 
    \(LanguageFA[\ ms, \ \(-4\)\ ]\), "\n", 
    \(LanguageFA[\ mf, \ \(-4\)\ ]\)}], "Input",
  CellLabel->"In[351]:=",
  CellTags->"ToFactorFA"],

Cell[TextData[{
  "An example for exluded factors.  The base language consists of all strings \
containing exactly 3 ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  "'s."
}], "ExampleText",
  CellTags->"ToFactorFA"],

Cell[BoxData[{
    \(m\  = \ CounterDFA[a, 3, 2]\), "\n", 
    \(mp\  = \ ToFactorFA[m, Type \[Rule] "\<NoPrefix\>"]\), "\n", 
    \(ms\  = \ ToFactorFA[m, Type \[Rule] "\<NoSuffix\>"]\), "\n", 
    \(mf\  = \ ToFactorFA[m, Type \[Rule] "\<NoFactor\>"]\)}], "Input",
  CellLabel->"In[355]:=",
  CellTags->"ToFactorFA"],

Cell["We compare accepted words of length up to 5.", "ExampleText",
  CellTags->"ToFactorFA"],

Cell[BoxData[{
    \(LanguageFA[\ m, \ \(-5\)\ ]\), "\n", 
    \(LanguageFA[\ mp, \ \(-5\)\ ]\), "\n", 
    \(LanguageFA[\ ms, \ \(-5\)\ ]\), "\n", 
    \(LanguageFA[\ mf, \ \(-5\)\ ]\)}], "Input",
  CellLabel->"In[359]:=",
  CellTags->"ToFactorFA"],

Cell["\<\
For the no-factor automaton it is easy to construct a reference \
machine and verify that they are equivalent.\
\>", "ExampleText",
  CellTags->"ToFactorFA"],

Cell[BoxData[{
    \(\(mref\  = \ 
        IntersectionFA[\[IndentingNewLine]RegularExpressionToFA[
            RET[\ \ a, \ RES[REP[a, b]], a\ ], 3\ ], 
          CounterDFA[a, 3, 2]];\)\), "\n", 
    \(EquivalentQFA[\ mf, \ mref\ ]\)}], "Input",
  CellLabel->"In[363]:=",
  CellTags->"ToFactorFA"],

Cell["ToFrontierDFA", "ObjectName",
  CellTags->"ToFrontierDFA"],

Cell["Function", "Highlight",
  CellTags->"ToFrontierDFA"],

Cell[TextData[{
  StyleBox["ToFrontierDFA[M]", "MR"],
  "  converts DFA ",
  StyleBox["M", "MR"],
  "  into the corresponding frontier automaton ",
  StyleBox["(g,F)", "MR"],
  "."
}], "Usage",
  CellTags->"ToFrontierDFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Periodic", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ToFrontierDFA"],

Cell[TextData[{
  "The frontier rewrite system ",
  StyleBox["g", "MR"],
  " is a list of word pairs, ",
  StyleBox["F", "MR"],
  " a list of words."
}], "Notes",
  CellTags->"ToFrontierDFA"],

Cell[TextData[{
  "With option ",
  StyleBox["Periodic->True", "MR"],
  " returns a list of rewrite rules ",
  Cell[BoxData[
      \(TraditionalForm\`\((u, v)\)\)]],
  " where ",
  Cell[BoxData[
      \(TraditionalForm\`v\)]],
  " is a prefix of ",
  Cell[BoxData[
      \(TraditionalForm\`u\)]],
  ", and a list of words corresponding to the final states of the automaton."
}], "Notes",
  CellTags->"ToFrontierDFA"],

Cell[TextData[{
  " See also ",
  ButtonBox["FrontierToDFA",
    ButtonStyle->"AddOnsLink"],
  "  and  ",
  ButtonBox["FrontierQ",
    ButtonData:>"FrontierQ",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToFrontierDFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToFrontierDFA"],

Cell[TextData[{
  "We start with a DFA for all words of the form ",
  Cell[BoxData[
      \(TraditionalForm\`\(\((a + b)\)\^*\) b\ \((a + b)\)\)]],
  ".  The plain frontier rewrite system looks like so."
}], "ExampleText",
  CellTags->"ToFrontierDFA"],

Cell[BoxData[
    \(\(m = DFA[4, 2, {{1, 3, 1, 3}, {2, 4, 2, 4}}, 1, {3, 4}];\)\)], "Input",\

  CellLabel->"In[102]:=",
  CellTags->"ToFrontierDFA"],

Cell[BoxData[{
    \(\({\[CapitalGamma], F}\  = \ ToFrontierDFA[m];\)\), "\n", 
    \(PrintEquations[\[CapitalGamma]]\), "\n", 
    \(F\)}], "Input",
  CellLabel->"In[103]:=",
  AspectRatioFixed->True,
  CellTags->"ToFrontierDFA"],

Cell["\<\
In the 4th rule, the right hand side is not a prefix of the left \
hand side.  Here is the corresponding periodic rewrite system.\
\>", \
"ExampleText",
  CellTags->"ToFrontierDFA"],

Cell[BoxData[{
    \(\({\[CapitalGamma], F} = 
        ToFrontierDFA[m, Periodic \[Rule] True];\)\), "\n", 
    \(PrintEquations[\[CapitalGamma]]\), "\n", 
    \(F\)}], "Input",
  CellLabel->"In[76]:=",
  AspectRatioFixed->True,
  CellTags->"ToFrontierDFA"],

Cell[BoxData[{
    \(\({\[CapitalGamma], F} = 
        ToFrontierDFA[m, Periodic \[Rule] True];\)\), "\n", 
    \(PrintEquations[\[CapitalGamma]]\), "\n", 
    \(F\)}], "Input",
  CellLabel->"In[82]:=",
  AspectRatioFixed->True,
  CellTags->"ToFrontierDFA"],

Cell["\<\
We can translate back to a DFA, and check equivalence with the \
original machine. \
\>", "ExampleText",
  CellTags->"ToFrontierDFA"],

Cell[BoxData[{
    \(\(mm\  = \ FrontierToDFA[\[CapitalGamma], F];\)\), "\n", 
    \(EquivalentQFA[m, mm]\)}], "Input",
  CellLabel->"In[85]:=",
  CellTags->"ToFrontierDFA"],

Cell["ToGeneratorsFA", "ObjectName",
  CellTags->"ToGeneratorsFA"],

Cell["Function", "Highlight",
  CellTags->"ToGeneratorsFA"],

Cell[TextData[{
  StyleBox["ToGeneratorsFA[M]", "MR"],
  "  converts the transition matrix of a DFA into a list of generators for \
the transition semigroup of the machine."
}], "Usage",
  CellTags->"ToGeneratorsFA"],

Cell["\<\
Also works on other types of machines provided they are \
deterministic. Does not check.\
\>", "Notes",
  CellTags->"ToGeneratorsFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  "  and ",
  ButtonBox["SyntacticSG",
    ButtonData:>"SyntacticSG",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToGeneratorsFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToGeneratorsFA"],

Cell["A DFA and the corresponding generators.", "ExampleText",
  CellTags->"ToGeneratorsFA"],

Cell[BoxData[
    \(\(m = 
        DFA[8, 2, {{1, 3, 5, 7, 1, 3, 5, 7}, {2, 4, 6, 8, 2, 4, 6, 8}}, 
          1, {5, 6, 7, 8}];\)\)], "Input",
  CellLabel->"In[15]:=",
  CellTags->"ToGeneratorsFA"],

Cell[BoxData[
    \(ToGeneratorsFA[m]\)], "Input",
  CellLabel->"In[16]:=",
  CellTags->"ToGeneratorsFA"],

Cell["Converting a deterministic semiautomaton.", "ExampleText",
  CellTags->"ToGeneratorsFA"],

Cell[BoxData[{
    \(sa\  = \ ToSA[CA[90]]\), "\n", 
    \(ToGeneratorsFA[sa]\)}], "Input",
  CellLabel->"In[19]:=",
  CellTags->"ToGeneratorsFA"],

Cell[BoxData[
    \(GenerateSG[%]\)], "Input",
  CellLabel->"In[21]:=",
  CellTags->"ToGeneratorsFA"],

Cell["ToGraphFA", "ObjectName",
  CellTags->"ToGraphFA"],

Cell["Function", "Highlight",
  CellTags->"ToGraphFA"],

Cell[TextData[{
  StyleBox["ToGraphFA[M]", "MR"],
  "  converts a finites state machine M into the underlying transition \
diagram: a directed graph with point set Q. "
}], "Usage",
  CellTags->"ToGraphFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Unlabeled", "\[Rule]", "True", "False"},
          {"Normalize", "\[Rule]", "$defaultnormalize", \(0, 1, 2\)}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ToGraphFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Unlabeled->False", "MR"],
  " generates a labeled edge list."
}], "Notes",
  CellTags->"ToGraphFA"],

Cell[TextData[{
  "This is used in many algorithms such as ",
  ButtonBox["AccessibleFA",
    ButtonData:>"AccessibleFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToGraphFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToGraphFA"],

Cell["A small semiautomaton and its diagram.", "ExampleText",
  CellTags->"ToGraphFA"],

Cell[BoxData[
    \(\(m = 
        SA[4, 2, {{1, 1, 1}, {1, 2, 2}, {2, 2, 3}, {2, 2, 4}, {3, 1, 1}, {3, 
              2, 2}, {4, 2, 3}, {4, 1, 4}}];\)\)], "Input",
  CellLabel->"In[148]:=",
  CellTags->"ToGraphFA"],

Cell[BoxData[
    \(g\  = \ ToGraphFA[m]\)], "Input",
  CellLabel->"In[149]:=",
  CellTags->"ToGraphFA"],

Cell["The automaton is transitive.", "ExampleText",
  CellTags->"ToGraphFA"],

Cell[BoxData[
    \(StronglyConnectedComponents[g]\)], "Input",
  CellLabel->"In[92]:=",
  CellTags->"ToGraphFA"],

Cell["ToKernelFA", "ObjectName",
  CellTags->"ToKernelFA"],

Cell["Function", "Highlight",
  CellTags->"ToKernelFA"],

Cell[TextData[{
  StyleBox["ToKernelFA[M,L,opts]", "MR"],
  "  generates the kernel automaton of ",
  StyleBox["M", "MR"],
  "  generated from the states in list ",
  StyleBox["L", "MR"],
  ". "
}], "Usage",
  CellTags->"ToKernelFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[Rule]", "1", "-"},
          {"Full", "\[Rule]", "False", "-"},
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ToKernelFA"],

Cell[TextData[{
  "The entries in ",
  StyleBox["L", "MR"],
  " can be single integers in ",
  Cell[BoxData[
      \(TraditionalForm\`\([n]\)\)]],
  ", where ",
  StyleBox["n", "MR"],
  " is the size of ",
  StyleBox["M", "MR"],
  ", or lists of such integers. "
}], "Notes",
  CellTags->"ToKernelFA"],

Cell[TextData[{
  "The default for ",
  StyleBox["L", "MR"],
  "  is ",
  Cell[BoxData[
      \(TraditionalForm\`{{p\_1}, \[Ellipsis], {p\_m}}\)]],
  " where the ",
  Cell[BoxData[
      \(TraditionalForm\`p\_i\)]],
  " are the initial states of ",
  StyleBox["M", "MR"],
  ". "
}], "Notes",
  CellTags->"ToKernelFA"],

Cell[TextData[{
  "Option  ",
  StyleBox["Normalize->1", "MR"],
  "  controls the state set of the kernel automaton."
}], "Notes",
  CellTags->"ToKernelFA"],

Cell[TextData[{
  "Option  ",
  StyleBox["Full->True", "MR"],
  "  returns a pair  ",
  StyleBox["{m,L}", "MR"],
  "  where ",
  StyleBox["m", "MR"],
  " is the kernel automaton and ",
  StyleBox["L", "MR"],
  " a list of the length of the minimal witnesses for the states in ",
  StyleBox["m", "MR"],
  ". "
}], "Notes",
  CellTags->"ToKernelFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToDFA",
    ButtonData:>"ToDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToKernelFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToKernelFA"],

Cell["\<\
A small semiautomaton and several associated kernel automata.\
\>", \
"ExampleText",
  CellTags->"ToKernelFA"],

Cell[BoxData[
    \(\(m = 
        SA[4, \(-2\), {{1, 1, 1}, {1, 2, 2}, {2, 2, 3}, {2, 2, 4}, {3, 1, 
              1}, {3, 2, 2}, {4, 2, 3}, {4, 1, 4}}];\)\)], "Input",
  CellLabel->"In[492]:=",
  CellTags->"ToKernelFA"],

Cell[BoxData[{
    \(ToKernelFA[m]\), "\n", 
    \(States[%]\)}], "Input",
  CellLabel->"In[493]:=",
  CellTags->"ToKernelFA"],

Cell["\<\
It takes at most 3 steps to reach any of the states in the kernel \
automaton from the 4 starting points.\
\>", "ExampleText",
  CellTags->"ToKernelFA"],

Cell[BoxData[
    \(ToKernelFA[m, Full \[Rule] True]\  // \ Last\)], "Input",
  CellLabel->"In[496]:=",
  CellTags->"ToKernelFA"],

Cell["With only one generator  {1}  we obtain a smaller automaton.", \
"ExampleText",
  CellTags->"ToKernelFA"],

Cell[BoxData[{
    \(ToKernelFA[m, {1}]\), "\n", 
    \(States[%]\)}], "Input",
  CellLabel->"In[497]:=",
  CellTags->"ToKernelFA"],

Cell[TextData[{
  "The automaton that would essentially be produced by ",
  StyleBox["ToDFA", "MR"],
  ".  This time it takes up to 5 steps to reach some of the states."
}], "ExampleText",
  CellTags->"ToKernelFA"],

Cell[BoxData[{
    \(\({mm, L}\  = \ 
        ToKernelFA[m, {{1, 2, 3, 4}}, Full \[Rule] True];\)\), "\n", 
    \(States[mm]\), "\n", 
    \(L\)}], "Input",
  CellLabel->"In[499]:=",
  CellTags->"ToKernelFA"],

Cell["ToMatrix", "ObjectName",
  CellTags->"ToMatrix"],

Cell["Option", "Highlight",
  CellTags->"ToMatrix"],

Cell[TextData[{
  StyleBox["ToMatrix", "MR"],
  "  is an option for ",
  StyleBox["ConvertToFA", "MR"],
  ". "
}], "Usage",
  CellTags->"ToMatrix"],

Cell["\<\
Translates a list of transitions into a transition matrix, as \
needed for DFAs and NFAs.\
\>", "Notes",
  CellTags->"ToMatrix"],

Cell[TextData[{
  "See also ",
  ButtonBox["ConvertToFA",
    ButtonData:>"ConvertToFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToMatrix"],

Cell["ToNFA", "ObjectName",
  CellTags->"ToNFA"],

Cell["Function", "Highlight",
  CellTags->"ToNFA"],

Cell[TextData[{
  StyleBox["ToNFA[M]", "MR"],
  "  converts FSM ",
  StyleBox["M", "MR"],
  " to an equivalent NFA."
}], "Usage",
  CellTags->"ToNFA"],

Cell["\<\
 If M is a semiautomaton all states will be initial as well as \
final.\
\>", "Notes",
  CellTags->"ToNFA"],

Cell[TextData[{
  " See also ",
  ButtonBox["ToSA",
    ButtonData:>"ToSA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToFA",
    ButtonData:>"ToFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToDFA",
    ButtonData:>"ToDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToNFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToNFA"],

Cell["A semiautomaton and the corresponding NFA.", "ExampleText",
  CellTags->"ToNFA"],

Cell[BoxData[
    \(\(m\  = \ 
        SA[4, 2, {{1, 1, 1}, {1, 2, 2}, {2, 2, 3}, {2, 2, 4}, {3, 1, 1}, {3, 
              2, 2}, {4, 2, 3}, {4, 1, 4}}];\)\)], "Input",
  CellLabel->"In[160]:=",
  CellTags->"ToNFA"],

Cell[BoxData[
    \(ToNFA[m]\)], "Input",
  CellLabel->"In[161]:=",
  CellTags->"ToNFA"],

Cell[BoxData[
    \(Transitions[%]\  // \ MatrixForm\)], "Input",
  CellLabel->"In[162]:=",
  CellTags->"ToNFA"],

Cell["ToRegularExpressionFA", "ObjectName",
  CellTags->"ToRegularExpressionFA"],

Cell["Function", "Highlight",
  CellTags->"ToRegularExpressionFA"],

Cell[TextData[{
  StyleBox["ToRegularExpressionFA[M]", "MR"],
  "  uses Kleene's algorithm to convert an FA into an equivalent regular \
expression. "
}], "Usage",
  CellTags->"ToRegularExpressionFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"RegExpSimp", "\[Rule]", "True", "-"},
          {"DoTrace", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ToRegularExpressionFA"],

Cell[TextData[{
  "Option ",
  StyleBox["RegExpSimp->False", "MR"],
  " turns the built-in simplification rules off.  This will ordinarily lead \
to very large expressions."
}], "Notes",
  CellTags->"ToRegularExpressionFA"],

Cell["\<\
Since simplification is very primitive, use only on small machines.\
\
\>", "Notes",
  CellTags->"ToRegularExpressionFA"],

Cell[TextData[{
  "See ",
  ButtonBox["$regexrules",
    ButtonData:>"$regexrules",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["RegExpSimp",
    ButtonData:>"RegExpSimp",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToRegularExpressionFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToRegularExpressionFA"],

Cell["A small nondeterministic machine.", "ExampleText",
  CellTags->"ToRegularExpressionFA"],

Cell[BoxData[
    \(\(m = 
        FA[4, 2, {{1, 1, 1}, {1, 2, 1}, {1, 2, 2}, {2, 1, 3}, {2, 2, 3}, {3, 
              1, 4}, {3, 2, 4}}, {1}, {4}];\)\)], "Input",
  CellLabel->"In[187]:=",
  CellTags->"ToRegularExpressionFA"],

Cell[BoxData[
    \(ToRegularExpressionFA[m]\)], "Input",
  CellLabel->"In[195]:=",
  CellTags->"ToRegularExpressionFA"],

Cell[BoxData[
    \(RET[
      REP["\<b\>", 
        RET[REP[\[Epsilon], "\<a\>", "\<b\>"], 
          RES[REP["\<a\>", "\<b\>"]], "\<b\>"]], REP["\<a\>", "\<b\>"], 
      REP["\<a\>", "\<b\>"]]\)], "Input",
  CellLabel->"Out[184]=",
  CellTags->"ToRegularExpressionFA"],

Cell[BoxData[
    \(%\  /. \ {REP\  \[Rule] \ Plus, 
        RET \[Rule] \ NonCommutativeMultiply}\)], "Input",
  CellLabel->"In[64]:=",
  CellTags->"ToRegularExpressionFA"],

Cell[TextData[{
  "Our simplification rules are too weak to rewrite this to ",
  Cell[BoxData[
      \(TraditionalForm\`\(\((a + b)\)\^\(\(\ \)\(*\)\)\) 
        b\ \((a + b)\) \((a + b)\)\)]],
  ". "
}], "ExampleText",
  CellTags->"ToRegularExpressionFA"],

Cell["ToSA", "ObjectName",
  CellTags->"ToSA"],

Cell["Function", "Highlight",
  CellTags->"ToSA"],

Cell[TextData[{
  StyleBox["ToSA[M]", "MR"],
  "  converts any finite state machine to the corresponding semiautomaton. "
}], "Usage",
  CellTags->"ToSA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ToSA"],

Cell["\<\
Also converts a cellular automaton to the corresponding de Bruijn \
semiautomaton.\
\>", "Notes",
  CellTags->"ToSA"],

Cell["\<\
Stripps off initial and final states  and converts to a list of \
transitions if necessary. \
\>", "Notes",
  CellTags->"ToSA"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToFA",
    ButtonData:>"ToFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToNFA",
    ButtonData:>"ToNFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToDFA",
    ButtonData:>"ToDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToSA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToSA"],

Cell["Removing initial and final states from a DFA. ", "ExampleText",
  CellTags->"ToSA"],

Cell["m = CounterDFA[b,3,2]", "Input",
  CellLabel->"In[3]:=",
  CellTags->"ToSA"],

Cell[BoxData[
    \(sa\  = \ ToSA[m]\)], "Input",
  CellLabel->"In[6]:=",
  CellTags->"ToSA"],

Cell["\<\
The semiautomaton is still deterministic but no longer equivalent \
to the original machine.\
\>", "ExampleText",
  CellTags->"ToSA"],

Cell[BoxData[{
    \(DeterministicQFA[sa]\), "\n", 
    \(EquivalentQFA[\ sa, \ m\ ]\)}], "Input",
  CellLabel->"In[7]:=",
  CellTags->"ToSA"],

Cell["Converting a cellular automaton.", "ExampleText",
  CellTags->"ToSA"],

Cell[BoxData[
    \(ToSA[\ \ CA[3, \(-2\), 90], \ Normalize \[Rule] 1\ ]\)], "Input",
  CellLabel->"In[13]:=",
  CellTags->"ToSA"],

Cell[BoxData[
    \(%\  // \ States\)], "Input",
  CellLabel->"In[14]:=",
  CellTags->"ToSA"],

Cell["ToSeries", "ObjectName",
  CellTags->"ToSeries"],

Cell["Function", "Highlight",
  CellTags->"ToSeries"],

Cell[TextData[{
  StyleBox["ToSeries[L]", "MR"],
  "  converts a list ",
  StyleBox["L", "MR"],
  " with multiplicities into a formal power series of words with integer \
coefficients. "
}], "Usage",
  CellTags->"ToSeries"],

Cell[TextData[{
  "Uses ",
  StyleBox["HoldForm", "MR"],
  " to bypass Mathematica's sorting algorithm."
}], "Notes",
  CellTags->"ToSeries"],

Cell["Mostly useful for display purposes.", "Notes",
  CellTags->"ToSeries"],

Cell[TextData[{
  "See ",
  ButtonBox["LanguageFA",
    ButtonData:>"LanguageFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Multiplicity",
    ButtonData:>"Multiplicity",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToSeries"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToSeries"],

Cell["Nothing at all.", "ExampleText",
  CellTags->"ToSeries"],

Cell["m = ToSA[CA[110,3,2]];", "Input",
  CellLabel->"In[203]:=",
  CellTags->"ToSeries"],

Cell[BoxData[
    \(LanguageFA[\ m, \ 4, \ Multiplicity \[Rule] True]\)], "Input",
  CellLabel->"In[220]:=",
  CellTags->"ToSeries"],

Cell[BoxData[
    \(ToSeries[%]\)], "Input",
  CellLabel->"In[221]:=",
  CellTags->"ToSeries"],

Cell["ToTransitions", "ObjectName",
  CellTags->"ToTransitions"],

Cell["Function", "Highlight",
  CellTags->"ToTransitions"],

Cell[TextData[{
  StyleBox["ToTransitions[T,K]", "MR"],
  "  translates a list of transitions with string labels into a proper \
transition list in positional notation with respect to alphabet ",
  StyleBox["K", "MR"],
  "."
}], "Usage",
  CellTags->"ToTransitions"],

Cell["\<\
The labels are required to be single characters, not strings.\
\>", \
"Notes",
  CellTags->"ToTransitions"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToTransitions"],

Cell["\<\
Translating string labeled transitions into positional \
representation, as required by all algorithms.\
\>", "ExampleText",
  CellTags->"ToTransitions"],

Cell["ToTransitions[{{1,\"a\",2},{2,\"b\",3}},2]", "Input",
  CellLabel->"In[232]:=",
  CellTags->"ToTransitions"],

Cell["TransferSequenceFA", "ObjectName",
  CellTags->"TransferSequenceFA"],

Cell["Function", "Highlight",
  CellTags->"TransferSequenceFA"],

Cell[TextData[{
  StyleBox["TransferSequenceFA[M,S,T]", "MR"],
  "  computes a least word in length-lex order  that leads to state ",
  StyleBox["t", "MR"],
  " from state ",
  StyleBox["s", "MR"],
  " in machine ",
  StyleBox["M", "MR"],
  ", if such a word exists, for all pairs ",
  StyleBox["(s,t)", "MR"],
  " with ",
  StyleBox["s", "MR"],
  " in ",
  StyleBox["S", "MR"],
  ", ",
  StyleBox["t", "MR"],
  " in ",
  StyleBox["T", "MR"],
  "."
}], "Usage",
  CellTags->"TransferSequenceFA"],

Cell[TextData[{
  "Default for S is ",
  StyleBox["Initial[M]", "MR"],
  " and for T ",
  StyleBox["Final[M]", "MR"],
  ". "
}], "Notes",
  CellTags->"TransferSequenceFA"],

Cell[TextData[{
  "The output is given as a  ",
  Cell[BoxData[
      \(TraditionalForm\`S\ \[Cross]\ T\)]],
  "  matrix of words. A dash in position ",
  Cell[BoxData[
      \(TraditionalForm\`\((s, t)\)\)]],
  " indicates that there is no computation im M from s to t."
}], "Notes",
  CellTags->"TransferSequenceFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"TransferSequenceFA"],

Cell["A DFA based on a de Bruijn graph.", "ExampleText",
  CellTags->"TransferSequenceFA"],

Cell[BoxData[
    \(\(m\  = \ 
        DFA[8, 2, {{1, 3, 5, 7, 1, 3, 5, 7}, {2, 4, 6, 8, 2, 4, 6, 8}}, 
          1, {5, 6, 7, 8}\ ];\)\)], "Input",
  CellLabel->"In[239]:=",
  CellTags->"TransferSequenceFA"],

Cell["TransferSequenceFA[ m ]", "Input",
  CellLabel->"In[240]:=",
  CellTags->"TransferSequenceFA"],

Cell["TransferSequenceFA[ m, Range[8], Range[8] ] // MatrixForm", "Input",
  CellLabel->"In[241]:=",
  CellTags->"TransferSequenceFA"],

Cell["TransitionFunctionFA", "ObjectName",
  CellTags->"TransitionFunctionFA"],

Cell["Function", "Highlight",
  CellTags->"TransitionFunctionFA"],

Cell[TextData[{
  StyleBox["TransitionFunctionFA[M,delta]", "MR"],
  "  assigns to ",
  StyleBox["delta", "MR"],
  " the transition function of machine ",
  StyleBox["M", "MR"],
  "."
}], "Usage",
  CellTags->"TransitionFunctionFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"TransitionFunctionFA"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["HoldRest", "MR"],
  "."
}], "Notes",
  CellTags->"TransitionFunctionFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  " causes ",
  StyleBox["delta", "MR"],
  " to compute a whole trace of the computation."
}], "Notes",
  CellTags->"TransitionFunctionFA"],

Cell[TextData[{
  "Assigns to ",
  StyleBox["delta", "MR"],
  " the transition function  ",
  Cell[BoxData[
      \(TraditionalForm\`Q\ \[Cross]\ \(K\^*\)\[LongRightArrow]\ Q\)]],
  " (which may be multi-valued for nondeterministic machines). "
}], "Notes",
  CellTags->"TransitionFunctionFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["Transitions",
    ButtonData:>"Transitions",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["TransitionMatrixFA",
    ButtonData:>"TransitionMatrixFA",
    ButtonStyle->"AddOnsLink"],
  " ."
}], "Notes",
  CellTags->"TransitionFunctionFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"TransitionFunctionFA"],

Cell["The transition function of a DFA is point-valued.", "ExampleText",
  CellTags->"TransitionFunctionFA"],

Cell[BoxData[{
    \(\(m = BuildDFA[aaa, abb, bbb, 2];\)\), "\n", 
    \(TransitionFunctionFA[m, \[Delta]]\), "\n", 
    \(\[Delta][1, aaa]\)}], "Input",
  CellLabel->"In[44]:=",
  CellTags->"TransitionFunctionFA"],

Cell["But for an FA the return value is always a list.", "ExampleText",
  CellTags->"TransitionFunctionFA"],

Cell[BoxData[{
    \(\(m = IthSymbolFA[a, \(-3\)];\)\), "\n", 
    \(TransitionFunctionFA[m, \[Delta]]\), "\n", 
    \(\[Delta][1, aaa]\), "\n", 
    \(\[Delta][2, aaa]\)}], "Input",
  CellLabel->"In[47]:=",
  CellTags->"TransitionFunctionFA"],

Cell["A more complete trace of the computation.", "ExampleText",
  CellTags->"TransitionFunctionFA"],

Cell[BoxData[{
    \(TransitionFunctionFA[m, \[Delta], Full \[Rule] True]\), "\n", 
    \(\[Delta][1, aaa]\), "\n", 
    \(\[Delta][2, aaa]\)}], "Input",
  CellLabel->"In[51]:=",
  CellTags->"TransitionFunctionFA"],

Cell["TransitionMatrixFA", "ObjectName",
  CellTags->"TransitionMatrixFA"],

Cell["Function", "Highlight",
  CellTags->"TransitionMatrixFA"],

Cell[TextData[{
  StyleBox["TransitionMatrixFA[M,f]", "MR"],
  "  assigns to ",
  StyleBox["f", "MR"],
  " the natural homomorphism from all words to matrices over the natural \
numbers given by machine ",
  StyleBox["M", "MR"],
  "."
}], "Usage",
  CellTags->"TransitionMatrixFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Type", "\[Rule]", "None", \(Boolean, Tropical\)},
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"TransitionMatrixFA"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["HoldRest", "MR"],
  "."
}], "Notes",
  CellTags->"TransitionMatrixFA"],

Cell[TextData[{
  "Option  ",
  StyleBox["Type->Boolean", "MR"],
  "  selects the Boolean semirgin while   ",
  StyleBox["Type->Tropical", "MR"],
  "  selects the tropical semiring."
}], "Notes",
  CellTags->"TransitionMatrixFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  "  produces ordinary matrices, rather than ",
  ButtonBox["SparseArray",
    ButtonStyle->"RefGuideLink"],
  " objects as by default."
}], "Notes",
  CellTags->"TransitionMatrixFA"],

Cell[TextData[{
  "The homomorphism has the form  ",
  Cell[BoxData[
      \(TraditionalForm\`f\  : \ \(K\^*\)\[LongRightArrow]\ Q\[Cross]
          Q\ \[LongRightArrow]\ R\)]],
  "  where ",
  Cell[BoxData[
      \(TraditionalForm\`R\)]],
  " is a semiring. The default for ",
  Cell[BoxData[
      \(TraditionalForm\`R\)]],
  " is the semiring of natural numbers. "
}], "Notes",
  CellTags->"TransitionMatrixFA"],

Cell[TextData[{
  "Note that Boolean matrices are implemented as binary matrices (see ",
  ButtonBox["Boolean",
    ButtonStyle->"AddOnsLink"],
  ")."
}], "Notes",
  CellTags->"TransitionMatrixFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["TransitionsToMatrix",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["TransitionFunctionFA",
    ButtonData:>"TransitionFunctionFA",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["FullTransitionMatrixFA",
    ButtonData:>"FullTransitionMatrixFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"TransitionMatrixFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"TransitionMatrixFA"],

Cell[TextData[{
  "The function ",
  StyleBox["f", "MR"],
  " takes as input words, and returns as output the corresponding matrix."
}], "ExampleText",
  CellTags->"TransitionMatrixFA"],

Cell[BoxData[{
    \(\(m = IthSymbolFA[a, \(-4\)];\)\), "\n", 
    \(TransitionMatrixFA[m, f]\)}], "Input",
  CellLabel->"In[127]:=",
  CellTags->"TransitionMatrixFA"],

Cell[BoxData[
    \(f[bb]\  // \ MatrixForm\)], "Input",
  CellLabel->"In[131]:=",
  CellTags->"TransitionMatrixFA"],

Cell[BoxData[
    \(f[aaaa]\  // \ MatrixForm\)], "Input",
  CellLabel->"In[130]:=",
  CellTags->"TransitionMatrixFA"],

Cell[TextData[{
  "Function ",
  StyleBox["f", "MR"],
  " is a homomorphism."
}], "ExampleText",
  CellTags->"TransitionMatrixFA"],

Cell[BoxData[{
    \(f[ba]\  . \ f[baa] // \ MatrixForm\), "\n", 
    \(f[babaa] // \ MatrixForm\)}], "Input",
  CellLabel->"In[138]:=",
  CellTags->"TransitionMatrixFA"],

Cell["TransitionsToMatrix", "ObjectName",
  CellTags->"TransitionsToMatrix"],

Cell["Function", "Highlight",
  CellTags->"TransitionsToMatrix"],

Cell[TextData[{
  StyleBox["TransitionsToMatrix[n,k,T]", "MR"],
  "  converts a list of transitions ",
  StyleBox["T", "MR"],
  " into the corresponding ",
  StyleBox["k", "MR"],
  " by ",
  StyleBox["n", "MR"],
  " matrix, where ",
  StyleBox["k", "MR"],
  " is the alphabet size and ",
  StyleBox["n", "MR"],
  " the number of states. "
}], "Usage",
  CellTags->"TransitionsToMatrix"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Type", "\[Rule]", \(Boolean, Tropical\)}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"TransitionsToMatrix"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["HoldRest", "MR"],
  "."
}], "Notes",
  CellTags->"TransitionsToMatrix"],

Cell[TextData[{
  "Option ",
  StyleBox["Type->Boolean", "MR"],
  " selects the Boolean semiring"
}], "Notes",
  CellTags->"TransitionsToMatrix"],

Cell["\<\
If the transitions are deterministic and complete, a matrix for a \
DFA is produced (an integer matrix of depth 2), otherwise the matrix for an \
NFA is computed (an integer matrix of depth 3).\
\>", "Notes",
  CellTags->"TransitionsToMatrix"],

Cell[TextData[{
  "See also ",
  ButtonBox["TransitionMatrixFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["FullTransitionMatrixFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["TransitionFunctionFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"TransitionsToMatrix"],

Cell["Further Examples", "ExampleSection",
  CellTags->"TransitionsToMatrix"],

Cell["\<\
Translating a list of transitions into the transition matrix for a \
corresponding NFA.\
\>", "ExampleText",
  CellTags->"TransitionsToMatrix"],

Cell[BoxData[
    \(tr\  = \ {{1, 1, 1}, {1, 1, 2}, {2, 2, 4}, {3, 2, 1}, {4, 2, 3}, {4, 1, 
          4}}\)], "Input",
  CellLabel->"In[81]:=",
  CellTags->"TransitionsToMatrix"],

Cell[BoxData[
    \(TransitionsToMatrix[4, 2, tr]\  // \ MatrixForm\)], "Input",
  CellLabel->"In[86]:=",
  CellTags->"TransitionsToMatrix"],

Cell[TextData[{
  "For DFA we want a ",
  Cell[BoxData[
      \(TraditionalForm\`k\ \[Cross]\ n\)]],
  " integer matrix as output."
}], "ExampleText",
  CellTags->"TransitionsToMatrix"],

Cell[BoxData[
    \(\(tr\  = \ {{1, 1, 2}, {2, 1, 3}, {3, 1, 4}, {4, 1, 5}, {5, 1, 5}, {1, 
            2, 1}, {2, 2, 2}, {3, 2, 3}, {4, 2, 4}, {5, 2, 5}, {1, 3, 1}, {2, 
            3, 2}, {3, 3, 3}, {4, 3, 4}, {5, 3, 5}};\)\)], "Input",
  CellLabel->"In[94]:=",
  CellTags->"TransitionsToMatrix"],

Cell[BoxData[
    \(TransitionsToMatrix[5, 3, tr, Type \[Rule] "\<DFA\>"]\  // \ 
      MatrixForm\)], "Input",
  CellLabel->"In[96]:=",
  CellTags->"TransitionsToMatrix"],

Cell["Transitions", "ObjectName",
  CellTags->"Transitions"],

Cell["Function", "Highlight",
  CellTags->"Transitions"],

Cell[TextData[{
  StyleBox["Transitions[M]", "MR"],
  "  returns the transition matrix (for DFAs) or transitions list (for all \
others), where M is a finite state machine. "
}], "Usage",
  CellTags->"Transitions"],

Cell[TextData[{
  "The entries in a labeled transition list are of the ",
  StyleBox["{p,a,q}", "MR"],
  " where ",
  StyleBox["p", "MR"],
  " and ",
  StyleBox["q", "MR"],
  " are states in positional notation, and ",
  StyleBox["a", "MR"],
  " is a symbol from the alphabet, also in positional notation (0 represents \
epsilon)."
}], "Notes",
  CellTags->"Transitions"],

Cell[TextData[{
  "See also ",
  ButtonBox["FullTransitions",
    ButtonData:>"FullTransitions",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["TransitionFunctionFA",
    ButtonData:>"TransitionFunctionFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Transitions"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Transitions"],

Cell["\<\
The type of the output depends on the type of the input \
machine.\
\>", "ExampleText",
  CellTags->"Transitions"],

Cell["\<\
m = BuildDFA[aaa,abb,bbb,2];
Transitions[m]\
\>", "Input",
  CellLabel->"In[275]:=",
  CellTags->"Transitions"],

Cell["\<\
m = IthSymbolFA[a,-3];
Transitions[m]\
\>", "Input",
  CellLabel->"In[277]:=",
  CellTags->"Transitions"],

Cell["TransitiveLanguageQFA", "ObjectName",
  CellTags->"TransitiveLanguageQFA"],

Cell["Function", "Highlight",
  CellTags->"TransitiveLanguageQFA"],

Cell[TextData[{
  StyleBox["TransitiveLanguageQFA[M]", "MR"],
  "  tests whether the language accepted by ",
  StyleBox["M", "MR"],
  " is transitive. "
}], "Usage",
  CellTags->"TransitiveLanguageQFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"TransitiveLanguageQFA"],

Cell[TextData[{
  "The algorithms assumes that ",
  Cell[BoxData[
      \(TraditionalForm\`M\)]],
  " is factorial, does not check. "
}], "Notes",
  CellTags->"TransitiveLanguageQFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["TransitiveQFA",
    ButtonData:>"TransitiveQFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"TransitiveLanguageQFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"TransitiveLanguageQFA"],

Cell["\<\
Transitive semiautomata naturally produce transitive \
languages.\
\>", "ExampleText",
  CellTags->"TransitiveLanguageQFA"],

Cell[" s = ToSA[CA[110]]", "Input",
  CellLabel->"In[711]:=",
  CellTags->"TransitiveLanguageQFA"],

Cell[BoxData[
    \(TransitiveLanguageQFA[s]\)], "Input",
  CellLabel->"In[712]:=",
  CellTags->"TransitiveLanguageQFA"],

Cell["TransitiveQFA", "ObjectName",
  CellTags->"TransitiveQFA"],

Cell["Function", "Highlight",
  CellTags->"TransitiveQFA"],

Cell[TextData[{
  StyleBox["TransitiveQFA[M,opts]", "MR"],
  "  tests whether M is transitive. "
}], "Usage",
  CellTags->"TransitiveQFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"DoTrace", "\[Rule]", "False", "-"},
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"TransitiveQFA"],

Cell[TextData[{
  "Option ",
  StyleBox["DoTrace->True", "MR"],
  " causes the strongly connected components of the transition diagram of M \
to be displayed."
}], "Notes",
  CellTags->"TransitiveQFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["TransitiveLangQFA",
    ButtonData:>"TransitiveLangQFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"TransitiveQFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"TransitiveQFA"],

Cell["\<\
The de Bruijn automata associated with linear cellular automata are \
naturally transitive.\
\>", "ExampleText",
  CellTags->"TransitiveQFA"],

Cell[" s = ToSA[CA[110]]", "Input",
  CellLabel->"In[285]:=",
  CellTags->"TransitiveQFA"],

Cell[BoxData[
    \(TransitiveQFA[s]\)], "Input",
  CellLabel->"In[286]:=",
  CellTags->"TransitiveQFA"],

Cell[BoxData[
    \(TransitiveQFA[s, DoTrace \[Rule] True]\)], "Input",
  CellLabel->"In[165]:=",
  CellTags->"TransitiveQFA"],

Cell["TransOnly", "ObjectName",
  CellTags->"TransOnly"],

Cell["Option", "Highlight",
  CellTags->"TransOnly"],

Cell[TextData[{
  StyleBox["TransOnly", "MR"],
  "  is an option for ",
  StyleBox["PrintFA", "MR"],
  ", "
}], "Usage",
  CellTags->"TransOnly"],

Cell[TextData[{
  "Option ",
  StyleBox["TransOnly->True", "MR"],
  "  causes only the transitions to be printed."
}], "Notes",
  CellTags->"TransOnly"],

Cell[TextData[{
  "See also ",
  ButtonBox["PrintFA",
    ButtonData:>"PrintFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"TransOnly"],

Cell["Further Examples", "ExampleSection",
  CellTags->"TransOnly"],

Cell["Printing a semiautomaton.", "ExampleText",
  CellTags->"TransOnly"],

Cell[BoxData[
    \(\(m = 
        SA[4, 2, {{1, 1, 1}, {1, 2, 2}, {2, 1, 3}, {2, 2, 4}, {3, 2, 1}, {3, 
              1, 2}, {4, 2, 3}, {4, 1, 4}}];\)\)], "Input",
  CellLabel->"In[103]:=",
  CellTags->"TransOnly"],

Cell[BoxData[
    \(PrintFA[m]\)], "Input",
  CellLabel->"In[104]:=",
  CellTags->"TransOnly"],

Cell[BoxData[
    \(PrintFA[m, TransOnly \[Rule] True]\)], "Input",
  CellLabel->"In[105]:=",
  CellTags->"TransOnly"],

Cell["TrapStatesFA", "ObjectName",
  CellTags->"TrapStatesFA"],

Cell["Function", "Highlight",
  CellTags->"TrapStatesFA"],

Cell[TextData[{
  StyleBox["TrapStatesFA[M]", "MR"],
  "  returns all the trap states of machine ",
  StyleBox["M", "MR"],
  "."
}], "Usage",
  CellTags->"TrapStatesFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "True", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"TrapStatesFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->False", "MR"],
  "  returns only the first trap state that fails to be final; returns 0 if \
no such state exists."
}], "Notes",
  CellTags->"TrapStatesFA"],

Cell["\<\
Trap state here means any state without out-going transitions, \
finality is not taken into account.\
\>", "Notes",
  CellTags->"TrapStatesFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["SinkQFA",
    ButtonData:>"SinkQFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"TrapStatesFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"TrapStatesFA"],

Cell["\<\
A finite state machine with two trap states, one of them being \
final.\
\>", "ExampleText",
  CellTags->"TrapStatesFA"],

Cell["\<\
m = FA[5,2,{{1,1,2},{2,1,3},{3,1,3},{1,2,4},{4,2,5}},{1},{3}]\
\>", \
"Input",
  CellLabel->"In[175]:=",
  CellTags->"TrapStatesFA"],

Cell[BoxData[{
    \(TrapStatesFA[m]\), "\n", 
    \(TrapStatesFA[m, Full \[Rule] False]\)}], "Input",
  CellLabel->"In[176]:=",
  CellTags->"TrapStatesFA"],

Cell["TrimFA", "ObjectName",
  CellTags->"TrimFA"],

Cell["Function", "Highlight",
  CellTags->"TrimFA"],

Cell[TextData[{
  StyleBox["TrimFA[M]", "MR"],
  "  constructs the trim part of FA M: only states that are accessible from \
the initial states and coaccessible from the final states are retained."
}], "Usage",
  CellTags->"TrimFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"TrimFA"],

Cell[TextData[{
  "Can also be used in the form ",
  StyleBox["TrimFA[M,S,T]", "MR"],
  " where S replaces the initial states, and T replaces the final states. "
}], "Notes",
  CellTags->"TrimFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["AccessibleFA",
    ButtonData:>"AccessibleFA",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["CoAccessibleFA",
    ButtonData:>"CoAccessibleFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"TrimFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"TrimFA"],

Cell["\<\
 A machine on 5 states whose trim part has size 3.The trim part is \
equivalent to the original machine.\
\>", "ExampleText",
  CellTags->"TrimFA"],

Cell[BoxData[
    \(m\  = \ 
      FA[\ 5, \ 2, \ {{1, 1, 3}, {3, 2, 4}, {2, 1, 1}, {2, 2, 
            5}}, {1}, {4}\ ]\)], "Input",
  CellLabel->"In[514]:=",
  CellTags->"TrimFA"],

Cell[BoxData[{
    \(ma\  = \ TrimFA[m]\), "\n", 
    \(EquivalentQFA[m, ma]\)}], "Input",
  CellLabel->"In[515]:=",
  CellTags->"TrimFA"],

Cell["TropicalIdentityMatrix", "ObjectName",
  CellTags->"TropicalIdentityMatrix"],

Cell["Function", "Highlight",
  CellTags->"TropicalIdentityMatrix"],

Cell[TextData[{
  StyleBox["TropicalIdentityMatrix[n]", "MR"],
  "  generates the ",
  StyleBox["n", "MR"],
  " by ",
  StyleBox["n", "MR"],
  " identity matrix in the tropical semiring. "
}], "Usage",
  CellTags->"TropicalIdentityMatrix"],

Cell[TextData[{
  "See also ",
  ButtonBox["Tropical",
    ButtonData:>"Tropical",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["TS",
    ButtonData:>"TS",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["LoopAutomatonFA",
    ButtonData:>"LoopAutomatonFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"TropicalIdentityMatrix"],

Cell["Further Examples", "ExampleSection",
  CellTags->"TropicalIdentityMatrix"],

Cell[TextData[{
  "The tropical ",
  Cell[BoxData[
      \(TraditionalForm\`4\[Cross]\ 4\)]],
  " identity matrix."
}], "ExampleText",
  CellTags->"TropicalIdentityMatrix"],

Cell["TropicalIdentityMatrix[4] // MatrixForm", "Input",
  CellLabel->"In[295]:=",
  CellTags->"TropicalIdentityMatrix"],

Cell["Tropical", "ObjectName",
  CellTags->"Tropical"],

Cell["Option", "Highlight",
  CellTags->"Tropical"],

Cell[TextData[{
  StyleBox["Tropical", "MR"],
  "  is an option for ",
  StyleBox["TransitionMatrixFA", "MR"],
  ", "
}], "Usage",
  CellTags->"Tropical"],

Cell["\<\
Causes the matrix to be generated over the tropical semiring.\
\>", \
"Notes",
  CellTags->"Tropical"],

Cell[TextData[{
  " See also ",
  ButtonBox["LoopAutomatonFA",
    ButtonData:>"LoopAutomatonFA",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["TransitionMatrixFA, Tropical",
    ButtonData:>"TransitionMatrixFA, Tropical",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"Tropical"],

Cell["TS", "ObjectName",
  CellTags->"TS"],

Cell["Constructor", "Highlight",
  CellTags->"TS"],

Cell[TextData[{
  StyleBox["TS[x]", "MR"],
  "  denotes an element in the tropical semiring; ",
  StyleBox["x", "MR"],
  " is a non-negative integer or ",
  StyleBox["Infinity", "MR"],
  ". "
}], "Usage",
  CellTags->"TS"],

Cell[TextData[{
  "The display format of ",
  StyleBox["TS[Infinity]", "MR"],
  " is \[Infinity], and ",
  StyleBox["TS[x]", "MR"],
  " is displayed as ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " for all natural numbers ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  "."
}], "Notes",
  CellTags->"TS"],

Cell[TextData[{
  "The arithmetic operations ",
  StyleBox["Plus", "MR"],
  ", ",
  StyleBox["Times", "MR"],
  " and ",
  StyleBox["Power", "MR"],
  " work as usual but their interpretation now is min, plus and times, \
respectively."
}], "Notes",
  CellTags->"TS"],

Cell[TextData[{
  "See ",
  ButtonBox["LoopAutomatonFA",
    ButtonData:>"LoopAutomatonFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Tropical",
    ButtonData:>"Tropical",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["TropicalIdentityMatrix",
    ButtonData:>"TropicalIdentityMatrix",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"TS"],

Cell["Further Examples", "ExampleSection",
  CellTags->"TS"],

Cell["Arithmetic over the tropical semiring.", "ExampleText",
  CellTags->"TS"],

Cell["TS[1] + TS[3]", "Input",
  CellLabel->"In[60]:=",
  CellTags->"TS"],

Cell["TS[1] * TS[3]", "Input",
  CellLabel->"In[61]:=",
  CellTags->"TS"],

Cell["TS[2]^3", "Input",
  CellLabel->"In[63]:=",
  CellTags->"TS"],

Cell["TS[1] + TS[Infinity]", "Input",
  CellLabel->"In[64]:=",
  CellTags->"TS"],

Cell["TS[1] * TS[Infinity]", "Input",
  CellLabel->"In[65]:=",
  CellTags->"TS"],

Cell["TS[Infinity]^3", "Input",
  CellLabel->"In[66]:=",
  CellTags->"TS"],

Cell["UnionFA", "ObjectName",
  CellTags->"UnionFA"],

Cell["Function", "Highlight",
  CellTags->"UnionFA"],

Cell[TextData[{
  StyleBox["UnionFA[M1,M2]", "MR"],
  "  generates a finite state machine that accepts the union of the \
acceptance languages of ",
  StyleBox["M1", "MR"],
  " and ",
  StyleBox["M2", "MR"],
  ". "
}], "Usage",
  CellTags->"UnionFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"},
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"UnionFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  " constructs full product machine."
}], "Notes",
  CellTags->"UnionFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Normalize->1", "MR"],
  " preserves state set ",
  Cell[BoxData[
      \(TraditionalForm\`Q\_1\[Cross]Q\_2\)]],
  ". "
}], "Notes",
  CellTags->"UnionFA"],

Cell["\<\
Uses a product machine construction for DFAs and disjoint union for \
other machines. \
\>", "Notes",
  CellTags->"UnionFA"],

Cell["Is polyadic.", "Notes",
  CellTags->"UnionFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["SumFA",
    ButtonData:>"SumFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ProductFA",
    ButtonData:>"ProductFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["IntersectionFA",
    ButtonData:>"IntersectionFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"UnionFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"UnionFA"],

Cell["\<\
Two machines for unary languages. The first accepts all words of \
length a multiple of 3, and the second all words of length a multiple of 4.\
\
\>", "ExampleText",
  CellTags->"UnionFA"],

Cell[BoxData[{
    \(m3 = CounterDFA[a, 0, 1, Modulus \[Rule] 3]\), "\n", 
    \(m4 = CounterDFA[a, 0, 1, Modulus \[Rule] 4]\)}], "Input",
  CellLabel->"In[173]:=",
  CellTags->"UnionFA"],

Cell["\<\
The union machine accepts all words of length a multiple of 3 or 4.\
\
\>", "ExampleText",
  CellTags->"UnionFA"],

Cell[BoxData[{
    \(UnionFA[m3, m4]\), "\n", 
    \(LanguageFA[\ %, \ \(-25\), \ SizeOnly \[Rule] True\ ]\)}], "Input",
  CellLabel->"In[175]:=",
  CellTags->"UnionFA"],

Cell["Unlabeled", "ObjectName",
  CellTags->"Unlabeled"],

Cell["Option", "Highlight",
  CellTags->"Unlabeled"],

Cell[TextData[{
  StyleBox["Unlabeled", "MR"],
  "   is an option for ",
  StyleBox["ToGraphFA", "MR"],
  ", causes an unlabeled graph to be generated."
}], "Usage",
  CellTags->"Unlabeled"],

Cell[TextData[{
  "Default value is ",
  StyleBox["True", "MR"],
  "."
}], "Notes",
  CellTags->"Unlabeled"],

Cell[TextData[{
  "See  ",
  ButtonBox["ToGraphFA",
    ButtonData:>"ToGraphFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Unlabeled"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Unlabeled"],

Cell["Converting a machine into a graph.", "ExampleText",
  CellTags->"Unlabeled"],

Cell[BoxData[
    \(\(m = 
        FA[4, 2, {{1, 1, 1}, {1, 2, 1}, {1, 2, 2}, {2, 1, 3}, {2, 2, 3}, {3, 
              1, 4}, {3, 2, 4}}, {1}, {4}];\)\)], "Input",
  CellLabel->"In[100]:=",
  CellTags->"Unlabeled"],

Cell["ToGraphFA[m]", "Input",
  CellLabel->"In[101]:=",
  CellTags->"Unlabeled"],

Cell[BoxData[
    \(ToGraphFA[m, Unlabeled \[Rule] False]\)], "Input",
  CellLabel->"In[102]:=",
  CellTags->"Unlabeled"],

Cell["VerifyFA", "ObjectName",
  CellTags->"VerifyFA"],

Cell["Function", "Highlight",
  CellTags->"VerifyFA"],

Cell[TextData[{
  StyleBox["VerifyFA[M]", "MR"],
  "  checks if ",
  StyleBox["M", "MR"],
  " is properly constructed. Recommended if a machine is constructed 'by \
hand'. "
}], "Usage",
  CellTags->"VerifyFA"],

Cell[TextData[{
  "Currently, ",
  StyleBox["VerifyFA", "MR"],
  " is quite feeble and needs improvement."
}], "Notes",
  CellTags->"VerifyFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"VerifyFA"],

Cell["WordToFA", "ObjectName",
  CellTags->"WordToFA"],

Cell["Function", "Highlight",
  CellTags->"WordToFA"],

Cell[TextData[{
  StyleBox["WordToFA[w]", "MR"],
  "  constructs a machine that accepts all factors of word ",
  StyleBox["w", "MR"],
  ".  "
}], "Usage",
  CellTags->"WordToFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"},
          {"Type", "\[Rule]", 
            StyleBox["\<\"\\\"Factor\\\"\"\>",
              "MR"], "-"},
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"WordToFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Type->\"Prefix\"", "MR"],
  " generates a machine accepting all prefixes of the given word instead."
}], "Notes",
  CellTags->"WordToFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Type->\"Suffix\"", "MR"],
  " generates a machine accepting all suffixes of the given word instead."
}], "Notes",
  CellTags->"WordToFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True ", "MR"],
  " generates a machine accepting all words that have ",
  Cell[BoxData[
      \(TraditionalForm\`w\)]],
  " as a factor (prefix, suffix)."
}], "Notes",
  CellTags->"WordToFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToFactorFA",
    ButtonData:>"ToFactorFA",
    ButtonStyle->"AddOnsLink"],
  "  and  ",
  ButtonBox["BuildDFA",
    ButtonData:>"BuildDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WordToFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordToFA"],

Cell[TextData[{
  " A machine that accepts all factors of ",
  Cell[BoxData[
      FormBox[
        StyleBox["aabbcc",
          FontSlant->"Italic"], TraditionalForm]]],
  "."
}], "ExampleText",
  CellTags->"WordToFA"],

Cell[BoxData[{
    \(WordToFA[aabbcc, 3\ ]\), "\n", 
    \(LanguageFA[\ %, \ Full \[Rule] True\ ]\)}], "Input",
  CellLabel->"In[81]:=",
  CellTags->"WordToFA"],

Cell[TextData[{
  " A machine that accepts all prefixes of ",
  Cell[BoxData[
      FormBox[
        StyleBox["aabbcc",
          FontSlant->"Italic"], TraditionalForm]]],
  "."
}], "ExampleText",
  CellTags->"WordToFA"],

Cell[BoxData[{
    \(WordToFA[aabbcc, 3, Type \[Rule] "\<Prefix\>"]\), "\n", 
    \(LanguageFA[\ %, \ Full \[Rule] True\ ]\)}], "Input",
  CellLabel->"In[83]:=",
  CellTags->"WordToFA"],

Cell[TextData[{
  " And a machine that accepts all suffixes of ",
  Cell[BoxData[
      FormBox[
        StyleBox["aabbcc",
          FontSlant->"Italic"], TraditionalForm]]],
  "."
}], "ExampleText",
  CellTags->"WordToFA"],

Cell[BoxData[{
    \(WordToFA[aabbcc, 3, Type \[Rule] "\<Suffix\>"]\), "\n", 
    \(LanguageFA[\ %, \ Full \[Rule] True\ ]\)}], "Input",
  CellLabel->"In[85]:=",
  CellTags->"WordToFA"],

Cell["\<\
The corresponding minimal automaton is rather surprisingly small.\
\
\>", "ExampleText",
  CellTags->"WordToFA"],

Cell[BoxData[
    \(Table[\ 
      Size@\(MinimizeFA@
          WordToFA[\ RandomWord[10, 3], 3\ ]\), \ {10}]\)], "Input",
  CellLabel->"In[88]:=",
  CellTags->"WordToFA"],

Cell["\<\
A soundness check: we construct a suffix machine directly as a \
quotient machine.\
\>", "ExampleText",
  CellTags->"WordToFA"],

Cell[BoxData[{
    \(\(m0\  = \ WordToFA[aabbcc, 3, Type \[Rule] "\<Suffix\>"];\)\), "\n", 
    \(\(m1\  = \ BuildDFA[\ WordSuffixes[aabbcc], 3];\)\), "\n", 
    \(EquivalentQFA[m0, m1]\)}], "Input",
  CellLabel->"In[89]:=",
  CellTags->"WordToFA"],

Cell["\<\
Next we construct machines that accept all words containing the \
given word as a factor.\
\>", "ExampleText",
  CellTags->"WordToFA"],

Cell[BoxData[{
    \(m\  = \ WordToFA[abc, 3, Full \[Rule] True\ ]\), "\n", 
    \(LanguageFA[\ %, \ \(-5\)\ ]\)}], "Input",
  CellLabel->"In[92]:=",
  CellTags->"WordToFA"],

Cell["\<\
We can verify the correctness of the last machine by checking \
equivalence with a lovingly handcrafted reference machine.\
\>", "ExampleText",\

  CellTags->"WordToFA"],

Cell[BoxData[{
    \(\(mref\  = \ 
        FA[\ 4, \ 
          3, \ {{1, 1, 1}, {1, 2, 1}, {1, 3, 1}, {1, 1, 2}, {2, 2, 3}, {3, 3, 
              4}, \[IndentingNewLine]{4, 1, 4}, {4, 2, 4}, {4, 3, 
              4}}, \ {1}, \ {4}\ ];\)\), "\n", 
    \(EquivalentQFA[m, mref]\)}], "Input",
  CellLabel->"In[94]:=",
  CellTags->"WordToFA"],

Cell["$regexrules", "ObjectName",
  CellTags->"$regexrules"],

Cell["SysVariable", "Highlight",
  CellTags->"$regexrules"],

Cell[TextData[{
  StyleBox["$regexrules", "MR"],
  "   is a set of simplification rules for regular expressions. "
}], "Usage",
  CellTags->"$regexrules"],

Cell[TextData[{
  "This rule table is used in ",
  ButtonBox["RESimplify",
    ButtonStyle->"AddOnsLink"],
  " which also addresses some formatting issues."
}], "Notes",
  CellTags->"$regexrules"],

Cell["\<\
The rules can be augmented by the user. Make sure not to introduce \
any cycles, though. \
\>", "Notes",
  CellTags->"$regexrules"],

Cell["\<\
Simplification PSPACE hard, so one should not expect great effects \
from a simple collection of rewrite rules. \
\>", "Notes",
  CellTags->"$regexrules"],

Cell[TextData[{
  "See also ",
  ButtonBox["ArdenSolve",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SubstituteRE",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"$regexrules"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$regexrules"],

Cell[TextData[{
  "The rules in ",
  StyleBox["$regexrules", "MR"],
  " perform some simplification of regular expressions. "
}], "ExampleText",
  CellTags->"$regexrules"],

Cell["REP[REP[a,b],c] /. $regexrules", "Input",
  CellLabel->"In[69]:=",
  CellTags->"$regexrules"],

Cell["REP[RES[a],Eps] //. $regexrules", "Input",
  CellLabel->"In[70]:=",
  CellTags->"$regexrules"],

Cell["ArdenSolve", "ObjectName",
  CellTags->"ArdenSolve"],

Cell["Function", "Highlight",
  CellTags->"ArdenSolve"],

Cell[TextData[{
  StyleBox["ArdenSolve[eq]", "MR"],
  "  solves a linear equation ",
  StyleBox["eq", "MR"],
  " over the language semiring using Arden's lemma. The second component of \
the resulting equation is a rational expression, possibly containing \
additional variables. "
}], "Usage",
  CellTags->"ArdenSolve"],

Cell[TextData[{
  "Use in conjunction with ",
  StyleBox["SubstituteRE", "MR"],
  "."
}], "Notes",
  CellTags->"ArdenSolve"],

Cell[TextData[{
  "Since the simplification methods for regular expressions used in ",
  StyleBox["ArdenSolve", "MR"],
  " are rather weak one can only tackle small systems."
}], "Notes",
  CellTags->"ArdenSolve"],

Cell[TextData[{
  "See also ",
  ButtonBox["SubstituteRE",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToEquationsDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ArdenSolve"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ArdenSolve"],

Cell[TextData[{
  "A DFA for all words containing an even number of ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  "'s and ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  "'s, converted into a system of equations. "
}], "ExampleText",
  CellTags->"ArdenSolve"],

Cell[BoxData[{
    \(ClearAll[X]\), "\n", 
    \(FormatX[]\), "\n", 
    \(m\  = \ DFA[4, 2, {{3, 4, 1, 2}, {2, 1, 4, 3}}, 1, {1}\ ]\), "\n", 
    \(\({e1, e2, e3, e4} = ToEquationsDFA[m];\)\), "\n", 
    \(\({e1, e2, e3, e4}\  // \ TableForm\)\  // \ PrintRE\)}], "Input",
  CellLabel->"In[77]:=",
  CellTags->"ArdenSolve"],

Cell[TextData[{
  "Substitute to express ",
  Cell[BoxData[
      \(TraditionalForm\`X\_1\)]],
  " in terms of ",
  Cell[BoxData[
      \(TraditionalForm\`X\_1\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`X\_4\)]],
  ". "
}], "ExampleText",
  CellTags->"ArdenSolve"],

Cell[BoxData[
    \(e11 = SubstituteRE[e1, e2, e3]\)], "Input",
  CellLabel->"In[82]:=",
  AspectRatioFixed->True,
  CellTags->"ArdenSolve"],

Cell[TextData[{
  "Substitute to express ",
  Cell[BoxData[
      \(TraditionalForm\`X\_4\)]],
  " in terms of ",
  Cell[BoxData[
      \(TraditionalForm\`X\_1\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`X\_4\)]],
  ", and solve the equation."
}], "ExampleText",
  CellTags->"ArdenSolve"],

Cell[BoxData[
    \(e44 = SubstituteRE[e4, e2, e3]\)], "Input",
  CellLabel->"In[83]:=",
  AspectRatioFixed->True,
  CellTags->"ArdenSolve"],

Cell[BoxData[
    \(e5 = ArdenSolve[e44]\)], "Input",
  CellLabel->"In[84]:=",
  AspectRatioFixed->True,
  CellTags->"ArdenSolve"],

Cell["\<\
Substitute back, and solve one more time for the final \
answer.\
\>", "ExampleText",
  CellTags->"ArdenSolve"],

Cell[BoxData[
    \(e6 = SubstituteRE[e11, e5]\)], "Input",
  CellLabel->"In[85]:=",
  AspectRatioFixed->True,
  CellTags->"ArdenSolve"],

Cell[BoxData[
    \(sol = ArdenSolve[e6]\)], "Input",
  CellLabel->"In[86]:=",
  AspectRatioFixed->True,
  CellTags->"ArdenSolve"],

Cell["\<\
Note that the last expression could be simplified as follows. Our \
simplification rules are not powerful enough to do this automatically.\
\>", \
"ExampleText",
  CellTags->"ArdenSolve"],

Cell[BoxData[
    \(re\  = \ 
      RES[\ REP[\ aa, \ bb, \ 
          RET[\ REP[ab, ba], RES[REP[aa, bb]], REP[ab, ba]]\ ]]\)], "Input",
  CellLabel->"In[87]:=",
  CellTags->"ArdenSolve"],

Cell["\<\
We can check that the two regular expressions are indeed \
equivalent.\
\>", "ExampleText",
  CellTags->"ArdenSolve"],

Cell[BoxData[{
    \(\(m1\  = \ RegularExpressionToFA[sol[\([2]\)]];\)\), "\n", 
    \(\(m2\  = \ RegularExpressionToFA[re];\)\), "\n", 
    \(EquivalentQFA[m1, m2]\)}], "Input",
  CellLabel->"In[88]:=",
  CellTags->"ArdenSolve"],

Cell["Of course, they are also equivalent to the original DFA.", "ExampleText",
  CellTags->"ArdenSolve"],

Cell[BoxData[
    \(EquivalentQFA[m1, m]\)], "Input",
  CellLabel->"In[91]:=",
  CellTags->"ArdenSolve"],

Cell["Empty", "ObjectName",
  CellTags->"Empty"],

Cell["Option", "Highlight",
  CellTags->"Empty"],

Cell[TextData[{
  StyleBox["Empty", "MR"],
  "  is a regular expression that denotes the empty set."
}], "Usage",
  CellTags->"Empty"],

Cell[TextData[{
  "See also ",
  ButtonBox["RegularExpressionToFA",
    ButtonData:>"RegularExpressionToFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Empty"],

Cell["FormatX", "ObjectName",
  CellTags->"FormatX"],

Cell["Function", "Highlight",
  CellTags->"FormatX"],

Cell[TextData[{
  StyleBox["FormatX[True]", "MR"],
  " formats ",
  StyleBox["X", "MR"],
  " so ",
  StyleBox["X[i]", "MR"],
  " prints in subscript notation. "
}], "Usage",
  CellTags->"FormatX"],

Cell["\<\
Mildly useful to display a system of linear equations over the \
language semiring. \
\>", "Notes",
  CellTags->"FormatX"],

Cell[TextData[{
  StyleBox["FormatX[False]", "MR"],
  " turns formatting off. Default is formatting on."
}], "Notes",
  CellTags->"FormatX"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToEquationsDFA",
    ButtonData:>"ToEquationsDFA",
    ButtonStyle->"AddOnsLink"],
  ". "
}], "Notes",
  CellTags->"FormatX"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FormatX"],

Cell["Converting a DFA into a system of equations.", "ExampleText",
  CellTags->"FormatX"],

Cell[BoxData[{
    \(FormatX[]\), "\n", 
    \(\(eq\  = \ 
        ToEquationsDFA[CounterDFA[b, 0, 2, Modulus \[Rule] 3]];\)\), "\n", 
    \(eq\  // \ TableForm\)}], "Input",
  CellLabel->"In[183]:=",
  CellTags->"FormatX"],

Cell["The system in algebraic form.", "ExampleText",
  CellTags->"FormatX"],

Cell[BoxData[
    \(%\  /. \ {REP \[Rule] Plus, RET \[Rule] NonCommutativeMultiply}\  // \ 
      TableForm\)], "Input",
  CellLabel->"In[186]:=",
  CellTags->"FormatX"],

Cell["Unformatted display can be useful for debugging.", "ExampleText",
  CellTags->"FormatX"],

Cell[BoxData[{
    \(FormatX[False]\), "\n", 
    \(eq\  // \ TableForm\)}], "Input",
  CellLabel->"In[200]:=",
  CellTags->"FormatX"],

Cell["REEpsQ", "ObjectName",
  CellTags->"REEpsQ"],

Cell["Function", "Highlight",
  CellTags->"REEpsQ"],

Cell[TextData[{
  StyleBox["REEpsQ[reg]", "MR"],
  "  tests whether the language denoted by regular expression ",
  StyleBox["reg", "MR"],
  " contains epsilon. "
}], "Usage",
  CellTags->"REEpsQ"],

Cell[TextData[{
  "The regular expression must be proper, i.e., it must be built solely from \
the headers ",
  StyleBox["REP", "MR"],
  ", ",
  StyleBox["RES", "MR"],
  " and ",
  StyleBox["RET", "MR"],
  " (rather than arithmetic operators such as ",
  StyleBox["Plus", "MR"],
  " and ",
  StyleBox["NonCommutativeMultiply", "MR"],
  "). "
}], "Notes",
  CellTags->"REEpsQ"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToRegExp",
    ButtonData:>"ToRegExp",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"REEpsQ"],

Cell["Further Examples", "ExampleSection",
  CellTags->"REEpsQ"],

Cell["Some simple regular expressions. ", "ExampleText",
  CellTags->"REEpsQ"],

Cell["REEpsQ[ REP[a,b,c] ]", "Input",
  CellLabel->"In[109]:=",
  CellTags->"REEpsQ"],

Cell["REEpsQ[ RES[REP[a,b,c]] ]", "Input",
  CellLabel->"In[110]:=",
  CellTags->"REEpsQ"],

Cell["REEpsQ[ RET[a,RES[REP[a,b,c]]] ]", "Input",
  CellLabel->"In[111]:=",
  CellTags->"REEpsQ"],

Cell["RegExpSimp", "ObjectName",
  CellTags->"RegExpSimp"],

Cell["Option", "Highlight",
  CellTags->"RegExpSimp"],

Cell[TextData[{
  StyleBox["RegExpSimp", "MR"],
  "  is an option for ",
  StyleBox["ToRegularExpressionFA", "MR"],
  ". "
}], "Usage",
  CellTags->"RegExpSimp"],

Cell["\<\
Forces Kleene's algorithm to be used without simplification. Do not \
use on machines with more than about 6 states.\
\>", "Notes",
  CellTags->"RegExpSimp"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToRegularExpressionFA",
    ButtonData:>"ToRegularExpressionFA",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"RegExpSimp"],

Cell["RegularExpressionToBEFA", "ObjectName",
  CellTags->"RegularExpressionToBEFA"],

Cell["Function", "Highlight",
  CellTags->"RegularExpressionToBEFA"],

Cell[TextData[{
  StyleBox["RegularExpressionToBEFA[exp]", "MR"],
  "  computes the BEFA for regular expression ",
  StyleBox["exp", "MR"],
  "."
}], "Usage",
  CellTags->"RegularExpressionToBEFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"SizeOnly", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"RegularExpressionToBEFA"],

Cell[TextData[{
  "Option ",
  StyleBox["SizeOnly->True", "MR"],
  " will return the size of the BEFA without computing it."
}], "Notes",
  CellTags->"RegularExpressionToBEFA"],

Cell["This algorithm uses McNaughton-Yamada automata.", "Notes",
  CellTags->"RegularExpressionToBEFA"],

Cell["\<\
The begin/exit automata will in general have \[Epsilon] \
transitions.\
\>", "Notes",
  CellTags->"RegularExpressionToBEFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["BEFA",
    ButtonData:>"BEFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["RegularExpressionToFA",
    ButtonData:>"RegularExpressionToFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"RegularExpressionToBEFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RegularExpressionToBEFA"],

Cell[TextData[{
  "A regular expression for ",
  Cell[BoxData[
      \(TraditionalForm\`\(\((a + b)\)\^\(\(\ \)\(*\)\)\) 
        b\ \((a + b)\)\^\(\(\ \)\(2\)\)\)]],
  " and the corresponding BEFA."
}], "ExampleText",
  CellTags->"RegularExpressionToBEFA"],

Cell[BoxData[
    \(re = RET[RES[REP[a, b]], b, REP[a, b], REP[a, b]]\)], "Input",
  CellLabel->"In[751]:=",
  CellTags->"RegularExpressionToBEFA"],

Cell[BoxData[
    \(RegularExpressionToBEFA[re]\)], "Input",
  CellLabel->"In[755]:=",
  CellTags->"RegularExpressionToBEFA"],

Cell["Computing the size of the BEFA only.", "ExampleText",
  CellTags->"RegularExpressionToBEFA"],

Cell[BoxData[
    \(RegularExpressionToBEFA[re, SizeOnly \[Rule] True\ ]\)], "Input",
  CellLabel->"In[756]:=",
  CellTags->"RegularExpressionToBEFA"],

Cell["RegularExpressionToFA", "ObjectName",
  CellTags->"RegularExpressionToFA"],

Cell["Function", "Highlight",
  CellTags->"RegularExpressionToFA"],

Cell[TextData[{
  StyleBox["RegularExpressionToFA[exp,K]", "MR"],
  "  computes an FA for regular expression ",
  StyleBox["exp", "MR"],
  " over alphabet ",
  StyleBox["K", "MR"],
  ". "
}], "Usage",
  CellTags->"RegularExpressionToFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"SizeOnly", "\[Rule]", "False", "-"},
          {"EpsElim", "\[Rule]", "True", "-"},
          {"Thompson", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"RegularExpressionToFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Thompson->True", "MR"],
  " will build a Thompson style FA with epsilon-moves. "
}], "Notes",
  CellTags->"RegularExpressionToFA"],

Cell[TextData[{
  "Option ",
  StyleBox["EpsElim->True", "MR"],
  " constructs a Thompson style FA and performs eps-elimination. "
}], "Notes",
  CellTags->"RegularExpressionToFA"],

Cell[TextData[{
  "Option ",
  StyleBox["SizeOnly->True", "MR"],
  " will return the size of the FA without computing it."
}], "Notes",
  CellTags->"RegularExpressionToFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["RegularExpressionToNNFA, RegularExpressionToBEFA.",
    ButtonData:>"RegularExpressionToNNFA, RegularExpressionToBEFA.",
    ButtonStyle->"AddOnsLink"]
}], "Notes",
  CellTags->"RegularExpressionToFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RegularExpressionToFA"],

Cell[TextData[{
  "A regular expression for ",
  Cell[BoxData[
      \(TraditionalForm\`\(\((a + b)\)\^\(\(\ \)\(*\)\)\) 
        b\ \((a + b)\)\^\(\(\ \)\(2\)\)\)]],
  " and the corresponding  FA."
}], "ExampleText",
  CellTags->"RegularExpressionToFA"],

Cell[BoxData[
    \(re = RET[RES[REP[a, b]], b, REP[a, b], REP[a, b]]\)], "Input",
  CellLabel->"In[15]:=",
  CellTags->"RegularExpressionToFA"],

Cell[BoxData[
    \(m\  = \ RegularExpressionToFA[re]\)], "Input",
  CellLabel->"In[17]:=",
  CellTags->"RegularExpressionToFA"],

Cell[BoxData[
    \(LanguageFA[m, \(-5\)]\)], "Input",
  CellLabel->"In[18]:=",
  CellTags->"RegularExpressionToFA"],

Cell["\<\
The machine obtained from Thompson's algorithm, without epsilon \
elimination.\
\>", "ExampleText",
  CellTags->"RegularExpressionToFA"],

Cell[BoxData[
    \(m\  = \ 
      RegularExpressionToFA[re, 2, Thompson \[Rule] True, \ 
        EpsElim \[Rule] False]\)], "Input",
  CellLabel->"In[19]:=",
  CellTags->"RegularExpressionToFA"],

Cell["\<\
Computing the size only, without actually constructing the machine.\
\
\>", "ExampleText",
  CellTags->"RegularExpressionToFA"],

Cell[BoxData[
    \(m\  = \ 
      RegularExpressionToFA[re, 2, SizeOnly \[Rule] True\ ]\)], "Input",
  CellLabel->"In[24]:=",
  CellTags->"RegularExpressionToFA"],

Cell["RegularExpressionToNNFA", "ObjectName",
  CellTags->"RegularExpressionToNNFA"],

Cell["Function", "Highlight",
  CellTags->"RegularExpressionToNNFA"],

Cell[TextData[{
  StyleBox["RegularExpressionToNNFA[exp]", "MR"],
  "  computes the NNFA for regular expression ",
  StyleBox["exp", "MR"],
  "."
}], "Usage",
  CellTags->"RegularExpressionToNNFA"],

Cell["\<\
This algorithm uses McNaughton-Yamada automata (no epsilon \
transitions).\
\>", "Notes",
  CellTags->"RegularExpressionToNNFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["NNFA",
    ButtonData:>"NNFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["RegularExpressionToFA",
    ButtonData:>"RegularExpressionToFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"RegularExpressionToNNFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RegularExpressionToNNFA"],

Cell[TextData[{
  "A regular expression for ",
  Cell[BoxData[
      \(TraditionalForm\`\(\((a + b)\)\^\(\(\ \)\(*\)\)\) 
        b\ \((a + b)\)\^\(\(\ \)\(2\)\)\)]],
  " and the corresponding NNFA."
}], "ExampleText",
  CellTags->"RegularExpressionToNNFA"],

Cell[BoxData[
    \(re = RET[RES[REP[a, b]], b, REP[a, b], REP[a, b]]\)], "Input",
  CellLabel->"In[761]:=",
  CellTags->"RegularExpressionToNNFA"],

Cell[BoxData[
    \(RegularExpressionToNNFA[re]\)], "Input",
  CellLabel->"In[762]:=",
  CellTags->"RegularExpressionToNNFA"],

Cell["Computing the size of the NNFA only.", "ExampleText",
  CellTags->"RegularExpressionToNNFA"],

Cell[BoxData[
    \(RegularExpressionToNNFA[re, SizeOnly \[Rule] True\ ]\)], "Input",
  CellLabel->"In[763]:=",
  CellTags->"RegularExpressionToNNFA"],

Cell["REP", "ObjectName",
  CellTags->"REP"],

Cell["Constructor", "Highlight",
  CellTags->"REP"],

Cell[TextData[{
  StyleBox["REP", "MR"],
  "  is a header for regular expressions, denotes union."
}], "Usage",
  CellTags->"REP"],

Cell[TextData[{
  " ",
  StyleBox["REP", "MR"],
  " is polyadic."
}], "Notes",
  CellTags->"REP"],

Cell[TextData[{
  "See also ",
  ButtonBox["RET",
    ButtonData:>"RET",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["RES",
    ButtonData:>"RES",
    ButtonStyle->"AddOnsLink"],
  StyleBox[",", "Text"],
  ButtonBox[" RegularExpressionToFA",
    ButtonData:>" RegularExpressionToFA",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"REP"],

Cell["Further Examples", "ExampleSection",
  CellTags->"REP"],

Cell[TextData[{
  "A slightly complicated regular expression for the language of all words \
containing an even number of ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  "'s and ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  "'s."
}], "ExampleText",
  CellTags->"REP"],

Cell["\<\
roo = REP[ab,ba]
ree = RES[ REP[aa,bb,RET[roo,RES[REP[aa,bb]],roo]]]\
\>", "Input",
  CellLabel->"In[80]:=",
  CellTags->"REP"],

Cell[BoxData[
    \(m\  = \ RegularExpressionToFA[ree]\)], "Input",
  CellLabel->"In[82]:=",
  CellTags->"REP"],

Cell[BoxData[
    \(LanguageFA[m, \(-6\)]\)], "Input",
  CellLabel->"In[83]:=",
  CellTags->"REP"],

Cell["REQuotient", "ObjectName",
  CellTags->"REQuotient"],

Cell["Function", "Highlight",
  CellTags->"REQuotient"],

Cell[TextData[{
  StyleBox["REQuotient[w,exp]", "MR"],
  "  computes the left quotient of the given rational expression ",
  StyleBox["exp", "MR"],
  " by the word ",
  StyleBox["w", "MR"],
  "."
}], "Usage",
  CellTags->"REQuotient"],

Cell[TextData[{
  StyleBox["REQuotient", "MR"],
  "  computes a rational expression for ",
  Cell[BoxData[
      \(TraditionalForm\`\(w\^\(\(-1\)\(\ \)\)\) L\)]],
  "  where ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  " is the language denoted by the rational expression in the input. Thus, \
only words ",
  Cell[BoxData[
      \(TraditionalForm\`u\)]],
  "  are retained such that  ",
  Cell[BoxData[
      \(TraditionalForm\`w\[VeryThinSpace]u\  \[Element] \ L\)]],
  ". "
}], "Notes",
  CellTags->"REQuotient"],

Cell[TextData[{
  "Note that no attempt at simplification is made (neither for the result nor \
for the intermediate expressions when ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(|\)\(w\)\(|\)\(\(>\)\(1\)\)\)\)]],
  "). Thus, the size of the expressions may grow exponentially."
}], "Notes",
  CellTags->"REQuotient"],

Cell[TextData[{
  "See also ",
  ButtonBox["RegularExpressionToFA",
    ButtonData:>"RegularExpressionToFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"REQuotient"],

Cell["Further Examples", "ExampleSection",
  CellTags->"REQuotient"],

Cell[TextData[{
  "Removing a prefix  ",
  Cell[BoxData[
      FormBox[
        StyleBox["aba",
          FontSlant->"Italic"], TraditionalForm]]],
  " from all words should produce an expression for all words."
}], "ExampleText",
  CellTags->"REQuotient"],

Cell["REQuotient[ aba, RES[REP[a,b] ]]", "Input",
  CellLabel->"In[167]:=",
  CellTags->"REQuotient"],

Cell[TextData[{
  "The last expression ought to simplify to ",
  Cell[BoxData[
      \(TraditionalForm\`\((a + b)\)\^\(\(\ \)\(*\)\)\)]],
  "."
}], "ExampleText",
  CellTags->"REQuotient"],

Cell[BoxData[
    \(RESimplify[%]\)], "Input",
  CellLabel->"In[168]:=",
  CellTags->"REQuotient"],

Cell["RES", "ObjectName",
  CellTags->"RES"],

Cell["Constructor", "Highlight",
  CellTags->"RES"],

Cell[TextData[{
  StyleBox["RES", "MR"],
  "  is a header for regular expressions, denotes Kleene star."
}], "Usage",
  CellTags->"RES"],

Cell[TextData[{
  " ",
  StyleBox["RES", "MR"],
  " is unary."
}], "Notes",
  CellTags->"RES"],

Cell[TextData[{
  "See also ",
  ButtonBox["REP",
    ButtonData:>"REP",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["RET",
    ButtonData:>"RET",
    ButtonStyle->"AddOnsLink"],
  StyleBox[",", "Text"],
  ButtonBox[" RegularExpressionToFA",
    ButtonData:>" RegularExpressionToFA",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"RES"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RES"],

Cell[TextData[{
  "A slightly complicated regular expression for the language of all words \
containing an even number of ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  "'s and ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  "'s."
}], "ExampleText",
  CellTags->"RES"],

Cell["\<\
roo = REP[ab,ba]
ree = RES[ REP[aa,bb,RET[roo,RES[REP[aa,bb]],roo]]]\
\>", "Input",
  CellLabel->"In[418]:=",
  CellTags->"RES"],

Cell[BoxData[
    \(m\  = \ RegularExpressionToFA[ree]\)], "Input",
  CellLabel->"In[420]:=",
  CellTags->"RES"],

Cell[BoxData[
    \(LanguageFA[m, \(-6\)]\)], "Input",
  CellLabel->"In[421]:=",
  CellTags->"RES"],

Cell["\<\
We compute the Parikh vectors to verify that the counts are in fact \
even/even.\
\>", "ExampleText",
  CellTags->"RES"],

Cell[BoxData[
    \(ParikhVector[Flatten[%], 2]\  // \ Union\)], "Input",
  CellLabel->"In[422]:=",
  CellTags->"RES"],

Cell["RESimplify", "ObjectName",
  CellTags->"RESimplify"],

Cell["Function", "Highlight",
  CellTags->"RESimplify"],

Cell[TextData[{
  StyleBox["RESimplify[exp]", "MR"],
  "  tries to simplify the regular expression ",
  StyleBox["exp", "MR"],
  " using the rules defined in ",
  StyleBox["$regexrules", "MR"],
  "."
}], "Usage",
  CellTags->"RESimplify"],

Cell["\<\
Uses only rewrite rules and cannot simplify mildly complicated \
expressions. \
\>", "Notes",
  CellTags->"RESimplify"],

Cell[TextData[{
  "See also ",
  ButtonBox["ArdenSolve",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SubstituteRE",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["$regexrules",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"RESimplify"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RESimplify"],

Cell[TextData[{
  "Very primitive simplifications work.  For example,  ",
  Cell[BoxData[
      \(TraditionalForm\`a\  + \ a\  \[Rule] \ a\)]],
  ". "
}], "ExampleText",
  CellTags->"RESimplify"],

Cell["RESimplify[ RES[REP[a,a]] ]", "Input",
  CellLabel->"In[66]:=",
  CellTags->"RESimplify"],

Cell[TextData[{
  "Or  ",
  Cell[BoxData[
      \(TraditionalForm\`\((a + b)\)\^\(\(*\)\(\ \)\) + \ \[Epsilon]\ \  + \ \
\[EmptySet]\  \[Rule] \ \(\((a + b)\)\^*\)\)]],
  ". "
}], "ExampleText",
  CellTags->"RESimplify"],

Cell["RESimplify[ REP[RES[REP[a,b]],Eps,Empty] ]", "Input",
  CellLabel->"In[68]:=",
  CellTags->"RESimplify"],

Cell["RET", "ObjectName",
  CellTags->"RET"],

Cell["Constructor", "Highlight",
  CellTags->"RET"],

Cell[TextData[{
  StyleBox["RET", "MR"],
  "  is a header for regular expressions, denotes concatenation."
}], "Usage",
  CellTags->"RET"],

Cell[TextData[{
  " ",
  StyleBox["RET", "MR"],
  " is polyadic."
}], "Notes",
  CellTags->"RET"],

Cell[TextData[{
  "See also ",
  ButtonBox["REP",
    ButtonData:>"REP",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["RES",
    ButtonData:>"RES",
    ButtonStyle->"AddOnsLink"],
  StyleBox[",", "Text"],
  ButtonBox[" RegularExpressionToFA",
    ButtonData:>" RegularExpressionToFA",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"RET"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RET"],

Cell[TextData[{
  "A slightly complicated regular expression for the language of all words \
containing an even number of ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  "'s and ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  "'s."
}], "ExampleText",
  CellTags->"RET"],

Cell["\<\
roo = REP[ab,ba]
ree = RES[ REP[aa,bb,RET[roo,RES[REP[aa,bb]],roo]]]\
\>", "Input",
  CellLabel->"In[80]:=",
  CellTags->"RET"],

Cell[BoxData[
    \(m\  = \ RegularExpressionToFA[ree]\)], "Input",
  CellLabel->"In[82]:=",
  CellTags->"RET"],

Cell[BoxData[
    \(LanguageFA[m, \(-6\)]\)], "Input",
  CellLabel->"In[83]:=",
  CellTags->"RET"],

Cell["SubstituteRE", "ObjectName",
  CellTags->"SubstituteRE"],

Cell["Function", "Highlight",
  CellTags->"SubstituteRE"],

Cell[TextData[{
  StyleBox["SubstituteRE[eq1,eq2]", "MR"],
  "  substitutes a linear equation ",
  StyleBox["eq2", "MR"],
  " over the language semiring into ",
  StyleBox["eq1", "MR"],
  ". "
}], "Usage",
  CellTags->"SubstituteRE"],

Cell[TextData[{
  "Mildy useful in conjunction with ",
  StyleBox["ArdenSolve", "MR"],
  "."
}], "Notes",
  CellTags->"SubstituteRE"],

Cell[TextData[{
  "See also ",
  ButtonBox["ArdenSolve",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToEquationsDFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SubstituteRE"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SubstituteRE"],

Cell["\<\
Turn or formatting and generate a small system of equations by \
converting the transition matrix of a DFA. \
\>", "ExampleText",
  CellTags->"SubstituteRE"],

Cell[BoxData[{
    \(ClearAll[X]\), "\n", 
    \(FormatX[]\), "\n", 
    \(m\  = \ CounterDFA[b, 2, 2]\), "\n", 
    \(\({e1, e2, e3, e4}\  = \ ToEquationsDFA[m];\)\), "\n", 
    \({e1, e2, e3, e4}\ \  // TableForm\)}], "Input",
  CellLabel->"In[44]:=",
  CellTags->"SubstituteRE"],

Cell["The result of substituting one equation into another.", "ExampleText",
  CellTags->"SubstituteRE"],

Cell[BoxData[
    \(SubstituteRE[e1, e2]\)], "Input",
  CellLabel->"In[51]:=",
  CellTags->"SubstituteRE"],

Cell[TextData[{
  "Using Arden's lemma one can express ",
  Cell[BoxData[
      \(TraditionalForm\`X\_1\)]],
  " in terms of ",
  Cell[BoxData[
      \(TraditionalForm\`X\_2\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`X\_3\)]],
  "."
}], "ExampleText",
  CellTags->"SubstituteRE"],

Cell[BoxData[{
    \(ArdenSolve[%]\), "\n", 
    \(%\  // \ PrintRE\)}], "Input",
  CellLabel->"In[52]:=",
  CellTags->"SubstituteRE"],

Cell["We can verify the last equation via the transition function.", \
"ExampleText",
  CellTags->"SubstituteRE"],

Cell[BoxData[{
    \(TransitionFunctionFA[m, \[Delta]]\), "\n", 
    \(\[Delta][1, aaaaaaaaaaaaaaaaa]\), "\n", 
    \(\[Delta][1, ba]\), "\n", 
    \(\[Delta][1, bb]\)}], "Input",
  CellLabel->"In[58]:=",
  CellTags->"SubstituteRE"],

Cell["ToRegExp", "ObjectName",
  CellTags->"ToRegExp"],

Cell["Function", "Highlight",
  CellTags->"ToRegExp"],

Cell[TextData[{
  StyleBox["ToRegExp[reg]", "MR"],
  "  converts an expression containing arithmetic operators +  and ** (",
  StyleBox["Plus", "MR"],
  " and ",
  StyleBox["NonCommutativeMultiply", "MR"],
  ") into the corresponding regular expression containing only headers ",
  StyleBox["REP", "MR"],
  ", ",
  StyleBox["RET", "MR"],
  " and ",
  StyleBox["RES", "MR"],
  ". "
}], "Usage",
  CellTags->"ToRegExp"],

Cell["\<\
This conversion is used automatically in all commands operating on \
regular expressions.\
\>", "Notes",
  CellTags->"ToRegExp"],

Cell[TextData[{
  "See also ",
  ButtonBox["RegularExpressionToFA,",
    ButtonData:>"RegularExpressionToFA,",
    ButtonStyle->"AddOnsLink"],
  " ",
  ButtonBox["REP",
    ButtonData:>"REP",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["RET",
    ButtonData:>"RET",
    ButtonStyle->"AddOnsLink"],
  " , ",
  ButtonBox["RES",
    ButtonData:>"RES",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["RESimplify",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToRegExp"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToRegExp"],

Cell["\<\
Using arithmetic notation makes it somewhat easier to write \
complicated regular expressions. \
\>", "ExampleText",
  CellTags->"ToRegExp"],

Cell["RES[ aa + bb + (ab+ba)**RES[aa+bb]**(ab+ba)]", "Input",
  CellLabel->"In[71]:=",
  CellTags->"ToRegExp"],

Cell[BoxData[
    \(ToRegExp[%]\)], "Input",
  CellLabel->"In[72]:=",
  CellTags->"ToRegExp"],

Cell[BoxData[
    \(aa\  + \ aa\  // \ ToRegExp\)], "Input",
  CellTags->"ToRegExp"],

Cell["BFunctionToCA", "ObjectName",
  CellTags->"BFunctionToCA"],

Cell["Function", "Highlight",
  CellTags->"BFunctionToCA"],

Cell[TextData[{
  StyleBox["BFunctionToCA[f,w]", "MR"],
  "  constructs a CA of width ",
  StyleBox["w", "MR"],
  " over alphabet ",
  StyleBox["{0,1}", "MR"],
  " whose local map is given by ",
  StyleBox["f", "MR"],
  ", a Boolean expression in variables ",
  StyleBox["x1", "MR"],
  ", ",
  StyleBox["x2", "MR"],
  ", \[Ellipsis] "
}], "Usage",
  CellTags->"BFunctionToCA"],

Cell[TextData[{
  StyleBox["Plus", "MR"],
  " and ",
  StyleBox["Times", "MR"],
  " can be used instead of logical ",
  StyleBox["Or", "MR"],
  " and ",
  StyleBox["And", "MR"],
  ".   Negated variables can be denoted by ",
  StyleBox["y1", "MR"],
  " and so forth. "
}], "Notes",
  CellTags->"BFunctionToCA"],

Cell[TextData[{
  "Subscripted variables ",
  Cell[BoxData[
      \(TraditionalForm\`x\_i\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`y\_i\)]],
  " can be used instead."
}], "Notes",
  CellTags->"BFunctionToCA"],

Cell[TextData[{
  "See ",
  ButtonBox["PolynomialToCA",
    ButtonData:>"PolynomialToCA",
    ButtonStyle->"AddOnsLink"],
  "  for conversion of polynomials modulo ",
  Cell[BoxData[
      \(TraditionalForm\`k\)]],
  " to cellular automata."
}], "Notes",
  CellTags->"BFunctionToCA"],

Cell[TextData[{
  "See also  ",
  ButtonBox["FunctionToCA",
    ButtonData:>"FunctionToCA",
    ButtonStyle->"AddOnsLink"],
  "  and ",
  ButtonBox["RulesToCA",
    ButtonData:>"RulesToCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"BFunctionToCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"BFunctionToCA"],

Cell["\<\
Exclusive or on the left right neighbors produces ECA with rule \
number 90.\
\>", "ExampleText",
  CellTags->"BFunctionToCA"],

Cell[BoxData[
    \(BFunctionToCA[\ Xor[x\_1, x\_3], \ 3\ ]\)], "Input",
  CellLabel->"In[46]:=",
  CellTags->"BFunctionToCA"],

Cell["Another elementary CA. ", "ExampleText",
  CellTags->"BFunctionToCA"],

Cell[BoxData[{
    \(BFunctionToCA[x1 + \((\(! x2\))\)\ x3, 3]\), "\n", 
    \(PrintCA[%]\)}], "Input",
  CellLabel->"In[49]:=",
  CellTags->"BFunctionToCA"],

Cell[TextData[{
  "The same CA built using the convention  ",
  Cell[BoxData[
      \(TraditionalForm\`y\_i \[Rule] \ \[Not] x\_i\)]],
  ". "
}], "ExampleText",
  CellTags->"BFunctionToCA"],

Cell[BoxData[
    \(BFunctionToCA[x\_1 + y\_2\ x\_3, 3]\)], "Input",
  CellLabel->"In[54]:=",
  CellTags->"BFunctionToCA"],

Cell["A binary CA of width 4.", "ExampleText",
  CellTags->"BFunctionToCA"],

Cell[BoxData[{
    \(BFunctionToCA[x\_1\ x\_3 + \ \(y\_2\) y\_4, 4]\), "\n", 
    \(PrintCA[%]\)}], "Input",
  CellLabel->"In[100]:=",
  CellTags->"BFunctionToCA"],

Cell["CA", "ObjectName",
  CellTags->"CA"],

Cell["Constructor", "Highlight",
  CellTags->"CA"],

Cell[TextData[{
  StyleBox["CA[w,k,r]", "MR"],
  "  denotes a linear cellular automaton with width ",
  StyleBox["w", "MR"],
  ", alphabet ",
  StyleBox["k", "MR"],
  " and local rule given by the rule number ",
  StyleBox["r", "MR"],
  ". "
}], "Usage",
  CellTags->"CA"],

Cell[TextData[{
  StyleBox["CA", "MR"],
  " is a constructor and will automatically supplement some missing parameter \
values.  For elementary cellular automata it suffices to specify only the \
rule number ",
  Cell[BoxData[
      \(TraditionalForm\`r\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`0\  \[LessEqual] \ r\  < \ 256\)]],
  ". "
}], "Notes",
  CellTags->"CA"],

Cell[TextData[{
  "See also  ",
  ButtonBox["FunctionToCA",
    ButtonStyle->"AddOnsLink"],
  "  and  ",
  ButtonBox["BFunctionToCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"CA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"CA"],

Cell["\<\
A cellular automaton of width 3 over a four-letter alphabet and its \
rule-table.\
\>", "ExampleText",
  CellTags->"CA"],

Cell[BoxData[{
    \(\(C = 
        CA[3, \(-4\), 316801401093077184121484526515959168785];\)\), "\n", 
    \(PrintCA[C]\)}], "Input",
  CellLabel->"In[119]:=",
  CellTags->"CA"],

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`C\)]],
  " has the rather unusual property of being reversible on infinite \
configurations. The following plot uses cyclic boundary conditions."
}], "ExampleText",
  CellTags->"CA"],

Cell[BoxData[{
    \(\(X\  = \ 
        Join[Table[0, {50}], 
          Table[Random[Integer, {0, 3}], {50}]];\)\), "\n", 
    \(\(cols\  = \ {0 \[Rule] \ Cyan, 1 \[Rule] Blue, 2 \[Rule] Red, 
          3 \[Rule] Yellow};\)\), "\n", 
    \(\(EvolutionCA[C, X, 100, RasterStyle \[Rule] cols];\)\)}], "Input",
  CellLabel->"In[53]:=",
  CellTags->"CA"],

Cell["\<\
Elementary cellular automata automatically supply the width and \
alphabet field.\
\>", "ExampleText",
  CellTags->"CA"],

Cell[" CA[110]", "Input",
  CellLabel->"In[118]:=",
  CellTags->"CA"],

Cell["ClassifyCA", "ObjectName",
  CellTags->"ClassifyCA"],

Cell["Function", "Highlight",
  CellTags->"ClassifyCA"],

Cell[TextData[{
  StyleBox["ClassifyCA[C]", "MR"],
  " classifies a cellular automaton ",
  StyleBox["C", "MR"],
  " according to properties of the global map. The algorithm distinguishes \
between injective, open, and surjective global maps."
}], "Usage",
  CellTags->"ClassifyCA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "True"},
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ClassifyCA"],

Cell[TextData[{
  "Return values are as follows: \n",
  Cell[BoxData[GridBox[{
          {"0", 
            StyleBox["nothing",
              "MR"]},
          {"1", 
            StyleBox["surjective",
              "MR"]},
          {"2", 
            StyleBox["open",
              "MR"]},
          {"3", 
            StyleBox["injective",
              "MR"]}
          }]]]
}], "Notes",
  CellTags->"ClassifyCA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  "  returns descriptive strings rather than an integer."
}], "Notes",
  CellTags->"ClassifyCA"],

Cell["\<\
Note that for global maps of cellular automata on infinite \
configurations injective implies open, which in turn implies \
surjective.\
\>", "Notes",
  CellTags->"ClassifyCA"],

Cell["\<\
The algorithm uses the product automaton of the corresponding de \
Bruijn automaton and is thus quadratic.\
\>", "Notes",
  CellTags->"ClassifyCA"],

Cell[TextData[{
  "See also  ",
  ButtonBox["SquareSA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["AmbiguousQFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ClassifyCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ClassifyCA"],

Cell["Some elementary cellular automata using verbose mode.", "ExampleText",
  CellTags->"ClassifyCA"],

Cell[BoxData[
    \(ClassifyCA[CA[110], Full \[Rule] True\ ]\)], "Input",
  CellLabel->"In[16]:=",
  CellTags->"ClassifyCA"],

Cell[BoxData[
    \(ClassifyCA[CA[30], Full \[Rule] True]\)], "Input",
  CellLabel->"In[17]:=",
  CellTags->"ClassifyCA"],

Cell[BoxData[
    \(ClassifyCA[CA[90], Full \[Rule] True]\)], "Input",
  CellLabel->"In[18]:=",
  CellTags->"ClassifyCA"],

Cell[BoxData[
    \(ClassifyCA[CA[51], Full \[Rule] True]\)], "Input",
  CellLabel->"In[19]:=",
  CellTags->"ClassifyCA"],

Cell["\<\
A CA of width 2 over a 4-symbol alphabet.  The output is an integer \
indicating the type of the automaton.\
\>", "ExampleText",
  CellTags->"ClassifyCA"],

Cell[BoxData[
    \(ClassifyCA[CA[2, \(-4\), 508289460]]\)], "Input",
  CellLabel->"In[20]:=",
  CellTags->"ClassifyCA"],

Cell["\<\
This output format is in particular more convenient of tables.  \
Here are all elementary CAs.\
\>", "ExampleText",
  CellTags->"ClassifyCA"],

Cell[BoxData[
    \(\(Table[\ ClassifyCA[CA[r]], \ {r, 0, 255}\ ]\  // \ 
        Frequencies\)\  // \ TableForm\)], "Input",
  CellLabel->"In[21]:=",
  CellTags->"ClassifyCA"],

Cell["ComposeCA", "ObjectName",
  CellTags->"ComposeCA"],

Cell["Function", "Highlight",
  CellTags->"ComposeCA"],

Cell[TextData[{
  StyleBox["ComposeCA[C1,C2]", "MR"],
  "  composes cellular automata ",
  StyleBox["C1", "MR"],
  " and ",
  StyleBox["C2", "MR"],
  " .  The cellular automata are assumed to be over the same alphabet."
}], "Usage",
  CellTags->"ComposeCA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ComposeCA"],

Cell[TextData[{
  "Composition is in diagrammatic order, so that first the local rule of ",
  Cell[BoxData[
      \(TraditionalForm\`C\_1\)]],
  " is applied, and then the local rule of ",
  Cell[BoxData[
      \(TraditionalForm\`C\_2\)]],
  ". "
}], "Notes",
  CellTags->"ComposeCA"],

Cell[TextData[{
  "The width of the resulting automaton is ",
  Cell[BoxData[
      \(TraditionalForm\`w\_1 + w\_2 - 1\)]],
  ". "
}], "Notes",
  CellTags->"ComposeCA"],

Cell[TextData[{
  "See also ",
  ButtonBox["IterateCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ComposeCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ComposeCA"],

Cell["A hybrid automaton composed of ECA rules 97 and 30.", "ExampleText",
  CellTags->"ComposeCA"],

Cell[BoxData[
    \(cc = ComposeCA[CA[97], CA[30]]\)], "Input",
  CellLabel->"In[48]:=",
  CellTags->"ComposeCA"],

Cell[BoxData[
    \(\(EvolutionCA[cc, 100, \ Boundary \[Rule] Fixed\ ];\)\)], "Input",
  CellLabel->"In[49]:=",
  CellTags->"ComposeCA"],

Cell["Cyclic", "ObjectName",
  CellTags->"Cyclic"],

Cell["Option", "Highlight",
  CellTags->"Cyclic"],

Cell[TextData[{
  StyleBox["Cyclic", "MR"],
  "  is a possible value for option ",
  StyleBox["Boundary", "MR"],
  " in ",
  StyleBox["ToBlocks", "MR"],
  " and various operations on cellular automata."
}], "Usage",
  CellTags->"Cyclic"],

Cell[TextData[{
  "See ",
  ButtonBox["ToBlocks",
    ButtonData:>"ToBlocks",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["EvolutionCA",
    ButtonData:>"EvolutionCA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["OrbitCA",
    ButtonData:>"OrbitCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Cyclic"],

Cell["EvolutionCA", "ObjectName",
  CellTags->"EvolutionCA"],

Cell["Function", "Highlight",
  CellTags->"EvolutionCA"],

Cell[TextData[{
  StyleBox["EvolutionCA[C,X,t,opts]", "MR"],
  "  plots t steps in the evolution of pattern X under the global map of \
cellular automaton C. "
}], "Usage",
  CellTags->"EvolutionCA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Boundary", "\[Rule]", "Cyclic", "-"},
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"EvolutionCA"],

Cell[TextData[{
  "Option ",
  StyleBox["Boundary->Fixed", "MR"],
  " uses fixed boundary conditions."
}], "Notes",
  CellTags->"EvolutionCA"],

Cell[TextData[{
  "The default for ",
  StyleBox["X", "MR"],
  " is a one-point seed of size ",
  Cell[BoxData[
      \(TraditionalForm\`2  t + 1\)]],
  ", generated by ",
  ButtonBox["SeedConfiguration",
    ButtonData:>"SeedConfiguration",
    ButtonStyle->"AddOnsLink"],
  ". "
}], "Notes",
  CellTags->"EvolutionCA"],

Cell[TextData[{
  "The default for ",
  StyleBox["t", "MR"],
  " is 50."
}], "Notes",
  CellTags->"EvolutionCA"],

Cell[TextData[{
  "Uses ",
  ButtonBox["PlotMatrix",
    ButtonStyle->"AddOnsLink"],
  "  to render the graphics."
}], "Notes",
  CellTags->"EvolutionCA"],

Cell[TextData[{
  "See also ",
  Cell[BoxData[
      FormBox[
        ButtonBox["GlobalMapCA",
          ButtonData:>"GlobalMapCA",
          ButtonStyle->"AddOnsLink"], TraditionalForm]]],
  " and ",
  ButtonBox["OrbitCA",
    ButtonData:>"OrbitCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"EvolutionCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"EvolutionCA"],

Cell["A classical example: additive rule 90.", "ExampleText",
  CellTags->"EvolutionCA"],

Cell["EvolutionCA[CA[90]];", "Input",
  CellLabel->"In[665]:=",
  CellTags->"EvolutionCA"],

Cell["\<\
The same CA with different boundary conditions, and a different \
initial configuration. The orbit ends in the fixed point \"all-zeros\".\
\>", \
"ExampleText",
  CellTags->"EvolutionCA"],

Cell[BoxData[{
    \(\(X = SeedConfiguration[{1, 1, 1}, 15];\)\), "\n", 
    \(\(EvolutionCA[CA[90], X, 10, \ Boundary \[Rule] Fixed, 
        GridLines \[Rule] True];\)\)}], "Input",
  CellLabel->"In[786]:=",
  CellTags->"EvolutionCA"],

Cell["DependencyCA", "ObjectName",
  CellTags->"DependencyCA"],

Cell["Function", "Highlight",
  CellTags->"DependencyCA"],

Cell[TextData[{
  StyleBox["DependencyCA[C]", "MR"],
  "  returns a list of the indices of the variables upon which the local map \
of cellular automaton ",
  StyleBox["C", "MR"],
  " depends. "
}], "Usage",
  CellTags->"DependencyCA"],

Cell[TextData[{
  "See also ",
  ButtonBox["Dependency",
    ButtonData:>"Dependency",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DependencyCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DependencyCA"],

Cell["\<\
The following cellular automaton has width 4, but the local map \
depends only on arguments 1 and 3. \
\>", "ExampleText",
  CellTags->"DependencyCA"],

Cell[BoxData[{
    \(c = CA[4, \(-2\), 13260]\), "\n", 
    \(DependencyCA[c]\)}], "Input",
  CellLabel->"In[2]:=",
  CellTags->"DependencyCA"],

Cell[BoxData[
    \(PrintCA[c]\)], "Input",
  CellLabel->"In[4]:=",
  CellTags->"DependencyCA"],

Cell["FSurjectiveCA", "ObjectName",
  CellTags->"FSurjectiveCA"],

Cell["Function", "Highlight",
  CellTags->"FSurjectiveCA"],

Cell[TextData[{
  StyleBox["FSurjectiveCA[C]", "MR"],
  "  tests whether a given surjective CA ",
  StyleBox["C", "MR"],
  "  is  surjective on all finite configurations."
}], "Usage",
  CellTags->"FSurjectiveCA"],

Cell[TextData[{
  " The cellular automaton ",
  StyleBox["C", "MR"],
  "  must be quiescent (a configuration of all 0's is mapped to all 0's)."
}], "Notes",
  CellTags->"FSurjectiveCA"],

Cell[TextData[{
  " ",
  Cell[BoxData[
      \(TraditionalForm\`F\)]],
  "-surjectivity is equivalent to the preimage of ",
  StyleBox["0",
    FontWeight->"Bold"],
  " being ",
  StyleBox["0",
    FontWeight->"Bold"],
  ". The latter property can be tested in the de Bruijn graph of the CA."
}], "Notes",
  CellTags->"FSurjectiveCA"],

Cell[TextData[{
  "See also ",
  ButtonBox["ClassifyCA",
    ButtonStyle->"AddOnsLink"],
  "  and  ",
  ButtonBox["ToWelchCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"FSurjectiveCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FSurjectiveCA"],

Cell["\<\
By compactness, F-surjectivity implies surjectivity. We determine \
which surjective elementary CAs are F-surjective.\
\>", "ExampleText",
  CellTags->"FSurjectiveCA"],

Cell[BoxData[
    \(\(surj\  = \ {30, 60, 86, 90, 102, 106, 120, 150, 154, 166, 170, 180, 
          204, 210, 240};\)\)], "Input",
  CellLabel->"In[417]:=",
  CellTags->"FSurjectiveCA"],

Cell[BoxData[
    \(Select[\ surj, \ FSurjectiveCA[CA[#]] &\ ]\)], "Input",
  CellLabel->"In[418]:=",
  CellTags->"FSurjectiveCA"],

Cell["FunctionToCA", "ObjectName",
  CellTags->"FunctionToCA"],

Cell["Function", "Highlight",
  CellTags->"FunctionToCA"],

Cell[TextData[{
  StyleBox["FunctionToCA[f,w,k]", "MR"],
  "  constructs a cellular automaton of width ",
  StyleBox["w", "MR"],
  " over the digit alphabet of size ",
  StyleBox["k", "MR"],
  " whose local map is given by ",
  StyleBox["f", "MR"],
  "."
}], "Usage",
  CellTags->"FunctionToCA"],

Cell[TextData[{
  " The input function ",
  StyleBox["f", "MR"],
  " must be a ",
  StyleBox["w", "MR"],
  "-ary function of integer arguments and return integer results in the \
proper range. "
}], "Notes",
  CellTags->"FunctionToCA"],

Cell[TextData[{
  "See also ",
  ButtonBox["BFunctionToCA",
    ButtonData:>"BFunctionToCA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["PolynomialToCA",
    ButtonData:>"PolynomialToCA",
    ButtonStyle->"AddOnsLink"],
  "  and ",
  ButtonBox["RulesToCA",
    ButtonData:>"RulesToCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"FunctionToCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FunctionToCA"],

Cell["\<\
Modular arithmetic can be used to construct cellular \
automata.\
\>", "ExampleText",
  CellTags->"FunctionToCA"],

Cell[BoxData[
    \(c = FunctionToCA[Mod[\(+##1\), 3] &, 3, \(-3\)]\)], "Input",
  CellLabel->"In[421]:=",
  CellTags->"FunctionToCA"],

Cell["These automata have open global maps. ", "ExampleText",
  CellTags->"FunctionToCA"],

Cell[BoxData[
    \(ClassifyCA[c]\)], "Input",
  CellLabel->"In[422]:=",
  CellTags->"FunctionToCA"],

Cell[BoxData[
    \(\(EvolutionCA[c, 50];\)\)], "Input",
  CellLabel->"In[106]:=",
  CellTags->"FunctionToCA"],

Cell["GlobalMapCA", "ObjectName",
  CellTags->"GlobalMapCA"],

Cell["Function", "Highlight",
  CellTags->"GlobalMapCA"],

Cell[TextData[{
  StyleBox["GlobalMapCA[f,C]", "MR"],
  "  assigns the global map of cellular automaton ",
  StyleBox["C", "MR"],
  " to symbol ",
  StyleBox["f", "MR"],
  ". Valid input for ",
  StyleBox["f", "MR"],
  " is a  list of integers in the appropriate range."
}], "Usage",
  CellTags->"GlobalMapCA"],

Cell[TextData[{
  "Options\n",
  Cell[BoxData[GridBox[{
          {"Boundary", "\[Rule]", "Cyclic", "-"},
          {"WordMap", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"GlobalMapCA"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["HoldFirst", "MR"],
  "."
}], "Notes",
  CellTags->"GlobalMapCA"],

Cell[TextData[{
  "Option ",
  StyleBox["Boundary->Fixed", "MR"],
  " uses fixed boundary conditions."
}], "Notes",
  CellTags->"GlobalMapCA"],

Cell[TextData[{
  "Option ",
  StyleBox["WordMap->True", "MR"],
  " can be used to generate a function that operates on strings rather than \
lists."
}], "Notes",
  CellTags->"GlobalMapCA"],

Cell[TextData[{
  "See also ",
  ButtonBox["LocalMapCA",
    ButtonData:>"LocalMapCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"GlobalMapCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"GlobalMapCA"],

Cell["A well-known additive cellular automaton: rule 90.", "ExampleText",
  CellTags->"GlobalMapCA"],

Cell[BoxData[
    \(GlobalMapCA[f, CA[90]]\)], "Input",
  CellLabel->"In[426]:=",
  CellTags->"GlobalMapCA"],

Cell[BoxData[
    \(\(\(NestList[\ f, \ {0, 0, 0, 1, 0, 0, 0}, \ 10\ ]\)\(\ \)\)\)], "Input",\

  CellLabel->"In[427]:=",
  CellTags->"GlobalMapCA"],

Cell[BoxData[
    \(TableForm[%, TableSpacing \[Rule] {0, 0}]\)], "Input",
  CellLabel->"In[271]:=",
  CellTags->"GlobalMapCA"],

Cell["We can test additivity computationally.", "ExampleText",
  CellTags->"GlobalMapCA"],

Cell[BoxData[{
    \(\(X\  = \ Table[Random[Integer, {0, 1}], {50}];\)\), "\n", 
    \(\(Y\  = \ Table[Random[Integer, {0, 1}], {50}];\)\), "\n", 
    \(Mod[\ f[X] + f[Y], 2]\  === f[Mod[X + Y, 2]]\)}], "Input",
  CellLabel->"In[276]:=",
  CellTags->"GlobalMapCA"],

Cell["InverseCA", "ObjectName",
  CellTags->"InverseCA"],

Cell["Function", "Highlight",
  CellTags->"InverseCA"],

Cell[TextData[{
  StyleBox["InverseCA[C]", "MR"],
  "  computes the inverse of a reversible cellular automaton ",
  StyleBox["C", "MR"],
  "."
}], "Usage",
  CellTags->"InverseCA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"},
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"InverseCA"],

Cell[TextData[{
  "The algorithm expects ",
  StyleBox["C", "MR"],
  " to be reversible,  it does not check."
}], "Notes",
  CellTags->"InverseCA"],

Cell[TextData[{
  "Option  ",
  StyleBox["Full->True", "MR"],
  "  does not apply the operation ",
  ButtonBox["SimplifyCA",
    ButtonStyle->"AddOnsLink"],
  ". "
}], "Notes",
  CellTags->"InverseCA"],

Cell[TextData[{
  "See also ",
  ButtonBox["WelchCA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DefiniteQFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"InverseCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"InverseCA"],

Cell["Verify that the CA below is reversible.", "ExampleText",
  CellTags->"InverseCA"],

Cell[BoxData[{
    \(\(C = CA[3915, 4, \(-2\)];\)\), "\n", 
    \(ClassifyCA[C]\)}], "Input",
  CellLabel->"In[934]:=",
  AspectRatioFixed->True,
  CellTags->"InverseCA"],

Cell["\<\
Compute its inverse and verify that the composition of the two CAs \
is the identity.\
\>", "ExampleText",
  CellTags->"InverseCA"],

Cell[BoxData[{
    \(Ci = InverseCA[C]\), "\n", 
    \(CC = ComposeCA[C, Ci]\), "\n", 
    \(SimplifyCA[CC]\)}], "Input",
  CellLabel->"In[936]:=",
  AspectRatioFixed->True,
  CellTags->"InverseCA"],

Cell["IterateCA", "ObjectName",
  CellTags->"IterateCA"],

Cell["Function", "Highlight",
  CellTags->"IterateCA"],

Cell[TextData[{
  StyleBox["IterateCA[C,t]", "MR"],
  "  returns the cellular automaton corresponding to iterating ",
  StyleBox["C", "MR"],
  "  ",
  StyleBox["t", "MR"],
  "-times. "
}], "Usage",
  CellTags->"IterateCA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[RuleDelayed]", "$defaultnormalize", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"IterateCA"],

Cell[TextData[{
  "Note that iteration produces an exponential blow-up in the size of the \
cellular automaton: the width of the output is ",
  Cell[BoxData[
      \(TraditionalForm\`t\ \((w - 1)\) + 1\)]],
  ", where ",
  Cell[BoxData[
      \(TraditionalForm\`w\)]],
  " is the width of ",
  Cell[BoxData[
      \(TraditionalForm\`C\)]],
  "."
}], "Notes",
  CellTags->"IterateCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"IterateCA"],

Cell["\<\
Iterating an CA with an open global map will produce another CA \
with an open global map.\
\>", "ExampleText",
  CellTags->"IterateCA"],

Cell["c = IterateCA[ CA[90], 3 ]", "Input",
  CellLabel->"In[182]:=",
  CellTags->"IterateCA"],

Cell[BoxData[{
    \(\($RecursionLimit = 10000;\)\), "\n", 
    \(ClassifyCA[c]\)}], "Input",
  CellLabel->"In[183]:=",
  CellTags->"IterateCA"],

Cell["LocalMapCA", "ObjectName",
  CellTags->"LocalMapCA"],

Cell["Function", "Highlight",
  CellTags->"LocalMapCA"],

Cell[TextData[{
  StyleBox["LocalMapCA[f,C]", "MR"],
  "  assigns the local map of cellular automaton ",
  StyleBox["C", "MR"],
  " to symbol ",
  StyleBox["f", "MR"],
  ". "
}], "Usage",
  CellTags->"LocalMapCA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"WordMap", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"LocalMapCA"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["HoldFirst", "MR"],
  "."
}], "Notes",
  CellTags->"LocalMapCA"],

Cell[TextData[{
  "Option ",
  StyleBox["Boundary->Fixed", "MR"],
  " uses fixed boundary conditions."
}], "Notes",
  CellTags->"LocalMapCA"],

Cell[TextData[{
  "Option ",
  StyleBox["WordMap->True", "MR"],
  " can be used to generate a function that operates on strings rather than \
lists."
}], "Notes",
  CellTags->"LocalMapCA"],

Cell[TextData[{
  "See also ",
  ButtonBox["GlobalMapCA",
    ButtonData:>"GlobalMapCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"LocalMapCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"LocalMapCA"],

Cell["A well-known additive cellular automaton: rule 90.", "ExampleText",
  CellTags->"LocalMapCA"],

Cell[BoxData[
    \(LocalMapCA[f, CA[90]]\)], "Input",
  CellLabel->"In[473]:=",
  CellTags->"LocalMapCA"],

Cell[BoxData[
    \(Apply[\ f, \ Tuples[3, {0, 1}], \ {1}]\)], "Input",
  CellLabel->"In[474]:=",
  CellTags->"LocalMapCA"],

Cell["\<\
The global map of the cellular automaton is obtained by applying \
the local one in parallel to all blocks of length 3.\
\>", "ExampleText",
  CellTags->"LocalMapCA"],

Cell[BoxData[{
    \(\(X\  = \ {0, 0, 0, 1, 0, 0, 0};\)\), "\n", 
    \(Apply[\ f, Partition[X, 3, 1, 2], \ {1}]\)}], "Input",
  CellLabel->"In[61]:=",
  CellTags->"LocalMapCA"],

Cell["NumberOfCA", "ObjectName",
  CellTags->"NumberOfCA"],

Cell["Function", "Highlight",
  CellTags->"NumberOfCA"],

Cell[TextData[{
  StyleBox["NumberOfCA[w,k]", "MR"],
  "  returnes the number of cellular automata of width ",
  StyleBox["w", "MR"],
  " and and alphabet of size ",
  StyleBox["k", "MR"],
  ". "
}], "Usage",
  CellTags->"NumberOfCA"],

Cell[TextData[{
  "Returns ",
  Cell[BoxData[
      \(TraditionalForm\`k\^\(k\^w\)\)]],
  "."
}], "Notes",
  CellTags->"NumberOfCA"],

Cell[TextData[{
  "See also ",
  ButtonBox["NumberOfTotalisticCA",
    ButtonData:>"NumberOfTotalisticCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"NumberOfCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"NumberOfCA"],

Cell["Super-exponential blow-up.", "ExampleText",
  CellTags->"NumberOfCA"],

Cell[" Table[{w,NumberOfCA[w,2]}, {w,5} ] // TableForm", "Input",
  CellLabel->"In[62]:=",
  CellTags->"NumberOfCA"],

Cell["NumberOfTotalisticCA", "ObjectName",
  CellTags->"NumberOfTotalisticCA"],

Cell["Function", "Highlight",
  CellTags->"NumberOfTotalisticCA"],

Cell[TextData[{
  StyleBox["NumberOfTotalisticCA[w,K]", "MR"],
  "  determinces the number of totalistic cellular automata of width ",
  StyleBox["w", "MR"],
  " and and alphabet of size ",
  StyleBox["k", "MR"],
  ". "
}], "Usage",
  CellTags->"NumberOfTotalisticCA"],

Cell[TextData[{
  "Returns ",
  Cell[BoxData[
      \(TraditionalForm\`k\^\(w(k - 1) + 1\)\)]],
  "."
}], "Notes",
  CellTags->"NumberOfTotalisticCA"],

Cell[TextData[{
  "See also  ",
  ButtonBox["NumberOfCA",
    ButtonData:>"NumberOfCA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToTotalisticRuleCA",
    ButtonData:>"ToTotalisticRuleCA",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["TotalisticRuleToCA",
    ButtonData:>"TotalisticRuleToCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"NumberOfTotalisticCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"NumberOfTotalisticCA"],

Cell["Exponential blow-up.", "ExampleText",
  CellTags->"NumberOfTotalisticCA"],

Cell[" Table[{w,NumberOfTotalisticCA[w,3]}, {w,5} ] // TableForm", "Input",
  CellLabel->"In[70]:=",
  CellTags->"NumberOfTotalisticCA"],

Cell["OrbitCA", "ObjectName",
  CellTags->"OrbitCA"],

Cell["Function", "Highlight",
  CellTags->"OrbitCA"],

Cell[TextData[{
  StyleBox["OrbitCA[C,pat,t,opts]", "MR"],
  "  computes the orbit of pattern ",
  StyleBox["pat", "MR"],
  " under application of the global map of cellular automaton ",
  StyleBox["C", "MR"],
  "  up to time ",
  StyleBox["t", "MR"],
  ". "
}], "Usage",
  CellTags->"OrbitCA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Boundary", "\[Rule]", "Cyclic", "-"},
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"},
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"OrbitCA"],

Cell[TextData[{
  "Option ",
  StyleBox["Boundary->Fixed", "MR"],
  " uses fixed boundary conditions."
}], "Notes",
  CellTags->"OrbitCA"],

Cell[TextData[{
  "The output is a matrix of integers that can be rendered by ",
  ButtonBox["PlotMatrix",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "MR"]
}], "Notes",
  CellTags->"OrbitCA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  "  produces ordinary matrices, rather than ",
  ButtonBox["PackedArray",
    ButtonStyle->"RefGuideLink"],
  " objects as by default.  "
}], "Notes",
  CellTags->"OrbitCA"],

Cell["\<\
At present, the input configuration can be given as a string rather \
than a list, in which case the orbit is returnes as a list of strings.  This \
leads to considerable memory savings compared to an ordinary matrix of \
integers.  However, packed arrays may make this feature obsolete in the \
future.\
\>", "Notes",
  CellTags->"OrbitCA"],

Cell[TextData[{
  "See also ",
  ButtonBox["GlobalMapCA",
    ButtonData:>"GlobalMapCA",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["EvolutionCA",
    ButtonData:>"EvolutionCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"OrbitCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"OrbitCA"],

Cell["\<\
A classical example: additive rule 90 on a random initial \
configuration.\
\>", "ExampleText",
  CellTags->"OrbitCA"],

Cell["\<\
X = Table[ Random[Integer,{0,1}],{50} ];
orb = OrbitCA[ CA[90], X, 100 ];\
\>", "Input",
  CellLabel->"In[481]:=",
  CellTags->"OrbitCA"],

Cell[BoxData[
    \(orb\  // \ Dimensions\)], "Input",
  CellLabel->"In[483]:=",
  CellTags->"OrbitCA"],

Cell["\<\
The relative frequencies of 0's and 1's is approximately 1/2.\
\>", \
"ExampleText",
  CellTags->"OrbitCA"],

Cell[BoxData[
    \(\(Flatten[orb]\  // \ Frequencies\)\  // \ TableForm\)], "Input",
  CellLabel->"In[20]:=",
  CellTags->"OrbitCA"],

Cell["Padding", "ObjectName",
  CellTags->"Padding"],

Cell["Option", "Highlight",
  CellTags->"Padding"],

Cell[TextData[{
  StyleBox["Padding", "MR"],
  " is an option for ",
  StyleBox["SeedConfiguration", "MR"],
  " that controls the pattern used to fill in the configuration."
}], "Usage",
  CellTags->"Padding"],

Cell[TextData[{
  "Default is ",
  StyleBox["0", "MR"],
  ",  possible values are integers or lists of integers."
}], "Notes",
  CellTags->"Padding"],

Cell[TextData[{
  "See also  ",
  ButtonBox["SeedConfiguration",
    ButtonData:>"SeedConfiguration",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["EvolutionCA",
    ButtonData:>"EvolutionCA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["OrbitCA",
    ButtonData:>"OrbitCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Padding"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Padding"],

Cell["\<\
Some lists suitable as starting configurations for cellular \
automata.\
\>", "ExampleText",
  CellTags->"Padding"],

Cell[BoxData[
    \(SeedConfiguration[{1, 2}, 10]\)], "Input",
  CellLabel->"In[18]:=",
  CellTags->"Padding"],

Cell[BoxData[
    \(SeedConfiguration[{1, 1}, 10, Padding \[Rule] 3]\)], "Input",
  CellLabel->"In[19]:=",
  CellTags->"Padding"],

Cell[BoxData[
    \(SeedConfiguration[{1, 1}, 10, Padding \[Rule] \ {0, 2}]\)], "Input",
  CellLabel->"In[20]:=",
  CellTags->"Padding"],

Cell["PolynomialToCA", "ObjectName",
  CellTags->"PolynomialToCA"],

Cell["Function", "Highlight",
  CellTags->"PolynomialToCA"],

Cell[TextData[{
  StyleBox["PolynomialToCA[pol,w,k]", "MR"],
  "  constructs a CA of width ",
  StyleBox["w", "MR"],
  " over the digit alphabet of size ",
  StyleBox["k", "MR"],
  " whose local map is given by the polynomial ",
  StyleBox["pol", "MR"],
  ". "
}], "Usage",
  CellTags->"PolynomialToCA"],

Cell[TextData[{
  "The values of ",
  StyleBox["pol", "MR"],
  " are taken modulo ",
  StyleBox["k", "MR"],
  ",  and the variables are assumed to be  ",
  StyleBox["x1", "MR"],
  ", ",
  StyleBox["x2", "MR"],
  ", \[Ellipsis] "
}], "Notes",
  CellTags->"PolynomialToCA"],

Cell[TextData[{
  "See also ",
  ButtonBox["BFunctionToCA",
    ButtonData:>"BFunctionToCA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["FunctionToCA",
    ButtonData:>"FunctionToCA",
    ButtonStyle->"AddOnsLink"],
  "  and ",
  ButtonBox["RulesToCA",
    ButtonData:>"RulesToCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"PolynomialToCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"PolynomialToCA"],

Cell["\<\
Addition modulo 2 of the left and right neighbors produces ECA with \
rule number 90.\
\>", "ExampleText",
  CellTags->"PolynomialToCA"],

Cell[BoxData[{
    \(PolynomialToCA[x1 + x3, 3, \(-2\)]\), "\n", 
    \(PrintCA[%]\)}], "Input",
  CellLabel->"In[815]:=",
  CellTags->"PolynomialToCA"],

Cell["PrintCA", "ObjectName",
  CellTags->"PrintCA"],

Cell["Function", "Highlight",
  CellTags->"PrintCA"],

Cell[TextData[{
  StyleBox["PrintCA[C]", "MR"],
  "  pretty-prints the local function of cellular automaton  ",
  StyleBox["C", "MR"],
  "  as a table of replacement rules."
}], "Usage",
  CellTags->"PrintCA"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToRulesCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"PrintCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"PrintCA"],

Cell["Pretty-printing the rule table of some cellular automata.", \
"ExampleText",
  CellTags->"PrintCA"],

Cell["CA[110] // PrintCA", "Input",
  CellLabel->"In[928]:=",
  CellTags->"PrintCA"],

Cell["CA[4,2,12345] // PrintCA", "Input",
  CellLabel->"In[929]:=",
  CellTags->"PrintCA"],

Cell["RuleCA", "ObjectName",
  CellTags->"RuleCA"],

Cell["Function", "Highlight",
  CellTags->"RuleCA"],

Cell[TextData[{
  StyleBox["RuleCA[C]", "MR"],
  "  returns the rule number of a cellular automaton ",
  StyleBox["C", "MR"],
  "."
}], "Usage",
  CellTags->"RuleCA"],

Cell[TextData[{
  "The number is the numerical value of the rule table, construed as a ",
  Cell[BoxData[
      \(TraditionalForm\`k\^\(\(\ \)\(w\)\)\)]],
  "-digit number in base ",
  Cell[BoxData[
      \(TraditionalForm\`k\)]],
  "  where ",
  Cell[BoxData[
      \(TraditionalForm\`k\)]],
  " is the size of the alphabet of ",
  Cell[BoxData[
      \(TraditionalForm\`C\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`w\)]],
  " its width."
}], "Notes",
  CellTags->"RuleCA"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"RuleCA"],

Cell[TextData[{
  "See also  ",
  ButtonBox["CA",
    ButtonData:>"CA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WidthCA",
    ButtonData:>"WidthCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"RuleCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RuleCA"],

Cell["The width is simply the first field.", "ExampleText",
  CellTags->"RuleCA"],

Cell["\<\
ECA[110]
WidthCA[%]\
\>", "Input",
  CellLabel->"In[18]:=",
  CellTags->"RuleCA"],

Cell["RulesToCA", "ObjectName",
  CellTags->"RulesToCA"],

Cell["Function", "Highlight",
  CellTags->"RulesToCA"],

Cell[TextData[{
  StyleBox["RulesToCA[rw,w,Q]", "MR"],
  "  constructs a cellular automaton of width ",
  StyleBox["w", "MR"],
  " over the alphabet ",
  StyleBox["Q", "MR"],
  " whose local map is determined by a collection ",
  StyleBox["rw", "MR"],
  " of rewrite rules."
}], "Usage",
  CellTags->"RulesToCA"],

Cell[TextData[{
  "The rewrite rules   ",
  StyleBox["rw ", "MR"],
  " must convert lists of length ",
  Cell[BoxData[
      \(TraditionalForm\`w\)]],
  " over ",
  Cell[BoxData[
      \(TraditionalForm\`Q\)]],
  "  into elements of ",
  Cell[BoxData[
      \(TraditionalForm\`Q\)]],
  ".  Make sure that there is some default rule such as  ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          StyleBox[\({_, x_, _}\),
            "MR"], 
          StyleBox[" ",
            "MR"], 
          StyleBox["\[Rule]",
            "MR"], 
          StyleBox[" ",
            "MR"], 
          StyleBox["x",
            "MR",
            FontSlant->"Plain"]}], TraditionalForm]]],
  " to capture otherwise undefined cases."
}], "Notes",
  CellTags->"RulesToCA"],

Cell[TextData[{
  "Here  ",
  Cell[BoxData[
      \(TraditionalForm\`Q\)]],
  "  can be an arbitrary list of letters and/or symbols, but the output CA \
will be defined over an appropriate digit alphabet.  "
}], "Notes",
  CellTags->"RulesToCA"],

Cell[TextData[{
  "See also  ",
  ButtonBox["FunctionToCA",
    ButtonData:>"FunctionToCA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["BFunctionToCA",
    ButtonData:>"BFunctionToCA",
    ButtonStyle->"AddOnsLink"],
  "  and  ",
  ButtonBox["PolynomialToCA",
    ButtonData:>"PolynomialToCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"RulesToCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RulesToCA"],

Cell["\<\
A three-state cellular automaton that tests whether a string of \
\"parentheses\" is balanced.\
\>", "ExampleText",
  CellTags->"RulesToCA"],

Cell[BoxData[{
    \(Clear[L, R]\), "\n", 
    \(\(rules\  = \ {\[IndentingNewLine]{_, L, R}\  \[Rule] \ 
            0, \[IndentingNewLine]{L, R, _}\  \[Rule] \ 
            0, \[IndentingNewLine]{_, 0, R}\  \[Rule] \ 
            R, \[IndentingNewLine]{0, R, _}\  \[Rule] \ 
            0, \[IndentingNewLine]{_, x_, _} \[Rule] \ 
            x\[IndentingNewLine]};\)\), "\n", 
    \(\(Q\  = \ {0, L, R};\)\), "\n", 
    \(C\  = \ RulesToCA[\ rules, \ 3, \ Q\ ]\)}], "Input",
  CellLabel->"In[51]:=",
  CellTags->"RulesToCA"],

Cell["\<\
An example of a successful computation.  Here 1 stands for \"(\" \
and 2 stands for \")\". \
\>", "ExampleText",
  CellTags->"RulesToCA"],

Cell[BoxData[
    \(\(X\  = \ {1, 2, 1, 1, 1, 2, 1, 2, 2, 1, 2, 1, 1, 2, 2, 2, 1, 1, 2, 2, 
          1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 2, 1, 1, 2, 2};\)\)], "Input",\

  CellLabel->"In[55]:=",
  CellTags->"RulesToCA"],

Cell[BoxData[
    \(\(EvolutionCA[\ C, \ X\ , 20, GridLines \[Rule] True, \ 
        Boundary \[Rule] Fixed, \ \[IndentingNewLine]RasterStyle \[Rule] \ 
          Join[Take[$rastercolors, 2], {2 \[Rule] Red}]];\)\)], "Input",
  CellLabel->"In[56]:=",
  CellTags->"RulesToCA"],

Cell["SeedConfiguration", "ObjectName",
  CellTags->"SeedConfiguration"],

Cell["Function", "Highlight",
  CellTags->"SeedConfiguration"],

Cell[TextData[{
  StyleBox["SeedConfiguration[{w},n]", "MR"],
  "  produces a seed configuration of length ",
  StyleBox["n", "MR"],
  " with ",
  StyleBox["w", "MR"],
  " at the center. "
}], "Usage",
  CellTags->"SeedConfiguration"],

Cell[TextData[{
  "Options\n",
  Cell[BoxData[GridBox[{
          {"Where", "\[Rule]", "Center", \(Left, Right\)},
          {"Padding", "\[Rule]", "0", "-"},
          {"Type", "\[Rule]", "List", "String"},
          {"Random", "\[Rule]", "0", "k"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"SeedConfiguration"],

Cell[TextData[{
  "Option  ",
  StyleBox["Where->Left ", "MR"],
  " places ",
  Cell[BoxData[
      \(TraditionalForm\`w\)]],
  " at the left end of the configuration.\nOption  ",
  StyleBox["Padding->p ", "MR"],
  " pads with ",
  Cell[BoxData[
      \(TraditionalForm\`p\)]],
  " rather than 0.\nOption  ",
  StyleBox["Type->String ", "MR"],
  " causes a string rather than a list integers to be returned.\nOption  ",
  StyleBox["SeedConfiguration[n,Random->k] ", "MR"],
  " generates a random configuration of length ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " over ",
  Cell[BoxData[
      \(TraditionalForm\`k\)]],
  " letters."
}], "Notes",
  CellTags->"SeedConfiguration"],

Cell[TextData[{
  StyleBox["SeedConfiguration[{w},n,z]", "MR"],
  "  use symbol ",
  Cell[BoxData[
      \(TraditionalForm\`z\)]],
  "  in place of 0. Also works with words rather than lists."
}], "Notes",
  CellTags->"SeedConfiguration"],

Cell["This is useful mostly in connection with cellular automata.", "Notes",
  CellTags->"SeedConfiguration"],

Cell[TextData[{
  "See also ",
  ButtonBox["EvolutionCA",
    ButtonData:>"EvolutionCA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["OrbitCA",
    ButtonData:>"OrbitCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SeedConfiguration"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SeedConfiguration"],

Cell["Some seed configurations of list type.", "ExampleText",
  CellTags->"SeedConfiguration"],

Cell[BoxData[
    \(SeedConfiguration[{1, 2, 3}, 15]\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->"SeedConfiguration"],

Cell[BoxData[
    \(SeedConfiguration[{1, 2, 3}, 15, Where \[Rule] Left]\)], "Input",
  CellLabel->"In[14]:=",
  CellTags->"SeedConfiguration"],

Cell["Changing the background padding.", "ExampleText",
  CellTags->"SeedConfiguration"],

Cell[BoxData[
    \(SeedConfiguration[{1, 1}, 15, Padding \[Rule] 2\ ]\)], "Input",
  CellLabel->"In[15]:=",
  CellTags->"SeedConfiguration"],

Cell[BoxData[
    \(SeedConfiguration[{1, 1}, 15, Padding \[Rule] {0, 2}\ ]\)], "Input",
  CellLabel->"In[16]:=",
  CellTags->"SeedConfiguration"],

Cell[TextData[{
  "A random seed configuration.  The elements are drawn uniformly from ",
  Cell[BoxData[
      \(TraditionalForm\`\([0, 3]\)\)]],
  "."
}], "ExampleText",
  CellTags->"SeedConfiguration"],

Cell[BoxData[
    \(SeedConfiguration[15, \ Random \[Rule] \ 4]\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->"SeedConfiguration"],

Cell[TextData[{
  "It is possible to construct seed configurations of type string with ",
  StyleBox["SeedConfiguration", "MR"],
  ". The main purpose of representing orbits as lists of strings rather than \
ordinary matrices of integers was to save memory; packed arrays may make this \
feature superfluous."
}], "ExampleText",
  CellTags->"SeedConfiguration"],

Cell[BoxData[
    \(SeedConfiguration[\ "\<bbb\>", \ 15, Type \[Rule] String\ ]\)], "Input",\

  CellLabel->"In[6]:=",
  CellTags->"SeedConfiguration"],

Cell["SimplifyCA", "ObjectName",
  CellTags->"SimplifyCA"],

Cell["Function", "Highlight",
  CellTags->"SimplifyCA"],

Cell[TextData[{
  StyleBox["SimplifyCA[C]", "MR"],
  "  eliminates the initial and final segment of variables that are \
superfluous in the local rule of cellular automaton ",
  StyleBox["C", "MR"],
  ",  if any. "
}], "Usage",
  CellTags->"SimplifyCA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"SimplifyCA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  " also returns the number of deleted variables."
}], "Notes",
  CellTags->"SimplifyCA"],

Cell[TextData[{
  "See also ",
  ButtonBox["ClassifyCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SimplifyCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SimplifyCA"],

Cell["\<\
Composition with ECA 204 whose global map is the identity produces \
a new CA that has width 4, but is essentially the same as the original CA.  \
Here is an example with rule 90. \
\>", "ExampleText",
  CellTags->"SimplifyCA"],

Cell[BoxData[
    \(CA[204]\  // \ PrintCA\)], "Input",
  CellLabel->"In[95]:=",
  CellTags->"SimplifyCA"],

Cell[BoxData[{
    \(c = ComposeCA[CA[90], \ CA[204]]\), "\n", 
    \(SimplifyCA[c]\)}], "Input",
  CellLabel->"In[96]:=",
  CellTags->"SimplifyCA"],

Cell["\<\
Elimination of useless variables is also helpful in checking the \
correctness of the inversion algorithm.\
\>", "ExampleText",
  CellTags->"SimplifyCA"],

Cell[BoxData[{
    \(\(C = CA[4, \(-2\), 3915];\)\), "\n", 
    \(\(Ci = InverseCA[C];\)\), "\n", 
    \(CC = ComposeCA[C, Ci]\), "\n", 
    \(SimplifyCA[CC]\)}], "Input",
  CellLabel->"In[1]:=",
  AspectRatioFixed->True,
  CellTags->"SimplifyCA"],

Cell[BoxData[
    \(PrintCA[%]\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->"SimplifyCA"],

Cell["ToRulesCA", "ObjectName",
  CellTags->"ToRulesCA"],

Cell["Function", "Highlight",
  CellTags->"ToRulesCA"],

Cell[TextData[{
  StyleBox["ToRulesCA[C]", "MR"],
  "  returns  a list of Mathematica replacement rules corresponding to the \
local map of cellular automaton C."
}], "Usage",
  CellTags->"ToRulesCA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"WordMap", "\[Rule]", "True", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ToRulesCA"],

Cell[TextData[{
  "Option ",
  StyleBox["WordMap->False", "MR"],
  " produces replacement rules for lists rather than words."
}], "Notes",
  CellTags->"ToRulesCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToRulesCA"],

Cell[TextData[{
  "The ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " rules corresponding to ECA number 90. "
}], "ExampleText",
  CellTags->"ToRulesCA"],

Cell[BoxData[
    \(ToRulesCA[CA[90]]\)], "Input",
  CellLabel->"In[511]:=",
  CellTags->"ToRulesCA"],

Cell["Same in list form.", "ExampleText",
  CellTags->"ToRulesCA"],

Cell[BoxData[
    \(ToRulesCA[CA[90], WordMap \[Rule] False]\)], "Input",
  CellLabel->"In[83]:=",
  CellTags->"ToRulesCA"],

Cell["TotalisticRuleToCA", "ObjectName",
  CellTags->"TotalisticRuleToCA"],

Cell["Function", "Highlight",
  CellTags->"TotalisticRuleToCA"],

Cell[TextData[{
  StyleBox["TotalisticRuleToCA[w,k,r]", "MR"],
  "  produces the ordinary CA with totalistic rule number ",
  StyleBox["r", "MR"],
  " and width ",
  StyleBox["w", "MR"],
  ". "
}], "Usage",
  CellTags->"TotalisticRuleToCA"],

Cell[TextData[{
  "The alphabet is always of the form ",
  Cell[BoxData[
      \(TraditionalForm\`{0, 1,  ... , k - 1}\)]],
  "."
}], "Notes",
  CellTags->"TotalisticRuleToCA"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToTotalisticRuleCA",
    ButtonData:>"ToTotalisticRuleCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"TotalisticRuleToCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"TotalisticRuleToCA"],

Cell["\<\
Conversion of a totalistic rule number to a standard rule number. \
\
\>", "ExampleText",
  CellTags->"TotalisticRuleToCA"],

Cell["NumberOfTotalisticCA[3,2]", "Input",
  CellLabel->"In[4]:=",
  CellTags->"TotalisticRuleToCA"],

Cell[BoxData[
    \(Table[\ TotalisticRuleToCA[3, 2, r], {r, 0, 15}]\)], "Input",
  CellLabel->"In[519]:=",
  CellTags->"TotalisticRuleToCA"],

Cell["ToTotalisticRuleCA", "ObjectName",
  CellTags->"ToTotalisticRuleCA"],

Cell["Function", "Highlight",
  CellTags->"ToTotalisticRuleCA"],

Cell[TextData[{
  StyleBox["ToTotalisticRuleCA[C]", "MR"],
  "  determines the totalistic rule number ",
  StyleBox["r", "MR"],
  " of a given CA ",
  StyleBox["C", "MR"],
  ". "
}], "Usage",
  CellTags->"ToTotalisticRuleCA"],

Cell[TextData[{
  "No test is performed whether ",
  StyleBox["C", "MR"],
  " is indeed totalistic."
}], "Notes",
  CellTags->"ToTotalisticRuleCA"],

Cell[TextData[{
  "See also ",
  ButtonBox["TotalisticRuleToCA",
    ButtonData:>"TotalisticRuleToCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToTotalisticRuleCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToTotalisticRuleCA"],

Cell["Rule 150 is totalistic.", "ExampleText",
  CellTags->"ToTotalisticRuleCA"],

Cell["ToTotalisticRuleCA[CA[3,2,150]]", "Input",
  CellLabel->"In[520]:=",
  CellTags->"ToTotalisticRuleCA"],

Cell["WelchCA", "ObjectName",
  CellTags->"WelchCA"],

Cell["Function", "Highlight",
  CellTags->"WelchCA"],

Cell[TextData[{
  StyleBox["WelchCA[C]", "MR"],
  "  computes the Welch indices of a surjective cellular automaton ",
  StyleBox["C", "MR"],
  ". "
}], "Usage",
  CellTags->"WelchCA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"},
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"WelchCA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  " also produces the actual Welch automata."
}], "Notes",
  CellTags->"WelchCA"],

Cell[TextData[{
  "The algorithm does not check that ",
  Cell[BoxData[
      \(TraditionalForm\`C\)]],
  " is in fact surjective."
}], "Notes",
  CellTags->"WelchCA"],

Cell[TextData[{
  "See also  ",
  ButtonBox["ToSA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WelchCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WelchCA"],

Cell["\<\
Here is a list of all surjective elementary CAs, and their Welch \
indices.\
\>", "ExampleText",
  CellTags->"WelchCA"],

Cell[BoxData[
    \(\(surj\  = \ {30, 60, 86, 90, 102, 106, 120, 150, 154, 166, 170, 180, 
          204, 210, 240};\)\)], "Input",
  CellLabel->"In[50]:=",
  CellTags->"WelchCA"],

Cell["WelchCA[CA[#]]& /@ surj // TableForm", "Input",
  CellLabel->"In[52]:=",
  CellTags->"WelchCA"],

Cell["The actual Welch automata for rule .", "ExampleText",
  CellTags->"WelchCA"],

Cell[BoxData[
    \(WelchCA[CA[204], Full \[Rule] True]\)], "Input",
  CellLabel->"In[55]:=",
  CellTags->"WelchCA"],

Cell[BoxData[
    \(States[%[\([2, 1]\)]]\)], "Input",
  CellLabel->"In[56]:=",
  CellTags->"WelchCA"],

Cell["Where", "ObjectName",
  CellTags->"Where"],

Cell["Option", "Highlight",
  CellTags->"Where"],

Cell[TextData[{
  StyleBox["Where", "MR"],
  " is an option for ",
  StyleBox["SeedConfiguration", "MR"],
  " that controls positioning of the seed pattern. "
}], "Usage",
  CellTags->"Where"],

Cell[TextData[{
  "Default is ",
  StyleBox["Center", "MR"],
  ", other possible values are ",
  StyleBox["Left", "MR"],
  " or ",
  StyleBox["Right", "MR"],
  "."
}], "Notes",
  CellTags->"Where"],

Cell[TextData[{
  "See also  ",
  ButtonBox["SeedConfiguration",
    ButtonData:>"SeedConfiguration",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["EvolutionCA",
    ButtonData:>"EvolutionCA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["OrbitCA",
    ButtonData:>"OrbitCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Where"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Where"],

Cell["\<\
Some lists suitable as starting configurations for cellular \
automata.\
\>", "ExampleText",
  CellTags->"Where"],

Cell["SeedConfiguration[{1,2},10]", "Input",
  CellLabel->"In[96]:=",
  CellTags->"Where"],

Cell["SeedConfiguration[{1,2},10,Where->Left]", "Input",
  CellLabel->"In[98]:=",
  CellTags->"Where"],

Cell["SeedConfiguration[{1,2},10,Where->Right]", "Input",
  CellLabel->"In[99]:=",
  CellTags->"Where"],

Cell["WidthCA", "ObjectName",
  CellTags->"WidthCA"],

Cell["Function", "Highlight",
  CellTags->"WidthCA"],

Cell[TextData[{
  StyleBox["WidthCA[C]", "MR"],
  "  returns the width of a cellular automaton ",
  StyleBox["C", "MR"],
  "."
}], "Usage",
  CellTags->"WidthCA"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"WidthCA"],

Cell[TextData[{
  "See also  ",
  ButtonBox["CA",
    ButtonData:>"CA",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["RuleCA",
    ButtonData:>"RuleCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WidthCA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WidthCA"],

Cell["\<\
The width is simply the first field of a cellular automaton object.\
\
\>", "ExampleText",
  CellTags->"WidthCA"],

Cell["\<\
CA[110]
WidthCA[%]\
\>", "Input",
  CellLabel->"In[18]:=",
  CellTags->"WidthCA"],

Cell["WordMap", "ObjectName",
  CellTags->"WordMap"],

Cell["Option", "Highlight",
  CellTags->"WordMap"],

Cell[TextData[{
  StyleBox["WordMap", "MR"],
  "  is an option for an number of functions dealing with cellular automata."
}], "Usage",
  CellTags->"WordMap"],

Cell[TextData[{
  "Associated Functions:\n",
  ButtonBox["GlobalMapCA",
    ButtonStyle->"AddOnsLink"],
  "\n",
  ButtonBox["LocalMapCA",
    ButtonStyle->"AddOnsLink"],
  "\n",
  ButtonBox["OrbitCA",
    ButtonStyle->"AddOnsLink"],
  "\n",
  ButtonBox["ToRulesCA",
    ButtonStyle->"AddOnsLink"]
}], "Notes",
  CellTags->"WordMap"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordMap"],

Cell["\<\
The local map of a CA usually operates sequences of integers, but \
can be made to work on strings instead.\
\>", "ExampleText",
  CellTags->"WordMap"],

Cell[BoxData[
    \(LocalMapCA[f, CA[90], WordMap \[Rule] True]\)], "Input",
  CellLabel->"In[21]:=",
  CellTags->"WordMap"],

Cell[BoxData[
    \(\(?f\)\)], "Input",
  CellLabel->"In[22]:=",
  CellTags->"WordMap"],

Cell["AddEdge", "ObjectName",
  CellTags->"AddEdge"],

Cell["Function", "Highlight",
  CellTags->"AddEdge"],

Cell[TextData[{
  StyleBox["AddEdge[G,e]", "MR"],
  "  adds edge ",
  StyleBox["e", "MR"],
  " to graph ",
  StyleBox["G", "MR"],
  ". "
}], "Usage",
  CellTags->"AddEdge"],

Cell[TextData[{
  "Edge ",
  StyleBox["e", "MR"],
  " must be given as a pair of vertices in positional notation.  The source \
and target vertex must be present in ",
  StyleBox["G", "MR"],
  "."
}], "Notes",
  CellTags->"AddEdge"],

Cell[TextData[{
  "See also ",
  ButtonBox["Graph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DeleteEdge",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DeleteVertex",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["AddVertex",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"AddEdge"],

Cell["Further Examples", "ExampleSection",
  CellTags->"AddEdge"],

Cell["Adding a back-edge to a path produces a lasso.", "ExampleText",
  CellTags->"AddEdge"],

Cell["G = PathGraph[8]", "Input",
  CellLabel->"In[89]:=",
  CellTags->"AddEdge"],

Cell[BoxData[
    \(AddEdge[\ G, \ {8, 3}]\)], "Input",
  CellLabel->"In[90]:=",
  CellTags->"AddEdge"],

Cell[BoxData[
    \(StronglyConnectedComponents[%]\)], "Input",
  CellLabel->"In[91]:=",
  CellTags->"AddEdge"],

Cell["AddVertex", "ObjectName",
  CellTags->"AddVertex"],

Cell["Function", "Highlight",
  CellTags->"AddVertex"],

Cell[TextData[{
  StyleBox["AddVertex[G]", "MR"],
  "  adds a new vertex to graph ",
  StyleBox["G", "MR"],
  ". "
}], "Usage",
  CellTags->"AddVertex"],

Cell[TextData[{
  "Form  ",
  StyleBox["AddVertex[G,x]", "MR"],
  " also adds ",
  StyleBox["x", "MR"],
  " to the structured vertex set. "
}], "Notes",
  CellTags->"AddVertex"],

Cell[TextData[{
  "The new vertex has position ",
  Cell[BoxData[
      \(TraditionalForm\`n + 1\)]],
  ",  ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " the size of ",
  Cell[BoxData[
      \(TraditionalForm\`G\)]],
  ", and is isolated."
}], "Notes",
  CellTags->"AddVertex"],

Cell[TextData[{
  "See also ",
  ButtonBox["Graph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["AddEdge",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DeleteEdge",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DeleteVertex",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"AddVertex"],

Cell["Further Examples", "ExampleSection",
  CellTags->"AddVertex"],

Cell["Adding a vertex to a graph, without any new edges.", "ExampleText",
  CellTags->"AddVertex"],

Cell[BoxData[
    \(\(G = CirculantGraph[4, {1}\ ];\)\)], "Input",
  CellLabel->"In[165]:=",
  CellTags->"AddVertex"],

Cell[BoxData[
    \(\(SetVertices[\ 
        G, \ {"\<a\>", "\<b\>", "\<c\>", "\<d\>"}];\)\)], "Input",
  CellLabel->"In[166]:=",
  CellTags->"AddVertex"],

Cell[BoxData[{
    \(G\  // \ VertexSet\), "\n", 
    \(AddVertex[G, "\<e\>"]\  // \ VertexSet\)}], "Input",
  CellLabel->"In[168]:=",
  CellTags->"AddVertex"],

Cell["BiAccessibleGraph", "ObjectName",
  CellTags->"BiAccessibleGraph"],

Cell["Function", "Highlight",
  CellTags->"BiAccessibleGraph"],

Cell[TextData[{
  StyleBox["BiAccessibleGraph[G,S,T]", "MR"],
  "  constructs the subgraph of ",
  StyleBox["G", "MR"],
  " induced by the vertices reachable from ",
  StyleBox["S", "MR"],
  " and coreachable from ",
  StyleBox["T", "MR"],
  "."
}], "Usage",
  CellTags->"BiAccessibleGraph"],

Cell["\<\
Both S and T are lists of integers, corresponding to vertices in G.\
\
\>", "Notes",
  CellTags->"BiAccessibleGraph"],

Cell[TextData[{
  "Returns a ",
  StyleBox["Graph", "MR"],
  " object that maintains the corresponding vertex set of the original \
graph."
}], "Notes",
  CellTags->"BiAccessibleGraph"],

Cell[TextData[{
  "See also ",
  ButtonBox["AccessibleGraph",
    ButtonData:>"AccessibleGraph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SubGraph",
    ButtonData:>"SubGraph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DegreeGraph",
    ButtonData:>"DegreeGraph",
    ButtonStyle->"AddOnsLink"],
  StyleBox[",", "Text"],
  " ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"BiAccessibleGraph"],

Cell["Further Examples", "ExampleSection",
  CellTags->"BiAccessibleGraph"],

Cell["\<\
A path graph. All points are accessible from vertex 1, and \
coaccessible from vertex 8.\
\>", "ExampleText",
  CellTags->"BiAccessibleGraph"],

Cell[BoxData[
    \(\(g\  = \ 
        Graph[8, 
          7, "\<EList\>", {{1, 2}, {2, 3}, {3, 4}, {4, 5}, {5, 6}, {6, 
              7}, {7, 8}}];\)\)], "Input",
  CellLabel->"In[533]:=",
  CellTags->"BiAccessibleGraph"],

Cell["BiAccessibleGraph[g,{1},{8}]", "Input",
  CellLabel->"In[534]:=",
  CellTags->"BiAccessibleGraph"],

Cell["\<\
Only 3 points are accissible from vertex 3, and coaccessible from \
vertex 5.\
\>", "ExampleText",
  CellTags->"BiAccessibleGraph"],

Cell["\<\
BiAccessibleGraph[g,{3},{5}]
VertexSet[%]\
\>", "Input",
  CellLabel->"In[535]:=",
  CellTags->"BiAccessibleGraph"],

Cell["AccessibleGraph", "ObjectName",
  CellTags->"AccessibleGraph"],

Cell["Function", "Highlight",
  CellTags->"AccessibleGraph"],

Cell[TextData[{
  StyleBox["AccessibleGraph[G,A]", "MR"],
  "  constructs the subgraph of G induced by the vertices reachable from set \
",
  StyleBox["A", "MR"],
  ".  ",
  StyleBox["AccessibleGraph", "MR"],
  " uses the function ",
  StyleBox["WeaklyConnectedComponents", "MR"],
  "."
}], "Usage",
  CellTags->"AccessibleGraph"],

Cell["\<\
A is a list of integers, corresponding to vertices in G in \
positional notation.\
\>", "Notes",
  CellTags->"AccessibleGraph"],

Cell[TextData[{
  "See also ",
  ButtonBox["BiAccessibleGraph",
    ButtonData:>"BiAccessibleGraph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SubGraph",
    ButtonData:>"SubGraph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DegreeGraph",
    ButtonData:>"DegreeGraph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"AccessibleGraph"],

Cell["Further Examples", "ExampleSection",
  CellTags->"AccessibleGraph"],

Cell["A circulant graph. ", "ExampleText",
  CellTags->"AccessibleGraph"],

Cell["g = CirculantGraph[10,{2,-2}]", "Input",
  CellLabel->"In[539]:=",
  CellTags->"AccessibleGraph"],

Cell["Only the odd vertices are reachable from vertex 1.", "ExampleText",
  CellTags->"AccessibleGraph"],

Cell["\<\
AccessibleGraph[g,1]
VertexSet[%]\
\>", "Input",
  CellLabel->"In[540]:=",
  CellTags->"AccessibleGraph"],

Cell["All vertices are reachable from {1,2}.", "ExampleText",
  CellTags->"AccessibleGraph"],

Cell["AccessibleGraph[g,{1,2}] // VertexSet", "Input",
  CellLabel->"In[177]:=",
  CellTags->"AccessibleGraph"],

Cell["CirculantGraph", "ObjectName",
  CellTags->"CirculantGraph"],

Cell["Function", "Highlight",
  CellTags->"CirculantGraph"],

Cell[TextData[{
  StyleBox["CirculantGraph[n,{d1,...,dr}]", "MR"],
  "  generates the circulant graph on ",
  StyleBox["n", "MR"],
  " nodes with strides given by the ",
  StyleBox["d", "MR"],
  "-list."
}], "Usage",
  CellTags->"CirculantGraph"],

Cell[TextData[{
  "A circulant graph can be construed as having as vertex set the modular \
numbers ",
  Cell[BoxData[
      \(TraditionalForm\`{0, 1, \[Ellipsis], n - 1}\)]],
  " and edges  ",
  Cell[BoxData[
      \(TraditionalForm\`\((x, x + d\_i\ mod\ n)\)\)]],
  ". "
}], "Notes",
  CellTags->"CirculantGraph"],

Cell[TextData[{
  "Circulant graphs are ",
  Cell[BoxData[
      \(TraditionalForm\`\((r, r)\)\)]],
  "-regular."
}], "Notes",
  CellTags->"CirculantGraph"],

Cell["The graph is generated in edge-list form.", "Notes",
  CellTags->"CirculantGraph"],

Cell[TextData[{
  "See also ",
  ButtonBox["PathGraph",
    ButtonData:>"PathGraph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"CirculantGraph"],

Cell["Further Examples", "ExampleSection",
  CellTags->"CirculantGraph"],

Cell["A circulant graph on 10 points.", "ExampleText",
  CellTags->"CirculantGraph"],

Cell["g = CirculantGraph[10,{2,4}]", "Input",
  CellLabel->"In[544]:=",
  CellTags->"CirculantGraph"],

Cell[BoxData[
    \(StronglyConnectedComponents[g]\)], "Input",
  CellLabel->"In[545]:=",
  CellTags->"CirculantGraph"],

Cell["A transitive circulant graph on 10 points.", "ExampleText",
  CellTags->"CirculantGraph"],

Cell["g = CirculantGraph[10,{2,3}]", "Input",
  CellLabel->"In[180]:=",
  CellTags->"CirculantGraph"],

Cell[BoxData[
    \(StronglyConnectedComponents[g]\)], "Input",
  CellLabel->"In[181]:=",
  CellTags->"CirculantGraph"],

Cell["CyclicQGraph", "ObjectName",
  CellTags->"CyclicQGraph"],

Cell["Function", "Highlight",
  CellTags->"CyclicQGraph"],

Cell[TextData[{
  StyleBox["CyclicQGraph[G]", "MR"],
  "  tests whether graph ",
  StyleBox["G", "MR"],
  " is cyclic."
}], "Usage",
  CellTags->"CyclicQGraph"],

Cell["Is linear time.", "Notes",
  CellTags->"CyclicQGraph"],

Cell[TextData[{
  "See also ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"CyclicQGraph"],

Cell["Further Examples", "ExampleSection",
  CellTags->"CyclicQGraph"],

Cell["A plain cycle. ", "ExampleText",
  CellTags->"CyclicQGraph"],

Cell[BoxData[
    \(\(g = 
        Graph[8, 
          8, "\<EList\>", {{1, 2}, {2, 3}, {3, 4}, {4, 5}, {5, 6}, {6, 
              7}, {7, 8}, {8, 1}}];\)\)], "Input",
  CellLabel->"In[191]:=",
  CellTags->"CyclicQGraph"],

Cell[BoxData[
    \(CyclicQGraph[g]\)], "Input",
  CellLabel->"In[192]:=",
  CellTags->"CyclicQGraph"],

Cell[BoxData[
    \(gg\  = \ DeleteVertex[g, 1]\)], "Input",
  CellLabel->"In[196]:=",
  CellTags->"CyclicQGraph"],

Cell[BoxData[
    \(CyclicQGraph[gg]\)], "Input",
  CellLabel->"In[197]:=",
  CellTags->"CyclicQGraph"],

Cell["DeBruijnEdges", "ObjectName",
  CellTags->"DeBruijnEdges"],

Cell["Function", "Highlight",
  CellTags->"DeBruijnEdges"],

Cell[TextData[{
  StyleBox["DeBruijnEdges[w,K]", "MR"],
  "  generates an edge list for the de Bruijn graph of width w over alphabet \
K."
}], "Usage",
  CellTags->"DeBruijnEdges"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[Rule]", "1", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"DeBruijnEdges"],

Cell[TextData[{
  " The default alphabet is ",
  Cell[BoxData[
      \(TraditionalForm\`K\  = \ {0, 1}\)]],
  "."
}], "Notes",
  CellTags->"DeBruijnEdges"],

Cell[TextData[{
  "Option ",
  StyleBox["Normalize->0", "MR"],
  " produces edge list in positional notation, suitable for a ",
  StyleBox["Graph", "MR"],
  " object."
}], "Notes",
  CellTags->"DeBruijnEdges"],

Cell[TextData[{
  "See also ",
  ButtonBox["DeBruijnGraph",
    ButtonData:>"DeBruijnGraph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DeBruijnEdges"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DeBruijnEdges"],

Cell[TextData[{
  "Edges for the de Bruijn graph of width 3 over ",
  Cell[BoxData[
      \(TraditionalForm\`{0, 1}\)]],
  ". "
}], "ExampleText",
  CellTags->"DeBruijnEdges"],

Cell[BoxData[
    \(DeBruijnEdges[3, \(-2\)]\)], "Input",
  CellLabel->"In[203]:=",
  CellTags->"DeBruijnEdges"],

Cell["The same in positional notation.", "ExampleText",
  CellTags->"DeBruijnEdges"],

Cell[BoxData[
    \(DeBruijnEdges[3, \(-2\), Normalize \[Rule] 0]\)], "Input",
  CellLabel->"In[204]:=",
  CellTags->"DeBruijnEdges"],

Cell["DeBruijnGraph", "ObjectName",
  CellTags->"DeBruijnGraph"],

Cell["Function", "Highlight",
  CellTags->"DeBruijnGraph"],

Cell[TextData[{
  StyleBox["DeBruijnGraph[w,K]", "MR"],
  "  generates the de Bruijn graph of width w over alphabet K."
}], "Usage",
  CellTags->"DeBruijnGraph"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[Rule]", "0", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"DeBruijnGraph"],

Cell[TextData[{
  " The default alphabet is ",
  Cell[BoxData[
      \(TraditionalForm\`K\  = \ {0, 1}\)]],
  "."
}], "Notes",
  CellTags->"DeBruijnGraph"],

Cell[TextData[{
  "The nodes in this graph are words of length ",
  Cell[BoxData[
      \(TraditionalForm\`w\)]],
  " over ",
  Cell[BoxData[
      \(TraditionalForm\`K\)]],
  ", and the edges are of the form ",
  Cell[BoxData[
      \(TraditionalForm\`\((ax, xb)\)\)]],
  " where ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " is a word of length ",
  Cell[BoxData[
      \(TraditionalForm\`w - 1\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  " are symbols. These graphs are ",
  Cell[BoxData[
      \(TraditionalForm\`\((k, k)\)\)]],
  "-regular: all nodes have indegree and outdegree ",
  Cell[BoxData[
      \(TraditionalForm\`k\  = \ \(\(|\)\(K\)\(|\)\)\)]],
  ". They are also Eulerian and Hamiltonian. "
}], "Notes",
  CellTags->"DeBruijnGraph"],

Cell[TextData[{
  "See also ",
  ButtonBox["DeBruijnEdges",
    ButtonData:>"DeBruijnEdges",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DeBruijnGraph"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DeBruijnGraph"],

Cell[TextData[{
  "The de Bruijn graph of width 3 over ",
  Cell[BoxData[
      \(TraditionalForm\`{0, 1}\)]],
  ". "
}], "ExampleText",
  CellTags->"DeBruijnGraph"],

Cell[BoxData[
    \(g\  = \ DeBruijnGraph[3, \(-2\), Normalize \[Rule] 1]\)], "Input",
  CellLabel->"In[512]:=",
  CellTags->"DeBruijnGraph"],

Cell[BoxData[
    \(VertexSet[%]\)], "Input",
  CellLabel->"In[218]:=",
  CellTags->"DeBruijnGraph"],

Cell["\<\
This graph is strongly transitive: there is a path of length 3 \
between any two vertices.\
\>", "ExampleText",
  CellTags->"DeBruijnGraph"],

Cell[BoxData[{
    \(\(A\  = \ Edges[ToAdjacencyMatrix[g]];\)\), "\n", 
    \(MatrixPower[A, 3]\  // \ MatrixForm\)}], "Input",
  CellLabel->"In[219]:=",
  CellTags->"DeBruijnGraph"],

Cell["DegreeGraph", "ObjectName",
  CellTags->"DegreeGraph"],

Cell["Function", "Highlight",
  CellTags->"DegreeGraph"],

Cell[TextData[{
  StyleBox["DegreeGraph[G,v]", "MR"],
  "  returns the degree of vertex ",
  StyleBox["v", "MR"],
  " in graph ",
  StyleBox["G", "MR"],
  ". "
}], "Usage",
  CellTags->"DegreeGraph"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"DegreeGraph"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  " causes the pair ",
  StyleBox["{in-deg,out-deg}", "MR"],
  " to be returned."
}], "Notes",
  CellTags->"DegreeGraph"],

Cell[TextData[{
  "To obtain a list of all degrees in a graph use ",
  StyleBox["DegreeGraph[G]", "MR"],
  "."
}], "Notes",
  CellTags->"DegreeGraph"],

Cell[TextData[{
  "See also ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DegreeGraph"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DegreeGraph"],

Cell["\<\
In a circulant graph, all vertices have the same in-degree and \
out-degree. \
\>", "ExampleText",
  CellTags->"DegreeGraph"],

Cell["g = CirculantGraph[8,{1,3}]", "Input",
  CellLabel->"In[442]:=",
  CellTags->"DegreeGraph"],

Cell[BoxData[
    \(DegreeGraph[g, 5]\)], "Input",
  CellLabel->"In[443]:=",
  CellTags->"DegreeGraph"],

Cell[BoxData[
    \(DegreeGraph[g, 5, Full \[Rule] True\ ]\)], "Input",
  CellLabel->"In[444]:=",
  CellTags->"DegreeGraph"],

Cell[BoxData[
    \(DegreeGraph[g, Full \[Rule] True\ ]\)], "Input",
  CellLabel->"In[445]:=",
  CellTags->"DegreeGraph"],

Cell["DeleteEdge", "ObjectName",
  CellTags->"DeleteEdge"],

Cell["Function", "Highlight",
  CellTags->"DeleteEdge"],

Cell[TextData[{
  StyleBox["DeleteEdge[G,e]", "MR"],
  "  removes edge ",
  StyleBox["e", "MR"],
  " from graph ",
  StyleBox["G", "MR"],
  ". "
}], "Usage",
  CellTags->"DeleteEdge"],

Cell[TextData[{
  "Edge ",
  StyleBox["e", "MR"],
  " must be given as a pair of vertices in positional notation."
}], "Notes",
  CellTags->"DeleteEdge"],

Cell[TextData[{
  "See also ",
  ButtonBox["Graph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["AddEdge",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["AddVertex",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DeleteVertex",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DeleteEdge"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DeleteEdge"],

Cell["A path, and a disconnected graph after removal of one edge.", \
"ExampleText",
  CellTags->"DeleteEdge"],

Cell["G = PathGraph[8]", "Input",
  CellLabel->"In[115]:=",
  CellTags->"DeleteEdge"],

Cell[BoxData[{
    \(DeleteEdge[\ G, \ {4, 5}]\), "\n", 
    \(WeaklyConnectedComponents[%, {1}]\)}], "Input",
  CellLabel->"In[127]:=",
  CellTags->"DeleteEdge"],

Cell["DeleteVertex", "ObjectName",
  CellTags->"DeleteVertex"],

Cell["Function", "Highlight",
  CellTags->"DeleteVertex"],

Cell[TextData[{
  StyleBox["DeleteVertex", "MR"],
  StyleBox["[G,x]", "MR"],
  "  removes vertex ",
  StyleBox["x", "MR"],
  " from graph ",
  StyleBox["G", "MR"],
  ". "
}], "Usage",
  CellTags->"DeleteVertex"],

Cell[TextData[{
  "Vertex ",
  StyleBox["x", "MR"],
  " must be given in positional notation."
}], "Notes",
  CellTags->"DeleteVertex"],

Cell[TextData[{
  "See also ",
  ButtonBox["Graph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["AddEdge",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["AddVertex",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DeleteEdge",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DeleteVertex"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DeleteVertex"],

Cell["A path, and a disconnected graph after removal of one point.", \
"ExampleText",
  CellTags->"DeleteVertex"],

Cell["G = PathGraph[8]", "Input",
  CellLabel->"In[618]:=",
  CellTags->"DeleteVertex"],

Cell[BoxData[
    \(GG\  = \ DeleteVertex[G, 5]\)], "Input",
  CellLabel->"In[619]:=",
  CellTags->"DeleteVertex"],

Cell[BoxData[{
    \(WeaklyConnectedComponents[G, 1]\), "\n", 
    \(WeaklyConnectedComponents[GG, 1]\)}], "Input",
  CellLabel->"In[54]:=",
  CellTags->"DeleteVertex"],

Cell["EdgeQ", "ObjectName",
  CellTags->"EdgeQ"],

Cell["Function", "Highlight",
  CellTags->"EdgeQ"],

Cell[TextData[{
  StyleBox["EdgeQ[G,x,y]", "MR"],
  "  tests whether the directed edge  ",
  StyleBox["(x,y)", "MR"],
  "  is present in graph ",
  StyleBox["G", "MR"],
  "."
}], "Usage",
  CellTags->"EdgeQ"],

Cell[TextData[{
  "Both ",
  StyleBox["x", "MR"],
  " and ",
  StyleBox["y", "MR"],
  " must be in positional notation."
}], "Notes",
  CellTags->"EdgeQ"],

Cell[TextData[{
  "Can also be used in the form  ",
  StyleBox["EdgeQ[G,{x,y}]", "MR"],
  ". "
}], "Notes",
  CellTags->"EdgeQ"],

Cell[TextData[{
  "See also ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"EdgeQ"],

Cell["Edges", "ObjectName",
  CellTags->"Edges"],

Cell["Function", "Highlight",
  CellTags->"Edges"],

Cell[TextData[{
  StyleBox["Edges[G]", "MR"],
  "  returns the edges of a graph G"
}], "Usage",
  CellTags->"Edges"],

Cell["\<\
 The type of the output depends on type of graph and could be an \
edge list, an adjacency list, or an adjacency matrix.\
\>", "Notes",
  CellTags->"Edges"],

Cell[TextData[{
  "See ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["TypeGraph",
    ButtonData:>"TypeGraph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Edges"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Edges"],

Cell["The edges in a circulant graph.", "ExampleText",
  CellTags->"Edges"],

Cell["g = CirculantGraph[8,{1,3}]", "Input",
  CellLabel->"In[565]:=",
  CellTags->"Edges"],

Cell[BoxData[
    \(Edges[g]\)], "Input",
  CellLabel->"In[566]:=",
  CellTags->"Edges"],

Cell["Graph", "ObjectName",
  CellTags->"Graph"],

Cell["Constructor", "Highlight",
  CellTags->"Graph"],

Cell[TextData[{
  StyleBox["Graph[n,e,t,A]", "MR"],
  "  represents a directed graph on ",
  StyleBox["n", "MR"],
  " nodes and ",
  StyleBox["e", "MR"],
  " edges with adjacencies given by A. Field ",
  StyleBox["t", "MR"],
  " indicates the type of the graph. "
}], "Usage",
  CellTags->"Graph"],

Cell["\<\
 Graphs can be represented by an edge list (EList), a labeled edge \
list (LEList), an adjacency matrix (AMat) or an adjacency list (AList).\
\>", \
"Notes",
  CellTags->"Graph"],

Cell[" Graphs types are indicated by strings, not symbols.", "Notes",
  CellTags->"Graph"],

Cell[TextData[{
  "There is a hidden field ",
  StyleBox["V", "MR"],
  " for the underlying vertex set. V can be empty (in which case the vertex \
set is assumed to be ",
  Cell[BoxData[
      \(TraditionalForm\`{1, 2, \[Ellipsis], n}\)]],
  ") or as an arbitrary list of length ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  ". "
}], "Notes",
  CellTags->"Graph"],

Cell[TextData[{
  "See also ",
  ButtonBox["Edges",
    ButtonData:>"Edges",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["VertexSet",
    ButtonData:>"VertexSet",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["WeaklyConnectedComponents",
    ButtonData:>"WeaklyConnectedComponents",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["StronglyConnectedComponents",
    ButtonData:>"StronglyConnectedComponents",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["NormalizeGraph",
    ButtonData:>"NormalizeGraph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Graph"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Graph"],

Cell["A simple 4-point graph, in edge-list representation.", "ExampleText",
  CellTags->"Graph"],

Cell[BoxData[
    \(\(g = 
        Graph[4, 
          4, "\<EList\>", {{1, 2}, {2, 3}, {3, 1}, {4, 1}}];\)\)], "Input",
  CellLabel->"In[160]:=",
  CellTags->"Graph"],

Cell[BoxData[{
    \(Edges[g]\), "\n", 
    \(VertexSet[g]\)}], "Input",
  CellLabel->"In[161]:=",
  CellTags->"Graph"],

Cell["\<\
The same graph in adjacency matrix and adjacency list \
representation.\
\>", "ExampleText",
  CellTags->"Graph"],

Cell[BoxData[
    \(ToAdjacencyMatrix[g]\)], "Input",
  CellLabel->"In[165]:=",
  CellTags->"Graph"],

Cell[BoxData[
    \(ToAdjacencyList[g]\)], "Input",
  CellLabel->"In[166]:=",
  CellTags->"Graph"],

Cell["Some connected components.", "ExampleText",
  CellTags->"Graph"],

Cell[BoxData[
    \(WeaklyConnectedComponents[g, 3]\)], "Input",
  CellLabel->"In[169]:=",
  CellTags->"Graph"],

Cell[BoxData[
    \(StronglyConnectedComponents[g]\)], "Input",
  CellLabel->"In[168]:=",
  CellTags->"Graph"],

Cell["GridGraph", "ObjectName",
  CellTags->"GridGraph"],

Cell["Constructor", "Highlight",
  CellTags->"GridGraph"],

Cell[TextData[{
  StyleBox["GridGraph[n,m,opts]", "MR"],
  "  constructs a directed grid graph on ",
  StyleBox["n", "MR"],
  " by ",
  StyleBox["m", "MR"],
  " nodes."
}], "Usage",
  CellTags->"GridGraph"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Type", "\[Rule]", "None", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"GridGraph"],

Cell[TextData[{
  "Option  ",
  StyleBox["Type->\"Cylinder\"", "MR"],
  "  causes a cylinder graph to be constructed."
}], "Notes",
  CellTags->"GridGraph"],

Cell[TextData[{
  "Option  ",
  StyleBox["Type->\"Torus\"", "MR"],
  "  causes a torus graph to be constructed."
}], "Notes",
  CellTags->"GridGraph"],

Cell[TextData[{
  "The vertex set is of the form ",
  Cell[BoxData[
      \(TraditionalForm\`{1, \[Ellipsis], n}\[Cross]{1, \[Ellipsis], m}\)]],
  "."
}], "Notes",
  CellTags->"GridGraph"],

Cell[TextData[{
  "See also ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["NormalizeGraph",
    ButtonData:>"NormalizeGraph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"GridGraph"],

Cell["Further Examples", "ExampleSection",
  CellTags->"GridGraph"],

Cell["A grid graph and one of its weakly connected components.", "ExampleText",
  CellTags->"GridGraph"],

Cell[BoxData[
    \(G = GridGraph[3, 5]\)], "Input",
  CellLabel->"In[1540]:=",
  CellTags->"GridGraph"],

Cell[BoxData[
    \(V\  = \ VertexSet[G]\)], "Input",
  CellLabel->"In[1541]:=",
  CellTags->"GridGraph"],

Cell[BoxData[
    \(V[\([\ WeaklyConnectedComponents[G, 7]]\)]\)], "Input",
  CellLabel->"In[1543]:=",
  CellTags->"GridGraph"],

Cell["The cylinder version has 3  strongly connected components.", \
"ExampleText",
  CellTags->"GridGraph"],

Cell[BoxData[
    \(GridGraph[3, 5, Type \[Rule] "\<Cylinder\>"]\  // \ 
      StronglyConnectedComponents\)], "Input",
  CellLabel->"In[11]:=",
  CellTags->"GridGraph"],

Cell["LabelEdgeList", "ObjectName",
  CellTags->"LabelEdgeList"],

Cell["Function", "Highlight",
  CellTags->"LabelEdgeList"],

Cell[TextData[{
  StyleBox["LabelEdgeList[E,lab]", "MR"],
  "  inserts labels into the edge list ",
  StyleBox["E", "MR"],
  " of a graph."
}], "Usage",
  CellTags->"LabelEdgeList"],

Cell["The labels should be given in positional notation.", "Notes",
  CellTags->"LabelEdgeList"],

Cell[TextData[{
  "See ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["TypeGraph",
    ButtonData:>"TypeGraph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"LabelEdgeList"],

Cell["Further Examples", "ExampleSection",
  CellTags->"LabelEdgeList"],

Cell["A path gaph, and its labeled version.", "ExampleText",
  CellTags->"LabelEdgeList"],

Cell["g = PathGraph[4]", "Input",
  CellLabel->"In[569]:=",
  CellTags->"LabelEdgeList"],

Cell[BoxData[
    \(LabelEdgeList[\ g, {1, 1, 2}]\)], "Input",
  CellLabel->"In[570]:=",
  CellTags->"LabelEdgeList"],

Cell["LongestPath", "ObjectName",
  CellTags->"LongestPath"],

Cell["Function", "Highlight",
  CellTags->"LongestPath"],

Cell[TextData[{
  StyleBox["LongestPath[G]", "MR"],
  "  determines the length of a longest path in an acyclic graph ",
  StyleBox["G", "MR"],
  ". "
}], "Usage",
  CellTags->"LongestPath"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"LongestPath"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  " produces the lengths of such longest paths for all vertices."
}], "Notes",
  CellTags->"LongestPath"],

Cell[TextData[{
  StyleBox["LongestPath", "MR"],
  " returns the length of a path ending at the target vertex."
}], "Notes",
  CellTags->"LongestPath"],

Cell["\<\
These paths start at an indegree zero point and end at outdegree \
zero vertices or vertices belonging to a non-trivial strongly connected \
component. \
\>", "Notes",
  CellTags->"LongestPath"],

Cell[TextData[{
  "See also  ",
  ButtonBox["ToDirectedAcyclicGraph",
    ButtonData:>"ToDirectedAcyclicGraph",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["StronglyConnectedComponents",
    ButtonData:>"StronglyConnectedComponents",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"LongestPath"],

Cell["Further Examples", "ExampleSection",
  CellTags->"LongestPath"],

Cell["A small acyclic graph.", "ExampleText",
  CellTags->"LongestPath"],

Cell[BoxData[
    \(\(G\  = \ 
        Graph[6, 
          6, "\<EList\>", {{1, 2}, {1, 5}, {2, 3}, {3, 4}, {3, 6}, {4, 
              5}}];\)\)], "Input",
  CellLabel->"In[573]:=",
  CellTags->"LongestPath"],

Cell[BoxData[
    \(LongestPath[\ G\ ]\)], "Input",
  CellLabel->"In[574]:=",
  CellTags->"LongestPath"],

Cell["The longest path ending at a vertex.", "ExampleText",
  CellTags->"LongestPath"],

Cell[BoxData[
    \(LongestPath[\ G, Full \[Rule] True]\)], "Input",
  CellLabel->"In[575]:=",
  CellTags->"LongestPath"],

Cell["NormalizeGraph", "ObjectName",
  CellTags->"NormalizeGraph"],

Cell["Function", "Highlight",
  CellTags->"NormalizeGraph"],

Cell[TextData[{
  StyleBox["NormalizeGraph[M]", "MR"],
  "  normalizes graph ",
  StyleBox["G", "MR"],
  " to have vertex set [n]."
}], "Usage",
  CellTags->"NormalizeGraph"],

Cell["\<\
The structured vertex set in a normalized graph is the empty list. \
Thus, normalization may significantly reduce the size of the datastructure.\
\
\>", "Notes",
  CellTags->"NormalizeGraph"],

Cell["\<\
Depending on the formatting level, the structured part may be \
invisible. \
\>", "Notes",
  CellTags->"NormalizeGraph"],

Cell[TextData[{
  "See also ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["VertexSet",
    ButtonData:>"VertexSet",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["FormatAutomata",
    ButtonData:>"FormatAutomata",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"NormalizeGraph"],

Cell["Further Examples", "ExampleSection",
  CellTags->"NormalizeGraph"],

Cell[TextData[{
  "A graph with a structured vertex set, obtained from a ",
  StyleBox["NonTransientGraph", "MR"],
  " operation."
}], "ExampleText",
  CellTags->"NormalizeGraph"],

Cell[BoxData[{
    \(g\  = \ 
      Graph[3, 2, "\<EList\>", {{1, 2}, {2, 3}}, {{2, 3, 4}, {5}, {6, 
            7}}]\), "\n", 
    \(VertexSet[g]\)}], "Input",
  CellLabel->"In[578]:=",
  CellTags->"NormalizeGraph"],

Cell[BoxData[{
    \(gg\  = \ NormalizeGraph[g]\), "\n", 
    \(VertexSet[gg]\)}], "Input",
  CellLabel->"In[582]:=",
  CellTags->"NormalizeGraph"],

Cell["NonTransientGraph", "ObjectName",
  CellTags->"NonTransientGraph"],

Cell["Function", "Highlight",
  CellTags->"NonTransientGraph"],

Cell[TextData[{
  StyleBox["NonTransientGraph[G]", "MR"],
  "  returns the non-transient part of the collapse of ",
  StyleBox["G", "MR"],
  "."
}], "Usage",
  CellTags->"NonTransientGraph"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[Rule]", "1", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"NonTransientGraph"],

Cell[TextData[{
  " ",
  StyleBox["NonTransientGraph[G]", "MR"],
  "  removes all transient points; only strongly connected components that \
lie on a biinfinite path are retained and collapsed into a single point."
}], "Notes",
  CellTags->"NonTransientGraph"],

Cell[TextData[StyleBox["The vertex set of the resulting graph is structured \
by default.", "Text"]], "Notes",
  CellTags->"NonTransientGraph"],

Cell[TextData[{
  "See also ",
  ButtonBox["WeaklyConnectedComponents",
    ButtonData:>"WeaklyConnectedComponents",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["StronglyConnectedComponents",
    ButtonData:>"StronglyConnectedComponents",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"NonTransientGraph"],

Cell["Further Examples", "ExampleSection",
  CellTags->"NonTransientGraph"],

Cell["A graph with several transient points.", "ExampleText",
  CellTags->"NonTransientGraph"],

Cell[BoxData[
    \(\(g\  = \ 
        Graph[9, 
          10, "\<EList\>", {{1, 2}, {2, 3}, {3, 4}, {4, 2}, {3, 5}, {5, 
              6}, {5, 9}, {6, 7}, {7, 6}, {7, 8}}];\)\)], "Input",
  CellLabel->"In[586]:=",
  CellTags->"NonTransientGraph"],

Cell[BoxData[{
    \(NonTransientGraph[g]\), "\n", 
    \(VertexSet[%]\)}], "Input",
  CellLabel->"In[587]:=",
  CellTags->"NonTransientGraph"],

Cell["NonTrivial", "ObjectName",
  CellTags->"NonTrivial"],

Cell["Option", "Highlight",
  CellTags->"NonTrivial"],

Cell[TextData[{
  StyleBox["NonTrivial", "MR"],
  "  is an option for command ",
  StyleBox["StronglyConnectedComponents", "MR"],
  ". "
}], "Usage",
  CellTags->"NonTrivial"],

Cell["\<\
Causes only those stronglyconnected components to be generated that \
contain at least one edge (possibly a self-loop).\
\>", "Notes",
  CellTags->"NonTrivial"],

Cell[TextData[{
  "See ",
  ButtonBox["StronglyConnectedComponents",
    ButtonData:>"StronglyConnectedComponents",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"NonTrivial"],

Cell["NumEdges", "ObjectName",
  CellTags->"NumEdges"],

Cell["Function", "Highlight",
  CellTags->"NumEdges"],

Cell[TextData[{
  StyleBox["NumEdges[G]", "MR"],
  "  returns the number of edges of a graph ",
  StyleBox["G", "MR"],
  "."
}], "Usage",
  CellTags->"NumEdges"],

Cell[TextData[{
  "See ",
  ButtonBox["NumVertices",
    ButtonData:>"NumVertices",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Edges",
    ButtonData:>"Edges",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["VertexSet",
    ButtonData:>"VertexSet",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"NumEdges"],

Cell["Further Examples", "ExampleSection",
  CellTags->"NumEdges"],

Cell["Counting edges in some graphs.", "ExampleText",
  CellTags->"NumEdges"],

Cell[BoxData[
    \(CirculantGraph[10, {1, 2, 3}]\  // \ NumEdges\)], "Input",
  CellLabel->"In[591]:=",
  CellTags->"NumEdges"],

Cell[BoxData[
    \(PathGraph[10]\  // \ NumEdges\)], "Input",
  CellLabel->"In[592]:=",
  CellTags->"NumEdges"],

Cell["NumVertices", "ObjectName",
  CellTags->"NumVertices"],

Cell["Function", "Highlight",
  CellTags->"NumVertices"],

Cell[TextData[{
  StyleBox["NumVertices[G]", "MR"],
  "  returns the number of vertices of a graph ",
  StyleBox["G", "MR"],
  "."
}], "Usage",
  CellTags->"NumVertices"],

Cell[TextData[{
  "See ",
  ButtonBox["NumEdges",
    ButtonData:>"NumEdges",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Edges",
    ButtonData:>"Edges",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["VertexSet",
    ButtonData:>"VertexSet",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"NumVertices"],

Cell["Further Examples", "ExampleSection",
  CellTags->"NumVertices"],

Cell["Counting vertices in some graphs.", "ExampleText",
  CellTags->"NumVertices"],

Cell[BoxData[
    \(CirculantGraph[10, {1, 2, 3}]\  // \ NumVertices\)], "Input",
  CellLabel->"In[595]:=",
  CellTags->"NumVertices"],

Cell[BoxData[
    \(PathGraph[10]\  // \ NumVertices\)], "Input",
  CellLabel->"In[596]:=",
  CellTags->"NumVertices"],

Cell["PathGraph", "ObjectName",
  CellTags->"PathGraph"],

Cell["Function", "Highlight",
  CellTags->"PathGraph"],

Cell[TextData[{
  StyleBox["PathGraph[n]", "MR"],
  "  generates a path of length ",
  StyleBox["n", "MR"],
  " in edge-list form. "
}], "Usage",
  CellTags->"PathGraph"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"PathGraph"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  "  adds a self-loop at the last vertex."
}], "Notes",
  CellTags->"PathGraph"],

Cell[TextData[{
  "See also ",
  ButtonBox["CirculantGraph",
    ButtonData:>"CirculantGraph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"PathGraph"],

Cell["Further Examples", "ExampleSection",
  CellTags->"PathGraph"],

Cell["A path graph on 10 points.", "ExampleText",
  CellTags->"PathGraph"],

Cell[BoxData[
    \(g = PathGraph[10]\)], "Input",
  CellLabel->"In[182]:=",
  CellTags->"PathGraph"],

Cell[BoxData[
    \(WeaklyConnectedComponents[g, 3]\)], "Input",
  CellLabel->"In[183]:=",
  CellTags->"PathGraph"],

Cell["The modified version with self-loop.", "ExampleText",
  CellTags->"PathGraph"],

Cell[BoxData[
    \(g = PathGraph[10, Full \[Rule] True]\)], "Input",
  CellLabel->"In[599]:=",
  CellTags->"PathGraph"],

Cell[BoxData[
    \(StronglyConnectedComponents[g, NonTrivial \[Rule] True]\)], "Input",
  CellLabel->"In[600]:=",
  CellTags->"PathGraph"],

Cell["PermuteGraph", "ObjectName",
  CellTags->"PermuteGraph"],

Cell["Function", "Highlight",
  CellTags->"PermuteGraph"],

Cell[TextData[{
  StyleBox["PermuteGraph[G,p]", "MR"],
  "  applies isomorphism ",
  StyleBox["p", "MR"],
  " to ",
  StyleBox["G", "MR"],
  ". "
}], "Usage",
  CellTags->"PermuteGraph"],

Cell[TextData[{
  "List ",
  StyleBox["p", "MR"],
  " must be a permutation on [n]."
}], "Notes",
  CellTags->"PermuteGraph"],

Cell[TextData[{
  "See also ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"PermuteGraph"],

Cell["Further Examples", "ExampleSection",
  CellTags->"PermuteGraph"],

Cell["\<\
A path graph in standard form and after permuting the vertex set.\
\
\>", "ExampleText",
  CellTags->"PermuteGraph"],

Cell["g = PathGraph[6]", "Input",
  CellLabel->"In[604]:=",
  CellTags->"PermuteGraph"],

Cell[BoxData[
    \(gg\  = \ PermuteGraph[g, {2, 1, 3, 4, 6, 5}]\)], "Input",
  CellLabel->"In[605]:=",
  CellTags->"PermuteGraph"],

Cell["ProductGraph", "ObjectName",
  CellTags->"ProductGraph"],

Cell["Function", "Highlight",
  CellTags->"ProductGraph"],

Cell[TextData[{
  StyleBox["ProductGraph[G1,G2]", "MR"],
  "  computes the Cartesian product of two  graphs ",
  StyleBox["G1", "MR"],
  " and ",
  StyleBox["G2", "MR"],
  "."
}], "Usage",
  CellTags->"ProductGraph"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "True", "-"},
          {"Normalize", "\[Rule]", "1", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ProductGraph"],

Cell[TextData[{
  "The result graph has as vertex set the Cartesian product of ",
  StyleBox["V1", "MR"],
  " and ",
  StyleBox["V2", "MR"],
  ", and has an edge ",
  Cell[BoxData[
      \(TraditionalForm\`\((\((p, q)\), \((p', q')\))\)\)]],
  " whenever ",
  Cell[BoxData[
      \(TraditionalForm\`p = p'\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\((q, q')\)\ \  \[Element] \ E\_2\)]],
  ", or ",
  Cell[BoxData[
      \(TraditionalForm\`\((p, p')\)\ \  \[Element] \ E\_1\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`q = q'\)]],
  ". "
}], "Notes",
  CellTags->"ProductGraph"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->False", "MR"],
  " produces the strict product instead:  ",
  Cell[BoxData[
      \(TraditionalForm\`\((p, p')\)\  \[Element] \ E\_1\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\((q, q')\)\  \[Element] \ E\_2\)]],
  "."
}], "Notes",
  CellTags->"ProductGraph"],

Cell[TextData[{
  "See also ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["ProductLabeledGraph",
    ButtonData:>"ProductLabeledGraph",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"ProductGraph"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ProductGraph"],

Cell["Two small graphs and their product, a cylinder.", "ExampleText",
  CellTags->"ProductGraph"],

Cell[BoxData[{
    \(\(g1 = PathGraph[4];\)\), "\n", 
    \(\(g2 = CirculantGraph[3, {1}];\)\)}], "Input",
  CellLabel->"In[1604]:=",
  CellTags->"ProductGraph"],

Cell[BoxData[
    \(g = ProductGraph[g1, g2]\)], "Input",
  CellLabel->"In[1609]:=",
  CellTags->"ProductGraph"],

Cell["By default, the vertex set is now a collection of pairs.", "ExampleText",
  CellTags->"ProductGraph"],

Cell[BoxData[
    \(VertexSet[g]\)], "Input",
  CellLabel->"In[1610]:=",
  CellTags->"ProductGraph"],

Cell["Various components of the product graph.", "ExampleText",
  CellTags->"ProductGraph"],

Cell[BoxData[
    \(WeaklyConnectedComponents[g, 1]\)], "Input",
  CellLabel->"In[1611]:=",
  CellTags->"ProductGraph"],

Cell[BoxData[
    \(WeaklyConnectedComponents[g, 7]\)], "Input",
  CellLabel->"In[1612]:=",
  CellTags->"ProductGraph"],

Cell[BoxData[
    \(StronglyConnectedComponents[g]\)], "Input",
  CellLabel->"In[1613]:=",
  CellTags->"ProductGraph"],

Cell["ProductLabeledGraph", "ObjectName",
  CellTags->"ProductLabeledGraph"],

Cell["Function", "Highlight",
  CellTags->"ProductLabeledGraph"],

Cell[TextData[{
  StyleBox["ProductLabeledGraph[G1,G2]", "MR"],
  "  computes the product of two labeled graphs ",
  StyleBox["G1", "MR"],
  " and ",
  StyleBox["G2", "MR"],
  "."
}], "Usage",
  CellTags->"ProductLabeledGraph"],

Cell[TextData[{
  "The result graph has as vertex set the Cartesian product of ",
  StyleBox["V1", "MR"],
  " and ",
  StyleBox["V2", "MR"],
  ", and has an edge ",
  Cell[BoxData[
      \(TraditionalForm\`\((\((p, q)\), a, \((p', q')\))\)\)]],
  " whenever  ",
  Cell[BoxData[
      \(TraditionalForm\`\((p, a, p')\)\ \  \[Element] \ E\_1\)]],
  "  and   ",
  Cell[BoxData[
      \(TraditionalForm\`\((q, a, q')\)\ \  \[Element] \ E\_2\)]],
  "."
}], "Notes",
  CellTags->"ProductLabeledGraph"],

Cell[TextData[{
  "See also ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["ProductGraph",
    ButtonData:>"ProductGraph",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"ProductLabeledGraph"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ProductLabeledGraph"],

Cell["\<\
A labeled graph on 4 points, whose product with itself has two \
strongly connected components that are isolated in the collapse of the graph.\
\
\>", "ExampleText",
  CellTags->"ProductLabeledGraph"],

Cell[BoxData[
    \(g\  = \ 
      Graph[4, 8, "\<LEList\>", {{1, 1, 1}, {1, 2, 2}, {2, 1, 3}, {2, 2, 
            4}, {3, 2, 1}, {3, 1, 2}, {4, 2, 3}, {4, 1, 4}}]\)], "Input",
  CellLabel->"In[8]:=",
  CellTags->"ProductLabeledGraph"],

Cell[BoxData[
    \(gg\  = \ ProductLabeledGraph[g, g]\)], "Input",
  CellLabel->"In[9]:=",
  CellTags->"ProductLabeledGraph"],

Cell[BoxData[
    \(StronglyConnectedComponents[gg]\)], "Input",
  CellLabel->"In[10]:=",
  CellTags->"ProductLabeledGraph"],

Cell[BoxData[
    \(ToDirectedAcyclicGraph[gg]\)], "Input",
  CellLabel->"In[11]:=",
  CellTags->"ProductLabeledGraph"],

Cell["ReverseGraph", "ObjectName",
  CellTags->"ReverseGraph"],

Cell["Function", "Highlight",
  CellTags->"ReverseGraph"],

Cell[TextData[{
  StyleBox["ReverseGraph[G]", "MR"],
  "  reverses all edges in a graph ",
  StyleBox["G", "MR"],
  "."
}], "Usage",
  CellTags->"ReverseGraph"],

Cell[TextData[{
  StyleBox["ReverseGraph", "MR"],
  " retains the type of the graph."
}], "Notes",
  CellTags->"ReverseGraph"],

Cell[TextData[{
  "See also ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["PermuteGraph",
    ButtonData:>"PermuteGraph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ReverseGraph"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ReverseGraph"],

Cell["A path graph and its reversal.", "ExampleText",
  CellTags->"ReverseGraph"],

Cell["g = PathGraph[6]", "Input",
  CellLabel->"In[624]:=",
  CellTags->"ReverseGraph"],

Cell[BoxData[
    \(ReverseGraph[g]\)], "Input",
  CellLabel->"In[625]:=",
  CellTags->"ReverseGraph"],

Cell[BoxData[
    \(\(g\  // \ ToAdjacencyMatrix\)\  // \ ReverseGraph\)], "Input",
  CellLabel->"In[226]:=",
  CellTags->"ReverseGraph"],

Cell["SetVertices", "ObjectName",
  CellTags->"SetVertices"],

Cell["Function", "Highlight",
  CellTags->"SetVertices"],

Cell[TextData[{
  StyleBox["SetVertices[G,V]", "MR"],
  "  is a mutating operation that sets the vertex set of ",
  StyleBox["G", "MR"],
  " to ",
  StyleBox["V", "MR"],
  "."
}], "Usage",
  CellTags->"SetVertices"],

Cell["\<\
The new vertex set has to have the same cardinality as the old \
vertex set.\
\>", "Notes",
  CellTags->"SetVertices"],

Cell[TextData[{
  "See also ",
  ButtonBox["SetAlphabet",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SetVertices"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SetVertices"],

Cell["\<\
A de Bruijn graph whose natural vertex set consists of words of \
length 2.\
\>", "ExampleText",
  CellTags->"SetVertices"],

Cell[BoxData[
    \(G\  = \ DeBruijnGraph[2, 2, Normalize \[Rule] 1]\)], "Input",
  CellLabel->"In[89]:=",
  CellTags->"SetVertices"],

Cell[BoxData[
    \(VertexSet[G]\)], "Input",
  CellLabel->"In[90]:=",
  CellTags->"SetVertices"],

Cell["\<\
This switches the vertex set to one that records only the Parikh \
vectors of the words.\
\>", "ExampleText",
  CellTags->"SetVertices"],

Cell[BoxData[{
    \(\(SetVertices[G, ParikhVector[VertexSet[G]]\ ];\)\), "\n", 
    \(VertexSet[G]\)}], "Input",
  CellLabel->"In[92]:=",
  CellTags->"SetVertices"],

Cell["ShortestPath", "ObjectName",
  CellTags->"ShortestPath"],

Cell["Function", "Highlight",
  CellTags->"ShortestPath"],

Cell[TextData[{
  StyleBox["ShortestPath[G,s]", "MR"],
  "  constructs a shortest path from source vertex ",
  StyleBox["s", "MR"],
  " in graph ",
  StyleBox["G", "MR"],
  " to all vertices reachable from ",
  StyleBox["s", "MR"],
  "."
}], "Usage",
  CellTags->"ShortestPath"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ShortestPath"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  " will produce the actual paths."
}], "Notes",
  CellTags->"ShortestPath"],

Cell[TextData[{
  "Output is of the form ",
  StyleBox["(u,d)", "MR"],
  " where ",
  StyleBox["u", "MR"],
  " is a vertex and ",
  StyleBox["d", "MR"],
  " is its distance from ",
  StyleBox["s", "MR"],
  "."
}], "Notes",
  CellTags->"ShortestPath"],

Cell[TextData[{
  "See also ",
  ButtonBox["WeaklyConnectedComponents",
    ButtonData:>"WeaklyConnectedComponents",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["LongestPath",
    ButtonData:>"LongestPath",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ShortestPath"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ShortestPath"],

Cell["\<\
A circulant graph and the shortest path associated with vertex 1. \
\
\>", "ExampleText",
  CellTags->"ShortestPath"],

Cell[BoxData[
    \(G = CirculantGraph[10, {1, \(-3\)}]\)], "Input",
  CellLabel->"In[628]:=",
  CellTags->"ShortestPath"],

Cell[BoxData[
    \(ShortestPath[G, 1]\)], "Input",
  CellLabel->"In[629]:=",
  CellTags->"ShortestPath"],

Cell[BoxData[
    \(ShortestPath[G, 1, Full \[Rule] True]\)], "Input",
  CellLabel->"In[84]:=",
  CellTags->"ShortestPath"],

Cell["StronglyConnectedComponents", "ObjectName",
  CellTags->"StronglyConnectedComponents"],

Cell["Function", "Highlight",
  CellTags->"StronglyConnectedComponents"],

Cell[TextData[{
  StyleBox["StronglyConnectedComponents[G]", "MR"],
  "  computes the strongly connected components of graph G using a linear \
algorithm. "
}], "Usage",
  CellTags->"StronglyConnectedComponents"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"NonTrivial", "\[Rule]", "False", "-"},
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"StronglyConnectedComponents"],

Cell[TextData[{
  "Option ",
  StyleBox["NonTrivial->True", "MR"],
  " causes only nontrivial components (containing at least one edge) to be \
returned."
}], "Notes",
  CellTags->"StronglyConnectedComponents"],

Cell["Uses Tarjan's linear time algorithm.", "Notes",
  CellTags->"StronglyConnectedComponents"],

Cell[TextData[{
  "See also ",
  ButtonBox["WeaklyConnectedComponents",
    ButtonData:>"WeaklyConnectedComponents",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["NonTransientGraph",
    ButtonData:>"NonTransientGraph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["LongestPath",
    ButtonData:>"LongestPath",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"StronglyConnectedComponents"],

Cell["Further Examples", "ExampleSection",
  CellTags->"StronglyConnectedComponents"],

Cell["A graph with several strongly connected components.", "ExampleText",
  CellTags->"StronglyConnectedComponents"],

Cell[BoxData[
    \(\(g\  = \ 
        Graph[9, 
          10, "\<EList\>", {{1, 2}, {2, 3}, {3, 4}, {4, 2}, {3, 5}, {5, 
              6}, {5, 9}, {6, 7}, {7, 6}, {7, 8}}];\)\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->"StronglyConnectedComponents"],

Cell[BoxData[
    \(StronglyConnectedComponents[g]\)], "Input",
  CellLabel->"In[9]:=",
  CellTags->"StronglyConnectedComponents"],

Cell["Only two of these components are non-trivial.", "ExampleText",
  CellTags->"StronglyConnectedComponents"],

Cell[BoxData[
    \(StronglyConnectedComponents[g, NonTrivial \[Rule] True]\)], "Input",
  CellLabel->"In[10]:=",
  CellTags->"StronglyConnectedComponents"],

Cell["SubGraph", "ObjectName",
  CellTags->"SubGraph"],

Cell["Function", "Highlight",
  CellTags->"SubGraph"],

Cell[TextData[{
  StyleBox["SubGraph[G,L]", "MR"],
  "  constructs the subgraph of ",
  StyleBox["G", "MR"],
  " induced by vertex set ",
  StyleBox["L", "MR"],
  ". "
}], "Usage",
  CellTags->"SubGraph"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[Rule]", "1", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"SubGraph"],

Cell[TextData[{
  StyleBox["L", "MR"],
  " must be given in positional notation."
}], "Notes",
  CellTags->"SubGraph"],

Cell[TextData[{
  "See also ",
  ButtonBox["PermuteGraph",
    ButtonData:>"PermuteGraph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SubGraph"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SubGraph"],

Cell["\<\
A path graph, and a vertex induced subgraph with two components. \
\
\>", "ExampleText",
  CellTags->"SubGraph"],

Cell["g = PathGraph[8]", "Input",
  CellLabel->"In[639]:=",
  CellTags->"SubGraph"],

Cell[BoxData[
    \(gg\  = \ SubGraph[g, {1, 2, 3, 6, 7, 8}]\)], "Input",
  CellLabel->"In[640]:=",
  CellTags->"SubGraph"],

Cell[BoxData[
    \(VertexSet[gg]\)], "Input",
  CellLabel->"In[641]:=",
  CellTags->"SubGraph"],

Cell[BoxData[{
    \(WeaklyConnectedComponents[gg, 1]\), "\n", 
    \(WeaklyConnectedComponents[gg, 4]\)}], "Input",
  CellLabel->"In[242]:=",
  CellTags->"SubGraph"],

Cell["ToAdjacencyList", "ObjectName",
  CellTags->"ToAdjacencyList"],

Cell["Function", "Highlight",
  CellTags->"ToAdjacencyList"],

Cell[TextData[{
  StyleBox["ToAdjacencyList[G]", "MR"],
  "  converts a graph ",
  StyleBox["G", "MR"],
  " into adjacency list form."
}], "Usage",
  CellTags->"ToAdjacencyList"],

Cell["\<\
The size of the data structure is slightly less than in edge list \
form.\
\>", "Notes",
  CellTags->"ToAdjacencyList"],

Cell["\<\
Note that in out-degree 2 regular graphs one could confuse edge \
list form with adjacency list form.\
\>", "Notes",
  CellTags->"ToAdjacencyList"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToAdjacencyMatrix",
    ButtonData:>"ToAdjacencyMatrix",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToEdgeList",
    ButtonData:>"ToEdgeList",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToAdjacencyList"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToAdjacencyList"],

Cell["Converting from edge list to adjacency lists.", "ExampleText",
  CellTags->"ToAdjacencyList"],

Cell["g = DeBruijnGraph[3,2]", "Input",
  CellLabel->"In[644]:=",
  CellTags->"ToAdjacencyList"],

Cell[BoxData[
    \(\(ToAdjacencyList[g]\  // \ Edges\)\  // \ MatrixForm\)], "Input",
  CellLabel->"In[649]:=",
  CellTags->"ToAdjacencyList"],

Cell[BoxData[
    \(ToAdjacencyList[g]\)], "Input",
  CellLabel->"In[650]:=",
  CellTags->"ToAdjacencyList"],

Cell["ToAdjacencyMatrix", "ObjectName",
  CellTags->"ToAdjacencyMatrix"],

Cell["Function", "Highlight",
  CellTags->"ToAdjacencyMatrix"],

Cell[TextData[{
  StyleBox["ToAdjacencyMatrix[G]", "MR"],
  "  converts a graph ",
  StyleBox["G", "MR"],
  "  into adjacency matrix form."
}], "Usage",
  CellTags->"ToAdjacencyMatrix"],

Cell["\<\
Note that the size of the data structure may increase quadratically \
in this conversion.\
\>", "Notes",
  CellTags->"ToAdjacencyMatrix"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToAdjacencyList",
    ButtonData:>"ToAdjacencyList",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToEdgeList",
    ButtonData:>"ToEdgeList",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToAdjacencyMatrix"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToAdjacencyMatrix"],

Cell["Converting from edge list to adjacency matrix.", "ExampleText",
  CellTags->"ToAdjacencyMatrix"],

Cell["g = DeBruijnGraph[3,2]", "Input",
  CellLabel->"In[653]:=",
  CellTags->"ToAdjacencyMatrix"],

Cell[BoxData[
    \(\(ToAdjacencyMatrix[g]\  // \ Edges\)\  // \ MatrixForm\)], "Input",
  CellLabel->"In[656]:=",
  CellTags->"ToAdjacencyMatrix"],

Cell["ToDirectedAcyclicGraph", "ObjectName",
  CellTags->"ToDirectedAcyclicGraph"],

Cell["Function", "Highlight",
  CellTags->"ToDirectedAcyclicGraph"],

Cell[TextData[{
  StyleBox["ToDirectedAcyclicGraph[G]", "MR"],
  "  computes the directed acyclic graph obtained from ",
  StyleBox["G", "MR"],
  " by collapsing its strongly connected components into points and \
preserving edges between these points."
}], "Usage",
  CellTags->"ToDirectedAcyclicGraph"],

Cell[TextData[{
  "Options\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[Rule]", "1", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ToDirectedAcyclicGraph"],

Cell["\<\
By default, the state set of the collapse is the collection of \
strongly connected componentes of the input graph.\
\>", "Notes",
  CellTags->"ToDirectedAcyclicGraph"],

Cell[TextData[{
  "See also ",
  ButtonBox["StronglyConnectedComponents",
    ButtonData:>"StronglyConnectedComponents",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["NonTransientGraph",
    ButtonData:>"NonTransientGraph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["LongestPath",
    ButtonData:>"LongestPath",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToDirectedAcyclicGraph"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToDirectedAcyclicGraph"],

Cell["A graph with several strongly connected components.", "ExampleText",
  CellTags->"ToDirectedAcyclicGraph"],

Cell[BoxData[
    \(\(g\  = \ 
        Graph[9, 
          10, "\<EList\>", {{1, 2}, {2, 3}, {3, 4}, {4, 2}, {3, 5}, {5, 
              6}, {5, 9}, {6, 7}, {7, 6}, {7, 8}}];\)\)], "Input",
  CellLabel->"In[660]:=",
  CellTags->"ToDirectedAcyclicGraph"],

Cell["The corresponding collapse.", "ExampleText",
  CellTags->"ToDirectedAcyclicGraph"],

Cell[BoxData[{
    \(ToDirectedAcyclicGraph[g]\), "\n", 
    \(VertexSet[%]\)}], "Input",
  CellLabel->"In[661]:=",
  CellTags->"ToDirectedAcyclicGraph"],

Cell["ToEdgeList", "ObjectName",
  CellTags->"ToEdgeList"],

Cell["Function", "Highlight",
  CellTags->"ToEdgeList"],

Cell[TextData[{
  StyleBox["ToEdgeList[G]", "MR"],
  "  converts a graph ",
  StyleBox["G", "MR"],
  " into edge list form."
}], "Usage",
  CellTags->"ToEdgeList"],

Cell[TextData[{
  "See also ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToAdjacencyList",
    ButtonData:>"ToAdjacencyList",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToAdjacencyMatrix",
    ButtonData:>"ToAdjacencyMatrix",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToEdgeList"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToEdgeList"],

Cell["A graph with an adjacency matrix, and its edgelist form.", "ExampleText",
  CellTags->"ToEdgeList"],

Cell[BoxData[
    \(\(g = 
        Graph[8, 16, 
          3, {{1, 1, 0, 0, 0, 0, 0, 0}, {0, 0, 1, 1, 0, 0, 0, 0}, {0, 0, 0, 
              0, 1, 1, 0, 0}, {0, 0, 0, 0, 0, 0, 1, 1}, {1, 1, 0, 0, 0, 0, 0, 
              0}, {0, 0, 1, 1, 0, 0, 0, 0}, {0, 0, 0, 0, 1, 1, 0, 0}, {0, 0, 
              0, 0, 0, 0, 1, 1}}];\)\)], "Input",
  CellLabel->"In[665]:=",
  CellTags->"ToEdgeList"],

Cell[BoxData[
    \(Edges[g]\  // \ MatrixForm\)], "Input",
  CellLabel->"In[666]:=",
  CellTags->"ToEdgeList"],

Cell[BoxData[
    \(ToEdgeList[g]\  // \ Edges\)], "Input",
  CellLabel->"In[667]:=",
  CellTags->"ToEdgeList"],

Cell["ToTDGraph", "ObjectName",
  CellTags->"ToTDGraph"],

Cell["Function", "Highlight",
  CellTags->"ToTDGraph"],

Cell[TextData[{
  StyleBox["ToTDGraph[{RR},m]", "MR"],
  "  converts a sequence of relations ",
  StyleBox["RR", "MR"],
  " to a time dependent graph over ground set ",
  StyleBox["m", "MR"],
  "."
}], "Usage",
  CellTags->"ToTDGraph"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Normalize", "\[Rule]", "0", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ToTDGraph"],

Cell["This is useful mostly for plotting.", "Notes",
  CellTags->"ToTDGraph"],

Cell[TextData[{
  "See ",
  ButtonBox["PlotTDGraph",
    ButtonData:>"PlotTDGraph",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"ToTDGraph"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToTDGraph"],

Cell[TextData[{
  "Two relations based on proper divisibility on ",
  Cell[BoxData[
      \(TraditionalForm\`{1, 2, \[Ellipsis], 8}\)]],
  ". "
}], "ExampleText",
  CellTags->"ToTDGraph"],

Cell[BoxData[
    \(R1 = 
      Select[Tuples[2, 8], 
        Mod[#\[LeftDoubleBracket]2\[RightDoubleBracket], \
#\[LeftDoubleBracket]1\[RightDoubleBracket]] == 
              0 && #\[LeftDoubleBracket]1\[RightDoubleBracket] \[NotEqual] #\
\[LeftDoubleBracket]2\[RightDoubleBracket] &]\)], "Input",
  CellLabel->"In[670]:=",
  CellTags->"ToTDGraph"],

Cell[BoxData[
    \(\(R2\  = \ Reverse\  /@ \ R1;\)\)], "Input",
  CellLabel->"In[671]:=",
  CellTags->"ToTDGraph"],

Cell["ToTDGraph[{R1,R2}, 8]", "Input",
  CellLabel->"In[672]:=",
  CellTags->"ToTDGraph"],

Cell[BoxData[
    \(\(PlotTDGraph[%, 8];\)\)], "Input",
  CellLabel->"In[673]:=",
  CellTags->"ToTDGraph"],

Cell["TypeGraph", "ObjectName",
  CellTags->"TypeGraph"],

Cell["Function", "Highlight",
  CellTags->"TypeGraph"],

Cell[TextData[{
  StyleBox["TypeGraph[G]", "MR"],
  "  returns the type of a graph ",
  StyleBox["G", "MR"],
  "."
}], "Usage",
  CellTags->"TypeGraph"],

Cell["\<\
 Graphs can be represented by an edge list (EList), a labeled edge \
list (LEList), an adjacency matrix (AMat) or an adjacency list (AList).\
\>", \
"Notes",
  CellTags->"TypeGraph"],

Cell[" Graphs types are indicated by strings, not symbols.", "Notes",
  CellTags->"TypeGraph"],

Cell[TextData[{
  "See also ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"TypeGraph"],

Cell["Further Examples", "ExampleSection",
  CellTags->"TypeGraph"],

Cell["A simple 4-point graph, in edge-list representation.", "ExampleText",
  CellTags->"TypeGraph"],

Cell[BoxData[
    \(\(g = 
        Graph[4, 
          4, "\<EList\>", {{1, 2}, {2, 3}, {3, 1}, {4, 1}}];\)\)], "Input",
  CellLabel->"In[76]:=",
  CellTags->"TypeGraph"],

Cell[BoxData[
    \(TypeGraph[g]\)], "Input",
  CellLabel->"In[77]:=",
  CellTags->"TypeGraph"],

Cell[BoxData[
    \(\(g\  // \ ToAdjacencyList\)\  // \ TypeGraph\)], "Input",
  CellLabel->"In[171]:=",
  CellTags->"TypeGraph"],

Cell[BoxData[
    \(\(g\  // \ ToAdjacencyMatrix\)\  // \ TypeGraph\)], "Input",
  CellLabel->"In[172]:=",
  CellTags->"TypeGraph"],

Cell["VertexSet", "ObjectName",
  CellTags->"VertexSet"],

Cell["Function", "Highlight",
  CellTags->"VertexSet"],

Cell[TextData[{
  StyleBox["VertexSet[G]", "MR"],
  "  returns the vertex set of a graph ",
  StyleBox["G", "MR"],
  "."
}], "Usage",
  CellTags->"VertexSet"],

Cell[TextData[{
  "If there is no structured vertex set, ",
  Cell[BoxData[
      \(TraditionalForm\`{1, 2, \[Ellipsis], n}\)]],
  "  is returned."
}], "Notes",
  CellTags->"VertexSet"],

Cell[TextData[{
  "See also ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Edges",
    ButtonData:>"Edges",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"VertexSet"],

Cell["Further Examples", "ExampleSection",
  CellTags->"VertexSet"],

Cell["Some graph operations produce structured vertex sets.", "ExampleText",
  CellTags->"VertexSet"],

Cell[BoxData[
    \(\(g = DeBruijnGraph[3, 2, Normalize \[Rule] 1];\)\)], "Input",
  CellLabel->"In[676]:=",
  CellTags->"VertexSet"],

Cell[BoxData[
    \(VertexSet[g]\)], "Input",
  CellLabel->"In[677]:=",
  CellTags->"VertexSet"],

Cell["WeaklyConnectedComponents", "ObjectName",
  CellTags->"WeaklyConnectedComponents"],

Cell["Function", "Highlight",
  CellTags->"WeaklyConnectedComponents"],

Cell[TextData[{
  StyleBox["WeaklyConnectedComponents[G,X]", "MR"],
  "  computes the weakly connected component of vertex set ",
  StyleBox["X", "MR"],
  " in graph ",
  StyleBox["G", "MR"],
  ". "
}], "Usage",
  CellTags->"WeaklyConnectedComponents"],

Cell["\<\
X can be a single vertex or a list of vertices; in either case, \
given in positional notation.\
\>", "Notes",
  CellTags->"WeaklyConnectedComponents"],

Cell[TextData[{
  "See also ",
  ButtonBox["StronglyConnectedComponents",
    ButtonData:>"StronglyConnectedComponents",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Graph",
    ButtonData:>"Graph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WeaklyConnectedComponents"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WeaklyConnectedComponents"],

Cell["Components in a circulant graph", "ExampleText",
  CellTags->"WeaklyConnectedComponents"],

Cell["g = CirculantGraph[ 12, {3,-3} ]", "Input",
  CellLabel->"In[680]:=",
  CellTags->"WeaklyConnectedComponents"],

Cell[BoxData[{
    \(WeaklyConnectedComponents[g, 1]\), "\n", 
    \(WeaklyConnectedComponents[g, 2]\), "\n", 
    \(WeaklyConnectedComponents[g, {1, 2}]\)}], "Input",
  CellLabel->"In[681]:=",
  CellTags->"WeaklyConnectedComponents"],

Cell["$generateSGBound", "ObjectName",
  CellTags->"$generateSGBound"],

Cell["SysVariable", "Highlight",
  CellTags->"$generateSGBound"],

Cell[TextData[{
  StyleBox["$generateSGBound", "MR"],
  "  is a default bound for the size of a semigroup generated  by ",
  StyleBox["GenerateSG", "MR"],
  ". "
}], "Usage",
  CellTags->"$generateSGBound"],

Cell["Default is 1000.", "Notes",
  CellTags->"$generateSGBound"],

Cell[TextData[{
  "See ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"$generateSGBound"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$generateSGBound"],

Cell["The system default value is 5000:", "ExampleText",
  CellTags->"$generateSGBound"],

Cell[BoxData[
    \($generateSGBound\)], "Input",
  CellLabel->"In[686]:=",
  CellTags->"$generateSGBound"],

Cell["$identitystring", "ObjectName",
  CellTags->"$identitystring"],

Cell["SysVariable", "Highlight",
  CellTags->"$identitystring"],

Cell[TextData[{
  StyleBox["$identitystring", "MR"],
  "   denotes the identity element of a monoid in rewrite rules produced by \
",
  StyleBox["GenerateSG", "MR"],
  "."
}], "Usage",
  CellTags->"$identitystring"],

Cell["\<\
 Default is 1 and should be changed if digit alphabets are \
used.\
\>", "Notes",
  CellTags->"$identitystring"],

Cell[TextData[{
  "See ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToIdentityString",
    ButtonData:>"ToIdentityString",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"$identitystring"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$identitystring"],

Cell["The system default value is \"1\":", "ExampleText",
  CellTags->"$identitystring"],

Cell[BoxData[
    \(\(\($identitystring\)\(\ \)\)\)], "Input",
  CellLabel->"In[687]:=",
  CellTags->"$identitystring"],

Cell["$nullstring", "ObjectName",
  CellTags->"$nullstring"],

Cell["SysVariable", "Highlight",
  CellTags->"$nullstring"],

Cell[TextData[{
  StyleBox["$nullstring", "MR"],
  "   denotes the null element of a semigroup in rewrite rules produced by ",
  StyleBox["GenerateSG", "MR"],
  ". "
}], "Usage",
  CellTags->"$nullstring"],

Cell["\<\
Default is 0 and should be changed if digit alphabets are \
used.\
\>", "Notes",
  CellTags->"$nullstring"],

Cell[TextData[{
  "See ",
  ButtonBox["SimplifyEquations",
    ButtonData:>"SimplifyEquations",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["WordReduce",
    ButtonData:>"WordReduce",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"$nullstring"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$nullstring"],

Cell["The system default value is \"0\":", "ExampleText",
  CellTags->"$nullstring"],

Cell[BoxData[
    \(\(\($nullstring\)\(\ \)\)\)], "Input",
  CellLabel->"In[688]:=",
  CellTags->"$nullstring"],

Cell["AllTimes", "ObjectName",
  CellTags->"AllTimes"],

Cell["Function", "Highlight",
  CellTags->"AllTimes"],

Cell[TextData[{
  StyleBox["AllTimes[L1,L2]", "MR"],
  "  computes all products between elements in ",
  StyleBox["L1", "MR"],
  " and ",
  StyleBox["L2", "MR"],
  " using ",
  StyleBox["NonCommutativeMultiply", "MR"],
  "."
}], "Usage",
  CellTags->"AllTimes"],

Cell[TextData[{
  "Repetitions in the output are not eliminated, so the length of the output \
list is always the product of the lengths of ",
  StyleBox["L1", "MR"],
  " and ",
  StyleBox["L2", "MR"],
  "."
}], "Notes",
  CellTags->"AllTimes"],

Cell["Handles more than two lists properly.", "Notes",
  CellTags->"AllTimes"],

Cell[TextData[{
  "See also ",
  ButtonBox["T",
    ButtonData:>"T",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SG",
    ButtonData:>"SG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"AllTimes"],

Cell["Further Examples", "ExampleSection",
  CellTags->"AllTimes"],

Cell["Transformations use NonCommutativeMultiply as composition.", \
"ExampleText",
  CellTags->"AllTimes"],

Cell["AllTimes[ {T[2,3,1],T[2,1,3]}, {T[2,3,1],T[2,1,3]}]", "Input",
  CellLabel->"In[689]:=",
  CellTags->"AllTimes"],

Cell["\<\
Another application will generate the whole symmetric group on 3 \
letters.\
\>", "ExampleText",
  CellTags->"AllTimes"],

Cell["\<\
AllTimes[ %, {T[2,3,1],T[2,1,3]}]
% // Union\
\>", "Input",
  CellLabel->"In[2]:=",
  CellTags->"AllTimes"],

Cell["AlternatingGroupSG", "ObjectName",
  CellTags->"AlternatingGroupSG"],

Cell[TextData[{
  StyleBox["AlternatingGroupSG[n]", "MR"],
  "  returns the alternating group on ",
  StyleBox["n", "MR"],
  " elements. The group is returned as a semigroup object. "
}], "Usage",
  CellTags->"AlternatingGroupSG"],

Cell[TextData[{
  "Since the alternating group has ",
  Cell[BoxData[
      \(TraditionalForm\`\(n!\)/2\)]],
  " elements, this can only be used for small ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  "."
}], "Notes",
  CellTags->"AlternatingGroupSG"],

Cell[TextData[{
  "See also ",
  ButtonBox["DihedralGroupSG",
    ButtonData:>"DihedralGroupSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SymmetricGroupSG",
    ButtonData:>"SymmetricGroupSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SG",
    ButtonData:>"SG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["T",
    ButtonData:>"T",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"AlternatingGroupSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"AlternatingGroupSG"],

Cell["The alternating group on 4 elements. ", "ExampleText",
  CellTags->"AlternatingGroupSG"],

Cell["AlternatingGroupSG[4]", "Input",
  CellLabel->"In[694]:=",
  CellTags->"AlternatingGroupSG"],

Cell[TextData[{
  "As permutations, the elements all have signature ",
  Cell[BoxData[
      \(TraditionalForm\`\(+1\)\)]],
  "."
}], "ExampleText",
  CellTags->"AlternatingGroupSG"],

Cell["\<\

Signature /@ ToList[%]\
\>", "Input",
  CellLabel->"In[695]:=",
  CellTags->"AlternatingGroupSG"],

Cell["AperiodicQSG", "ObjectName",
  CellTags->"AperiodicQSG"],

Cell["Function", "Highlight",
  CellTags->"AperiodicQSG"],

Cell[TextData[{
  StyleBox["AperiodicQSG[S]", "MR"],
  "  determines whether semigroup ",
  StyleBox["S", "MR"],
  " is aperiodic. "
}], "Usage",
  CellTags->"AperiodicQSG"],

Cell[TextData[{
  "Also allows as input a list or sequence of generators for ",
  StyleBox["S", "MR"],
  "."
}], "Notes",
  CellTags->"AperiodicQSG"],

Cell[TextData[{
  "A single transformation ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " is also accepted as input.  Tests whether there exists ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " such that ",
  Cell[BoxData[
      \(TraditionalForm\`x\^\(\(\ \)\(n + 1\)\) = \ x\^\(\(\ \)\(n\)\)\)]],
  "."
}], "Notes",
  CellTags->"AperiodicQSG"],

Cell[TextData[{
  "See also ",
  ButtonBox["AperiodicSG",
    ButtonData:>"AperiodicSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SG",
    ButtonData:>"SG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["T",
    ButtonData:>"T",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"AperiodicQSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"AperiodicQSG"],

Cell["\<\
An aperiodic transformation semigroup with lots of constants.\
\>", \
"ExampleText",
  CellTags->"AperiodicQSG"],

Cell[BoxData[
    \(\(S\  = \ 
        SG[T[1, 1, 1, 1, 1, 1, 1, 1], T[1, 4, 6, 8, 6, 8, 4, 1], 
          T[1, 8, 8, 1, 8, 1, 8, 1], T[2, 2, 2, 2, 2, 2, 2, 2], 
          T[2, 3, 5, 7, 5, 7, 3, 2], T[2, 7, 7, 2, 7, 2, 7, 2], 
          T[3, 3, 3, 3, 3, 3, 3, 3], T[3, 5, 5, 3, 5, 3, 5, 3], 
          T[4, 4, 4, 4, 4, 4, 4, 4], T[4, 6, 6, 4, 6, 4, 6, 4], 
          T[5, 5, 5, 5, 5, 5, 5, 5], T[6, 6, 6, 6, 6, 6, 6, 6], 
          T[7, 7, 7, 7, 7, 7, 7, 7], T[8, 8, 8, 8, 8, 8, 8, 8]];\)\)], "Input",\

  CellLabel->"In[698]:=",
  CellTags->"AperiodicQSG"],

Cell[BoxData[
    \(AperiodicQSG[S]\)], "Input",
  CellLabel->"In[699]:=",
  CellTags->"AperiodicQSG"],

Cell["But the symmetric group fails to be aperiodic.", "ExampleText",
  CellTags->"AperiodicQSG"],

Cell["AperiodicQSG[{T[2,3,4,1],T[2,1,3,4]}]", "Input",
  CellLabel->"In[700]:=",
  CellTags->"AperiodicQSG"],

Cell["AperiodicSG", "ObjectName",
  CellTags->"AperiodicSG"],

Cell["Function", "Highlight",
  CellTags->"AperiodicSG"],

Cell[TextData[{
  StyleBox["AperiodicSG[S]", "MR"],
  "  returns the aperiodic elements of semigroup ",
  StyleBox["S", "MR"],
  "."
}], "Usage",
  CellTags->"AperiodicSG"],

Cell[TextData[{
  "A transformation is aperiodic if ",
  Cell[BoxData[
      \(TraditionalForm\`x\^\(n + 1\) = x\^n\)]],
  " for some ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  ". "
}], "Notes",
  CellTags->"AperiodicSG"],

Cell[TextData[{
  "See also  ",
  ButtonBox["AperiodicQSG",
    ButtonData:>"AperiodicQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["SG",
    ButtonData:>"SG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["T",
    ButtonData:>"T",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"AperiodicSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"AperiodicSG"],

Cell["\<\
The full transformation semigroup on 3 elements contains 16 \
aperiodic elements. \
\>", "ExampleText",
  CellTags->"AperiodicSG"],

Cell[BoxData[
    \(\(S\  = \ TransformationSG[3];\)\)], "Input",
  CellLabel->"In[714]:=",
  CellTags->"AperiodicSG"],

Cell["\<\
AperiodicSG[S]
% // Length\
\>", "Input",
  CellLabel->"In[715]:=",
  CellTags->"AperiodicSG"],

Cell["BlocksToDClass", "ObjectName",
  CellTags->"BlocksToDClass"],

Cell["Function", "Highlight",
  CellTags->"BlocksToDClass"],

Cell[TextData[{
  StyleBox["BlocksToDClass[db,S]", "MR"],
  "  converts a D-class in semigroup ",
  StyleBox["S", "MR"],
  " from block representation to multiplier representation."
}], "Usage",
  CellTags->"BlocksToDClass"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"BlocksToDClass"],

Cell[TextData[{
  "The input is a matrix of the form  ",
  Cell[BoxData[
      \(TraditionalForm\`\((H\_\(i\[InvisibleComma]j\))\)\)]],
  "  where each ",
  Cell[BoxData[
      \(TraditionalForm\`H\)]],
  "-class ",
  Cell[BoxData[
      \(TraditionalForm\`H\_\(i\[InvisibleComma]j\)\)]],
  " is given as a list of transformations. The output is of the form ",
  StyleBox["DC[L,R,A]. ", "MR"]
}], "Notes",
  CellTags->"BlocksToDClass"],

Cell[TextData[{
  "The dimensions of the tensor can be determined with ",
  ButtonBox["DClassSize",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"BlocksToDClass"],

Cell[TextData[{
  "See also ",
  ButtonBox["DClassToBlocks",
    ButtonData:>"DClassToBlocks",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DC",
    ButtonData:>"DC",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"BlocksToDClass"],

Cell["Further Examples", "ExampleSection",
  CellTags->"BlocksToDClass"],

Cell["\<\
A small D-class represented by a matrix of H-classes, and the \
corresponding representation in standard form.\
\>", "ExampleText",
  CellTags->"BlocksToDClass"],

Cell[BoxData[
    \(\(d\  = \ {{{T[1, 4, 4, 4]}, {T[2, 4, 4, 4]}, {T[3, 4, 4, 4]}}, {{T[4, 
                4, 1, 4]}, {T[4, 4, 2, 4]}, {T[4, 4, 3, 4]}}, {{T[4, 1, 4, 
                4]}, {T[4, 2, 4, 4]}, {T[4, 3, 4, 4]}}};\)\)], "Input",
  CellLabel->"In[785]:=",
  CellTags->"BlocksToDClass"],

Cell[BoxData[
    \(Dimensions[d]\)], "Input",
  CellLabel->"In[786]:=",
  CellTags->"BlocksToDClass"],

Cell[BoxData[
    \(BlocksToDClass[d]\)], "Input",
  CellLabel->"In[787]:=",
  CellTags->"BlocksToDClass"],

Cell["Converting back to block form.", "ExampleText",
  CellTags->"BlocksToDClass"],

Cell[BoxData[
    \(DClassToBlocks[%] === d\)], "Input",
  CellLabel->"In[788]:=",
  CellTags->"BlocksToDClass"],

Cell["BoundCSM", "ObjectName",
  CellTags->"BoundCSM"],

Cell["Option", "Highlight",
  CellTags->"BoundCSM"],

Cell[TextData[{
  StyleBox["BoundCSM", "MR"],
  "  is an option for ",
  StyleBox["GenerateCSM", "MR"],
  ", limits the size of the generated cyclic semimodule. "
}], "Usage",
  CellTags->"BoundCSM"],

Cell["Default is 2000.", "Notes",
  CellTags->"BoundCSM"],

Cell[TextData[{
  "See also ",
  ButtonBox["GenerateCSM",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"BoundCSM"],

Cell["BoundGSG", "ObjectName",
  CellTags->"BoundGSG"],

Cell["SysVariable", "Highlight",
  CellTags->"BoundGSG"],

Cell[TextData[{
  StyleBox["BoundGSG", "MR"],
  "  is an option that controls the size of a semigroup generated by ",
  StyleBox["GenerateSG", "MR"],
  ". "
}], "Usage",
  CellTags->"BoundGSG"],

Cell[TextData[{
  "Default is ",
  StyleBox["$generateSGBound", "MR"],
  " = 5000."
}], "Notes",
  CellTags->"BoundGSG"],

Cell[TextData[{
  "See also ",
  ButtonBox["$generateSGBound",
    ButtonData:>"$generateSGBound",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"BoundGSG"],

Cell["CollapseT", "ObjectName",
  CellTags->"CollapseT"],

Cell["Function", "Highlight",
  CellTags->"CollapseT"],

Cell[TextData[{
  StyleBox["CollapseT[t]", "MR"],
  "  produces the image transformation associated with transformation ",
  StyleBox["t", "MR"],
  ". "
}], "Usage",
  CellTags->"CollapseT"],

Cell[TextData[{
  "If ",
  StyleBox["t", "MR"],
  " lies in a group, ",
  StyleBox["CollapseT[t]", "MR"],
  " produces a permutation."
}], "Notes",
  CellTags->"CollapseT"],

Cell[TextData[{
  "All permutations are fixed points of  ",
  StyleBox["CollapseT", "MR"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"CollapseT"],

Cell[TextData[{
  "See also ",
  ButtonBox["OrderT",
    ButtonData:>"OrderT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ImageT",
    ButtonData:>"ImageT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["RankT",
    ButtonData:>"RankT",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["KernelT",
    ButtonData:>"KernelT",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["KernelImageToT",
    ButtonData:>"KernelImageToT",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"CollapseT"],

Cell["Further Examples", "ExampleSection",
  CellTags->"CollapseT"],

Cell["A transformation and its collapse.", "ExampleText",
  CellTags->"CollapseT"],

Cell["t = T[2,3,4,1,1]", "Input",
  CellLabel->"In[724]:=",
  CellTags->"CollapseT"],

Cell[BoxData[{
    \(ImageT[t]\), "\n", 
    \(KernelT[t]\)}], "Input",
  CellLabel->"In[725]:=",
  CellTags->"CollapseT"],

Cell[BoxData[
    \(CollapseT[t]\)], "Input",
  CellLabel->"In[727]:=",
  CellTags->"CollapseT"],

Cell["ConstantT", "ObjectName",
  CellTags->"ConstantT"],

Cell["Constructor", "Highlight",
  CellTags->"ConstantT"],

Cell[TextData[{
  "ConstantT",
  StyleBox["[k,n]", "MR"],
  "  returns the constant transformation ",
  StyleBox["T[k,\[Ellipsis],k]", "MR"],
  " of length ",
  StyleBox["n", "MR"],
  "."
}], "Usage",
  CellTags->"ConstantT"],

Cell[TextData[{
  "See also ",
  ButtonBox["OneT",
    ButtonData:>"OneT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["CycleToT",
    ButtonData:>"CycleToT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["T",
    ButtonData:>"T",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ConstantT"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ConstantT"],

Cell["A constant transformation.", "ExampleText",
  CellTags->"ConstantT"],

Cell[BoxData[{
    \(ConstantT[1, 5]\), "\n", 
    \(OrderT[%]\)}], "Input",
  CellLabel->"In[373]:=",
  CellTags->"ConstantT"],

Cell["CycleDecompositionT", "ObjectName",
  CellTags->"CycleDecompositionT"],

Cell["Function", "Highlight",
  CellTags->"CycleDecompositionT"],

Cell[TextData[{
  StyleBox["CycleDecompositionT[t]", "MR"],
  "  returns the cycle decomposition of a permutation t. "
}], "Usage",
  CellTags->"CycleDecompositionT"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"CycleDecompositionT"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  " causes fixed points to be displayed."
}], "Notes",
  CellTags->"CycleDecompositionT"],

Cell["\<\
No test is made whether the given transformation is in fact a \
permutation, for arbitrary transformations a list of some orbits is produced.\
\
\>", "Notes",
  CellTags->"CycleDecompositionT"],

Cell[TextData[{
  "See also ",
  ButtonBox["CycleToT",
    ButtonData:>"CycleToT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["OrderT",
    ButtonData:>"OrderT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["T",
    ButtonData:>"T",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"CycleDecompositionT"],

Cell["Further Examples", "ExampleSection",
  CellTags->"CycleDecompositionT"],

Cell["A permutation and its cycle decomposition.", "ExampleText",
  CellTags->"CycleDecompositionT"],

Cell["\<\
t = T[ 2, 3, 1, 5, 4, 6 ];
CycleDecompositionT[t]\
\>", "Input",
  CellLabel->"In[324]:=",
  CellTags->"CycleDecompositionT"],

Cell[BoxData[
    \(CycleDecompositionT[t, Full \[Rule] True]\)], "Input",
  CellLabel->"In[326]:=",
  CellTags->"CycleDecompositionT"],

Cell["CycleToT", "ObjectName",
  CellTags->"CycleToT"],

Cell["Function", "Highlight",
  CellTags->"CycleToT"],

Cell[TextData[{
  StyleBox["CycleToT[cycles,n]", "MR"],
  "  converts a cycle decomposition to a permutation, given as a \
transformation."
}], "Usage",
  CellTags->"CycleToT"],

Cell["Fixed points are added automatically.", "Notes",
  CellTags->"CycleToT"],

Cell[TextData[{
  "See also ",
  ButtonBox["CycleDecompositionT",
    ButtonData:>"CycleDecompositionT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["OrderT",
    ButtonData:>"OrderT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["T",
    ButtonData:>"T",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"CycleToT"],

Cell["Further Examples", "ExampleSection",
  CellTags->"CycleToT"],

Cell["A permutation with two cycles.", "ExampleText",
  CellTags->"CycleToT"],

Cell[BoxData[
    \(t = CycleToT[{{1, 2, 3}, {4, 5}}, 5]\)], "Input",
  CellLabel->"In[735]:=",
  CellTags->"CycleToT"],

Cell[BoxData[
    \(OrderT[t]\)], "Input",
  CellLabel->"In[322]:=",
  CellTags->"CycleToT"],

Cell[BoxData[
    \(\(PlotT[t, 6, DoTrace \[Rule] {1, 4}, Full \[Rule] True];\)\)], "Input",\

  CellLabel->"In[317]:=",
  CellTags->"CycleToT"],

Cell[BoxData[
    \(CycleToT[{{4, 5}}, 5]\)], "Input",
  CellLabel->"In[323]:=",
  CellTags->"CycleToT"],

Cell["DC", "ObjectName",
  CellTags->"DC"],

Cell["Constructor", "Highlight",
  CellTags->"DC"],

Cell[TextData[{
  StyleBox["DC[L,R,H]", "MR"],
  "  is an expression denoting a D-class in a transformation semigroup. "
}], "Usage",
  CellTags->"DC"],

Cell[TextData[{
  StyleBox["H", "MR"],
  " is the basic ",
  Cell[BoxData[
      \(TraditionalForm\`H\)]],
  "-class, ",
  StyleBox["L", "MR"],
  " a list of left multipliers and ",
  StyleBox["R", "MR"],
  " a list of right multipliers."
}], "Notes",
  CellTags->"DC"],

Cell[TextData[{
  "See also ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["DClassSize",
    ButtonData:>"DClassSize",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SG",
    ButtonData:>"SG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["T",
    ButtonData:>"T",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DC"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DC"],

Cell["A  D-class, its size and its dimensions.", "ExampleText",
  CellTags->"DC"],

Cell[BoxData[
    \(\(d\  = \ 
        DC[{T[1, 1, 1, 4], T[1, 1, 4, 1], T[1, 1, 4, 4], T[1, 4, 1, 1], 
            T[1, 4, 1, 4], T[1, 4, 4, 1], T[1, 4, 4, 4]}, {T[1, 1, 1, 2], 
            T[1, 1, 1, 3], T[1, 1, 1, 4], T[2, 1, 1, 3], T[2, 1, 1, 4], 
            T[3, 1, 1, 4]}, {T[1, 1, 1, 4], T[4, 4, 4, 1]}];\)\)], "Input",
  CellLabel->"In[150]:=",
  CellTags->"DC"],

Cell[BoxData[{
    \(DClassSize[d]\), "\n", 
    \(DClassSize[d, Full \[Rule] True]\)}], "Input",
  CellLabel->"In[22]:=",
  CellTags->"DC"],

Cell["DClasses", "ObjectName",
  CellTags->"DClasses"],

Cell["Option", "Highlight",
  CellTags->"DClasses"],

Cell[TextData[{
  StyleBox["DClasses", "MR"],
  "  is an option for ",
  StyleBox["GenerateSG", "MR"],
  "."
}], "Usage",
  CellTags->"DClasses"],

Cell[TextData[{
  "Default value is ",
  StyleBox["False", "MR"],
  "."
}], "Notes",
  CellTags->"DClasses"],

Cell[TextData[{
  "See also ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DC",
    ButtonData:>"DC",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DClassDecomposition",
    ButtonData:>"DClassDecomposition",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DClasses"],

Cell["DClassDecomposition", "ObjectName",
  CellTags->"DClassDecomposition"],

Cell["Function", "Highlight",
  CellTags->"DClassDecomposition"],

Cell[TextData[{
  StyleBox["DClassDecomposition[S]", "MR"],
  "  generates a list of all regular D-classes of semigroup ",
  StyleBox["S", "MR"],
  ". "
}], "Usage",
  CellTags->"DClassDecomposition"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"RegularOnly", "\[Rule]", "True", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"DClassDecomposition"],

Cell[TextData[{
  "Option ",
  StyleBox["RegularOnly->False", "MR"],
  " causes all D-classes to be generated. "
}], "Notes",
  CellTags->"DClassDecomposition"],

Cell[TextData[{
  "Note that this involves computation of left and right ideals generated by \
irregular elements of ",
  StyleBox["S", "MR"],
  " and becomes quite slow for large ",
  StyleBox["S", "MR"],
  "."
}], "Notes",
  CellTags->"DClassDecomposition"],

Cell["\<\
At present, there is no external implementation of this \
operation.\
\>", "Notes",
  CellTags->"DClassDecomposition"],

Cell[TextData[{
  "Note that this involves computation of left and right ideals generated by \
irregular elements of ",
  StyleBox["S", "MR"],
  " and becomes quite slow for large ",
  StyleBox["S", "MR"],
  "."
}], "Notes",
  CellTags->"DClassDecomposition"],

Cell[TextData[{
  "See also ",
  ButtonBox["DClasses",
    ButtonData:>"DClasses",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DC",
    ButtonData:>"DC",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DClassDecomposition"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DClassDecomposition"],

Cell["\<\
The full transformation semigroup on four points, and its \
decomposition.\
\>", "ExampleText",
  CellTags->"DClassDecomposition"],

Cell[BoxData[{
    \(\(S = TransformationSG[4];\)\), "\n", 
    \(S\  // \ Length\)}], "Input",
  CellLabel->"In[898]:=",
  CellTags->"DClassDecomposition"],

Cell[BoxData[
    \(\(dcl\  = \ DClassDecomposition[S];\)\)], "Input",
  CellLabel->"In[740]:=",
  CellTags->"DClassDecomposition"],

Cell[BoxData[{
    \(DClassSize[dcl]\), "\n", 
    \(DClassSize[dcl, Full \[Rule] True]\)}], "Input",
  CellLabel->"In[741]:=",
  CellTags->"DClassDecomposition"],

Cell["The second D-class in the decomposition, in block form.", "ExampleText",
  CellTags->"DClassDecomposition"],

Cell[BoxData[
    \(\(dcl[\([2]\)]\  // \ DClassToBlocks\)\ \  // \ MatrixForm\)], "Input",
  CellLabel->"In[126]:=",
  CellTags->"DClassDecomposition"],

Cell["DClassSize", "ObjectName",
  CellTags->"DClassSize"],

Cell["Function", "Highlight",
  CellTags->"DClassSize"],

Cell[TextData[{
  StyleBox["DClassSize[dc]", "MR"],
  "  returns the size of D-class ",
  StyleBox["dc", "MR"],
  ". "
}], "Usage",
  CellTags->"DClassSize"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"DClassSize"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  "  returns the dimensions of the ",
  Cell[BoxData[
      \(TraditionalForm\`D\)]],
  "-class instead: a triple  number of ",
  Cell[BoxData[
      \(TraditionalForm\`R\)]],
  "-classes, number of ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  "-classes, size of ",
  Cell[BoxData[
      \(TraditionalForm\`H\)]],
  "-classes."
}], "Notes",
  CellTags->"DClassSize"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"DClassSize"],

Cell[TextData[{
  "See also ",
  ButtonBox["DClassDecomposition",
    ButtonData:>"DClassDecomposition",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DC",
    ButtonData:>"DC",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DClassSize"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DClassSize"],

Cell["A D-class, in standard form.", "ExampleText",
  CellTags->"DClassSize"],

Cell[BoxData[{
    \(\(d\  = \ 
        DC[{T[1, 1, 1, 4], T[1, 1, 4, 1], T[1, 1, 4, 4], T[1, 4, 1, 1], 
            T[1, 4, 1, 4], T[1, 4, 4, 1], T[1, 4, 4, 4]}, {T[1, 1, 1, 2], 
            T[1, 1, 1, 3], T[1, 1, 1, 4], T[2, 1, 1, 3], T[2, 1, 1, 4], 
            T[3, 1, 1, 4]}, {T[1, 1, 1, 4], T[4, 4, 4, 1]}];\)\), "\n", 
    \(DClassSize[d]\), "\n", 
    \(DClassSize[d, Full \[Rule] False]\)}], "Input",
  CellLabel->"In[18]:=",
  CellTags->"DClassSize"],

Cell["The 84 elements in arranged in the usual grid.", "ExampleText",
  CellTags->"DClassSize"],

Cell[BoxData[
    \(\(d\  // \ DClassToBlocks\)\  // \ MatrixForm\)], "Input",
  CellLabel->"In[21]:=",
  CellTags->"DClassSize"],

Cell["DClassT", "ObjectName",
  CellTags->"DClassT"],

Cell["Function", "Highlight",
  CellTags->"DClassT"],

Cell[TextData[{
  StyleBox["DClassT[x,S]", "MR"],
  "  determines the D-class of a regular element ",
  StyleBox["x", "MR"],
  " in a semigroup ",
  StyleBox["S", "MR"],
  ". "
}], "Usage",
  CellTags->"DClassT"],

Cell["Coordinates are given as left and right multipliers.", "Notes",
  CellTags->"DClassT"],

Cell[TextData[{
  "See also ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DC",
    ButtonData:>"DC",
    ButtonStyle->"AddOnsLink"],
  ". "
}], "Notes",
  CellTags->"DClassT"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DClassT"],

Cell["The full transformation semigroup on 3 points. ", "ExampleText",
  CellTags->"DClassT"],

Cell[BoxData[{
    \(\(S = TransformationSG[3];\)\), "\n", 
    \(Length[S]\)}], "Input",
  CellLabel->"In[54]:=",
  CellTags->"DClassT"],

Cell[BoxData[{
    \(DClassT[T[1, 1, 1], S]\), "\n", 
    \(\(%\  // \ DClassToBlocks\)\  // \ MatrixForm\)}], "Input",
  CellLabel->"In[56]:=",
  CellTags->"DClassT"],

Cell[BoxData[{
    \(DClassT[T[1, 2, 2], S]\), "\n", 
    \(\(%\  // \ DClassToBlocks\)\  // \ MatrixForm\)}], "Input",
  CellLabel->"In[58]:=",
  CellTags->"DClassT"],

Cell[BoxData[{
    \(DClassT[T[1, 2, 3], S]\), "\n", 
    \(\(%\  // \ DClassToBlocks\)\  // \ MatrixForm\)}], "Input",
  CellLabel->"In[60]:=",
  CellTags->"DClassT"],

Cell["DClassToBlocks", "ObjectName",
  CellTags->"DClassToBlocks"],

Cell["Function", "Highlight",
  CellTags->"DClassToBlocks"],

Cell[TextData[{
  StyleBox["DClassToBlocks[dc]", "MR"],
  "  converts a D-class in standard form into a matrix of H-classes (a tensor \
of rank 3)."
}], "Usage",
  CellTags->"DClassToBlocks"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"DClassToBlocks"],

Cell[TextData[{
  "The output is a matrix of the form  ",
  Cell[BoxData[
      \(TraditionalForm\`\((H\_\(i\[InvisibleComma]j\))\)\)]],
  "  where each H-class ",
  Cell[BoxData[
      \(TraditionalForm\`H\_\(i\[InvisibleComma]j\)\)]],
  " is given as a list of transformations."
}], "Notes",
  CellTags->"DClassToBlocks"],

Cell[TextData[{
  "The dimensions of the tensor can be determined with ",
  ButtonBox["DClassSize",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DClassToBlocks"],

Cell[TextData[{
  "See also ",
  ButtonBox["BlocksToClass",
    ButtonData:>"BlocksToClass",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["DC",
    ButtonData:>"DC",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DClassToBlocks"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DClassToBlocks"],

Cell["A small D-class in standard representation.", "ExampleText",
  CellTags->"DClassToBlocks"],

Cell[BoxData[{
    \(\(d\  = \ 
        DC[{T[1, 2, 3, 4], T[2, 4, 1, 4], T[3, 1, 4, 4]}, {T[1, 2, 3, 4], 
            T[2, 4, 1, 4], T[3, 1, 4, 4]}, {T[1, 4, 4, 4]}];\)\), "\n", 
    \(\(d\  // \ ToList\)\  // \ TableForm\)}], "Input",
  CellLabel->"In[754]:=",
  CellTags->"DClassToBlocks"],

Cell[BoxData[
    \(DClassToBlocks[d]\  // \ MatrixForm\)], "Input",
  CellLabel->"In[756]:=",
  CellTags->"DClassToBlocks"],

Cell["DihedralGroupSG", "ObjectName",
  CellTags->"DihedralGroupSG"],

Cell["Function", "Highlight",
  CellTags->"DihedralGroupSG"],

Cell[TextData[{
  StyleBox["DihedralGroupSG[n]", "MR"],
  "  returns the dihedral group on ",
  StyleBox["n", "MR"],
  " points. "
}], "Usage",
  CellTags->"DihedralGroupSG"],

Cell["The group is returned as a semigroup object.", "Notes",
  CellTags->"DihedralGroupSG"],

Cell[TextData[{
  "The generated group has ",
  Cell[BoxData[
      \(TraditionalForm\`2  n\)]],
  " elements, corresponding to ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " rations (which form a cyclic subgroup) and ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " reflections."
}], "Notes",
  CellTags->"DihedralGroupSG"],

Cell[TextData[{
  "See also ",
  ButtonBox["AlternatingGroupSG",
    ButtonData:>"AlternatingGroupSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SymmetricGroupSG",
    ButtonData:>"SymmetricGroupSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SG",
    ButtonData:>"SG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["T",
    ButtonData:>"T",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DihedralGroupSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DihedralGroupSG"],

Cell["Generating a dihedral group.", "ExampleText",
  CellTags->"DihedralGroupSG"],

Cell[BoxData[
    \(S = DihedralGroupSG[6]\)], "Input",
  CellLabel->"In[141]:=",
  CellTags->"DihedralGroupSG"],

Cell[TextData[{
  "The cycle structure of these permutations can be visualized using ",
  StyleBox["PlotT", "MR"],
  ". "
}], "ExampleText",
  CellTags->"DihedralGroupSG"],

Cell[BoxData[{
    \(\(\(PlotT[#, 5, PlotStyle \[Rule] {0.05, 0.04}, 
            DisplayFunction \[Rule] Identity] &\) /@ ToList[S];\)\), "\n", 
    \(\(ShowArray[%];\)\)}], "Input",
  CellLabel->"In[144]:=",
  CellTags->"DihedralGroupSG"],

Cell["GEN", "ObjectName",
  CellTags->"GEN"],

Cell["Constructor", "Highlight",
  CellTags->"GEN"],

Cell[TextData[{
  StyleBox["GEN[n,k,{g1,...,gk}]", "MR"],
  "  is a collection of k generators, suitable as input to ",
  StyleBox["GenerateSG", "MR"],
  ". "
}], "Usage",
  CellTags->"GEN"],

Cell[TextData[{
  " The generators ",
  StyleBox["gi", "MR"],
  " can be transformations of size ",
  StyleBox["n", "MR"],
  " or ",
  StyleBox["n", "MR"],
  " by ",
  StyleBox["n", "MR"],
  " matrices over a semiring."
}], "Notes",
  CellTags->"GEN"],

Cell[TextData[{
  "See ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SG",
    ButtonData:>"SG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["T",
    ButtonData:>"T",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"GEN"],

Cell["GenerateCSM", "ObjectName",
  CellTags->"GenerateCSM"],

Cell["Function", "Highlight",
  CellTags->"GenerateCSM"],

Cell[TextData[{
  StyleBox["GenerateCSM[q0,dot,K,opts]", "MR"],
  "  produces the cyclic right semimodule generated by ",
  StyleBox["q0", "MR"],
  " and operation ",
  StyleBox["dot", "MR"],
  ", a right action of the monoid of words over ",
  StyleBox["K", "MR"],
  "."
}], "Usage",
  CellTags->"GenerateCSM"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Equations", "\[Rule]", "False", "-"},
          {"Direction", "\[Rule]", "Right", "-"},
          {"BoundCSM", "\[Rule]", "2000", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"GenerateCSM"],

Cell[TextData[{
  "Option ",
  StyleBox["Equations->True", "MR"],
  "  causes a list of witnesses and rewrite rules to be returned in addition \
to the cyclic semimodule. "
}], "Notes",
  CellTags->"GenerateCSM"],

Cell[TextData[{
  "Option ",
  StyleBox["Direction->Left", "MR"],
  "  will build a left semimodule.  Operation ",
  StyleBox["dot", "MR"],
  " is required to be a left action in this case."
}], "Notes",
  CellTags->"GenerateCSM"],

Cell[TextData[{
  "Option ",
  StyleBox["BoundCSM->n", "MR"],
  "  allows to construct larger modules."
}], "Notes",
  CellTags->"GenerateCSM"],

Cell[TextData[{
  "The return object  ",
  StyleBox["dot[p,s]", "MR"],
  "  must be an element of the semimodule where ",
  StyleBox["p", "MR"],
  " is an element of the semimodule and ",
  StyleBox["s", "MR"],
  " is a symbol in the alphabet. "
}], "Notes",
  CellTags->"GenerateCSM"],

Cell["\<\
Can be used to construct the accessible parts of a variety of \
automata.\
\>", "Notes",
  CellTags->"GenerateCSM"],

Cell[TextData[{
  "See also ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["GenerateCayleyGraph",
    ButtonData:>"GenerateCayleyGraph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"GenerateCSM"],

Cell["Further Examples", "ExampleSection",
  CellTags->"GenerateCSM"],

Cell["\<\
A small example over a one-letter alphabet: a lasso with a loop of \
size 2.\
\>", "ExampleText",
  CellTags->"GenerateCSM"],

Cell[BoxData[{
    \(op[p_, "\<a\>"] := p + 1 /; p < 7\), "\n", 
    \(op[_, "\<a\>"] := 5\)}], "Input",
  CellLabel->"In[89]:=",
  ImageRegion->{{0, 1}, {0, 1}},
  CellTags->"GenerateCSM"],

Cell[BoxData[{
    \(\({Q, W, eq} = 
        GenerateCSM[0, op, {a}, Equations \[Rule] True];\)\), "\n", 
    \(Q\), "\n", 
    \(PrintWitnesses[Q, W]\)}], "Input",
  CellLabel->"In[91]:=",
  ImageRegion->{{0, 1}, {0, 1}},
  CellTags->"GenerateCSM"],

Cell["\<\
The \"equations\" here are in essence just a trace of the \
construction of the semimodule.  The generator corresponds to \[Epsilon] \
.\
\>", "ExampleText",
  CellTags->"GenerateCSM"],

Cell[BoxData[
    \(PrintEquations[eq]\)], "Input",
  CellLabel->"In[94]:=",
  ImageRegion->{{0, 1}, {0, 1}},
  CellTags->"GenerateCSM"],

Cell["\<\
A two-letter example.  The semimodule has 15 elements in this case.\
\
\>", "ExampleText",
  CellTags->"GenerateCSM"],

Cell[BoxData[{
    \(\(dot[{p_, q_}, "\<a\>"] := {Mod[p + 2, 5], q};\)\), "\n", 
    \(\(dot[{p_, q_}, "\<b\>"] := {p, Mod[q + 1, 3]};\)\), "\n", 
    \(GenerateCSM[{0, 0}, dot, {a, b}]\)}], "Input",
  CellLabel->"In[95]:=",
  ImageRegion->{{0, 1}, {0, 1}},
  CellTags->"GenerateCSM"],

Cell["\<\
On occasion it is preferable to return the equations not in word \
form, but as triples indicating which action transformed the left-hand-side \
into the right-hand-side.  One can easily reconstruct the ordinary equations \
from this extended form.\
\>", "ExampleText",
  CellTags->"GenerateCSM"],

Cell[BoxData[{
    \(\({Q, W, eqq}\  = \ 
        GenerateCSM[{0, 0}, dot, {a, b}, 
          Equations \[Rule] Full\ ]\ ;\)\), "\n", 
    \(eqq\)}], "Input",
  CellLabel->"In[98]:=",
  ImageRegion->{{0, 1}, {0, 1}},
  CellTags->"GenerateCSM"],

Cell[BoxData[
    \(Map[\ W[\([#]\)] &, \(Drop[#, {2}] &\)\  /@ \ eqq, {1}]\ \  // \ 
      PrintEquations\)], "Input",
  CellLabel->"In[410]:=",
  CellTags->"GenerateCSM"],

Cell["GenerateCayleyGraph", "ObjectName",
  CellTags->"GenerateCayleyGraph"],

Cell["Function", "Highlight",
  CellTags->"GenerateCayleyGraph"],

Cell[TextData[{
  StyleBox["GenerateCayleyGraph[q0,dot,K,opts]", "MR"],
  "  produces the Cayley graph for the right semimodule generated by ",
  StyleBox["q0", "MR"],
  " and operation ",
  StyleBox["dot", "MR"],
  ", a right action of the monoid of words over ",
  StyleBox["K", "MR"],
  "."
}], "Usage",
  CellTags->"GenerateCayleyGraph"],

Cell[TextData[{
  "Options\n",
  Cell[BoxData[GridBox[{
          {"Type", "\[Rule]", "None", "-"},
          {"Direction", "\[Rule]", "Right", "-"},
          {"BoundCSM", "\[Rule]", "1000", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"GenerateCayleyGraph"],

Cell[TextData[{
  "Option ",
  StyleBox["Equations->True", "MR"],
  "  causes a list of witnesses and rewrite rules to be returned in addition \
to the cyclic semimodule. "
}], "Notes",
  CellTags->"GenerateCayleyGraph"],

Cell[TextData[{
  "Option ",
  StyleBox["Direction->Left", "MR"],
  "  will build the Cayley graph for a left action ",
  StyleBox["dot", "MR"],
  " ."
}], "Notes",
  CellTags->"GenerateCayleyGraph"],

Cell[TextData[{
  "Option ",
  StyleBox["BoundCSM->n", "MR"],
  "  allows to construct larger graphs."
}], "Notes",
  CellTags->"GenerateCayleyGraph"],

Cell[TextData[{
  "The return object  ",
  StyleBox["dot[p,s]", "MR"],
  "  must be an element of the semimodule where ",
  StyleBox["p", "MR"],
  " is an element of the semimodule and ",
  StyleBox["s", "MR"],
  " is a symbol in the alphabet. "
}], "Notes",
  CellTags->"GenerateCayleyGraph"],

Cell[TextData[{
  StyleBox["GenerateCayleyGraph", "MR"],
  " is really just a wrapper for ",
  StyleBox["GenerateCSM", "MR"],
  "."
}], "Notes",
  CellTags->"GenerateCayleyGraph"],

Cell[TextData[{
  "See also ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["GenerateCSM",
    ButtonData:>"GenerateCSM",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"GenerateCayleyGraph"],

Cell["Further Examples", "ExampleSection",
  CellTags->"GenerateCayleyGraph"],

Cell["\<\
The Cayley graph for a one-generated semimodule.  The periodic part \
has size 3.\
\>", "ExampleText",
  CellTags->"GenerateCayleyGraph"],

Cell[BoxData[{
    \(Clear[op]\), "\n", 
    \(op[p_, "\<a\>"] := p + 1 /; p < 7\), "\n", 
    \(op[_, "\<a\>"] := 5\)}], "Input",
  CellLabel->"In[470]:=",
  ImageRegion->{{0, 1}, {0, 1}},
  CellTags->"GenerateCayleyGraph"],

Cell[BoxData[
    \(GenerateCayleyGraph[\ 0, \ op, \ {a}\ ]\)], "Input",
  CellLabel->"In[473]:=",
  CellTags->"GenerateCayleyGraph"],

Cell[BoxData[
    \(StronglyConnectedComponents[\ %, \ 
      NonTrivial \[Rule] True\ ]\)], "Input",
  CellLabel->"In[474]:=",
  CellTags->"GenerateCayleyGraph"],

Cell[TextData[{
  "The Cayley graph of the group ",
  Cell[BoxData[
      \(TraditionalForm\`\[DoubleStruckCapitalZ]\_\(\(\ \)\(2\)\)\[Cross]\ \
\[DoubleStruckCapitalZ]\_\(\(\ \)\(3\)\)\)]],
  ".  The graph is in- and out-degree regular."
}], "ExampleText",
  CellTags->"GenerateCayleyGraph"],

Cell[BoxData[{
    \(Clear[op]\), "\n", 
    \(\(op[{p_, q_}, "\<a\>"] := \ {\ Mod[p + 1, 2], \ q\ };\)\), "\n", 
    \(\(op[{p_, q_}, "\<b\>"] := \ {\ p, \ Mod[q + 1, 3]\ };\)\)}], "Input",
  CellLabel->"In[475]:=",
  ImageRegion->{{0, 1}, {0, 1}},
  CellTags->"GenerateCayleyGraph"],

Cell[BoxData[
    \(G\  = \ 
      GenerateCayleyGraph[\ \ {0, 0}, \ op, \ {a, b}, \ 
        Type \[Rule] Full\ ]\)], "Input",
  CellLabel->"In[478]:=",
  CellTags->"GenerateCayleyGraph"],

Cell[BoxData[
    \(DegreeGraph[G, Full \[Rule] True]\  // \ TableForm\)], "Input",
  CellLabel->"In[479]:=",
  CellTags->"GenerateCayleyGraph"],

Cell["GenerateSG", "ObjectName",
  CellTags->"GenerateSG"],

Cell["Function", "Highlight",
  CellTags->"GenerateSG"],

Cell[TextData[{
  StyleBox["GenerateSG[gen]", "MR"],
  "  produces the semigroup generated by the generators ",
  StyleBox["gen", "MR"],
  ".  "
}], "Usage",
  CellTags->"GenerateSG"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"},
          {"Equations", "\[Rule]", "False", "-"},
          {"Filter", "\[Rule]", "Identity", "-"},
          {"DClasses", "\[Rule]", "False", "-"},
          {"BoundGSG", "\[RuleDelayed]", "$generateSGBound", "-"},
          {"External", "\[RuleDelayed]", "$defaultexternal", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"GenerateSG"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  " causes the corresponding monoid to be generated. "
}], "Notes",
  CellTags->"GenerateSG"],

Cell[TextData[{
  "Option ",
  StyleBox["Equations->True", "MR"],
  " returns a triple ",
  StyleBox["{S,W,eq}", "MR"],
  " where ",
  StyleBox["S", "MR"],
  " is the semigroup, ",
  StyleBox["W", "MR"],
  " is a list of corresponding witnesses, and ",
  StyleBox["eq", "MR"],
  " is the canonical rewrite system for  ",
  StyleBox["S", "MR"],
  ".   The witnesses are minimal with respect to length-lex order and the \
syntactic homomorphism."
}], "Notes",
  CellTags->"GenerateSG"],

Cell[TextData[{
  "Option ",
  StyleBox["DClasses->True", "MR"],
  " in addition returns the D-class decomposition of the semigroup."
}], "Notes",
  CellTags->"GenerateSG"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  " causes the corresponding monoid to be generated. "
}], "Notes",
  CellTags->"GenerateSG"],

Cell[TextData[{
  "Option ",
  StyleBox["BoundGSG->m", "MR"],
  "  allows for the construction of larger semigroups. Default value is \
1000."
}], "Notes",
  CellTags->"GenerateSG"],

Cell[TextData[{
  "With the internal implementation one can also generate semigroups of \
matrices. The operations for the matrix semiring must be overloaded on ",
  StyleBox["Plus", "MR"],
  " and ",
  StyleBox["Times", "MR"],
  ", operation ",
  StyleBox["Dot", "MR"],
  " is used for matrix multiplication."
}], "Notes",
  CellTags->"GenerateSG"],

Cell[TextData[{
  StyleBox["NonCommutativeMultiply", "MR"],
  " is used as multiplication for transformation semigroups. "
}], "Notes",
  CellTags->"GenerateSG"],

Cell[TextData[{
  "See also ",
  ButtonBox["SyntacticSG",
    ButtonData:>"SyntacticSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["GEN",
    ButtonData:>"GEN",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SG",
    ButtonData:>"SG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DC",
    ButtonData:>"DC",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"GenerateSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"GenerateSG"],

Cell["\<\
Two generators suffice to produce the symmetric group, say, on 3 \
points.\
\>", "ExampleText",
  CellTags->"GenerateSG"],

Cell["\<\
g = GEN[3,2,{T[2,3,1],T[2,1,3]}];
GenerateSG[ g ]
% // Length\
\>", "Input",
  CellLabel->"In[1]:=",
  CellTags->"GenerateSG"],

Cell["\<\
We can produce additional information: witnesses and the \
corresponding equations.\
\>", "ExampleText",
  CellTags->"GenerateSG"],

Cell[BoxData[
    \(\({S, W, eq}\  = \ GenerateSG[g, Equations \[Rule] True];\)\)], "Input",\

  CellLabel->"In[59]:=",
  CellTags->"GenerateSG"],

Cell[BoxData[
    \(PrintWitnesses[S, W]\)], "Input",
  CellLabel->"In[60]:=",
  CellTags->"GenerateSG"],

Cell["\<\
The witnesses show how to multiply the generators to obain a \
certain element in the group.\
\>", "ExampleText",
  CellTags->"GenerateSG"],

Cell[BoxData[
    \(T[2, 3, 1] ** T[2, 1, 3]\)], "Input",
  CellLabel->"In[61]:=",
  CellTags->"GenerateSG"],

Cell["\<\
The equations, or rewrite rules, show how to simplify more \
complicated products of the generators symbolically.\
\>", "ExampleText",
  CellTags->"GenerateSG"],

Cell[BoxData[
    \(PrintEquations[eq]\)], "Input",
  CellLabel->"In[62]:=",
  CellTags->"GenerateSG"],

Cell[BoxData[
    \(T[2, 3, 1]\^3 === T[2, 1, 3]\^2\)], "Input",
  CellLabel->"In[63]:=",
  CellTags->"GenerateSG"],

Cell[BoxData[
    \(WordReduce[\ aabababa, \ eq, \ DoTrace \[Rule] True\ ]\)], "Input",
  CellLabel->"In[64]:=",
  CellTags->"GenerateSG"],

Cell["\<\
Here is a larger example, complete with D-class decomposition. \
There are 3 regular D-classes in this case. \
\>", "ExampleText",
  CellTags->"GenerateSG"],

Cell[BoxData[
    \(\(g\  = \ GEN[4, 2, {T[2, 4, 1, 4], T[3, 1, 4, 4]}];\)\)], "Input",
  CellLabel->"In[786]:=",
  CellTags->"GenerateSG"],

Cell[BoxData[
    \(\({S, W, eq, dcl}\  = \ 
        GenerateSG[g, DClasses \[Rule] True];\)\)], "Input",
  CellLabel->"In[793]:=",
  CellTags->"GenerateSG"],

Cell[BoxData[{
    \(S\  // \ Length\), "\n", 
    \(DClassSize[dcl, Full \[Rule] True]\)}], "Input",
  CellLabel->"In[794]:=",
  CellTags->"GenerateSG"],

Cell[TextData[{
  "The second ",
  Cell[BoxData[
      \(TraditionalForm\`D\)]],
  "-class, once as a matrix of transformations, and once in witness form. "
}], "ExampleText",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"GenerateSG"],

Cell[BoxData[{
    \(\(AssignFunction[h, ToList[S], W, Opts \[Rule] Listable];\)\), "\n", 
    \(\(D2\  = \ 
        DClassToBlocks[
          dcl\[LeftDoubleBracket]2\[RightDoubleBracket]];\)\), "\n", 
    \(D2\  // \ MatrixForm\), "\n", 
    \(h[D2]\  // \ MatrixForm\)}], "Input",
  CellLabel->"In[396]:=",
  AspectRatioFixed->True,
  CellTags->"GenerateSG"],

Cell[TextData[{
  "All the ",
  Cell[BoxData[
      \(TraditionalForm\`H\)]],
  "-classes have size 1, so the semigroup is aperiodic."
}], "ExampleText",
  Evaluatable->False,
  AspectRatioFixed->True,
  CellTags->"GenerateSG"],

Cell[BoxData[
    \(AperiodicQSG[S]\)], "Input",
  CellLabel->"In[399]:=",
  CellTags->"GenerateSG"],

Cell["GreenHQT", "ObjectName",
  CellTags->"GreenHQT"],

Cell["Function", "Highlight",
  CellTags->"GreenHQT"],

Cell[TextData[{
  StyleBox["GreenRQT[x,y,S]", "MR"],
  "  tests whether transformations ",
  StyleBox["x", "MR"],
  " and ",
  StyleBox["y", "MR"],
  " are J-equivalent in semigroup ",
  StyleBox["S", "MR"],
  "."
}], "Usage",
  CellTags->"GreenHQT"],

Cell[TextData[{
  "See also  ",
  ButtonBox["GreenJQT",
    ButtonData:>"GreenJQT",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["GreenLQT",
    ButtonData:>"GreenLQT",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["GreenRQT",
    ButtonData:>"GreenRQT",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"GreenHQT"],

Cell["GreenJQT", "ObjectName",
  CellTags->"GreenJQT"],

Cell["Function", "Highlight",
  CellTags->"GreenJQT"],

Cell[TextData[{
  StyleBox["GreenRQT[x,y,S]", "MR"],
  "  tests whether transformations ",
  StyleBox["x", "MR"],
  " and ",
  StyleBox["y", "MR"],
  " are J-equivalent in semigroup ",
  StyleBox["S", "MR"],
  "."
}], "Usage",
  CellTags->"GreenJQT"],

Cell[TextData[{
  "For finite semigroups Green's relations ",
  Cell[BoxData[
      \(TraditionalForm\`J\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`D\)]],
  " agree.  Note that this comand use brute-force computation of two-sided \
ideals and is quite inefficient."
}], "Notes",
  CellTags->"GreenJQT"],

Cell[TextData[{
  "See ",
  StyleBox[" ", "Text"],
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  StyleBox[" and  ", "Text"],
  ButtonBox["DClassDecomposition",
    ButtonData:>"DClassDecomposition",
    ButtonStyle->"AddOnsLink"],
  StyleBox["  for better ways to generate ", "Text"],
  Cell[BoxData[
      \(TraditionalForm\`D\)]],
  "-classes."
}], "Notes",
  CellTags->"GreenJQT"],

Cell[TextData[{
  "See also  ",
  ButtonBox["GreenHQT",
    ButtonData:>"GreenHQT",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["GreenLQT",
    ButtonData:>"GreenLQT",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["GreenRQT",
    ButtonData:>"GreenRQT",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"GreenJQT"],

Cell["GreenLQT", "ObjectName",
  CellTags->"GreenLQT"],

Cell["Function", "Highlight",
  CellTags->"GreenLQT"],

Cell[TextData[{
  StyleBox["GreenRQT[x,y,S]", "MR"],
  "  tests whether transformations ",
  StyleBox["x", "MR"],
  " and ",
  StyleBox["y", "MR"],
  " are L-equivalent in semigroup ",
  StyleBox["S", "MR"],
  "."
}], "Usage",
  CellTags->"GreenLQT"],

Cell[TextData[{
  "See also  ",
  ButtonBox["GreenHQT",
    ButtonData:>"GreenHQT",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["GreenJQT",
    ButtonData:>"GreenJQT",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["GreenRQT",
    ButtonData:>"GreenRQT",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"GreenLQT"],

Cell["GreenRQT", "ObjectName",
  CellTags->"GreenRQT"],

Cell["Function", "Highlight",
  CellTags->"GreenRQT"],

Cell[TextData[{
  StyleBox["GreenRQT[x,y,S]", "MR"],
  "  tests whether transformations ",
  StyleBox["x", "MR"],
  " and ",
  StyleBox["y", "MR"],
  " are R-equivalent in semigroup ",
  StyleBox["S", "MR"],
  "."
}], "Usage",
  CellTags->"GreenRQT"],

Cell[TextData[{
  "See also  ",
  ButtonBox["GreenHQT",
    ButtonData:>"GreenHQT",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["GreenJQT",
    ButtonData:>"GreenJQT",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["GreenLQT",
    ButtonData:>"GreenLQT",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"GreenRQT"],

Cell["HClassSize", "ObjectName",
  CellTags->"HClassSize"],

Cell["Function", "Highlight",
  CellTags->"HClassSize"],

Cell[TextData[{
  StyleBox["HClassSize[dc]", "MR"],
  "  returns the size of the H-classes of D-class ",
  StyleBox["dc", "MR"],
  "."
}], "Usage",
  CellTags->"HClassSize"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"HClassSize"],

Cell[TextData[{
  "See also  ",
  ButtonBox["DClassSize",
    ButtonData:>"DClassSize",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"HClassSize"],

Cell["HClassT", "ObjectName",
  CellTags->"HClassT"],

Cell["Function", "Highlight",
  CellTags->"HClassT"],

Cell[TextData[{
  StyleBox["HClassT[x,S]", "MR"],
  "  returns the H-class of a regular element ",
  StyleBox["x", "MR"],
  " in semigroup ",
  StyleBox["S", "MR"],
  "."
}], "Usage",
  CellTags->"HClassT"],

Cell["Returns a flat list of transformations.", "Notes",
  CellTags->"HClassT"],

Cell[TextData[{
  "See ",
  ButtonBox["DClassT",
    ButtonData:>"DClassT",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"HClassT"],

Cell["HTrivialQSG", "ObjectName",
  CellTags->"HTrivialQSG"],

Cell["Function", "Highlight",
  CellTags->"HTrivialQSG"],

Cell[TextData[{
  StyleBox["HTrivialQSG[S]", "MR"],
  "  tests whether semigroup ",
  StyleBox["S", "MR"],
  " is H-trivial."
}], "Usage",
  CellTags->"HTrivialQSG"],

Cell["Performs D-class decomposition, may be slow.", "Notes",
  CellTags->"HTrivialQSG"],

Cell[TextData[{
  "See also  ",
  ButtonBox["AperiodicQSG",
    ButtonData:>"AperiodicQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["JTrivialQSG",
    ButtonData:>"JTrivialQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["LocallyCommQSG",
    ButtonData:>"LocallyCommQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["LocallyIdemQSG",
    ButtonData:>"LocallyIdemQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["MonoidQSG",
    ButtonData:>"MonoidQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["NilpotentQSG",
    ButtonData:>"NilpotentQSG",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"HTrivialQSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"HTrivialQSG"],

Cell["A syntactic semigroup that is H-trivial.", "ExampleText",
  CellTags->"HTrivialQSG"],

Cell[BoxData[{
    \(\(m = 
        MinimizeFA[RegularExpressionToFA[RES[ab + ba], {a, b}]];\)\), "\n", 
    \(S = SyntacticSG[m]\)}], "Input",
  CellLabel->"In[849]:=",
  CellTags->"HTrivialQSG"],

Cell[BoxData[
    \(HTrivialQSG[S]\)], "Input",
  CellLabel->"In[851]:=",
  CellTags->"HTrivialQSG"],

Cell["IdealT", "ObjectName",
  CellTags->"IdealT"],

Cell["Function", "Highlight",
  CellTags->"IdealT"],

Cell[TextData[{
  StyleBox["IdealT[x,S]", "MR"],
  "  computes the two-sided ideal generated by transformation ",
  StyleBox["x", "MR"],
  " in semigroup ",
  StyleBox["S", "MR"],
  "."
}], "Usage",
  CellTags->"IdealT"],

Cell["Returns a list of transformations.", "Notes",
  CellTags->"IdealT"],

Cell[TextData[{
  "See also ",
  ButtonBox["LeftIdealT",
    ButtonData:>"LeftIdealT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["RightIdealT",
    ButtonData:>"RightIdealT",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"IdealT"],

Cell["Further Examples", "ExampleSection",
  CellTags->"IdealT"],

Cell["\<\
The full transformation semigroup on 3 points, and some of its \
ideals.\
\>", "ExampleText",
  CellTags->"IdealT"],

Cell[BoxData[
    \(\(S = TransformationSG[3];\)\)], "Input",
  CellLabel->"In[852]:=",
  CellTags->"IdealT"],

Cell[BoxData[
    \(IdealT[\ S[\([1]\)], \ S\ ]\)], "Input",
  CellLabel->"In[853]:=",
  CellTags->"IdealT"],

Cell[BoxData[
    \(IdealT[\ S[\([2]\)], \ S\ ]\)], "Input",
  CellLabel->"In[854]:=",
  CellTags->"IdealT"],

Cell["IdemSG", "ObjectName",
  CellTags->"IdemSG"],

Cell["Function", "Highlight",
  CellTags->"IdemSG"],

Cell[TextData[{
  StyleBox["IdemSG[S]", "MR"],
  "  lists the idempotent elements of a semigroup ",
  StyleBox["S", "MR"],
  "."
}], "Usage",
  CellTags->"IdemSG"],

Cell["Returns a list of transformations.", "Notes",
  CellTags->"IdemSG"],

Cell[TextData[{
  StyleBox["IdemSG[x]", "MR"],
  "  tests whether transformation ",
  StyleBox["x", "MR"],
  " is idempotent."
}], "Notes",
  CellTags->"IdemSG"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"IdemSG"],

Cell[TextData[{
  "See also ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["InGroupSG",
    ButtonData:>"InGroupSG",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"IdemSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"IdemSG"],

Cell["\<\
The idempotent elements of the full transformation semigroup on 3 \
points.\
\>", "ExampleText",
  CellTags->"IdemSG"],

Cell[BoxData[
    \(\(S = \ TransformationSG[3];\)\)], "Input",
  CellLabel->"In[862]:=",
  CellTags->"IdemSG"],

Cell[BoxData[
    \(Select[ToList[S], IdemSG\ ]\)], "Input",
  CellLabel->"In[863]:=",
  CellTags->"IdemSG"],

Cell["We can also operate directly on the semigroup:", "ExampleText",
  CellTags->"IdemSG"],

Cell[BoxData[
    \(IdemSG[S]\)], "Input",
  CellLabel->"In[865]:=",
  CellTags->"IdemSG"],

Cell["ImageT", "ObjectName",
  CellTags->"ImageT"],

Cell["Function", "Highlight",
  CellTags->"ImageT"],

Cell[TextData[{
  StyleBox["ImageT[t]", "MR"],
  "  gives the range of transformation ",
  StyleBox["t", "MR"],
  " as an ordered list."
}], "Usage",
  CellTags->"ImageT"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"ImageT"],

Cell[TextData[{
  "See also ",
  ButtonBox["OrderT",
    ButtonData:>"OrderT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["CollapseT",
    ButtonData:>"CollapseT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["RankT",
    ButtonData:>"RankT",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["KernelT",
    ButtonData:>"KernelT",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"ImageT"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ImageT"],

Cell["The images of all transformations on 3 points.", "ExampleText",
  CellTags->"ImageT"],

Cell[BoxData[
    \(\({#, ImageT[#]} &\) /@ 
        ToList@TransformationSG[3] // \ \(Curry[TableForm]\)[
        TableDepth \[Rule] 2]\)], "Input",
  CellLabel->"In[873]:=",
  CellTags->"ImageT"],

Cell["InGroupSG", "ObjectName",
  CellTags->"InGroupSG"],

Cell["Function", "Highlight",
  CellTags->"InGroupSG"],

Cell[TextData[{
  StyleBox["InGroupSG[S]", "MR"],
  "  produces all elements in semigroup ",
  StyleBox["S", "MR"],
  " that belong to a subgroup of ",
  StyleBox["S", "MR"],
  "."
}], "Usage",
  CellTags->"InGroupSG"],

Cell["Returns a list of transformations.", "Notes",
  CellTags->"InGroupSG"],

Cell[TextData[{
  StyleBox["InGroupSG[x]", "MR"],
  "  tests whether transformation ",
  StyleBox["x", "MR"],
  " lies in a group."
}], "Notes",
  CellTags->"InGroupSG"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"InGroupSG"],

Cell[TextData[{
  "See also ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["IdemSG",
    ButtonData:>"IdemSG",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"InGroupSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"InGroupSG"],

Cell["The full transformation semigroup on 3 points.", "ExampleText",
  CellTags->"InGroupSG"],

Cell[BoxData[
    \(\(S = TransformationSG[3];\)\)], "Input",
  CellLabel->"In[881]:=",
  CellTags->"InGroupSG"],

Cell[BoxData[{
    \(IdemSG[S]\), "\n", 
    \(InGroupSG[S]\)}], "Input",
  CellLabel->"In[882]:=",
  CellTags->"InGroupSG"],

Cell[BoxData[
    \(Complement[ToList[S], InGroupSG[S]]\)], "Input",
  CellLabel->"In[59]:=",
  CellTags->"InGroupSG"],

Cell["InverseT", "ObjectName",
  CellTags->"InverseT"],

Cell["Function", "Highlight",
  CellTags->"InverseT"],

Cell[TextData[{
  StyleBox["InverseT[x]", "MR"],
  "  computes the inverse of a transformation ",
  StyleBox["x", "MR"],
  "."
}], "Usage",
  CellTags->"InverseT"],

Cell[TextData[{
  "The inverse is in the group generated by ",
  StyleBox["x", "MR"],
  ", if it exists."
}], "Notes",
  CellTags->"InverseT"],

Cell[TextData[{
  "See also ",
  ButtonBox["OrderT",
    ButtonData:>"OrderT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["CollapseT",
    ButtonData:>"CollapseT",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"InverseT"],

Cell["Further Examples", "ExampleSection",
  CellTags->"InverseT"],

Cell["The inverses of the symmetric group on 3 points.", "ExampleText",
  CellTags->"InverseT"],

Cell["\<\
S = SymmetricGroupSG[3];
{#,InverseT[#]}& /@ ToList[S] // TableForm\
\>", "Input",
  CellLabel->"In[888]:=",
  CellTags->"InverseT"],

Cell["JTrivialQSG", "ObjectName",
  CellTags->"JTrivialQSG"],

Cell["Function", "Highlight",
  CellTags->"JTrivialQSG"],

Cell[TextData[{
  StyleBox["JTrivialQSG[S]", "MR"],
  "  tests whether semigroup ",
  StyleBox["S", "MR"],
  " is J-trivial"
}], "Usage",
  CellTags->"JTrivialQSG"],

Cell["Tests whether all J-classes have cardinality 1. ", "Notes",
  CellTags->"JTrivialQSG"],

Cell[TextData[{
  "Since ",
  StyleBox["S", "MR"],
  " is finite this is the same as being D-trivial."
}], "Notes",
  CellTags->"JTrivialQSG"],

Cell[TextData[{
  "See also  ",
  ButtonBox["AperiodicQSG",
    ButtonData:>"AperiodicQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["HTrivialQSG",
    ButtonData:>"HTrivialQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["LocallyCommQSG",
    ButtonData:>"LocallyCommQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["LocallyIdemQSG",
    ButtonData:>"LocallyIdemQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["MonoidQSG",
    ButtonData:>"MonoidQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["NilpotentQSG",
    ButtonData:>"NilpotentQSG",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"JTrivialQSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"JTrivialQSG"],

Cell["Some J-trivial semigroups.", "ExampleText",
  CellTags->"JTrivialQSG"],

Cell[BoxData[
    \(\(\(S\)\(\ \)\(=\)\(\ \)\(JTrivialSG[4]\)\(\ \)\)\)], "Input",
  CellLabel->"In[937]:=",
  CellTags->"JTrivialQSG"],

Cell[BoxData[
    \(JTrivialQSG[S]\)], "Input",
  CellLabel->"In[938]:=",
  CellTags->"JTrivialQSG"],

Cell[BoxData[
    \(\(\(S\)\(\ \)\(=\)\(\ \)\(JTrivialSG[4, 3]\)\(\ \)\)\)], "Input",
  CellLabel->"In[939]:=",
  CellTags->"JTrivialQSG"],

Cell[BoxData[
    \(JTrivialQSG[S]\)], "Input",
  CellLabel->"In[940]:=",
  CellTags->"JTrivialQSG"],

Cell["JTrivialSG", "ObjectName",
  CellTags->"JTrivialSG"],

Cell["Function", "Highlight",
  CellTags->"JTrivialSG"],

Cell[TextData[{
  StyleBox["JTrivialSG[n]", "MR"],
  "  generates the J-trivial semigroup of all order preserving inflationary \
transformations. "
}], "Usage",
  CellTags->"JTrivialSG"],

Cell[TextData[{
  StyleBox["JTrivialSG[n,b]", "MR"],
  " generates only those transformations with values at least ",
  StyleBox["b", "MR"],
  "."
}], "Notes",
  CellTags->"JTrivialSG"],

Cell[TextData[{
  "See also ",
  ButtonBox["JTrivialQSG",
    ButtonData:>"JTrivialQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["SymmetricGroupSG",
    ButtonData:>"SymmetricGroupSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["AlternatingGroupSG",
    ButtonData:>"AlternatingGroupSG",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"JTrivialSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"JTrivialSG"],

Cell["Some J-trivial semigroups.", "ExampleText",
  CellTags->"JTrivialSG"],

Cell[BoxData[
    \(JTrivialSG[4]\)], "Input",
  CellLabel->"In[934]:=",
  CellTags->"JTrivialSG"],

Cell[BoxData[
    \(JTrivialSG[4, 2]\)], "Input",
  CellLabel->"In[935]:=",
  CellTags->"JTrivialSG"],

Cell["JTrivialQSG[%]", "Input",
  CellLabel->"In[936]:=",
  CellTags->"JTrivialSG"],

Cell["KernelImageToT", "ObjectName",
  CellTags->"KernelImageToT"],

Cell["Function", "Highlight",
  CellTags->"KernelImageToT"],

Cell[TextData[{
  StyleBox["KernelImageToT[ker,im]", "MR"],
  "  reconstructs a transformation given its range and kernel. "
}], "Usage",
  CellTags->"KernelImageToT"],

Cell["\<\
Note that this works only because kernels and images are ordered \
lists rather than sets.\
\>", "Notes",
  CellTags->"KernelImageToT"],

Cell[TextData[{
  "See also ",
  ButtonBox["OrderT",
    ButtonData:>"OrderT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["CollapseT",
    ButtonData:>"CollapseT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ImageT",
    ButtonData:>"ImageT",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["KernelT",
    ButtonData:>"KernelT",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"KernelImageToT"],

Cell["Further Examples", "ExampleSection",
  CellTags->"KernelImageToT"],

Cell["\<\
Given the image and kernel we can reconstruct the transformation.\
\
\>", "ExampleText",
  CellTags->"KernelImageToT"],

Cell["t = RandomT[10]", "Input",
  CellLabel->"In[773]:=",
  CellTags->"KernelImageToT"],

Cell[BoxData[{
    \(it\  = \ ImageT[t]\), "\n", 
    \(kt\  = \ KernelT[t]\)}], "Input",
  CellLabel->"In[774]:=",
  CellTags->"KernelImageToT"],

Cell[BoxData[
    \(KernelImageToT[kt, it]\)], "Input",
  CellLabel->"In[776]:=",
  CellTags->"KernelImageToT"],

Cell["KernelT", "ObjectName",
  CellTags->"KernelT"],

Cell["Function", "Highlight",
  CellTags->"KernelT"],

Cell[TextData[{
  StyleBox["KernelT[t]", "MR"],
  "  gives the kernel of transformation ",
  StyleBox["t", "MR"],
  "."
}], "Usage",
  CellTags->"KernelT"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"KernelT"],

Cell[TextData[{
  "Note that the kernel classes are ordered according to ",
  StyleBox["ImageT[t]", "MR"],
  ". Hence, to compare two kernels as equivalence relations one has to sort \
them first."
}], "Notes",
  CellTags->"KernelT"],

Cell[TextData[{
  "See also ",
  ButtonBox["OrderT",
    ButtonData:>"OrderT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["CollapseT",
    ButtonData:>"CollapseT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ImageT",
    ButtonData:>"ImageT",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["RankT",
    ButtonData:>"RankT",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"KernelT"],

Cell["Further Examples", "ExampleSection",
  CellTags->"KernelT"],

Cell["The kernels of all transformations on 3 points.", "ExampleText",
  CellTags->"KernelT"],

Cell[BoxData[
    \(\({#, KernelT[#]} &\) /@ 
        ToList@TransformationSG[3] // \ \(Curry[TableForm]\)[
        TableDepth \[Rule] 2]\)], "Input",
  CellLabel->"In[782]:=",
  CellTags->"KernelT"],

Cell["LeftIdealT", "ObjectName",
  CellTags->"LeftIdealT"],

Cell["Function", "Highlight",
  CellTags->"LeftIdealT"],

Cell[TextData[{
  StyleBox["LeftIdealT[x,S]", "MR"],
  "  computes the left ideal generated by transformation x in semigroup S."
}], "Usage",
  CellTags->"LeftIdealT"],

Cell["Returns a list of transformations.", "Notes",
  CellTags->"LeftIdealT"],

Cell[TextData[{
  "See also ",
  ButtonBox["IdealT",
    ButtonData:>"IdealT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["RightIdealT",
    ButtonData:>"RightIdealT",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"LeftIdealT"],

Cell["Further Examples", "ExampleSection",
  CellTags->"LeftIdealT"],

Cell["\<\
The full transformation semigroup on 3 points, and some of its left \
ideals.\
\>", "ExampleText",
  CellTags->"LeftIdealT"],

Cell[BoxData[
    \(\(S = TransformationSG[3];\)\)], "Input",
  CellLabel->"In[945]:=",
  CellTags->"LeftIdealT"],

Cell[BoxData[
    \(LeftIdealT[\ S[\([1]\)], \ S\ ]\)], "Input",
  CellLabel->"In[946]:=",
  CellTags->"LeftIdealT"],

Cell[BoxData[
    \(LeftIdealT[\ S[\([2]\)], \ S\ ]\)], "Input",
  CellLabel->"In[949]:=",
  CellTags->"LeftIdealT"],

Cell["LeftNullSG", "ObjectName",
  CellTags->"LeftNullSG"],

Cell["Function", "Highlight",
  CellTags->"LeftNullSG"],

Cell[TextData[{
  StyleBox["LeftNullSG[S]", "MR"],
  "  lists the left nulls of a semigroup ",
  StyleBox["S", "MR"],
  "."
}], "Usage",
  CellTags->"LeftNullSG"],

Cell["Returns a list of transformations.", "Notes",
  CellTags->"LeftNullSG"],

Cell[TextData[{
  StyleBox["LeftNullSG[x,S]", "MR"],
  " tests whether ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " is a left null in ",
  Cell[BoxData[
      \(TraditionalForm\`S\)]],
  "."
}], "Notes",
  CellTags->"LeftNullSG"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"LeftNullSG"],

Cell[TextData[{
  "See also  ",
  ButtonBox["OneSG",
    ButtonData:>"OneSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["RightNullSG",
    ButtonData:>"RightNullSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["NullSG",
    ButtonData:>"NullSG",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"LeftNullSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"LeftNullSG"],

Cell["\<\
The full transformation semigroup on 3 points has no \
left-nulls.\
\>", "ExampleText",
  CellTags->"LeftNullSG"],

Cell["S = TransformationSG[3];", "Input",
  CellLabel->"In[961]:=",
  CellTags->"LeftNullSG"],

Cell[BoxData[
    \(Select[\ ToList[S], \ LeftNullSG[#, S] &\ ]\)], "Input",
  CellTags->"LeftNullSG"],

Cell[BoxData[
    \(LeftNullSG[S]\)], "Input",
  CellLabel->"In[962]:=",
  CellTags->"LeftNullSG"],

Cell["LeftOneSG", "ObjectName",
  CellTags->"LeftOneSG"],

Cell["Function", "Highlight",
  CellTags->"LeftOneSG"],

Cell[TextData[{
  StyleBox["LeftOneSG[S]", "MR"],
  "  lists the left identities of a semigroup ",
  StyleBox["S", "MR"],
  "."
}], "Usage",
  CellTags->"LeftOneSG"],

Cell["Returns a list of transformations.", "Notes",
  CellTags->"LeftOneSG"],

Cell[TextData[{
  StyleBox["LeftOne", "MR"],
  StyleBox["SG", "MR"],
  StyleBox["[x,S]", "MR"],
  "  tests whether transformation ",
  StyleBox["x", "MR"],
  " is a left identity in ",
  StyleBox["S", "MR"],
  "."
}], "Notes",
  CellTags->"LeftOneSG"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"LeftOneSG"],

Cell[TextData[{
  "See also  ",
  ButtonBox["OneSG",
    ButtonData:>"OneSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["RightOneSG",
    ButtonData:>"RightOneSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["NullSG",
    ButtonData:>"NullSG",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"LeftOneSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"LeftOneSG"],

Cell["\<\
The full transformation semigroup on 3 points has only the identity \
as left-one.\
\>", "ExampleText",
  CellTags->"LeftOneSG"],

Cell["S = TransformationSG[3];", "Input",
  CellLabel->"In[89]:=",
  CellTags->"LeftOneSG"],

Cell[BoxData[
    \(Select[\ ToList[S], \ LeftOneSG[#, S] &\ ]\)], "Input",
  CellLabel->"In[966]:=",
  CellTags->"LeftOneSG"],

Cell[BoxData[
    \(LeftOneSG[S]\)], "Input",
  CellLabel->"In[90]:=",
  CellTags->"LeftOneSG"],

Cell["LocallyCommutativeQSG", "ObjectName",
  CellTags->"LocallyCommutativeQSG"],

Cell["Function", "Highlight",
  CellTags->"LocallyCommutativeQSG"],

Cell[TextData[{
  StyleBox["LocallyCommutativeQSG[S]", "MR"],
  "  tests if semigroup ",
  StyleBox["S", "MR"],
  " is locally commutative"
}], "Usage",
  CellTags->"LocallyCommutativeQSG"],

Cell[TextData[{
  "Tests if  ",
  Cell[BoxData[
      \(TraditionalForm\`e\ S\ e\)]],
  " is commutative for each idempotent element ",
  Cell[BoxData[
      \(TraditionalForm\`e\  \[Element] \ S\)]],
  "."
}], "Notes",
  CellTags->"LocallyCommutativeQSG"],

Cell[TextData[{
  "May require ",
  Cell[BoxData[
      \(TraditionalForm\`O(n\^3)\)]],
  " semigroup operations where ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " is the size of the semigroup."
}], "Notes",
  CellTags->"LocallyCommutativeQSG"],

Cell[TextData[{
  "See also  ",
  ButtonBox["AperiodicQSG",
    ButtonData:>"AperiodicQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["JTrivialQSG",
    ButtonData:>"JTrivialQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["LocallyIdempotentQSG",
    ButtonData:>"LocallyIdempotentQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["MonoidQSG",
    ButtonData:>"MonoidQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["NilpotentQSG",
    ButtonData:>"NilpotentQSG",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"LocallyCommutativeQSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"LocallyCommutativeQSG"],

Cell["A DFA whose syntactic semigroup is locally commutative.", "ExampleText",
  CellTags->"LocallyCommutativeQSG"],

Cell["\<\
m = DFA[9,2,{{2,4,2,6,2,6,8,4,8},{3,3,5,7,6,6,9,7,6}},1,{4,7,8,9}];\
\
\>", "Input",
  CellLabel->"In[123]:=",
  CellTags->"LocallyCommutativeQSG"],

Cell["The semigroup has 28 elements and contains 16 idempotents.", \
"ExampleText",
  CellTags->"LocallyCommutativeQSG"],

Cell[BoxData[{
    \(\(S\  = \ SyntacticSG[m];\)\), "\n", 
    \(S\  // \ Length\), "\n", 
    \(IdemSG[S]\  // \ Length\)}], "Input",
  CellLabel->"In[130]:=",
  CellTags->"LocallyCommutativeQSG"],

Cell[BoxData[
    \(LocallyCommutativeQSG[S]\)], "Input",
  CellLabel->"In[125]:=",
  CellTags->"LocallyCommutativeQSG"],

Cell["LocallyIdempotentQSG", "ObjectName",
  CellTags->"LocallyIdempotentQSG"],

Cell["Function", "Highlight",
  CellTags->"LocallyIdempotentQSG"],

Cell[TextData[{
  StyleBox["LocallyIdempotentQSG[S]", "MR"],
  "  tests if semigroup ",
  StyleBox["S", "MR"],
  " is locally idempotent."
}], "Usage",
  CellTags->"LocallyIdempotentQSG"],

Cell[TextData[{
  "Tests if the subsemigroup ",
  Cell[BoxData[
      \(TraditionalForm\`e\ S\ e\)]],
  " is idempotent for each idempotent element ",
  Cell[BoxData[
      \(TraditionalForm\`e\  \[Element] \ S\)]],
  "."
}], "Notes",
  CellTags->"LocallyIdempotentQSG"],

Cell[TextData[{
  "May require ",
  Cell[BoxData[
      \(TraditionalForm\`O(n\^2)\)]],
  " semigroup operations where ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " is the size of the semigroup."
}], "Notes",
  CellTags->"LocallyIdempotentQSG"],

Cell[TextData[{
  "See also  ",
  ButtonBox["AperiodicQSG",
    ButtonData:>"AperiodicQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["JTrivialQSG",
    ButtonData:>"JTrivialQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["HTrivialQSG",
    ButtonData:>"HTrivialQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["LocallyCommutativeQSG",
    ButtonData:>"LocallyCommutativeQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["MonoidQSG",
    ButtonData:>"MonoidQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["NilpotentQSG",
    ButtonData:>"NilpotentQSG",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"LocallyIdempotentQSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"LocallyIdempotentQSG"],

Cell["A DFA whose syntactic semigroup is locally idempotent.", "ExampleText",
  CellTags->"LocallyIdempotentQSG"],

Cell["\<\
m = DFA[9,2,{{2,4,2,6,2,6,8,4,8},{3,3,5,7,6,6,9,7,6}},1,{4,7,8,9}];\
\
\>", "Input",
  CellLabel->"In[140]:=",
  CellTags->"LocallyIdempotentQSG"],

Cell[BoxData[{
    \(\(S\  = \ SyntacticSG[m];\)\), "\n", 
    \(S\  // \ Length\), "\n", 
    \(IdemSG[S]\  // \ Length\)}], "Input",
  CellLabel->"In[141]:=",
  CellTags->"LocallyIdempotentQSG"],

Cell[BoxData[
    \(LocallyIdempotentQSG[S]\)], "Input",
  CellLabel->"In[144]:=",
  CellTags->"LocallyIdempotentQSG"],

Cell["Merge12T", "ObjectName",
  CellTags->"Merge12T"],

Cell["Function", "Highlight",
  CellTags->"Merge12T"],

Cell[TextData[{
  StyleBox["Merge12T[n]", "MR"],
  "  returns the transformation ",
  StyleBox["T[1,1,3,..,n]", "MR"],
  "."
}], "Usage",
  CellTags->"Merge12T"],

Cell["Useful as a generator for the full transformation semigroup.", "Notes",
  CellTags->"Merge12T"],

Cell[TextData[{
  "See ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["ShiftT",
    ButtonData:>"ShiftT",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["TranspositionT",
    ButtonData:>"TranspositionT",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Merge12T"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Merge12T"],

Cell["Generating the full transformation semigroup.", "ExampleText",
  CellTags->"Merge12T"],

Cell[BoxData[
    \(g\  = \ 
      GEN[4, 3, {Merge12T[4], TranspositionT[4, 1], LShiftT[4]}]\)], "Input",
  CellLabel->"In[993]:=",
  CellTags->"Merge12T"],

Cell[BoxData[
    \(GenerateSG[g]\  // \ Length\)], "Input",
  CellLabel->"In[994]:=",
  CellTags->"Merge12T"],

Cell["Without Merge12 we obtain the symmetric group on 4 letters.", \
"ExampleText",
  CellTags->"Merge12T"],

Cell[BoxData[
    \(g\  = \ GEN[4, 3, {TranspositionT[4, 1], LShiftT[4]}]\)], "Input",
  CellLabel->"In[995]:=",
  CellTags->"Merge12T"],

Cell[BoxData[
    \(GenerateSG[g]\  // \ Length\)], "Input",
  CellLabel->"In[996]:=",
  CellTags->"Merge12T"],

Cell["MakeMonoidSG", "ObjectName",
  CellTags->"MakeMonoidSG"],

Cell["Function", "Highlight",
  CellTags->"MakeMonoidSG"],

Cell[TextData[{
  StyleBox["MakeMonoidSG[S]", "MR"],
  "  turns semigroup ",
  StyleBox["S", "MR"],
  " into a monoid by adjoining a unit, if necessary."
}], "Usage",
  CellTags->"MakeMonoidSG"],

Cell[TextData[{
  "See also ",
  ButtonBox["MonoidQSG",
    ButtonData:>"MonoidQSG",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"MakeMonoidSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"MakeMonoidSG"],

Cell["A semigroup without a unit.", "ExampleText",
  CellTags->"MakeMonoidSG"],

Cell[BoxData[
    \(\(S = SG[T[1, 1, 1], T[2, 2, 2], T[3, 3, 3]];\)\)], "Input",
  CellLabel->"In[997]:=",
  CellTags->"MakeMonoidSG"],

Cell[BoxData[
    \(MakeMonoidSG[S]\)], "Input",
  CellLabel->"In[998]:=",
  CellTags->"MakeMonoidSG"],

Cell["MonoidQSG", "ObjectName",
  CellTags->"MonoidQSG"],

Cell["Function", "Highlight",
  CellTags->"MonoidQSG"],

Cell[TextData[{
  StyleBox["MonoidQSG[S]", "MR"],
  "  tests whether semigroup ",
  StyleBox["S", "MR"],
  " is a monoid."
}], "Usage",
  CellTags->"MonoidQSG"],

Cell[TextData[{
  "See also  ",
  ButtonBox["AperiodicQSG",
    ButtonData:>"AperiodicQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["JTrivialQSG",
    ButtonData:>"JTrivialQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["LocallyCommQSG",
    ButtonData:>"LocallyCommQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["LocallyIdemQSG",
    ButtonData:>"LocallyIdemQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["HTrivialQSG",
    ButtonData:>"HTrivialQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["NilpotentQSG",
    ButtonData:>"NilpotentQSG",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"MonoidQSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"MonoidQSG"],

Cell["A semigroup without a unit.", "ExampleText",
  CellTags->"MonoidQSG"],

Cell[BoxData[
    \(\(S = SG[T[1, 1, 1], T[2, 2, 2], T[3, 3, 3]];\)\)], "Input",
  CellLabel->"In[1001]:=",
  CellTags->"MonoidQSG"],

Cell[BoxData[
    \(MonoidQSG[S]\)], "Input",
  CellLabel->"In[1002]:=",
  CellTags->"MonoidQSG"],

Cell["NilpotentQSG", "ObjectName",
  CellTags->"NilpotentQSG"],

Cell["Function", "Highlight",
  CellTags->"NilpotentQSG"],

Cell[TextData[{
  StyleBox["NilpotentQSG[S]", "MR"],
  "  tests whether semigroup ",
  StyleBox["S", "MR"],
  " is nilpotent, i.e., whether null is the only idempotent element."
}], "Usage",
  CellTags->"NilpotentQSG"],

Cell[TextData[{
  "See also  ",
  ButtonBox["AperiodicQSG",
    ButtonData:>"AperiodicQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["JTrivialQSG",
    ButtonData:>"JTrivialQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["LocallyCommQSG",
    ButtonData:>"LocallyCommQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["LocallyIdemQSG",
    ButtonData:>"LocallyIdemQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["MonoidQSG",
    ButtonData:>"MonoidQSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["HTrivialQSG",
    ButtonData:>"HTrivialQSG",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"NilpotentQSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"NilpotentQSG"],

Cell["A nilpotent semigroup obtained from a cofinite language.", "ExampleText",
  CellTags->"NilpotentQSG"],

Cell[BoxData[
    \(m = ComplementFA[BuildDFA[Words[3, {a, b}], {a, b}]]\)], "Input",
  CellLabel->"In[1005]:=",
  CellTags->"NilpotentQSG"],

Cell[BoxData[
    \(S\  = \ SyntacticSG[m]\)], "Input",
  CellLabel->"In[1006]:=",
  CellTags->"NilpotentQSG"],

Cell[BoxData[
    \(NilpotentQSG[S]\)], "Input",
  CellLabel->"In[1007]:=",
  CellTags->"NilpotentQSG"],

Cell["NullSG", "ObjectName",
  CellTags->"NullSG"],

Cell["Function", "Highlight",
  CellTags->"NullSG"],

Cell[TextData[{
  StyleBox["NullSG[S]", "MR"],
  "  lists the null element of a semigroup ",
  StyleBox["S", "MR"],
  ", if it exists."
}], "Usage",
  CellTags->"NullSG"],

Cell[TextData[{
  StyleBox["NullSG[x,S]", "MR"],
  "  tests whether transformation ",
  StyleBox["x", "MR"],
  " is a null in semigroup ",
  StyleBox["S", "MR"],
  "."
}], "Notes",
  CellTags->"NullSG"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"NullSG"],

Cell[TextData[{
  "See also ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["LeftNullSG",
    ButtonData:>"LeftNullSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["RightNullSG",
    ButtonData:>"RightNullSG",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"NullSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"NullSG"],

Cell["A semigroup with a null.", "ExampleText",
  CellTags->"NullSG"],

Cell[BoxData[
    \(\(S = 
        SG[T[1, 1, 4, 4, 5, 6], T[1, 1, 6, 6, 5, 4], T[1, 4, 5, 6, 5, 4], 
          T[1, 6, 5, 4, 5, 6], T[2, 2, 3, 3, 5, 6], T[2, 2, 6, 6, 5, 3], 
          T[2, 3, 5, 6, 5, 3], T[2, 6, 5, 3, 5, 6], T[3, 3, 3, 3, 5, 5], 
          T[3, 3, 5, 5, 5, 3], T[3, 5, 5, 3, 5, 5], T[4, 4, 4, 4, 5, 5], 
          T[4, 4, 5, 5, 5, 4], T[4, 5, 5, 4, 5, 5], T[5, 5, 5, 5, 5, 5], 
          T[6, 5, 5, 6, 5, 5], T[6, 6, 5, 5, 5, 6], 
          T[6, 6, 6, 6, 5, 5]];\)\)], "Input",
  CellLabel->"In[1010]:=",
  CellTags->"NullSG"],

Cell[BoxData[{
    \(NullSG[S]\), "\n", 
    \(IdemSG[S]\)}], "Input",
  CellLabel->"In[1011]:=",
  CellTags->"NullSG"],

Cell[BoxData[
    \(NullSG[T[5, 5, 5, 5, 5, 5], S]\)], "Input",
  CellLabel->"In[1013]:=",
  CellTags->"NullSG"],

Cell["OneSG", "ObjectName",
  CellTags->"OneSG"],

Cell["Function", "Highlight",
  CellTags->"OneSG"],

Cell[TextData[{
  StyleBox["OneSG[S]", "MR"],
  "  lists the identity of a semigroup ",
  StyleBox["S", "MR"],
  ", if it exists."
}], "Usage",
  CellTags->"OneSG"],

Cell[TextData[{
  StyleBox["OneSG[t,S]", "MR"],
  "  tests whether transformation ",
  StyleBox["t", "MR"],
  " is the identity in semigroup ",
  StyleBox["S", "MR"],
  "."
}], "Notes",
  CellTags->"OneSG"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"OneSG"],

Cell[TextData[{
  "See also ",
  ButtonBox["OneT",
    ButtonData:>"OneT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["NullSG",
    ButtonData:>"NullSG",
    ButtonStyle->"AddOnsLink"],
  ". "
}], "Notes",
  CellTags->"OneSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"OneSG"],

Cell["A small semigroup with no apparent identity element.", "ExampleText",
  CellTags->"OneSG"],

Cell[BoxData[
    \(\(S\  = \ 
        SG[T[1, 1, 3, 4], T[1, 1, 4, 3], T[3, 3, 1, 4], T[3, 3, 4, 1], 
          T[4, 4, 1, 3], T[4, 4, 3, 1]];\)\)], "Input",
  CellLabel->"In[1019]:=",
  CellTags->"OneSG"],

Cell[BoxData[
    \(\(\(\ \)\(\(OneSG[#, S] &\)\  /@ \ ToList[S]\)\)\)], "Input",
  CellLabel->"In[1022]:=",
  CellTags->"OneSG"],

Cell["The identity of the alternating group on 4 points.", "ExampleText",
  CellTags->"OneSG"],

Cell["S = AlternatingGroupSG[4]", "Input",
  CellLabel->"In[1025]:=",
  CellTags->"OneSG"],

Cell[BoxData[
    \(OneSG[S]\)], "Input",
  CellLabel->"In[1026]:=",
  CellTags->"OneSG"],

Cell["The following semigroup has no identity, only right-nulls.", \
"ExampleText",
  CellTags->"OneSG"],

Cell[BoxData[
    \(S\ \  = \ SG @@ Table[ConstantT[i, 3], {i, 3}]\)], "Input",
  CellLabel->"In[389]:=",
  CellTags->"OneSG"],

Cell[BoxData[
    \(OneSG[S]\)], "Input",
  CellLabel->"In[390]:=",
  CellTags->"OneSG"],

Cell[BoxData[
    \(RightNullSG[S]\)], "Input",
  CellLabel->"In[391]:=",
  CellTags->"OneSG"],

Cell["OneT", "ObjectName",
  CellTags->"OneT"],

Cell["Constructor", "Highlight",
  CellTags->"OneT"],

Cell[TextData[{
  StyleBox["OneT[t]", "MR"],
  "  produces the identity transformation of size ",
  StyleBox["Length[t]", "MR"],
  " where ",
  StyleBox["t", "MR"],
  " is any transformation. "
}], "Usage",
  CellTags->"OneT"],

Cell[TextData[{
  StyleBox["OneT[k]", "MR"],
  "  for an integer ",
  StyleBox["k", "MR"],
  StyleBox[" also return the appropriate identity transformation.", "Text"]
}], "Notes",
  CellTags->"OneT"],

Cell[TextData[{
  "See also  ",
  ButtonBox["OneSG",
    ButtonData:>"OneSG",
    ButtonStyle->"AddOnsLink"],
  ". "
}], "Notes",
  CellTags->"OneT"],

Cell["Further Examples", "ExampleSection",
  CellTags->"OneT"],

Cell["The identity transformation on 5 points.", "ExampleText",
  CellTags->"OneT"],

Cell[" OneT[5]", "Input",
  CellLabel->"In[1029]:=",
  CellTags->"OneT"],

Cell[BoxData[
    \(OneSG[%, \ SymmetricGroupSG[5]\ ]\)], "Input",
  CellLabel->"In[1030]:=",
  CellTags->"OneT"],

Cell["OrderT", "ObjectName",
  CellTags->"OrderT"],

Cell["Function", "Highlight",
  CellTags->"OrderT"],

Cell[TextData[{
  StyleBox["OrderT[x]", "MR"],
  "  returns the order of a permutation x."
}], "Usage",
  CellTags->"OrderT"],

Cell[TextData[{
  "See also ",
  ButtonBox["InGroupT",
    ButtonData:>"InGroupT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["InverseT",
    ButtonData:>"InverseT",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"OrderT"],

Cell["Further Examples", "ExampleSection",
  CellTags->"OrderT"],

Cell["\<\
The alternating group on 5 points, and the orders of its \
elements.\
\>", "ExampleText",
  CellTags->"OrderT"],

Cell["S = AlternatingGroupSG[5];", "Input",
  CellLabel->"In[133]:=",
  CellTags->"OrderT"],

Cell[BoxData[{
    \(OrderT[ToList[S]]\), "\n", 
    \(Frequencies[%]\  // \ TableForm\)}], "Input",
  CellLabel->"In[134]:=",
  CellTags->"OrderT"],

Cell["PrintDClass", "ObjectName",
  CellTags->"PrintDClass"],

Cell["Function", "Highlight",
  CellTags->"PrintDClass"],

Cell[TextData[{
  StyleBox["PrintDClass[dc]", "MR"],
  "  pretty prints a D-class ",
  StyleBox["dc", "MR"],
  "."
}], "Usage",
  CellTags->"PrintDClass"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"PrintDClass"],

Cell[TextData[{
  "Produces a rather verbose listing of the ",
  Cell[BoxData[
      \(TraditionalForm\`D\)]],
  "-class, ordered by images and kernels. "
}], "Notes",
  CellTags->"PrintDClass"],

Cell[TextData[{
  "See also ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DClassDecomposition",
    ButtonData:>"DClassDecomposition",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DC",
    ButtonData:>"DC",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"PrintDClass"],

Cell["Further Examples", "ExampleSection",
  CellTags->"PrintDClass"],

Cell["A D-class, in standard form.", "ExampleText",
  CellTags->"PrintDClass"],

Cell[BoxData[
    \(\(d\  = \ 
        DC[{T[1, 1, 1, 4], T[1, 1, 4, 1], T[1, 1, 4, 4], T[1, 4, 1, 1], 
            T[1, 4, 1, 4], T[1, 4, 4, 1], T[1, 4, 4, 4]}, {T[1, 1, 1, 2], 
            T[1, 1, 1, 3], T[1, 1, 1, 4], T[2, 1, 1, 3], T[2, 1, 1, 4], 
            T[3, 1, 1, 4]}, {T[1, 1, 1, 4], T[4, 4, 4, 1]}];\)\)], "Input",
  CellLabel->"In[146]:=",
  CellTags->"PrintDClass"],

Cell[BoxData[
    \(PrintDClass[d]\)], "Input",
  CellLabel->"In[147]:=",
  CellTags->"PrintDClass"],

Cell["PrintEquations", "ObjectName",
  CellTags->"PrintEquations"],

Cell["Function", "Highlight",
  CellTags->"PrintEquations"],

Cell[TextData[{
  StyleBox["PrintEquations[eq]", "MR"],
  "  pretty prints a list of equations in word form. "
}], "Usage",
  CellTags->"PrintEquations"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Cols", "\[Rule]", "None", "-"},
          {"Algebraic", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"PrintEquations"],

Cell[TextData[{
  "Option ",
  StyleBox["Algebraic->True", "MR"],
  " writes words in algebraic notation."
}], "Notes",
  CellTags->"PrintEquations"],

Cell[TextData[{
  "See ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SyntacticSG",
    ButtonData:>"SyntacticSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Equations",
    ButtonData:>"Equations",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["PrintWitnesses",
    ButtonData:>"PrintWitnesses",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"PrintEquations"],

Cell["Further Examples", "ExampleSection",
  CellTags->"PrintEquations"],

Cell["\<\
The full transformation semigroup on 3 points, and the \
corresponding equations.\
\>", "ExampleText",
  CellTags->"PrintEquations"],

Cell[BoxData[
    \(\({S, W, eq} = 
        TransformationSG[3, Equations \[Rule] True];\)\)], "Input",
  CellLabel->"In[1045]:=",
  CellTags->"PrintEquations"],

Cell[BoxData[
    \(Length[eq]\)], "Input",
  CellLabel->"In[1046]:=",
  CellTags->"PrintEquations"],

Cell["The rewrite rules can be simplified considerably.", "ExampleText",
  CellTags->"PrintEquations"],

Cell[BoxData[{
    \(\(eqs\  = \ SimplifyEquations[S, W, eq];\)\), "\n", 
    \(PrintEquations[eqs]\)}], "Input",
  CellLabel->"In[1047]:=",
  CellTags->"PrintEquations"],

Cell["PrintWitnesses", "ObjectName",
  CellTags->"PrintWitnesses"],

Cell["Function", "Highlight",
  CellTags->"PrintWitnesses"],

Cell[TextData[{
  StyleBox["PrintWitnesses[S,wit]", "MR"],
  "  prints a table of witnesses and semigroup elements for semigroup ",
  StyleBox["S", "MR"],
  ". "
}], "Usage",
  CellTags->"PrintWitnesses"],

Cell[TextData[{
  "See ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SyntacticSG",
    ButtonData:>"SyntacticSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Equations",
    ButtonData:>"Equations",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["PrintEquations",
    ButtonData:>"PrintEquations",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"PrintWitnesses"],

Cell["Further Examples", "ExampleSection",
  CellTags->"PrintWitnesses"],

Cell["\<\
The full transformation semigroup on 3 points, and the \
corresponding witnesses.\
\>", "ExampleText",
  CellTags->"PrintWitnesses"],

Cell[BoxData[
    \(\({S, W, eq} = 
        TransformationSG[3, Equations \[Rule] True];\)\)], "Input",
  CellLabel->"In[1049]:=",
  CellTags->"PrintWitnesses"],

Cell[BoxData[
    \(PrintWitnesses[S, W]\)], "Input",
  CellLabel->"In[1050]:=",
  CellTags->"PrintWitnesses"],

Cell["RegularSG", "ObjectName",
  CellTags->"RegularSG"],

Cell["Function", "Highlight",
  CellTags->"RegularSG"],

Cell[TextData[{
  StyleBox["RegularSG[S]", "MR"],
  "  returns the regular elements in semigroup ",
  StyleBox["S", "MR"],
  "."
}], "Usage",
  CellTags->"RegularSG"],

Cell["Returns a flat list of transformations.", "Notes",
  CellTags->"RegularSG"],

Cell[TextData[{
  StyleBox["RegularSG[x,S]", "MR"],
  "  tests whether ",
  StyleBox["x", "MR"],
  " is a regular element in semigroup ",
  StyleBox["S", "MR"],
  "."
}], "Notes",
  CellTags->"RegularSG"],

Cell[TextData[{
  "See also ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"RegularSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RegularSG"],

Cell["A three-element semigroup with only one regular element.", "ExampleText",
  CellTags->"RegularSG"],

Cell[BoxData[
    \(\(S\  = \ SG[T[2, 2, 1, 1], T[2, 2, 1, 2], T[2, 2, 2, 2]];\)\)], "Input",\

  CellLabel->"In[1057]:=",
  CellTags->"RegularSG"],

Cell[BoxData[
    \(Select[ToList[S], RegularSG[#, S] &]\)], "Input",
  CellLabel->"In[1053]:=",
  CellTags->"RegularSG"],

Cell[BoxData[
    \(RegularSG[S]\)], "Input",
  CellLabel->"In[1058]:=",
  CellTags->"RegularSG"],

Cell[BoxData[
    \(NullSG[S]\)], "Input",
  CellLabel->"In[1059]:=",
  CellTags->"RegularSG"],

Cell["RandomT", "ObjectName",
  CellTags->"RandomT"],

Cell["Function", "Highlight",
  CellTags->"RandomT"],

Cell[TextData[{
  StyleBox["RandomT[n]", "MR"],
  "  generates a random transformation on [n]. "
}], "Usage",
  CellTags->"RandomT"],

Cell[TextData[{
  "The entries are chosen uniformly from ",
  Cell[BoxData[
      \(TraditionalForm\`\([n]\)\)]],
  "."
}], "Notes",
  CellTags->"RandomT"],

Cell[TextData[{
  "See also ",
  ButtonBox["T",
    ButtonData:>"T",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"RandomT"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RandomT"],

Cell["Some random transformations.", "ExampleText",
  CellTags->"RandomT"],

Cell["Table[ RandomT[8], {5} ]", "Input",
  CellLabel->"In[87]:=",
  CellTags->"RandomT"],

Cell["RankT", "ObjectName",
  CellTags->"RankT"],

Cell["Function", "Highlight",
  CellTags->"RankT"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"RankT"],

Cell[TextData[{
  StyleBox["RankT[t]", "MR"],
  "  gives the rank of transformation ",
  StyleBox["t", "MR"],
  ", i.e., the cardinality of its image and kernel."
}], "Usage",
  CellTags->"RankT"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"RankT"],

Cell[TextData[{
  "See also ",
  ButtonBox["OrderT",
    ButtonData:>"OrderT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["CollapseT",
    ButtonData:>"CollapseT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ImageT",
    ButtonData:>"ImageT",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["KernelT",
    ButtonData:>"KernelT",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"RankT"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RankT"],

Cell["The images of all transformations on 3 points.", "ExampleText",
  CellTags->"RankT"],

Cell[BoxData[
    \(\({#, RankT[#]} &\) /@ ToList@TransformationSG[3] // \ 
      TableForm\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->"RankT"],

Cell["RegularOnly", "ObjectName",
  CellTags->"RegularOnly"],

Cell["Option", "Highlight",
  CellTags->"RegularOnly"],

Cell[TextData[{
  StyleBox["RegularOnly", "MR"],
  "  is an option for command ",
  StyleBox["DClassDecomposition", "MR"],
  ". "
}], "Usage",
  CellTags->"RegularOnly"],

Cell[TextData[{
  "Option ",
  StyleBox["RegularOnly->False", "MR"],
  " causes all D-classes to be generated, rather than just the regular ones. \
"
}], "Notes",
  CellTags->"RegularOnly"],

Cell["\<\
This option causes the internal version of the decomposition \
algorithm to be applied and should only be used for small semigroups.\
\>", \
"Notes",
  CellTags->"RegularOnly"],

Cell[TextData[{
  "See also ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["DClasses",
    ButtonData:>"DClasses",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"RegularOnly"],

Cell["RightIdealT", "ObjectName",
  CellTags->"RightIdealT"],

Cell["Function", "Highlight",
  CellTags->"RightIdealT"],

Cell[TextData[{
  StyleBox["RightIdealT[x,T]", "MR"],
  "  computes the right ideal generated by transformation x in semigroup S."
}], "Usage",
  CellTags->"RightIdealT"],

Cell["Returns a list of transformations.", "Notes",
  CellTags->"RightIdealT"],

Cell[TextData[{
  "See also ",
  ButtonBox["LeftIdealT",
    ButtonData:>"LeftIdealT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["IdealT",
    ButtonData:>"IdealT",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"RightIdealT"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RightIdealT"],

Cell["\<\
The full transformation semigroup on 3 points, and some of its \
right ideals.\
\>", "ExampleText",
  CellTags->"RightIdealT"],

Cell[BoxData[
    \(\(S = TransformationSG[3];\)\)], "Input",
  CellLabel->"In[1065]:=",
  CellTags->"RightIdealT"],

Cell[BoxData[
    \(RightIdealT[\ S[\([1]\)], \ S\ ]\)], "Input",
  CellLabel->"In[1066]:=",
  CellTags->"RightIdealT"],

Cell[BoxData[
    \(RightIdealT[\ S[\([2]\)], \ S\ ]\)], "Input",
  CellLabel->"In[1067]:=",
  CellTags->"RightIdealT"],

Cell["RightNullSG", "ObjectName",
  CellTags->"RightNullSG"],

Cell["Function", "Highlight",
  CellTags->"RightNullSG"],

Cell[TextData[{
  StyleBox["LeftNullSG[S]", "MR"],
  "  lists the right nulls of a semigroup ",
  StyleBox["S", "MR"],
  "."
}], "Usage",
  CellTags->"RightNullSG"],

Cell["Returns a list of transformations.", "Notes",
  CellTags->"RightNullSG"],

Cell[TextData[{
  StyleBox["RightNullSG[x,S]", "MR"],
  "  tests whether transformation ",
  StyleBox["x", "MR"],
  " is a right null in semigroup ",
  StyleBox["S", "MR"],
  "."
}], "Notes",
  CellTags->"RightNullSG"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"RightNullSG"],

Cell[TextData[{
  "See also  ",
  ButtonBox["OneSG",
    ButtonData:>"OneSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["LeftNullSG",
    ButtonData:>"LeftNullSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["NullSG",
    ButtonData:>"NullSG",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"RightNullSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RightNullSG"],

Cell["\<\
The full transformation semigroup on 3 points has 3 right-nulls, \
the constant transformations.\
\>", "ExampleText",
  CellTags->"RightNullSG"],

Cell["S = TransformationSG[3];", "Input",
  CellLabel->"In[1076]:=",
  CellTags->"RightNullSG"],

Cell[BoxData[
    \(Select[\ ToList[S], \ RightNullSG[#, S] &\ ]\)], "Input",
  CellLabel->"In[1073]:=",
  CellTags->"RightNullSG"],

Cell[BoxData[
    \(RightNullSG[S]\)], "Input",
  CellLabel->"In[1077]:=",
  CellTags->"RightNullSG"],

Cell["RightOneSG", "ObjectName",
  CellTags->"RightOneSG"],

Cell["Function", "Highlight",
  CellTags->"RightOneSG"],

Cell[TextData[{
  StyleBox["RightOneSG[S]", "MR"],
  "  lists the right identities of a semigroup ",
  StyleBox["S", "MR"],
  "."
}], "Usage",
  CellTags->"RightOneSG"],

Cell["Returns a list of transformations.", "Notes",
  CellTags->"RightOneSG"],

Cell[TextData[{
  StyleBox["RightOneSG[x,S]", "MR"],
  "  tests whether transformation ",
  StyleBox["x", "MR"],
  " is a right identity in semigroup ",
  StyleBox["S", "MR"],
  "."
}], "Notes",
  CellTags->"RightOneSG"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["Listable", "MR"],
  "."
}], "Notes",
  CellTags->"RightOneSG"],

Cell[TextData[{
  "See also  ",
  ButtonBox["OneSG",
    ButtonData:>"OneSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["LeftOneSG",
    ButtonData:>"LeftOneSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["NullSG",
    ButtonData:>"NullSG",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"RightOneSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RightOneSG"],

Cell["\<\
The full transformation semigroup on 3 points has only the identity \
as right-one.\
\>", "ExampleText",
  CellTags->"RightOneSG"],

Cell["S = TransformationSG[3];", "Input",
  CellLabel->"In[96]:=",
  CellTags->"RightOneSG"],

Cell[BoxData[
    \(Select[\ ToList[S], \ RightOneSG[#, S] &\ ]\)], "Input",
  CellLabel->"In[1083]:=",
  CellTags->"RightOneSG"],

Cell[BoxData[
    \(RightOneSG[S]\)], "Input",
  CellLabel->"In[97]:=",
  CellTags->"RightOneSG"],

Cell["ShiftT", "ObjectName",
  CellTags->"ShiftT"],

Cell["Constructor", "Highlight",
  CellTags->"ShiftT"],

Cell[TextData[{
  StyleBox["ShiftT[n]", "MR"],
  "  returns the right shift transformation ",
  StyleBox["T[n,1,2,..,n-1]", "MR"],
  "."
}], "Usage",
  CellTags->"ShiftT"],

Cell[TextData[{
  StyleBox["ShiftT[n,k]", "MR"],
  "  produces a right-shift by ",
  StyleBox["k", "MR"],
  " places."
}], "Notes",
  CellTags->"ShiftT"],

Cell[TextData[{
  StyleBox["ShiftT[n,k]", "MR"],
  "  Negative values of k produce a left-shift."
}], "Notes",
  CellTags->"ShiftT"],

Cell[TextData[{
  "See also ",
  StyleBox[" ", "Text"],
  ButtonBox["ConstantT",
    ButtonData:>"ConstantT",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["OneT",
    ButtonData:>"OneT",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", and ", "Text"],
  ButtonBox["RandomT",
    ButtonData:>"RandomT",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"ShiftT"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ShiftT"],

Cell["The semigroup generated by the right shift on 5 points. ", "ExampleText",
  CellTags->"ShiftT"],

Cell[BoxData[
    \(GenerateSG[\ ShiftT[10]]\)], "Input",
  CellLabel->"In[1090]:=",
  CellTags->"ShiftT"],

Cell["\<\
The sizes of the semigroups obtained by changing the shift factor. \
\
\>", "ExampleText",
  CellTags->"ShiftT"],

Cell[BoxData[
    \(Table[\ Length[GenerateSG[\ ShiftT[10, k]]], \ {k, 0, 9}\ ]\)], "Input",\

  CellLabel->"In[1091]:=",
  CellTags->"ShiftT"],

Cell["SG", "ObjectName",
  CellTags->"SG"],

Cell["Constructor", "Highlight",
  CellTags->"SG"],

Cell[TextData[{
  StyleBox["SG", "MR"],
  "  is a header used for transformation semigroups."
}], "Usage",
  CellTags->"SG"],

Cell[TextData[{
  " Elements are transformations, i.e., expressions of the form ",
  StyleBox["T[a1,..,an]", "MR"],
  " where ",
  Cell[BoxData[
      \(TraditionalForm\`1\  \[LessEqual] \ a\_\(\(\ \)\(i\)\) \[LessEqual] \ 
        n\)]],
  "."
}], "Notes",
  CellTags->"SG"],

Cell[TextData[{
  "See also ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["SymmetricGroupSG",
    ButtonData:>"SymmetricGroupSG",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["AlternatingGroupSG",
    ButtonData:>"AlternatingGroupSG",
    ButtonStyle->"AddOnsLink"],
  StyleBox[",", "Text"],
  " ",
  ButtonBox["T",
    ButtonData:>"T",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SG"],

Cell["\<\
The symmetric group on 3 points is in particular a semigroup.\
\>", \
"ExampleText",
  CellTags->"SG"],

Cell[BoxData[
    \(SymmetricGroupSG[3]\)], "Input",
  CellLabel->"In[85]:=",
  CellTags->"SG"],

Cell["SolveSG", "ObjectName",
  CellTags->"SolveSG"],

Cell["Function", "Highlight",
  CellTags->"SolveSG"],

Cell[TextData[{
  StyleBox["SolveSG[a,b,S]", "MR"],
  "  returns a list of all solutions of equation  ",
  StyleBox["a**x = b", "MR"],
  "  in a transformation semigroup ",
  StyleBox["S", "MR"],
  "."
}], "Usage",
  CellTags->"SolveSG"],

Cell[TextData[{
  "Options\n",
  Cell[BoxData[
      FormBox[GridBox[{
            {"Full", "\[Rule]", "True", "-"},
            {"Where", "\[Rule]", "Right", "Left"}
            },
          ColumnWidths->Automatic,
          ColumnAlignments->{Left}], TraditionalForm]]]
}], "Notes",
  CellTags->"SolveSG"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->False", "MR"],
  " returns only the first solution, or ",
  StyleBox["$Failed", "MR"],
  " if none is exists."
}], "Notes",
  CellTags->"SolveSG"],

Cell[TextData[{
  "Option ",
  StyleBox["Where->Left", "MR"],
  " solves   ",
  StyleBox["x**a = b", "MR"],
  "  instead."
}], "Notes",
  CellTags->"SolveSG"],

Cell[TextData[{
  "See also ",
  ButtonBox["SG",
    ButtonData:>"SG",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["T",
    ButtonData:>"T",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SolveSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SolveSG"],

Cell["In a group all equations have unique solutions.", "ExampleText",
  CellTags->"SolveSG"],

Cell["S = SymmetricGroupSG[3]", "Input",
  CellLabel->"In[1094]:=",
  CellTags->"SolveSG"],

Cell[BoxData[{
    \(SolveSG[\ T[1, 3, 2], T[2, 3, 1], S]\), "\n", 
    \(SolveSG[\ T[1, 3, 2], T[2, 3, 1], S, \ Where \[Rule] Left]\)}], "Input",\

  CellLabel->"In[1095]:=",
  CellTags->"SolveSG"],

Cell["But in general semigroups the situation is more complicated.", \
"ExampleText",
  CellTags->"SolveSG"],

Cell["S = TransformationSG[3];", "Input",
  CellLabel->"In[158]:=",
  CellTags->"SolveSG"],

Cell[BoxData[{
    \(SolveSG[\ T[1, 2, 2], T[2, 3, 3], S\ ]\), "\n", 
    \(SolveSG[\ T[1, 2, 2], T[2, 3, 3], S, \ Where \[Rule] Left]\)}], "Input",\

  CellLabel->"In[159]:=",
  CellTags->"SolveSG"],

Cell["SymmetricGroupSG", "ObjectName",
  CellTags->"SymmetricGroupSG"],

Cell["Function", "Highlight",
  CellTags->"SymmetricGroupSG"],

Cell[TextData[{
  StyleBox["SymmetricGroupSG[n]", "MR"],
  "  returns the symmetric group on n points."
}], "Usage",
  CellTags->"SymmetricGroupSG"],

Cell[TextData[{
  "Since the symmetric group has  ",
  Cell[BoxData[
      \(TraditionalForm\`\(n!\)\)]],
  "  elements, this can only be used for small ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  "."
}], "Notes",
  CellTags->"SymmetricGroupSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SymmetricGroupSG"],

Cell["The symmetric group on 4 points.", "ExampleText",
  CellTags->"SymmetricGroupSG"],

Cell["SymmetricGroupSG[4]", "Input",
  CellLabel->"In[90]:=",
  CellTags->"SymmetricGroupSG"],

Cell["SyntacticCongruence", "ObjectName",
  CellTags->"SyntacticCongruence"],

Cell["Function", "Highlight",
  CellTags->"SyntacticCongruence"],

Cell[TextData[{
  StyleBox["SyntacticCongruence[M,r]", "MR"],
  "  determines the syntactic congruence classes of all words of length ",
  StyleBox["r", "MR"],
  ". "
}], "Usage",
  CellTags->"SyntacticCongruence"],

Cell[TextData[{
  "Can also be used in the form ",
  StyleBox["SyntacticCongruence[gen,r]", "MR"],
  " where ",
  StyleBox["gen", "MR"],
  " is a generator object."
}], "Notes",
  CellTags->"SyntacticCongruence"],

Cell[TextData[{
  "See also ",
  ButtonBox["SyntacticSG",
    ButtonData:>"SyntacticSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["ToGeneratorsFA",
    ButtonData:>"ToGeneratorsFA",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["GEN",
    ButtonData:>"GEN",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"SyntacticCongruence"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SyntacticCongruence"],

Cell["\<\
A counting machine and its syntactic congruence classes for words \
of length 3.\
\>", "ExampleText",
  CellTags->"SyntacticCongruence"],

Cell[BoxData[
    \(m = CounterDFA[b, 0, 2, Modulus \[Rule] 3]\)], "Input",
  CellLabel->"In[202]:=",
  CellTags->"SyntacticCongruence"],

Cell[BoxData[
    \(sc\  = \ SyntacticCongruence[m, 3]\)], "Input",
  CellLabel->"In[203]:=",
  CellTags->"SyntacticCongruence"],

Cell["The equivalence is induced by the syntactic homomorphism.", \
"ExampleText",
  CellTags->"SyntacticCongruence"],

Cell[BoxData[{
    \(SyntacticHomomorphism[f, m]\), "\n", 
    \(Map[\ f, \ sc, \ {1}\ ]\)}], "Input",
  CellLabel->"In[204]:=",
  CellTags->"SyntacticCongruence"],

Cell["SyntacticHomomorphism", "ObjectName",
  CellTags->"SyntacticHomomorphism"],

Cell["Function", "Highlight",
  CellTags->"SyntacticHomomorphism"],

Cell[TextData[{
  StyleBox["SyntacticHomomorphism[f,M]", "MR"],
  "  assigns the syntactic homomorphism to symbol ",
  StyleBox["f", "MR"],
  ". "
}], "Usage",
  CellTags->"SyntacticHomomorphism"],

Cell[TextData[{
  "Has attribute ",
  StyleBox["HoldFirst", "MR"],
  "."
}], "Notes",
  CellTags->"SyntacticHomomorphism"],

Cell[TextData[{
  " ",
  Cell[BoxData[
      \(TraditionalForm\`f\  : \ \(K\^*\)\ \[LongRightArrow]\ S\)]],
  " is the natural homomorphism from the set of all words to the transition \
semigroup of DFA ",
  StyleBox["M", "MR"],
  "."
}], "Notes",
  CellTags->"SyntacticHomomorphism"],

Cell[TextData[{
  "See also ",
  ButtonBox["SyntacticSG",
    ButtonData:>"SyntacticSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["ToGeneratorsFA",
    ButtonData:>"ToGeneratorsFA",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"SyntacticHomomorphism"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SyntacticHomomorphism"],

Cell["A counting machine and its syntactic homomorphism.", "ExampleText",
  CellTags->"SyntacticHomomorphism"],

Cell["m = CounterDFA[ b,0,2,Modulus->3]", "Input",
  CellLabel->"In[208]:=",
  CellTags->"SyntacticHomomorphism"],

Cell[BoxData[
    \(SyntacticHomomorphism[f, m]\)], "Input",
  CellLabel->"In[209]:=",
  CellTags->"SyntacticHomomorphism"],

Cell[BoxData[
    \(f[b]\)], "Input",
  CellLabel->"In[210]:=",
  CellTags->"SyntacticHomomorphism"],

Cell[BoxData[
    \(f\  /@ \ Words[3]\)], "Input",
  CellLabel->"In[211]:=",
  CellTags->"SyntacticHomomorphism"],

Cell["SyntacticSG", "ObjectName",
  CellTags->"SyntacticSG"],

Cell["Function", "Highlight",
  CellTags->"SyntacticSG"],

Cell[TextData[{
  StyleBox["SyntacticSG[M]", "MR"],
  "  generates the transition semigroup of ",
  StyleBox["M", "MR"],
  ". "
}], "Usage",
  CellTags->"SyntacticSG"],

Cell[TextData[{
  "Make sure to minimize ",
  StyleBox["M", "MR"],
  " to obtain the syntactic semigroup of the acceptance language of ",
  StyleBox["M", "MR"],
  "."
}], "Notes",
  CellTags->"SyntacticSG"],

Cell[TextData[{
  "See also ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["ToGeneratorsFA",
    ButtonData:>"ToGeneratorsFA",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"SyntacticSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SyntacticSG"],

Cell["\<\
A counting machine and it syntactic semigroup, which happens to be \
a group.\
\>", "ExampleText",
  CellTags->"SyntacticSG"],

Cell[BoxData[
    \(m = CounterDFA[b, 3, 2, Modulus \[Rule] 5]\)], "Input",
  CellLabel->"In[217]:=",
  CellTags->"SyntacticSG"],

Cell[BoxData[
    \(SyntacticSG[m]\)], "Input",
  CellLabel->"In[218]:=",
  CellTags->"SyntacticSG"],

Cell["T", "ObjectName",
  CellTags->"T"],

Cell["Constructor", "Highlight",
  CellTags->"T"],

Cell[TextData[{
  StyleBox["T[a1,...,ak]", "MR"],
  "  denotes an element in a transformation semigroup"
}], "Usage",
  CellTags->"T"],

Cell[TextData[{
  "We must have  ",
  Cell[BoxData[
      \(TraditionalForm\`1\  \[LessEqual] \ a\_\(\(\ \)\(i\)\) \[LessEqual] \ 
        n\)]],
  ". "
}], "Notes",
  CellTags->"T"],

Cell[TextData[{
  "Composition is defined on ",
  StyleBox["NonCommutativeMultiply", "MR"],
  " and uses diagrammatic order:  ",
  StyleBox["s**t", "MR"],
  "  means: apply ",
  StyleBox["s", "MR"],
  " first, then apply ",
  StyleBox["t", "MR"],
  "."
}], "Notes",
  CellTags->"T"],

Cell[TextData[{
  "See also ",
  ButtonBox["OrderT",
    ButtonData:>"OrderT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["RankT",
    ButtonData:>"RankT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ImageT",
    ButtonData:>"ImageT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["KernelT",
    ButtonData:>"KernelT",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SG",
    ButtonData:>"SG",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"T"],

Cell["Further Examples", "ExampleSection",
  CellTags->"T"],

Cell["\<\
Some transformations and their products. Note that this operation \
is not commutative.\
\>", "ExampleText",
  CellTags->"T"],

Cell["\<\
s = T[2,3,1,1]
t = T[2,1,4,3]\
\>", "Input",
  CellLabel->"In[89]:=",
  CellTags->"T"],

Cell[BoxData[{
    \(s ** t\), "\n", 
    \(t ** s\)}], "Input",
  CellLabel->"In[91]:=",
  CellTags->"T"],

Cell["TableSG", "ObjectName",
  CellTags->"TableSG"],

Cell["Function", "Highlight",
  CellTags->"TableSG"],

Cell[TextData[{
  StyleBox["TableSG[S,W]", "MR"],
  "  prints the multiplication table of semigroup ",
  StyleBox["S", "MR"],
  " using witnesses ",
  StyleBox["W", "MR"],
  "."
}], "Usage",
  CellTags->"TableSG"],

Cell[TextData[{
  "Substitutes ",
  StyleBox["$identitystring", "MR"],
  " and ",
  StyleBox["$nullstring", "MR"],
  "  for identity and null, if they exist."
}], "Notes",
  CellTags->"TableSG"],

Cell["Mildly useful for semigroups of at most 20 elements.", "Notes",
  CellTags->"TableSG"],

Cell[TextData[{
  "See ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"TableSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"TableSG"],

Cell["\<\
The multiplication table for the symmetric group on 3 points.\
\>", \
"ExampleText",
  CellTags->"TableSG"],

Cell[BoxData[
    \(\({S, W, eq} = 
        SymmetricGroupSG[3, Equations \[Rule] True];\)\)], "Input",
  CellLabel->"In[155]:=",
  CellTags->"TableSG"],

Cell[BoxData[
    \(TableSG[\ S, W]\)], "Input",
  CellLabel->"In[156]:=",
  CellTags->"TableSG"],

Cell["ToIdentityString", "ObjectName",
  CellTags->"ToIdentityString"],

Cell["Function", "Highlight",
  CellTags->"ToIdentityString"],

Cell[TextData[{
  StyleBox["ToIdentityString[x]", "MR"],
  "  replaces all occurrences of the empty string in ",
  StyleBox["x", "MR"],
  " by ",
  StyleBox["$identitystring", "MR"],
  "."
}], "Usage",
  CellTags->"ToIdentityString"],

Cell[TextData[{
  "See also ",
  ButtonBox["$identitystring",
    ButtonData:>"$identitystring",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SimplifyEquations",
    ButtonData:>"SimplifyEquations",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToIdentityString"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToIdentityString"],

Cell["Cleaning up rewrite systems.", "ExampleText",
  CellTags->"ToIdentityString"],

Cell["ToIdentityString[ \"aaaa\"->\"\" ]", "Input",
  CellLabel->"In[1119]:=",
  CellTags->"ToIdentityString"],

Cell["TransformationSG", "ObjectName",
  CellTags->"TransformationSG"],

Cell["Function", "Highlight",
  CellTags->"TransformationSG"],

Cell[TextData[{
  StyleBox["TransformationSG[n]", "MR"],
  "  returns the transformation semigroup on ",
  StyleBox["n", "MR"],
  " points. "
}], "Usage",
  CellTags->"TransformationSG"],

Cell[TextData[{
  " Since the full transformation semigroup has ",
  Cell[BoxData[
      \(TraditionalForm\`n\^\(\(\ \)\(n\)\)\)]],
  " elements, this can only be used for small ",
  StyleBox["n", "MR"],
  "."
}], "Notes",
  CellTags->"TransformationSG"],

Cell[TextData[{
  "See ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SymmetricGroupSG",
    ButtonData:>"SymmetricGroupSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["AlternatingGroupSG",
    ButtonData:>"AlternatingGroupSG",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"TransformationSG"],

Cell["Further Examples", "ExampleSection",
  CellTags->"TransformationSG"],

Cell["The full transformation semigroup on 3 points.", "ExampleText",
  CellTags->"TransformationSG"],

Cell["S = TransformationSG[3]", "Input",
  CellLabel->"In[123]:=",
  CellTags->"TransformationSG"],

Cell[BoxData[
    \(S\  // \ Length\)], "Input",
  CellLabel->"In[124]:=",
  CellTags->"TransformationSG"],

Cell["TransientPeriodicT", "ObjectName",
  CellTags->"TransientPeriodicT"],

Cell["Function", "Highlight",
  CellTags->"TransientPeriodicT"],

Cell[TextData[{
  StyleBox["TransientPeriodicT[x]", "MR"],
  "  returns the length of the transient and periodic part of the semigroup \
generated by transformation ",
  StyleBox["x", "MR"],
  ". "
}], "Usage",
  CellTags->"TransientPeriodicT"],

Cell["\<\
If x is a permutation, the length of the periodic part is the order \
of x in the group generated by it.\
\>", "Notes",
  CellTags->"TransientPeriodicT"],

Cell[TextData[{
  "See also ",
  ButtonBox["OrderT",
    ButtonData:>"OrderT",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"TransientPeriodicT"],

Cell["Further Examples", "ExampleSection",
  CellTags->"TransientPeriodicT"],

Cell["\<\
The transients and periods for all elements of the full \
transformation semigroup on 3 points.\
\>", "ExampleText",
  CellTags->"TransientPeriodicT"],

Cell[BoxData[
    \(\(S = TransformationSG[3];\)\)], "Input",
  CellLabel->"In[1124]:=",
  CellTags->"TransientPeriodicT"],

Cell[BoxData[
    \(TransientPeriodicT\  /@ \ ToList[S]\)], "Input",
  CellLabel->"In[1125]:=",
  CellTags->"TransientPeriodicT"],

Cell[BoxData[
    \(Frequencies[%]\  // \ \(Curry[TableForm]\)[
        TableDepth \[Rule] 2]\)], "Input",
  CellLabel->"In[1126]:=",
  CellTags->"TransientPeriodicT"],

Cell["TranspositionT", "ObjectName",
  CellTags->"TranspositionT"],

Cell["Function", "Highlight",
  CellTags->"TranspositionT"],

Cell[TextData[{
  StyleBox["TranspositionT[n,i]", "MR"],
  "  returns the transposition ",
  StyleBox["(i,i+1)", "MR"],
  "  as a transformation on ",
  StyleBox["n", "MR"],
  "."
}], "Usage",
  CellTags->"TranspositionT"],

Cell[TextData[{
  "Position ",
  StyleBox["i", "MR"],
  " has default value 1."
}], "Notes",
  CellTags->"TranspositionT"],

Cell[TextData[{
  "See also ",
  ButtonBox["Merge12T",
    ButtonData:>"Merge12T",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ShiftT",
    ButtonData:>"ShiftT",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"TranspositionT"],

Cell["Further Examples", "ExampleSection",
  CellTags->"TranspositionT"],

Cell["Transpositions can be used to generate the symmetric group.", \
"ExampleText",
  CellTags->"TranspositionT"],

Cell["TranspositionT[4,1]", "Input",
  CellLabel->"In[1130]:=",
  CellTags->"TranspositionT"],

Cell["GenerateSG[GEN[4,3,Table[TranspositionT[4,i],{i,3}]]]", "Input",
  CellLabel->"In[1133]:=",
  CellTags->"TranspositionT"],

Cell[BoxData[
    \(%\  // \ Length\)], "Input",
  CellLabel->"In[1134]:=",
  CellTags->"TranspositionT"],

Cell["UnitElement", "ObjectName",
  CellTags->"UnitElement"],

Cell["Function", "Highlight",
  CellTags->"UnitElement"],

Cell[TextData[{
  StyleBox["UnitElement[x]", "MR"],
  "  returns the unit element in the monoid to which ",
  StyleBox["x", "MR"],
  " naturally belongs. "
}], "Usage",
  CellTags->"UnitElement"],

Cell[TextData[{
  "More precisely, ",
  StyleBox["x", "MR"],
  " can be a transformation, a Boolean matrix, a matrix over the tropical \
semiring, or an integer matrix. "
}], "Notes",
  CellTags->"UnitElement"],

Cell[TextData[{
  "Used internally in ",
  StyleBox["GenerateSG", "MR"],
  "."
}], "Notes",
  CellTags->"UnitElement"],

Cell[TextData[{
  "See ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"UnitElement"],

Cell["Further Examples", "ExampleSection",
  CellTags->"UnitElement"],

Cell["Some unit elements.", "ExampleText",
  CellTags->"UnitElement"],

Cell["UnitElement[T[2,2,2]]", "Input",
  CellLabel->"In[159]:=",
  CellTags->"UnitElement"],

Cell["UnitElement[{{1,1},{0,0}}]", "Input",
  CellLabel->"In[160]:=",
  CellTags->"UnitElement"],

Cell["$rewriteSimpBound", "ObjectName",
  CellTags->"$rewriteSimpBound"],

Cell["SysVariable", "Highlight",
  CellTags->"$rewriteSimpBound"],

Cell[TextData[{
  StyleBox["$rewriteSimpBound", "MR"],
  "  is a default bound for the number of transformation steps taken in the \
normalization algorithm for rewrite systems. "
}], "Usage",
  CellTags->"$rewriteSimpBound"],

Cell["Default value is 100.", "Notes",
  CellTags->"$rewriteSimpBound"],

Cell[TextData[{
  "See ",
  ButtonBox["SimplifyRewriteSystem",
    ButtonData:>"SimplifyRewriteSystem",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"$rewriteSimpBound"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$rewriteSimpBound"],

Cell["The system default value is 100:", "ExampleText",
  CellTags->"$rewriteSimpBound"],

Cell[BoxData[
    \($rewriteSimpBound\)], "Input",
  CellLabel->"In[1144]:=",
  CellTags->"$rewriteSimpBound"],

Cell["AllStringReplace", "ObjectName",
  CellTags->"AllStringReplace"],

Cell["Type: functional", "Highlight",
  CellTags->"AllStringReplace"],

Cell[TextData[{
  StyleBox["AllStringReplace[eq][w] ", "MR"],
  " generates all words obtainable from word ",
  StyleBox["w", "MR"],
  " by applying the rewrite equations in ",
  StyleBox["eq", "MR"],
  " once."
}], "Usage",
  CellTags->"AllStringReplace"],

Cell[TextData[{
  "The rewrite equations can be given as a list of pairs of the form ",
  StyleBox["{lhs,rhs}", "MR"],
  ", both words, or as a list of rules  ",
  StyleBox["lhs\[Rule]rhs", "MR"],
  "."
}], "Notes",
  CellTags->"AllStringReplace"],

Cell[TextData[{
  " ",
  StyleBox["w", "MR"],
  " can also be a list of words."
}], "Notes",
  CellTags->"AllStringReplace"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordQ",
    ButtonData:>"WordQ",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"AllStringReplace"],

Cell["Further Examples", "ExampleSection",
  CellTags->"AllStringReplace"],

Cell["All possible single substitutions are performed.", "ExampleText",
  CellTags->"AllStringReplace"],

Cell["AllStringReplace[{{aa,c},{ab,d}}][aab]", "Input",
  CellLabel->"In[1149]:=",
  CellTags->"AllStringReplace"],

Cell[BoxData[
    \(\(AllStringReplace[{aa \[Rule] c, ab \[Rule] d}]\)[aab]\)], "Input",
  CellLabel->"In[1150]:=",
  CellTags->"AllStringReplace"],

Cell[BoxData[
    \(\(AllStringReplace[a \[Rule] \[Epsilon]]\)[ababa]\)], "Input",
  CellLabel->"In[1147]:=",
  CellTags->"AllStringReplace"],

Cell["BoundETW", "ObjectName",
  CellTags->"BoundETW"],

Cell["Option", "Highlight",
  CellTags->"BoundETW"],

Cell[TextData[{
  StyleBox["BoundETW", "MR"],
  "  is an option for ",
  StyleBox["EquationsToWitnesses", "MR"],
  ". "
}], "Usage",
  CellTags->"BoundETW"],

Cell["Default is 500.", "Notes",
  CellTags->"BoundETW"],

Cell[TextData[{
  "See ",
  ButtonBox["EquationsToWitnesses",
    ButtonData:>"EquationsToWitnesses",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"BoundETW"],

Cell["BoundWR", "ObjectName",
  CellTags->"BoundWR"],

Cell["Option", "Highlight",
  CellTags->"BoundWR"],

Cell[TextData[{
  StyleBox["BoundWR", "MR"],
  "  is an upper bound for the number of reduction steps in ",
  StyleBox["WordReduce", "MR"],
  ". "
}], "Usage",
  CellTags->"BoundWR"],

Cell["Default is 500.", "Notes",
  CellTags->"BoundWR"],

Cell[TextData[{
  "See ",
  ButtonBox["WordReduce",
    ButtonData:>"WordReduce",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"BoundWR"],

Cell["Cols", "ObjectName",
  CellTags->"Cols"],

Cell["Option", "Highlight",
  CellTags->"Cols"],

Cell[TextData[{
  StyleBox["Cols", "MR"],
  "  is an option ",
  StyleBox["PrintEquations", "MR"],
  "."
}], "Usage",
  CellTags->"Cols"],

Cell[TextData[{
  "See ",
  ButtonBox["PrintEquations",
    ButtonData:>"PrintEquations",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Cols"],

Cell["EquationsToWitnesses", "ObjectName",
  CellTags->"EquationsToWitnesses"],

Cell["Function", "Highlight",
  CellTags->"EquationsToWitnesses"],

Cell[TextData[{
  StyleBox["EquationsToWitnesses[eq,K]", "MR"],
  "  generates witnesses for the semigroup on generators ",
  StyleBox["K", "MR"],
  " with defining relationes given by ",
  StyleBox["eq", "MR"],
  ", a rewrite system. "
}], "Usage",
  CellTags->"EquationsToWitnesses"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"},
          {"BoundETW", "\[Rule]", "500", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"EquationsToWitnesses"],

Cell[TextData[{
  "Option ",
  StyleBox["BoundETW->n", "MR"],
  " defines ",
  StyleBox["n", "MR"],
  " as upper for the number of witnesses produced, has default 500."
}], "Notes",
  CellTags->"EquationsToWitnesses"],

Cell["\<\
Note that the equations are only used in directed form left to \
right, no attempt is made to generate the Thue congruence. \
\>", "Notes",
  CellTags->"EquationsToWitnesses"],

Cell[TextData[{
  "See also ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"EquationsToWitnesses"],

Cell["Further Examples", "ExampleSection",
  CellTags->"EquationsToWitnesses"],

Cell[TextData[{
  "A simple one-letter example:  rule ",
  Cell[BoxData[
      \(TraditionalForm\`a\^\(\(\ \)\(5\)\) \[Rule] \ a\)]],
  "."
}], "ExampleText",
  CellTags->"EquationsToWitnesses"],

Cell[BoxData[
    \(EquationsToWitnesses[{{aaaaa, a}}, {a}]\)], "Input",
  CellLabel->"In[1153]:=",
  AspectRatioFixed->True,
  CellTags->"EquationsToWitnesses"],

Cell[BoxData[
    \(EquationsToWitnesses[{{aaaaa, a}}, {a}, Full \[Rule] True]\)], "Input",
  CellLabel->"In[1154]:=",
  AspectRatioFixed->True,
  CellTags->"EquationsToWitnesses"],

Cell[TextData[{
  "An two-letter example: one rule ",
  Cell[BoxData[
      \(TraditionalForm\`a\^\(\(\ \)\(5\)\) \[Rule] \ a\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`b\^\(\(\ \)\(3\)\) \[Rule] b\^\(\(\ \)\(2\)\)\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`b\[VeryThinSpace]a \[Rule] a\[VeryThinSpace]b\)]],
  "."
}], "ExampleText",
  CellTags->"EquationsToWitnesses"],

Cell[BoxData[{
    \(\(eq = {{aaaaa, a}, {bbb, bb}, {ba, ab}};\)\), "\n", 
    \(S = EquationsToWitnesses[eq, {a, b}]\)}], "Input",
  CellLabel->"In[1155]:=",
  CellTags->"EquationsToWitnesses"],

Cell["\<\
All products of these words are indeed reducible to an element in \
S.\
\>", "ExampleText",
  CellTags->"EquationsToWitnesses"],

Cell[BoxData[{
    \(\(SS\  = \ Concatenate[S, S];\)\), "\n", 
    \(\(WordReduce[\ SS, \ eq\ ]\  // \ Union\)\  // \ WordSort\)}], "Input",
  CellLabel->"In[1157]:=",
  CellTags->"EquationsToWitnesses"],

Cell[BoxData[
    \(SubsetQ[%, S]\)], "Input",
  CellLabel->"In[1159]:=",
  CellTags->"EquationsToWitnesses"],

Cell["Equations", "ObjectName",
  CellTags->"Equations"],

Cell["Option", "Highlight",
  CellTags->"Equations"],

Cell[TextData[{
  StyleBox["Equations", "MR"],
  "  is an option for various functions."
}], "Usage",
  CellTags->"Equations"],

Cell[TextData[{
  "See ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["GenerateCSM",
    ButtonData:>"GenerateCSM",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Equations"],

Cell["FromIdentityString", "ObjectName",
  CellTags->"FromIdentityString"],

Cell["Function", "Highlight",
  CellTags->"FromIdentityString"],

Cell[TextData[{
  StyleBox["FromIdentityString[x]", "MR"],
  "  replaces all occurences of ",
  StyleBox["$identitystring", "MR"],
  " in ",
  StyleBox["x", "MR"],
  " by the empty string."
}], "Usage",
  CellTags->"FromIdentityString"],

Cell[TextData[{
  StyleBox["$identitystring", "MR"],
  " is by default \"1\"."
}], "Notes",
  CellTags->"FromIdentityString"],

Cell[TextData[StyleBox["Used internally to clean up rewrite systems in \
algebraic notation.", "Text"]], "Notes",
  CellTags->"FromIdentityString"],

Cell[TextData[{
  "See also  ",
  ButtonBox["AlgebraicForm",
    ButtonData:>"AlgebraicForm",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["EquationsToWitnesses",
    ButtonData:>"EquationsToWitnesses",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["FromIdentityString",
    ButtonData:>"FromIdentityString",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["identitystring",
    ButtonData:>"identitystring",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["SimplifyEquations",
    ButtonData:>"SimplifyEquations",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["SimplifyRewriteSystem",
    ButtonData:>"SimplifyRewriteSystem",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["TableSG",
    ButtonData:>"TableSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["ToIdentityString",
    ButtonData:>"ToIdentityString",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["WordReduce",
    ButtonData:>"WordReduce",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"FromIdentityString"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FromIdentityString"],

Cell["Removing elements introduced for pretty-printing.", "ExampleText",
  CellTags->"FromIdentityString"],

Cell["eq = {{\"aaa\",\"1\"},{\"bbb\",\"1\"}}; ", "Input",
  CellLabel->"In[621]:=",
  CellTags->"FromIdentityString"],

Cell["eq // PrintEquations", "Input",
  CellLabel->"In[629]:=",
  CellTags->"FromIdentityString"],

Cell["FromIdentityString[eq] // PrintEquations", "Input",
  CellLabel->"In[666]:=",
  CellTags->"FromIdentityString"],

Cell["IrreducibleQ", "ObjectName",
  CellTags->"IrreducibleQ"],

Cell["Function", "Highlight",
  CellTags->"IrreducibleQ"],

Cell[TextData[{
  StyleBox["IrreducibleQ[w,eq]", "MR"],
  "  test is word ",
  StyleBox["w", "MR"],
  " is irreducible with respect to the rewrite system ",
  StyleBox["eq", "MR"],
  ". "
}], "Usage",
  CellTags->"IrreducibleQ"],

Cell[TextData[{
  "Note that only the left hand sides of ",
  StyleBox["eq", "MR"],
  " are used."
}], "Notes",
  CellTags->"IrreducibleQ"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordReduce",
    ButtonData:>"WordReduce",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SimplifyEquations",
    ButtonData:>"SimplifyEquations",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"IrreducibleQ"],

Cell["Further Examples", "ExampleSection",
  CellTags->"IrreducibleQ"],

Cell["A small rewrite system.", "ExampleText",
  CellTags->"IrreducibleQ"],

Cell[BoxData[
    \(\(eq = {aaa \[Rule] b, aba \[Rule] a};\)\)], "Input",
  CellLabel->"In[916]:=",
  CellTags->"IrreducibleQ"],

Cell[BoxData[
    \(IrreducibleQ[\ abababa, \ eq\ ]\)], "Input",
  CellLabel->"In[918]:=",
  CellTags->"IrreducibleQ"],

Cell[BoxData[
    \(IrreducibleQ[\ abbabb, \ eq\ ]\)], "Input",
  CellTags->"IrreducibleQ"],

Cell["PairsToRules", "ObjectName",
  CellTags->"PairsToRules"],

Cell["Function", "Highlight",
  CellTags->"PairsToRules"],

Cell[TextData[{
  StyleBox["PairsToRules[rw]", "MR"],
  "  replaces word pairs by rules."
}], "Usage",
  CellTags->"PairsToRules"],

Cell[TextData[{
  "See also ",
  ButtonBox["RulesToPairs",
    ButtonData:>"RulesToPairs",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["SimplifyEquations",
    ButtonData:>"SimplifyEquations",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["SimplifyRewriteSystem",
    ButtonData:>"SimplifyRewriteSystem",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["WordReduce",
    ButtonData:>"WordReduce",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"PairsToRules"],

Cell["Further Examples", "ExampleSection",
  CellTags->"PairsToRules"],

Cell["Rewrite systems can be displayed as lists of rules. ", "ExampleText",
  CellTags->"PairsToRules"],

Cell[BoxData[
    \(\(rw\  = \ {{"\<bc\>", "\<c\>"}, {"\<cc\>", "\<c\>"}, {"\<aaa\>", "\<bb\
\>"}, {"\<aab\>", "\<ba\>"}, {"\<aba\>", "\<b\>"}};\)\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->"PairsToRules"],

Cell[BoxData[
    \(PairsToRules[rw]\  // \ TableForm\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->"PairsToRules"],

Cell["RulesToPairs", "ObjectName",
  CellTags->"RulesToPairs"],

Cell["Function", "Highlight",
  CellTags->"RulesToPairs"],

Cell[TextData[{
  StyleBox["RulesToPairs[rw]", "MR"],
  "  replaces rewrite rules by word pairs."
}], "Usage",
  CellTags->"RulesToPairs"],

Cell[TextData[{
  "See also ",
  ButtonBox["PairsToRules",
    ButtonData:>"PairsToRules",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["SimplifyEquations",
    ButtonData:>"SimplifyEquations",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["SimplifyRewriteSystem",
    ButtonData:>"SimplifyRewriteSystem",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["WordReduce",
    ButtonData:>"WordReduce",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"RulesToPairs"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RulesToPairs"],

Cell["A rewrite systems using rules, and its pair form.", "ExampleText",
  CellTags->"RulesToPairs"],

Cell[BoxData[
    \(\(eq\  = \ {"\<bc\>" \[Rule] "\<c\>", "\<cc\>" \[Rule] "\<c\>", \
"\<aaa\>" \[Rule] "\<bb\>", "\<aab\>" \[Rule] "\<ba\>", "\<aba\>" \[Rule] \
"\<Eps\>"};\)\)], "Input",
  CellLabel->"In[53]:=",
  CellTags->"RulesToPairs"],

Cell[BoxData[
    \(RulesToPairs[eq]\)], "Input",
  CellLabel->"In[55]:=",
  CellTags->"RulesToPairs"],

Cell["SimplifyEquations", "ObjectName",
  CellTags->"SimplifyEquations"],

Cell["Function", "Highlight",
  CellTags->"SimplifyEquations"],

Cell[TextData[{
  StyleBox["SimplifyEquations[S,wit,eq]", "MR"],
  "  simplifies the rewrite system associated with a semigroup produces by ",
  StyleBox["GenerateSG", "MR"],
  "."
}], "Usage",
  CellTags->"SimplifyEquations"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "True", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"SimplifyEquations"],

Cell[TextData[{
  " Also rewrites null as ",
  StyleBox["$nullstring", "MR"],
  " (default: 0) and one as ",
  StyleBox["$identitystring", "MR"],
  " (default: 1), if there is a null and/or one in ",
  StyleBox["S", "MR"],
  ". "
}], "Notes",
  CellTags->"SimplifyEquations"],

Cell[TextData[{
  "See also ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SimplifyRewriteSystem",
    ButtonData:>"SimplifyRewriteSystem",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SimplifyEquations"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SimplifyEquations"],

Cell["\<\
The full transformation semigroup on 3 points has an identity, but \
no null.\
\>", "ExampleText",
  CellTags->"SimplifyEquations"],

Cell[BoxData[
    \(\({S, W, rw} = 
        TransformationSG[3, Equations \[Rule] True];\)\)], "Input",
  CellLabel->"In[13]:=",
  CellTags->"SimplifyEquations"],

Cell[BoxData[
    \(rw\  // \ Length\)], "Input",
  CellLabel->"In[14]:=",
  CellTags->"SimplifyEquations"],

Cell[BoxData[
    \(SimplifyEquations[S, W, rw]\  // \ PrintEquations\)], "Input",
  CellLabel->"In[15]:=",
  CellTags->"SimplifyEquations"],

Cell["SimplifyRewriteSystem", "ObjectName",
  CellTags->"SimplifyRewriteSystem"],

Cell["Function", "Highlight",
  CellTags->"SimplifyRewriteSystem"],

Cell[TextData[{
  StyleBox["SimplifyRewriteSystem[eq]", "MR"],
  "  normalizes a rewrite system given by the directed equations ",
  StyleBox["eq", "MR"],
  ". "
}], "Usage",
  CellTags->"SimplifyRewriteSystem"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"DoTrace", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"SimplifyRewriteSystem"],

Cell[TextData[{
  "In the output, both the LHS and the RHS of all equations will be \
irreducible. The Thue congruence determined by ",
  StyleBox["eq", "MR"],
  " is not affected by this transformation."
}], "Notes",
  CellTags->"SimplifyRewriteSystem"],

Cell[TextData[{
  "Note that ",
  StyleBox["SimplifyRewriteSystem", "MR"],
  " is implemented only internally and rather slow; do not use on more than \
50 equations. "
}], "Notes",
  CellTags->"SimplifyRewriteSystem"],

Cell[TextData[{
  "See also ",
  ButtonBox["SimplifyEquations",
    ButtonData:>"SimplifyEquations",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SimplifyRewriteSystem"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SimplifyRewriteSystem"],

Cell["\<\
The rewrite system produced by generating the full transformation \
semigroup on 3 points is rather large, but can be simplified.\
\>", \
"ExampleText",
  CellTags->"SimplifyRewriteSystem"],

Cell[BoxData[
    \(\({S, W, eq} = 
        TransformationSG[3, Equations \[Rule] True];\)\)], "Input",
  CellLabel->"In[18]:=",
  CellTags->"SimplifyRewriteSystem"],

Cell[BoxData[
    \(eq\  // \ Length\)], "Input",
  CellLabel->"In[19]:=",
  CellTags->"SimplifyRewriteSystem"],

Cell[BoxData[
    \(\(eqs\  = \ SimplifyRewriteSystem[eq];\)\)], "Input",
  CellLabel->"In[20]:=",
  CellTags->"SimplifyRewriteSystem"],

Cell[BoxData[
    \(PrintEquations[eqs]\)], "Input",
  CellLabel->"In[21]:=",
  CellTags->"SimplifyRewriteSystem"],

Cell["WordReduce", "ObjectName",
  CellTags->"WordReduce"],

Cell["Function", "Highlight",
  CellTags->"WordReduce"],

Cell[TextData[{
  StyleBox["WordReduce[w,eq,opt]", "MR"],
  "  reduces word ",
  StyleBox["w", "MR"],
  " according to equations ",
  StyleBox["eq", "MR"],
  ". "
}], "Usage",
  CellTags->"WordReduce"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"DoTrace", "\[Rule]", "False", "-"},
          {"BoundWR", "\[Rule]", "500", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"WordReduce"],

Cell[TextData[{
  "Option ",
  StyleBox["DoTrace->True", "MR"],
  " causes a trace to be displayed.  Uses ",
  ButtonBox["StringReplace",
    ButtonStyle->"RefGuideLink"],
  " internally and may therefore perform several substitutions in one step. "
}], "Notes",
  CellTags->"WordReduce"],

Cell[TextData[{
  "Option ",
  StyleBox["BoundRW->500", "MR"],
  " bounds the number of substitution steps."
}], "Notes",
  CellTags->"WordReduce"],

Cell["\<\
The equations are only applied in directed form from left to right. \
\
\>", "Notes",
  CellTags->"WordReduce"],

Cell[TextData[{
  "See also ",
  ButtonBox["SimplifyEquations",
    ButtonData:>"SimplifyEquations",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SimplifyRewriteSystem",
    ButtonData:>"SimplifyRewriteSystem",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"WordReduce"],

Cell["Further Examples", "ExampleSection",
  CellTags->"WordReduce"],

Cell["A small rewrite system.", "ExampleText",
  CellTags->"WordReduce"],

Cell[BoxData[
    \(\(eq = {{aaa, \[Epsilon]}, {bbb, \[Epsilon]}, {ba, ab}};\)\)], "Input",
  CellLabel->"In[24]:=",
  CellTags->"WordReduce"],

Cell[BoxData[
    \(WordReduce[\ RandomWord[10, 2], \ eq\ ]\)], "Input",
  CellLabel->"In[25]:=",
  CellTags->"WordReduce"],

Cell[BoxData[
    \(WordReduce[\ RandomWord[10, 2], \ eq\ , 
      DoTrace \[Rule] True]\)], "Input",
  CellLabel->"In[26]:=",
  CellTags->"WordReduce"],

Cell["All words that are irreducible with respect to eq.", "ExampleText",
  CellTags->"WordReduce"],

Cell[BoxData[
    \(\(WordReduce[\ Words[\(-6\)], eq]\  // \ Union\)\  // \ 
      WordSort\)], "Input",
  CellLabel->"In[27]:=",
  CellTags->"WordReduce"],

Cell["BooleanComplement", "ObjectName",
  CellTags->"BooleanComplement"],

Cell["Function", "Highlight",
  CellTags->"BooleanComplement"],

Cell[TextData[{
  StyleBox["BooleanComplement[M1,M2]", "MR"],
  "  computes the Boolean difference (pointwise) of two Boolean matrices ",
  StyleBox["M1", "MR"],
  " and ",
  StyleBox["M2", "MR"],
  ", corresponding to relational complement. Also works with a single matrix. \
"
}], "Usage",
  CellTags->"BooleanComplement"],

Cell["\<\
Note that for  efficiency reasons Boolean matrices are implemented \
using 0/1-integer matrices. \
\>", "Notes",
  CellTags->"BooleanComplement"],

Cell[TextData[{
  "See ",
  ButtonBox["RelationToMatrix",
    ButtonData:>"RelationToMatrix",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanIntersection",
    ButtonData:>"BooleanIntersection",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanUnion",
    ButtonData:>"BooleanUnion",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanComposition",
    ButtonData:>"BooleanComposition",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanConverse",
    ButtonData:>"BooleanConverse",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanPower",
    ButtonData:>"BooleanPower",
    ButtonStyle->"AddOnsLink"],
  StyleBox[". ", "Text"]
}], "Notes",
  CellTags->"BooleanComplement"],

Cell["Further Examples", "ExampleSection",
  CellTags->"BooleanComplement"],

Cell["Two Boolean matrices, and their complement.", "ExampleText",
  CellTags->"BooleanComplement"],

Cell["\<\
M1 = {{1,1,1},{1,1,1},{1,1,1}};
M2 = {{0,1,1},{1,0,1},{1,1,0}};\
\>", "Input",
  CellLabel->"In[30]:=",
  CellTags->"BooleanComplement"],

Cell["BooleanComplement[M1,M2]", "Input",
  CellLabel->"In[32]:=",
  CellTags->"BooleanComplement"],

Cell["BooleanComplement[M1]", "Input",
  CellLabel->"In[56]:=",
  CellTags->"BooleanComplement"],

Cell["BooleanComposition", "ObjectName",
  CellTags->"BooleanComposition"],

Cell["Function", "Highlight",
  CellTags->"BooleanComposition"],

Cell[TextData[{
  StyleBox["BooleanComposition[M1,M2]", "MR"],
  "  computes the Boolean dot product of two Boolean matrices ",
  StyleBox["M1", "MR"],
  " and ",
  StyleBox["M2", "MR"],
  ", corresponding to relational composition. "
}], "Usage",
  CellTags->"BooleanComposition"],

Cell["\<\
Note that for  efficiency reasons Boolean matrices are implemented \
using 0/1-integer matrices. \
\>", "Notes",
  CellTags->"BooleanComposition"],

Cell[TextData[{
  "See ",
  ButtonBox["RelationToMatrix",
    ButtonData:>"RelationToMatrix",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanIntersection",
    ButtonData:>"BooleanIntersection",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanComplement",
    ButtonData:>"BooleanComplement",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanUnion",
    ButtonData:>"BooleanUnion",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanConverse",
    ButtonData:>"BooleanConverse",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanPower",
    ButtonData:>"BooleanPower",
    ButtonStyle->"AddOnsLink"],
  StyleBox[". ", "Text"]
}], "Notes",
  CellTags->"BooleanComposition"],

Cell["Further Examples", "ExampleSection",
  CellTags->"BooleanComposition"],

Cell[TextData[{
  "The matrix representing the less-than relation on ",
  Cell[BoxData[
      \(TraditionalForm\`{1, 2, \[Ellipsis], 6}\)]],
  "."
}], "ExampleText",
  CellTags->"BooleanComposition"],

Cell["L = RelationToMatrix[Less,Range[6]]", "Input",
  CellLabel->"In[74]:=",
  CellTags->"BooleanComposition"],

Cell["\<\
Composing this matrix with itself produces the following matrices. \
\
\>", "ExampleText",
  CellTags->"BooleanComposition"],

Cell[BoxData[{
    \(\(L2 = BooleanComposition[L, L];\)\), "\n", 
    \(L2\  // \ MatrixForm\)}], "Input",
  CellLabel->"In[75]:=",
  CellTags->"BooleanComposition"],

Cell[BoxData[{
    \(\(L5 = BooleanComposition[L, L, L, L, L];\)\), "\n", 
    \(L5\  // \ MatrixForm\)}], "Input",
  CellLabel->"In[77]:=",
  CellTags->"BooleanComposition"],

Cell["BooleanConverse", "ObjectName",
  CellTags->"BooleanConverse"],

Cell["Function", "Highlight",
  CellTags->"BooleanConverse"],

Cell[TextData[{
  StyleBox["BooleanConverse[M]", "MR"],
  "  computes the transpose of a Boolean matrix ",
  StyleBox["M", "MR"],
  ", corresponding to relational converse. "
}], "Usage",
  CellTags->"BooleanConverse"],

Cell[TextData[{
  "Use ",
  ButtonBox["RelationToMatrix",
    ButtonData:>"RelationToMatrix",
    ButtonStyle->"AddOnsLink"],
  " to convert a relation given by a function into a matrix."
}], "Notes",
  CellTags->"BooleanConverse"],

Cell["\<\
Note that for  efficiency reasons Boolean matrices are implemented \
using 0/1-integer matrices. \
\>", "Notes",
  CellTags->"BooleanConverse"],

Cell[TextData[{
  "See also ",
  ButtonBox["RelationToMatrix",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanIntersection",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanComplement",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanComposition",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanUnion",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanPower",
    ButtonStyle->"AddOnsLink"],
  StyleBox[". ", "Text"]
}], "Notes",
  CellTags->"BooleanConverse"],

Cell["Further Examples", "ExampleSection",
  CellTags->"BooleanConverse"],

Cell[TextData[{
  "The matrix representing the less-than relation on ",
  Cell[BoxData[
      \(TraditionalForm\`{1, 2, \[Ellipsis], 6}\)]],
  ", and its converse."
}], "ExampleText",
  CellTags->"BooleanConverse"],

Cell["\<\
L = RelationToMatrix[Range[6],Less];
L // MatrixForm\
\>", "Input",
  CellLabel->"In[218]:=",
  CellTags->"BooleanConverse"],

Cell["BooleanConverse[L] // MatrixForm", "Input",
  CellLabel->"In[220]:=",
  CellTags->"BooleanConverse"],

Cell["BooleanIntersection", "ObjectName",
  CellTags->"BooleanIntersection"],

Cell["Function", "Highlight",
  CellTags->"BooleanIntersection"],

Cell[TextData[{
  StyleBox["BooleanIntersection[M1,M2]", "MR"],
  "  computes the Boolean product (pointwise) of two Boolean matrices ",
  StyleBox["M1", "MR"],
  " and ",
  StyleBox["M2", "MR"],
  ", corresponding to relational intersection. "
}], "Usage",
  CellTags->"BooleanIntersection"],

Cell["\<\
Note that for  efficiency reasons Boolean matrices are implemented \
using 0/1-integer matrices. \
\>", "Notes",
  CellTags->"BooleanIntersection"],

Cell[TextData[{
  "See ",
  ButtonBox["RelationToMatrix",
    ButtonData:>"RelationToMatrix",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanUnion",
    ButtonData:>"BooleanUnion",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanComplement",
    ButtonData:>"BooleanComplement",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanComposition",
    ButtonData:>"BooleanComposition",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanConverse",
    ButtonData:>"BooleanConverse",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanPower",
    ButtonData:>"BooleanPower",
    ButtonStyle->"AddOnsLink"],
  StyleBox[". ", "Text"]
}], "Notes",
  CellTags->"BooleanIntersection"],

Cell["Further Examples", "ExampleSection",
  CellTags->"BooleanIntersection"],

Cell["\<\
The intersection of less-than-equal and larger-than-equal is the \
identity. \
\>", "ExampleText",
  CellTags->"BooleanIntersection"],

Cell["\<\
L1 = RelationToMatrix[LessEqual,Range[6]];
L1 // MatrixForm\
\>", "Input",
  CellLabel->"In[87]:=",
  CellTags->"BooleanIntersection"],

Cell["\<\
L2 = RelationToMatrix[GreaterEqual,Range[6]];
L2 // MatrixForm\
\>", "Input",
  CellLabel->"In[89]:=",
  CellTags->"BooleanIntersection"],

Cell["BooleanIntersection[L1,L2] // MatrixForm", "Input",
  CellLabel->"In[91]:=",
  CellTags->"BooleanIntersection"],

Cell["BooleanPower", "ObjectName",
  CellTags->"BooleanPower"],

Cell["Function", "Highlight",
  CellTags->"BooleanPower"],

Cell[TextData[{
  StyleBox["BooleanPower[M,n]", "MR"],
  "  computes the ",
  StyleBox["n", "MR"],
  "-th power of the Boolean matrix ",
  StyleBox["M", "MR"],
  ", corresponding to relational iteration. "
}], "Usage",
  CellTags->"BooleanPower"],

Cell["\<\
Note that for  efficiency reasons Boolean matrices are implemented \
using 0/1-integer matrices. \
\>", "Notes",
  CellTags->"BooleanPower"],

Cell[TextData[{
  "See ",
  ButtonBox["RelationToMatrix",
    ButtonData:>"RelationToMatrix",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanIntersection",
    ButtonData:>"BooleanIntersection",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanComplement",
    ButtonData:>"BooleanComplement",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanComposition",
    ButtonData:>"BooleanComposition",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanConverse",
    ButtonData:>"BooleanConverse",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanUnion",
    ButtonData:>"BooleanUnion",
    ButtonStyle->"AddOnsLink"],
  StyleBox[". ", "Text"]
}], "Notes",
  CellTags->"BooleanPower"],

Cell["Further Examples", "ExampleSection",
  CellTags->"BooleanPower"],

Cell[TextData[{
  "The matrix representing the less-than relation on ",
  Cell[BoxData[
      \(TraditionalForm\`{1, 2, \[Ellipsis], 6}\)]],
  "."
}], "ExampleText",
  CellTags->"BooleanPower"],

Cell["L = RelationToMatrix[Less,Range[6]]", "Input",
  CellLabel->"In[94]:=",
  CellTags->"BooleanPower"],

Cell["\<\
The 5th power of this matrix, only point 1 is related to point 6.\
\
\>", "ExampleText",
  CellTags->"BooleanPower"],

Cell[BoxData[{
    \(\(L5 = BooleanPower[L, 5];\)\), "\n", 
    \(L5\  // \ MatrixForm\)}], "Input",
  CellLabel->"In[95]:=",
  CellTags->"BooleanPower"],

Cell["BooleanUnion", "ObjectName",
  CellTags->"BooleanUnion"],

Cell["Function", "Highlight",
  CellTags->"BooleanUnion"],

Cell[TextData[{
  StyleBox["BooleanUnion[M1,M2]", "MR"],
  "  computes the Boolean sum (pointwise) of two Boolean matrices ",
  StyleBox["M1", "MR"],
  " and ",
  StyleBox["M2", "MR"],
  ", corresponding to relational union."
}], "Usage",
  CellTags->"BooleanUnion"],

Cell["\<\
Note that for  efficiency reasons Boolean matrices are implemented \
using 0/1-integer matrices. \
\>", "Notes",
  CellTags->"BooleanUnion"],

Cell[TextData[{
  "See ",
  ButtonBox["RelationToMatrix",
    ButtonData:>"RelationToMatrix",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanIntersection",
    ButtonData:>"BooleanIntersection",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanComplement",
    ButtonData:>"BooleanComplement",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanComposition",
    ButtonData:>"BooleanComposition",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanConverse",
    ButtonData:>"BooleanConverse",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanPower",
    ButtonData:>"BooleanPower",
    ButtonStyle->"AddOnsLink"],
  StyleBox[". ", "Text"]
}], "Notes",
  CellTags->"BooleanUnion"],

Cell["Further Examples", "ExampleSection",
  CellTags->"BooleanUnion"],

Cell["\<\
The union of less-than and larger-than is the complement of the \
identity. \
\>", "ExampleText",
  CellTags->"BooleanUnion"],

Cell["\<\
L1 = RelationToMatrix[Range[6],Less];
L1 // MatrixForm\
\>", "Input",
  CellLabel->"In[5]:=",
  CellTags->"BooleanUnion"],

Cell["\<\
L2 = RelationToMatrix[Range[6],Greater];
L2 // MatrixForm\
\>", "Input",
  CellLabel->"In[3]:=",
  CellTags->"BooleanUnion"],

Cell["BooleanUnion[L1,L2] // MatrixForm", "Input",
  CellLabel->"In[101]:=",
  CellTags->"BooleanUnion"],

Cell["Boolean", "ObjectName",
  CellTags->"Boolean"],

Cell["Option", "Highlight",
  CellTags->"Boolean"],

Cell[TextData[{
  StyleBox["Boolean", "MR"],
  "  is an option for ",
  StyleBox["TransitionMatrixFA", "MR"],
  ", causes the constructed homomorphism to take values in the semiring of \
Boolean matrices. "
}], "Usage",
  CellTags->"Boolean"],

Cell["\<\
Note that for  efficiency reasons Boolean matrices are implemented \
using 0/1-integer matrices. \
\>", "Notes",
  CellTags->"Boolean"],

Cell[TextData[{
  "See also ",
  ButtonBox["TransitionMatrixFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["FullTransitionMatrixFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Boolean"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Boolean"],

Cell["A nondeterministic machine", "ExampleText",
  CellTags->"Boolean"],

Cell[BoxData[
    \(M = IthSymbolFA[a, \(-4\)]\)], "Input",
  CellLabel->"In[42]:=",
  CellTags->"Boolean"],

Cell[BoxData[
    \(TransitionMatrixFA[M, f, Boolean \[Rule] True]\)], "Input",
  CellLabel->"In[43]:=",
  CellTags->"Boolean"],

Cell["f[a] // MatrixForm", "Input",
  CellLabel->"In[48]:=",
  CellTags->"Boolean"],

Cell["f[aaaa] // MatrixForm", "Input",
  CellLabel->"In[52]:=",
  CellTags->"Boolean"],

Cell["RelationToMatrix", "ObjectName",
  CellTags->"RelationToMatrix"],

Cell["Function", "Highlight",
  CellTags->"RelationToMatrix"],

Cell[TextData[{
  StyleBox["RelationToMatrix[A,rel]", "MR"],
  "  converts a binary relation ",
  StyleBox["rel", "MR"],
  " on carrier set ",
  StyleBox["A ", "MR"],
  "into a square Boolean matrix."
}], "Usage",
  CellTags->"RelationToMatrix"],

Cell[TextData[{
  "The relation must be given by a binary Boolean function over the ground \
set ",
  Cell[BoxData[
      \(TraditionalForm\`A\)]],
  ". "
}], "Notes",
  CellTags->"RelationToMatrix"],

Cell["For efficiency reasons, the output is a 0/1-integer matrix.", "Notes",
  CellTags->"RelationToMatrix"],

Cell[TextData[{
  "See also  ",
  ButtonBox["BooleanComplement",
    ButtonData:>"BooleanComplement",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["BooleanComposition",
    ButtonData:>"BooleanComposition",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["BooleanConverse",
    ButtonData:>"BooleanConverse",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["BooleanIntersection",
    ButtonData:>"BooleanIntersection",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["BooleanPower",
    ButtonData:>"BooleanPower",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["BooleanUnion",
    ButtonData:>"BooleanUnion",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["Boolean",
    ButtonData:>"Boolean",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"RelationToMatrix"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RelationToMatrix"],

Cell["Converting to a Boolean matrix.", "ExampleText",
  CellTags->"RelationToMatrix"],

Cell["RelationToMatrix[ Range[8], Less ] // MatrixForm", "Input",
  CellLabel->"In[56]:=",
  CellTags->"RelationToMatrix"],

Cell["ToClasses", "ObjectName",
  CellTags->"ToClasses"],

Cell["Function", "Highlight",
  CellTags->"ToClasses"],

Cell[TextData[{
  StyleBox["ToClasses[L,R]", "MR"],
  "  generates the equivalence classes of list ",
  StyleBox["L", "MR"],
  " with respect to equivalence relation ",
  StyleBox["R", "MR"],
  ". "
}], "Usage",
  CellTags->"ToClasses"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Type", "\[Rule]", "Relation", \(Matrix, Function, Index\)}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ToClasses"],

Cell[TextData[{
  "The relation ",
  StyleBox["R", "MR"],
  " can be given as a binary relation, a Boolean function (option ",
  StyleBox["Type->Function", "MR"],
  "), a binary matrix (option ",
  StyleBox["Type->Matrix", "MR"],
  "), or an index (option ",
  StyleBox["Type->Index", "MR"],
  "). "
}], "Notes",
  CellTags->"ToClasses"],

Cell[TextData[{
  "In the last case, ",
  StyleBox["R", "MR"],
  " must be a list of integers of length ",
  StyleBox["Length[L]", "MR"],
  "."
}], "Notes",
  CellTags->"ToClasses"],

Cell[TextData[{
  "See ",
  ButtonBox["MakeIndex",
    ButtonData:>"MakeIndex",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["RelationToMatrix",
    ButtonData:>"RelationToMatrix",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToClasses"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToClasses"],

Cell["\<\
An equivalence relation represented by a Boolean-valued function. \
\
\>", "ExampleText",
  CellTags->"ToClasses"],

Cell[BoxData[
    \(\(\[Rho][x_, y_]\  := \ Mod[x, 5] \[Equal] Mod[y, 5];\)\)], "Input",
  CellLabel->"In[134]:=",
  CellTags->"ToClasses"],

Cell[BoxData[
    \(ToClasses[\ Range[12], \ \[Rho]\ ]\)], "Input",
  CellLabel->"In[135]:=",
  CellTags->"ToClasses"],

Cell["Here is the Boolean matrix representation.", "ExampleText",
  CellTags->"ToClasses"],

Cell[BoxData[
    \(M\  = \ RelationToMatrix[Range[12], \[Rho]]\)], "Input",
  CellLabel->"In[137]:=",
  CellTags->"ToClasses"],

Cell[BoxData[
    \(ToClasses[\ Range[12], \ M, \ Type \[Rule] Matrix\ ]\)], "Input",
  CellLabel->"In[138]:=",
  CellTags->"ToClasses"],

Cell["The function representation.", "ExampleText",
  CellTags->"ToClasses"],

Cell[BoxData[
    \(\(f[x_]\  := \ Mod[x, 5];\)\)], "Input",
  CellLabel->"In[139]:=",
  CellTags->"ToClasses"],

Cell[BoxData[
    \(ToClasses[\ Range[12], \ f, \ Type \[Rule] Function\ ]\)], "Input",
  CellLabel->"In[140]:=",
  CellTags->"ToClasses"],

Cell["Lastly, the index.", "ExampleText",
  CellTags->"ToClasses"],

Cell[BoxData[
    \(ind\  = \ \(MakeIndex[Range[12], \[Rho]]\)[\([1]\)]\)], "Input",
  CellLabel->"In[144]:=",
  CellTags->"ToClasses"],

Cell[BoxData[
    \(ToClasses[\ Range[12], \ ind, \ Type \[Rule] Index\ ]\)], "Input",
  CellLabel->"In[145]:=",
  CellTags->"ToClasses"],

Cell["TransitiveClosure", "ObjectName",
  CellTags->"TransitiveClosure"],

Cell["Function", "Highlight",
  CellTags->"TransitiveClosure"],

Cell[TextData[{
  StyleBox["TransitiveClosure[B]", "MR"],
  "  determines the transitive closure of Boolean matrix ",
  StyleBox["B", "MR"],
  ". "
}], "Usage",
  CellTags->"TransitiveClosure"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Reflexive", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"TransitiveClosure"],

Cell[TextData[{
  "Option ",
  StyleBox["Reflexive->True", "MR"],
  " causes the transitive reflexive closure (rather than the transitive \
closure) to be computed."
}], "Notes",
  CellTags->"TransitiveClosure"],

Cell[TextData[{
  "Over the semiring of Boolean matrices, the output is   ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalSigma]\_\(\(\ \)\(i\  \[GreaterEqual] \ 1\)\
\)\ B\^\(\(\ \)\(i\)\)\)]],
  "  and  ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalSigma]\_\(\(\ \)\(i\  \[GreaterEqual] \ 0\)\
\)\ B\^\(\(\ \)\(i\)\)\)]],
  " for the reflexive transitive closure."
}], "Notes",
  CellTags->"TransitiveClosure"],

Cell[TextData[{
  "The implementation uses ",
  ButtonBox["FixedPoint",
    ButtonStyle->"RefGuideLink"],
  " on the operations ",
  Cell[BoxData[
      \(TraditionalForm\`X\ \  \[RightTeeArrow] \ 
        X\  + \ X\ \[CenterDot]\ X\)]],
  "  starting at ",
  Cell[BoxData[
      \(TraditionalForm\`B\)]],
  "  and  ",
  Cell[BoxData[
      \(TraditionalForm\`X\ \  \[RightTeeArrow] \ X\ \[CenterDot]\ X\)]],
  "  starting at ",
  Cell[BoxData[
      \(TraditionalForm\`B\  + \ I\)]],
  " in the reflexive case.  "
}], "Notes",
  CellTags->"TransitiveClosure"],

Cell["For efficiency reasons, all matrices are  0/1-integer.", "Notes",
  CellTags->"TransitiveClosure"],

Cell[TextData[{
  "See also ",
  ButtonBox["Closure",
    ButtonData:>"Closure",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ClosureSearch",
    ButtonData:>"ClosureSearch",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["BooleanPower",
    ButtonData:>"BooleanPower",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"TransitiveClosure"],

Cell["Further Examples", "ExampleSection",
  CellTags->"TransitiveClosure"],

Cell["A circulant matrix and its closure.", "ExampleText",
  CellTags->"TransitiveClosure"],

Cell[BoxData[
    \(\(M = {{0, 0, 1, 0, 1, 0, 0, 0}, {0, 0, 0, 1, 0, 1, 0, 0}, {0, 0, 0, 0, 
            1, 0, 1, 0}, {0, 0, 0, 0, 0, 1, 0, 1}, {1, 0, 0, 0, 0, 0, 1, 
            0}, {0, 1, 0, 0, 0, 0, 0, 1}, {1, 0, 1, 0, 0, 0, 0, 0}, {0, 1, 0, 
            1, 0, 0, 0, 0}};\)\)], "Input",
  CellLabel->"In[60]:=",
  CellTags->"TransitiveClosure"],

Cell[BoxData[
    \(M\  // \ MatrixForm\)], "Input",
  CellLabel->"In[61]:=",
  CellTags->"TransitiveClosure"],

Cell[BoxData[
    \(TransitiveClosure[M] // \ MatrixForm\)], "Input",
  CellLabel->"In[62]:=",
  CellTags->"TransitiveClosure"],

Cell["Warshall", "ObjectName",
  CellTags->"Warshall"],

Cell["Function", "Highlight",
  CellTags->"Warshall"],

Cell[TextData[{
  StyleBox["Warshall[B]", "MR"],
  "  computes the transitive closure of Boolean matrix ",
  StyleBox["B", "MR"],
  " using Warshall's algorithm. "
}], "Usage",
  CellTags->"Warshall"],

Cell[TextData[{
  "Options\n",
  Cell[BoxData[GridBox[{
          {"Reflexive", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]],
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"Warshall"],

Cell["\<\
For efficiency reasons, the matrix is required to be a 0/1-integer \
matrix. \
\>", "Notes",
  CellTags->"Warshall"],

Cell[TextData[{
  "Option ",
  StyleBox["Reflexive->True", "MR"],
  " causes the tranisitive reflexive closure to be computed."
}], "Notes",
  CellTags->"Warshall"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  " causes the tranisitive reduction to be computed alongside with the \
transitive closure.  This requires the input matrix to be acyclic, does not \
check."
}], "Notes",
  CellTags->"Warshall"],

Cell["\<\
The algorithm is cubic, so it cannot be used for very large \
matrices.\
\>", "Notes",
  CellTags->"Warshall"],

Cell[TextData[{
  "See also ",
  ButtonBox["TransitiveClosure",
    ButtonData:>"TransitiveClosure",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Closure",
    ButtonData:>"Closure",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Warshall"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Warshall"],

Cell[TextData[{
  "The successor relation on ",
  Cell[BoxData[
      \(TraditionalForm\`\([6]\)\)]],
  ", and its transitive closure."
}], "ExampleText",
  CellTags->"Warshall"],

Cell["\<\
M = RelationToMatrix[ #1+1==#2&, Range[6] ];
% // MatrixForm\
\>", "Input",
  CellLabel->"In[197]:=",
  CellTags->"Warshall"],

Cell[BoxData[
    \(Warshall[M]\  // MatrixForm\)], "Input",
  CellLabel->"In[199]:=",
  CellTags->"Warshall"],

Cell[BoxData[
    \(Warshall[M, Reflexive \[Rule] True\ ]\  // MatrixForm\)], "Input",
  CellLabel->"In[200]:=",
  CellTags->"Warshall"],

Cell["\<\
The transitive reduction of a divisor lattice.  Matrix M represents \
the divisor relation.\
\>", "ExampleText",
  CellTags->"Warshall"],

Cell[BoxData[{
    \(\(d = Divisors[2\^4\ 3\^3\ 7\^3];\)\), "\n", 
    \(\(n = Length[d];\)\), "\n", 
    \(\(M = 
        Partition[\((If[
                  Mod[#\[LeftDoubleBracket]2\[RightDoubleBracket], #\
\[LeftDoubleBracket]1\[RightDoubleBracket]] == 0, 1, 0] &)\) /@ 
            CartesianProduct[d, d], n];\)\), "\n", 
    \(\({A, B} = Warshall[M, Full \[Rule] True];\)\), "\n", 
    \(\(PlotMatrix[B];\)\)}], "Input",
  CellLabel->"In[88]:=",
  CellTags->"Warshall"],

Cell["\<\
The transitive closure of the reduction is the other output \
component.\
\>", "ExampleText",
  CellTags->"Warshall"],

Cell[BoxData[
    \(A == Warshall[B]\)], "Input",
  CellLabel->"In[93]:=",
  CellTags->"Warshall"],

Cell["AnalyzeOrbit", "ObjectName",
  CellTags->"AnalyzeOrbit"],

Cell["Function", "Highlight",
  CellTags->"AnalyzeOrbit"],

Cell[TextData[{
  StyleBox["AnalyzeOrbit[f,x]", "MR"],
  "  returns ",
  StyleBox["{t,p}", "MR"],
  " where ",
  StyleBox["t", "MR"],
  " is the length of the transient part of the orbit of point ",
  StyleBox["x", "MR"],
  " under map ",
  StyleBox["f", "MR"],
  ", and ",
  StyleBox["p", "MR"],
  " is the length of the cyclic part.  Option  b is the entry point of the \
transient part on the cycle. Computation is memoryless. "
}], "Usage",
  CellTags->"AnalyzeOrbit"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Cycle", "\[Rule]", "None", \(False, True\)},
          {"Periodic", "\[Rule]", "False", "True"},
          {"BoundAO", "\[Rule]", "1000", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"AnalyzeOrbit"],

Cell[TextData[{
  "Option ",
  StyleBox["Cycle->True", "MR"],
  " causes the limit cycle to be displayed, starting with the entry point, ",
  StyleBox["Cycle->False", "MR"],
  " displays only the entry point."
}], "Notes",
  CellTags->"AnalyzeOrbit"],

Cell[TextData[{
  "Option ",
  StyleBox["Periodic->True", "MR"],
  " does not affect the output, but improves speed for periodic orbits.  Note \
that no test is performed as to whether the orbit is indeed periodic."
}], "Notes",
  CellTags->"AnalyzeOrbit"],

Cell[TextData[{
  "Option ",
  StyleBox["BoundAO->n", "MR"],
  "  truncates the search for the anchor point on the limit cycle after ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " steps."
}], "Notes",
  CellTags->"AnalyzeOrbit"],

Cell["\<\
The computation is memoryless using Floyd's trick, so the only \
constraint is time. \
\>", "Notes",
  CellTags->"AnalyzeOrbit"],

Cell[TextData[{
  "If ",
  StyleBox["ff[x] = f[f[x]]", "MR"],
  " is slow to compute by composition, ",
  StyleBox["ff", "MR"],
  " can be supplied as a third argument: ",
  StyleBox["AnalyzeOrbit[f,a,ff]", "MR"],
  ". "
}], "Notes",
  CellTags->"AnalyzeOrbit"],

Cell[TextData[{
  "See also ",
  ButtonBox["OrbitCA",
    ButtonData:>"OrbitCA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["CA",
    ButtonData:>"CA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"AnalyzeOrbit"],

Cell["Further Examples", "ExampleSection",
  CellTags->"AnalyzeOrbit"],

Cell["A simple map on modular numbers. ", "ExampleText",
  CellTags->"AnalyzeOrbit"],

Cell["\<\
ClearAll[f]
f[x_] := Mod[x^2,11]\
\>", "Input",
  CellLabel->"In[21]:=",
  CellTags->"AnalyzeOrbit"],

Cell[BoxData[
    \(AnalyzeOrbit[f, 1]\)], "Input",
  CellLabel->"In[23]:=",
  CellTags->"AnalyzeOrbit"],

Cell[BoxData[
    \(AnalyzeOrbit[f, 3]\)], "Input",
  CellLabel->"In[24]:=",
  CellTags->"AnalyzeOrbit"],

Cell[BoxData[
    \(AnalyzeOrbit[f, 3, Cycle \[Rule] True]\)], "Input",
  CellLabel->"In[25]:=",
  CellTags->"AnalyzeOrbit"],

Cell[BoxData[
    \(AnalyzeOrbit[f, 3, Cycle \[Rule] False]\)], "Input",
  CellLabel->"In[26]:=",
  CellTags->"AnalyzeOrbit"],

Cell["\<\
The memoryless approach works well for larger objects such as \
configurations of cellular automata.\
\>", "ExampleText",
  CellTags->"AnalyzeOrbit"],

Cell[BoxData[{
    \(\(X = \ SeedConfiguration[\ {1}, 19];\)\), "\n", 
    \(GlobalMapCA[f, CA[90]]\), "\n", 
    \(AnalyzeOrbit[\ f, \ X\ ]\)}], "Input",
  CellLabel->"In[27]:=",
  CellTags->"AnalyzeOrbit"],

Cell[BoxData[
    \(\(EvolutionCA[CA[90], X, 512, MultiCol \[Rule] True, 
        NRows \[Rule] 100];\)\)], "Input",
  CellLabel->"In[30]:=",
  CellTags->"AnalyzeOrbit"],

Cell[TextData[{
  StyleBox["AnalyzeOrbit", "MR"],
  " automatically truncates searches that fail to find the anchor point on \
the limit cycle in less than 5000 steps."
}], "ExampleText",
  CellTags->"AnalyzeOrbit"],

Cell[BoxData[
    \(AnalyzeOrbit[# + 1 &, 0]\)], "Input",
  CellLabel->"In[31]:=",
  CellTags->"AnalyzeOrbit"],

Cell[TextData[{
  "You may have to adjust the option ",
  StyleBox["BoundAO", "MR"],
  " accordingly."
}], "ExampleText",
  CellTags->"AnalyzeOrbit"],

Cell[BoxData[{
    \(Clear[f]\), "\n", 
    \(\(f[6000]\  = \ 6000;\)\), "\n", 
    \(\(f[x_]\  := \ x + \ 1;\)\), "\n", 
    \(AnalyzeOrbit[\ f, \ 0, BoundAO \[Rule] \ 10^4]\)}], "Input",
  CellLabel->"In[37]:=",
  CellTags->"AnalyzeOrbit"],

Cell["BoundAO", "ObjectName",
  CellTags->"BoundAO"],

Cell["Option", "Highlight",
  CellTags->"BoundAO"],

Cell[TextData[{
  StyleBox["BoundAO", "MR"],
  "  is an option for ",
  StyleBox["AnalyzeOrbit", "MR"],
  " that causes the search to be truncated."
}], "Usage",
  CellTags->"BoundAO"],

Cell["Default value is 5000.", "Notes",
  CellTags->"BoundAO"],

Cell[TextData[{
  "See ",
  ButtonBox["AnalyzeOrbit",
    ButtonData:>"AnalyzeOrbit",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"BoundAO"],

Cell["CartesianProduct", "ObjectName",
  CellTags->"CartesianProduct"],

Cell["Function", "Highlight",
  CellTags->"CartesianProduct"],

Cell[TextData[{
  StyleBox["CartesianProduct[A,B]", "MR"],
  "  computes the Cartesian product of lists ",
  StyleBox["A", "MR"],
  " and ",
  StyleBox["B", "MR"],
  "."
}], "Usage",
  CellTags->"CartesianProduct"],

Cell[TextData[{
  StyleBox["CartesianProduct", "MR"],
  " is polyadic."
}], "Notes",
  CellTags->"CartesianProduct"],

Cell[TextData[{
  StyleBox["CartesianProduct", "MR"],
  " returns a flat list of pairs (",
  Cell[BoxData[
      FormBox[
        RowBox[{"k", Cell[""]}], TraditionalForm]]],
  "-tuples)."
}], "Notes",
  CellTags->"CartesianProduct"],

Cell[TextData[StyleBox["The length of the output list is the product of the \
lengths of all the input lists.", "Text"]], "Notes",
  CellTags->"CartesianProduct"],

Cell[TextData[{
  StyleBox["See also ", "Text"],
  ButtonBox["Tuples",
    ButtonStyle->"AddOnsLink"],
  StyleBox[",  ", "Text"],
  ButtonBox["Pairs",
    ButtonData:>"Pairs",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["ToPairs",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"CartesianProduct"],

Cell["Further Examples", "ExampleSection",
  CellTags->"CartesianProduct"],

Cell["A simple Cartesian product. ", "ExampleText",
  CellTags->"CartesianProduct"],

Cell["\<\
Clear[a,b,c,d]
CartesianProduct[{1,2,3},{a,b}]\
\>", "Input",
  CellLabel->"In[550]:=",
  CellTags->"CartesianProduct"],

Cell["A product with three terms, producing a list of triples.", "ExampleText",
  CellTags->"CartesianProduct"],

Cell["CartesianProduct[{1,2,3},{a,b},{c,d}]", "Input",
  CellLabel->"In[552]:=",
  CellTags->"CartesianProduct"],

Cell[TextData[{
  StyleBox["CartesianProduct", "MR"],
  " behaves properly with respect to empty lists and empty argument \
sequences."
}], "ExampleText",
  CellTags->"CartesianProduct"],

Cell["\<\
CartesianProduct[{1,2,3},{},{c,d}]
CartesianProduct[]\
\>", "Input",
  CellLabel->"In[553]:=",
  CellTags->"CartesianProduct"],

Cell["$closureBound", "ObjectName",
  CellTags->"$closureBound"],

Cell["SysVariable", "Highlight",
  CellTags->"$closureBound"],

Cell[TextData[{
  StyleBox["$closureBound", "MR"],
  "  is a default bound for the number of steps taken in ",
  StyleBox["Closure.", "MR"]
}], "Usage",
  CellTags->"$closureBound"],

Cell[" Default value is 500 to guard against exponential blow-up.", "Notes",
  CellTags->"$closureBound"],

Cell[TextData[{
  "See ",
  ButtonBox["Closure",
    ButtonData:>"Closure",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"$closureBound"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$closureBound"],

Cell["The system default value is 500:", "ExampleText",
  CellTags->"$closureBound"],

Cell[BoxData[
    \($closureBound\)], "Input",
  CellLabel->"In[78]:=",
  CellTags->"$closureBound"],

Cell["ClosureBound", "ObjectName",
  CellTags->"ClosureBound"],

Cell["Option", "Highlight",
  CellTags->"ClosureBound"],

Cell[TextData[{
  StyleBox["ClosureBound", "MR"],
  "  is an option for ",
  StyleBox["Closure", "MR"],
  " and ",
  StyleBox["ClosureSearch", "MR"],
  "."
}], "Usage",
  CellTags->"ClosureBound"],

Cell[TextData[{
  "See ",
  ButtonBox["Closure",
    ButtonData:>"Closure",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["ClosureSearch",
    ButtonData:>"ClosureSearch",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ClosureBound"],

Cell["$closureSearchBound", "ObjectName",
  CellTags->"$closureSearchBound"],

Cell["SysVariable", "Highlight",
  CellTags->"$closureSearchBound"],

Cell[TextData[{
  StyleBox["$closureSearchBound", "MR"],
  "   is a default bound for ",
  StyleBox["ClosureSearch", "MR"],
  ". "
}], "Usage",
  CellTags->"$closureSearchBound"],

Cell["\<\
Default value is 500 for the size of closure set to be \
computed.\
\>", "Notes",
  CellTags->"$closureSearchBound"],

Cell[TextData[{
  "See ",
  ButtonBox["ClosureSearch",
    ButtonData:>"ClosureSearch",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"$closureSearchBound"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$closureSearchBound"],

Cell["The system default value is 500:", "ExampleText",
  CellTags->"$closureSearchBound"],

Cell[BoxData[
    \($closureSearchBound\)], "Input",
  CellLabel->"In[79]:=",
  CellTags->"$closureSearchBound"],

Cell["ClosureSearch", "ObjectName",
  CellTags->"ClosureSearch"],

Cell["Function", "Highlight",
  CellTags->"ClosureSearch"],

Cell[TextData[{
  StyleBox["ClosureSearch[L,targ,ff]", "MR"],
  "  determines whether ",
  StyleBox["targ", "MR"],
  " occurs in the closure of list ",
  StyleBox["L", "MR"],
  " under the function ",
  StyleBox["ff", "MR"],
  "."
}], "Usage",
  CellTags->"ClosureSearch"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"SetClosure", "\[Rule]", "False", "-"},
          {"ClosureBound", "\[RuleDelayed]", "$closureSearchBound", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ClosureSearch"],

Cell[TextData[{
  "Input ",
  StyleBox["ff", "MR"],
  " can also be a sequence of functions."
}], "Notes",
  CellTags->"ClosureSearch"],

Cell[TextData[{
  "Option  ",
  StyleBox["SetClosure->True", "MR"],
  "  causes the intermediate results to be  properly flattened so that ",
  StyleBox["ff", "MR"],
  " can be of type   ",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau]\ \[LongRightArrow]\ \(list(\[Tau])\)\)]],
  ". "
}], "Notes",
  CellTags->"ClosureSearch"],

Cell["\<\
If the target is found, the search is immediately truncated, so \
that computation of the full closure can sometimes be avoided.\
\>", "Notes",
  CellTags->"ClosureSearch"],

Cell[TextData[{
  "See also ",
  ButtonBox["Closure",
    ButtonData:>"Closure",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ClosureBound",
    ButtonData:>"ClosureBound",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["TransitiveClosure",
    ButtonData:>"TransitiveClosure",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"ClosureSearch"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ClosureSearch"],

Cell["Combining shift with shuffle produces all permutations.", "ExampleText",
  CellTags->"ClosureSearch"],

Cell[BoxData[{
    \(\(shuffle[L_List] := Flatten[Thread[Partition[L, 2]]];\)\), "\n", 
    \(shuffle[Range[8]]\)}], "Input",
  CellLabel->"In[81]:=",
  CellTags->"ClosureSearch"],

Cell[BoxData[
    \(Length[Closure[{Range[6]}, RotateLeft, shuffle]]\)], "Input",
  CellLabel->"In[83]:=",
  CellTags->"ClosureSearch"],

Cell["The bound has to be large enough.", "ExampleText",
  CellTags->"ClosureSearch"],

Cell[BoxData[
    \(ClosureSearch[{Range[6]}, {6, 5, 4, 3, 2, 1}, RotateLeft, shuffle, 
      ClosureBound \[Rule] 1000]\)], "Input",
  CellLabel->"In[84]:=",
  CellTags->"ClosureSearch"],

Cell[BoxData[
    \(ClosureSearch[{Range[6]}, {1, 1, 1, 1, 1, 1}, RotateLeft, shuffle, 
      ClosureBound \[Rule] 1000]\)], "Input",
  CellLabel->"In[361]:=",
  CellTags->"ClosureSearch"],

Cell["Closure", "ObjectName",
  CellTags->"Closure"],

Cell["Function", "Highlight",
  CellTags->"Closure"],

Cell[TextData[{
  StyleBox["Closure[L,ff]", "MR"],
  "  computes the closure of list L under the function ff  "
}], "Usage",
  CellTags->"Closure"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"SetClosure", "\[Rule]", "False", "-"},
          {"ClosureBound", "\[Rule]", "$closureBound", "-"},
          {"Rank", "\[Rule]", "False", "-"},
          {"DoTrace", "\[Rule]", "False", "-"},
          {"Index", "\[Rule]", "1", "-"},
          {"Filter", "\[Rule]", "Identity", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"Closure"],

Cell[TextData[{
  "Input ",
  StyleBox["ff", "MR"],
  " can also be a sequence of functions."
}], "Notes",
  CellTags->"Closure"],

Cell[TextData[{
  "Option  ",
  StyleBox["SetClosure->True", "MR"],
  "  causes the intermediate results to be  properly flattened so that ",
  StyleBox["ff", "MR"],
  " can be of type   ",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau]\ \[LongRightArrow]\ \(list(\[Tau])\)\)]],
  ". "
}], "Notes",
  CellTags->"Closure"],

Cell[TextData[{
  "Option  ",
  StyleBox["ClosureBound->B", "MR"],
  "  can be used to control the maximal size of generated objects. Default \
value is ",
  ButtonBox["$closureBound",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Closure"],

Cell[TextData[{
  "Option  ",
  StyleBox["Rank->True", "MR"],
  "  causes an additional list of the ranks of all generated objects to be \
returned."
}], "Notes",
  CellTags->"Closure"],

Cell[TextData[{
  "Option  ",
  StyleBox["DoTrace->True", "MR"],
  "  causes an additional trace of all generated objects to be returned."
}], "Notes",
  CellTags->"Closure"],

Cell[TextData[{
  "Option  ",
  StyleBox["Index->0", "MR"],
  "  uses 0-indexing for the trace."
}], "Notes",
  CellTags->"Closure"],

Cell[TextData[{
  "See also ",
  ButtonBox["ClosureSearch",
    ButtonData:>"ClosureSearch",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ClosureBound",
    ButtonData:>"ClosureBound",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["TransitiveClosure",
    ButtonData:>"TransitiveClosure",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"Closure"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Closure"],

Cell["All possible cyclic shifts of a list.", "ExampleText",
  CellTags->"Closure"],

Cell["Closure[ { Range[4] }, RotateLeft ]", "Input",
  CellLabel->"In[111]:=",
  CellTags->"Closure"],

Cell["Closure[ {{1,2,1,2}}, RotateLeft ]", "Input",
  CellLabel->"In[112]:=",
  CellTags->"Closure"],

Cell["Combining shift with shuffle produces all permutations.", "ExampleText",
  CellTags->"Closure"],

Cell[BoxData[{
    \(\(shuffle[L_List] := Flatten[Thread[Partition[L, 2]]];\)\), "\n", 
    \(shuffle[Range[8]]\)}], "Input",
  CellLabel->"In[105]:=",
  CellTags->"Closure"],

Cell["Closure[ { Range[4] }, RotateLeft, shuffle ]", "Input",
  CellLabel->"In[107]:=",
  CellTags->"Closure"],

Cell["\<\
6 applications of these operations are necessary to produce all \
permutations.\
\>", "ExampleText",
  CellTags->"Closure"],

Cell[BoxData[{
    \(\({L, rks} = 
        Closure[{Range[4]}, RotateLeft, shuffle, 
          Rank \[Rule] True];\)\), "\n", 
    \(rks\)}], "Input",
  CellLabel->"In[108]:=",
  CellTags->"Closure"],

Cell["\<\
Sometimes the return type of the operation is naturally a list of \
objects. Here is the operation \"follow all edges in a circulant graph\". \
Note that there is only one set-valued function here.\
\>", "ExampleText",
  CellTags->"Closure"],

Cell[BoxData[
    \(Closure[{0}, Mod[{#1 + 2, #1 + 4}, 20] &, 
      SetClosure \[Rule] True]\)], "Input",
  CellLabel->"In[356]:=",
  CellTags->"Closure"],

Cell["\<\
We can keep track of which function has caused a certain element to \
be included in the closure. The example uses two point-valued functions.  \
Needless to say, this is useful for the construction of the accessible parts \
of certain finite state machines.\
\>", "ExampleText",
  CellTags->"Closure"],

Cell[BoxData[
    \(\({S, trc}\  = \ 
        Closure[{0}, {Mod[# + 4, 12] &, \ Mod[# + 3, 12] &}, \ 
          DoTrace \[Rule] True];\)\)], "Input",
  CellLabel->"In[363]:=",
  CellTags->"Closure"],

Cell[BoxData[
    \(S\)], "Input",
  CellLabel->"In[364]:=",
  CellTags->"Closure"],

Cell[BoxData[
    \(trc\)], "Input",
  CellLabel->"In[365]:=",
  CellTags->"Closure"],

Cell["Cycle", "ObjectName",
  CellTags->"Cycle"],

Cell["Option", "Highlight",
  CellTags->"Cycle"],

Cell[TextData[{
  StyleBox["Cycle", "MR"],
  "  is an option for ",
  StyleBox["AnalyzeOrbit", "MR"],
  " that causes the limit cycle to be displayed."
}], "Usage",
  CellTags->"Cycle"],

Cell[TextData[{
  "See ",
  ButtonBox["AnalyzeOrbit",
    ButtonData:>"AnalyzeOrbit",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Cycle"],

Cell["DeleteDuplicates", "ObjectName",
  CellTags->"DeleteDuplicates"],

Cell["Function", "Highlight",
  CellTags->"DeleteDuplicates"],

Cell[TextData[{
  StyleBox["DeleteDuplicates", "MR"],
  StyleBox["[L]", "MR"],
  "  removes duplicates from list ",
  StyleBox["L", "MR"],
  ". "
}], "Usage",
  CellTags->"DeleteDuplicates"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"OutPairs", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"DeleteDuplicates"],

Cell[TextData[{
  "Option ",
  StyleBox["OutPairs->True", "MR"],
  "  generates output in pair form. Only applies to the two-list situation."
}], "Notes",
  CellTags->"DeleteDuplicates"],

Cell[TextData[{
  StyleBox["DeleteDuplicates[L]", "MR"],
  "  does not sort the list but retains the original order of the elements, \
uses ",
  ButtonBox["Reap",
    ButtonStyle->"RefGuideLink"],
  "."
}], "Notes",
  CellTags->"DeleteDuplicates"],

Cell[TextData[{
  StyleBox["DeleteDuplicates", "MR"],
  StyleBox["[L1,L2]", "MR"],
  " removes duplicates from list ",
  StyleBox["L1", "MR"],
  " and corresponding elements from list ",
  StyleBox["L2", "MR"],
  ".  Both lists must have the same length. Here pairs are sorted."
}], "Notes",
  CellTags->"DeleteDuplicates"],

Cell[TextData[{
  "See also ",
  ButtonBox["DClassT",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DeleteDuplicates"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DeleteDuplicates"],

Cell["A list with repetitions and the uniqued list.", "ExampleText",
  CellTags->"DeleteDuplicates"],

Cell["\<\
L = Mod[ Range[10], 4 ]
DeleteDuplicates[L]  \
\>", "Input",
  CellLabel->"In[568]:=",
  CellTags->"DeleteDuplicates"],

Cell["Working on pairs of lists.", "ExampleText",
  CellTags->"DeleteDuplicates"],

Cell[BoxData[
    \(DeleteDuplicates[L, MakeAlphabet[10]]\)], "Input",
  CellLabel->"In[570]:=",
  CellTags->"DeleteDuplicates"],

Cell[BoxData[
    \(DeleteDuplicates[L, MakeAlphabet[10], 
      OutPairs \[Rule] True\ ]\)], "Input",
  CellLabel->"In[572]:=",
  CellTags->"DeleteDuplicates"],

Cell["DisjointQ", "ObjectName",
  CellTags->"DisjointQ"],

Cell["Function", "Highlight",
  CellTags->"DisjointQ"],

Cell[TextData[{
  StyleBox["DisjointQ[L1,L2]", "MR"],
  "  tests whether lists ",
  StyleBox["L1", "MR"],
  " and ",
  StyleBox["L2", "MR"],
  " are disjoint."
}], "Usage",
  CellTags->"DisjointQ"],

Cell[TextData[{
  "See also ",
  ButtonBox["SubsetQ",
    ButtonData:>"SubsetQ",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DisjointQ"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DisjointQ"],

Cell["Only top level elments are considered.", "ExampleText",
  CellTags->"DisjointQ"],

Cell[" DisjointQ[{1,2,3},{{1},4,5}]", "Input",
  CellLabel->"In[90]:=",
  CellTags->"DisjointQ"],

Cell[" DisjointQ[{1,2,3},{1,4,5}]", "Input",
  CellLabel->"In[87]:=",
  CellTags->"DisjointQ"],

Cell["EnumForm", "ObjectName",
  CellTags->"EnumForm"],

Cell["Function", "Highlight",
  CellTags->"EnumForm"],

Cell[TextData[{
  StyleBox["EnumForm[L]", "MR"],
  "  displays list ",
  StyleBox["L", "MR"],
  " in ",
  StyleBox["ColumnForm", "MR"],
  " with line numbers starting at 1."
}], "Usage",
  CellTags->"EnumForm"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Index", "\[Rule]", "1", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"EnumForm"],

Cell[TextData[{
  "Option ",
  StyleBox["Index->n", "MR"],
  "  changes the starting value to ",
  StyleBox["n", "MR"],
  "."
}], "Notes",
  CellTags->"EnumForm"],

Cell[TextData[{
  "Uses ",
  StyleBox["ColonForm", "MR"],
  " to generate the line numbers. "
}], "Notes",
  CellTags->"EnumForm"],

Cell[TextData[{
  "See ",
  ButtonBox["EnumForm0",
    ButtonData:>"EnumForm0",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"],
  " "
}], "Notes",
  CellTags->"EnumForm"],

Cell["Further Examples", "ExampleSection",
  CellTags->"EnumForm"],

Cell["Displaying a few words in a acceptance language.", "ExampleText",
  CellTags->"EnumForm"],

Cell[BoxData[{
    \(\(m = CounterDFA[a, 1, 2, Modulus \[Rule] 3];\)\), "\n", 
    \(L = LanguageFA[m, \(-5\)]\)}], "Input",
  CellLabel->"In[221]:=",
  CellTags->"EnumForm"],

Cell[BoxData[
    \(EnumForm[L]\)], "Input",
  CellLabel->"In[223]:=",
  CellTags->"EnumForm"],

Cell["It is more natural to start numbering at 0 in this case. ", \
"ExampleText",
  CellTags->"EnumForm"],

Cell[BoxData[
    \(EnumForm0[L]\)], "Input",
  CellLabel->"In[224]:=",
  CellTags->"EnumForm"],

Cell["External", "ObjectName",
  CellTags->"External"],

Cell["Option", "Highlight",
  CellTags->"External"],

Cell[TextData[{
  StyleBox["External", "MR"],
  "  is an option for various commands with an external implementation. "
}], "Usage",
  CellTags->"External"],

Cell[TextData[{
  "Option  ",
  StyleBox["External->True", "MR"],
  "  causes the external code to be uses, assuming that the appropriate ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  "  has been established. "
}], "Notes",
  CellTags->"External"],

Cell[TextData[{
  "Has default value ",
  StyleBox["$defaultexternal", "MR"],
  "."
}], "Notes",
  CellTags->"External"],

Cell["\<\
The internal and external algorithms produce essentially the same \
output.\
\>", "Notes",
  CellTags->"External"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToDFA",
    ButtonData:>"ToDFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["MinimizeFA",
    ButtonData:>"MinimizeFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["EvolutionCA",
    ButtonData:>"EvolutionCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"External"],

Cell["FastPowerSet", "ObjectName",
  CellTags->"FastPowerSet"],

Cell["Function", "Highlight",
  CellTags->"FastPowerSet"],

Cell[TextData[{
  StyleBox["FastPowerSet[L]", "MR"],
  "   uses pattern matching to generate a power set."
}], "Usage",
  CellTags->"FastPowerSet"],

Cell[TextData[{
  "Is currently faster than the inductive method used in ",
  StyleBox["PowerSet", "MR"],
  "."
}], "Notes",
  CellTags->"FastPowerSet"],

Cell[TextData[{
  "See ",
  ButtonBox["PowerSet",
    ButtonData:>"PowerSet",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"FastPowerSet"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FastPowerSet"],

Cell["\<\
Note that the methods produce the subsets in different order.\
\>", \
"ExampleText",
  CellTags->"FastPowerSet"],

Cell[BoxData[
    \(Timing[\(P1 = FastPowerSet[Range[12]];\)]\)], "Input",
  CellLabel->"In[728]:=",
  CellTags->"FastPowerSet"],

Cell[BoxData[
    \(Timing[\(P2 = PowerSet[Range[12]];\)]\)], "Input",
  CellLabel->"In[729]:=",
  CellTags->"FastPowerSet"],

Cell[BoxData[{
    \(P1 === P2\), "\n", 
    \(Sort[P1] === P2\)}], "Input",
  CellLabel->"In[730]:=",
  CellTags->"FastPowerSet"],

Cell["Filter", "ObjectName",
  CellTags->"Filter"],

Cell["Option", "Highlight",
  CellTags->"Filter"],

Cell[TextData[{
  StyleBox["Filter", "MR"],
  " is an option that can be used in GenerateSG to further process the \
objects generated by semigroup multiplications. "
}], "Usage",
  CellTags->"Filter"],

Cell["Useful to enforce modular arithmetic.", "Notes",
  CellTags->"Filter"],

Cell["\<\
Note that this option is only available for the internal algorithm.\
\
\>", "Notes",
  CellTags->"Filter"],

Cell[TextData[{
  "See also ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Filter"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Filter"],

Cell["Constructing a semigroup of 0/1-matrices with 4 generators.", \
"ExampleText",
  CellTags->"Filter"],

Cell[BoxData[{
    \(\(r\  = \ {{0, 1, 0}, {0, 0, 1}, {1, 0, 0}};\)\), "\n", 
    \(\(t\  = \ {{0, 1, 0}, {1, 0, 0}, {0, 0, 1}};\)\), "\n", 
    \(\(m\  = \ {{1, 0, 0}, {1, 0, 0}, {0, 0, 1}};\)\)}], "Input",
  CellLabel->"In[104]:=",
  CellTags->"Filter"],

Cell[BoxData[
    \(\(g\  = \ GEN[\ 3, 3, \ {r, t, m}\ ];\)\)], "Input",
  CellLabel->"In[107]:=",
  CellTags->"Filter"],

Cell[BoxData[
    \(\(S\  = \ 
        GenerateSG[\ g, \ \[IndentingNewLine]\tMatrix \[Rule] True, \ 
          Unit \[Rule] IdentityMatrix, \ Filter \[Rule] Sign\ ];\)\)], "Input",\

  CellLabel->"In[108]:=",
  CellTags->"Filter"],

Cell[BoxData[
    \(MatrixForm\  /@ \ S\)], "Input",
  CellLabel->"In[109]:=",
  CellTags->"Filter"],

Cell["Frequencies", "ObjectName",
  CellTags->"Frequencies"],

Cell["Function", "Highlight",
  CellTags->"Frequencies"],

Cell[TextData[{
  StyleBox["Frequencies[L]", "MR"],
  "  counts the occurrences of all elements in the list ",
  StyleBox["L", "MR"],
  "."
}], "Usage",
  CellTags->"Frequencies"],

Cell[TextData[{
  "Returns a list of pairs  ",
  Cell[BoxData[
      \(TraditionalForm\`\((\ object, \ count\ )\)\)]],
  ". "
}], "Notes",
  CellTags->"Frequencies"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Frequencies"],

Cell["Frequencies of numbers in a random sequence.", "ExampleText",
  CellTags->"Frequencies"],

Cell[BoxData[{
    \(\(L = Table[Random[Integer, {1, 5}], {500}];\)\), "\n", 
    \(Frequencies[L]\), "\n", 
    \(%\  // \ TableForm\)}], "Input",
  CellLabel->"In[58]:=",
  CellTags->"Frequencies"],

Cell["FromBitVector", "ObjectName",
  CellTags->"FromBitVector"],

Cell["Function", "Highlight",
  CellTags->"FromBitVector"],

Cell[TextData[{
  StyleBox["FromBitVector[B,U]", "MR"],
  "  transforms bitvector ",
  StyleBox["B", "MR"],
  " into a subset of universe ",
  StyleBox["U", "MR"],
  ". "
}], "Usage",
  CellTags->"FromBitVector"],

Cell["A bitvector is understood to be a 0/1-integer list.", "Notes",
  CellTags->"FromBitVector"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToBitVector",
    ButtonData:>"ToBitVector",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"FromBitVector"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FromBitVector"],

Cell["Conversion from a bitvector to an actual subset.", "ExampleText",
  CellTags->"FromBitVector"],

Cell["all = Tuples[3,{0,1}]", "Input",
  CellLabel->"In[103]:=",
  CellTags->"FromBitVector"],

Cell[BoxData[
    \(\(FromBitVector[#, {a, b, c}] &\)\  /@ \ all\)], "Input",
  CellLabel->"In[104]:=",
  CellTags->"FromBitVector"],

Cell["Index", "ObjectName",
  CellTags->"Index"],

Cell["Option", "Highlight",
  CellTags->"Index"],

Cell[TextData[{
  StyleBox["Index", "MR"],
  "  is a possible value for option ",
  StyleBox["Type", "MR"],
  " in function ",
  StyleBox["ToClasses", "MR"],
  "."
}], "Usage",
  CellTags->"Index"],

Cell[TextData[{
  "See ",
  ButtonBox["ToClasses",
    ButtonData:>"ToClasses",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Type",
    ButtonData:>"Type",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Index"],

Cell["Kary", "ObjectName",
  CellTags->"Kary"],

Cell["Option", "Highlight",
  CellTags->"Kary"],

Cell[TextData[{
  StyleBox["Kary", "MR"],
  "  is a Boolean option for ",
  StyleBox["AssignFunction", "MR"],
  ", causes the function to be treated as a ",
  StyleBox["k", "MR"],
  "-ary function."
}], "Usage",
  CellTags->"Kary"],

Cell[TextData[{
  "See ",
  ButtonBox["AssignFunction",
    ButtonData:>"AssignFunction",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Kary"],

Cell["LongestCommonSubsequence", "ObjectName",
  CellTags->"LongestCommonSubsequence"],

Cell["Function", "Highlight",
  CellTags->"LongestCommonSubsequence"],

Cell[TextData[{
  StyleBox["LongestCommonSubsequence[A,B]", "MR"],
  "  produces a longest common subsequence of the two lists ",
  StyleBox["A", "MR"],
  " and ",
  StyleBox["B", "MR"],
  "."
}], "Usage",
  CellTags->"LongestCommonSubsequence"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "True", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"LongestCommonSubsequence"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->False", "MR"],
  "  returns only the length of a LCS, not a witness."
}], "Notes",
  CellTags->"LongestCommonSubsequence"],

Cell["Uses the standard quadratic dynamic programming approach.", "Notes",
  CellTags->"LongestCommonSubsequence"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordBinomial",
    ButtonData:>"WordBinomial",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"LongestCommonSubsequence"],

Cell["Further Examples", "ExampleSection",
  CellTags->"LongestCommonSubsequence"],

Cell["A longest common subsequence.", "ExampleText",
  CellTags->"LongestCommonSubsequence"],

Cell[BoxData[{
    \(LongestCommonSubsequence[\ {0, 2, 3, 2}, {0, 1, 1, 3, \ 2, \ 3, 1, 0}, 
      Full \[Rule] False]\), "\n", 
    \(LongestCommonSubsequence[\ {0, 2, 3, 2}, {0, 1, 1, 3, \ 2, \ 3, 1, 
        0}]\)}], "Input",
  CellLabel->"In[746]:=",
  CellTags->"LongestCommonSubsequence"],

Cell["\<\
The LCS algorithm can be used to check for a longest increasing \
subsequence.\
\>", "ExampleText",
  CellTags->"LongestCommonSubsequence"],

Cell[BoxData[{
    \(Table[\ RandomInteger[20], {30}]\), "\n", 
    \(LongestCommonSubsequence[\ Union[%], %]\)}], "Input",
  CellLabel->"In[736]:=",
  CellTags->"LongestCommonSubsequence"],

Cell["MakeIndex", "ObjectName",
  CellTags->"MakeIndex"],

Cell["Function", "Highlight",
  CellTags->"MakeIndex"],

Cell[TextData[{
  StyleBox["MakeIndex[L,R]", "MR"],
  "  computes an index of list ",
  StyleBox["L", "MR"],
  " with respect to equivalence relation ",
  StyleBox["R", "MR"],
  ". "
}], "Usage",
  CellTags->"MakeIndex"],

Cell[TextData[{
  StyleBox["R", "MR"],
  " has default ",
  StyleBox["SameQ", "MR"],
  ". "
}], "Notes",
  CellTags->"MakeIndex"],

Cell[TextData[StyleBox["Returns the index and the number of classes.", \
"Text"]], "Notes",
  CellTags->"MakeIndex"],

Cell[TextData[{
  "The index is the canonical kernel function that maps ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " to  ",
  Cell[BoxData[
      \(TraditionalForm\`min(\ y\  | \ x\ R\ y\ )\)]],
  ".  As a function ",
  Cell[BoxData[
      \(TraditionalForm\`\([n]\)\[LongRightArrow]\([n]\)\)]],
  ", the index is therefore idempotent and deflationary."
}], "Notes",
  CellTags->"MakeIndex"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToClasses",
    ButtonData:>"ToClasses",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"MakeIndex"],

Cell["Further Examples", "ExampleSection",
  CellTags->"MakeIndex"],

Cell["A congruence relation with four classes.", "ExampleText",
  CellTags->"MakeIndex"],

Cell[BoxData[
    \(MakeIndex[Range[0, 11], Mod[#1, 4] == Mod[#2, 4] &]\)], "Input",
  CellLabel->"In[727]:=",
  CellTags->"MakeIndex"],

Cell["Matrix", "ObjectName",
  CellTags->"Matrix"],

Cell["Option", "Highlight",
  CellTags->"Matrix"],

Cell[TextData[{
  StyleBox["Matrix", "MR"],
  "  is a possible value for option ",
  StyleBox["Type", "MR"],
  " in function ",
  StyleBox["ToClasses", "MR"],
  "."
}], "Usage",
  CellTags->"Matrix"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToClasses",
    ButtonData:>"ToClasses",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Matrix"],

Cell["NonDecreasingSequences", "ObjectName",
  CellTags->"NonDecreasingSequences"],

Cell["Function", "Highlight",
  CellTags->"NonDecreasingSequences"],

Cell[TextData[{
  StyleBox["NonDecreasingSequences[n,k]", "MR"],
  "  generates all non-decreasing sequences of length ",
  StyleBox["k", "MR"],
  " over ",
  StyleBox["[n]", "MR"],
  "."
}], "Usage",
  CellTags->"NonDecreasingSequences"],

Cell[TextData[{
  "Also accepts lists as input in place of ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  ".  The order is then given by the original placement."
}], "Notes",
  CellTags->"NonDecreasingSequences"],

Cell[TextData[{
  "Output is the same as for ",
  StyleBox["PowerSet[n,k]", "MR"],
  "  but the implementation too cute to abandon."
}], "Notes",
  CellTags->"NonDecreasingSequences"],

Cell[TextData[{
  "See also ",
  ButtonBox["Tuples",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["ToPairs",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["PowerSet",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"NonDecreasingSequences"],

Cell["Further Examples", "ExampleSection",
  CellTags->"NonDecreasingSequences"],

Cell[TextData[{
  "All non-decreasing sequences over ",
  Cell[BoxData[
      \(TraditionalForm\`\([3]\)\  = \ {1, 2, 3}\)]],
  " of length 5."
}], "ExampleText",
  CellTags->"NonDecreasingSequences"],

Cell["NonDecreasingSequences[3,5]", "Input",
  CellLabel->"In[23]:=",
  CellTags->"NonDecreasingSequences"],

Cell["A general list as input.", "ExampleText",
  CellTags->"NonDecreasingSequences"],

Cell["\<\
Clear[a,b,c,d]
NonDecreasingSequences[{a,b,c,d},2,Full->False]\
\>", "Input",
  CellLabel->"In[61]:=",
  CellTags->"NonDecreasingSequences"],

Cell["\<\
Unordered pairs as in the last example can also be produced in \
other ways.\
\>", "ExampleText",
  CellTags->"NonDecreasingSequences"],

Cell[BoxData[
    \(PowerSet[{a, b, c, d}, 2]\)], "Input",
  CellLabel->"In[71]:=",
  CellTags->"NonDecreasingSequences"],

Cell[BoxData[
    \(ToPairs[{a, b, c, d}]\)], "Input",
  CellLabel->"In[64]:=",
  CellTags->"NonDecreasingSequences"],

Cell["NoRest", "ObjectName",
  CellTags->"NoRest"],

Cell["Option", "Highlight",
  CellTags->"NoRest"],

Cell[TextData[{
  StyleBox["NoRest", "MR"],
  "  is an option for ",
  StyleBox["PlotMatrix", "MR"],
  "."
}], "Usage",
  CellTags->"NoRest"],

Cell[TextData[{
  "See ",
  ButtonBox["PlotMatrix",
    ButtonData:>"PlotMatrix",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"NoRest"],

Cell["OutPairs", "ObjectName",
  CellTags->"OutPairs"],

Cell["Option", "Highlight",
  CellTags->"OutPairs"],

Cell[TextData[{
  StyleBox["OutPairs", "MR"],
  "  is an option for command ",
  StyleBox["DeleteDuplicates", "MR"],
  ". "
}], "Usage",
  CellTags->"OutPairs"],

Cell["Causes the ouput to be returned in pair-list format.", "Notes",
  CellTags->"OutPairs"],

Cell[TextData[{
  "See ",
  ButtonBox["DeleteDuplicates",
    ButtonData:>"DeleteDuplicates",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"OutPairs"],

Cell["Pairs", "ObjectName",
  CellTags->"Pairs"],

Cell["Function", "Highlight",
  CellTags->"Pairs"],

Cell[TextData[{
  StyleBox["Pairs[A,B]", "MR"],
  "  generates a list of pairs with elements taken from lists ",
  StyleBox["A", "MR"],
  " and ",
  StyleBox["B", "MR"],
  "."
}], "Usage",
  CellTags->"Pairs"],

Cell[TextData[{
  StyleBox["Pairs", "MR"],
  " is polyadic."
}], "Notes",
  CellTags->"Pairs"],

Cell[TextData[{
  "All the input lists must have the same length, ",
  StyleBox["Pairs", "MR"],
  " does not check. "
}], "Notes",
  CellTags->"Pairs"],

Cell["The output list has the same length as the input lists.", "Notes",
  CellTags->"Pairs"],

Cell[TextData[{
  StyleBox["See also ", "Text"],
  ButtonBox["CartesianProduct",
    ButtonData:>"CartesianProduct",
    ButtonStyle->"AddOnsLink"],
  StyleBox[". ", "Text"]
}], "Notes",
  CellTags->"Pairs"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Pairs"],

Cell["Generating tuples.", "ExampleText",
  CellTags->"Pairs"],

Cell["Pairs[{1,2,3},{a,b,c}]", "Input",
  CellLabel->"In[119]:=",
  CellTags->"Pairs"],

Cell["Pairs[{1,2,3},{a,b,c},{1,2,3}]", "Input",
  CellLabel->"In[336]:=",
  CellTags->"Pairs"],

Cell["Pairs[{},{}]", "Input",
  CellLabel->"In[337]:=",
  CellTags->"Pairs"],

Cell["Pairs[{1},{}]", "Input",
  CellLabel->"In[338]:=",
  CellTags->"Pairs"],

Cell["Partitions", "ObjectName",
  CellTags->"Partitions"],

Cell["Function", "Highlight",
  CellTags->"Partitions"],

Cell[TextData[{
  StyleBox["Partitions[n]", "MR"],
  "  returns the partitions of ",
  StyleBox["n", "MR"],
  ". "
}], "Usage",
  CellTags->"Partitions"],

Cell[TextData[{
  StyleBox["Partitions[n,m]", "MR"],
  " returns the partitions of ",
  StyleBox["n", "MR"],
  " whose largest component is at most ",
  StyleBox["m", "MR"],
  "."
}], "Notes",
  CellTags->"Partitions"],

Cell[TextData[{
  "See also ",
  ButtonBox["PartitionsP",
    ButtonData:>"PartitionsP",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["PartitionQ",
    ButtonData:>"PartitionQ",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Partitions"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Partitions"],

Cell[TextData[{
  "All partitions of ",
  Cell[BoxData[
      \(TraditionalForm\`\([5]\)\)]],
  "."
}], "ExampleText",
  CellTags->"Partitions"],

Cell["Partitions[5]", "Input",
  CellLabel->"In[122]:=",
  CellTags->"Partitions"],

Cell["Periodic", "ObjectName",
  CellTags->"Periodic"],

Cell["Option", "Highlight",
  CellTags->"Periodic"],

Cell[TextData[{
  StyleBox["Periodic", "MR"],
  "  is an option that speeds up the computation in ",
  StyleBox["AnalyzeOrbit", "MR"],
  " if the orbit is known to be periodic. "
}], "Usage",
  CellTags->"Periodic"],

Cell[TextData[{
  "Default is ",
  StyleBox["Periodic->False", "MR"],
  "."
}], "Notes",
  CellTags->"Periodic"],

Cell[TextData[{
  "See ",
  ButtonBox["AnalyzeOrbit",
    ButtonData:>"AnalyzeOrbit",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Periodic"],

Cell["PowerSet", "ObjectName",
  CellTags->"PowerSet"],

Cell["Function", "Highlight",
  CellTags->"PowerSet"],

Cell[TextData[{
  StyleBox["PowerSet[L,k]", "MR"],
  "  produces all ",
  StyleBox["k", "MR"],
  "-element subsets of ",
  StyleBox["L", "MR"],
  "."
}], "Usage",
  CellTags->"PowerSet"],

Cell[TextData[{
  " If ",
  StyleBox["k", "MR"],
  " is omitted, the full power set is generated. "
}], "Notes",
  CellTags->"PowerSet"],

Cell[TextData[{
  StyleBox["Range[n]", "MR"],
  " can be denoted by ",
  StyleBox["n", "MR"],
  "."
}], "Notes",
  CellTags->"PowerSet"],

Cell[TextData[{
  StyleBox["See also ", "Text"],
  ButtonBox["FastPowerSet",
    ButtonData:>"FastPowerSet",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"PowerSet"],

Cell["Further Examples", "ExampleSection",
  CellTags->"PowerSet"],

Cell[TextData[{
  "The power set of ",
  Cell[BoxData[
      \(TraditionalForm\`{a, b, c, d}\)]],
  "."
}], "ExampleText",
  CellTags->"PowerSet"],

Cell["PowerSet[{a,b,c,d}]", "Input",
  CellLabel->"In[148]:=",
  CellTags->"PowerSet"],

Cell["PowerSet[{a,b,c,d},2]", "Input",
  CellLabel->"In[149]:=",
  CellTags->"PowerSet"],

Cell["RandomPermutation", "ObjectName",
  CellTags->"RandomPermutation"],

Cell["Function", "Highlight",
  CellTags->"RandomPermutation"],

Cell[TextData[{
  StyleBox["RandomPermutation[k]", "MR"],
  "  produces a random permutation on ",
  StyleBox["k", "MR"],
  " points."
}], "Usage",
  CellTags->"RandomPermutation"],

Cell["\<\
Elements are drawn uniformly from the set of all \
permutations.\
\>", "Notes",
  CellTags->"RandomPermutation"],

Cell[TextData[{
  "See also ",
  ButtonBox["Tuples",
    ButtonData:>"Tuples",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"RandomPermutation"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RandomPermutation"],

Cell[TextData[{
  "A random permutation of ",
  Cell[BoxData[
      \(TraditionalForm\`\([10]\)\)]],
  ". "
}], "ExampleText",
  CellTags->"RandomPermutation"],

Cell["RandomPermutation[10]", "Input",
  CellLabel->"In[47]:=",
  CellTags->"RandomPermutation"],

Cell["Rank", "ObjectName",
  CellTags->"Rank"],

Cell["Option", "Highlight",
  CellTags->"Rank"],

Cell[TextData[{
  StyleBox["Rank", "MR"],
  "  is an option for ",
  StyleBox["Closure", "MR"],
  "."
}], "Usage",
  CellTags->"Rank"],

Cell[TextData[{
  "See also ",
  ButtonBox["Closure",
    ButtonData:>"Closure",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Rank"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Rank"],

Cell["RankingRules", "ObjectName",
  CellTags->"RankingRules"],

Cell["Function", "Highlight",
  CellTags->"RankingRules"],

Cell[TextData[{
  StyleBox["RankingRules[L]", "MR"],
  "  produces a list of rules that replace the elements of list ",
  StyleBox["L", "MR"],
  " by their ranks."
}], "Usage",
  CellTags->"RankingRules"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Direction", "\[Rule]", "Forward", "-"},
          {"Index", "\[Rule]", "1", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"RankingRules"],

Cell[TextData[{
  "Option ",
  StyleBox["Index->n", "MR"],
  " starts numbering at ",
  StyleBox["n", "MR"],
  ".  Mostly useful for 0-indexing."
}], "Notes",
  CellTags->"RankingRules"],

Cell[TextData[{
  "Option ",
  StyleBox["Type->Backward", "MR"],
  " generates rules for the corresponding unranking operation from integers \
to list elements."
}], "Notes",
  CellTags->"RankingRules"],

Cell[TextData[{
  "The ranks are the positions of the elements in the list.  The list is \
assumed to be repetition-free, ",
  StyleBox["RankingRules", "MR"],
  " does not check."
}], "Notes",
  CellTags->"RankingRules"],

Cell[TextData[{
  "Applying the rules generated by ",
  StyleBox["RankingRules", "MR"],
  "  has the same effect as applying the canonical ranking function but is \
often more convenient."
}], "Notes",
  CellTags->"RankingRules"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RankingRules"],

Cell["\<\
All permutations of a four-element set.  The collection is \
naturally ordered  lexicographically.\
\>", "ExampleText",
  CellTags->"RankingRules"],

Cell["P = Permutations[Range[4]]", "Input",
  CellLabel->"In[698]:=",
  CellTags->"RankingRules"],

Cell["The corresponding ranking rules.", "ExampleText",
  CellTags->"RankingRules"],

Cell[BoxData[{
    \(\(rules\  = \ RankingRules[P];\)\), "\n", 
    \(P\  /. \ rules\), "\n", 
    \(Select[P, Signature[#] \[Equal] 1 &]\  /. \ rules\)}], "Input",
  CellLabel->"In[714]:=",
  CellTags->"RankingRules"],

Cell["Unranking the last list of ranks.", "ExampleText",
  CellTags->"RankingRules"],

Cell[BoxData[
    \(%\  /. \ RankingRules[P, Direction \[Rule] Backward]\)], "Input",
  CellLabel->"In[717]:=",
  CellTags->"RankingRules"],

Cell["The same effect using an unranking function.", "ExampleText",
  CellTags->"RankingRules"],

Cell[BoxData[{
    \(AssignFunction[unrnk, Range[24], P]\), "\n", 
    \(unrnk\  /@ \ {1, 4, 5, 8, 9, 12, 13, 16, 17, 20, 21, 24}\)}], "Input",
  CellLabel->"In[725]:=",
  CellTags->"RankingRules"],

Cell["Reflexive", "ObjectName",
  CellTags->"Reflexive"],

Cell["Option", "Highlight",
  CellTags->"Reflexive"],

Cell[TextData[{
  StyleBox["Reflexive", "MR"],
  "  is an option for function ",
  StyleBox["TransitiveClosure", "MR"]
}], "Usage",
  CellTags->"Reflexive"],

Cell[TextData[{
  " Default value is ",
  StyleBox["False", "MR"],
  "."
}], "Notes",
  CellTags->"Reflexive"],

Cell[TextData[{
  "See also ",
  ButtonBox["TransitiveClosure",
    ButtonData:>"TransitiveClosure",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Reflexive"],

Cell["Relation", "ObjectName",
  CellTags->"Relation"],

Cell["Option", "Highlight",
  CellTags->"Relation"],

Cell[TextData[{
  StyleBox["Relation", "MR"],
  "  is a possible value for option ",
  StyleBox["Type", "MR"],
  " in function ",
  StyleBox["ToClasses", "MR"],
  "."
}], "Usage",
  CellTags->"Relation"],

Cell[TextData[{
  "See ",
  ButtonBox["ToClasses",
    ButtonData:>"ToClasses",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Relation"],

Cell["RootList", "ObjectName",
  CellTags->"RootList"],

Cell["Function", "Highlight",
  CellTags->"RootList"],

Cell[TextData[{
  StyleBox["RootList[L]", "MR"],
  "  computes the root of a list ",
  StyleBox["L", "MR"],
  ", i.e., the shortest list ",
  StyleBox["K", "MR"],
  " such that ",
  StyleBox["L", "MR"],
  " is a repetition of copies of ",
  StyleBox["K", "MR"],
  "."
}], "Usage",
  CellTags->"RootList"],

Cell[TextData[{
  "Unlike ",
  StyleBox["WordRoot", "MR"],
  ", uses a brute force algorithm."
}], "Notes",
  CellTags->"RootList"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordPrimitiveCount",
    ButtonData:>"WordPrimitiveCount",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["WordPrimitiveQ",
    ButtonData:>"WordPrimitiveQ",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"RootList"],

Cell["Further Examples", "ExampleSection",
  CellTags->"RootList"],

Cell["A repetitive list, and its root.", "ExampleText",
  CellTags->"RootList"],

Cell[BoxData[
    \(L = Flatten[Table[{1, 2, 3}, {5}]]\)], "Input",
  CellLabel->"In[140]:=",
  CellTags->"RootList"],

Cell[BoxData[
    \(RootList[L]\)], "Input",
  CellLabel->"In[141]:=",
  CellTags->"RootList"],

Cell["SelectorQ", "ObjectName",
  CellTags->"SelectorQ"],

Cell["Function", "Highlight",
  CellTags->"SelectorQ"],

Cell[TextData[{
  StyleBox["SelectorQ[A,P]", "MR"],
  "  tests whether list ",
  StyleBox["A", "MR"],
  " selects precisely one element of each of the lists in ",
  StyleBox["P", "MR"],
  "."
}], "Usage",
  CellTags->"SelectorQ"],

Cell[TextData[{
  "If ",
  StyleBox["P", "MR"],
  " is in particular a partition this tests whether ",
  StyleBox["A", "MR"],
  " is a set of representatives for ",
  StyleBox["P", "MR"],
  "."
}], "Notes",
  CellTags->"SelectorQ"],

Cell[TextData[{
  "See also ",
  ButtonBox["Partitions",
    ButtonData:>"Partitions",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SelectorQ"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SelectorQ"],

Cell[TextData[{
  "A partition of ",
  Cell[BoxData[
      \(TraditionalForm\`\([12]\)\)]],
  ", and a system of representatives."
}], "ExampleText",
  CellTags->"SelectorQ"],

Cell[BoxData[
    \(\(P = {{1, 4, 7, 10}, {2, 5, 8, 11}, {3, 6, 9, 12}};\)\)], "Input",
  CellLabel->"In[149]:=",
  CellTags->"SelectorQ"],

Cell[BoxData[
    \(SelectorQ[\ {10, 11, 12}, \ P\ ]\)], "Input",
  CellLabel->"In[150]:=",
  CellTags->"SelectorQ"],

Cell[BoxData[
    \(SelectorQ[\ {9, 11, 12}, \ P\ ]\)], "Input",
  CellLabel->"In[151]:=",
  CellTags->"SelectorQ"],

Cell["SetClosure", "ObjectName",
  CellTags->"SetClosure"],

Cell["Option", "Highlight",
  CellTags->"SetClosure"],

Cell[TextData[{
  StyleBox["SetClosure", "MR"],
  "  is an option in ",
  StyleBox["Closure", "MR"],
  " and ",
  StyleBox["ClosureSearch", "MR"],
  " that determines the signature of the closure function."
}], "Usage",
  CellTags->"SetClosure"],

Cell[TextData[{
  "See ",
  ButtonBox["Closure",
    ButtonData:>"Closure",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ClosureSearch",
    ButtonData:>"ClosureSearch",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SetClosure"],

Cell["Shuffle", "ObjectName",
  CellTags->"Shuffle"],

Cell["Function", "Highlight",
  CellTags->"Shuffle"],

Cell[TextData[{
  StyleBox["Shuffle[A,B]", "MR"],
  "  shuffles two lists ",
  StyleBox["A", "MR"],
  " and ",
  StyleBox["B", "MR"],
  "."
}], "Usage",
  CellTags->"Shuffle"],

Cell[TextData[{
  "Also accepts input ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " for ",
  Cell[BoxData[
      FormBox[
        StyleBox[\(Range[n]\),
          "MR"], TraditionalForm]]],
  "."
}], "Notes",
  CellTags->"Shuffle"],

Cell[TextData[{
  "See also ",
  ButtonBox["WordShuffle",
    ButtonData:>"WordShuffle",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ShuffleProductFA",
    ButtonData:>"ShuffleProductFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Shuffle"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Shuffle"],

Cell["All possible order preserving interleavings of two lists.", \
"ExampleText",
  CellTags->"Shuffle"],

Cell["Shuffle[ {1,2,3}, {a,b} ]", "Input",
  CellLabel->"In[773]:=",
  CellTags->"Shuffle"],

Cell[BoxData[
    \(Length[%]\  \[Equal] \ Binomial[5, 2]\)], "Input",
  CellLabel->"In[774]:=",
  CellTags->"Shuffle"],

Cell["SubsetQ", "ObjectName",
  CellTags->"SubsetQ"],

Cell["Function", "Highlight",
  CellTags->"SubsetQ"],

Cell[TextData[{
  StyleBox["SubsetQ[S1,S2]", "MR"],
  "  tests whether ",
  StyleBox["S1", "MR"],
  " is a subset of ",
  StyleBox["S2", "MR"],
  "."
}], "Usage",
  CellTags->"SubsetQ"],

Cell[TextData[{
  "Multiplicities are disregarded by ",
  StyleBox["SubsetQ", "MR"],
  "."
}], "Notes",
  CellTags->"SubsetQ"],

Cell[TextData[{
  "See also ",
  ButtonBox["PowerSet",
    ButtonData:>"PowerSet",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"SubsetQ"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SubsetQ"],

Cell["Lists are being treated as sets.", "ExampleText",
  CellTags->"SubsetQ"],

Cell["SubsetQ[ {1,5}, Range[5] ]", "Input",
  CellLabel->"In[98]:=",
  CellTags->"SubsetQ"],

Cell["SubsetQ[ {1,6}, Range[5] ]", "Input",
  CellLabel->"In[99]:=",
  CellTags->"SubsetQ"],

Cell["ToBitVector", "ObjectName",
  CellTags->"ToBitVector"],

Cell["Function", "Highlight",
  CellTags->"ToBitVector"],

Cell[TextData[{
  StyleBox["ToBitVector[S,U]", "MR"],
  "  transforms set ",
  StyleBox["S", "MR"],
  " into a bitvector over universe ",
  StyleBox["U", "MR"],
  "."
}], "Usage",
  CellTags->"ToBitVector"],

Cell["A bitvector is understood to be a 0/1-integer list.", "Notes",
  CellTags->"ToBitVector"],

Cell[TextData[{
  "See also ",
  ButtonBox["FromBitVector",
    ButtonData:>"FromBitVector",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ToBitVector"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToBitVector"],

Cell["Conversion from  an actual subset to a bitvector.", "ExampleText",
  CellTags->"ToBitVector"],

Cell["all = PowerSet[{a,b,c}]", "Input",
  CellLabel->"In[45]:=",
  CellTags->"ToBitVector"],

Cell[BoxData[
    \(\(ToBitVector[#, {a, b, c}] &\)\  /@ \ all\)], "Input",
  CellLabel->"In[46]:=",
  CellTags->"ToBitVector"],

Cell["ToList", "ObjectName",
  CellTags->"ToList"],

Cell["Function", "Highlight",
  CellTags->"ToList"],

Cell[TextData[{
  StyleBox["ToList[S]", "MR"],
  "  replaces the head of object ",
  StyleBox["S", "MR"],
  " by ",
  StyleBox["List", "MR"],
  "."
}], "Usage",
  CellTags->"ToList"],

Cell["ToPairs", "ObjectName",
  CellTags->"ToPairs"],

Cell["Function", "Highlight",
  CellTags->"ToPairs"],

Cell[TextData[{
  StyleBox["ToPairs[A]", "MR"],
  "  returns a list of all unordered pairs of elements in list ",
  StyleBox["A", "MR"],
  "."
}], "Usage",
  CellTags->"ToPairs"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"ToPairs"],

Cell[TextData[{
  StyleBox["ToPairs", "MR"],
  "  returns the ordered pairs in standard lexicographic order."
}], "Notes",
  CellTags->"ToPairs"],

Cell[TextData[{
  "Option  ",
  StyleBox["Full->True", "MR"],
  "  also includes the diagonal, i.e., pairs of the form  ",
  Cell[BoxData[
      \(TraditionalForm\`{x, x}\)]],
  "."
}], "Notes",
  CellTags->"ToPairs"],

Cell[TextData[{
  StyleBox["The length of the output list is ", "Text"],
  Cell[BoxData[
      \(TraditionalForm\`\(n(n - 1)\)/2\)]],
  StyleBox[" where ", "Text"],
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " is ",
  StyleBox["the length of the input list (", "Text"],
  Cell[BoxData[
      \(TraditionalForm\`\(n(n + 1)\)/2\)]],
  " in ",
  StyleBox["Full", "MR"],
  " mode)."
}], "Notes",
  CellTags->"ToPairs"],

Cell[TextData[{
  "See also ",
  ButtonBox["Pairs",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["NonDecreasing",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["PowerSet",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["CartesianProduct",
    ButtonStyle->"AddOnsLink"],
  ". "
}], "Notes",
  CellTags->"ToPairs"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ToPairs"],

Cell["Ordered pairs, without and with diagonal elements.", "ExampleText",
  CellTags->"ToPairs"],

Cell["\<\
Clear[a,b,c,d]
ToPairs[{a,b,c,d}]\
\>", "Input",
  CellLabel->"In[529]:=",
  CellTags->"ToPairs"],

Cell[BoxData[
    \(ToPairs[{a, b, c, d}, Full \[Rule] True]\)], "Input",
  CellLabel->"In[531]:=",
  CellTags->"ToPairs"],

Cell[TextData[{
  StyleBox["ToPairs", "MR"],
  " is slightly faster on long lists than selecting elements from the \
Cartesian product."
}], "ExampleText",
  CellTags->"ToPairs"],

Cell[BoxData[
    \(Select[CartesianProduct[Range[100], Range[100]], OrderedQ] === 
      Sort[ToPairs[Range[100], Full \[Rule] True]]\)], "Input",
  CellLabel->"In[537]:=",
  CellTags->"ToPairs"],

Cell[BoxData[{
    \(\(Select[CartesianProduct[Range[100], Range[100]], OrderedQ];\)\  // \ 
      AbsoluteTiming\), "\n", 
    \(\(Sort[ToPairs[Range[100], Full \[Rule] True]];\)\  // \ 
      AbsoluteTiming\)}], "Input",
  CellLabel->"In[538]:=",
  CellTags->"ToPairs"],

Cell["Tuples", "ObjectName",
  CellTags->"Tuples"],

Cell["Function", "Highlight",
  CellTags->"Tuples"],

Cell[TextData[{
  StyleBox["Tuples[L,k]", "MR"],
  "  produces a list of all tuples of length ",
  StyleBox["k", "MR"],
  " with elements taken from list ",
  StyleBox["L", "MR"],
  ". "
}], "Usage",
  CellTags->"Tuples"],

Cell[TextData[{
  StyleBox["L = Range[r]", "MR"],
  " can be abbreviated as ",
  StyleBox["r", "MR"],
  "."
}], "Notes",
  CellTags->"Tuples"],

Cell[TextData[{
  "For negative values of ",
  StyleBox["k", "MR"],
  ",  all tuples of length ",
  Cell[BoxData[
      \(TraditionalForm\`s\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`0\  \[LessEqual] s\  \[LessEqual] k\)]],
  ",  are returned as a flattened list."
}], "Notes",
  CellTags->"Tuples"],

Cell[TextData[{
  StyleBox["Tuples", "MR"],
  " is just a wrapper for ",
  ButtonBox["CartesianProduct",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Tuples"],

Cell[TextData[{
  "See also ",
  ButtonBox["Permutations",
    ButtonData:>"Permutations",
    ButtonStyle->"RefGuideLink"],
  "."
}], "Notes",
  CellTags->"Tuples"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Tuples"],

Cell["All tuples of length 3 over a two-element list.", "ExampleText",
  CellTags->"Tuples"],

Cell[BoxData[
    \(Tuples[{a, b}, 3]\)], "Input",
  CellLabel->"In[678]:=",
  CellTags->"Tuples"],

Cell["\<\
All tuples up to length 3. Note that the empty list is \
included.\
\>", "ExampleText",
  CellTags->"Tuples"],

Cell[BoxData[
    \(Tuples[{a, b}, \(-3\)]\)], "Input",
  CellLabel->"In[689]:=",
  CellTags->"Tuples"],

Cell["Abbreviating special lists as integers.", "ExampleText",
  CellTags->"Tuples"],

Cell[BoxData[{
    \(Tuples[2, 3]\), "\n", 
    \(Tuples[\(-2\), 3]\)}], "Input",
  CellLabel->"In[680]:=",
  CellTags->"Tuples"],

Cell["Type", "ObjectName",
  CellTags->"Type"],

Cell["Option", "Highlight",
  CellTags->"Type"],

Cell[TextData[{
  StyleBox["Type", "MR"],
  "  is an option for ",
  StyleBox["ToClasses", "MR"],
  ". "
}], "Usage",
  CellTags->"Type"],

Cell[TextData[{
  "Possible values are: ",
  StyleBox["Relation", "MR"],
  " (default), ",
  StyleBox["Function", "MR"],
  ", ",
  StyleBox["Matrix", "MR"],
  ", ",
  StyleBox["Index", "MR"],
  "."
}], "Notes",
  CellTags->"Type"],

Cell[TextData[{
  "See ",
  ButtonBox["ToClasses",
    ButtonData:>"ToClasses",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Type"],

Cell["$extlink", "ObjectName",
  CellTags->"$extlink"],

Cell["SysVariable", "Highlight",
  CellTags->"$extlink"],

Cell[TextData[{
  StyleBox["$extlink", "MR"],
  "  is the MathLink",
  " connection to the external code."
}], "Usage",
  CellTags->"$extlink"],

Cell[TextData[{
  "Is set to ",
  StyleBox["True", "MR"],
  " automatically if the ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  "  connection to ",
  StyleBox["extaut", "Input"],
  " has been properly established."
}], "Notes",
  CellTags->"$extlink"],

Cell[TextData[{
  "Can be overridden locally using option ",
  StyleBox["External->False", "MR"],
  "."
}], "Notes",
  CellTags->"$extlink"],

Cell[TextData[{
  "See also ",
  ButtonBox["$defaultexternal",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["External",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"$extlink"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$extlink"],

Cell[TextData[{
  "By default the value of ",
  StyleBox["$extlink", "MR"],
  " is ",
  StyleBox["Null", "MR"],
  ":"
}], "ExampleText",
  CellTags->"$extlink"],

Cell[BoxData[
    \($extlink\)], "Input",
  CellLabel->"In[170]:=",
  CellTags->"$extlink"],

Cell["$defaultexternal", "ObjectName",
  CellTags->"$defaultexternal"],

Cell["SysVariable", "Highlight",
  CellTags->"$defaultexternal"],

Cell[TextData[{
  StyleBox["$defaultexternal", "MR"],
  "  determines whether an external algorithm is used in a number of \
commands."
}], "Usage",
  CellTags->"$defaultexternal"],

Cell[TextData[{
  "Is set to ",
  StyleBox["True", "MR"],
  " automatically if the ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  "  connection to ",
  StyleBox["extaut", "Input"],
  " has been properly established."
}], "Notes",
  CellTags->"$defaultexternal"],

Cell[TextData[{
  "Can be overridden locally using option ",
  StyleBox["External->False", "MR"],
  "."
}], "Notes",
  CellTags->"$defaultexternal"],

Cell[TextData[{
  "See also ",
  ButtonBox["$extlink",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["ToDFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["MinimizeFA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["EvolutionCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"$defaultexternal"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$defaultexternal"],

Cell["The system default value is False:", "ExampleText",
  CellTags->"$defaultexternal"],

Cell[BoxData[
    \($defaultexternal\)], "Input",
  CellLabel->"In[170]:=",
  CellTags->"$defaultexternal"],

Cell["$LLL", "ObjectName",
  CellTags->"$LLL"],

Cell["SysVariable", "Highlight",
  CellTags->"$LLL"],

Cell[TextData[{
  StyleBox["$LLL", "MR"],
  "   is a global parameter for the recursive language map in command ",
  StyleBox["LanguageFA", "MR"],
  " with option ",
  StyleBox["Implementation->3", "MR"],
  " .  "
}], "Usage",
  CellTags->"$LLL"],

Cell[TextData[{
  StyleBox["$LLL[s,p,q]", "MR"],
  " is the collection of all words of length ",
  Cell[BoxData[
      \(TraditionalForm\`s\)]],
  " leading from state ",
  Cell[BoxData[
      \(TraditionalForm\`p\)]],
  " to state ",
  Cell[BoxData[
      \(TraditionalForm\`q\)]],
  ".  ",
  "This works only for DFAs. "
}], "Notes",
  CellTags->"$LLL"],

Cell["\<\
Implemented only for DFAs.   Mildly useful to produce  a trace of \
the algorithm.\
\>", "Notes",
  CellTags->"$LLL"],

Cell[TextData[{
  "See also ",
  ButtonBox["LanguageFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"$LLL"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$LLL"],

Cell[TextData[{
  "A DFA for the language \"3rd symbol from the end is an ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  "\". "
}], "ExampleText",
  CellTags->"$LLL"],

Cell[BoxData[
    \(\(M\  = \ 
        DFA[8, 2, {{2, 3, 4, 4, 6, 3, 6, 2}, {1, 7, 5, 5, 8, 7, 8, 1}}, 
          1, {4, 5, 6, 8}];\)\)], "Input",
  CellLabel->"In[111]:=",
  CellTags->"$LLL"],

Cell["The acceptance language up to words of length 4.", "ExampleText",
  CellTags->"$LLL"],

Cell[BoxData[
    \(LanguageFA[M, \(-4\), Implementation \[Rule] 3]\)], "Input",
  CellLabel->"In[112]:=",
  CellTags->"$LLL"],

Cell[TextData[{
  "At level 1, ",
  StyleBox["$LLL", "MR"],
  " is essentially just the transition matrix of the automaton."
}], "ExampleText",
  CellTags->"$LLL"],

Cell[BoxData[
    \(Table[$LLL[1, p, q], {p, 8}, {q, 8}]\  // \ MatrixForm\)], "Input",
  CellLabel->"In[113]:=",
  CellTags->"$LLL"],

Cell["At level 2 we get transfer sequences of length 2.", "ExampleText",
  CellTags->"$LLL"],

Cell[BoxData[
    \(Table[$LLL[2, p, q], {p, 8}, {q, 8}\ ]\  // \ MatrixForm\)], "Input",
  CellLabel->"In[109]:=",
  CellTags->"$LLL"],

Cell["\<\
At level 4, transfer sequences of length 4; here from state 1 to \
any target state. \
\>", "ExampleText",
  CellTags->"$LLL"],

Cell[BoxData[
    \(Table[$LLL[4, 1, p], {p, 8}]\  // \ TableForm\)], "Input",
  CellLabel->"In[100]:=",
  CellTags->"$LLL"],

Cell["$shortlines", "ObjectName",
  CellTags->"$shortlines"],

Cell["SysVariable", "Highlight",
  CellTags->"$shortlines"],

Cell[TextData[{
  StyleBox["$shortlines", "MR"],
  "  is controls the size of expression printed inside machines when \
formatting is turned on. "
}], "Usage",
  CellTags->"$shortlines"],

Cell[TextData[{
  "Formatting applies   ",
  StyleBox["$Short[#,$shortlines]& ", "MR"],
  " to potentially long expressions."
}], "Notes",
  CellTags->"$shortlines"],

Cell["Default value is 4. ", "Notes",
  CellTags->"$shortlines"],

Cell[TextData[{
  "See also ",
  ButtonBox["FormatAutomata",
    ButtonData:>"FormatAutomata",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"$shortlines"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$shortlines"],

Cell["The system default value is 4:", "ExampleText",
  CellTags->"$shortlines"],

Cell[BoxData[
    \($shortlines\)], "Input",
  CellLabel->"In[345]:=",
  CellTags->"$shortlines"],

Cell["The value can be adjusted locally.", "ExampleText",
  CellTags->"$shortlines"],

Cell[BoxData[
    \(m = IthSymbolFA[b, \(-5\), Full \[Rule] True]\)], "Input",
  CellLabel->"In[346]:=",
  CellTags->"$shortlines"],

Cell[BoxData[{
    \(\($shortlines = 1;\)\), "\n", 
    \(m\)}], "Input",
  CellLabel->"In[347]:=",
  CellTags->"$shortlines"],

Cell[BoxData[
    \(\($shortlines = 4;\)\)], "Input",
  CellTags->"$shortlines"],

Cell["$transdispwidth", "ObjectName",
  CellTags->"$transdispwidth"],

Cell["SysVariable", "Highlight",
  CellTags->"$transdispwidth"],

Cell[TextData[{
  StyleBox["$transdispwidth", "MR"],
  "   controls the number of transitions displayed in one row in ",
  StyleBox["PrintFA", "MR"],
  ". "
}], "Usage",
  CellTags->"$transdispwidth"],

Cell["Default is 20.", "Notes",
  CellTags->"$transdispwidth"],

Cell[TextData[{
  "See ",
  ButtonBox["PrintFA",
    ButtonData:>"PrintFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"$transdispwidth"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$transdispwidth"],

Cell["The value can be adjusted locally.", "ExampleText",
  CellTags->"$transdispwidth"],

Cell["m = IthSymbolFA[b,-10]", "Input",
  CellLabel->"In[197]:=",
  CellTags->"$transdispwidth"],

Cell[BoxData[
    \(Block[{$transdispwidth = 6}, PrintFA[m]]\)], "Input",
  CellLabel->"In[198]:=",
  CellTags->"$transdispwidth"],

Cell["All", "ObjectName",
  CellTags->"All"],

Cell["Option", "Highlight",
  CellTags->"All"],

Cell[TextData[{
  StyleBox["All", "MR"],
  "  is an option value for various functions."
}], "Usage",
  CellTags->"All"],

Cell[TextData[{
  StyleBox["See also ", "Text"],
  ButtonBox["PlotFunction",
    ButtonData:>"PlotFunction",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["PlotRelation",
    ButtonData:>"PlotRelation",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["PlotTDGraph",
    ButtonData:>"PlotTDGraph",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"All"],

Cell["Further Examples", "ExampleSection",
  CellTags->"All"],

Cell[TextData[{
  "The function ",
  StyleBox["PlotRelation", "MR"],
  " by default uses ",
  StyleBox["DoTrace->All", "MR"],
  " which may produce confusing graphics. "
}], "ExampleText",
  CellTags->"All"],

Cell["PlotRelation[(Mod[#2,#1]===0)&,Range[10],2 ]", "Input",
  CellLabel->"In[93]:=",
  CellTags->"All"],

Cell["Tracing individual points improves the clarity. ", "ExampleText",
  CellTags->"All"],

Cell["PlotRelation[(Mod[#2,#1]===0)&,Range[10],2,DoTrace->{2} ]", "Input",
  CellLabel->"In[96]:=",
  CellTags->"All"],

Cell["AssignFunction", "ObjectName",
  CellTags->"AssignFunction"],

Cell["Function", "Highlight",
  CellTags->"AssignFunction"],

Cell[TextData[{
  StyleBox["AssignFunction[h,P]", "MR"],
  "  defines ",
  StyleBox["h", "MR"],
  " to be function with graph ",
  StyleBox["P", "MR"],
  ", where ",
  StyleBox["P", "MR"],
  " is a list of pairs, representing the functional digraph of ",
  StyleBox["h", "MR"],
  ". "
}], "Usage",
  CellTags->"AssignFunction"],

Cell[TextData[{
  " ",
  StyleBox["AssignFunction[h,D,C]", "MR"],
  " defines the function that maps the i-th element of the domain list ",
  StyleBox["D", "MR"],
  " to the i-th element of the codomain list ",
  StyleBox["C", "MR"],
  " (",
  StyleBox["D", "MR"],
  " and ",
  StyleBox["C", "MR"],
  " must have the same length). "
}], "Notes",
  CellTags->"AssignFunction"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Opts", "\[Rule]", \({}\), "-"},
          {"Kary", "\[Rule]", "False", "True"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"AssignFunction"],

Cell[TextData[{
  "Option ",
  StyleBox["Opts->attributes", "MR"],
  " assigns the new function the given attributes. "
}], "Notes",
  CellTags->"AssignFunction"],

Cell[TextData[{
  "Option ",
  StyleBox["Kary->True", "MR"],
  " makes the new function ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  "-ary, assumes domain elements are lists."
}], "Notes",
  CellTags->"AssignFunction"],

Cell[TextData[{
  "See also  ",
  ButtonBox["Pairs",
    ButtonData:>"Pairs",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"AssignFunction"],

Cell["Further Examples", "ExampleSection",
  CellTags->"AssignFunction"],

Cell["A simple function on a domain of size 3.", "ExampleText",
  CellTags->"AssignFunction"],

Cell["\<\
AssignFunction[ f, {1,2,3}, {a,b,c} ]
f[2]\
\>", "Input",
  CellLabel->"In[266]:=",
  CellTags->"AssignFunction"],

Cell["\<\
The same function, but this time Listable and specified as a list \
of argument-value pairs.\
\>", "ExampleText",
  CellTags->"AssignFunction"],

Cell[BoxData[{
    \(AssignFunction[g, {{1, a}, {2, b}, {3, c}}, 
      Opts \[Rule] Listable]\), "\n", 
    \(g[{1, 2, 3}]\)}], "Input",
  CellLabel->"In[404]:=",
  CellTags->"AssignFunction"],

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`k\)]],
  "-ary functions can be defined by handing over lists of the arguments in \
conjunction with option ",
  StyleBox["Kary->True", "MR"],
  "."
}], "ExampleText",
  CellTags->"AssignFunction"],

Cell[BoxData[{
    \(\(A\  = \ CartesianProduct[2, 2];\)\), "\n", 
    \(\(B\  = \ Range[4];\)\), "\n", 
    \(AssignFunction[\ h, \ A, B, \ Kary \[Rule] True\ ]\)}], "Input",
  CellLabel->"In[419]:=",
  CellTags->"AssignFunction"],

Cell[BoxData[
    \(?? h\)], "Input",
  CellLabel->"In[422]:=",
  CellTags->"AssignFunction"],

Cell["Curry", "ObjectName",
  CellTags->"Curry"],

Cell["Functional", "Highlight",
  CellTags->"Curry"],

Cell[TextData[{
  StyleBox["Curry[f]", "MR"],
  "  curries a function ",
  StyleBox["f", "MR"],
  " of two variables."
}], "Usage",
  CellTags->"Curry"],

Cell[TextData[{
  "Thus ",
  StyleBox["Curry[f][y][x] = f[x,y]", "MR"],
  ". "
}], "Notes",
  CellTags->"Curry"],

Cell["\<\
This is useful on occasion when a function of two variables is to \
be used in postfix notation.\
\>", "Notes",
  CellTags->"Curry"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Curry"],

Cell["Using a functional in postfix notation.", "ExampleText",
  CellTags->"Curry"],

Cell[BoxData[
    \({a, b, c}\  // \ \(Curry[Append]\)[0]\)], "Input",
  CellLabel->"In[20]:=",
  CellTags->"Curry"],

Cell[BoxData[
    \(Table[i + j, {i, 3}, {j, 3}]\  // \ \(Curry[TableForm]\)[
        TableSpacing \[Rule] {0, 0}]\)], "Input",
  CellLabel->"In[23]:=",
  CellTags->"Curry"],

Cell["Dependency", "ObjectName",
  CellTags->"Dependency"],

Cell["Function", "Highlight",
  CellTags->"Dependency"],

Cell[TextData[{
  StyleBox["Dependency[V,k]", "MR"],
  "  computes the indices of the variables on which the ",
  StyleBox["k", "MR"],
  "-ary function represented by the value list ",
  StyleBox["V", "MR"],
  " depends. "
}], "Usage",
  CellTags->"Dependency"],

Cell[TextData[{
  "Here ",
  StyleBox["V", "MR"],
  " must have length ",
  Cell[BoxData[
      \(TraditionalForm\`n\^\(\(\ \)\(k\)\)\)]],
  " and represents a function ",
  Cell[BoxData[
      \(TraditionalForm\`f\)]],
  " of the form ",
  Cell[BoxData[
      \(TraditionalForm\`A\^\(\(\ \)\(k\)\)\[LongRightArrow]B\)]],
  ". "
}], "Notes",
  CellTags->"Dependency"],

Cell[TextData[{
  "Parameter ",
  StyleBox["k", "MR"],
  " is optional and has default 2."
}], "Notes",
  CellTags->"Dependency"],

Cell[TextData[{
  "See also ",
  ButtonBox["DependencyCA",
    ButtonData:>"DependencyCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Dependency"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Dependency"],

Cell["A function of three variables depending only on two.", "ExampleText",
  CellTags->"Dependency"],

Cell[BoxData[
    \(V\  = \ 
      Apply[\ Function[{x, y, z}, 
          Mod[x + z, 2]], \[IndentingNewLine]\t\t\tCartesianProduct[2, 2, 
          2], {1}]\)], "Input",
  CellLabel->"In[203]:=",
  CellTags->"Dependency"],

Cell[BoxData[
    \(Dependency[V]\)], "Input",
  CellLabel->"In[204]:=",
  CellTags->"Dependency"],

Cell["DoTrace", "ObjectName",
  CellTags->"DoTrace"],

Cell["Option", "Highlight",
  CellTags->"DoTrace"],

Cell[TextData[{
  StyleBox["DoTrace", "MR"],
  "  is an option for various functions, causes additional output to be \
generated."
}], "Usage",
  CellTags->"DoTrace"],

Cell[TextData[{
  "See  ",
  ButtonBox["PlotFunction",
    ButtonData:>"PlotFunction",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["PlotRelation",
    ButtonData:>"PlotRelation",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["PlotTDGraph",
    ButtonData:>"PlotTDGraph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["SimplifyRewriteSystem",
    ButtonData:>"SimplifyRewriteSystem",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["WordReduce",
    ButtonData:>"WordReduce",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DoTrace"],

Cell["Drei", "ObjectName",
  CellTags->"Drei"],

Cell["Type: fctassg", "Highlight",
  CellTags->"Drei"],

Cell[TextData[{
  StyleBox["Drei[x]", "MR"],
  "  returns the third part of object ",
  StyleBox["x", "MR"],
  "."
}], "Usage",
  CellTags->"Drei"],

Cell[TextData[{
  "Thus ",
  StyleBox["Drei[x] = x[[3]]", "MR"],
  "."
}], "Notes",
  CellTags->"Drei"],

Cell[TextData[{
  "See also ",
  ButtonBox["Zwei",
    ButtonData:>"Zwei",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["MinusEins",
    ButtonData:>"MinusEins",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["MinusZwei",
    ButtonData:>"MinusZwei",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Drei"],

Cell["ExtAutomata", "ObjectName",
  CellTags->"ExtAutomata"],

Cell["Function", "Highlight",
  CellTags->"ExtAutomata"],

Cell[TextData[{
  StyleBox["ExtAutomata[bool]", "MR"],
  "  controls the use of external code via MathLink."
}], "Usage",
  CellTags->"ExtAutomata"],

Cell[TextData[{
  StyleBox["ExtAutomata[True]", "MR"],
  "  turns the ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " connection on."
}], "Notes",
  CellTags->"ExtAutomata"],

Cell[TextData[{
  StyleBox["ExtAutomata[False]", "MR"],
  "  turns the ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " connection off."
}], "Notes",
  CellTags->"ExtAutomata"],

Cell[TextData[{
  "See also ",
  ButtonBox["$extlink",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["$defaultexternal",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ExtAutomata"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ExtAutomata"],

Cell["\<\
This will only work if you have installed the external code.   The \
speed-up shoul be a factor of about 50.\
\>", "ExampleText",
  CellTags->"ExtAutomata"],

Cell[BoxData[
    \(\(m\  = IthSymbolFA[a, \(-12\)];\)\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->"ExtAutomata"],

Cell[BoxData[{
    \(ExtAutomata[]\), "\n", 
    \(\(ToDFA[m];\)\  // \ AbsoluteTiming\)}], "Input",
  CellLabel->"In[2]:=",
  CellTags->"ExtAutomata"],

Cell[BoxData[{
    \(ExtAutomata[False]\), "\n", 
    \(\(ToDFA[m];\)\  // \ AbsoluteTiming\)}], "Input",
  CellLabel->"In[4]:=",
  CellTags->"ExtAutomata"],

Cell["FlattenOne", "ObjectName",
  CellTags->"FlattenOne"],

Cell["Function", "Highlight",
  CellTags->"FlattenOne"],

Cell[TextData[{
  StyleBox["FlattenOne[x]", "MR"],
  "  is shorthand for ",
  StyleBox["Flatten[x,1]", "MR"],
  "."
}], "Usage",
  CellTags->"FlattenOne"],

Cell["Useful in postfix notation.", "Notes",
  CellTags->"FlattenOne"],

Cell[TextData[{
  "See also ",
  ButtonBox["Curry",
    ButtonData:>"Curry",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"FlattenOne"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FlattenOne"],

Cell["\<\
A deeply nested list, the von Neumann ordinal 3, and its successive \
flattenings.\
\>", "ExampleText",
  CellTags->"FlattenOne"],

Cell["vN = Nest[ {#,{#}}&, {}, 3 ]", "Input",
  CellLabel->"In[468]:=",
  CellTags->"FlattenOne"],

Cell[BoxData[
    \(NestList[\ FlattenOne, \ vN, \ 6\ ]\  // \ ColumnForm\)], "Input",
  CellLabel->"In[469]:=",
  CellTags->"FlattenOne"],

Cell["FormatAutomata", "ObjectName",
  CellTags->"FormatAutomata"],

Cell["Function", "Highlight",
  CellTags->"FormatAutomata"],

Cell[TextData[{
  StyleBox["FormatAutomata[fmt]", "MR"],
  "  controls formatting of the output form of finite state machines and \
graphs. "
}], "Usage",
  CellTags->"FormatAutomata"],

Cell[TextData[{
  StyleBox["FormatAutomata[None]", "MR"],
  "  turns all formatting off. This may lead to the display of very large \
expressions."
}], "Notes",
  CellTags->"FormatAutomata"],

Cell[TextData[{
  " ",
  StyleBox["FormatAutomata[Some]", "MR"],
  " supresses display of the structured part of machines. "
}], "Notes",
  CellTags->"FormatAutomata"],

Cell[TextData[{
  StyleBox["FormatAutomata[Automatic]", "MR"],
  " applies ",
  StyleBox["Short", "MR"],
  " to potentially large parts of the expression. "
}], "Notes",
  CellTags->"FormatAutomata"],

Cell[TextData[{
  StyleBox["FormatAutomata[SizeOnly]", "MR"],
  " returns the type of the automaton, the number of states and the size of \
the alphabet."
}], "Notes",
  CellTags->"FormatAutomata"],

Cell[TextData[{
  StyleBox["FormatAutomata[Full]", "MR"],
  " only prints a string such as ",
  StyleBox["-DFA-", "MR"],
  "."
}], "Notes",
  CellTags->"FormatAutomata"],

Cell["\<\
Note that formatted expression may cause problems when copied to \
produce input for further computations.\
\>", "Notes",
  CellTags->"FormatAutomata"],

Cell[TextData[{
  "Formatting also affects the way the empty word is printed by modifying ",
  ButtonBox["$Post",
    ButtonStyle->"RefGuideLink"],
  "."
}], "Notes",
  CellTags->"FormatAutomata"],

Cell[TextData[{
  "See also ",
  ButtonBox["$shortlines",
    ButtonData:>"shortlines",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"FormatAutomata"],

Cell["Further Examples", "ExampleSection",
  CellTags->"FormatAutomata"],

Cell["The same DFA, displayed in four different ways. ", "ExampleText",
  CellTags->"FormatAutomata"],

Cell[BoxData[
    \(\(m = ToDFA[IthSymbolFA[a, \(-5\)], Normalize \[Rule] 1];\)\)], "Input",\

  CellLabel->"In[61]:=",
  CellTags->"FormatAutomata"],

Cell[BoxData[{
    \(FormatAutomata[None]\), "\n", 
    \(m\)}], "Input",
  CellLabel->"In[62]:=",
  CellTags->"FormatAutomata"],

Cell[BoxData[{
    \(FormatAutomata[Some]\), "\n", 
    \(m\)}], "Input",
  CellLabel->"In[64]:=",
  CellTags->"FormatAutomata"],

Cell[BoxData[{
    \(FormatAutomata[Automatic]\), "\n", 
    \(m\)}], "Input",
  CellLabel->"In[70]:=",
  CellTags->"FormatAutomata"],

Cell[BoxData[{
    \(FormatAutomata[SizeOnly]\), "\n", 
    \(m\)}], "Input",
  CellLabel->"In[72]:=",
  CellTags->"FormatAutomata"],

Cell[BoxData[{
    \(FormatAutomata[Full]\), "\n", 
    \(m\)}], "Input",
  CellLabel->"In[72]:=",
  CellTags->"FormatAutomata"],

Cell["Back to the default.", "ExampleText",
  CellTags->"FormatAutomata"],

Cell[BoxData[
    \(FormatAutomata[Automatic]\)], "Input",
  CellLabel->"In[74]:=",
  CellTags->"FormatAutomata"],

Cell["Full", "ObjectName",
  CellTags->"Full"],

Cell["Option", "Highlight",
  CellTags->"Full"],

Cell[TextData[{
  StyleBox["Full", "MR"],
  "  is an option for various functions."
}], "Usage",
  CellTags->"Full"],

Cell[TextData[{
  "See also ",
  ButtonBox["CycleDecompositionT",
    ButtonData:>"CycleDecompositionT",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["CyclicQGraph",
    ButtonData:>"CyclicQGraph",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["DegreeGraph",
    ButtonData:>"DegreeGraph",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["EquationsToWitnesses",
    ButtonData:>"EquationsToWitnesses",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["GenerateCSM",
    ButtonData:>"GenerateCSM",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["GenerateSG",
    ButtonData:>"GenerateSG",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["GridLine",
    ButtonData:>"GridLine",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["GridPoints",
    ButtonData:>"GridPoints",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["LongestPath",
    ButtonData:>"LongestPath",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["MakeAlphabet",
    ButtonData:>"MakeAlphabet",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["OrderT",
    ButtonData:>"OrderT",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["PathGraph",
    ButtonData:>"PathGraph",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["PlotFunction",
    ButtonData:>"PlotFunction",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["PlotRelation",
    ButtonData:>"PlotRelation",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["PlotTDGraph",
    ButtonData:>"PlotTDGraph",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["ProductGraph",
    ButtonData:>"ProductGraph",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["ShortestPath",
    ButtonData:>"ShortestPath",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["SimplifyEquations",
    ButtonData:>"SimplifyEquations",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["StaircasePlot",
    ButtonData:>"StaircasePlot",
    ButtonStyle->"AddOnsLink"],
  ". "
}], "Notes",
  CellTags->"Full"],

Cell["MinusEins", "ObjectName",
  CellTags->"MinusEins"],

Cell["Type: fctassg", "Highlight",
  CellTags->"MinusEins"],

Cell[TextData[{
  StyleBox["MinusEins", "MR"],
  "  returns the last ",
  "part of object ",
  StyleBox["x", "MR"],
  "."
}], "Usage",
  CellTags->"MinusEins"],

Cell[TextData[{
  "Thus ",
  StyleBox["MinusEins[x] = x[[-1]]", "MR"],
  "."
}], "Notes",
  CellTags->"MinusEins"],

Cell[TextData[{
  "See also ",
  ButtonBox["Zwei",
    ButtonData:>"Zwei",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Drei",
    ButtonData:>"Drei",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["MinusZwei",
    ButtonData:>"MinusZwei",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"MinusEins"],

Cell["MinusZwei", "ObjectName",
  CellTags->"MinusZwei"],

Cell["Type: fctassg", "Highlight",
  CellTags->"MinusZwei"],

Cell[TextData[{
  StyleBox["MinusZwei[x]", "MR"],
  "  returns the second to the last part of object ",
  StyleBox["x", "MR"],
  "."
}], "Usage",
  CellTags->"MinusZwei"],

Cell[TextData[{
  "Thus ",
  StyleBox["MinusZwei[x] = x[[-2]]", "MR"],
  "."
}], "Notes",
  CellTags->"MinusZwei"],

Cell[TextData[{
  "See also ",
  ButtonBox["Zwei",
    ButtonData:>"Zwei",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["Drei",
    ButtonData:>"Drei",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["MinusEins",
    ButtonData:>"MinusEins",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"MinusZwei"],

Cell["Opts", "ObjectName",
  CellTags->"Opts"],

Cell["Option", "Highlight",
  CellTags->"Opts"],

Cell[TextData[{
  StyleBox["Opts", "MR"],
  "  is an option for ",
  StyleBox["AssignFunction", "MR"],
  ". "
}], "Usage",
  CellTags->"Opts"],

Cell["Controls the attributes of the defined function.", "Notes",
  CellTags->"Opts"],

Cell[TextData[{
  "See ",
  ButtonBox["AssignFunction",
    ButtonData:>"AssignFunction",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Opts"],

Cell["PositionList", "ObjectName",
  CellTags->"PositionList"],

Cell["Function", "Highlight",
  CellTags->"PositionList"],

Cell[TextData[{
  StyleBox["PositionList[A,B,opts]", "MR"],
  "  generates a flattened list of the positions of the elements of list ",
  StyleBox["B", "MR"],
  " in list ",
  StyleBox["A", "MR"],
  ".  "
}], "Usage",
  CellTags->"PositionList"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Type", "\[Rule]", "None", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"PositionList"],

Cell[TextData[{
  "Option ",
  StyleBox["Type->Permutation", "MR"],
  "  can be used to speed up the computation when both lists are permutations \
over the same ground set."
}], "Notes",
  CellTags->"PositionList"],

Cell[TextData[{
  "All elements in ",
  StyleBox["B", "MR"],
  " must occur in ",
  StyleBox["A", "MR"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"PositionList"],

Cell["Further Examples", "ExampleSection",
  CellTags->"PositionList"],

Cell["\<\
A random list of integers, and the positions of the occurring \
elements.\
\>", "ExampleText",
  CellTags->"PositionList"],

Cell["A = Table[Random[Integer,{1,5}],{20}]", "Input",
  CellLabel->"In[513]:=",
  CellTags->"PositionList"],

Cell[BoxData[{
    \(\(B = \ Union[A];\)\), "\n", 
    \(PositionList[A, B]\)}], "Input",
  CellLabel->"In[519]:=",
  CellTags->"PositionList"],

Cell[BoxData[
    \(PositionList[A, Take[B, 1]]\)], "Input",
  CellLabel->"In[521]:=",
  CellTags->"PositionList"],

Cell[TextData[{
  "If both lists are permutations and long, the computation will be \
significantly faster using the option ",
  StyleBox["Type->Permution", "MR"],
  "."
}], "ExampleText",
  CellTags->"PositionList"],

Cell["A = RandomPermutation[8000];", "Input",
  CellLabel->"In[554]:=",
  CellTags->"PositionList"],

Cell[BoxData[
    \(\(p1\  = PositionList[Range[8000], A];\)\  // \ Timing\)], "Input",
  CellLabel->"In[555]:=",
  CellTags->"PositionList"],

Cell[BoxData[
    \(\(p2\  = \ 
          PositionList[Range[8000], A, Type \[Rule] Permutation\ ];\)\  // \ 
      Timing\)], "Input",
  CellLabel->"In[556]:=",
  CellTags->"PositionList"],

Cell[BoxData[
    \(p1 === p2\)], "Input",
  CellLabel->"In[557]:=",
  CellTags->"PositionList"],

Cell["PositionOne", "ObjectName",
  CellTags->"PositionOne"],

Cell["Function", "Highlight",
  CellTags->"PositionOne"],

Cell[TextData[{
  StyleBox["PositionOne[x,y]", "MR"],
  "  evaluates to ",
  StyleBox["Position[x,y][[1,1]]", "MR"],
  "."
}], "Usage",
  CellTags->"PositionOne"],

Cell["Used internally.", "Notes",
  CellTags->"PositionOne"],

Cell["Further Examples", "ExampleSection",
  CellTags->"PositionOne"],

Cell["\<\
Extraction the position of the first occurrence of an element in a \
list.\
\>", "ExampleText",
  CellTags->"PositionOne"],

Cell["A = Table[Random[Integer,{1,5}],{20}]", "Input",
  CellLabel->"In[472]:=",
  CellTags->"PositionOne"],

Cell[BoxData[
    \(PositionOne[A, 3]\)], "Input",
  CellLabel->"In[473]:=",
  CellTags->"PositionOne"],

Cell["ReplaceParts", "ObjectName",
  CellTags->"ReplaceParts"],

Cell["Function", "Highlight",
  CellTags->"ReplaceParts"],

Cell[TextData[{
  StyleBox["ReplaceParts[x,r1,p1,r2,p2]", "MR"],
  "  performs several consecutive replacements."
}], "Usage",
  CellTags->"ReplaceParts"],

Cell["Used internally.  Is polyadic.", "Notes",
  CellTags->"ReplaceParts"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ReplaceParts"],

Cell["Multiple replacements.", "ExampleText",
  CellTags->"ReplaceParts"],

Cell[BoxData[
    \(ReplaceParts[\ Range[5], a, 1, b, 2]\)], "Input",
  CellLabel->"In[137]:=",
  CellTags->"ReplaceParts"],

Cell["SelectOptions", "ObjectName",
  CellTags->"SelectOptions"],

Cell["Function", "Highlight",
  CellTags->"SelectOptions"],

Cell[TextData[{
  StyleBox["SelectOptions[cmd,opts]", "MR"],
  "  selects those options from ",
  StyleBox["opts", "MR"],
  " that make sense for ",
  StyleBox["cmd", "MR"],
  ". "
}], "Usage",
  CellTags->"SelectOptions"],

Cell[TextData[{
  StyleBox["SelectOptions[cmd,opts,{excl}]", "MR"],
  " excludes options whose names are listed in ",
  StyleBox["{excl}", "MR"],
  "."
}], "Notes",
  CellTags->"SelectOptions"],

Cell["Status", "ObjectName",
  CellTags->"Status"],

Cell["Function", "Highlight",
  CellTags->"Status"],

Cell[TextData[{
  StyleBox["Status[]", "MR"],
  "  displays some information about the ongoing session."
}], "Usage",
  CellTags->"Status"],

Cell["Further Examples", "ExampleSection",
  CellTags->"Status"],

Cell["A status message.", "ExampleText",
  CellTags->"Status"],

Cell["Status[]", "Input",
  CellLabel->"In[157]:=",
  CellTags->"Status"],

Cell["SymbolList", "ObjectName",
  CellTags->"SymbolList"],

Cell["Function", "Highlight",
  CellTags->"SymbolList"],

Cell[TextData[{
  StyleBox["SymbolList[sym,k]", "MR"],
  "  generates a list of symbols of the form ",
  StyleBox["sym1", "MR"],
  ", ",
  StyleBox["sym2", "MR"],
  ", ..., ",
  StyleBox["symk", "MR"],
  "."
}], "Usage",
  CellTags->"SymbolList"],

Cell[TextData[{
  StyleBox["k", "MR"],
  " can also be replaced by a list."
}], "Notes",
  CellTags->"SymbolList"],

Cell["Further Examples", "ExampleSection",
  CellTags->"SymbolList"],

Cell["A list of symbols.", "ExampleText",
  CellTags->"SymbolList"],

Cell["\<\
SymbolList[ \"fred\", 5 ]
% // FullForm\
\>", "Input",
  CellLabel->"In[162]:=",
  CellTags->"SymbolList"],

Cell["Zwei", "ObjectName",
  CellTags->"Zwei"],

Cell["Type: fctassg", "Highlight",
  CellTags->"Zwei"],

Cell[TextData[{
  StyleBox["Zwei[x]", "MR"],
  "  extracts the zweiten part of   ",
  StyleBox["x", "MR"],
  "."
}], "Usage",
  CellTags->"Zwei"],

Cell[TextData[{
  "Thus,  ",
  StyleBox["Zwei[x] = x[[2]]", "MR"],
  "."
}], "Notes",
  CellTags->"Zwei"],

Cell[TextData[{
  "See also ",
  ButtonBox["Drei",
    ButtonData:>"Drei",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["MinusEins",
    ButtonData:>"MinusEins",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["MinusZwei",
    ButtonData:>"MinusZwei",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"Zwei"],

Cell["$AutomataReleaseNumber", "ObjectName",
  CellTags->"$AutomataReleaseNumber"],

Cell["SysVariable", "Highlight",
  CellTags->"$AutomataReleaseNumber"],

Cell[TextData[{
  StyleBox["$AutomataReleaseNumber", "MR"],
  "   is the release number of the current Automata package."
}], "Usage",
  CellTags->"$AutomataReleaseNumber"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$AutomataReleaseNumber"],

Cell["This is release 5.0:", "ExampleText",
  CellTags->"$AutomataReleaseNumber"],

Cell[BoxData[
    \($AutomataReleaseNumber\)], "Input",
  CellLabel->"In[218]:=",
  CellTags->"$AutomataReleaseNumber"],

Cell["PostLoad", "ObjectName",
  CellTags->"PostLoad"],

Cell["PostLoad", "Highlight",
  CellTags->"PostLoad"],

Cell[TextData[{
  StyleBox["postload", "MR"],
  "  sets some system parameters. Is executed automatically after the package \
is loaded.  "
}], "Usage",
  CellTags->"PostLoad"],

Cell["PreLoad", "ObjectName",
  CellTags->"PreLoad"],

Cell["PreLoad", "Highlight",
  CellTags->"PreLoad"],

Cell[TextData[{
  StyleBox["preload", "MR"],
  "  defines load messages. Is executed automatically before the package is \
loaded."
}], "Usage",
  CellTags->"PreLoad"],

Cell["$deffatline", "ObjectName",
  CellTags->"$deffatline"],

Cell["SysVariable", "Highlight",
  CellTags->"$deffatline"],

Cell[TextData[{
  StyleBox["$deffatline", "MR"],
  "   controls the width of emphasized lines in ",
  StyleBox["PlotTDGraph", "MR"],
  ". "
}], "Usage",
  CellTags->"$deffatline"],

Cell[TextData[{
  "The default is 0.005 and can be changed via option ",
  StyleBox["LineStyle", "MR"],
  "."
}], "Notes",
  CellTags->"$deffatline"],

Cell[TextData[{
  "See also ",
  ButtonBox["PlotTDGraph",
    ButtonData:>"PlotTDGraph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"$deffatline"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$deffatline"],

Cell["The system default value is 0.005:", "ExampleText",
  CellTags->"$deffatline"],

Cell[BoxData[
    \($deffatline\)], "Input",
  CellLabel->"In[219]:=",
  CellTags->"$deffatline"],

Cell["$deflabeloffset", "ObjectName",
  CellTags->"$deflabeloffset"],

Cell["SysVariable", "Highlight",
  CellTags->"$deflabeloffset"],

Cell[TextData[{
  StyleBox["$deflabeloffset", "MR"],
  "   controls the position of labels in ",
  StyleBox["GridPoints", "MR"],
  ". "
}], "Usage",
  CellTags->"$deflabeloffset"],

Cell["Default value is 0.2.", "Notes",
  CellTags->"$deflabeloffset"],

Cell[TextData[{
  "See also ",
  ButtonBox["GridPoints",
    ButtonData:>"GridPoints",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["PlotTDGraph",
    ButtonData:>"PlotTDGraph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"$deflabeloffset"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$deflabeloffset"],

Cell["The system default value is 0.2:", "ExampleText",
  CellTags->"$deflabeloffset"],

Cell[BoxData[
    \($deflabeloffset\)], "Input",
  CellLabel->"In[220]:=",
  CellTags->"$deflabeloffset"],

Cell["$defpointsize", "ObjectName",
  CellTags->"$defpointsize"],

Cell["SysVariable", "Highlight",
  CellTags->"$defpointsize"],

Cell[TextData[{
  StyleBox["$defpointsize", "MR"],
  "  controls the pointsize in function ",
  StyleBox["GridPoints", "MR"],
  ". "
}], "Usage",
  CellTags->"$defpointsize"],

Cell[TextData[{
  "The default is 0.02 and can be changed via option ",
  StyleBox["PlotStyle", "MR"],
  "."
}], "Notes",
  CellTags->"$defpointsize"],

Cell[TextData[{
  "See also ",
  ButtonBox["GridPoints",
    ButtonData:>"GridPoints",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["PlotTDGraph",
    ButtonData:>"PlotTDGraph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"$defpointsize"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$defpointsize"],

Cell[TextData[{
  "Controlling point size in ",
  StyleBox["GridPoints", "MR"],
  "."
}], "ExampleText",
  CellTags->"$defpointsize"],

Cell["GridPoints[4,4] // Show;", "Input",
  CellLabel->"In[221]:=",
  CellTags->"$defpointsize"],

Cell[BoxData[
    \(\(Block[{$defpointsize = 0.05}, GridPoints[4, 4]] // Show;\)\)], "Input",\

  CellLabel->"In[222]:=",
  CellTags->"$defpointsize"],

Cell["$defthinline", "ObjectName",
  CellTags->"$defthinline"],

Cell["SysVariable", "Highlight",
  CellTags->"$defthinline"],

Cell[TextData[{
  StyleBox["$defthinline", "MR"],
  " controls the width of plain lines in ",
  StyleBox["PlotTDGraph", "MR"],
  ". "
}], "Usage",
  CellTags->"$defthinline"],

Cell[TextData[{
  "The default is 0.005 and can be changed via option ",
  StyleBox["LineStyle", "MR"],
  "."
}], "Notes",
  CellTags->"$defthinline"],

Cell[TextData[{
  "See also ",
  ButtonBox["GridLine",
    ButtonData:>"GridLine",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["PlotTDGraph",
    ButtonData:>"PlotTDGraph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"$defthinline"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$defthinline"],

Cell[TextData[{
  "Controlling line thickness in ",
  StyleBox["GridLine", "MR"],
  "."
}], "ExampleText",
  CellTags->"$defthinline"],

Cell["GridLine[4,4] // Show;", "Input",
  CellLabel->"In[100]:=",
  CellTags->"$defthinline"],

Cell["\<\
Block[{$defthinline=0.02},
GridLine[4,4]] // Show;\
\>", "Input",
  CellLabel->"In[101]:=",
  CellTags->"$defthinline"],

Cell["\<\
Block[{$defthinline=0.02},
GridLine[4,4]] // Show;\
\>", "Input",
  CellLabel->"In[101]:=",
  CellTags->"$defthinline"],

Cell["$plotrangeoffset", "ObjectName",
  CellTags->"$plotrangeoffset"],

Cell["SysVariable", "Highlight",
  CellTags->"$plotrangeoffset"],

Cell[TextData[{
  StyleBox["$plotrangeoffset", "MR"],
  "   controls the offset for ",
  StyleBox["MakePlotRange", "MR"],
  ". "
}], "Usage",
  CellTags->"$plotrangeoffset"],

Cell["Default value is 0.8.", "Notes",
  CellTags->"$plotrangeoffset"],

Cell[TextData[{
  "See ",
  ButtonBox["MakePlotRange",
    ButtonData:>"MakePlotRange",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"$plotrangeoffset"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$plotrangeoffset"],

Cell["The default value is 0.8:", "ExampleText",
  CellTags->"$plotrangeoffset"],

Cell[BoxData[
    \($plotrangeoffset\)], "Input",
  CellLabel->"In[223]:=",
  CellTags->"$plotrangeoffset"],

Cell["$rastercolors", "ObjectName",
  CellTags->"$rastercolors"],

Cell["SysVariable", "Highlight",
  CellTags->"$rastercolors"],

Cell[TextData[{
  StyleBox["$rastercolors", "MR"],
  "   is a list of rules that determines plot colors for raster array plots. \
"
}], "Usage",
  CellTags->"$rastercolors"],

Cell[TextData[{
  "By default, only matrices whose elements are integers in the range ",
  Cell[BoxData[
      \(TraditionalForm\`\([0, 9]\)\)]],
  " can be rendered. "
}], "Notes",
  CellTags->"$rastercolors"],

Cell[TextData[{
  "See also  ",
  ButtonBox["PlotMatrix",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["EvolutionCA",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["RasterStyle",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["HueRasterColors",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"$rastercolors"],

Cell["Further Examples", "ExampleSection",
  CellTags->"$rastercolors"],

Cell[TextData[{
  "The default values are quite arbitrary, but seem to produce reasonable \
pictures for typical input to ",
  StyleBox["PlotMatrix", "MR"],
  ". "
}], "ExampleText",
  CellTags->"$rastercolors"],

Cell[BoxData[
    \($rastercolors\)], "Input",
  CellLabel->"In[36]:=",
  CellTags->"$rastercolors"],

Cell[BoxData[
    \(\(PlotMatrix[Partition[Range[0, 9], 5]];\)\)], "Input",
  CellLabel->"In[37]:=",
  CellTags->"$rastercolors"],

Cell["\<\
For generating hard copy, it may be preferable to replace colors by \
black and white only.\
\>", "ExampleText",
  CellTags->"$rastercolors"],

Cell[BoxData[
    \(\(EvolutionCA[\ CA[90], \ 
        RasterStyle \[Rule] {0 \[Rule] White, 1 \[Rule] Black}];\)\)], "Input",\

  CellLabel->"In[38]:=",
  CellTags->"$rastercolors"],

Cell["ColorQ", "ObjectName",
  CellTags->"ColorQ"],

Cell["Function", "Highlight",
  CellTags->"ColorQ"],

Cell[TextData[{
  StyleBox["ColorQ[x]", "MR"],
  "  tests whether ",
  StyleBox["x", "MR"],
  " is a color specification."
}], "Usage",
  CellTags->"ColorQ"],

Cell["Used internally in some plotting commands. ", "Notes",
  CellTags->"ColorQ"],

Cell[TextData[{
  "See ",
  ButtonBox["GridLine",
    ButtonData:>"GridLine",
    ButtonStyle->"AddOnsLink"],
  StyleBox[",", "Text"],
  " ",
  ButtonBox["PlotTDGraph",
    ButtonData:>"PlotTDGraph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ColorQ"],

Cell["DoShow", "ObjectName",
  CellTags->"DoShow"],

Cell["Function", "Highlight",
  CellTags->"DoShow"],

Cell[TextData[{
  StyleBox["DoShow[gr]", "MR"],
  "  displays graphics object ",
  StyleBox["gr", "MR"],
  " by setting the ",
  StyleBox["DisplayFunction", "MR"],
  " to ",
  StyleBox["$DisplayFunction", "MR"],
  ". "
}], "Usage",
  CellTags->"DoShow"],

Cell[TextData[{
  "Used in conjunction with ",
  StyleBox["DisplayFunction->Identity", "MR"],
  "."
}], "Notes",
  CellTags->"DoShow"],

Cell[TextData[{
  "See also ",
  ButtonBox["ShowArray",
    ButtonData:>"ShowArray",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"DoShow"],

Cell["Further Examples", "ExampleSection",
  CellTags->"DoShow"],

Cell["Postponing the display of graphics.", "ExampleText",
  CellTags->"DoShow"],

Cell[BoxData[
    \(\(gr = 
        PlotT[T[2, 3, 4, 1, 4, 5, 6], 5, 
          DisplayFunction \[Rule] Identity];\)\)], "Input",
  CellLabel->"In[140]:=",
  CellTags->"DoShow"],

Cell[BoxData[
    \(\(DoShow[gr];\)\)], "Input",
  CellLabel->"In[141]:=",
  CellTags->"DoShow"],

Cell["GridLine", "ObjectName",
  CellTags->"GridLine"],

Cell["Function", "Highlight",
  CellTags->"GridLine"],

Cell[TextData[{
  StyleBox["GridLine[n,m]", "MR"],
  "   generates a graphics object representing an ",
  StyleBox["n", "MR"],
  " by ",
  StyleBox["m", "MR"],
  " grid of points. "
}], "Usage",
  CellTags->"GridLine"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"PlotStyle", "\[Rule]", \({GrayLevel[0.8], $defthinline}\), "-"},
          {"GridOffset", "\[Rule]", \({0, 0}\), "-"},
          {"Full", "\[Rule]", "All", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"GridLine"],

Cell["Line size and color can be controlled by options.", "Notes",
  CellTags->"GridLine"],

Cell[TextData[{
  "See also ",
  ButtonBox["GridPoints",
    ButtonData:>"GridPoints",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["PlotTDGraph",
    ButtonData:>"PlotTDGraph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"GridLine"],

Cell["Further Examples", "ExampleSection",
  CellTags->"GridLine"],

Cell["The basic grid.", "ExampleText",
  CellTags->"GridLine"],

Cell[BoxData[
    \(\(GridLine[5, 5]\  // \ Show;\)\)], "Input",
  CellLabel->"In[232]:=",
  CellTags->"GridLine"],

Cell["Using options to control details.", "ExampleText",
  CellTags->"GridLine"],

Cell[BoxData[{
    \(\(gr1 = GridLine[4, 5];\)\), "\n", 
    \(\(gr2 = 
        GridLine[4, 5, PlotStyle \[Rule] {Red, 0.02}, 
          Full \[Rule] {{1, 1}, {1, 2}, {2, 2}, {2, 3}, {3, 3}, {4, 3}, {4, 
                4}, {5, 4}}];\)\)}], "Input",
  CellLabel->"In[233]:=",
  AspectRatioFixed->True,
  CellTags->"GridLine"],

Cell["Show[gr1,gr2];", "Input",
  CellLabel->"In[235]:=",
  CellTags->"GridLine"],

Cell["GridOffset", "ObjectName",
  CellTags->"GridOffset"],

Cell["Option", "Highlight",
  CellTags->"GridOffset"],

Cell[TextData[{
  StyleBox["GridOffset", "MR"],
  "  is an option for ",
  StyleBox["GridLine", "MR"],
  "."
}], "Usage",
  CellTags->"GridOffset"],

Cell[TextData[{
  "Determines starting coordinates for the grid. Default value is   ",
  StyleBox["{1,1}", "MR"],
  "."
}], "Notes",
  CellTags->"GridOffset"],

Cell[TextData[{
  "See also ",
  ButtonBox["GridLine",
    ButtonData:>"GridLine",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"GridOffset"],

Cell["GridPoints", "ObjectName",
  CellTags->"GridPoints"],

Cell["Function", "Highlight",
  CellTags->"GridPoints"],

Cell[TextData[{
  StyleBox["GridPoints[n,m]", "MR"],
  "  generates a graphics object representing an ",
  StyleBox["n", "MR"],
  " by ",
  StyleBox["m", "MR"],
  " grid of points. "
}], "Usage",
  CellTags->"GridPoints"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"LabelGrid", "\[Rule]", "None", "-"},
          {"Full", "\[Rule]", "All", "-"},
          {"LabelPoints", "\[Rule]", "None", "-"},
          {"PlotStyle", 
            "\[RuleDelayed]", \({{Blue, $defpointsize}, {Red, 
                1.2  $defpointsize}}\), "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"GridPoints"],

Cell["Dot size and color can be controlled by options.", "Notes",
  CellTags->"GridPoints"],

Cell[TextData[{
  "See also ",
  ButtonBox["GridLine",
    ButtonData:>"GridLine",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["PlotTDGraph",
    ButtonData:>"PlotTDGraph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"GridPoints"],

Cell["Further Examples", "ExampleSection",
  CellTags->"GridPoints"],

Cell["The basic image.", "ExampleText",
  CellTags->"GridPoints"],

Cell[BoxData[
    \(\(GridPoints[5, 5]\  // \ Show;\)\)], "Input",
  CellLabel->"In[835]:=",
  AspectRatioFixed->True,
  CellTags->"GridPoints"],

Cell["Cum bells and whistles.", "ExampleText",
  CellTags->"GridPoints"],

Cell[BoxData[
    \(\(GridPoints[20, 20, Full \[Rule] \((Mod[#1 + #2\^2, 7] == 0 &)\), 
          PlotStyle \[Rule] {{Cyan,  .02}, {Yellow,  .03}}, 
          Frame \[Rule] True, FrameTicks \[Rule] None, 
          Background \[Rule] LightSkyBlue]\ \  // \ Show;\)\)], "Input",
  CellLabel->"In[836]:=",
  AspectRatioFixed->True,
  CellTags->"GridPoints"],

Cell["HueRasterColors", "ObjectName",
  CellTags->"HueRasterColors"],

Cell["Function", "Highlight",
  CellTags->"HueRasterColors"],

Cell[TextData[{
  StyleBox["HueRasterColors[p]", "MR"],
  "  can be used in ",
  StyleBox["PlotMatrix", "MR"],
  " to control colors. "
}], "Usage",
  CellTags->"HueRasterColors"],

Cell[TextData[{
  "Provides an alternative to ",
  StyleBox["$rastercolor", "MR"],
  "."
}], "Notes",
  CellTags->"HueRasterColors"],

Cell[TextData[{
  "See also ",
  ButtonBox["PlotMatrix",
    ButtonData:>"PlotMatrix",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["EvolutionCA",
    ButtonData:>"EvolutionCA",
    ButtonStyle->"AddOnsLink"],
  StyleBox[", ", "Text"],
  ButtonBox["RasterStyle",
    ButtonData:>"RasterStyle",
    ButtonStyle->"AddOnsLink"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"HueRasterColors"],

Cell["Further Examples", "ExampleSection",
  CellTags->"HueRasterColors"],

Cell["Exclusive or, in hues.", "ExampleText",
  CellTags->"HueRasterColors"],

Cell[BoxData[
    \(\(m = Table[BitXor[i, j], {i, 0, 31}, {j, 0, 31}];\)\)], "Input",
  CellLabel->"In[234]:=",
  CellTags->"HueRasterColors"],

Cell[BoxData[
    \(\(PlotMatrix[m, RasterStyle \[Rule] HueRasterColors[32]];\)\)], "Input",\

  CellLabel->"In[236]:=",
  CellTags->"HueRasterColors"],

Cell["LabelGrid", "ObjectName",
  CellTags->"LabelGrid"],

Cell["Option", "Highlight",
  CellTags->"LabelGrid"],

Cell[TextData[{
  StyleBox["LabelGrid", "MR"],
  "  is an option for ",
  StyleBox["GridPoints", "MR"],
  ", causes the first and last column of grid points to be labeled by \
integers."
}], "Usage",
  CellTags->"LabelGrid"],

Cell[TextData[{
  "See ",
  ButtonBox["GridPoints",
    ButtonData:>"GridPoints",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"LabelGrid"],

Cell["LabelPoints", "ObjectName",
  CellTags->"LabelPoints"],

Cell["Option", "Highlight",
  CellTags->"LabelPoints"],

Cell[TextData[{
  StyleBox["LabelPoints", "MR"],
  "  is an option for ",
  StyleBox["GridPoints", "MR"],
  ", causes all grid points to be labeled by integers."
}], "Usage",
  CellTags->"LabelPoints"],

Cell[TextData[{
  " Default is ",
  StyleBox["LabelPoints->None", "MR"],
  ", change to ",
  StyleBox["LabelPoints->labelingfct", "MR"],
  "."
}], "Notes",
  CellTags->"LabelPoints"],

Cell[TextData[{
  "See ",
  ButtonBox["GridPoints",
    ButtonData:>"GridPoints",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"LabelPoints"],

Cell["LineStyle", "ObjectName",
  CellTags->"LineStyle"],

Cell["Option", "Highlight",
  CellTags->"LineStyle"],

Cell[TextData[{
  StyleBox["LineStyle", "MR"],
  "  is an option for ",
  StyleBox["PlotTDGraph", "MR"],
  " that controls thickness and color of the lines in the plot."
}], "Usage",
  CellTags->"LineStyle"],

Cell[TextData[{
  "Usage: ",
  StyleBox["LineStyle->{{col1,thick1},{col2,thick2}}", "MR"],
  "."
}], "Notes",
  CellTags->"LineStyle"],

Cell[TextData[{
  "See ",
  ButtonBox["PlotTDGraph",
    ButtonData:>"PlotTDGraph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"LineStyle"],

Cell["MakePlotRange", "ObjectName",
  CellTags->"MakePlotRange"],

Cell["Function", "Highlight",
  CellTags->"MakePlotRange"],

Cell[TextData[{
  StyleBox["MakePlotRange[x1,x2,y1,y2]", "MR"],
  "  constructs data for the ",
  StyleBox["PlotRange", "MR"],
  " option, adds proper margins. "
}], "Usage",
  CellTags->"MakePlotRange"],

Cell["Used mostly internally in plotting functions.", "Notes",
  CellTags->"MakePlotRange"],

Cell[TextData[{
  "See also  ",
  ButtonBox["$plotrangeoffset",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"MakePlotRange"],

Cell["MultiCol", "ObjectName",
  CellTags->"MultiCol"],

Cell["Option", "Highlight",
  CellTags->"MultiCol"],

Cell[TextData[{
  StyleBox["MultiCol", "MR"],
  "  is an option for ",
  StyleBox["PlotMatrix", "MR"],
  "."
}], "Usage",
  CellTags->"MultiCol"],

Cell[TextData[{
  "See ",
  ButtonBox["PlotMatrix",
    ButtonData:>"PlotMatrix",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"MultiCol"],

Cell["NRows", "ObjectName",
  CellTags->"NRows"],

Cell["Option", "Highlight",
  CellTags->"NRows"],

Cell[TextData[{
  StyleBox["NRows", "MR"],
  "  is an option for ",
  StyleBox["PlotMatrix", "MR"],
  "."
}], "Usage",
  CellTags->"NRows"],

Cell[TextData[{
  "See ",
  ButtonBox["PlotMatrix",
    ButtonData:>"PlotMatrix",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"NRows"],

Cell["PlotComputationFA", "ObjectName",
  CellTags->"PlotComputationFA"],

Cell["Function", "Highlight",
  CellTags->"PlotComputationFA"],

Cell[TextData[{
  StyleBox["PlotComputationFA[M,w,opts]", "MR"],
  "  graphs the computation of FSM  ",
  StyleBox["M", "MR"],
  "  on input word ",
  StyleBox["w", "MR"],
  ". "
}], "Usage",
  CellTags->"PlotComputationFA"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"DoTrace", "\[Rule]", "Automatic", "-"},
          {"LabelGrid", "\[Rule]", "None", "-"},
          {"Full", "\[Rule]", "False", "-"},
          {"Frame", "\[Rule]", "True", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"PlotComputationFA"],

Cell[TextData[{
  "Option ",
  StyleBox["Full->True", "MR"],
  " causes all applicable transitions to be displayed (rather than just \
persistent branches in the computation)."
}], "Notes",
  CellTags->"PlotComputationFA"],

Cell[TextData[{
  "Option ",
  StyleBox["DoTrace->I", "MR"],
  "  treats ",
  StyleBox["I", "MR"],
  " as the set of initial states."
}], "Notes",
  CellTags->"PlotComputationFA"],

Cell[TextData[{
  "See also ",
  ButtonBox["ComputationFA",
    ButtonData:>"ComputationFA",
    ButtonStyle->"AddOnsLink"],
  "  and  ",
  ButtonBox["ComputationGraphFA",
    ButtonData:>"ComputationGraphFA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"PlotComputationFA"],

Cell["Further Examples", "ExampleSection",
  CellTags->"PlotComputationFA"],

Cell[TextData[{
  "A machine for the language of all words whose 4th symbol from the end is \
an ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  ". "
}], "ExampleText",
  CellTags->"PlotComputationFA"],

Cell[BoxData[
    \(m = IthSymbolFA[a, \(-4\)]\)], "Input",
  CellLabel->"In[120]:=",
  CellTags->"PlotComputationFA"],

Cell["\<\
There is exactly one accepting computation in this machine for each \
word in the language.\
\>", "ExampleText",
  CellTags->"PlotComputationFA"],

Cell[BoxData[
    \(\(PlotComputationFA[m, aabaa\ ];\)\)], "Input",
  CellLabel->"In[123]:=",
  CellTags->"PlotComputationFA"],

Cell["\<\
Here all the unsuccessful attempts are included in the \
picture.\
\>", "ExampleText",
  CellTags->"PlotComputationFA"],

Cell[BoxData[
    \(\(PlotComputationFA[m, aabaa, Full \[Rule] True, 
        LabelGrid \[Rule] Automatic\ ];\)\)], "Input",
  CellLabel->"In[122]:=",
  CellTags->"PlotComputationFA"],

Cell["PlotFunction", "ObjectName",
  CellTags->"PlotFunction"],

Cell["Function", "Highlight",
  CellTags->"PlotFunction"],

Cell[TextData[{
  StyleBox["PlotFunction[f,A,n,opts]", "MR"],
  "  plots a picture of a function ",
  StyleBox["f", "MR"],
  " on domain ",
  StyleBox["A", "MR"],
  ", iterated ",
  StyleBox["n", "MR"],
  " steps. "
}], "Usage",
  CellTags->"PlotFunction"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Frame", "\[Rule]", "True", "-"},
          {"Full", "\[Rule]", "False", "-"},
          {"DoTrace", "\[Rule]", "All", "-"},
          {"LineStyle", 
            "\[RuleDelayed]", \({{GrayLevel[ .5], $defthinline}, {Black, \
$deffatline}}\), "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"PlotFunction"],

Cell[TextData[{
  " See also ",
  ButtonBox["PlotTDGraph",
    ButtonData:>"PlotTDGraph",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"PlotFunction"],

Cell["Further Examples", "ExampleSection",
  CellTags->"PlotFunction"],

Cell["A modular polynomial, iterated 5 times.", "ExampleText",
  CellTags->"PlotFunction"],

Cell[BoxData[{
    \(\(f[x_] := Mod[x\^2 + 1, 11];\)\), "\n", 
    \(\(PlotFunction[f, Range[0, 10], 5];\)\)}], "Input",
  CellLabel->"In[236]:=",
  CellTags->"PlotFunction"],

Cell["PlotMatrix", "ObjectName",
  CellTags->"PlotMatrix"],

Cell["Function", "Highlight",
  CellTags->"PlotMatrix"],

Cell[TextData[{
  StyleBox["PlotMatrix[M,opts]", "MR"],
  "  plots a raster image of matrix ",
  StyleBox["M", "MR"],
  ". "
}], "Usage",
  CellTags->"PlotMatrix"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Frame", "\[Rule]", "True", "-"},
          {"GridLines", "\[Rule]", "False", "-"},
          {"MultiCol", "\[Rule]", "False", "-"},
          {"NRows", "\[Rule]", "30", "-"},
          {"RView", "\[Rule]", "False", "-"},
          {"NoRest", "\[Rule]", "False", "-"},
          {"RasterStyle", "\[RuleDelayed]", "$rastercolors", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"PlotMatrix"],

Cell[TextData[{
  "The default colors are controlled by ",
  StyleBox["$rastercolors", "MR"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"PlotMatrix"],

Cell[TextData[{
  "Option ",
  StyleBox["RasterStyle->Automatic", "MR"],
  " uses HueRasterColors. "
}], "Notes",
  CellTags->"PlotMatrix"],

Cell[TextData[{
  "Option ",
  StyleBox["GridLines->True", "MR"],
  " superimposes a light gray grid. "
}], "Notes",
  CellTags->"PlotMatrix"],

Cell[TextData[{
  "Option ",
  StyleBox["Frame->False", "MR"],
  " turns off the surrounding frame. "
}], "Notes",
  CellTags->"PlotMatrix"],

Cell[TextData[{
  "Option ",
  StyleBox["MultiCol->Turns", "MR"],
  " turns on multi-column mode and chops the matrix into several blocks. "
}], "Notes",
  CellTags->"PlotMatrix"],

Cell[TextData[{
  "Default number of rows in each block (except perhaps the last) is 30, can \
be changed with option ",
  StyleBox["NRow->nrows", "MR"],
  ". "
}], "Notes",
  CellTags->"PlotMatrix"],

Cell[TextData[{
  "Option ",
  StyleBox["RView->True", "MR"],
  " rotates the blocks sideways, and prints them in a single column. "
}], "Notes",
  CellTags->"PlotMatrix"],

Cell[TextData[{
  "Option ",
  StyleBox["NoRest->True", "MR"],
  " supresses trailing blocks of size less than ",
  StyleBox["NRows", "MR"],
  "."
}], "Notes",
  CellTags->"PlotMatrix"],

Cell[TextData[{
  "See also ",
  ButtonBox["$rastercolors",
    ButtonData:>"$rastercolors",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["HueRasterColors",
    ButtonData:>"HueRasterColors",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["EvolutionCA",
    ButtonData:>"EvolutionCA",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"PlotMatrix"],

Cell["Further Examples", "ExampleSection",
  CellTags->"PlotMatrix"],

Cell["A random matrix, and its visual representation.", "ExampleText",
  CellTags->"PlotMatrix"],

Cell[TextData[{
  "M = Table[Random[Integer,{0,3}],{10},{10}];",
  "\n",
  Cell[BoxData[
      \(\(PlotMatrix[M];\)\)], "Input",
    CellLabel->"In[438]:="]
}], "Input",
  CellLabel->"In[440]:=",
  CellTags->"PlotMatrix"],

Cell["Same picture with grid lines. ", "ExampleText",
  CellTags->"PlotMatrix"],

Cell[BoxData[
    \(\(PlotMatrix[M, GridLines \[Rule] True];\)\)], "Input",
  CellLabel->"In[442]:=",
  CellTags->"PlotMatrix"],

Cell["Chopping up a large matrix into pieces.", "ExampleText",
  CellTags->"PlotMatrix"],

Cell[BoxData[{
    \(\(orb\  = \ 
        OrbitCA[\ CA[90], \ SeedConfiguration[{1}, 21], \ 199, \ 
          Boundary \[Rule] Fixed\ ];\)\), "\n", 
    \(\(PlotMatrix[\ orb, \ MultiCol \[Rule] True, \ 
        NRows \[Rule] 50];\)\)}], "Input",
  CellLabel->"In[443]:=",
  CellTags->"PlotMatrix"],

Cell["PlotRelation", "ObjectName",
  CellTags->"PlotRelation"],

Cell["Function", "Highlight",
  CellTags->"PlotRelation"],

Cell[TextData[{
  StyleBox["PlotRelation[R,A,n,opts]", "MR"],
  "  plots a picture of a binary relation ",
  StyleBox["R", "MR"],
  " on domain ",
  StyleBox["A", "MR"],
  ", iterated ",
  StyleBox["n", "MR"],
  " steps. "
}], "Usage",
  CellTags->"PlotRelation"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"DoTrace", "\[Rule]", "All", "-"},
          {"Full", "\[Rule]", "False", "-"},
          {"PlotStyle", 
            "\[RuleDelayed]", \({{Blue, $defpointsize}, {Red, 
                1.2  $defpointsize}}\), "-"},
          {"LineStyle", 
            "\[RuleDelayed]", \({{GrayLevel[ .5], $defthinline}, {Black, \
$deffatline}}\), "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"PlotRelation"],

Cell[TextData[{
  "See also ",
  ButtonBox["PlotTDGraph",
    ButtonData:>"PlotTDGraph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["PlotT",
    ButtonData:>"PlotT",
    ButtonStyle->"AddOnsLink"],
  ". "
}], "Notes",
  CellTags->"PlotRelation"],

Cell["Further Examples", "ExampleSection",
  CellTags->"PlotRelation"],

Cell["A relation based on squaring modulo 20.", "ExampleText",
  CellTags->"PlotRelation"],

Cell[BoxData[
    \(\(\[Rho][x_, y_]\  := \ Mod[x\^2, 20]\  \[Equal] \ y;\)\)], "Input",
  CellLabel->"In[167]:=",
  CellTags->"PlotRelation"],

Cell[BoxData[
    \(\(PlotRelation[\ \[Rho], \ Range[19], 3\ ];\)\)], "Input",
  CellLabel->"In[169]:=",
  CellTags->"PlotRelation"],

Cell["PlotTDGraph", "ObjectName",
  CellTags->"PlotTDGraph"],

Cell["Function", "Highlight",
  CellTags->"PlotTDGraph"],

Cell[TextData[{
  StyleBox["PlotTDGraph[gr,n]", "MR"],
  "  plots a picture of a time dependent graph ",
  StyleBox["gr", "MR"],
  " of order ",
  StyleBox["n", "MR"],
  "."
}], "Usage",
  CellTags->"PlotTDGraph"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"DoTrace", "\[Rule]", "All", "-"},
          {"Full", "\[Rule]", "False", "-"},
          {"PlotStyle", 
            "\[RuleDelayed]", \({{Blue, $defpointsize}, {Red, 
                1.2  $defpointsize}}\), "-"},
          {"LineStyle", 
            "\[RuleDelayed]", \({{GrayLevel[ .5], $defthinline}, {Black, \
$deffatline}}\), "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"PlotTDGraph"],

Cell[TextData[{
  "Option are ",
  StyleBox["Full->False", "MR"],
  StyleBox[" shows all potential paths.", "Text"]
}], "Notes",
  CellTags->"PlotTDGraph"],

Cell[TextData[{
  "Option are ",
  StyleBox["DoTrace->A", "MR"],
  StyleBox["  only follows paths starting at vertices in ", "Text"],
  StyleBox["A", "MR"],
  StyleBox[". ", "Text"]
}], "Notes",
  CellTags->"PlotTDGraph"],

Cell[TextData[{
  "See also ",
  ButtonBox["ToTDGraph",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["PlotFunction",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["GridPoints",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["PlotT",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"PlotTDGraph"],

Cell["Further Examples", "ExampleSection",
  CellTags->"PlotTDGraph"],

Cell[TextData[{
  "Two relations based on proper divisibility on ",
  Cell[BoxData[
      \(TraditionalForm\`{1, 2, \[Ellipsis], 8}\)]],
  ". "
}], "ExampleText",
  CellTags->"PlotTDGraph"],

Cell[BoxData[{
    \(\(R1 = 
        Select[Tuples[8, 2], 
          Mod[#\[LeftDoubleBracket]2\[RightDoubleBracket], #\
\[LeftDoubleBracket]1\[RightDoubleBracket]] == 
                0 && #\[LeftDoubleBracket]1\[RightDoubleBracket] \[NotEqual] \
#\[LeftDoubleBracket]2\[RightDoubleBracket] &];\)\), "\n", 
    \(\(R2 = Reverse /@ R1;\)\), "\n", 
    \(g = ToTDGraph[{R1, R2}, 8]\)}], "Input",
  CellLabel->"In[329]:=",
  CellTags->"PlotTDGraph"],

Cell[BoxData[
    \(\(PlotTDGraph[g, 8];\)\)], "Input",
  CellLabel->"In[332]:=",
  CellTags->"PlotTDGraph"],

Cell[BoxData[
    \(\(PlotTDGraph[g, 8, \ DoTrace \[Rule] {3}, \ 
        Full \[Rule] True\ ];\)\)], "Input",
  CellLabel->"In[335]:=",
  CellTags->"PlotTDGraph"],

Cell["PlotT", "ObjectName",
  CellTags->"PlotT"],

Cell["Function", "Highlight",
  CellTags->"PlotT"],

Cell[TextData[{
  StyleBox["PlotT[t,n]", "MR"],
  "  plots the first ",
  StyleBox["n", "MR"],
  " iterations of transformation ",
  StyleBox["t", "MR"],
  ". "
}], "Usage",
  CellTags->"PlotT"],

Cell[TextData[{
  "Options are passed on to ",
  ButtonBox["PlotFunction",
    ButtonStyle->"AddOnsLink"],
  " and ",
  ButtonBox["PlotRelation",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"PlotT"],

Cell[TextData[{
  "Default for ",
  StyleBox["n", "MR"],
  " is 8."
}], "Notes",
  CellTags->"PlotT"],

Cell[TextData[{
  "See also ",
  ButtonBox["T",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["PlotTDGraph",
    ButtonStyle->"AddOnsLink"],
  ",  ",
  ButtonBox["GridPoints",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"PlotT"],

Cell["Further Examples", "ExampleSection",
  CellTags->"PlotT"],

Cell["A random transformation and its visual representation.", "ExampleText",
  CellTags->"PlotT"],

Cell["t = RandomT[10]", "Input",
  CellLabel->"In[811]:=",
  CellTags->"PlotT"],

Cell[BoxData[
    \(\(PlotT[t];\)\)], "Input",
  CellLabel->"In[812]:=",
  CellTags->"PlotT"],

Cell["Tinkering with options.", "ExampleText",
  CellTags->"PlotT"],

Cell[BoxData[
    \(\(PlotT[t, 5, \ Frame \[Rule] True, \ 
        Background \[Rule] LightYellow\ ];\)\)], "Input",
  CellLabel->"In[813]:=",
  CellTags->"PlotT"],

Cell["RasterStyle", "ObjectName",
  CellTags->"RasterStyle"],

Cell["Option", "Highlight",
  CellTags->"RasterStyle"],

Cell[TextData[{
  StyleBox["RasterStyle", "MR"],
  "  is an option for ",
  StyleBox["PlotMatrix", "MR"],
  ", determines the colors used in the raster graphics. "
}], "Usage",
  CellTags->"RasterStyle"],

Cell[TextData[{
  "Default is ",
  StyleBox["$rastercolors", "MR"],
  ", ",
  StyleBox["RasterStyle->Automatic", "MR"],
  " uses ",
  StyleBox["HueRasterColors", "MR"],
  "."
}], "Notes",
  CellTags->"RasterStyle"],

Cell[TextData[{
  "See also ",
  ButtonBox["PlotMatrix",
    ButtonData:>"PlotMatrix",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["HueRasterColors",
    ButtonData:>"HueRasterColors",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"RasterStyle"],

Cell["RView", "ObjectName",
  CellTags->"RView"],

Cell["Option", "Highlight",
  CellTags->"RView"],

Cell[TextData[{
  StyleBox["RView", "MR"],
  "  is an option for ",
  StyleBox["PlotMatrix", "MR"],
  "."
}], "Usage",
  CellTags->"RView"],

Cell[TextData[{
  "See ",
  ButtonBox["PlotMatrix",
    ButtonData:>"PlotMatrix",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"RView"],

Cell["ShowArray", "ObjectName",
  CellTags->"ShowArray"],

Cell["Function", "Highlight",
  CellTags->"ShowArray"],

Cell[TextData[{
  StyleBox["ShowArray[L,k]", "MR"],
  "  produces a graphics array of width ",
  StyleBox["k", "MR"],
  " from a linear list ",
  StyleBox["L", "MR"],
  " of graphics objects. "
}], "Usage",
  CellTags->"ShowArray"],

Cell[TextData[{
  "Parameter ",
  StyleBox["k", "MR"],
  " has the default value ",
  StyleBox["Ceiling[Sqrt[Length[L]]]", "MR"],
  StyleBox[".", "Text"]
}], "Notes",
  CellTags->"ShowArray"],

Cell[TextData[{
  "See also ",
  ButtonBox["PlotMatrix",
    ButtonData:>"PlotMatrix",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"ShowArray"],

Cell["Further Examples", "ExampleSection",
  CellTags->"ShowArray"],

Cell["Arranging graphics objects in a grid. ", "ExampleText",
  CellTags->"ShowArray"],

Cell[BoxData[
    \(\(gr = \((PlotT[#1, 4, PlotStyle \[Rule] {0.05, 0.05}, 
                DisplayFunction \[Rule] Identity] &)\) /@ 
          ToList[SymmetricGroupSG[3]];\)\)], "Input",
  CellLabel->"In[238]:=",
  CellTags->"ShowArray"],

Cell[BoxData[
    \(\(ShowArray[gr];\)\)], "Input",
  CellLabel->"In[239]:=",
  CellTags->"ShowArray"],

Cell["StaircasePlot", "ObjectName",
  CellTags->"StaircasePlot"],

Cell["Function", "Highlight",
  CellTags->"StaircasePlot"],

Cell[TextData[{
  StyleBox["StaircasePlot[L,opts]", "MR"],
  "  generates a staircase plot of the data in the numerical list ",
  StyleBox["L", "MR"],
  ". "
}], "Usage",
  CellTags->"StaircasePlot"],

Cell[TextData[{
  "Options",
  "\n",
  Cell[BoxData[GridBox[{
          {"Full", "\[Rule]", "False", "-"},
          {"PlotStyle", "\[Rule]", "Blue", "-"}
          },
        ColumnWidths->Automatic,
        ColumnAlignments->{Left}]]]
}], "Notes",
  CellTags->"StaircasePlot"],

Cell[TextData[{
  "All options are forwarded to ",
  StyleBox["Graphics", "MR"],
  "."
}], "Notes",
  CellTags->"StaircasePlot"],

Cell[TextData[{
  "See also ",
  ButtonBox["PlotMatrix",
    ButtonData:>"PlotMatrix",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["PlotT",
    ButtonData:>"PlotT",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["GridLine",
    ButtonData:>"GridLine",
    ButtonStyle->"AddOnsLink"],
  ", ",
  ButtonBox["GridPoints",
    ButtonData:>"GridPoints",
    ButtonStyle->"AddOnsLink"],
  "."
}], "Notes",
  CellTags->"StaircasePlot"],

Cell["Further Examples", "ExampleSection",
  CellTags->"StaircasePlot"],

Cell["A staircase plot of the first 20 primes.", "ExampleText",
  CellTags->"StaircasePlot"],

Cell[BoxData[
    \(\(StaircasePlot[\ Prime[Range[20]], Frame \[Rule] True, 
        FrameTicks \[Rule] None, 
        Background \[Rule] LightYellow\ ];\)\)], "Input",
  CellLabel->"In[911]:=",
  CellTags->"StaircasePlot"]
},
FrontEndVersion->"5.0 for X",
ScreenRectangle->{{0, 1280}, {0, 1024}},
AutoGeneratedPackage->None,
WindowSize->{520, 600},
WindowMargins->{{380, Automatic}, {Automatic, 212}},
Visible->False,
LanguageCategory->"Mathematica",
Magnification->1,
StyleDefinitions -> "HelpBrowser.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "$alphabet"->{
    Cell[1754, 51, 56, 1, 32, "ObjectName",
      CellTags->"$alphabet"],
    Cell[1813, 54, 57, 1, 50, "Highlight",
      CellTags->"$alphabet"],
    Cell[1873, 57, 183, 5, 47, "Usage",
      CellTags->"$alphabet"],
    Cell[2059, 64, 178, 6, 24, "Notes",
      CellTags->"$alphabet"],
    Cell[2240, 72, 147, 4, 24, "Notes",
      CellTags->"$alphabet"],
    Cell[2390, 78, 67, 1, 39, "ExampleSection",
      CellTags->"$alphabet"],
    Cell[2460, 81, 82, 1, 34, "ExampleText",
      CellTags->"$alphabet"],
    Cell[2545, 84, 76, 2, 28, "Input",
      CellTags->"$alphabet"],
    Cell[2624, 88, 120, 3, 26, "Input",
      CellTags->"$alphabet"],
    Cell[2747, 93, 137, 3, 26, "Input",
      CellTags->"$alphabet"]},
  "AlgebraicForm"->{
    Cell[2887, 98, 64, 1, 32, "ObjectName",
      CellTags->"AlgebraicForm"],
    Cell[2954, 101, 58, 1, 50, "Highlight",
      CellTags->"AlgebraicForm"],
    Cell[3015, 104, 199, 6, 31, "Usage",
      CellTags->"AlgebraicForm"],
    Cell[3217, 112, 175, 7, 24, "Notes",
      CellTags->"AlgebraicForm"],
    Cell[3395, 121, 71, 1, 39, "ExampleSection",
      CellTags->"AlgebraicForm"],
    Cell[3469, 124, 128, 4, 34, "ExampleText",
      CellTags->"AlgebraicForm"],
    Cell[3600, 130, 119, 3, 26, "Input",
      CellTags->"AlgebraicForm"],
    Cell[3722, 135, 113, 3, 26, "Input",
      CellTags->"AlgebraicForm"]},
  "Alphabet"->{
    Cell[3838, 140, 54, 1, 32, "ObjectName",
      CellTags->"Alphabet"],
    Cell[3895, 143, 53, 1, 50, "Highlight",
      CellTags->"Alphabet"],
    Cell[3951, 146, 161, 6, 31, "Usage",
      CellTags->"Alphabet"],
    Cell[4115, 154, 273, 10, 61, "Notes",
      CellTags->"Alphabet"],
    Cell[4391, 166, 250, 11, 24, "Notes",
      CellTags->"Alphabet"],
    Cell[4644, 179, 291, 11, 40, "Notes",
      CellTags->"Alphabet"],
    Cell[4938, 192, 110, 5, 24, "Notes",
      CellTags->"Alphabet"],
    Cell[5051, 199, 132, 4, 24, "Notes",
      CellTags->"Alphabet"],
    Cell[5186, 205, 82, 1, 24, "Notes",
      CellTags->"Alphabet"],
    Cell[5271, 208, 363, 15, 24, "Notes",
      CellTags->"Alphabet"],
    Cell[5637, 225, 66, 1, 39, "ExampleSection",
      CellTags->"Alphabet"],
    Cell[5706, 228, 142, 4, 34, "ExampleText",
      CellTags->"Alphabet"],
    Cell[5851, 234, 146, 4, 42, "Input",
      CellTags->"Alphabet"],
    Cell[6000, 240, 95, 1, 34, "ExampleText",
      CellTags->"Alphabet"],
    Cell[6098, 243, 188, 4, 42, "Input",
      CellTags->"Alphabet"],
    Cell[6289, 249, 70, 1, 34, "ExampleText",
      CellTags->"Alphabet"],
    Cell[6362, 252, 86, 2, 28, "Input",
      CellTags->"Alphabet"]},
  "Alphabetic"->{
    Cell[6451, 256, 58, 1, 32, "ObjectName",
      CellTags->"Alphabetic"],
    Cell[6512, 259, 53, 1, 50, "Highlight",
      CellTags->"Alphabetic"],
    Cell[6568, 262, 197, 8, 31, "Usage",
      CellTags->"Alphabetic"],
    Cell[6768, 272, 203, 6, 24, "Notes",
      CellTags->"Alphabetic"],
    Cell[6974, 280, 270, 11, 24, "Notes",
      CellTags->"Alphabetic"]},
  "AlphabetQ"->{
    Cell[7247, 293, 56, 1, 32, "ObjectName",
      CellTags->"AlphabetQ"],
    Cell[7306, 296, 54, 1, 50, "Highlight",
      CellTags->"AlphabetQ"],
    Cell[7363, 299, 161, 6, 31, "Usage",
      CellTags->"AlphabetQ"],
    Cell[7527, 307, 141, 4, 24, "Notes",
      CellTags->"AlphabetQ"],
    Cell[7671, 313, 159, 4, 40, "Notes",
      CellTags->"AlphabetQ"],
    Cell[7833, 319, 265, 9, 24, "Notes",
      CellTags->"AlphabetQ"],
    Cell[8101, 330, 308, 12, 24, "Notes",
      CellTags->"AlphabetQ"],
    Cell[8412, 344, 363, 15, 24, "Notes",
      CellTags->"AlphabetQ"],
    Cell[8778, 361, 67, 1, 39, "ExampleSection",
      CellTags->"AlphabetQ"],
    Cell[8848, 364, 70, 1, 34, "ExampleText",
      CellTags->"AlphabetQ"],
    Cell[8921, 367, 133, 4, 42, "Input",
      CellTags->"AlphabetQ"],
    Cell[9057, 373, 96, 1, 34, "ExampleText",
      CellTags->"AlphabetQ"],
    Cell[9156, 376, 150, 4, 42, "Input",
      CellTags->"AlphabetQ"],
    Cell[9309, 382, 67, 1, 34, "ExampleText",
      CellTags->"AlphabetQ"],
    Cell[9379, 385, 110, 5, 45, "Input",
      CellTags->"AlphabetQ"]},
  "AlphabetSize"->{
    Cell[9492, 392, 62, 1, 32, "ObjectName",
      CellTags->"AlphabetSize"],
    Cell[9557, 395, 57, 1, 50, "Highlight",
      CellTags->"AlphabetSize"],
    Cell[9617, 398, 165, 6, 31, "Usage",
      CellTags->"AlphabetSize"],
    Cell[9785, 406, 73, 1, 24, "Notes",
      CellTags->"AlphabetSize"],
    Cell[9861, 409, 358, 15, 24, "Notes",
      CellTags->"AlphabetSize"],
    Cell[10222, 426, 70, 1, 39, "ExampleSection",
      CellTags->"AlphabetSize"],
    Cell[10295, 429, 89, 1, 34, "ExampleText",
      CellTags->"AlphabetSize"],
    Cell[10387, 432, 207, 8, 41, "Input",
      CellTags->"AlphabetSize"],
    Cell[10597, 442, 76, 1, 34, "ExampleText",
      CellTags->"AlphabetSize"],
    Cell[10676, 445, 110, 3, 26, "Input",
      CellTags->"AlphabetSize"],
    Cell[10789, 450, 107, 3, 26, "Input",
      CellTags->"AlphabetSize"]},
  "Concatenate"->{
    Cell[10899, 455, 60, 1, 32, "ObjectName",
      CellTags->"Concatenate"],
    Cell[10962, 458, 56, 1, 50, "Highlight",
      CellTags->"Concatenate"],
    Cell[11021, 461, 184, 8, 31, "Usage",
      CellTags->"Concatenate"],
    Cell[11208, 471, 107, 4, 24, "Notes",
      CellTags->"Concatenate"],
    Cell[11318, 477, 178, 5, 24, "Notes",
      CellTags->"Concatenate"],
    Cell[11499, 484, 126, 4, 24, "Notes",
      CellTags->"Concatenate"],
    Cell[11628, 490, 236, 11, 24, "Notes",
      CellTags->"Concatenate"],
    Cell[11867, 503, 69, 1, 39, "ExampleSection",
      CellTags->"Concatenate"],
    Cell[11939, 506, 179, 4, 50, "ExampleText",
      CellTags->"Concatenate"],
    Cell[12121, 512, 135, 4, 42, "Input",
      CellTags->"Concatenate"],
    Cell[12259, 518, 128, 4, 34, "ExampleText",
      CellTags->"Concatenate"],
    Cell[12390, 524, 113, 3, 26, "Input",
      CellTags->"Concatenate"],
    Cell[12506, 529, 112, 3, 26, "Input",
      CellTags->"Concatenate"],
    Cell[12621, 534, 130, 3, 26, "Input",
      CellTags->"Concatenate"]},
  "Eps"->{
    Cell[12754, 539, 44, 1, 32, "ObjectName",
      CellTags->"Eps"],
    Cell[12801, 542, 51, 1, 50, "Highlight",
      CellTags->"Eps"],
    Cell[12855, 545, 115, 4, 31, "Usage",
      CellTags->"Eps"],
    Cell[12973, 551, 171, 6, 24, "Notes",
      CellTags->"Eps"],
    Cell[13147, 559, 299, 11, 39, "Notes",
      CellTags->"Eps"],
    Cell[13449, 572, 284, 11, 40, "Notes",
      CellTags->"Eps"],
    Cell[13736, 585, 141, 7, 24, "Notes",
      CellTags->"Eps"],
    Cell[13880, 594, 61, 1, 39, "ExampleSection",
      CellTags->"Eps"],
    Cell[13944, 597, 214, 7, 50, "ExampleText",
      CellTags->"Eps"],
    Cell[14161, 606, 102, 3, 26, "Input",
      CellTags->"Eps"],
    Cell[14266, 611, 273, 8, 50, "ExampleText",
      CellTags->"Eps"],
    Cell[14542, 621, 164, 5, 58, "Input",
      CellTags->"Eps"],
    Cell[14709, 628, 103, 3, 26, "Input",
      CellTags->"Eps"]},
  "FromEps"->{
    Cell[14815, 633, 52, 1, 32, "ObjectName",
      CellTags->"FromEps"],
    Cell[14870, 636, 52, 1, 50, "Highlight",
      CellTags->"FromEps"],
    Cell[14925, 639, 232, 8, 47, "Usage",
      CellTags->"FromEps"],
    Cell[15160, 649, 96, 1, 24, "Notes",
      CellTags->"FromEps"],
    Cell[15259, 652, 232, 11, 24, "Notes",
      CellTags->"FromEps"],
    Cell[15494, 665, 65, 1, 39, "ExampleSection",
      CellTags->"FromEps"],
    Cell[15562, 668, 86, 1, 34, "ExampleText",
      CellTags->"FromEps"],
    Cell[15651, 671, 243, 6, 58, "Input",
      CellTags->"FromEps"]},
  "FromFrontier"->{
    Cell[15897, 679, 62, 1, 32, "ObjectName",
      CellTags->"FromFrontier"],
    Cell[15962, 682, 57, 1, 50, "Highlight",
      CellTags->"FromFrontier"],
    Cell[16022, 685, 194, 6, 31, "Usage",
      CellTags->"FromFrontier"],
    Cell[16219, 693, 228, 9, 44, "Notes",
      CellTags->"FromFrontier"],
    Cell[16450, 704, 218, 8, 24, "Notes",
      CellTags->"FromFrontier"],
    Cell[16671, 714, 173, 6, 24, "Notes",
      CellTags->"FromFrontier"],
    Cell[16847, 722, 245, 8, 24, "Notes",
      CellTags->"FromFrontier"],
    Cell[17095, 732, 264, 11, 24, "Notes",
      CellTags->"FromFrontier"],
    Cell[17362, 745, 70, 1, 39, "ExampleSection",
      CellTags->"FromFrontier"],
    Cell[17435, 748, 94, 1, 34, "ExampleText",
      CellTags->"FromFrontier"],
    Cell[17532, 751, 167, 4, 42, "Input",
      CellTags->"FromFrontier"],
    Cell[17702, 757, 96, 2, 28, "Input",
      CellTags->"FromFrontier"],
    Cell[17801, 761, 87, 1, 34, "ExampleText",
      CellTags->"FromFrontier"],
    Cell[17891, 764, 119, 3, 26, "Input",
      CellTags->"FromFrontier"]},
  "FrontierQ"->{
    Cell[18013, 769, 56, 1, 32, "ObjectName",
      CellTags->"FrontierQ"],
    Cell[18072, 772, 54, 1, 50, "Highlight",
      CellTags->"FrontierQ"],
    Cell[18129, 775, 218, 8, 31, "Usage",
      CellTags->"FrontierQ"],
    Cell[18350, 785, 153, 5, 24, "Notes",
      CellTags->"FrontierQ"],
    Cell[18506, 792, 365, 15, 24, "Notes",
      CellTags->"FrontierQ"],
    Cell[18874, 809, 67, 1, 39, "ExampleSection",
      CellTags->"FrontierQ"],
    Cell[18944, 812, 74, 1, 34, "ExampleText",
      CellTags->"FrontierQ"],
    Cell[19021, 815, 128, 3, 26, "Input",
      CellTags->"FrontierQ"],
    Cell[19152, 820, 90, 1, 34, "ExampleText",
      CellTags->"FrontierQ"],
    Cell[19245, 823, 128, 3, 26, "Input",
      CellTags->"FrontierQ"]},
  "MakeAlphabet"->{
    Cell[19376, 828, 62, 1, 32, "ObjectName",
      CellTags->"MakeAlphabet"],
    Cell[19441, 831, 57, 1, 50, "Highlight",
      CellTags->"MakeAlphabet"],
    Cell[19501, 834, 259, 10, 31, "Usage",
      CellTags->"MakeAlphabet"],
    Cell[19763, 846, 228, 9, 44, "Notes",
      CellTags->"MakeAlphabet"],
    Cell[19994, 857, 255, 11, 24, "Notes",
      CellTags->"MakeAlphabet"],
    Cell[20252, 870, 170, 5, 24, "Notes",
      CellTags->"MakeAlphabet"],
    Cell[20425, 877, 263, 11, 24, "Notes",
      CellTags->"MakeAlphabet"],
    Cell[20691, 890, 70, 1, 39, "ExampleSection",
      CellTags->"MakeAlphabet"],
    Cell[20764, 893, 89, 1, 34, "ExampleText",
      CellTags->"MakeAlphabet"],
    Cell[20856, 896, 122, 6, 62, "Input",
      CellTags->"MakeAlphabet"],
    Cell[20981, 904, 125, 3, 26, "Input",
      CellTags->"MakeAlphabet"],
    Cell[21109, 909, 68, 1, 34, "ExampleText",
      CellTags->"MakeAlphabet"],
    Cell[21180, 912, 108, 5, 45, "Input",
      CellTags->"MakeAlphabet"]},
  "MakeWord"->{
    Cell[21291, 919, 54, 1, 32, "ObjectName",
      CellTags->"MakeWord"],
    Cell[21348, 922, 53, 1, 50, "Highlight",
      CellTags->"MakeWord"],
    Cell[21404, 925, 311, 12, 47, "Usage",
      CellTags->"MakeWord"],
    Cell[21718, 939, 65, 1, 24, "Notes",
      CellTags->"MakeWord"],
    Cell[21786, 942, 78, 1, 24, "Notes",
      CellTags->"MakeWord"],
    Cell[21867, 945, 159, 7, 24, "Notes",
      CellTags->"MakeWord"],
    Cell[22029, 954, 66, 1, 39, "ExampleSection",
      CellTags->"MakeWord"],
    Cell[22098, 957, 68, 1, 34, "ExampleText",
      CellTags->"MakeWord"],
    Cell[22169, 960, 113, 5, 45, "Input",
      CellTags->"MakeWord"],
    Cell[22285, 967, 93, 3, 26, "Input",
      CellTags->"MakeWord"]},
  "ParikhVector"->{
    Cell[22381, 972, 62, 1, 32, "ObjectName",
      CellTags->"ParikhVector"],
    Cell[22446, 975, 57, 1, 50, "Highlight",
      CellTags->"ParikhVector"],
    Cell[22506, 978, 223, 8, 31, "Usage",
      CellTags->"ParikhVector"],
    Cell[22732, 988, 265, 10, 24, "Notes",
      CellTags->"ParikhVector"],
    Cell[23000, 1000, 75, 1, 24, "Notes",
      CellTags->"ParikhVector"],
    Cell[23078, 1003, 70, 1, 39, "ExampleSection",
      CellTags->"ParikhVector"],
    Cell[23151, 1006, 143, 4, 34, "ExampleText",
      CellTags->"ParikhVector"],
    Cell[23297, 1012, 98, 2, 28, "Input",
      CellTags->"ParikhVector"],
    Cell[23398, 1016, 136, 4, 34, "ExampleText",
      CellTags->"ParikhVector"],
    Cell[23537, 1022, 203, 5, 42, "Input",
      CellTags->"ParikhVector"]},
  "RandomWord"->{
    Cell[23743, 1029, 58, 1, 32, "ObjectName",
      CellTags->"RandomWord"],
    Cell[23804, 1032, 55, 1, 50, "Highlight",
      CellTags->"RandomWord"],
    Cell[23862, 1035, 150, 4, 31, "Usage",
      CellTags->"RandomWord"],
    Cell[24015, 1041, 101, 1, 24, "Notes",
      CellTags->"RandomWord"],
    Cell[24119, 1044, 155, 7, 24, "Notes",
      CellTags->"RandomWord"],
    Cell[24277, 1053, 68, 1, 39, "ExampleSection",
      CellTags->"RandomWord"],
    Cell[24348, 1056, 62, 1, 34, "ExampleText",
      CellTags->"RandomWord"],
    Cell[24413, 1059, 85, 2, 28, "Input",
      CellTags->"RandomWord"],
    Cell[24501, 1063, 86, 2, 28, "Input",
      CellTags->"RandomWord"]},
  "ToBlocks"->{
    Cell[24590, 1067, 54, 1, 32, "ObjectName",
      CellTags->"ToBlocks"],
    Cell[24647, 1070, 53, 1, 50, "Highlight",
      CellTags->"ToBlocks"],
    Cell[24703, 1073, 232, 8, 31, "Usage",
      CellTags->"ToBlocks"],
    Cell[24938, 1083, 470, 17, 44, "Notes",
      CellTags->"ToBlocks"],
    Cell[25411, 1102, 132, 5, 24, "Notes",
      CellTags->"ToBlocks"],
    Cell[25546, 1109, 253, 11, 24, "Notes",
      CellTags->"ToBlocks"],
    Cell[25802, 1122, 161, 5, 24, "Notes",
      CellTags->"ToBlocks"],
    Cell[25966, 1129, 188, 7, 24, "Notes",
      CellTags->"ToBlocks"],
    Cell[26157, 1138, 66, 1, 39, "ExampleSection",
      CellTags->"ToBlocks"],
    Cell[26226, 1141, 75, 1, 34, "ExampleText",
      CellTags->"ToBlocks"],
    Cell[26304, 1144, 103, 3, 26, "Input",
      CellTags->"ToBlocks"],
    Cell[26410, 1149, 127, 3, 26, "Input",
      CellTags->"ToBlocks"],
    Cell[26540, 1154, 67, 1, 34, "ExampleText",
      CellTags->"ToBlocks"],
    Cell[26610, 1157, 132, 3, 26, "Input",
      CellTags->"ToBlocks"]},
  "ToEps"->{
    Cell[26745, 1162, 48, 1, 32, "ObjectName",
      CellTags->"ToEps"],
    Cell[26796, 1165, 50, 1, 50, "Highlight",
      CellTags->"ToEps"],
    Cell[26849, 1168, 212, 8, 31, "Usage",
      CellTags->"ToEps"],
    Cell[27064, 1178, 98, 1, 24, "Notes",
      CellTags->"ToEps"],
    Cell[27165, 1181, 152, 7, 24, "Notes",
      CellTags->"ToEps"],
    Cell[27320, 1190, 63, 1, 39, "ExampleSection",
      CellTags->"ToEps"],
    Cell[27386, 1193, 125, 4, 34, "ExampleText",
      CellTags->"ToEps"],
    Cell[27514, 1199, 98, 3, 26, "Input",
      CellTags->"ToEps"]},
  "ToFrontier"->{
    Cell[27615, 1204, 58, 1, 32, "ObjectName",
      CellTags->"ToFrontier"],
    Cell[27676, 1207, 55, 1, 50, "Highlight",
      CellTags->"ToFrontier"],
    Cell[27734, 1210, 234, 8, 47, "Usage",
      CellTags->"ToFrontier"],
    Cell[27971, 1220, 158, 6, 24, "Notes",
      CellTags->"ToFrontier"],
    Cell[28132, 1228, 156, 5, 24, "Notes",
      CellTags->"ToFrontier"],
    Cell[28291, 1235, 266, 11, 24, "Notes",
      CellTags->"ToFrontier"],
    Cell[28560, 1248, 68, 1, 39, "ExampleSection",
      CellTags->"ToFrontier"],
    Cell[28631, 1251, 92, 1, 34, "ExampleText",
      CellTags->"ToFrontier"],
    Cell[28726, 1254, 173, 4, 42, "Input",
      CellTags->"ToFrontier"],
    Cell[28902, 1260, 82, 2, 28, "Input",
      CellTags->"ToFrontier"]},
  "ToIndex"->{
    Cell[28987, 1264, 52, 1, 32, "ObjectName",
      CellTags->"ToIndex"],
    Cell[29042, 1267, 52, 1, 50, "Highlight",
      CellTags->"ToIndex"],
    Cell[29097, 1270, 214, 8, 31, "Usage",
      CellTags->"ToIndex"],
    Cell[29314, 1280, 91, 1, 24, "Notes",
      CellTags->"ToIndex"],
    Cell[29408, 1283, 90, 1, 24, "Notes",
      CellTags->"ToIndex"],
    Cell[29501, 1286, 152, 7, 24, "Notes",
      CellTags->"ToIndex"],
    Cell[29656, 1295, 65, 1, 39, "ExampleSection",
      CellTags->"ToIndex"],
    Cell[29724, 1298, 79, 1, 34, "ExampleText",
      CellTags->"ToIndex"],
    Cell[29806, 1301, 85, 2, 28, "Input",
      CellTags->"ToIndex"],
    Cell[29894, 1305, 100, 3, 26, "Input",
      CellTags->"ToIndex"]},
  "ToWord"->{
    Cell[29997, 1310, 50, 1, 32, "ObjectName",
      CellTags->"ToWord"],
    Cell[30050, 1313, 51, 1, 50, "Highlight",
      CellTags->"ToWord"],
    Cell[30104, 1316, 196, 7, 31, "Usage",
      CellTags->"ToWord"],
    Cell[30303, 1325, 157, 6, 24, "Notes",
      CellTags->"ToWord"],
    Cell[30463, 1333, 112, 4, 24, "Notes",
      CellTags->"ToWord"],
    Cell[30578, 1339, 126, 6, 24, "Notes",
      CellTags->"ToWord"],
    Cell[30707, 1347, 64, 1, 39, "ExampleSection",
      CellTags->"ToWord"],
    Cell[30774, 1350, 85, 1, 34, "ExampleText",
      CellTags->"ToWord"],
    Cell[30862, 1353, 83, 2, 28, "Input",
      CellTags->"ToWord"],
    Cell[30948, 1357, 96, 2, 28, "Input",
      CellTags->"ToWord"],
    Cell[31047, 1361, 66, 1, 34, "ExampleText",
      CellTags->"ToWord"],
    Cell[31116, 1364, 110, 3, 26, "Input",
      CellTags->"ToWord"]},
  "WordBinomial"->{
    Cell[31229, 1369, 62, 1, 32, "ObjectName",
      CellTags->"WordBinomial"],
    Cell[31294, 1372, 57, 1, 50, "Highlight",
      CellTags->"WordBinomial"],
    Cell[31354, 1375, 242, 9, 47, "Usage",
      CellTags->"WordBinomial"],
    Cell[31599, 1386, 231, 9, 44, "Notes",
      CellTags->"WordBinomial"],
    Cell[31833, 1397, 207, 9, 24, "Notes",
      CellTags->"WordBinomial"],
    Cell[32043, 1408, 151, 4, 24, "Notes",
      CellTags->"WordBinomial"],
    Cell[32197, 1414, 194, 7, 24, "Notes",
      CellTags->"WordBinomial"],
    Cell[32394, 1423, 70, 1, 39, "ExampleSection",
      CellTags->"WordBinomial"],
    Cell[32467, 1426, 154, 4, 34, "ExampleText",
      CellTags->"WordBinomial"],
    Cell[32624, 1432, 142, 4, 42, "Input",
      CellTags->"WordBinomial"],
    Cell[32769, 1438, 99, 1, 34, "ExampleText",
      CellTags->"WordBinomial"],
    Cell[32871, 1441, 202, 5, 42, "Input",
      CellTags->"WordBinomial"]},
  "WordConjugateQ"->{
    Cell[33076, 1448, 66, 1, 32, "ObjectName",
      CellTags->"WordConjugateQ"],
    Cell[33145, 1451, 59, 1, 50, "Highlight",
      CellTags->"WordConjugateQ"],
    Cell[33207, 1454, 205, 8, 31, "Usage",
      CellTags->"WordConjugateQ"],
    Cell[33415, 1464, 114, 5, 24, "Notes",
      CellTags->"WordConjugateQ"],
    Cell[33532, 1471, 186, 6, 24, "Notes",
      CellTags->"WordConjugateQ"],
    Cell[33721, 1479, 168, 7, 24, "Notes",
      CellTags->"WordConjugateQ"],
    Cell[33892, 1488, 72, 1, 39, "ExampleSection",
      CellTags->"WordConjugateQ"],
    Cell[33967, 1491, 89, 1, 34, "ExampleText",
      CellTags->"WordConjugateQ"],
    Cell[34059, 1494, 100, 2, 28, "Input",
      CellTags->"WordConjugateQ"],
    Cell[34162, 1498, 100, 2, 28, "Input",
      CellTags->"WordConjugateQ"],
    Cell[34265, 1502, 118, 2, 28, "Input",
      CellTags->"WordConjugateQ"]},
  "WordConjugates"->{
    Cell[34386, 1506, 66, 1, 32, "ObjectName",
      CellTags->"WordConjugates"],
    Cell[34455, 1509, 59, 1, 50, "Highlight",
      CellTags->"WordConjugates"],
    Cell[34517, 1512, 193, 6, 31, "Usage",
      CellTags->"WordConjugates"],
    Cell[34713, 1520, 114, 5, 24, "Notes",
      CellTags->"WordConjugates"],
    Cell[34830, 1527, 141, 6, 24, "Notes",
      CellTags->"WordConjugates"],
    Cell[34974, 1535, 72, 1, 39, "ExampleSection",
      CellTags->"WordConjugates"],
    Cell[35049, 1538, 283, 10, 50, "ExampleText",
      CellTags->"WordConjugates"],
    Cell[35335, 1550, 96, 2, 28, "Input",
      CellTags->"WordConjugates"],
    Cell[35434, 1554, 95, 2, 28, "Input",
      CellTags->"WordConjugates"]},
  "WordFactorizations"->{
    Cell[35532, 1558, 74, 1, 32, "ObjectName",
      CellTags->"WordFactorizations"],
    Cell[35609, 1561, 63, 1, 50, "Highlight",
      CellTags->"WordFactorizations"],
    Cell[35675, 1564, 213, 8, 31, "Usage",
      CellTags->"WordFactorizations"],
    Cell[35891, 1574, 215, 8, 24, "Notes",
      CellTags->"WordFactorizations"],
    Cell[36109, 1584, 214, 9, 24, "Notes",
      CellTags->"WordFactorizations"],
    Cell[36326, 1595, 76, 1, 39, "ExampleSection",
      CellTags->"WordFactorizations"],
    Cell[36405, 1598, 205, 6, 34, "ExampleText",
      CellTags->"WordFactorizations"],
    Cell[36613, 1606, 103, 2, 28, "Input",
      CellTags->"WordFactorizations"],
    Cell[36719, 1610, 125, 3, 26, "Input",
      CellTags->"WordFactorizations"]},
  "WordFactorQ"->{
    Cell[36847, 1615, 60, 1, 32, "ObjectName",
      CellTags->"WordFactorQ"],
    Cell[36910, 1618, 56, 1, 50, "Highlight",
      CellTags->"WordFactorQ"],
    Cell[36969, 1621, 210, 8, 31, "Usage",
      CellTags->"WordFactorQ"],
    Cell[37182, 1631, 109, 4, 24, "Notes",
      CellTags->"WordFactorQ"],
    Cell[37294, 1637, 423, 18, 24, "Notes",
      CellTags->"WordFactorQ"],
    Cell[37720, 1657, 69, 1, 39, "ExampleSection",
      CellTags->"WordFactorQ"],
    Cell[37792, 1660, 151, 4, 50, "ExampleText",
      CellTags->"WordFactorQ"],
    Cell[37946, 1666, 105, 3, 26, "Input",
      CellTags->"WordFactorQ"],
    Cell[38054, 1671, 105, 3, 26, "Input",
      CellTags->"WordFactorQ"],
    Cell[38162, 1676, 135, 3, 26, "Input",
      CellTags->"WordFactorQ"]},
  "WordFactors"->{
    Cell[38300, 1681, 60, 1, 32, "ObjectName",
      CellTags->"WordFactors"],
    Cell[38363, 1684, 56, 1, 50, "Highlight",
      CellTags->"WordFactors"],
    Cell[38422, 1687, 195, 6, 31, "Usage",
      CellTags->"WordFactors"],
    Cell[38620, 1695, 226, 9, 44, "Notes",
      CellTags->"WordFactors"],
    Cell[38849, 1706, 169, 7, 24, "Notes",
      CellTags->"WordFactors"],
    Cell[39021, 1715, 111, 5, 24, "Notes",
      CellTags->"WordFactors"],
    Cell[39135, 1722, 210, 8, 24, "Notes",
      CellTags->"WordFactors"],
    Cell[39348, 1732, 476, 19, 24, "Notes",
      CellTags->"WordFactors"],
    Cell[39827, 1753, 69, 1, 39, "ExampleSection",
      CellTags->"WordFactors"],
    Cell[39899, 1756, 163, 6, 34, "ExampleText",
      CellTags->"WordFactors"],
    Cell[40065, 1764, 90, 2, 28, "Input",
      CellTags->"WordFactors"],
    Cell[40158, 1768, 90, 2, 28, "Input",
      CellTags->"WordFactors"],
    Cell[40251, 1772, 80, 1, 34, "ExampleText",
      CellTags->"WordFactors"],
    Cell[40334, 1775, 92, 2, 28, "Input",
      CellTags->"WordFactors"]},
  "WordFusion"->{
    Cell[40429, 1779, 58, 1, 32, "ObjectName",
      CellTags->"WordFusion"],
    Cell[40490, 1782, 55, 1, 50, "Highlight",
      CellTags->"WordFusion"],
    Cell[40548, 1785, 223, 10, 31, "Usage",
      CellTags->"WordFusion"],
    Cell[40774, 1797, 150, 5, 24, "Notes",
      CellTags->"WordFusion"],
    Cell[40927, 1804, 195, 8, 24, "Notes",
      CellTags->"WordFusion"],
    Cell[41125, 1814, 68, 1, 39, "ExampleSection",
      CellTags->"WordFusion"],
    Cell[41196, 1817, 220, 9, 34, "ExampleText",
      CellTags->"WordFusion"],
    Cell[41419, 1828, 96, 2, 28, "Input",
      CellTags->"WordFusion"],
    Cell[41518, 1832, 96, 2, 28, "Input",
      CellTags->"WordFusion"]},
  "WordLength"->{
    Cell[41617, 1836, 58, 1, 32, "ObjectName",
      CellTags->"WordLength"],
    Cell[41678, 1839, 55, 1, 50, "Highlight",
      CellTags->"WordLength"],
    Cell[41736, 1842, 148, 5, 31, "Usage",
      CellTags->"WordLength"],
    Cell[41887, 1849, 110, 5, 24, "Notes",
      CellTags->"WordLength"],
    Cell[42000, 1856, 233, 7, 40, "Notes",
      CellTags->"WordLength"],
    Cell[42236, 1865, 260, 10, 24, "Notes",
      CellTags->"WordLength"],
    Cell[42499, 1877, 135, 6, 24, "Notes",
      CellTags->"WordLength"],
    Cell[42637, 1885, 68, 1, 39, "ExampleSection",
      CellTags->"WordLength"],
    Cell[42708, 1888, 126, 4, 34, "ExampleText",
      CellTags->"WordLength"],
    Cell[42837, 1894, 99, 5, 45, "Input",
      CellTags->"WordLength"],
    Cell[42939, 1901, 101, 5, 45, "Input",
      CellTags->"WordLength"]},
  "WordOrder"->{
    Cell[43043, 1908, 56, 1, 32, "ObjectName",
      CellTags->"WordOrder"],
    Cell[43102, 1911, 54, 1, 50, "Highlight",
      CellTags->"WordOrder"],
    Cell[43159, 1914, 230, 8, 31, "Usage",
      CellTags->"WordOrder"],
    Cell[43392, 1924, 257, 8, 40, "Notes",
      CellTags->"WordOrder"],
    Cell[43652, 1934, 444, 19, 40, "Notes",
      CellTags->"WordOrder"],
    Cell[44099, 1955, 130, 6, 24, "Notes",
      CellTags->"WordOrder"],
    Cell[44232, 1963, 67, 1, 39, "ExampleSection",
      CellTags->"WordOrder"],
    Cell[44302, 1966, 98, 1, 34, "ExampleText",
      CellTags->"WordOrder"],
    Cell[44403, 1969, 80, 2, 28, "Input",
      CellTags->"WordOrder"],
    Cell[44486, 1973, 153, 4, 26, "Input",
      CellTags->"WordOrder"]},
  "WordPeriod"->{
    Cell[44642, 1979, 58, 1, 32, "ObjectName",
      CellTags->"WordPeriod"],
    Cell[44703, 1982, 55, 1, 50, "Highlight",
      CellTags->"WordPeriod"],
    Cell[44761, 1985, 163, 6, 31, "Usage",
      CellTags->"WordPeriod"],
    Cell[44927, 1993, 226, 9, 44, "Notes",
      CellTags->"WordPeriod"],
    Cell[45156, 2004, 110, 5, 24, "Notes",
      CellTags->"WordPeriod"],
    Cell[45269, 2011, 218, 8, 24, "Notes",
      CellTags->"WordPeriod"],
    Cell[45490, 2021, 393, 13, 24, "Notes",
      CellTags->"WordPeriod"],
    Cell[45886, 2036, 361, 15, 24, "Notes",
      CellTags->"WordPeriod"],
    Cell[46250, 2053, 68, 1, 39, "ExampleSection",
      CellTags->"WordPeriod"],
    Cell[46321, 2056, 145, 4, 34, "ExampleText",
      CellTags->"WordPeriod"],
    Cell[46469, 2062, 211, 4, 42, "Input",
      CellTags->"WordPeriod"],
    Cell[46683, 2068, 137, 4, 34, "ExampleText",
      CellTags->"WordPeriod"],
    Cell[46823, 2074, 118, 3, 26, "Input",
      CellTags->"WordPeriod"],
    Cell[46944, 2079, 132, 3, 26, "Input",
      CellTags->"WordPeriod"],
    Cell[47079, 2084, 71, 1, 34, "ExampleText",
      CellTags->"WordPeriod"],
    Cell[47153, 2087, 130, 3, 26, "Input",
      CellTags->"WordPeriod"]},
  "WordPower"->{
    Cell[47286, 2092, 56, 1, 32, "ObjectName",
      CellTags->"WordPower"],
    Cell[47345, 2095, 54, 1, 50, "Highlight",
      CellTags->"WordPower"],
    Cell[47402, 2098, 184, 8, 31, "Usage",
      CellTags->"WordPower"],
    Cell[47589, 2108, 109, 5, 24, "Notes",
      CellTags->"WordPower"],
    Cell[47701, 2115, 284, 11, 24, "Notes",
      CellTags->"WordPower"],
    Cell[47988, 2128, 67, 1, 39, "ExampleSection",
      CellTags->"WordPower"],
    Cell[48058, 2131, 123, 4, 34, "ExampleText",
      CellTags->"WordPower"],
    Cell[48184, 2137, 142, 3, 26, "Input",
      CellTags->"WordPower"],
    Cell[48329, 2142, 111, 3, 26, "Input",
      CellTags->"WordPower"],
    Cell[48443, 2147, 111, 3, 26, "Input",
      CellTags->"WordPower"],
    Cell[48557, 2152, 105, 3, 26, "Input",
      CellTags->"WordPower"]},
  "WordPrefixQ"->{
    Cell[48665, 2157, 60, 1, 32, "ObjectName",
      CellTags->"WordPrefixQ"],
    Cell[48728, 2160, 56, 1, 50, "Highlight",
      CellTags->"WordPrefixQ"],
    Cell[48787, 2163, 199, 8, 31, "Usage",
      CellTags->"WordPrefixQ"],
    Cell[48989, 2173, 109, 4, 24, "Notes",
      CellTags->"WordPrefixQ"],
    Cell[49101, 2179, 350, 15, 24, "Notes",
      CellTags->"WordPrefixQ"],
    Cell[49454, 2196, 69, 1, 39, "ExampleSection",
      CellTags->"WordPrefixQ"],
    Cell[49526, 2199, 77, 1, 34, "ExampleText",
      CellTags->"WordPrefixQ"],
    Cell[49606, 2202, 105, 3, 26, "Input",
      CellTags->"WordPrefixQ"],
    Cell[49714, 2207, 105, 3, 26, "Input",
      CellTags->"WordPrefixQ"],
    Cell[49822, 2212, 114, 3, 26, "Input",
      CellTags->"WordPrefixQ"],
    Cell[49939, 2217, 136, 3, 26, "Input",
      CellTags->"WordPrefixQ"]},
  "WordPrefixes"->{
    Cell[50078, 2222, 62, 1, 32, "ObjectName",
      CellTags->"WordPrefixes"],
    Cell[50143, 2225, 57, 1, 50, "Highlight",
      CellTags->"WordPrefixes"],
    Cell[50203, 2228, 180, 6, 31, "Usage",
      CellTags->"WordPrefixes"],
    Cell[50386, 2236, 227, 9, 44, "Notes",
      CellTags->"WordPrefixes"],
    Cell[50616, 2247, 171, 7, 24, "Notes",
      CellTags->"WordPrefixes"],
    Cell[50790, 2256, 112, 5, 24, "Notes",
      CellTags->"WordPrefixes"],
    Cell[50905, 2263, 463, 19, 24, "Notes",
      CellTags->"WordPrefixes"],
    Cell[51371, 2284, 70, 1, 39, "ExampleSection",
      CellTags->"WordPrefixes"],
    Cell[51444, 2287, 173, 6, 34, "ExampleText",
      CellTags->"WordPrefixes"],
    Cell[51620, 2295, 92, 2, 28, "Input",
      CellTags->"WordPrefixes"],
    Cell[51715, 2299, 95, 2, 28, "Input",
      CellTags->"WordPrefixes"],
    Cell[51813, 2303, 72, 1, 34, "ExampleText",
      CellTags->"WordPrefixes"],
    Cell[51888, 2306, 125, 3, 26, "Input",
      CellTags->"WordPrefixes"]},
  "WordPrimitiveCount"->{
    Cell[52016, 2311, 74, 1, 32, "ObjectName",
      CellTags->"WordPrimitiveCount"],
    Cell[52093, 2314, 63, 1, 50, "Highlight",
      CellTags->"WordPrimitiveCount"],
    Cell[52159, 2317, 246, 8, 47, "Usage",
      CellTags->"WordPrimitiveCount"],
    Cell[52408, 2327, 118, 5, 24, "Notes",
      CellTags->"WordPrimitiveCount"],
    Cell[52529, 2334, 160, 6, 24, "Notes",
      CellTags->"WordPrimitiveCount"],
    Cell[52692, 2342, 371, 15, 24, "Notes",
      CellTags->"WordPrimitiveCount"],
    Cell[53066, 2359, 76, 1, 39, "ExampleSection",
      CellTags->"WordPrimitiveCount"],
    Cell[53145, 2362, 82, 1, 34, "ExampleText",
      CellTags->"WordPrimitiveCount"],
    Cell[53230, 2365, 135, 2, 28, "Input",
      CellTags->"WordPrimitiveCount"]},
  "WordPrimitiveQ"->{
    Cell[53368, 2369, 66, 1, 32, "ObjectName",
      CellTags->"WordPrimitiveQ"],
    Cell[53437, 2372, 59, 1, 50, "Highlight",
      CellTags->"WordPrimitiveQ"],
    Cell[53499, 2375, 205, 6, 31, "Usage",
      CellTags->"WordPrimitiveQ"],
    Cell[53707, 2383, 114, 5, 24, "Notes",
      CellTags->"WordPrimitiveQ"],
    Cell[53824, 2390, 168, 6, 24, "Notes",
      CellTags->"WordPrimitiveQ"],
    Cell[53995, 2398, 373, 15, 24, "Notes",
      CellTags->"WordPrimitiveQ"],
    Cell[54371, 2415, 72, 1, 39, "ExampleSection",
      CellTags->"WordPrimitiveQ"],
    Cell[54446, 2418, 137, 4, 34, "ExampleText",
      CellTags->"WordPrimitiveQ"],
    Cell[54586, 2424, 168, 4, 42, "Input",
      CellTags->"WordPrimitiveQ"],
    Cell[54757, 2430, 114, 2, 34, "ExampleText",
      CellTags->"WordPrimitiveQ"],
    Cell[54874, 2434, 145, 3, 26, "Input",
      CellTags->"WordPrimitiveQ"],
    Cell[55022, 2439, 141, 4, 34, "ExampleText",
      CellTags->"WordPrimitiveQ"],
    Cell[55166, 2445, 134, 3, 26, "Input",
      CellTags->"WordPrimitiveQ"]},
  "WordQ"->{
    Cell[55303, 2450, 48, 1, 32, "ObjectName",
      CellTags->"WordQ"],
    Cell[55354, 2453, 50, 1, 50, "Highlight",
      CellTags->"WordQ"],
    Cell[55407, 2456, 143, 4, 31, "Usage",
      CellTags->"WordQ"],
    Cell[55553, 2462, 67, 1, 24, "Notes",
      CellTags->"WordQ"],
    Cell[55623, 2465, 105, 5, 24, "Notes",
      CellTags->"WordQ"],
    Cell[55731, 2472, 1945, 79, 72, "Notes",
      CellTags->"WordQ"],
    Cell[57679, 2553, 63, 1, 39, "ExampleSection",
      CellTags->"WordQ"],
    Cell[57745, 2556, 72, 1, 34, "ExampleText",
      CellTags->"WordQ"],
    Cell[57820, 2559, 131, 3, 26, "Input",
      CellTags->"WordQ"]},
  "WordReverse"->{
    Cell[57954, 2564, 60, 1, 32, "ObjectName",
      CellTags->"WordReverse"],
    Cell[58017, 2567, 56, 1, 50, "Highlight",
      CellTags->"WordReverse"],
    Cell[58076, 2570, 257, 8, 47, "Usage",
      CellTags->"WordReverse"],
    Cell[58336, 2580, 111, 5, 24, "Notes",
      CellTags->"WordReverse"],
    Cell[58450, 2587, 78, 1, 24, "Notes",
      CellTags->"WordReverse"],
    Cell[58531, 2590, 195, 9, 24, "Notes",
      CellTags->"WordReverse"],
    Cell[58729, 2601, 69, 1, 39, "ExampleSection",
      CellTags->"WordReverse"],
    Cell[58801, 2604, 103, 1, 34, "ExampleText",
      CellTags->"WordReverse"],
    Cell[58907, 2607, 89, 2, 28, "Input",
      CellTags->"WordReverse"],
    Cell[58999, 2611, 73, 1, 34, "ExampleText",
      CellTags->"WordReverse"],
    Cell[59075, 2614, 117, 3, 26, "Input",
      CellTags->"WordReverse"],
    Cell[59195, 2619, 81, 1, 34, "ExampleText",
      CellTags->"WordReverse"],
    Cell[59279, 2622, 137, 3, 26, "Input",
      CellTags->"WordReverse"]},
  "WordRoot"->{
    Cell[59419, 2627, 54, 1, 32, "ObjectName",
      CellTags->"WordRoot"],
    Cell[59476, 2630, 53, 1, 50, "Highlight",
      CellTags->"WordRoot"],
    Cell[59532, 2633, 289, 12, 31, "Usage",
      CellTags->"WordRoot"],
    Cell[59824, 2647, 108, 5, 24, "Notes",
      CellTags->"WordRoot"],
    Cell[59935, 2654, 378, 15, 24, "Notes",
      CellTags->"WordRoot"],
    Cell[60316, 2671, 66, 1, 39, "ExampleSection",
      CellTags->"WordRoot"],
    Cell[60385, 2674, 92, 1, 34, "ExampleText",
      CellTags->"WordRoot"],
    Cell[60480, 2677, 107, 2, 28, "Input",
      CellTags->"WordRoot"],
    Cell[60590, 2681, 94, 3, 26, "Input",
      CellTags->"WordRoot"]},
  "WordRotate"->{
    Cell[60687, 2686, 58, 1, 32, "ObjectName",
      CellTags->"WordRotate"],
    Cell[60748, 2689, 55, 1, 50, "Highlight",
      CellTags->"WordRotate"],
    Cell[60806, 2692, 199, 8, 31, "Usage",
      CellTags->"WordRotate"],
    Cell[61008, 2702, 110, 5, 24, "Notes",
      CellTags->"WordRotate"],
    Cell[61121, 2709, 97, 4, 24, "Notes",
      CellTags->"WordRotate"],
    Cell[61221, 2715, 68, 1, 24, "Notes",
      CellTags->"WordRotate"],
    Cell[61292, 2718, 278, 11, 24, "Notes",
      CellTags->"WordRotate"],
    Cell[61573, 2731, 68, 1, 39, "ExampleSection",
      CellTags->"WordRotate"],
    Cell[61644, 2734, 73, 1, 34, "ExampleText",
      CellTags->"WordRotate"],
    Cell[61720, 2737, 104, 2, 28, "Input",
      CellTags->"WordRotate"],
    Cell[61827, 2741, 109, 3, 26, "Input",
      CellTags->"WordRotate"]},
  "Words"->{
    Cell[61939, 2746, 48, 1, 32, "ObjectName",
      CellTags->"Words"],
    Cell[61990, 2749, 50, 1, 50, "Highlight",
      CellTags->"Words"],
    Cell[62043, 2752, 198, 8, 31, "Usage",
      CellTags->"Words"],
    Cell[62244, 2762, 220, 9, 44, "Notes",
      CellTags->"Words"],
    Cell[62467, 2773, 263, 11, 24, "Notes",
      CellTags->"Words"],
    Cell[62733, 2786, 166, 7, 24, "Notes",
      CellTags->"Words"],
    Cell[62902, 2795, 84, 1, 24, "Notes",
      CellTags->"Words"],
    Cell[62989, 2798, 174, 8, 24, "Notes",
      CellTags->"Words"],
    Cell[63166, 2808, 123, 6, 24, "Notes",
      CellTags->"Words"],
    Cell[63292, 2816, 63, 1, 39, "ExampleSection",
      CellTags->"Words"],
    Cell[63358, 2819, 89, 1, 34, "ExampleText",
      CellTags->"Words"],
    Cell[63450, 2822, 73, 2, 28, "Input",
      CellTags->"Words"],
    Cell[63526, 2826, 147, 4, 42, "Input",
      CellTags->"Words"],
    Cell[63676, 2832, 91, 3, 26, "Input",
      CellTags->"Words"],
    Cell[63770, 2837, 108, 3, 26, "Input",
      CellTags->"Words"],
    Cell[63881, 2842, 214, 6, 50, "ExampleText",
      CellTags->"Words"],
    Cell[64098, 2850, 114, 3, 26, "Input",
      CellTags->"Words"]},
  "WordShuffle"->{
    Cell[64215, 2855, 60, 1, 32, "ObjectName",
      CellTags->"WordShuffle"],
    Cell[64278, 2858, 56, 1, 50, "Highlight",
      CellTags->"WordShuffle"],
    Cell[64337, 2861, 204, 8, 31, "Usage",
      CellTags->"WordShuffle"],
    Cell[64544, 2871, 81, 1, 24, "Notes",
      CellTags->"WordShuffle"],
    Cell[64628, 2874, 141, 6, 24, "Notes",
      CellTags->"WordShuffle"],
    Cell[64772, 2882, 69, 1, 39, "ExampleSection",
      CellTags->"WordShuffle"],
    Cell[64844, 2885, 172, 4, 50, "ExampleText",
      CellTags->"WordShuffle"],
    Cell[65019, 2891, 92, 2, 28, "Input",
      CellTags->"WordShuffle"],
    Cell[65114, 2895, 93, 2, 28, "Input",
      CellTags->"WordShuffle"],
    Cell[65210, 2899, 161, 4, 34, "ExampleText",
      CellTags->"WordShuffle"],
    Cell[65374, 2905, 96, 2, 28, "Input",
      CellTags->"WordShuffle"],
    Cell[65473, 2909, 100, 2, 28, "Input",
      CellTags->"WordShuffle"],
    Cell[65576, 2913, 123, 4, 34, "ExampleText",
      CellTags->"WordShuffle"],
    Cell[65702, 2919, 138, 3, 26, "Input",
      CellTags->"WordShuffle"]},
  "WordSort"->{
    Cell[65843, 2924, 54, 1, 32, "ObjectName",
      CellTags->"WordSort"],
    Cell[65900, 2927, 53, 1, 50, "Highlight",
      CellTags->"WordSort"],
    Cell[65956, 2930, 206, 6, 31, "Usage",
      CellTags->"WordSort"],
    Cell[66165, 2938, 526, 13, 24, "Notes",
      CellTags->"WordSort"],
    Cell[66694, 2953, 136, 4, 24, "Notes",
      CellTags->"WordSort"],
    Cell[66833, 2959, 159, 7, 24, "Notes",
      CellTags->"WordSort"],
    Cell[66995, 2968, 66, 1, 39, "ExampleSection",
      CellTags->"WordSort"],
    Cell[67064, 2971, 147, 4, 34, "ExampleText",
      CellTags->"WordSort"],
    Cell[67214, 2977, 84, 2, 28, "Input",
      CellTags->"WordSort"],
    Cell[67301, 2981, 94, 3, 26, "Input",
      CellTags->"WordSort"]},
  "WordSuffixQ"->{
    Cell[67398, 2986, 60, 1, 32, "ObjectName",
      CellTags->"WordSuffixQ"],
    Cell[67461, 2989, 56, 1, 50, "Highlight",
      CellTags->"WordSuffixQ"],
    Cell[67520, 2992, 199, 8, 31, "Usage",
      CellTags->"WordSuffixQ"],
    Cell[67722, 3002, 109, 4, 24, "Notes",
      CellTags->"WordSuffixQ"],
    Cell[67834, 3008, 350, 15, 24, "Notes",
      CellTags->"WordSuffixQ"],
    Cell[68187, 3025, 69, 1, 39, "ExampleSection",
      CellTags->"WordSuffixQ"],
    Cell[68259, 3028, 77, 1, 34, "ExampleText",
      CellTags->"WordSuffixQ"],
    Cell[68339, 3031, 105, 3, 26, "Input",
      CellTags->"WordSuffixQ"],
    Cell[68447, 3036, 105, 3, 26, "Input",
      CellTags->"WordSuffixQ"],
    Cell[68555, 3041, 114, 3, 26, "Input",
      CellTags->"WordSuffixQ"],
    Cell[68672, 3046, 136, 3, 26, "Input",
      CellTags->"WordSuffixQ"]},
  "WordSuffixes"->{
    Cell[68811, 3051, 62, 1, 32, "ObjectName",
      CellTags->"WordSuffixes"],
    Cell[68876, 3054, 57, 1, 50, "Highlight",
      CellTags->"WordSuffixes"],
    Cell[68936, 3057, 179, 6, 31, "Usage",
      CellTags->"WordSuffixes"],
    Cell[69118, 3065, 227, 9, 44, "Notes",
      CellTags->"WordSuffixes"],
    Cell[69348, 3076, 171, 7, 24, "Notes",
      CellTags->"WordSuffixes"],
    Cell[69522, 3085, 112, 5, 24, "Notes",
      CellTags->"WordSuffixes"],
    Cell[69637, 3092, 363, 15, 24, "Notes",
      CellTags->"WordSuffixes"],
    Cell[70003, 3109, 70, 1, 39, "ExampleSection",
      CellTags->"WordSuffixes"],
    Cell[70076, 3112, 173, 6, 34, "ExampleText",
      CellTags->"WordSuffixes"],
    Cell[70252, 3120, 92, 2, 28, "Input",
      CellTags->"WordSuffixes"],
    Cell[70347, 3124, 95, 2, 28, "Input",
      CellTags->"WordSuffixes"],
    Cell[70445, 3128, 72, 1, 34, "ExampleText",
      CellTags->"WordSuffixes"],
    Cell[70520, 3131, 125, 3, 26, "Input",
      CellTags->"WordSuffixes"]},
  "$bottom"->{
    Cell[70648, 3136, 52, 1, 32, "ObjectName",
      CellTags->"$bottom"],
    Cell[70703, 3139, 55, 1, 50, "Highlight",
      CellTags->"$bottom"],
    Cell[70761, 3142, 195, 6, 31, "Usage",
      CellTags->"$bottom"],
    Cell[70959, 3150, 60, 1, 24, "Notes",
      CellTags->"$bottom"],
    Cell[71022, 3153, 114, 4, 24, "Notes",
      CellTags->"$bottom"],
    Cell[71139, 3159, 160, 7, 24, "Notes",
      CellTags->"$bottom"],
    Cell[71302, 3168, 65, 1, 39, "ExampleSection",
      CellTags->"$bottom"],
    Cell[71370, 3171, 105, 2, 34, "ExampleText",
      CellTags->"$bottom"],
    Cell[71478, 3175, 249, 6, 42, "Input",
      CellTags->"$bottom"],
    Cell[71730, 3183, 173, 5, 58, "Input",
      CellTags->"$bottom"],
    Cell[71906, 3190, 107, 2, 34, "ExampleText",
      CellTags->"$bottom"],
    Cell[72016, 3194, 170, 4, 42, "Input",
      CellTags->"$bottom"]},
  "$defaultnormalize"->{
    Cell[72189, 3200, 72, 1, 32, "ObjectName",
      CellTags->"$defaultnormalize"],
    Cell[72264, 3203, 65, 1, 50, "Highlight",
      CellTags->"$defaultnormalize"],
    Cell[72332, 3206, 200, 6, 31, "Usage",
      CellTags->"$defaultnormalize"],
    Cell[72535, 3214, 183, 5, 24, "Notes",
      CellTags->"$defaultnormalize"],
    Cell[72721, 3221, 328, 10, 40, "Notes",
      CellTags->"$defaultnormalize"],
    Cell[73052, 3233, 444, 12, 56, "Notes",
      CellTags->"$defaultnormalize"],
    Cell[73499, 3247, 258, 7, 40, "Notes",
      CellTags->"$defaultnormalize"],
    Cell[73760, 3256, 338, 7, 56, "Notes",
      CellTags->"$defaultnormalize"],
    Cell[74101, 3265, 266, 11, 24, "Notes",
      CellTags->"$defaultnormalize"],
    Cell[74370, 3278, 75, 1, 39, "ExampleSection",
      CellTags->"$defaultnormalize"],
    Cell[74448, 3281, 95, 1, 34, "ExampleText",
      CellTags->"$defaultnormalize"],
    Cell[74546, 3284, 112, 2, 28, "Input",
      CellTags->"$defaultnormalize"],
    Cell[74661, 3288, 80, 1, 34, "ExampleText",
      CellTags->"$defaultnormalize"],
    Cell[74744, 3291, 168, 4, 26, "Input",
      CellTags->"$defaultnormalize"],
    Cell[74915, 3297, 163, 4, 26, "Input",
      CellTags->"$defaultnormalize"]},
  "AcceptQFA"->{
    Cell[75081, 3303, 56, 1, 32, "ObjectName",
      CellTags->"AcceptQFA"],
    Cell[75140, 3306, 54, 1, 50, "Highlight",
      CellTags->"AcceptQFA"],
    Cell[75197, 3309, 193, 8, 31, "Usage",
      CellTags->"AcceptQFA"],
    Cell[75393, 3319, 281, 10, 61, "Notes",
      CellTags->"AcceptQFA"],
    Cell[75677, 3331, 144, 5, 24, "Notes",
      CellTags->"AcceptQFA"],
    Cell[75824, 3338, 267, 11, 24, "Notes",
      CellTags->"AcceptQFA"],
    Cell[76094, 3351, 274, 12, 24, "Notes",
      CellTags->"AcceptQFA"],
    Cell[76371, 3365, 67, 1, 39, "ExampleSection",
      CellTags->"AcceptQFA"],
    Cell[76441, 3368, 169, 6, 34, "ExampleText",
      CellTags->"AcceptQFA"],
    Cell[76613, 3376, 87, 2, 28, "Input",
      CellTags->"AcceptQFA"],
    Cell[76703, 3380, 103, 3, 26, "Input",
      CellTags->"AcceptQFA"],
    Cell[76809, 3385, 125, 3, 26, "Input",
      CellTags->"AcceptQFA"],
    Cell[76937, 3390, 79, 1, 34, "ExampleText",
      CellTags->"AcceptQFA"],
    Cell[77019, 3393, 123, 3, 26, "Input",
      CellTags->"AcceptQFA"],
    Cell[77145, 3398, 100, 1, 34, "ExampleText",
      CellTags->"AcceptQFA"],
    Cell[77248, 3401, 106, 3, 26, "Input",
      CellTags->"AcceptQFA"]},
  "AccessibleFA"->{
    Cell[77357, 3406, 62, 1, 32, "ObjectName",
      CellTags->"AccessibleFA"],
    Cell[77422, 3409, 57, 1, 50, "Highlight",
      CellTags->"AccessibleFA"],
    Cell[77482, 3412, 171, 6, 31, "Usage",
      CellTags->"AccessibleFA"],
    Cell[77656, 3420, 319, 10, 61, "Notes",
      CellTags->"AccessibleFA"],
    Cell[77978, 3432, 92, 1, 24, "Notes",
      CellTags->"AccessibleFA"],
    Cell[78073, 3435, 129, 4, 24, "Notes",
      CellTags->"AccessibleFA"],
    Cell[78205, 3441, 264, 11, 24, "Notes",
      CellTags->"AccessibleFA"],
    Cell[78472, 3454, 70, 1, 39, "ExampleSection",
      CellTags->"AccessibleFA"],
    Cell[78545, 3457, 180, 4, 50, "ExampleText",
      CellTags->"AccessibleFA"],
    Cell[78728, 3463, 206, 5, 42, "Input",
      CellTags->"AccessibleFA"],
    Cell[78937, 3470, 150, 4, 42, "Input",
      CellTags->"AccessibleFA"],
    Cell[79090, 3476, 106, 1, 34, "ExampleText",
      CellTags->"AccessibleFA"],
    Cell[79199, 3479, 136, 3, 26, "Input",
      CellTags->"AccessibleFA"],
    Cell[79338, 3484, 136, 3, 26, "Input",
      CellTags->"AccessibleFA"]},
  "AddTransitionFA"->{
    Cell[79477, 3489, 68, 1, 32, "ObjectName",
      CellTags->"AddTransitionFA"],
    Cell[79548, 3492, 60, 1, 50, "Highlight",
      CellTags->"AddTransitionFA"],
    Cell[79611, 3495, 202, 8, 31, "Usage",
      CellTags->"AddTransitionFA"],
    Cell[79816, 3505, 165, 5, 24, "Notes",
      CellTags->"AddTransitionFA"],
    Cell[79984, 3512, 261, 9, 40, "Notes",
      CellTags->"AddTransitionFA"],
    Cell[80248, 3523, 296, 11, 24, "Notes",
      CellTags->"AddTransitionFA"],
    Cell[80547, 3536, 73, 1, 39, "ExampleSection",
      CellTags->"AddTransitionFA"],
    Cell[80623, 3539, 94, 1, 34, "ExampleText",
      CellTags->"AddTransitionFA"],
    Cell[80720, 3542, 204, 5, 42, "Input",
      CellTags->"AddTransitionFA"],
    Cell[80927, 3549, 187, 6, 34, "ExampleText",
      CellTags->"AddTransitionFA"],
    Cell[81117, 3557, 172, 4, 42, "Input",
      CellTags->"AddTransitionFA"],
    Cell[81292, 3563, 178, 6, 34, "ExampleText",
      CellTags->"AddTransitionFA"],
    Cell[81473, 3571, 179, 4, 42, "Input",
      CellTags->"AddTransitionFA"]},
  "AllDFA"->{
    Cell[81655, 3577, 50, 1, 32, "ObjectName",
      CellTags->"AllDFA"],
    Cell[81708, 3580, 51, 1, 50, "Highlight",
      CellTags->"AllDFA"],
    Cell[81762, 3583, 172, 6, 31, "Usage",
      CellTags->"AllDFA"],
    Cell[81937, 3591, 225, 9, 44, "Notes",
      CellTags->"AllDFA"],
    Cell[82165, 3602, 148, 5, 24, "Notes",
      CellTags->"AllDFA"],
    Cell[82316, 3609, 256, 12, 24, "Notes",
      CellTags->"AllDFA"],
    Cell[82575, 3623, 64, 1, 39, "ExampleSection",
      CellTags->"AllDFA"],
    Cell[82642, 3626, 179, 6, 34, "ExampleText",
      CellTags->"AllDFA"],
    Cell[82824, 3634, 118, 3, 26, "Input",
      CellTags->"AllDFA"],
    Cell[82945, 3639, 102, 3, 26, "Input",
      CellTags->"AllDFA"]},
  "AmbiguousQFA"->{
    Cell[83050, 3644, 62, 1, 32, "ObjectName",
      CellTags->"AmbiguousQFA"],
    Cell[83115, 3647, 57, 1, 50, "Highlight",
      CellTags->"AmbiguousQFA"],
    Cell[83175, 3650, 168, 6, 31, "Usage",
      CellTags->"AmbiguousQFA"],
    Cell[83346, 3658, 295, 10, 61, "Notes",
      CellTags->"AmbiguousQFA"],
    Cell[83644, 3670, 196, 6, 40, "Notes",
      CellTags->"AmbiguousQFA"],
    Cell[83843, 3678, 134, 4, 24, "Notes",
      CellTags->"AmbiguousQFA"],
    Cell[83980, 3684, 303, 13, 24, "Notes",
      CellTags->"AmbiguousQFA"],
    Cell[84286, 3699, 70, 1, 39, "ExampleSection",
      CellTags->"AmbiguousQFA"],
    Cell[84359, 3702, 81, 1, 34, "ExampleText",
      CellTags->"AmbiguousQFA"],
    Cell[84443, 3705, 222, 8, 41, "Input",
      CellTags->"AmbiguousQFA"],
    Cell[84668, 3715, 104, 1, 34, "ExampleText",
      CellTags->"AmbiguousQFA"],
    Cell[84775, 3718, 147, 4, 42, "Input",
      CellTags->"AmbiguousQFA"],
    Cell[84925, 3724, 178, 4, 50, "ExampleText",
      CellTags->"AmbiguousQFA"],
    Cell[85106, 3730, 166, 4, 42, "Input",
      CellTags->"AmbiguousQFA"],
    Cell[85275, 3736, 133, 3, 26, "Input",
      CellTags->"AmbiguousQFA"]},
  "BEFA"->{
    Cell[85411, 3741, 46, 1, 32, "ObjectName",
      CellTags->"BEFA"],
    Cell[85460, 3744, 52, 1, 50, "Highlight",
      CellTags->"BEFA"],
    Cell[85515, 3747, 238, 6, 47, "Usage",
      CellTags->"BEFA"],
    Cell[85756, 3755, 273, 8, 40, "Notes",
      CellTags->"BEFA"],
    Cell[86032, 3765, 267, 9, 40, "Notes",
      CellTags->"BEFA"],
    Cell[86302, 3776, 95, 1, 24, "Notes",
      CellTags->"BEFA"],
    Cell[86400, 3779, 140, 6, 24, "Notes",
      CellTags->"BEFA"],
    Cell[86543, 3787, 62, 1, 39, "ExampleSection",
      CellTags->"BEFA"],
    Cell[86608, 3790, 193, 6, 34, "ExampleText",
      CellTags->"BEFA"],
    Cell[86804, 3798, 178, 4, 24, "Input",
      CellTags->"BEFA"],
    Cell[86985, 3804, 102, 2, 34, "ExampleText",
      CellTags->"BEFA"],
    Cell[87090, 3808, 168, 5, 45, "Input",
      CellTags->"BEFA"]},
  "BEFAToFA"->{
    Cell[87261, 3815, 54, 1, 32, "ObjectName",
      CellTags->"BEFAToFA"],
    Cell[87318, 3818, 53, 1, 50, "Highlight",
      CellTags->"BEFAToFA"],
    Cell[87374, 3821, 170, 7, 31, "Usage",
      CellTags->"BEFAToFA"],
    Cell[87547, 3830, 97, 1, 24, "Notes",
      CellTags->"BEFAToFA"],
    Cell[87647, 3833, 264, 11, 24, "Notes",
      CellTags->"BEFAToFA"],
    Cell[87914, 3846, 66, 1, 39, "ExampleSection",
      CellTags->"BEFAToFA"],
    Cell[87983, 3849, 86, 1, 34, "ExampleText",
      CellTags->"BEFAToFA"],
    Cell[88072, 3852, 127, 3, 26, "Input",
      CellTags->"BEFAToFA"],
    Cell[88202, 3857, 95, 3, 26, "Input",
      CellTags->"BEFAToFA"]},
  "BehaviorFA"->{
    Cell[88300, 3862, 58, 1, 32, "ObjectName",
      CellTags->"BehaviorFA"],
    Cell[88361, 3865, 55, 1, 50, "Highlight",
      CellTags->"BehaviorFA"],
    Cell[88419, 3868, 251, 9, 47, "Usage",
      CellTags->"BehaviorFA"],
    Cell[88673, 3879, 367, 12, 40, "Notes",
      CellTags->"BehaviorFA"],
    Cell[89043, 3893, 185, 7, 24, "Notes",
      CellTags->"BehaviorFA"],
    Cell[89231, 3902, 174, 4, 40, "Notes",
      CellTags->"BehaviorFA"],
    Cell[89408, 3908, 195, 9, 24, "Notes",
      CellTags->"BehaviorFA"],
    Cell[89606, 3919, 68, 1, 39, "ExampleSection",
      CellTags->"BehaviorFA"],
    Cell[89677, 3922, 199, 6, 34, "ExampleText",
      CellTags->"BehaviorFA"],
    Cell[89879, 3930, 127, 3, 26, "Input",
      CellTags->"BehaviorFA"],
    Cell[90009, 3935, 132, 4, 34, "ExampleText",
      CellTags->"BehaviorFA"],
    Cell[90144, 3941, 109, 3, 26, "Input",
      CellTags->"BehaviorFA"],
    Cell[90256, 3946, 187, 6, 34, "ExampleText",
      CellTags->"BehaviorFA"],
    Cell[90446, 3954, 141, 4, 42, "Input",
      CellTags->"BehaviorFA"],
    Cell[90590, 3960, 109, 3, 26, "Input",
      CellTags->"BehaviorFA"]},
  "Boundary"->{
    Cell[90702, 3965, 54, 1, 32, "ObjectName",
      CellTags->"Boundary"],
    Cell[90759, 3968, 51, 1, 50, "Highlight",
      CellTags->"Boundary"],
    Cell[90813, 3971, 144, 6, 31, "Usage",
      CellTags->"Boundary"],
    Cell[90960, 3979, 186, 9, 24, "Notes",
      CellTags->"Boundary"],
    Cell[91149, 3990, 345, 15, 24, "Notes",
      CellTags->"Boundary"]},
  "BuildBEFA"->{
    Cell[91497, 4007, 56, 1, 32, "ObjectName",
      CellTags->"BuildBEFA"],
    Cell[91556, 4010, 54, 1, 50, "Highlight",
      CellTags->"BuildBEFA"],
    Cell[91613, 4013, 197, 7, 31, "Usage",
      CellTags->"BuildBEFA"],
    Cell[91813, 4022, 117, 5, 24, "Notes",
      CellTags->"BuildBEFA"],
    Cell[91933, 4029, 134, 4, 24, "Notes",
      CellTags->"BuildBEFA"],
    Cell[92070, 4035, 206, 9, 24, "Notes",
      CellTags->"BuildBEFA"],
    Cell[92279, 4046, 67, 1, 39, "ExampleSection",
      CellTags->"BuildBEFA"],
    Cell[92349, 4049, 231, 7, 50, "ExampleText",
      CellTags->"BuildBEFA"],
    Cell[92583, 4058, 85, 2, 28, "Input",
      CellTags->"BuildBEFA"]},
  "BuildDFA"->{
    Cell[92671, 4062, 54, 1, 32, "ObjectName",
      CellTags->"BuildDFA"],
    Cell[92728, 4065, 53, 1, 50, "Highlight",
      CellTags->"BuildDFA"],
    Cell[92784, 4068, 230, 7, 47, "Usage",
      CellTags->"BuildDFA"],
    Cell[93017, 4077, 248, 9, 44, "Notes",
      CellTags->"BuildDFA"],
    Cell[93268, 4088, 257, 8, 40, "Notes",
      CellTags->"BuildDFA"],
    Cell[93528, 4098, 79, 1, 24, "Notes",
      CellTags->"BuildDFA"],
    Cell[93610, 4101, 164, 7, 24, "Notes",
      CellTags->"BuildDFA"],
    Cell[93777, 4110, 66, 1, 39, "ExampleSection",
      CellTags->"BuildDFA"],
    Cell[93846, 4113, 75, 1, 34, "ExampleText",
      CellTags->"BuildDFA"],
    Cell[93924, 4116, 152, 4, 42, "Input",
      CellTags->"BuildDFA"],
    Cell[94079, 4122, 149, 6, 34, "ExampleText",
      CellTags->"BuildDFA"],
    Cell[94231, 4130, 184, 5, 58, "Input",
      CellTags->"BuildDFA"],
    Cell[94418, 4137, 123, 4, 34, "ExampleText",
      CellTags->"BuildDFA"],
    Cell[94544, 4143, 138, 3, 26, "Input",
      CellTags->"BuildDFA"]},
  "BuildNNFA"->{
    Cell[94685, 4148, 56, 1, 32, "ObjectName",
      CellTags->"BuildNNFA"],
    Cell[94744, 4151, 54, 1, 50, "Highlight",
      CellTags->"BuildNNFA"],
    Cell[94801, 4154, 213, 6, 47, "Usage",
      CellTags->"BuildNNFA"],
    Cell[95017, 4162, 117, 5, 24, "Notes",
      CellTags->"BuildNNFA"],
    Cell[95137, 4169, 145, 4, 24, "Notes",
      CellTags->"BuildNNFA"],
    Cell[95285, 4175, 205, 9, 24, "Notes",
      CellTags->"BuildNNFA"],
    Cell[95493, 4186, 67, 1, 39, "ExampleSection",
      CellTags->"BuildNNFA"],
    Cell[95563, 4189, 73, 1, 34, "ExampleText",
      CellTags->"BuildNNFA"],
    Cell[95639, 4192, 82, 2, 28, "Input",
      CellTags->"BuildNNFA"]},
  "BuildStarDFA"->{
    Cell[95724, 4196, 62, 1, 32, "ObjectName",
      CellTags->"BuildStarDFA"],
    Cell[95789, 4199, 57, 1, 50, "Highlight",
      CellTags->"BuildStarDFA"],
    Cell[95849, 4202, 229, 7, 47, "Usage",
      CellTags->"BuildStarDFA"],
    Cell[96081, 4211, 91, 1, 24, "Notes",
      CellTags->"BuildStarDFA"],
    Cell[96175, 4214, 195, 9, 24, "Notes",
      CellTags->"BuildStarDFA"],
    Cell[96373, 4225, 70, 1, 39, "ExampleSection",
      CellTags->"BuildStarDFA"],
    Cell[96446, 4228, 400, 14, 34, "ExampleText",
      CellTags->"BuildStarDFA"],
    Cell[96849, 4244, 112, 3, 26, "Input",
      CellTags->"BuildStarDFA"],
    Cell[96964, 4249, 134, 3, 26, "Input",
      CellTags->"BuildStarDFA"],
    Cell[97101, 4254, 125, 4, 34, "ExampleText",
      CellTags->"BuildStarDFA"],
    Cell[97229, 4260, 144, 3, 26, "Input",
      CellTags->"BuildStarDFA"]},
  "CirculantSA"->{
    Cell[97376, 4265, 60, 1, 32, "ObjectName",
      CellTags->"CirculantSA"],
    Cell[97439, 4268, 56, 1, 50, "Highlight",
      CellTags->"CirculantSA"],
    Cell[97498, 4271, 258, 7, 47, "Usage",
      CellTags->"CirculantSA"],
    Cell[97759, 4280, 325, 12, 40, "Notes",
      CellTags->"CirculantSA"],
    Cell[98087, 4294, 133, 5, 24, "Notes",
      CellTags->"CirculantSA"],
    Cell[98223, 4301, 260, 8, 24, "Notes",
      CellTags->"CirculantSA"],
    Cell[98486, 4311, 172, 7, 24, "Notes",
      CellTags->"CirculantSA"],
    Cell[98661, 4320, 69, 1, 39, "ExampleSection",
      CellTags->"CirculantSA"],
    Cell[98733, 4323, 245, 9, 34, "ExampleText",
      CellTags->"CirculantSA"],
    Cell[98981, 4334, 138, 3, 26, "Input",
      CellTags->"CirculantSA"],
    Cell[99122, 4339, 102, 3, 26, "Input",
      CellTags->"CirculantSA"]},
  "CoAccessibleFA"->{
    Cell[99227, 4344, 66, 1, 32, "ObjectName",
      CellTags->"CoAccessibleFA"],
    Cell[99296, 4347, 59, 1, 50, "Highlight",
      CellTags->"CoAccessibleFA"],
    Cell[99358, 4350, 191, 6, 31, "Usage",
      CellTags->"CoAccessibleFA"],
    Cell[99552, 4358, 321, 10, 61, "Notes",
      CellTags->"CoAccessibleFA"],
    Cell[99876, 4370, 167, 5, 24, "Notes",
      CellTags->"CoAccessibleFA"],
    Cell[100046, 4377, 262, 11, 24, "Notes",
      CellTags->"CoAccessibleFA"],
    Cell[100311, 4390, 72, 1, 39, "ExampleSection",
      CellTags->"CoAccessibleFA"],
    Cell[100386, 4393, 125, 4, 34, "ExampleText",
      CellTags->"CoAccessibleFA"],
    Cell[100514, 4399, 195, 5, 42, "Input",
      CellTags->"CoAccessibleFA"],
    Cell[100712, 4406, 145, 4, 42, "Input",
      CellTags->"CoAccessibleFA"],
    Cell[100860, 4412, 108, 1, 34, "ExampleText",
      CellTags->"CoAccessibleFA"],
    Cell[100971, 4415, 141, 3, 26, "Input",
      CellTags->"CoAccessibleFA"],
    Cell[101115, 4420, 141, 3, 26, "Input",
      CellTags->"CoAccessibleFA"]},
  "ComplementFA"->{
    Cell[101259, 4425, 62, 1, 32, "ObjectName",
      CellTags->"ComplementFA"],
    Cell[101324, 4428, 57, 1, 50, "Highlight",
      CellTags->"ComplementFA"],
    Cell[101384, 4431, 215, 7, 47, "Usage",
      CellTags->"ComplementFA"],
    Cell[101602, 4440, 286, 7, 40, "Notes",
      CellTags->"ComplementFA"],
    Cell[101891, 4449, 255, 8, 24, "Notes",
      CellTags->"ComplementFA"],
    Cell[102149, 4459, 524, 24, 40, "Notes",
      CellTags->"ComplementFA"],
    Cell[102676, 4485, 70, 1, 39, "ExampleSection",
      CellTags->"ComplementFA"],
    Cell[102749, 4488, 134, 4, 34, "ExampleText",
      CellTags->"ComplementFA"],
    Cell[102886, 4494, 98, 2, 28, "Input",
      CellTags->"ComplementFA"],
    Cell[102987, 4498, 111, 3, 26, "Input",
      CellTags->"ComplementFA"],
    Cell[103101, 4503, 134, 3, 26, "Input",
      CellTags->"ComplementFA"],
    Cell[103238, 4508, 100, 2, 28, "Input",
      CellTags->"ComplementFA"],
    Cell[103341, 4512, 110, 2, 34, "ExampleText",
      CellTags->"ComplementFA"],
    Cell[103454, 4516, 121, 3, 26, "Input",
      CellTags->"ComplementFA"],
    Cell[103578, 4521, 134, 3, 26, "Input",
      CellTags->"ComplementFA"]},
  "CompleteFA"->{
    Cell[103715, 4526, 58, 1, 32, "ObjectName",
      CellTags->"CompleteFA"],
    Cell[103776, 4529, 55, 1, 50, "Highlight",
      CellTags->"CompleteFA"],
    Cell[103834, 4532, 186, 6, 31, "Usage",
      CellTags->"CompleteFA"],
    Cell[104023, 4540, 126, 4, 24, "Notes",
      CellTags->"CompleteFA"],
    Cell[104152, 4546, 195, 9, 24, "Notes",
      CellTags->"CompleteFA"],
    Cell[104350, 4557, 68, 1, 39, "ExampleSection",
      CellTags->"CompleteFA"],
    Cell[104421, 4560, 102, 1, 34, "ExampleText",
      CellTags->"CompleteFA"],
    Cell[104526, 4563, 98, 2, 28, "Input",
      CellTags->"CompleteFA"],
    Cell[104627, 4567, 98, 3, 26, "Input",
      CellTags->"CompleteFA"],
    Cell[104728, 4572, 105, 3, 26, "Input",
      CellTags->"CompleteFA"]},
  "CompleteQFA"->{
    Cell[104836, 4577, 60, 1, 32, "ObjectName",
      CellTags->"CompleteQFA"],
    Cell[104899, 4580, 56, 1, 50, "Highlight",
      CellTags->"CompleteQFA"],
    Cell[104958, 4583, 179, 6, 31, "Usage",
      CellTags->"CompleteQFA"],
    Cell[105140, 4591, 160, 4, 40, "Notes",
      CellTags->"CompleteQFA"],
    Cell[105303, 4597, 99, 1, 24, "Notes",
      CellTags->"CompleteQFA"],
    Cell[105405, 4600, 195, 9, 24, "Notes",
      CellTags->"CompleteQFA"],
    Cell[105603, 4611, 69, 1, 39, "ExampleSection",
      CellTags->"CompleteQFA"],
    Cell[105675, 4614, 96, 1, 34, "ExampleText",
      CellTags->"CompleteQFA"],
    Cell[105774, 4617, 123, 5, 45, "Input",
      CellTags->"CompleteQFA"],
    Cell[105900, 4624, 112, 2, 34, "ExampleText",
      CellTags->"CompleteQFA"],
    Cell[106015, 4628, 103, 2, 28, "Input",
      CellTags->"CompleteQFA"]},
  "ComputationFA"->{
    Cell[106121, 4632, 64, 1, 32, "ObjectName",
      CellTags->"ComputationFA"],
    Cell[106188, 4635, 58, 1, 50, "Highlight",
      CellTags->"ComputationFA"],
    Cell[106249, 4638, 260, 8, 47, "Usage",
      CellTags->"ComputationFA"],
    Cell[106512, 4648, 228, 9, 44, "Notes",
      CellTags->"ComputationFA"],
    Cell[106743, 4659, 164, 5, 24, "Notes",
      CellTags->"ComputationFA"],
    Cell[106910, 4666, 195, 5, 40, "Notes",
      CellTags->"ComputationFA"],
    Cell[107108, 4673, 148, 4, 24, "Notes",
      CellTags->"ComputationFA"],
    Cell[107259, 4679, 295, 11, 24, "Notes",
      CellTags->"ComputationFA"],
    Cell[107557, 4692, 71, 1, 39, "ExampleSection",
      CellTags->"ComputationFA"],
    Cell[107631, 4695, 202, 7, 34, "ExampleText",
      CellTags->"ComputationFA"],
    Cell[107836, 4704, 114, 3, 26, "Input",
      CellTags->"ComputationFA"],
    Cell[107953, 4709, 211, 8, 34, "ExampleText",
      CellTags->"ComputationFA"],
    Cell[108167, 4719, 118, 3, 26, "Input",
      CellTags->"ComputationFA"],
    Cell[108288, 4724, 93, 1, 34, "ExampleText",
      CellTags->"ComputationFA"],
    Cell[108384, 4727, 138, 3, 26, "Input",
      CellTags->"ComputationFA"],
    Cell[108525, 4732, 105, 1, 34, "ExampleText",
      CellTags->"ComputationFA"],
    Cell[108633, 4735, 137, 3, 26, "Input",
      CellTags->"ComputationFA"]},
  "ComputationGraphFA"->{
    Cell[108773, 4740, 74, 1, 32, "ObjectName",
      CellTags->"ComputationGraphFA"],
    Cell[108850, 4743, 63, 1, 50, "Highlight",
      CellTags->"ComputationGraphFA"],
    Cell[108916, 4746, 253, 8, 47, "Usage",
      CellTags->"ComputationGraphFA"],
    Cell[109172, 4756, 295, 9, 40, "Notes",
      CellTags->"ComputationGraphFA"],
    Cell[109470, 4767, 178, 6, 24, "Notes",
      CellTags->"ComputationGraphFA"],
    Cell[109651, 4775, 384, 15, 24, "Notes",
      CellTags->"ComputationGraphFA"],
    Cell[110038, 4792, 76, 1, 39, "ExampleSection",
      CellTags->"ComputationGraphFA"],
    Cell[110117, 4795, 207, 7, 34, "ExampleText",
      CellTags->"ComputationGraphFA"],
    Cell[110327, 4804, 119, 3, 26, "Input",
      CellTags->"ComputationGraphFA"],
    Cell[110449, 4809, 165, 4, 26, "Input",
      CellTags->"ComputationGraphFA"]},
  "ConcatenateFA"->{
    Cell[110617, 4815, 64, 1, 32, "ObjectName",
      CellTags->"ConcatenateFA"],
    Cell[110684, 4818, 58, 1, 50, "Highlight",
      CellTags->"ConcatenateFA"],
    Cell[110745, 4821, 257, 9, 47, "Usage",
      CellTags->"ConcatenateFA"],
    Cell[111005, 4832, 300, 10, 61, "Notes",
      CellTags->"ConcatenateFA"],
    Cell[111308, 4844, 193, 6, 24, "Notes",
      CellTags->"ConcatenateFA"],
    Cell[111504, 4852, 247, 7, 24, "Notes",
      CellTags->"ConcatenateFA"],
    Cell[111754, 4861, 285, 9, 24, "Notes",
      CellTags->"ConcatenateFA"],
    Cell[112042, 4872, 110, 4, 24, "Notes",
      CellTags->"ConcatenateFA"],
    Cell[112155, 4878, 459, 21, 24, "Notes",
      CellTags->"ConcatenateFA"],
    Cell[112617, 4901, 71, 1, 39, "ExampleSection",
      CellTags->"ConcatenateFA"],
    Cell[112691, 4904, 103, 1, 34, "ExampleText",
      CellTags->"ConcatenateFA"],
    Cell[112797, 4907, 100, 2, 28, "Input",
      CellTags->"ConcatenateFA"],
    Cell[112900, 4911, 118, 3, 26, "Input",
      CellTags->"ConcatenateFA"],
    Cell[113021, 4916, 141, 3, 26, "Input",
      CellTags->"ConcatenateFA"]},
  "ConvertToFA"->{
    Cell[113165, 4921, 60, 1, 32, "ObjectName",
      CellTags->"ConvertToFA"],
    Cell[113228, 4924, 56, 1, 50, "Highlight",
      CellTags->"ConvertToFA"],
    Cell[113287, 4927, 372, 16, 47, "Usage",
      CellTags->"ConvertToFA"],
    Cell[113662, 4945, 300, 10, 61, "Notes",
      CellTags->"ConvertToFA"],
    Cell[113965, 4957, 218, 6, 40, "Notes",
      CellTags->"ConvertToFA"],
    Cell[114186, 4965, 273, 12, 24, "Notes",
      CellTags->"ConvertToFA"],
    Cell[114462, 4979, 69, 1, 39, "ExampleSection",
      CellTags->"ConvertToFA"],
    Cell[114534, 4982, 509, 15, 66, "ExampleText",
      CellTags->"ConvertToFA"],
    Cell[115046, 4999, 252, 7, 42, "Input",
      CellTags->"ConvertToFA"],
    Cell[115301, 5008, 107, 3, 26, "Input",
      CellTags->"ConvertToFA"],
    Cell[115411, 5013, 110, 2, 34, "ExampleText",
      CellTags->"ConvertToFA"],
    Cell[115524, 5017, 315, 7, 58, "Input",
      CellTags->"ConvertToFA"],
    Cell[115842, 5026, 126, 3, 26, "Input",
      CellTags->"ConvertToFA"]},
  "CounterDFA"->{
    Cell[115971, 5031, 58, 1, 32, "ObjectName",
      CellTags->"CounterDFA"],
    Cell[116032, 5034, 55, 1, 50, "Highlight",
      CellTags->"CounterDFA"],
    Cell[116090, 5037, 310, 12, 47, "Usage",
      CellTags->"CounterDFA"],
    Cell[116403, 5051, 519, 18, 61, "Notes",
      CellTags->"CounterDFA"],
    Cell[116925, 5071, 283, 9, 40, "Notes",
      CellTags->"CounterDFA"],
    Cell[117211, 5082, 386, 14, 40, "Notes",
      CellTags->"CounterDFA"],
    Cell[117600, 5098, 205, 9, 24, "Notes",
      CellTags->"CounterDFA"],
    Cell[117808, 5109, 178, 7, 24, "Notes",
      CellTags->"CounterDFA"],
    Cell[117989, 5118, 68, 1, 39, "ExampleSection",
      CellTags->"CounterDFA"],
    Cell[118060, 5121, 153, 6, 34, "ExampleText",
      CellTags->"CounterDFA"],
    Cell[118216, 5129, 150, 4, 42, "Input",
      CellTags->"CounterDFA"],
    Cell[118369, 5135, 160, 6, 34, "ExampleText",
      CellTags->"CounterDFA"],
    Cell[118532, 5143, 159, 4, 42, "Input",
      CellTags->"CounterDFA"],
    Cell[118694, 5149, 159, 6, 34, "ExampleText",
      CellTags->"CounterDFA"],
    Cell[118856, 5157, 173, 4, 42, "Input",
      CellTags->"CounterDFA"]},
  "CoverQDFA"->{
    Cell[119032, 5163, 56, 1, 32, "ObjectName",
      CellTags->"CoverQDFA"],
    Cell[119091, 5166, 54, 1, 50, "Highlight",
      CellTags->"CoverQDFA"],
    Cell[119148, 5169, 194, 8, 31, "Usage",
      CellTags->"CoverQDFA"],
    Cell[119345, 5179, 228, 9, 44, "Notes",
      CellTags->"CoverQDFA"],
    Cell[119576, 5190, 182, 6, 40, "Notes",
      CellTags->"CoverQDFA"],
    Cell[119761, 5198, 97, 1, 24, "Notes",
      CellTags->"CoverQDFA"],
    Cell[119861, 5201, 259, 10, 24, "Notes",
      CellTags->"CoverQDFA"],
    Cell[120123, 5213, 384, 16, 24, "Notes",
      CellTags->"CoverQDFA"],
    Cell[120510, 5231, 67, 1, 39, "ExampleSection",
      CellTags->"CoverQDFA"],
    Cell[120580, 5234, 132, 4, 34, "ExampleText",
      CellTags->"CoverQDFA"],
    Cell[120715, 5240, 166, 4, 42, "Input",
      CellTags->"CoverQDFA"],
    Cell[120884, 5246, 100, 3, 26, "Input",
      CellTags->"CoverQDFA"],
    Cell[120987, 5251, 122, 3, 26, "Input",
      CellTags->"CoverQDFA"]},
  "DefiniteQFA"->{
    Cell[121112, 5256, 60, 1, 32, "ObjectName",
      CellTags->"DefiniteQFA"],
    Cell[121175, 5259, 56, 1, 50, "Highlight",
      CellTags->"DefiniteQFA"],
    Cell[121234, 5262, 343, 10, 69, "Usage",
      CellTags->"DefiniteQFA"],
    Cell[121580, 5274, 294, 10, 61, "Notes",
      CellTags->"DefiniteQFA"],
    Cell[121877, 5286, 164, 5, 24, "Notes",
      CellTags->"DefiniteQFA"],
    Cell[122044, 5293, 170, 4, 40, "Notes",
      CellTags->"DefiniteQFA"],
    Cell[122217, 5299, 231, 10, 24, "Notes",
      CellTags->"DefiniteQFA"],
    Cell[122451, 5311, 69, 1, 39, "ExampleSection",
      CellTags->"DefiniteQFA"],
    Cell[122523, 5314, 71, 1, 34, "ExampleText",
      CellTags->"DefiniteQFA"],
    Cell[122597, 5317, 197, 5, 42, "Input",
      CellTags->"DefiniteQFA"],
    Cell[122797, 5324, 105, 1, 34, "ExampleText",
      CellTags->"DefiniteQFA"],
    Cell[122905, 5327, 118, 3, 26, "Input",
      CellTags->"DefiniteQFA"],
    Cell[123026, 5332, 118, 3, 26, "Input",
      CellTags->"DefiniteQFA"],
    Cell[123147, 5337, 124, 3, 26, "Input",
      CellTags->"DefiniteQFA"],
    Cell[123274, 5342, 124, 3, 26, "Input",
      CellTags->"DefiniteQFA"]},
  "DegreesFA"->{
    Cell[123401, 5347, 56, 1, 32, "ObjectName",
      CellTags->"DegreesFA"],
    Cell[123460, 5350, 54, 1, 50, "Highlight",
      CellTags->"DegreesFA"],
    Cell[123517, 5353, 343, 14, 47, "Usage",
      CellTags->"DegreesFA"],
    Cell[123863, 5369, 276, 10, 61, "Notes",
      CellTags->"DegreesFA"],
    Cell[124142, 5381, 223, 6, 40, "Notes",
      CellTags->"DegreesFA"],
    Cell[124368, 5389, 153, 5, 24, "Notes",
      CellTags->"DegreesFA"],
    Cell[124524, 5396, 155, 4, 40, "Notes",
      CellTags->"DegreesFA"],
    Cell[124682, 5402, 203, 9, 24, "Notes",
      CellTags->"DegreesFA"],
    Cell[124888, 5413, 67, 1, 39, "ExampleSection",
      CellTags->"DegreesFA"],
    Cell[124958, 5416, 135, 4, 34, "ExampleText",
      CellTags->"DegreesFA"],
    Cell[125096, 5422, 97, 2, 28, "Input",
      CellTags->"DegreesFA"],
    Cell[125196, 5426, 114, 3, 26, "Input",
      CellTags->"DegreesFA"]},
  "DeleteStateFA"->{
    Cell[125313, 5431, 64, 1, 32, "ObjectName",
      CellTags->"DeleteStateFA"],
    Cell[125380, 5434, 58, 1, 50, "Highlight",
      CellTags->"DeleteStateFA"],
    Cell[125441, 5437, 191, 8, 31, "Usage",
      CellTags->"DeleteStateFA"],
    Cell[125635, 5447, 230, 9, 44, "Notes",
      CellTags->"DeleteStateFA"],
    Cell[125868, 5458, 180, 6, 24, "Notes",
      CellTags->"DeleteStateFA"],
    Cell[126051, 5466, 180, 6, 24, "Notes",
      CellTags->"DeleteStateFA"],
    Cell[126234, 5474, 216, 9, 24, "Notes",
      CellTags->"DeleteStateFA"],
    Cell[126453, 5485, 71, 1, 39, "ExampleSection",
      CellTags->"DeleteStateFA"],
    Cell[126527, 5488, 129, 4, 34, "ExampleText",
      CellTags->"DeleteStateFA"],
    Cell[126659, 5494, 315, 7, 74, "Input",
      CellTags->"DeleteStateFA"],
    Cell[126977, 5503, 276, 5, 58, "Input",
      CellTags->"DeleteStateFA"]},
  "DeterministicQFA"->{
    Cell[127256, 5510, 70, 1, 32, "ObjectName",
      CellTags->"DeterministicQFA"],
    Cell[127329, 5513, 61, 1, 50, "Highlight",
      CellTags->"DeterministicQFA"],
    Cell[127393, 5516, 177, 6, 31, "Usage",
      CellTags->"DeterministicQFA"],
    Cell[127573, 5524, 294, 10, 61, "Notes",
      CellTags->"DeterministicQFA"],
    Cell[127870, 5536, 420, 17, 40, "Notes",
      CellTags->"DeterministicQFA"],
    Cell[128293, 5555, 187, 5, 40, "Notes",
      CellTags->"DeterministicQFA"],
    Cell[128483, 5562, 360, 12, 40, "Notes",
      CellTags->"DeterministicQFA"],
    Cell[128846, 5576, 302, 13, 24, "Notes",
      CellTags->"DeterministicQFA"],
    Cell[129151, 5591, 74, 1, 39, "ExampleSection",
      CellTags->"DeterministicQFA"],
    Cell[129228, 5594, 157, 4, 50, "ExampleText",
      CellTags->"DeterministicQFA"],
    Cell[129388, 5600, 208, 5, 58, "Input",
      CellTags->"DeterministicQFA"],
    Cell[129599, 5607, 151, 4, 34, "ExampleText",
      CellTags->"DeterministicQFA"],
    Cell[129753, 5613, 165, 4, 42, "Input",
      CellTags->"DeterministicQFA"],
    Cell[129921, 5619, 150, 4, 34, "ExampleText",
      CellTags->"DeterministicQFA"],
    Cell[130074, 5625, 337, 7, 58, "Input",
      CellTags->"DeterministicQFA"]},
  "DFA"->{
    Cell[130414, 5634, 44, 1, 32, "ObjectName",
      CellTags->"DFA"],
    Cell[130461, 5637, 51, 1, 50, "Highlight",
      CellTags->"DFA"],
    Cell[130515, 5640, 142, 4, 31, "Usage",
      CellTags->"DFA"],
    Cell[130660, 5646, 319, 12, 40, "Notes",
      CellTags->"DFA"],
    Cell[130982, 5660, 247, 6, 40, "Notes",
      CellTags->"DFA"],
    Cell[131232, 5668, 197, 9, 24, "Notes",
      CellTags->"DFA"],
    Cell[131432, 5679, 188, 8, 24, "Notes",
      CellTags->"DFA"],
    Cell[131623, 5689, 352, 9, 56, "Notes",
      CellTags->"DFA"],
    Cell[131978, 5700, 342, 16, 24, "Notes",
      CellTags->"DFA"],
    Cell[132323, 5718, 61, 1, 39, "ExampleSection",
      CellTags->"DFA"],
    Cell[132387, 5721, 402, 10, 66, "ExampleText",
      CellTags->"DFA"],
    Cell[132792, 5733, 98, 3, 26, "Input",
      CellTags->"DFA"],
    Cell[132893, 5738, 199, 5, 26, "Input",
      CellTags->"DFA"],
    Cell[133095, 5745, 117, 4, 42, "Input",
      CellTags->"DFA"],
    Cell[133215, 5751, 103, 3, 26, "Input",
      CellTags->"DFA"]},
  "DivisibilityDFA"->{
    Cell[133321, 5756, 68, 1, 32, "ObjectName",
      CellTags->"DivisibilityDFA"],
    Cell[133392, 5759, 60, 1, 50, "Highlight",
      CellTags->"DivisibilityDFA"],
    Cell[133455, 5762, 253, 8, 47, "Usage",
      CellTags->"DivisibilityDFA"],
    Cell[133711, 5772, 332, 11, 79, "Notes",
      CellTags->"DivisibilityDFA"],
    Cell[134046, 5785, 176, 5, 24, "Notes",
      CellTags->"DivisibilityDFA"],
    Cell[134225, 5792, 156, 5, 24, "Notes",
      CellTags->"DivisibilityDFA"],
    Cell[134384, 5799, 231, 8, 24, "Notes",
      CellTags->"DivisibilityDFA"],
    Cell[134618, 5809, 1094, 34, 72, "Notes",
      CellTags->"DivisibilityDFA"],
    Cell[135715, 5845, 320, 9, 40, "Notes",
      CellTags->"DivisibilityDFA"],
    Cell[136038, 5856, 242, 10, 24, "Notes",
      CellTags->"DivisibilityDFA"],
    Cell[136283, 5868, 73, 1, 39, "ExampleSection",
      CellTags->"DivisibilityDFA"],
    Cell[136359, 5871, 284, 9, 50, "ExampleText",
      CellTags->"DivisibilityDFA"],
    Cell[136646, 5882, 210, 5, 58, "Input",
      CellTags->"DivisibilityDFA"],
    Cell[136859, 5889, 107, 1, 34, "ExampleText",
      CellTags->"DivisibilityDFA"],
    Cell[136969, 5892, 126, 3, 26, "Input",
      CellTags->"DivisibilityDFA"],
    Cell[137098, 5897, 105, 3, 26, "Input",
      CellTags->"DivisibilityDFA"],
    Cell[137206, 5902, 112, 3, 26, "Input",
      CellTags->"DivisibilityDFA"],
    Cell[137321, 5907, 221, 9, 34, "ExampleText",
      CellTags->"DivisibilityDFA"],
    Cell[137545, 5918, 183, 4, 42, "Input",
      CellTags->"DivisibilityDFA"],
    Cell[137731, 5924, 169, 6, 34, "ExampleText",
      CellTags->"DivisibilityDFA"],
    Cell[137903, 5932, 180, 4, 26, "Input",
      CellTags->"DivisibilityDFA"],
    Cell[138086, 5938, 230, 7, 50, "ExampleText",
      CellTags->"DivisibilityDFA"],
    Cell[138319, 5947, 195, 4, 42, "Input",
      CellTags->"DivisibilityDFA"]},
  "EdgeLU"->{
    Cell[138517, 5953, 50, 1, 32, "ObjectName",
      CellTags->"EdgeLU"],
    Cell[138570, 5956, 49, 1, 50, "Highlight",
      CellTags->"EdgeLU"],
    Cell[138622, 5959, 147, 6, 31, "Usage",
      CellTags->"EdgeLU"],
    Cell[138772, 5967, 98, 1, 24, "Notes",
      CellTags->"EdgeLU"],
    Cell[138873, 5970, 166, 7, 24, "Notes",
      CellTags->"EdgeLU"]},
  "EmptyDFA"->{
    Cell[139042, 5979, 54, 1, 32, "ObjectName",
      CellTags->"EmptyDFA"],
    Cell[139099, 5982, 53, 1, 50, "Highlight",
      CellTags->"EmptyDFA"],
    Cell[139155, 5985, 185, 6, 31, "Usage",
      CellTags->"EmptyDFA"],
    Cell[139343, 5993, 256, 12, 24, "Notes",
      CellTags->"EmptyDFA"],
    Cell[139602, 6007, 66, 1, 39, "ExampleSection",
      CellTags->"EmptyDFA"],
    Cell[139671, 6010, 108, 2, 34, "ExampleText",
      CellTags->"EmptyDFA"],
    Cell[139782, 6014, 82, 2, 28, "Input",
      CellTags->"EmptyDFA"],
    Cell[139867, 6018, 104, 3, 26, "Input",
      CellTags->"EmptyDFA"]},
  "EmptyQFA"->{
    Cell[139974, 6023, 54, 1, 32, "ObjectName",
      CellTags->"EmptyQFA"],
    Cell[140031, 6026, 53, 1, 50, "Highlight",
      CellTags->"EmptyQFA"],
    Cell[140087, 6029, 172, 6, 31, "Usage",
      CellTags->"EmptyQFA"],
    Cell[140262, 6037, 125, 4, 24, "Notes",
      CellTags->"EmptyQFA"],
    Cell[140390, 6043, 128, 6, 24, "Notes",
      CellTags->"EmptyQFA"],
    Cell[140521, 6051, 66, 1, 39, "ExampleSection",
      CellTags->"EmptyQFA"],
    Cell[140590, 6054, 102, 1, 34, "ExampleText",
      CellTags->"EmptyQFA"],
    Cell[140695, 6057, 304, 8, 58, "Input",
      CellTags->"EmptyQFA"],
    Cell[141002, 6067, 175, 5, 58, "Input",
      CellTags->"EmptyQFA"]},
  "EpsElim"->{
    Cell[141180, 6074, 52, 1, 32, "ObjectName",
      CellTags->"EpsElim"],
    Cell[141235, 6077, 50, 1, 50, "Highlight",
      CellTags->"EpsElim"],
    Cell[141288, 6080, 194, 8, 31, "Usage",
      CellTags->"EpsElim"],
    Cell[141485, 6090, 175, 5, 24, "Notes",
      CellTags->"EpsElim"],
    Cell[141663, 6097, 289, 11, 24, "Notes",
      CellTags->"EpsElim"]},
  "EpsilonFreeQ"->{
    Cell[141955, 6110, 62, 1, 32, "ObjectName",
      CellTags->"EpsilonFreeQ"],
    Cell[142020, 6113, 57, 1, 50, "Highlight",
      CellTags->"EpsilonFreeQ"],
    Cell[142080, 6116, 140, 4, 31, "Usage",
      CellTags->"EpsilonFreeQ"],
    Cell[142223, 6122, 345, 17, 24, "Notes",
      CellTags->"EpsilonFreeQ"],
    Cell[142571, 6141, 145, 6, 24, "Notes",
      CellTags->"EpsilonFreeQ"],
    Cell[142719, 6149, 70, 1, 39, "ExampleSection",
      CellTags->"EpsilonFreeQ"],
    Cell[142792, 6152, 134, 4, 34, "ExampleText",
      CellTags->"EpsilonFreeQ"],
    Cell[142929, 6158, 117, 2, 28, "Input",
      CellTags->"EpsilonFreeQ"],
    Cell[143049, 6162, 102, 3, 26, "Input",
      CellTags->"EpsilonFreeQ"]},
  "EpsilonEliminationFA"->{
    Cell[143154, 6167, 78, 1, 32, "ObjectName",
      CellTags->"EpsilonEliminationFA"],
    Cell[143235, 6170, 65, 1, 50, "Highlight",
      CellTags->"EpsilonEliminationFA"],
    Cell[143303, 6173, 178, 4, 31, "Usage",
      CellTags->"EpsilonEliminationFA"],
    Cell[143484, 6179, 375, 11, 79, "Notes",
      CellTags->"EpsilonEliminationFA"],
    Cell[143862, 6192, 837, 29, 56, "Notes",
      CellTags->"EpsilonEliminationFA"],
    Cell[144702, 6223, 186, 4, 40, "Notes",
      CellTags->"EpsilonEliminationFA"],
    Cell[144891, 6229, 187, 4, 40, "Notes",
      CellTags->"EpsilonEliminationFA"],
    Cell[145081, 6235, 138, 6, 24, "Notes",
      CellTags->"EpsilonEliminationFA"],
    Cell[145222, 6243, 78, 1, 39, "ExampleSection",
      CellTags->"EpsilonEliminationFA"],
    Cell[145303, 6246, 160, 4, 50, "ExampleText",
      CellTags->"EpsilonEliminationFA"],
    Cell[145466, 6252, 667, 11, 122, "Input",
      CellTags->"EpsilonEliminationFA"],
    Cell[146136, 6265, 132, 3, 26, "Input",
      CellTags->"EpsilonEliminationFA"],
    Cell[146271, 6270, 115, 3, 26, "Input",
      CellTags->"EpsilonEliminationFA"],
    Cell[146389, 6275, 86, 1, 34, "ExampleText",
      CellTags->"EpsilonEliminationFA"],
    Cell[146478, 6278, 176, 4, 26, "Input",
      CellTags->"EpsilonEliminationFA"],
    Cell[146657, 6284, 97, 3, 26, "Input",
      CellTags->"EpsilonEliminationFA"],
    Cell[146757, 6289, 96, 3, 26, "Input",
      CellTags->"EpsilonEliminationFA"],
    Cell[146856, 6294, 97, 3, 26, "Input",
      CellTags->"EpsilonEliminationFA"]},
  "EquivalentQFA"->{
    Cell[146956, 6299, 64, 1, 32, "ObjectName",
      CellTags->"EquivalentQFA"],
    Cell[147023, 6302, 58, 1, 50, "Highlight",
      CellTags->"EquivalentQFA"],
    Cell[147084, 6305, 155, 4, 31, "Usage",
      CellTags->"EquivalentQFA"],
    Cell[147242, 6311, 299, 10, 61, "Notes",
      CellTags->"EquivalentQFA"],
    Cell[147544, 6323, 617, 20, 56, "Notes",
      CellTags->"EquivalentQFA"],
    Cell[148164, 6345, 369, 7, 94, "Notes",
      CellTags->"EquivalentQFA"],
    Cell[148536, 6354, 216, 5, 40, "Notes",
      CellTags->"EquivalentQFA"],
    Cell[148755, 6361, 197, 9, 24, "Notes",
      CellTags->"EquivalentQFA"],
    Cell[148955, 6372, 71, 1, 39, "ExampleSection",
      CellTags->"EquivalentQFA"],
    Cell[149029, 6375, 80, 1, 34, "ExampleText",
      CellTags->"EquivalentQFA"],
    Cell[149112, 6378, 223, 5, 42, "Input",
      CellTags->"EquivalentQFA"],
    Cell[149338, 6385, 280, 6, 58, "Input",
      CellTags->"EquivalentQFA"],
    Cell[149621, 6393, 202, 5, 42, "Input",
      CellTags->"EquivalentQFA"],
    Cell[149826, 6400, 115, 3, 26, "Input",
      CellTags->"EquivalentQFA"],
    Cell[149944, 6405, 96, 1, 34, "ExampleText",
      CellTags->"EquivalentQFA"],
    Cell[150043, 6408, 115, 3, 26, "Input",
      CellTags->"EquivalentQFA"],
    Cell[150161, 6413, 194, 5, 50, "ExampleText",
      CellTags->"EquivalentQFA"],
    Cell[150358, 6420, 165, 4, 26, "Input",
      CellTags->"EquivalentQFA"],
    Cell[150526, 6426, 121, 3, 26, "Input",
      CellTags->"EquivalentQFA"]},
  "FactorialQFA"->{
    Cell[150650, 6431, 62, 1, 32, "ObjectName",
      CellTags->"FactorialQFA"],
    Cell[150715, 6434, 57, 1, 50, "Highlight",
      CellTags->"FactorialQFA"],
    Cell[150775, 6437, 219, 6, 47, "Usage",
      CellTags->"FactorialQFA"],
    Cell[150997, 6445, 295, 10, 61, "Notes",
      CellTags->"FactorialQFA"],
    Cell[151295, 6457, 221, 6, 40, "Notes",
      CellTags->"FactorialQFA"],
    Cell[151519, 6465, 138, 6, 24, "Notes",
      CellTags->"FactorialQFA"],
    Cell[151660, 6473, 70, 1, 39, "ExampleSection",
      CellTags->"FactorialQFA"],
    Cell[151733, 6476, 81, 1, 34, "ExampleText",
      CellTags->"FactorialQFA"],
    Cell[151817, 6479, 185, 5, 42, "Input",
      CellTags->"FactorialQFA"],
    Cell[152005, 6486, 100, 3, 26, "Input",
      CellTags->"FactorialQFA"],
    Cell[152108, 6491, 86, 1, 34, "ExampleText",
      CellTags->"FactorialQFA"],
    Cell[152197, 6494, 119, 3, 26, "Input",
      CellTags->"FactorialQFA"]},
  "FA"->{
    Cell[152319, 6499, 42, 1, 32, "ObjectName",
      CellTags->"FA"],
    Cell[152364, 6502, 50, 1, 50, "Highlight",
      CellTags->"FA"],
    Cell[152417, 6505, 124, 4, 31, "Usage",
      CellTags->"FA"],
    Cell[152544, 6511, 231, 9, 24, "Notes",
      CellTags->"FA"],
    Cell[152778, 6522, 168, 7, 24, "Notes",
      CellTags->"FA"],
    Cell[152949, 6531, 261, 9, 24, "Notes",
      CellTags->"FA"],
    Cell[153213, 6542, 183, 7, 24, "Notes",
      CellTags->"FA"],
    Cell[153399, 6551, 303, 15, 24, "Notes",
      CellTags->"FA"],
    Cell[153705, 6568, 60, 1, 39, "ExampleSection",
      CellTags->"FA"],
    Cell[153768, 6571, 52, 1, 34, "ExampleText",
      CellTags->"FA"],
    Cell[153823, 6574, 207, 5, 42, "Input",
      CellTags->"FA"],
    Cell[154033, 6581, 142, 5, 58, "Input",
      CellTags->"FA"]},
  "FinalVectorFA"->{
    Cell[154178, 6588, 64, 1, 32, "ObjectName",
      CellTags->"FinalVectorFA"],
    Cell[154245, 6591, 58, 1, 50, "Highlight",
      CellTags->"FinalVectorFA"],
    Cell[154306, 6594, 159, 4, 31, "Usage",
      CellTags->"FinalVectorFA"],
    Cell[154468, 6600, 92, 1, 24, "Notes",
      CellTags->"FinalVectorFA"],
    Cell[154563, 6603, 339, 14, 24, "Notes",
      CellTags->"FinalVectorFA"],
    Cell[154905, 6619, 71, 1, 39, "ExampleSection",
      CellTags->"FinalVectorFA"],
    Cell[154979, 6622, 96, 1, 34, "ExampleText",
      CellTags->"FinalVectorFA"],
    Cell[155078, 6625, 227, 6, 74, "Input",
      CellTags->"FinalVectorFA"]},
  "Final"->{
    Cell[155308, 6633, 48, 1, 32, "ObjectName",
      CellTags->"Final"],
    Cell[155359, 6636, 50, 1, 50, "Highlight",
      CellTags->"Final"],
    Cell[155412, 6639, 198, 6, 31, "Usage",
      CellTags->"Final"],
    Cell[155613, 6647, 257, 11, 24, "Notes",
      CellTags->"Final"],
    Cell[155873, 6660, 63, 1, 39, "ExampleSection",
      CellTags->"Final"],
    Cell[155939, 6663, 141, 4, 34, "ExampleText",
      CellTags->"Final"],
    Cell[156083, 6669, 151, 4, 26, "Input",
      CellTags->"Final"],
    Cell[156237, 6675, 88, 3, 26, "Input",
      CellTags->"Final"]},
  "FiniteComplementSA"->{
    Cell[156328, 6680, 74, 1, 32, "ObjectName",
      CellTags->"FiniteComplementSA"],
    Cell[156405, 6683, 63, 1, 50, "Highlight",
      CellTags->"FiniteComplementSA"],
    Cell[156471, 6686, 281, 7, 47, "Usage",
      CellTags->"FiniteComplementSA"],
    Cell[156755, 6695, 242, 9, 44, "Notes",
      CellTags->"FiniteComplementSA"],
    Cell[157000, 6706, 178, 6, 24, "Notes",
      CellTags->"FiniteComplementSA"],
    Cell[157181, 6714, 117, 4, 24, "Notes",
      CellTags->"FiniteComplementSA"],
    Cell[157301, 6720, 143, 6, 24, "Notes",
      CellTags->"FiniteComplementSA"],
    Cell[157447, 6728, 76, 1, 39, "ExampleSection",
      CellTags->"FiniteComplementSA"],
    Cell[157526, 6731, 340, 13, 34, "ExampleText",
      CellTags->"FiniteComplementSA"],
    Cell[157869, 6746, 128, 3, 26, "Input",
      CellTags->"FiniteComplementSA"],
    Cell[158000, 6751, 114, 3, 26, "Input",
      CellTags->"FiniteComplementSA"],
    Cell[158117, 6756, 108, 3, 26, "Input",
      CellTags->"FiniteComplementSA"]},
  "FinLU"->{
    Cell[158228, 6761, 48, 1, 32, "ObjectName",
      CellTags->"FinLU"],
    Cell[158279, 6764, 48, 1, 50, "Highlight",
      CellTags->"FinLU"],
    Cell[158330, 6767, 145, 6, 31, "Usage",
      CellTags->"FinLU"],
    Cell[158478, 6775, 76, 1, 24, "Notes",
      CellTags->"FinLU"],
    Cell[158557, 6778, 165, 7, 24, "Notes",
      CellTags->"FinLU"]},
  "Fixed"->{
    Cell[158725, 6787, 48, 1, 32, "ObjectName",
      CellTags->"Fixed"],
    Cell[158776, 6790, 48, 1, 50, "Highlight",
      CellTags->"Fixed"],
    Cell[158827, 6793, 137, 6, 31, "Usage",
      CellTags->"Fixed"],
    Cell[158967, 6801, 82, 1, 24, "Notes",
      CellTags->"Fixed"],
    Cell[159052, 6804, 149, 7, 24, "Notes",
      CellTags->"Fixed"]},
  "BoundLFA"->{
    Cell[159204, 6813, 54, 1, 32, "ObjectName",
      CellTags->"BoundLFA"],
    Cell[159261, 6816, 51, 1, 50, "Highlight",
      CellTags->"BoundLFA"],
    Cell[159315, 6819, 145, 6, 31, "Usage",
      CellTags->"BoundLFA"],
    Cell[159463, 6827, 98, 1, 24, "Notes",
      CellTags->"BoundLFA"],
    Cell[159564, 6830, 119, 5, 24, "Notes",
      CellTags->"BoundLFA"],
    Cell[159686, 6837, 156, 7, 24, "Notes",
      CellTags->"BoundLFA"]},
  "FrontierToDFA"->{
    Cell[159845, 6846, 64, 1, 32, "ObjectName",
      CellTags->"FrontierToDFA"],
    Cell[159912, 6849, 58, 1, 50, "Highlight",
      CellTags->"FrontierToDFA"],
    Cell[159973, 6852, 229, 8, 31, "Usage",
      CellTags->"FrontierToDFA"],
    Cell[160205, 6862, 218, 8, 24, "Notes",
      CellTags->"FrontierToDFA"],
    Cell[160426, 6872, 273, 11, 24, "Notes",
      CellTags->"FrontierToDFA"],
    Cell[160702, 6885, 71, 1, 39, "ExampleSection",
      CellTags->"FrontierToDFA"],
    Cell[160776, 6888, 151, 4, 34, "ExampleText",
      CellTags->"FrontierToDFA"],
    Cell[160930, 6894, 288, 6, 42, "Input",
      CellTags->"FrontierToDFA"],
    Cell[161221, 6902, 128, 4, 34, "ExampleText",
      CellTags->"FrontierToDFA"],
    Cell[161352, 6908, 118, 3, 26, "Input",
      CellTags->"FrontierToDFA"],
    Cell[161473, 6913, 161, 4, 26, "Input",
      CellTags->"FrontierToDFA"],
    Cell[161637, 6919, 172, 4, 50, "ExampleText",
      CellTags->"FrontierToDFA"],
    Cell[161812, 6925, 96, 3, 26, "Input",
      CellTags->"FrontierToDFA"],
    Cell[161911, 6930, 90, 1, 34, "ExampleText",
      CellTags->"FrontierToDFA"],
    Cell[162004, 6933, 114, 3, 26, "Input",
      CellTags->"FrontierToDFA"],
    Cell[162121, 6938, 102, 1, 34, "ExampleText",
      CellTags->"FrontierToDFA"],
    Cell[162226, 6941, 142, 3, 26, "Input",
      CellTags->"FrontierToDFA"]},
  "FSMQ"->{
    Cell[162371, 6946, 46, 1, 32, "ObjectName",
      CellTags->"FSMQ"],
    Cell[162420, 6949, 49, 1, 50, "Highlight",
      CellTags->"FSMQ"],
    Cell[162472, 6952, 154, 6, 31, "Usage",
      CellTags->"FSMQ"],
    Cell[162629, 6960, 147, 5, 24, "Notes",
      CellTags->"FSMQ"],
    Cell[162779, 6967, 62, 1, 39, "ExampleSection",
      CellTags->"FSMQ"],
    Cell[162844, 6970, 91, 1, 34, "ExampleText",
      CellTags->"FSMQ"],
    Cell[162938, 6973, 95, 2, 28, "Input",
      CellTags->"FSMQ"],
    Cell[163036, 6977, 116, 3, 26, "Input",
      CellTags->"FSMQ"],
    Cell[163155, 6982, 94, 3, 26, "Input",
      CellTags->"FSMQ"]},
  "FullQFA"->{
    Cell[163252, 6987, 52, 1, 32, "ObjectName",
      CellTags->"FullQFA"],
    Cell[163307, 6990, 52, 1, 50, "Highlight",
      CellTags->"FullQFA"],
    Cell[163362, 6993, 171, 6, 31, "Usage",
      CellTags->"FullQFA"],
    Cell[163536, 7001, 188, 5, 40, "Notes",
      CellTags->"FullQFA"],
    Cell[163727, 7008, 257, 12, 24, "Notes",
      CellTags->"FullQFA"],
    Cell[163987, 7022, 65, 1, 39, "ExampleSection",
      CellTags->"FullQFA"],
    Cell[164055, 7025, 194, 6, 34, "ExampleText",
      CellTags->"FullQFA"],
    Cell[164252, 7033, 216, 4, 42, "Input",
      CellTags->"FullQFA"],
    Cell[164471, 7039, 157, 4, 42, "Input",
      CellTags->"FullQFA"]},
  "FullTransitionMatrixFA"->{
    Cell[164631, 7045, 82, 1, 32, "ObjectName",
      CellTags->"FullTransitionMatrixFA"],
    Cell[164716, 7048, 67, 1, 50, "Highlight",
      CellTags->"FullTransitionMatrixFA"],
    Cell[164786, 7051, 231, 5, 47, "Usage",
      CellTags->"FullTransitionMatrixFA"],
    Cell[165020, 7058, 294, 9, 61, "Notes",
      CellTags->"FullTransitionMatrixFA"],
    Cell[165317, 7069, 164, 5, 24, "Notes",
      CellTags->"FullTransitionMatrixFA"],
    Cell[165484, 7076, 248, 8, 24, "Notes",
      CellTags->"FullTransitionMatrixFA"],
    Cell[165735, 7086, 153, 6, 24, "Notes",
      CellTags->"FullTransitionMatrixFA"],
    Cell[165891, 7094, 80, 1, 39, "ExampleSection",
      CellTags->"FullTransitionMatrixFA"],
    Cell[165974, 7097, 115, 1, 34, "ExampleText",
      CellTags->"FullTransitionMatrixFA"],
    Cell[166092, 7100, 108, 2, 28, "Input",
      CellTags->"FullTransitionMatrixFA"],
    Cell[166203, 7104, 173, 4, 42, "Input",
      CellTags->"FullTransitionMatrixFA"],
    Cell[166379, 7110, 141, 4, 34, "ExampleText",
      CellTags->"FullTransitionMatrixFA"],
    Cell[166523, 7116, 130, 3, 26, "Input",
      CellTags->"FullTransitionMatrixFA"]},
  "FullTransitions"->{
    Cell[166656, 7121, 68, 1, 32, "ObjectName",
      CellTags->"FullTransitions"],
    Cell[166727, 7124, 60, 1, 50, "Highlight",
      CellTags->"FullTransitions"],
    Cell[166790, 7127, 192, 6, 31, "Usage",
      CellTags->"FullTransitions"],
    Cell[166985, 7135, 383, 12, 40, "Notes",
      CellTags->"FullTransitions"],
    Cell[167371, 7149, 293, 11, 24, "Notes",
      CellTags->"FullTransitions"],
    Cell[167667, 7162, 73, 1, 39, "ExampleSection",
      CellTags->"FullTransitions"],
    Cell[167743, 7165, 107, 1, 34, "ExampleText",
      CellTags->"FullTransitions"],
    Cell[167853, 7168, 101, 2, 28, "Input",
      CellTags->"FullTransitions"],
    Cell[167957, 7172, 125, 3, 26, "Input",
      CellTags->"FullTransitions"]},
  "FunctionToDFA"->{
    Cell[168085, 7177, 64, 1, 32, "ObjectName",
      CellTags->"FunctionToDFA"],
    Cell[168152, 7180, 58, 1, 50, "Highlight",
      CellTags->"FunctionToDFA"],
    Cell[168213, 7183, 309, 14, 31, "Usage",
      CellTags->"FunctionToDFA"],
    Cell[168525, 7199, 303, 10, 61, "Notes",
      CellTags->"FunctionToDFA"],
    Cell[168831, 7211, 244, 7, 24, "Notes",
      CellTags->"FunctionToDFA"],
    Cell[169078, 7220, 481, 14, 56, "Notes",
      CellTags->"FunctionToDFA"],
    Cell[169562, 7236, 315, 10, 40, "Notes",
      CellTags->"FunctionToDFA"],
    Cell[169880, 7248, 268, 11, 24, "Notes",
      CellTags->"FunctionToDFA"],
    Cell[170151, 7261, 71, 1, 39, "ExampleSection",
      CellTags->"FunctionToDFA"],
    Cell[170225, 7264, 105, 1, 34, "ExampleText",
      CellTags->"FunctionToDFA"],
    Cell[170333, 7267, 122, 3, 26, "Input",
      CellTags->"FunctionToDFA"],
    Cell[170458, 7272, 264, 7, 42, "Input",
      CellTags->"FunctionToDFA"],
    Cell[170725, 7281, 312, 10, 50, "ExampleText",
      CellTags->"FunctionToDFA"],
    Cell[171040, 7293, 115, 3, 26, "Input",
      CellTags->"FunctionToDFA"]},
  "FunctionToSA"->{
    Cell[171158, 7298, 62, 1, 32, "ObjectName",
      CellTags->"FunctionToSA"],
    Cell[171223, 7301, 57, 1, 50, "Highlight",
      CellTags->"FunctionToSA"],
    Cell[171283, 7304, 248, 10, 31, "Usage",
      CellTags->"FunctionToSA"],
    Cell[171534, 7316, 302, 10, 61, "Notes",
      CellTags->"FunctionToSA"],
    Cell[171839, 7328, 254, 7, 24, "Notes",
      CellTags->"FunctionToSA"],
    Cell[172096, 7337, 316, 11, 40, "Notes",
      CellTags->"FunctionToSA"],
    Cell[172415, 7350, 314, 10, 40, "Notes",
      CellTags->"FunctionToSA"],
    Cell[172732, 7362, 307, 11, 24, "Notes",
      CellTags->"FunctionToSA"],
    Cell[173042, 7375, 70, 1, 39, "ExampleSection",
      CellTags->"FunctionToSA"],
    Cell[173115, 7378, 142, 4, 34, "ExampleText",
      CellTags->"FunctionToSA"],
    Cell[173260, 7384, 251, 6, 74, "Input",
      CellTags->"FunctionToSA"],
    Cell[173514, 7392, 170, 5, 26, "Input",
      CellTags->"FunctionToSA"],
    Cell[173687, 7399, 83, 1, 34, "ExampleText",
      CellTags->"FunctionToSA"],
    Cell[173773, 7402, 120, 3, 26, "Input",
      CellTags->"FunctionToSA"],
    Cell[173896, 7407, 132, 3, 26, "Input",
      CellTags->"FunctionToSA"]},
  "GenerateDFA"->{
    Cell[174031, 7412, 60, 1, 32, "ObjectName",
      CellTags->"GenerateDFA"],
    Cell[174094, 7415, 56, 1, 50, "Highlight",
      CellTags->"GenerateDFA"],
    Cell[174153, 7418, 277, 8, 47, "Usage",
      CellTags->"GenerateDFA"],
    Cell[174433, 7428, 296, 10, 61, "Notes",
      CellTags->"GenerateDFA"],
    Cell[174732, 7440, 534, 19, 56, "Notes",
      CellTags->"GenerateDFA"],
    Cell[175269, 7461, 300, 9, 40, "Notes",
      CellTags->"GenerateDFA"],
    Cell[175572, 7472, 250, 9, 24, "Notes",
      CellTags->"GenerateDFA"],
    Cell[175825, 7483, 274, 11, 24, "Notes",
      CellTags->"GenerateDFA"],
    Cell[176102, 7496, 69, 1, 39, "ExampleSection",
      CellTags->"GenerateDFA"],
    Cell[176174, 7499, 229, 7, 34, "ExampleText",
      CellTags->"GenerateDFA"],
    Cell[176406, 7508, 202, 4, 42, "Input",
      CellTags->"GenerateDFA"],
    Cell[176611, 7514, 203, 6, 26, "Input",
      CellTags->"GenerateDFA"],
    Cell[176817, 7522, 184, 5, 26, "Input",
      CellTags->"GenerateDFA"],
    Cell[177004, 7529, 254, 7, 34, "ExampleText",
      CellTags->"GenerateDFA"],
    Cell[177261, 7538, 124, 3, 28, "Input",
      CellTags->"GenerateDFA"],
    Cell[177388, 7543, 157, 4, 26, "Input",
      CellTags->"GenerateDFA"]},
  "GenerateQuotients"->{
    Cell[177548, 7549, 72, 1, 32, "ObjectName",
      CellTags->"GenerateQuotients"],
    Cell[177623, 7552, 62, 1, 50, "Highlight",
      CellTags->"GenerateQuotients"],
    Cell[177688, 7555, 232, 8, 31, "Usage",
      CellTags->"GenerateQuotients"],
    Cell[177923, 7565, 173, 5, 24, "Notes",
      CellTags->"GenerateQuotients"],
    Cell[178099, 7572, 476, 19, 24, "Notes",
      CellTags->"GenerateQuotients"],
    Cell[178578, 7593, 75, 1, 39, "ExampleSection",
      CellTags->"GenerateQuotients"],
    Cell[178656, 7596, 146, 4, 34, "ExampleText",
      CellTags->"GenerateQuotients"],
    Cell[178805, 7602, 94, 2, 28, "Input",
      CellTags->"GenerateQuotients"],
    Cell[178902, 7606, 115, 3, 26, "Input",
      CellTags->"GenerateQuotients"],
    Cell[179020, 7611, 298, 5, 58, "Input",
      CellTags->"GenerateQuotients"]},
  "GrowthSeriesDFA"->{
    Cell[179321, 7618, 68, 1, 32, "ObjectName",
      CellTags->"GrowthSeriesDFA"],
    Cell[179392, 7621, 60, 1, 50, "Highlight",
      CellTags->"GrowthSeriesDFA"],
    Cell[179455, 7624, 281, 8, 47, "Usage",
      CellTags->"GrowthSeriesDFA"],
    Cell[179739, 7634, 517, 18, 61, "Notes",
      CellTags->"GrowthSeriesDFA"],
    Cell[180259, 7654, 203, 6, 40, "Notes",
      CellTags->"GrowthSeriesDFA"],
    Cell[180465, 7662, 208, 6, 40, "Notes",
      CellTags->"GrowthSeriesDFA"],
    Cell[180676, 7670, 161, 4, 40, "Notes",
      CellTags->"GrowthSeriesDFA"],
    Cell[180840, 7676, 124, 4, 24, "Notes",
      CellTags->"GrowthSeriesDFA"],
    Cell[180967, 7682, 271, 11, 24, "Notes",
      CellTags->"GrowthSeriesDFA"],
    Cell[181241, 7695, 73, 1, 39, "ExampleSection",
      CellTags->"GrowthSeriesDFA"],
    Cell[181317, 7698, 155, 4, 50, "ExampleText",
      CellTags->"GrowthSeriesDFA"],
    Cell[181475, 7704, 203, 5, 42, "Input",
      CellTags->"GrowthSeriesDFA"],
    Cell[181681, 7711, 147, 4, 42, "Input",
      CellTags->"GrowthSeriesDFA"],
    Cell[181831, 7717, 117, 3, 26, "Input",
      CellTags->"GrowthSeriesDFA"],
    Cell[181951, 7722, 114, 2, 34, "ExampleText",
      CellTags->"GrowthSeriesDFA"],
    Cell[182068, 7726, 134, 3, 26, "Input",
      CellTags->"GrowthSeriesDFA"],
    Cell[182205, 7731, 110, 1, 34, "ExampleText",
      CellTags->"GrowthSeriesDFA"],
    Cell[182318, 7734, 169, 4, 26, "Input",
      CellTags->"GrowthSeriesDFA"]},
  "HomomorphismFA"->{
    Cell[182490, 7740, 66, 1, 32, "ObjectName",
      CellTags->"HomomorphismFA"],
    Cell[182559, 7743, 59, 1, 50, "Highlight",
      CellTags->"HomomorphismFA"],
    Cell[182621, 7746, 257, 8, 47, "Usage",
      CellTags->"HomomorphismFA"],
    Cell[182881, 7756, 164, 5, 24, "Notes",
      CellTags->"HomomorphismFA"],
    Cell[183048, 7763, 189, 7, 24, "Notes",
      CellTags->"HomomorphismFA"],
    Cell[183240, 7772, 245, 10, 24, "Notes",
      CellTags->"HomomorphismFA"],
    Cell[183488, 7784, 72, 1, 39, "ExampleSection",
      CellTags->"HomomorphismFA"],
    Cell[183563, 7787, 303, 12, 34, "ExampleText",
      CellTags->"HomomorphismFA"],
    Cell[183869, 7801, 222, 5, 42, "Input",
      CellTags->"HomomorphismFA"],
    Cell[184094, 7808, 110, 3, 26, "Input",
      CellTags->"HomomorphismFA"]},
  "Hopcroft"->{
    Cell[184207, 7813, 54, 1, 32, "ObjectName",
      CellTags->"Hopcroft"],
    Cell[184264, 7816, 51, 1, 50, "Highlight",
      CellTags->"Hopcroft"],
    Cell[184318, 7819, 148, 6, 31, "Usage",
      CellTags->"Hopcroft"],
    Cell[184469, 7827, 162, 6, 24, "Notes",
      CellTags->"Hopcroft"],
    Cell[184634, 7835, 145, 4, 24, "Notes",
      CellTags->"Hopcroft"],
    Cell[184782, 7841, 135, 6, 24, "Notes",
      CellTags->"Hopcroft"]},
  "HurwitzProduct"->{
    Cell[184920, 7849, 66, 1, 32, "ObjectName",
      CellTags->"HurwitzProduct"],
    Cell[184989, 7852, 59, 1, 50, "Highlight",
      CellTags->"HurwitzProduct"],
    Cell[185051, 7855, 164, 4, 31, "Usage",
      CellTags->"HurwitzProduct"],
    Cell[185218, 7861, 84, 1, 24, "Notes",
      CellTags->"HurwitzProduct"],
    Cell[185305, 7864, 418, 15, 24, "Notes",
      CellTags->"HurwitzProduct"],
    Cell[185726, 7881, 72, 1, 39, "ExampleSection",
      CellTags->"HurwitzProduct"],
    Cell[185801, 7884, 114, 2, 34, "ExampleText",
      CellTags->"HurwitzProduct"],
    Cell[185918, 7888, 142, 6, 45, "Input",
      CellTags->"HurwitzProduct"],
    Cell[186063, 7896, 160, 6, 45, "Input",
      CellTags->"HurwitzProduct"],
    Cell[186226, 7904, 91, 3, 26, "Input",
      CellTags->"HurwitzProduct"],
    Cell[186320, 7909, 112, 3, 28, "Input",
      CellTags->"HurwitzProduct"],
    Cell[186435, 7914, 387, 7, 58, "Input",
      CellTags->"HurwitzProduct"],
    Cell[186825, 7923, 196, 6, 50, "ExampleText",
      CellTags->"HurwitzProduct"],
    Cell[187024, 7931, 229, 5, 42, "Input",
      CellTags->"HurwitzProduct"]},
  "IdealBaseFA"->{
    Cell[187256, 7938, 60, 1, 32, "ObjectName",
      CellTags->"IdealBaseFA"],
    Cell[187319, 7941, 56, 1, 50, "Highlight",
      CellTags->"IdealBaseFA"],
    Cell[187378, 7944, 183, 6, 31, "Usage",
      CellTags->"IdealBaseFA"],
    Cell[187564, 7952, 254, 9, 24, "Notes",
      CellTags->"IdealBaseFA"],
    Cell[187821, 7963, 238, 8, 24, "Notes",
      CellTags->"IdealBaseFA"],
    Cell[188062, 7973, 258, 11, 24, "Notes",
      CellTags->"IdealBaseFA"],
    Cell[188323, 7986, 69, 1, 39, "ExampleSection",
      CellTags->"IdealBaseFA"],
    Cell[188395, 7989, 98, 1, 34, "ExampleText",
      CellTags->"IdealBaseFA"],
    Cell[188496, 7992, 158, 4, 42, "Input",
      CellTags->"IdealBaseFA"],
    Cell[188657, 7998, 375, 12, 50, "ExampleText",
      CellTags->"IdealBaseFA"],
    Cell[189035, 8012, 154, 4, 42, "Input",
      CellTags->"IdealBaseFA"],
    Cell[189192, 8018, 93, 1, 34, "ExampleText",
      CellTags->"IdealBaseFA"],
    Cell[189288, 8021, 250, 6, 58, "Input",
      CellTags->"IdealBaseFA"],
    Cell[189541, 8029, 277, 8, 50, "ExampleText",
      CellTags->"IdealBaseFA"],
    Cell[189821, 8039, 287, 6, 58, "Input",
      CellTags->"IdealBaseFA"],
    Cell[190111, 8047, 119, 3, 26, "Input",
      CellTags->"IdealBaseFA"]},
  "IdealQFA"->{
    Cell[190233, 8052, 54, 1, 32, "ObjectName",
      CellTags->"IdealQFA"],
    Cell[190290, 8055, 53, 1, 50, "Highlight",
      CellTags->"IdealQFA"],
    Cell[190346, 8058, 168, 6, 31, "Usage",
      CellTags->"IdealQFA"],
    Cell[190517, 8066, 147, 5, 24, "Notes",
      CellTags->"IdealQFA"],
    Cell[190667, 8073, 163, 7, 24, "Notes",
      CellTags->"IdealQFA"],
    Cell[190833, 8082, 66, 1, 39, "ExampleSection",
      CellTags->"IdealQFA"],
    Cell[190902, 8085, 95, 1, 34, "ExampleText",
      CellTags->"IdealQFA"],
    Cell[191000, 8088, 124, 3, 26, "Input",
      CellTags->"IdealQFA"],
    Cell[191127, 8093, 93, 3, 26, "Input",
      CellTags->"IdealQFA"]},
  "Implementation"->{
    Cell[191223, 8098, 66, 1, 32, "ObjectName",
      CellTags->"Implementation"],
    Cell[191292, 8101, 57, 1, 50, "Highlight",
      CellTags->"Implementation"],
    Cell[191352, 8104, 172, 6, 31, "Usage",
      CellTags->"Implementation"],
    Cell[191527, 8112, 91, 1, 24, "Notes",
      CellTags->"Implementation"],
    Cell[191621, 8115, 162, 7, 24, "Notes",
      CellTags->"Implementation"]},
  "InfiniteQFA"->{
    Cell[191786, 8124, 60, 1, 32, "ObjectName",
      CellTags->"InfiniteQFA"],
    Cell[191849, 8127, 56, 1, 50, "Highlight",
      CellTags->"InfiniteQFA"],
    Cell[191908, 8130, 181, 6, 31, "Usage",
      CellTags->"InfiniteQFA"],
    Cell[192092, 8138, 130, 5, 24, "Notes",
      CellTags->"InfiniteQFA"],
    Cell[192225, 8145, 157, 7, 24, "Notes",
      CellTags->"InfiniteQFA"],
    Cell[192385, 8154, 69, 1, 39, "ExampleSection",
      CellTags->"InfiniteQFA"],
    Cell[192457, 8157, 83, 1, 34, "ExampleText",
      CellTags->"InfiniteQFA"],
    Cell[192543, 8160, 100, 2, 28, "Input",
      CellTags->"InfiniteQFA"],
    Cell[192646, 8164, 100, 3, 26, "Input",
      CellTags->"InfiniteQFA"],
    Cell[192749, 8169, 100, 1, 34, "ExampleText",
      CellTags->"InfiniteQFA"],
    Cell[192852, 8172, 84, 2, 28, "Input",
      CellTags->"InfiniteQFA"],
    Cell[192939, 8176, 101, 3, 26, "Input",
      CellTags->"InfiniteQFA"]},
  "IniLU"->{
    Cell[193043, 8181, 48, 1, 32, "ObjectName",
      CellTags->"IniLU"],
    Cell[193094, 8184, 48, 1, 50, "Highlight",
      CellTags->"IniLU"],
    Cell[193145, 8187, 145, 6, 31, "Usage",
      CellTags->"IniLU"],
    Cell[193293, 8195, 79, 1, 24, "Notes",
      CellTags->"IniLU"],
    Cell[193375, 8198, 165, 7, 24, "Notes",
      CellTags->"IniLU"]},
  "InitialVectorFA"->{
    Cell[193543, 8207, 68, 1, 32, "ObjectName",
      CellTags->"InitialVectorFA"],
    Cell[193614, 8210, 60, 1, 50, "Highlight",
      CellTags->"InitialVectorFA"],
    Cell[193677, 8213, 167, 4, 31, "Usage",
      CellTags->"InitialVectorFA"],
    Cell[193847, 8219, 94, 1, 24, "Notes",
      CellTags->"InitialVectorFA"],
    Cell[193944, 8222, 267, 11, 24, "Notes",
      CellTags->"InitialVectorFA"],
    Cell[194214, 8235, 73, 1, 39, "ExampleSection",
      CellTags->"InitialVectorFA"],
    Cell[194290, 8238, 151, 4, 34, "ExampleText",
      CellTags->"InitialVectorFA"],
    Cell[194444, 8244, 161, 4, 26, "Input",
      CellTags->"InitialVectorFA"],
    Cell[194608, 8250, 108, 3, 26, "Input",
      CellTags->"InitialVectorFA"]},
  "Initial"->{
    Cell[194719, 8255, 52, 1, 32, "ObjectName",
      CellTags->"Initial"],
    Cell[194774, 8258, 52, 1, 50, "Highlight",
      CellTags->"Initial"],
    Cell[194829, 8261, 178, 5, 31, "Usage",
      CellTags->"Initial"],
    Cell[195010, 8268, 259, 11, 24, "Notes",
      CellTags->"Initial"],
    Cell[195272, 8281, 65, 1, 39, "ExampleSection",
      CellTags->"Initial"],
    Cell[195340, 8284, 143, 4, 34, "ExampleText",
      CellTags->"Initial"],
    Cell[195486, 8290, 153, 4, 26, "Input",
      CellTags->"Initial"],
    Cell[195642, 8296, 92, 3, 26, "Input",
      CellTags->"Initial"]},
  "IntersectionFA"->{
    Cell[195737, 8301, 66, 1, 32, "ObjectName",
      CellTags->"IntersectionFA"],
    Cell[195806, 8304, 59, 1, 50, "Highlight",
      CellTags->"IntersectionFA"],
    Cell[195868, 8307, 271, 9, 47, "Usage",
      CellTags->"IntersectionFA"],
    Cell[196142, 8318, 299, 10, 61, "Notes",
      CellTags->"IntersectionFA"],
    Cell[196444, 8330, 141, 5, 24, "Notes",
      CellTags->"IntersectionFA"],
    Cell[196588, 8337, 203, 8, 24, "Notes",
      CellTags->"IntersectionFA"],
    Cell[196794, 8347, 83, 1, 24, "Notes",
      CellTags->"IntersectionFA"],
    Cell[196880, 8350, 59, 1, 24, "Notes",
      CellTags->"IntersectionFA"],
    Cell[196942, 8353, 341, 15, 24, "Notes",
      CellTags->"IntersectionFA"],
    Cell[197286, 8370, 72, 1, 39, "ExampleSection",
      CellTags->"IntersectionFA"],
    Cell[197361, 8373, 205, 5, 50, "ExampleText",
      CellTags->"IntersectionFA"],
    Cell[197569, 8380, 194, 4, 42, "Input",
      CellTags->"IntersectionFA"],
    Cell[197766, 8386, 133, 4, 34, "ExampleText",
      CellTags->"IntersectionFA"],
    Cell[197902, 8392, 183, 4, 42, "Input",
      CellTags->"IntersectionFA"]},
  "IrreducibleFA"->{
    Cell[198088, 8398, 64, 1, 32, "ObjectName",
      CellTags->"IrreducibleFA"],
    Cell[198155, 8401, 58, 1, 50, "Highlight",
      CellTags->"IrreducibleFA"],
    Cell[198216, 8404, 267, 9, 47, "Usage",
      CellTags->"IrreducibleFA"],
    Cell[198486, 8415, 298, 10, 61, "Notes",
      CellTags->"IrreducibleFA"],
    Cell[198787, 8427, 401, 15, 40, "Notes",
      CellTags->"IrreducibleFA"],
    Cell[199191, 8444, 146, 5, 24, "Notes",
      CellTags->"IrreducibleFA"],
    Cell[199340, 8451, 167, 7, 24, "Notes",
      CellTags->"IrreducibleFA"],
    Cell[199510, 8460, 71, 1, 39, "ExampleSection",
      CellTags->"IrreducibleFA"],
    Cell[199584, 8463, 221, 9, 34, "ExampleText",
      CellTags->"IrreducibleFA"],
    Cell[199808, 8474, 82, 2, 28, "Input",
      CellTags->"IrreducibleFA"],
    Cell[199893, 8478, 114, 3, 26, "Input",
      CellTags->"IrreducibleFA"],
    Cell[200010, 8483, 114, 3, 26, "Input",
      CellTags->"IrreducibleFA"],
    Cell[200127, 8488, 96, 1, 34, "ExampleText",
      CellTags->"IrreducibleFA"],
    Cell[200226, 8491, 142, 3, 26, "Input",
      CellTags->"IrreducibleFA"]},
  "IsomorphicQDFA"->{
    Cell[200371, 8496, 66, 1, 32, "ObjectName",
      CellTags->"IsomorphicQDFA"],
    Cell[200440, 8499, 59, 1, 50, "Highlight",
      CellTags->"IsomorphicQDFA"],
    Cell[200502, 8502, 157, 4, 31, "Usage",
      CellTags->"IsomorphicQDFA"],
    Cell[200662, 8508, 300, 10, 61, "Notes",
      CellTags->"IsomorphicQDFA"],
    Cell[200965, 8520, 150, 4, 24, "Notes",
      CellTags->"IsomorphicQDFA"],
    Cell[201118, 8526, 167, 5, 24, "Notes",
      CellTags->"IsomorphicQDFA"],
    Cell[201288, 8533, 163, 4, 40, "Notes",
      CellTags->"IsomorphicQDFA"],
    Cell[201454, 8539, 72, 1, 39, "ExampleSection",
      CellTags->"IsomorphicQDFA"],
    Cell[201529, 8542, 77, 1, 34, "ExampleText",
      CellTags->"IsomorphicQDFA"],
    Cell[201609, 8545, 335, 8, 74, "Input",
      CellTags->"IsomorphicQDFA"],
    Cell[201947, 8555, 110, 3, 26, "Input",
      CellTags->"IsomorphicQDFA"],
    Cell[202060, 8560, 132, 3, 26, "Input",
      CellTags->"IsomorphicQDFA"]},
  "IthSymbolFA"->{
    Cell[202195, 8565, 60, 1, 32, "ObjectName",
      CellTags->"IthSymbolFA"],
    Cell[202258, 8568, 56, 1, 50, "Highlight",
      CellTags->"IthSymbolFA"],
    Cell[202317, 8571, 226, 7, 47, "Usage",
      CellTags->"IthSymbolFA"],
    Cell[202546, 8580, 296, 10, 61, "Notes",
      CellTags->"IthSymbolFA"],
    Cell[202845, 8592, 212, 6, 40, "Notes",
      CellTags->"IthSymbolFA"],
    Cell[203060, 8600, 312, 12, 24, "Notes",
      CellTags->"IthSymbolFA"],
    Cell[203375, 8614, 166, 7, 24, "Notes",
      CellTags->"IthSymbolFA"],
    Cell[203544, 8623, 69, 1, 39, "ExampleSection",
      CellTags->"IthSymbolFA"],
    Cell[203616, 8626, 206, 6, 34, "ExampleText",
      CellTags->"IthSymbolFA"],
    Cell[203825, 8634, 113, 3, 26, "Input",
      CellTags->"IthSymbolFA"],
    Cell[203941, 8639, 111, 3, 26, "Input",
      CellTags->"IthSymbolFA"],
    Cell[204055, 8644, 129, 4, 34, "ExampleText",
      CellTags->"IthSymbolFA"],
    Cell[204187, 8650, 141, 3, 26, "Input",
      CellTags->"IthSymbolFA"],
    Cell[204331, 8655, 189, 6, 34, "ExampleText",
      CellTags->"IthSymbolFA"],
    Cell[204523, 8663, 174, 5, 26, "Input",
      CellTags->"IthSymbolFA"],
    Cell[204700, 8670, 112, 3, 26, "Input",
      CellTags->"IthSymbolFA"],
    Cell[204815, 8675, 165, 4, 50, "ExampleText",
      CellTags->"IthSymbolFA"],
    Cell[204983, 8681, 193, 5, 58, "Input",
      CellTags->"IthSymbolFA"]},
  "JoinSA"->{
    Cell[205179, 8688, 50, 1, 32, "ObjectName",
      CellTags->"JoinSA"],
    Cell[205232, 8691, 51, 1, 50, "Highlight",
      CellTags->"JoinSA"],
    Cell[205286, 8694, 190, 6, 31, "Usage",
      CellTags->"JoinSA"],
    Cell[205479, 8702, 246, 9, 44, "Notes",
      CellTags->"JoinSA"],
    Cell[205728, 8713, 158, 5, 24, "Notes",
      CellTags->"JoinSA"],
    Cell[205889, 8720, 149, 7, 24, "Notes",
      CellTags->"JoinSA"],
    Cell[206041, 8729, 64, 1, 39, "ExampleSection",
      CellTags->"JoinSA"],
    Cell[206108, 8732, 169, 4, 50, "ExampleText",
      CellTags->"JoinSA"],
    Cell[206280, 8738, 271, 7, 42, "Input",
      CellTags->"JoinSA"],
    Cell[206554, 8747, 92, 1, 34, "ExampleText",
      CellTags->"JoinSA"],
    Cell[206649, 8750, 118, 3, 26, "Input",
      CellTags->"JoinSA"],
    Cell[206770, 8755, 85, 1, 34, "ExampleText",
      CellTags->"JoinSA"],
    Cell[206858, 8758, 130, 3, 26, "Input",
      CellTags->"JoinSA"],
    Cell[206991, 8763, 229, 7, 50, "ExampleText",
      CellTags->"JoinSA"],
    Cell[207223, 8772, 226, 5, 58, "Input",
      CellTags->"JoinSA"]},
  "LanguageFA"->{
    Cell[207452, 8779, 58, 1, 32, "ObjectName",
      CellTags->"LanguageFA"],
    Cell[207513, 8782, 55, 1, 50, "Highlight",
      CellTags->"LanguageFA"],
    Cell[207571, 8785, 207, 8, 31, "Usage",
      CellTags->"LanguageFA"],
    Cell[207781, 8795, 474, 14, 133, "Notes",
      CellTags->"LanguageFA"],
    Cell[208258, 8811, 181, 7, 24, "Notes",
      CellTags->"LanguageFA"],
    Cell[208442, 8820, 211, 7, 24, "Notes",
      CellTags->"LanguageFA"],
    Cell[208656, 8829, 183, 5, 40, "Notes",
      CellTags->"LanguageFA"],
    Cell[208842, 8836, 156, 5, 24, "Notes",
      CellTags->"LanguageFA"],
    Cell[209001, 8843, 347, 10, 56, "Notes",
      CellTags->"LanguageFA"],
    Cell[209351, 8855, 398, 14, 40, "Notes",
      CellTags->"LanguageFA"],
    Cell[209752, 8871, 205, 7, 24, "Notes",
      CellTags->"LanguageFA"],
    Cell[209960, 8880, 278, 12, 24, "Notes",
      CellTags->"LanguageFA"],
    Cell[210241, 8894, 68, 1, 39, "ExampleSection",
      CellTags->"LanguageFA"],
    Cell[210312, 8897, 134, 4, 34, "ExampleText",
      CellTags->"LanguageFA"],
    Cell[210449, 8903, 193, 5, 42, "Input",
      CellTags->"LanguageFA"],
    Cell[210645, 8910, 105, 3, 26, "Input",
      CellTags->"LanguageFA"],
    Cell[210753, 8915, 154, 4, 26, "Input",
      CellTags->"LanguageFA"],
    Cell[210910, 8921, 110, 2, 34, "ExampleText",
      CellTags->"LanguageFA"],
    Cell[211023, 8925, 218, 5, 42, "Input",
      CellTags->"LanguageFA"],
    Cell[211244, 8932, 175, 4, 42, "Input",
      CellTags->"LanguageFA"],
    Cell[211422, 8938, 124, 4, 34, "ExampleText",
      CellTags->"LanguageFA"],
    Cell[211549, 8944, 127, 3, 26, "Input",
      CellTags->"LanguageFA"],
    Cell[211679, 8949, 144, 4, 34, "ExampleText",
      CellTags->"LanguageFA"],
    Cell[211826, 8955, 224, 5, 42, "Input",
      CellTags->"LanguageFA"],
    Cell[212053, 8962, 128, 3, 26, "Input",
      CellTags->"LanguageFA"],
    Cell[212184, 8967, 127, 3, 26, "Input",
      CellTags->"LanguageFA"],
    Cell[212314, 8972, 85, 1, 34, "ExampleText",
      CellTags->"LanguageFA"],
    Cell[212402, 8975, 114, 3, 26, "Input",
      CellTags->"LanguageFA"],
    Cell[212519, 8980, 128, 3, 26, "Input",
      CellTags->"LanguageFA"],
    Cell[212650, 8985, 116, 3, 26, "Input",
      CellTags->"LanguageFA"]},
  "LanguageSort"->{
    Cell[212769, 8990, 62, 1, 32, "ObjectName",
      CellTags->"LanguageSort"],
    Cell[212834, 8993, 57, 1, 50, "Highlight",
      CellTags->"LanguageSort"],
    Cell[212894, 8996, 207, 6, 31, "Usage",
      CellTags->"LanguageSort"],
    Cell[213104, 9004, 145, 5, 24, "Notes",
      CellTags->"LanguageSort"],
    Cell[213252, 9011, 178, 7, 24, "Notes",
      CellTags->"LanguageSort"],
    Cell[213433, 9020, 70, 1, 39, "ExampleSection",
      CellTags->"LanguageSort"],
    Cell[213506, 9023, 171, 4, 50, "ExampleText",
      CellTags->"LanguageSort"],
    Cell[213680, 9029, 246, 6, 42, "Input",
      CellTags->"LanguageSort"],
    Cell[213929, 9037, 204, 5, 42, "Input",
      CellTags->"LanguageSort"]},
  "LanguageToSeries"->{
    Cell[214136, 9044, 70, 1, 32, "ObjectName",
      CellTags->"LanguageToSeries"],
    Cell[214209, 9047, 61, 1, 50, "Highlight",
      CellTags->"LanguageToSeries"],
    Cell[214273, 9050, 175, 6, 31, "Usage",
      CellTags->"LanguageToSeries"],
    Cell[214451, 9058, 156, 5, 24, "Notes",
      CellTags->"LanguageToSeries"],
    Cell[214610, 9065, 269, 11, 24, "Notes",
      CellTags->"LanguageToSeries"],
    Cell[214882, 9078, 74, 1, 39, "ExampleSection",
      CellTags->"LanguageToSeries"],
    Cell[214959, 9081, 77, 1, 34, "ExampleText",
      CellTags->"LanguageToSeries"],
    Cell[215039, 9084, 83, 2, 28, "Input",
      CellTags->"LanguageToSeries"],
    Cell[215125, 9088, 110, 3, 26, "Input",
      CellTags->"LanguageToSeries"],
    Cell[215238, 9093, 102, 3, 26, "Input",
      CellTags->"LanguageToSeries"]},
  "LeftQuotient"->{
    Cell[215343, 9098, 62, 1, 32, "ObjectName",
      CellTags->"LeftQuotient"],
    Cell[215408, 9101, 57, 1, 50, "Highlight",
      CellTags->"LeftQuotient"],
    Cell[215468, 9104, 231, 8, 31, "Usage",
      CellTags->"LeftQuotient"],
    Cell[215702, 9114, 182, 7, 24, "Notes",
      CellTags->"LeftQuotient"],
    Cell[215887, 9123, 481, 19, 24, "Notes",
      CellTags->"LeftQuotient"],
    Cell[216371, 9144, 70, 1, 39, "ExampleSection",
      CellTags->"LeftQuotient"],
    Cell[216444, 9147, 145, 4, 34, "ExampleText",
      CellTags->"LeftQuotient"],
    Cell[216592, 9153, 89, 2, 28, "Input",
      CellTags->"LeftQuotient"],
    Cell[216684, 9157, 105, 3, 26, "Input",
      CellTags->"LeftQuotient"],
    Cell[216792, 9162, 106, 3, 26, "Input",
      CellTags->"LeftQuotient"],
    Cell[216901, 9167, 107, 3, 26, "Input",
      CellTags->"LeftQuotient"],
    Cell[217011, 9172, 106, 3, 26, "Input",
      CellTags->"LeftQuotient"]},
  "LengthDFA"->{
    Cell[217120, 9177, 56, 1, 32, "ObjectName",
      CellTags->"LengthDFA"],
    Cell[217179, 9180, 54, 1, 50, "Highlight",
      CellTags->"LengthDFA"],
    Cell[217236, 9183, 228, 8, 31, "Usage",
      CellTags->"LengthDFA"],
    Cell[217467, 9193, 293, 10, 61, "Notes",
      CellTags->"LengthDFA"],
    Cell[217763, 9205, 192, 9, 24, "Notes",
      CellTags->"LengthDFA"],
    Cell[217958, 9216, 229, 9, 24, "Notes",
      CellTags->"LengthDFA"],
    Cell[218190, 9227, 161, 5, 24, "Notes",
      CellTags->"LengthDFA"],
    Cell[218354, 9234, 162, 7, 24, "Notes",
      CellTags->"LengthDFA"],
    Cell[218519, 9243, 67, 1, 39, "ExampleSection",
      CellTags->"LengthDFA"],
    Cell[218589, 9246, 92, 1, 34, "ExampleText",
      CellTags->"LengthDFA"],
    Cell[218684, 9249, 130, 3, 26, "Input",
      CellTags->"LengthDFA"],
    Cell[218817, 9254, 134, 3, 26, "Input",
      CellTags->"LengthDFA"],
    Cell[218954, 9259, 94, 1, 34, "ExampleText",
      CellTags->"LengthDFA"],
    Cell[219051, 9262, 194, 4, 42, "Input",
      CellTags->"LengthDFA"],
    Cell[219248, 9268, 66, 1, 34, "ExampleText",
      CellTags->"LengthDFA"],
    Cell[219317, 9271, 221, 5, 42, "Input",
      CellTags->"LengthDFA"]},
  "LocalQSA"->{
    Cell[219541, 9278, 54, 1, 32, "ObjectName",
      CellTags->"LocalQSA"],
    Cell[219598, 9281, 53, 1, 50, "Highlight",
      CellTags->"LocalQSA"],
    Cell[219654, 9284, 170, 6, 31, "Usage",
      CellTags->"LocalQSA"],
    Cell[219827, 9292, 224, 9, 44, "Notes",
      CellTags->"LocalQSA"],
    Cell[220054, 9303, 220, 7, 40, "Notes",
      CellTags->"LocalQSA"],
    Cell[220277, 9312, 212, 6, 40, "Notes",
      CellTags->"LocalQSA"],
    Cell[220492, 9320, 261, 11, 24, "Notes",
      CellTags->"LocalQSA"],
    Cell[220756, 9333, 66, 1, 39, "ExampleSection",
      CellTags->"LocalQSA"],
    Cell[220825, 9336, 228, 5, 50, "ExampleText",
      CellTags->"LocalQSA"],
    Cell[221056, 9343, 109, 3, 26, "Input",
      CellTags->"LocalQSA"],
    Cell[221168, 9348, 145, 4, 42, "Input",
      CellTags->"LocalQSA"],
    Cell[221316, 9354, 96, 3, 26, "Input",
      CellTags->"LocalQSA"],
    Cell[221415, 9359, 115, 3, 26, "Input",
      CellTags->"LocalQSA"],
    Cell[221533, 9364, 63, 1, 34, "ExampleText",
      CellTags->"LocalQSA"],
    Cell[221599, 9367, 122, 3, 26, "Input",
      CellTags->"LocalQSA"]},
  "LocalUniversalFA"->{
    Cell[221724, 9372, 70, 1, 32, "ObjectName",
      CellTags->"LocalUniversalFA"],
    Cell[221797, 9375, 61, 1, 50, "Highlight",
      CellTags->"LocalUniversalFA"],
    Cell[221861, 9378, 231, 8, 31, "Usage",
      CellTags->"LocalUniversalFA"],
    Cell[222095, 9388, 437, 12, 97, "Notes",
      CellTags->"LocalUniversalFA"],
    Cell[222535, 9402, 292, 11, 40, "Notes",
      CellTags->"LocalUniversalFA"],
    Cell[222830, 9415, 83, 1, 24, "Notes",
      CellTags->"LocalUniversalFA"],
    Cell[222916, 9418, 175, 7, 24, "Notes",
      CellTags->"LocalUniversalFA"],
    Cell[223094, 9427, 74, 1, 39, "ExampleSection",
      CellTags->"LocalUniversalFA"],
    Cell[223171, 9430, 335, 9, 50, "ExampleText",
      CellTags->"LocalUniversalFA"],
    Cell[223509, 9441, 296, 7, 58, "Input",
      CellTags->"LocalUniversalFA"],
    Cell[223808, 9450, 100, 3, 26, "Input",
      CellTags->"LocalUniversalFA"],
    Cell[223911, 9455, 173, 6, 34, "ExampleText",
      CellTags->"LocalUniversalFA"],
    Cell[224087, 9463, 118, 3, 26, "Input",
      CellTags->"LocalUniversalFA"]},
  "LoopAutomatonFA"->{
    Cell[224208, 9468, 68, 1, 32, "ObjectName",
      CellTags->"LoopAutomatonFA"],
    Cell[224279, 9471, 60, 1, 50, "Highlight",
      CellTags->"LoopAutomatonFA"],
    Cell[224342, 9474, 172, 6, 31, "Usage",
      CellTags->"LoopAutomatonFA"],
    Cell[224517, 9482, 104, 1, 24, "Notes",
      CellTags->"LoopAutomatonFA"],
    Cell[224624, 9485, 271, 11, 24, "Notes",
      CellTags->"LoopAutomatonFA"],
    Cell[224898, 9498, 73, 1, 39, "ExampleSection",
      CellTags->"LoopAutomatonFA"],
    Cell[224974, 9501, 176, 4, 50, "ExampleText",
      CellTags->"LoopAutomatonFA"],
    Cell[225153, 9507, 197, 4, 42, "Input",
      CellTags->"LoopAutomatonFA"],
    Cell[225353, 9513, 121, 3, 26, "Input",
      CellTags->"LoopAutomatonFA"],
    Cell[225477, 9518, 219, 6, 50, "ExampleText",
      CellTags->"LoopAutomatonFA"],
    Cell[225699, 9526, 190, 4, 42, "Input",
      CellTags->"LoopAutomatonFA"],
    Cell[225892, 9532, 128, 4, 34, "ExampleText",
      CellTags->"LoopAutomatonFA"],
    Cell[226023, 9538, 167, 4, 26, "Input",
      CellTags->"LoopAutomatonFA"]},
  "MakeHomomorphismDFA"->{
    Cell[226193, 9544, 76, 1, 32, "ObjectName",
      CellTags->"MakeHomomorphismDFA"],
    Cell[226272, 9547, 64, 1, 50, "Highlight",
      CellTags->"MakeHomomorphismDFA"],
    Cell[226339, 9550, 284, 10, 47, "Usage",
      CellTags->"MakeHomomorphismDFA"],
    Cell[226626, 9562, 185, 7, 24, "Notes",
      CellTags->"MakeHomomorphismDFA"],
    Cell[226814, 9571, 120, 5, 24, "Notes",
      CellTags->"MakeHomomorphismDFA"],
    Cell[226937, 9578, 173, 7, 24, "Notes",
      CellTags->"MakeHomomorphismDFA"],
    Cell[227113, 9587, 221, 9, 24, "Notes",
      CellTags->"MakeHomomorphismDFA"],
    Cell[227337, 9598, 77, 1, 39, "ExampleSection",
      CellTags->"MakeHomomorphismDFA"],
    Cell[227417, 9601, 98, 1, 34, "ExampleText",
      CellTags->"MakeHomomorphismDFA"],
    Cell[227518, 9604, 175, 4, 42, "Input",
      CellTags->"MakeHomomorphismDFA"],
    Cell[227696, 9610, 140, 4, 34, "ExampleText",
      CellTags->"MakeHomomorphismDFA"],
    Cell[227839, 9616, 132, 3, 26, "Input",
      CellTags->"MakeHomomorphismDFA"],
    Cell[227974, 9621, 237, 5, 58, "Input",
      CellTags->"MakeHomomorphismDFA"]},
  "MakeHomomorphism"->{
    Cell[228214, 9628, 70, 1, 32, "ObjectName",
      CellTags->"MakeHomomorphism"],
    Cell[228287, 9631, 61, 1, 50, "Highlight",
      CellTags->"MakeHomomorphism"],
    Cell[228351, 9634, 267, 10, 47, "Usage",
      CellTags->"MakeHomomorphism"],
    Cell[228621, 9646, 231, 9, 44, "Notes",
      CellTags->"MakeHomomorphism"],
    Cell[228855, 9657, 193, 7, 24, "Notes",
      CellTags->"MakeHomomorphism"],
    Cell[229051, 9666, 117, 5, 24, "Notes",
      CellTags->"MakeHomomorphism"],
    Cell[229171, 9673, 208, 8, 24, "Notes",
      CellTags->"MakeHomomorphism"],
    Cell[229382, 9683, 177, 7, 24, "Notes",
      CellTags->"MakeHomomorphism"],
    Cell[229562, 9692, 74, 1, 39, "ExampleSection",
      CellTags->"MakeHomomorphism"],
    Cell[229639, 9695, 148, 4, 34, "ExampleText",
      CellTags->"MakeHomomorphism"],
    Cell[229790, 9701, 107, 2, 28, "Input",
      CellTags->"MakeHomomorphism"],
    Cell[229900, 9705, 184, 5, 42, "Input",
      CellTags->"MakeHomomorphism"],
    Cell[230087, 9712, 132, 2, 34, "ExampleText",
      Evaluatable->False,
      CellTags->"MakeHomomorphism"],
    Cell[230222, 9716, 104, 3, 26, "Input",
      CellTags->"MakeHomomorphism"]},
  "MakeTrim"->{
    Cell[230329, 9721, 54, 1, 32, "ObjectName",
      CellTags->"MakeTrim"],
    Cell[230386, 9724, 51, 1, 50, "Highlight",
      CellTags->"MakeTrim"],
    Cell[230440, 9727, 154, 6, 31, "Usage",
      CellTags->"MakeTrim"],
    Cell[230597, 9735, 147, 4, 24, "Notes",
      CellTags->"MakeTrim"],
    Cell[230747, 9741, 161, 7, 24, "Notes",
      CellTags->"MakeTrim"]},
  "MatrixToTransitions"->{
    Cell[230911, 9750, 76, 1, 32, "ObjectName",
      CellTags->"MatrixToTransitions"],
    Cell[230990, 9753, 64, 1, 50, "Highlight",
      CellTags->"MatrixToTransitions"],
    Cell[231057, 9756, 232, 6, 47, "Usage",
      CellTags->"MatrixToTransitions"],
    Cell[231292, 9764, 97, 1, 24, "Notes",
      CellTags->"MatrixToTransitions"],
    Cell[231392, 9767, 280, 11, 24, "Notes",
      CellTags->"MatrixToTransitions"],
    Cell[231675, 9780, 77, 1, 39, "ExampleSection",
      CellTags->"MatrixToTransitions"],
    Cell[231755, 9783, 130, 4, 34, "ExampleText",
      CellTags->"MatrixToTransitions"],
    Cell[231888, 9789, 149, 3, 26, "Input",
      CellTags->"MatrixToTransitions"],
    Cell[232040, 9794, 100, 2, 28, "Input",
      CellTags->"MatrixToTransitions"]},
  "MinimalFischerFA"->{
    Cell[232143, 9798, 70, 1, 32, "ObjectName",
      CellTags->"MinimalFischerFA"],
    Cell[232216, 9801, 61, 1, 50, "Highlight",
      CellTags->"MinimalFischerFA"],
    Cell[232280, 9804, 207, 6, 47, "Usage",
      CellTags->"MinimalFischerFA"],
    Cell[232490, 9812, 368, 11, 79, "Notes",
      CellTags->"MinimalFischerFA"],
    Cell[232861, 9825, 250, 8, 40, "Notes",
      CellTags->"MinimalFischerFA"],
    Cell[233114, 9835, 218, 6, 40, "Notes",
      CellTags->"MinimalFischerFA"],
    Cell[233335, 9843, 74, 1, 39, "ExampleSection",
      CellTags->"MinimalFischerFA"],
    Cell[233412, 9846, 106, 1, 34, "ExampleText",
      CellTags->"MinimalFischerFA"],
    Cell[233521, 9849, 92, 2, 28, "Input",
      CellTags->"MinimalFischerFA"],
    Cell[233616, 9853, 118, 3, 26, "Input",
      CellTags->"MinimalFischerFA"],
    Cell[233737, 9858, 111, 3, 26, "Input",
      CellTags->"MinimalFischerFA"],
    Cell[233851, 9863, 115, 3, 26, "Input",
      CellTags->"MinimalFischerFA"]},
  "MinimizeDFA"->{
    Cell[233969, 9868, 60, 1, 32, "ObjectName",
      CellTags->"MinimizeDFA"],
    Cell[234032, 9871, 56, 1, 50, "Highlight",
      CellTags->"MinimizeDFA"],
    Cell[234091, 9874, 169, 6, 31, "Usage",
      CellTags->"MinimizeDFA"],
    Cell[234263, 9882, 348, 11, 79, "Notes",
      CellTags->"MinimizeDFA"],
    Cell[234614, 9895, 305, 9, 40, "Notes",
      CellTags->"MinimizeDFA"],
    Cell[234922, 9906, 146, 5, 24, "Notes",
      CellTags->"MinimizeDFA"],
    Cell[235071, 9913, 272, 8, 40, "Notes",
      CellTags->"MinimizeDFA"],
    Cell[235346, 9923, 270, 11, 24, "Notes",
      CellTags->"MinimizeDFA"],
    Cell[235619, 9936, 69, 1, 39, "ExampleSection",
      CellTags->"MinimizeDFA"],
    Cell[235691, 9939, 78, 1, 34, "ExampleText",
      CellTags->"MinimizeDFA"],
    Cell[235772, 9942, 138, 3, 26, "Input",
      CellTags->"MinimizeDFA"],
    Cell[235913, 9947, 286, 8, 74, "Input",
      CellTags->"MinimizeDFA"]},
  "MinimizeFA"->{
    Cell[236202, 9957, 58, 1, 32, "ObjectName",
      CellTags->"MinimizeFA"],
    Cell[236263, 9960, 55, 1, 50, "Highlight",
      CellTags->"MinimizeFA"],
    Cell[236321, 9963, 232, 6, 47, "Usage",
      CellTags->"MinimizeFA"],
    Cell[236556, 9971, 365, 11, 79, "Notes",
      CellTags->"MinimizeFA"],
    Cell[236924, 9984, 222, 6, 40, "Notes",
      CellTags->"MinimizeFA"],
    Cell[237149, 9992, 209, 6, 40, "Notes",
      CellTags->"MinimizeFA"],
    Cell[237361, 10000, 362, 15, 24, "Notes",
      CellTags->"MinimizeFA"],
    Cell[237726, 10017, 68, 1, 39, "ExampleSection",
      CellTags->"MinimizeFA"],
    Cell[237797, 10020, 158, 4, 50, "ExampleText",
      CellTags->"MinimizeFA"],
    Cell[237958, 10026, 125, 5, 45, "Input",
      CellTags->"MinimizeFA"],
    Cell[238086, 10033, 147, 4, 26, "Input",
      CellTags->"MinimizeFA"]},
  "NFA"->{
    Cell[238236, 10039, 44, 1, 32, "ObjectName",
      CellTags->"NFA"],
    Cell[238283, 10042, 51, 1, 50, "Highlight",
      CellTags->"NFA"],
    Cell[238337, 10045, 136, 4, 31, "Usage",
      CellTags->"NFA"],
    Cell[238476, 10051, 276, 11, 24, "Notes",
      CellTags->"NFA"],
    Cell[238755, 10064, 169, 7, 24, "Notes",
      CellTags->"NFA"],
    Cell[238927, 10073, 262, 9, 24, "Notes",
      CellTags->"NFA"],
    Cell[239192, 10084, 184, 7, 24, "Notes",
      CellTags->"NFA"],
    Cell[239379, 10093, 302, 15, 24, "Notes",
      CellTags->"NFA"],
    Cell[239684, 10110, 61, 1, 39, "ExampleSection",
      CellTags->"NFA"],
    Cell[239748, 10113, 54, 1, 34, "ExampleText",
      CellTags->"NFA"],
    Cell[239805, 10116, 173, 5, 26, "Input",
      CellTags->"NFA"],
    Cell[239981, 10123, 109, 3, 26, "Input",
      CellTags->"NFA"]},
  "NNFAToFA"->{
    Cell[240093, 10128, 54, 1, 32, "ObjectName",
      CellTags->"NNFAToFA"],
    Cell[240150, 10131, 53, 1, 50, "Highlight",
      CellTags->"NNFAToFA"],
    Cell[240206, 10134, 177, 6, 31, "Usage",
      CellTags->"NNFAToFA"],
    Cell[240386, 10142, 140, 4, 24, "Notes",
      CellTags->"NNFAToFA"],
    Cell[240529, 10148, 187, 7, 24, "Notes",
      CellTags->"NNFAToFA"],
    Cell[240719, 10157, 66, 1, 39, "ExampleSection",
      CellTags->"NNFAToFA"],
    Cell[240788, 10160, 242, 7, 34, "ExampleText",
      CellTags->"NNFAToFA"],
    Cell[241033, 10169, 110, 2, 28, "Input",
      CellTags->"NNFAToFA"],
    Cell[241146, 10173, 109, 3, 26, "Input",
      CellTags->"NNFAToFA"],
    Cell[241258, 10178, 67, 1, 34, "ExampleText",
      CellTags->"NNFAToFA"],
    Cell[241328, 10181, 77, 2, 28, "Input",
      CellTags->"NNFAToFA"],
    Cell[241408, 10185, 103, 3, 26, "Input",
      CellTags->"NNFAToFA"]},
  "NNFA"->{
    Cell[241514, 10190, 46, 1, 32, "ObjectName",
      CellTags->"NNFA"],
    Cell[241563, 10193, 52, 1, 50, "Highlight",
      CellTags->"NNFA"],
    Cell[241618, 10196, 157, 4, 31, "Usage",
      CellTags->"NNFA"],
    Cell[241778, 10202, 291, 6, 56, "Notes",
      CellTags->"NNFA"],
    Cell[242072, 10210, 270, 11, 24, "Notes",
      CellTags->"NNFA"],
    Cell[242345, 10223, 62, 1, 39, "ExampleSection",
      CellTags->"NNFA"],
    Cell[242410, 10226, 56, 1, 34, "ExampleText",
      CellTags->"NNFA"],
    Cell[242469, 10229, 104, 2, 28, "Input",
      CellTags->"NNFA"]},
  "Normalize"->{
    Cell[242576, 10233, 56, 1, 32, "ObjectName",
      CellTags->"Normalize"],
    Cell[242635, 10236, 52, 1, 50, "Highlight",
      CellTags->"Normalize"],
    Cell[242690, 10239, 137, 4, 31, "Usage",
      CellTags->"Normalize"],
    Cell[242830, 10245, 269, 9, 40, "Notes",
      CellTags->"Normalize"],
    Cell[243102, 10256, 813, 26, 88, "Notes",
      CellTags->"Normalize"],
    Cell[243918, 10284, 147, 5, 24, "Notes",
      CellTags->"Normalize"],
    Cell[244068, 10291, 268, 11, 24, "Notes",
      CellTags->"Normalize"],
    Cell[244339, 10304, 67, 1, 39, "ExampleSection",
      CellTags->"Normalize"],
    Cell[244409, 10307, 79, 1, 34, "ExampleText",
      CellTags->"Normalize"],
    Cell[244491, 10310, 268, 6, 74, "Input",
      CellTags->"Normalize"],
    Cell[244762, 10318, 100, 3, 26, "Input",
      CellTags->"Normalize"],
    Cell[244865, 10323, 126, 4, 34, "ExampleText",
      CellTags->"Normalize"],
    Cell[244994, 10329, 134, 3, 26, "Input",
      CellTags->"Normalize"],
    Cell[245131, 10334, 193, 5, 50, "ExampleText",
      CellTags->"Normalize"],
    Cell[245327, 10341, 134, 3, 26, "Input",
      CellTags->"Normalize"],
    Cell[245464, 10346, 191, 5, 50, "ExampleText",
      CellTags->"Normalize"],
    Cell[245658, 10353, 331, 6, 74, "Input",
      CellTags->"Normalize"]},
  "NormalizeControl"->{
    Cell[245992, 10361, 70, 1, 32, "ObjectName",
      CellTags->"NormalizeControl"],
    Cell[246065, 10364, 61, 1, 50, "Highlight",
      CellTags->"NormalizeControl"],
    Cell[246129, 10367, 158, 4, 31, "Usage",
      CellTags->"NormalizeControl"],
    Cell[246290, 10373, 176, 5, 24, "Notes",
      CellTags->"NormalizeControl"],
    Cell[246469, 10380, 154, 5, 24, "Notes",
      CellTags->"NormalizeControl"],
    Cell[246626, 10387, 163, 4, 40, "Notes",
      CellTags->"NormalizeControl"],
    Cell[246792, 10393, 74, 1, 39, "ExampleSection",
      CellTags->"NormalizeControl"],
    Cell[246869, 10396, 157, 4, 34, "ExampleText",
      CellTags->"NormalizeControl"],
    Cell[247029, 10402, 152, 4, 42, "Input",
      CellTags->"NormalizeControl"],
    Cell[247184, 10408, 173, 4, 42, "Input",
      CellTags->"NormalizeControl"],
    Cell[247360, 10414, 128, 4, 34, "ExampleText",
      CellTags->"NormalizeControl"],
    Cell[247491, 10420, 187, 5, 58, "Input",
      CellTags->"NormalizeControl"]},
  "NormalizeFA"->{
    Cell[247681, 10427, 60, 1, 32, "ObjectName",
      CellTags->"NormalizeFA"],
    Cell[247744, 10430, 56, 1, 50, "Highlight",
      CellTags->"NormalizeFA"],
    Cell[247803, 10433, 160, 6, 31, "Usage",
      CellTags->"NormalizeFA"],
    Cell[247966, 10441, 291, 9, 40, "Notes",
      CellTags->"NormalizeFA"],
    Cell[248260, 10452, 139, 4, 24, "Notes",
      CellTags->"NormalizeFA"],
    Cell[248402, 10458, 185, 7, 24, "Notes",
      CellTags->"NormalizeFA"],
    Cell[248590, 10467, 69, 1, 39, "ExampleSection",
      CellTags->"NormalizeFA"],
    Cell[248662, 10470, 133, 4, 34, "ExampleText",
      CellTags->"NormalizeFA"],
    Cell[248798, 10476, 171, 4, 42, "Input",
      CellTags->"NormalizeFA"],
    Cell[248972, 10482, 138, 4, 42, "Input",
      CellTags->"NormalizeFA"]},
  "NumberOfDFA"->{
    Cell[249113, 10488, 60, 1, 32, "ObjectName",
      CellTags->"NumberOfDFA"],
    Cell[249176, 10491, 56, 1, 50, "Highlight",
      CellTags->"NumberOfDFA"],
    Cell[249235, 10494, 210, 8, 31, "Usage",
      CellTags->"NumberOfDFA"],
    Cell[249448, 10504, 171, 5, 24, "Notes",
      CellTags->"NumberOfDFA"],
    Cell[249622, 10511, 354, 15, 24, "Notes",
      CellTags->"NumberOfDFA"],
    Cell[249979, 10528, 69, 1, 39, "ExampleSection",
      CellTags->"NumberOfDFA"],
    Cell[250051, 10531, 91, 1, 34, "ExampleText",
      CellTags->"NumberOfDFA"],
    Cell[250145, 10534, 115, 2, 28, "Input",
      CellTags->"NumberOfDFA"]},
  "NumTransitions"->{
    Cell[250263, 10538, 66, 1, 32, "ObjectName",
      CellTags->"NumTransitions"],
    Cell[250332, 10541, 59, 1, 50, "Highlight",
      CellTags->"NumTransitions"],
    Cell[250394, 10544, 179, 6, 31, "Usage",
      CellTags->"NumTransitions"],
    Cell[250576, 10552, 155, 7, 24, "Notes",
      CellTags->"NumTransitions"],
    Cell[250734, 10561, 72, 1, 39, "ExampleSection",
      CellTags->"NumTransitions"],
    Cell[250809, 10564, 109, 1, 34, "ExampleText",
      CellTags->"NumTransitions"],
    Cell[250921, 10567, 209, 5, 42, "Input",
      CellTags->"NumTransitions"],
    Cell[251133, 10574, 106, 3, 26, "Input",
      CellTags->"NumTransitions"]},
  "PermutationSA"->{
    Cell[251242, 10579, 64, 1, 32, "ObjectName",
      CellTags->"PermutationSA"],
    Cell[251309, 10582, 58, 1, 50, "Highlight",
      CellTags->"PermutationSA"],
    Cell[251370, 10585, 213, 6, 47, "Usage",
      CellTags->"PermutationSA"],
    Cell[251586, 10593, 267, 9, 40, "Notes",
      CellTags->"PermutationSA"],
    Cell[251856, 10604, 213, 9, 24, "Notes",
      CellTags->"PermutationSA"],
    Cell[252072, 10615, 71, 1, 39, "ExampleSection",
      CellTags->"PermutationSA"],
    Cell[252146, 10618, 87, 1, 34, "ExampleText",
      CellTags->"PermutationSA"],
    Cell[252236, 10621, 113, 2, 28, "Input",
      CellTags->"PermutationSA"],
    Cell[252352, 10625, 131, 4, 34, "ExampleText",
      CellTags->"PermutationSA"],
    Cell[252486, 10631, 156, 4, 42, "Input",
      CellTags->"PermutationSA"],
    Cell[252645, 10637, 176, 4, 26, "Input",
      CellTags->"PermutationSA"],
    Cell[252824, 10643, 55, 1, 26, "Input",
      CellTags->"PermutationSA"]},
  "PermuteFA"->{
    Cell[252882, 10646, 56, 1, 32, "ObjectName",
      CellTags->"PermuteFA"],
    Cell[252941, 10649, 54, 1, 50, "Highlight",
      CellTags->"PermuteFA"],
    Cell[252998, 10652, 215, 8, 31, "Usage",
      CellTags->"PermuteFA"],
    Cell[253216, 10662, 211, 7, 24, "Notes",
      CellTags->"PermuteFA"],
    Cell[253430, 10671, 67, 1, 39, "ExampleSection",
      CellTags->"PermuteFA"],
    Cell[253500, 10674, 104, 1, 34, "ExampleText",
      CellTags->"PermuteFA"],
    Cell[253607, 10677, 106, 2, 28, "Input",
      CellTags->"PermuteFA"],
    Cell[253716, 10681, 122, 3, 26, "Input",
      CellTags->"PermuteFA"],
    Cell[253841, 10686, 105, 3, 26, "Input",
      CellTags->"PermuteFA"]},
  "PrintFA"->{
    Cell[253949, 10691, 52, 1, 32, "ObjectName",
      CellTags->"PrintFA"],
    Cell[254004, 10694, 52, 1, 50, "Highlight",
      CellTags->"PrintFA"],
    Cell[254059, 10697, 158, 6, 31, "Usage",
      CellTags->"PrintFA"],
    Cell[254220, 10705, 273, 10, 61, "Notes",
      CellTags->"PrintFA"],
    Cell[254496, 10717, 144, 5, 24, "Notes",
      CellTags->"PrintFA"],
    Cell[254643, 10724, 135, 5, 24, "Notes",
      CellTags->"PrintFA"],
    Cell[254781, 10731, 79, 1, 24, "Notes",
      CellTags->"PrintFA"],
    Cell[254863, 10734, 65, 1, 39, "ExampleSection",
      CellTags->"PrintFA"],
    Cell[254931, 10737, 61, 1, 34, "ExampleText",
      CellTags->"PrintFA"],
    Cell[254995, 10740, 91, 2, 28, "Input",
      CellTags->"PrintFA"],
    Cell[255089, 10744, 92, 3, 26, "Input",
      CellTags->"PrintFA"],
    Cell[255184, 10749, 61, 1, 34, "ExampleText",
      CellTags->"PrintFA"],
    Cell[255248, 10752, 93, 2, 28, "Input",
      CellTags->"PrintFA"],
    Cell[255344, 10756, 92, 3, 26, "Input",
      CellTags->"PrintFA"]},
  "ProductFA"->{
    Cell[255439, 10761, 56, 1, 32, "ObjectName",
      CellTags->"ProductFA"],
    Cell[255498, 10764, 54, 1, 50, "Highlight",
      CellTags->"ProductFA"],
    Cell[255555, 10767, 199, 8, 31, "Usage",
      CellTags->"ProductFA"],
    Cell[255757, 10777, 350, 11, 79, "Notes",
      CellTags->"ProductFA"],
    Cell[256110, 10790, 249, 6, 62, "Notes",
      CellTags->"ProductFA"],
    Cell[256362, 10798, 235, 6, 40, "Notes",
      CellTags->"ProductFA"],
    Cell[256600, 10806, 446, 19, 24, "Notes",
      CellTags->"ProductFA"],
    Cell[257049, 10827, 67, 1, 39, "ExampleSection",
      CellTags->"ProductFA"],
    Cell[257119, 10830, 97, 1, 34, "ExampleText",
      CellTags->"ProductFA"],
    Cell[257219, 10833, 128, 5, 45, "Input",
      CellTags->"ProductFA"],
    Cell[257350, 10840, 110, 3, 26, "Input",
      CellTags->"ProductFA"],
    Cell[257463, 10845, 244, 9, 34, "ExampleText",
      CellTags->"ProductFA"],
    Cell[257710, 10856, 104, 3, 26, "Input",
      CellTags->"ProductFA"]},
  "ProductTransitions"->{
    Cell[257817, 10861, 74, 1, 32, "ObjectName",
      CellTags->"ProductTransitions"],
    Cell[257894, 10864, 63, 1, 50, "Highlight",
      CellTags->"ProductTransitions"],
    Cell[257960, 10867, 199, 5, 47, "Usage",
      CellTags->"ProductTransitions"],
    Cell[258162, 10874, 230, 9, 44, "Notes",
      CellTags->"ProductTransitions"],
    Cell[258395, 10885, 446, 16, 40, "Notes",
      CellTags->"ProductTransitions"],
    Cell[258844, 10903, 253, 11, 24, "Notes",
      CellTags->"ProductTransitions"],
    Cell[259100, 10916, 276, 11, 24, "Notes",
      CellTags->"ProductTransitions"],
    Cell[259379, 10929, 286, 12, 24, "Notes",
      CellTags->"ProductTransitions"],
    Cell[259668, 10943, 76, 1, 39, "ExampleSection",
      CellTags->"ProductTransitions"],
    Cell[259747, 10946, 112, 1, 34, "ExampleText",
      CellTags->"ProductTransitions"],
    Cell[259862, 10949, 203, 4, 42, "Input",
      CellTags->"ProductTransitions"],
    Cell[260068, 10955, 120, 3, 26, "Input",
      CellTags->"ProductTransitions"],
    Cell[260191, 10960, 138, 3, 26, "Input",
      CellTags->"ProductTransitions"],
    Cell[260332, 10965, 138, 3, 26, "Input",
      CellTags->"ProductTransitions"]},
  "QuotientDFA"->{
    Cell[260473, 10970, 60, 1, 32, "ObjectName",
      CellTags->"QuotientDFA"],
    Cell[260536, 10973, 56, 1, 50, "Highlight",
      CellTags->"QuotientDFA"],
    Cell[260595, 10976, 228, 8, 31, "Usage",
      CellTags->"QuotientDFA"],
    Cell[260826, 10986, 251, 9, 44, "Notes",
      CellTags->"QuotientDFA"],
    Cell[261080, 10997, 74, 1, 24, "Notes",
      CellTags->"QuotientDFA"],
    Cell[261157, 11000, 580, 23, 40, "Notes",
      CellTags->"QuotientDFA"],
    Cell[261740, 11025, 69, 1, 39, "ExampleSection",
      CellTags->"QuotientDFA"],
    Cell[261812, 11028, 155, 4, 50, "ExampleText",
      CellTags->"QuotientDFA"],
    Cell[261970, 11034, 88, 2, 28, "Input",
      CellTags->"QuotientDFA"],
    Cell[262061, 11038, 128, 3, 26, "Input",
      CellTags->"QuotientDFA"],
    Cell[262192, 11043, 113, 3, 26, "Input",
      CellTags->"QuotientDFA"],
    Cell[262308, 11048, 111, 2, 34, "ExampleText",
      CellTags->"QuotientDFA"],
    Cell[262422, 11052, 142, 4, 42, "Input",
      CellTags->"QuotientDFA"]},
  "QuotientMachineFA"->{
    Cell[262567, 11058, 72, 1, 32, "ObjectName",
      CellTags->"QuotientMachineFA"],
    Cell[262642, 11061, 62, 1, 50, "Highlight",
      CellTags->"QuotientMachineFA"],
    Cell[262707, 11064, 368, 14, 47, "Usage",
      CellTags->"QuotientMachineFA"],
    Cell[263078, 11080, 257, 9, 44, "Notes",
      CellTags->"QuotientMachineFA"],
    Cell[263338, 11091, 271, 7, 24, "Notes",
      CellTags->"QuotientMachineFA"],
    Cell[263612, 11100, 108, 1, 24, "Notes",
      CellTags->"QuotientMachineFA"],
    Cell[263723, 11103, 475, 19, 24, "Notes",
      CellTags->"QuotientMachineFA"],
    Cell[264201, 11124, 75, 1, 39, "ExampleSection",
      CellTags->"QuotientMachineFA"],
    Cell[264279, 11127, 378, 12, 34, "ExampleText",
      CellTags->"QuotientMachineFA"],
    Cell[264660, 11141, 174, 5, 42, "Input",
      CellTags->"QuotientMachineFA"],
    Cell[264837, 11148, 143, 3, 26, "Input",
      CellTags->"QuotientMachineFA"],
    Cell[264983, 11153, 162, 4, 26, "Input",
      CellTags->"QuotientMachineFA"],
    Cell[265148, 11159, 107, 2, 28, "Input",
      CellTags->"QuotientMachineFA"],
    Cell[265258, 11163, 119, 3, 26, "Input",
      CellTags->"QuotientMachineFA"],
    Cell[265380, 11168, 103, 1, 34, "ExampleText",
      CellTags->"QuotientMachineFA"],
    Cell[265486, 11171, 118, 3, 26, "Input",
      CellTags->"QuotientMachineFA"]},
  "RandomDFA"->{
    Cell[265607, 11176, 56, 1, 32, "ObjectName",
      CellTags->"RandomDFA"],
    Cell[265666, 11179, 54, 1, 50, "Highlight",
      CellTags->"RandomDFA"],
    Cell[265723, 11182, 147, 4, 31, "Usage",
      CellTags->"RandomDFA"],
    Cell[265873, 11188, 102, 1, 24, "Notes",
      CellTags->"RandomDFA"],
    Cell[265978, 11191, 356, 15, 24, "Notes",
      CellTags->"RandomDFA"],
    Cell[266337, 11208, 67, 1, 39, "ExampleSection",
      CellTags->"RandomDFA"],
    Cell[266407, 11211, 99, 1, 34, "ExampleText",
      CellTags->"RandomDFA"],
    Cell[266509, 11214, 82, 2, 28, "Input",
      CellTags->"RandomDFA"],
    Cell[266594, 11218, 82, 2, 28, "Input",
      CellTags->"RandomDFA"]},
  "RankOfDFA"->{
    Cell[266679, 11222, 56, 1, 32, "ObjectName",
      CellTags->"RankOfDFA"],
    Cell[266738, 11225, 54, 1, 50, "Highlight",
      CellTags->"RankOfDFA"],
    Cell[266795, 11228, 194, 6, 31, "Usage",
      CellTags->"RankOfDFA"],
    Cell[266992, 11236, 352, 15, 24, "Notes",
      CellTags->"RankOfDFA"],
    Cell[267347, 11253, 67, 1, 39, "ExampleSection",
      CellTags->"RankOfDFA"],
    Cell[267417, 11256, 66, 1, 34, "ExampleText",
      CellTags->"RankOfDFA"],
    Cell[267486, 11259, 129, 3, 26, "Input",
      CellTags->"RankOfDFA"],
    Cell[267618, 11264, 100, 3, 26, "Input",
      CellTags->"RankOfDFA"],
    Cell[267721, 11269, 110, 3, 26, "Input",
      CellTags->"RankOfDFA"]},
  "RankToDFA"->{
    Cell[267834, 11274, 56, 1, 32, "ObjectName",
      CellTags->"RankToDFA"],
    Cell[267893, 11277, 54, 1, 50, "Highlight",
      CellTags->"RankToDFA"],
    Cell[267950, 11280, 275, 10, 47, "Usage",
      CellTags->"RankToDFA"],
    Cell[268228, 11292, 96, 1, 24, "Notes",
      CellTags->"RankToDFA"],
    Cell[268327, 11295, 369, 17, 24, "Notes",
      CellTags->"RankToDFA"],
    Cell[268699, 11314, 67, 1, 39, "ExampleSection",
      CellTags->"RankToDFA"],
    Cell[268769, 11317, 61, 1, 34, "ExampleText",
      CellTags->"RankToDFA"],
    Cell[268833, 11320, 133, 5, 45, "Input",
      CellTags->"RankToDFA"]},
  "DeleteTransitionFA"->{
    Cell[268969, 11327, 74, 1, 32, "ObjectName",
      CellTags->"DeleteTransitionFA"],
    Cell[269046, 11330, 63, 1, 50, "Highlight",
      CellTags->"DeleteTransitionFA"],
    Cell[269112, 11333, 302, 11, 47, "Usage",
      CellTags->"DeleteTransitionFA"],
    Cell[269417, 11346, 265, 8, 40, "Notes",
      CellTags->"DeleteTransitionFA"],
    Cell[269685, 11356, 293, 11, 24, "Notes",
      CellTags->"DeleteTransitionFA"],
    Cell[269981, 11369, 76, 1, 39, "ExampleSection",
      CellTags->"DeleteTransitionFA"],
    Cell[270060, 11372, 96, 1, 34, "ExampleText",
      CellTags->"DeleteTransitionFA"],
    Cell[270159, 11375, 224, 5, 42, "Input",
      CellTags->"DeleteTransitionFA"],
    Cell[270386, 11382, 106, 3, 26, "Input",
      CellTags->"DeleteTransitionFA"],
    Cell[270495, 11387, 152, 4, 34, "ExampleText",
      CellTags->"DeleteTransitionFA"],
    Cell[270650, 11393, 132, 3, 26, "Input",
      CellTags->"DeleteTransitionFA"],
    Cell[270785, 11398, 106, 3, 26, "Input",
      CellTags->"DeleteTransitionFA"]},
  "ReverseFA"->{
    Cell[270894, 11403, 56, 1, 32, "ObjectName",
      CellTags->"ReverseFA"],
    Cell[270953, 11406, 54, 1, 50, "Highlight",
      CellTags->"ReverseFA"],
    Cell[271010, 11409, 207, 6, 31, "Usage",
      CellTags->"ReverseFA"],
    Cell[271220, 11417, 117, 4, 24, "Notes",
      CellTags->"ReverseFA"],
    Cell[271340, 11423, 67, 1, 39, "ExampleSection",
      CellTags->"ReverseFA"],
    Cell[271410, 11426, 178, 6, 34, "ExampleText",
      CellTags->"ReverseFA"],
    Cell[271591, 11434, 127, 3, 26, "Input",
      CellTags->"ReverseFA"],
    Cell[271721, 11439, 101, 3, 26, "Input",
      CellTags->"ReverseFA"],
    Cell[271825, 11444, 112, 3, 26, "Input",
      CellTags->"ReverseFA"]},
  "SA"->{
    Cell[271940, 11449, 42, 1, 32, "ObjectName",
      CellTags->"SA"],
    Cell[271985, 11452, 50, 1, 50, "Highlight",
      CellTags->"SA"],
    Cell[272038, 11455, 109, 4, 31, "Usage",
      CellTags->"SA"],
    Cell[272150, 11461, 231, 9, 24, "Notes",
      CellTags->"SA"],
    Cell[272384, 11472, 166, 5, 24, "Notes",
      CellTags->"SA"],
    Cell[272553, 11479, 79, 1, 24, "Notes",
      CellTags->"SA"],
    Cell[272635, 11482, 183, 7, 24, "Notes",
      CellTags->"SA"],
    Cell[272821, 11491, 303, 15, 24, "Notes",
      CellTags->"SA"],
    Cell[273127, 11508, 60, 1, 39, "ExampleSection",
      CellTags->"SA"],
    Cell[273190, 11511, 63, 1, 34, "ExampleText",
      CellTags->"SA"],
    Cell[273256, 11514, 212, 5, 42, "Input",
      CellTags->"SA"],
    Cell[273471, 11521, 141, 5, 58, "Input",
      CellTags->"SA"]},
  "SelectFA"->{
    Cell[273615, 11528, 54, 1, 32, "ObjectName",
      CellTags->"SelectFA"],
    Cell[273672, 11531, 53, 1, 50, "Highlight",
      CellTags->"SelectFA"],
    Cell[273728, 11534, 208, 8, 31, "Usage",
      CellTags->"SelectFA"],
    Cell[273939, 11544, 225, 9, 44, "Notes",
      CellTags->"SelectFA"],
    Cell[274167, 11555, 91, 1, 24, "Notes",
      CellTags->"SelectFA"],
    Cell[274261, 11558, 114, 4, 24, "Notes",
      CellTags->"SelectFA"],
    Cell[274378, 11564, 66, 1, 39, "ExampleSection",
      CellTags->"SelectFA"],
    Cell[274447, 11567, 254, 7, 34, "ExampleText",
      CellTags->"SelectFA"],
    Cell[274704, 11576, 298, 7, 58, "Input",
      InitializationCell->True,
      CellTags->"SelectFA"],
    Cell[275005, 11585, 179, 5, 26, "Input",
      InitializationCell->True,
      CellTags->"SelectFA"],
    Cell[275187, 11592, 121, 4, 26, "Input",
      InitializationCell->True,
      CellTags->"SelectFA"],
    Cell[275311, 11598, 138, 4, 34, "ExampleText",
      CellTags->"SelectFA"],
    Cell[275452, 11604, 143, 4, 26, "Input",
      InitializationCell->True,
      CellTags->"SelectFA"],
    Cell[275598, 11610, 131, 4, 26, "Input",
      InitializationCell->True,
      CellTags->"SelectFA"]},
  "SetAlphabet"->{
    Cell[275732, 11616, 60, 1, 32, "ObjectName",
      CellTags->"SetAlphabet"],
    Cell[275795, 11619, 56, 1, 50, "Highlight",
      CellTags->"SetAlphabet"],
    Cell[275854, 11622, 213, 8, 31, "Usage",
      CellTags->"SetAlphabet"],
    Cell[276070, 11632, 122, 4, 24, "Notes",
      CellTags->"SetAlphabet"],
    Cell[276195, 11638, 133, 6, 24, "Notes",
      CellTags->"SetAlphabet"],
    Cell[276331, 11646, 69, 1, 39, "ExampleSection",
      CellTags->"SetAlphabet"],
    Cell[276403, 11649, 209, 7, 36, "ExampleText",
      CellTags->"SetAlphabet"],
    Cell[276615, 11658, 116, 3, 26, "Input",
      CellTags->"SetAlphabet"],
    Cell[276734, 11663, 96, 3, 26, "Input",
      CellTags->"SetAlphabet"],
    Cell[276833, 11668, 214, 9, 34, "ExampleText",
      CellTags->"SetAlphabet"],
    Cell[277050, 11679, 154, 4, 42, "Input",
      CellTags->"SetAlphabet"],
    Cell[277207, 11685, 90, 1, 34, "ExampleText",
      CellTags->"SetAlphabet"],
    Cell[277300, 11688, 111, 3, 26, "Input",
      CellTags->"SetAlphabet"]},
  "SetFinalFA"->{
    Cell[277414, 11693, 58, 1, 32, "ObjectName",
      CellTags->"SetFinalFA"],
    Cell[277475, 11696, 55, 1, 50, "Highlight",
      CellTags->"SetFinalFA"],
    Cell[277533, 11699, 205, 8, 31, "Usage",
      CellTags->"SetFinalFA"],
    Cell[277741, 11709, 142, 4, 24, "Notes",
      CellTags->"SetFinalFA"],
    Cell[277886, 11715, 186, 6, 24, "Notes",
      CellTags->"SetFinalFA"],
    Cell[278075, 11723, 241, 6, 40, "Notes",
      CellTags->"SetFinalFA"],
    Cell[278319, 11731, 410, 17, 24, "Notes",
      CellTags->"SetFinalFA"],
    Cell[278732, 11750, 68, 1, 39, "ExampleSection",
      CellTags->"SetFinalFA"],
    Cell[278803, 11753, 293, 8, 50, "ExampleText",
      CellTags->"SetFinalFA"],
    Cell[279099, 11763, 248, 6, 42, "Input",
      CellTags->"SetFinalFA"],
    Cell[279350, 11771, 111, 3, 26, "Input",
      CellTags->"SetFinalFA"],
    Cell[279464, 11776, 205, 7, 34, "ExampleText",
      CellTags->"SetFinalFA"],
    Cell[279672, 11785, 106, 3, 26, "Input",
      CellTags->"SetFinalFA"],
    Cell[279781, 11790, 255, 8, 34, "ExampleText",
      CellTags->"SetFinalFA"],
    Cell[280039, 11800, 190, 4, 42, "Input",
      CellTags->"SetFinalFA"]},
  "SetInitialFA"->{
    Cell[280232, 11806, 62, 1, 32, "ObjectName",
      CellTags->"SetInitialFA"],
    Cell[280297, 11809, 57, 1, 50, "Highlight",
      CellTags->"SetInitialFA"],
    Cell[280357, 11812, 210, 8, 31, "Usage",
      CellTags->"SetInitialFA"],
    Cell[280570, 11822, 151, 5, 24, "Notes",
      CellTags->"SetInitialFA"],
    Cell[280724, 11829, 188, 6, 24, "Notes",
      CellTags->"SetInitialFA"],
    Cell[280915, 11837, 245, 6, 40, "Notes",
      CellTags->"SetInitialFA"],
    Cell[281163, 11845, 340, 15, 24, "Notes",
      CellTags->"SetInitialFA"],
    Cell[281506, 11862, 70, 1, 39, "ExampleSection",
      CellTags->"SetInitialFA"],
    Cell[281579, 11865, 265, 7, 50, "ExampleText",
      CellTags->"SetInitialFA"],
    Cell[281847, 11874, 216, 5, 42, "Input",
      CellTags->"SetInitialFA"],
    Cell[282066, 11881, 104, 3, 26, "Input",
      CellTags->"SetInitialFA"],
    Cell[282173, 11886, 107, 3, 26, "Input",
      CellTags->"SetInitialFA"],
    Cell[282283, 11891, 152, 4, 42, "Input",
      CellTags->"SetInitialFA"]},
  "SetInitialFinalFA"->{
    Cell[282438, 11897, 72, 1, 32, "ObjectName",
      CellTags->"SetInitialFinalFA"],
    Cell[282513, 11900, 62, 1, 50, "Highlight",
      CellTags->"SetInitialFinalFA"],
    Cell[282578, 11903, 276, 10, 47, "Usage",
      CellTags->"SetInitialFinalFA"],
    Cell[282857, 11915, 195, 7, 24, "Notes",
      CellTags->"SetInitialFinalFA"],
    Cell[283055, 11924, 193, 6, 24, "Notes",
      CellTags->"SetInitialFinalFA"],
    Cell[283251, 11932, 255, 6, 40, "Notes",
      CellTags->"SetInitialFinalFA"],
    Cell[283509, 11940, 404, 17, 24, "Notes",
      CellTags->"SetInitialFinalFA"],
    Cell[283916, 11959, 75, 1, 39, "ExampleSection",
      CellTags->"SetInitialFinalFA"],
    Cell[283994, 11962, 156, 4, 34, "ExampleText",
      CellTags->"SetInitialFinalFA"],
    Cell[284153, 11968, 222, 5, 42, "Input",
      CellTags->"SetInitialFinalFA"],
    Cell[284378, 11975, 128, 3, 26, "Input",
      CellTags->"SetInitialFinalFA"],
    Cell[284509, 11980, 113, 3, 26, "Input",
      CellTags->"SetInitialFinalFA"],
    Cell[284625, 11985, 247, 8, 34, "ExampleText",
      CellTags->"SetInitialFinalFA"],
    Cell[284875, 11995, 196, 4, 42, "Input",
      CellTags->"SetInitialFinalFA"],
    Cell[285074, 12001, 193, 4, 42, "Input",
      CellTags->"SetInitialFinalFA"]},
  "SetStates"->{
    Cell[285270, 12007, 56, 1, 32, "ObjectName",
      CellTags->"SetStates"],
    Cell[285329, 12010, 54, 1, 50, "Highlight",
      CellTags->"SetStates"],
    Cell[285386, 12013, 207, 8, 31, "Usage",
      CellTags->"SetStates"],
    Cell[285596, 12023, 122, 4, 24, "Notes",
      CellTags->"SetStates"],
    Cell[285721, 12029, 133, 6, 24, "Notes",
      CellTags->"SetStates"],
    Cell[285857, 12037, 67, 1, 39, "ExampleSection",
      CellTags->"SetStates"],
    Cell[285927, 12040, 144, 4, 34, "ExampleText",
      CellTags->"SetStates"],
    Cell[286074, 12046, 139, 3, 26, "Input",
      CellTags->"SetStates"],
    Cell[286216, 12051, 92, 3, 26, "Input",
      CellTags->"SetStates"],
    Cell[286311, 12056, 149, 4, 34, "ExampleText",
      CellTags->"SetStates"],
    Cell[286463, 12062, 153, 4, 42, "Input",
      CellTags->"SetStates"],
    Cell[286619, 12068, 71, 1, 34, "ExampleText",
      CellTags->"SetStates"],
    Cell[286693, 12071, 107, 3, 26, "Input",
      CellTags->"SetStates"]},
  "ShowAll"->{
    Cell[286803, 12076, 52, 1, 32, "ObjectName",
      CellTags->"ShowAll"],
    Cell[286858, 12079, 50, 1, 50, "Highlight",
      CellTags->"ShowAll"],
    Cell[286911, 12082, 152, 6, 31, "Usage",
      CellTags->"ShowAll"],
    Cell[287066, 12090, 100, 1, 24, "Notes",
      CellTags->"ShowAll"],
    Cell[287169, 12093, 132, 5, 24, "Notes",
      CellTags->"ShowAll"],
    Cell[287304, 12100, 174, 7, 24, "Notes",
      CellTags->"ShowAll"]},
  "ShowEdges"->{
    Cell[287481, 12109, 56, 1, 32, "ObjectName",
      CellTags->"ShowEdges"],
    Cell[287540, 12112, 52, 1, 50, "Highlight",
      CellTags->"ShowEdges"],
    Cell[287595, 12115, 154, 6, 31, "Usage",
      CellTags->"ShowEdges"],
    Cell[287752, 12123, 140, 4, 24, "Notes",
      CellTags->"ShowEdges"],
    Cell[287895, 12129, 176, 7, 24, "Notes",
      CellTags->"ShowEdges"]},
  "ShuffleProductFA"->{
    Cell[288074, 12138, 70, 1, 32, "ObjectName",
      CellTags->"ShuffleProductFA"],
    Cell[288147, 12141, 61, 1, 50, "Highlight",
      CellTags->"ShuffleProductFA"],
    Cell[288211, 12144, 282, 9, 47, "Usage",
      CellTags->"ShuffleProductFA"],
    Cell[288496, 12155, 256, 9, 44, "Notes",
      CellTags->"ShuffleProductFA"],
    Cell[288755, 12166, 107, 1, 24, "Notes",
      CellTags->"ShuffleProductFA"],
    Cell[288865, 12169, 171, 7, 24, "Notes",
      CellTags->"ShuffleProductFA"],
    Cell[289039, 12178, 74, 1, 39, "ExampleSection",
      CellTags->"ShuffleProductFA"],
    Cell[289116, 12181, 287, 9, 50, "ExampleText",
      CellTags->"ShuffleProductFA"],
    Cell[289406, 12192, 136, 5, 45, "Input",
      CellTags->"ShuffleProductFA"],
    Cell[289545, 12199, 123, 3, 26, "Input",
      CellTags->"ShuffleProductFA"],
    Cell[289671, 12204, 138, 3, 26, "Input",
      CellTags->"ShuffleProductFA"]},
  "SinkQDFA"->{
    Cell[289812, 12209, 54, 1, 32, "ObjectName",
      CellTags->"SinkQDFA"],
    Cell[289869, 12212, 53, 1, 50, "Highlight",
      CellTags->"SinkQDFA"],
    Cell[289925, 12215, 161, 6, 31, "Usage",
      CellTags->"SinkQDFA"],
    Cell[290089, 12223, 299, 9, 40, "Notes",
      CellTags->"SinkQDFA"],
    Cell[290391, 12234, 166, 7, 24, "Notes",
      CellTags->"SinkQDFA"],
    Cell[290560, 12243, 66, 1, 39, "ExampleSection",
      CellTags->"SinkQDFA"],
    Cell[290629, 12246, 109, 2, 34, "ExampleText",
      CellTags->"SinkQDFA"],
    Cell[290741, 12250, 177, 4, 42, "Input",
      CellTags->"SinkQDFA"],
    Cell[290921, 12256, 149, 6, 34, "ExampleText",
      CellTags->"SinkQDFA"],
    Cell[291073, 12264, 94, 3, 26, "Input",
      CellTags->"SinkQDFA"],
    Cell[291170, 12269, 139, 4, 34, "ExampleText",
      CellTags->"SinkQDFA"],
    Cell[291312, 12275, 151, 4, 26, "Input",
      CellTags->"SinkQDFA"]},
  "SizeOnly"->{
    Cell[291466, 12281, 54, 1, 32, "ObjectName",
      CellTags->"SizeOnly"],
    Cell[291523, 12284, 51, 1, 50, "Highlight",
      CellTags->"SizeOnly"],
    Cell[291577, 12287, 148, 5, 31, "Usage",
      CellTags->"SizeOnly"],
    Cell[291728, 12294, 294, 10, 40, "Notes",
      CellTags->"SizeOnly"],
    Cell[292025, 12306, 209, 7, 40, "Notes",
      CellTags->"SizeOnly"],
    Cell[292237, 12315, 279, 11, 24, "Notes",
      CellTags->"SizeOnly"],
    Cell[292519, 12328, 66, 1, 39, "ExampleSection",
      CellTags->"SizeOnly"],
    Cell[292588, 12331, 65, 1, 34, "ExampleText",
      CellTags->"SizeOnly"],
    Cell[292656, 12334, 213, 5, 42, "Input",
      CellTags->"SizeOnly"],
    Cell[292872, 12341, 136, 3, 26, "Input",
      CellTags->"SizeOnly"]},
  "Size"->{
    Cell[293011, 12346, 46, 1, 32, "ObjectName",
      CellTags->"Size"],
    Cell[293060, 12349, 49, 1, 50, "Highlight",
      CellTags->"Size"],
    Cell[293112, 12352, 126, 4, 31, "Usage",
      CellTags->"Size"],
    Cell[293241, 12358, 220, 9, 44, "Notes",
      CellTags->"Size"],
    Cell[293464, 12369, 262, 7, 40, "Notes",
      CellTags->"Size"],
    Cell[293729, 12378, 104, 5, 24, "Notes",
      CellTags->"Size"],
    Cell[293836, 12385, 62, 1, 39, "ExampleSection",
      CellTags->"Size"],
    Cell[293901, 12388, 88, 1, 34, "ExampleText",
      CellTags->"Size"],
    Cell[293992, 12391, 155, 4, 42, "Input",
      CellTags->"Size"],
    Cell[294150, 12397, 88, 1, 34, "ExampleText",
      CellTags->"Size"],
    Cell[294241, 12400, 131, 3, 26, "Input",
      CellTags->"Size"]},
  "Some"->{
    Cell[294375, 12405, 46, 1, 32, "ObjectName",
      CellTags->"Some"],
    Cell[294424, 12408, 47, 1, 50, "Highlight",
      CellTags->"Some"],
    Cell[294474, 12411, 141, 6, 31, "Usage",
      CellTags->"Some"],
    Cell[294618, 12419, 160, 7, 24, "Notes",
      CellTags->"Some"]},
  "SquareSA"->{
    Cell[294781, 12428, 54, 1, 32, "ObjectName",
      CellTags->"SquareSA"],
    Cell[294838, 12431, 53, 1, 50, "Highlight",
      CellTags->"SquareSA"],
    Cell[294894, 12434, 187, 6, 31, "Usage",
      CellTags->"SquareSA"],
    Cell[295084, 12442, 302, 10, 61, "Notes",
      CellTags->"SquareSA"],
    Cell[295389, 12454, 220, 8, 24, "Notes",
      CellTags->"SquareSA"],
    Cell[295612, 12464, 159, 5, 24, "Notes",
      CellTags->"SquareSA"],
    Cell[295774, 12471, 154, 5, 24, "Notes",
      CellTags->"SquareSA"],
    Cell[295931, 12478, 274, 8, 40, "Notes",
      CellTags->"SquareSA"],
    Cell[296208, 12488, 100, 1, 24, "Notes",
      CellTags->"SquareSA"],
    Cell[296311, 12491, 258, 11, 24, "Notes",
      CellTags->"SquareSA"],
    Cell[296572, 12504, 66, 1, 39, "ExampleSection",
      CellTags->"SquareSA"],
    Cell[296641, 12507, 227, 9, 34, "ExampleText",
      CellTags->"SquareSA"],
    Cell[296871, 12518, 213, 5, 42, "Input",
      CellTags->"SquareSA"],
    Cell[297087, 12525, 93, 3, 26, "Input",
      CellTags->"SquareSA"],
    Cell[297183, 12530, 562, 19, 66, "ExampleText",
      CellTags->"SquareSA"],
    Cell[297748, 12551, 160, 4, 42, "Input",
      CellTags->"SquareSA"],
    Cell[297911, 12557, 109, 3, 26, "Input",
      CellTags->"SquareSA"]},
  "StarFA"->{
    Cell[298023, 12562, 50, 1, 32, "ObjectName",
      CellTags->"StarFA"],
    Cell[298076, 12565, 51, 1, 50, "Highlight",
      CellTags->"StarFA"],
    Cell[298130, 12568, 196, 7, 31, "Usage",
      CellTags->"StarFA"],
    Cell[298329, 12577, 305, 10, 61, "Notes",
      CellTags->"StarFA"],
    Cell[298637, 12589, 176, 5, 24, "Notes",
      CellTags->"StarFA"],
    Cell[298816, 12596, 438, 19, 24, "Notes",
      CellTags->"StarFA"],
    Cell[299257, 12617, 64, 1, 39, "ExampleSection",
      CellTags->"StarFA"],
    Cell[299324, 12620, 189, 6, 34, "ExampleText",
      CellTags->"StarFA"],
    Cell[299516, 12628, 83, 2, 28, "Input",
      CellTags->"StarFA"],
    Cell[299602, 12632, 98, 3, 26, "Input",
      CellTags->"StarFA"],
    Cell[299703, 12637, 134, 3, 26, "Input",
      CellTags->"StarFA"]},
  "StateMergeDFA"->{
    Cell[299840, 12642, 64, 1, 32, "ObjectName",
      CellTags->"StateMergeDFA"],
    Cell[299907, 12645, 58, 1, 50, "Highlight",
      CellTags->"StateMergeDFA"],
    Cell[299968, 12648, 227, 6, 47, "Usage",
      CellTags->"StateMergeDFA"],
    Cell[300198, 12656, 350, 11, 79, "Notes",
      CellTags->"StateMergeDFA"],
    Cell[300551, 12669, 228, 5, 40, "Notes",
      CellTags->"StateMergeDFA"],
    Cell[300782, 12676, 187, 7, 24, "Notes",
      CellTags->"StateMergeDFA"],
    Cell[300972, 12685, 202, 7, 24, "Notes",
      CellTags->"StateMergeDFA"],
    Cell[301177, 12694, 71, 1, 39, "ExampleSection",
      CellTags->"StateMergeDFA"],
    Cell[301251, 12697, 242, 7, 34, "ExampleText",
      CellTags->"StateMergeDFA"],
    Cell[301496, 12706, 220, 5, 42, "Input",
      CellTags->"StateMergeDFA"],
    Cell[301719, 12713, 105, 1, 34, "ExampleText",
      CellTags->"StateMergeDFA"],
    Cell[301827, 12716, 112, 3, 26, "Input",
      CellTags->"StateMergeDFA"],
    Cell[301942, 12721, 110, 3, 26, "Input",
      CellTags->"StateMergeDFA"]},
  "States"->{
    Cell[302055, 12726, 50, 1, 32, "ObjectName",
      CellTags->"States"],
    Cell[302108, 12729, 51, 1, 50, "Highlight",
      CellTags->"States"],
    Cell[302162, 12732, 164, 6, 31, "Usage",
      CellTags->"States"],
    Cell[302329, 12740, 196, 6, 24, "Notes",
      CellTags->"States"],
    Cell[302528, 12748, 255, 11, 24, "Notes",
      CellTags->"States"],
    Cell[302786, 12761, 64, 1, 39, "ExampleSection",
      CellTags->"States"],
    Cell[302853, 12764, 140, 4, 34, "ExampleText",
      CellTags->"States"],
    Cell[302996, 12770, 130, 4, 42, "Input",
      CellTags->"States"],
    Cell[303129, 12776, 106, 2, 34, "ExampleText",
      CellTags->"States"],
    Cell[303238, 12780, 150, 4, 42, "Input",
      CellTags->"States"]},
  "SubsetQFA"->{
    Cell[303391, 12786, 56, 1, 32, "ObjectName",
      CellTags->"SubsetQFA"],
    Cell[303450, 12789, 54, 1, 50, "Highlight",
      CellTags->"SubsetQFA"],
    Cell[303507, 12792, 244, 8, 47, "Usage",
      CellTags->"SubsetQFA"],
    Cell[303754, 12802, 247, 9, 44, "Notes",
      CellTags->"SubsetQFA"],
    Cell[304004, 12813, 198, 6, 40, "Notes",
      CellTags->"SubsetQFA"],
    Cell[304205, 12821, 168, 7, 24, "Notes",
      CellTags->"SubsetQFA"],
    Cell[304376, 12830, 67, 1, 39, "ExampleSection",
      CellTags->"SubsetQFA"],
    Cell[304446, 12833, 125, 4, 34, "ExampleText",
      CellTags->"SubsetQFA"],
    Cell[304574, 12839, 155, 4, 42, "Input",
      CellTags->"SubsetQFA"],
    Cell[304732, 12845, 102, 3, 26, "Input",
      CellTags->"SubsetQFA"],
    Cell[304837, 12850, 103, 3, 26, "Input",
      CellTags->"SubsetQFA"]},
  "SubwordCounterDFA"->{
    Cell[304943, 12855, 72, 1, 32, "ObjectName",
      CellTags->"SubwordCounterDFA"],
    Cell[305018, 12858, 62, 1, 50, "Highlight",
      CellTags->"SubwordCounterDFA"],
    Cell[305083, 12861, 297, 10, 47, "Usage",
      CellTags->"SubwordCounterDFA"],
    Cell[305383, 12873, 301, 10, 61, "Notes",
      CellTags->"SubwordCounterDFA"],
    Cell[305687, 12885, 531, 17, 62, "Notes",
      CellTags->"SubwordCounterDFA"],
    Cell[306221, 12904, 192, 9, 24, "Notes",
      CellTags->"SubwordCounterDFA"],
    Cell[306416, 12915, 328, 9, 40, "Notes",
      CellTags->"SubwordCounterDFA"],
    Cell[306747, 12926, 170, 7, 24, "Notes",
      CellTags->"SubwordCounterDFA"],
    Cell[306920, 12935, 75, 1, 39, "ExampleSection",
      CellTags->"SubwordCounterDFA"],
    Cell[306998, 12938, 138, 4, 34, "ExampleText",
      CellTags->"SubwordCounterDFA"],
    Cell[307139, 12944, 229, 6, 58, "Input",
      CellTags->"SubwordCounterDFA"],
    Cell[307371, 12952, 203, 5, 42, "Input",
      CellTags->"SubwordCounterDFA"],
    Cell[307577, 12959, 210, 6, 34, "ExampleText",
      CellTags->"SubwordCounterDFA"],
    Cell[307790, 12967, 147, 3, 26, "Input",
      CellTags->"SubwordCounterDFA"],
    Cell[307940, 12972, 107, 3, 26, "Input",
      CellTags->"SubwordCounterDFA"],
    Cell[308050, 12977, 110, 3, 26, "Input",
      CellTags->"SubwordCounterDFA"]},
  "SubwordDFA"->{
    Cell[308163, 12982, 58, 1, 32, "ObjectName",
      CellTags->"SubwordDFA"],
    Cell[308224, 12985, 55, 1, 50, "Highlight",
      CellTags->"SubwordDFA"],
    Cell[308282, 12988, 234, 8, 47, "Usage",
      CellTags->"SubwordDFA"],
    Cell[308519, 12998, 250, 9, 44, "Notes",
      CellTags->"SubwordDFA"],
    Cell[308772, 13009, 142, 5, 24, "Notes",
      CellTags->"SubwordDFA"],
    Cell[308917, 13016, 178, 7, 24, "Notes",
      CellTags->"SubwordDFA"],
    Cell[309098, 13025, 68, 1, 39, "ExampleSection",
      CellTags->"SubwordDFA"],
    Cell[309169, 13028, 97, 1, 34, "ExampleText",
      CellTags->"SubwordDFA"],
    Cell[309269, 13031, 132, 3, 26, "Input",
      CellTags->"SubwordDFA"],
    Cell[309404, 13036, 93, 3, 26, "Input",
      CellTags->"SubwordDFA"],
    Cell[309500, 13041, 105, 3, 26, "Input",
      CellTags->"SubwordDFA"],
    Cell[309608, 13046, 113, 3, 26, "Input",
      CellTags->"SubwordDFA"]},
  "SumFA"->{
    Cell[309724, 13051, 48, 1, 32, "ObjectName",
      CellTags->"SumFA"],
    Cell[309775, 13054, 50, 1, 50, "Highlight",
      CellTags->"SumFA"],
    Cell[309828, 13057, 199, 8, 31, "Usage",
      CellTags->"SumFA"],
    Cell[310030, 13067, 245, 9, 44, "Notes",
      CellTags->"SumFA"],
    Cell[310278, 13078, 146, 5, 24, "Notes",
      CellTags->"SumFA"],
    Cell[310427, 13085, 187, 5, 40, "Notes",
      CellTags->"SumFA"],
    Cell[310617, 13092, 142, 4, 24, "Notes",
      CellTags->"SumFA"],
    Cell[310762, 13098, 350, 15, 24, "Notes",
      CellTags->"SumFA"],
    Cell[311115, 13115, 63, 1, 39, "ExampleSection",
      CellTags->"SumFA"],
    Cell[311181, 13118, 142, 4, 34, "ExampleText",
      CellTags->"SumFA"],
    Cell[311326, 13124, 132, 6, 62, "Input",
      CellTags->"SumFA"],
    Cell[311461, 13132, 100, 3, 26, "Input",
      CellTags->"SumFA"],
    Cell[311564, 13137, 140, 4, 34, "ExampleText",
      CellTags->"SumFA"],
    Cell[311707, 13143, 105, 3, 26, "Input",
      CellTags->"SumFA"],
    Cell[311815, 13148, 99, 3, 26, "Input",
      CellTags->"SumFA"]},
  "SwitchTransitionFA"->{
    Cell[311917, 13153, 74, 1, 32, "ObjectName",
      CellTags->"SwitchTransitionFA"],
    Cell[311994, 13156, 63, 1, 50, "Highlight",
      CellTags->"SwitchTransitionFA"],
    Cell[312060, 13159, 251, 10, 31, "Usage",
      CellTags->"SwitchTransitionFA"],
    Cell[312314, 13171, 191, 7, 24, "Notes",
      CellTags->"SwitchTransitionFA"],
    Cell[312508, 13180, 245, 7, 40, "Notes",
      CellTags->"SwitchTransitionFA"],
    Cell[312756, 13189, 296, 11, 24, "Notes",
      CellTags->"SwitchTransitionFA"],
    Cell[313055, 13202, 76, 1, 39, "ExampleSection",
      CellTags->"SwitchTransitionFA"],
    Cell[313134, 13205, 200, 5, 50, "ExampleText",
      CellTags->"SwitchTransitionFA"],
    Cell[313337, 13212, 223, 5, 42, "Input",
      CellTags->"SwitchTransitionFA"],
    Cell[313563, 13219, 129, 3, 26, "Input",
      CellTags->"SwitchTransitionFA"],
    Cell[313695, 13224, 114, 3, 26, "Input",
      CellTags->"SwitchTransitionFA"]},
  "TheInitial"->{
    Cell[313812, 13229, 58, 1, 32, "ObjectName",
      CellTags->"TheInitial"],
    Cell[313873, 13232, 55, 1, 50, "Highlight",
      CellTags->"TheInitial"],
    Cell[313931, 13235, 186, 6, 31, "Usage",
      CellTags->"TheInitial"],
    Cell[314120, 13243, 180, 7, 24, "Notes",
      CellTags->"TheInitial"],
    Cell[314303, 13252, 68, 1, 39, "ExampleSection",
      CellTags->"TheInitial"],
    Cell[314374, 13255, 128, 4, 34, "ExampleText",
      CellTags->"TheInitial"],
    Cell[314505, 13261, 102, 2, 28, "Input",
      CellTags->"TheInitial"],
    Cell[314610, 13265, 127, 4, 42, "Input",
      CellTags->"TheInitial"]},
  "Thompson"->{
    Cell[314740, 13271, 54, 1, 32, "ObjectName",
      CellTags->"Thompson"],
    Cell[314797, 13274, 51, 1, 50, "Highlight",
      CellTags->"Thompson"],
    Cell[314851, 13277, 156, 6, 31, "Usage",
      CellTags->"Thompson"],
    Cell[315010, 13285, 213, 7, 40, "Notes",
      CellTags->"Thompson"],
    Cell[315226, 13294, 183, 7, 24, "Notes",
      CellTags->"Thompson"]},
  "ToCharacters"->{
    Cell[315412, 13303, 62, 1, 32, "ObjectName",
      CellTags->"ToCharacters"],
    Cell[315477, 13306, 57, 1, 50, "Highlight",
      CellTags->"ToCharacters"],
    Cell[315537, 13309, 160, 4, 31, "Usage",
      CellTags->"ToCharacters"],
    Cell[315700, 13315, 164, 5, 24, "Notes",
      CellTags->"ToCharacters"],
    Cell[315867, 13322, 86, 1, 24, "Notes",
      CellTags->"ToCharacters"],
    Cell[315956, 13325, 70, 1, 39, "ExampleSection",
      CellTags->"ToCharacters"],
    Cell[316029, 13328, 124, 4, 34, "ExampleText",
      CellTags->"ToCharacters"],
    Cell[316156, 13334, 111, 5, 45, "Input",
      CellTags->"ToCharacters"],
    Cell[316270, 13341, 111, 3, 26, "Input",
      CellTags->"ToCharacters"]},
  "ToDFA"->{
    Cell[316384, 13346, 48, 1, 32, "ObjectName",
      CellTags->"ToDFA"],
    Cell[316435, 13349, 50, 1, 50, "Highlight",
      CellTags->"ToDFA"],
    Cell[316488, 13352, 142, 4, 31, "Usage",
      CellTags->"ToDFA"],
    Cell[316633, 13358, 406, 12, 97, "Notes",
      CellTags->"ToDFA"],
    Cell[317042, 13372, 212, 8, 24, "Notes",
      CellTags->"ToDFA"],
    Cell[317257, 13382, 224, 8, 24, "Notes",
      CellTags->"ToDFA"],
    Cell[317484, 13392, 209, 5, 40, "Notes",
      CellTags->"ToDFA"],
    Cell[317696, 13399, 278, 8, 40, "Notes",
      CellTags->"ToDFA"],
    Cell[317977, 13409, 150, 5, 24, "Notes",
      CellTags->"ToDFA"],
    Cell[318130, 13416, 161, 4, 40, "Notes",
      CellTags->"ToDFA"],
    Cell[318294, 13422, 316, 15, 24, "Notes",
      CellTags->"ToDFA"],
    Cell[318613, 13439, 63, 1, 39, "ExampleSection",
      CellTags->"ToDFA"],
    Cell[318679, 13442, 147, 4, 50, "ExampleText",
      CellTags->"ToDFA"],
    Cell[318829, 13448, 146, 4, 42, "Input",
      CellTags->"ToDFA"],
    Cell[318978, 13454, 100, 3, 26, "Input",
      CellTags->"ToDFA"],
    Cell[319081, 13459, 260, 7, 50, "ExampleText",
      CellTags->"ToDFA"],
    Cell[319344, 13468, 200, 5, 58, "Input",
      CellTags->"ToDFA"]},
  "ToEquationsDFA"->{
    Cell[319547, 13475, 66, 1, 32, "ObjectName",
      CellTags->"ToEquationsDFA"],
    Cell[319616, 13478, 59, 1, 50, "Highlight",
      CellTags->"ToEquationsDFA"],
    Cell[319678, 13481, 175, 6, 31, "Usage",
      CellTags->"ToEquationsDFA"],
    Cell[319856, 13489, 173, 4, 40, "Notes",
      CellTags->"ToEquationsDFA"],
    Cell[320032, 13495, 267, 11, 24, "Notes",
      CellTags->"ToEquationsDFA"],
    Cell[320302, 13508, 72, 1, 39, "ExampleSection",
      CellTags->"ToEquationsDFA"],
    Cell[320377, 13511, 239, 7, 50, "ExampleText",
      CellTags->"ToEquationsDFA"],
    Cell[320619, 13520, 106, 2, 28, "Input",
      CellTags->"ToEquationsDFA"],
    Cell[320728, 13524, 151, 4, 42, "Input",
      CellTags->"ToEquationsDFA"],
    Cell[320882, 13530, 82, 1, 34, "ExampleText",
      CellTags->"ToEquationsDFA"],
    Cell[320967, 13533, 194, 4, 42, "Input",
      CellTags->"ToEquationsDFA"]},
  "ToFA"->{
    Cell[321164, 13539, 46, 1, 32, "ObjectName",
      CellTags->"ToFA"],
    Cell[321213, 13542, 49, 1, 50, "Highlight",
      CellTags->"ToFA"],
    Cell[321265, 13545, 152, 6, 31, "Usage",
      CellTags->"ToFA"],
    Cell[321420, 13553, 244, 9, 44, "Notes",
      CellTags->"ToFA"],
    Cell[321667, 13564, 175, 6, 24, "Notes",
      CellTags->"ToFA"],
    Cell[321845, 13572, 123, 4, 24, "Notes",
      CellTags->"ToFA"],
    Cell[321971, 13578, 303, 15, 24, "Notes",
      CellTags->"ToFA"],
    Cell[322277, 13595, 62, 1, 39, "ExampleSection",
      CellTags->"ToFA"],
    Cell[322342, 13598, 69, 1, 34, "ExampleText",
      CellTags->"ToFA"],
    Cell[322414, 13601, 187, 5, 42, "Input",
      CellTags->"ToFA"],
    Cell[322604, 13608, 69, 2, 28, "Input",
      CellTags->"ToFA"],
    Cell[322676, 13612, 97, 3, 26, "Input",
      CellTags->"ToFA"],
    Cell[322776, 13617, 75, 1, 34, "ExampleText",
      CellTags->"ToFA"],
    Cell[322854, 13620, 91, 3, 26, "Input",
      CellTags->"ToFA"]},
  "ToFactorFA"->{
    Cell[322948, 13625, 58, 1, 32, "ObjectName",
      CellTags->"ToFactorFA"],
    Cell[323009, 13628, 55, 1, 50, "Highlight",
      CellTags->"ToFactorFA"],
    Cell[323067, 13631, 258, 9, 47, "Usage",
      CellTags->"ToFactorFA"],
    Cell[323328, 13642, 332, 12, 61, "Notes",
      CellTags->"ToFactorFA"],
    Cell[323663, 13656, 458, 15, 112, "Notes",
      CellTags->"ToFactorFA"],
    Cell[324124, 13673, 310, 9, 40, "Notes",
      CellTags->"ToFactorFA"],
    Cell[324437, 13684, 413, 13, 40, "Notes",
      CellTags->"ToFactorFA"],
    Cell[324853, 13699, 357, 11, 40, "Notes",
      CellTags->"ToFactorFA"],
    Cell[325213, 13712, 164, 4, 40, "Notes",
      CellTags->"ToFactorFA"],
    Cell[325380, 13718, 355, 15, 24, "Notes",
      CellTags->"ToFactorFA"],
    Cell[325738, 13735, 68, 1, 39, "ExampleSection",
      CellTags->"ToFactorFA"],
    Cell[325809, 13738, 293, 7, 50, "ExampleText",
      CellTags->"ToFactorFA"],
    Cell[326105, 13747, 198, 5, 42, "Input",
      CellTags->"ToFactorFA"],
    Cell[326306, 13754, 268, 5, 58, "Input",
      CellTags->"ToFactorFA"],
    Cell[326577, 13761, 126, 4, 34, "ExampleText",
      CellTags->"ToFactorFA"],
    Cell[326706, 13767, 249, 6, 74, "Input",
      CellTags->"ToFactorFA"],
    Cell[326958, 13775, 220, 7, 50, "ExampleText",
      CellTags->"ToFactorFA"],
    Cell[327181, 13784, 318, 6, 74, "Input",
      CellTags->"ToFactorFA"],
    Cell[327502, 13792, 93, 1, 34, "ExampleText",
      CellTags->"ToFactorFA"],
    Cell[327598, 13795, 249, 6, 74, "Input",
      CellTags->"ToFactorFA"],
    Cell[327850, 13803, 167, 4, 50, "ExampleText",
      CellTags->"ToFactorFA"],
    Cell[328020, 13809, 299, 7, 74, "Input",
      CellTags->"ToFactorFA"]},
  "ToFrontierDFA"->{
    Cell[328322, 13818, 64, 1, 32, "ObjectName",
      CellTags->"ToFrontierDFA"],
    Cell[328389, 13821, 58, 1, 50, "Highlight",
      CellTags->"ToFrontierDFA"],
    Cell[328450, 13824, 222, 8, 31, "Usage",
      CellTags->"ToFrontierDFA"],
    Cell[328675, 13834, 233, 9, 44, "Notes",
      CellTags->"ToFrontierDFA"],
    Cell[328911, 13845, 191, 7, 24, "Notes",
      CellTags->"ToFrontierDFA"],
    Cell[329105, 13854, 416, 14, 40, "Notes",
      CellTags->"ToFrontierDFA"],
    Cell[329524, 13870, 239, 10, 24, "Notes",
      CellTags->"ToFrontierDFA"],
    Cell[329766, 13882, 71, 1, 39, "ExampleSection",
      CellTags->"ToFrontierDFA"],
    Cell[329840, 13885, 251, 6, 50, "ExampleText",
      CellTags->"ToFrontierDFA"],
    Cell[330094, 13893, 149, 4, 26, "Input",
      CellTags->"ToFrontierDFA"],
    Cell[330246, 13899, 230, 6, 58, "Input",
      CellTags->"ToFrontierDFA"],
    Cell[330479, 13907, 191, 5, 50, "ExampleText",
      CellTags->"ToFrontierDFA"],
    Cell[330673, 13914, 257, 7, 58, "Input",
      CellTags->"ToFrontierDFA"],
    Cell[330933, 13923, 257, 7, 58, "Input",
      CellTags->"ToFrontierDFA"],
    Cell[331193, 13932, 143, 4, 34, "ExampleText",
      CellTags->"ToFrontierDFA"],
    Cell[331339, 13938, 173, 4, 42, "Input",
      CellTags->"ToFrontierDFA"]},
  "ToGeneratorsFA"->{
    Cell[331515, 13944, 66, 1, 32, "ObjectName",
      CellTags->"ToGeneratorsFA"],
    Cell[331584, 13947, 59, 1, 50, "Highlight",
      CellTags->"ToGeneratorsFA"],
    Cell[331646, 13950, 216, 5, 47, "Usage",
      CellTags->"ToGeneratorsFA"],
    Cell[331865, 13957, 143, 4, 24, "Notes",
      CellTags->"ToGeneratorsFA"],
    Cell[332011, 13963, 269, 11, 24, "Notes",
      CellTags->"ToGeneratorsFA"],
    Cell[332283, 13976, 72, 1, 39, "ExampleSection",
      CellTags->"ToGeneratorsFA"],
    Cell[332358, 13979, 92, 1, 34, "ExampleText",
      CellTags->"ToGeneratorsFA"],
    Cell[332453, 13982, 197, 5, 42, "Input",
      CellTags->"ToGeneratorsFA"],
    Cell[332653, 13989, 105, 3, 26, "Input",
      CellTags->"ToGeneratorsFA"],
    Cell[332761, 13994, 94, 1, 34, "ExampleText",
      CellTags->"ToGeneratorsFA"],
    Cell[332858, 13997, 146, 4, 42, "Input",
      CellTags->"ToGeneratorsFA"],
    Cell[333007, 14003, 101, 3, 26, "Input",
      CellTags->"ToGeneratorsFA"]},
  "ToGraphFA"->{
    Cell[333111, 14008, 56, 1, 32, "ObjectName",
      CellTags->"ToGraphFA"],
    Cell[333170, 14011, 54, 1, 50, "Highlight",
      CellTags->"ToGraphFA"],
    Cell[333227, 14014, 206, 5, 47, "Usage",
      CellTags->"ToGraphFA"],
    Cell[333436, 14021, 303, 10, 61, "Notes",
      CellTags->"ToGraphFA"],
    Cell[333742, 14033, 140, 5, 24, "Notes",
      CellTags->"ToGraphFA"],
    Cell[333885, 14040, 197, 7, 24, "Notes",
      CellTags->"ToGraphFA"],
    Cell[334085, 14049, 67, 1, 39, "ExampleSection",
      CellTags->"ToGraphFA"],
    Cell[334155, 14052, 86, 1, 34, "ExampleText",
      CellTags->"ToGraphFA"],
    Cell[334244, 14055, 215, 5, 42, "Input",
      CellTags->"ToGraphFA"],
    Cell[334462, 14062, 104, 3, 26, "Input",
      CellTags->"ToGraphFA"],
    Cell[334569, 14067, 76, 1, 34, "ExampleText",
      CellTags->"ToGraphFA"],
    Cell[334648, 14070, 113, 3, 26, "Input",
      CellTags->"ToGraphFA"]},
  "ToKernelFA"->{
    Cell[334764, 14075, 58, 1, 32, "ObjectName",
      CellTags->"ToKernelFA"],
    Cell[334825, 14078, 55, 1, 50, "Highlight",
      CellTags->"ToKernelFA"],
    Cell[334883, 14081, 233, 8, 47, "Usage",
      CellTags->"ToKernelFA"],
    Cell[335119, 14091, 339, 11, 79, "Notes",
      CellTags->"ToKernelFA"],
    Cell[335461, 14104, 301, 12, 24, "Notes",
      CellTags->"ToKernelFA"],
    Cell[335765, 14118, 317, 13, 24, "Notes",
      CellTags->"ToKernelFA"],
    Cell[336085, 14133, 156, 5, 24, "Notes",
      CellTags->"ToKernelFA"],
    Cell[336244, 14140, 347, 13, 40, "Notes",
      CellTags->"ToKernelFA"],
    Cell[336594, 14155, 153, 7, 24, "Notes",
      CellTags->"ToKernelFA"],
    Cell[336750, 14164, 68, 1, 39, "ExampleSection",
      CellTags->"ToKernelFA"],
    Cell[336821, 14167, 120, 4, 34, "ExampleText",
      CellTags->"ToKernelFA"],
    Cell[336944, 14173, 221, 5, 42, "Input",
      CellTags->"ToKernelFA"],
    Cell[337168, 14180, 126, 4, 42, "Input",
      CellTags->"ToKernelFA"],
    Cell[337297, 14186, 162, 4, 50, "ExampleText",
      CellTags->"ToKernelFA"],
    Cell[337462, 14192, 129, 3, 26, "Input",
      CellTags->"ToKernelFA"],
    Cell[337594, 14197, 111, 2, 34, "ExampleText",
      CellTags->"ToKernelFA"],
    Cell[337708, 14201, 131, 4, 42, "Input",
      CellTags->"ToKernelFA"],
    Cell[337842, 14207, 214, 5, 50, "ExampleText",
      CellTags->"ToKernelFA"],
    Cell[338059, 14214, 208, 6, 58, "Input",
      CellTags->"ToKernelFA"]},
  "ToMatrix"->{
    Cell[338270, 14222, 54, 1, 32, "ObjectName",
      CellTags->"ToMatrix"],
    Cell[338327, 14225, 51, 1, 50, "Highlight",
      CellTags->"ToMatrix"],
    Cell[338381, 14228, 147, 6, 31, "Usage",
      CellTags->"ToMatrix"],
    Cell[338531, 14236, 138, 4, 24, "Notes",
      CellTags->"ToMatrix"],
    Cell[338672, 14242, 163, 7, 24, "Notes",
      CellTags->"ToMatrix"]},
  "ToNFA"->{
    Cell[338838, 14251, 48, 1, 32, "ObjectName",
      CellTags->"ToNFA"],
    Cell[338889, 14254, 50, 1, 50, "Highlight",
      CellTags->"ToNFA"],
    Cell[338942, 14257, 150, 6, 31, "Usage",
      CellTags->"ToNFA"],
    Cell[339095, 14265, 117, 4, 24, "Notes",
      CellTags->"ToNFA"],
    Cell[339215, 14271, 317, 15, 24, "Notes",
      CellTags->"ToNFA"],
    Cell[339535, 14288, 63, 1, 39, "ExampleSection",
      CellTags->"ToNFA"],
    Cell[339601, 14291, 86, 1, 34, "ExampleText",
      CellTags->"ToNFA"],
    Cell[339690, 14294, 215, 5, 42, "Input",
      CellTags->"ToNFA"],
    Cell[339908, 14301, 88, 3, 26, "Input",
      CellTags->"ToNFA"],
    Cell[339999, 14306, 112, 3, 26, "Input",
      CellTags->"ToNFA"]},
  "ToRegularExpressionFA"->{
    Cell[340114, 14311, 80, 1, 32, "ObjectName",
      CellTags->"ToRegularExpressionFA"],
    Cell[340197, 14314, 66, 1, 50, "Highlight",
      CellTags->"ToRegularExpressionFA"],
    Cell[340266, 14317, 200, 5, 47, "Usage",
      CellTags->"ToRegularExpressionFA"],
    Cell[340469, 14324, 290, 10, 61, "Notes",
      CellTags->"ToRegularExpressionFA"],
    Cell[340762, 14336, 223, 6, 40, "Notes",
      CellTags->"ToRegularExpressionFA"],
    Cell[340988, 14344, 131, 4, 24, "Notes",
      CellTags->"ToRegularExpressionFA"],
    Cell[341122, 14350, 270, 11, 24, "Notes",
      CellTags->"ToRegularExpressionFA"],
    Cell[341395, 14363, 79, 1, 39, "ExampleSection",
      CellTags->"ToRegularExpressionFA"],
    Cell[341477, 14366, 93, 1, 34, "ExampleText",
      CellTags->"ToRegularExpressionFA"],
    Cell[341573, 14369, 226, 5, 42, "Input",
      CellTags->"ToRegularExpressionFA"],
    Cell[341802, 14376, 120, 3, 26, "Input",
      CellTags->"ToRegularExpressionFA"],
    Cell[341925, 14381, 270, 7, 42, "Input",
      CellTags->"ToRegularExpressionFA"],
    Cell[342198, 14390, 173, 4, 26, "Input",
      CellTags->"ToRegularExpressionFA"],
    Cell[342374, 14396, 256, 7, 34, "ExampleText",
      CellTags->"ToRegularExpressionFA"]},
  "ToSA"->{
    Cell[342633, 14405, 46, 1, 32, "ObjectName",
      CellTags->"ToSA"],
    Cell[342682, 14408, 49, 1, 50, "Highlight",
      CellTags->"ToSA"],
    Cell[342734, 14411, 154, 4, 31, "Usage",
      CellTags->"ToSA"],
    Cell[342891, 14417, 244, 9, 44, "Notes",
      CellTags->"ToSA"],
    Cell[343138, 14428, 127, 4, 24, "Notes",
      CellTags->"ToSA"],
    Cell[343268, 14434, 137, 4, 24, "Notes",
      CellTags->"ToSA"],
    Cell[343408, 14440, 317, 15, 24, "Notes",
      CellTags->"ToSA"],
    Cell[343728, 14457, 62, 1, 39, "ExampleSection",
      CellTags->"ToSA"],
    Cell[343793, 14460, 89, 1, 34, "ExampleText",
      CellTags->"ToSA"],
    Cell[343885, 14463, 82, 2, 28, "Input",
      CellTags->"ToSA"],
    Cell[343970, 14467, 93, 3, 26, "Input",
      CellTags->"ToSA"],
    Cell[344066, 14472, 143, 4, 34, "ExampleText",
      CellTags->"ToSA"],
    Cell[344212, 14478, 142, 4, 42, "Input",
      CellTags->"ToSA"],
    Cell[344357, 14484, 75, 1, 34, "ExampleText",
      CellTags->"ToSA"],
    Cell[344435, 14487, 130, 3, 26, "Input",
      CellTags->"ToSA"],
    Cell[344568, 14492, 93, 3, 26, "Input",
      CellTags->"ToSA"]},
  "ToSeries"->{
    Cell[344664, 14497, 54, 1, 32, "ObjectName",
      CellTags->"ToSeries"],
    Cell[344721, 14500, 53, 1, 50, "Highlight",
      CellTags->"ToSeries"],
    Cell[344777, 14503, 223, 7, 47, "Usage",
      CellTags->"ToSeries"],
    Cell[345003, 14512, 141, 5, 24, "Notes",
      CellTags->"ToSeries"],
    Cell[345147, 14519, 76, 1, 24, "Notes",
      CellTags->"ToSeries"],
    Cell[345226, 14522, 256, 11, 24, "Notes",
      CellTags->"ToSeries"],
    Cell[345485, 14535, 66, 1, 39, "ExampleSection",
      CellTags->"ToSeries"],
    Cell[345554, 14538, 62, 1, 34, "ExampleText",
      CellTags->"ToSeries"],
    Cell[345619, 14541, 89, 2, 28, "Input",
      CellTags->"ToSeries"],
    Cell[345711, 14545, 132, 3, 26, "Input",
      CellTags->"ToSeries"],
    Cell[345846, 14550, 94, 3, 26, "Input",
      CellTags->"ToSeries"]},
  "ToTransitions"->{
    Cell[345943, 14555, 64, 1, 32, "ObjectName",
      CellTags->"ToTransitions"],
    Cell[346010, 14558, 58, 1, 50, "Highlight",
      CellTags->"ToTransitions"],
    Cell[346071, 14561, 266, 7, 47, "Usage",
      CellTags->"ToTransitions"],
    Cell[346340, 14570, 117, 4, 24, "Notes",
      CellTags->"ToTransitions"],
    Cell[346460, 14576, 71, 1, 39, "ExampleSection",
      CellTags->"ToTransitions"],
    Cell[346534, 14579, 163, 4, 50, "ExampleText",
      CellTags->"ToTransitions"],
    Cell[346700, 14585, 114, 2, 28, "Input",
      CellTags->"ToTransitions"]},
  "TransferSequenceFA"->{
    Cell[346817, 14589, 74, 1, 32, "ObjectName",
      CellTags->"TransferSequenceFA"],
    Cell[346894, 14592, 63, 1, 50, "Highlight",
      CellTags->"TransferSequenceFA"],
    Cell[346960, 14595, 495, 20, 47, "Usage",
      CellTags->"TransferSequenceFA"],
    Cell[347458, 14617, 171, 7, 24, "Notes",
      CellTags->"TransferSequenceFA"],
    Cell[347632, 14626, 318, 9, 40, "Notes",
      CellTags->"TransferSequenceFA"],
    Cell[347953, 14637, 76, 1, 39, "ExampleSection",
      CellTags->"TransferSequenceFA"],
    Cell[348032, 14640, 90, 1, 34, "ExampleText",
      CellTags->"TransferSequenceFA"],
    Cell[348125, 14643, 208, 5, 42, "Input",
      CellTags->"TransferSequenceFA"],
    Cell[348336, 14650, 100, 2, 28, "Input",
      CellTags->"TransferSequenceFA"],
    Cell[348439, 14654, 134, 2, 28, "Input",
      CellTags->"TransferSequenceFA"]},
  "TransitionFunctionFA"->{
    Cell[348576, 14658, 78, 1, 32, "ObjectName",
      CellTags->"TransitionFunctionFA"],
    Cell[348657, 14661, 65, 1, 50, "Highlight",
      CellTags->"TransitionFunctionFA"],
    Cell[348725, 14664, 232, 8, 31, "Usage",
      CellTags->"TransitionFunctionFA"],
    Cell[348960, 14674, 236, 9, 44, "Notes",
      CellTags->"TransitionFunctionFA"],
    Cell[349199, 14685, 120, 5, 24, "Notes",
      CellTags->"TransitionFunctionFA"],
    Cell[349322, 14692, 200, 7, 24, "Notes",
      CellTags->"TransitionFunctionFA"],
    Cell[349525, 14701, 293, 8, 40, "Notes",
      CellTags->"TransitionFunctionFA"],
    Cell[349821, 14711, 288, 11, 24, "Notes",
      CellTags->"TransitionFunctionFA"],
    Cell[350112, 14724, 78, 1, 39, "ExampleSection",
      CellTags->"TransitionFunctionFA"],
    Cell[350193, 14727, 108, 1, 34, "ExampleText",
      CellTags->"TransitionFunctionFA"],
    Cell[350304, 14730, 214, 5, 58, "Input",
      CellTags->"TransitionFunctionFA"],
    Cell[350521, 14737, 107, 1, 34, "ExampleText",
      CellTags->"TransitionFunctionFA"],
    Cell[350631, 14740, 243, 6, 74, "Input",
      CellTags->"TransitionFunctionFA"],
    Cell[350877, 14748, 100, 1, 34, "ExampleText",
      CellTags->"TransitionFunctionFA"],
    Cell[350980, 14751, 214, 5, 58, "Input",
      CellTags->"TransitionFunctionFA"]},
  "TransitionMatrixFA"->{
    Cell[351197, 14758, 74, 1, 32, "ObjectName",
      CellTags->"TransitionMatrixFA"],
    Cell[351274, 14761, 63, 1, 50, "Highlight",
      CellTags->"TransitionMatrixFA"],
    Cell[351340, 14764, 281, 9, 47, "Usage",
      CellTags->"TransitionMatrixFA"],
    Cell[351624, 14775, 296, 10, 61, "Notes",
      CellTags->"TransitionMatrixFA"],
    Cell[351923, 14787, 118, 5, 24, "Notes",
      CellTags->"TransitionMatrixFA"],
    Cell[352044, 14794, 229, 7, 40, "Notes",
      CellTags->"TransitionMatrixFA"],
    Cell[352276, 14803, 244, 8, 24, "Notes",
      CellTags->"TransitionMatrixFA"],
    Cell[352523, 14813, 414, 13, 40, "Notes",
      CellTags->"TransitionMatrixFA"],
    Cell[352940, 14828, 197, 6, 24, "Notes",
      CellTags->"TransitionMatrixFA"],
    Cell[353140, 14836, 390, 14, 40, "Notes",
      CellTags->"TransitionMatrixFA"],
    Cell[353533, 14852, 76, 1, 39, "ExampleSection",
      CellTags->"TransitionMatrixFA"],
    Cell[353612, 14855, 185, 5, 34, "ExampleText",
      CellTags->"TransitionMatrixFA"],
    Cell[353800, 14862, 167, 4, 42, "Input",
      CellTags->"TransitionMatrixFA"],
    Cell[353970, 14868, 116, 3, 26, "Input",
      CellTags->"TransitionMatrixFA"],
    Cell[354089, 14873, 118, 3, 26, "Input",
      CellTags->"TransitionMatrixFA"],
    Cell[354210, 14878, 130, 5, 34, "ExampleText",
      CellTags->"TransitionMatrixFA"],
    Cell[354343, 14885, 170, 4, 42, "Input",
      CellTags->"TransitionMatrixFA"]},
  "TransitionsToMatrix"->{
    Cell[354516, 14891, 76, 1, 32, "ObjectName",
      CellTags->"TransitionsToMatrix"],
    Cell[354595, 14894, 64, 1, 50, "Highlight",
      CellTags->"TransitionsToMatrix"],
    Cell[354662, 14897, 386, 14, 47, "Usage",
      CellTags->"TransitionsToMatrix"],
    Cell[355051, 14913, 244, 9, 44, "Notes",
      CellTags->"TransitionsToMatrix"],
    Cell[355298, 14924, 119, 5, 24, "Notes",
      CellTags->"TransitionsToMatrix"],
    Cell[355420, 14931, 145, 5, 24, "Notes",
      CellTags->"TransitionsToMatrix"],
    Cell[355568, 14938, 253, 5, 40, "Notes",
      CellTags->"TransitionsToMatrix"],
    Cell[355824, 14945, 304, 12, 24, "Notes",
      CellTags->"TransitionsToMatrix"],
    Cell[356131, 14959, 77, 1, 39, "ExampleSection",
      CellTags->"TransitionsToMatrix"],
    Cell[356211, 14962, 153, 4, 34, "ExampleText",
      CellTags->"TransitionsToMatrix"],
    Cell[356367, 14968, 179, 4, 26, "Input",
      CellTags->"TransitionsToMatrix"],
    Cell[356549, 14974, 140, 3, 26, "Input",
      CellTags->"TransitionsToMatrix"],
    Cell[356692, 14979, 185, 6, 34, "ExampleText",
      CellTags->"TransitionsToMatrix"],
    Cell[356880, 14987, 298, 5, 58, "Input",
      CellTags->"TransitionsToMatrix"],
    Cell[357181, 14994, 171, 4, 26, "Input",
      CellTags->"TransitionsToMatrix"]},
  "Transitions"->{
    Cell[357355, 15000, 60, 1, 32, "ObjectName",
      CellTags->"Transitions"],
    Cell[357418, 15003, 56, 1, 50, "Highlight",
      CellTags->"Transitions"],
    Cell[357477, 15006, 214, 5, 47, "Usage",
      CellTags->"Transitions"],
    Cell[357694, 15013, 371, 12, 40, "Notes",
      CellTags->"Transitions"],
    Cell[358068, 15027, 293, 11, 24, "Notes",
      CellTags->"Transitions"],
    Cell[358364, 15040, 69, 1, 39, "ExampleSection",
      CellTags->"Transitions"],
    Cell[358436, 15043, 124, 4, 34, "ExampleText",
      CellTags->"Transitions"],
    Cell[358563, 15049, 121, 5, 45, "Input",
      CellTags->"Transitions"],
    Cell[358687, 15056, 115, 5, 45, "Input",
      CellTags->"Transitions"]},
  "TransitiveLanguageQFA"->{
    Cell[358805, 15063, 80, 1, 32, "ObjectName",
      CellTags->"TransitiveLanguageQFA"],
    Cell[358888, 15066, 66, 1, 50, "Highlight",
      CellTags->"TransitiveLanguageQFA"],
    Cell[358957, 15069, 202, 6, 31, "Usage",
      CellTags->"TransitiveLanguageQFA"],
    Cell[359162, 15077, 259, 9, 44, "Notes",
      CellTags->"TransitiveLanguageQFA"],
    Cell[359424, 15088, 183, 6, 24, "Notes",
      CellTags->"TransitiveLanguageQFA"],
    Cell[359610, 15096, 180, 7, 24, "Notes",
      CellTags->"TransitiveLanguageQFA"],
    Cell[359793, 15105, 79, 1, 39, "ExampleSection",
      CellTags->"TransitiveLanguageQFA"],
    Cell[359875, 15108, 133, 4, 34, "ExampleText",
      CellTags->"TransitiveLanguageQFA"],
    Cell[360011, 15114, 98, 2, 28, "Input",
      CellTags->"TransitiveLanguageQFA"],
    Cell[360112, 15118, 120, 3, 26, "Input",
      CellTags->"TransitiveLanguageQFA"]},
  "TransitiveQFA"->{
    Cell[360235, 15123, 64, 1, 32, "ObjectName",
      CellTags->"TransitiveQFA"],
    Cell[360302, 15126, 58, 1, 50, "Highlight",
      CellTags->"TransitiveQFA"],
    Cell[360363, 15129, 138, 4, 31, "Usage",
      CellTags->"TransitiveQFA"],
    Cell[360504, 15135, 299, 10, 61, "Notes",
      CellTags->"TransitiveQFA"],
    Cell[360806, 15147, 201, 6, 40, "Notes",
      CellTags->"TransitiveQFA"],
    Cell[361010, 15155, 180, 7, 24, "Notes",
      CellTags->"TransitiveQFA"],
    Cell[361193, 15164, 71, 1, 39, "ExampleSection",
      CellTags->"TransitiveQFA"],
    Cell[361267, 15167, 151, 4, 34, "ExampleText",
      CellTags->"TransitiveQFA"],
    Cell[361421, 15173, 90, 2, 28, "Input",
      CellTags->"TransitiveQFA"],
    Cell[361514, 15177, 104, 3, 26, "Input",
      CellTags->"TransitiveQFA"],
    Cell[361621, 15182, 126, 3, 26, "Input",
      CellTags->"TransitiveQFA"]},
  "TransOnly"->{
    Cell[361750, 15187, 56, 1, 32, "ObjectName",
      CellTags->"TransOnly"],
    Cell[361809, 15190, 52, 1, 50, "Highlight",
      CellTags->"TransOnly"],
    Cell[361864, 15193, 145, 6, 31, "Usage",
      CellTags->"TransOnly"],
    Cell[362012, 15201, 152, 5, 24, "Notes",
      CellTags->"TransOnly"],
    Cell[362167, 15208, 156, 7, 24, "Notes",
      CellTags->"TransOnly"],
    Cell[362326, 15217, 67, 1, 39, "ExampleSection",
      CellTags->"TransOnly"],
    Cell[362396, 15220, 73, 1, 34, "ExampleText",
      CellTags->"TransOnly"],
    Cell[362472, 15223, 215, 5, 42, "Input",
      CellTags->"TransOnly"],
    Cell[362690, 15230, 94, 3, 26, "Input",
      CellTags->"TransOnly"],
    Cell[362787, 15235, 118, 3, 26, "Input",
      CellTags->"TransOnly"]},
  "TrapStatesFA"->{
    Cell[362908, 15240, 62, 1, 32, "ObjectName",
      CellTags->"TrapStatesFA"],
    Cell[362973, 15243, 57, 1, 50, "Highlight",
      CellTags->"TrapStatesFA"],
    Cell[363033, 15246, 169, 6, 31, "Usage",
      CellTags->"TrapStatesFA"],
    Cell[363205, 15254, 227, 9, 44, "Notes",
      CellTags->"TrapStatesFA"],
    Cell[363435, 15265, 203, 6, 40, "Notes",
      CellTags->"TrapStatesFA"],
    Cell[363641, 15273, 153, 4, 24, "Notes",
      CellTags->"TrapStatesFA"],
    Cell[363797, 15279, 159, 7, 24, "Notes",
      CellTags->"TrapStatesFA"],
    Cell[363959, 15288, 70, 1, 39, "ExampleSection",
      CellTags->"TrapStatesFA"],
    Cell[364032, 15291, 130, 4, 34, "ExampleText",
      CellTags->"TrapStatesFA"],
    Cell[364165, 15297, 142, 5, 28, "Input",
      CellTags->"TrapStatesFA"],
    Cell[364310, 15304, 156, 4, 42, "Input",
      CellTags->"TrapStatesFA"]},
  "TrimFA"->{
    Cell[364469, 15310, 50, 1, 32, "ObjectName",
      CellTags->"TrimFA"],
    Cell[364522, 15313, 51, 1, 50, "Highlight",
      CellTags->"TrimFA"],
    Cell[364576, 15316, 232, 5, 47, "Usage",
      CellTags->"TrimFA"],
    Cell[364811, 15323, 246, 9, 44, "Notes",
      CellTags->"TrimFA"],
    Cell[365060, 15334, 196, 5, 40, "Notes",
      CellTags->"TrimFA"],
    Cell[365259, 15341, 270, 11, 24, "Notes",
      CellTags->"TrimFA"],
    Cell[365532, 15354, 64, 1, 39, "ExampleSection",
      CellTags->"TrimFA"],
    Cell[365599, 15357, 157, 4, 50, "ExampleText",
      CellTags->"TrimFA"],
    Cell[365759, 15363, 181, 5, 26, "Input",
      CellTags->"TrimFA"],
    Cell[365943, 15370, 138, 4, 42, "Input",
      CellTags->"TrimFA"]},
  "TropicalIdentityMatrix"->{
    Cell[366084, 15376, 82, 1, 32, "ObjectName",
      CellTags->"TropicalIdentityMatrix"],
    Cell[366169, 15379, 67, 1, 50, "Highlight",
      CellTags->"TropicalIdentityMatrix"],
    Cell[366239, 15382, 239, 8, 31, "Usage",
      CellTags->"TropicalIdentityMatrix"],
    Cell[366481, 15392, 357, 15, 24, "Notes",
      CellTags->"TropicalIdentityMatrix"],
    Cell[366841, 15409, 80, 1, 39, "ExampleSection",
      CellTags->"TropicalIdentityMatrix"],
    Cell[366924, 15412, 172, 6, 34, "ExampleText",
      CellTags->"TropicalIdentityMatrix"],
    Cell[367099, 15420, 120, 2, 28, "Input",
      CellTags->"TropicalIdentityMatrix"]},
  "Tropical"->{
    Cell[367222, 15424, 54, 1, 32, "ObjectName",
      CellTags->"Tropical"],
    Cell[367279, 15427, 51, 1, 50, "Highlight",
      CellTags->"Tropical"],
    Cell[367333, 15430, 154, 6, 31, "Usage",
      CellTags->"Tropical"],
    Cell[367490, 15438, 112, 4, 24, "Notes",
      CellTags->"Tropical"],
    Cell[367605, 15444, 340, 11, 24, "Notes",
      CellTags->"Tropical"]},
  "TS"->{
    Cell[367948, 15457, 42, 1, 32, "ObjectName",
      CellTags->"TS"],
    Cell[367993, 15460, 50, 1, 50, "Highlight",
      CellTags->"TS"],
    Cell[368046, 15463, 222, 8, 31, "Usage",
      CellTags->"TS"],
    Cell[368271, 15473, 318, 13, 24, "Notes",
      CellTags->"TS"],
    Cell[368592, 15488, 265, 10, 40, "Notes",
      CellTags->"TS"],
    Cell[368860, 15500, 372, 15, 24, "Notes",
      CellTags->"TS"],
    Cell[369235, 15517, 60, 1, 39, "ExampleSection",
      CellTags->"TS"],
    Cell[369298, 15520, 79, 1, 34, "ExampleText",
      CellTags->"TS"],
    Cell[369380, 15523, 73, 2, 28, "Input",
      CellTags->"TS"],
    Cell[369456, 15527, 73, 2, 28, "Input",
      CellTags->"TS"],
    Cell[369532, 15531, 67, 2, 28, "Input",
      CellTags->"TS"],
    Cell[369602, 15535, 80, 2, 28, "Input",
      CellTags->"TS"],
    Cell[369685, 15539, 80, 2, 28, "Input",
      CellTags->"TS"],
    Cell[369768, 15543, 74, 2, 28, "Input",
      CellTags->"TS"]},
  "UnionFA"->{
    Cell[369845, 15547, 52, 1, 32, "ObjectName",
      CellTags->"UnionFA"],
    Cell[369900, 15550, 52, 1, 50, "Highlight",
      CellTags->"UnionFA"],
    Cell[369955, 15553, 250, 9, 47, "Usage",
      CellTags->"UnionFA"],
    Cell[370208, 15564, 292, 10, 61, "Notes",
      CellTags->"UnionFA"],
    Cell[370503, 15576, 134, 5, 24, "Notes",
      CellTags->"UnionFA"],
    Cell[370640, 15583, 195, 8, 24, "Notes",
      CellTags->"UnionFA"],
    Cell[370838, 15593, 134, 4, 24, "Notes",
      CellTags->"UnionFA"],
    Cell[370975, 15599, 52, 1, 24, "Notes",
      CellTags->"UnionFA"],
    Cell[371030, 15602, 348, 15, 24, "Notes",
      CellTags->"UnionFA"],
    Cell[371381, 15619, 65, 1, 39, "ExampleSection",
      CellTags->"UnionFA"],
    Cell[371449, 15622, 198, 5, 50, "ExampleText",
      CellTags->"UnionFA"],
    Cell[371650, 15629, 187, 4, 42, "Input",
      CellTags->"UnionFA"],
    Cell[371840, 15635, 123, 4, 34, "ExampleText",
      CellTags->"UnionFA"],
    Cell[371966, 15641, 169, 4, 42, "Input",
      CellTags->"UnionFA"]},
  "Unlabeled"->{
    Cell[372138, 15647, 56, 1, 32, "ObjectName",
      CellTags->"Unlabeled"],
    Cell[372197, 15650, 52, 1, 50, "Highlight",
      CellTags->"Unlabeled"],
    Cell[372252, 15653, 190, 6, 31, "Usage",
      CellTags->"Unlabeled"],
    Cell[372445, 15661, 108, 5, 24, "Notes",
      CellTags->"Unlabeled"],
    Cell[372556, 15668, 156, 7, 24, "Notes",
      CellTags->"Unlabeled"],
    Cell[372715, 15677, 67, 1, 39, "ExampleSection",
      CellTags->"Unlabeled"],
    Cell[372785, 15680, 82, 1, 34, "ExampleText",
      CellTags->"Unlabeled"],
    Cell[372870, 15683, 214, 5, 42, "Input",
      CellTags->"Unlabeled"],
    Cell[373087, 15690, 80, 2, 28, "Input",
      CellTags->"Unlabeled"],
    Cell[373170, 15694, 121, 3, 26, "Input",
      CellTags->"Unlabeled"]},
  "VerifyFA"->{
    Cell[373294, 15699, 54, 1, 32, "ObjectName",
      CellTags->"VerifyFA"],
    Cell[373351, 15702, 53, 1, 50, "Highlight",
      CellTags->"VerifyFA"],
    Cell[373407, 15705, 210, 7, 47, "Usage",
      CellTags->"VerifyFA"],
    Cell[373620, 15714, 143, 5, 24, "Notes",
      CellTags->"VerifyFA"],
    Cell[373766, 15721, 66, 1, 39, "ExampleSection",
      CellTags->"VerifyFA"]},
  "WordToFA"->{
    Cell[373835, 15724, 54, 1, 32, "ObjectName",
      CellTags->"WordToFA"],
    Cell[373892, 15727, 53, 1, 50, "Highlight",
      CellTags->"WordToFA"],
    Cell[373948, 15730, 178, 6, 31, "Usage",
      CellTags->"WordToFA"],
    Cell[374129, 15738, 398, 13, 79, "Notes",
      CellTags->"WordToFA"],
    Cell[374530, 15753, 178, 5, 24, "Notes",
      CellTags->"WordToFA"],
    Cell[374711, 15760, 178, 5, 24, "Notes",
      CellTags->"WordToFA"],
    Cell[374892, 15767, 238, 8, 40, "Notes",
      CellTags->"WordToFA"],
    Cell[375133, 15777, 258, 11, 24, "Notes",
      CellTags->"WordToFA"],
    Cell[375394, 15790, 66, 1, 39, "ExampleSection",
      CellTags->"WordToFA"],
    Cell[375463, 15793, 219, 8, 34, "ExampleText",
      CellTags->"WordToFA"],
    Cell[375685, 15803, 160, 4, 42, "Input",
      CellTags->"WordToFA"],
    Cell[375848, 15809, 220, 8, 34, "ExampleText",
      CellTags->"WordToFA"],
    Cell[376071, 15819, 185, 4, 42, "Input",
      CellTags->"WordToFA"],
    Cell[376259, 15825, 224, 8, 34, "ExampleText",
      CellTags->"WordToFA"],
    Cell[376486, 15835, 185, 4, 42, "Input",
      CellTags->"WordToFA"],
    Cell[376674, 15841, 122, 4, 34, "ExampleText",
      CellTags->"WordToFA"],
    Cell[376799, 15847, 171, 5, 26, "Input",
      CellTags->"WordToFA"],
    Cell[376973, 15854, 137, 4, 34, "ExampleText",
      CellTags->"WordToFA"],
    Cell[377113, 15860, 248, 5, 58, "Input",
      CellTags->"WordToFA"],
    Cell[377364, 15867, 144, 4, 34, "ExampleText",
      CellTags->"WordToFA"],
    Cell[377511, 15873, 173, 4, 42, "Input",
      CellTags->"WordToFA"],
    Cell[377687, 15879, 179, 5, 50, "ExampleText",
      CellTags->"WordToFA"],
    Cell[377869, 15886, 339, 8, 74, "Input",
      CellTags->"WordToFA"]},
  "$regexrules"->{
    Cell[378211, 15896, 60, 1, 32, "ObjectName",
      CellTags->"$regexrules"],
    Cell[378274, 15899, 59, 1, 50, "Highlight",
      CellTags->"$regexrules"],
    Cell[378336, 15902, 154, 4, 31, "Usage",
      CellTags->"$regexrules"],
    Cell[378493, 15908, 196, 6, 24, "Notes",
      CellTags->"$regexrules"],
    Cell[378692, 15916, 141, 4, 24, "Notes",
      CellTags->"$regexrules"],
    Cell[378836, 15922, 164, 4, 40, "Notes",
      CellTags->"$regexrules"],
    Cell[379003, 15928, 202, 9, 24, "Notes",
      CellTags->"$regexrules"],
    Cell[379208, 15939, 69, 1, 39, "ExampleSection",
      CellTags->"$regexrules"],
    Cell[379280, 15942, 171, 5, 34, "ExampleText",
      CellTags->"$regexrules"],
    Cell[379454, 15949, 99, 2, 28, "Input",
      CellTags->"$regexrules"],
    Cell[379556, 15953, 100, 2, 28, "Input",
      CellTags->"$regexrules"]},
  "ArdenSolve"->{
    Cell[379659, 15957, 58, 1, 32, "ObjectName",
      CellTags->"ArdenSolve"],
    Cell[379720, 15960, 55, 1, 50, "Highlight",
      CellTags->"ArdenSolve"],
    Cell[379778, 15963, 320, 8, 63, "Usage",
      CellTags->"ArdenSolve"],
    Cell[380101, 15973, 124, 5, 24, "Notes",
      CellTags->"ArdenSolve"],
    Cell[380228, 15980, 213, 5, 40, "Notes",
      CellTags->"ArdenSolve"],
    Cell[380444, 15987, 205, 9, 24, "Notes",
      CellTags->"ArdenSolve"],
    Cell[380652, 15998, 68, 1, 39, "ExampleSection",
      CellTags->"ArdenSolve"],
    Cell[380723, 16001, 271, 9, 50, "ExampleText",
      CellTags->"ArdenSolve"],
    Cell[380997, 16012, 324, 7, 90, "Input",
      CellTags->"ArdenSolve"],
    Cell[381324, 16021, 278, 12, 34, "ExampleText",
      CellTags->"ArdenSolve"],
    Cell[381605, 16035, 140, 4, 26, "Input",
      CellTags->"ArdenSolve"],
    Cell[381748, 16041, 301, 12, 34, "ExampleText",
      CellTags->"ArdenSolve"],
    Cell[382052, 16055, 140, 4, 26, "Input",
      CellTags->"ArdenSolve"],
    Cell[382195, 16061, 130, 4, 26, "Input",
      CellTags->"ArdenSolve"],
    Cell[382328, 16067, 121, 4, 34, "ExampleText",
      CellTags->"ArdenSolve"],
    Cell[382452, 16073, 136, 4, 26, "Input",
      CellTags->"ArdenSolve"],
    Cell[382591, 16079, 130, 4, 26, "Input",
      CellTags->"ArdenSolve"],
    Cell[382724, 16085, 197, 5, 50, "ExampleText",
      CellTags->"ArdenSolve"],
    Cell[382924, 16092, 188, 5, 42, "Input",
      CellTags->"ArdenSolve"],
    Cell[383115, 16099, 127, 4, 34, "ExampleText",
      CellTags->"ArdenSolve"],
    Cell[383245, 16105, 229, 5, 58, "Input",
      CellTags->"ArdenSolve"],
    Cell[383477, 16112, 105, 1, 34, "ExampleText",
      CellTags->"ArdenSolve"],
    Cell[383585, 16115, 104, 3, 26, "Input",
      CellTags->"ArdenSolve"]},
  "Empty"->{
    Cell[383692, 16120, 48, 1, 32, "ObjectName",
      CellTags->"Empty"],
    Cell[383743, 16123, 48, 1, 50, "Highlight",
      CellTags->"Empty"],
    Cell[383794, 16126, 134, 4, 31, "Usage",
      CellTags->"Empty"],
    Cell[383931, 16132, 180, 7, 24, "Notes",
      CellTags->"Empty"]},
  "FormatX"->{
    Cell[384114, 16141, 52, 1, 32, "ObjectName",
      CellTags->"FormatX"],
    Cell[384169, 16144, 52, 1, 50, "Highlight",
      CellTags->"FormatX"],
    Cell[384224, 16147, 196, 8, 31, "Usage",
      CellTags->"FormatX"],
    Cell[384423, 16157, 132, 4, 24, "Notes",
      CellTags->"FormatX"],
    Cell[384558, 16163, 140, 4, 24, "Notes",
      CellTags->"FormatX"],
    Cell[384701, 16169, 169, 7, 24, "Notes",
      CellTags->"FormatX"],
    Cell[384873, 16178, 65, 1, 39, "ExampleSection",
      CellTags->"FormatX"],
    Cell[384941, 16181, 90, 1, 34, "ExampleText",
      CellTags->"FormatX"],
    Cell[385034, 16184, 223, 6, 58, "Input",
      CellTags->"FormatX"],
    Cell[385260, 16192, 75, 1, 34, "ExampleText",
      CellTags->"FormatX"],
    Cell[385338, 16195, 169, 4, 26, "Input",
      CellTags->"FormatX"],
    Cell[385510, 16201, 94, 1, 34, "ExampleText",
      CellTags->"FormatX"],
    Cell[385607, 16204, 134, 4, 42, "Input",
      CellTags->"FormatX"]},
  "REEpsQ"->{
    Cell[385744, 16210, 50, 1, 32, "ObjectName",
      CellTags->"REEpsQ"],
    Cell[385797, 16213, 51, 1, 50, "Highlight",
      CellTags->"REEpsQ"],
    Cell[385851, 16216, 197, 6, 31, "Usage",
      CellTags->"REEpsQ"],
    Cell[386051, 16224, 376, 14, 40, "Notes",
      CellTags->"REEpsQ"],
    Cell[386430, 16240, 155, 7, 24, "Notes",
      CellTags->"REEpsQ"],
    Cell[386588, 16249, 64, 1, 39, "ExampleSection",
      CellTags->"REEpsQ"],
    Cell[386655, 16252, 78, 1, 34, "ExampleText",
      CellTags->"REEpsQ"],
    Cell[386736, 16255, 85, 2, 28, "Input",
      CellTags->"REEpsQ"],
    Cell[386824, 16259, 90, 2, 28, "Input",
      CellTags->"REEpsQ"],
    Cell[386917, 16263, 97, 2, 28, "Input",
      CellTags->"REEpsQ"]},
  "RegExpSimp"->{
    Cell[387017, 16267, 58, 1, 32, "ObjectName",
      CellTags->"RegExpSimp"],
    Cell[387078, 16270, 53, 1, 50, "Highlight",
      CellTags->"RegExpSimp"],
    Cell[387134, 16273, 161, 6, 31, "Usage",
      CellTags->"RegExpSimp"],
    Cell[387298, 16281, 167, 4, 40, "Notes",
      CellTags->"RegExpSimp"],
    Cell[387468, 16287, 203, 7, 24, "Notes",
      CellTags->"RegExpSimp"]},
  "RegularExpressionToBEFA"->{
    Cell[387674, 16296, 84, 1, 32, "ObjectName",
      CellTags->"RegularExpressionToBEFA"],
    Cell[387761, 16299, 68, 1, 50, "Highlight",
      CellTags->"RegularExpressionToBEFA"],
    Cell[387832, 16302, 197, 6, 31, "Usage",
      CellTags->"RegularExpressionToBEFA"],
    Cell[388032, 16310, 243, 9, 44, "Notes",
      CellTags->"RegularExpressionToBEFA"],
    Cell[388278, 16321, 176, 5, 24, "Notes",
      CellTags->"RegularExpressionToBEFA"],
    Cell[388457, 16328, 103, 1, 24, "Notes",
      CellTags->"RegularExpressionToBEFA"],
    Cell[388563, 16331, 134, 4, 24, "Notes",
      CellTags->"RegularExpressionToBEFA"],
    Cell[388700, 16337, 282, 11, 24, "Notes",
      CellTags->"RegularExpressionToBEFA"],
    Cell[388985, 16350, 81, 1, 39, "ExampleSection",
      CellTags->"RegularExpressionToBEFA"],
    Cell[389069, 16353, 257, 7, 34, "ExampleText",
      CellTags->"RegularExpressionToBEFA"],
    Cell[389329, 16362, 147, 3, 26, "Input",
      CellTags->"RegularExpressionToBEFA"],
    Cell[389479, 16367, 125, 3, 26, "Input",
      CellTags->"RegularExpressionToBEFA"],
    Cell[389607, 16372, 98, 1, 34, "ExampleText",
      CellTags->"RegularExpressionToBEFA"],
    Cell[389708, 16375, 150, 3, 26, "Input",
      CellTags->"RegularExpressionToBEFA"]},
  "RegularExpressionToFA"->{
    Cell[389861, 16380, 80, 1, 32, "ObjectName",
      CellTags->"RegularExpressionToFA"],
    Cell[389944, 16383, 66, 1, 50, "Highlight",
      CellTags->"RegularExpressionToFA"],
    Cell[390013, 16386, 237, 8, 47, "Usage",
      CellTags->"RegularExpressionToFA"],
    Cell[390253, 16396, 337, 11, 79, "Notes",
      CellTags->"RegularExpressionToFA"],
    Cell[390593, 16409, 171, 5, 24, "Notes",
      CellTags->"RegularExpressionToFA"],
    Cell[390767, 16416, 180, 5, 24, "Notes",
      CellTags->"RegularExpressionToFA"],
    Cell[390950, 16423, 172, 5, 24, "Notes",
      CellTags->"RegularExpressionToFA"],
    Cell[391125, 16430, 245, 6, 24, "Notes",
      CellTags->"RegularExpressionToFA"],
    Cell[391373, 16438, 79, 1, 39, "ExampleSection",
      CellTags->"RegularExpressionToFA"],
    Cell[391455, 16441, 254, 7, 34, "ExampleText",
      CellTags->"RegularExpressionToFA"],
    Cell[391712, 16450, 144, 3, 26, "Input",
      CellTags->"RegularExpressionToFA"],
    Cell[391859, 16455, 128, 3, 26, "Input",
      CellTags->"RegularExpressionToFA"],
    Cell[391990, 16460, 116, 3, 26, "Input",
      CellTags->"RegularExpressionToFA"],
    Cell[392109, 16465, 146, 4, 34, "ExampleText",
      CellTags->"RegularExpressionToFA"],
    Cell[392258, 16471, 195, 5, 26, "Input",
      CellTags->"RegularExpressionToFA"],
    Cell[392456, 16478, 137, 4, 34, "ExampleText",
      CellTags->"RegularExpressionToFA"],
    Cell[392596, 16484, 163, 4, 26, "Input",
      CellTags->"RegularExpressionToFA"]},
  "RegularExpressionToNNFA"->{
    Cell[392762, 16490, 84, 1, 32, "ObjectName",
      CellTags->"RegularExpressionToNNFA"],
    Cell[392849, 16493, 68, 1, 50, "Highlight",
      CellTags->"RegularExpressionToNNFA"],
    Cell[392920, 16496, 197, 6, 31, "Usage",
      CellTags->"RegularExpressionToNNFA"],
    Cell[393120, 16504, 138, 4, 24, "Notes",
      CellTags->"RegularExpressionToNNFA"],
    Cell[393261, 16510, 282, 11, 24, "Notes",
      CellTags->"RegularExpressionToNNFA"],
    Cell[393546, 16523, 81, 1, 39, "ExampleSection",
      CellTags->"RegularExpressionToNNFA"],
    Cell[393630, 16526, 257, 7, 34, "ExampleText",
      CellTags->"RegularExpressionToNNFA"],
    Cell[393890, 16535, 147, 3, 26, "Input",
      CellTags->"RegularExpressionToNNFA"],
    Cell[394040, 16540, 125, 3, 26, "Input",
      CellTags->"RegularExpressionToNNFA"],
    Cell[394168, 16545, 98, 1, 34, "ExampleText",
      CellTags->"RegularExpressionToNNFA"],
    Cell[394269, 16548, 150, 3, 26, "Input",
      CellTags->"RegularExpressionToNNFA"]},
  "REP"->{
    Cell[394422, 16553, 44, 1, 32, "ObjectName",
      CellTags->"REP"],
    Cell[394469, 16556, 51, 1, 50, "Highlight",
      CellTags->"REP"],
    Cell[394523, 16559, 130, 4, 31, "Usage",
      CellTags->"REP"],
    Cell[394656, 16565, 97, 5, 24, "Notes",
      CellTags->"REP"],
    Cell[394756, 16572, 379, 15, 24, "Notes",
      CellTags->"REP"],
    Cell[395138, 16589, 61, 1, 39, "ExampleSection",
      CellTags->"REP"],
    Cell[395202, 16592, 279, 10, 50, "ExampleText",
      CellTags->"REP"],
    Cell[395484, 16604, 137, 5, 45, "Input",
      CellTags->"REP"],
    Cell[395624, 16611, 111, 3, 26, "Input",
      CellTags->"REP"],
    Cell[395738, 16616, 98, 3, 26, "Input",
      CellTags->"REP"]},
  "REQuotient"->{
    Cell[395839, 16621, 58, 1, 32, "ObjectName",
      CellTags->"REQuotient"],
    Cell[395900, 16624, 55, 1, 50, "Highlight",
      CellTags->"REQuotient"],
    Cell[395958, 16627, 234, 8, 47, "Usage",
      CellTags->"REQuotient"],
    Cell[396195, 16637, 523, 17, 40, "Notes",
      CellTags->"REQuotient"],
    Cell[396721, 16656, 320, 7, 40, "Notes",
      CellTags->"REQuotient"],
    Cell[397044, 16665, 185, 7, 24, "Notes",
      CellTags->"REQuotient"],
    Cell[397232, 16674, 68, 1, 39, "ExampleSection",
      CellTags->"REQuotient"],
    Cell[397303, 16677, 256, 8, 34, "ExampleText",
      CellTags->"REQuotient"],
    Cell[397562, 16687, 101, 2, 28, "Input",
      CellTags->"REQuotient"],
    Cell[397666, 16691, 188, 6, 34, "ExampleText",
      CellTags->"REQuotient"],
    Cell[397857, 16699, 98, 3, 26, "Input",
      CellTags->"REQuotient"]},
  "RES"->{
    Cell[397958, 16704, 44, 1, 32, "ObjectName",
      CellTags->"RES"],
    Cell[398005, 16707, 51, 1, 50, "Highlight",
      CellTags->"RES"],
    Cell[398059, 16710, 136, 4, 31, "Usage",
      CellTags->"RES"],
    Cell[398198, 16716, 94, 5, 24, "Notes",
      CellTags->"RES"],
    Cell[398295, 16723, 379, 15, 24, "Notes",
      CellTags->"RES"],
    Cell[398677, 16740, 61, 1, 39, "ExampleSection",
      CellTags->"RES"],
    Cell[398741, 16743, 279, 10, 50, "ExampleText",
      CellTags->"RES"],
    Cell[399023, 16755, 138, 5, 45, "Input",
      CellTags->"RES"],
    Cell[399164, 16762, 112, 3, 26, "Input",
      CellTags->"RES"],
    Cell[399279, 16767, 99, 3, 26, "Input",
      CellTags->"RES"],
    Cell[399381, 16772, 130, 4, 34, "ExampleText",
      CellTags->"RES"],
    Cell[399514, 16778, 118, 3, 26, "Input",
      CellTags->"RES"]},
  "RESimplify"->{
    Cell[399635, 16783, 58, 1, 32, "ObjectName",
      CellTags->"RESimplify"],
    Cell[399696, 16786, 55, 1, 50, "Highlight",
      CellTags->"RESimplify"],
    Cell[399754, 16789, 238, 8, 47, "Usage",
      CellTags->"RESimplify"],
    Cell[399995, 16799, 129, 4, 24, "Notes",
      CellTags->"RESimplify"],
    Cell[400127, 16805, 268, 12, 24, "Notes",
      CellTags->"RESimplify"],
    Cell[400398, 16819, 68, 1, 39, "ExampleSection",
      CellTags->"RESimplify"],
    Cell[400469, 16822, 195, 6, 34, "ExampleText",
      CellTags->"RESimplify"],
    Cell[400667, 16830, 95, 2, 28, "Input",
      CellTags->"RESimplify"],
    Cell[400765, 16834, 220, 7, 34, "ExampleText",
      CellTags->"RESimplify"],
    Cell[400988, 16843, 110, 2, 28, "Input",
      CellTags->"RESimplify"]},
  "RET"->{
    Cell[401101, 16847, 44, 1, 32, "ObjectName",
      CellTags->"RET"],
    Cell[401148, 16850, 51, 1, 50, "Highlight",
      CellTags->"RET"],
    Cell[401202, 16853, 138, 4, 31, "Usage",
      CellTags->"RET"],
    Cell[401343, 16859, 97, 5, 24, "Notes",
      CellTags->"RET"],
    Cell[401443, 16866, 379, 15, 24, "Notes",
      CellTags->"RET"],
    Cell[401825, 16883, 61, 1, 39, "ExampleSection",
      CellTags->"RET"],
    Cell[401889, 16886, 279, 10, 50, "ExampleText",
      CellTags->"RET"],
    Cell[402171, 16898, 137, 5, 45, "Input",
      CellTags->"RET"],
    Cell[402311, 16905, 111, 3, 26, "Input",
      CellTags->"RET"],
    Cell[402425, 16910, 98, 3, 26, "Input",
      CellTags->"RET"]},
  "SubstituteRE"->{
    Cell[402526, 16915, 62, 1, 32, "ObjectName",
      CellTags->"SubstituteRE"],
    Cell[402591, 16918, 57, 1, 50, "Highlight",
      CellTags->"SubstituteRE"],
    Cell[402651, 16921, 233, 8, 47, "Usage",
      CellTags->"SubstituteRE"],
    Cell[402887, 16931, 133, 5, 24, "Notes",
      CellTags->"SubstituteRE"],
    Cell[403023, 16938, 205, 9, 24, "Notes",
      CellTags->"SubstituteRE"],
    Cell[403231, 16949, 70, 1, 39, "ExampleSection",
      CellTags->"SubstituteRE"],
    Cell[403304, 16952, 167, 4, 50, "ExampleText",
      CellTags->"SubstituteRE"],
    Cell[403474, 16958, 281, 7, 90, "Input",
      CellTags->"SubstituteRE"],
    Cell[403758, 16967, 104, 1, 34, "ExampleText",
      CellTags->"SubstituteRE"],
    Cell[403865, 16970, 106, 3, 26, "Input",
      CellTags->"SubstituteRE"],
    Cell[403974, 16975, 293, 12, 34, "ExampleText",
      CellTags->"SubstituteRE"],
    Cell[404270, 16989, 134, 4, 42, "Input",
      CellTags->"SubstituteRE"],
    Cell[404407, 16995, 113, 2, 34, "ExampleText",
      CellTags->"SubstituteRE"],
    Cell[404523, 16999, 232, 6, 74, "Input",
      CellTags->"SubstituteRE"]},
  "ToRegExp"->{
    Cell[404758, 17007, 54, 1, 32, "ObjectName",
      CellTags->"ToRegExp"],
    Cell[404815, 17010, 53, 1, 50, "Highlight",
      CellTags->"ToRegExp"],
    Cell[404871, 17013, 417, 14, 63, "Usage",
      CellTags->"ToRegExp"],
    Cell[405291, 17029, 138, 4, 24, "Notes",
      CellTags->"ToRegExp"],
    Cell[405432, 17035, 498, 22, 24, "Notes",
      CellTags->"ToRegExp"],
    Cell[405933, 17059, 66, 1, 39, "ExampleSection",
      CellTags->"ToRegExp"],
    Cell[406002, 17062, 150, 4, 50, "ExampleText",
      CellTags->"ToRegExp"],
    Cell[406155, 17068, 110, 2, 28, "Input",
      CellTags->"ToRegExp"],
    Cell[406268, 17072, 93, 3, 26, "Input",
      CellTags->"ToRegExp"],
    Cell[406364, 17077, 84, 2, 26, "Input",
      CellTags->"ToRegExp"]},
  "BFunctionToCA"->{
    Cell[406451, 17081, 64, 1, 32, "ObjectName",
      CellTags->"BFunctionToCA"],
    Cell[406518, 17084, 58, 1, 50, "Highlight",
      CellTags->"BFunctionToCA"],
    Cell[406579, 17087, 376, 14, 47, "Usage",
      CellTags->"BFunctionToCA"],
    Cell[406958, 17103, 309, 12, 40, "Notes",
      CellTags->"BFunctionToCA"],
    Cell[407270, 17117, 224, 9, 24, "Notes",
      CellTags->"BFunctionToCA"],
    Cell[407497, 17128, 283, 10, 24, "Notes",
      CellTags->"BFunctionToCA"],
    Cell[407783, 17140, 269, 11, 24, "Notes",
      CellTags->"BFunctionToCA"],
    Cell[408055, 17153, 71, 1, 39, "ExampleSection",
      CellTags->"BFunctionToCA"],
    Cell[408129, 17156, 136, 4, 34, "ExampleText",
      CellTags->"BFunctionToCA"],
    Cell[408268, 17162, 126, 3, 26, "Input",
      CellTags->"BFunctionToCA"],
    Cell[408397, 17167, 75, 1, 34, "ExampleText",
      CellTags->"BFunctionToCA"],
    Cell[408475, 17170, 157, 4, 42, "Input",
      CellTags->"BFunctionToCA"],
    Cell[408635, 17176, 189, 6, 34, "ExampleText",
      CellTags->"BFunctionToCA"],
    Cell[408827, 17184, 122, 3, 26, "Input",
      CellTags->"BFunctionToCA"],
    Cell[408952, 17189, 75, 1, 34, "ExampleText",
      CellTags->"BFunctionToCA"],
    Cell[409030, 17192, 163, 4, 42, "Input",
      CellTags->"BFunctionToCA"]},
  "CA"->{
    Cell[409196, 17198, 42, 1, 32, "ObjectName",
      CellTags->"CA"],
    Cell[409241, 17201, 50, 1, 50, "Highlight",
      CellTags->"CA"],
    Cell[409294, 17204, 272, 10, 47, "Usage",
      CellTags->"CA"],
    Cell[409569, 17216, 381, 12, 40, "Notes",
      CellTags->"CA"],
    Cell[409953, 17230, 202, 9, 24, "Notes",
      CellTags->"CA"],
    Cell[410158, 17241, 60, 1, 39, "ExampleSection",
      CellTags->"CA"],
    Cell[410221, 17244, 130, 4, 34, "ExampleText",
      CellTags->"CA"],
    Cell[410354, 17250, 178, 5, 42, "Input",
      CellTags->"CA"],
    Cell[410535, 17257, 239, 6, 50, "ExampleText",
      CellTags->"CA"],
    Cell[410777, 17265, 350, 8, 58, "Input",
      CellTags->"CA"],
    Cell[411130, 17275, 130, 4, 34, "ExampleText",
      CellTags->"CA"],
    Cell[411263, 17281, 69, 2, 28, "Input",
      CellTags->"CA"]},
  "ClassifyCA"->{
    Cell[411335, 17285, 58, 1, 32, "ObjectName",
      CellTags->"ClassifyCA"],
    Cell[411396, 17288, 55, 1, 50, "Highlight",
      CellTags->"ClassifyCA"],
    Cell[411454, 17291, 282, 7, 47, "Usage",
      CellTags->"ClassifyCA"],
    Cell[411739, 17300, 296, 10, 61, "Notes",
      CellTags->"ClassifyCA"],
    Cell[412038, 17312, 417, 17, 97, "Notes",
      CellTags->"ClassifyCA"],
    Cell[412458, 17331, 157, 5, 24, "Notes",
      CellTags->"ClassifyCA"],
    Cell[412618, 17338, 186, 5, 40, "Notes",
      CellTags->"ClassifyCA"],
    Cell[412807, 17345, 157, 4, 40, "Notes",
      CellTags->"ClassifyCA"],
    Cell[412967, 17351, 200, 9, 24, "Notes",
      CellTags->"ClassifyCA"],
    Cell[413170, 17362, 68, 1, 39, "ExampleSection",
      CellTags->"ClassifyCA"],
    Cell[413241, 17365, 102, 1, 34, "ExampleText",
      CellTags->"ClassifyCA"],
    Cell[413346, 17368, 124, 3, 26, "Input",
      CellTags->"ClassifyCA"],
    Cell[413473, 17373, 121, 3, 26, "Input",
      CellTags->"ClassifyCA"],
    Cell[413597, 17378, 121, 3, 26, "Input",
      CellTags->"ClassifyCA"],
    Cell[413721, 17383, 121, 3, 26, "Input",
      CellTags->"ClassifyCA"],
    Cell[413845, 17388, 164, 4, 50, "ExampleText",
      CellTags->"ClassifyCA"],
    Cell[414012, 17394, 120, 3, 26, "Input",
      CellTags->"ClassifyCA"],
    Cell[414135, 17399, 151, 4, 34, "ExampleText",
      CellTags->"ClassifyCA"],
    Cell[414289, 17405, 176, 4, 26, "Input",
      CellTags->"ClassifyCA"]},
  "ComposeCA"->{
    Cell[414468, 17411, 56, 1, 32, "ObjectName",
      CellTags->"ComposeCA"],
    Cell[414527, 17414, 54, 1, 50, "Highlight",
      CellTags->"ComposeCA"],
    Cell[414584, 17417, 257, 8, 47, "Usage",
      CellTags->"ComposeCA"],
    Cell[414844, 17427, 247, 9, 44, "Notes",
      CellTags->"ComposeCA"],
    Cell[415094, 17438, 284, 9, 40, "Notes",
      CellTags->"ComposeCA"],
    Cell[415381, 17449, 168, 6, 24, "Notes",
      CellTags->"ComposeCA"],
    Cell[415552, 17457, 131, 6, 24, "Notes",
      CellTags->"ComposeCA"],
    Cell[415686, 17465, 67, 1, 39, "ExampleSection",
      CellTags->"ComposeCA"],
    Cell[415756, 17468, 99, 1, 34, "ExampleText",
      CellTags->"ComposeCA"],
    Cell[415858, 17471, 113, 3, 26, "Input",
      CellTags->"ComposeCA"],
    Cell[415974, 17476, 136, 3, 26, "Input",
      CellTags->"ComposeCA"]},
  "Cyclic"->{
    Cell[416113, 17481, 50, 1, 32, "ObjectName",
      CellTags->"Cyclic"],
    Cell[416166, 17484, 49, 1, 50, "Highlight",
      CellTags->"Cyclic"],
    Cell[416218, 17487, 237, 8, 47, "Usage",
      CellTags->"Cyclic"],
    Cell[416458, 17497, 338, 15, 24, "Notes",
      CellTags->"Cyclic"]},
  "EvolutionCA"->{
    Cell[416799, 17514, 60, 1, 32, "ObjectName",
      CellTags->"EvolutionCA"],
    Cell[416862, 17517, 56, 1, 50, "Highlight",
      CellTags->"EvolutionCA"],
    Cell[416921, 17520, 199, 5, 47, "Usage",
      CellTags->"EvolutionCA"],
    Cell[417123, 17527, 299, 10, 61, "Notes",
      CellTags->"EvolutionCA"],
    Cell[417425, 17539, 142, 5, 24, "Notes",
      CellTags->"EvolutionCA"],
    Cell[417570, 17546, 320, 12, 24, "Notes",
      CellTags->"EvolutionCA"],
    Cell[417893, 17560, 112, 5, 24, "Notes",
      CellTags->"EvolutionCA"],
    Cell[418008, 17567, 154, 6, 24, "Notes",
      CellTags->"EvolutionCA"],
    Cell[418165, 17575, 328, 13, 24, "Notes",
      CellTags->"EvolutionCA"],
    Cell[418496, 17590, 69, 1, 39, "ExampleSection",
      CellTags->"EvolutionCA"],
    Cell[418568, 17593, 88, 1, 34, "ExampleText",
      CellTags->"EvolutionCA"],
    Cell[418659, 17596, 90, 2, 28, "Input",
      CellTags->"EvolutionCA"],
    Cell[418752, 17600, 197, 5, 50, "ExampleText",
      CellTags->"EvolutionCA"],
    Cell[418952, 17607, 236, 5, 42, "Input",
      CellTags->"EvolutionCA"]},
  "DependencyCA"->{
    Cell[419191, 17614, 62, 1, 32, "ObjectName",
      CellTags->"DependencyCA"],
    Cell[419256, 17617, 57, 1, 50, "Highlight",
      CellTags->"DependencyCA"],
    Cell[419316, 17620, 235, 7, 47, "Usage",
      CellTags->"DependencyCA"],
    Cell[419554, 17629, 165, 7, 24, "Notes",
      CellTags->"DependencyCA"],
    Cell[419722, 17638, 70, 1, 39, "ExampleSection",
      CellTags->"DependencyCA"],
    Cell[419795, 17641, 160, 4, 50, "ExampleText",
      CellTags->"DependencyCA"],
    Cell[419958, 17647, 143, 4, 42, "Input",
      CellTags->"DependencyCA"],
    Cell[420104, 17653, 95, 3, 26, "Input",
      CellTags->"DependencyCA"]},
  "FSurjectiveCA"->{
    Cell[420202, 17658, 64, 1, 32, "ObjectName",
      CellTags->"FSurjectiveCA"],
    Cell[420269, 17661, 58, 1, 50, "Highlight",
      CellTags->"FSurjectiveCA"],
    Cell[420330, 17664, 213, 6, 47, "Usage",
      CellTags->"FSurjectiveCA"],
    Cell[420546, 17672, 185, 5, 24, "Notes",
      CellTags->"FSurjectiveCA"],
    Cell[420734, 17679, 334, 12, 40, "Notes",
      CellTags->"FSurjectiveCA"],
    Cell[421071, 17693, 206, 9, 24, "Notes",
      CellTags->"FSurjectiveCA"],
    Cell[421280, 17704, 71, 1, 39, "ExampleSection",
      CellTags->"FSurjectiveCA"],
    Cell[421354, 17707, 177, 4, 50, "ExampleText",
      CellTags->"FSurjectiveCA"],
    Cell[421534, 17713, 186, 4, 42, "Input",
      CellTags->"FSurjectiveCA"],
    Cell[421723, 17719, 130, 3, 26, "Input",
      CellTags->"FSurjectiveCA"]},
  "FunctionToCA"->{
    Cell[421856, 17724, 62, 1, 32, "ObjectName",
      CellTags->"FunctionToCA"],
    Cell[421921, 17727, 57, 1, 50, "Highlight",
      CellTags->"FunctionToCA"],
    Cell[421981, 17730, 295, 10, 47, "Usage",
      CellTags->"FunctionToCA"],
    Cell[422279, 17742, 235, 8, 40, "Notes",
      CellTags->"FunctionToCA"],
    Cell[422517, 17752, 374, 15, 24, "Notes",
      CellTags->"FunctionToCA"],
    Cell[422894, 17769, 70, 1, 39, "ExampleSection",
      CellTags->"FunctionToCA"],
    Cell[422967, 17772, 123, 4, 34, "ExampleText",
      CellTags->"FunctionToCA"],
    Cell[423093, 17778, 134, 3, 26, "Input",
      CellTags->"FunctionToCA"],
    Cell[423230, 17783, 89, 1, 34, "ExampleText",
      CellTags->"FunctionToCA"],
    Cell[423322, 17786, 100, 3, 26, "Input",
      CellTags->"FunctionToCA"],
    Cell[423425, 17791, 110, 3, 26, "Input",
      CellTags->"FunctionToCA"]},
  "GlobalMapCA"->{
    Cell[423538, 17796, 60, 1, 32, "ObjectName",
      CellTags->"GlobalMapCA"],
    Cell[423601, 17799, 56, 1, 50, "Highlight",
      CellTags->"GlobalMapCA"],
    Cell[423660, 17802, 310, 10, 47, "Usage",
      CellTags->"GlobalMapCA"],
    Cell[423973, 17814, 274, 9, 61, "Notes",
      CellTags->"GlobalMapCA"],
    Cell[424250, 17825, 112, 5, 24, "Notes",
      CellTags->"GlobalMapCA"],
    Cell[424365, 17832, 142, 5, 24, "Notes",
      CellTags->"GlobalMapCA"],
    Cell[424510, 17839, 189, 6, 40, "Notes",
      CellTags->"GlobalMapCA"],
    Cell[424702, 17847, 164, 7, 24, "Notes",
      CellTags->"GlobalMapCA"],
    Cell[424869, 17856, 69, 1, 39, "ExampleSection",
      CellTags->"GlobalMapCA"],
    Cell[424941, 17859, 100, 1, 34, "ExampleText",
      CellTags->"GlobalMapCA"],
    Cell[425044, 17862, 108, 3, 26, "Input",
      CellTags->"GlobalMapCA"],
    Cell[425155, 17867, 148, 4, 26, "Input",
      CellTags->"GlobalMapCA"],
    Cell[425306, 17873, 127, 3, 26, "Input",
      CellTags->"GlobalMapCA"],
    Cell[425436, 17878, 89, 1, 34, "ExampleText",
      CellTags->"GlobalMapCA"],
    Cell[425528, 17881, 264, 5, 58, "Input",
      CellTags->"GlobalMapCA"]},
  "InverseCA"->{
    Cell[425795, 17888, 56, 1, 32, "ObjectName",
      CellTags->"InverseCA"],
    Cell[425854, 17891, 54, 1, 50, "Highlight",
      CellTags->"InverseCA"],
    Cell[425911, 17894, 180, 6, 31, "Usage",
      CellTags->"InverseCA"],
    Cell[426094, 17902, 292, 10, 61, "Notes",
      CellTags->"InverseCA"],
    Cell[426389, 17914, 147, 5, 24, "Notes",
      CellTags->"InverseCA"],
    Cell[426539, 17921, 201, 8, 24, "Notes",
      CellTags->"InverseCA"],
    Cell[426743, 17931, 196, 9, 24, "Notes",
      CellTags->"InverseCA"],
    Cell[426942, 17942, 67, 1, 39, "ExampleSection",
      CellTags->"InverseCA"],
    Cell[427012, 17945, 87, 1, 34, "ExampleText",
      CellTags->"InverseCA"],
    Cell[427102, 17948, 170, 5, 42, "Input",
      CellTags->"InverseCA"],
    Cell[427275, 17955, 141, 4, 34, "ExampleText",
      CellTags->"InverseCA"],
    Cell[427419, 17961, 198, 6, 58, "Input",
      CellTags->"InverseCA"]},
  "IterateCA"->{
    Cell[427620, 17969, 56, 1, 32, "ObjectName",
      CellTags->"IterateCA"],
    Cell[427679, 17972, 54, 1, 50, "Highlight",
      CellTags->"IterateCA"],
    Cell[427736, 17975, 222, 8, 31, "Usage",
      CellTags->"IterateCA"],
    Cell[427961, 17985, 249, 9, 44, "Notes",
      CellTags->"IterateCA"],
    Cell[428213, 17996, 383, 13, 40, "Notes",
      CellTags->"IterateCA"],
    Cell[428599, 18011, 67, 1, 39, "ExampleSection",
      CellTags->"IterateCA"],
    Cell[428669, 18014, 146, 4, 34, "ExampleText",
      CellTags->"IterateCA"],
    Cell[428818, 18020, 94, 2, 28, "Input",
      CellTags->"IterateCA"],
    Cell[428915, 18024, 144, 4, 42, "Input",
      CellTags->"IterateCA"]},
  "LocalMapCA"->{
    Cell[429062, 18030, 58, 1, 32, "ObjectName",
      CellTags->"LocalMapCA"],
    Cell[429123, 18033, 55, 1, 50, "Highlight",
      CellTags->"LocalMapCA"],
    Cell[429181, 18036, 213, 8, 31, "Usage",
      CellTags->"LocalMapCA"],
    Cell[429397, 18046, 229, 9, 44, "Notes",
      CellTags->"LocalMapCA"],
    Cell[429629, 18057, 111, 5, 24, "Notes",
      CellTags->"LocalMapCA"],
    Cell[429743, 18064, 141, 5, 24, "Notes",
      CellTags->"LocalMapCA"],
    Cell[429887, 18071, 188, 6, 40, "Notes",
      CellTags->"LocalMapCA"],
    Cell[430078, 18079, 165, 7, 24, "Notes",
      CellTags->"LocalMapCA"],
    Cell[430246, 18088, 68, 1, 39, "ExampleSection",
      CellTags->"LocalMapCA"],
    Cell[430317, 18091, 99, 1, 34, "ExampleText",
      CellTags->"LocalMapCA"],
    Cell[430419, 18094, 106, 3, 26, "Input",
      CellTags->"LocalMapCA"],
    Cell[430528, 18099, 123, 3, 26, "Input",
      CellTags->"LocalMapCA"],
    Cell[430654, 18104, 176, 4, 50, "ExampleText",
      CellTags->"LocalMapCA"],
    Cell[430833, 18110, 177, 4, 42, "Input",
      CellTags->"LocalMapCA"]},
  "NumberOfCA"->{
    Cell[431013, 18116, 58, 1, 32, "ObjectName",
      CellTags->"NumberOfCA"],
    Cell[431074, 18119, 55, 1, 50, "Highlight",
      CellTags->"NumberOfCA"],
    Cell[431132, 18122, 234, 8, 47, "Usage",
      CellTags->"NumberOfCA"],
    Cell[431369, 18132, 132, 6, 24, "Notes",
      CellTags->"NumberOfCA"],
    Cell[431504, 18140, 183, 7, 24, "Notes",
      CellTags->"NumberOfCA"],
    Cell[431690, 18149, 68, 1, 39, "ExampleSection",
      CellTags->"NumberOfCA"],
    Cell[431761, 18152, 75, 1, 34, "ExampleText",
      CellTags->"NumberOfCA"],
    Cell[431839, 18155, 116, 2, 28, "Input",
      CellTags->"NumberOfCA"]},
  "NumberOfTotalisticCA"->{
    Cell[431958, 18159, 78, 1, 32, "ObjectName",
      CellTags->"NumberOfTotalisticCA"],
    Cell[432039, 18162, 65, 1, 50, "Highlight",
      CellTags->"NumberOfTotalisticCA"],
    Cell[432107, 18165, 268, 8, 47, "Usage",
      CellTags->"NumberOfTotalisticCA"],
    Cell[432378, 18175, 150, 6, 24, "Notes",
      CellTags->"NumberOfTotalisticCA"],
    Cell[432531, 18183, 401, 15, 24, "Notes",
      CellTags->"NumberOfTotalisticCA"],
    Cell[432935, 18200, 78, 1, 39, "ExampleSection",
      CellTags->"NumberOfTotalisticCA"],
    Cell[433016, 18203, 79, 1, 34, "ExampleText",
      CellTags->"NumberOfTotalisticCA"],
    Cell[433098, 18206, 136, 2, 28, "Input",
      CellTags->"NumberOfTotalisticCA"]},
  "OrbitCA"->{
    Cell[433237, 18210, 52, 1, 32, "ObjectName",
      CellTags->"OrbitCA"],
    Cell[433292, 18213, 52, 1, 50, "Highlight",
      CellTags->"OrbitCA"],
    Cell[433347, 18216, 294, 10, 47, "Usage",
      CellTags->"OrbitCA"],
    Cell[433644, 18228, 340, 11, 79, "Notes",
      CellTags->"OrbitCA"],
    Cell[433987, 18241, 138, 5, 24, "Notes",
      CellTags->"OrbitCA"],
    Cell[434128, 18248, 196, 6, 24, "Notes",
      CellTags->"OrbitCA"],
    Cell[434327, 18256, 235, 8, 40, "Notes",
      CellTags->"OrbitCA"],
    Cell[434565, 18266, 351, 7, 56, "Notes",
      CellTags->"OrbitCA"],
    Cell[434919, 18275, 263, 11, 24, "Notes",
      CellTags->"OrbitCA"],
    Cell[435185, 18288, 65, 1, 39, "ExampleSection",
      CellTags->"OrbitCA"],
    Cell[435253, 18291, 128, 4, 34, "ExampleText",
      CellTags->"OrbitCA"],
    Cell[435384, 18297, 147, 5, 45, "Input",
      CellTags->"OrbitCA"],
    Cell[435534, 18304, 103, 3, 26, "Input",
      CellTags->"OrbitCA"],
    Cell[435640, 18309, 117, 4, 34, "ExampleText",
      CellTags->"OrbitCA"],
    Cell[435760, 18315, 133, 3, 26, "Input",
      CellTags->"OrbitCA"]},
  "Padding"->{
    Cell[435896, 18320, 52, 1, 32, "ObjectName",
      CellTags->"Padding"],
    Cell[435951, 18323, 50, 1, 50, "Highlight",
      CellTags->"Padding"],
    Cell[436004, 18326, 209, 6, 47, "Usage",
      CellTags->"Padding"],
    Cell[436216, 18334, 149, 5, 24, "Notes",
      CellTags->"Padding"],
    Cell[436368, 18341, 363, 15, 24, "Notes",
      CellTags->"Padding"],
    Cell[436734, 18358, 65, 1, 39, "ExampleSection",
      CellTags->"Padding"],
    Cell[436802, 18361, 125, 4, 34, "ExampleText",
      CellTags->"Padding"],
    Cell[436930, 18367, 110, 3, 26, "Input",
      CellTags->"Padding"],
    Cell[437043, 18372, 129, 3, 26, "Input",
      CellTags->"Padding"],
    Cell[437175, 18377, 136, 3, 26, "Input",
      CellTags->"Padding"]},
  "PolynomialToCA"->{
    Cell[437314, 18382, 66, 1, 32, "ObjectName",
      CellTags->"PolynomialToCA"],
    Cell[437383, 18385, 59, 1, 50, "Highlight",
      CellTags->"PolynomialToCA"],
    Cell[437445, 18388, 303, 10, 47, "Usage",
      CellTags->"PolynomialToCA"],
    Cell[437751, 18400, 271, 11, 24, "Notes",
      CellTags->"PolynomialToCA"],
    Cell[438025, 18413, 372, 15, 24, "Notes",
      CellTags->"PolynomialToCA"],
    Cell[438400, 18430, 72, 1, 39, "ExampleSection",
      CellTags->"PolynomialToCA"],
    Cell[438475, 18433, 146, 4, 34, "ExampleText",
      CellTags->"PolynomialToCA"],
    Cell[438624, 18439, 152, 4, 42, "Input",
      CellTags->"PolynomialToCA"]},
  "PrintCA"->{
    Cell[438779, 18445, 52, 1, 32, "ObjectName",
      CellTags->"PrintCA"],
    Cell[438834, 18448, 52, 1, 50, "Highlight",
      CellTags->"PrintCA"],
    Cell[438889, 18451, 209, 6, 47, "Usage",
      CellTags->"PrintCA"],
    Cell[439101, 18459, 129, 6, 24, "Notes",
      CellTags->"PrintCA"],
    Cell[439233, 18467, 65, 1, 39, "ExampleSection",
      CellTags->"PrintCA"],
    Cell[439301, 18470, 105, 2, 34, "ExampleText",
      CellTags->"PrintCA"],
    Cell[439409, 18474, 84, 2, 28, "Input",
      CellTags->"PrintCA"],
    Cell[439496, 18478, 90, 2, 28, "Input",
      CellTags->"PrintCA"]},
  "RuleCA"->{
    Cell[439589, 18482, 50, 1, 32, "ObjectName",
      CellTags->"RuleCA"],
    Cell[439642, 18485, 51, 1, 50, "Highlight",
      CellTags->"RuleCA"],
    Cell[439696, 18488, 166, 6, 31, "Usage",
      CellTags->"RuleCA"],
    Cell[439865, 18496, 488, 18, 40, "Notes",
      CellTags->"RuleCA"],
    Cell[440356, 18516, 106, 5, 24, "Notes",
      CellTags->"RuleCA"],
    Cell[440465, 18523, 235, 11, 24, "Notes",
      CellTags->"RuleCA"],
    Cell[440703, 18536, 64, 1, 39, "ExampleSection",
      CellTags->"RuleCA"],
    Cell[440770, 18539, 81, 1, 34, "ExampleText",
      CellTags->"RuleCA"],
    Cell[440854, 18542, 91, 5, 45, "Input",
      CellTags->"RuleCA"]},
  "RulesToCA"->{
    Cell[440948, 18549, 56, 1, 32, "ObjectName",
      CellTags->"RulesToCA"],
    Cell[441007, 18552, 54, 1, 50, "Highlight",
      CellTags->"RulesToCA"],
    Cell[441064, 18555, 312, 10, 47, "Usage",
      CellTags->"RulesToCA"],
    Cell[441379, 18567, 762, 29, 40, "Notes",
      CellTags->"RulesToCA"],
    Cell[442144, 18598, 245, 7, 40, "Notes",
      CellTags->"RulesToCA"],
    Cell[442392, 18607, 379, 15, 24, "Notes",
      CellTags->"RulesToCA"],
    Cell[442774, 18624, 67, 1, 39, "ExampleSection",
      CellTags->"RulesToCA"],
    Cell[442844, 18627, 150, 4, 34, "ExampleText",
      CellTags->"RulesToCA"],
    Cell[442997, 18633, 527, 11, 170, "Input",
      CellTags->"RulesToCA"],
    Cell[443527, 18646, 147, 4, 34, "ExampleText",
      CellTags->"RulesToCA"],
    Cell[443677, 18652, 223, 5, 42, "Input",
      CellTags->"RulesToCA"],
    Cell[443903, 18659, 275, 5, 42, "Input",
      CellTags->"RulesToCA"]},
  "SeedConfiguration"->{
    Cell[444181, 18666, 72, 1, 32, "ObjectName",
      CellTags->"SeedConfiguration"],
    Cell[444256, 18669, 62, 1, 50, "Highlight",
      CellTags->"SeedConfiguration"],
    Cell[444321, 18672, 234, 8, 31, "Usage",
      CellTags->"SeedConfiguration"],
    Cell[444558, 18682, 377, 11, 97, "Notes",
      CellTags->"SeedConfiguration"],
    Cell[444938, 18695, 690, 23, 106, "Notes",
      CellTags->"SeedConfiguration"],
    Cell[445631, 18720, 238, 7, 40, "Notes",
      CellTags->"SeedConfiguration"],
    Cell[445872, 18729, 109, 1, 24, "Notes",
      CellTags->"SeedConfiguration"],
    Cell[445984, 18732, 262, 11, 24, "Notes",
      CellTags->"SeedConfiguration"],
    Cell[446249, 18745, 75, 1, 39, "ExampleSection",
      CellTags->"SeedConfiguration"],
    Cell[446327, 18748, 94, 1, 34, "ExampleText",
      CellTags->"SeedConfiguration"],
    Cell[446424, 18751, 122, 3, 26, "Input",
      CellTags->"SeedConfiguration"],
    Cell[446549, 18756, 143, 3, 26, "Input",
      CellTags->"SeedConfiguration"],
    Cell[446695, 18761, 88, 1, 34, "ExampleText",
      CellTags->"SeedConfiguration"],
    Cell[446786, 18764, 141, 3, 26, "Input",
      CellTags->"SeedConfiguration"],
    Cell[446930, 18769, 146, 3, 26, "Input",
      CellTags->"SeedConfiguration"],
    Cell[447079, 18774, 204, 6, 34, "ExampleText",
      CellTags->"SeedConfiguration"],
    Cell[447286, 18782, 133, 3, 26, "Input",
      CellTags->"SeedConfiguration"],
    Cell[447422, 18787, 361, 7, 66, "ExampleText",
      CellTags->"SeedConfiguration"],
    Cell[447786, 18796, 151, 4, 26, "Input",
      CellTags->"SeedConfiguration"]},
  "SimplifyCA"->{
    Cell[447940, 18802, 58, 1, 32, "ObjectName",
      CellTags->"SimplifyCA"],
    Cell[448001, 18805, 55, 1, 50, "Highlight",
      CellTags->"SimplifyCA"],
    Cell[448059, 18808, 253, 7, 47, "Usage",
      CellTags->"SimplifyCA"],
    Cell[448315, 18817, 226, 9, 44, "Notes",
      CellTags->"SimplifyCA"],
    Cell[448544, 18828, 150, 5, 24, "Notes",
      CellTags->"SimplifyCA"],
    Cell[448697, 18835, 133, 6, 24, "Notes",
      CellTags->"SimplifyCA"],
    Cell[448833, 18843, 68, 1, 39, "ExampleSection",
      CellTags->"SimplifyCA"],
    Cell[448904, 18846, 237, 5, 50, "ExampleText",
      CellTags->"SimplifyCA"],
    Cell[449144, 18853, 106, 3, 26, "Input",
      CellTags->"SimplifyCA"],
    Cell[449253, 18858, 148, 4, 42, "Input",
      CellTags->"SimplifyCA"],
    Cell[449404, 18864, 163, 4, 50, "ExampleText",
      CellTags->"SimplifyCA"],
    Cell[449570, 18870, 247, 7, 74, "Input",
      CellTags->"SimplifyCA"],
    Cell[449820, 18879, 93, 3, 26, "Input",
      CellTags->"SimplifyCA"]},
  "ToRulesCA"->{
    Cell[449916, 18884, 56, 1, 32, "ObjectName",
      CellTags->"ToRulesCA"],
    Cell[449975, 18887, 54, 1, 50, "Highlight",
      CellTags->"ToRulesCA"],
    Cell[450032, 18890, 200, 5, 47, "Usage",
      CellTags->"ToRulesCA"],
    Cell[450235, 18897, 227, 9, 44, "Notes",
      CellTags->"ToRulesCA"],
    Cell[450465, 18908, 163, 5, 24, "Notes",
      CellTags->"ToRulesCA"],
    Cell[450631, 18915, 67, 1, 39, "ExampleSection",
      CellTags->"ToRulesCA"],
    Cell[450701, 18918, 165, 6, 34, "ExampleText",
      CellTags->"ToRulesCA"],
    Cell[450869, 18926, 101, 3, 26, "Input",
      CellTags->"ToRulesCA"],
    Cell[450973, 18931, 66, 1, 34, "ExampleText",
      CellTags->"ToRulesCA"],
    Cell[451042, 18934, 123, 3, 26, "Input",
      CellTags->"ToRulesCA"]},
  "TotalisticRuleToCA"->{
    Cell[451168, 18939, 74, 1, 32, "ObjectName",
      CellTags->"TotalisticRuleToCA"],
    Cell[451245, 18942, 63, 1, 50, "Highlight",
      CellTags->"TotalisticRuleToCA"],
    Cell[451311, 18945, 240, 8, 47, "Usage",
      CellTags->"TotalisticRuleToCA"],
    Cell[451554, 18955, 176, 6, 24, "Notes",
      CellTags->"TotalisticRuleToCA"],
    Cell[451733, 18963, 187, 7, 24, "Notes",
      CellTags->"TotalisticRuleToCA"],
    Cell[451923, 18972, 76, 1, 39, "ExampleSection",
      CellTags->"TotalisticRuleToCA"],
    Cell[452002, 18975, 133, 4, 34, "ExampleText",
      CellTags->"TotalisticRuleToCA"],
    Cell[452138, 18981, 100, 2, 28, "Input",
      CellTags->"TotalisticRuleToCA"],
    Cell[452241, 18985, 141, 3, 26, "Input",
      CellTags->"TotalisticRuleToCA"]},
  "ToTotalisticRuleCA"->{
    Cell[452385, 18990, 74, 1, 32, "ObjectName",
      CellTags->"ToTotalisticRuleCA"],
    Cell[452462, 18993, 63, 1, 50, "Highlight",
      CellTags->"ToTotalisticRuleCA"],
    Cell[452528, 18996, 225, 8, 31, "Usage",
      CellTags->"ToTotalisticRuleCA"],
    Cell[452756, 19006, 147, 5, 24, "Notes",
      CellTags->"ToTotalisticRuleCA"],
    Cell[452906, 19013, 187, 7, 24, "Notes",
      CellTags->"ToTotalisticRuleCA"],
    Cell[453096, 19022, 76, 1, 39, "ExampleSection",
      CellTags->"ToTotalisticRuleCA"],
    Cell[453175, 19025, 80, 1, 34, "ExampleText",
      CellTags->"ToTotalisticRuleCA"],
    Cell[453258, 19028, 108, 2, 28, "Input",
      CellTags->"ToTotalisticRuleCA"]},
  "WelchCA"->{
    Cell[453369, 19032, 52, 1, 32, "ObjectName",
      CellTags->"WelchCA"],
    Cell[453424, 19035, 52, 1, 50, "Highlight",
      CellTags->"WelchCA"],
    Cell[453479, 19038, 183, 6, 31, "Usage",
      CellTags->"WelchCA"],
    Cell[453665, 19046, 290, 10, 61, "Notes",
      CellTags->"WelchCA"],
    Cell[453958, 19058, 142, 5, 24, "Notes",
      CellTags->"WelchCA"],
    Cell[454103, 19065, 167, 6, 24, "Notes",
      CellTags->"WelchCA"],
    Cell[454273, 19073, 125, 6, 24, "Notes",
      CellTags->"WelchCA"],
    Cell[454401, 19081, 65, 1, 39, "ExampleSection",
      CellTags->"WelchCA"],
    Cell[454469, 19084, 129, 4, 34, "ExampleText",
      CellTags->"WelchCA"],
    Cell[454601, 19090, 179, 4, 42, "Input",
      CellTags->"WelchCA"],
    Cell[454783, 19096, 101, 2, 28, "Input",
      CellTags->"WelchCA"],
    Cell[454887, 19100, 82, 1, 34, "ExampleText",
      CellTags->"WelchCA"],
    Cell[454972, 19103, 116, 3, 26, "Input",
      CellTags->"WelchCA"],
    Cell[455091, 19108, 102, 3, 26, "Input",
      CellTags->"WelchCA"]},
  "Where"->{
    Cell[455196, 19113, 48, 1, 32, "ObjectName",
      CellTags->"Where"],
    Cell[455247, 19116, 48, 1, 50, "Highlight",
      CellTags->"Where"],
    Cell[455298, 19119, 192, 6, 31, "Usage",
      CellTags->"Where"],
    Cell[455493, 19127, 197, 9, 24, "Notes",
      CellTags->"Where"],
    Cell[455693, 19138, 361, 15, 24, "Notes",
      CellTags->"Where"],
    Cell[456057, 19155, 63, 1, 39, "ExampleSection",
      CellTags->"Where"],
    Cell[456123, 19158, 123, 4, 34, "ExampleText",
      CellTags->"Where"],
    Cell[456249, 19164, 90, 2, 28, "Input",
      CellTags->"Where"],
    Cell[456342, 19168, 102, 2, 28, "Input",
      CellTags->"Where"],
    Cell[456447, 19172, 103, 2, 28, "Input",
      CellTags->"Where"]},
  "WidthCA"->{
    Cell[456553, 19176, 52, 1, 32, "ObjectName",
      CellTags->"WidthCA"],
    Cell[456608, 19179, 52, 1, 50, "Highlight",
      CellTags->"WidthCA"],
    Cell[456663, 19182, 162, 6, 31, "Usage",
      CellTags->"WidthCA"],
    Cell[456828, 19190, 107, 5, 24, "Notes",
      CellTags->"WidthCA"],
    Cell[456938, 19197, 234, 11, 24, "Notes",
      CellTags->"WidthCA"],
    Cell[457175, 19210, 65, 1, 39, "ExampleSection",
      CellTags->"WidthCA"],
    Cell[457243, 19213, 123, 4, 34, "ExampleText",
      CellTags->"WidthCA"],
    Cell[457369, 19219, 91, 5, 45, "Input",
      CellTags->"WidthCA"]},
  "WordMap"->{
    Cell[457463, 19226, 52, 1, 32, "ObjectName",
      CellTags->"WordMap"],
    Cell[457518, 19229, 50, 1, 50, "Highlight",
      CellTags->"WordMap"],
    Cell[457571, 19232, 158, 4, 31, "Usage",
      CellTags->"WordMap"],
    Cell[457732, 19238, 332, 14, 108, "Notes",
      CellTags->"WordMap"],
    Cell[458067, 19254, 65, 1, 39, "ExampleSection",
      CellTags->"WordMap"],
    Cell[458135, 19257, 161, 4, 50, "ExampleText",
      CellTags->"WordMap"],
    Cell[458299, 19263, 124, 3, 26, "Input",
      CellTags->"WordMap"],
    Cell[458426, 19268, 87, 3, 26, "Input",
      CellTags->"WordMap"]},
  "AddEdge"->{
    Cell[458516, 19273, 52, 1, 32, "ObjectName",
      CellTags->"AddEdge"],
    Cell[458571, 19276, 52, 1, 50, "Highlight",
      CellTags->"AddEdge"],
    Cell[458626, 19279, 172, 8, 31, "Usage",
      CellTags->"AddEdge"],
    Cell[458801, 19289, 232, 8, 40, "Notes",
      CellTags->"AddEdge"],
    Cell[459036, 19299, 324, 15, 24, "Notes",
      CellTags->"AddEdge"],
    Cell[459363, 19316, 65, 1, 39, "ExampleSection",
      CellTags->"AddEdge"],
    Cell[459431, 19319, 92, 1, 34, "ExampleText",
      CellTags->"AddEdge"],
    Cell[459526, 19322, 81, 2, 28, "Input",
      CellTags->"AddEdge"],
    Cell[459610, 19326, 103, 3, 26, "Input",
      CellTags->"AddEdge"],
    Cell[459716, 19331, 111, 3, 26, "Input",
      CellTags->"AddEdge"]},
  "AddVertex"->{
    Cell[459830, 19336, 56, 1, 32, "ObjectName",
      CellTags->"AddVertex"],
    Cell[459889, 19339, 54, 1, 50, "Highlight",
      CellTags->"AddVertex"],
    Cell[459946, 19342, 152, 6, 31, "Usage",
      CellTags->"AddVertex"],
    Cell[460101, 19350, 177, 7, 24, "Notes",
      CellTags->"AddVertex"],
    Cell[460281, 19359, 286, 12, 24, "Notes",
      CellTags->"AddVertex"],
    Cell[460570, 19373, 324, 15, 24, "Notes",
      CellTags->"AddVertex"],
    Cell[460897, 19390, 67, 1, 39, "ExampleSection",
      CellTags->"AddVertex"],
    Cell[460967, 19393, 98, 1, 34, "ExampleText",
      CellTags->"AddVertex"],
    Cell[461068, 19396, 117, 3, 26, "Input",
      CellTags->"AddVertex"],
    Cell[461188, 19401, 154, 4, 26, "Input",
      CellTags->"AddVertex"],
    Cell[461345, 19407, 159, 4, 42, "Input",
      CellTags->"AddVertex"]},
  "BiAccessibleGraph"->{
    Cell[461507, 19413, 72, 1, 32, "ObjectName",
      CellTags->"BiAccessibleGraph"],
    Cell[461582, 19416, 62, 1, 50, "Highlight",
      CellTags->"BiAccessibleGraph"],
    Cell[461647, 19419, 291, 10, 47, "Usage",
      CellTags->"BiAccessibleGraph"],
    Cell[461941, 19431, 127, 4, 24, "Notes",
      CellTags->"BiAccessibleGraph"],
    Cell[462071, 19437, 185, 6, 24, "Notes",
      CellTags->"BiAccessibleGraph"],
    Cell[462259, 19445, 480, 20, 24, "Notes",
      CellTags->"BiAccessibleGraph"],
    Cell[462742, 19467, 75, 1, 39, "ExampleSection",
      CellTags->"BiAccessibleGraph"],
    Cell[462820, 19470, 152, 4, 34, "ExampleText",
      CellTags->"BiAccessibleGraph"],
    Cell[462975, 19476, 222, 6, 42, "Input",
      CellTags->"BiAccessibleGraph"],
    Cell[463200, 19484, 104, 2, 28, "Input",
      CellTags->"BiAccessibleGraph"],
    Cell[463307, 19488, 141, 4, 34, "ExampleText",
      CellTags->"BiAccessibleGraph"],
    Cell[463451, 19494, 125, 5, 45, "Input",
      CellTags->"BiAccessibleGraph"]},
  "AccessibleGraph"->{
    Cell[463579, 19501, 68, 1, 32, "ObjectName",
      CellTags->"AccessibleGraph"],
    Cell[463650, 19504, 60, 1, 50, "Highlight",
      CellTags->"AccessibleGraph"],
    Cell[463713, 19507, 330, 11, 47, "Usage",
      CellTags->"AccessibleGraph"],
    Cell[464046, 19520, 137, 4, 24, "Notes",
      CellTags->"AccessibleGraph"],
    Cell[464186, 19526, 458, 19, 24, "Notes",
      CellTags->"AccessibleGraph"],
    Cell[464647, 19547, 73, 1, 39, "ExampleSection",
      CellTags->"AccessibleGraph"],
    Cell[464723, 19550, 73, 1, 34, "ExampleText",
      CellTags->"AccessibleGraph"],
    Cell[464799, 19553, 103, 2, 28, "Input",
      CellTags->"AccessibleGraph"],
    Cell[464905, 19557, 104, 1, 34, "ExampleText",
      CellTags->"AccessibleGraph"],
    Cell[465012, 19560, 115, 5, 45, "Input",
      CellTags->"AccessibleGraph"],
    Cell[465130, 19567, 92, 1, 34, "ExampleText",
      CellTags->"AccessibleGraph"],
    Cell[465225, 19570, 111, 2, 28, "Input",
      CellTags->"AccessibleGraph"]},
  "CirculantGraph"->{
    Cell[465339, 19574, 66, 1, 32, "ObjectName",
      CellTags->"CirculantGraph"],
    Cell[465408, 19577, 59, 1, 50, "Highlight",
      CellTags->"CirculantGraph"],
    Cell[465470, 19580, 246, 8, 47, "Usage",
      CellTags->"CirculantGraph"],
    Cell[465719, 19590, 315, 10, 40, "Notes",
      CellTags->"CirculantGraph"],
    Cell[466037, 19602, 156, 6, 24, "Notes",
      CellTags->"CirculantGraph"],
    Cell[466196, 19610, 88, 1, 24, "Notes",
      CellTags->"CirculantGraph"],
    Cell[466287, 19613, 251, 11, 24, "Notes",
      CellTags->"CirculantGraph"],
    Cell[466541, 19626, 72, 1, 39, "ExampleSection",
      CellTags->"CirculantGraph"],
    Cell[466616, 19629, 84, 1, 34, "ExampleText",
      CellTags->"CirculantGraph"],
    Cell[466703, 19632, 101, 2, 28, "Input",
      CellTags->"CirculantGraph"],
    Cell[466807, 19636, 119, 3, 26, "Input",
      CellTags->"CirculantGraph"],
    Cell[466929, 19641, 95, 1, 34, "ExampleText",
      CellTags->"CirculantGraph"],
    Cell[467027, 19644, 101, 2, 28, "Input",
      CellTags->"CirculantGraph"],
    Cell[467131, 19648, 119, 3, 26, "Input",
      CellTags->"CirculantGraph"]},
  "CyclicQGraph"->{
    Cell[467253, 19653, 62, 1, 32, "ObjectName",
      CellTags->"CyclicQGraph"],
    Cell[467318, 19656, 57, 1, 50, "Highlight",
      CellTags->"CyclicQGraph"],
    Cell[467378, 19659, 160, 6, 31, "Usage",
      CellTags->"CyclicQGraph"],
    Cell[467541, 19667, 60, 1, 24, "Notes",
      CellTags->"CyclicQGraph"],
    Cell[467604, 19670, 155, 7, 24, "Notes",
      CellTags->"CyclicQGraph"],
    Cell[467762, 19679, 70, 1, 39, "ExampleSection",
      CellTags->"CyclicQGraph"],
    Cell[467835, 19682, 66, 1, 34, "ExampleText",
      CellTags->"CyclicQGraph"],
    Cell[467904, 19685, 221, 6, 42, "Input",
      CellTags->"CyclicQGraph"],
    Cell[468128, 19693, 102, 3, 26, "Input",
      CellTags->"CyclicQGraph"],
    Cell[468233, 19698, 114, 3, 26, "Input",
      CellTags->"CyclicQGraph"],
    Cell[468350, 19703, 103, 3, 26, "Input",
      CellTags->"CyclicQGraph"]},
  "DeBruijnEdges"->{
    Cell[468456, 19708, 64, 1, 32, "ObjectName",
      CellTags->"DeBruijnEdges"],
    Cell[468523, 19711, 58, 1, 50, "Highlight",
      CellTags->"DeBruijnEdges"],
    Cell[468584, 19714, 180, 5, 31, "Usage",
      CellTags->"DeBruijnEdges"],
    Cell[468767, 19721, 230, 9, 44, "Notes",
      CellTags->"DeBruijnEdges"],
    Cell[469000, 19732, 155, 6, 24, "Notes",
      CellTags->"DeBruijnEdges"],
    Cell[469158, 19740, 209, 7, 24, "Notes",
      CellTags->"DeBruijnEdges"],
    Cell[469370, 19749, 172, 7, 24, "Notes",
      CellTags->"DeBruijnEdges"],
    Cell[469545, 19758, 71, 1, 39, "ExampleSection",
      CellTags->"DeBruijnEdges"],
    Cell[469619, 19761, 175, 6, 34, "ExampleText",
      CellTags->"DeBruijnEdges"],
    Cell[469797, 19769, 112, 3, 26, "Input",
      CellTags->"DeBruijnEdges"],
    Cell[469912, 19774, 84, 1, 34, "ExampleText",
      CellTags->"DeBruijnEdges"],
    Cell[469999, 19777, 133, 3, 26, "Input",
      CellTags->"DeBruijnEdges"]},
  "DeBruijnGraph"->{
    Cell[470135, 19782, 64, 1, 32, "ObjectName",
      CellTags->"DeBruijnGraph"],
    Cell[470202, 19785, 58, 1, 50, "Highlight",
      CellTags->"DeBruijnGraph"],
    Cell[470263, 19788, 161, 4, 31, "Usage",
      CellTags->"DeBruijnGraph"],
    Cell[470427, 19794, 230, 9, 44, "Notes",
      CellTags->"DeBruijnGraph"],
    Cell[470660, 19805, 155, 6, 24, "Notes",
      CellTags->"DeBruijnGraph"],
    Cell[470818, 19813, 839, 30, 56, "Notes",
      CellTags->"DeBruijnGraph"],
    Cell[471660, 19845, 172, 7, 24, "Notes",
      CellTags->"DeBruijnGraph"],
    Cell[471835, 19854, 71, 1, 39, "ExampleSection",
      CellTags->"DeBruijnGraph"],
    Cell[471909, 19857, 165, 6, 34, "ExampleText",
      CellTags->"DeBruijnGraph"],
    Cell[472077, 19865, 141, 3, 26, "Input",
      CellTags->"DeBruijnGraph"],
    Cell[472221, 19870, 100, 3, 26, "Input",
      CellTags->"DeBruijnGraph"],
    Cell[472324, 19875, 150, 4, 34, "ExampleText",
      CellTags->"DeBruijnGraph"],
    Cell[472477, 19881, 182, 4, 42, "Input",
      CellTags->"DeBruijnGraph"]},
  "DegreeGraph"->{
    Cell[472662, 19887, 60, 1, 32, "ObjectName",
      CellTags->"DegreeGraph"],
    Cell[472725, 19890, 56, 1, 50, "Highlight",
      CellTags->"DegreeGraph"],
    Cell[472784, 19893, 199, 8, 31, "Usage",
      CellTags->"DegreeGraph"],
    Cell[472986, 19903, 227, 9, 44, "Notes",
      CellTags->"DegreeGraph"],
    Cell[473216, 19914, 182, 7, 24, "Notes",
      CellTags->"DegreeGraph"],
    Cell[473401, 19923, 150, 5, 24, "Notes",
      CellTags->"DegreeGraph"],
    Cell[473554, 19930, 154, 7, 24, "Notes",
      CellTags->"DegreeGraph"],
    Cell[473711, 19939, 69, 1, 39, "ExampleSection",
      CellTags->"DegreeGraph"],
    Cell[473783, 19942, 135, 4, 34, "ExampleText",
      CellTags->"DegreeGraph"],
    Cell[473921, 19948, 97, 2, 28, "Input",
      CellTags->"DegreeGraph"],
    Cell[474021, 19952, 103, 3, 26, "Input",
      CellTags->"DegreeGraph"],
    Cell[474127, 19957, 124, 3, 26, "Input",
      CellTags->"DegreeGraph"],
    Cell[474254, 19962, 121, 3, 26, "Input",
      CellTags->"DegreeGraph"]},
  "DeleteEdge"->{
    Cell[474378, 19967, 58, 1, 32, "ObjectName",
      CellTags->"DeleteEdge"],
    Cell[474439, 19970, 55, 1, 50, "Highlight",
      CellTags->"DeleteEdge"],
    Cell[474497, 19973, 183, 8, 31, "Usage",
      CellTags->"DeleteEdge"],
    Cell[474683, 19983, 153, 5, 24, "Notes",
      CellTags->"DeleteEdge"],
    Cell[474839, 19990, 324, 15, 24, "Notes",
      CellTags->"DeleteEdge"],
    Cell[475166, 20007, 68, 1, 39, "ExampleSection",
      CellTags->"DeleteEdge"],
    Cell[475237, 20010, 110, 2, 34, "ExampleText",
      CellTags->"DeleteEdge"],
    Cell[475350, 20014, 85, 2, 28, "Input",
      CellTags->"DeleteEdge"],
    Cell[475438, 20018, 162, 4, 42, "Input",
      CellTags->"DeleteEdge"]},
  "DeleteVertex"->{
    Cell[475603, 20024, 62, 1, 32, "ObjectName",
      CellTags->"DeleteVertex"],
    Cell[475668, 20027, 57, 1, 50, "Highlight",
      CellTags->"DeleteVertex"],
    Cell[475728, 20030, 211, 9, 31, "Usage",
      CellTags->"DeleteVertex"],
    Cell[475942, 20041, 135, 5, 24, "Notes",
      CellTags->"DeleteVertex"],
    Cell[476080, 20048, 324, 15, 24, "Notes",
      CellTags->"DeleteVertex"],
    Cell[476407, 20065, 70, 1, 39, "ExampleSection",
      CellTags->"DeleteVertex"],
    Cell[476480, 20068, 113, 2, 34, "ExampleText",
      CellTags->"DeleteVertex"],
    Cell[476596, 20072, 87, 2, 28, "Input",
      CellTags->"DeleteVertex"],
    Cell[476686, 20076, 114, 3, 26, "Input",
      CellTags->"DeleteVertex"],
    Cell[476803, 20081, 168, 4, 42, "Input",
      CellTags->"DeleteVertex"]},
  "EdgeQ"->{
    Cell[476974, 20087, 48, 1, 32, "ObjectName",
      CellTags->"EdgeQ"],
    Cell[477025, 20090, 50, 1, 50, "Highlight",
      CellTags->"EdgeQ"],
    Cell[477078, 20093, 208, 8, 31, "Usage",
      CellTags->"EdgeQ"],
    Cell[477289, 20103, 154, 7, 24, "Notes",
      CellTags->"EdgeQ"],
    Cell[477446, 20112, 128, 5, 24, "Notes",
      CellTags->"EdgeQ"],
    Cell[477577, 20119, 148, 7, 24, "Notes",
      CellTags->"EdgeQ"]},
  "Edges"->{
    Cell[477728, 20128, 48, 1, 32, "ObjectName",
      CellTags->"Edges"],
    Cell[477779, 20131, 50, 1, 50, "Highlight",
      CellTags->"Edges"],
    Cell[477832, 20134, 116, 4, 31, "Usage",
      CellTags->"Edges"],
    Cell[477951, 20140, 166, 4, 40, "Notes",
      CellTags->"Edges"],
    Cell[478120, 20146, 255, 11, 24, "Notes",
      CellTags->"Edges"],
    Cell[478378, 20159, 63, 1, 39, "ExampleSection",
      CellTags->"Edges"],
    Cell[478444, 20162, 75, 1, 34, "ExampleText",
      CellTags->"Edges"],
    Cell[478522, 20165, 91, 2, 28, "Input",
      CellTags->"Edges"],
    Cell[478616, 20169, 88, 3, 26, "Input",
      CellTags->"Edges"]},
  "Graph"->{
    Cell[478707, 20174, 48, 1, 32, "ObjectName",
      CellTags->"Graph"],
    Cell[478758, 20177, 53, 1, 50, "Highlight",
      CellTags->"Graph"],
    Cell[478814, 20180, 297, 10, 47, "Usage",
      CellTags->"Graph"],
    Cell[479114, 20192, 188, 5, 40, "Notes",
      CellTags->"Graph"],
    Cell[479305, 20199, 90, 1, 24, "Notes",
      CellTags->"Graph"],
    Cell[479398, 20202, 370, 12, 40, "Notes",
      CellTags->"Graph"],
    Cell[479771, 20216, 602, 23, 40, "Notes",
      CellTags->"Graph"],
    Cell[480376, 20241, 63, 1, 39, "ExampleSection",
      CellTags->"Graph"],
    Cell[480442, 20244, 96, 1, 34, "ExampleText",
      CellTags->"Graph"],
    Cell[480541, 20247, 167, 5, 26, "Input",
      CellTags->"Graph"],
    Cell[480711, 20254, 119, 4, 42, "Input",
      CellTags->"Graph"],
    Cell[480833, 20260, 123, 4, 34, "ExampleText",
      CellTags->"Graph"],
    Cell[480959, 20266, 100, 3, 26, "Input",
      CellTags->"Graph"],
    Cell[481062, 20271, 98, 3, 26, "Input",
      CellTags->"Graph"],
    Cell[481163, 20276, 70, 1, 34, "ExampleText",
      CellTags->"Graph"],
    Cell[481236, 20279, 111, 3, 26, "Input",
      CellTags->"Graph"],
    Cell[481350, 20284, 110, 3, 26, "Input",
      CellTags->"Graph"]},
  "GridGraph"->{
    Cell[481463, 20289, 56, 1, 32, "ObjectName",
      CellTags->"GridGraph"],
    Cell[481522, 20292, 57, 1, 50, "Highlight",
      CellTags->"GridGraph"],
    Cell[481582, 20295, 206, 8, 31, "Usage",
      CellTags->"GridGraph"],
    Cell[481791, 20305, 224, 9, 44, "Notes",
      CellTags->"GridGraph"],
    Cell[482018, 20316, 156, 5, 24, "Notes",
      CellTags->"GridGraph"],
    Cell[482177, 20323, 150, 5, 24, "Notes",
      CellTags->"GridGraph"],
    Cell[482330, 20330, 188, 6, 24, "Notes",
      CellTags->"GridGraph"],
    Cell[482521, 20338, 256, 11, 24, "Notes",
      CellTags->"GridGraph"],
    Cell[482780, 20351, 67, 1, 39, "ExampleSection",
      CellTags->"GridGraph"],
    Cell[482850, 20354, 104, 1, 34, "ExampleText",
      CellTags->"GridGraph"],
    Cell[482957, 20357, 104, 3, 26, "Input",
      CellTags->"GridGraph"],
    Cell[483064, 20362, 105, 3, 26, "Input",
      CellTags->"GridGraph"],
    Cell[483172, 20367, 127, 3, 26, "Input",
      CellTags->"GridGraph"],
    Cell[483302, 20372, 108, 2, 34, "ExampleText",
      CellTags->"GridGraph"],
    Cell[483413, 20376, 169, 4, 26, "Input",
      CellTags->"GridGraph"]},
  "LabelEdgeList"->{
    Cell[483585, 20382, 64, 1, 32, "ObjectName",
      CellTags->"LabelEdgeList"],
    Cell[483652, 20385, 58, 1, 50, "Highlight",
      CellTags->"LabelEdgeList"],
    Cell[483713, 20388, 181, 6, 31, "Usage",
      CellTags->"LabelEdgeList"],
    Cell[483897, 20396, 96, 1, 24, "Notes",
      CellTags->"LabelEdgeList"],
    Cell[483996, 20399, 245, 11, 24, "Notes",
      CellTags->"LabelEdgeList"],
    Cell[484244, 20412, 71, 1, 39, "ExampleSection",
      CellTags->"LabelEdgeList"],
    Cell[484318, 20415, 89, 1, 34, "ExampleText",
      CellTags->"LabelEdgeList"],
    Cell[484410, 20418, 88, 2, 28, "Input",
      CellTags->"LabelEdgeList"],
    Cell[484501, 20422, 117, 3, 26, "Input",
      CellTags->"LabelEdgeList"]},
  "LongestPath"->{
    Cell[484621, 20427, 60, 1, 32, "ObjectName",
      CellTags->"LongestPath"],
    Cell[484684, 20430, 56, 1, 50, "Highlight",
      CellTags->"LongestPath"],
    Cell[484743, 20433, 189, 6, 31, "Usage",
      CellTags->"LongestPath"],
    Cell[484935, 20441, 227, 9, 44, "Notes",
      CellTags->"LongestPath"],
    Cell[485165, 20452, 166, 5, 24, "Notes",
      CellTags->"LongestPath"],
    Cell[485334, 20459, 151, 4, 24, "Notes",
      CellTags->"LongestPath"],
    Cell[485488, 20465, 204, 5, 40, "Notes",
      CellTags->"LongestPath"],
    Cell[485695, 20472, 320, 11, 24, "Notes",
      CellTags->"LongestPath"],
    Cell[486018, 20485, 69, 1, 39, "ExampleSection",
      CellTags->"LongestPath"],
    Cell[486090, 20488, 72, 1, 34, "ExampleText",
      CellTags->"LongestPath"],
    Cell[486165, 20491, 208, 6, 42, "Input",
      CellTags->"LongestPath"],
    Cell[486376, 20499, 104, 3, 26, "Input",
      CellTags->"LongestPath"],
    Cell[486483, 20504, 86, 1, 34, "ExampleText",
      CellTags->"LongestPath"],
    Cell[486572, 20507, 121, 3, 26, "Input",
      CellTags->"LongestPath"]},
  "NormalizeGraph"->{
    Cell[486696, 20512, 66, 1, 32, "ObjectName",
      CellTags->"NormalizeGraph"],
    Cell[486765, 20515, 59, 1, 50, "Highlight",
      CellTags->"NormalizeGraph"],
    Cell[486827, 20518, 174, 6, 31, "Usage",
      CellTags->"NormalizeGraph"],
    Cell[487004, 20526, 201, 5, 40, "Notes",
      CellTags->"NormalizeGraph"],
    Cell[487208, 20533, 130, 4, 24, "Notes",
      CellTags->"NormalizeGraph"],
    Cell[487341, 20539, 355, 15, 24, "Notes",
      CellTags->"NormalizeGraph"],
    Cell[487699, 20556, 72, 1, 39, "ExampleSection",
      CellTags->"NormalizeGraph"],
    Cell[487774, 20559, 179, 5, 34, "ExampleText",
      CellTags->"NormalizeGraph"],
    Cell[487956, 20566, 216, 6, 42, "Input",
      CellTags->"NormalizeGraph"],
    Cell[488175, 20574, 147, 4, 42, "Input",
      CellTags->"NormalizeGraph"]},
  "NonTransientGraph"->{
    Cell[488325, 20580, 72, 1, 32, "ObjectName",
      CellTags->"NonTransientGraph"],
    Cell[488400, 20583, 62, 1, 50, "Highlight",
      CellTags->"NonTransientGraph"],
    Cell[488465, 20586, 190, 6, 31, "Usage",
      CellTags->"NonTransientGraph"],
    Cell[488658, 20594, 234, 9, 44, "Notes",
      CellTags->"NonTransientGraph"],
    Cell[488895, 20605, 261, 6, 40, "Notes",
      CellTags->"NonTransientGraph"],
    Cell[489159, 20613, 143, 2, 24, "Notes",
      CellTags->"NonTransientGraph"],
    Cell[489305, 20617, 416, 15, 24, "Notes",
      CellTags->"NonTransientGraph"],
    Cell[489724, 20634, 75, 1, 39, "ExampleSection",
      CellTags->"NonTransientGraph"],
    Cell[489802, 20637, 94, 1, 34, "ExampleText",
      CellTags->"NonTransientGraph"],
    Cell[489899, 20640, 247, 6, 42, "Input",
      CellTags->"NonTransientGraph"],
    Cell[490149, 20648, 143, 4, 42, "Input",
      CellTags->"NonTransientGraph"]},
  "NonTrivial"->{
    Cell[490295, 20654, 58, 1, 32, "ObjectName",
      CellTags->"NonTrivial"],
    Cell[490356, 20657, 53, 1, 50, "Highlight",
      CellTags->"NonTrivial"],
    Cell[490412, 20660, 175, 6, 31, "Usage",
      CellTags->"NonTrivial"],
    Cell[490590, 20668, 170, 4, 40, "Notes",
      CellTags->"NonTrivial"],
    Cell[490763, 20674, 192, 7, 24, "Notes",
      CellTags->"NonTrivial"]},
  "NumEdges"->{
    Cell[490958, 20683, 54, 1, 32, "ObjectName",
      CellTags->"NumEdges"],
    Cell[491015, 20686, 53, 1, 50, "Highlight",
      CellTags->"NumEdges"],
    Cell[491071, 20689, 161, 6, 31, "Usage",
      CellTags->"NumEdges"],
    Cell[491235, 20697, 338, 15, 24, "Notes",
      CellTags->"NumEdges"],
    Cell[491576, 20714, 66, 1, 39, "ExampleSection",
      CellTags->"NumEdges"],
    Cell[491645, 20717, 77, 1, 34, "ExampleText",
      CellTags->"NumEdges"],
    Cell[491725, 20720, 128, 3, 26, "Input",
      CellTags->"NumEdges"],
    Cell[491856, 20725, 112, 3, 26, "Input",
      CellTags->"NumEdges"]},
  "NumVertices"->{
    Cell[491971, 20730, 60, 1, 32, "ObjectName",
      CellTags->"NumVertices"],
    Cell[492034, 20733, 56, 1, 50, "Highlight",
      CellTags->"NumVertices"],
    Cell[492093, 20736, 170, 6, 31, "Usage",
      CellTags->"NumVertices"],
    Cell[492266, 20744, 335, 15, 24, "Notes",
      CellTags->"NumVertices"],
    Cell[492604, 20761, 69, 1, 39, "ExampleSection",
      CellTags->"NumVertices"],
    Cell[492676, 20764, 83, 1, 34, "ExampleText",
      CellTags->"NumVertices"],
    Cell[492762, 20767, 134, 3, 26, "Input",
      CellTags->"NumVertices"],
    Cell[492899, 20772, 118, 3, 26, "Input",
      CellTags->"NumVertices"]},
  "PathGraph"->{
    Cell[493020, 20777, 56, 1, 32, "ObjectName",
      CellTags->"PathGraph"],
    Cell[493079, 20780, 54, 1, 50, "Highlight",
      CellTags->"PathGraph"],
    Cell[493136, 20783, 170, 6, 31, "Usage",
      CellTags->"PathGraph"],
    Cell[493309, 20791, 225, 9, 44, "Notes",
      CellTags->"PathGraph"],
    Cell[493537, 20802, 141, 5, 24, "Notes",
      CellTags->"PathGraph"],
    Cell[493681, 20809, 256, 11, 24, "Notes",
      CellTags->"PathGraph"],
    Cell[493940, 20822, 67, 1, 39, "ExampleSection",
      CellTags->"PathGraph"],
    Cell[494010, 20825, 74, 1, 34, "ExampleText",
      CellTags->"PathGraph"],
    Cell[494087, 20828, 101, 3, 26, "Input",
      CellTags->"PathGraph"],
    Cell[494191, 20833, 115, 3, 26, "Input",
      CellTags->"PathGraph"],
    Cell[494309, 20838, 84, 1, 34, "ExampleText",
      CellTags->"PathGraph"],
    Cell[494396, 20841, 120, 3, 26, "Input",
      CellTags->"PathGraph"],
    Cell[494519, 20846, 139, 3, 26, "Input",
      CellTags->"PathGraph"]},
  "PermuteGraph"->{
    Cell[494661, 20851, 62, 1, 32, "ObjectName",
      CellTags->"PermuteGraph"],
    Cell[494726, 20854, 57, 1, 50, "Highlight",
      CellTags->"PermuteGraph"],
    Cell[494786, 20857, 186, 8, 31, "Usage",
      CellTags->"PermuteGraph"],
    Cell[494975, 20867, 125, 5, 24, "Notes",
      CellTags->"PermuteGraph"],
    Cell[495103, 20874, 155, 7, 24, "Notes",
      CellTags->"PermuteGraph"],
    Cell[495261, 20883, 70, 1, 39, "ExampleSection",
      CellTags->"PermuteGraph"],
    Cell[495334, 20886, 126, 4, 34, "ExampleText",
      CellTags->"PermuteGraph"],
    Cell[495463, 20892, 87, 2, 28, "Input",
      CellTags->"PermuteGraph"],
    Cell[495553, 20896, 131, 3, 26, "Input",
      CellTags->"PermuteGraph"]},
  "ProductGraph"->{
    Cell[495687, 20901, 62, 1, 32, "ObjectName",
      CellTags->"ProductGraph"],
    Cell[495752, 20904, 57, 1, 50, "Highlight",
      CellTags->"ProductGraph"],
    Cell[495812, 20907, 216, 8, 31, "Usage",
      CellTags->"ProductGraph"],
    Cell[496031, 20917, 273, 10, 61, "Notes",
      CellTags->"ProductGraph"],
    Cell[496307, 20929, 604, 22, 40, "Notes",
      CellTags->"ProductGraph"],
    Cell[496914, 20953, 320, 11, 24, "Notes",
      CellTags->"ProductGraph"],
    Cell[497237, 20966, 305, 11, 24, "Notes",
      CellTags->"ProductGraph"],
    Cell[497545, 20979, 70, 1, 39, "ExampleSection",
      CellTags->"ProductGraph"],
    Cell[497618, 20982, 98, 1, 34, "ExampleText",
      CellTags->"ProductGraph"],
    Cell[497719, 20985, 161, 4, 42, "Input",
      CellTags->"ProductGraph"],
    Cell[497883, 20991, 112, 3, 26, "Input",
      CellTags->"ProductGraph"],
    Cell[497998, 20996, 107, 1, 34, "ExampleText",
      CellTags->"ProductGraph"],
    Cell[498108, 20999, 100, 3, 26, "Input",
      CellTags->"ProductGraph"],
    Cell[498211, 21004, 91, 1, 34, "ExampleText",
      CellTags->"ProductGraph"],
    Cell[498305, 21007, 119, 3, 26, "Input",
      CellTags->"ProductGraph"],
    Cell[498427, 21012, 119, 3, 26, "Input",
      CellTags->"ProductGraph"],
    Cell[498549, 21017, 118, 3, 26, "Input",
      CellTags->"ProductGraph"]},
  "ProductLabeledGraph"->{
    Cell[498670, 21022, 76, 1, 32, "ObjectName",
      CellTags->"ProductLabeledGraph"],
    Cell[498749, 21025, 64, 1, 50, "Highlight",
      CellTags->"ProductLabeledGraph"],
    Cell[498816, 21028, 227, 8, 31, "Usage",
      CellTags->"ProductLabeledGraph"],
    Cell[499046, 21038, 495, 16, 40, "Notes",
      CellTags->"ProductLabeledGraph"],
    Cell[499544, 21056, 298, 11, 24, "Notes",
      CellTags->"ProductLabeledGraph"],
    Cell[499845, 21069, 77, 1, 39, "ExampleSection",
      CellTags->"ProductLabeledGraph"],
    Cell[499925, 21072, 210, 5, 50, "ExampleText",
      CellTags->"ProductLabeledGraph"],
    Cell[500138, 21079, 235, 5, 42, "Input",
      CellTags->"ProductLabeledGraph"],
    Cell[500376, 21086, 126, 3, 26, "Input",
      CellTags->"ProductLabeledGraph"],
    Cell[500505, 21091, 124, 3, 26, "Input",
      CellTags->"ProductLabeledGraph"],
    Cell[500632, 21096, 119, 3, 26, "Input",
      CellTags->"ProductLabeledGraph"]},
  "ReverseGraph"->{
    Cell[500754, 21101, 62, 1, 32, "ObjectName",
      CellTags->"ReverseGraph"],
    Cell[500819, 21104, 57, 1, 50, "Highlight",
      CellTags->"ReverseGraph"],
    Cell[500879, 21107, 160, 6, 31, "Usage",
      CellTags->"ReverseGraph"],
    Cell[501042, 21115, 126, 4, 24, "Notes",
      CellTags->"ReverseGraph"],
    Cell[501171, 21121, 255, 11, 24, "Notes",
      CellTags->"ReverseGraph"],
    Cell[501429, 21134, 70, 1, 39, "ExampleSection",
      CellTags->"ReverseGraph"],
    Cell[501502, 21137, 81, 1, 34, "ExampleText",
      CellTags->"ReverseGraph"],
    Cell[501586, 21140, 87, 2, 28, "Input",
      CellTags->"ReverseGraph"],
    Cell[501676, 21144, 102, 3, 26, "Input",
      CellTags->"ReverseGraph"],
    Cell[501781, 21149, 137, 3, 26, "Input",
      CellTags->"ReverseGraph"]},
  "SetVertices"->{
    Cell[501921, 21154, 60, 1, 32, "ObjectName",
      CellTags->"SetVertices"],
    Cell[501984, 21157, 56, 1, 50, "Highlight",
      CellTags->"SetVertices"],
    Cell[502043, 21160, 215, 8, 31, "Usage",
      CellTags->"SetVertices"],
    Cell[502261, 21170, 128, 4, 24, "Notes",
      CellTags->"SetVertices"],
    Cell[502392, 21176, 135, 6, 24, "Notes",
      CellTags->"SetVertices"],
    Cell[502530, 21184, 69, 1, 39, "ExampleSection",
      CellTags->"SetVertices"],
    Cell[502602, 21187, 133, 4, 34, "ExampleText",
      CellTags->"SetVertices"],
    Cell[502738, 21193, 133, 3, 26, "Input",
      CellTags->"SetVertices"],
    Cell[502874, 21198, 97, 3, 26, "Input",
      CellTags->"SetVertices"],
    Cell[502974, 21203, 146, 4, 34, "ExampleText",
      CellTags->"SetVertices"],
    Cell[503123, 21209, 165, 4, 42, "Input",
      CellTags->"SetVertices"]},
  "ShortestPath"->{
    Cell[503291, 21215, 62, 1, 32, "ObjectName",
      CellTags->"ShortestPath"],
    Cell[503356, 21218, 57, 1, 50, "Highlight",
      CellTags->"ShortestPath"],
    Cell[503416, 21221, 278, 10, 47, "Usage",
      CellTags->"ShortestPath"],
    Cell[503697, 21233, 228, 9, 44, "Notes",
      CellTags->"ShortestPath"],
    Cell[503928, 21244, 137, 5, 24, "Notes",
      CellTags->"ShortestPath"],
    Cell[504068, 21251, 250, 11, 24, "Notes",
      CellTags->"ShortestPath"],
    Cell[504321, 21264, 379, 15, 24, "Notes",
      CellTags->"ShortestPath"],
    Cell[504703, 21281, 70, 1, 39, "ExampleSection",
      CellTags->"ShortestPath"],
    Cell[504776, 21284, 127, 4, 34, "ExampleText",
      CellTags->"ShortestPath"],
    Cell[504906, 21290, 122, 3, 26, "Input",
      CellTags->"ShortestPath"],
    Cell[505031, 21295, 105, 3, 26, "Input",
      CellTags->"ShortestPath"],
    Cell[505139, 21300, 123, 3, 26, "Input",
      CellTags->"ShortestPath"]},
  "StronglyConnectedComponents"->{
    Cell[505265, 21305, 92, 1, 32, "ObjectName",
      CellTags->"StronglyConnectedComponents"],
    Cell[505360, 21308, 72, 1, 50, "Highlight",
      CellTags->"StronglyConnectedComponents"],
    Cell[505435, 21311, 212, 5, 47, "Usage",
      CellTags->"StronglyConnectedComponents"],
    Cell[505650, 21318, 316, 10, 61, "Notes",
      CellTags->"StronglyConnectedComponents"],
    Cell[505969, 21330, 210, 6, 40, "Notes",
      CellTags->"StronglyConnectedComponents"],
    Cell[506182, 21338, 96, 1, 24, "Notes",
      CellTags->"StronglyConnectedComponents"],
    Cell[506281, 21341, 504, 19, 24, "Notes",
      CellTags->"StronglyConnectedComponents"],
    Cell[506788, 21362, 85, 1, 39, "ExampleSection",
      CellTags->"StronglyConnectedComponents"],
    Cell[506876, 21365, 117, 1, 34, "ExampleText",
      CellTags->"StronglyConnectedComponents"],
    Cell[506996, 21368, 255, 6, 42, "Input",
      CellTags->"StronglyConnectedComponents"],
    Cell[507254, 21376, 130, 3, 26, "Input",
      CellTags->"StronglyConnectedComponents"],
    Cell[507387, 21381, 111, 1, 34, "ExampleText",
      CellTags->"StronglyConnectedComponents"],
    Cell[507501, 21384, 156, 3, 26, "Input",
      CellTags->"StronglyConnectedComponents"]},
  "SubGraph"->{
    Cell[507660, 21389, 54, 1, 32, "ObjectName",
      CellTags->"SubGraph"],
    Cell[507717, 21392, 53, 1, 50, "Highlight",
      CellTags->"SubGraph"],
    Cell[507773, 21395, 204, 8, 31, "Usage",
      CellTags->"SubGraph"],
    Cell[507980, 21405, 225, 9, 44, "Notes",
      CellTags->"SubGraph"],
    Cell[508208, 21416, 118, 4, 24, "Notes",
      CellTags->"SubGraph"],
    Cell[508329, 21422, 251, 11, 24, "Notes",
      CellTags->"SubGraph"],
    Cell[508583, 21435, 66, 1, 39, "ExampleSection",
      CellTags->"SubGraph"],
    Cell[508652, 21438, 122, 4, 34, "ExampleText",
      CellTags->"SubGraph"],
    Cell[508777, 21444, 83, 2, 28, "Input",
      CellTags->"SubGraph"],
    Cell[508863, 21448, 123, 3, 26, "Input",
      CellTags->"SubGraph"],
    Cell[508989, 21453, 96, 3, 26, "Input",
      CellTags->"SubGraph"],
    Cell[509088, 21458, 166, 4, 42, "Input",
      CellTags->"SubGraph"]},
  "ToAdjacencyList"->{
    Cell[509257, 21464, 68, 1, 32, "ObjectName",
      CellTags->"ToAdjacencyList"],
    Cell[509328, 21467, 60, 1, 50, "Highlight",
      CellTags->"ToAdjacencyList"],
    Cell[509391, 21470, 178, 6, 31, "Usage",
      CellTags->"ToAdjacencyList"],
    Cell[509572, 21478, 129, 4, 24, "Notes",
      CellTags->"ToAdjacencyList"],
    Cell[509704, 21484, 157, 4, 24, "Notes",
      CellTags->"ToAdjacencyList"],
    Cell[509864, 21490, 364, 15, 24, "Notes",
      CellTags->"ToAdjacencyList"],
    Cell[510231, 21507, 73, 1, 39, "ExampleSection",
      CellTags->"ToAdjacencyList"],
    Cell[510307, 21510, 99, 1, 34, "ExampleText",
      CellTags->"ToAdjacencyList"],
    Cell[510409, 21513, 96, 2, 28, "Input",
      CellTags->"ToAdjacencyList"],
    Cell[510508, 21517, 143, 3, 26, "Input",
      CellTags->"ToAdjacencyList"],
    Cell[510654, 21522, 108, 3, 26, "Input",
      CellTags->"ToAdjacencyList"]},
  "ToAdjacencyMatrix"->{
    Cell[510765, 21527, 72, 1, 32, "ObjectName",
      CellTags->"ToAdjacencyMatrix"],
    Cell[510840, 21530, 62, 1, 50, "Highlight",
      CellTags->"ToAdjacencyMatrix"],
    Cell[510905, 21533, 185, 6, 31, "Usage",
      CellTags->"ToAdjacencyMatrix"],
    Cell[511093, 21541, 147, 4, 24, "Notes",
      CellTags->"ToAdjacencyMatrix"],
    Cell[511243, 21547, 362, 15, 24, "Notes",
      CellTags->"ToAdjacencyMatrix"],
    Cell[511608, 21564, 75, 1, 39, "ExampleSection",
      CellTags->"ToAdjacencyMatrix"],
    Cell[511686, 21567, 102, 1, 34, "ExampleText",
      CellTags->"ToAdjacencyMatrix"],
    Cell[511791, 21570, 98, 2, 28, "Input",
      CellTags->"ToAdjacencyMatrix"],
    Cell[511892, 21574, 147, 3, 26, "Input",
      CellTags->"ToAdjacencyMatrix"]},
  "ToDirectedAcyclicGraph"->{
    Cell[512042, 21579, 82, 1, 32, "ObjectName",
      CellTags->"ToDirectedAcyclicGraph"],
    Cell[512127, 21582, 67, 1, 50, "Highlight",
      CellTags->"ToDirectedAcyclicGraph"],
    Cell[512197, 21585, 304, 7, 47, "Usage",
      CellTags->"ToDirectedAcyclicGraph"],
    Cell[512504, 21594, 233, 8, 44, "Notes",
      CellTags->"ToDirectedAcyclicGraph"],
    Cell[512740, 21604, 178, 4, 40, "Notes",
      CellTags->"ToDirectedAcyclicGraph"],
    Cell[512921, 21610, 503, 19, 24, "Notes",
      CellTags->"ToDirectedAcyclicGraph"],
    Cell[513427, 21631, 80, 1, 39, "ExampleSection",
      CellTags->"ToDirectedAcyclicGraph"],
    Cell[513510, 21634, 112, 1, 34, "ExampleText",
      CellTags->"ToDirectedAcyclicGraph"],
    Cell[513625, 21637, 252, 6, 42, "Input",
      CellTags->"ToDirectedAcyclicGraph"],
    Cell[513880, 21645, 88, 1, 34, "ExampleText",
      CellTags->"ToDirectedAcyclicGraph"],
    Cell[513971, 21648, 153, 4, 42, "Input",
      CellTags->"ToDirectedAcyclicGraph"]},
  "ToEdgeList"->{
    Cell[514127, 21654, 58, 1, 32, "ObjectName",
      CellTags->"ToEdgeList"],
    Cell[514188, 21657, 55, 1, 50, "Highlight",
      CellTags->"ToEdgeList"],
    Cell[514246, 21660, 163, 6, 31, "Usage",
      CellTags->"ToEdgeList"],
    Cell[514412, 21668, 369, 15, 24, "Notes",
      CellTags->"ToEdgeList"],
    Cell[514784, 21685, 68, 1, 39, "ExampleSection",
      CellTags->"ToEdgeList"],
    Cell[514855, 21688, 105, 1, 34, "ExampleText",
      CellTags->"ToEdgeList"],
    Cell[514963, 21691, 384, 8, 74, "Input",
      CellTags->"ToEdgeList"],
    Cell[515350, 21701, 111, 3, 26, "Input",
      CellTags->"ToEdgeList"],
    Cell[515464, 21706, 111, 3, 26, "Input",
      CellTags->"ToEdgeList"]},
  "ToTDGraph"->{
    Cell[515578, 21711, 56, 1, 32, "ObjectName",
      CellTags->"ToTDGraph"],
    Cell[515637, 21714, 54, 1, 50, "Highlight",
      CellTags->"ToTDGraph"],
    Cell[515694, 21717, 235, 8, 47, "Usage",
      CellTags->"ToTDGraph"],
    Cell[515932, 21727, 226, 9, 44, "Notes",
      CellTags->"ToTDGraph"],
    Cell[516161, 21738, 77, 1, 24, "Notes",
      CellTags->"ToTDGraph"],
    Cell[516241, 21741, 177, 7, 24, "Notes",
      CellTags->"ToTDGraph"],
    Cell[516421, 21750, 67, 1, 39, "ExampleSection",
      CellTags->"ToTDGraph"],
    Cell[516491, 21753, 187, 6, 34, "ExampleText",
      CellTags->"ToTDGraph"],
    Cell[516681, 21761, 349, 8, 26, "Input",
      CellTags->"ToTDGraph"],
    Cell[517033, 21771, 115, 3, 26, "Input",
      CellTags->"ToTDGraph"],
    Cell[517151, 21776, 89, 2, 28, "Input",
      CellTags->"ToTDGraph"],
    Cell[517243, 21780, 106, 3, 26, "Input",
      CellTags->"ToTDGraph"]},
  "TypeGraph"->{
    Cell[517352, 21785, 56, 1, 32, "ObjectName",
      CellTags->"TypeGraph"],
    Cell[517411, 21788, 54, 1, 50, "Highlight",
      CellTags->"TypeGraph"],
    Cell[517468, 21791, 152, 6, 31, "Usage",
      CellTags->"TypeGraph"],
    Cell[517623, 21799, 192, 5, 40, "Notes",
      CellTags->"TypeGraph"],
    Cell[517818, 21806, 94, 1, 24, "Notes",
      CellTags->"TypeGraph"],
    Cell[517915, 21809, 152, 7, 24, "Notes",
      CellTags->"TypeGraph"],
    Cell[518070, 21818, 67, 1, 39, "ExampleSection",
      CellTags->"TypeGraph"],
    Cell[518140, 21821, 100, 1, 34, "ExampleText",
      CellTags->"TypeGraph"],
    Cell[518243, 21824, 170, 5, 26, "Input",
      CellTags->"TypeGraph"],
    Cell[518416, 21831, 95, 3, 26, "Input",
      CellTags->"TypeGraph"],
    Cell[518514, 21836, 129, 3, 26, "Input",
      CellTags->"TypeGraph"],
    Cell[518646, 21841, 131, 3, 26, "Input",
      CellTags->"TypeGraph"]},
  "VertexSet"->{
    Cell[518780, 21846, 56, 1, 32, "ObjectName",
      CellTags->"VertexSet"],
    Cell[518839, 21849, 54, 1, 50, "Highlight",
      CellTags->"VertexSet"],
    Cell[518896, 21852, 158, 6, 31, "Usage",
      CellTags->"VertexSet"],
    Cell[519057, 21860, 185, 6, 24, "Notes",
      CellTags->"VertexSet"],
    Cell[519245, 21868, 238, 11, 24, "Notes",
      CellTags->"VertexSet"],
    Cell[519486, 21881, 67, 1, 39, "ExampleSection",
      CellTags->"VertexSet"],
    Cell[519556, 21884, 101, 1, 34, "ExampleText",
      CellTags->"VertexSet"],
    Cell[519660, 21887, 133, 3, 26, "Input",
      CellTags->"VertexSet"],
    Cell[519796, 21892, 96, 3, 26, "Input",
      CellTags->"VertexSet"]},
  "WeaklyConnectedComponents"->{
    Cell[519895, 21897, 88, 1, 32, "ObjectName",
      CellTags->"WeaklyConnectedComponents"],
    Cell[519986, 21900, 70, 1, 50, "Highlight",
      CellTags->"WeaklyConnectedComponents"],
    Cell[520059, 21903, 252, 8, 47, "Usage",
      CellTags->"WeaklyConnectedComponents"],
    Cell[520314, 21913, 161, 4, 24, "Notes",
      CellTags->"WeaklyConnectedComponents"],
    Cell[520478, 21919, 298, 11, 24, "Notes",
      CellTags->"WeaklyConnectedComponents"],
    Cell[520779, 21932, 83, 1, 39, "ExampleSection",
      CellTags->"WeaklyConnectedComponents"],
    Cell[520865, 21935, 95, 1, 34, "ExampleText",
      CellTags->"WeaklyConnectedComponents"],
    Cell[520963, 21938, 116, 2, 28, "Input",
      CellTags->"WeaklyConnectedComponents"],
    Cell[521082, 21942, 234, 5, 58, "Input",
      CellTags->"WeaklyConnectedComponents"]},
  "$generateSGBound"->{
    Cell[521319, 21949, 70, 1, 32, "ObjectName",
      CellTags->"$generateSGBound"],
    Cell[521392, 21952, 64, 1, 50, "Highlight",
      CellTags->"$generateSGBound"],
    Cell[521459, 21955, 206, 6, 31, "Usage",
      CellTags->"$generateSGBound"],
    Cell[521668, 21963, 65, 1, 24, "Notes",
      CellTags->"$generateSGBound"],
    Cell[521736, 21966, 164, 7, 24, "Notes",
      CellTags->"$generateSGBound"],
    Cell[521903, 21975, 74, 1, 39, "ExampleSection",
      CellTags->"$generateSGBound"],
    Cell[521980, 21978, 88, 1, 34, "ExampleText",
      CellTags->"$generateSGBound"],
    Cell[522071, 21981, 107, 3, 26, "Input",
      CellTags->"$generateSGBound"]},
  "$identitystring"->{
    Cell[522181, 21986, 68, 1, 32, "ObjectName",
      CellTags->"$identitystring"],
    Cell[522252, 21989, 63, 1, 50, "Highlight",
      CellTags->"$identitystring"],
    Cell[522318, 21992, 215, 7, 47, "Usage",
      CellTags->"$identitystring"],
    Cell[522536, 22001, 122, 4, 24, "Notes",
      CellTags->"$identitystring"],
    Cell[522661, 22007, 271, 11, 24, "Notes",
      CellTags->"$identitystring"],
    Cell[522935, 22020, 73, 1, 39, "ExampleSection",
      CellTags->"$identitystring"],
    Cell[523011, 22023, 88, 1, 34, "ExampleText",
      CellTags->"$identitystring"],
    Cell[523102, 22026, 119, 3, 26, "Input",
      CellTags->"$identitystring"]},
  "$nullstring"->{
    Cell[523224, 22031, 60, 1, 32, "ObjectName",
      CellTags->"$nullstring"],
    Cell[523287, 22034, 59, 1, 50, "Highlight",
      CellTags->"$nullstring"],
    Cell[523349, 22037, 205, 6, 47, "Usage",
      CellTags->"$nullstring"],
    Cell[523557, 22045, 117, 4, 24, "Notes",
      CellTags->"$nullstring"],
    Cell[523677, 22051, 269, 11, 24, "Notes",
      CellTags->"$nullstring"],
    Cell[523949, 22064, 69, 1, 39, "ExampleSection",
      CellTags->"$nullstring"],
    Cell[524021, 22067, 84, 1, 34, "ExampleText",
      CellTags->"$nullstring"],
    Cell[524108, 22070, 111, 3, 26, "Input",
      CellTags->"$nullstring"]},
  "AllTimes"->{
    Cell[524222, 22075, 54, 1, 32, "ObjectName",
      CellTags->"AllTimes"],
    Cell[524279, 22078, 53, 1, 50, "Highlight",
      CellTags->"AllTimes"],
    Cell[524335, 22081, 261, 10, 47, "Usage",
      CellTags->"AllTimes"],
    Cell[524599, 22093, 244, 8, 40, "Notes",
      CellTags->"AllTimes"],
    Cell[524846, 22103, 78, 1, 24, "Notes",
      CellTags->"AllTimes"],
    Cell[524927, 22106, 319, 15, 24, "Notes",
      CellTags->"AllTimes"],
    Cell[525249, 22123, 66, 1, 39, "ExampleSection",
      CellTags->"AllTimes"],
    Cell[525318, 22126, 107, 2, 34, "ExampleText",
      CellTags->"AllTimes"],
    Cell[525428, 22130, 118, 2, 28, "Input",
      CellTags->"AllTimes"],
    Cell[525549, 22134, 130, 4, 34, "ExampleText",
      CellTags->"AllTimes"],
    Cell[525682, 22140, 117, 5, 45, "Input",
      CellTags->"AllTimes"]},
  "AlternatingGroupSG"->{
    Cell[525802, 22147, 74, 1, 32, "ObjectName",
      CellTags->"AlternatingGroupSG"],
    Cell[525879, 22150, 230, 6, 47, "Usage",
      CellTags->"AlternatingGroupSG"],
    Cell[526112, 22158, 258, 9, 24, "Notes",
      CellTags->"AlternatingGroupSG"],
    Cell[526373, 22169, 543, 23, 24, "Notes",
      CellTags->"AlternatingGroupSG"],
    Cell[526919, 22194, 76, 1, 39, "ExampleSection",
      CellTags->"AlternatingGroupSG"],
    Cell[526998, 22197, 94, 1, 34, "ExampleText",
      CellTags->"AlternatingGroupSG"],
    Cell[527095, 22200, 98, 2, 28, "Input",
      CellTags->"AlternatingGroupSG"],
    Cell[527196, 22204, 182, 6, 34, "ExampleText",
      CellTags->"AlternatingGroupSG"],
    Cell[527381, 22212, 108, 5, 45, "Input",
      CellTags->"AlternatingGroupSG"]},
  "AperiodicQSG"->{
    Cell[527492, 22219, 62, 1, 32, "ObjectName",
      CellTags->"AperiodicQSG"],
    Cell[527557, 22222, 57, 1, 50, "Highlight",
      CellTags->"AperiodicQSG"],
    Cell[527617, 22225, 173, 6, 31, "Usage",
      CellTags->"AperiodicQSG"],
    Cell[527793, 22233, 149, 5, 24, "Notes",
      CellTags->"AperiodicQSG"],
    Cell[527945, 22240, 359, 12, 40, "Notes",
      CellTags->"AperiodicQSG"],
    Cell[528307, 22254, 421, 19, 24, "Notes",
      CellTags->"AperiodicQSG"],
    Cell[528731, 22275, 70, 1, 39, "ExampleSection",
      CellTags->"AperiodicQSG"],
    Cell[528804, 22278, 122, 4, 34, "ExampleText",
      CellTags->"AperiodicQSG"],
    Cell[528929, 22284, 557, 11, 122, "Input",
      CellTags->"AperiodicQSG"],
    Cell[529489, 22297, 102, 3, 26, "Input",
      CellTags->"AperiodicQSG"],
    Cell[529594, 22302, 97, 1, 34, "ExampleText",
      CellTags->"AperiodicQSG"],
    Cell[529694, 22305, 108, 2, 28, "Input",
      CellTags->"AperiodicQSG"]},
  "AperiodicSG"->{
    Cell[529805, 22309, 60, 1, 32, "ObjectName",
      CellTags->"AperiodicSG"],
    Cell[529868, 22312, 56, 1, 50, "Highlight",
      CellTags->"AperiodicSG"],
    Cell[529927, 22315, 172, 6, 31, "Usage",
      CellTags->"AperiodicSG"],
    Cell[530102, 22323, 231, 9, 24, "Notes",
      CellTags->"AperiodicSG"],
    Cell[530336, 22334, 426, 19, 24, "Notes",
      CellTags->"AperiodicSG"],
    Cell[530765, 22355, 69, 1, 39, "ExampleSection",
      CellTags->"AperiodicSG"],
    Cell[530837, 22358, 140, 4, 34, "ExampleText",
      CellTags->"AperiodicSG"],
    Cell[530980, 22364, 118, 3, 26, "Input",
      CellTags->"AperiodicSG"],
    Cell[531101, 22369, 104, 5, 45, "Input",
      CellTags->"AperiodicSG"]},
  "BlocksToDClass"->{
    Cell[531208, 22376, 66, 1, 32, "ObjectName",
      CellTags->"BlocksToDClass"],
    Cell[531277, 22379, 59, 1, 50, "Highlight",
      CellTags->"BlocksToDClass"],
    Cell[531339, 22382, 224, 6, 47, "Usage",
      CellTags->"BlocksToDClass"],
    Cell[531566, 22390, 114, 5, 24, "Notes",
      CellTags->"BlocksToDClass"],
    Cell[531683, 22397, 435, 13, 42, "Notes",
      CellTags->"BlocksToDClass"],
    Cell[532121, 22412, 180, 6, 24, "Notes",
      CellTags->"BlocksToDClass"],
    Cell[532304, 22420, 351, 15, 24, "Notes",
      CellTags->"BlocksToDClass"],
    Cell[532658, 22437, 72, 1, 39, "ExampleSection",
      CellTags->"BlocksToDClass"],
    Cell[532733, 22440, 171, 4, 50, "ExampleText",
      CellTags->"BlocksToDClass"],
    Cell[532907, 22446, 295, 5, 58, "Input",
      CellTags->"BlocksToDClass"],
    Cell[533205, 22453, 102, 3, 26, "Input",
      CellTags->"BlocksToDClass"],
    Cell[533310, 22458, 106, 3, 26, "Input",
      CellTags->"BlocksToDClass"],
    Cell[533419, 22463, 83, 1, 34, "ExampleText",
      CellTags->"BlocksToDClass"],
    Cell[533505, 22466, 112, 3, 26, "Input",
      CellTags->"BlocksToDClass"]},
  "BoundCSM"->{
    Cell[533620, 22471, 54, 1, 32, "ObjectName",
      CellTags->"BoundCSM"],
    Cell[533677, 22474, 51, 1, 50, "Highlight",
      CellTags->"BoundCSM"],
    Cell[533731, 22477, 199, 6, 31, "Usage",
      CellTags->"BoundCSM"],
    Cell[533933, 22485, 57, 1, 24, "Notes",
      CellTags->"BoundCSM"],
    Cell[533993, 22488, 132, 6, 24, "Notes",
      CellTags->"BoundCSM"]},
  "BoundGSG"->{
    Cell[534128, 22496, 54, 1, 32, "ObjectName",
      CellTags->"BoundGSG"],
    Cell[534185, 22499, 56, 1, 50, "Highlight",
      CellTags->"BoundGSG"],
    Cell[534244, 22502, 193, 6, 31, "Usage",
      CellTags->"BoundGSG"],
    Cell[534440, 22510, 120, 5, 24, "Notes",
      CellTags->"BoundGSG"],
    Cell[534563, 22517, 173, 7, 24, "Notes",
      CellTags->"BoundGSG"]},
  "CollapseT"->{
    Cell[534739, 22526, 56, 1, 32, "ObjectName",
      CellTags->"CollapseT"],
    Cell[534798, 22529, 54, 1, 50, "Highlight",
      CellTags->"CollapseT"],
    Cell[534855, 22532, 190, 6, 31, "Usage",
      CellTags->"CollapseT"],
    Cell[535048, 22540, 172, 7, 24, "Notes",
      CellTags->"CollapseT"],
    Cell[535223, 22549, 152, 5, 24, "Notes",
      CellTags->"CollapseT"],
    Cell[535378, 22556, 576, 23, 24, "Notes",
      CellTags->"CollapseT"],
    Cell[535957, 22581, 67, 1, 39, "ExampleSection",
      CellTags->"CollapseT"],
    Cell[536027, 22584, 82, 1, 34, "ExampleText",
      CellTags->"CollapseT"],
    Cell[536112, 22587, 84, 2, 28, "Input",
      CellTags->"CollapseT"],
    Cell[536199, 22591, 122, 4, 42, "Input",
      CellTags->"CollapseT"],
    Cell[536324, 22597, 96, 3, 26, "Input",
      CellTags->"CollapseT"]},
  "ConstantT"->{
    Cell[536423, 22602, 56, 1, 32, "ObjectName",
      CellTags->"ConstantT"],
    Cell[536482, 22605, 57, 1, 50, "Highlight",
      CellTags->"ConstantT"],
    Cell[536542, 22608, 225, 9, 31, "Usage",
      CellTags->"ConstantT"],
    Cell[536770, 22619, 320, 15, 24, "Notes",
      CellTags->"ConstantT"],
    Cell[537093, 22636, 67, 1, 39, "ExampleSection",
      CellTags->"ConstantT"],
    Cell[537163, 22639, 74, 1, 34, "ExampleText",
      CellTags->"ConstantT"],
    Cell[537240, 22642, 127, 4, 42, "Input",
      CellTags->"ConstantT"]},
  "CycleDecompositionT"->{
    Cell[537370, 22648, 76, 1, 32, "ObjectName",
      CellTags->"CycleDecompositionT"],
    Cell[537449, 22651, 64, 1, 50, "Highlight",
      CellTags->"CycleDecompositionT"],
    Cell[537516, 22654, 166, 4, 31, "Usage",
      CellTags->"CycleDecompositionT"],
    Cell[537685, 22660, 235, 9, 44, "Notes",
      CellTags->"CycleDecompositionT"],
    Cell[537923, 22671, 150, 5, 24, "Notes",
      CellTags->"CycleDecompositionT"],
    Cell[538076, 22678, 203, 5, 40, "Notes",
      CellTags->"CycleDecompositionT"],
    Cell[538282, 22685, 334, 15, 24, "Notes",
      CellTags->"CycleDecompositionT"],
    Cell[538619, 22702, 77, 1, 39, "ExampleSection",
      CellTags->"CycleDecompositionT"],
    Cell[538699, 22705, 100, 1, 34, "ExampleText",
      CellTags->"CycleDecompositionT"],
    Cell[538802, 22708, 135, 5, 45, "Input",
      CellTags->"CycleDecompositionT"],
    Cell[538940, 22715, 135, 3, 26, "Input",
      CellTags->"CycleDecompositionT"]},
  "CycleToT"->{
    Cell[539078, 22720, 54, 1, 32, "ObjectName",
      CellTags->"CycleToT"],
    Cell[539135, 22723, 53, 1, 50, "Highlight",
      CellTags->"CycleToT"],
    Cell[539191, 22726, 176, 5, 47, "Usage",
      CellTags->"CycleToT"],
    Cell[539370, 22733, 78, 1, 24, "Notes",
      CellTags->"CycleToT"],
    Cell[539451, 22736, 345, 15, 24, "Notes",
      CellTags->"CycleToT"],
    Cell[539799, 22753, 66, 1, 39, "ExampleSection",
      CellTags->"CycleToT"],
    Cell[539868, 22756, 77, 1, 34, "ExampleText",
      CellTags->"CycleToT"],
    Cell[539948, 22759, 119, 3, 26, "Input",
      CellTags->"CycleToT"],
    Cell[540070, 22764, 92, 3, 26, "Input",
      CellTags->"CycleToT"],
    Cell[540165, 22769, 144, 4, 26, "Input",
      CellTags->"CycleToT"],
    Cell[540312, 22775, 104, 3, 26, "Input",
      CellTags->"CycleToT"]},
  "DC"->{
    Cell[540419, 22780, 42, 1, 32, "ObjectName",
      CellTags->"DC"],
    Cell[540464, 22783, 50, 1, 50, "Highlight",
      CellTags->"DC"],
    Cell[540517, 22786, 151, 4, 31, "Usage",
      CellTags->"DC"],
    Cell[540671, 22792, 269, 11, 24, "Notes",
      CellTags->"DC"],
    Cell[540943, 22805, 410, 19, 24, "Notes",
      CellTags->"DC"],
    Cell[541356, 22826, 60, 1, 39, "ExampleSection",
      CellTags->"DC"],
    Cell[541419, 22829, 81, 1, 34, "ExampleText",
      CellTags->"DC"],
    Cell[541503, 22832, 371, 7, 74, "Input",
      CellTags->"DC"],
    Cell[541877, 22841, 140, 4, 42, "Input",
      CellTags->"DC"]},
  "DClasses"->{
    Cell[542020, 22847, 54, 1, 32, "ObjectName",
      CellTags->"DClasses"],
    Cell[542077, 22850, 51, 1, 50, "Highlight",
      CellTags->"DClasses"],
    Cell[542131, 22853, 145, 6, 31, "Usage",
      CellTags->"DClasses"],
    Cell[542279, 22861, 108, 5, 24, "Notes",
      CellTags->"DClasses"],
    Cell[542390, 22868, 355, 15, 24, "Notes",
      CellTags->"DClasses"]},
  "DClassDecomposition"->{
    Cell[542748, 22885, 76, 1, 32, "ObjectName",
      CellTags->"DClassDecomposition"],
    Cell[542827, 22888, 64, 1, 50, "Highlight",
      CellTags->"DClassDecomposition"],
    Cell[542894, 22891, 200, 6, 31, "Usage",
      CellTags->"DClassDecomposition"],
    Cell[543097, 22899, 241, 9, 44, "Notes",
      CellTags->"DClassDecomposition"],
    Cell[543341, 22910, 160, 5, 24, "Notes",
      CellTags->"DClassDecomposition"],
    Cell[543504, 22917, 259, 8, 40, "Notes",
      CellTags->"DClassDecomposition"],
    Cell[543766, 22927, 128, 4, 24, "Notes",
      CellTags->"DClassDecomposition"],
    Cell[543897, 22933, 259, 8, 40, "Notes",
      CellTags->"DClassDecomposition"],
    Cell[544159, 22943, 344, 15, 24, "Notes",
      CellTags->"DClassDecomposition"],
    Cell[544506, 22960, 77, 1, 39, "ExampleSection",
      CellTags->"DClassDecomposition"],
    Cell[544586, 22963, 140, 4, 34, "ExampleText",
      CellTags->"DClassDecomposition"],
    Cell[544729, 22969, 156, 4, 42, "Input",
      CellTags->"DClassDecomposition"],
    Cell[544888, 22975, 131, 3, 26, "Input",
      CellTags->"DClassDecomposition"],
    Cell[545022, 22980, 162, 4, 42, "Input",
      CellTags->"DClassDecomposition"],
    Cell[545187, 22986, 113, 1, 34, "ExampleText",
      CellTags->"DClassDecomposition"],
    Cell[545303, 22989, 152, 3, 26, "Input",
      CellTags->"DClassDecomposition"]},
  "DClassSize"->{
    Cell[545458, 22994, 58, 1, 32, "ObjectName",
      CellTags->"DClassSize"],
    Cell[545519, 22997, 55, 1, 50, "Highlight",
      CellTags->"DClassSize"],
    Cell[545577, 23000, 157, 6, 31, "Usage",
      CellTags->"DClassSize"],
    Cell[545737, 23008, 226, 9, 44, "Notes",
      CellTags->"DClassSize"],
    Cell[545966, 23019, 435, 17, 40, "Notes",
      CellTags->"DClassSize"],
    Cell[546404, 23038, 110, 5, 24, "Notes",
      CellTags->"DClassSize"],
    Cell[546517, 23045, 261, 11, 24, "Notes",
      CellTags->"DClassSize"],
    Cell[546781, 23058, 68, 1, 39, "ExampleSection",
      CellTags->"DClassSize"],
    Cell[546852, 23061, 77, 1, 34, "ExampleText",
      CellTags->"DClassSize"],
    Cell[546932, 23064, 460, 9, 106, "Input",
      CellTags->"DClassSize"],
    Cell[547395, 23075, 95, 1, 34, "ExampleText",
      CellTags->"DClassSize"],
    Cell[547493, 23078, 129, 3, 26, "Input",
      CellTags->"DClassSize"]},
  "DClassT"->{
    Cell[547625, 23083, 52, 1, 32, "ObjectName",
      CellTags->"DClassT"],
    Cell[547680, 23086, 52, 1, 50, "Highlight",
      CellTags->"DClassT"],
    Cell[547735, 23089, 212, 8, 31, "Usage",
      CellTags->"DClassT"],
    Cell[547950, 23099, 92, 1, 24, "Notes",
      CellTags->"DClassT"],
    Cell[548045, 23102, 241, 11, 24, "Notes",
      CellTags->"DClassT"],
    Cell[548289, 23115, 65, 1, 39, "ExampleSection",
      CellTags->"DClassT"],
    Cell[548357, 23118, 93, 1, 34, "ExampleText",
      CellTags->"DClassT"],
    Cell[548453, 23121, 137, 4, 42, "Input",
      CellTags->"DClassT"],
    Cell[548593, 23127, 167, 4, 42, "Input",
      CellTags->"DClassT"],
    Cell[548763, 23133, 167, 4, 42, "Input",
      CellTags->"DClassT"],
    Cell[548933, 23139, 167, 4, 42, "Input",
      CellTags->"DClassT"]},
  "DClassToBlocks"->{
    Cell[549103, 23145, 66, 1, 32, "ObjectName",
      CellTags->"DClassToBlocks"],
    Cell[549172, 23148, 59, 1, 50, "Highlight",
      CellTags->"DClassToBlocks"],
    Cell[549234, 23151, 191, 5, 47, "Usage",
      CellTags->"DClassToBlocks"],
    Cell[549428, 23158, 114, 5, 24, "Notes",
      CellTags->"DClassToBlocks"],
    Cell[549545, 23165, 323, 9, 42, "Notes",
      CellTags->"DClassToBlocks"],
    Cell[549871, 23176, 180, 6, 24, "Notes",
      CellTags->"DClassToBlocks"],
    Cell[550054, 23184, 351, 15, 24, "Notes",
      CellTags->"DClassToBlocks"],
    Cell[550408, 23201, 72, 1, 39, "ExampleSection",
      CellTags->"DClassToBlocks"],
    Cell[550483, 23204, 96, 1, 34, "ExampleText",
      CellTags->"DClassToBlocks"],
    Cell[550582, 23207, 292, 6, 58, "Input",
      CellTags->"DClassToBlocks"],
    Cell[550877, 23215, 124, 3, 26, "Input",
      CellTags->"DClassToBlocks"]},
  "DihedralGroupSG"->{
    Cell[551004, 23220, 68, 1, 32, "ObjectName",
      CellTags->"DihedralGroupSG"],
    Cell[551075, 23223, 60, 1, 50, "Highlight",
      CellTags->"DihedralGroupSG"],
    Cell[551138, 23226, 174, 6, 31, "Usage",
      CellTags->"DihedralGroupSG"],
    Cell[551315, 23234, 92, 1, 24, "Notes",
      CellTags->"DihedralGroupSG"],
    Cell[551410, 23237, 338, 12, 40, "Notes",
      CellTags->"DihedralGroupSG"],
    Cell[551751, 23251, 546, 23, 24, "Notes",
      CellTags->"DihedralGroupSG"],
    Cell[552300, 23276, 73, 1, 39, "ExampleSection",
      CellTags->"DihedralGroupSG"],
    Cell[552376, 23279, 82, 1, 34, "ExampleText",
      CellTags->"DihedralGroupSG"],
    Cell[552461, 23282, 112, 3, 26, "Input",
      CellTags->"DihedralGroupSG"],
    Cell[552576, 23287, 171, 5, 34, "ExampleText",
      CellTags->"DihedralGroupSG"],
    Cell[552750, 23294, 240, 5, 74, "Input",
      CellTags->"DihedralGroupSG"]},
  "GEN"->{
    Cell[552993, 23301, 44, 1, 32, "ObjectName",
      CellTags->"GEN"],
    Cell[553040, 23304, 51, 1, 50, "Highlight",
      CellTags->"GEN"],
    Cell[553094, 23307, 190, 6, 31, "Usage",
      CellTags->"GEN"],
    Cell[553287, 23315, 251, 11, 24, "Notes",
      CellTags->"GEN"],
    Cell[553541, 23328, 309, 15, 24, "Notes",
      CellTags->"GEN"]},
  "GenerateCSM"->{
    Cell[553853, 23345, 60, 1, 32, "ObjectName",
      CellTags->"GenerateCSM"],
    Cell[553916, 23348, 56, 1, 50, "Highlight",
      CellTags->"GenerateCSM"],
    Cell[553975, 23351, 311, 10, 47, "Usage",
      CellTags->"GenerateCSM"],
    Cell[554289, 23363, 330, 11, 79, "Notes",
      CellTags->"GenerateCSM"],
    Cell[554622, 23376, 212, 6, 40, "Notes",
      CellTags->"GenerateCSM"],
    Cell[554837, 23384, 230, 7, 40, "Notes",
      CellTags->"GenerateCSM"],
    Cell[555070, 23393, 143, 5, 24, "Notes",
      CellTags->"GenerateCSM"],
    Cell[555216, 23400, 285, 9, 40, "Notes",
      CellTags->"GenerateCSM"],
    Cell[555504, 23411, 125, 4, 24, "Notes",
      CellTags->"GenerateCSM"],
    Cell[555632, 23417, 278, 11, 24, "Notes",
      CellTags->"GenerateCSM"],
    Cell[555913, 23430, 69, 1, 39, "ExampleSection",
      CellTags->"GenerateCSM"],
    Cell[555985, 23433, 134, 4, 34, "ExampleText",
      CellTags->"GenerateCSM"],
    Cell[556122, 23439, 189, 5, 42, "Input",
      CellTags->"GenerateCSM"],
    Cell[556314, 23446, 249, 7, 58, "Input",
      CellTags->"GenerateCSM"],
    Cell[556566, 23455, 194, 5, 50, "ExampleText",
      CellTags->"GenerateCSM"],
    Cell[556763, 23462, 136, 4, 26, "Input",
      CellTags->"GenerateCSM"],
    Cell[556902, 23468, 127, 4, 34, "ExampleText",
      CellTags->"GenerateCSM"],
    Cell[557032, 23474, 284, 6, 58, "Input",
      CellTags->"GenerateCSM"],
    Cell[557319, 23482, 306, 6, 66, "ExampleText",
      CellTags->"GenerateCSM"],
    Cell[557628, 23490, 243, 7, 42, "Input",
      CellTags->"GenerateCSM"],
    Cell[557874, 23499, 172, 4, 26, "Input",
      CellTags->"GenerateCSM"]},
  "GenerateCayleyGraph"->{
    Cell[558049, 23505, 76, 1, 32, "ObjectName",
      CellTags->"GenerateCayleyGraph"],
    Cell[558128, 23508, 64, 1, 50, "Highlight",
      CellTags->"GenerateCayleyGraph"],
    Cell[558195, 23511, 341, 10, 47, "Usage",
      CellTags->"GenerateCayleyGraph"],
    Cell[558539, 23523, 326, 10, 79, "Notes",
      CellTags->"GenerateCayleyGraph"],
    Cell[558868, 23535, 220, 6, 40, "Notes",
      CellTags->"GenerateCayleyGraph"],
    Cell[559091, 23543, 199, 7, 24, "Notes",
      CellTags->"GenerateCayleyGraph"],
    Cell[559293, 23552, 150, 5, 24, "Notes",
      CellTags->"GenerateCayleyGraph"],
    Cell[559446, 23559, 293, 9, 40, "Notes",
      CellTags->"GenerateCayleyGraph"],
    Cell[559742, 23570, 179, 6, 24, "Notes",
      CellTags->"GenerateCayleyGraph"],
    Cell[559924, 23578, 270, 11, 24, "Notes",
      CellTags->"GenerateCayleyGraph"],
    Cell[560197, 23591, 77, 1, 39, "ExampleSection",
      CellTags->"GenerateCayleyGraph"],
    Cell[560277, 23594, 147, 4, 34, "ExampleText",
      CellTags->"GenerateCayleyGraph"],
    Cell[560427, 23600, 224, 6, 58, "Input",
      CellTags->"GenerateCayleyGraph"],
    Cell[560654, 23608, 133, 3, 26, "Input",
      CellTags->"GenerateCayleyGraph"],
    Cell[560790, 23613, 162, 4, 26, "Input",
      CellTags->"GenerateCayleyGraph"],
    Cell[560955, 23619, 292, 7, 34, "ExampleText",
      CellTags->"GenerateCayleyGraph"],
    Cell[561250, 23628, 284, 6, 58, "Input",
      CellTags->"GenerateCayleyGraph"],
    Cell[561537, 23636, 188, 5, 26, "Input",
      CellTags->"GenerateCayleyGraph"],
    Cell[561728, 23643, 144, 3, 26, "Input",
      CellTags->"GenerateCayleyGraph"]},
  "GenerateSG"->{
    Cell[561875, 23648, 58, 1, 32, "ObjectName",
      CellTags->"GenerateSG"],
    Cell[561936, 23651, 55, 1, 50, "Highlight",
      CellTags->"GenerateSG"],
    Cell[561994, 23654, 183, 6, 31, "Usage",
      CellTags->"GenerateSG"],
    Cell[562180, 23662, 509, 14, 133, "Notes",
      CellTags->"GenerateSG"],
    Cell[562692, 23678, 154, 5, 24, "Notes",
      CellTags->"GenerateSG"],
    Cell[562849, 23685, 483, 16, 56, "Notes",
      CellTags->"GenerateSG"],
    Cell[563335, 23703, 172, 5, 24, "Notes",
      CellTags->"GenerateSG"],
    Cell[563510, 23710, 154, 5, 24, "Notes",
      CellTags->"GenerateSG"],
    Cell[563667, 23717, 181, 6, 24, "Notes",
      CellTags->"GenerateSG"],
    Cell[563851, 23725, 349, 10, 56, "Notes",
      CellTags->"GenerateSG"],
    Cell[564203, 23737, 161, 4, 24, "Notes",
      CellTags->"GenerateSG"],
    Cell[564367, 23743, 407, 19, 24, "Notes",
      CellTags->"GenerateSG"],
    Cell[564777, 23764, 68, 1, 39, "ExampleSection",
      CellTags->"GenerateSG"],
    Cell[564848, 23767, 131, 4, 34, "ExampleText",
      CellTags->"GenerateSG"],
    Cell[564982, 23773, 136, 6, 62, "Input",
      CellTags->"GenerateSG"],
    Cell[565121, 23781, 140, 4, 34, "ExampleText",
      CellTags->"GenerateSG"],
    Cell[565264, 23787, 145, 4, 26, "Input",
      CellTags->"GenerateSG"],
    Cell[565412, 23793, 104, 3, 26, "Input",
      CellTags->"GenerateSG"],
    Cell[565519, 23798, 149, 4, 34, "ExampleText",
      CellTags->"GenerateSG"],
    Cell[565671, 23804, 108, 3, 26, "Input",
      CellTags->"GenerateSG"],
    Cell[565782, 23809, 170, 4, 50, "ExampleText",
      CellTags->"GenerateSG"],
    Cell[565955, 23815, 102, 3, 26, "Input",
      CellTags->"GenerateSG"],
    Cell[566060, 23820, 115, 3, 28, "Input",
      CellTags->"GenerateSG"],
    Cell[566178, 23825, 138, 3, 26, "Input",
      CellTags->"GenerateSG"],
    Cell[566319, 23830, 166, 4, 50, "ExampleText",
      CellTags->"GenerateSG"],
    Cell[566488, 23836, 139, 3, 26, "Input",
      CellTags->"GenerateSG"],
    Cell[566630, 23841, 157, 4, 26, "Input",
      CellTags->"GenerateSG"],
    Cell[566790, 23847, 153, 4, 42, "Input",
      CellTags->"GenerateSG"],
    Cell[566946, 23853, 249, 8, 34, "ExampleText",
      Evaluatable->False,
      CellTags->"GenerateSG"],
    Cell[567198, 23863, 361, 9, 74, "Input",
      CellTags->"GenerateSG"],
    Cell[567562, 23874, 227, 8, 34, "ExampleText",
      Evaluatable->False,
      CellTags->"GenerateSG"],
    Cell[567792, 23884, 100, 3, 26, "Input",
      CellTags->"GenerateSG"]},
  "GreenHQT"->{
    Cell[567895, 23889, 54, 1, 32, "ObjectName",
      CellTags->"GreenHQT"],
    Cell[567952, 23892, 53, 1, 50, "Highlight",
      CellTags->"GreenHQT"],
    Cell[568008, 23895, 250, 10, 31, "Usage",
      CellTags->"GreenHQT"],
    Cell[568261, 23907, 476, 19, 24, "Notes",
      CellTags->"GreenHQT"]},
  "GreenJQT"->{
    Cell[568740, 23928, 54, 1, 32, "ObjectName",
      CellTags->"GreenJQT"],
    Cell[568797, 23931, 53, 1, 50, "Highlight",
      CellTags->"GreenJQT"],
    Cell[568853, 23934, 250, 10, 31, "Usage",
      CellTags->"GreenJQT"],
    Cell[569106, 23946, 316, 10, 40, "Notes",
      CellTags->"GreenJQT"],
    Cell[569425, 23958, 427, 15, 24, "Notes",
      CellTags->"GreenJQT"],
    Cell[569855, 23975, 362, 15, 24, "Notes",
      CellTags->"GreenJQT"]},
  "GreenLQT"->{
    Cell[570220, 23992, 54, 1, 32, "ObjectName",
      CellTags->"GreenLQT"],
    Cell[570277, 23995, 53, 1, 50, "Highlight",
      CellTags->"GreenLQT"],
    Cell[570333, 23998, 250, 10, 31, "Usage",
      CellTags->"GreenLQT"],
    Cell[570586, 24010, 476, 19, 24, "Notes",
      CellTags->"GreenLQT"]},
  "GreenRQT"->{
    Cell[571065, 24031, 54, 1, 32, "ObjectName",
      CellTags->"GreenRQT"],
    Cell[571122, 24034, 53, 1, 50, "Highlight",
      CellTags->"GreenRQT"],
    Cell[571178, 24037, 250, 10, 31, "Usage",
      CellTags->"GreenRQT"],
    Cell[571431, 24049, 476, 19, 24, "Notes",
      CellTags->"GreenRQT"]},
  "HClassSize"->{
    Cell[571910, 24070, 58, 1, 32, "ObjectName",
      CellTags->"HClassSize"],
    Cell[571971, 24073, 55, 1, 50, "Highlight",
      CellTags->"HClassSize"],
    Cell[572029, 24076, 173, 6, 31, "Usage",
      CellTags->"HClassSize"],
    Cell[572205, 24084, 110, 5, 24, "Notes",
      CellTags->"HClassSize"],
    Cell[572318, 24091, 164, 7, 24, "Notes",
      CellTags->"HClassSize"]},
  "HClassT"->{
    Cell[572485, 24100, 52, 1, 32, "ObjectName",
      CellTags->"HClassT"],
    Cell[572540, 24103, 52, 1, 50, "Highlight",
      CellTags->"HClassT"],
    Cell[572595, 24106, 206, 8, 31, "Usage",
      CellTags->"HClassT"],
    Cell[572804, 24116, 79, 1, 24, "Notes",
      CellTags->"HClassT"],
    Cell[572886, 24119, 149, 7, 24, "Notes",
      CellTags->"HClassT"]},
  "HTrivialQSG"->{
    Cell[573038, 24128, 60, 1, 32, "ObjectName",
      CellTags->"HTrivialQSG"],
    Cell[573101, 24131, 56, 1, 50, "Highlight",
      CellTags->"HTrivialQSG"],
    Cell[573160, 24134, 165, 6, 31, "Usage",
      CellTags->"HTrivialQSG"],
    Cell[573328, 24142, 88, 1, 24, "Notes",
      CellTags->"HTrivialQSG"],
    Cell[573419, 24145, 692, 27, 40, "Notes",
      CellTags->"HTrivialQSG"],
    Cell[574114, 24174, 69, 1, 39, "ExampleSection",
      CellTags->"HTrivialQSG"],
    Cell[574186, 24177, 90, 1, 34, "ExampleText",
      CellTags->"HTrivialQSG"],
    Cell[574279, 24180, 196, 5, 42, "Input",
      CellTags->"HTrivialQSG"],
    Cell[574478, 24187, 100, 3, 26, "Input",
      CellTags->"HTrivialQSG"]},
  "IdealT"->{
    Cell[574581, 24192, 50, 1, 32, "ObjectName",
      CellTags->"IdealT"],
    Cell[574634, 24195, 51, 1, 50, "Highlight",
      CellTags->"IdealT"],
    Cell[574688, 24198, 220, 8, 31, "Usage",
      CellTags->"IdealT"],
    Cell[574911, 24208, 73, 1, 24, "Notes",
      CellTags->"IdealT"],
    Cell[574987, 24211, 257, 11, 24, "Notes",
      CellTags->"IdealT"],
    Cell[575247, 24224, 64, 1, 39, "ExampleSection",
      CellTags->"IdealT"],
    Cell[575314, 24227, 125, 4, 34, "ExampleText",
      CellTags->"IdealT"],
    Cell[575442, 24233, 109, 3, 26, "Input",
      CellTags->"IdealT"],
    Cell[575554, 24238, 108, 3, 26, "Input",
      CellTags->"IdealT"],
    Cell[575665, 24243, 108, 3, 26, "Input",
      CellTags->"IdealT"]},
  "IdemSG"->{
    Cell[575776, 24248, 50, 1, 32, "ObjectName",
      CellTags->"IdemSG"],
    Cell[575829, 24251, 51, 1, 50, "Highlight",
      CellTags->"IdemSG"],
    Cell[575883, 24254, 163, 6, 31, "Usage",
      CellTags->"IdemSG"],
    Cell[576049, 24262, 73, 1, 24, "Notes",
      CellTags->"IdemSG"],
    Cell[576125, 24265, 161, 6, 24, "Notes",
      CellTags->"IdemSG"],
    Cell[576289, 24273, 106, 5, 24, "Notes",
      CellTags->"IdemSG"],
    Cell[576398, 24280, 253, 11, 24, "Notes",
      CellTags->"IdemSG"],
    Cell[576654, 24293, 64, 1, 39, "ExampleSection",
      CellTags->"IdemSG"],
    Cell[576721, 24296, 128, 4, 34, "ExampleText",
      CellTags->"IdemSG"],
    Cell[576852, 24302, 111, 3, 26, "Input",
      CellTags->"IdemSG"],
    Cell[576966, 24307, 108, 3, 26, "Input",
      CellTags->"IdemSG"],
    Cell[577077, 24312, 91, 1, 34, "ExampleText",
      CellTags->"IdemSG"],
    Cell[577171, 24315, 90, 3, 26, "Input",
      CellTags->"IdemSG"]},
  "ImageT"->{
    Cell[577264, 24320, 50, 1, 32, "ObjectName",
      CellTags->"ImageT"],
    Cell[577317, 24323, 51, 1, 50, "Highlight",
      CellTags->"ImageT"],
    Cell[577371, 24326, 171, 6, 31, "Usage",
      CellTags->"ImageT"],
    Cell[577545, 24334, 106, 5, 24, "Notes",
      CellTags->"ImageT"],
    Cell[577654, 24341, 457, 19, 24, "Notes",
      CellTags->"ImageT"],
    Cell[578114, 24362, 64, 1, 39, "ExampleSection",
      CellTags->"ImageT"],
    Cell[578181, 24365, 91, 1, 34, "ExampleText",
      CellTags->"ImageT"],
    Cell[578275, 24368, 197, 5, 42, "Input",
      CellTags->"ImageT"]},
  "InGroupSG"->{
    Cell[578475, 24375, 56, 1, 32, "ObjectName",
      CellTags->"InGroupSG"],
    Cell[578534, 24378, 54, 1, 50, "Highlight",
      CellTags->"InGroupSG"],
    Cell[578591, 24381, 218, 8, 31, "Usage",
      CellTags->"InGroupSG"],
    Cell[578812, 24391, 76, 1, 24, "Notes",
      CellTags->"InGroupSG"],
    Cell[578891, 24394, 169, 6, 24, "Notes",
      CellTags->"InGroupSG"],
    Cell[579063, 24402, 109, 5, 24, "Notes",
      CellTags->"InGroupSG"],
    Cell[579175, 24409, 250, 11, 24, "Notes",
      CellTags->"InGroupSG"],
    Cell[579428, 24422, 67, 1, 39, "ExampleSection",
      CellTags->"InGroupSG"],
    Cell[579498, 24425, 94, 1, 34, "ExampleText",
      CellTags->"InGroupSG"],
    Cell[579595, 24428, 112, 3, 26, "Input",
      CellTags->"InGroupSG"],
    Cell[579710, 24433, 124, 4, 42, "Input",
      CellTags->"InGroupSG"],
    Cell[579837, 24439, 118, 3, 26, "Input",
      CellTags->"InGroupSG"]},
  "InverseT"->{
    Cell[579958, 24444, 54, 1, 32, "ObjectName",
      CellTags->"InverseT"],
    Cell[580015, 24447, 53, 1, 50, "Highlight",
      CellTags->"InverseT"],
    Cell[580071, 24450, 163, 6, 31, "Usage",
      CellTags->"InverseT"],
    Cell[580237, 24458, 142, 5, 24, "Notes",
      CellTags->"InverseT"],
    Cell[580382, 24465, 247, 11, 24, "Notes",
      CellTags->"InverseT"],
    Cell[580632, 24478, 66, 1, 39, "ExampleSection",
      CellTags->"InverseT"],
    Cell[580701, 24481, 95, 1, 34, "ExampleText",
      CellTags->"InverseT"],
    Cell[580799, 24484, 142, 5, 45, "Input",
      CellTags->"InverseT"]},
  "JTrivialQSG"->{
    Cell[580944, 24491, 60, 1, 32, "ObjectName",
      CellTags->"JTrivialQSG"],
    Cell[581007, 24494, 56, 1, 50, "Highlight",
      CellTags->"JTrivialQSG"],
    Cell[581066, 24497, 164, 6, 31, "Usage",
      CellTags->"JTrivialQSG"],
    Cell[581233, 24505, 92, 1, 24, "Notes",
      CellTags->"JTrivialQSG"],
    Cell[581328, 24508, 142, 5, 24, "Notes",
      CellTags->"JTrivialQSG"],
    Cell[581473, 24515, 692, 27, 40, "Notes",
      CellTags->"JTrivialQSG"],
    Cell[582168, 24544, 69, 1, 39, "ExampleSection",
      CellTags->"JTrivialQSG"],
    Cell[582240, 24547, 76, 1, 34, "ExampleText",
      CellTags->"JTrivialQSG"],
    Cell[582319, 24550, 135, 3, 26, "Input",
      CellTags->"JTrivialQSG"],
    Cell[582457, 24555, 100, 3, 26, "Input",
      CellTags->"JTrivialQSG"],
    Cell[582560, 24560, 138, 3, 26, "Input",
      CellTags->"JTrivialQSG"],
    Cell[582701, 24565, 100, 3, 26, "Input",
      CellTags->"JTrivialQSG"]},
  "JTrivialSG"->{
    Cell[582804, 24570, 58, 1, 32, "ObjectName",
      CellTags->"JTrivialSG"],
    Cell[582865, 24573, 55, 1, 50, "Highlight",
      CellTags->"JTrivialSG"],
    Cell[582923, 24576, 186, 5, 47, "Usage",
      CellTags->"JTrivialSG"],
    Cell[583112, 24583, 185, 6, 24, "Notes",
      CellTags->"JTrivialSG"],
    Cell[583300, 24591, 484, 19, 24, "Notes",
      CellTags->"JTrivialSG"],
    Cell[583787, 24612, 68, 1, 39, "ExampleSection",
      CellTags->"JTrivialSG"],
    Cell[583858, 24615, 75, 1, 34, "ExampleText",
      CellTags->"JTrivialSG"],
    Cell[583936, 24618, 98, 3, 26, "Input",
      CellTags->"JTrivialSG"],
    Cell[584037, 24623, 101, 3, 26, "Input",
      CellTags->"JTrivialSG"],
    Cell[584141, 24628, 83, 2, 28, "Input",
      CellTags->"JTrivialSG"]},
  "KernelImageToT"->{
    Cell[584227, 24632, 66, 1, 32, "ObjectName",
      CellTags->"KernelImageToT"],
    Cell[584296, 24635, 59, 1, 50, "Highlight",
      CellTags->"KernelImageToT"],
    Cell[584358, 24638, 167, 4, 31, "Usage",
      CellTags->"KernelImageToT"],
    Cell[584528, 24644, 145, 4, 24, "Notes",
      CellTags->"KernelImageToT"],
    Cell[584676, 24650, 467, 19, 24, "Notes",
      CellTags->"KernelImageToT"],
    Cell[585146, 24671, 72, 1, 39, "ExampleSection",
      CellTags->"KernelImageToT"],
    Cell[585221, 24674, 128, 4, 34, "ExampleText",
      CellTags->"KernelImageToT"],
    Cell[585352, 24680, 88, 2, 28, "Input",
      CellTags->"KernelImageToT"],
    Cell[585443, 24684, 145, 4, 42, "Input",
      CellTags->"KernelImageToT"],
    Cell[585591, 24690, 111, 3, 26, "Input",
      CellTags->"KernelImageToT"]},
  "KernelT"->{
    Cell[585705, 24695, 52, 1, 32, "ObjectName",
      CellTags->"KernelT"],
    Cell[585760, 24698, 52, 1, 50, "Highlight",
      CellTags->"KernelT"],
    Cell[585815, 24701, 155, 6, 31, "Usage",
      CellTags->"KernelT"],
    Cell[585973, 24709, 107, 5, 24, "Notes",
      CellTags->"KernelT"],
    Cell[586083, 24716, 233, 6, 40, "Notes",
      CellTags->"KernelT"],
    Cell[586319, 24724, 456, 19, 24, "Notes",
      CellTags->"KernelT"],
    Cell[586778, 24745, 65, 1, 39, "ExampleSection",
      CellTags->"KernelT"],
    Cell[586846, 24748, 93, 1, 34, "ExampleText",
      CellTags->"KernelT"],
    Cell[586942, 24751, 199, 5, 42, "Input",
      CellTags->"KernelT"]},
  "LeftIdealT"->{
    Cell[587144, 24758, 58, 1, 32, "ObjectName",
      CellTags->"LeftIdealT"],
    Cell[587205, 24761, 55, 1, 50, "Highlight",
      CellTags->"LeftIdealT"],
    Cell[587263, 24764, 167, 4, 31, "Usage",
      CellTags->"LeftIdealT"],
    Cell[587433, 24770, 77, 1, 24, "Notes",
      CellTags->"LeftIdealT"],
    Cell[587513, 24773, 253, 11, 24, "Notes",
      CellTags->"LeftIdealT"],
    Cell[587769, 24786, 68, 1, 39, "ExampleSection",
      CellTags->"LeftIdealT"],
    Cell[587840, 24789, 134, 4, 34, "ExampleText",
      CellTags->"LeftIdealT"],
    Cell[587977, 24795, 113, 3, 26, "Input",
      CellTags->"LeftIdealT"],
    Cell[588093, 24800, 116, 3, 26, "Input",
      CellTags->"LeftIdealT"],
    Cell[588212, 24805, 116, 3, 26, "Input",
      CellTags->"LeftIdealT"]},
  "LeftNullSG"->{
    Cell[588331, 24810, 58, 1, 32, "ObjectName",
      CellTags->"LeftNullSG"],
    Cell[588392, 24813, 55, 1, 50, "Highlight",
      CellTags->"LeftNullSG"],
    Cell[588450, 24816, 162, 6, 31, "Usage",
      CellTags->"LeftNullSG"],
    Cell[588615, 24824, 77, 1, 24, "Notes",
      CellTags->"LeftNullSG"],
    Cell[588695, 24827, 239, 10, 24, "Notes",
      CellTags->"LeftNullSG"],
    Cell[588937, 24839, 110, 5, 24, "Notes",
      CellTags->"LeftNullSG"],
    Cell[589050, 24846, 340, 15, 24, "Notes",
      CellTags->"LeftNullSG"],
    Cell[589393, 24863, 68, 1, 39, "ExampleSection",
      CellTags->"LeftNullSG"],
    Cell[589464, 24866, 123, 4, 34, "ExampleText",
      CellTags->"LeftNullSG"],
    Cell[589590, 24872, 93, 2, 28, "Input",
      CellTags->"LeftNullSG"],
    Cell[589686, 24876, 102, 2, 26, "Input",
      CellTags->"LeftNullSG"],
    Cell[589791, 24880, 98, 3, 26, "Input",
      CellTags->"LeftNullSG"]},
  "LeftOneSG"->{
    Cell[589892, 24885, 56, 1, 32, "ObjectName",
      CellTags->"LeftOneSG"],
    Cell[589951, 24888, 54, 1, 50, "Highlight",
      CellTags->"LeftOneSG"],
    Cell[590008, 24891, 165, 6, 31, "Usage",
      CellTags->"LeftOneSG"],
    Cell[590176, 24899, 76, 1, 24, "Notes",
      CellTags->"LeftOneSG"],
    Cell[590255, 24902, 251, 10, 24, "Notes",
      CellTags->"LeftOneSG"],
    Cell[590509, 24914, 109, 5, 24, "Notes",
      CellTags->"LeftOneSG"],
    Cell[590621, 24921, 337, 15, 24, "Notes",
      CellTags->"LeftOneSG"],
    Cell[590961, 24938, 67, 1, 39, "ExampleSection",
      CellTags->"LeftOneSG"],
    Cell[591031, 24941, 138, 4, 34, "ExampleText",
      CellTags->"LeftOneSG"],
    Cell[591172, 24947, 91, 2, 28, "Input",
      CellTags->"LeftOneSG"],
    Cell[591266, 24951, 126, 3, 26, "Input",
      CellTags->"LeftOneSG"],
    Cell[591395, 24956, 95, 3, 26, "Input",
      CellTags->"LeftOneSG"]},
  "LocallyCommutativeQSG"->{
    Cell[591493, 24961, 80, 1, 32, "ObjectName",
      CellTags->"LocallyCommutativeQSG"],
    Cell[591576, 24964, 66, 1, 50, "Highlight",
      CellTags->"LocallyCommutativeQSG"],
    Cell[591645, 24967, 189, 6, 31, "Usage",
      CellTags->"LocallyCommutativeQSG"],
    Cell[591837, 24975, 256, 9, 24, "Notes",
      CellTags->"LocallyCommutativeQSG"],
    Cell[592096, 24986, 254, 9, 24, "Notes",
      CellTags->"LocallyCommutativeQSG"],
    Cell[592353, 24997, 609, 23, 40, "Notes",
      CellTags->"LocallyCommutativeQSG"],
    Cell[592965, 25022, 79, 1, 39, "ExampleSection",
      CellTags->"LocallyCommutativeQSG"],
    Cell[593047, 25025, 115, 1, 34, "ExampleText",
      CellTags->"LocallyCommutativeQSG"],
    Cell[593165, 25028, 157, 5, 45, "Input",
      CellTags->"LocallyCommutativeQSG"],
    Cell[593325, 25035, 120, 2, 34, "ExampleText",
      CellTags->"LocallyCommutativeQSG"],
    Cell[593448, 25039, 197, 5, 58, "Input",
      CellTags->"LocallyCommutativeQSG"],
    Cell[593648, 25046, 120, 3, 26, "Input",
      CellTags->"LocallyCommutativeQSG"]},
  "LocallyIdempotentQSG"->{
    Cell[593771, 25051, 78, 1, 32, "ObjectName",
      CellTags->"LocallyIdempotentQSG"],
    Cell[593852, 25054, 65, 1, 50, "Highlight",
      CellTags->"LocallyIdempotentQSG"],
    Cell[593920, 25057, 187, 6, 31, "Usage",
      CellTags->"LocallyIdempotentQSG"],
    Cell[594110, 25065, 270, 9, 24, "Notes",
      CellTags->"LocallyIdempotentQSG"],
    Cell[594383, 25076, 253, 9, 24, "Notes",
      CellTags->"LocallyIdempotentQSG"],
    Cell[594639, 25087, 709, 27, 40, "Notes",
      CellTags->"LocallyIdempotentQSG"],
    Cell[595351, 25116, 78, 1, 39, "ExampleSection",
      CellTags->"LocallyIdempotentQSG"],
    Cell[595432, 25119, 113, 1, 34, "ExampleText",
      CellTags->"LocallyIdempotentQSG"],
    Cell[595548, 25122, 156, 5, 45, "Input",
      CellTags->"LocallyIdempotentQSG"],
    Cell[595707, 25129, 196, 5, 58, "Input",
      CellTags->"LocallyIdempotentQSG"],
    Cell[595906, 25136, 118, 3, 26, "Input",
      CellTags->"LocallyIdempotentQSG"]},
  "Merge12T"->{
    Cell[596027, 25141, 54, 1, 32, "ObjectName",
      CellTags->"Merge12T"],
    Cell[596084, 25144, 53, 1, 50, "Highlight",
      CellTags->"Merge12T"],
    Cell[596140, 25147, 161, 6, 31, "Usage",
      CellTags->"Merge12T"],
    Cell[596304, 25155, 101, 1, 24, "Notes",
      CellTags->"Merge12T"],
    Cell[596408, 25158, 350, 15, 24, "Notes",
      CellTags->"Merge12T"],
    Cell[596761, 25175, 66, 1, 39, "ExampleSection",
      CellTags->"Merge12T"],
    Cell[596830, 25178, 92, 1, 34, "ExampleText",
      CellTags->"Merge12T"],
    Cell[596925, 25181, 156, 4, 26, "Input",
      CellTags->"Merge12T"],
    Cell[597084, 25187, 110, 3, 26, "Input",
      CellTags->"Merge12T"],
    Cell[597197, 25192, 108, 2, 34, "ExampleText",
      CellTags->"Merge12T"],
    Cell[597308, 25196, 136, 3, 26, "Input",
      CellTags->"Merge12T"],
    Cell[597447, 25201, 110, 3, 26, "Input",
      CellTags->"Merge12T"]},
  "MakeMonoidSG"->{
    Cell[597560, 25206, 62, 1, 32, "ObjectName",
      CellTags->"MakeMonoidSG"],
    Cell[597625, 25209, 57, 1, 50, "Highlight",
      CellTags->"MakeMonoidSG"],
    Cell[597685, 25212, 194, 6, 31, "Usage",
      CellTags->"MakeMonoidSG"],
    Cell[597882, 25220, 163, 7, 24, "Notes",
      CellTags->"MakeMonoidSG"],
    Cell[598048, 25229, 70, 1, 39, "ExampleSection",
      CellTags->"MakeMonoidSG"],
    Cell[598121, 25232, 78, 1, 34, "ExampleText",
      CellTags->"MakeMonoidSG"],
    Cell[598202, 25235, 134, 3, 26, "Input",
      CellTags->"MakeMonoidSG"],
    Cell[598339, 25240, 102, 3, 26, "Input",
      CellTags->"MakeMonoidSG"]},
  "MonoidQSG"->{
    Cell[598444, 25245, 56, 1, 32, "ObjectName",
      CellTags->"MonoidQSG"],
    Cell[598503, 25248, 54, 1, 50, "Highlight",
      CellTags->"MonoidQSG"],
    Cell[598560, 25251, 160, 6, 31, "Usage",
      CellTags->"MonoidQSG"],
    Cell[598723, 25259, 694, 27, 40, "Notes",
      CellTags->"MonoidQSG"],
    Cell[599420, 25288, 67, 1, 39, "ExampleSection",
      CellTags->"MonoidQSG"],
    Cell[599490, 25291, 75, 1, 34, "ExampleText",
      CellTags->"MonoidQSG"],
    Cell[599568, 25294, 132, 3, 26, "Input",
      CellTags->"MonoidQSG"],
    Cell[599703, 25299, 97, 3, 26, "Input",
      CellTags->"MonoidQSG"]},
  "NilpotentQSG"->{
    Cell[599803, 25304, 62, 1, 32, "ObjectName",
      CellTags->"NilpotentQSG"],
    Cell[599868, 25307, 57, 1, 50, "Highlight",
      CellTags->"NilpotentQSG"],
    Cell[599928, 25310, 218, 6, 47, "Usage",
      CellTags->"NilpotentQSG"],
    Cell[600149, 25318, 691, 27, 40, "Notes",
      CellTags->"NilpotentQSG"],
    Cell[600843, 25347, 70, 1, 39, "ExampleSection",
      CellTags->"NilpotentQSG"],
    Cell[600916, 25350, 107, 1, 34, "ExampleText",
      CellTags->"NilpotentQSG"],
    Cell[601026, 25353, 140, 3, 26, "Input",
      CellTags->"NilpotentQSG"],
    Cell[601169, 25358, 110, 3, 26, "Input",
      CellTags->"NilpotentQSG"],
    Cell[601282, 25363, 103, 3, 26, "Input",
      CellTags->"NilpotentQSG"]},
  "NullSG"->{
    Cell[601388, 25368, 50, 1, 32, "ObjectName",
      CellTags->"NullSG"],
    Cell[601441, 25371, 51, 1, 50, "Highlight",
      CellTags->"NullSG"],
    Cell[601495, 25374, 170, 6, 31, "Usage",
      CellTags->"NullSG"],
    Cell[601668, 25382, 202, 8, 24, "Notes",
      CellTags->"NullSG"],
    Cell[601873, 25392, 106, 5, 24, "Notes",
      CellTags->"NullSG"],
    Cell[601982, 25399, 353, 15, 24, "Notes",
      CellTags->"NullSG"],
    Cell[602338, 25416, 64, 1, 39, "ExampleSection",
      CellTags->"NullSG"],
    Cell[602405, 25419, 69, 1, 34, "ExampleText",
      CellTags->"NullSG"],
    Cell[602477, 25422, 546, 10, 106, "Input",
      CellTags->"NullSG"],
    Cell[603026, 25434, 119, 4, 42, "Input",
      CellTags->"NullSG"],
    Cell[603148, 25440, 112, 3, 26, "Input",
      CellTags->"NullSG"]},
  "OneSG"->{
    Cell[603263, 25445, 48, 1, 32, "ObjectName",
      CellTags->"OneSG"],
    Cell[603314, 25448, 50, 1, 50, "Highlight",
      CellTags->"OneSG"],
    Cell[603367, 25451, 164, 6, 31, "Usage",
      CellTags->"OneSG"],
    Cell[603534, 25459, 206, 8, 24, "Notes",
      CellTags->"OneSG"],
    Cell[603743, 25469, 105, 5, 24, "Notes",
      CellTags->"OneSG"],
    Cell[603851, 25476, 235, 11, 24, "Notes",
      CellTags->"OneSG"],
    Cell[604089, 25489, 63, 1, 39, "ExampleSection",
      CellTags->"OneSG"],
    Cell[604155, 25492, 96, 1, 34, "ExampleText",
      CellTags->"OneSG"],
    Cell[604254, 25495, 206, 5, 42, "Input",
      CellTags->"OneSG"],
    Cell[604463, 25502, 129, 3, 26, "Input",
      CellTags->"OneSG"],
    Cell[604595, 25507, 94, 1, 34, "ExampleText",
      CellTags->"OneSG"],
    Cell[604692, 25510, 90, 2, 28, "Input",
      CellTags->"OneSG"],
    Cell[604785, 25514, 89, 3, 26, "Input",
      CellTags->"OneSG"],
    Cell[604877, 25519, 104, 2, 34, "ExampleText",
      CellTags->"OneSG"],
    Cell[604984, 25523, 126, 3, 26, "Input",
      CellTags->"OneSG"],
    Cell[605113, 25528, 88, 3, 26, "Input",
      CellTags->"OneSG"],
    Cell[605204, 25533, 94, 3, 26, "Input",
      CellTags->"OneSG"]},
  "OneT"->{
    Cell[605301, 25538, 46, 1, 32, "ObjectName",
      CellTags->"OneT"],
    Cell[605350, 25541, 52, 1, 50, "Highlight",
      CellTags->"OneT"],
    Cell[605405, 25544, 226, 8, 31, "Usage",
      CellTags->"OneT"],
    Cell[605634, 25554, 199, 6, 24, "Notes",
      CellTags->"OneT"],
    Cell[605836, 25562, 149, 7, 24, "Notes",
      CellTags->"OneT"],
    Cell[605988, 25571, 62, 1, 39, "ExampleSection",
      CellTags->"OneT"],
    Cell[606053, 25574, 83, 1, 34, "ExampleText",
      CellTags->"OneT"],
    Cell[606139, 25577, 72, 2, 28, "Input",
      CellTags->"OneT"],
    Cell[606214, 25581, 113, 3, 26, "Input",
      CellTags->"OneT"]},
  "OrderT"->{
    Cell[606330, 25586, 50, 1, 32, "ObjectName",
      CellTags->"OrderT"],
    Cell[606383, 25589, 51, 1, 50, "Highlight",
      CellTags->"OrderT"],
    Cell[606437, 25592, 125, 4, 31, "Usage",
      CellTags->"OrderT"],
    Cell[606565, 25598, 247, 11, 24, "Notes",
      CellTags->"OrderT"],
    Cell[606815, 25611, 64, 1, 39, "ExampleSection",
      CellTags->"OrderT"],
    Cell[606882, 25614, 121, 4, 34, "ExampleText",
      CellTags->"OrderT"],
    Cell[607006, 25620, 91, 2, 28, "Input",
      CellTags->"OrderT"],
    Cell[607100, 25624, 148, 4, 42, "Input",
      CellTags->"OrderT"]},
  "PrintDClass"->{
    Cell[607251, 25630, 60, 1, 32, "ObjectName",
      CellTags->"PrintDClass"],
    Cell[607314, 25633, 56, 1, 50, "Highlight",
      CellTags->"PrintDClass"],
    Cell[607373, 25636, 154, 6, 31, "Usage",
      CellTags->"PrintDClass"],
    Cell[607530, 25644, 111, 5, 24, "Notes",
      CellTags->"PrintDClass"],
    Cell[607644, 25651, 194, 6, 24, "Notes",
      CellTags->"PrintDClass"],
    Cell[607841, 25659, 358, 15, 24, "Notes",
      CellTags->"PrintDClass"],
    Cell[608202, 25676, 69, 1, 39, "ExampleSection",
      CellTags->"PrintDClass"],
    Cell[608274, 25679, 78, 1, 34, "ExampleText",
      CellTags->"PrintDClass"],
    Cell[608355, 25682, 380, 7, 74, "Input",
      CellTags->"PrintDClass"],
    Cell[608738, 25691, 100, 3, 26, "Input",
      CellTags->"PrintDClass"]},
  "PrintEquations"->{
    Cell[608841, 25696, 66, 1, 32, "ObjectName",
      CellTags->"PrintEquations"],
    Cell[608910, 25699, 59, 1, 50, "Highlight",
      CellTags->"PrintEquations"],
    Cell[608972, 25702, 153, 4, 31, "Usage",
      CellTags->"PrintEquations"],
    Cell[609128, 25708, 279, 10, 61, "Notes",
      CellTags->"PrintEquations"],
    Cell[609410, 25720, 149, 5, 24, "Notes",
      CellTags->"PrintEquations"],
    Cell[609562, 25727, 458, 19, 24, "Notes",
      CellTags->"PrintEquations"],
    Cell[610023, 25748, 72, 1, 39, "ExampleSection",
      CellTags->"PrintEquations"],
    Cell[610098, 25751, 142, 4, 34, "ExampleText",
      CellTags->"PrintEquations"],
    Cell[610243, 25757, 160, 4, 26, "Input",
      CellTags->"PrintEquations"],
    Cell[610406, 25763, 100, 3, 26, "Input",
      CellTags->"PrintEquations"],
    Cell[610509, 25768, 102, 1, 34, "ExampleText",
      CellTags->"PrintEquations"],
    Cell[610614, 25771, 170, 4, 42, "Input",
      CellTags->"PrintEquations"]},
  "PrintWitnesses"->{
    Cell[610787, 25777, 66, 1, 32, "ObjectName",
      CellTags->"PrintWitnesses"],
    Cell[610856, 25780, 59, 1, 50, "Highlight",
      CellTags->"PrintWitnesses"],
    Cell[610918, 25783, 204, 6, 31, "Usage",
      CellTags->"PrintWitnesses"],
    Cell[611125, 25791, 458, 19, 24, "Notes",
      CellTags->"PrintWitnesses"],
    Cell[611586, 25812, 72, 1, 39, "ExampleSection",
      CellTags->"PrintWitnesses"],
    Cell[611661, 25815, 142, 4, 34, "ExampleText",
      CellTags->"PrintWitnesses"],
    Cell[611806, 25821, 160, 4, 26, "Input",
      CellTags->"PrintWitnesses"],
    Cell[611969, 25827, 110, 3, 26, "Input",
      CellTags->"PrintWitnesses"]},
  "RegularSG"->{
    Cell[612082, 25832, 56, 1, 32, "ObjectName",
      CellTags->"RegularSG"],
    Cell[612141, 25835, 54, 1, 50, "Highlight",
      CellTags->"RegularSG"],
    Cell[612198, 25838, 166, 6, 31, "Usage",
      CellTags->"RegularSG"],
    Cell[612367, 25846, 81, 1, 24, "Notes",
      CellTags->"RegularSG"],
    Cell[612451, 25849, 204, 8, 24, "Notes",
      CellTags->"RegularSG"],
    Cell[612658, 25859, 162, 7, 24, "Notes",
      CellTags->"RegularSG"],
    Cell[612823, 25868, 67, 1, 39, "ExampleSection",
      CellTags->"RegularSG"],
    Cell[612893, 25871, 104, 1, 34, "ExampleText",
      CellTags->"RegularSG"],
    Cell[613000, 25874, 147, 4, 26, "Input",
      CellTags->"RegularSG"],
    Cell[613150, 25880, 121, 3, 26, "Input",
      CellTags->"RegularSG"],
    Cell[613274, 25885, 97, 3, 26, "Input",
      CellTags->"RegularSG"],
    Cell[613374, 25890, 94, 3, 26, "Input",
      CellTags->"RegularSG"]},
  "RandomT"->{
    Cell[613471, 25895, 52, 1, 32, "ObjectName",
      CellTags->"RandomT"],
    Cell[613526, 25898, 52, 1, 50, "Highlight",
      CellTags->"RandomT"],
    Cell[613581, 25901, 132, 4, 31, "Usage",
      CellTags->"RandomT"],
    Cell[613716, 25907, 155, 6, 24, "Notes",
      CellTags->"RandomT"],
    Cell[613874, 25915, 142, 7, 24, "Notes",
      CellTags->"RandomT"],
    Cell[614019, 25924, 65, 1, 39, "ExampleSection",
      CellTags->"RandomT"],
    Cell[614087, 25927, 74, 1, 34, "ExampleText",
      CellTags->"RandomT"],
    Cell[614164, 25930, 89, 2, 28, "Input",
      CellTags->"RandomT"]},
  "RankT"->{
    Cell[614256, 25934, 48, 1, 32, "ObjectName",
      CellTags->"RankT"],
    Cell[614307, 25937, 50, 1, 50, "Highlight",
      CellTags->"RankT"],
    Cell[614360, 25940, 105, 5, 24, "Notes",
      CellTags->"RankT"],
    Cell[614468, 25947, 196, 6, 31, "Usage",
      CellTags->"RankT"],
    Cell[614667, 25955, 105, 5, 24, "Notes",
      CellTags->"RankT"],
    Cell[614775, 25962, 458, 19, 24, "Notes",
      CellTags->"RankT"],
    Cell[615236, 25983, 63, 1, 39, "ExampleSection",
      CellTags->"RankT"],
    Cell[615302, 25986, 90, 1, 34, "ExampleText",
      CellTags->"RankT"],
    Cell[615395, 25989, 149, 4, 26, "Input",
      CellTags->"RankT"]},
  "RegularOnly"->{
    Cell[615547, 25995, 60, 1, 32, "ObjectName",
      CellTags->"RegularOnly"],
    Cell[615610, 25998, 54, 1, 50, "Highlight",
      CellTags->"RegularOnly"],
    Cell[615667, 26001, 169, 6, 31, "Usage",
      CellTags->"RegularOnly"],
    Cell[615839, 26009, 189, 6, 40, "Notes",
      CellTags->"RegularOnly"],
    Cell[616031, 26017, 186, 5, 40, "Notes",
      CellTags->"RegularOnly"],
    Cell[616220, 26024, 256, 11, 24, "Notes",
      CellTags->"RegularOnly"]},
  "RightIdealT"->{
    Cell[616479, 26037, 60, 1, 32, "ObjectName",
      CellTags->"RightIdealT"],
    Cell[616542, 26040, 56, 1, 50, "Highlight",
      CellTags->"RightIdealT"],
    Cell[616601, 26043, 170, 4, 31, "Usage",
      CellTags->"RightIdealT"],
    Cell[616774, 26049, 78, 1, 24, "Notes",
      CellTags->"RightIdealT"],
    Cell[616855, 26052, 252, 11, 24, "Notes",
      CellTags->"RightIdealT"],
    Cell[617110, 26065, 69, 1, 39, "ExampleSection",
      CellTags->"RightIdealT"],
    Cell[617182, 26068, 136, 4, 34, "ExampleText",
      CellTags->"RightIdealT"],
    Cell[617321, 26074, 115, 3, 26, "Input",
      CellTags->"RightIdealT"],
    Cell[617439, 26079, 119, 3, 26, "Input",
      CellTags->"RightIdealT"],
    Cell[617561, 26084, 119, 3, 26, "Input",
      CellTags->"RightIdealT"]},
  "RightNullSG"->{
    Cell[617683, 26089, 60, 1, 32, "ObjectName",
      CellTags->"RightNullSG"],
    Cell[617746, 26092, 56, 1, 50, "Highlight",
      CellTags->"RightNullSG"],
    Cell[617805, 26095, 164, 6, 31, "Usage",
      CellTags->"RightNullSG"],
    Cell[617972, 26103, 78, 1, 24, "Notes",
      CellTags->"RightNullSG"],
    Cell[618053, 26106, 218, 8, 24, "Notes",
      CellTags->"RightNullSG"],
    Cell[618274, 26116, 111, 5, 24, "Notes",
      CellTags->"RightNullSG"],
    Cell[618388, 26123, 357, 15, 24, "Notes",
      CellTags->"RightNullSG"],
    Cell[618748, 26140, 69, 1, 39, "ExampleSection",
      CellTags->"RightNullSG"],
    Cell[618820, 26143, 154, 4, 50, "ExampleText",
      CellTags->"RightNullSG"],
    Cell[618977, 26149, 95, 2, 28, "Input",
      CellTags->"RightNullSG"],
    Cell[619075, 26153, 131, 3, 26, "Input",
      CellTags->"RightNullSG"],
    Cell[619209, 26158, 101, 3, 26, "Input",
      CellTags->"RightNullSG"]},
  "RightOneSG"->{
    Cell[619313, 26163, 58, 1, 32, "ObjectName",
      CellTags->"RightOneSG"],
    Cell[619374, 26166, 55, 1, 50, "Highlight",
      CellTags->"RightOneSG"],
    Cell[619432, 26169, 168, 6, 31, "Usage",
      CellTags->"RightOneSG"],
    Cell[619603, 26177, 77, 1, 24, "Notes",
      CellTags->"RightOneSG"],
    Cell[619683, 26180, 220, 8, 24, "Notes",
      CellTags->"RightOneSG"],
    Cell[619906, 26190, 110, 5, 24, "Notes",
      CellTags->"RightOneSG"],
    Cell[620019, 26197, 336, 15, 24, "Notes",
      CellTags->"RightOneSG"],
    Cell[620358, 26214, 68, 1, 39, "ExampleSection",
      CellTags->"RightOneSG"],
    Cell[620429, 26217, 140, 4, 34, "ExampleText",
      CellTags->"RightOneSG"],
    Cell[620572, 26223, 92, 2, 28, "Input",
      CellTags->"RightOneSG"],
    Cell[620667, 26227, 129, 3, 26, "Input",
      CellTags->"RightOneSG"],
    Cell[620799, 26232, 97, 3, 26, "Input",
      CellTags->"RightOneSG"]},
  "ShiftT"->{
    Cell[620899, 26237, 50, 1, 32, "ObjectName",
      CellTags->"ShiftT"],
    Cell[620952, 26240, 54, 1, 50, "Highlight",
      CellTags->"ShiftT"],
    Cell[621009, 26243, 171, 6, 31, "Usage",
      CellTags->"ShiftT"],
    Cell[621183, 26251, 153, 6, 24, "Notes",
      CellTags->"ShiftT"],
    Cell[621339, 26259, 132, 4, 24, "Notes",
      CellTags->"ShiftT"],
    Cell[621474, 26265, 414, 16, 24, "Notes",
      CellTags->"ShiftT"],
    Cell[621891, 26283, 64, 1, 39, "ExampleSection",
      CellTags->"ShiftT"],
    Cell[621958, 26286, 101, 1, 34, "ExampleText",
      CellTags->"ShiftT"],
    Cell[622062, 26289, 106, 3, 26, "Input",
      CellTags->"ShiftT"],
    Cell[622171, 26294, 122, 4, 34, "ExampleText",
      CellTags->"ShiftT"],
    Cell[622296, 26300, 143, 4, 26, "Input",
      CellTags->"ShiftT"]},
  "SG"->{
    Cell[622442, 26306, 42, 1, 32, "ObjectName",
      CellTags->"SG"],
    Cell[622487, 26309, 50, 1, 50, "Highlight",
      CellTags->"SG"],
    Cell[622540, 26312, 124, 4, 31, "Usage",
      CellTags->"SG"],
    Cell[622667, 26318, 275, 9, 24, "Notes",
      CellTags->"SG"],
    Cell[622945, 26329, 513, 20, 24, "Notes",
      CellTags->"SG"],
    Cell[623461, 26351, 60, 1, 39, "ExampleSection",
      CellTags->"SG"],
    Cell[623524, 26354, 112, 4, 34, "ExampleText",
      CellTags->"SG"],
    Cell[623639, 26360, 95, 3, 26, "Input",
      CellTags->"SG"]},
  "SolveSG"->{
    Cell[623737, 26365, 52, 1, 32, "ObjectName",
      CellTags->"SolveSG"],
    Cell[623792, 26368, 52, 1, 50, "Highlight",
      CellTags->"SolveSG"],
    Cell[623847, 26371, 237, 8, 47, "Usage",
      CellTags->"SolveSG"],
    Cell[624087, 26381, 308, 10, 61, "Notes",
      CellTags->"SolveSG"],
    Cell[624398, 26393, 193, 7, 24, "Notes",
      CellTags->"SolveSG"],
    Cell[624594, 26402, 158, 7, 24, "Notes",
      CellTags->"SolveSG"],
    Cell[624755, 26411, 240, 11, 24, "Notes",
      CellTags->"SolveSG"],
    Cell[624998, 26424, 65, 1, 39, "ExampleSection",
      CellTags->"SolveSG"],
    Cell[625066, 26427, 93, 1, 34, "ExampleText",
      CellTags->"SolveSG"],
    Cell[625162, 26430, 90, 2, 28, "Input",
      CellTags->"SolveSG"],
    Cell[625255, 26434, 198, 5, 42, "Input",
      CellTags->"SolveSG"],
    Cell[625456, 26441, 108, 2, 34, "ExampleText",
      CellTags->"SolveSG"],
    Cell[625567, 26445, 90, 2, 28, "Input",
      CellTags->"SolveSG"],
    Cell[625660, 26449, 199, 5, 42, "Input",
      CellTags->"SolveSG"]},
  "SymmetricGroupSG"->{
    Cell[625862, 26456, 70, 1, 32, "ObjectName",
      CellTags->"SymmetricGroupSG"],
    Cell[625935, 26459, 61, 1, 50, "Highlight",
      CellTags->"SymmetricGroupSG"],
    Cell[625999, 26462, 148, 4, 31, "Usage",
      CellTags->"SymmetricGroupSG"],
    Cell[626150, 26468, 254, 9, 24, "Notes",
      CellTags->"SymmetricGroupSG"],
    Cell[626407, 26479, 74, 1, 39, "ExampleSection",
      CellTags->"SymmetricGroupSG"],
    Cell[626484, 26482, 87, 1, 34, "ExampleText",
      CellTags->"SymmetricGroupSG"],
    Cell[626574, 26485, 93, 2, 28, "Input",
      CellTags->"SymmetricGroupSG"]},
  "SyntacticCongruence"->{
    Cell[626670, 26489, 76, 1, 32, "ObjectName",
      CellTags->"SyntacticCongruence"],
    Cell[626749, 26492, 64, 1, 50, "Highlight",
      CellTags->"SyntacticCongruence"],
    Cell[626816, 26495, 214, 6, 47, "Usage",
      CellTags->"SyntacticCongruence"],
    Cell[627033, 26503, 212, 7, 24, "Notes",
      CellTags->"SyntacticCongruence"],
    Cell[627248, 26512, 510, 19, 24, "Notes",
      CellTags->"SyntacticCongruence"],
    Cell[627761, 26533, 77, 1, 39, "ExampleSection",
      CellTags->"SyntacticCongruence"],
    Cell[627841, 26536, 146, 4, 34, "ExampleText",
      CellTags->"SyntacticCongruence"],
    Cell[627990, 26542, 136, 3, 26, "Input",
      CellTags->"SyntacticCongruence"],
    Cell[628129, 26547, 128, 3, 26, "Input",
      CellTags->"SyntacticCongruence"],
    Cell[628260, 26552, 117, 2, 34, "ExampleText",
      CellTags->"SyntacticCongruence"],
    Cell[628380, 26556, 163, 4, 42, "Input",
      CellTags->"SyntacticCongruence"]},
  "SyntacticHomomorphism"->{
    Cell[628546, 26562, 80, 1, 32, "ObjectName",
      CellTags->"SyntacticHomomorphism"],
    Cell[628629, 26565, 66, 1, 50, "Highlight",
      CellTags->"SyntacticHomomorphism"],
    Cell[628698, 26568, 196, 6, 31, "Usage",
      CellTags->"SyntacticHomomorphism"],
    Cell[628897, 26576, 122, 5, 24, "Notes",
      CellTags->"SyntacticHomomorphism"],
    Cell[629022, 26583, 284, 9, 40, "Notes",
      CellTags->"SyntacticHomomorphism"],
    Cell[629309, 26594, 412, 15, 24, "Notes",
      CellTags->"SyntacticHomomorphism"],
    Cell[629724, 26611, 79, 1, 39, "ExampleSection",
      CellTags->"SyntacticHomomorphism"],
    Cell[629806, 26614, 110, 1, 34, "ExampleText",
      CellTags->"SyntacticHomomorphism"],
    Cell[629919, 26617, 113, 2, 28, "Input",
      CellTags->"SyntacticHomomorphism"],
    Cell[630035, 26621, 123, 3, 26, "Input",
      CellTags->"SyntacticHomomorphism"],
    Cell[630161, 26626, 100, 3, 26, "Input",
      CellTags->"SyntacticHomomorphism"],
    Cell[630264, 26631, 113, 3, 26, "Input",
      CellTags->"SyntacticHomomorphism"]},
  "SyntacticSG"->{
    Cell[630380, 26636, 60, 1, 32, "ObjectName",
      CellTags->"SyntacticSG"],
    Cell[630443, 26639, 56, 1, 50, "Highlight",
      CellTags->"SyntacticSG"],
    Cell[630502, 26642, 167, 6, 31, "Usage",
      CellTags->"SyntacticSG"],
    Cell[630672, 26650, 206, 7, 24, "Notes",
      CellTags->"SyntacticSG"],
    Cell[630881, 26659, 304, 11, 24, "Notes",
      CellTags->"SyntacticSG"],
    Cell[631188, 26672, 69, 1, 39, "ExampleSection",
      CellTags->"SyntacticSG"],
    Cell[631260, 26675, 135, 4, 34, "ExampleText",
      CellTags->"SyntacticSG"],
    Cell[631398, 26681, 128, 3, 26, "Input",
      CellTags->"SyntacticSG"],
    Cell[631529, 26686, 100, 3, 26, "Input",
      CellTags->"SyntacticSG"]},
  "T"->{
    Cell[631632, 26691, 40, 1, 32, "ObjectName",
      CellTags->"T"],
    Cell[631675, 26694, 49, 1, 50, "Highlight",
      CellTags->"T"],
    Cell[631727, 26697, 134, 4, 31, "Usage",
      CellTags->"T"],
    Cell[631864, 26703, 182, 7, 24, "Notes",
      CellTags->"T"],
    Cell[632049, 26712, 282, 11, 40, "Notes",
      CellTags->"T"],
    Cell[632334, 26725, 587, 27, 24, "Notes",
      CellTags->"T"],
    Cell[632924, 26754, 59, 1, 39, "ExampleSection",
      CellTags->"T"],
    Cell[632986, 26757, 135, 4, 34, "ExampleText",
      CellTags->"T"],
    Cell[633124, 26763, 96, 5, 45, "Input",
      CellTags->"T"],
    Cell[633223, 26770, 106, 4, 42, "Input",
      CellTags->"T"]},
  "TableSG"->{
    Cell[633332, 26776, 52, 1, 32, "ObjectName",
      CellTags->"TableSG"],
    Cell[633387, 26779, 52, 1, 50, "Highlight",
      CellTags->"TableSG"],
    Cell[633442, 26782, 213, 8, 31, "Usage",
      CellTags->"TableSG"],
    Cell[633658, 26792, 194, 7, 24, "Notes",
      CellTags->"TableSG"],
    Cell[633855, 26801, 92, 1, 24, "Notes",
      CellTags->"TableSG"],
    Cell[633950, 26804, 155, 7, 24, "Notes",
      CellTags->"TableSG"],
    Cell[634108, 26813, 65, 1, 39, "ExampleSection",
      CellTags->"TableSG"],
    Cell[634176, 26816, 117, 4, 34, "ExampleText",
      CellTags->"TableSG"],
    Cell[634296, 26822, 152, 4, 26, "Input",
      CellTags->"TableSG"],
    Cell[634451, 26828, 97, 3, 26, "Input",
      CellTags->"TableSG"]},
  "ToIdentityString"->{
    Cell[634551, 26833, 70, 1, 32, "ObjectName",
      CellTags->"ToIdentityString"],
    Cell[634624, 26836, 61, 1, 50, "Highlight",
      CellTags->"ToIdentityString"],
    Cell[634688, 26839, 233, 8, 47, "Usage",
      CellTags->"ToIdentityString"],
    Cell[634924, 26849, 289, 11, 24, "Notes",
      CellTags->"ToIdentityString"],
    Cell[635216, 26862, 74, 1, 39, "ExampleSection",
      CellTags->"ToIdentityString"],
    Cell[635293, 26865, 83, 1, 34, "ExampleText",
      CellTags->"ToIdentityString"],
    Cell[635379, 26868, 110, 2, 28, "Input",
      CellTags->"ToIdentityString"]},
  "TransformationSG"->{
    Cell[635492, 26872, 70, 1, 32, "ObjectName",
      CellTags->"TransformationSG"],
    Cell[635565, 26875, 61, 1, 50, "Highlight",
      CellTags->"TransformationSG"],
    Cell[635629, 26878, 186, 6, 31, "Usage",
      CellTags->"TransformationSG"],
    Cell[635818, 26886, 254, 8, 24, "Notes",
      CellTags->"TransformationSG"],
    Cell[636075, 26896, 384, 15, 24, "Notes",
      CellTags->"TransformationSG"],
    Cell[636462, 26913, 74, 1, 39, "ExampleSection",
      CellTags->"TransformationSG"],
    Cell[636539, 26916, 101, 1, 34, "ExampleText",
      CellTags->"TransformationSG"],
    Cell[636643, 26919, 98, 2, 28, "Input",
      CellTags->"TransformationSG"],
    Cell[636744, 26923, 106, 3, 26, "Input",
      CellTags->"TransformationSG"]},
  "TransientPeriodicT"->{
    Cell[636853, 26928, 74, 1, 32, "ObjectName",
      CellTags->"TransientPeriodicT"],
    Cell[636930, 26931, 63, 1, 50, "Highlight",
      CellTags->"TransientPeriodicT"],
    Cell[636996, 26934, 244, 7, 47, "Usage",
      CellTags->"TransientPeriodicT"],
    Cell[637243, 26943, 163, 4, 24, "Notes",
      CellTags->"TransientPeriodicT"],
    Cell[637409, 26949, 163, 7, 24, "Notes",
      CellTags->"TransientPeriodicT"],
    Cell[637575, 26958, 76, 1, 39, "ExampleSection",
      CellTags->"TransientPeriodicT"],
    Cell[637654, 26961, 160, 4, 34, "ExampleText",
      CellTags->"TransientPeriodicT"],
    Cell[637817, 26967, 122, 3, 26, "Input",
      CellTags->"TransientPeriodicT"],
    Cell[637942, 26972, 129, 3, 26, "Input",
      CellTags->"TransientPeriodicT"],
    Cell[638074, 26977, 167, 4, 26, "Input",
      CellTags->"TransientPeriodicT"]},
  "TranspositionT"->{
    Cell[638244, 26983, 66, 1, 32, "ObjectName",
      CellTags->"TranspositionT"],
    Cell[638313, 26986, 59, 1, 50, "Highlight",
      CellTags->"TranspositionT"],
    Cell[638375, 26989, 222, 8, 31, "Usage",
      CellTags->"TranspositionT"],
    Cell[638600, 26999, 122, 5, 24, "Notes",
      CellTags->"TranspositionT"],
    Cell[638725, 27006, 251, 11, 24, "Notes",
      CellTags->"TranspositionT"],
    Cell[638979, 27019, 72, 1, 39, "ExampleSection",
      CellTags->"TranspositionT"],
    Cell[639054, 27022, 114, 2, 34, "ExampleText",
      CellTags->"TranspositionT"],
    Cell[639171, 27026, 93, 2, 28, "Input",
      CellTags->"TranspositionT"],
    Cell[639267, 27030, 127, 2, 28, "Input",
      CellTags->"TranspositionT"],
    Cell[639397, 27034, 105, 3, 26, "Input",
      CellTags->"TranspositionT"]},
  "UnitElement"->{
    Cell[639505, 27039, 60, 1, 32, "ObjectName",
      CellTags->"UnitElement"],
    Cell[639568, 27042, 56, 1, 50, "Highlight",
      CellTags->"UnitElement"],
    Cell[639627, 27045, 195, 6, 31, "Usage",
      CellTags->"UnitElement"],
    Cell[639825, 27053, 210, 6, 40, "Notes",
      CellTags->"UnitElement"],
    Cell[640038, 27061, 118, 5, 24, "Notes",
      CellTags->"UnitElement"],
    Cell[640159, 27068, 159, 7, 24, "Notes",
      CellTags->"UnitElement"],
    Cell[640321, 27077, 69, 1, 39, "ExampleSection",
      CellTags->"UnitElement"],
    Cell[640393, 27080, 69, 1, 34, "ExampleText",
      CellTags->"UnitElement"],
    Cell[640465, 27083, 91, 2, 28, "Input",
      CellTags->"UnitElement"],
    Cell[640559, 27087, 96, 2, 28, "Input",
      CellTags->"UnitElement"]},
  "$rewriteSimpBound"->{
    Cell[640658, 27091, 72, 1, 32, "ObjectName",
      CellTags->"$rewriteSimpBound"],
    Cell[640733, 27094, 65, 1, 50, "Highlight",
      CellTags->"$rewriteSimpBound"],
    Cell[640801, 27097, 225, 5, 47, "Usage",
      CellTags->"$rewriteSimpBound"],
    Cell[641029, 27104, 71, 1, 24, "Notes",
      CellTags->"$rewriteSimpBound"],
    Cell[641103, 27107, 187, 7, 24, "Notes",
      CellTags->"$rewriteSimpBound"],
    Cell[641293, 27116, 75, 1, 39, "ExampleSection",
      CellTags->"$rewriteSimpBound"],
    Cell[641371, 27119, 88, 1, 34, "ExampleText",
      CellTags->"$rewriteSimpBound"],
    Cell[641462, 27122, 110, 3, 26, "Input",
      CellTags->"$rewriteSimpBound"]},
  "AllStringReplace"->{
    Cell[641575, 27127, 70, 1, 32, "ObjectName",
      CellTags->"AllStringReplace"],
    Cell[641648, 27130, 69, 1, 50, "Highlight",
      CellTags->"AllStringReplace"],
    Cell[641720, 27133, 256, 8, 47, "Usage",
      CellTags->"AllStringReplace"],
    Cell[641979, 27143, 247, 7, 40, "Notes",
      CellTags->"AllStringReplace"],
    Cell[642229, 27152, 124, 5, 24, "Notes",
      CellTags->"AllStringReplace"],
    Cell[642356, 27159, 159, 7, 24, "Notes",
      CellTags->"AllStringReplace"],
    Cell[642518, 27168, 74, 1, 39, "ExampleSection",
      CellTags->"AllStringReplace"],
    Cell[642595, 27171, 103, 1, 34, "ExampleText",
      CellTags->"AllStringReplace"],
    Cell[642701, 27174, 114, 2, 28, "Input",
      CellTags->"AllStringReplace"],
    Cell[642818, 27178, 147, 3, 26, "Input",
      CellTags->"AllStringReplace"],
    Cell[642968, 27183, 141, 3, 26, "Input",
      CellTags->"AllStringReplace"]},
  "BoundETW"->{
    Cell[643112, 27188, 54, 1, 32, "ObjectName",
      CellTags->"BoundETW"],
    Cell[643169, 27191, 51, 1, 50, "Highlight",
      CellTags->"BoundETW"],
    Cell[643223, 27194, 156, 6, 31, "Usage",
      CellTags->"BoundETW"],
    Cell[643382, 27202, 56, 1, 24, "Notes",
      CellTags->"BoundETW"],
    Cell[643441, 27205, 176, 7, 24, "Notes",
      CellTags->"BoundETW"]},
  "BoundWR"->{
    Cell[643620, 27214, 52, 1, 32, "ObjectName",
      CellTags->"BoundWR"],
    Cell[643675, 27217, 50, 1, 50, "Highlight",
      CellTags->"BoundWR"],
    Cell[643728, 27220, 182, 6, 31, "Usage",
      CellTags->"BoundWR"],
    Cell[643913, 27228, 55, 1, 24, "Notes",
      CellTags->"BoundWR"],
    Cell[643971, 27231, 155, 7, 24, "Notes",
      CellTags->"BoundWR"]},
  "Cols"->{
    Cell[644129, 27240, 46, 1, 32, "ObjectName",
      CellTags->"Cols"],
    Cell[644178, 27243, 47, 1, 50, "Highlight",
      CellTags->"Cols"],
    Cell[644228, 27246, 137, 6, 31, "Usage",
      CellTags->"Cols"],
    Cell[644368, 27254, 160, 7, 24, "Notes",
      CellTags->"Cols"]},
  "EquationsToWitnesses"->{
    Cell[644531, 27263, 78, 1, 32, "ObjectName",
      CellTags->"EquationsToWitnesses"],
    Cell[644612, 27266, 65, 1, 50, "Highlight",
      CellTags->"EquationsToWitnesses"],
    Cell[644680, 27269, 285, 8, 47, "Usage",
      CellTags->"EquationsToWitnesses"],
    Cell[644968, 27279, 283, 10, 61, "Notes",
      CellTags->"EquationsToWitnesses"],
    Cell[645254, 27291, 217, 7, 24, "Notes",
      CellTags->"EquationsToWitnesses"],
    Cell[645474, 27300, 185, 4, 40, "Notes",
      CellTags->"EquationsToWitnesses"],
    Cell[645662, 27306, 173, 7, 24, "Notes",
      CellTags->"EquationsToWitnesses"],
    Cell[645838, 27315, 78, 1, 39, "ExampleSection",
      CellTags->"EquationsToWitnesses"],
    Cell[645919, 27318, 194, 6, 34, "ExampleText",
      CellTags->"EquationsToWitnesses"],
    Cell[646116, 27326, 161, 4, 26, "Input",
      CellTags->"EquationsToWitnesses"],
    Cell[646280, 27332, 180, 4, 26, "Input",
      CellTags->"EquationsToWitnesses"],
    Cell[646463, 27338, 391, 12, 34, "ExampleText",
      CellTags->"EquationsToWitnesses"],
    Cell[646857, 27352, 194, 4, 42, "Input",
      CellTags->"EquationsToWitnesses"],
    Cell[647054, 27358, 137, 4, 34, "ExampleText",
      CellTags->"EquationsToWitnesses"],
    Cell[647194, 27364, 203, 4, 42, "Input",
      CellTags->"EquationsToWitnesses"],
    Cell[647400, 27370, 109, 3, 26, "Input",
      CellTags->"EquationsToWitnesses"]},
  "Equations"->{
    Cell[647512, 27375, 56, 1, 32, "ObjectName",
      CellTags->"Equations"],
    Cell[647571, 27378, 52, 1, 50, "Highlight",
      CellTags->"Equations"],
    Cell[647626, 27381, 126, 4, 31, "Usage",
      CellTags->"Equations"],
    Cell[647755, 27387, 255, 11, 24, "Notes",
      CellTags->"Equations"]},
  "FromIdentityString"->{
    Cell[648013, 27400, 74, 1, 32, "ObjectName",
      CellTags->"FromIdentityString"],
    Cell[648090, 27403, 63, 1, 50, "Highlight",
      CellTags->"FromIdentityString"],
    Cell[648156, 27406, 236, 8, 47, "Usage",
      CellTags->"FromIdentityString"],
    Cell[648395, 27416, 125, 4, 24, "Notes",
      CellTags->"FromIdentityString"],
    Cell[648523, 27422, 147, 2, 24, "Notes",
      CellTags->"FromIdentityString"],
    Cell[648673, 27426, 1040, 39, 56, "Notes",
      CellTags->"FromIdentityString"],
    Cell[649716, 27467, 76, 1, 39, "ExampleSection",
      CellTags->"FromIdentityString"],
    Cell[649795, 27470, 106, 1, 34, "ExampleText",
      CellTags->"FromIdentityString"],
    Cell[649904, 27473, 117, 2, 28, "Input",
      CellTags->"FromIdentityString"],
    Cell[650024, 27477, 97, 2, 28, "Input",
      CellTags->"FromIdentityString"],
    Cell[650124, 27481, 117, 2, 28, "Input",
      CellTags->"FromIdentityString"]},
  "IrreducibleQ"->{
    Cell[650244, 27485, 62, 1, 32, "ObjectName",
      CellTags->"IrreducibleQ"],
    Cell[650309, 27488, 57, 1, 50, "Highlight",
      CellTags->"IrreducibleQ"],
    Cell[650369, 27491, 228, 8, 31, "Usage",
      CellTags->"IrreducibleQ"],
    Cell[650600, 27501, 139, 5, 24, "Notes",
      CellTags->"IrreducibleQ"],
    Cell[650742, 27508, 275, 11, 24, "Notes",
      CellTags->"IrreducibleQ"],
    Cell[651020, 27521, 70, 1, 39, "ExampleSection",
      CellTags->"IrreducibleQ"],
    Cell[651093, 27524, 74, 1, 34, "ExampleText",
      CellTags->"IrreducibleQ"],
    Cell[651170, 27527, 127, 3, 26, "Input",
      CellTags->"IrreducibleQ"],
    Cell[651300, 27532, 118, 3, 26, "Input",
      CellTags->"IrreducibleQ"],
    Cell[651421, 27537, 91, 2, 26, "Input",
      CellTags->"IrreducibleQ"]},
  "PairsToRules"->{
    Cell[651515, 27541, 62, 1, 32, "ObjectName",
      CellTags->"PairsToRules"],
    Cell[651580, 27544, 57, 1, 50, "Highlight",
      CellTags->"PairsToRules"],
    Cell[651640, 27547, 130, 4, 31, "Usage",
      CellTags->"PairsToRules"],
    Cell[651773, 27553, 565, 19, 24, "Notes",
      CellTags->"PairsToRules"],
    Cell[652341, 27574, 70, 1, 39, "ExampleSection",
      CellTags->"PairsToRules"],
    Cell[652414, 27577, 103, 1, 34, "ExampleText",
      CellTags->"PairsToRules"],
    Cell[652520, 27580, 211, 4, 42, "Input",
      CellTags->"PairsToRules"],
    Cell[652734, 27586, 118, 3, 26, "Input",
      CellTags->"PairsToRules"]},
  "RulesToPairs"->{
    Cell[652855, 27591, 62, 1, 32, "ObjectName",
      CellTags->"RulesToPairs"],
    Cell[652920, 27594, 57, 1, 50, "Highlight",
      CellTags->"RulesToPairs"],
    Cell[652980, 27597, 138, 4, 31, "Usage",
      CellTags->"RulesToPairs"],
    Cell[653121, 27603, 565, 19, 24, "Notes",
      CellTags->"RulesToPairs"],
    Cell[653689, 27624, 70, 1, 39, "ExampleSection",
      CellTags->"RulesToPairs"],
    Cell[653762, 27627, 100, 1, 34, "ExampleText",
      CellTags->"RulesToPairs"],
    Cell[653865, 27630, 241, 5, 42, "Input",
      CellTags->"RulesToPairs"],
    Cell[654109, 27637, 102, 3, 26, "Input",
      CellTags->"RulesToPairs"]},
  "SimplifyEquations"->{
    Cell[654214, 27642, 72, 1, 32, "ObjectName",
      CellTags->"SimplifyEquations"],
    Cell[654289, 27645, 62, 1, 50, "Highlight",
      CellTags->"SimplifyEquations"],
    Cell[654354, 27648, 226, 6, 47, "Usage",
      CellTags->"SimplifyEquations"],
    Cell[654583, 27656, 232, 9, 44, "Notes",
      CellTags->"SimplifyEquations"],
    Cell[654818, 27667, 275, 9, 40, "Notes",
      CellTags->"SimplifyEquations"],
    Cell[655096, 27678, 288, 11, 24, "Notes",
      CellTags->"SimplifyEquations"],
    Cell[655387, 27691, 75, 1, 39, "ExampleSection",
      CellTags->"SimplifyEquations"],
    Cell[655465, 27694, 141, 4, 34, "ExampleText",
      CellTags->"SimplifyEquations"],
    Cell[655609, 27700, 161, 4, 26, "Input",
      CellTags->"SimplifyEquations"],
    Cell[655773, 27706, 107, 3, 26, "Input",
      CellTags->"SimplifyEquations"],
    Cell[655883, 27711, 140, 3, 26, "Input",
      CellTags->"SimplifyEquations"]},
  "SimplifyRewriteSystem"->{
    Cell[656026, 27716, 80, 1, 32, "ObjectName",
      CellTags->"SimplifyRewriteSystem"],
    Cell[656109, 27719, 66, 1, 50, "Highlight",
      CellTags->"SimplifyRewriteSystem"],
    Cell[656178, 27722, 211, 6, 47, "Usage",
      CellTags->"SimplifyRewriteSystem"],
    Cell[656392, 27730, 240, 9, 44, "Notes",
      CellTags->"SimplifyRewriteSystem"],
    Cell[656635, 27741, 254, 6, 40, "Notes",
      CellTags->"SimplifyRewriteSystem"],
    Cell[656892, 27749, 218, 6, 40, "Notes",
      CellTags->"SimplifyRewriteSystem"],
    Cell[657113, 27757, 188, 7, 24, "Notes",
      CellTags->"SimplifyRewriteSystem"],
    Cell[657304, 27766, 79, 1, 39, "ExampleSection",
      CellTags->"SimplifyRewriteSystem"],
    Cell[657386, 27769, 199, 5, 50, "ExampleText",
      CellTags->"SimplifyRewriteSystem"],
    Cell[657588, 27776, 165, 4, 26, "Input",
      CellTags->"SimplifyRewriteSystem"],
    Cell[657756, 27782, 111, 3, 26, "Input",
      CellTags->"SimplifyRewriteSystem"],
    Cell[657870, 27787, 135, 3, 26, "Input",
      CellTags->"SimplifyRewriteSystem"],
    Cell[658008, 27792, 114, 3, 26, "Input",
      CellTags->"SimplifyRewriteSystem"]},
  "WordReduce"->{
    Cell[658125, 27797, 58, 1, 32, "ObjectName",
      CellTags->"WordReduce"],
    Cell[658186, 27800, 55, 1, 50, "Highlight",
      CellTags->"WordReduce"],
    Cell[658244, 27803, 201, 8, 31, "Usage",
      CellTags->"WordReduce"],
    Cell[658448, 27813, 275, 10, 61, "Notes",
      CellTags->"WordReduce"],
    Cell[658726, 27825, 288, 8, 40, "Notes",
      CellTags->"WordReduce"],
    Cell[659017, 27835, 147, 5, 24, "Notes",
      CellTags->"WordReduce"],
    Cell[659167, 27842, 121, 4, 24, "Notes",
      CellTags->"WordReduce"],
    Cell[659291, 27848, 295, 11, 24, "Notes",
      CellTags->"WordReduce"],
    Cell[659589, 27861, 68, 1, 39, "ExampleSection",
      CellTags->"WordReduce"],
    Cell[659660, 27864, 72, 1, 34, "ExampleText",
      CellTags->"WordReduce"],
    Cell[659735, 27867, 142, 3, 26, "Input",
      CellTags->"WordReduce"],
    Cell[659880, 27872, 123, 3, 26, "Input",
      CellTags->"WordReduce"],
    Cell[660006, 27877, 152, 4, 26, "Input",
      CellTags->"WordReduce"],
    Cell[660161, 27883, 99, 1, 34, "ExampleText",
      CellTags->"WordReduce"],
    Cell[660263, 27886, 155, 4, 26, "Input",
      CellTags->"WordReduce"]},
  "BooleanComplement"->{
    Cell[660421, 27892, 72, 1, 32, "ObjectName",
      CellTags->"BooleanComplement"],
    Cell[660496, 27895, 62, 1, 50, "Highlight",
      CellTags->"BooleanComplement"],
    Cell[660561, 27898, 324, 9, 47, "Usage",
      CellTags->"BooleanComplement"],
    Cell[660888, 27909, 155, 4, 24, "Notes",
      CellTags->"BooleanComplement"],
    Cell[661046, 27915, 818, 27, 40, "Notes",
      CellTags->"BooleanComplement"],
    Cell[661867, 27944, 75, 1, 39, "ExampleSection",
      CellTags->"BooleanComplement"],
    Cell[661945, 27947, 99, 1, 34, "ExampleText",
      CellTags->"BooleanComplement"],
    Cell[662047, 27950, 146, 5, 45, "Input",
      CellTags->"BooleanComplement"],
    Cell[662196, 27957, 99, 2, 28, "Input",
      CellTags->"BooleanComplement"],
    Cell[662298, 27961, 96, 2, 28, "Input",
      CellTags->"BooleanComplement"]},
  "BooleanComposition"->{
    Cell[662397, 27965, 74, 1, 32, "ObjectName",
      CellTags->"BooleanComposition"],
    Cell[662474, 27968, 63, 1, 50, "Highlight",
      CellTags->"BooleanComposition"],
    Cell[662540, 27971, 281, 8, 47, "Usage",
      CellTags->"BooleanComposition"],
    Cell[662824, 27981, 156, 4, 24, "Notes",
      CellTags->"BooleanComposition"],
    Cell[662983, 27987, 817, 27, 40, "Notes",
      CellTags->"BooleanComposition"],
    Cell[663803, 28016, 76, 1, 39, "ExampleSection",
      CellTags->"BooleanComposition"],
    Cell[663882, 28019, 199, 6, 34, "ExampleText",
      CellTags->"BooleanComposition"],
    Cell[664084, 28027, 111, 2, 28, "Input",
      CellTags->"BooleanComposition"],
    Cell[664198, 28031, 134, 4, 34, "ExampleText",
      CellTags->"BooleanComposition"],
    Cell[664335, 28037, 165, 4, 42, "Input",
      CellTags->"BooleanComposition"],
    Cell[664503, 28043, 174, 4, 42, "Input",
      CellTags->"BooleanComposition"]},
  "BooleanConverse"->{
    Cell[664680, 28049, 68, 1, 32, "ObjectName",
      CellTags->"BooleanConverse"],
    Cell[664751, 28052, 60, 1, 50, "Highlight",
      CellTags->"BooleanConverse"],
    Cell[664814, 28055, 218, 6, 47, "Usage",
      CellTags->"BooleanConverse"],
    Cell[665035, 28063, 231, 7, 24, "Notes",
      CellTags->"BooleanConverse"],
    Cell[665269, 28072, 153, 4, 24, "Notes",
      CellTags->"BooleanConverse"],
    Cell[665425, 28078, 611, 21, 40, "Notes",
      CellTags->"BooleanConverse"],
    Cell[666039, 28101, 73, 1, 39, "ExampleSection",
      CellTags->"BooleanConverse"],
    Cell[666115, 28104, 214, 6, 34, "ExampleText",
      CellTags->"BooleanConverse"],
    Cell[666332, 28112, 134, 5, 45, "Input",
      CellTags->"BooleanConverse"],
    Cell[666469, 28119, 106, 2, 28, "Input",
      CellTags->"BooleanConverse"]},
  "BooleanIntersection"->{
    Cell[666578, 28123, 76, 1, 32, "ObjectName",
      CellTags->"BooleanIntersection"],
    Cell[666657, 28126, 64, 1, 50, "Highlight",
      CellTags->"BooleanIntersection"],
    Cell[666724, 28129, 292, 8, 47, "Usage",
      CellTags->"BooleanIntersection"],
    Cell[667019, 28139, 157, 4, 24, "Notes",
      CellTags->"BooleanIntersection"],
    Cell[667179, 28145, 816, 27, 40, "Notes",
      CellTags->"BooleanIntersection"],
    Cell[667998, 28174, 77, 1, 39, "ExampleSection",
      CellTags->"BooleanIntersection"],
    Cell[668078, 28177, 143, 4, 34, "ExampleText",
      CellTags->"BooleanIntersection"],
    Cell[668224, 28183, 144, 5, 45, "Input",
      CellTags->"BooleanIntersection"],
    Cell[668371, 28190, 147, 5, 45, "Input",
      CellTags->"BooleanIntersection"],
    Cell[668521, 28197, 117, 2, 28, "Input",
      CellTags->"BooleanIntersection"]},
  "BooleanPower"->{
    Cell[668641, 28201, 62, 1, 32, "ObjectName",
      CellTags->"BooleanPower"],
    Cell[668706, 28204, 57, 1, 50, "Highlight",
      CellTags->"BooleanPower"],
    Cell[668766, 28207, 246, 8, 47, "Usage",
      CellTags->"BooleanPower"],
    Cell[669015, 28217, 150, 4, 24, "Notes",
      CellTags->"BooleanPower"],
    Cell[669168, 28223, 823, 27, 40, "Notes",
      CellTags->"BooleanPower"],
    Cell[669994, 28252, 70, 1, 39, "ExampleSection",
      CellTags->"BooleanPower"],
    Cell[670067, 28255, 193, 6, 34, "ExampleText",
      CellTags->"BooleanPower"],
    Cell[670263, 28263, 105, 2, 28, "Input",
      CellTags->"BooleanPower"],
    Cell[670371, 28267, 126, 4, 34, "ExampleText",
      CellTags->"BooleanPower"],
    Cell[670500, 28273, 153, 4, 42, "Input",
      CellTags->"BooleanPower"]},
  "BooleanUnion"->{
    Cell[670656, 28279, 62, 1, 32, "ObjectName",
      CellTags->"BooleanUnion"],
    Cell[670721, 28282, 57, 1, 50, "Highlight",
      CellTags->"BooleanUnion"],
    Cell[670781, 28285, 266, 8, 47, "Usage",
      CellTags->"BooleanUnion"],
    Cell[671050, 28295, 150, 4, 24, "Notes",
      CellTags->"BooleanUnion"],
    Cell[671203, 28301, 823, 27, 40, "Notes",
      CellTags->"BooleanUnion"],
    Cell[672029, 28330, 70, 1, 39, "ExampleSection",
      CellTags->"BooleanUnion"],
    Cell[672102, 28333, 135, 4, 34, "ExampleText",
      CellTags->"BooleanUnion"],
    Cell[672240, 28339, 131, 5, 45, "Input",
      CellTags->"BooleanUnion"],
    Cell[672374, 28346, 134, 5, 45, "Input",
      CellTags->"BooleanUnion"],
    Cell[672511, 28353, 104, 2, 28, "Input",
      CellTags->"BooleanUnion"]},
  "Boolean"->{
    Cell[672618, 28357, 52, 1, 32, "ObjectName",
      CellTags->"Boolean"],
    Cell[672673, 28360, 50, 1, 50, "Highlight",
      CellTags->"Boolean"],
    Cell[672726, 28363, 242, 7, 47, "Usage",
      CellTags->"Boolean"],
    Cell[672971, 28372, 145, 4, 24, "Notes",
      CellTags->"Boolean"],
    Cell[673119, 28378, 216, 9, 24, "Notes",
      CellTags->"Boolean"],
    Cell[673338, 28389, 65, 1, 39, "ExampleSection",
      CellTags->"Boolean"],
    Cell[673406, 28392, 72, 1, 34, "ExampleText",
      CellTags->"Boolean"],
    Cell[673481, 28395, 107, 3, 26, "Input",
      CellTags->"Boolean"],
    Cell[673591, 28400, 127, 3, 26, "Input",
      CellTags->"Boolean"],
    Cell[673721, 28405, 83, 2, 28, "Input",
      CellTags->"Boolean"],
    Cell[673807, 28409, 86, 2, 28, "Input",
      CellTags->"Boolean"]},
  "RelationToMatrix"->{
    Cell[673896, 28413, 70, 1, 32, "ObjectName",
      CellTags->"RelationToMatrix"],
    Cell[673969, 28416, 61, 1, 50, "Highlight",
      CellTags->"RelationToMatrix"],
    Cell[674033, 28419, 245, 8, 47, "Usage",
      CellTags->"RelationToMatrix"],
    Cell[674281, 28429, 199, 7, 24, "Notes",
      CellTags->"RelationToMatrix"],
    Cell[674483, 28438, 108, 1, 24, "Notes",
      CellTags->"RelationToMatrix"],
    Cell[674594, 28441, 830, 31, 40, "Notes",
      CellTags->"RelationToMatrix"],
    Cell[675427, 28474, 74, 1, 39, "ExampleSection",
      CellTags->"RelationToMatrix"],
    Cell[675504, 28477, 86, 1, 34, "ExampleText",
      CellTags->"RelationToMatrix"],
    Cell[675593, 28480, 122, 2, 28, "Input",
      CellTags->"RelationToMatrix"]},
  "ToClasses"->{
    Cell[675718, 28484, 56, 1, 32, "ObjectName",
      CellTags->"ToClasses"],
    Cell[675777, 28487, 54, 1, 50, "Highlight",
      CellTags->"ToClasses"],
    Cell[675834, 28490, 236, 8, 47, "Usage",
      CellTags->"ToClasses"],
    Cell[676073, 28500, 252, 9, 44, "Notes",
      CellTags->"ToClasses"],
    Cell[676328, 28511, 337, 11, 40, "Notes",
      CellTags->"ToClasses"],
    Cell[676668, 28524, 181, 7, 24, "Notes",
      CellTags->"ToClasses"],
    Cell[676852, 28533, 263, 11, 24, "Notes",
      CellTags->"ToClasses"],
    Cell[677118, 28546, 67, 1, 39, "ExampleSection",
      CellTags->"ToClasses"],
    Cell[677188, 28549, 124, 4, 34, "ExampleText",
      CellTags->"ToClasses"],
    Cell[677315, 28555, 139, 3, 26, "Input",
      CellTags->"ToClasses"],
    Cell[677457, 28560, 118, 3, 26, "Input",
      CellTags->"ToClasses"],
    Cell[677578, 28565, 90, 1, 34, "ExampleText",
      CellTags->"ToClasses"],
    Cell[677671, 28568, 127, 3, 26, "Input",
      CellTags->"ToClasses"],
    Cell[677801, 28573, 136, 3, 26, "Input",
      CellTags->"ToClasses"],
    Cell[677940, 28578, 76, 1, 34, "ExampleText",
      CellTags->"ToClasses"],
    Cell[678019, 28581, 111, 3, 26, "Input",
      CellTags->"ToClasses"],
    Cell[678133, 28586, 138, 3, 26, "Input",
      CellTags->"ToClasses"],
    Cell[678274, 28591, 66, 1, 34, "ExampleText",
      CellTags->"ToClasses"],
    Cell[678343, 28594, 135, 3, 26, "Input",
      CellTags->"ToClasses"],
    Cell[678481, 28599, 137, 3, 26, "Input",
      CellTags->"ToClasses"]},
  "TransitiveClosure"->{
    Cell[678621, 28604, 72, 1, 32, "ObjectName",
      CellTags->"TransitiveClosure"],
    Cell[678696, 28607, 62, 1, 50, "Highlight",
      CellTags->"TransitiveClosure"],
    Cell[678761, 28610, 193, 6, 31, "Usage",
      CellTags->"TransitiveClosure"],
    Cell[678957, 28618, 238, 9, 44, "Notes",
      CellTags->"TransitiveClosure"],
    Cell[679198, 28629, 211, 6, 40, "Notes",
      CellTags->"TransitiveClosure"],
    Cell[679412, 28637, 424, 11, 40, "Notes",
      CellTags->"TransitiveClosure"],
    Cell[679839, 28650, 560, 19, 40, "Notes",
      CellTags->"TransitiveClosure"],
    Cell[680402, 28671, 104, 1, 24, "Notes",
      CellTags->"TransitiveClosure"],
    Cell[680509, 28674, 402, 15, 24, "Notes",
      CellTags->"TransitiveClosure"],
    Cell[680914, 28691, 75, 1, 39, "ExampleSection",
      CellTags->"TransitiveClosure"],
    Cell[680992, 28694, 91, 1, 34, "ExampleText",
      CellTags->"TransitiveClosure"],
    Cell[681086, 28697, 347, 6, 74, "Input",
      CellTags->"TransitiveClosure"],
    Cell[681436, 28705, 110, 3, 26, "Input",
      CellTags->"TransitiveClosure"],
    Cell[681549, 28710, 127, 3, 26, "Input",
      CellTags->"TransitiveClosure"]},
  "Warshall"->{
    Cell[681679, 28715, 54, 1, 32, "ObjectName",
      CellTags->"Warshall"],
    Cell[681736, 28718, 53, 1, 50, "Highlight",
      CellTags->"Warshall"],
    Cell[681792, 28721, 200, 6, 31, "Usage",
      CellTags->"Warshall"],
    Cell[681995, 28729, 383, 14, 64, "Notes",
      CellTags->"Warshall"],
    Cell[682381, 28745, 126, 4, 24, "Notes",
      CellTags->"Warshall"],
    Cell[682510, 28751, 164, 5, 24, "Notes",
      CellTags->"Warshall"],
    Cell[682677, 28758, 256, 7, 40, "Notes",
      CellTags->"Warshall"],
    Cell[682936, 28767, 120, 4, 24, "Notes",
      CellTags->"Warshall"],
    Cell[683059, 28773, 265, 11, 24, "Notes",
      CellTags->"Warshall"],
    Cell[683327, 28786, 66, 1, 39, "ExampleSection",
      CellTags->"Warshall"],
    Cell[683396, 28789, 178, 6, 34, "ExampleText",
      CellTags->"Warshall"],
    Cell[683577, 28797, 135, 5, 45, "Input",
      CellTags->"Warshall"],
    Cell[683715, 28804, 110, 3, 26, "Input",
      CellTags->"Warshall"],
    Cell[683828, 28809, 136, 3, 26, "Input",
      CellTags->"Warshall"],
    Cell[683967, 28814, 146, 4, 34, "ExampleText",
      CellTags->"Warshall"],
    Cell[684116, 28820, 475, 11, 111, "Input",
      CellTags->"Warshall"],
    Cell[684594, 28833, 127, 4, 34, "ExampleText",
      CellTags->"Warshall"],
    Cell[684724, 28839, 98, 3, 26, "Input",
      CellTags->"Warshall"]},
  "AnalyzeOrbit"->{
    Cell[684825, 28844, 62, 1, 32, "ObjectName",
      CellTags->"AnalyzeOrbit"],
    Cell[684890, 28847, 57, 1, 50, "Highlight",
      CellTags->"AnalyzeOrbit"],
    Cell[684950, 28850, 472, 15, 63, "Usage",
      CellTags->"AnalyzeOrbit"],
    Cell[685425, 28867, 339, 11, 79, "Notes",
      CellTags->"AnalyzeOrbit"],
    Cell[685767, 28880, 250, 7, 40, "Notes",
      CellTags->"AnalyzeOrbit"],
    Cell[686020, 28889, 256, 6, 40, "Notes",
      CellTags->"AnalyzeOrbit"],
    Cell[686279, 28897, 236, 8, 24, "Notes",
      CellTags->"AnalyzeOrbit"],
    Cell[686518, 28907, 138, 4, 24, "Notes",
      CellTags->"AnalyzeOrbit"],
    Cell[686659, 28913, 261, 9, 40, "Notes",
      CellTags->"AnalyzeOrbit"],
    Cell[686923, 28924, 239, 11, 24, "Notes",
      CellTags->"AnalyzeOrbit"],
    Cell[687165, 28937, 70, 1, 39, "ExampleSection",
      CellTags->"AnalyzeOrbit"],
    Cell[687238, 28940, 84, 1, 34, "ExampleText",
      CellTags->"AnalyzeOrbit"],
    Cell[687325, 28943, 110, 5, 45, "Input",
      CellTags->"AnalyzeOrbit"],
    Cell[687438, 28950, 104, 3, 26, "Input",
      CellTags->"AnalyzeOrbit"],
    Cell[687545, 28955, 104, 3, 26, "Input",
      CellTags->"AnalyzeOrbit"],
    Cell[687652, 28960, 124, 3, 26, "Input",
      CellTags->"AnalyzeOrbit"],
    Cell[687779, 28965, 125, 3, 26, "Input",
      CellTags->"AnalyzeOrbit"],
    Cell[687907, 28970, 159, 4, 50, "ExampleText",
      CellTags->"AnalyzeOrbit"],
    Cell[688069, 28976, 207, 5, 58, "Input",
      CellTags->"AnalyzeOrbit"],
    Cell[688279, 28983, 169, 4, 26, "Input",
      CellTags->"AnalyzeOrbit"],
    Cell[688451, 28989, 215, 5, 50, "ExampleText",
      CellTags->"AnalyzeOrbit"],
    Cell[688669, 28996, 110, 3, 26, "Input",
      CellTags->"AnalyzeOrbit"],
    Cell[688782, 29001, 149, 5, 34, "ExampleText",
      CellTags->"AnalyzeOrbit"],
    Cell[688934, 29008, 241, 6, 74, "Input",
      CellTags->"AnalyzeOrbit"]},
  "BoundAO"->{
    Cell[689178, 29016, 52, 1, 32, "ObjectName",
      CellTags->"BoundAO"],
    Cell[689233, 29019, 50, 1, 50, "Highlight",
      CellTags->"BoundAO"],
    Cell[689286, 29022, 184, 6, 31, "Usage",
      CellTags->"BoundAO"],
    Cell[689473, 29030, 62, 1, 24, "Notes",
      CellTags->"BoundAO"],
    Cell[689538, 29033, 159, 7, 24, "Notes",
      CellTags->"BoundAO"]},
  "CartesianProduct"->{
    Cell[689700, 29042, 70, 1, 32, "ObjectName",
      CellTags->"CartesianProduct"],
    Cell[689773, 29045, 61, 1, 50, "Highlight",
      CellTags->"CartesianProduct"],
    Cell[689837, 29048, 214, 8, 31, "Usage",
      CellTags->"CartesianProduct"],
    Cell[690054, 29058, 116, 4, 24, "Notes",
      CellTags->"CartesianProduct"],
    Cell[690173, 29064, 233, 8, 26, "Notes",
      CellTags->"CartesianProduct"],
    Cell[690409, 29074, 162, 2, 24, "Notes",
      CellTags->"CartesianProduct"],
    Cell[690574, 29078, 357, 13, 24, "Notes",
      CellTags->"CartesianProduct"],
    Cell[690934, 29093, 74, 1, 39, "ExampleSection",
      CellTags->"CartesianProduct"],
    Cell[691011, 29096, 83, 1, 34, "ExampleText",
      CellTags->"CartesianProduct"],
    Cell[691097, 29099, 129, 5, 45, "Input",
      CellTags->"CartesianProduct"],
    Cell[691229, 29106, 111, 1, 34, "ExampleText",
      CellTags->"CartesianProduct"],
    Cell[691343, 29109, 112, 2, 28, "Input",
      CellTags->"CartesianProduct"],
    Cell[691458, 29113, 186, 5, 50, "ExampleText",
      CellTags->"CartesianProduct"],
    Cell[691647, 29120, 136, 5, 45, "Input",
      CellTags->"CartesianProduct"]},
  "$closureBound"->{
    Cell[691786, 29127, 64, 1, 32, "ObjectName",
      CellTags->"$closureBound"],
    Cell[691853, 29130, 61, 1, 50, "Highlight",
      CellTags->"$closureBound"],
    Cell[691917, 29133, 181, 5, 31, "Usage",
      CellTags->"$closureBound"],
    Cell[692101, 29140, 105, 1, 24, "Notes",
      CellTags->"$closureBound"],
    Cell[692209, 29143, 155, 7, 24, "Notes",
      CellTags->"$closureBound"],
    Cell[692367, 29152, 71, 1, 39, "ExampleSection",
      CellTags->"$closureBound"],
    Cell[692441, 29155, 84, 1, 34, "ExampleText",
      CellTags->"$closureBound"],
    Cell[692528, 29158, 100, 3, 26, "Input",
      CellTags->"$closureBound"]},
  "ClosureBound"->{
    Cell[692631, 29163, 62, 1, 32, "ObjectName",
      CellTags->"ClosureBound"],
    Cell[692696, 29166, 55, 1, 50, "Highlight",
      CellTags->"ClosureBound"],
    Cell[692754, 29169, 196, 8, 31, "Usage",
      CellTags->"ClosureBound"],
    Cell[692953, 29179, 259, 11, 24, "Notes",
      CellTags->"ClosureBound"]},
  "$closureSearchBound"->{
    Cell[693215, 29192, 76, 1, 32, "ObjectName",
      CellTags->"$closureSearchBound"],
    Cell[693294, 29195, 67, 1, 50, "Highlight",
      CellTags->"$closureSearchBound"],
    Cell[693364, 29198, 178, 6, 31, "Usage",
      CellTags->"$closureSearchBound"],
    Cell[693545, 29206, 126, 4, 24, "Notes",
      CellTags->"$closureSearchBound"],
    Cell[693674, 29212, 173, 7, 24, "Notes",
      CellTags->"$closureSearchBound"],
    Cell[693850, 29221, 77, 1, 39, "ExampleSection",
      CellTags->"$closureSearchBound"],
    Cell[693930, 29224, 90, 1, 34, "ExampleText",
      CellTags->"$closureSearchBound"],
    Cell[694023, 29227, 112, 3, 26, "Input",
      CellTags->"$closureSearchBound"]},
  "ClosureSearch"->{
    Cell[694138, 29232, 64, 1, 32, "ObjectName",
      CellTags->"ClosureSearch"],
    Cell[694205, 29235, 58, 1, 50, "Highlight",
      CellTags->"ClosureSearch"],
    Cell[694266, 29238, 272, 10, 47, "Usage",
      CellTags->"ClosureSearch"],
    Cell[694541, 29250, 309, 10, 61, "Notes",
      CellTags->"ClosureSearch"],
    Cell[694853, 29262, 135, 5, 24, "Notes",
      CellTags->"ClosureSearch"],
    Cell[694991, 29269, 328, 10, 40, "Notes",
      CellTags->"ClosureSearch"],
    Cell[695322, 29281, 182, 4, 40, "Notes",
      CellTags->"ClosureSearch"],
    Cell[695507, 29287, 406, 15, 24, "Notes",
      CellTags->"ClosureSearch"],
    Cell[695916, 29304, 71, 1, 39, "ExampleSection",
      CellTags->"ClosureSearch"],
    Cell[695990, 29307, 107, 1, 34, "ExampleText",
      CellTags->"ClosureSearch"],
    Cell[696100, 29310, 179, 4, 42, "Input",
      CellTags->"ClosureSearch"],
    Cell[696282, 29316, 135, 3, 26, "Input",
      CellTags->"ClosureSearch"],
    Cell[696420, 29321, 85, 1, 34, "ExampleText",
      CellTags->"ClosureSearch"],
    Cell[696508, 29324, 187, 4, 42, "Input",
      CellTags->"ClosureSearch"],
    Cell[696698, 29330, 188, 4, 42, "Input",
      CellTags->"ClosureSearch"]},
  "Closure"->{
    Cell[696889, 29336, 52, 1, 32, "ObjectName",
      CellTags->"Closure"],
    Cell[696944, 29339, 52, 1, 50, "Highlight",
      CellTags->"Closure"],
    Cell[696999, 29342, 147, 4, 31, "Usage",
      CellTags->"Closure"],
    Cell[697149, 29348, 475, 14, 133, "Notes",
      CellTags->"Closure"],
    Cell[697627, 29364, 129, 5, 24, "Notes",
      CellTags->"Closure"],
    Cell[697759, 29371, 322, 10, 40, "Notes",
      CellTags->"Closure"],
    Cell[698084, 29383, 258, 9, 40, "Notes",
      CellTags->"Closure"],
    Cell[698345, 29394, 185, 6, 24, "Notes",
      CellTags->"Closure"],
    Cell[698533, 29402, 174, 5, 24, "Notes",
      CellTags->"Closure"],
    Cell[698710, 29409, 132, 5, 24, "Notes",
      CellTags->"Closure"],
    Cell[698845, 29416, 412, 15, 24, "Notes",
      CellTags->"Closure"],
    Cell[699260, 29433, 65, 1, 39, "ExampleSection",
      CellTags->"Closure"],
    Cell[699328, 29436, 83, 1, 34, "ExampleText",
      CellTags->"Closure"],
    Cell[699414, 29439, 101, 2, 28, "Input",
      CellTags->"Closure"],
    Cell[699518, 29443, 100, 2, 28, "Input",
      CellTags->"Closure"],
    Cell[699621, 29447, 101, 1, 34, "ExampleText",
      CellTags->"Closure"],
    Cell[699725, 29450, 174, 4, 42, "Input",
      CellTags->"Closure"],
    Cell[699902, 29456, 110, 2, 28, "Input",
      CellTags->"Closure"],
    Cell[700015, 29460, 133, 4, 34, "ExampleText",
      CellTags->"Closure"],
    Cell[700151, 29466, 199, 6, 42, "Input",
      CellTags->"Closure"],
    Cell[700353, 29474, 251, 5, 66, "ExampleText",
      CellTags->"Closure"],
    Cell[700607, 29481, 155, 4, 26, "Input",
      CellTags->"Closure"],
    Cell[700765, 29487, 312, 6, 66, "ExampleText",
      CellTags->"Closure"],
    Cell[701080, 29495, 198, 5, 42, "Input",
      CellTags->"Closure"],
    Cell[701281, 29502, 83, 3, 26, "Input",
      CellTags->"Closure"],
    Cell[701367, 29507, 85, 3, 26, "Input",
      CellTags->"Closure"]},
  "Cycle"->{
    Cell[701455, 29512, 48, 1, 32, "ObjectName",
      CellTags->"Cycle"],
    Cell[701506, 29515, 48, 1, 50, "Highlight",
      CellTags->"Cycle"],
    Cell[701557, 29518, 185, 6, 31, "Usage",
      CellTags->"Cycle"],
    Cell[701745, 29526, 157, 7, 24, "Notes",
      CellTags->"Cycle"]},
  "DeleteDuplicates"->{
    Cell[701905, 29535, 70, 1, 32, "ObjectName",
      CellTags->"DeleteDuplicates"],
    Cell[701978, 29538, 61, 1, 50, "Highlight",
      CellTags->"DeleteDuplicates"],
    Cell[702042, 29541, 190, 7, 31, "Usage",
      CellTags->"DeleteDuplicates"],
    Cell[702235, 29550, 236, 9, 44, "Notes",
      CellTags->"DeleteDuplicates"],
    Cell[702474, 29561, 186, 5, 24, "Notes",
      CellTags->"DeleteDuplicates"],
    Cell[702663, 29568, 247, 8, 40, "Notes",
      CellTags->"DeleteDuplicates"],
    Cell[702913, 29578, 323, 9, 40, "Notes",
      CellTags->"DeleteDuplicates"],
    Cell[703239, 29589, 136, 6, 24, "Notes",
      CellTags->"DeleteDuplicates"],
    Cell[703378, 29597, 74, 1, 39, "ExampleSection",
      CellTags->"DeleteDuplicates"],
    Cell[703455, 29600, 100, 1, 34, "ExampleText",
      CellTags->"DeleteDuplicates"],
    Cell[703558, 29603, 128, 5, 45, "Input",
      CellTags->"DeleteDuplicates"],
    Cell[703689, 29610, 81, 1, 34, "ExampleText",
      CellTags->"DeleteDuplicates"],
    Cell[703773, 29613, 128, 3, 26, "Input",
      CellTags->"DeleteDuplicates"],
    Cell[703904, 29618, 160, 4, 26, "Input",
      CellTags->"DeleteDuplicates"]},
  "DisjointQ"->{
    Cell[704067, 29624, 56, 1, 32, "ObjectName",
      CellTags->"DisjointQ"],
    Cell[704126, 29627, 54, 1, 50, "Highlight",
      CellTags->"DisjointQ"],
    Cell[704183, 29630, 197, 8, 31, "Usage",
      CellTags->"DisjointQ"],
    Cell[704383, 29640, 156, 7, 24, "Notes",
      CellTags->"DisjointQ"],
    Cell[704542, 29649, 67, 1, 39, "ExampleSection",
      CellTags->"DisjointQ"],
    Cell[704612, 29652, 86, 1, 34, "ExampleText",
      CellTags->"DisjointQ"],
    Cell[704701, 29655, 96, 2, 28, "Input",
      CellTags->"DisjointQ"],
    Cell[704800, 29659, 94, 2, 28, "Input",
      CellTags->"DisjointQ"]},
  "EnumForm"->{
    Cell[704897, 29663, 54, 1, 32, "ObjectName",
      CellTags->"EnumForm"],
    Cell[704954, 29666, 53, 1, 50, "Highlight",
      CellTags->"EnumForm"],
    Cell[705010, 29669, 210, 8, 31, "Usage",
      CellTags->"EnumForm"],
    Cell[705223, 29679, 221, 9, 44, "Notes",
      CellTags->"EnumForm"],
    Cell[705447, 29690, 162, 7, 24, "Notes",
      CellTags->"EnumForm"],
    Cell[705612, 29699, 130, 5, 24, "Notes",
      CellTags->"EnumForm"],
    Cell[705745, 29706, 179, 8, 24, "Notes",
      CellTags->"EnumForm"],
    Cell[705927, 29716, 66, 1, 39, "ExampleSection",
      CellTags->"EnumForm"],
    Cell[705996, 29719, 95, 1, 34, "ExampleText",
      CellTags->"EnumForm"],
    Cell[706094, 29722, 174, 4, 42, "Input",
      CellTags->"EnumForm"],
    Cell[706271, 29728, 94, 3, 26, "Input",
      CellTags->"EnumForm"],
    Cell[706368, 29733, 106, 2, 34, "ExampleText",
      CellTags->"EnumForm"],
    Cell[706477, 29737, 95, 3, 26, "Input",
      CellTags->"EnumForm"]},
  "External"->{
    Cell[706575, 29742, 54, 1, 32, "ObjectName",
      CellTags->"External"],
    Cell[706632, 29745, 51, 1, 50, "Highlight",
      CellTags->"External"],
    Cell[706686, 29748, 156, 4, 31, "Usage",
      CellTags->"External"],
    Cell[706845, 29754, 255, 8, 40, "Notes",
      CellTags->"External"],
    Cell[707103, 29764, 120, 5, 24, "Notes",
      CellTags->"External"],
    Cell[707226, 29771, 124, 4, 24, "Notes",
      CellTags->"External"],
    Cell[707353, 29777, 441, 19, 24, "Notes",
      CellTags->"External"]},
  "FastPowerSet"->{
    Cell[707797, 29798, 62, 1, 32, "ObjectName",
      CellTags->"FastPowerSet"],
    Cell[707862, 29801, 57, 1, 50, "Highlight",
      CellTags->"FastPowerSet"],
    Cell[707922, 29804, 147, 4, 31, "Usage",
      CellTags->"FastPowerSet"],
    Cell[708072, 29810, 152, 5, 24, "Notes",
      CellTags->"FastPowerSet"],
    Cell[708227, 29817, 156, 7, 24, "Notes",
      CellTags->"FastPowerSet"],
    Cell[708386, 29826, 70, 1, 39, "ExampleSection",
      CellTags->"FastPowerSet"],
    Cell[708459, 29829, 122, 4, 34, "ExampleText",
      CellTags->"FastPowerSet"],
    Cell[708584, 29835, 128, 3, 26, "Input",
      CellTags->"FastPowerSet"],
    Cell[708715, 29840, 124, 3, 26, "Input",
      CellTags->"FastPowerSet"],
    Cell[708842, 29845, 130, 4, 42, "Input",
      CellTags->"FastPowerSet"]},
  "Filter"->{
    Cell[708975, 29851, 50, 1, 32, "ObjectName",
      CellTags->"Filter"],
    Cell[709028, 29854, 49, 1, 50, "Highlight",
      CellTags->"Filter"],
    Cell[709080, 29857, 201, 5, 47, "Usage",
      CellTags->"Filter"],
    Cell[709284, 29864, 76, 1, 24, "Notes",
      CellTags->"Filter"],
    Cell[709363, 29867, 116, 4, 24, "Notes",
      CellTags->"Filter"],
    Cell[709482, 29873, 159, 7, 24, "Notes",
      CellTags->"Filter"],
    Cell[709644, 29882, 64, 1, 39, "ExampleSection",
      CellTags->"Filter"],
    Cell[709711, 29885, 106, 2, 34, "ExampleText",
      CellTags->"Filter"],
    Cell[709820, 29889, 255, 5, 58, "Input",
      CellTags->"Filter"],
    Cell[710078, 29896, 120, 3, 26, "Input",
      CellTags->"Filter"],
    Cell[710201, 29901, 231, 6, 42, "Input",
      CellTags->"Filter"],
    Cell[710435, 29909, 100, 3, 26, "Input",
      CellTags->"Filter"]},
  "Frequencies"->{
    Cell[710538, 29914, 60, 1, 32, "ObjectName",
      CellTags->"Frequencies"],
    Cell[710601, 29917, 56, 1, 50, "Highlight",
      CellTags->"Frequencies"],
    Cell[710660, 29920, 179, 6, 31, "Usage",
      CellTags->"Frequencies"],
    Cell[710842, 29928, 165, 6, 24, "Notes",
      CellTags->"Frequencies"],
    Cell[711010, 29936, 69, 1, 39, "ExampleSection",
      CellTags->"Frequencies"],
    Cell[711082, 29939, 94, 1, 34, "ExampleText",
      CellTags->"Frequencies"],
    Cell[711179, 29942, 199, 5, 58, "Input",
      CellTags->"Frequencies"]},
  "FromBitVector"->{
    Cell[711381, 29949, 64, 1, 32, "ObjectName",
      CellTags->"FromBitVector"],
    Cell[711448, 29952, 58, 1, 50, "Highlight",
      CellTags->"FromBitVector"],
    Cell[711509, 29955, 212, 8, 31, "Usage",
      CellTags->"FromBitVector"],
    Cell[711724, 29965, 97, 1, 24, "Notes",
      CellTags->"FromBitVector"],
    Cell[711824, 29968, 168, 7, 24, "Notes",
      CellTags->"FromBitVector"],
    Cell[711995, 29977, 71, 1, 39, "ExampleSection",
      CellTags->"FromBitVector"],
    Cell[712069, 29980, 100, 1, 34, "ExampleText",
      CellTags->"FromBitVector"],
    Cell[712172, 29983, 93, 2, 28, "Input",
      CellTags->"FromBitVector"],
    Cell[712268, 29987, 132, 3, 26, "Input",
      CellTags->"FromBitVector"]},
  "Index"->{
    Cell[712403, 29992, 48, 1, 32, "ObjectName",
      CellTags->"Index"],
    Cell[712454, 29995, 48, 1, 50, "Highlight",
      CellTags->"Index"],
    Cell[712505, 29998, 197, 8, 31, "Usage",
      CellTags->"Index"],
    Cell[712705, 30008, 235, 11, 24, "Notes",
      CellTags->"Index"]},
  "Kary"->{
    Cell[712943, 30021, 46, 1, 32, "ObjectName",
      CellTags->"Kary"],
    Cell[712992, 30024, 47, 1, 50, "Highlight",
      CellTags->"Kary"],
    Cell[713042, 30027, 231, 8, 47, "Usage",
      CellTags->"Kary"],
    Cell[713276, 30037, 160, 7, 24, "Notes",
      CellTags->"Kary"]},
  "LongestCommonSubsequence"->{
    Cell[713439, 30046, 86, 1, 32, "ObjectName",
      CellTags->"LongestCommonSubsequence"],
    Cell[713528, 30049, 69, 1, 50, "Highlight",
      CellTags->"LongestCommonSubsequence"],
    Cell[713600, 30052, 245, 8, 47, "Usage",
      CellTags->"LongestCommonSubsequence"],
    Cell[713848, 30062, 239, 9, 44, "Notes",
      CellTags->"LongestCommonSubsequence"],
    Cell[714090, 30073, 169, 5, 24, "Notes",
      CellTags->"LongestCommonSubsequence"],
    Cell[714262, 30080, 114, 1, 24, "Notes",
      CellTags->"LongestCommonSubsequence"],
    Cell[714379, 30083, 181, 7, 24, "Notes",
      CellTags->"LongestCommonSubsequence"],
    Cell[714563, 30092, 82, 1, 39, "ExampleSection",
      CellTags->"LongestCommonSubsequence"],
    Cell[714648, 30095, 92, 1, 34, "ExampleText",
      CellTags->"LongestCommonSubsequence"],
    Cell[714743, 30098, 294, 6, 58, "Input",
      CellTags->"LongestCommonSubsequence"],
    Cell[715040, 30106, 149, 4, 34, "ExampleText",
      CellTags->"LongestCommonSubsequence"],
    Cell[715192, 30112, 189, 4, 42, "Input",
      CellTags->"LongestCommonSubsequence"]},
  "MakeIndex"->{
    Cell[715384, 30118, 56, 1, 32, "ObjectName",
      CellTags->"MakeIndex"],
    Cell[715443, 30121, 54, 1, 50, "Highlight",
      CellTags->"MakeIndex"],
    Cell[715500, 30124, 220, 8, 31, "Usage",
      CellTags->"MakeIndex"],
    Cell[715723, 30134, 129, 6, 24, "Notes",
      CellTags->"MakeIndex"],
    Cell[715855, 30142, 116, 2, 24, "Notes",
      CellTags->"MakeIndex"],
    Cell[715974, 30146, 401, 12, 40, "Notes",
      CellTags->"MakeIndex"],
    Cell[716378, 30160, 160, 7, 24, "Notes",
      CellTags->"MakeIndex"],
    Cell[716541, 30169, 67, 1, 39, "ExampleSection",
      CellTags->"MakeIndex"],
    Cell[716611, 30172, 88, 1, 34, "ExampleText",
      CellTags->"MakeIndex"],
    Cell[716702, 30175, 135, 3, 26, "Input",
      CellTags->"MakeIndex"]},
  "Matrix"->{
    Cell[716840, 30180, 50, 1, 32, "ObjectName",
      CellTags->"Matrix"],
    Cell[716893, 30183, 49, 1, 50, "Highlight",
      CellTags->"Matrix"],
    Cell[716945, 30186, 199, 8, 31, "Usage",
      CellTags->"Matrix"],
    Cell[717147, 30196, 157, 7, 24, "Notes",
      CellTags->"Matrix"]},
  "NonDecreasingSequences"->{
    Cell[717307, 30205, 82, 1, 32, "ObjectName",
      CellTags->"NonDecreasingSequences"],
    Cell[717392, 30208, 67, 1, 50, "Highlight",
      CellTags->"NonDecreasingSequences"],
    Cell[717462, 30211, 238, 8, 31, "Usage",
      CellTags->"NonDecreasingSequences"],
    Cell[717703, 30221, 218, 6, 24, "Notes",
      CellTags->"NonDecreasingSequences"],
    Cell[717924, 30229, 183, 5, 24, "Notes",
      CellTags->"NonDecreasingSequences"],
    Cell[718110, 30236, 270, 12, 24, "Notes",
      CellTags->"NonDecreasingSequences"],
    Cell[718383, 30250, 80, 1, 39, "ExampleSection",
      CellTags->"NonDecreasingSequences"],
    Cell[718466, 30253, 200, 6, 34, "ExampleText",
      CellTags->"NonDecreasingSequences"],
    Cell[718669, 30261, 107, 2, 28, "Input",
      CellTags->"NonDecreasingSequences"],
    Cell[718779, 30265, 85, 1, 34, "ExampleText",
      CellTags->"NonDecreasingSequences"],
    Cell[718867, 30268, 150, 5, 45, "Input",
      CellTags->"NonDecreasingSequences"],
    Cell[719020, 30275, 145, 4, 34, "ExampleText",
      CellTags->"NonDecreasingSequences"],
    Cell[719168, 30281, 121, 3, 26, "Input",
      CellTags->"NonDecreasingSequences"],
    Cell[719292, 30286, 117, 3, 26, "Input",
      CellTags->"NonDecreasingSequences"]},
  "NoRest"->{
    Cell[719412, 30291, 50, 1, 32, "ObjectName",
      CellTags->"NoRest"],
    Cell[719465, 30294, 49, 1, 50, "Highlight",
      CellTags->"NoRest"],
    Cell[719517, 30297, 141, 6, 31, "Usage",
      CellTags->"NoRest"],
    Cell[719661, 30305, 154, 7, 24, "Notes",
      CellTags->"NoRest"]},
  "OutPairs"->{
    Cell[719818, 30314, 54, 1, 32, "ObjectName",
      CellTags->"OutPairs"],
    Cell[719875, 30317, 51, 1, 50, "Highlight",
      CellTags->"OutPairs"],
    Cell[719929, 30320, 160, 6, 31, "Usage",
      CellTags->"OutPairs"],
    Cell[720092, 30328, 93, 1, 24, "Notes",
      CellTags->"OutPairs"],
    Cell[720188, 30331, 168, 7, 24, "Notes",
      CellTags->"OutPairs"]},
  "Pairs"->{
    Cell[720359, 30340, 48, 1, 32, "ObjectName",
      CellTags->"Pairs"],
    Cell[720410, 30343, 50, 1, 50, "Highlight",
      CellTags->"Pairs"],
    Cell[720463, 30346, 209, 8, 31, "Usage",
      CellTags->"Pairs"],
    Cell[720675, 30356, 94, 4, 24, "Notes",
      CellTags->"Pairs"],
    Cell[720772, 30362, 151, 5, 24, "Notes",
      CellTags->"Pairs"],
    Cell[720926, 30369, 93, 1, 24, "Notes",
      CellTags->"Pairs"],
    Cell[721022, 30372, 207, 7, 24, "Notes",
      CellTags->"Pairs"],
    Cell[721232, 30381, 63, 1, 39, "ExampleSection",
      CellTags->"Pairs"],
    Cell[721298, 30384, 62, 1, 34, "ExampleText",
      CellTags->"Pairs"],
    Cell[721363, 30387, 86, 2, 28, "Input",
      CellTags->"Pairs"],
    Cell[721452, 30391, 94, 2, 28, "Input",
      CellTags->"Pairs"],
    Cell[721549, 30395, 76, 2, 28, "Input",
      CellTags->"Pairs"],
    Cell[721628, 30399, 77, 2, 28, "Input",
      CellTags->"Pairs"]},
  "Partitions"->{
    Cell[721708, 30403, 58, 1, 32, "ObjectName",
      CellTags->"Partitions"],
    Cell[721769, 30406, 55, 1, 50, "Highlight",
      CellTags->"Partitions"],
    Cell[721827, 30409, 153, 6, 31, "Usage",
      CellTags->"Partitions"],
    Cell[721983, 30417, 218, 8, 24, "Notes",
      CellTags->"Partitions"],
    Cell[722204, 30427, 261, 11, 24, "Notes",
      CellTags->"Partitions"],
    Cell[722468, 30440, 68, 1, 39, "ExampleSection",
      CellTags->"Partitions"],
    Cell[722539, 30443, 144, 6, 34, "ExampleText",
      CellTags->"Partitions"],
    Cell[722686, 30451, 82, 2, 28, "Input",
      CellTags->"Partitions"]},
  "Periodic"->{
    Cell[722771, 30455, 54, 1, 32, "ObjectName",
      CellTags->"Periodic"],
    Cell[722828, 30458, 51, 1, 50, "Highlight",
      CellTags->"Periodic"],
    Cell[722882, 30461, 215, 6, 47, "Usage",
      CellTags->"Periodic"],
    Cell[723100, 30469, 112, 5, 24, "Notes",
      CellTags->"Periodic"],
    Cell[723215, 30476, 160, 7, 24, "Notes",
      CellTags->"Periodic"]},
  "PowerSet"->{
    Cell[723378, 30485, 54, 1, 32, "ObjectName",
      CellTags->"PowerSet"],
    Cell[723435, 30488, 53, 1, 50, "Highlight",
      CellTags->"PowerSet"],
    Cell[723491, 30491, 186, 8, 31, "Usage",
      CellTags->"PowerSet"],
    Cell[723680, 30501, 136, 5, 24, "Notes",
      CellTags->"PowerSet"],
    Cell[723819, 30508, 136, 6, 24, "Notes",
      CellTags->"PowerSet"],
    Cell[723958, 30516, 201, 7, 24, "Notes",
      CellTags->"PowerSet"],
    Cell[724162, 30525, 66, 1, 39, "ExampleSection",
      CellTags->"PowerSet"],
    Cell[724231, 30528, 146, 6, 34, "ExampleText",
      CellTags->"PowerSet"],
    Cell[724380, 30536, 86, 2, 28, "Input",
      CellTags->"PowerSet"],
    Cell[724469, 30540, 88, 2, 28, "Input",
      CellTags->"PowerSet"]},
  "RandomPermutation"->{
    Cell[724560, 30544, 72, 1, 32, "ObjectName",
      CellTags->"RandomPermutation"],
    Cell[724635, 30547, 62, 1, 50, "Highlight",
      CellTags->"RandomPermutation"],
    Cell[724700, 30550, 180, 6, 31, "Usage",
      CellTags->"RandomPermutation"],
    Cell[724883, 30558, 122, 4, 24, "Notes",
      CellTags->"RandomPermutation"],
    Cell[725008, 30564, 162, 7, 24, "Notes",
      CellTags->"RandomPermutation"],
    Cell[725173, 30573, 75, 1, 39, "ExampleSection",
      CellTags->"RandomPermutation"],
    Cell[725251, 30576, 159, 6, 34, "ExampleText",
      CellTags->"RandomPermutation"],
    Cell[725413, 30584, 96, 2, 28, "Input",
      CellTags->"RandomPermutation"]},
  "Rank"->{
    Cell[725512, 30588, 46, 1, 32, "ObjectName",
      CellTags->"Rank"],
    Cell[725561, 30591, 47, 1, 50, "Highlight",
      CellTags->"Rank"],
    Cell[725611, 30594, 134, 6, 31, "Usage",
      CellTags->"Rank"],
    Cell[725748, 30602, 151, 7, 24, "Notes",
      CellTags->"Rank"],
    Cell[725902, 30611, 62, 1, 39, "ExampleSection",
      CellTags->"Rank"]},
  "RankingRules"->{
    Cell[725967, 30614, 62, 1, 32, "ObjectName",
      CellTags->"RankingRules"],
    Cell[726032, 30617, 57, 1, 50, "Highlight",
      CellTags->"RankingRules"],
    Cell[726092, 30620, 204, 6, 31, "Usage",
      CellTags->"RankingRules"],
    Cell[726299, 30628, 277, 10, 61, "Notes",
      CellTags->"RankingRules"],
    Cell[726579, 30640, 186, 7, 24, "Notes",
      CellTags->"RankingRules"],
    Cell[726768, 30649, 202, 6, 40, "Notes",
      CellTags->"RankingRules"],
    Cell[726973, 30657, 220, 6, 40, "Notes",
      CellTags->"RankingRules"],
    Cell[727196, 30665, 229, 6, 40, "Notes",
      CellTags->"RankingRules"],
    Cell[727428, 30673, 70, 1, 39, "ExampleSection",
      CellTags->"RankingRules"],
    Cell[727501, 30676, 157, 4, 50, "ExampleText",
      CellTags->"RankingRules"],
    Cell[727661, 30682, 97, 2, 28, "Input",
      CellTags->"RankingRules"],
    Cell[727761, 30686, 83, 1, 34, "ExampleText",
      CellTags->"RankingRules"],
    Cell[727847, 30689, 218, 5, 58, "Input",
      CellTags->"RankingRules"],
    Cell[728068, 30696, 84, 1, 34, "ExampleText",
      CellTags->"RankingRules"],
    Cell[728155, 30699, 139, 3, 26, "Input",
      CellTags->"RankingRules"],
    Cell[728297, 30704, 95, 1, 34, "ExampleText",
      CellTags->"RankingRules"],
    Cell[728395, 30707, 197, 4, 42, "Input",
      CellTags->"RankingRules"]},
  "Reflexive"->{
    Cell[728595, 30713, 56, 1, 32, "ObjectName",
      CellTags->"Reflexive"],
    Cell[728654, 30716, 52, 1, 50, "Highlight",
      CellTags->"Reflexive"],
    Cell[728709, 30719, 156, 5, 31, "Usage",
      CellTags->"Reflexive"],
    Cell[728868, 30726, 110, 5, 24, "Notes",
      CellTags->"Reflexive"],
    Cell[728981, 30733, 176, 7, 24, "Notes",
      CellTags->"Reflexive"]},
  "Relation"->{
    Cell[729160, 30742, 54, 1, 32, "ObjectName",
      CellTags->"Relation"],
    Cell[729217, 30745, 51, 1, 50, "Highlight",
      CellTags->"Relation"],
    Cell[729271, 30748, 203, 8, 31, "Usage",
      CellTags->"Relation"],
    Cell[729477, 30758, 154, 7, 24, "Notes",
      CellTags->"Relation"]},
  "RootList"->{
    Cell[729634, 30767, 54, 1, 32, "ObjectName",
      CellTags->"RootList"],
    Cell[729691, 30770, 53, 1, 50, "Highlight",
      CellTags->"RootList"],
    Cell[729747, 30773, 304, 12, 47, "Usage",
      CellTags->"RootList"],
    Cell[730054, 30787, 131, 5, 24, "Notes",
      CellTags->"RootList"],
    Cell[730188, 30794, 281, 11, 24, "Notes",
      CellTags->"RootList"],
    Cell[730472, 30807, 66, 1, 39, "ExampleSection",
      CellTags->"RootList"],
    Cell[730541, 30810, 79, 1, 34, "ExampleText",
      CellTags->"RootList"],
    Cell[730623, 30813, 117, 3, 26, "Input",
      CellTags->"RootList"],
    Cell[730743, 30818, 94, 3, 26, "Input",
      CellTags->"RootList"]},
  "SelectorQ"->{
    Cell[730840, 30823, 56, 1, 32, "ObjectName",
      CellTags->"SelectorQ"],
    Cell[730899, 30826, 54, 1, 50, "Highlight",
      CellTags->"SelectorQ"],
    Cell[730956, 30829, 229, 8, 31, "Usage",
      CellTags->"SelectorQ"],
    Cell[731188, 30839, 231, 9, 24, "Notes",
      CellTags->"SelectorQ"],
    Cell[731422, 30850, 162, 7, 24, "Notes",
      CellTags->"SelectorQ"],
    Cell[731587, 30859, 67, 1, 39, "ExampleSection",
      CellTags->"SelectorQ"],
    Cell[731657, 30862, 174, 6, 34, "ExampleText",
      CellTags->"SelectorQ"],
    Cell[731834, 30870, 138, 3, 26, "Input",
      CellTags->"SelectorQ"],
    Cell[731975, 30875, 116, 3, 26, "Input",
      CellTags->"SelectorQ"],
    Cell[732094, 30880, 115, 3, 26, "Input",
      CellTags->"SelectorQ"]},
  "SetClosure"->{
    Cell[732212, 30885, 58, 1, 32, "ObjectName",
      CellTags->"SetClosure"],
    Cell[732273, 30888, 53, 1, 50, "Highlight",
      CellTags->"SetClosure"],
    Cell[732329, 30891, 245, 8, 47, "Usage",
      CellTags->"SetClosure"],
    Cell[732577, 30901, 254, 11, 24, "Notes",
      CellTags->"SetClosure"]},
  "Shuffle"->{
    Cell[732834, 30914, 52, 1, 32, "ObjectName",
      CellTags->"Shuffle"],
    Cell[732889, 30917, 52, 1, 50, "Highlight",
      CellTags->"Shuffle"],
    Cell[732944, 30920, 175, 8, 31, "Usage",
      CellTags->"Shuffle"],
    Cell[733122, 30930, 240, 11, 24, "Notes",
      CellTags->"Shuffle"],
    Cell[733365, 30943, 270, 11, 24, "Notes",
      CellTags->"Shuffle"],
    Cell[733638, 30956, 65, 1, 39, "ExampleSection",
      CellTags->"Shuffle"],
    Cell[733706, 30959, 105, 2, 34, "ExampleText",
      CellTags->"Shuffle"],
    Cell[733814, 30963, 91, 2, 28, "Input",
      CellTags->"Shuffle"],
    Cell[733908, 30967, 119, 3, 26, "Input",
      CellTags->"Shuffle"]},
  "SubsetQ"->{
    Cell[734030, 30972, 52, 1, 32, "ObjectName",
      CellTags->"SubsetQ"],
    Cell[734085, 30975, 52, 1, 50, "Highlight",
      CellTags->"SubsetQ"],
    Cell[734140, 30978, 185, 8, 31, "Usage",
      CellTags->"SubsetQ"],
    Cell[734328, 30988, 126, 5, 24, "Notes",
      CellTags->"SubsetQ"],
    Cell[734457, 30995, 156, 7, 24, "Notes",
      CellTags->"SubsetQ"],
    Cell[734616, 31004, 65, 1, 39, "ExampleSection",
      CellTags->"SubsetQ"],
    Cell[734684, 31007, 78, 1, 34, "ExampleText",
      CellTags->"SubsetQ"],
    Cell[734765, 31010, 91, 2, 28, "Input",
      CellTags->"SubsetQ"],
    Cell[734859, 31014, 91, 2, 28, "Input",
      CellTags->"SubsetQ"]},
  "ToBitVector"->{
    Cell[734953, 31018, 60, 1, 32, "ObjectName",
      CellTags->"ToBitVector"],
    Cell[735016, 31021, 56, 1, 50, "Highlight",
      CellTags->"ToBitVector"],
    Cell[735075, 31024, 206, 8, 31, "Usage",
      CellTags->"ToBitVector"],
    Cell[735284, 31034, 95, 1, 24, "Notes",
      CellTags->"ToBitVector"],
    Cell[735382, 31037, 170, 7, 24, "Notes",
      CellTags->"ToBitVector"],
    Cell[735555, 31046, 69, 1, 39, "ExampleSection",
      CellTags->"ToBitVector"],
    Cell[735627, 31049, 99, 1, 34, "ExampleText",
      CellTags->"ToBitVector"],
    Cell[735729, 31052, 92, 2, 28, "Input",
      CellTags->"ToBitVector"],
    Cell[735824, 31056, 127, 3, 26, "Input",
      CellTags->"ToBitVector"]},
  "ToList"->{
    Cell[735954, 31061, 50, 1, 32, "ObjectName",
      CellTags->"ToList"],
    Cell[736007, 31064, 51, 1, 50, "Highlight",
      CellTags->"ToList"],
    Cell[736061, 31067, 182, 8, 31, "Usage",
      CellTags->"ToList"]},
  "ToPairs"->{
    Cell[736246, 31077, 52, 1, 32, "ObjectName",
      CellTags->"ToPairs"],
    Cell[736301, 31080, 52, 1, 50, "Highlight",
      CellTags->"ToPairs"],
    Cell[736356, 31083, 178, 6, 31, "Usage",
      CellTags->"ToPairs"],
    Cell[736537, 31091, 223, 9, 44, "Notes",
      CellTags->"ToPairs"],
    Cell[736763, 31102, 145, 4, 24, "Notes",
      CellTags->"ToPairs"],
    Cell[736911, 31108, 217, 8, 24, "Notes",
      CellTags->"ToPairs"],
    Cell[737131, 31118, 421, 15, 40, "Notes",
      CellTags->"ToPairs"],
    Cell[737555, 31135, 331, 15, 24, "Notes",
      CellTags->"ToPairs"],
    Cell[737889, 31152, 65, 1, 39, "ExampleSection",
      CellTags->"ToPairs"],
    Cell[737957, 31155, 96, 1, 34, "ExampleText",
      CellTags->"ToPairs"],
    Cell[738056, 31158, 107, 5, 45, "Input",
      CellTags->"ToPairs"],
    Cell[738166, 31165, 122, 3, 26, "Input",
      CellTags->"ToPairs"],
    Cell[738291, 31170, 178, 5, 50, "ExampleText",
      CellTags->"ToPairs"],
    Cell[738472, 31177, 196, 4, 42, "Input",
      CellTags->"ToPairs"],
    Cell[738671, 31183, 271, 6, 58, "Input",
      CellTags->"ToPairs"]},
  "Tuples"->{
    Cell[738945, 31191, 50, 1, 32, "ObjectName",
      CellTags->"Tuples"],
    Cell[738998, 31194, 51, 1, 50, "Highlight",
      CellTags->"Tuples"],
    Cell[739052, 31197, 221, 8, 31, "Usage",
      CellTags->"Tuples"],
    Cell[739276, 31207, 142, 6, 24, "Notes",
      CellTags->"Tuples"],
    Cell[739421, 31215, 311, 11, 24, "Notes",
      CellTags->"Tuples"],
    Cell[739735, 31228, 177, 7, 24, "Notes",
      CellTags->"Tuples"],
    Cell[739915, 31237, 165, 7, 24, "Notes",
      CellTags->"Tuples"],
    Cell[740083, 31246, 64, 1, 39, "ExampleSection",
      CellTags->"Tuples"],
    Cell[740150, 31249, 92, 1, 34, "ExampleText",
      CellTags->"Tuples"],
    Cell[740245, 31252, 98, 3, 26, "Input",
      CellTags->"Tuples"],
    Cell[740346, 31257, 119, 4, 34, "ExampleText",
      CellTags->"Tuples"],
    Cell[740468, 31263, 103, 3, 26, "Input",
      CellTags->"Tuples"],
    Cell[740574, 31268, 84, 1, 34, "ExampleText",
      CellTags->"Tuples"],
    Cell[740661, 31271, 129, 4, 42, "Input",
      CellTags->"Tuples"]},
  "Type"->{
    Cell[740793, 31277, 46, 1, 32, "ObjectName",
      CellTags->"Type"],
    Cell[740842, 31280, 47, 1, 50, "Highlight",
      CellTags->"Type"],
    Cell[740892, 31283, 137, 6, 31, "Usage",
      CellTags->"Type"],
    Cell[741032, 31291, 230, 11, 24, "Notes",
      CellTags->"Type"],
    Cell[741265, 31304, 150, 7, 24, "Notes",
      CellTags->"Type"]},
  "$extlink"->{
    Cell[741418, 31313, 54, 1, 32, "ObjectName",
      CellTags->"$extlink"],
    Cell[741475, 31316, 56, 1, 50, "Highlight",
      CellTags->"$extlink"],
    Cell[741534, 31319, 143, 5, 31, "Usage",
      CellTags->"$extlink"],
    Cell[741680, 31326, 260, 10, 25, "Notes",
      CellTags->"$extlink"],
    Cell[741943, 31338, 140, 5, 24, "Notes",
      CellTags->"$extlink"],
    Cell[742086, 31345, 201, 9, 24, "Notes",
      CellTags->"$extlink"],
    Cell[742290, 31356, 66, 1, 39, "ExampleSection",
      CellTags->"$extlink"],
    Cell[742359, 31359, 160, 7, 34, "ExampleText",
      CellTags->"$extlink"],
    Cell[742522, 31368, 91, 3, 26, "Input",
      CellTags->"$extlink"]},
  "$defaultexternal"->{
    Cell[742616, 31373, 70, 1, 32, "ObjectName",
      CellTags->"$defaultexternal"],
    Cell[742689, 31376, 64, 1, 50, "Highlight",
      CellTags->"$defaultexternal"],
    Cell[742756, 31379, 180, 5, 31, "Usage",
      CellTags->"$defaultexternal"],
    Cell[742939, 31386, 268, 10, 25, "Notes",
      CellTags->"$defaultexternal"],
    Cell[743210, 31398, 148, 5, 24, "Notes",
      CellTags->"$defaultexternal"],
    Cell[743361, 31405, 397, 18, 24, "Notes",
      CellTags->"$defaultexternal"],
    Cell[743761, 31425, 74, 1, 39, "ExampleSection",
      CellTags->"$defaultexternal"],
    Cell[743838, 31428, 89, 1, 34, "ExampleText",
      CellTags->"$defaultexternal"],
    Cell[743930, 31431, 107, 3, 26, "Input",
      CellTags->"$defaultexternal"]},
  "$LLL"->{
    Cell[744040, 31436, 46, 1, 32, "ObjectName",
      CellTags->"$LLL"],
    Cell[744089, 31439, 52, 1, 50, "Highlight",
      CellTags->"$LLL"],
    Cell[744144, 31442, 246, 8, 47, "Usage",
      CellTags->"$LLL"],
    Cell[744393, 31452, 355, 14, 40, "Notes",
      CellTags->"$LLL"],
    Cell[744751, 31468, 127, 4, 24, "Notes",
      CellTags->"$LLL"],
    Cell[744881, 31474, 127, 6, 24, "Notes",
      CellTags->"$LLL"],
    Cell[745011, 31482, 62, 1, 39, "ExampleSection",
      CellTags->"$LLL"],
    Cell[745076, 31485, 172, 6, 34, "ExampleText",
      CellTags->"$LLL"],
    Cell[745251, 31493, 192, 5, 42, "Input",
      CellTags->"$LLL"],
    Cell[745446, 31500, 91, 1, 34, "ExampleText",
      CellTags->"$LLL"],
    Cell[745540, 31503, 126, 3, 26, "Input",
      CellTags->"$LLL"],
    Cell[745669, 31508, 163, 5, 34, "ExampleText",
      CellTags->"$LLL"],
    Cell[745835, 31515, 133, 3, 26, "Input",
      CellTags->"$LLL"],
    Cell[745971, 31520, 92, 1, 34, "ExampleText",
      CellTags->"$LLL"],
    Cell[746066, 31523, 135, 3, 26, "Input",
      CellTags->"$LLL"],
    Cell[746204, 31528, 136, 4, 34, "ExampleText",
      CellTags->"$LLL"],
    Cell[746343, 31534, 124, 3, 26, "Input",
      CellTags->"$LLL"]},
  "$shortlines"->{
    Cell[746470, 31539, 60, 1, 32, "ObjectName",
      CellTags->"$shortlines"],
    Cell[746533, 31542, 59, 1, 50, "Highlight",
      CellTags->"$shortlines"],
    Cell[746595, 31545, 186, 5, 47, "Usage",
      CellTags->"$shortlines"],
    Cell[746784, 31552, 165, 5, 24, "Notes",
      CellTags->"$shortlines"],
    Cell[746952, 31559, 64, 1, 24, "Notes",
      CellTags->"$shortlines"],
    Cell[747019, 31562, 172, 7, 24, "Notes",
      CellTags->"$shortlines"],
    Cell[747194, 31571, 69, 1, 39, "ExampleSection",
      CellTags->"$shortlines"],
    Cell[747266, 31574, 80, 1, 34, "ExampleText",
      CellTags->"$shortlines"],
    Cell[747349, 31577, 97, 3, 26, "Input",
      CellTags->"$shortlines"],
    Cell[747449, 31582, 84, 1, 34, "ExampleText",
      CellTags->"$shortlines"],
    Cell[747536, 31585, 131, 3, 26, "Input",
      CellTags->"$shortlines"],
    Cell[747670, 31590, 126, 4, 42, "Input",
      CellTags->"$shortlines"],
    Cell[747799, 31596, 80, 2, 26, "Input",
      CellTags->"$shortlines"]},
  "$transdispwidth"->{
    Cell[747882, 31600, 68, 1, 32, "ObjectName",
      CellTags->"$transdispwidth"],
    Cell[747953, 31603, 63, 1, 50, "Highlight",
      CellTags->"$transdispwidth"],
    Cell[748019, 31606, 200, 6, 31, "Usage",
      CellTags->"$transdispwidth"],
    Cell[748222, 31614, 62, 1, 24, "Notes",
      CellTags->"$transdispwidth"],
    Cell[748287, 31617, 157, 7, 24, "Notes",
      CellTags->"$transdispwidth"],
    Cell[748447, 31626, 73, 1, 39, "ExampleSection",
      CellTags->"$transdispwidth"],
    Cell[748523, 31629, 88, 1, 34, "ExampleText",
      CellTags->"$transdispwidth"],
    Cell[748614, 31632, 96, 2, 28, "Input",
      CellTags->"$transdispwidth"],
    Cell[748713, 31636, 130, 3, 26, "Input",
      CellTags->"$transdispwidth"]},
  "All"->{
    Cell[748846, 31641, 44, 1, 32, "ObjectName",
      CellTags->"All"],
    Cell[748893, 31644, 46, 1, 50, "Highlight",
      CellTags->"All"],
    Cell[748942, 31647, 120, 4, 31, "Usage",
      CellTags->"All"],
    Cell[749065, 31653, 430, 15, 24, "Notes",
      CellTags->"All"],
    Cell[749498, 31670, 61, 1, 39, "ExampleSection",
      CellTags->"All"],
    Cell[749562, 31673, 207, 7, 50, "ExampleText",
      CellTags->"All"],
    Cell[749772, 31682, 105, 2, 28, "Input",
      CellTags->"All"],
    Cell[749880, 31686, 90, 1, 34, "ExampleText",
      CellTags->"All"],
    Cell[749973, 31689, 118, 2, 28, "Input",
      CellTags->"All"]},
  "AssignFunction"->{
    Cell[750094, 31693, 66, 1, 32, "ObjectName",
      CellTags->"AssignFunction"],
    Cell[750163, 31696, 59, 1, 50, "Highlight",
      CellTags->"AssignFunction"],
    Cell[750225, 31699, 327, 12, 47, "Usage",
      CellTags->"AssignFunction"],
    Cell[750555, 31713, 375, 13, 40, "Notes",
      CellTags->"AssignFunction"],
    Cell[750933, 31728, 277, 10, 61, "Notes",
      CellTags->"AssignFunction"],
    Cell[751213, 31740, 162, 5, 24, "Notes",
      CellTags->"AssignFunction"],
    Cell[751378, 31747, 226, 8, 24, "Notes",
      CellTags->"AssignFunction"],
    Cell[751607, 31757, 158, 7, 24, "Notes",
      CellTags->"AssignFunction"],
    Cell[751768, 31766, 72, 1, 39, "ExampleSection",
      CellTags->"AssignFunction"],
    Cell[751843, 31769, 93, 1, 34, "ExampleText",
      CellTags->"AssignFunction"],
    Cell[751939, 31772, 123, 5, 45, "Input",
      CellTags->"AssignFunction"],
    Cell[752065, 31779, 153, 4, 34, "ExampleText",
      CellTags->"AssignFunction"],
    Cell[752221, 31785, 193, 5, 42, "Input",
      CellTags->"AssignFunction"],
    Cell[752417, 31792, 254, 8, 50, "ExampleText",
      CellTags->"AssignFunction"],
    Cell[752674, 31802, 231, 5, 58, "Input",
      CellTags->"AssignFunction"],
    Cell[752908, 31809, 93, 3, 26, "Input",
      CellTags->"AssignFunction"]},
  "Curry"->{
    Cell[753004, 31814, 48, 1, 32, "ObjectName",
      CellTags->"Curry"],
    Cell[753055, 31817, 52, 1, 50, "Highlight",
      CellTags->"Curry"],
    Cell[753110, 31820, 152, 6, 31, "Usage",
      CellTags->"Curry"],
    Cell[753265, 31828, 112, 5, 24, "Notes",
      CellTags->"Curry"],
    Cell[753380, 31835, 142, 4, 24, "Notes",
      CellTags->"Curry"],
    Cell[753525, 31841, 63, 1, 39, "ExampleSection",
      CellTags->"Curry"],
    Cell[753591, 31844, 83, 1, 34, "ExampleText",
      CellTags->"Curry"],
    Cell[753677, 31847, 116, 3, 26, "Input",
      CellTags->"Curry"],
    Cell[753796, 31852, 173, 4, 42, "Input",
      CellTags->"Curry"]},
  "Dependency"->{
    Cell[753972, 31858, 58, 1, 32, "ObjectName",
      CellTags->"Dependency"],
    Cell[754033, 31861, 55, 1, 50, "Highlight",
      CellTags->"Dependency"],
    Cell[754091, 31864, 261, 8, 47, "Usage",
      CellTags->"Dependency"],
    Cell[754355, 31874, 367, 14, 24, "Notes",
      CellTags->"Dependency"],
    Cell[754725, 31890, 129, 5, 24, "Notes",
      CellTags->"Dependency"],
    Cell[754857, 31897, 167, 7, 24, "Notes",
      CellTags->"Dependency"],
    Cell[755027, 31906, 68, 1, 39, "ExampleSection",
      CellTags->"Dependency"],
    Cell[755098, 31909, 101, 1, 34, "ExampleText",
      CellTags->"Dependency"],
    Cell[755202, 31912, 222, 6, 42, "Input",
      CellTags->"Dependency"],
    Cell[755427, 31920, 98, 3, 26, "Input",
      CellTags->"Dependency"]},
  "DoTrace"->{
    Cell[755528, 31925, 52, 1, 32, "ObjectName",
      CellTags->"DoTrace"],
    Cell[755583, 31928, 50, 1, 50, "Highlight",
      CellTags->"DoTrace"],
    Cell[755636, 31931, 166, 5, 31, "Usage",
      CellTags->"DoTrace"],
    Cell[755805, 31938, 572, 23, 40, "Notes",
      CellTags->"DoTrace"]},
  "Drei"->{
    Cell[756380, 31963, 46, 1, 32, "ObjectName",
      CellTags->"Drei"],
    Cell[756429, 31966, 54, 1, 50, "Highlight",
      CellTags->"Drei"],
    Cell[756486, 31969, 147, 6, 31, "Usage",
      CellTags->"Drei"],
    Cell[756636, 31977, 103, 5, 24, "Notes",
      CellTags->"Drei"],
    Cell[756742, 31984, 333, 15, 24, "Notes",
      CellTags->"Drei"]},
  "ExtAutomata"->{
    Cell[757078, 32001, 60, 1, 32, "ObjectName",
      CellTags->"ExtAutomata"],
    Cell[757141, 32004, 56, 1, 50, "Highlight",
      CellTags->"ExtAutomata"],
    Cell[757200, 32007, 148, 4, 31, "Usage",
      CellTags->"ExtAutomata"],
    Cell[757351, 32013, 181, 7, 24, "Notes",
      CellTags->"ExtAutomata"],
    Cell[757535, 32022, 183, 7, 24, "Notes",
      CellTags->"ExtAutomata"],
    Cell[757721, 32031, 204, 9, 24, "Notes",
      CellTags->"ExtAutomata"],
    Cell[757928, 32042, 69, 1, 39, "ExampleSection",
      CellTags->"ExtAutomata"],
    Cell[758000, 32045, 166, 4, 50, "ExampleText",
      CellTags->"ExtAutomata"],
    Cell[758169, 32051, 118, 3, 26, "Input",
      CellTags->"ExtAutomata"],
    Cell[758290, 32056, 151, 4, 42, "Input",
      CellTags->"ExtAutomata"],
    Cell[758444, 32062, 156, 4, 42, "Input",
      CellTags->"ExtAutomata"]},
  "FlattenOne"->{
    Cell[758603, 32068, 58, 1, 32, "ObjectName",
      CellTags->"FlattenOne"],
    Cell[758664, 32071, 55, 1, 50, "Highlight",
      CellTags->"FlattenOne"],
    Cell[758722, 32074, 154, 6, 31, "Usage",
      CellTags->"FlattenOne"],
    Cell[758879, 32082, 70, 1, 24, "Notes",
      CellTags->"FlattenOne"],
    Cell[758952, 32085, 153, 7, 24, "Notes",
      CellTags->"FlattenOne"],
    Cell[759108, 32094, 68, 1, 39, "ExampleSection",
      CellTags->"FlattenOne"],
    Cell[759179, 32097, 139, 4, 34, "ExampleText",
      CellTags->"FlattenOne"],
    Cell[759321, 32103, 97, 2, 28, "Input",
      CellTags->"FlattenOne"],
    Cell[759421, 32107, 138, 3, 26, "Input",
      CellTags->"FlattenOne"]},
  "FormatAutomata"->{
    Cell[759562, 32112, 66, 1, 32, "ObjectName",
      CellTags->"FormatAutomata"],
    Cell[759631, 32115, 59, 1, 50, "Highlight",
      CellTags->"FormatAutomata"],
    Cell[759693, 32118, 184, 5, 47, "Usage",
      CellTags->"FormatAutomata"],
    Cell[759880, 32125, 190, 5, 40, "Notes",
      CellTags->"FormatAutomata"],
    Cell[760073, 32132, 167, 5, 24, "Notes",
      CellTags->"FormatAutomata"],
    Cell[760243, 32139, 199, 6, 24, "Notes",
      CellTags->"FormatAutomata"],
    Cell[760445, 32147, 197, 5, 40, "Notes",
      CellTags->"FormatAutomata"],
    Cell[760645, 32154, 169, 6, 24, "Notes",
      CellTags->"FormatAutomata"],
    Cell[760817, 32162, 161, 4, 40, "Notes",
      CellTags->"FormatAutomata"],
    Cell[760981, 32168, 196, 6, 24, "Notes",
      CellTags->"FormatAutomata"],
    Cell[761180, 32176, 168, 7, 24, "Notes",
      CellTags->"FormatAutomata"],
    Cell[761351, 32185, 72, 1, 39, "ExampleSection",
      CellTags->"FormatAutomata"],
    Cell[761426, 32188, 101, 1, 34, "ExampleText",
      CellTags->"FormatAutomata"],
    Cell[761530, 32191, 149, 4, 26, "Input",
      CellTags->"FormatAutomata"],
    Cell[761682, 32197, 128, 4, 42, "Input",
      CellTags->"FormatAutomata"],
    Cell[761813, 32203, 128, 4, 42, "Input",
      CellTags->"FormatAutomata"],
    Cell[761944, 32209, 133, 4, 42, "Input",
      CellTags->"FormatAutomata"],
    Cell[762080, 32215, 132, 4, 42, "Input",
      CellTags->"FormatAutomata"],
    Cell[762215, 32221, 128, 4, 42, "Input",
      CellTags->"FormatAutomata"],
    Cell[762346, 32227, 73, 1, 34, "ExampleText",
      CellTags->"FormatAutomata"],
    Cell[762422, 32230, 113, 3, 26, "Input",
      CellTags->"FormatAutomata"]},
  "Full"->{
    Cell[762538, 32235, 46, 1, 32, "ObjectName",
      CellTags->"Full"],
    Cell[762587, 32238, 47, 1, 50, "Highlight",
      CellTags->"Full"],
    Cell[762637, 32241, 116, 4, 31, "Usage",
      CellTags->"Full"],
    Cell[762756, 32247, 1980, 79, 72, "Notes",
      CellTags->"Full"]},
  "MinusEins"->{
    Cell[764739, 32328, 56, 1, 32, "ObjectName",
      CellTags->"MinusEins"],
    Cell[764798, 32331, 59, 1, 50, "Highlight",
      CellTags->"MinusEins"],
    Cell[764860, 32334, 159, 7, 31, "Usage",
      CellTags->"MinusEins"],
    Cell[765022, 32343, 114, 5, 24, "Notes",
      CellTags->"MinusEins"],
    Cell[765139, 32350, 346, 15, 24, "Notes",
      CellTags->"MinusEins"]},
  "MinusZwei"->{
    Cell[765488, 32367, 56, 1, 32, "ObjectName",
      CellTags->"MinusZwei"],
    Cell[765547, 32370, 59, 1, 50, "Highlight",
      CellTags->"MinusZwei"],
    Cell[765609, 32373, 170, 6, 31, "Usage",
      CellTags->"MinusZwei"],
    Cell[765782, 32381, 114, 5, 24, "Notes",
      CellTags->"MinusZwei"],
    Cell[765899, 32388, 346, 15, 24, "Notes",
      CellTags->"MinusZwei"]},
  "Opts"->{
    Cell[766248, 32405, 46, 1, 32, "ObjectName",
      CellTags->"Opts"],
    Cell[766297, 32408, 47, 1, 50, "Highlight",
      CellTags->"Opts"],
    Cell[766347, 32411, 142, 6, 31, "Usage",
      CellTags->"Opts"],
    Cell[766492, 32419, 85, 1, 24, "Notes",
      CellTags->"Opts"],
    Cell[766580, 32422, 160, 7, 24, "Notes",
      CellTags->"Opts"]},
  "PositionList"->{
    Cell[766743, 32431, 62, 1, 32, "ObjectName",
      CellTags->"PositionList"],
    Cell[766808, 32434, 57, 1, 50, "Highlight",
      CellTags->"PositionList"],
    Cell[766868, 32437, 245, 8, 47, "Usage",
      CellTags->"PositionList"],
    Cell[767116, 32447, 227, 9, 44, "Notes",
      CellTags->"PositionList"],
    Cell[767346, 32458, 215, 6, 40, "Notes",
      CellTags->"PositionList"],
    Cell[767564, 32466, 169, 7, 24, "Notes",
      CellTags->"PositionList"],
    Cell[767736, 32475, 70, 1, 39, "ExampleSection",
      CellTags->"PositionList"],
    Cell[767809, 32478, 132, 4, 34, "ExampleText",
      CellTags->"PositionList"],
    Cell[767944, 32484, 108, 2, 28, "Input",
      CellTags->"PositionList"],
    Cell[768055, 32488, 143, 4, 42, "Input",
      CellTags->"PositionList"],
    Cell[768201, 32494, 114, 3, 26, "Input",
      CellTags->"PositionList"],
    Cell[768318, 32499, 216, 6, 50, "ExampleText",
      CellTags->"PositionList"],
    Cell[768537, 32507, 99, 2, 28, "Input",
      CellTags->"PositionList"],
    Cell[768639, 32511, 141, 3, 26, "Input",
      CellTags->"PositionList"],
    Cell[768783, 32516, 189, 5, 26, "Input",
      CellTags->"PositionList"],
    Cell[768975, 32523, 96, 3, 26, "Input",
      CellTags->"PositionList"]},
  "PositionOne"->{
    Cell[769074, 32528, 60, 1, 32, "ObjectName",
      CellTags->"PositionOne"],
    Cell[769137, 32531, 56, 1, 50, "Highlight",
      CellTags->"PositionOne"],
    Cell[769196, 32534, 162, 6, 31, "Usage",
      CellTags->"PositionOne"],
    Cell[769361, 32542, 60, 1, 24, "Notes",
      CellTags->"PositionOne"],
    Cell[769424, 32545, 69, 1, 39, "ExampleSection",
      CellTags->"PositionOne"],
    Cell[769496, 32548, 132, 4, 34, "ExampleText",
      CellTags->"PositionOne"],
    Cell[769631, 32554, 107, 2, 28, "Input",
      CellTags->"PositionOne"],
    Cell[769741, 32558, 103, 3, 26, "Input",
      CellTags->"PositionOne"]},
  "ReplaceParts"->{
    Cell[769847, 32563, 62, 1, 32, "ObjectName",
      CellTags->"ReplaceParts"],
    Cell[769912, 32566, 57, 1, 50, "Highlight",
      CellTags->"ReplaceParts"],
    Cell[769972, 32569, 154, 4, 31, "Usage",
      CellTags->"ReplaceParts"],
    Cell[770129, 32575, 75, 1, 24, "Notes",
      CellTags->"ReplaceParts"],
    Cell[770207, 32578, 70, 1, 39, "ExampleSection",
      CellTags->"ReplaceParts"],
    Cell[770280, 32581, 73, 1, 34, "ExampleText",
      CellTags->"ReplaceParts"],
    Cell[770356, 32584, 123, 3, 26, "Input",
      CellTags->"ReplaceParts"]},
  "SelectOptions"->{
    Cell[770482, 32589, 64, 1, 32, "ObjectName",
      CellTags->"SelectOptions"],
    Cell[770549, 32592, 58, 1, 50, "Highlight",
      CellTags->"SelectOptions"],
    Cell[770610, 32595, 222, 8, 31, "Usage",
      CellTags->"SelectOptions"],
    Cell[770835, 32605, 193, 6, 24, "Notes",
      CellTags->"SelectOptions"]},
  "Status"->{
    Cell[771031, 32613, 50, 1, 32, "ObjectName",
      CellTags->"Status"],
    Cell[771084, 32616, 51, 1, 50, "Highlight",
      CellTags->"Status"],
    Cell[771138, 32619, 139, 4, 31, "Usage",
      CellTags->"Status"],
    Cell[771280, 32625, 64, 1, 39, "ExampleSection",
      CellTags->"Status"],
    Cell[771347, 32628, 62, 1, 34, "ExampleText",
      CellTags->"Status"],
    Cell[771412, 32631, 73, 2, 28, "Input",
      CellTags->"Status"]},
  "SymbolList"->{
    Cell[771488, 32635, 58, 1, 32, "ObjectName",
      CellTags->"SymbolList"],
    Cell[771549, 32638, 55, 1, 50, "Highlight",
      CellTags->"SymbolList"],
    Cell[771607, 32641, 246, 10, 31, "Usage",
      CellTags->"SymbolList"],
    Cell[771856, 32653, 114, 4, 24, "Notes",
      CellTags->"SymbolList"],
    Cell[771973, 32659, 68, 1, 39, "ExampleSection",
      CellTags->"SymbolList"],
    Cell[772044, 32662, 67, 1, 34, "ExampleText",
      CellTags->"SymbolList"],
    Cell[772114, 32665, 116, 5, 45, "Input",
      CellTags->"SymbolList"]},
  "Zwei"->{
    Cell[772233, 32672, 46, 1, 32, "ObjectName",
      CellTags->"Zwei"],
    Cell[772282, 32675, 54, 1, 50, "Highlight",
      CellTags->"Zwei"],
    Cell[772339, 32678, 145, 6, 31, "Usage",
      CellTags->"Zwei"],
    Cell[772487, 32686, 105, 5, 24, "Notes",
      CellTags->"Zwei"],
    Cell[772595, 32693, 333, 15, 24, "Notes",
      CellTags->"Zwei"]},
  "$AutomataReleaseNumber"->{
    Cell[772931, 32710, 82, 1, 32, "ObjectName",
      CellTags->"$AutomataReleaseNumber"],
    Cell[773016, 32713, 70, 1, 50, "Highlight",
      CellTags->"$AutomataReleaseNumber"],
    Cell[773089, 32716, 172, 4, 31, "Usage",
      CellTags->"$AutomataReleaseNumber"],
    Cell[773264, 32722, 80, 1, 39, "ExampleSection",
      CellTags->"$AutomataReleaseNumber"],
    Cell[773347, 32725, 81, 1, 34, "ExampleText",
      CellTags->"$AutomataReleaseNumber"],
    Cell[773431, 32728, 119, 3, 26, "Input",
      CellTags->"$AutomataReleaseNumber"]},
  "PostLoad"->{
    Cell[773553, 32733, 54, 1, 32, "ObjectName",
      CellTags->"PostLoad"],
    Cell[773610, 32736, 53, 1, 50, "Highlight",
      CellTags->"PostLoad"],
    Cell[773666, 32739, 176, 5, 31, "Usage",
      CellTags->"PostLoad"]},
  "PreLoad"->{
    Cell[773845, 32746, 52, 1, 32, "ObjectName",
      CellTags->"PreLoad"],
    Cell[773900, 32749, 51, 1, 50, "Highlight",
      CellTags->"PreLoad"],
    Cell[773954, 32752, 167, 5, 31, "Usage",
      CellTags->"PreLoad"]},
  "$deffatline"->{
    Cell[774124, 32759, 60, 1, 32, "ObjectName",
      CellTags->"$deffatline"],
    Cell[774187, 32762, 59, 1, 50, "Highlight",
      CellTags->"$deffatline"],
    Cell[774249, 32765, 179, 6, 31, "Usage",
      CellTags->"$deffatline"],
    Cell[774431, 32773, 149, 5, 24, "Notes",
      CellTags->"$deffatline"],
    Cell[774583, 32780, 166, 7, 24, "Notes",
      CellTags->"$deffatline"],
    Cell[774752, 32789, 69, 1, 39, "ExampleSection",
      CellTags->"$deffatline"],
    Cell[774824, 32792, 84, 1, 34, "ExampleText",
      CellTags->"$deffatline"],
    Cell[774911, 32795, 97, 3, 26, "Input",
      CellTags->"$deffatline"]},
  "$deflabeloffset"->{
    Cell[775011, 32800, 68, 1, 32, "ObjectName",
      CellTags->"$deflabeloffset"],
    Cell[775082, 32803, 63, 1, 50, "Highlight",
      CellTags->"$deflabeloffset"],
    Cell[775148, 32806, 179, 6, 31, "Usage",
      CellTags->"$deflabeloffset"],
    Cell[775330, 32814, 69, 1, 24, "Notes",
      CellTags->"$deflabeloffset"],
    Cell[775402, 32817, 266, 11, 24, "Notes",
      CellTags->"$deflabeloffset"],
    Cell[775671, 32830, 73, 1, 39, "ExampleSection",
      CellTags->"$deflabeloffset"],
    Cell[775747, 32833, 86, 1, 34, "ExampleText",
      CellTags->"$deflabeloffset"],
    Cell[775836, 32836, 105, 3, 26, "Input",
      CellTags->"$deflabeloffset"]},
  "$defpointsize"->{
    Cell[775944, 32841, 64, 1, 32, "ObjectName",
      CellTags->"$defpointsize"],
    Cell[776011, 32844, 61, 1, 50, "Highlight",
      CellTags->"$defpointsize"],
    Cell[776075, 32847, 174, 6, 31, "Usage",
      CellTags->"$defpointsize"],
    Cell[776252, 32855, 150, 5, 24, "Notes",
      CellTags->"$defpointsize"],
    Cell[776405, 32862, 264, 11, 24, "Notes",
      CellTags->"$defpointsize"],
    Cell[776672, 32875, 71, 1, 39, "ExampleSection",
      CellTags->"$defpointsize"],
    Cell[776746, 32878, 133, 5, 34, "ExampleText",
      CellTags->"$defpointsize"],
    Cell[776882, 32885, 96, 2, 28, "Input",
      CellTags->"$defpointsize"],
    Cell[776981, 32889, 150, 4, 26, "Input",
      CellTags->"$defpointsize"]},
  "$defthinline"->{
    Cell[777134, 32895, 62, 1, 32, "ObjectName",
      CellTags->"$defthinline"],
    Cell[777199, 32898, 60, 1, 50, "Highlight",
      CellTags->"$defthinline"],
    Cell[777262, 32901, 174, 6, 31, "Usage",
      CellTags->"$defthinline"],
    Cell[777439, 32909, 150, 5, 24, "Notes",
      CellTags->"$defthinline"],
    Cell[777592, 32916, 259, 11, 24, "Notes",
      CellTags->"$defthinline"],
    Cell[777854, 32929, 70, 1, 39, "ExampleSection",
      CellTags->"$defthinline"],
    Cell[777927, 32932, 134, 5, 34, "ExampleText",
      CellTags->"$defthinline"],
    Cell[778064, 32939, 93, 2, 28, "Input",
      CellTags->"$defthinline"],
    Cell[778160, 32943, 129, 5, 45, "Input",
      CellTags->"$defthinline"],
    Cell[778292, 32950, 129, 5, 45, "Input",
      CellTags->"$defthinline"]},
  "$plotrangeoffset"->{
    Cell[778424, 32957, 70, 1, 32, "ObjectName",
      CellTags->"$plotrangeoffset"],
    Cell[778497, 32960, 64, 1, 50, "Highlight",
      CellTags->"$plotrangeoffset"],
    Cell[778564, 32963, 173, 6, 31, "Usage",
      CellTags->"$plotrangeoffset"],
    Cell[778740, 32971, 70, 1, 24, "Notes",
      CellTags->"$plotrangeoffset"],
    Cell[778813, 32974, 170, 7, 24, "Notes",
      CellTags->"$plotrangeoffset"],
    Cell[778986, 32983, 74, 1, 39, "ExampleSection",
      CellTags->"$plotrangeoffset"],
    Cell[779063, 32986, 80, 1, 34, "ExampleText",
      CellTags->"$plotrangeoffset"],
    Cell[779146, 32989, 107, 3, 26, "Input",
      CellTags->"$plotrangeoffset"]},
  "$rastercolors"->{
    Cell[779256, 32994, 64, 1, 32, "ObjectName",
      CellTags->"$rastercolors"],
    Cell[779323, 32997, 61, 1, 50, "Highlight",
      CellTags->"$rastercolors"],
    Cell[779387, 33000, 173, 5, 31, "Usage",
      CellTags->"$rastercolors"],
    Cell[779563, 33007, 210, 6, 24, "Notes",
      CellTags->"$rastercolors"],
    Cell[779776, 33015, 342, 15, 24, "Notes",
      CellTags->"$rastercolors"],
    Cell[780121, 33032, 71, 1, 39, "ExampleSection",
      CellTags->"$rastercolors"],
    Cell[780195, 33035, 211, 6, 50, "ExampleText",
      CellTags->"$rastercolors"],
    Cell[780409, 33043, 100, 3, 26, "Input",
      CellTags->"$rastercolors"],
    Cell[780512, 33048, 129, 3, 26, "Input",
      CellTags->"$rastercolors"],
    Cell[780644, 33053, 151, 4, 34, "ExampleText",
      CellTags->"$rastercolors"],
    Cell[780798, 33059, 182, 5, 26, "Input",
      CellTags->"$rastercolors"]},
  "ColorQ"->{
    Cell[780983, 33066, 50, 1, 32, "ObjectName",
      CellTags->"ColorQ"],
    Cell[781036, 33069, 51, 1, 50, "Highlight",
      CellTags->"ColorQ"],
    Cell[781090, 33072, 157, 6, 31, "Usage",
      CellTags->"ColorQ"],
    Cell[781250, 33080, 82, 1, 24, "Notes",
      CellTags->"ColorQ"],
    Cell[781335, 33083, 272, 12, 24, "Notes",
      CellTags->"ColorQ"]},
  "DoShow"->{
    Cell[781610, 33097, 50, 1, 32, "ObjectName",
      CellTags->"DoShow"],
    Cell[781663, 33100, 51, 1, 50, "Highlight",
      CellTags->"DoShow"],
    Cell[781717, 33103, 253, 10, 47, "Usage",
      CellTags->"DoShow"],
    Cell[781973, 33115, 134, 5, 24, "Notes",
      CellTags->"DoShow"],
    Cell[782110, 33122, 157, 7, 24, "Notes",
      CellTags->"DoShow"],
    Cell[782270, 33131, 64, 1, 39, "ExampleSection",
      CellTags->"DoShow"],
    Cell[782337, 33134, 80, 1, 34, "ExampleText",
      CellTags->"DoShow"],
    Cell[782420, 33137, 177, 5, 26, "Input",
      CellTags->"DoShow"],
    Cell[782600, 33144, 96, 3, 26, "Input",
      CellTags->"DoShow"]},
  "GridLine"->{
    Cell[782699, 33149, 54, 1, 32, "ObjectName",
      CellTags->"GridLine"],
    Cell[782756, 33152, 53, 1, 50, "Highlight",
      CellTags->"GridLine"],
    Cell[782812, 33155, 218, 8, 31, "Usage",
      CellTags->"GridLine"],
    Cell[783033, 33165, 353, 11, 79, "Notes",
      CellTags->"GridLine"],
    Cell[783389, 33178, 90, 1, 24, "Notes",
      CellTags->"GridLine"],
    Cell[783482, 33181, 259, 11, 24, "Notes",
      CellTags->"GridLine"],
    Cell[783744, 33194, 66, 1, 39, "ExampleSection",
      CellTags->"GridLine"],
    Cell[783813, 33197, 62, 1, 34, "ExampleText",
      CellTags->"GridLine"],
    Cell[783878, 33200, 114, 3, 26, "Input",
      CellTags->"GridLine"],
    Cell[783995, 33205, 80, 1, 34, "ExampleText",
      CellTags->"GridLine"],
    Cell[784078, 33208, 325, 8, 58, "Input",
      CellTags->"GridLine"],
    Cell[784406, 33218, 81, 2, 28, "Input",
      CellTags->"GridLine"]},
  "GridOffset"->{
    Cell[784490, 33222, 58, 1, 32, "ObjectName",
      CellTags->"GridOffset"],
    Cell[784551, 33225, 53, 1, 50, "Highlight",
      CellTags->"GridOffset"],
    Cell[784607, 33228, 147, 6, 31, "Usage",
      CellTags->"GridOffset"],
    Cell[784757, 33236, 158, 5, 24, "Notes",
      CellTags->"GridOffset"],
    Cell[784918, 33243, 159, 7, 24, "Notes",
      CellTags->"GridOffset"]},
  "GridPoints"->{
    Cell[785080, 33252, 58, 1, 32, "ObjectName",
      CellTags->"GridPoints"],
    Cell[785141, 33255, 55, 1, 50, "Highlight",
      CellTags->"GridPoints"],
    Cell[785199, 33258, 221, 8, 31, "Usage",
      CellTags->"GridPoints"],
    Cell[785423, 33268, 458, 14, 97, "Notes",
      CellTags->"GridPoints"],
    Cell[785884, 33284, 91, 1, 24, "Notes",
      CellTags->"GridPoints"],
    Cell[785978, 33287, 257, 11, 24, "Notes",
      CellTags->"GridPoints"],
    Cell[786238, 33300, 68, 1, 39, "ExampleSection",
      CellTags->"GridPoints"],
    Cell[786309, 33303, 65, 1, 34, "ExampleText",
      CellTags->"GridPoints"],
    Cell[786377, 33306, 144, 4, 26, "Input",
      CellTags->"GridPoints"],
    Cell[786524, 33312, 72, 1, 34, "ExampleText",
      CellTags->"GridPoints"],
    Cell[786599, 33315, 355, 7, 66, "Input",
      CellTags->"GridPoints"]},
  "HueRasterColors"->{
    Cell[786957, 33324, 68, 1, 32, "ObjectName",
      CellTags->"HueRasterColors"],
    Cell[787028, 33327, 60, 1, 50, "Highlight",
      CellTags->"HueRasterColors"],
    Cell[787091, 33330, 179, 6, 31, "Usage",
      CellTags->"HueRasterColors"],
    Cell[787273, 33338, 132, 5, 24, "Notes",
      CellTags->"HueRasterColors"],
    Cell[787408, 33345, 400, 15, 24, "Notes",
      CellTags->"HueRasterColors"],
    Cell[787811, 33362, 73, 1, 39, "ExampleSection",
      CellTags->"HueRasterColors"],
    Cell[787887, 33365, 76, 1, 34, "ExampleText",
      CellTags->"HueRasterColors"],
    Cell[787966, 33368, 142, 3, 26, "Input",
      CellTags->"HueRasterColors"],
    Cell[788111, 33373, 151, 4, 26, "Input",
      CellTags->"HueRasterColors"]},
  "LabelGrid"->{
    Cell[788265, 33379, 56, 1, 32, "ObjectName",
      CellTags->"LabelGrid"],
    Cell[788324, 33382, 52, 1, 50, "Highlight",
      CellTags->"LabelGrid"],
    Cell[788379, 33385, 224, 7, 47, "Usage",
      CellTags->"LabelGrid"],
    Cell[788606, 33394, 157, 7, 24, "Notes",
      CellTags->"LabelGrid"]},
  "LabelPoints"->{
    Cell[788766, 33403, 60, 1, 32, "ObjectName",
      CellTags->"LabelPoints"],
    Cell[788829, 33406, 54, 1, 50, "Highlight",
      CellTags->"LabelPoints"],
    Cell[788886, 33409, 201, 6, 31, "Usage",
      CellTags->"LabelPoints"],
    Cell[789090, 33417, 182, 7, 24, "Notes",
      CellTags->"LabelPoints"],
    Cell[789275, 33426, 159, 7, 24, "Notes",
      CellTags->"LabelPoints"]},
  "LineStyle"->{
    Cell[789437, 33435, 56, 1, 32, "ObjectName",
      CellTags->"LineStyle"],
    Cell[789496, 33438, 52, 1, 50, "Highlight",
      CellTags->"LineStyle"],
    Cell[789551, 33441, 207, 6, 47, "Usage",
      CellTags->"LineStyle"],
    Cell[789761, 33449, 134, 5, 24, "Notes",
      CellTags->"LineStyle"],
    Cell[789898, 33456, 159, 7, 24, "Notes",
      CellTags->"LineStyle"]},
  "MakePlotRange"->{
    Cell[790060, 33465, 64, 1, 32, "ObjectName",
      CellTags->"MakePlotRange"],
    Cell[790127, 33468, 58, 1, 50, "Highlight",
      CellTags->"MakePlotRange"],
    Cell[790188, 33471, 203, 6, 47, "Usage",
      CellTags->"MakePlotRange"],
    Cell[790394, 33479, 91, 1, 24, "Notes",
      CellTags->"MakePlotRange"],
    Cell[790488, 33482, 143, 6, 24, "Notes",
      CellTags->"MakePlotRange"]},
  "MultiCol"->{
    Cell[790634, 33490, 54, 1, 32, "ObjectName",
      CellTags->"MultiCol"],
    Cell[790691, 33493, 51, 1, 50, "Highlight",
      CellTags->"MultiCol"],
    Cell[790745, 33496, 145, 6, 31, "Usage",
      CellTags->"MultiCol"],
    Cell[790893, 33504, 156, 7, 24, "Notes",
      CellTags->"MultiCol"]},
  "NRows"->{
    Cell[791052, 33513, 48, 1, 32, "ObjectName",
      CellTags->"NRows"],
    Cell[791103, 33516, 48, 1, 50, "Highlight",
      CellTags->"NRows"],
    Cell[791154, 33519, 139, 6, 31, "Usage",
      CellTags->"NRows"],
    Cell[791296, 33527, 153, 7, 24, "Notes",
      CellTags->"NRows"]},
  "PlotComputationFA"->{
    Cell[791452, 33536, 72, 1, 32, "ObjectName",
      CellTags->"PlotComputationFA"],
    Cell[791527, 33539, 62, 1, 50, "Highlight",
      CellTags->"PlotComputationFA"],
    Cell[791592, 33542, 224, 8, 31, "Usage",
      CellTags->"PlotComputationFA"],
    Cell[791819, 33552, 379, 12, 97, "Notes",
      CellTags->"PlotComputationFA"],
    Cell[792201, 33566, 222, 6, 40, "Notes",
      CellTags->"PlotComputationFA"],
    Cell[792426, 33574, 179, 7, 24, "Notes",
      CellTags->"PlotComputationFA"],
    Cell[792608, 33583, 293, 11, 24, "Notes",
      CellTags->"PlotComputationFA"],
    Cell[792904, 33596, 75, 1, 39, "ExampleSection",
      CellTags->"PlotComputationFA"],
    Cell[792982, 33599, 206, 7, 34, "ExampleText",
      CellTags->"PlotComputationFA"],
    Cell[793191, 33608, 118, 3, 26, "Input",
      CellTags->"PlotComputationFA"],
    Cell[793312, 33613, 155, 4, 34, "ExampleText",
      CellTags->"PlotComputationFA"],
    Cell[793470, 33619, 126, 3, 26, "Input",
      CellTags->"PlotComputationFA"],
    Cell[793599, 33624, 129, 4, 34, "ExampleText",
      CellTags->"PlotComputationFA"],
    Cell[793731, 33630, 183, 4, 26, "Input",
      CellTags->"PlotComputationFA"]},
  "PlotFunction"->{
    Cell[793917, 33636, 62, 1, 32, "ObjectName",
      CellTags->"PlotFunction"],
    Cell[793982, 33639, 57, 1, 50, "Highlight",
      CellTags->"PlotFunction"],
    Cell[794042, 33642, 256, 10, 31, "Usage",
      CellTags->"PlotFunction"],
    Cell[794301, 33654, 442, 14, 97, "Notes",
      CellTags->"PlotFunction"],
    Cell[794746, 33670, 168, 7, 24, "Notes",
      CellTags->"PlotFunction"],
    Cell[794917, 33679, 70, 1, 39, "ExampleSection",
      CellTags->"PlotFunction"],
    Cell[794990, 33682, 90, 1, 34, "ExampleText",
      CellTags->"PlotFunction"],
    Cell[795083, 33685, 174, 4, 47, "Input",
      CellTags->"PlotFunction"]},
  "PlotMatrix"->{
    Cell[795260, 33691, 58, 1, 32, "ObjectName",
      CellTags->"PlotMatrix"],
    Cell[795321, 33694, 55, 1, 50, "Highlight",
      CellTags->"PlotMatrix"],
    Cell[795379, 33697, 163, 6, 31, "Usage",
      CellTags->"PlotMatrix"],
    Cell[795545, 33705, 528, 15, 151, "Notes",
      CellTags->"PlotMatrix"],
    Cell[796076, 33722, 156, 5, 24, "Notes",
      CellTags->"PlotMatrix"],
    Cell[796235, 33729, 139, 5, 24, "Notes",
      CellTags->"PlotMatrix"],
    Cell[796377, 33736, 142, 5, 24, "Notes",
      CellTags->"PlotMatrix"],
    Cell[796522, 33743, 140, 5, 24, "Notes",
      CellTags->"PlotMatrix"],
    Cell[796665, 33750, 179, 5, 24, "Notes",
      CellTags->"PlotMatrix"],
    Cell[796847, 33757, 199, 6, 40, "Notes",
      CellTags->"PlotMatrix"],
    Cell[797049, 33765, 171, 5, 24, "Notes",
      CellTags->"PlotMatrix"],
    Cell[797223, 33772, 185, 7, 24, "Notes",
      CellTags->"PlotMatrix"],
    Cell[797411, 33781, 373, 15, 24, "Notes",
      CellTags->"PlotMatrix"],
    Cell[797787, 33798, 68, 1, 39, "ExampleSection",
      CellTags->"PlotMatrix"],
    Cell[797858, 33801, 96, 1, 34, "ExampleText",
      CellTags->"PlotMatrix"],
    Cell[797957, 33804, 221, 8, 41, "Input",
      CellTags->"PlotMatrix"],
    Cell[798181, 33814, 79, 1, 34, "ExampleText",
      CellTags->"PlotMatrix"],
    Cell[798263, 33817, 127, 3, 26, "Input",
      CellTags->"PlotMatrix"],
    Cell[798393, 33822, 88, 1, 34, "ExampleText",
      CellTags->"PlotMatrix"],
    Cell[798484, 33825, 297, 7, 58, "Input",
      CellTags->"PlotMatrix"]},
  "PlotRelation"->{
    Cell[798784, 33834, 62, 1, 32, "ObjectName",
      CellTags->"PlotRelation"],
    Cell[798849, 33837, 57, 1, 50, "Highlight",
      CellTags->"PlotRelation"],
    Cell[798909, 33840, 263, 10, 47, "Usage",
      CellTags->"PlotRelation"],
    Cell[799175, 33852, 531, 16, 97, "Notes",
      CellTags->"PlotRelation"],
    Cell[799709, 33870, 254, 11, 24, "Notes",
      CellTags->"PlotRelation"],
    Cell[799966, 33883, 70, 1, 39, "ExampleSection",
      CellTags->"PlotRelation"],
    Cell[800039, 33886, 90, 1, 34, "ExampleText",
      CellTags->"PlotRelation"],
    Cell[800132, 33889, 142, 3, 30, "Input",
      CellTags->"PlotRelation"],
    Cell[800277, 33894, 132, 3, 26, "Input",
      CellTags->"PlotRelation"]},
  "PlotTDGraph"->{
    Cell[800412, 33899, 60, 1, 32, "ObjectName",
      CellTags->"PlotTDGraph"],
    Cell[800475, 33902, 56, 1, 50, "Highlight",
      CellTags->"PlotTDGraph"],
    Cell[800534, 33905, 213, 8, 31, "Usage",
      CellTags->"PlotTDGraph"],
    Cell[800750, 33915, 530, 16, 97, "Notes",
      CellTags->"PlotTDGraph"],
    Cell[801283, 33933, 155, 5, 24, "Notes",
      CellTags->"PlotTDGraph"],
    Cell[801441, 33940, 221, 7, 24, "Notes",
      CellTags->"PlotTDGraph"],
    Cell[801665, 33949, 328, 15, 24, "Notes",
      CellTags->"PlotTDGraph"],
    Cell[801996, 33966, 69, 1, 39, "ExampleSection",
      CellTags->"PlotTDGraph"],
    Cell[802068, 33969, 189, 6, 34, "ExampleText",
      CellTags->"PlotTDGraph"],
    Cell[802260, 33977, 447, 10, 58, "Input",
      CellTags->"PlotTDGraph"],
    Cell[802710, 33989, 108, 3, 26, "Input",
      CellTags->"PlotTDGraph"],
    Cell[802821, 33994, 163, 4, 26, "Input",
      CellTags->"PlotTDGraph"]},
  "PlotT"->{
    Cell[802987, 34000, 48, 1, 32, "ObjectName",
      CellTags->"PlotT"],
    Cell[803038, 34003, 50, 1, 50, "Highlight",
      CellTags->"PlotT"],
    Cell[803091, 34006, 194, 8, 31, "Usage",
      CellTags->"PlotT"],
    Cell[803288, 34016, 217, 9, 24, "Notes",
      CellTags->"PlotT"],
    Cell[803508, 34027, 101, 5, 24, "Notes",
      CellTags->"PlotT"],
    Cell[803612, 34034, 254, 12, 24, "Notes",
      CellTags->"PlotT"],
    Cell[803869, 34048, 63, 1, 39, "ExampleSection",
      CellTags->"PlotT"],
    Cell[803935, 34051, 98, 1, 34, "ExampleText",
      CellTags->"PlotT"],
    Cell[804036, 34054, 79, 2, 28, "Input",
      CellTags->"PlotT"],
    Cell[804118, 34058, 93, 3, 26, "Input",
      CellTags->"PlotT"],
    Cell[804214, 34063, 67, 1, 34, "ExampleText",
      CellTags->"PlotT"],
    Cell[804284, 34066, 163, 4, 26, "Input",
      CellTags->"PlotT"]},
  "RasterStyle"->{
    Cell[804450, 34072, 60, 1, 32, "ObjectName",
      CellTags->"RasterStyle"],
    Cell[804513, 34075, 54, 1, 50, "Highlight",
      CellTags->"RasterStyle"],
    Cell[804570, 34078, 203, 6, 31, "Usage",
      CellTags->"RasterStyle"],
    Cell[804776, 34086, 214, 9, 24, "Notes",
      CellTags->"RasterStyle"],
    Cell[804993, 34097, 270, 11, 24, "Notes",
      CellTags->"RasterStyle"]},
  "RView"->{
    Cell[805266, 34110, 48, 1, 32, "ObjectName",
      CellTags->"RView"],
    Cell[805317, 34113, 48, 1, 50, "Highlight",
      CellTags->"RView"],
    Cell[805368, 34116, 139, 6, 31, "Usage",
      CellTags->"RView"],
    Cell[805510, 34124, 153, 7, 24, "Notes",
      CellTags->"RView"]},
  "ShowArray"->{
    Cell[805666, 34133, 56, 1, 32, "ObjectName",
      CellTags->"ShowArray"],
    Cell[805725, 34136, 54, 1, 50, "Highlight",
      CellTags->"ShowArray"],
    Cell[805782, 34139, 231, 8, 31, "Usage",
      CellTags->"ShowArray"],
    Cell[806016, 34149, 191, 7, 24, "Notes",
      CellTags->"ShowArray"],
    Cell[806210, 34158, 162, 7, 24, "Notes",
      CellTags->"ShowArray"],
    Cell[806375, 34167, 67, 1, 39, "ExampleSection",
      CellTags->"ShowArray"],
    Cell[806445, 34170, 86, 1, 34, "ExampleText",
      CellTags->"ShowArray"],
    Cell[806534, 34173, 239, 5, 42, "Input",
      CellTags->"ShowArray"],
    Cell[806776, 34180, 102, 3, 26, "Input",
      CellTags->"ShowArray"]},
  "StaircasePlot"->{
    Cell[806881, 34185, 64, 1, 32, "ObjectName",
      CellTags->"StaircasePlot"],
    Cell[806948, 34188, 58, 1, 50, "Highlight",
      CellTags->"StaircasePlot"],
    Cell[807009, 34191, 199, 6, 31, "Usage",
      CellTags->"StaircasePlot"],
    Cell[807211, 34199, 278, 10, 61, "Notes",
      CellTags->"StaircasePlot"],
    Cell[807492, 34211, 128, 5, 24, "Notes",
      CellTags->"StaircasePlot"],
    Cell[807623, 34218, 440, 19, 24, "Notes",
      CellTags->"StaircasePlot"],
    Cell[808066, 34239, 71, 1, 39, "ExampleSection",
      CellTags->"StaircasePlot"],
    Cell[808140, 34242, 92, 1, 34, "ExampleText",
      CellTags->"StaircasePlot"],
    Cell[808235, 34245, 223, 5, 42, "Input",
      CellTags->"StaircasePlot"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"$alphabet", 809177, 34272},
  {"AlgebraicForm", 809918, 34293},
  {"Alphabet", 810555, 34310},
  {"Alphabetic", 811797, 34345},
  {"AlphabetQ", 812183, 34356},
  {"AlphabetSize", 813301, 34387},
  {"Concatenate", 814166, 34410},
  {"Eps", 815242, 34439},
  {"FromEps", 816145, 34466},
  {"FromFrontier", 816746, 34483},
  {"FrontierQ", 817839, 34512},
  {"MakeAlphabet", 818606, 34533},
  {"MakeWord", 819626, 34560},
  {"ParikhVector", 820375, 34581},
  {"RandomWord", 821176, 34602},
  {"ToBlocks", 821877, 34621},
  {"ToEps", 822926, 34650},
  {"ToFrontier", 823516, 34667},
  {"ToIndex", 824292, 34688},
  {"ToWord", 825034, 34709},
  {"WordBinomial", 825918, 34734},
  {"WordConjugateQ", 826879, 34759},
  {"WordConjugates", 827780, 34782},
  {"WordFactorizations", 828529, 34801},
  {"WordFactorQ", 829308, 34820},
  {"WordFactors", 830100, 34841},
  {"WordFusion", 831193, 34870},
  {"WordLength", 831898, 34889},
  {"WordOrder", 832751, 34912},
  {"WordPeriod", 833520, 34933},
  {"WordPower", 834756, 34966},
  {"WordPrefixQ", 835601, 34989},
  {"WordPrefixes", 836468, 35012},
  {"WordPrimitiveCount", 837508, 35039},
  {"WordPrimitiveQ", 838290, 35058},
  {"WordQ", 839352, 35085},
  {"WordReverse", 840014, 35104},
  {"WordRoot", 841035, 35131},
  {"WordRotate", 841723, 35150},
  {"Words", 842570, 35173},
  {"WordShuffle", 843787, 35208},
  {"WordSort", 844883, 35237},
  {"WordSuffixQ", 845643, 35258},
  {"WordSuffixes", 846510, 35281},
  {"$bottom", 847539, 35308},
  {"$defaultnormalize", 848443, 35333},
  {"AcceptQFA", 849704, 35364},
  {"AccessibleFA", 850927, 35397},
  {"AddTransitionFA", 852043, 35426},
  {"AllDFA", 853118, 35453},
  {"AmbiguousQFA", 853861, 35474},
  {"BEFA", 855048, 35505},
  {"BEFAToFA", 855907, 35530},
  {"BehaviorFA", 856593, 35549},
  {"Boundary", 857754, 35580},
  {"BuildBEFA", 858140, 35591},
  {"BuildDFA", 858834, 35610},
  {"BuildNNFA", 859891, 35639},
  {"BuildStarDFA", 860588, 35658},
  {"CirculantSA", 861471, 35681},
  {"CoAccessibleFA", 862341, 35704},
  {"ComplementFA", 863413, 35731},
  {"CompleteFA", 864614, 35762},
  {"CompleteQFA", 865403, 35783},
  {"ComputationFA", 866286, 35806},
  {"ComputationGraphFA", 867678, 35841},
  {"ConcatenateFA", 868552, 35862},
  {"ConvertToFA", 869687, 35891},
  {"CounterDFA", 870725, 35918},
  {"CoverQDFA", 871904, 35949},
  {"DefiniteQFA", 872908, 35976},
  {"DegreesFA", 874095, 36007},
  {"DeleteStateFA", 875027, 36032},
  {"DeterministicQFA", 875931, 36055},
  {"DFA", 877195, 36086},
  {"DivisibilityDFA", 878262, 36117},
  {"EdgeLU", 880166, 36164},
  {"EmptyDFA", 880544, 36175},
  {"EmptyQFA", 881166, 36192},
  {"EpsElim", 881860, 36211},
  {"EpsilonFreeQ", 882249, 36222},
  {"EpsilonEliminationFA", 882993, 36241},
  {"EquivalentQFA", 884572, 36278},
  {"FactorialQFA", 886108, 36317},
  {"FA", 887069, 36342},
  {"FinalVectorFA", 887916, 36367},
  {"Final", 888574, 36384},
  {"FiniteComplementSA", 889182, 36401},
  {"FinLU", 890214, 36426},
  {"Fixed", 890584, 36437},
  {"BoundLFA", 890957, 36448},
  {"FrontierToDFA", 891423, 36461},
  {"FSMQ", 892806, 36496},
  {"FullQFA", 893461, 36515},
  {"FullTransitionMatrixFA", 894162, 36534},
  {"FullTransitions", 895344, 36561},
  {"FunctionToDFA", 896109, 36580},
  {"FunctionToSA", 897255, 36609},
  {"GenerateDFA", 898461, 36640},
  {"GenerateQuotients", 899733, 36673},
  {"GrowthSeriesDFA", 900598, 36694},
  {"HomomorphismFA", 902096, 36731},
  {"Hopcroft", 902926, 36752},
  {"HurwitzProduct", 903394, 36765},
  {"IdealBaseFA", 904546, 36794},
  {"IdealQFA", 905818, 36827},
  {"Implementation", 906517, 36846},
  {"InfiniteQFA", 906938, 36857},
  {"IniLU", 907889, 36882},
  {"InitialVectorFA", 908269, 36893},
  {"Initial", 909026, 36912},
  {"IntersectionFA", 909646, 36929},
  {"IrreducibleFA", 910801, 36958},
  {"IsomorphicQDFA", 911943, 36987},
  {"IthSymbolFA", 912932, 37012},
  {"JoinSA", 914356, 37049},
  {"LanguageFA", 915477, 37080},
  {"LanguageSort", 917797, 37141},
  {"LanguageToSeries", 918536, 37160},
  {"LeftQuotient", 919387, 37181},
  {"LengthDFA", 920350, 37206},
  {"LocalQSA", 921582, 37239},
  {"LocalUniversalFA", 922728, 37270},
  {"LoopAutomatonFA", 923833, 37297},
  {"MakeHomomorphismDFA", 924935, 37324},
  {"MakeHomomorphism", 926079, 37351},
  {"MakeTrim", 927284, 37381},
  {"MatrixToTransitions", 927684, 37392},
  {"MinimalFischerFA", 928486, 37411},
  {"MinimizeDFA", 929499, 37436},
  {"MinimizeFA", 930452, 37461},
  {"NFA", 931317, 37484},
  {"NNFAToFA", 932184, 37509},
  {"NNFA", 933110, 37534},
  {"Normalize", 933709, 37551},
  {"NormalizeControl", 935049, 37586},
  {"NormalizeFA", 936080, 37611},
  {"NumberOfDFA", 936891, 37632},
  {"NumTransitions", 937551, 37649},
  {"PermutationSA", 938234, 37666},
  {"PermuteFA", 939225, 37691},
  {"PrintFA", 939937, 37710},
  {"ProductFA", 940998, 37739},
  {"ProductTransitions", 942028, 37766},
  {"QuotientDFA", 943249, 37795},
  {"QuotientMachineFA", 944302, 37822},
  {"RandomDFA", 945678, 37855},
  {"RankOfDFA", 946390, 37874},
  {"RankToDFA", 947104, 37893},
  {"DeleteTransitionFA", 947752, 37910},
  {"ReverseFA", 948806, 37935},
  {"SA", 949513, 37954},
  {"SelectFA", 950366, 37979},
  {"SetAlphabet", 951608, 38013},
  {"SetFinalFA", 952659, 38040},
  {"SetInitialFA", 953854, 38071},
  {"SetInitialFinalFA", 954916, 38098},
  {"SetStates", 956208, 38129},
  {"ShowAll", 957230, 38156},
  {"ShowEdges", 957693, 38169},
  {"ShuffleProductFA", 958100, 38180},
  {"SinkQDFA", 959040, 38203},
  {"SizeOnly", 959977, 38228},
  {"Size", 960752, 38249},
  {"Some", 961560, 38272},
  {"SquareSA", 961864, 38281},
  {"StarFA", 963166, 38316},
  {"StateMergeDFA", 964001, 38339},
  {"States", 965071, 38366},
  {"SubsetQFA", 965837, 38387},
  {"SubwordCounterDFA", 966709, 38410},
  {"SubwordDFA", 968086, 38443},
  {"SumFA", 969031, 38468},
  {"SwitchTransitionFA", 970150, 38499},
  {"TheInitial", 971116, 38522},
  {"Thompson", 971762, 38539},
  {"ToCharacters", 972160, 38550},
  {"ToDFA", 972896, 38569},
  {"ToEquationsDFA", 974155, 38604},
  {"ToFA", 975075, 38627},
  {"ToFactorFA", 976097, 38656},
  {"ToFrontierDFA", 977841, 38701},
  {"ToGeneratorsFA", 979163, 38734},
  {"ToGraphFA", 980167, 38759},
  {"ToKernelFA", 981112, 38784},
  {"ToMatrix", 982616, 38823},
  {"ToNFA", 983007, 38834},
  {"ToRegularExpressionFA", 983767, 38855},
  {"ToSA", 985026, 38884},
  {"ToSeries", 986192, 38917},
  {"ToTransitions", 987046, 38940},
  {"TransferSequenceFA", 987647, 38955},
  {"TransitionFunctionFA", 988537, 38976},
  {"TransitionMatrixFA", 989888, 39007},
  {"TransitionsToMatrix", 991374, 39042},
  {"Transitions", 992698, 39073},
  {"TransitiveLanguageQFA", 993444, 39092},
  {"TransitiveQFA", 994356, 39113},
  {"TransOnly", 995264, 39136},
  {"TrapStatesFA", 996054, 39157},
  {"TrimFA", 996948, 39180},
  {"TropicalIdentityMatrix", 997721, 39201},
  {"Tropical", 998376, 39216},
  {"TS", 998765, 39227},
  {"UnionFA", 999750, 39256},
  {"Unlabeled", 1000818, 39285},
  {"VerifyFA", 1001604, 39306},
  {"WordToFA", 1002006, 39317},
  {"$regexrules", 1003787, 39364},
  {"ArdenSolve", 1004673, 39387},
  {"Empty", 1006494, 39434},
  {"FormatX", 1006801, 39443},
  {"REEpsQ", 1007795, 39470},
  {"RegExpSimp", 1008552, 39491},
  {"RegularExpressionToBEFA", 1008971, 39502},
  {"RegularExpressionToFA", 1010274, 39531},
  {"RegularExpressionToNNFA", 1011819, 39566},
  {"REP", 1012837, 39589},
  {"REQuotient", 1013567, 39610},
  {"RES", 1014442, 39633},
  {"RESimplify", 1015316, 39658},
  {"RET", 1016115, 39679},
  {"SubstituteRE", 1016847, 39700},
  {"ToRegExp", 1017996, 39729},
  {"BFunctionToCA", 1018779, 39750},
  {"CA", 1020095, 39783},
  {"ClassifyCA", 1020964, 39808},
  {"ComposeCA", 1022461, 39847},
  {"Cyclic", 1023321, 39870},
  {"EvolutionCA", 1023637, 39879},
  {"DependencyCA", 1024764, 39908},
  {"FSurjectiveCA", 1025430, 39925},
  {"FunctionToCA", 1026263, 39946},
  {"GlobalMapCA", 1027169, 39969},
  {"InverseCA", 1028369, 40000},
  {"IterateCA", 1029314, 40025},
  {"LocalMapCA", 1030029, 40044},
  {"NumberOfCA", 1031138, 40073},
  {"NumberOfTotalisticCA", 1031795, 40090},
  {"OrbitCA", 1032520, 40107},
  {"Padding", 1033661, 40138},
  {"PolynomialToCA", 1034436, 40159},
  {"PrintCA", 1035116, 40176},
  {"RuleCA", 1035734, 40193},
  {"RulesToCA", 1036421, 40212},
  {"SeedConfiguration", 1037379, 40237},
  {"SimplifyCA", 1039016, 40276},
  {"ToRulesCA", 1040048, 40303},
  {"TotalisticRuleToCA", 1040851, 40324},
  {"ToTotalisticRuleCA", 1041655, 40343},
  {"WelchCA", 1042363, 40360},
  {"Where", 1043426, 40389},
  {"WidthCA", 1044173, 40410},
  {"WordMap", 1044794, 40427},
  {"AddEdge", 1045416, 40444},
  {"AddVertex", 1046184, 40465},
  {"BiAccessibleGraph", 1047057, 40488},
  {"AccessibleGraph", 1048106, 40513},
  {"CirculantGraph", 1049134, 40538},
  {"CyclicQGraph", 1050301, 40567},
  {"DeBruijnEdges", 1051200, 40590},
  {"DeBruijnGraph", 1052196, 40615},
  {"DegreeGraph", 1053271, 40642},
  {"DeleteEdge", 1054311, 40669},
  {"DeleteVertex", 1055037, 40688},
  {"EdgeQ", 1055852, 40709},
  {"Edges", 1056299, 40722},
  {"Graph", 1056971, 40741},
  {"GridGraph", 1058228, 40776},
  {"LabelEdgeList", 1059403, 40807},
  {"LongestPath", 1060153, 40826},
  {"NormalizeGraph", 1061280, 40855},
  {"NonTransientGraph", 1062128, 40876},
  {"NonTrivial", 1063081, 40899},
  {"NumEdges", 1063485, 40910},
  {"NumVertices", 1064118, 40927},
  {"PathGraph", 1064773, 40944},
  {"PermuteGraph", 1065795, 40971},
  {"ProductGraph", 1066538, 40990},
  {"ProductLabeledGraph", 1067927, 41025},
  {"ReverseGraph", 1068906, 41048},
  {"SetVertices", 1069726, 41069},
  {"ShortestPath", 1070620, 41092},
  {"StronglyConnectedComponents", 1071616, 41117},
  {"SubGraph", 1072870, 41144},
  {"ToAdjacencyList", 1073802, 41169},
  {"ToAdjacencyMatrix", 1074739, 41192},
  {"ToDirectedAcyclicGraph", 1075538, 41211},
  {"ToEdgeList", 1076552, 41234},
  {"ToTDGraph", 1077276, 41253},
  {"TypeGraph", 1078213, 41278},
  {"VertexSet", 1079150, 41303},
  {"WeaklyConnectedComponents", 1079880, 41322},
  {"$generateSGBound", 1080745, 41341},
  {"$identitystring", 1081444, 41358},
  {"$nullstring", 1082133, 41375},
  {"AllTimes", 1082787, 41392},
  {"AlternatingGroupSG", 1083656, 41415},
  {"AperiodicQSG", 1084456, 41434},
  {"AperiodicSG", 1085442, 41459},
  {"BlocksToDClass", 1086180, 41478},
  {"BoundCSM", 1087345, 41507},
  {"BoundGSG", 1087738, 41518},
  {"CollapseT", 1088133, 41529},
  {"ConstantT", 1088995, 41552},
  {"CycleDecompositionT", 1089569, 41567},
  {"CycleToT", 1090543, 41590},
  {"DC", 1091387, 41613},
  {"DClasses", 1092038, 41632},
  {"DClassDecomposition", 1092444, 41643},
  {"DClassSize", 1093851, 41676},
  {"DClassT", 1094807, 41701},
  {"DClassToBlocks", 1095653, 41724},
  {"DihedralGroupSG", 1096578, 41747},
  {"GEN", 1097508, 41770},
  {"GenerateCSM", 1097882, 41781},
  {"GenerateCayleyGraph", 1099569, 41824},
  {"GenerateSG", 1101233, 41863},
  {"GreenHQT", 1103796, 41930},
  {"GreenJQT", 1104118, 41939},
  {"GreenLQT", 1104590, 41952},
  {"GreenRQT", 1104912, 41961},
  {"HClassSize", 1105236, 41970},
  {"HClassT", 1105639, 41981},
  {"HTrivialQSG", 1106030, 41992},
  {"IdealT", 1106758, 42011},
  {"IdemSG", 1107514, 42032},
  {"ImageT", 1108490, 42059},
  {"InGroupSG", 1109105, 42076},
  {"InverseT", 1110042, 42101},
  {"JTrivialQSG", 1110675, 42118},
  {"JTrivialSG", 1111638, 42143},
  {"KernelImageToT", 1112440, 42164},
  {"KernelT", 1113277, 42185},
  {"LeftIdealT", 1113974, 42204},
  {"LeftNullSG", 1114774, 42225},
  {"LeftOneSG", 1115724, 42250},
  {"LocallyCommutativeQSG", 1116674, 42275},
  {"LocallyIdempotentQSG", 1117775, 42300},
  {"Merge12T", 1118760, 42323},
  {"MakeMonoidSG", 1119696, 42348},
  {"MonoidQSG", 1120358, 42365},
  {"NilpotentQSG", 1120999, 42382},
  {"NullSG", 1121738, 42401},
  {"OneSG", 1122567, 42424},
  {"OneT", 1123815, 42459},
  {"OrderT", 1124479, 42478},
  {"PrintDClass", 1125096, 42495},
  {"PrintEquations", 1125910, 42516},
  {"PrintWitnesses", 1126922, 42541},
  {"RegularSG", 1127602, 42558},
  {"RandomT", 1128536, 42583},
  {"RankT", 1129153, 42600},
  {"RegularOnly", 1129833, 42619},
  {"RightIdealT", 1130323, 42632},
  {"RightNullSG", 1131134, 42653},
  {"RightOneSG", 1132097, 42678},
  {"ShiftT", 1133043, 42703},
  {"SG", 1133874, 42726},
  {"SolveSG", 1134455, 42743},
  {"SymmetricGroupSG", 1135528, 42772},
  {"SyntacticCongruence", 1136149, 42787},
  {"SyntacticHomomorphism", 1137142, 42810},
  {"SyntacticSG", 1138228, 42835},
  {"T", 1138953, 42854},
  {"TableSG", 1139661, 42875},
  {"ToIdentityString", 1140435, 42896},
  {"TransformationSG", 1141055, 42911},
  {"TransientPeriodicT", 1141841, 42930},
  {"TranspositionT", 1142725, 42951},
  {"UnitElement", 1143566, 42972},
  {"$rewriteSimpBound", 1144380, 42993},
  {"AllStringReplace", 1145088, 43010},
  {"BoundETW", 1146028, 43033},
  {"BoundWR", 1146420, 43044},
  {"Cols", 1146804, 43055},
  {"EquationsToWitnesses", 1147120, 43064},
  {"Equations", 1148549, 43097},
  {"FromIdentityString", 1148884, 43106},
  {"IrreducibleQ", 1149851, 43129},
  {"PairsToRules", 1150672, 43150},
  {"RulesToPairs", 1151339, 43167},
  {"SimplifyEquations", 1152011, 43184},
  {"SimplifyRewriteSystem", 1152976, 43207},
  {"WordReduce", 1154147, 43234},
  {"BooleanComplement", 1155340, 43265},
  {"BooleanComposition", 1156216, 43286},
  {"BooleanConverse", 1157192, 43309},
  {"BooleanIntersection", 1158052, 43330},
  {"BooleanPower", 1158945, 43351},
  {"BooleanUnion", 1159774, 43372},
  {"Boolean", 1160592, 43393},
  {"RelationToMatrix", 1161438, 43416},
  {"ToClasses", 1162215, 43435},
  {"TransitiveClosure", 1163779, 43476},
  {"Warshall", 1164981, 43505},
  {"AnalyzeOrbit", 1166370, 43542},
  {"BoundAO", 1168299, 43591},
  {"CartesianProduct", 1168695, 43602},
  {"$closureBound", 1169898, 43631},
  {"ClosureBound", 1170571, 43648},
  {"$closureSearchBound", 1170919, 43657},
  {"ClosureSearch", 1171641, 43674},
  {"Closure", 1172872, 43705},
  {"Cycle", 1174830, 43758},
  {"DeleteDuplicates", 1175146, 43767},
  {"DisjointQ", 1176338, 43796},
  {"EnumForm", 1176973, 43813},
  {"External", 1177976, 43840},
  {"FastPowerSet", 1178523, 43855},
  {"Filter", 1179339, 43876},
  {"Frequencies", 1180243, 43901},
  {"FromBitVector", 1180824, 43916},
  {"Index", 1181568, 43935},
  {"Kary", 1181873, 43944},
  {"LongestCommonSubsequence", 1182193, 43953},
  {"MakeIndex", 1183317, 43978},
  {"Matrix", 1184103, 43999},
  {"NonDecreasingSequences", 1184429, 44008},
  {"NoRest", 1185709, 44037},
  {"OutPairs", 1186021, 44046},
  {"Pairs", 1186411, 44057},
  {"Partitions", 1187367, 44084},
  {"Periodic", 1188013, 44101},
  {"PowerSet", 1188407, 44112},
  {"RandomPermutation", 1189192, 44133},
  {"Rank", 1189889, 44150},
  {"RankingRules", 1190275, 44161},
  {"Reflexive", 1191655, 44196},
  {"Relation", 1192054, 44207},
  {"RootList", 1192374, 44216},
  {"SelectorQ", 1193079, 44235},
  {"SetClosure", 1193869, 44256},
  {"Shuffle", 1194197, 44265},
  {"SubsetQ", 1194892, 44284},
  {"ToBitVector", 1195587, 44303},
  {"ToList", 1196313, 44322},
  {"ToPairs", 1196552, 44329},
  {"Tuples", 1197690, 44360},
  {"Type", 1198742, 44389},
  {"$extlink", 1199117, 44400},
  {"$defaultexternal", 1199829, 44419},
  {"$LLL", 1200602, 44438},
  {"$shortlines", 1201856, 44473},
  {"$transdispwidth", 1202903, 44500},
  {"All", 1203662, 44519},
  {"AssignFunction", 1204333, 44538},
  {"Curry", 1205661, 44571},
  {"Dependency", 1206339, 44590},
  {"DoTrace", 1207136, 44611},
  {"Drei", 1207449, 44620},
  {"ExtAutomata", 1207827, 44631},
  {"FlattenOne", 1208714, 44654},
  {"FormatAutomata", 1209440, 44673},
  {"Full", 1211159, 44716},
  {"MinusEins", 1211466, 44725},
  {"MinusZwei", 1211867, 44736},
  {"Opts", 1212263, 44747},
  {"PositionList", 1212640, 44758},
  {"PositionOne", 1213933, 44791},
  {"ReplaceParts", 1214590, 44808},
  {"SelectOptions", 1215177, 44823},
  {"Status", 1215515, 44832},
  {"SymbolList", 1215985, 44845},
  {"Zwei", 1216551, 44860},
  {"$AutomataReleaseNumber", 1216940, 44871},
  {"PostLoad", 1217505, 44884},
  {"PreLoad", 1217750, 44891},
  {"$deffatline", 1217996, 44898},
  {"$deflabeloffset", 1218655, 44915},
  {"$defpointsize", 1219345, 44932},
  {"$defthinline", 1220098, 44951},
  {"$plotrangeoffset", 1220924, 44972},
  {"$rastercolors", 1221621, 44989},
  {"ColorQ", 1222533, 45012},
  {"DoShow", 1222915, 45023},
  {"GridLine", 1223600, 45042},
  {"GridOffset", 1224532, 45067},
  {"GridPoints", 1224938, 45078},
  {"HueRasterColors", 1225824, 45101},
  {"LabelGrid", 1226592, 45120},
  {"LabelPoints", 1226919, 45129},
  {"LineStyle", 1227329, 45140},
  {"MakePlotRange", 1227733, 45151},
  {"MultiCol", 1228151, 45162},
  {"NRows", 1228468, 45171},
  {"PlotComputationFA", 1228785, 45180},
  {"PlotFunction", 1230003, 45209},
  {"PlotMatrix", 1230668, 45226},
  {"PlotRelation", 1232242, 45267},
  {"PlotTDGraph", 1232987, 45286},
  {"PlotT", 1233949, 45311},
  {"RasterStyle", 1234845, 45336},
  {"RView", 1235252, 45347},
  {"ShowArray", 1235561, 45356},
  {"StaircasePlot", 1236278, 45375}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 56, 1, 32, "ObjectName",
  CellTags->"$alphabet"],
Cell[1813, 54, 57, 1, 50, "Highlight",
  CellTags->"$alphabet"],
Cell[1873, 57, 183, 5, 47, "Usage",
  CellTags->"$alphabet"],
Cell[2059, 64, 178, 6, 24, "Notes",
  CellTags->"$alphabet"],
Cell[2240, 72, 147, 4, 24, "Notes",
  CellTags->"$alphabet"],
Cell[2390, 78, 67, 1, 39, "ExampleSection",
  CellTags->"$alphabet"],
Cell[2460, 81, 82, 1, 34, "ExampleText",
  CellTags->"$alphabet"],
Cell[2545, 84, 76, 2, 28, "Input",
  CellTags->"$alphabet"],
Cell[2624, 88, 120, 3, 26, "Input",
  CellTags->"$alphabet"],
Cell[2747, 93, 137, 3, 26, "Input",
  CellTags->"$alphabet"],
Cell[2887, 98, 64, 1, 32, "ObjectName",
  CellTags->"AlgebraicForm"],
Cell[2954, 101, 58, 1, 50, "Highlight",
  CellTags->"AlgebraicForm"],
Cell[3015, 104, 199, 6, 31, "Usage",
  CellTags->"AlgebraicForm"],
Cell[3217, 112, 175, 7, 24, "Notes",
  CellTags->"AlgebraicForm"],
Cell[3395, 121, 71, 1, 39, "ExampleSection",
  CellTags->"AlgebraicForm"],
Cell[3469, 124, 128, 4, 34, "ExampleText",
  CellTags->"AlgebraicForm"],
Cell[3600, 130, 119, 3, 26, "Input",
  CellTags->"AlgebraicForm"],
Cell[3722, 135, 113, 3, 26, "Input",
  CellTags->"AlgebraicForm"],
Cell[3838, 140, 54, 1, 32, "ObjectName",
  CellTags->"Alphabet"],
Cell[3895, 143, 53, 1, 50, "Highlight",
  CellTags->"Alphabet"],
Cell[3951, 146, 161, 6, 31, "Usage",
  CellTags->"Alphabet"],
Cell[4115, 154, 273, 10, 61, "Notes",
  CellTags->"Alphabet"],
Cell[4391, 166, 250, 11, 24, "Notes",
  CellTags->"Alphabet"],
Cell[4644, 179, 291, 11, 40, "Notes",
  CellTags->"Alphabet"],
Cell[4938, 192, 110, 5, 24, "Notes",
  CellTags->"Alphabet"],
Cell[5051, 199, 132, 4, 24, "Notes",
  CellTags->"Alphabet"],
Cell[5186, 205, 82, 1, 24, "Notes",
  CellTags->"Alphabet"],
Cell[5271, 208, 363, 15, 24, "Notes",
  CellTags->"Alphabet"],
Cell[5637, 225, 66, 1, 39, "ExampleSection",
  CellTags->"Alphabet"],
Cell[5706, 228, 142, 4, 34, "ExampleText",
  CellTags->"Alphabet"],
Cell[5851, 234, 146, 4, 42, "Input",
  CellTags->"Alphabet"],
Cell[6000, 240, 95, 1, 34, "ExampleText",
  CellTags->"Alphabet"],
Cell[6098, 243, 188, 4, 42, "Input",
  CellTags->"Alphabet"],
Cell[6289, 249, 70, 1, 34, "ExampleText",
  CellTags->"Alphabet"],
Cell[6362, 252, 86, 2, 28, "Input",
  CellTags->"Alphabet"],
Cell[6451, 256, 58, 1, 32, "ObjectName",
  CellTags->"Alphabetic"],
Cell[6512, 259, 53, 1, 50, "Highlight",
  CellTags->"Alphabetic"],
Cell[6568, 262, 197, 8, 31, "Usage",
  CellTags->"Alphabetic"],
Cell[6768, 272, 203, 6, 24, "Notes",
  CellTags->"Alphabetic"],
Cell[6974, 280, 270, 11, 24, "Notes",
  CellTags->"Alphabetic"],
Cell[7247, 293, 56, 1, 32, "ObjectName",
  CellTags->"AlphabetQ"],
Cell[7306, 296, 54, 1, 50, "Highlight",
  CellTags->"AlphabetQ"],
Cell[7363, 299, 161, 6, 31, "Usage",
  CellTags->"AlphabetQ"],
Cell[7527, 307, 141, 4, 24, "Notes",
  CellTags->"AlphabetQ"],
Cell[7671, 313, 159, 4, 40, "Notes",
  CellTags->"AlphabetQ"],
Cell[7833, 319, 265, 9, 24, "Notes",
  CellTags->"AlphabetQ"],
Cell[8101, 330, 308, 12, 24, "Notes",
  CellTags->"AlphabetQ"],
Cell[8412, 344, 363, 15, 24, "Notes",
  CellTags->"AlphabetQ"],
Cell[8778, 361, 67, 1, 39, "ExampleSection",
  CellTags->"AlphabetQ"],
Cell[8848, 364, 70, 1, 34, "ExampleText",
  CellTags->"AlphabetQ"],
Cell[8921, 367, 133, 4, 42, "Input",
  CellTags->"AlphabetQ"],
Cell[9057, 373, 96, 1, 34, "ExampleText",
  CellTags->"AlphabetQ"],
Cell[9156, 376, 150, 4, 42, "Input",
  CellTags->"AlphabetQ"],
Cell[9309, 382, 67, 1, 34, "ExampleText",
  CellTags->"AlphabetQ"],
Cell[9379, 385, 110, 5, 45, "Input",
  CellTags->"AlphabetQ"],
Cell[9492, 392, 62, 1, 32, "ObjectName",
  CellTags->"AlphabetSize"],
Cell[9557, 395, 57, 1, 50, "Highlight",
  CellTags->"AlphabetSize"],
Cell[9617, 398, 165, 6, 31, "Usage",
  CellTags->"AlphabetSize"],
Cell[9785, 406, 73, 1, 24, "Notes",
  CellTags->"AlphabetSize"],
Cell[9861, 409, 358, 15, 24, "Notes",
  CellTags->"AlphabetSize"],
Cell[10222, 426, 70, 1, 39, "ExampleSection",
  CellTags->"AlphabetSize"],
Cell[10295, 429, 89, 1, 34, "ExampleText",
  CellTags->"AlphabetSize"],
Cell[10387, 432, 207, 8, 41, "Input",
  CellTags->"AlphabetSize"],
Cell[10597, 442, 76, 1, 34, "ExampleText",
  CellTags->"AlphabetSize"],
Cell[10676, 445, 110, 3, 26, "Input",
  CellTags->"AlphabetSize"],
Cell[10789, 450, 107, 3, 26, "Input",
  CellTags->"AlphabetSize"],
Cell[10899, 455, 60, 1, 32, "ObjectName",
  CellTags->"Concatenate"],
Cell[10962, 458, 56, 1, 50, "Highlight",
  CellTags->"Concatenate"],
Cell[11021, 461, 184, 8, 31, "Usage",
  CellTags->"Concatenate"],
Cell[11208, 471, 107, 4, 24, "Notes",
  CellTags->"Concatenate"],
Cell[11318, 477, 178, 5, 24, "Notes",
  CellTags->"Concatenate"],
Cell[11499, 484, 126, 4, 24, "Notes",
  CellTags->"Concatenate"],
Cell[11628, 490, 236, 11, 24, "Notes",
  CellTags->"Concatenate"],
Cell[11867, 503, 69, 1, 39, "ExampleSection",
  CellTags->"Concatenate"],
Cell[11939, 506, 179, 4, 50, "ExampleText",
  CellTags->"Concatenate"],
Cell[12121, 512, 135, 4, 42, "Input",
  CellTags->"Concatenate"],
Cell[12259, 518, 128, 4, 34, "ExampleText",
  CellTags->"Concatenate"],
Cell[12390, 524, 113, 3, 26, "Input",
  CellTags->"Concatenate"],
Cell[12506, 529, 112, 3, 26, "Input",
  CellTags->"Concatenate"],
Cell[12621, 534, 130, 3, 26, "Input",
  CellTags->"Concatenate"],
Cell[12754, 539, 44, 1, 32, "ObjectName",
  CellTags->"Eps"],
Cell[12801, 542, 51, 1, 50, "Highlight",
  CellTags->"Eps"],
Cell[12855, 545, 115, 4, 31, "Usage",
  CellTags->"Eps"],
Cell[12973, 551, 171, 6, 24, "Notes",
  CellTags->"Eps"],
Cell[13147, 559, 299, 11, 39, "Notes",
  CellTags->"Eps"],
Cell[13449, 572, 284, 11, 40, "Notes",
  CellTags->"Eps"],
Cell[13736, 585, 141, 7, 24, "Notes",
  CellTags->"Eps"],
Cell[13880, 594, 61, 1, 39, "ExampleSection",
  CellTags->"Eps"],
Cell[13944, 597, 214, 7, 50, "ExampleText",
  CellTags->"Eps"],
Cell[14161, 606, 102, 3, 26, "Input",
  CellTags->"Eps"],
Cell[14266, 611, 273, 8, 50, "ExampleText",
  CellTags->"Eps"],
Cell[14542, 621, 164, 5, 58, "Input",
  CellTags->"Eps"],
Cell[14709, 628, 103, 3, 26, "Input",
  CellTags->"Eps"],
Cell[14815, 633, 52, 1, 32, "ObjectName",
  CellTags->"FromEps"],
Cell[14870, 636, 52, 1, 50, "Highlight",
  CellTags->"FromEps"],
Cell[14925, 639, 232, 8, 47, "Usage",
  CellTags->"FromEps"],
Cell[15160, 649, 96, 1, 24, "Notes",
  CellTags->"FromEps"],
Cell[15259, 652, 232, 11, 24, "Notes",
  CellTags->"FromEps"],
Cell[15494, 665, 65, 1, 39, "ExampleSection",
  CellTags->"FromEps"],
Cell[15562, 668, 86, 1, 34, "ExampleText",
  CellTags->"FromEps"],
Cell[15651, 671, 243, 6, 58, "Input",
  CellTags->"FromEps"],
Cell[15897, 679, 62, 1, 32, "ObjectName",
  CellTags->"FromFrontier"],
Cell[15962, 682, 57, 1, 50, "Highlight",
  CellTags->"FromFrontier"],
Cell[16022, 685, 194, 6, 31, "Usage",
  CellTags->"FromFrontier"],
Cell[16219, 693, 228, 9, 44, "Notes",
  CellTags->"FromFrontier"],
Cell[16450, 704, 218, 8, 24, "Notes",
  CellTags->"FromFrontier"],
Cell[16671, 714, 173, 6, 24, "Notes",
  CellTags->"FromFrontier"],
Cell[16847, 722, 245, 8, 24, "Notes",
  CellTags->"FromFrontier"],
Cell[17095, 732, 264, 11, 24, "Notes",
  CellTags->"FromFrontier"],
Cell[17362, 745, 70, 1, 39, "ExampleSection",
  CellTags->"FromFrontier"],
Cell[17435, 748, 94, 1, 34, "ExampleText",
  CellTags->"FromFrontier"],
Cell[17532, 751, 167, 4, 42, "Input",
  CellTags->"FromFrontier"],
Cell[17702, 757, 96, 2, 28, "Input",
  CellTags->"FromFrontier"],
Cell[17801, 761, 87, 1, 34, "ExampleText",
  CellTags->"FromFrontier"],
Cell[17891, 764, 119, 3, 26, "Input",
  CellTags->"FromFrontier"],
Cell[18013, 769, 56, 1, 32, "ObjectName",
  CellTags->"FrontierQ"],
Cell[18072, 772, 54, 1, 50, "Highlight",
  CellTags->"FrontierQ"],
Cell[18129, 775, 218, 8, 31, "Usage",
  CellTags->"FrontierQ"],
Cell[18350, 785, 153, 5, 24, "Notes",
  CellTags->"FrontierQ"],
Cell[18506, 792, 365, 15, 24, "Notes",
  CellTags->"FrontierQ"],
Cell[18874, 809, 67, 1, 39, "ExampleSection",
  CellTags->"FrontierQ"],
Cell[18944, 812, 74, 1, 34, "ExampleText",
  CellTags->"FrontierQ"],
Cell[19021, 815, 128, 3, 26, "Input",
  CellTags->"FrontierQ"],
Cell[19152, 820, 90, 1, 34, "ExampleText",
  CellTags->"FrontierQ"],
Cell[19245, 823, 128, 3, 26, "Input",
  CellTags->"FrontierQ"],
Cell[19376, 828, 62, 1, 32, "ObjectName",
  CellTags->"MakeAlphabet"],
Cell[19441, 831, 57, 1, 50, "Highlight",
  CellTags->"MakeAlphabet"],
Cell[19501, 834, 259, 10, 31, "Usage",
  CellTags->"MakeAlphabet"],
Cell[19763, 846, 228, 9, 44, "Notes",
  CellTags->"MakeAlphabet"],
Cell[19994, 857, 255, 11, 24, "Notes",
  CellTags->"MakeAlphabet"],
Cell[20252, 870, 170, 5, 24, "Notes",
  CellTags->"MakeAlphabet"],
Cell[20425, 877, 263, 11, 24, "Notes",
  CellTags->"MakeAlphabet"],
Cell[20691, 890, 70, 1, 39, "ExampleSection",
  CellTags->"MakeAlphabet"],
Cell[20764, 893, 89, 1, 34, "ExampleText",
  CellTags->"MakeAlphabet"],
Cell[20856, 896, 122, 6, 62, "Input",
  CellTags->"MakeAlphabet"],
Cell[20981, 904, 125, 3, 26, "Input",
  CellTags->"MakeAlphabet"],
Cell[21109, 909, 68, 1, 34, "ExampleText",
  CellTags->"MakeAlphabet"],
Cell[21180, 912, 108, 5, 45, "Input",
  CellTags->"MakeAlphabet"],
Cell[21291, 919, 54, 1, 32, "ObjectName",
  CellTags->"MakeWord"],
Cell[21348, 922, 53, 1, 50, "Highlight",
  CellTags->"MakeWord"],
Cell[21404, 925, 311, 12, 47, "Usage",
  CellTags->"MakeWord"],
Cell[21718, 939, 65, 1, 24, "Notes",
  CellTags->"MakeWord"],
Cell[21786, 942, 78, 1, 24, "Notes",
  CellTags->"MakeWord"],
Cell[21867, 945, 159, 7, 24, "Notes",
  CellTags->"MakeWord"],
Cell[22029, 954, 66, 1, 39, "ExampleSection",
  CellTags->"MakeWord"],
Cell[22098, 957, 68, 1, 34, "ExampleText",
  CellTags->"MakeWord"],
Cell[22169, 960, 113, 5, 45, "Input",
  CellTags->"MakeWord"],
Cell[22285, 967, 93, 3, 26, "Input",
  CellTags->"MakeWord"],
Cell[22381, 972, 62, 1, 32, "ObjectName",
  CellTags->"ParikhVector"],
Cell[22446, 975, 57, 1, 50, "Highlight",
  CellTags->"ParikhVector"],
Cell[22506, 978, 223, 8, 31, "Usage",
  CellTags->"ParikhVector"],
Cell[22732, 988, 265, 10, 24, "Notes",
  CellTags->"ParikhVector"],
Cell[23000, 1000, 75, 1, 24, "Notes",
  CellTags->"ParikhVector"],
Cell[23078, 1003, 70, 1, 39, "ExampleSection",
  CellTags->"ParikhVector"],
Cell[23151, 1006, 143, 4, 34, "ExampleText",
  CellTags->"ParikhVector"],
Cell[23297, 1012, 98, 2, 28, "Input",
  CellTags->"ParikhVector"],
Cell[23398, 1016, 136, 4, 34, "ExampleText",
  CellTags->"ParikhVector"],
Cell[23537, 1022, 203, 5, 42, "Input",
  CellTags->"ParikhVector"],
Cell[23743, 1029, 58, 1, 32, "ObjectName",
  CellTags->"RandomWord"],
Cell[23804, 1032, 55, 1, 50, "Highlight",
  CellTags->"RandomWord"],
Cell[23862, 1035, 150, 4, 31, "Usage",
  CellTags->"RandomWord"],
Cell[24015, 1041, 101, 1, 24, "Notes",
  CellTags->"RandomWord"],
Cell[24119, 1044, 155, 7, 24, "Notes",
  CellTags->"RandomWord"],
Cell[24277, 1053, 68, 1, 39, "ExampleSection",
  CellTags->"RandomWord"],
Cell[24348, 1056, 62, 1, 34, "ExampleText",
  CellTags->"RandomWord"],
Cell[24413, 1059, 85, 2, 28, "Input",
  CellTags->"RandomWord"],
Cell[24501, 1063, 86, 2, 28, "Input",
  CellTags->"RandomWord"],
Cell[24590, 1067, 54, 1, 32, "ObjectName",
  CellTags->"ToBlocks"],
Cell[24647, 1070, 53, 1, 50, "Highlight",
  CellTags->"ToBlocks"],
Cell[24703, 1073, 232, 8, 31, "Usage",
  CellTags->"ToBlocks"],
Cell[24938, 1083, 470, 17, 44, "Notes",
  CellTags->"ToBlocks"],
Cell[25411, 1102, 132, 5, 24, "Notes",
  CellTags->"ToBlocks"],
Cell[25546, 1109, 253, 11, 24, "Notes",
  CellTags->"ToBlocks"],
Cell[25802, 1122, 161, 5, 24, "Notes",
  CellTags->"ToBlocks"],
Cell[25966, 1129, 188, 7, 24, "Notes",
  CellTags->"ToBlocks"],
Cell[26157, 1138, 66, 1, 39, "ExampleSection",
  CellTags->"ToBlocks"],
Cell[26226, 1141, 75, 1, 34, "ExampleText",
  CellTags->"ToBlocks"],
Cell[26304, 1144, 103, 3, 26, "Input",
  CellTags->"ToBlocks"],
Cell[26410, 1149, 127, 3, 26, "Input",
  CellTags->"ToBlocks"],
Cell[26540, 1154, 67, 1, 34, "ExampleText",
  CellTags->"ToBlocks"],
Cell[26610, 1157, 132, 3, 26, "Input",
  CellTags->"ToBlocks"],
Cell[26745, 1162, 48, 1, 32, "ObjectName",
  CellTags->"ToEps"],
Cell[26796, 1165, 50, 1, 50, "Highlight",
  CellTags->"ToEps"],
Cell[26849, 1168, 212, 8, 31, "Usage",
  CellTags->"ToEps"],
Cell[27064, 1178, 98, 1, 24, "Notes",
  CellTags->"ToEps"],
Cell[27165, 1181, 152, 7, 24, "Notes",
  CellTags->"ToEps"],
Cell[27320, 1190, 63, 1, 39, "ExampleSection",
  CellTags->"ToEps"],
Cell[27386, 1193, 125, 4, 34, "ExampleText",
  CellTags->"ToEps"],
Cell[27514, 1199, 98, 3, 26, "Input",
  CellTags->"ToEps"],
Cell[27615, 1204, 58, 1, 32, "ObjectName",
  CellTags->"ToFrontier"],
Cell[27676, 1207, 55, 1, 50, "Highlight",
  CellTags->"ToFrontier"],
Cell[27734, 1210, 234, 8, 47, "Usage",
  CellTags->"ToFrontier"],
Cell[27971, 1220, 158, 6, 24, "Notes",
  CellTags->"ToFrontier"],
Cell[28132, 1228, 156, 5, 24, "Notes",
  CellTags->"ToFrontier"],
Cell[28291, 1235, 266, 11, 24, "Notes",
  CellTags->"ToFrontier"],
Cell[28560, 1248, 68, 1, 39, "ExampleSection",
  CellTags->"ToFrontier"],
Cell[28631, 1251, 92, 1, 34, "ExampleText",
  CellTags->"ToFrontier"],
Cell[28726, 1254, 173, 4, 42, "Input",
  CellTags->"ToFrontier"],
Cell[28902, 1260, 82, 2, 28, "Input",
  CellTags->"ToFrontier"],
Cell[28987, 1264, 52, 1, 32, "ObjectName",
  CellTags->"ToIndex"],
Cell[29042, 1267, 52, 1, 50, "Highlight",
  CellTags->"ToIndex"],
Cell[29097, 1270, 214, 8, 31, "Usage",
  CellTags->"ToIndex"],
Cell[29314, 1280, 91, 1, 24, "Notes",
  CellTags->"ToIndex"],
Cell[29408, 1283, 90, 1, 24, "Notes",
  CellTags->"ToIndex"],
Cell[29501, 1286, 152, 7, 24, "Notes",
  CellTags->"ToIndex"],
Cell[29656, 1295, 65, 1, 39, "ExampleSection",
  CellTags->"ToIndex"],
Cell[29724, 1298, 79, 1, 34, "ExampleText",
  CellTags->"ToIndex"],
Cell[29806, 1301, 85, 2, 28, "Input",
  CellTags->"ToIndex"],
Cell[29894, 1305, 100, 3, 26, "Input",
  CellTags->"ToIndex"],
Cell[29997, 1310, 50, 1, 32, "ObjectName",
  CellTags->"ToWord"],
Cell[30050, 1313, 51, 1, 50, "Highlight",
  CellTags->"ToWord"],
Cell[30104, 1316, 196, 7, 31, "Usage",
  CellTags->"ToWord"],
Cell[30303, 1325, 157, 6, 24, "Notes",
  CellTags->"ToWord"],
Cell[30463, 1333, 112, 4, 24, "Notes",
  CellTags->"ToWord"],
Cell[30578, 1339, 126, 6, 24, "Notes",
  CellTags->"ToWord"],
Cell[30707, 1347, 64, 1, 39, "ExampleSection",
  CellTags->"ToWord"],
Cell[30774, 1350, 85, 1, 34, "ExampleText",
  CellTags->"ToWord"],
Cell[30862, 1353, 83, 2, 28, "Input",
  CellTags->"ToWord"],
Cell[30948, 1357, 96, 2, 28, "Input",
  CellTags->"ToWord"],
Cell[31047, 1361, 66, 1, 34, "ExampleText",
  CellTags->"ToWord"],
Cell[31116, 1364, 110, 3, 26, "Input",
  CellTags->"ToWord"],
Cell[31229, 1369, 62, 1, 32, "ObjectName",
  CellTags->"WordBinomial"],
Cell[31294, 1372, 57, 1, 50, "Highlight",
  CellTags->"WordBinomial"],
Cell[31354, 1375, 242, 9, 47, "Usage",
  CellTags->"WordBinomial"],
Cell[31599, 1386, 231, 9, 44, "Notes",
  CellTags->"WordBinomial"],
Cell[31833, 1397, 207, 9, 24, "Notes",
  CellTags->"WordBinomial"],
Cell[32043, 1408, 151, 4, 24, "Notes",
  CellTags->"WordBinomial"],
Cell[32197, 1414, 194, 7, 24, "Notes",
  CellTags->"WordBinomial"],
Cell[32394, 1423, 70, 1, 39, "ExampleSection",
  CellTags->"WordBinomial"],
Cell[32467, 1426, 154, 4, 34, "ExampleText",
  CellTags->"WordBinomial"],
Cell[32624, 1432, 142, 4, 42, "Input",
  CellTags->"WordBinomial"],
Cell[32769, 1438, 99, 1, 34, "ExampleText",
  CellTags->"WordBinomial"],
Cell[32871, 1441, 202, 5, 42, "Input",
  CellTags->"WordBinomial"],
Cell[33076, 1448, 66, 1, 32, "ObjectName",
  CellTags->"WordConjugateQ"],
Cell[33145, 1451, 59, 1, 50, "Highlight",
  CellTags->"WordConjugateQ"],
Cell[33207, 1454, 205, 8, 31, "Usage",
  CellTags->"WordConjugateQ"],
Cell[33415, 1464, 114, 5, 24, "Notes",
  CellTags->"WordConjugateQ"],
Cell[33532, 1471, 186, 6, 24, "Notes",
  CellTags->"WordConjugateQ"],
Cell[33721, 1479, 168, 7, 24, "Notes",
  CellTags->"WordConjugateQ"],
Cell[33892, 1488, 72, 1, 39, "ExampleSection",
  CellTags->"WordConjugateQ"],
Cell[33967, 1491, 89, 1, 34, "ExampleText",
  CellTags->"WordConjugateQ"],
Cell[34059, 1494, 100, 2, 28, "Input",
  CellTags->"WordConjugateQ"],
Cell[34162, 1498, 100, 2, 28, "Input",
  CellTags->"WordConjugateQ"],
Cell[34265, 1502, 118, 2, 28, "Input",
  CellTags->"WordConjugateQ"],
Cell[34386, 1506, 66, 1, 32, "ObjectName",
  CellTags->"WordConjugates"],
Cell[34455, 1509, 59, 1, 50, "Highlight",
  CellTags->"WordConjugates"],
Cell[34517, 1512, 193, 6, 31, "Usage",
  CellTags->"WordConjugates"],
Cell[34713, 1520, 114, 5, 24, "Notes",
  CellTags->"WordConjugates"],
Cell[34830, 1527, 141, 6, 24, "Notes",
  CellTags->"WordConjugates"],
Cell[34974, 1535, 72, 1, 39, "ExampleSection",
  CellTags->"WordConjugates"],
Cell[35049, 1538, 283, 10, 50, "ExampleText",
  CellTags->"WordConjugates"],
Cell[35335, 1550, 96, 2, 28, "Input",
  CellTags->"WordConjugates"],
Cell[35434, 1554, 95, 2, 28, "Input",
  CellTags->"WordConjugates"],
Cell[35532, 1558, 74, 1, 32, "ObjectName",
  CellTags->"WordFactorizations"],
Cell[35609, 1561, 63, 1, 50, "Highlight",
  CellTags->"WordFactorizations"],
Cell[35675, 1564, 213, 8, 31, "Usage",
  CellTags->"WordFactorizations"],
Cell[35891, 1574, 215, 8, 24, "Notes",
  CellTags->"WordFactorizations"],
Cell[36109, 1584, 214, 9, 24, "Notes",
  CellTags->"WordFactorizations"],
Cell[36326, 1595, 76, 1, 39, "ExampleSection",
  CellTags->"WordFactorizations"],
Cell[36405, 1598, 205, 6, 34, "ExampleText",
  CellTags->"WordFactorizations"],
Cell[36613, 1606, 103, 2, 28, "Input",
  CellTags->"WordFactorizations"],
Cell[36719, 1610, 125, 3, 26, "Input",
  CellTags->"WordFactorizations"],
Cell[36847, 1615, 60, 1, 32, "ObjectName",
  CellTags->"WordFactorQ"],
Cell[36910, 1618, 56, 1, 50, "Highlight",
  CellTags->"WordFactorQ"],
Cell[36969, 1621, 210, 8, 31, "Usage",
  CellTags->"WordFactorQ"],
Cell[37182, 1631, 109, 4, 24, "Notes",
  CellTags->"WordFactorQ"],
Cell[37294, 1637, 423, 18, 24, "Notes",
  CellTags->"WordFactorQ"],
Cell[37720, 1657, 69, 1, 39, "ExampleSection",
  CellTags->"WordFactorQ"],
Cell[37792, 1660, 151, 4, 50, "ExampleText",
  CellTags->"WordFactorQ"],
Cell[37946, 1666, 105, 3, 26, "Input",
  CellTags->"WordFactorQ"],
Cell[38054, 1671, 105, 3, 26, "Input",
  CellTags->"WordFactorQ"],
Cell[38162, 1676, 135, 3, 26, "Input",
  CellTags->"WordFactorQ"],
Cell[38300, 1681, 60, 1, 32, "ObjectName",
  CellTags->"WordFactors"],
Cell[38363, 1684, 56, 1, 50, "Highlight",
  CellTags->"WordFactors"],
Cell[38422, 1687, 195, 6, 31, "Usage",
  CellTags->"WordFactors"],
Cell[38620, 1695, 226, 9, 44, "Notes",
  CellTags->"WordFactors"],
Cell[38849, 1706, 169, 7, 24, "Notes",
  CellTags->"WordFactors"],
Cell[39021, 1715, 111, 5, 24, "Notes",
  CellTags->"WordFactors"],
Cell[39135, 1722, 210, 8, 24, "Notes",
  CellTags->"WordFactors"],
Cell[39348, 1732, 476, 19, 24, "Notes",
  CellTags->"WordFactors"],
Cell[39827, 1753, 69, 1, 39, "ExampleSection",
  CellTags->"WordFactors"],
Cell[39899, 1756, 163, 6, 34, "ExampleText",
  CellTags->"WordFactors"],
Cell[40065, 1764, 90, 2, 28, "Input",
  CellTags->"WordFactors"],
Cell[40158, 1768, 90, 2, 28, "Input",
  CellTags->"WordFactors"],
Cell[40251, 1772, 80, 1, 34, "ExampleText",
  CellTags->"WordFactors"],
Cell[40334, 1775, 92, 2, 28, "Input",
  CellTags->"WordFactors"],
Cell[40429, 1779, 58, 1, 32, "ObjectName",
  CellTags->"WordFusion"],
Cell[40490, 1782, 55, 1, 50, "Highlight",
  CellTags->"WordFusion"],
Cell[40548, 1785, 223, 10, 31, "Usage",
  CellTags->"WordFusion"],
Cell[40774, 1797, 150, 5, 24, "Notes",
  CellTags->"WordFusion"],
Cell[40927, 1804, 195, 8, 24, "Notes",
  CellTags->"WordFusion"],
Cell[41125, 1814, 68, 1, 39, "ExampleSection",
  CellTags->"WordFusion"],
Cell[41196, 1817, 220, 9, 34, "ExampleText",
  CellTags->"WordFusion"],
Cell[41419, 1828, 96, 2, 28, "Input",
  CellTags->"WordFusion"],
Cell[41518, 1832, 96, 2, 28, "Input",
  CellTags->"WordFusion"],
Cell[41617, 1836, 58, 1, 32, "ObjectName",
  CellTags->"WordLength"],
Cell[41678, 1839, 55, 1, 50, "Highlight",
  CellTags->"WordLength"],
Cell[41736, 1842, 148, 5, 31, "Usage",
  CellTags->"WordLength"],
Cell[41887, 1849, 110, 5, 24, "Notes",
  CellTags->"WordLength"],
Cell[42000, 1856, 233, 7, 40, "Notes",
  CellTags->"WordLength"],
Cell[42236, 1865, 260, 10, 24, "Notes",
  CellTags->"WordLength"],
Cell[42499, 1877, 135, 6, 24, "Notes",
  CellTags->"WordLength"],
Cell[42637, 1885, 68, 1, 39, "ExampleSection",
  CellTags->"WordLength"],
Cell[42708, 1888, 126, 4, 34, "ExampleText",
  CellTags->"WordLength"],
Cell[42837, 1894, 99, 5, 45, "Input",
  CellTags->"WordLength"],
Cell[42939, 1901, 101, 5, 45, "Input",
  CellTags->"WordLength"],
Cell[43043, 1908, 56, 1, 32, "ObjectName",
  CellTags->"WordOrder"],
Cell[43102, 1911, 54, 1, 50, "Highlight",
  CellTags->"WordOrder"],
Cell[43159, 1914, 230, 8, 31, "Usage",
  CellTags->"WordOrder"],
Cell[43392, 1924, 257, 8, 40, "Notes",
  CellTags->"WordOrder"],
Cell[43652, 1934, 444, 19, 40, "Notes",
  CellTags->"WordOrder"],
Cell[44099, 1955, 130, 6, 24, "Notes",
  CellTags->"WordOrder"],
Cell[44232, 1963, 67, 1, 39, "ExampleSection",
  CellTags->"WordOrder"],
Cell[44302, 1966, 98, 1, 34, "ExampleText",
  CellTags->"WordOrder"],
Cell[44403, 1969, 80, 2, 28, "Input",
  CellTags->"WordOrder"],
Cell[44486, 1973, 153, 4, 26, "Input",
  CellTags->"WordOrder"],
Cell[44642, 1979, 58, 1, 32, "ObjectName",
  CellTags->"WordPeriod"],
Cell[44703, 1982, 55, 1, 50, "Highlight",
  CellTags->"WordPeriod"],
Cell[44761, 1985, 163, 6, 31, "Usage",
  CellTags->"WordPeriod"],
Cell[44927, 1993, 226, 9, 44, "Notes",
  CellTags->"WordPeriod"],
Cell[45156, 2004, 110, 5, 24, "Notes",
  CellTags->"WordPeriod"],
Cell[45269, 2011, 218, 8, 24, "Notes",
  CellTags->"WordPeriod"],
Cell[45490, 2021, 393, 13, 24, "Notes",
  CellTags->"WordPeriod"],
Cell[45886, 2036, 361, 15, 24, "Notes",
  CellTags->"WordPeriod"],
Cell[46250, 2053, 68, 1, 39, "ExampleSection",
  CellTags->"WordPeriod"],
Cell[46321, 2056, 145, 4, 34, "ExampleText",
  CellTags->"WordPeriod"],
Cell[46469, 2062, 211, 4, 42, "Input",
  CellTags->"WordPeriod"],
Cell[46683, 2068, 137, 4, 34, "ExampleText",
  CellTags->"WordPeriod"],
Cell[46823, 2074, 118, 3, 26, "Input",
  CellTags->"WordPeriod"],
Cell[46944, 2079, 132, 3, 26, "Input",
  CellTags->"WordPeriod"],
Cell[47079, 2084, 71, 1, 34, "ExampleText",
  CellTags->"WordPeriod"],
Cell[47153, 2087, 130, 3, 26, "Input",
  CellTags->"WordPeriod"],
Cell[47286, 2092, 56, 1, 32, "ObjectName",
  CellTags->"WordPower"],
Cell[47345, 2095, 54, 1, 50, "Highlight",
  CellTags->"WordPower"],
Cell[47402, 2098, 184, 8, 31, "Usage",
  CellTags->"WordPower"],
Cell[47589, 2108, 109, 5, 24, "Notes",
  CellTags->"WordPower"],
Cell[47701, 2115, 284, 11, 24, "Notes",
  CellTags->"WordPower"],
Cell[47988, 2128, 67, 1, 39, "ExampleSection",
  CellTags->"WordPower"],
Cell[48058, 2131, 123, 4, 34, "ExampleText",
  CellTags->"WordPower"],
Cell[48184, 2137, 142, 3, 26, "Input",
  CellTags->"WordPower"],
Cell[48329, 2142, 111, 3, 26, "Input",
  CellTags->"WordPower"],
Cell[48443, 2147, 111, 3, 26, "Input",
  CellTags->"WordPower"],
Cell[48557, 2152, 105, 3, 26, "Input",
  CellTags->"WordPower"],
Cell[48665, 2157, 60, 1, 32, "ObjectName",
  CellTags->"WordPrefixQ"],
Cell[48728, 2160, 56, 1, 50, "Highlight",
  CellTags->"WordPrefixQ"],
Cell[48787, 2163, 199, 8, 31, "Usage",
  CellTags->"WordPrefixQ"],
Cell[48989, 2173, 109, 4, 24, "Notes",
  CellTags->"WordPrefixQ"],
Cell[49101, 2179, 350, 15, 24, "Notes",
  CellTags->"WordPrefixQ"],
Cell[49454, 2196, 69, 1, 39, "ExampleSection",
  CellTags->"WordPrefixQ"],
Cell[49526, 2199, 77, 1, 34, "ExampleText",
  CellTags->"WordPrefixQ"],
Cell[49606, 2202, 105, 3, 26, "Input",
  CellTags->"WordPrefixQ"],
Cell[49714, 2207, 105, 3, 26, "Input",
  CellTags->"WordPrefixQ"],
Cell[49822, 2212, 114, 3, 26, "Input",
  CellTags->"WordPrefixQ"],
Cell[49939, 2217, 136, 3, 26, "Input",
  CellTags->"WordPrefixQ"],
Cell[50078, 2222, 62, 1, 32, "ObjectName",
  CellTags->"WordPrefixes"],
Cell[50143, 2225, 57, 1, 50, "Highlight",
  CellTags->"WordPrefixes"],
Cell[50203, 2228, 180, 6, 31, "Usage",
  CellTags->"WordPrefixes"],
Cell[50386, 2236, 227, 9, 44, "Notes",
  CellTags->"WordPrefixes"],
Cell[50616, 2247, 171, 7, 24, "Notes",
  CellTags->"WordPrefixes"],
Cell[50790, 2256, 112, 5, 24, "Notes",
  CellTags->"WordPrefixes"],
Cell[50905, 2263, 463, 19, 24, "Notes",
  CellTags->"WordPrefixes"],
Cell[51371, 2284, 70, 1, 39, "ExampleSection",
  CellTags->"WordPrefixes"],
Cell[51444, 2287, 173, 6, 34, "ExampleText",
  CellTags->"WordPrefixes"],
Cell[51620, 2295, 92, 2, 28, "Input",
  CellTags->"WordPrefixes"],
Cell[51715, 2299, 95, 2, 28, "Input",
  CellTags->"WordPrefixes"],
Cell[51813, 2303, 72, 1, 34, "ExampleText",
  CellTags->"WordPrefixes"],
Cell[51888, 2306, 125, 3, 26, "Input",
  CellTags->"WordPrefixes"],
Cell[52016, 2311, 74, 1, 32, "ObjectName",
  CellTags->"WordPrimitiveCount"],
Cell[52093, 2314, 63, 1, 50, "Highlight",
  CellTags->"WordPrimitiveCount"],
Cell[52159, 2317, 246, 8, 47, "Usage",
  CellTags->"WordPrimitiveCount"],
Cell[52408, 2327, 118, 5, 24, "Notes",
  CellTags->"WordPrimitiveCount"],
Cell[52529, 2334, 160, 6, 24, "Notes",
  CellTags->"WordPrimitiveCount"],
Cell[52692, 2342, 371, 15, 24, "Notes",
  CellTags->"WordPrimitiveCount"],
Cell[53066, 2359, 76, 1, 39, "ExampleSection",
  CellTags->"WordPrimitiveCount"],
Cell[53145, 2362, 82, 1, 34, "ExampleText",
  CellTags->"WordPrimitiveCount"],
Cell[53230, 2365, 135, 2, 28, "Input",
  CellTags->"WordPrimitiveCount"],
Cell[53368, 2369, 66, 1, 32, "ObjectName",
  CellTags->"WordPrimitiveQ"],
Cell[53437, 2372, 59, 1, 50, "Highlight",
  CellTags->"WordPrimitiveQ"],
Cell[53499, 2375, 205, 6, 31, "Usage",
  CellTags->"WordPrimitiveQ"],
Cell[53707, 2383, 114, 5, 24, "Notes",
  CellTags->"WordPrimitiveQ"],
Cell[53824, 2390, 168, 6, 24, "Notes",
  CellTags->"WordPrimitiveQ"],
Cell[53995, 2398, 373, 15, 24, "Notes",
  CellTags->"WordPrimitiveQ"],
Cell[54371, 2415, 72, 1, 39, "ExampleSection",
  CellTags->"WordPrimitiveQ"],
Cell[54446, 2418, 137, 4, 34, "ExampleText",
  CellTags->"WordPrimitiveQ"],
Cell[54586, 2424, 168, 4, 42, "Input",
  CellTags->"WordPrimitiveQ"],
Cell[54757, 2430, 114, 2, 34, "ExampleText",
  CellTags->"WordPrimitiveQ"],
Cell[54874, 2434, 145, 3, 26, "Input",
  CellTags->"WordPrimitiveQ"],
Cell[55022, 2439, 141, 4, 34, "ExampleText",
  CellTags->"WordPrimitiveQ"],
Cell[55166, 2445, 134, 3, 26, "Input",
  CellTags->"WordPrimitiveQ"],
Cell[55303, 2450, 48, 1, 32, "ObjectName",
  CellTags->"WordQ"],
Cell[55354, 2453, 50, 1, 50, "Highlight",
  CellTags->"WordQ"],
Cell[55407, 2456, 143, 4, 31, "Usage",
  CellTags->"WordQ"],
Cell[55553, 2462, 67, 1, 24, "Notes",
  CellTags->"WordQ"],
Cell[55623, 2465, 105, 5, 24, "Notes",
  CellTags->"WordQ"],
Cell[55731, 2472, 1945, 79, 72, "Notes",
  CellTags->"WordQ"],
Cell[57679, 2553, 63, 1, 39, "ExampleSection",
  CellTags->"WordQ"],
Cell[57745, 2556, 72, 1, 34, "ExampleText",
  CellTags->"WordQ"],
Cell[57820, 2559, 131, 3, 26, "Input",
  CellTags->"WordQ"],
Cell[57954, 2564, 60, 1, 32, "ObjectName",
  CellTags->"WordReverse"],
Cell[58017, 2567, 56, 1, 50, "Highlight",
  CellTags->"WordReverse"],
Cell[58076, 2570, 257, 8, 47, "Usage",
  CellTags->"WordReverse"],
Cell[58336, 2580, 111, 5, 24, "Notes",
  CellTags->"WordReverse"],
Cell[58450, 2587, 78, 1, 24, "Notes",
  CellTags->"WordReverse"],
Cell[58531, 2590, 195, 9, 24, "Notes",
  CellTags->"WordReverse"],
Cell[58729, 2601, 69, 1, 39, "ExampleSection",
  CellTags->"WordReverse"],
Cell[58801, 2604, 103, 1, 34, "ExampleText",
  CellTags->"WordReverse"],
Cell[58907, 2607, 89, 2, 28, "Input",
  CellTags->"WordReverse"],
Cell[58999, 2611, 73, 1, 34, "ExampleText",
  CellTags->"WordReverse"],
Cell[59075, 2614, 117, 3, 26, "Input",
  CellTags->"WordReverse"],
Cell[59195, 2619, 81, 1, 34, "ExampleText",
  CellTags->"WordReverse"],
Cell[59279, 2622, 137, 3, 26, "Input",
  CellTags->"WordReverse"],
Cell[59419, 2627, 54, 1, 32, "ObjectName",
  CellTags->"WordRoot"],
Cell[59476, 2630, 53, 1, 50, "Highlight",
  CellTags->"WordRoot"],
Cell[59532, 2633, 289, 12, 31, "Usage",
  CellTags->"WordRoot"],
Cell[59824, 2647, 108, 5, 24, "Notes",
  CellTags->"WordRoot"],
Cell[59935, 2654, 378, 15, 24, "Notes",
  CellTags->"WordRoot"],
Cell[60316, 2671, 66, 1, 39, "ExampleSection",
  CellTags->"WordRoot"],
Cell[60385, 2674, 92, 1, 34, "ExampleText",
  CellTags->"WordRoot"],
Cell[60480, 2677, 107, 2, 28, "Input",
  CellTags->"WordRoot"],
Cell[60590, 2681, 94, 3, 26, "Input",
  CellTags->"WordRoot"],
Cell[60687, 2686, 58, 1, 32, "ObjectName",
  CellTags->"WordRotate"],
Cell[60748, 2689, 55, 1, 50, "Highlight",
  CellTags->"WordRotate"],
Cell[60806, 2692, 199, 8, 31, "Usage",
  CellTags->"WordRotate"],
Cell[61008, 2702, 110, 5, 24, "Notes",
  CellTags->"WordRotate"],
Cell[61121, 2709, 97, 4, 24, "Notes",
  CellTags->"WordRotate"],
Cell[61221, 2715, 68, 1, 24, "Notes",
  CellTags->"WordRotate"],
Cell[61292, 2718, 278, 11, 24, "Notes",
  CellTags->"WordRotate"],
Cell[61573, 2731, 68, 1, 39, "ExampleSection",
  CellTags->"WordRotate"],
Cell[61644, 2734, 73, 1, 34, "ExampleText",
  CellTags->"WordRotate"],
Cell[61720, 2737, 104, 2, 28, "Input",
  CellTags->"WordRotate"],
Cell[61827, 2741, 109, 3, 26, "Input",
  CellTags->"WordRotate"],
Cell[61939, 2746, 48, 1, 32, "ObjectName",
  CellTags->"Words"],
Cell[61990, 2749, 50, 1, 50, "Highlight",
  CellTags->"Words"],
Cell[62043, 2752, 198, 8, 31, "Usage",
  CellTags->"Words"],
Cell[62244, 2762, 220, 9, 44, "Notes",
  CellTags->"Words"],
Cell[62467, 2773, 263, 11, 24, "Notes",
  CellTags->"Words"],
Cell[62733, 2786, 166, 7, 24, "Notes",
  CellTags->"Words"],
Cell[62902, 2795, 84, 1, 24, "Notes",
  CellTags->"Words"],
Cell[62989, 2798, 174, 8, 24, "Notes",
  CellTags->"Words"],
Cell[63166, 2808, 123, 6, 24, "Notes",
  CellTags->"Words"],
Cell[63292, 2816, 63, 1, 39, "ExampleSection",
  CellTags->"Words"],
Cell[63358, 2819, 89, 1, 34, "ExampleText",
  CellTags->"Words"],
Cell[63450, 2822, 73, 2, 28, "Input",
  CellTags->"Words"],
Cell[63526, 2826, 147, 4, 42, "Input",
  CellTags->"Words"],
Cell[63676, 2832, 91, 3, 26, "Input",
  CellTags->"Words"],
Cell[63770, 2837, 108, 3, 26, "Input",
  CellTags->"Words"],
Cell[63881, 2842, 214, 6, 50, "ExampleText",
  CellTags->"Words"],
Cell[64098, 2850, 114, 3, 26, "Input",
  CellTags->"Words"],
Cell[64215, 2855, 60, 1, 32, "ObjectName",
  CellTags->"WordShuffle"],
Cell[64278, 2858, 56, 1, 50, "Highlight",
  CellTags->"WordShuffle"],
Cell[64337, 2861, 204, 8, 31, "Usage",
  CellTags->"WordShuffle"],
Cell[64544, 2871, 81, 1, 24, "Notes",
  CellTags->"WordShuffle"],
Cell[64628, 2874, 141, 6, 24, "Notes",
  CellTags->"WordShuffle"],
Cell[64772, 2882, 69, 1, 39, "ExampleSection",
  CellTags->"WordShuffle"],
Cell[64844, 2885, 172, 4, 50, "ExampleText",
  CellTags->"WordShuffle"],
Cell[65019, 2891, 92, 2, 28, "Input",
  CellTags->"WordShuffle"],
Cell[65114, 2895, 93, 2, 28, "Input",
  CellTags->"WordShuffle"],
Cell[65210, 2899, 161, 4, 34, "ExampleText",
  CellTags->"WordShuffle"],
Cell[65374, 2905, 96, 2, 28, "Input",
  CellTags->"WordShuffle"],
Cell[65473, 2909, 100, 2, 28, "Input",
  CellTags->"WordShuffle"],
Cell[65576, 2913, 123, 4, 34, "ExampleText",
  CellTags->"WordShuffle"],
Cell[65702, 2919, 138, 3, 26, "Input",
  CellTags->"WordShuffle"],
Cell[65843, 2924, 54, 1, 32, "ObjectName",
  CellTags->"WordSort"],
Cell[65900, 2927, 53, 1, 50, "Highlight",
  CellTags->"WordSort"],
Cell[65956, 2930, 206, 6, 31, "Usage",
  CellTags->"WordSort"],
Cell[66165, 2938, 526, 13, 24, "Notes",
  CellTags->"WordSort"],
Cell[66694, 2953, 136, 4, 24, "Notes",
  CellTags->"WordSort"],
Cell[66833, 2959, 159, 7, 24, "Notes",
  CellTags->"WordSort"],
Cell[66995, 2968, 66, 1, 39, "ExampleSection",
  CellTags->"WordSort"],
Cell[67064, 2971, 147, 4, 34, "ExampleText",
  CellTags->"WordSort"],
Cell[67214, 2977, 84, 2, 28, "Input",
  CellTags->"WordSort"],
Cell[67301, 2981, 94, 3, 26, "Input",
  CellTags->"WordSort"],
Cell[67398, 2986, 60, 1, 32, "ObjectName",
  CellTags->"WordSuffixQ"],
Cell[67461, 2989, 56, 1, 50, "Highlight",
  CellTags->"WordSuffixQ"],
Cell[67520, 2992, 199, 8, 31, "Usage",
  CellTags->"WordSuffixQ"],
Cell[67722, 3002, 109, 4, 24, "Notes",
  CellTags->"WordSuffixQ"],
Cell[67834, 3008, 350, 15, 24, "Notes",
  CellTags->"WordSuffixQ"],
Cell[68187, 3025, 69, 1, 39, "ExampleSection",
  CellTags->"WordSuffixQ"],
Cell[68259, 3028, 77, 1, 34, "ExampleText",
  CellTags->"WordSuffixQ"],
Cell[68339, 3031, 105, 3, 26, "Input",
  CellTags->"WordSuffixQ"],
Cell[68447, 3036, 105, 3, 26, "Input",
  CellTags->"WordSuffixQ"],
Cell[68555, 3041, 114, 3, 26, "Input",
  CellTags->"WordSuffixQ"],
Cell[68672, 3046, 136, 3, 26, "Input",
  CellTags->"WordSuffixQ"],
Cell[68811, 3051, 62, 1, 32, "ObjectName",
  CellTags->"WordSuffixes"],
Cell[68876, 3054, 57, 1, 50, "Highlight",
  CellTags->"WordSuffixes"],
Cell[68936, 3057, 179, 6, 31, "Usage",
  CellTags->"WordSuffixes"],
Cell[69118, 3065, 227, 9, 44, "Notes",
  CellTags->"WordSuffixes"],
Cell[69348, 3076, 171, 7, 24, "Notes",
  CellTags->"WordSuffixes"],
Cell[69522, 3085, 112, 5, 24, "Notes",
  CellTags->"WordSuffixes"],
Cell[69637, 3092, 363, 15, 24, "Notes",
  CellTags->"WordSuffixes"],
Cell[70003, 3109, 70, 1, 39, "ExampleSection",
  CellTags->"WordSuffixes"],
Cell[70076, 3112, 173, 6, 34, "ExampleText",
  CellTags->"WordSuffixes"],
Cell[70252, 3120, 92, 2, 28, "Input",
  CellTags->"WordSuffixes"],
Cell[70347, 3124, 95, 2, 28, "Input",
  CellTags->"WordSuffixes"],
Cell[70445, 3128, 72, 1, 34, "ExampleText",
  CellTags->"WordSuffixes"],
Cell[70520, 3131, 125, 3, 26, "Input",
  CellTags->"WordSuffixes"],
Cell[70648, 3136, 52, 1, 32, "ObjectName",
  CellTags->"$bottom"],
Cell[70703, 3139, 55, 1, 50, "Highlight",
  CellTags->"$bottom"],
Cell[70761, 3142, 195, 6, 31, "Usage",
  CellTags->"$bottom"],
Cell[70959, 3150, 60, 1, 24, "Notes",
  CellTags->"$bottom"],
Cell[71022, 3153, 114, 4, 24, "Notes",
  CellTags->"$bottom"],
Cell[71139, 3159, 160, 7, 24, "Notes",
  CellTags->"$bottom"],
Cell[71302, 3168, 65, 1, 39, "ExampleSection",
  CellTags->"$bottom"],
Cell[71370, 3171, 105, 2, 34, "ExampleText",
  CellTags->"$bottom"],
Cell[71478, 3175, 249, 6, 42, "Input",
  CellTags->"$bottom"],
Cell[71730, 3183, 173, 5, 58, "Input",
  CellTags->"$bottom"],
Cell[71906, 3190, 107, 2, 34, "ExampleText",
  CellTags->"$bottom"],
Cell[72016, 3194, 170, 4, 42, "Input",
  CellTags->"$bottom"],
Cell[72189, 3200, 72, 1, 32, "ObjectName",
  CellTags->"$defaultnormalize"],
Cell[72264, 3203, 65, 1, 50, "Highlight",
  CellTags->"$defaultnormalize"],
Cell[72332, 3206, 200, 6, 31, "Usage",
  CellTags->"$defaultnormalize"],
Cell[72535, 3214, 183, 5, 24, "Notes",
  CellTags->"$defaultnormalize"],
Cell[72721, 3221, 328, 10, 40, "Notes",
  CellTags->"$defaultnormalize"],
Cell[73052, 3233, 444, 12, 56, "Notes",
  CellTags->"$defaultnormalize"],
Cell[73499, 3247, 258, 7, 40, "Notes",
  CellTags->"$defaultnormalize"],
Cell[73760, 3256, 338, 7, 56, "Notes",
  CellTags->"$defaultnormalize"],
Cell[74101, 3265, 266, 11, 24, "Notes",
  CellTags->"$defaultnormalize"],
Cell[74370, 3278, 75, 1, 39, "ExampleSection",
  CellTags->"$defaultnormalize"],
Cell[74448, 3281, 95, 1, 34, "ExampleText",
  CellTags->"$defaultnormalize"],
Cell[74546, 3284, 112, 2, 28, "Input",
  CellTags->"$defaultnormalize"],
Cell[74661, 3288, 80, 1, 34, "ExampleText",
  CellTags->"$defaultnormalize"],
Cell[74744, 3291, 168, 4, 26, "Input",
  CellTags->"$defaultnormalize"],
Cell[74915, 3297, 163, 4, 26, "Input",
  CellTags->"$defaultnormalize"],
Cell[75081, 3303, 56, 1, 32, "ObjectName",
  CellTags->"AcceptQFA"],
Cell[75140, 3306, 54, 1, 50, "Highlight",
  CellTags->"AcceptQFA"],
Cell[75197, 3309, 193, 8, 31, "Usage",
  CellTags->"AcceptQFA"],
Cell[75393, 3319, 281, 10, 61, "Notes",
  CellTags->"AcceptQFA"],
Cell[75677, 3331, 144, 5, 24, "Notes",
  CellTags->"AcceptQFA"],
Cell[75824, 3338, 267, 11, 24, "Notes",
  CellTags->"AcceptQFA"],
Cell[76094, 3351, 274, 12, 24, "Notes",
  CellTags->"AcceptQFA"],
Cell[76371, 3365, 67, 1, 39, "ExampleSection",
  CellTags->"AcceptQFA"],
Cell[76441, 3368, 169, 6, 34, "ExampleText",
  CellTags->"AcceptQFA"],
Cell[76613, 3376, 87, 2, 28, "Input",
  CellTags->"AcceptQFA"],
Cell[76703, 3380, 103, 3, 26, "Input",
  CellTags->"AcceptQFA"],
Cell[76809, 3385, 125, 3, 26, "Input",
  CellTags->"AcceptQFA"],
Cell[76937, 3390, 79, 1, 34, "ExampleText",
  CellTags->"AcceptQFA"],
Cell[77019, 3393, 123, 3, 26, "Input",
  CellTags->"AcceptQFA"],
Cell[77145, 3398, 100, 1, 34, "ExampleText",
  CellTags->"AcceptQFA"],
Cell[77248, 3401, 106, 3, 26, "Input",
  CellTags->"AcceptQFA"],
Cell[77357, 3406, 62, 1, 32, "ObjectName",
  CellTags->"AccessibleFA"],
Cell[77422, 3409, 57, 1, 50, "Highlight",
  CellTags->"AccessibleFA"],
Cell[77482, 3412, 171, 6, 31, "Usage",
  CellTags->"AccessibleFA"],
Cell[77656, 3420, 319, 10, 61, "Notes",
  CellTags->"AccessibleFA"],
Cell[77978, 3432, 92, 1, 24, "Notes",
  CellTags->"AccessibleFA"],
Cell[78073, 3435, 129, 4, 24, "Notes",
  CellTags->"AccessibleFA"],
Cell[78205, 3441, 264, 11, 24, "Notes",
  CellTags->"AccessibleFA"],
Cell[78472, 3454, 70, 1, 39, "ExampleSection",
  CellTags->"AccessibleFA"],
Cell[78545, 3457, 180, 4, 50, "ExampleText",
  CellTags->"AccessibleFA"],
Cell[78728, 3463, 206, 5, 42, "Input",
  CellTags->"AccessibleFA"],
Cell[78937, 3470, 150, 4, 42, "Input",
  CellTags->"AccessibleFA"],
Cell[79090, 3476, 106, 1, 34, "ExampleText",
  CellTags->"AccessibleFA"],
Cell[79199, 3479, 136, 3, 26, "Input",
  CellTags->"AccessibleFA"],
Cell[79338, 3484, 136, 3, 26, "Input",
  CellTags->"AccessibleFA"],
Cell[79477, 3489, 68, 1, 32, "ObjectName",
  CellTags->"AddTransitionFA"],
Cell[79548, 3492, 60, 1, 50, "Highlight",
  CellTags->"AddTransitionFA"],
Cell[79611, 3495, 202, 8, 31, "Usage",
  CellTags->"AddTransitionFA"],
Cell[79816, 3505, 165, 5, 24, "Notes",
  CellTags->"AddTransitionFA"],
Cell[79984, 3512, 261, 9, 40, "Notes",
  CellTags->"AddTransitionFA"],
Cell[80248, 3523, 296, 11, 24, "Notes",
  CellTags->"AddTransitionFA"],
Cell[80547, 3536, 73, 1, 39, "ExampleSection",
  CellTags->"AddTransitionFA"],
Cell[80623, 3539, 94, 1, 34, "ExampleText",
  CellTags->"AddTransitionFA"],
Cell[80720, 3542, 204, 5, 42, "Input",
  CellTags->"AddTransitionFA"],
Cell[80927, 3549, 187, 6, 34, "ExampleText",
  CellTags->"AddTransitionFA"],
Cell[81117, 3557, 172, 4, 42, "Input",
  CellTags->"AddTransitionFA"],
Cell[81292, 3563, 178, 6, 34, "ExampleText",
  CellTags->"AddTransitionFA"],
Cell[81473, 3571, 179, 4, 42, "Input",
  CellTags->"AddTransitionFA"],
Cell[81655, 3577, 50, 1, 32, "ObjectName",
  CellTags->"AllDFA"],
Cell[81708, 3580, 51, 1, 50, "Highlight",
  CellTags->"AllDFA"],
Cell[81762, 3583, 172, 6, 31, "Usage",
  CellTags->"AllDFA"],
Cell[81937, 3591, 225, 9, 44, "Notes",
  CellTags->"AllDFA"],
Cell[82165, 3602, 148, 5, 24, "Notes",
  CellTags->"AllDFA"],
Cell[82316, 3609, 256, 12, 24, "Notes",
  CellTags->"AllDFA"],
Cell[82575, 3623, 64, 1, 39, "ExampleSection",
  CellTags->"AllDFA"],
Cell[82642, 3626, 179, 6, 34, "ExampleText",
  CellTags->"AllDFA"],
Cell[82824, 3634, 118, 3, 26, "Input",
  CellTags->"AllDFA"],
Cell[82945, 3639, 102, 3, 26, "Input",
  CellTags->"AllDFA"],
Cell[83050, 3644, 62, 1, 32, "ObjectName",
  CellTags->"AmbiguousQFA"],
Cell[83115, 3647, 57, 1, 50, "Highlight",
  CellTags->"AmbiguousQFA"],
Cell[83175, 3650, 168, 6, 31, "Usage",
  CellTags->"AmbiguousQFA"],
Cell[83346, 3658, 295, 10, 61, "Notes",
  CellTags->"AmbiguousQFA"],
Cell[83644, 3670, 196, 6, 40, "Notes",
  CellTags->"AmbiguousQFA"],
Cell[83843, 3678, 134, 4, 24, "Notes",
  CellTags->"AmbiguousQFA"],
Cell[83980, 3684, 303, 13, 24, "Notes",
  CellTags->"AmbiguousQFA"],
Cell[84286, 3699, 70, 1, 39, "ExampleSection",
  CellTags->"AmbiguousQFA"],
Cell[84359, 3702, 81, 1, 34, "ExampleText",
  CellTags->"AmbiguousQFA"],
Cell[84443, 3705, 222, 8, 41, "Input",
  CellTags->"AmbiguousQFA"],
Cell[84668, 3715, 104, 1, 34, "ExampleText",
  CellTags->"AmbiguousQFA"],
Cell[84775, 3718, 147, 4, 42, "Input",
  CellTags->"AmbiguousQFA"],
Cell[84925, 3724, 178, 4, 50, "ExampleText",
  CellTags->"AmbiguousQFA"],
Cell[85106, 3730, 166, 4, 42, "Input",
  CellTags->"AmbiguousQFA"],
Cell[85275, 3736, 133, 3, 26, "Input",
  CellTags->"AmbiguousQFA"],
Cell[85411, 3741, 46, 1, 32, "ObjectName",
  CellTags->"BEFA"],
Cell[85460, 3744, 52, 1, 50, "Highlight",
  CellTags->"BEFA"],
Cell[85515, 3747, 238, 6, 47, "Usage",
  CellTags->"BEFA"],
Cell[85756, 3755, 273, 8, 40, "Notes",
  CellTags->"BEFA"],
Cell[86032, 3765, 267, 9, 40, "Notes",
  CellTags->"BEFA"],
Cell[86302, 3776, 95, 1, 24, "Notes",
  CellTags->"BEFA"],
Cell[86400, 3779, 140, 6, 24, "Notes",
  CellTags->"BEFA"],
Cell[86543, 3787, 62, 1, 39, "ExampleSection",
  CellTags->"BEFA"],
Cell[86608, 3790, 193, 6, 34, "ExampleText",
  CellTags->"BEFA"],
Cell[86804, 3798, 178, 4, 24, "Input",
  CellTags->"BEFA"],
Cell[86985, 3804, 102, 2, 34, "ExampleText",
  CellTags->"BEFA"],
Cell[87090, 3808, 168, 5, 45, "Input",
  CellTags->"BEFA"],
Cell[87261, 3815, 54, 1, 32, "ObjectName",
  CellTags->"BEFAToFA"],
Cell[87318, 3818, 53, 1, 50, "Highlight",
  CellTags->"BEFAToFA"],
Cell[87374, 3821, 170, 7, 31, "Usage",
  CellTags->"BEFAToFA"],
Cell[87547, 3830, 97, 1, 24, "Notes",
  CellTags->"BEFAToFA"],
Cell[87647, 3833, 264, 11, 24, "Notes",
  CellTags->"BEFAToFA"],
Cell[87914, 3846, 66, 1, 39, "ExampleSection",
  CellTags->"BEFAToFA"],
Cell[87983, 3849, 86, 1, 34, "ExampleText",
  CellTags->"BEFAToFA"],
Cell[88072, 3852, 127, 3, 26, "Input",
  CellTags->"BEFAToFA"],
Cell[88202, 3857, 95, 3, 26, "Input",
  CellTags->"BEFAToFA"],
Cell[88300, 3862, 58, 1, 32, "ObjectName",
  CellTags->"BehaviorFA"],
Cell[88361, 3865, 55, 1, 50, "Highlight",
  CellTags->"BehaviorFA"],
Cell[88419, 3868, 251, 9, 47, "Usage",
  CellTags->"BehaviorFA"],
Cell[88673, 3879, 367, 12, 40, "Notes",
  CellTags->"BehaviorFA"],
Cell[89043, 3893, 185, 7, 24, "Notes",
  CellTags->"BehaviorFA"],
Cell[89231, 3902, 174, 4, 40, "Notes",
  CellTags->"BehaviorFA"],
Cell[89408, 3908, 195, 9, 24, "Notes",
  CellTags->"BehaviorFA"],
Cell[89606, 3919, 68, 1, 39, "ExampleSection",
  CellTags->"BehaviorFA"],
Cell[89677, 3922, 199, 6, 34, "ExampleText",
  CellTags->"BehaviorFA"],
Cell[89879, 3930, 127, 3, 26, "Input",
  CellTags->"BehaviorFA"],
Cell[90009, 3935, 132, 4, 34, "ExampleText",
  CellTags->"BehaviorFA"],
Cell[90144, 3941, 109, 3, 26, "Input",
  CellTags->"BehaviorFA"],
Cell[90256, 3946, 187, 6, 34, "ExampleText",
  CellTags->"BehaviorFA"],
Cell[90446, 3954, 141, 4, 42, "Input",
  CellTags->"BehaviorFA"],
Cell[90590, 3960, 109, 3, 26, "Input",
  CellTags->"BehaviorFA"],
Cell[90702, 3965, 54, 1, 32, "ObjectName",
  CellTags->"Boundary"],
Cell[90759, 3968, 51, 1, 50, "Highlight",
  CellTags->"Boundary"],
Cell[90813, 3971, 144, 6, 31, "Usage",
  CellTags->"Boundary"],
Cell[90960, 3979, 186, 9, 24, "Notes",
  CellTags->"Boundary"],
Cell[91149, 3990, 345, 15, 24, "Notes",
  CellTags->"Boundary"],
Cell[91497, 4007, 56, 1, 32, "ObjectName",
  CellTags->"BuildBEFA"],
Cell[91556, 4010, 54, 1, 50, "Highlight",
  CellTags->"BuildBEFA"],
Cell[91613, 4013, 197, 7, 31, "Usage",
  CellTags->"BuildBEFA"],
Cell[91813, 4022, 117, 5, 24, "Notes",
  CellTags->"BuildBEFA"],
Cell[91933, 4029, 134, 4, 24, "Notes",
  CellTags->"BuildBEFA"],
Cell[92070, 4035, 206, 9, 24, "Notes",
  CellTags->"BuildBEFA"],
Cell[92279, 4046, 67, 1, 39, "ExampleSection",
  CellTags->"BuildBEFA"],
Cell[92349, 4049, 231, 7, 50, "ExampleText",
  CellTags->"BuildBEFA"],
Cell[92583, 4058, 85, 2, 28, "Input",
  CellTags->"BuildBEFA"],
Cell[92671, 4062, 54, 1, 32, "ObjectName",
  CellTags->"BuildDFA"],
Cell[92728, 4065, 53, 1, 50, "Highlight",
  CellTags->"BuildDFA"],
Cell[92784, 4068, 230, 7, 47, "Usage",
  CellTags->"BuildDFA"],
Cell[93017, 4077, 248, 9, 44, "Notes",
  CellTags->"BuildDFA"],
Cell[93268, 4088, 257, 8, 40, "Notes",
  CellTags->"BuildDFA"],
Cell[93528, 4098, 79, 1, 24, "Notes",
  CellTags->"BuildDFA"],
Cell[93610, 4101, 164, 7, 24, "Notes",
  CellTags->"BuildDFA"],
Cell[93777, 4110, 66, 1, 39, "ExampleSection",
  CellTags->"BuildDFA"],
Cell[93846, 4113, 75, 1, 34, "ExampleText",
  CellTags->"BuildDFA"],
Cell[93924, 4116, 152, 4, 42, "Input",
  CellTags->"BuildDFA"],
Cell[94079, 4122, 149, 6, 34, "ExampleText",
  CellTags->"BuildDFA"],
Cell[94231, 4130, 184, 5, 58, "Input",
  CellTags->"BuildDFA"],
Cell[94418, 4137, 123, 4, 34, "ExampleText",
  CellTags->"BuildDFA"],
Cell[94544, 4143, 138, 3, 26, "Input",
  CellTags->"BuildDFA"],
Cell[94685, 4148, 56, 1, 32, "ObjectName",
  CellTags->"BuildNNFA"],
Cell[94744, 4151, 54, 1, 50, "Highlight",
  CellTags->"BuildNNFA"],
Cell[94801, 4154, 213, 6, 47, "Usage",
  CellTags->"BuildNNFA"],
Cell[95017, 4162, 117, 5, 24, "Notes",
  CellTags->"BuildNNFA"],
Cell[95137, 4169, 145, 4, 24, "Notes",
  CellTags->"BuildNNFA"],
Cell[95285, 4175, 205, 9, 24, "Notes",
  CellTags->"BuildNNFA"],
Cell[95493, 4186, 67, 1, 39, "ExampleSection",
  CellTags->"BuildNNFA"],
Cell[95563, 4189, 73, 1, 34, "ExampleText",
  CellTags->"BuildNNFA"],
Cell[95639, 4192, 82, 2, 28, "Input",
  CellTags->"BuildNNFA"],
Cell[95724, 4196, 62, 1, 32, "ObjectName",
  CellTags->"BuildStarDFA"],
Cell[95789, 4199, 57, 1, 50, "Highlight",
  CellTags->"BuildStarDFA"],
Cell[95849, 4202, 229, 7, 47, "Usage",
  CellTags->"BuildStarDFA"],
Cell[96081, 4211, 91, 1, 24, "Notes",
  CellTags->"BuildStarDFA"],
Cell[96175, 4214, 195, 9, 24, "Notes",
  CellTags->"BuildStarDFA"],
Cell[96373, 4225, 70, 1, 39, "ExampleSection",
  CellTags->"BuildStarDFA"],
Cell[96446, 4228, 400, 14, 34, "ExampleText",
  CellTags->"BuildStarDFA"],
Cell[96849, 4244, 112, 3, 26, "Input",
  CellTags->"BuildStarDFA"],
Cell[96964, 4249, 134, 3, 26, "Input",
  CellTags->"BuildStarDFA"],
Cell[97101, 4254, 125, 4, 34, "ExampleText",
  CellTags->"BuildStarDFA"],
Cell[97229, 4260, 144, 3, 26, "Input",
  CellTags->"BuildStarDFA"],
Cell[97376, 4265, 60, 1, 32, "ObjectName",
  CellTags->"CirculantSA"],
Cell[97439, 4268, 56, 1, 50, "Highlight",
  CellTags->"CirculantSA"],
Cell[97498, 4271, 258, 7, 47, "Usage",
  CellTags->"CirculantSA"],
Cell[97759, 4280, 325, 12, 40, "Notes",
  CellTags->"CirculantSA"],
Cell[98087, 4294, 133, 5, 24, "Notes",
  CellTags->"CirculantSA"],
Cell[98223, 4301, 260, 8, 24, "Notes",
  CellTags->"CirculantSA"],
Cell[98486, 4311, 172, 7, 24, "Notes",
  CellTags->"CirculantSA"],
Cell[98661, 4320, 69, 1, 39, "ExampleSection",
  CellTags->"CirculantSA"],
Cell[98733, 4323, 245, 9, 34, "ExampleText",
  CellTags->"CirculantSA"],
Cell[98981, 4334, 138, 3, 26, "Input",
  CellTags->"CirculantSA"],
Cell[99122, 4339, 102, 3, 26, "Input",
  CellTags->"CirculantSA"],
Cell[99227, 4344, 66, 1, 32, "ObjectName",
  CellTags->"CoAccessibleFA"],
Cell[99296, 4347, 59, 1, 50, "Highlight",
  CellTags->"CoAccessibleFA"],
Cell[99358, 4350, 191, 6, 31, "Usage",
  CellTags->"CoAccessibleFA"],
Cell[99552, 4358, 321, 10, 61, "Notes",
  CellTags->"CoAccessibleFA"],
Cell[99876, 4370, 167, 5, 24, "Notes",
  CellTags->"CoAccessibleFA"],
Cell[100046, 4377, 262, 11, 24, "Notes",
  CellTags->"CoAccessibleFA"],
Cell[100311, 4390, 72, 1, 39, "ExampleSection",
  CellTags->"CoAccessibleFA"],
Cell[100386, 4393, 125, 4, 34, "ExampleText",
  CellTags->"CoAccessibleFA"],
Cell[100514, 4399, 195, 5, 42, "Input",
  CellTags->"CoAccessibleFA"],
Cell[100712, 4406, 145, 4, 42, "Input",
  CellTags->"CoAccessibleFA"],
Cell[100860, 4412, 108, 1, 34, "ExampleText",
  CellTags->"CoAccessibleFA"],
Cell[100971, 4415, 141, 3, 26, "Input",
  CellTags->"CoAccessibleFA"],
Cell[101115, 4420, 141, 3, 26, "Input",
  CellTags->"CoAccessibleFA"],
Cell[101259, 4425, 62, 1, 32, "ObjectName",
  CellTags->"ComplementFA"],
Cell[101324, 4428, 57, 1, 50, "Highlight",
  CellTags->"ComplementFA"],
Cell[101384, 4431, 215, 7, 47, "Usage",
  CellTags->"ComplementFA"],
Cell[101602, 4440, 286, 7, 40, "Notes",
  CellTags->"ComplementFA"],
Cell[101891, 4449, 255, 8, 24, "Notes",
  CellTags->"ComplementFA"],
Cell[102149, 4459, 524, 24, 40, "Notes",
  CellTags->"ComplementFA"],
Cell[102676, 4485, 70, 1, 39, "ExampleSection",
  CellTags->"ComplementFA"],
Cell[102749, 4488, 134, 4, 34, "ExampleText",
  CellTags->"ComplementFA"],
Cell[102886, 4494, 98, 2, 28, "Input",
  CellTags->"ComplementFA"],
Cell[102987, 4498, 111, 3, 26, "Input",
  CellTags->"ComplementFA"],
Cell[103101, 4503, 134, 3, 26, "Input",
  CellTags->"ComplementFA"],
Cell[103238, 4508, 100, 2, 28, "Input",
  CellTags->"ComplementFA"],
Cell[103341, 4512, 110, 2, 34, "ExampleText",
  CellTags->"ComplementFA"],
Cell[103454, 4516, 121, 3, 26, "Input",
  CellTags->"ComplementFA"],
Cell[103578, 4521, 134, 3, 26, "Input",
  CellTags->"ComplementFA"],
Cell[103715, 4526, 58, 1, 32, "ObjectName",
  CellTags->"CompleteFA"],
Cell[103776, 4529, 55, 1, 50, "Highlight",
  CellTags->"CompleteFA"],
Cell[103834, 4532, 186, 6, 31, "Usage",
  CellTags->"CompleteFA"],
Cell[104023, 4540, 126, 4, 24, "Notes",
  CellTags->"CompleteFA"],
Cell[104152, 4546, 195, 9, 24, "Notes",
  CellTags->"CompleteFA"],
Cell[104350, 4557, 68, 1, 39, "ExampleSection",
  CellTags->"CompleteFA"],
Cell[104421, 4560, 102, 1, 34, "ExampleText",
  CellTags->"CompleteFA"],
Cell[104526, 4563, 98, 2, 28, "Input",
  CellTags->"CompleteFA"],
Cell[104627, 4567, 98, 3, 26, "Input",
  CellTags->"CompleteFA"],
Cell[104728, 4572, 105, 3, 26, "Input",
  CellTags->"CompleteFA"],
Cell[104836, 4577, 60, 1, 32, "ObjectName",
  CellTags->"CompleteQFA"],
Cell[104899, 4580, 56, 1, 50, "Highlight",
  CellTags->"CompleteQFA"],
Cell[104958, 4583, 179, 6, 31, "Usage",
  CellTags->"CompleteQFA"],
Cell[105140, 4591, 160, 4, 40, "Notes",
  CellTags->"CompleteQFA"],
Cell[105303, 4597, 99, 1, 24, "Notes",
  CellTags->"CompleteQFA"],
Cell[105405, 4600, 195, 9, 24, "Notes",
  CellTags->"CompleteQFA"],
Cell[105603, 4611, 69, 1, 39, "ExampleSection",
  CellTags->"CompleteQFA"],
Cell[105675, 4614, 96, 1, 34, "ExampleText",
  CellTags->"CompleteQFA"],
Cell[105774, 4617, 123, 5, 45, "Input",
  CellTags->"CompleteQFA"],
Cell[105900, 4624, 112, 2, 34, "ExampleText",
  CellTags->"CompleteQFA"],
Cell[106015, 4628, 103, 2, 28, "Input",
  CellTags->"CompleteQFA"],
Cell[106121, 4632, 64, 1, 32, "ObjectName",
  CellTags->"ComputationFA"],
Cell[106188, 4635, 58, 1, 50, "Highlight",
  CellTags->"ComputationFA"],
Cell[106249, 4638, 260, 8, 47, "Usage",
  CellTags->"ComputationFA"],
Cell[106512, 4648, 228, 9, 44, "Notes",
  CellTags->"ComputationFA"],
Cell[106743, 4659, 164, 5, 24, "Notes",
  CellTags->"ComputationFA"],
Cell[106910, 4666, 195, 5, 40, "Notes",
  CellTags->"ComputationFA"],
Cell[107108, 4673, 148, 4, 24, "Notes",
  CellTags->"ComputationFA"],
Cell[107259, 4679, 295, 11, 24, "Notes",
  CellTags->"ComputationFA"],
Cell[107557, 4692, 71, 1, 39, "ExampleSection",
  CellTags->"ComputationFA"],
Cell[107631, 4695, 202, 7, 34, "ExampleText",
  CellTags->"ComputationFA"],
Cell[107836, 4704, 114, 3, 26, "Input",
  CellTags->"ComputationFA"],
Cell[107953, 4709, 211, 8, 34, "ExampleText",
  CellTags->"ComputationFA"],
Cell[108167, 4719, 118, 3, 26, "Input",
  CellTags->"ComputationFA"],
Cell[108288, 4724, 93, 1, 34, "ExampleText",
  CellTags->"ComputationFA"],
Cell[108384, 4727, 138, 3, 26, "Input",
  CellTags->"ComputationFA"],
Cell[108525, 4732, 105, 1, 34, "ExampleText",
  CellTags->"ComputationFA"],
Cell[108633, 4735, 137, 3, 26, "Input",
  CellTags->"ComputationFA"],
Cell[108773, 4740, 74, 1, 32, "ObjectName",
  CellTags->"ComputationGraphFA"],
Cell[108850, 4743, 63, 1, 50, "Highlight",
  CellTags->"ComputationGraphFA"],
Cell[108916, 4746, 253, 8, 47, "Usage",
  CellTags->"ComputationGraphFA"],
Cell[109172, 4756, 295, 9, 40, "Notes",
  CellTags->"ComputationGraphFA"],
Cell[109470, 4767, 178, 6, 24, "Notes",
  CellTags->"ComputationGraphFA"],
Cell[109651, 4775, 384, 15, 24, "Notes",
  CellTags->"ComputationGraphFA"],
Cell[110038, 4792, 76, 1, 39, "ExampleSection",
  CellTags->"ComputationGraphFA"],
Cell[110117, 4795, 207, 7, 34, "ExampleText",
  CellTags->"ComputationGraphFA"],
Cell[110327, 4804, 119, 3, 26, "Input",
  CellTags->"ComputationGraphFA"],
Cell[110449, 4809, 165, 4, 26, "Input",
  CellTags->"ComputationGraphFA"],
Cell[110617, 4815, 64, 1, 32, "ObjectName",
  CellTags->"ConcatenateFA"],
Cell[110684, 4818, 58, 1, 50, "Highlight",
  CellTags->"ConcatenateFA"],
Cell[110745, 4821, 257, 9, 47, "Usage",
  CellTags->"ConcatenateFA"],
Cell[111005, 4832, 300, 10, 61, "Notes",
  CellTags->"ConcatenateFA"],
Cell[111308, 4844, 193, 6, 24, "Notes",
  CellTags->"ConcatenateFA"],
Cell[111504, 4852, 247, 7, 24, "Notes",
  CellTags->"ConcatenateFA"],
Cell[111754, 4861, 285, 9, 24, "Notes",
  CellTags->"ConcatenateFA"],
Cell[112042, 4872, 110, 4, 24, "Notes",
  CellTags->"ConcatenateFA"],
Cell[112155, 4878, 459, 21, 24, "Notes",
  CellTags->"ConcatenateFA"],
Cell[112617, 4901, 71, 1, 39, "ExampleSection",
  CellTags->"ConcatenateFA"],
Cell[112691, 4904, 103, 1, 34, "ExampleText",
  CellTags->"ConcatenateFA"],
Cell[112797, 4907, 100, 2, 28, "Input",
  CellTags->"ConcatenateFA"],
Cell[112900, 4911, 118, 3, 26, "Input",
  CellTags->"ConcatenateFA"],
Cell[113021, 4916, 141, 3, 26, "Input",
  CellTags->"ConcatenateFA"],
Cell[113165, 4921, 60, 1, 32, "ObjectName",
  CellTags->"ConvertToFA"],
Cell[113228, 4924, 56, 1, 50, "Highlight",
  CellTags->"ConvertToFA"],
Cell[113287, 4927, 372, 16, 47, "Usage",
  CellTags->"ConvertToFA"],
Cell[113662, 4945, 300, 10, 61, "Notes",
  CellTags->"ConvertToFA"],
Cell[113965, 4957, 218, 6, 40, "Notes",
  CellTags->"ConvertToFA"],
Cell[114186, 4965, 273, 12, 24, "Notes",
  CellTags->"ConvertToFA"],
Cell[114462, 4979, 69, 1, 39, "ExampleSection",
  CellTags->"ConvertToFA"],
Cell[114534, 4982, 509, 15, 66, "ExampleText",
  CellTags->"ConvertToFA"],
Cell[115046, 4999, 252, 7, 42, "Input",
  CellTags->"ConvertToFA"],
Cell[115301, 5008, 107, 3, 26, "Input",
  CellTags->"ConvertToFA"],
Cell[115411, 5013, 110, 2, 34, "ExampleText",
  CellTags->"ConvertToFA"],
Cell[115524, 5017, 315, 7, 58, "Input",
  CellTags->"ConvertToFA"],
Cell[115842, 5026, 126, 3, 26, "Input",
  CellTags->"ConvertToFA"],
Cell[115971, 5031, 58, 1, 32, "ObjectName",
  CellTags->"CounterDFA"],
Cell[116032, 5034, 55, 1, 50, "Highlight",
  CellTags->"CounterDFA"],
Cell[116090, 5037, 310, 12, 47, "Usage",
  CellTags->"CounterDFA"],
Cell[116403, 5051, 519, 18, 61, "Notes",
  CellTags->"CounterDFA"],
Cell[116925, 5071, 283, 9, 40, "Notes",
  CellTags->"CounterDFA"],
Cell[117211, 5082, 386, 14, 40, "Notes",
  CellTags->"CounterDFA"],
Cell[117600, 5098, 205, 9, 24, "Notes",
  CellTags->"CounterDFA"],
Cell[117808, 5109, 178, 7, 24, "Notes",
  CellTags->"CounterDFA"],
Cell[117989, 5118, 68, 1, 39, "ExampleSection",
  CellTags->"CounterDFA"],
Cell[118060, 5121, 153, 6, 34, "ExampleText",
  CellTags->"CounterDFA"],
Cell[118216, 5129, 150, 4, 42, "Input",
  CellTags->"CounterDFA"],
Cell[118369, 5135, 160, 6, 34, "ExampleText",
  CellTags->"CounterDFA"],
Cell[118532, 5143, 159, 4, 42, "Input",
  CellTags->"CounterDFA"],
Cell[118694, 5149, 159, 6, 34, "ExampleText",
  CellTags->"CounterDFA"],
Cell[118856, 5157, 173, 4, 42, "Input",
  CellTags->"CounterDFA"],
Cell[119032, 5163, 56, 1, 32, "ObjectName",
  CellTags->"CoverQDFA"],
Cell[119091, 5166, 54, 1, 50, "Highlight",
  CellTags->"CoverQDFA"],
Cell[119148, 5169, 194, 8, 31, "Usage",
  CellTags->"CoverQDFA"],
Cell[119345, 5179, 228, 9, 44, "Notes",
  CellTags->"CoverQDFA"],
Cell[119576, 5190, 182, 6, 40, "Notes",
  CellTags->"CoverQDFA"],
Cell[119761, 5198, 97, 1, 24, "Notes",
  CellTags->"CoverQDFA"],
Cell[119861, 5201, 259, 10, 24, "Notes",
  CellTags->"CoverQDFA"],
Cell[120123, 5213, 384, 16, 24, "Notes",
  CellTags->"CoverQDFA"],
Cell[120510, 5231, 67, 1, 39, "ExampleSection",
  CellTags->"CoverQDFA"],
Cell[120580, 5234, 132, 4, 34, "ExampleText",
  CellTags->"CoverQDFA"],
Cell[120715, 5240, 166, 4, 42, "Input",
  CellTags->"CoverQDFA"],
Cell[120884, 5246, 100, 3, 26, "Input",
  CellTags->"CoverQDFA"],
Cell[120987, 5251, 122, 3, 26, "Input",
  CellTags->"CoverQDFA"],
Cell[121112, 5256, 60, 1, 32, "ObjectName",
  CellTags->"DefiniteQFA"],
Cell[121175, 5259, 56, 1, 50, "Highlight",
  CellTags->"DefiniteQFA"],
Cell[121234, 5262, 343, 10, 69, "Usage",
  CellTags->"DefiniteQFA"],
Cell[121580, 5274, 294, 10, 61, "Notes",
  CellTags->"DefiniteQFA"],
Cell[121877, 5286, 164, 5, 24, "Notes",
  CellTags->"DefiniteQFA"],
Cell[122044, 5293, 170, 4, 40, "Notes",
  CellTags->"DefiniteQFA"],
Cell[122217, 5299, 231, 10, 24, "Notes",
  CellTags->"DefiniteQFA"],
Cell[122451, 5311, 69, 1, 39, "ExampleSection",
  CellTags->"DefiniteQFA"],
Cell[122523, 5314, 71, 1, 34, "ExampleText",
  CellTags->"DefiniteQFA"],
Cell[122597, 5317, 197, 5, 42, "Input",
  CellTags->"DefiniteQFA"],
Cell[122797, 5324, 105, 1, 34, "ExampleText",
  CellTags->"DefiniteQFA"],
Cell[122905, 5327, 118, 3, 26, "Input",
  CellTags->"DefiniteQFA"],
Cell[123026, 5332, 118, 3, 26, "Input",
  CellTags->"DefiniteQFA"],
Cell[123147, 5337, 124, 3, 26, "Input",
  CellTags->"DefiniteQFA"],
Cell[123274, 5342, 124, 3, 26, "Input",
  CellTags->"DefiniteQFA"],
Cell[123401, 5347, 56, 1, 32, "ObjectName",
  CellTags->"DegreesFA"],
Cell[123460, 5350, 54, 1, 50, "Highlight",
  CellTags->"DegreesFA"],
Cell[123517, 5353, 343, 14, 47, "Usage",
  CellTags->"DegreesFA"],
Cell[123863, 5369, 276, 10, 61, "Notes",
  CellTags->"DegreesFA"],
Cell[124142, 5381, 223, 6, 40, "Notes",
  CellTags->"DegreesFA"],
Cell[124368, 5389, 153, 5, 24, "Notes",
  CellTags->"DegreesFA"],
Cell[124524, 5396, 155, 4, 40, "Notes",
  CellTags->"DegreesFA"],
Cell[124682, 5402, 203, 9, 24, "Notes",
  CellTags->"DegreesFA"],
Cell[124888, 5413, 67, 1, 39, "ExampleSection",
  CellTags->"DegreesFA"],
Cell[124958, 5416, 135, 4, 34, "ExampleText",
  CellTags->"DegreesFA"],
Cell[125096, 5422, 97, 2, 28, "Input",
  CellTags->"DegreesFA"],
Cell[125196, 5426, 114, 3, 26, "Input",
  CellTags->"DegreesFA"],
Cell[125313, 5431, 64, 1, 32, "ObjectName",
  CellTags->"DeleteStateFA"],
Cell[125380, 5434, 58, 1, 50, "Highlight",
  CellTags->"DeleteStateFA"],
Cell[125441, 5437, 191, 8, 31, "Usage",
  CellTags->"DeleteStateFA"],
Cell[125635, 5447, 230, 9, 44, "Notes",
  CellTags->"DeleteStateFA"],
Cell[125868, 5458, 180, 6, 24, "Notes",
  CellTags->"DeleteStateFA"],
Cell[126051, 5466, 180, 6, 24, "Notes",
  CellTags->"DeleteStateFA"],
Cell[126234, 5474, 216, 9, 24, "Notes",
  CellTags->"DeleteStateFA"],
Cell[126453, 5485, 71, 1, 39, "ExampleSection",
  CellTags->"DeleteStateFA"],
Cell[126527, 5488, 129, 4, 34, "ExampleText",
  CellTags->"DeleteStateFA"],
Cell[126659, 5494, 315, 7, 74, "Input",
  CellTags->"DeleteStateFA"],
Cell[126977, 5503, 276, 5, 58, "Input",
  CellTags->"DeleteStateFA"],
Cell[127256, 5510, 70, 1, 32, "ObjectName",
  CellTags->"DeterministicQFA"],
Cell[127329, 5513, 61, 1, 50, "Highlight",
  CellTags->"DeterministicQFA"],
Cell[127393, 5516, 177, 6, 31, "Usage",
  CellTags->"DeterministicQFA"],
Cell[127573, 5524, 294, 10, 61, "Notes",
  CellTags->"DeterministicQFA"],
Cell[127870, 5536, 420, 17, 40, "Notes",
  CellTags->"DeterministicQFA"],
Cell[128293, 5555, 187, 5, 40, "Notes",
  CellTags->"DeterministicQFA"],
Cell[128483, 5562, 360, 12, 40, "Notes",
  CellTags->"DeterministicQFA"],
Cell[128846, 5576, 302, 13, 24, "Notes",
  CellTags->"DeterministicQFA"],
Cell[129151, 5591, 74, 1, 39, "ExampleSection",
  CellTags->"DeterministicQFA"],
Cell[129228, 5594, 157, 4, 50, "ExampleText",
  CellTags->"DeterministicQFA"],
Cell[129388, 5600, 208, 5, 58, "Input",
  CellTags->"DeterministicQFA"],
Cell[129599, 5607, 151, 4, 34, "ExampleText",
  CellTags->"DeterministicQFA"],
Cell[129753, 5613, 165, 4, 42, "Input",
  CellTags->"DeterministicQFA"],
Cell[129921, 5619, 150, 4, 34, "ExampleText",
  CellTags->"DeterministicQFA"],
Cell[130074, 5625, 337, 7, 58, "Input",
  CellTags->"DeterministicQFA"],
Cell[130414, 5634, 44, 1, 32, "ObjectName",
  CellTags->"DFA"],
Cell[130461, 5637, 51, 1, 50, "Highlight",
  CellTags->"DFA"],
Cell[130515, 5640, 142, 4, 31, "Usage",
  CellTags->"DFA"],
Cell[130660, 5646, 319, 12, 40, "Notes",
  CellTags->"DFA"],
Cell[130982, 5660, 247, 6, 40, "Notes",
  CellTags->"DFA"],
Cell[131232, 5668, 197, 9, 24, "Notes",
  CellTags->"DFA"],
Cell[131432, 5679, 188, 8, 24, "Notes",
  CellTags->"DFA"],
Cell[131623, 5689, 352, 9, 56, "Notes",
  CellTags->"DFA"],
Cell[131978, 5700, 342, 16, 24, "Notes",
  CellTags->"DFA"],
Cell[132323, 5718, 61, 1, 39, "ExampleSection",
  CellTags->"DFA"],
Cell[132387, 5721, 402, 10, 66, "ExampleText",
  CellTags->"DFA"],
Cell[132792, 5733, 98, 3, 26, "Input",
  CellTags->"DFA"],
Cell[132893, 5738, 199, 5, 26, "Input",
  CellTags->"DFA"],
Cell[133095, 5745, 117, 4, 42, "Input",
  CellTags->"DFA"],
Cell[133215, 5751, 103, 3, 26, "Input",
  CellTags->"DFA"],
Cell[133321, 5756, 68, 1, 32, "ObjectName",
  CellTags->"DivisibilityDFA"],
Cell[133392, 5759, 60, 1, 50, "Highlight",
  CellTags->"DivisibilityDFA"],
Cell[133455, 5762, 253, 8, 47, "Usage",
  CellTags->"DivisibilityDFA"],
Cell[133711, 5772, 332, 11, 79, "Notes",
  CellTags->"DivisibilityDFA"],
Cell[134046, 5785, 176, 5, 24, "Notes",
  CellTags->"DivisibilityDFA"],
Cell[134225, 5792, 156, 5, 24, "Notes",
  CellTags->"DivisibilityDFA"],
Cell[134384, 5799, 231, 8, 24, "Notes",
  CellTags->"DivisibilityDFA"],
Cell[134618, 5809, 1094, 34, 72, "Notes",
  CellTags->"DivisibilityDFA"],
Cell[135715, 5845, 320, 9, 40, "Notes",
  CellTags->"DivisibilityDFA"],
Cell[136038, 5856, 242, 10, 24, "Notes",
  CellTags->"DivisibilityDFA"],
Cell[136283, 5868, 73, 1, 39, "ExampleSection",
  CellTags->"DivisibilityDFA"],
Cell[136359, 5871, 284, 9, 50, "ExampleText",
  CellTags->"DivisibilityDFA"],
Cell[136646, 5882, 210, 5, 58, "Input",
  CellTags->"DivisibilityDFA"],
Cell[136859, 5889, 107, 1, 34, "ExampleText",
  CellTags->"DivisibilityDFA"],
Cell[136969, 5892, 126, 3, 26, "Input",
  CellTags->"DivisibilityDFA"],
Cell[137098, 5897, 105, 3, 26, "Input",
  CellTags->"DivisibilityDFA"],
Cell[137206, 5902, 112, 3, 26, "Input",
  CellTags->"DivisibilityDFA"],
Cell[137321, 5907, 221, 9, 34, "ExampleText",
  CellTags->"DivisibilityDFA"],
Cell[137545, 5918, 183, 4, 42, "Input",
  CellTags->"DivisibilityDFA"],
Cell[137731, 5924, 169, 6, 34, "ExampleText",
  CellTags->"DivisibilityDFA"],
Cell[137903, 5932, 180, 4, 26, "Input",
  CellTags->"DivisibilityDFA"],
Cell[138086, 5938, 230, 7, 50, "ExampleText",
  CellTags->"DivisibilityDFA"],
Cell[138319, 5947, 195, 4, 42, "Input",
  CellTags->"DivisibilityDFA"],
Cell[138517, 5953, 50, 1, 32, "ObjectName",
  CellTags->"EdgeLU"],
Cell[138570, 5956, 49, 1, 50, "Highlight",
  CellTags->"EdgeLU"],
Cell[138622, 5959, 147, 6, 31, "Usage",
  CellTags->"EdgeLU"],
Cell[138772, 5967, 98, 1, 24, "Notes",
  CellTags->"EdgeLU"],
Cell[138873, 5970, 166, 7, 24, "Notes",
  CellTags->"EdgeLU"],
Cell[139042, 5979, 54, 1, 32, "ObjectName",
  CellTags->"EmptyDFA"],
Cell[139099, 5982, 53, 1, 50, "Highlight",
  CellTags->"EmptyDFA"],
Cell[139155, 5985, 185, 6, 31, "Usage",
  CellTags->"EmptyDFA"],
Cell[139343, 5993, 256, 12, 24, "Notes",
  CellTags->"EmptyDFA"],
Cell[139602, 6007, 66, 1, 39, "ExampleSection",
  CellTags->"EmptyDFA"],
Cell[139671, 6010, 108, 2, 34, "ExampleText",
  CellTags->"EmptyDFA"],
Cell[139782, 6014, 82, 2, 28, "Input",
  CellTags->"EmptyDFA"],
Cell[139867, 6018, 104, 3, 26, "Input",
  CellTags->"EmptyDFA"],
Cell[139974, 6023, 54, 1, 32, "ObjectName",
  CellTags->"EmptyQFA"],
Cell[140031, 6026, 53, 1, 50, "Highlight",
  CellTags->"EmptyQFA"],
Cell[140087, 6029, 172, 6, 31, "Usage",
  CellTags->"EmptyQFA"],
Cell[140262, 6037, 125, 4, 24, "Notes",
  CellTags->"EmptyQFA"],
Cell[140390, 6043, 128, 6, 24, "Notes",
  CellTags->"EmptyQFA"],
Cell[140521, 6051, 66, 1, 39, "ExampleSection",
  CellTags->"EmptyQFA"],
Cell[140590, 6054, 102, 1, 34, "ExampleText",
  CellTags->"EmptyQFA"],
Cell[140695, 6057, 304, 8, 58, "Input",
  CellTags->"EmptyQFA"],
Cell[141002, 6067, 175, 5, 58, "Input",
  CellTags->"EmptyQFA"],
Cell[141180, 6074, 52, 1, 32, "ObjectName",
  CellTags->"EpsElim"],
Cell[141235, 6077, 50, 1, 50, "Highlight",
  CellTags->"EpsElim"],
Cell[141288, 6080, 194, 8, 31, "Usage",
  CellTags->"EpsElim"],
Cell[141485, 6090, 175, 5, 24, "Notes",
  CellTags->"EpsElim"],
Cell[141663, 6097, 289, 11, 24, "Notes",
  CellTags->"EpsElim"],
Cell[141955, 6110, 62, 1, 32, "ObjectName",
  CellTags->"EpsilonFreeQ"],
Cell[142020, 6113, 57, 1, 50, "Highlight",
  CellTags->"EpsilonFreeQ"],
Cell[142080, 6116, 140, 4, 31, "Usage",
  CellTags->"EpsilonFreeQ"],
Cell[142223, 6122, 345, 17, 24, "Notes",
  CellTags->"EpsilonFreeQ"],
Cell[142571, 6141, 145, 6, 24, "Notes",
  CellTags->"EpsilonFreeQ"],
Cell[142719, 6149, 70, 1, 39, "ExampleSection",
  CellTags->"EpsilonFreeQ"],
Cell[142792, 6152, 134, 4, 34, "ExampleText",
  CellTags->"EpsilonFreeQ"],
Cell[142929, 6158, 117, 2, 28, "Input",
  CellTags->"EpsilonFreeQ"],
Cell[143049, 6162, 102, 3, 26, "Input",
  CellTags->"EpsilonFreeQ"],
Cell[143154, 6167, 78, 1, 32, "ObjectName",
  CellTags->"EpsilonEliminationFA"],
Cell[143235, 6170, 65, 1, 50, "Highlight",
  CellTags->"EpsilonEliminationFA"],
Cell[143303, 6173, 178, 4, 31, "Usage",
  CellTags->"EpsilonEliminationFA"],
Cell[143484, 6179, 375, 11, 79, "Notes",
  CellTags->"EpsilonEliminationFA"],
Cell[143862, 6192, 837, 29, 56, "Notes",
  CellTags->"EpsilonEliminationFA"],
Cell[144702, 6223, 186, 4, 40, "Notes",
  CellTags->"EpsilonEliminationFA"],
Cell[144891, 6229, 187, 4, 40, "Notes",
  CellTags->"EpsilonEliminationFA"],
Cell[145081, 6235, 138, 6, 24, "Notes",
  CellTags->"EpsilonEliminationFA"],
Cell[145222, 6243, 78, 1, 39, "ExampleSection",
  CellTags->"EpsilonEliminationFA"],
Cell[145303, 6246, 160, 4, 50, "ExampleText",
  CellTags->"EpsilonEliminationFA"],
Cell[145466, 6252, 667, 11, 122, "Input",
  CellTags->"EpsilonEliminationFA"],
Cell[146136, 6265, 132, 3, 26, "Input",
  CellTags->"EpsilonEliminationFA"],
Cell[146271, 6270, 115, 3, 26, "Input",
  CellTags->"EpsilonEliminationFA"],
Cell[146389, 6275, 86, 1, 34, "ExampleText",
  CellTags->"EpsilonEliminationFA"],
Cell[146478, 6278, 176, 4, 26, "Input",
  CellTags->"EpsilonEliminationFA"],
Cell[146657, 6284, 97, 3, 26, "Input",
  CellTags->"EpsilonEliminationFA"],
Cell[146757, 6289, 96, 3, 26, "Input",
  CellTags->"EpsilonEliminationFA"],
Cell[146856, 6294, 97, 3, 26, "Input",
  CellTags->"EpsilonEliminationFA"],
Cell[146956, 6299, 64, 1, 32, "ObjectName",
  CellTags->"EquivalentQFA"],
Cell[147023, 6302, 58, 1, 50, "Highlight",
  CellTags->"EquivalentQFA"],
Cell[147084, 6305, 155, 4, 31, "Usage",
  CellTags->"EquivalentQFA"],
Cell[147242, 6311, 299, 10, 61, "Notes",
  CellTags->"EquivalentQFA"],
Cell[147544, 6323, 617, 20, 56, "Notes",
  CellTags->"EquivalentQFA"],
Cell[148164, 6345, 369, 7, 94, "Notes",
  CellTags->"EquivalentQFA"],
Cell[148536, 6354, 216, 5, 40, "Notes",
  CellTags->"EquivalentQFA"],
Cell[148755, 6361, 197, 9, 24, "Notes",
  CellTags->"EquivalentQFA"],
Cell[148955, 6372, 71, 1, 39, "ExampleSection",
  CellTags->"EquivalentQFA"],
Cell[149029, 6375, 80, 1, 34, "ExampleText",
  CellTags->"EquivalentQFA"],
Cell[149112, 6378, 223, 5, 42, "Input",
  CellTags->"EquivalentQFA"],
Cell[149338, 6385, 280, 6, 58, "Input",
  CellTags->"EquivalentQFA"],
Cell[149621, 6393, 202, 5, 42, "Input",
  CellTags->"EquivalentQFA"],
Cell[149826, 6400, 115, 3, 26, "Input",
  CellTags->"EquivalentQFA"],
Cell[149944, 6405, 96, 1, 34, "ExampleText",
  CellTags->"EquivalentQFA"],
Cell[150043, 6408, 115, 3, 26, "Input",
  CellTags->"EquivalentQFA"],
Cell[150161, 6413, 194, 5, 50, "ExampleText",
  CellTags->"EquivalentQFA"],
Cell[150358, 6420, 165, 4, 26, "Input",
  CellTags->"EquivalentQFA"],
Cell[150526, 6426, 121, 3, 26, "Input",
  CellTags->"EquivalentQFA"],
Cell[150650, 6431, 62, 1, 32, "ObjectName",
  CellTags->"FactorialQFA"],
Cell[150715, 6434, 57, 1, 50, "Highlight",
  CellTags->"FactorialQFA"],
Cell[150775, 6437, 219, 6, 47, "Usage",
  CellTags->"FactorialQFA"],
Cell[150997, 6445, 295, 10, 61, "Notes",
  CellTags->"FactorialQFA"],
Cell[151295, 6457, 221, 6, 40, "Notes",
  CellTags->"FactorialQFA"],
Cell[151519, 6465, 138, 6, 24, "Notes",
  CellTags->"FactorialQFA"],
Cell[151660, 6473, 70, 1, 39, "ExampleSection",
  CellTags->"FactorialQFA"],
Cell[151733, 6476, 81, 1, 34, "ExampleText",
  CellTags->"FactorialQFA"],
Cell[151817, 6479, 185, 5, 42, "Input",
  CellTags->"FactorialQFA"],
Cell[152005, 6486, 100, 3, 26, "Input",
  CellTags->"FactorialQFA"],
Cell[152108, 6491, 86, 1, 34, "ExampleText",
  CellTags->"FactorialQFA"],
Cell[152197, 6494, 119, 3, 26, "Input",
  CellTags->"FactorialQFA"],
Cell[152319, 6499, 42, 1, 32, "ObjectName",
  CellTags->"FA"],
Cell[152364, 6502, 50, 1, 50, "Highlight",
  CellTags->"FA"],
Cell[152417, 6505, 124, 4, 31, "Usage",
  CellTags->"FA"],
Cell[152544, 6511, 231, 9, 24, "Notes",
  CellTags->"FA"],
Cell[152778, 6522, 168, 7, 24, "Notes",
  CellTags->"FA"],
Cell[152949, 6531, 261, 9, 24, "Notes",
  CellTags->"FA"],
Cell[153213, 6542, 183, 7, 24, "Notes",
  CellTags->"FA"],
Cell[153399, 6551, 303, 15, 24, "Notes",
  CellTags->"FA"],
Cell[153705, 6568, 60, 1, 39, "ExampleSection",
  CellTags->"FA"],
Cell[153768, 6571, 52, 1, 34, "ExampleText",
  CellTags->"FA"],
Cell[153823, 6574, 207, 5, 42, "Input",
  CellTags->"FA"],
Cell[154033, 6581, 142, 5, 58, "Input",
  CellTags->"FA"],
Cell[154178, 6588, 64, 1, 32, "ObjectName",
  CellTags->"FinalVectorFA"],
Cell[154245, 6591, 58, 1, 50, "Highlight",
  CellTags->"FinalVectorFA"],
Cell[154306, 6594, 159, 4, 31, "Usage",
  CellTags->"FinalVectorFA"],
Cell[154468, 6600, 92, 1, 24, "Notes",
  CellTags->"FinalVectorFA"],
Cell[154563, 6603, 339, 14, 24, "Notes",
  CellTags->"FinalVectorFA"],
Cell[154905, 6619, 71, 1, 39, "ExampleSection",
  CellTags->"FinalVectorFA"],
Cell[154979, 6622, 96, 1, 34, "ExampleText",
  CellTags->"FinalVectorFA"],
Cell[155078, 6625, 227, 6, 74, "Input",
  CellTags->"FinalVectorFA"],
Cell[155308, 6633, 48, 1, 32, "ObjectName",
  CellTags->"Final"],
Cell[155359, 6636, 50, 1, 50, "Highlight",
  CellTags->"Final"],
Cell[155412, 6639, 198, 6, 31, "Usage",
  CellTags->"Final"],
Cell[155613, 6647, 257, 11, 24, "Notes",
  CellTags->"Final"],
Cell[155873, 6660, 63, 1, 39, "ExampleSection",
  CellTags->"Final"],
Cell[155939, 6663, 141, 4, 34, "ExampleText",
  CellTags->"Final"],
Cell[156083, 6669, 151, 4, 26, "Input",
  CellTags->"Final"],
Cell[156237, 6675, 88, 3, 26, "Input",
  CellTags->"Final"],
Cell[156328, 6680, 74, 1, 32, "ObjectName",
  CellTags->"FiniteComplementSA"],
Cell[156405, 6683, 63, 1, 50, "Highlight",
  CellTags->"FiniteComplementSA"],
Cell[156471, 6686, 281, 7, 47, "Usage",
  CellTags->"FiniteComplementSA"],
Cell[156755, 6695, 242, 9, 44, "Notes",
  CellTags->"FiniteComplementSA"],
Cell[157000, 6706, 178, 6, 24, "Notes",
  CellTags->"FiniteComplementSA"],
Cell[157181, 6714, 117, 4, 24, "Notes",
  CellTags->"FiniteComplementSA"],
Cell[157301, 6720, 143, 6, 24, "Notes",
  CellTags->"FiniteComplementSA"],
Cell[157447, 6728, 76, 1, 39, "ExampleSection",
  CellTags->"FiniteComplementSA"],
Cell[157526, 6731, 340, 13, 34, "ExampleText",
  CellTags->"FiniteComplementSA"],
Cell[157869, 6746, 128, 3, 26, "Input",
  CellTags->"FiniteComplementSA"],
Cell[158000, 6751, 114, 3, 26, "Input",
  CellTags->"FiniteComplementSA"],
Cell[158117, 6756, 108, 3, 26, "Input",
  CellTags->"FiniteComplementSA"],
Cell[158228, 6761, 48, 1, 32, "ObjectName",
  CellTags->"FinLU"],
Cell[158279, 6764, 48, 1, 50, "Highlight",
  CellTags->"FinLU"],
Cell[158330, 6767, 145, 6, 31, "Usage",
  CellTags->"FinLU"],
Cell[158478, 6775, 76, 1, 24, "Notes",
  CellTags->"FinLU"],
Cell[158557, 6778, 165, 7, 24, "Notes",
  CellTags->"FinLU"],
Cell[158725, 6787, 48, 1, 32, "ObjectName",
  CellTags->"Fixed"],
Cell[158776, 6790, 48, 1, 50, "Highlight",
  CellTags->"Fixed"],
Cell[158827, 6793, 137, 6, 31, "Usage",
  CellTags->"Fixed"],
Cell[158967, 6801, 82, 1, 24, "Notes",
  CellTags->"Fixed"],
Cell[159052, 6804, 149, 7, 24, "Notes",
  CellTags->"Fixed"],
Cell[159204, 6813, 54, 1, 32, "ObjectName",
  CellTags->"BoundLFA"],
Cell[159261, 6816, 51, 1, 50, "Highlight",
  CellTags->"BoundLFA"],
Cell[159315, 6819, 145, 6, 31, "Usage",
  CellTags->"BoundLFA"],
Cell[159463, 6827, 98, 1, 24, "Notes",
  CellTags->"BoundLFA"],
Cell[159564, 6830, 119, 5, 24, "Notes",
  CellTags->"BoundLFA"],
Cell[159686, 6837, 156, 7, 24, "Notes",
  CellTags->"BoundLFA"],
Cell[159845, 6846, 64, 1, 32, "ObjectName",
  CellTags->"FrontierToDFA"],
Cell[159912, 6849, 58, 1, 50, "Highlight",
  CellTags->"FrontierToDFA"],
Cell[159973, 6852, 229, 8, 31, "Usage",
  CellTags->"FrontierToDFA"],
Cell[160205, 6862, 218, 8, 24, "Notes",
  CellTags->"FrontierToDFA"],
Cell[160426, 6872, 273, 11, 24, "Notes",
  CellTags->"FrontierToDFA"],
Cell[160702, 6885, 71, 1, 39, "ExampleSection",
  CellTags->"FrontierToDFA"],
Cell[160776, 6888, 151, 4, 34, "ExampleText",
  CellTags->"FrontierToDFA"],
Cell[160930, 6894, 288, 6, 42, "Input",
  CellTags->"FrontierToDFA"],
Cell[161221, 6902, 128, 4, 34, "ExampleText",
  CellTags->"FrontierToDFA"],
Cell[161352, 6908, 118, 3, 26, "Input",
  CellTags->"FrontierToDFA"],
Cell[161473, 6913, 161, 4, 26, "Input",
  CellTags->"FrontierToDFA"],
Cell[161637, 6919, 172, 4, 50, "ExampleText",
  CellTags->"FrontierToDFA"],
Cell[161812, 6925, 96, 3, 26, "Input",
  CellTags->"FrontierToDFA"],
Cell[161911, 6930, 90, 1, 34, "ExampleText",
  CellTags->"FrontierToDFA"],
Cell[162004, 6933, 114, 3, 26, "Input",
  CellTags->"FrontierToDFA"],
Cell[162121, 6938, 102, 1, 34, "ExampleText",
  CellTags->"FrontierToDFA"],
Cell[162226, 6941, 142, 3, 26, "Input",
  CellTags->"FrontierToDFA"],
Cell[162371, 6946, 46, 1, 32, "ObjectName",
  CellTags->"FSMQ"],
Cell[162420, 6949, 49, 1, 50, "Highlight",
  CellTags->"FSMQ"],
Cell[162472, 6952, 154, 6, 31, "Usage",
  CellTags->"FSMQ"],
Cell[162629, 6960, 147, 5, 24, "Notes",
  CellTags->"FSMQ"],
Cell[162779, 6967, 62, 1, 39, "ExampleSection",
  CellTags->"FSMQ"],
Cell[162844, 6970, 91, 1, 34, "ExampleText",
  CellTags->"FSMQ"],
Cell[162938, 6973, 95, 2, 28, "Input",
  CellTags->"FSMQ"],
Cell[163036, 6977, 116, 3, 26, "Input",
  CellTags->"FSMQ"],
Cell[163155, 6982, 94, 3, 26, "Input",
  CellTags->"FSMQ"],
Cell[163252, 6987, 52, 1, 32, "ObjectName",
  CellTags->"FullQFA"],
Cell[163307, 6990, 52, 1, 50, "Highlight",
  CellTags->"FullQFA"],
Cell[163362, 6993, 171, 6, 31, "Usage",
  CellTags->"FullQFA"],
Cell[163536, 7001, 188, 5, 40, "Notes",
  CellTags->"FullQFA"],
Cell[163727, 7008, 257, 12, 24, "Notes",
  CellTags->"FullQFA"],
Cell[163987, 7022, 65, 1, 39, "ExampleSection",
  CellTags->"FullQFA"],
Cell[164055, 7025, 194, 6, 34, "ExampleText",
  CellTags->"FullQFA"],
Cell[164252, 7033, 216, 4, 42, "Input",
  CellTags->"FullQFA"],
Cell[164471, 7039, 157, 4, 42, "Input",
  CellTags->"FullQFA"],
Cell[164631, 7045, 82, 1, 32, "ObjectName",
  CellTags->"FullTransitionMatrixFA"],
Cell[164716, 7048, 67, 1, 50, "Highlight",
  CellTags->"FullTransitionMatrixFA"],
Cell[164786, 7051, 231, 5, 47, "Usage",
  CellTags->"FullTransitionMatrixFA"],
Cell[165020, 7058, 294, 9, 61, "Notes",
  CellTags->"FullTransitionMatrixFA"],
Cell[165317, 7069, 164, 5, 24, "Notes",
  CellTags->"FullTransitionMatrixFA"],
Cell[165484, 7076, 248, 8, 24, "Notes",
  CellTags->"FullTransitionMatrixFA"],
Cell[165735, 7086, 153, 6, 24, "Notes",
  CellTags->"FullTransitionMatrixFA"],
Cell[165891, 7094, 80, 1, 39, "ExampleSection",
  CellTags->"FullTransitionMatrixFA"],
Cell[165974, 7097, 115, 1, 34, "ExampleText",
  CellTags->"FullTransitionMatrixFA"],
Cell[166092, 7100, 108, 2, 28, "Input",
  CellTags->"FullTransitionMatrixFA"],
Cell[166203, 7104, 173, 4, 42, "Input",
  CellTags->"FullTransitionMatrixFA"],
Cell[166379, 7110, 141, 4, 34, "ExampleText",
  CellTags->"FullTransitionMatrixFA"],
Cell[166523, 7116, 130, 3, 26, "Input",
  CellTags->"FullTransitionMatrixFA"],
Cell[166656, 7121, 68, 1, 32, "ObjectName",
  CellTags->"FullTransitions"],
Cell[166727, 7124, 60, 1, 50, "Highlight",
  CellTags->"FullTransitions"],
Cell[166790, 7127, 192, 6, 31, "Usage",
  CellTags->"FullTransitions"],
Cell[166985, 7135, 383, 12, 40, "Notes",
  CellTags->"FullTransitions"],
Cell[167371, 7149, 293, 11, 24, "Notes",
  CellTags->"FullTransitions"],
Cell[167667, 7162, 73, 1, 39, "ExampleSection",
  CellTags->"FullTransitions"],
Cell[167743, 7165, 107, 1, 34, "ExampleText",
  CellTags->"FullTransitions"],
Cell[167853, 7168, 101, 2, 28, "Input",
  CellTags->"FullTransitions"],
Cell[167957, 7172, 125, 3, 26, "Input",
  CellTags->"FullTransitions"],
Cell[168085, 7177, 64, 1, 32, "ObjectName",
  CellTags->"FunctionToDFA"],
Cell[168152, 7180, 58, 1, 50, "Highlight",
  CellTags->"FunctionToDFA"],
Cell[168213, 7183, 309, 14, 31, "Usage",
  CellTags->"FunctionToDFA"],
Cell[168525, 7199, 303, 10, 61, "Notes",
  CellTags->"FunctionToDFA"],
Cell[168831, 7211, 244, 7, 24, "Notes",
  CellTags->"FunctionToDFA"],
Cell[169078, 7220, 481, 14, 56, "Notes",
  CellTags->"FunctionToDFA"],
Cell[169562, 7236, 315, 10, 40, "Notes",
  CellTags->"FunctionToDFA"],
Cell[169880, 7248, 268, 11, 24, "Notes",
  CellTags->"FunctionToDFA"],
Cell[170151, 7261, 71, 1, 39, "ExampleSection",
  CellTags->"FunctionToDFA"],
Cell[170225, 7264, 105, 1, 34, "ExampleText",
  CellTags->"FunctionToDFA"],
Cell[170333, 7267, 122, 3, 26, "Input",
  CellTags->"FunctionToDFA"],
Cell[170458, 7272, 264, 7, 42, "Input",
  CellTags->"FunctionToDFA"],
Cell[170725, 7281, 312, 10, 50, "ExampleText",
  CellTags->"FunctionToDFA"],
Cell[171040, 7293, 115, 3, 26, "Input",
  CellTags->"FunctionToDFA"],
Cell[171158, 7298, 62, 1, 32, "ObjectName",
  CellTags->"FunctionToSA"],
Cell[171223, 7301, 57, 1, 50, "Highlight",
  CellTags->"FunctionToSA"],
Cell[171283, 7304, 248, 10, 31, "Usage",
  CellTags->"FunctionToSA"],
Cell[171534, 7316, 302, 10, 61, "Notes",
  CellTags->"FunctionToSA"],
Cell[171839, 7328, 254, 7, 24, "Notes",
  CellTags->"FunctionToSA"],
Cell[172096, 7337, 316, 11, 40, "Notes",
  CellTags->"FunctionToSA"],
Cell[172415, 7350, 314, 10, 40, "Notes",
  CellTags->"FunctionToSA"],
Cell[172732, 7362, 307, 11, 24, "Notes",
  CellTags->"FunctionToSA"],
Cell[173042, 7375, 70, 1, 39, "ExampleSection",
  CellTags->"FunctionToSA"],
Cell[173115, 7378, 142, 4, 34, "ExampleText",
  CellTags->"FunctionToSA"],
Cell[173260, 7384, 251, 6, 74, "Input",
  CellTags->"FunctionToSA"],
Cell[173514, 7392, 170, 5, 26, "Input",
  CellTags->"FunctionToSA"],
Cell[173687, 7399, 83, 1, 34, "ExampleText",
  CellTags->"FunctionToSA"],
Cell[173773, 7402, 120, 3, 26, "Input",
  CellTags->"FunctionToSA"],
Cell[173896, 7407, 132, 3, 26, "Input",
  CellTags->"FunctionToSA"],
Cell[174031, 7412, 60, 1, 32, "ObjectName",
  CellTags->"GenerateDFA"],
Cell[174094, 7415, 56, 1, 50, "Highlight",
  CellTags->"GenerateDFA"],
Cell[174153, 7418, 277, 8, 47, "Usage",
  CellTags->"GenerateDFA"],
Cell[174433, 7428, 296, 10, 61, "Notes",
  CellTags->"GenerateDFA"],
Cell[174732, 7440, 534, 19, 56, "Notes",
  CellTags->"GenerateDFA"],
Cell[175269, 7461, 300, 9, 40, "Notes",
  CellTags->"GenerateDFA"],
Cell[175572, 7472, 250, 9, 24, "Notes",
  CellTags->"GenerateDFA"],
Cell[175825, 7483, 274, 11, 24, "Notes",
  CellTags->"GenerateDFA"],
Cell[176102, 7496, 69, 1, 39, "ExampleSection",
  CellTags->"GenerateDFA"],
Cell[176174, 7499, 229, 7, 34, "ExampleText",
  CellTags->"GenerateDFA"],
Cell[176406, 7508, 202, 4, 42, "Input",
  CellTags->"GenerateDFA"],
Cell[176611, 7514, 203, 6, 26, "Input",
  CellTags->"GenerateDFA"],
Cell[176817, 7522, 184, 5, 26, "Input",
  CellTags->"GenerateDFA"],
Cell[177004, 7529, 254, 7, 34, "ExampleText",
  CellTags->"GenerateDFA"],
Cell[177261, 7538, 124, 3, 28, "Input",
  CellTags->"GenerateDFA"],
Cell[177388, 7543, 157, 4, 26, "Input",
  CellTags->"GenerateDFA"],
Cell[177548, 7549, 72, 1, 32, "ObjectName",
  CellTags->"GenerateQuotients"],
Cell[177623, 7552, 62, 1, 50, "Highlight",
  CellTags->"GenerateQuotients"],
Cell[177688, 7555, 232, 8, 31, "Usage",
  CellTags->"GenerateQuotients"],
Cell[177923, 7565, 173, 5, 24, "Notes",
  CellTags->"GenerateQuotients"],
Cell[178099, 7572, 476, 19, 24, "Notes",
  CellTags->"GenerateQuotients"],
Cell[178578, 7593, 75, 1, 39, "ExampleSection",
  CellTags->"GenerateQuotients"],
Cell[178656, 7596, 146, 4, 34, "ExampleText",
  CellTags->"GenerateQuotients"],
Cell[178805, 7602, 94, 2, 28, "Input",
  CellTags->"GenerateQuotients"],
Cell[178902, 7606, 115, 3, 26, "Input",
  CellTags->"GenerateQuotients"],
Cell[179020, 7611, 298, 5, 58, "Input",
  CellTags->"GenerateQuotients"],
Cell[179321, 7618, 68, 1, 32, "ObjectName",
  CellTags->"GrowthSeriesDFA"],
Cell[179392, 7621, 60, 1, 50, "Highlight",
  CellTags->"GrowthSeriesDFA"],
Cell[179455, 7624, 281, 8, 47, "Usage",
  CellTags->"GrowthSeriesDFA"],
Cell[179739, 7634, 517, 18, 61, "Notes",
  CellTags->"GrowthSeriesDFA"],
Cell[180259, 7654, 203, 6, 40, "Notes",
  CellTags->"GrowthSeriesDFA"],
Cell[180465, 7662, 208, 6, 40, "Notes",
  CellTags->"GrowthSeriesDFA"],
Cell[180676, 7670, 161, 4, 40, "Notes",
  CellTags->"GrowthSeriesDFA"],
Cell[180840, 7676, 124, 4, 24, "Notes",
  CellTags->"GrowthSeriesDFA"],
Cell[180967, 7682, 271, 11, 24, "Notes",
  CellTags->"GrowthSeriesDFA"],
Cell[181241, 7695, 73, 1, 39, "ExampleSection",
  CellTags->"GrowthSeriesDFA"],
Cell[181317, 7698, 155, 4, 50, "ExampleText",
  CellTags->"GrowthSeriesDFA"],
Cell[181475, 7704, 203, 5, 42, "Input",
  CellTags->"GrowthSeriesDFA"],
Cell[181681, 7711, 147, 4, 42, "Input",
  CellTags->"GrowthSeriesDFA"],
Cell[181831, 7717, 117, 3, 26, "Input",
  CellTags->"GrowthSeriesDFA"],
Cell[181951, 7722, 114, 2, 34, "ExampleText",
  CellTags->"GrowthSeriesDFA"],
Cell[182068, 7726, 134, 3, 26, "Input",
  CellTags->"GrowthSeriesDFA"],
Cell[182205, 7731, 110, 1, 34, "ExampleText",
  CellTags->"GrowthSeriesDFA"],
Cell[182318, 7734, 169, 4, 26, "Input",
  CellTags->"GrowthSeriesDFA"],
Cell[182490, 7740, 66, 1, 32, "ObjectName",
  CellTags->"HomomorphismFA"],
Cell[182559, 7743, 59, 1, 50, "Highlight",
  CellTags->"HomomorphismFA"],
Cell[182621, 7746, 257, 8, 47, "Usage",
  CellTags->"HomomorphismFA"],
Cell[182881, 7756, 164, 5, 24, "Notes",
  CellTags->"HomomorphismFA"],
Cell[183048, 7763, 189, 7, 24, "Notes",
  CellTags->"HomomorphismFA"],
Cell[183240, 7772, 245, 10, 24, "Notes",
  CellTags->"HomomorphismFA"],
Cell[183488, 7784, 72, 1, 39, "ExampleSection",
  CellTags->"HomomorphismFA"],
Cell[183563, 7787, 303, 12, 34, "ExampleText",
  CellTags->"HomomorphismFA"],
Cell[183869, 7801, 222, 5, 42, "Input",
  CellTags->"HomomorphismFA"],
Cell[184094, 7808, 110, 3, 26, "Input",
  CellTags->"HomomorphismFA"],
Cell[184207, 7813, 54, 1, 32, "ObjectName",
  CellTags->"Hopcroft"],
Cell[184264, 7816, 51, 1, 50, "Highlight",
  CellTags->"Hopcroft"],
Cell[184318, 7819, 148, 6, 31, "Usage",
  CellTags->"Hopcroft"],
Cell[184469, 7827, 162, 6, 24, "Notes",
  CellTags->"Hopcroft"],
Cell[184634, 7835, 145, 4, 24, "Notes",
  CellTags->"Hopcroft"],
Cell[184782, 7841, 135, 6, 24, "Notes",
  CellTags->"Hopcroft"],
Cell[184920, 7849, 66, 1, 32, "ObjectName",
  CellTags->"HurwitzProduct"],
Cell[184989, 7852, 59, 1, 50, "Highlight",
  CellTags->"HurwitzProduct"],
Cell[185051, 7855, 164, 4, 31, "Usage",
  CellTags->"HurwitzProduct"],
Cell[185218, 7861, 84, 1, 24, "Notes",
  CellTags->"HurwitzProduct"],
Cell[185305, 7864, 418, 15, 24, "Notes",
  CellTags->"HurwitzProduct"],
Cell[185726, 7881, 72, 1, 39, "ExampleSection",
  CellTags->"HurwitzProduct"],
Cell[185801, 7884, 114, 2, 34, "ExampleText",
  CellTags->"HurwitzProduct"],
Cell[185918, 7888, 142, 6, 45, "Input",
  CellTags->"HurwitzProduct"],
Cell[186063, 7896, 160, 6, 45, "Input",
  CellTags->"HurwitzProduct"],
Cell[186226, 7904, 91, 3, 26, "Input",
  CellTags->"HurwitzProduct"],
Cell[186320, 7909, 112, 3, 28, "Input",
  CellTags->"HurwitzProduct"],
Cell[186435, 7914, 387, 7, 58, "Input",
  CellTags->"HurwitzProduct"],
Cell[186825, 7923, 196, 6, 50, "ExampleText",
  CellTags->"HurwitzProduct"],
Cell[187024, 7931, 229, 5, 42, "Input",
  CellTags->"HurwitzProduct"],
Cell[187256, 7938, 60, 1, 32, "ObjectName",
  CellTags->"IdealBaseFA"],
Cell[187319, 7941, 56, 1, 50, "Highlight",
  CellTags->"IdealBaseFA"],
Cell[187378, 7944, 183, 6, 31, "Usage",
  CellTags->"IdealBaseFA"],
Cell[187564, 7952, 254, 9, 24, "Notes",
  CellTags->"IdealBaseFA"],
Cell[187821, 7963, 238, 8, 24, "Notes",
  CellTags->"IdealBaseFA"],
Cell[188062, 7973, 258, 11, 24, "Notes",
  CellTags->"IdealBaseFA"],
Cell[188323, 7986, 69, 1, 39, "ExampleSection",
  CellTags->"IdealBaseFA"],
Cell[188395, 7989, 98, 1, 34, "ExampleText",
  CellTags->"IdealBaseFA"],
Cell[188496, 7992, 158, 4, 42, "Input",
  CellTags->"IdealBaseFA"],
Cell[188657, 7998, 375, 12, 50, "ExampleText",
  CellTags->"IdealBaseFA"],
Cell[189035, 8012, 154, 4, 42, "Input",
  CellTags->"IdealBaseFA"],
Cell[189192, 8018, 93, 1, 34, "ExampleText",
  CellTags->"IdealBaseFA"],
Cell[189288, 8021, 250, 6, 58, "Input",
  CellTags->"IdealBaseFA"],
Cell[189541, 8029, 277, 8, 50, "ExampleText",
  CellTags->"IdealBaseFA"],
Cell[189821, 8039, 287, 6, 58, "Input",
  CellTags->"IdealBaseFA"],
Cell[190111, 8047, 119, 3, 26, "Input",
  CellTags->"IdealBaseFA"],
Cell[190233, 8052, 54, 1, 32, "ObjectName",
  CellTags->"IdealQFA"],
Cell[190290, 8055, 53, 1, 50, "Highlight",
  CellTags->"IdealQFA"],
Cell[190346, 8058, 168, 6, 31, "Usage",
  CellTags->"IdealQFA"],
Cell[190517, 8066, 147, 5, 24, "Notes",
  CellTags->"IdealQFA"],
Cell[190667, 8073, 163, 7, 24, "Notes",
  CellTags->"IdealQFA"],
Cell[190833, 8082, 66, 1, 39, "ExampleSection",
  CellTags->"IdealQFA"],
Cell[190902, 8085, 95, 1, 34, "ExampleText",
  CellTags->"IdealQFA"],
Cell[191000, 8088, 124, 3, 26, "Input",
  CellTags->"IdealQFA"],
Cell[191127, 8093, 93, 3, 26, "Input",
  CellTags->"IdealQFA"],
Cell[191223, 8098, 66, 1, 32, "ObjectName",
  CellTags->"Implementation"],
Cell[191292, 8101, 57, 1, 50, "Highlight",
  CellTags->"Implementation"],
Cell[191352, 8104, 172, 6, 31, "Usage",
  CellTags->"Implementation"],
Cell[191527, 8112, 91, 1, 24, "Notes",
  CellTags->"Implementation"],
Cell[191621, 8115, 162, 7, 24, "Notes",
  CellTags->"Implementation"],
Cell[191786, 8124, 60, 1, 32, "ObjectName",
  CellTags->"InfiniteQFA"],
Cell[191849, 8127, 56, 1, 50, "Highlight",
  CellTags->"InfiniteQFA"],
Cell[191908, 8130, 181, 6, 31, "Usage",
  CellTags->"InfiniteQFA"],
Cell[192092, 8138, 130, 5, 24, "Notes",
  CellTags->"InfiniteQFA"],
Cell[192225, 8145, 157, 7, 24, "Notes",
  CellTags->"InfiniteQFA"],
Cell[192385, 8154, 69, 1, 39, "ExampleSection",
  CellTags->"InfiniteQFA"],
Cell[192457, 8157, 83, 1, 34, "ExampleText",
  CellTags->"InfiniteQFA"],
Cell[192543, 8160, 100, 2, 28, "Input",
  CellTags->"InfiniteQFA"],
Cell[192646, 8164, 100, 3, 26, "Input",
  CellTags->"InfiniteQFA"],
Cell[192749, 8169, 100, 1, 34, "ExampleText",
  CellTags->"InfiniteQFA"],
Cell[192852, 8172, 84, 2, 28, "Input",
  CellTags->"InfiniteQFA"],
Cell[192939, 8176, 101, 3, 26, "Input",
  CellTags->"InfiniteQFA"],
Cell[193043, 8181, 48, 1, 32, "ObjectName",
  CellTags->"IniLU"],
Cell[193094, 8184, 48, 1, 50, "Highlight",
  CellTags->"IniLU"],
Cell[193145, 8187, 145, 6, 31, "Usage",
  CellTags->"IniLU"],
Cell[193293, 8195, 79, 1, 24, "Notes",
  CellTags->"IniLU"],
Cell[193375, 8198, 165, 7, 24, "Notes",
  CellTags->"IniLU"],
Cell[193543, 8207, 68, 1, 32, "ObjectName",
  CellTags->"InitialVectorFA"],
Cell[193614, 8210, 60, 1, 50, "Highlight",
  CellTags->"InitialVectorFA"],
Cell[193677, 8213, 167, 4, 31, "Usage",
  CellTags->"InitialVectorFA"],
Cell[193847, 8219, 94, 1, 24, "Notes",
  CellTags->"InitialVectorFA"],
Cell[193944, 8222, 267, 11, 24, "Notes",
  CellTags->"InitialVectorFA"],
Cell[194214, 8235, 73, 1, 39, "ExampleSection",
  CellTags->"InitialVectorFA"],
Cell[194290, 8238, 151, 4, 34, "ExampleText",
  CellTags->"InitialVectorFA"],
Cell[194444, 8244, 161, 4, 26, "Input",
  CellTags->"InitialVectorFA"],
Cell[194608, 8250, 108, 3, 26, "Input",
  CellTags->"InitialVectorFA"],
Cell[194719, 8255, 52, 1, 32, "ObjectName",
  CellTags->"Initial"],
Cell[194774, 8258, 52, 1, 50, "Highlight",
  CellTags->"Initial"],
Cell[194829, 8261, 178, 5, 31, "Usage",
  CellTags->"Initial"],
Cell[195010, 8268, 259, 11, 24, "Notes",
  CellTags->"Initial"],
Cell[195272, 8281, 65, 1, 39, "ExampleSection",
  CellTags->"Initial"],
Cell[195340, 8284, 143, 4, 34, "ExampleText",
  CellTags->"Initial"],
Cell[195486, 8290, 153, 4, 26, "Input",
  CellTags->"Initial"],
Cell[195642, 8296, 92, 3, 26, "Input",
  CellTags->"Initial"],
Cell[195737, 8301, 66, 1, 32, "ObjectName",
  CellTags->"IntersectionFA"],
Cell[195806, 8304, 59, 1, 50, "Highlight",
  CellTags->"IntersectionFA"],
Cell[195868, 8307, 271, 9, 47, "Usage",
  CellTags->"IntersectionFA"],
Cell[196142, 8318, 299, 10, 61, "Notes",
  CellTags->"IntersectionFA"],
Cell[196444, 8330, 141, 5, 24, "Notes",
  CellTags->"IntersectionFA"],
Cell[196588, 8337, 203, 8, 24, "Notes",
  CellTags->"IntersectionFA"],
Cell[196794, 8347, 83, 1, 24, "Notes",
  CellTags->"IntersectionFA"],
Cell[196880, 8350, 59, 1, 24, "Notes",
  CellTags->"IntersectionFA"],
Cell[196942, 8353, 341, 15, 24, "Notes",
  CellTags->"IntersectionFA"],
Cell[197286, 8370, 72, 1, 39, "ExampleSection",
  CellTags->"IntersectionFA"],
Cell[197361, 8373, 205, 5, 50, "ExampleText",
  CellTags->"IntersectionFA"],
Cell[197569, 8380, 194, 4, 42, "Input",
  CellTags->"IntersectionFA"],
Cell[197766, 8386, 133, 4, 34, "ExampleText",
  CellTags->"IntersectionFA"],
Cell[197902, 8392, 183, 4, 42, "Input",
  CellTags->"IntersectionFA"],
Cell[198088, 8398, 64, 1, 32, "ObjectName",
  CellTags->"IrreducibleFA"],
Cell[198155, 8401, 58, 1, 50, "Highlight",
  CellTags->"IrreducibleFA"],
Cell[198216, 8404, 267, 9, 47, "Usage",
  CellTags->"IrreducibleFA"],
Cell[198486, 8415, 298, 10, 61, "Notes",
  CellTags->"IrreducibleFA"],
Cell[198787, 8427, 401, 15, 40, "Notes",
  CellTags->"IrreducibleFA"],
Cell[199191, 8444, 146, 5, 24, "Notes",
  CellTags->"IrreducibleFA"],
Cell[199340, 8451, 167, 7, 24, "Notes",
  CellTags->"IrreducibleFA"],
Cell[199510, 8460, 71, 1, 39, "ExampleSection",
  CellTags->"IrreducibleFA"],
Cell[199584, 8463, 221, 9, 34, "ExampleText",
  CellTags->"IrreducibleFA"],
Cell[199808, 8474, 82, 2, 28, "Input",
  CellTags->"IrreducibleFA"],
Cell[199893, 8478, 114, 3, 26, "Input",
  CellTags->"IrreducibleFA"],
Cell[200010, 8483, 114, 3, 26, "Input",
  CellTags->"IrreducibleFA"],
Cell[200127, 8488, 96, 1, 34, "ExampleText",
  CellTags->"IrreducibleFA"],
Cell[200226, 8491, 142, 3, 26, "Input",
  CellTags->"IrreducibleFA"],
Cell[200371, 8496, 66, 1, 32, "ObjectName",
  CellTags->"IsomorphicQDFA"],
Cell[200440, 8499, 59, 1, 50, "Highlight",
  CellTags->"IsomorphicQDFA"],
Cell[200502, 8502, 157, 4, 31, "Usage",
  CellTags->"IsomorphicQDFA"],
Cell[200662, 8508, 300, 10, 61, "Notes",
  CellTags->"IsomorphicQDFA"],
Cell[200965, 8520, 150, 4, 24, "Notes",
  CellTags->"IsomorphicQDFA"],
Cell[201118, 8526, 167, 5, 24, "Notes",
  CellTags->"IsomorphicQDFA"],
Cell[201288, 8533, 163, 4, 40, "Notes",
  CellTags->"IsomorphicQDFA"],
Cell[201454, 8539, 72, 1, 39, "ExampleSection",
  CellTags->"IsomorphicQDFA"],
Cell[201529, 8542, 77, 1, 34, "ExampleText",
  CellTags->"IsomorphicQDFA"],
Cell[201609, 8545, 335, 8, 74, "Input",
  CellTags->"IsomorphicQDFA"],
Cell[201947, 8555, 110, 3, 26, "Input",
  CellTags->"IsomorphicQDFA"],
Cell[202060, 8560, 132, 3, 26, "Input",
  CellTags->"IsomorphicQDFA"],
Cell[202195, 8565, 60, 1, 32, "ObjectName",
  CellTags->"IthSymbolFA"],
Cell[202258, 8568, 56, 1, 50, "Highlight",
  CellTags->"IthSymbolFA"],
Cell[202317, 8571, 226, 7, 47, "Usage",
  CellTags->"IthSymbolFA"],
Cell[202546, 8580, 296, 10, 61, "Notes",
  CellTags->"IthSymbolFA"],
Cell[202845, 8592, 212, 6, 40, "Notes",
  CellTags->"IthSymbolFA"],
Cell[203060, 8600, 312, 12, 24, "Notes",
  CellTags->"IthSymbolFA"],
Cell[203375, 8614, 166, 7, 24, "Notes",
  CellTags->"IthSymbolFA"],
Cell[203544, 8623, 69, 1, 39, "ExampleSection",
  CellTags->"IthSymbolFA"],
Cell[203616, 8626, 206, 6, 34, "ExampleText",
  CellTags->"IthSymbolFA"],
Cell[203825, 8634, 113, 3, 26, "Input",
  CellTags->"IthSymbolFA"],
Cell[203941, 8639, 111, 3, 26, "Input",
  CellTags->"IthSymbolFA"],
Cell[204055, 8644, 129, 4, 34, "ExampleText",
  CellTags->"IthSymbolFA"],
Cell[204187, 8650, 141, 3, 26, "Input",
  CellTags->"IthSymbolFA"],
Cell[204331, 8655, 189, 6, 34, "ExampleText",
  CellTags->"IthSymbolFA"],
Cell[204523, 8663, 174, 5, 26, "Input",
  CellTags->"IthSymbolFA"],
Cell[204700, 8670, 112, 3, 26, "Input",
  CellTags->"IthSymbolFA"],
Cell[204815, 8675, 165, 4, 50, "ExampleText",
  CellTags->"IthSymbolFA"],
Cell[204983, 8681, 193, 5, 58, "Input",
  CellTags->"IthSymbolFA"],
Cell[205179, 8688, 50, 1, 32, "ObjectName",
  CellTags->"JoinSA"],
Cell[205232, 8691, 51, 1, 50, "Highlight",
  CellTags->"JoinSA"],
Cell[205286, 8694, 190, 6, 31, "Usage",
  CellTags->"JoinSA"],
Cell[205479, 8702, 246, 9, 44, "Notes",
  CellTags->"JoinSA"],
Cell[205728, 8713, 158, 5, 24, "Notes",
  CellTags->"JoinSA"],
Cell[205889, 8720, 149, 7, 24, "Notes",
  CellTags->"JoinSA"],
Cell[206041, 8729, 64, 1, 39, "ExampleSection",
  CellTags->"JoinSA"],
Cell[206108, 8732, 169, 4, 50, "ExampleText",
  CellTags->"JoinSA"],
Cell[206280, 8738, 271, 7, 42, "Input",
  CellTags->"JoinSA"],
Cell[206554, 8747, 92, 1, 34, "ExampleText",
  CellTags->"JoinSA"],
Cell[206649, 8750, 118, 3, 26, "Input",
  CellTags->"JoinSA"],
Cell[206770, 8755, 85, 1, 34, "ExampleText",
  CellTags->"JoinSA"],
Cell[206858, 8758, 130, 3, 26, "Input",
  CellTags->"JoinSA"],
Cell[206991, 8763, 229, 7, 50, "ExampleText",
  CellTags->"JoinSA"],
Cell[207223, 8772, 226, 5, 58, "Input",
  CellTags->"JoinSA"],
Cell[207452, 8779, 58, 1, 32, "ObjectName",
  CellTags->"LanguageFA"],
Cell[207513, 8782, 55, 1, 50, "Highlight",
  CellTags->"LanguageFA"],
Cell[207571, 8785, 207, 8, 31, "Usage",
  CellTags->"LanguageFA"],
Cell[207781, 8795, 474, 14, 133, "Notes",
  CellTags->"LanguageFA"],
Cell[208258, 8811, 181, 7, 24, "Notes",
  CellTags->"LanguageFA"],
Cell[208442, 8820, 211, 7, 24, "Notes",
  CellTags->"LanguageFA"],
Cell[208656, 8829, 183, 5, 40, "Notes",
  CellTags->"LanguageFA"],
Cell[208842, 8836, 156, 5, 24, "Notes",
  CellTags->"LanguageFA"],
Cell[209001, 8843, 347, 10, 56, "Notes",
  CellTags->"LanguageFA"],
Cell[209351, 8855, 398, 14, 40, "Notes",
  CellTags->"LanguageFA"],
Cell[209752, 8871, 205, 7, 24, "Notes",
  CellTags->"LanguageFA"],
Cell[209960, 8880, 278, 12, 24, "Notes",
  CellTags->"LanguageFA"],
Cell[210241, 8894, 68, 1, 39, "ExampleSection",
  CellTags->"LanguageFA"],
Cell[210312, 8897, 134, 4, 34, "ExampleText",
  CellTags->"LanguageFA"],
Cell[210449, 8903, 193, 5, 42, "Input",
  CellTags->"LanguageFA"],
Cell[210645, 8910, 105, 3, 26, "Input",
  CellTags->"LanguageFA"],
Cell[210753, 8915, 154, 4, 26, "Input",
  CellTags->"LanguageFA"],
Cell[210910, 8921, 110, 2, 34, "ExampleText",
  CellTags->"LanguageFA"],
Cell[211023, 8925, 218, 5, 42, "Input",
  CellTags->"LanguageFA"],
Cell[211244, 8932, 175, 4, 42, "Input",
  CellTags->"LanguageFA"],
Cell[211422, 8938, 124, 4, 34, "ExampleText",
  CellTags->"LanguageFA"],
Cell[211549, 8944, 127, 3, 26, "Input",
  CellTags->"LanguageFA"],
Cell[211679, 8949, 144, 4, 34, "ExampleText",
  CellTags->"LanguageFA"],
Cell[211826, 8955, 224, 5, 42, "Input",
  CellTags->"LanguageFA"],
Cell[212053, 8962, 128, 3, 26, "Input",
  CellTags->"LanguageFA"],
Cell[212184, 8967, 127, 3, 26, "Input",
  CellTags->"LanguageFA"],
Cell[212314, 8972, 85, 1, 34, "ExampleText",
  CellTags->"LanguageFA"],
Cell[212402, 8975, 114, 3, 26, "Input",
  CellTags->"LanguageFA"],
Cell[212519, 8980, 128, 3, 26, "Input",
  CellTags->"LanguageFA"],
Cell[212650, 8985, 116, 3, 26, "Input",
  CellTags->"LanguageFA"],
Cell[212769, 8990, 62, 1, 32, "ObjectName",
  CellTags->"LanguageSort"],
Cell[212834, 8993, 57, 1, 50, "Highlight",
  CellTags->"LanguageSort"],
Cell[212894, 8996, 207, 6, 31, "Usage",
  CellTags->"LanguageSort"],
Cell[213104, 9004, 145, 5, 24, "Notes",
  CellTags->"LanguageSort"],
Cell[213252, 9011, 178, 7, 24, "Notes",
  CellTags->"LanguageSort"],
Cell[213433, 9020, 70, 1, 39, "ExampleSection",
  CellTags->"LanguageSort"],
Cell[213506, 9023, 171, 4, 50, "ExampleText",
  CellTags->"LanguageSort"],
Cell[213680, 9029, 246, 6, 42, "Input",
  CellTags->"LanguageSort"],
Cell[213929, 9037, 204, 5, 42, "Input",
  CellTags->"LanguageSort"],
Cell[214136, 9044, 70, 1, 32, "ObjectName",
  CellTags->"LanguageToSeries"],
Cell[214209, 9047, 61, 1, 50, "Highlight",
  CellTags->"LanguageToSeries"],
Cell[214273, 9050, 175, 6, 31, "Usage",
  CellTags->"LanguageToSeries"],
Cell[214451, 9058, 156, 5, 24, "Notes",
  CellTags->"LanguageToSeries"],
Cell[214610, 9065, 269, 11, 24, "Notes",
  CellTags->"LanguageToSeries"],
Cell[214882, 9078, 74, 1, 39, "ExampleSection",
  CellTags->"LanguageToSeries"],
Cell[214959, 9081, 77, 1, 34, "ExampleText",
  CellTags->"LanguageToSeries"],
Cell[215039, 9084, 83, 2, 28, "Input",
  CellTags->"LanguageToSeries"],
Cell[215125, 9088, 110, 3, 26, "Input",
  CellTags->"LanguageToSeries"],
Cell[215238, 9093, 102, 3, 26, "Input",
  CellTags->"LanguageToSeries"],
Cell[215343, 9098, 62, 1, 32, "ObjectName",
  CellTags->"LeftQuotient"],
Cell[215408, 9101, 57, 1, 50, "Highlight",
  CellTags->"LeftQuotient"],
Cell[215468, 9104, 231, 8, 31, "Usage",
  CellTags->"LeftQuotient"],
Cell[215702, 9114, 182, 7, 24, "Notes",
  CellTags->"LeftQuotient"],
Cell[215887, 9123, 481, 19, 24, "Notes",
  CellTags->"LeftQuotient"],
Cell[216371, 9144, 70, 1, 39, "ExampleSection",
  CellTags->"LeftQuotient"],
Cell[216444, 9147, 145, 4, 34, "ExampleText",
  CellTags->"LeftQuotient"],
Cell[216592, 9153, 89, 2, 28, "Input",
  CellTags->"LeftQuotient"],
Cell[216684, 9157, 105, 3, 26, "Input",
  CellTags->"LeftQuotient"],
Cell[216792, 9162, 106, 3, 26, "Input",
  CellTags->"LeftQuotient"],
Cell[216901, 9167, 107, 3, 26, "Input",
  CellTags->"LeftQuotient"],
Cell[217011, 9172, 106, 3, 26, "Input",
  CellTags->"LeftQuotient"],
Cell[217120, 9177, 56, 1, 32, "ObjectName",
  CellTags->"LengthDFA"],
Cell[217179, 9180, 54, 1, 50, "Highlight",
  CellTags->"LengthDFA"],
Cell[217236, 9183, 228, 8, 31, "Usage",
  CellTags->"LengthDFA"],
Cell[217467, 9193, 293, 10, 61, "Notes",
  CellTags->"LengthDFA"],
Cell[217763, 9205, 192, 9, 24, "Notes",
  CellTags->"LengthDFA"],
Cell[217958, 9216, 229, 9, 24, "Notes",
  CellTags->"LengthDFA"],
Cell[218190, 9227, 161, 5, 24, "Notes",
  CellTags->"LengthDFA"],
Cell[218354, 9234, 162, 7, 24, "Notes",
  CellTags->"LengthDFA"],
Cell[218519, 9243, 67, 1, 39, "ExampleSection",
  CellTags->"LengthDFA"],
Cell[218589, 9246, 92, 1, 34, "ExampleText",
  CellTags->"LengthDFA"],
Cell[218684, 9249, 130, 3, 26, "Input",
  CellTags->"LengthDFA"],
Cell[218817, 9254, 134, 3, 26, "Input",
  CellTags->"LengthDFA"],
Cell[218954, 9259, 94, 1, 34, "ExampleText",
  CellTags->"LengthDFA"],
Cell[219051, 9262, 194, 4, 42, "Input",
  CellTags->"LengthDFA"],
Cell[219248, 9268, 66, 1, 34, "ExampleText",
  CellTags->"LengthDFA"],
Cell[219317, 9271, 221, 5, 42, "Input",
  CellTags->"LengthDFA"],
Cell[219541, 9278, 54, 1, 32, "ObjectName",
  CellTags->"LocalQSA"],
Cell[219598, 9281, 53, 1, 50, "Highlight",
  CellTags->"LocalQSA"],
Cell[219654, 9284, 170, 6, 31, "Usage",
  CellTags->"LocalQSA"],
Cell[219827, 9292, 224, 9, 44, "Notes",
  CellTags->"LocalQSA"],
Cell[220054, 9303, 220, 7, 40, "Notes",
  CellTags->"LocalQSA"],
Cell[220277, 9312, 212, 6, 40, "Notes",
  CellTags->"LocalQSA"],
Cell[220492, 9320, 261, 11, 24, "Notes",
  CellTags->"LocalQSA"],
Cell[220756, 9333, 66, 1, 39, "ExampleSection",
  CellTags->"LocalQSA"],
Cell[220825, 9336, 228, 5, 50, "ExampleText",
  CellTags->"LocalQSA"],
Cell[221056, 9343, 109, 3, 26, "Input",
  CellTags->"LocalQSA"],
Cell[221168, 9348, 145, 4, 42, "Input",
  CellTags->"LocalQSA"],
Cell[221316, 9354, 96, 3, 26, "Input",
  CellTags->"LocalQSA"],
Cell[221415, 9359, 115, 3, 26, "Input",
  CellTags->"LocalQSA"],
Cell[221533, 9364, 63, 1, 34, "ExampleText",
  CellTags->"LocalQSA"],
Cell[221599, 9367, 122, 3, 26, "Input",
  CellTags->"LocalQSA"],
Cell[221724, 9372, 70, 1, 32, "ObjectName",
  CellTags->"LocalUniversalFA"],
Cell[221797, 9375, 61, 1, 50, "Highlight",
  CellTags->"LocalUniversalFA"],
Cell[221861, 9378, 231, 8, 31, "Usage",
  CellTags->"LocalUniversalFA"],
Cell[222095, 9388, 437, 12, 97, "Notes",
  CellTags->"LocalUniversalFA"],
Cell[222535, 9402, 292, 11, 40, "Notes",
  CellTags->"LocalUniversalFA"],
Cell[222830, 9415, 83, 1, 24, "Notes",
  CellTags->"LocalUniversalFA"],
Cell[222916, 9418, 175, 7, 24, "Notes",
  CellTags->"LocalUniversalFA"],
Cell[223094, 9427, 74, 1, 39, "ExampleSection",
  CellTags->"LocalUniversalFA"],
Cell[223171, 9430, 335, 9, 50, "ExampleText",
  CellTags->"LocalUniversalFA"],
Cell[223509, 9441, 296, 7, 58, "Input",
  CellTags->"LocalUniversalFA"],
Cell[223808, 9450, 100, 3, 26, "Input",
  CellTags->"LocalUniversalFA"],
Cell[223911, 9455, 173, 6, 34, "ExampleText",
  CellTags->"LocalUniversalFA"],
Cell[224087, 9463, 118, 3, 26, "Input",
  CellTags->"LocalUniversalFA"],
Cell[224208, 9468, 68, 1, 32, "ObjectName",
  CellTags->"LoopAutomatonFA"],
Cell[224279, 9471, 60, 1, 50, "Highlight",
  CellTags->"LoopAutomatonFA"],
Cell[224342, 9474, 172, 6, 31, "Usage",
  CellTags->"LoopAutomatonFA"],
Cell[224517, 9482, 104, 1, 24, "Notes",
  CellTags->"LoopAutomatonFA"],
Cell[224624, 9485, 271, 11, 24, "Notes",
  CellTags->"LoopAutomatonFA"],
Cell[224898, 9498, 73, 1, 39, "ExampleSection",
  CellTags->"LoopAutomatonFA"],
Cell[224974, 9501, 176, 4, 50, "ExampleText",
  CellTags->"LoopAutomatonFA"],
Cell[225153, 9507, 197, 4, 42, "Input",
  CellTags->"LoopAutomatonFA"],
Cell[225353, 9513, 121, 3, 26, "Input",
  CellTags->"LoopAutomatonFA"],
Cell[225477, 9518, 219, 6, 50, "ExampleText",
  CellTags->"LoopAutomatonFA"],
Cell[225699, 9526, 190, 4, 42, "Input",
  CellTags->"LoopAutomatonFA"],
Cell[225892, 9532, 128, 4, 34, "ExampleText",
  CellTags->"LoopAutomatonFA"],
Cell[226023, 9538, 167, 4, 26, "Input",
  CellTags->"LoopAutomatonFA"],
Cell[226193, 9544, 76, 1, 32, "ObjectName",
  CellTags->"MakeHomomorphismDFA"],
Cell[226272, 9547, 64, 1, 50, "Highlight",
  CellTags->"MakeHomomorphismDFA"],
Cell[226339, 9550, 284, 10, 47, "Usage",
  CellTags->"MakeHomomorphismDFA"],
Cell[226626, 9562, 185, 7, 24, "Notes",
  CellTags->"MakeHomomorphismDFA"],
Cell[226814, 9571, 120, 5, 24, "Notes",
  CellTags->"MakeHomomorphismDFA"],
Cell[226937, 9578, 173, 7, 24, "Notes",
  CellTags->"MakeHomomorphismDFA"],
Cell[227113, 9587, 221, 9, 24, "Notes",
  CellTags->"MakeHomomorphismDFA"],
Cell[227337, 9598, 77, 1, 39, "ExampleSection",
  CellTags->"MakeHomomorphismDFA"],
Cell[227417, 9601, 98, 1, 34, "ExampleText",
  CellTags->"MakeHomomorphismDFA"],
Cell[227518, 9604, 175, 4, 42, "Input",
  CellTags->"MakeHomomorphismDFA"],
Cell[227696, 9610, 140, 4, 34, "ExampleText",
  CellTags->"MakeHomomorphismDFA"],
Cell[227839, 9616, 132, 3, 26, "Input",
  CellTags->"MakeHomomorphismDFA"],
Cell[227974, 9621, 237, 5, 58, "Input",
  CellTags->"MakeHomomorphismDFA"],
Cell[228214, 9628, 70, 1, 32, "ObjectName",
  CellTags->"MakeHomomorphism"],
Cell[228287, 9631, 61, 1, 50, "Highlight",
  CellTags->"MakeHomomorphism"],
Cell[228351, 9634, 267, 10, 47, "Usage",
  CellTags->"MakeHomomorphism"],
Cell[228621, 9646, 231, 9, 44, "Notes",
  CellTags->"MakeHomomorphism"],
Cell[228855, 9657, 193, 7, 24, "Notes",
  CellTags->"MakeHomomorphism"],
Cell[229051, 9666, 117, 5, 24, "Notes",
  CellTags->"MakeHomomorphism"],
Cell[229171, 9673, 208, 8, 24, "Notes",
  CellTags->"MakeHomomorphism"],
Cell[229382, 9683, 177, 7, 24, "Notes",
  CellTags->"MakeHomomorphism"],
Cell[229562, 9692, 74, 1, 39, "ExampleSection",
  CellTags->"MakeHomomorphism"],
Cell[229639, 9695, 148, 4, 34, "ExampleText",
  CellTags->"MakeHomomorphism"],
Cell[229790, 9701, 107, 2, 28, "Input",
  CellTags->"MakeHomomorphism"],
Cell[229900, 9705, 184, 5, 42, "Input",
  CellTags->"MakeHomomorphism"],
Cell[230087, 9712, 132, 2, 34, "ExampleText",
  Evaluatable->False,
  CellTags->"MakeHomomorphism"],
Cell[230222, 9716, 104, 3, 26, "Input",
  CellTags->"MakeHomomorphism"],
Cell[230329, 9721, 54, 1, 32, "ObjectName",
  CellTags->"MakeTrim"],
Cell[230386, 9724, 51, 1, 50, "Highlight",
  CellTags->"MakeTrim"],
Cell[230440, 9727, 154, 6, 31, "Usage",
  CellTags->"MakeTrim"],
Cell[230597, 9735, 147, 4, 24, "Notes",
  CellTags->"MakeTrim"],
Cell[230747, 9741, 161, 7, 24, "Notes",
  CellTags->"MakeTrim"],
Cell[230911, 9750, 76, 1, 32, "ObjectName",
  CellTags->"MatrixToTransitions"],
Cell[230990, 9753, 64, 1, 50, "Highlight",
  CellTags->"MatrixToTransitions"],
Cell[231057, 9756, 232, 6, 47, "Usage",
  CellTags->"MatrixToTransitions"],
Cell[231292, 9764, 97, 1, 24, "Notes",
  CellTags->"MatrixToTransitions"],
Cell[231392, 9767, 280, 11, 24, "Notes",
  CellTags->"MatrixToTransitions"],
Cell[231675, 9780, 77, 1, 39, "ExampleSection",
  CellTags->"MatrixToTransitions"],
Cell[231755, 9783, 130, 4, 34, "ExampleText",
  CellTags->"MatrixToTransitions"],
Cell[231888, 9789, 149, 3, 26, "Input",
  CellTags->"MatrixToTransitions"],
Cell[232040, 9794, 100, 2, 28, "Input",
  CellTags->"MatrixToTransitions"],
Cell[232143, 9798, 70, 1, 32, "ObjectName",
  CellTags->"MinimalFischerFA"],
Cell[232216, 9801, 61, 1, 50, "Highlight",
  CellTags->"MinimalFischerFA"],
Cell[232280, 9804, 207, 6, 47, "Usage",
  CellTags->"MinimalFischerFA"],
Cell[232490, 9812, 368, 11, 79, "Notes",
  CellTags->"MinimalFischerFA"],
Cell[232861, 9825, 250, 8, 40, "Notes",
  CellTags->"MinimalFischerFA"],
Cell[233114, 9835, 218, 6, 40, "Notes",
  CellTags->"MinimalFischerFA"],
Cell[233335, 9843, 74, 1, 39, "ExampleSection",
  CellTags->"MinimalFischerFA"],
Cell[233412, 9846, 106, 1, 34, "ExampleText",
  CellTags->"MinimalFischerFA"],
Cell[233521, 9849, 92, 2, 28, "Input",
  CellTags->"MinimalFischerFA"],
Cell[233616, 9853, 118, 3, 26, "Input",
  CellTags->"MinimalFischerFA"],
Cell[233737, 9858, 111, 3, 26, "Input",
  CellTags->"MinimalFischerFA"],
Cell[233851, 9863, 115, 3, 26, "Input",
  CellTags->"MinimalFischerFA"],
Cell[233969, 9868, 60, 1, 32, "ObjectName",
  CellTags->"MinimizeDFA"],
Cell[234032, 9871, 56, 1, 50, "Highlight",
  CellTags->"MinimizeDFA"],
Cell[234091, 9874, 169, 6, 31, "Usage",
  CellTags->"MinimizeDFA"],
Cell[234263, 9882, 348, 11, 79, "Notes",
  CellTags->"MinimizeDFA"],
Cell[234614, 9895, 305, 9, 40, "Notes",
  CellTags->"MinimizeDFA"],
Cell[234922, 9906, 146, 5, 24, "Notes",
  CellTags->"MinimizeDFA"],
Cell[235071, 9913, 272, 8, 40, "Notes",
  CellTags->"MinimizeDFA"],
Cell[235346, 9923, 270, 11, 24, "Notes",
  CellTags->"MinimizeDFA"],
Cell[235619, 9936, 69, 1, 39, "ExampleSection",
  CellTags->"MinimizeDFA"],
Cell[235691, 9939, 78, 1, 34, "ExampleText",
  CellTags->"MinimizeDFA"],
Cell[235772, 9942, 138, 3, 26, "Input",
  CellTags->"MinimizeDFA"],
Cell[235913, 9947, 286, 8, 74, "Input",
  CellTags->"MinimizeDFA"],
Cell[236202, 9957, 58, 1, 32, "ObjectName",
  CellTags->"MinimizeFA"],
Cell[236263, 9960, 55, 1, 50, "Highlight",
  CellTags->"MinimizeFA"],
Cell[236321, 9963, 232, 6, 47, "Usage",
  CellTags->"MinimizeFA"],
Cell[236556, 9971, 365, 11, 79, "Notes",
  CellTags->"MinimizeFA"],
Cell[236924, 9984, 222, 6, 40, "Notes",
  CellTags->"MinimizeFA"],
Cell[237149, 9992, 209, 6, 40, "Notes",
  CellTags->"MinimizeFA"],
Cell[237361, 10000, 362, 15, 24, "Notes",
  CellTags->"MinimizeFA"],
Cell[237726, 10017, 68, 1, 39, "ExampleSection",
  CellTags->"MinimizeFA"],
Cell[237797, 10020, 158, 4, 50, "ExampleText",
  CellTags->"MinimizeFA"],
Cell[237958, 10026, 125, 5, 45, "Input",
  CellTags->"MinimizeFA"],
Cell[238086, 10033, 147, 4, 26, "Input",
  CellTags->"MinimizeFA"],
Cell[238236, 10039, 44, 1, 32, "ObjectName",
  CellTags->"NFA"],
Cell[238283, 10042, 51, 1, 50, "Highlight",
  CellTags->"NFA"],
Cell[238337, 10045, 136, 4, 31, "Usage",
  CellTags->"NFA"],
Cell[238476, 10051, 276, 11, 24, "Notes",
  CellTags->"NFA"],
Cell[238755, 10064, 169, 7, 24, "Notes",
  CellTags->"NFA"],
Cell[238927, 10073, 262, 9, 24, "Notes",
  CellTags->"NFA"],
Cell[239192, 10084, 184, 7, 24, "Notes",
  CellTags->"NFA"],
Cell[239379, 10093, 302, 15, 24, "Notes",
  CellTags->"NFA"],
Cell[239684, 10110, 61, 1, 39, "ExampleSection",
  CellTags->"NFA"],
Cell[239748, 10113, 54, 1, 34, "ExampleText",
  CellTags->"NFA"],
Cell[239805, 10116, 173, 5, 26, "Input",
  CellTags->"NFA"],
Cell[239981, 10123, 109, 3, 26, "Input",
  CellTags->"NFA"],
Cell[240093, 10128, 54, 1, 32, "ObjectName",
  CellTags->"NNFAToFA"],
Cell[240150, 10131, 53, 1, 50, "Highlight",
  CellTags->"NNFAToFA"],
Cell[240206, 10134, 177, 6, 31, "Usage",
  CellTags->"NNFAToFA"],
Cell[240386, 10142, 140, 4, 24, "Notes",
  CellTags->"NNFAToFA"],
Cell[240529, 10148, 187, 7, 24, "Notes",
  CellTags->"NNFAToFA"],
Cell[240719, 10157, 66, 1, 39, "ExampleSection",
  CellTags->"NNFAToFA"],
Cell[240788, 10160, 242, 7, 34, "ExampleText",
  CellTags->"NNFAToFA"],
Cell[241033, 10169, 110, 2, 28, "Input",
  CellTags->"NNFAToFA"],
Cell[241146, 10173, 109, 3, 26, "Input",
  CellTags->"NNFAToFA"],
Cell[241258, 10178, 67, 1, 34, "ExampleText",
  CellTags->"NNFAToFA"],
Cell[241328, 10181, 77, 2, 28, "Input",
  CellTags->"NNFAToFA"],
Cell[241408, 10185, 103, 3, 26, "Input",
  CellTags->"NNFAToFA"],
Cell[241514, 10190, 46, 1, 32, "ObjectName",
  CellTags->"NNFA"],
Cell[241563, 10193, 52, 1, 50, "Highlight",
  CellTags->"NNFA"],
Cell[241618, 10196, 157, 4, 31, "Usage",
  CellTags->"NNFA"],
Cell[241778, 10202, 291, 6, 56, "Notes",
  CellTags->"NNFA"],
Cell[242072, 10210, 270, 11, 24, "Notes",
  CellTags->"NNFA"],
Cell[242345, 10223, 62, 1, 39, "ExampleSection",
  CellTags->"NNFA"],
Cell[242410, 10226, 56, 1, 34, "ExampleText",
  CellTags->"NNFA"],
Cell[242469, 10229, 104, 2, 28, "Input",
  CellTags->"NNFA"],
Cell[242576, 10233, 56, 1, 32, "ObjectName",
  CellTags->"Normalize"],
Cell[242635, 10236, 52, 1, 50, "Highlight",
  CellTags->"Normalize"],
Cell[242690, 10239, 137, 4, 31, "Usage",
  CellTags->"Normalize"],
Cell[242830, 10245, 269, 9, 40, "Notes",
  CellTags->"Normalize"],
Cell[243102, 10256, 813, 26, 88, "Notes",
  CellTags->"Normalize"],
Cell[243918, 10284, 147, 5, 24, "Notes",
  CellTags->"Normalize"],
Cell[244068, 10291, 268, 11, 24, "Notes",
  CellTags->"Normalize"],
Cell[244339, 10304, 67, 1, 39, "ExampleSection",
  CellTags->"Normalize"],
Cell[244409, 10307, 79, 1, 34, "ExampleText",
  CellTags->"Normalize"],
Cell[244491, 10310, 268, 6, 74, "Input",
  CellTags->"Normalize"],
Cell[244762, 10318, 100, 3, 26, "Input",
  CellTags->"Normalize"],
Cell[244865, 10323, 126, 4, 34, "ExampleText",
  CellTags->"Normalize"],
Cell[244994, 10329, 134, 3, 26, "Input",
  CellTags->"Normalize"],
Cell[245131, 10334, 193, 5, 50, "ExampleText",
  CellTags->"Normalize"],
Cell[245327, 10341, 134, 3, 26, "Input",
  CellTags->"Normalize"],
Cell[245464, 10346, 191, 5, 50, "ExampleText",
  CellTags->"Normalize"],
Cell[245658, 10353, 331, 6, 74, "Input",
  CellTags->"Normalize"],
Cell[245992, 10361, 70, 1, 32, "ObjectName",
  CellTags->"NormalizeControl"],
Cell[246065, 10364, 61, 1, 50, "Highlight",
  CellTags->"NormalizeControl"],
Cell[246129, 10367, 158, 4, 31, "Usage",
  CellTags->"NormalizeControl"],
Cell[246290, 10373, 176, 5, 24, "Notes",
  CellTags->"NormalizeControl"],
Cell[246469, 10380, 154, 5, 24, "Notes",
  CellTags->"NormalizeControl"],
Cell[246626, 10387, 163, 4, 40, "Notes",
  CellTags->"NormalizeControl"],
Cell[246792, 10393, 74, 1, 39, "ExampleSection",
  CellTags->"NormalizeControl"],
Cell[246869, 10396, 157, 4, 34, "ExampleText",
  CellTags->"NormalizeControl"],
Cell[247029, 10402, 152, 4, 42, "Input",
  CellTags->"NormalizeControl"],
Cell[247184, 10408, 173, 4, 42, "Input",
  CellTags->"NormalizeControl"],
Cell[247360, 10414, 128, 4, 34, "ExampleText",
  CellTags->"NormalizeControl"],
Cell[247491, 10420, 187, 5, 58, "Input",
  CellTags->"NormalizeControl"],
Cell[247681, 10427, 60, 1, 32, "ObjectName",
  CellTags->"NormalizeFA"],
Cell[247744, 10430, 56, 1, 50, "Highlight",
  CellTags->"NormalizeFA"],
Cell[247803, 10433, 160, 6, 31, "Usage",
  CellTags->"NormalizeFA"],
Cell[247966, 10441, 291, 9, 40, "Notes",
  CellTags->"NormalizeFA"],
Cell[248260, 10452, 139, 4, 24, "Notes",
  CellTags->"NormalizeFA"],
Cell[248402, 10458, 185, 7, 24, "Notes",
  CellTags->"NormalizeFA"],
Cell[248590, 10467, 69, 1, 39, "ExampleSection",
  CellTags->"NormalizeFA"],
Cell[248662, 10470, 133, 4, 34, "ExampleText",
  CellTags->"NormalizeFA"],
Cell[248798, 10476, 171, 4, 42, "Input",
  CellTags->"NormalizeFA"],
Cell[248972, 10482, 138, 4, 42, "Input",
  CellTags->"NormalizeFA"],
Cell[249113, 10488, 60, 1, 32, "ObjectName",
  CellTags->"NumberOfDFA"],
Cell[249176, 10491, 56, 1, 50, "Highlight",
  CellTags->"NumberOfDFA"],
Cell[249235, 10494, 210, 8, 31, "Usage",
  CellTags->"NumberOfDFA"],
Cell[249448, 10504, 171, 5, 24, "Notes",
  CellTags->"NumberOfDFA"],
Cell[249622, 10511, 354, 15, 24, "Notes",
  CellTags->"NumberOfDFA"],
Cell[249979, 10528, 69, 1, 39, "ExampleSection",
  CellTags->"NumberOfDFA"],
Cell[250051, 10531, 91, 1, 34, "ExampleText",
  CellTags->"NumberOfDFA"],
Cell[250145, 10534, 115, 2, 28, "Input",
  CellTags->"NumberOfDFA"],
Cell[250263, 10538, 66, 1, 32, "ObjectName",
  CellTags->"NumTransitions"],
Cell[250332, 10541, 59, 1, 50, "Highlight",
  CellTags->"NumTransitions"],
Cell[250394, 10544, 179, 6, 31, "Usage",
  CellTags->"NumTransitions"],
Cell[250576, 10552, 155, 7, 24, "Notes",
  CellTags->"NumTransitions"],
Cell[250734, 10561, 72, 1, 39, "ExampleSection",
  CellTags->"NumTransitions"],
Cell[250809, 10564, 109, 1, 34, "ExampleText",
  CellTags->"NumTransitions"],
Cell[250921, 10567, 209, 5, 42, "Input",
  CellTags->"NumTransitions"],
Cell[251133, 10574, 106, 3, 26, "Input",
  CellTags->"NumTransitions"],
Cell[251242, 10579, 64, 1, 32, "ObjectName",
  CellTags->"PermutationSA"],
Cell[251309, 10582, 58, 1, 50, "Highlight",
  CellTags->"PermutationSA"],
Cell[251370, 10585, 213, 6, 47, "Usage",
  CellTags->"PermutationSA"],
Cell[251586, 10593, 267, 9, 40, "Notes",
  CellTags->"PermutationSA"],
Cell[251856, 10604, 213, 9, 24, "Notes",
  CellTags->"PermutationSA"],
Cell[252072, 10615, 71, 1, 39, "ExampleSection",
  CellTags->"PermutationSA"],
Cell[252146, 10618, 87, 1, 34, "ExampleText",
  CellTags->"PermutationSA"],
Cell[252236, 10621, 113, 2, 28, "Input",
  CellTags->"PermutationSA"],
Cell[252352, 10625, 131, 4, 34, "ExampleText",
  CellTags->"PermutationSA"],
Cell[252486, 10631, 156, 4, 42, "Input",
  CellTags->"PermutationSA"],
Cell[252645, 10637, 176, 4, 26, "Input",
  CellTags->"PermutationSA"],
Cell[252824, 10643, 55, 1, 26, "Input",
  CellTags->"PermutationSA"],
Cell[252882, 10646, 56, 1, 32, "ObjectName",
  CellTags->"PermuteFA"],
Cell[252941, 10649, 54, 1, 50, "Highlight",
  CellTags->"PermuteFA"],
Cell[252998, 10652, 215, 8, 31, "Usage",
  CellTags->"PermuteFA"],
Cell[253216, 10662, 211, 7, 24, "Notes",
  CellTags->"PermuteFA"],
Cell[253430, 10671, 67, 1, 39, "ExampleSection",
  CellTags->"PermuteFA"],
Cell[253500, 10674, 104, 1, 34, "ExampleText",
  CellTags->"PermuteFA"],
Cell[253607, 10677, 106, 2, 28, "Input",
  CellTags->"PermuteFA"],
Cell[253716, 10681, 122, 3, 26, "Input",
  CellTags->"PermuteFA"],
Cell[253841, 10686, 105, 3, 26, "Input",
  CellTags->"PermuteFA"],
Cell[253949, 10691, 52, 1, 32, "ObjectName",
  CellTags->"PrintFA"],
Cell[254004, 10694, 52, 1, 50, "Highlight",
  CellTags->"PrintFA"],
Cell[254059, 10697, 158, 6, 31, "Usage",
  CellTags->"PrintFA"],
Cell[254220, 10705, 273, 10, 61, "Notes",
  CellTags->"PrintFA"],
Cell[254496, 10717, 144, 5, 24, "Notes",
  CellTags->"PrintFA"],
Cell[254643, 10724, 135, 5, 24, "Notes",
  CellTags->"PrintFA"],
Cell[254781, 10731, 79, 1, 24, "Notes",
  CellTags->"PrintFA"],
Cell[254863, 10734, 65, 1, 39, "ExampleSection",
  CellTags->"PrintFA"],
Cell[254931, 10737, 61, 1, 34, "ExampleText",
  CellTags->"PrintFA"],
Cell[254995, 10740, 91, 2, 28, "Input",
  CellTags->"PrintFA"],
Cell[255089, 10744, 92, 3, 26, "Input",
  CellTags->"PrintFA"],
Cell[255184, 10749, 61, 1, 34, "ExampleText",
  CellTags->"PrintFA"],
Cell[255248, 10752, 93, 2, 28, "Input",
  CellTags->"PrintFA"],
Cell[255344, 10756, 92, 3, 26, "Input",
  CellTags->"PrintFA"],
Cell[255439, 10761, 56, 1, 32, "ObjectName",
  CellTags->"ProductFA"],
Cell[255498, 10764, 54, 1, 50, "Highlight",
  CellTags->"ProductFA"],
Cell[255555, 10767, 199, 8, 31, "Usage",
  CellTags->"ProductFA"],
Cell[255757, 10777, 350, 11, 79, "Notes",
  CellTags->"ProductFA"],
Cell[256110, 10790, 249, 6, 62, "Notes",
  CellTags->"ProductFA"],
Cell[256362, 10798, 235, 6, 40, "Notes",
  CellTags->"ProductFA"],
Cell[256600, 10806, 446, 19, 24, "Notes",
  CellTags->"ProductFA"],
Cell[257049, 10827, 67, 1, 39, "ExampleSection",
  CellTags->"ProductFA"],
Cell[257119, 10830, 97, 1, 34, "ExampleText",
  CellTags->"ProductFA"],
Cell[257219, 10833, 128, 5, 45, "Input",
  CellTags->"ProductFA"],
Cell[257350, 10840, 110, 3, 26, "Input",
  CellTags->"ProductFA"],
Cell[257463, 10845, 244, 9, 34, "ExampleText",
  CellTags->"ProductFA"],
Cell[257710, 10856, 104, 3, 26, "Input",
  CellTags->"ProductFA"],
Cell[257817, 10861, 74, 1, 32, "ObjectName",
  CellTags->"ProductTransitions"],
Cell[257894, 10864, 63, 1, 50, "Highlight",
  CellTags->"ProductTransitions"],
Cell[257960, 10867, 199, 5, 47, "Usage",
  CellTags->"ProductTransitions"],
Cell[258162, 10874, 230, 9, 44, "Notes",
  CellTags->"ProductTransitions"],
Cell[258395, 10885, 446, 16, 40, "Notes",
  CellTags->"ProductTransitions"],
Cell[258844, 10903, 253, 11, 24, "Notes",
  CellTags->"ProductTransitions"],
Cell[259100, 10916, 276, 11, 24, "Notes",
  CellTags->"ProductTransitions"],
Cell[259379, 10929, 286, 12, 24, "Notes",
  CellTags->"ProductTransitions"],
Cell[259668, 10943, 76, 1, 39, "ExampleSection",
  CellTags->"ProductTransitions"],
Cell[259747, 10946, 112, 1, 34, "ExampleText",
  CellTags->"ProductTransitions"],
Cell[259862, 10949, 203, 4, 42, "Input",
  CellTags->"ProductTransitions"],
Cell[260068, 10955, 120, 3, 26, "Input",
  CellTags->"ProductTransitions"],
Cell[260191, 10960, 138, 3, 26, "Input",
  CellTags->"ProductTransitions"],
Cell[260332, 10965, 138, 3, 26, "Input",
  CellTags->"ProductTransitions"],
Cell[260473, 10970, 60, 1, 32, "ObjectName",
  CellTags->"QuotientDFA"],
Cell[260536, 10973, 56, 1, 50, "Highlight",
  CellTags->"QuotientDFA"],
Cell[260595, 10976, 228, 8, 31, "Usage",
  CellTags->"QuotientDFA"],
Cell[260826, 10986, 251, 9, 44, "Notes",
  CellTags->"QuotientDFA"],
Cell[261080, 10997, 74, 1, 24, "Notes",
  CellTags->"QuotientDFA"],
Cell[261157, 11000, 580, 23, 40, "Notes",
  CellTags->"QuotientDFA"],
Cell[261740, 11025, 69, 1, 39, "ExampleSection",
  CellTags->"QuotientDFA"],
Cell[261812, 11028, 155, 4, 50, "ExampleText",
  CellTags->"QuotientDFA"],
Cell[261970, 11034, 88, 2, 28, "Input",
  CellTags->"QuotientDFA"],
Cell[262061, 11038, 128, 3, 26, "Input",
  CellTags->"QuotientDFA"],
Cell[262192, 11043, 113, 3, 26, "Input",
  CellTags->"QuotientDFA"],
Cell[262308, 11048, 111, 2, 34, "ExampleText",
  CellTags->"QuotientDFA"],
Cell[262422, 11052, 142, 4, 42, "Input",
  CellTags->"QuotientDFA"],
Cell[262567, 11058, 72, 1, 32, "ObjectName",
  CellTags->"QuotientMachineFA"],
Cell[262642, 11061, 62, 1, 50, "Highlight",
  CellTags->"QuotientMachineFA"],
Cell[262707, 11064, 368, 14, 47, "Usage",
  CellTags->"QuotientMachineFA"],
Cell[263078, 11080, 257, 9, 44, "Notes",
  CellTags->"QuotientMachineFA"],
Cell[263338, 11091, 271, 7, 24, "Notes",
  CellTags->"QuotientMachineFA"],
Cell[263612, 11100, 108, 1, 24, "Notes",
  CellTags->"QuotientMachineFA"],
Cell[263723, 11103, 475, 19, 24, "Notes",
  CellTags->"QuotientMachineFA"],
Cell[264201, 11124, 75, 1, 39, "ExampleSection",
  CellTags->"QuotientMachineFA"],
Cell[264279, 11127, 378, 12, 34, "ExampleText",
  CellTags->"QuotientMachineFA"],
Cell[264660, 11141, 174, 5, 42, "Input",
  CellTags->"QuotientMachineFA"],
Cell[264837, 11148, 143, 3, 26, "Input",
  CellTags->"QuotientMachineFA"],
Cell[264983, 11153, 162, 4, 26, "Input",
  CellTags->"QuotientMachineFA"],
Cell[265148, 11159, 107, 2, 28, "Input",
  CellTags->"QuotientMachineFA"],
Cell[265258, 11163, 119, 3, 26, "Input",
  CellTags->"QuotientMachineFA"],
Cell[265380, 11168, 103, 1, 34, "ExampleText",
  CellTags->"QuotientMachineFA"],
Cell[265486, 11171, 118, 3, 26, "Input",
  CellTags->"QuotientMachineFA"],
Cell[265607, 11176, 56, 1, 32, "ObjectName",
  CellTags->"RandomDFA"],
Cell[265666, 11179, 54, 1, 50, "Highlight",
  CellTags->"RandomDFA"],
Cell[265723, 11182, 147, 4, 31, "Usage",
  CellTags->"RandomDFA"],
Cell[265873, 11188, 102, 1, 24, "Notes",
  CellTags->"RandomDFA"],
Cell[265978, 11191, 356, 15, 24, "Notes",
  CellTags->"RandomDFA"],
Cell[266337, 11208, 67, 1, 39, "ExampleSection",
  CellTags->"RandomDFA"],
Cell[266407, 11211, 99, 1, 34, "ExampleText",
  CellTags->"RandomDFA"],
Cell[266509, 11214, 82, 2, 28, "Input",
  CellTags->"RandomDFA"],
Cell[266594, 11218, 82, 2, 28, "Input",
  CellTags->"RandomDFA"],
Cell[266679, 11222, 56, 1, 32, "ObjectName",
  CellTags->"RankOfDFA"],
Cell[266738, 11225, 54, 1, 50, "Highlight",
  CellTags->"RankOfDFA"],
Cell[266795, 11228, 194, 6, 31, "Usage",
  CellTags->"RankOfDFA"],
Cell[266992, 11236, 352, 15, 24, "Notes",
  CellTags->"RankOfDFA"],
Cell[267347, 11253, 67, 1, 39, "ExampleSection",
  CellTags->"RankOfDFA"],
Cell[267417, 11256, 66, 1, 34, "ExampleText",
  CellTags->"RankOfDFA"],
Cell[267486, 11259, 129, 3, 26, "Input",
  CellTags->"RankOfDFA"],
Cell[267618, 11264, 100, 3, 26, "Input",
  CellTags->"RankOfDFA"],
Cell[267721, 11269, 110, 3, 26, "Input",
  CellTags->"RankOfDFA"],
Cell[267834, 11274, 56, 1, 32, "ObjectName",
  CellTags->"RankToDFA"],
Cell[267893, 11277, 54, 1, 50, "Highlight",
  CellTags->"RankToDFA"],
Cell[267950, 11280, 275, 10, 47, "Usage",
  CellTags->"RankToDFA"],
Cell[268228, 11292, 96, 1, 24, "Notes",
  CellTags->"RankToDFA"],
Cell[268327, 11295, 369, 17, 24, "Notes",
  CellTags->"RankToDFA"],
Cell[268699, 11314, 67, 1, 39, "ExampleSection",
  CellTags->"RankToDFA"],
Cell[268769, 11317, 61, 1, 34, "ExampleText",
  CellTags->"RankToDFA"],
Cell[268833, 11320, 133, 5, 45, "Input",
  CellTags->"RankToDFA"],
Cell[268969, 11327, 74, 1, 32, "ObjectName",
  CellTags->"DeleteTransitionFA"],
Cell[269046, 11330, 63, 1, 50, "Highlight",
  CellTags->"DeleteTransitionFA"],
Cell[269112, 11333, 302, 11, 47, "Usage",
  CellTags->"DeleteTransitionFA"],
Cell[269417, 11346, 265, 8, 40, "Notes",
  CellTags->"DeleteTransitionFA"],
Cell[269685, 11356, 293, 11, 24, "Notes",
  CellTags->"DeleteTransitionFA"],
Cell[269981, 11369, 76, 1, 39, "ExampleSection",
  CellTags->"DeleteTransitionFA"],
Cell[270060, 11372, 96, 1, 34, "ExampleText",
  CellTags->"DeleteTransitionFA"],
Cell[270159, 11375, 224, 5, 42, "Input",
  CellTags->"DeleteTransitionFA"],
Cell[270386, 11382, 106, 3, 26, "Input",
  CellTags->"DeleteTransitionFA"],
Cell[270495, 11387, 152, 4, 34, "ExampleText",
  CellTags->"DeleteTransitionFA"],
Cell[270650, 11393, 132, 3, 26, "Input",
  CellTags->"DeleteTransitionFA"],
Cell[270785, 11398, 106, 3, 26, "Input",
  CellTags->"DeleteTransitionFA"],
Cell[270894, 11403, 56, 1, 32, "ObjectName",
  CellTags->"ReverseFA"],
Cell[270953, 11406, 54, 1, 50, "Highlight",
  CellTags->"ReverseFA"],
Cell[271010, 11409, 207, 6, 31, "Usage",
  CellTags->"ReverseFA"],
Cell[271220, 11417, 117, 4, 24, "Notes",
  CellTags->"ReverseFA"],
Cell[271340, 11423, 67, 1, 39, "ExampleSection",
  CellTags->"ReverseFA"],
Cell[271410, 11426, 178, 6, 34, "ExampleText",
  CellTags->"ReverseFA"],
Cell[271591, 11434, 127, 3, 26, "Input",
  CellTags->"ReverseFA"],
Cell[271721, 11439, 101, 3, 26, "Input",
  CellTags->"ReverseFA"],
Cell[271825, 11444, 112, 3, 26, "Input",
  CellTags->"ReverseFA"],
Cell[271940, 11449, 42, 1, 32, "ObjectName",
  CellTags->"SA"],
Cell[271985, 11452, 50, 1, 50, "Highlight",
  CellTags->"SA"],
Cell[272038, 11455, 109, 4, 31, "Usage",
  CellTags->"SA"],
Cell[272150, 11461, 231, 9, 24, "Notes",
  CellTags->"SA"],
Cell[272384, 11472, 166, 5, 24, "Notes",
  CellTags->"SA"],
Cell[272553, 11479, 79, 1, 24, "Notes",
  CellTags->"SA"],
Cell[272635, 11482, 183, 7, 24, "Notes",
  CellTags->"SA"],
Cell[272821, 11491, 303, 15, 24, "Notes",
  CellTags->"SA"],
Cell[273127, 11508, 60, 1, 39, "ExampleSection",
  CellTags->"SA"],
Cell[273190, 11511, 63, 1, 34, "ExampleText",
  CellTags->"SA"],
Cell[273256, 11514, 212, 5, 42, "Input",
  CellTags->"SA"],
Cell[273471, 11521, 141, 5, 58, "Input",
  CellTags->"SA"],
Cell[273615, 11528, 54, 1, 32, "ObjectName",
  CellTags->"SelectFA"],
Cell[273672, 11531, 53, 1, 50, "Highlight",
  CellTags->"SelectFA"],
Cell[273728, 11534, 208, 8, 31, "Usage",
  CellTags->"SelectFA"],
Cell[273939, 11544, 225, 9, 44, "Notes",
  CellTags->"SelectFA"],
Cell[274167, 11555, 91, 1, 24, "Notes",
  CellTags->"SelectFA"],
Cell[274261, 11558, 114, 4, 24, "Notes",
  CellTags->"SelectFA"],
Cell[274378, 11564, 66, 1, 39, "ExampleSection",
  CellTags->"SelectFA"],
Cell[274447, 11567, 254, 7, 34, "ExampleText",
  CellTags->"SelectFA"],
Cell[274704, 11576, 298, 7, 58, "Input",
  InitializationCell->True,
  CellTags->"SelectFA"],
Cell[275005, 11585, 179, 5, 26, "Input",
  InitializationCell->True,
  CellTags->"SelectFA"],
Cell[275187, 11592, 121, 4, 26, "Input",
  InitializationCell->True,
  CellTags->"SelectFA"],
Cell[275311, 11598, 138, 4, 34, "ExampleText",
  CellTags->"SelectFA"],
Cell[275452, 11604, 143, 4, 26, "Input",
  InitializationCell->True,
  CellTags->"SelectFA"],
Cell[275598, 11610, 131, 4, 26, "Input",
  InitializationCell->True,
  CellTags->"SelectFA"],
Cell[275732, 11616, 60, 1, 32, "ObjectName",
  CellTags->"SetAlphabet"],
Cell[275795, 11619, 56, 1, 50, "Highlight",
  CellTags->"SetAlphabet"],
Cell[275854, 11622, 213, 8, 31, "Usage",
  CellTags->"SetAlphabet"],
Cell[276070, 11632, 122, 4, 24, "Notes",
  CellTags->"SetAlphabet"],
Cell[276195, 11638, 133, 6, 24, "Notes",
  CellTags->"SetAlphabet"],
Cell[276331, 11646, 69, 1, 39, "ExampleSection",
  CellTags->"SetAlphabet"],
Cell[276403, 11649, 209, 7, 36, "ExampleText",
  CellTags->"SetAlphabet"],
Cell[276615, 11658, 116, 3, 26, "Input",
  CellTags->"SetAlphabet"],
Cell[276734, 11663, 96, 3, 26, "Input",
  CellTags->"SetAlphabet"],
Cell[276833, 11668, 214, 9, 34, "ExampleText",
  CellTags->"SetAlphabet"],
Cell[277050, 11679, 154, 4, 42, "Input",
  CellTags->"SetAlphabet"],
Cell[277207, 11685, 90, 1, 34, "ExampleText",
  CellTags->"SetAlphabet"],
Cell[277300, 11688, 111, 3, 26, "Input",
  CellTags->"SetAlphabet"],
Cell[277414, 11693, 58, 1, 32, "ObjectName",
  CellTags->"SetFinalFA"],
Cell[277475, 11696, 55, 1, 50, "Highlight",
  CellTags->"SetFinalFA"],
Cell[277533, 11699, 205, 8, 31, "Usage",
  CellTags->"SetFinalFA"],
Cell[277741, 11709, 142, 4, 24, "Notes",
  CellTags->"SetFinalFA"],
Cell[277886, 11715, 186, 6, 24, "Notes",
  CellTags->"SetFinalFA"],
Cell[278075, 11723, 241, 6, 40, "Notes",
  CellTags->"SetFinalFA"],
Cell[278319, 11731, 410, 17, 24, "Notes",
  CellTags->"SetFinalFA"],
Cell[278732, 11750, 68, 1, 39, "ExampleSection",
  CellTags->"SetFinalFA"],
Cell[278803, 11753, 293, 8, 50, "ExampleText",
  CellTags->"SetFinalFA"],
Cell[279099, 11763, 248, 6, 42, "Input",
  CellTags->"SetFinalFA"],
Cell[279350, 11771, 111, 3, 26, "Input",
  CellTags->"SetFinalFA"],
Cell[279464, 11776, 205, 7, 34, "ExampleText",
  CellTags->"SetFinalFA"],
Cell[279672, 11785, 106, 3, 26, "Input",
  CellTags->"SetFinalFA"],
Cell[279781, 11790, 255, 8, 34, "ExampleText",
  CellTags->"SetFinalFA"],
Cell[280039, 11800, 190, 4, 42, "Input",
  CellTags->"SetFinalFA"],
Cell[280232, 11806, 62, 1, 32, "ObjectName",
  CellTags->"SetInitialFA"],
Cell[280297, 11809, 57, 1, 50, "Highlight",
  CellTags->"SetInitialFA"],
Cell[280357, 11812, 210, 8, 31, "Usage",
  CellTags->"SetInitialFA"],
Cell[280570, 11822, 151, 5, 24, "Notes",
  CellTags->"SetInitialFA"],
Cell[280724, 11829, 188, 6, 24, "Notes",
  CellTags->"SetInitialFA"],
Cell[280915, 11837, 245, 6, 40, "Notes",
  CellTags->"SetInitialFA"],
Cell[281163, 11845, 340, 15, 24, "Notes",
  CellTags->"SetInitialFA"],
Cell[281506, 11862, 70, 1, 39, "ExampleSection",
  CellTags->"SetInitialFA"],
Cell[281579, 11865, 265, 7, 50, "ExampleText",
  CellTags->"SetInitialFA"],
Cell[281847, 11874, 216, 5, 42, "Input",
  CellTags->"SetInitialFA"],
Cell[282066, 11881, 104, 3, 26, "Input",
  CellTags->"SetInitialFA"],
Cell[282173, 11886, 107, 3, 26, "Input",
  CellTags->"SetInitialFA"],
Cell[282283, 11891, 152, 4, 42, "Input",
  CellTags->"SetInitialFA"],
Cell[282438, 11897, 72, 1, 32, "ObjectName",
  CellTags->"SetInitialFinalFA"],
Cell[282513, 11900, 62, 1, 50, "Highlight",
  CellTags->"SetInitialFinalFA"],
Cell[282578, 11903, 276, 10, 47, "Usage",
  CellTags->"SetInitialFinalFA"],
Cell[282857, 11915, 195, 7, 24, "Notes",
  CellTags->"SetInitialFinalFA"],
Cell[283055, 11924, 193, 6, 24, "Notes",
  CellTags->"SetInitialFinalFA"],
Cell[283251, 11932, 255, 6, 40, "Notes",
  CellTags->"SetInitialFinalFA"],
Cell[283509, 11940, 404, 17, 24, "Notes",
  CellTags->"SetInitialFinalFA"],
Cell[283916, 11959, 75, 1, 39, "ExampleSection",
  CellTags->"SetInitialFinalFA"],
Cell[283994, 11962, 156, 4, 34, "ExampleText",
  CellTags->"SetInitialFinalFA"],
Cell[284153, 11968, 222, 5, 42, "Input",
  CellTags->"SetInitialFinalFA"],
Cell[284378, 11975, 128, 3, 26, "Input",
  CellTags->"SetInitialFinalFA"],
Cell[284509, 11980, 113, 3, 26, "Input",
  CellTags->"SetInitialFinalFA"],
Cell[284625, 11985, 247, 8, 34, "ExampleText",
  CellTags->"SetInitialFinalFA"],
Cell[284875, 11995, 196, 4, 42, "Input",
  CellTags->"SetInitialFinalFA"],
Cell[285074, 12001, 193, 4, 42, "Input",
  CellTags->"SetInitialFinalFA"],
Cell[285270, 12007, 56, 1, 32, "ObjectName",
  CellTags->"SetStates"],
Cell[285329, 12010, 54, 1, 50, "Highlight",
  CellTags->"SetStates"],
Cell[285386, 12013, 207, 8, 31, "Usage",
  CellTags->"SetStates"],
Cell[285596, 12023, 122, 4, 24, "Notes",
  CellTags->"SetStates"],
Cell[285721, 12029, 133, 6, 24, "Notes",
  CellTags->"SetStates"],
Cell[285857, 12037, 67, 1, 39, "ExampleSection",
  CellTags->"SetStates"],
Cell[285927, 12040, 144, 4, 34, "ExampleText",
  CellTags->"SetStates"],
Cell[286074, 12046, 139, 3, 26, "Input",
  CellTags->"SetStates"],
Cell[286216, 12051, 92, 3, 26, "Input",
  CellTags->"SetStates"],
Cell[286311, 12056, 149, 4, 34, "ExampleText",
  CellTags->"SetStates"],
Cell[286463, 12062, 153, 4, 42, "Input",
  CellTags->"SetStates"],
Cell[286619, 12068, 71, 1, 34, "ExampleText",
  CellTags->"SetStates"],
Cell[286693, 12071, 107, 3, 26, "Input",
  CellTags->"SetStates"],
Cell[286803, 12076, 52, 1, 32, "ObjectName",
  CellTags->"ShowAll"],
Cell[286858, 12079, 50, 1, 50, "Highlight",
  CellTags->"ShowAll"],
Cell[286911, 12082, 152, 6, 31, "Usage",
  CellTags->"ShowAll"],
Cell[287066, 12090, 100, 1, 24, "Notes",
  CellTags->"ShowAll"],
Cell[287169, 12093, 132, 5, 24, "Notes",
  CellTags->"ShowAll"],
Cell[287304, 12100, 174, 7, 24, "Notes",
  CellTags->"ShowAll"],
Cell[287481, 12109, 56, 1, 32, "ObjectName",
  CellTags->"ShowEdges"],
Cell[287540, 12112, 52, 1, 50, "Highlight",
  CellTags->"ShowEdges"],
Cell[287595, 12115, 154, 6, 31, "Usage",
  CellTags->"ShowEdges"],
Cell[287752, 12123, 140, 4, 24, "Notes",
  CellTags->"ShowEdges"],
Cell[287895, 12129, 176, 7, 24, "Notes",
  CellTags->"ShowEdges"],
Cell[288074, 12138, 70, 1, 32, "ObjectName",
  CellTags->"ShuffleProductFA"],
Cell[288147, 12141, 61, 1, 50, "Highlight",
  CellTags->"ShuffleProductFA"],
Cell[288211, 12144, 282, 9, 47, "Usage",
  CellTags->"ShuffleProductFA"],
Cell[288496, 12155, 256, 9, 44, "Notes",
  CellTags->"ShuffleProductFA"],
Cell[288755, 12166, 107, 1, 24, "Notes",
  CellTags->"ShuffleProductFA"],
Cell[288865, 12169, 171, 7, 24, "Notes",
  CellTags->"ShuffleProductFA"],
Cell[289039, 12178, 74, 1, 39, "ExampleSection",
  CellTags->"ShuffleProductFA"],
Cell[289116, 12181, 287, 9, 50, "ExampleText",
  CellTags->"ShuffleProductFA"],
Cell[289406, 12192, 136, 5, 45, "Input",
  CellTags->"ShuffleProductFA"],
Cell[289545, 12199, 123, 3, 26, "Input",
  CellTags->"ShuffleProductFA"],
Cell[289671, 12204, 138, 3, 26, "Input",
  CellTags->"ShuffleProductFA"],
Cell[289812, 12209, 54, 1, 32, "ObjectName",
  CellTags->"SinkQDFA"],
Cell[289869, 12212, 53, 1, 50, "Highlight",
  CellTags->"SinkQDFA"],
Cell[289925, 12215, 161, 6, 31, "Usage",
  CellTags->"SinkQDFA"],
Cell[290089, 12223, 299, 9, 40, "Notes",
  CellTags->"SinkQDFA"],
Cell[290391, 12234, 166, 7, 24, "Notes",
  CellTags->"SinkQDFA"],
Cell[290560, 12243, 66, 1, 39, "ExampleSection",
  CellTags->"SinkQDFA"],
Cell[290629, 12246, 109, 2, 34, "ExampleText",
  CellTags->"SinkQDFA"],
Cell[290741, 12250, 177, 4, 42, "Input",
  CellTags->"SinkQDFA"],
Cell[290921, 12256, 149, 6, 34, "ExampleText",
  CellTags->"SinkQDFA"],
Cell[291073, 12264, 94, 3, 26, "Input",
  CellTags->"SinkQDFA"],
Cell[291170, 12269, 139, 4, 34, "ExampleText",
  CellTags->"SinkQDFA"],
Cell[291312, 12275, 151, 4, 26, "Input",
  CellTags->"SinkQDFA"],
Cell[291466, 12281, 54, 1, 32, "ObjectName",
  CellTags->"SizeOnly"],
Cell[291523, 12284, 51, 1, 50, "Highlight",
  CellTags->"SizeOnly"],
Cell[291577, 12287, 148, 5, 31, "Usage",
  CellTags->"SizeOnly"],
Cell[291728, 12294, 294, 10, 40, "Notes",
  CellTags->"SizeOnly"],
Cell[292025, 12306, 209, 7, 40, "Notes",
  CellTags->"SizeOnly"],
Cell[292237, 12315, 279, 11, 24, "Notes",
  CellTags->"SizeOnly"],
Cell[292519, 12328, 66, 1, 39, "ExampleSection",
  CellTags->"SizeOnly"],
Cell[292588, 12331, 65, 1, 34, "ExampleText",
  CellTags->"SizeOnly"],
Cell[292656, 12334, 213, 5, 42, "Input",
  CellTags->"SizeOnly"],
Cell[292872, 12341, 136, 3, 26, "Input",
  CellTags->"SizeOnly"],
Cell[293011, 12346, 46, 1, 32, "ObjectName",
  CellTags->"Size"],
Cell[293060, 12349, 49, 1, 50, "Highlight",
  CellTags->"Size"],
Cell[293112, 12352, 126, 4, 31, "Usage",
  CellTags->"Size"],
Cell[293241, 12358, 220, 9, 44, "Notes",
  CellTags->"Size"],
Cell[293464, 12369, 262, 7, 40, "Notes",
  CellTags->"Size"],
Cell[293729, 12378, 104, 5, 24, "Notes",
  CellTags->"Size"],
Cell[293836, 12385, 62, 1, 39, "ExampleSection",
  CellTags->"Size"],
Cell[293901, 12388, 88, 1, 34, "ExampleText",
  CellTags->"Size"],
Cell[293992, 12391, 155, 4, 42, "Input",
  CellTags->"Size"],
Cell[294150, 12397, 88, 1, 34, "ExampleText",
  CellTags->"Size"],
Cell[294241, 12400, 131, 3, 26, "Input",
  CellTags->"Size"],
Cell[294375, 12405, 46, 1, 32, "ObjectName",
  CellTags->"Some"],
Cell[294424, 12408, 47, 1, 50, "Highlight",
  CellTags->"Some"],
Cell[294474, 12411, 141, 6, 31, "Usage",
  CellTags->"Some"],
Cell[294618, 12419, 160, 7, 24, "Notes",
  CellTags->"Some"],
Cell[294781, 12428, 54, 1, 32, "ObjectName",
  CellTags->"SquareSA"],
Cell[294838, 12431, 53, 1, 50, "Highlight",
  CellTags->"SquareSA"],
Cell[294894, 12434, 187, 6, 31, "Usage",
  CellTags->"SquareSA"],
Cell[295084, 12442, 302, 10, 61, "Notes",
  CellTags->"SquareSA"],
Cell[295389, 12454, 220, 8, 24, "Notes",
  CellTags->"SquareSA"],
Cell[295612, 12464, 159, 5, 24, "Notes",
  CellTags->"SquareSA"],
Cell[295774, 12471, 154, 5, 24, "Notes",
  CellTags->"SquareSA"],
Cell[295931, 12478, 274, 8, 40, "Notes",
  CellTags->"SquareSA"],
Cell[296208, 12488, 100, 1, 24, "Notes",
  CellTags->"SquareSA"],
Cell[296311, 12491, 258, 11, 24, "Notes",
  CellTags->"SquareSA"],
Cell[296572, 12504, 66, 1, 39, "ExampleSection",
  CellTags->"SquareSA"],
Cell[296641, 12507, 227, 9, 34, "ExampleText",
  CellTags->"SquareSA"],
Cell[296871, 12518, 213, 5, 42, "Input",
  CellTags->"SquareSA"],
Cell[297087, 12525, 93, 3, 26, "Input",
  CellTags->"SquareSA"],
Cell[297183, 12530, 562, 19, 66, "ExampleText",
  CellTags->"SquareSA"],
Cell[297748, 12551, 160, 4, 42, "Input",
  CellTags->"SquareSA"],
Cell[297911, 12557, 109, 3, 26, "Input",
  CellTags->"SquareSA"],
Cell[298023, 12562, 50, 1, 32, "ObjectName",
  CellTags->"StarFA"],
Cell[298076, 12565, 51, 1, 50, "Highlight",
  CellTags->"StarFA"],
Cell[298130, 12568, 196, 7, 31, "Usage",
  CellTags->"StarFA"],
Cell[298329, 12577, 305, 10, 61, "Notes",
  CellTags->"StarFA"],
Cell[298637, 12589, 176, 5, 24, "Notes",
  CellTags->"StarFA"],
Cell[298816, 12596, 438, 19, 24, "Notes",
  CellTags->"StarFA"],
Cell[299257, 12617, 64, 1, 39, "ExampleSection",
  CellTags->"StarFA"],
Cell[299324, 12620, 189, 6, 34, "ExampleText",
  CellTags->"StarFA"],
Cell[299516, 12628, 83, 2, 28, "Input",
  CellTags->"StarFA"],
Cell[299602, 12632, 98, 3, 26, "Input",
  CellTags->"StarFA"],
Cell[299703, 12637, 134, 3, 26, "Input",
  CellTags->"StarFA"],
Cell[299840, 12642, 64, 1, 32, "ObjectName",
  CellTags->"StateMergeDFA"],
Cell[299907, 12645, 58, 1, 50, "Highlight",
  CellTags->"StateMergeDFA"],
Cell[299968, 12648, 227, 6, 47, "Usage",
  CellTags->"StateMergeDFA"],
Cell[300198, 12656, 350, 11, 79, "Notes",
  CellTags->"StateMergeDFA"],
Cell[300551, 12669, 228, 5, 40, "Notes",
  CellTags->"StateMergeDFA"],
Cell[300782, 12676, 187, 7, 24, "Notes",
  CellTags->"StateMergeDFA"],
Cell[300972, 12685, 202, 7, 24, "Notes",
  CellTags->"StateMergeDFA"],
Cell[301177, 12694, 71, 1, 39, "ExampleSection",
  CellTags->"StateMergeDFA"],
Cell[301251, 12697, 242, 7, 34, "ExampleText",
  CellTags->"StateMergeDFA"],
Cell[301496, 12706, 220, 5, 42, "Input",
  CellTags->"StateMergeDFA"],
Cell[301719, 12713, 105, 1, 34, "ExampleText",
  CellTags->"StateMergeDFA"],
Cell[301827, 12716, 112, 3, 26, "Input",
  CellTags->"StateMergeDFA"],
Cell[301942, 12721, 110, 3, 26, "Input",
  CellTags->"StateMergeDFA"],
Cell[302055, 12726, 50, 1, 32, "ObjectName",
  CellTags->"States"],
Cell[302108, 12729, 51, 1, 50, "Highlight",
  CellTags->"States"],
Cell[302162, 12732, 164, 6, 31, "Usage",
  CellTags->"States"],
Cell[302329, 12740, 196, 6, 24, "Notes",
  CellTags->"States"],
Cell[302528, 12748, 255, 11, 24, "Notes",
  CellTags->"States"],
Cell[302786, 12761, 64, 1, 39, "ExampleSection",
  CellTags->"States"],
Cell[302853, 12764, 140, 4, 34, "ExampleText",
  CellTags->"States"],
Cell[302996, 12770, 130, 4, 42, "Input",
  CellTags->"States"],
Cell[303129, 12776, 106, 2, 34, "ExampleText",
  CellTags->"States"],
Cell[303238, 12780, 150, 4, 42, "Input",
  CellTags->"States"],
Cell[303391, 12786, 56, 1, 32, "ObjectName",
  CellTags->"SubsetQFA"],
Cell[303450, 12789, 54, 1, 50, "Highlight",
  CellTags->"SubsetQFA"],
Cell[303507, 12792, 244, 8, 47, "Usage",
  CellTags->"SubsetQFA"],
Cell[303754, 12802, 247, 9, 44, "Notes",
  CellTags->"SubsetQFA"],
Cell[304004, 12813, 198, 6, 40, "Notes",
  CellTags->"SubsetQFA"],
Cell[304205, 12821, 168, 7, 24, "Notes",
  CellTags->"SubsetQFA"],
Cell[304376, 12830, 67, 1, 39, "ExampleSection",
  CellTags->"SubsetQFA"],
Cell[304446, 12833, 125, 4, 34, "ExampleText",
  CellTags->"SubsetQFA"],
Cell[304574, 12839, 155, 4, 42, "Input",
  CellTags->"SubsetQFA"],
Cell[304732, 12845, 102, 3, 26, "Input",
  CellTags->"SubsetQFA"],
Cell[304837, 12850, 103, 3, 26, "Input",
  CellTags->"SubsetQFA"],
Cell[304943, 12855, 72, 1, 32, "ObjectName",
  CellTags->"SubwordCounterDFA"],
Cell[305018, 12858, 62, 1, 50, "Highlight",
  CellTags->"SubwordCounterDFA"],
Cell[305083, 12861, 297, 10, 47, "Usage",
  CellTags->"SubwordCounterDFA"],
Cell[305383, 12873, 301, 10, 61, "Notes",
  CellTags->"SubwordCounterDFA"],
Cell[305687, 12885, 531, 17, 62, "Notes",
  CellTags->"SubwordCounterDFA"],
Cell[306221, 12904, 192, 9, 24, "Notes",
  CellTags->"SubwordCounterDFA"],
Cell[306416, 12915, 328, 9, 40, "Notes",
  CellTags->"SubwordCounterDFA"],
Cell[306747, 12926, 170, 7, 24, "Notes",
  CellTags->"SubwordCounterDFA"],
Cell[306920, 12935, 75, 1, 39, "ExampleSection",
  CellTags->"SubwordCounterDFA"],
Cell[306998, 12938, 138, 4, 34, "ExampleText",
  CellTags->"SubwordCounterDFA"],
Cell[307139, 12944, 229, 6, 58, "Input",
  CellTags->"SubwordCounterDFA"],
Cell[307371, 12952, 203, 5, 42, "Input",
  CellTags->"SubwordCounterDFA"],
Cell[307577, 12959, 210, 6, 34, "ExampleText",
  CellTags->"SubwordCounterDFA"],
Cell[307790, 12967, 147, 3, 26, "Input",
  CellTags->"SubwordCounterDFA"],
Cell[307940, 12972, 107, 3, 26, "Input",
  CellTags->"SubwordCounterDFA"],
Cell[308050, 12977, 110, 3, 26, "Input",
  CellTags->"SubwordCounterDFA"],
Cell[308163, 12982, 58, 1, 32, "ObjectName",
  CellTags->"SubwordDFA"],
Cell[308224, 12985, 55, 1, 50, "Highlight",
  CellTags->"SubwordDFA"],
Cell[308282, 12988, 234, 8, 47, "Usage",
  CellTags->"SubwordDFA"],
Cell[308519, 12998, 250, 9, 44, "Notes",
  CellTags->"SubwordDFA"],
Cell[308772, 13009, 142, 5, 24, "Notes",
  CellTags->"SubwordDFA"],
Cell[308917, 13016, 178, 7, 24, "Notes",
  CellTags->"SubwordDFA"],
Cell[309098, 13025, 68, 1, 39, "ExampleSection",
  CellTags->"SubwordDFA"],
Cell[309169, 13028, 97, 1, 34, "ExampleText",
  CellTags->"SubwordDFA"],
Cell[309269, 13031, 132, 3, 26, "Input",
  CellTags->"SubwordDFA"],
Cell[309404, 13036, 93, 3, 26, "Input",
  CellTags->"SubwordDFA"],
Cell[309500, 13041, 105, 3, 26, "Input",
  CellTags->"SubwordDFA"],
Cell[309608, 13046, 113, 3, 26, "Input",
  CellTags->"SubwordDFA"],
Cell[309724, 13051, 48, 1, 32, "ObjectName",
  CellTags->"SumFA"],
Cell[309775, 13054, 50, 1, 50, "Highlight",
  CellTags->"SumFA"],
Cell[309828, 13057, 199, 8, 31, "Usage",
  CellTags->"SumFA"],
Cell[310030, 13067, 245, 9, 44, "Notes",
  CellTags->"SumFA"],
Cell[310278, 13078, 146, 5, 24, "Notes",
  CellTags->"SumFA"],
Cell[310427, 13085, 187, 5, 40, "Notes",
  CellTags->"SumFA"],
Cell[310617, 13092, 142, 4, 24, "Notes",
  CellTags->"SumFA"],
Cell[310762, 13098, 350, 15, 24, "Notes",
  CellTags->"SumFA"],
Cell[311115, 13115, 63, 1, 39, "ExampleSection",
  CellTags->"SumFA"],
Cell[311181, 13118, 142, 4, 34, "ExampleText",
  CellTags->"SumFA"],
Cell[311326, 13124, 132, 6, 62, "Input",
  CellTags->"SumFA"],
Cell[311461, 13132, 100, 3, 26, "Input",
  CellTags->"SumFA"],
Cell[311564, 13137, 140, 4, 34, "ExampleText",
  CellTags->"SumFA"],
Cell[311707, 13143, 105, 3, 26, "Input",
  CellTags->"SumFA"],
Cell[311815, 13148, 99, 3, 26, "Input",
  CellTags->"SumFA"],
Cell[311917, 13153, 74, 1, 32, "ObjectName",
  CellTags->"SwitchTransitionFA"],
Cell[311994, 13156, 63, 1, 50, "Highlight",
  CellTags->"SwitchTransitionFA"],
Cell[312060, 13159, 251, 10, 31, "Usage",
  CellTags->"SwitchTransitionFA"],
Cell[312314, 13171, 191, 7, 24, "Notes",
  CellTags->"SwitchTransitionFA"],
Cell[312508, 13180, 245, 7, 40, "Notes",
  CellTags->"SwitchTransitionFA"],
Cell[312756, 13189, 296, 11, 24, "Notes",
  CellTags->"SwitchTransitionFA"],
Cell[313055, 13202, 76, 1, 39, "ExampleSection",
  CellTags->"SwitchTransitionFA"],
Cell[313134, 13205, 200, 5, 50, "ExampleText",
  CellTags->"SwitchTransitionFA"],
Cell[313337, 13212, 223, 5, 42, "Input",
  CellTags->"SwitchTransitionFA"],
Cell[313563, 13219, 129, 3, 26, "Input",
  CellTags->"SwitchTransitionFA"],
Cell[313695, 13224, 114, 3, 26, "Input",
  CellTags->"SwitchTransitionFA"],
Cell[313812, 13229, 58, 1, 32, "ObjectName",
  CellTags->"TheInitial"],
Cell[313873, 13232, 55, 1, 50, "Highlight",
  CellTags->"TheInitial"],
Cell[313931, 13235, 186, 6, 31, "Usage",
  CellTags->"TheInitial"],
Cell[314120, 13243, 180, 7, 24, "Notes",
  CellTags->"TheInitial"],
Cell[314303, 13252, 68, 1, 39, "ExampleSection",
  CellTags->"TheInitial"],
Cell[314374, 13255, 128, 4, 34, "ExampleText",
  CellTags->"TheInitial"],
Cell[314505, 13261, 102, 2, 28, "Input",
  CellTags->"TheInitial"],
Cell[314610, 13265, 127, 4, 42, "Input",
  CellTags->"TheInitial"],
Cell[314740, 13271, 54, 1, 32, "ObjectName",
  CellTags->"Thompson"],
Cell[314797, 13274, 51, 1, 50, "Highlight",
  CellTags->"Thompson"],
Cell[314851, 13277, 156, 6, 31, "Usage",
  CellTags->"Thompson"],
Cell[315010, 13285, 213, 7, 40, "Notes",
  CellTags->"Thompson"],
Cell[315226, 13294, 183, 7, 24, "Notes",
  CellTags->"Thompson"],
Cell[315412, 13303, 62, 1, 32, "ObjectName",
  CellTags->"ToCharacters"],
Cell[315477, 13306, 57, 1, 50, "Highlight",
  CellTags->"ToCharacters"],
Cell[315537, 13309, 160, 4, 31, "Usage",
  CellTags->"ToCharacters"],
Cell[315700, 13315, 164, 5, 24, "Notes",
  CellTags->"ToCharacters"],
Cell[315867, 13322, 86, 1, 24, "Notes",
  CellTags->"ToCharacters"],
Cell[315956, 13325, 70, 1, 39, "ExampleSection",
  CellTags->"ToCharacters"],
Cell[316029, 13328, 124, 4, 34, "ExampleText",
  CellTags->"ToCharacters"],
Cell[316156, 13334, 111, 5, 45, "Input",
  CellTags->"ToCharacters"],
Cell[316270, 13341, 111, 3, 26, "Input",
  CellTags->"ToCharacters"],
Cell[316384, 13346, 48, 1, 32, "ObjectName",
  CellTags->"ToDFA"],
Cell[316435, 13349, 50, 1, 50, "Highlight",
  CellTags->"ToDFA"],
Cell[316488, 13352, 142, 4, 31, "Usage",
  CellTags->"ToDFA"],
Cell[316633, 13358, 406, 12, 97, "Notes",
  CellTags->"ToDFA"],
Cell[317042, 13372, 212, 8, 24, "Notes",
  CellTags->"ToDFA"],
Cell[317257, 13382, 224, 8, 24, "Notes",
  CellTags->"ToDFA"],
Cell[317484, 13392, 209, 5, 40, "Notes",
  CellTags->"ToDFA"],
Cell[317696, 13399, 278, 8, 40, "Notes",
  CellTags->"ToDFA"],
Cell[317977, 13409, 150, 5, 24, "Notes",
  CellTags->"ToDFA"],
Cell[318130, 13416, 161, 4, 40, "Notes",
  CellTags->"ToDFA"],
Cell[318294, 13422, 316, 15, 24, "Notes",
  CellTags->"ToDFA"],
Cell[318613, 13439, 63, 1, 39, "ExampleSection",
  CellTags->"ToDFA"],
Cell[318679, 13442, 147, 4, 50, "ExampleText",
  CellTags->"ToDFA"],
Cell[318829, 13448, 146, 4, 42, "Input",
  CellTags->"ToDFA"],
Cell[318978, 13454, 100, 3, 26, "Input",
  CellTags->"ToDFA"],
Cell[319081, 13459, 260, 7, 50, "ExampleText",
  CellTags->"ToDFA"],
Cell[319344, 13468, 200, 5, 58, "Input",
  CellTags->"ToDFA"],
Cell[319547, 13475, 66, 1, 32, "ObjectName",
  CellTags->"ToEquationsDFA"],
Cell[319616, 13478, 59, 1, 50, "Highlight",
  CellTags->"ToEquationsDFA"],
Cell[319678, 13481, 175, 6, 31, "Usage",
  CellTags->"ToEquationsDFA"],
Cell[319856, 13489, 173, 4, 40, "Notes",
  CellTags->"ToEquationsDFA"],
Cell[320032, 13495, 267, 11, 24, "Notes",
  CellTags->"ToEquationsDFA"],
Cell[320302, 13508, 72, 1, 39, "ExampleSection",
  CellTags->"ToEquationsDFA"],
Cell[320377, 13511, 239, 7, 50, "ExampleText",
  CellTags->"ToEquationsDFA"],
Cell[320619, 13520, 106, 2, 28, "Input",
  CellTags->"ToEquationsDFA"],
Cell[320728, 13524, 151, 4, 42, "Input",
  CellTags->"ToEquationsDFA"],
Cell[320882, 13530, 82, 1, 34, "ExampleText",
  CellTags->"ToEquationsDFA"],
Cell[320967, 13533, 194, 4, 42, "Input",
  CellTags->"ToEquationsDFA"],
Cell[321164, 13539, 46, 1, 32, "ObjectName",
  CellTags->"ToFA"],
Cell[321213, 13542, 49, 1, 50, "Highlight",
  CellTags->"ToFA"],
Cell[321265, 13545, 152, 6, 31, "Usage",
  CellTags->"ToFA"],
Cell[321420, 13553, 244, 9, 44, "Notes",
  CellTags->"ToFA"],
Cell[321667, 13564, 175, 6, 24, "Notes",
  CellTags->"ToFA"],
Cell[321845, 13572, 123, 4, 24, "Notes",
  CellTags->"ToFA"],
Cell[321971, 13578, 303, 15, 24, "Notes",
  CellTags->"ToFA"],
Cell[322277, 13595, 62, 1, 39, "ExampleSection",
  CellTags->"ToFA"],
Cell[322342, 13598, 69, 1, 34, "ExampleText",
  CellTags->"ToFA"],
Cell[322414, 13601, 187, 5, 42, "Input",
  CellTags->"ToFA"],
Cell[322604, 13608, 69, 2, 28, "Input",
  CellTags->"ToFA"],
Cell[322676, 13612, 97, 3, 26, "Input",
  CellTags->"ToFA"],
Cell[322776, 13617, 75, 1, 34, "ExampleText",
  CellTags->"ToFA"],
Cell[322854, 13620, 91, 3, 26, "Input",
  CellTags->"ToFA"],
Cell[322948, 13625, 58, 1, 32, "ObjectName",
  CellTags->"ToFactorFA"],
Cell[323009, 13628, 55, 1, 50, "Highlight",
  CellTags->"ToFactorFA"],
Cell[323067, 13631, 258, 9, 47, "Usage",
  CellTags->"ToFactorFA"],
Cell[323328, 13642, 332, 12, 61, "Notes",
  CellTags->"ToFactorFA"],
Cell[323663, 13656, 458, 15, 112, "Notes",
  CellTags->"ToFactorFA"],
Cell[324124, 13673, 310, 9, 40, "Notes",
  CellTags->"ToFactorFA"],
Cell[324437, 13684, 413, 13, 40, "Notes",
  CellTags->"ToFactorFA"],
Cell[324853, 13699, 357, 11, 40, "Notes",
  CellTags->"ToFactorFA"],
Cell[325213, 13712, 164, 4, 40, "Notes",
  CellTags->"ToFactorFA"],
Cell[325380, 13718, 355, 15, 24, "Notes",
  CellTags->"ToFactorFA"],
Cell[325738, 13735, 68, 1, 39, "ExampleSection",
  CellTags->"ToFactorFA"],
Cell[325809, 13738, 293, 7, 50, "ExampleText",
  CellTags->"ToFactorFA"],
Cell[326105, 13747, 198, 5, 42, "Input",
  CellTags->"ToFactorFA"],
Cell[326306, 13754, 268, 5, 58, "Input",
  CellTags->"ToFactorFA"],
Cell[326577, 13761, 126, 4, 34, "ExampleText",
  CellTags->"ToFactorFA"],
Cell[326706, 13767, 249, 6, 74, "Input",
  CellTags->"ToFactorFA"],
Cell[326958, 13775, 220, 7, 50, "ExampleText",
  CellTags->"ToFactorFA"],
Cell[327181, 13784, 318, 6, 74, "Input",
  CellTags->"ToFactorFA"],
Cell[327502, 13792, 93, 1, 34, "ExampleText",
  CellTags->"ToFactorFA"],
Cell[327598, 13795, 249, 6, 74, "Input",
  CellTags->"ToFactorFA"],
Cell[327850, 13803, 167, 4, 50, "ExampleText",
  CellTags->"ToFactorFA"],
Cell[328020, 13809, 299, 7, 74, "Input",
  CellTags->"ToFactorFA"],
Cell[328322, 13818, 64, 1, 32, "ObjectName",
  CellTags->"ToFrontierDFA"],
Cell[328389, 13821, 58, 1, 50, "Highlight",
  CellTags->"ToFrontierDFA"],
Cell[328450, 13824, 222, 8, 31, "Usage",
  CellTags->"ToFrontierDFA"],
Cell[328675, 13834, 233, 9, 44, "Notes",
  CellTags->"ToFrontierDFA"],
Cell[328911, 13845, 191, 7, 24, "Notes",
  CellTags->"ToFrontierDFA"],
Cell[329105, 13854, 416, 14, 40, "Notes",
  CellTags->"ToFrontierDFA"],
Cell[329524, 13870, 239, 10, 24, "Notes",
  CellTags->"ToFrontierDFA"],
Cell[329766, 13882, 71, 1, 39, "ExampleSection",
  CellTags->"ToFrontierDFA"],
Cell[329840, 13885, 251, 6, 50, "ExampleText",
  CellTags->"ToFrontierDFA"],
Cell[330094, 13893, 149, 4, 26, "Input",
  CellTags->"ToFrontierDFA"],
Cell[330246, 13899, 230, 6, 58, "Input",
  CellTags->"ToFrontierDFA"],
Cell[330479, 13907, 191, 5, 50, "ExampleText",
  CellTags->"ToFrontierDFA"],
Cell[330673, 13914, 257, 7, 58, "Input",
  CellTags->"ToFrontierDFA"],
Cell[330933, 13923, 257, 7, 58, "Input",
  CellTags->"ToFrontierDFA"],
Cell[331193, 13932, 143, 4, 34, "ExampleText",
  CellTags->"ToFrontierDFA"],
Cell[331339, 13938, 173, 4, 42, "Input",
  CellTags->"ToFrontierDFA"],
Cell[331515, 13944, 66, 1, 32, "ObjectName",
  CellTags->"ToGeneratorsFA"],
Cell[331584, 13947, 59, 1, 50, "Highlight",
  CellTags->"ToGeneratorsFA"],
Cell[331646, 13950, 216, 5, 47, "Usage",
  CellTags->"ToGeneratorsFA"],
Cell[331865, 13957, 143, 4, 24, "Notes",
  CellTags->"ToGeneratorsFA"],
Cell[332011, 13963, 269, 11, 24, "Notes",
  CellTags->"ToGeneratorsFA"],
Cell[332283, 13976, 72, 1, 39, "ExampleSection",
  CellTags->"ToGeneratorsFA"],
Cell[332358, 13979, 92, 1, 34, "ExampleText",
  CellTags->"ToGeneratorsFA"],
Cell[332453, 13982, 197, 5, 42, "Input",
  CellTags->"ToGeneratorsFA"],
Cell[332653, 13989, 105, 3, 26, "Input",
  CellTags->"ToGeneratorsFA"],
Cell[332761, 13994, 94, 1, 34, "ExampleText",
  CellTags->"ToGeneratorsFA"],
Cell[332858, 13997, 146, 4, 42, "Input",
  CellTags->"ToGeneratorsFA"],
Cell[333007, 14003, 101, 3, 26, "Input",
  CellTags->"ToGeneratorsFA"],
Cell[333111, 14008, 56, 1, 32, "ObjectName",
  CellTags->"ToGraphFA"],
Cell[333170, 14011, 54, 1, 50, "Highlight",
  CellTags->"ToGraphFA"],
Cell[333227, 14014, 206, 5, 47, "Usage",
  CellTags->"ToGraphFA"],
Cell[333436, 14021, 303, 10, 61, "Notes",
  CellTags->"ToGraphFA"],
Cell[333742, 14033, 140, 5, 24, "Notes",
  CellTags->"ToGraphFA"],
Cell[333885, 14040, 197, 7, 24, "Notes",
  CellTags->"ToGraphFA"],
Cell[334085, 14049, 67, 1, 39, "ExampleSection",
  CellTags->"ToGraphFA"],
Cell[334155, 14052, 86, 1, 34, "ExampleText",
  CellTags->"ToGraphFA"],
Cell[334244, 14055, 215, 5, 42, "Input",
  CellTags->"ToGraphFA"],
Cell[334462, 14062, 104, 3, 26, "Input",
  CellTags->"ToGraphFA"],
Cell[334569, 14067, 76, 1, 34, "ExampleText",
  CellTags->"ToGraphFA"],
Cell[334648, 14070, 113, 3, 26, "Input",
  CellTags->"ToGraphFA"],
Cell[334764, 14075, 58, 1, 32, "ObjectName",
  CellTags->"ToKernelFA"],
Cell[334825, 14078, 55, 1, 50, "Highlight",
  CellTags->"ToKernelFA"],
Cell[334883, 14081, 233, 8, 47, "Usage",
  CellTags->"ToKernelFA"],
Cell[335119, 14091, 339, 11, 79, "Notes",
  CellTags->"ToKernelFA"],
Cell[335461, 14104, 301, 12, 24, "Notes",
  CellTags->"ToKernelFA"],
Cell[335765, 14118, 317, 13, 24, "Notes",
  CellTags->"ToKernelFA"],
Cell[336085, 14133, 156, 5, 24, "Notes",
  CellTags->"ToKernelFA"],
Cell[336244, 14140, 347, 13, 40, "Notes",
  CellTags->"ToKernelFA"],
Cell[336594, 14155, 153, 7, 24, "Notes",
  CellTags->"ToKernelFA"],
Cell[336750, 14164, 68, 1, 39, "ExampleSection",
  CellTags->"ToKernelFA"],
Cell[336821, 14167, 120, 4, 34, "ExampleText",
  CellTags->"ToKernelFA"],
Cell[336944, 14173, 221, 5, 42, "Input",
  CellTags->"ToKernelFA"],
Cell[337168, 14180, 126, 4, 42, "Input",
  CellTags->"ToKernelFA"],
Cell[337297, 14186, 162, 4, 50, "ExampleText",
  CellTags->"ToKernelFA"],
Cell[337462, 14192, 129, 3, 26, "Input",
  CellTags->"ToKernelFA"],
Cell[337594, 14197, 111, 2, 34, "ExampleText",
  CellTags->"ToKernelFA"],
Cell[337708, 14201, 131, 4, 42, "Input",
  CellTags->"ToKernelFA"],
Cell[337842, 14207, 214, 5, 50, "ExampleText",
  CellTags->"ToKernelFA"],
Cell[338059, 14214, 208, 6, 58, "Input",
  CellTags->"ToKernelFA"],
Cell[338270, 14222, 54, 1, 32, "ObjectName",
  CellTags->"ToMatrix"],
Cell[338327, 14225, 51, 1, 50, "Highlight",
  CellTags->"ToMatrix"],
Cell[338381, 14228, 147, 6, 31, "Usage",
  CellTags->"ToMatrix"],
Cell[338531, 14236, 138, 4, 24, "Notes",
  CellTags->"ToMatrix"],
Cell[338672, 14242, 163, 7, 24, "Notes",
  CellTags->"ToMatrix"],
Cell[338838, 14251, 48, 1, 32, "ObjectName",
  CellTags->"ToNFA"],
Cell[338889, 14254, 50, 1, 50, "Highlight",
  CellTags->"ToNFA"],
Cell[338942, 14257, 150, 6, 31, "Usage",
  CellTags->"ToNFA"],
Cell[339095, 14265, 117, 4, 24, "Notes",
  CellTags->"ToNFA"],
Cell[339215, 14271, 317, 15, 24, "Notes",
  CellTags->"ToNFA"],
Cell[339535, 14288, 63, 1, 39, "ExampleSection",
  CellTags->"ToNFA"],
Cell[339601, 14291, 86, 1, 34, "ExampleText",
  CellTags->"ToNFA"],
Cell[339690, 14294, 215, 5, 42, "Input",
  CellTags->"ToNFA"],
Cell[339908, 14301, 88, 3, 26, "Input",
  CellTags->"ToNFA"],
Cell[339999, 14306, 112, 3, 26, "Input",
  CellTags->"ToNFA"],
Cell[340114, 14311, 80, 1, 32, "ObjectName",
  CellTags->"ToRegularExpressionFA"],
Cell[340197, 14314, 66, 1, 50, "Highlight",
  CellTags->"ToRegularExpressionFA"],
Cell[340266, 14317, 200, 5, 47, "Usage",
  CellTags->"ToRegularExpressionFA"],
Cell[340469, 14324, 290, 10, 61, "Notes",
  CellTags->"ToRegularExpressionFA"],
Cell[340762, 14336, 223, 6, 40, "Notes",
  CellTags->"ToRegularExpressionFA"],
Cell[340988, 14344, 131, 4, 24, "Notes",
  CellTags->"ToRegularExpressionFA"],
Cell[341122, 14350, 270, 11, 24, "Notes",
  CellTags->"ToRegularExpressionFA"],
Cell[341395, 14363, 79, 1, 39, "ExampleSection",
  CellTags->"ToRegularExpressionFA"],
Cell[341477, 14366, 93, 1, 34, "ExampleText",
  CellTags->"ToRegularExpressionFA"],
Cell[341573, 14369, 226, 5, 42, "Input",
  CellTags->"ToRegularExpressionFA"],
Cell[341802, 14376, 120, 3, 26, "Input",
  CellTags->"ToRegularExpressionFA"],
Cell[341925, 14381, 270, 7, 42, "Input",
  CellTags->"ToRegularExpressionFA"],
Cell[342198, 14390, 173, 4, 26, "Input",
  CellTags->"ToRegularExpressionFA"],
Cell[342374, 14396, 256, 7, 34, "ExampleText",
  CellTags->"ToRegularExpressionFA"],
Cell[342633, 14405, 46, 1, 32, "ObjectName",
  CellTags->"ToSA"],
Cell[342682, 14408, 49, 1, 50, "Highlight",
  CellTags->"ToSA"],
Cell[342734, 14411, 154, 4, 31, "Usage",
  CellTags->"ToSA"],
Cell[342891, 14417, 244, 9, 44, "Notes",
  CellTags->"ToSA"],
Cell[343138, 14428, 127, 4, 24, "Notes",
  CellTags->"ToSA"],
Cell[343268, 14434, 137, 4, 24, "Notes",
  CellTags->"ToSA"],
Cell[343408, 14440, 317, 15, 24, "Notes",
  CellTags->"ToSA"],
Cell[343728, 14457, 62, 1, 39, "ExampleSection",
  CellTags->"ToSA"],
Cell[343793, 14460, 89, 1, 34, "ExampleText",
  CellTags->"ToSA"],
Cell[343885, 14463, 82, 2, 28, "Input",
  CellTags->"ToSA"],
Cell[343970, 14467, 93, 3, 26, "Input",
  CellTags->"ToSA"],
Cell[344066, 14472, 143, 4, 34, "ExampleText",
  CellTags->"ToSA"],
Cell[344212, 14478, 142, 4, 42, "Input",
  CellTags->"ToSA"],
Cell[344357, 14484, 75, 1, 34, "ExampleText",
  CellTags->"ToSA"],
Cell[344435, 14487, 130, 3, 26, "Input",
  CellTags->"ToSA"],
Cell[344568, 14492, 93, 3, 26, "Input",
  CellTags->"ToSA"],
Cell[344664, 14497, 54, 1, 32, "ObjectName",
  CellTags->"ToSeries"],
Cell[344721, 14500, 53, 1, 50, "Highlight",
  CellTags->"ToSeries"],
Cell[344777, 14503, 223, 7, 47, "Usage",
  CellTags->"ToSeries"],
Cell[345003, 14512, 141, 5, 24, "Notes",
  CellTags->"ToSeries"],
Cell[345147, 14519, 76, 1, 24, "Notes",
  CellTags->"ToSeries"],
Cell[345226, 14522, 256, 11, 24, "Notes",
  CellTags->"ToSeries"],
Cell[345485, 14535, 66, 1, 39, "ExampleSection",
  CellTags->"ToSeries"],
Cell[345554, 14538, 62, 1, 34, "ExampleText",
  CellTags->"ToSeries"],
Cell[345619, 14541, 89, 2, 28, "Input",
  CellTags->"ToSeries"],
Cell[345711, 14545, 132, 3, 26, "Input",
  CellTags->"ToSeries"],
Cell[345846, 14550, 94, 3, 26, "Input",
  CellTags->"ToSeries"],
Cell[345943, 14555, 64, 1, 32, "ObjectName",
  CellTags->"ToTransitions"],
Cell[346010, 14558, 58, 1, 50, "Highlight",
  CellTags->"ToTransitions"],
Cell[346071, 14561, 266, 7, 47, "Usage",
  CellTags->"ToTransitions"],
Cell[346340, 14570, 117, 4, 24, "Notes",
  CellTags->"ToTransitions"],
Cell[346460, 14576, 71, 1, 39, "ExampleSection",
  CellTags->"ToTransitions"],
Cell[346534, 14579, 163, 4, 50, "ExampleText",
  CellTags->"ToTransitions"],
Cell[346700, 14585, 114, 2, 28, "Input",
  CellTags->"ToTransitions"],
Cell[346817, 14589, 74, 1, 32, "ObjectName",
  CellTags->"TransferSequenceFA"],
Cell[346894, 14592, 63, 1, 50, "Highlight",
  CellTags->"TransferSequenceFA"],
Cell[346960, 14595, 495, 20, 47, "Usage",
  CellTags->"TransferSequenceFA"],
Cell[347458, 14617, 171, 7, 24, "Notes",
  CellTags->"TransferSequenceFA"],
Cell[347632, 14626, 318, 9, 40, "Notes",
  CellTags->"TransferSequenceFA"],
Cell[347953, 14637, 76, 1, 39, "ExampleSection",
  CellTags->"TransferSequenceFA"],
Cell[348032, 14640, 90, 1, 34, "ExampleText",
  CellTags->"TransferSequenceFA"],
Cell[348125, 14643, 208, 5, 42, "Input",
  CellTags->"TransferSequenceFA"],
Cell[348336, 14650, 100, 2, 28, "Input",
  CellTags->"TransferSequenceFA"],
Cell[348439, 14654, 134, 2, 28, "Input",
  CellTags->"TransferSequenceFA"],
Cell[348576, 14658, 78, 1, 32, "ObjectName",
  CellTags->"TransitionFunctionFA"],
Cell[348657, 14661, 65, 1, 50, "Highlight",
  CellTags->"TransitionFunctionFA"],
Cell[348725, 14664, 232, 8, 31, "Usage",
  CellTags->"TransitionFunctionFA"],
Cell[348960, 14674, 236, 9, 44, "Notes",
  CellTags->"TransitionFunctionFA"],
Cell[349199, 14685, 120, 5, 24, "Notes",
  CellTags->"TransitionFunctionFA"],
Cell[349322, 14692, 200, 7, 24, "Notes",
  CellTags->"TransitionFunctionFA"],
Cell[349525, 14701, 293, 8, 40, "Notes",
  CellTags->"TransitionFunctionFA"],
Cell[349821, 14711, 288, 11, 24, "Notes",
  CellTags->"TransitionFunctionFA"],
Cell[350112, 14724, 78, 1, 39, "ExampleSection",
  CellTags->"TransitionFunctionFA"],
Cell[350193, 14727, 108, 1, 34, "ExampleText",
  CellTags->"TransitionFunctionFA"],
Cell[350304, 14730, 214, 5, 58, "Input",
  CellTags->"TransitionFunctionFA"],
Cell[350521, 14737, 107, 1, 34, "ExampleText",
  CellTags->"TransitionFunctionFA"],
Cell[350631, 14740, 243, 6, 74, "Input",
  CellTags->"TransitionFunctionFA"],
Cell[350877, 14748, 100, 1, 34, "ExampleText",
  CellTags->"TransitionFunctionFA"],
Cell[350980, 14751, 214, 5, 58, "Input",
  CellTags->"TransitionFunctionFA"],
Cell[351197, 14758, 74, 1, 32, "ObjectName",
  CellTags->"TransitionMatrixFA"],
Cell[351274, 14761, 63, 1, 50, "Highlight",
  CellTags->"TransitionMatrixFA"],
Cell[351340, 14764, 281, 9, 47, "Usage",
  CellTags->"TransitionMatrixFA"],
Cell[351624, 14775, 296, 10, 61, "Notes",
  CellTags->"TransitionMatrixFA"],
Cell[351923, 14787, 118, 5, 24, "Notes",
  CellTags->"TransitionMatrixFA"],
Cell[352044, 14794, 229, 7, 40, "Notes",
  CellTags->"TransitionMatrixFA"],
Cell[352276, 14803, 244, 8, 24, "Notes",
  CellTags->"TransitionMatrixFA"],
Cell[352523, 14813, 414, 13, 40, "Notes",
  CellTags->"TransitionMatrixFA"],
Cell[352940, 14828, 197, 6, 24, "Notes",
  CellTags->"TransitionMatrixFA"],
Cell[353140, 14836, 390, 14, 40, "Notes",
  CellTags->"TransitionMatrixFA"],
Cell[353533, 14852, 76, 1, 39, "ExampleSection",
  CellTags->"TransitionMatrixFA"],
Cell[353612, 14855, 185, 5, 34, "ExampleText",
  CellTags->"TransitionMatrixFA"],
Cell[353800, 14862, 167, 4, 42, "Input",
  CellTags->"TransitionMatrixFA"],
Cell[353970, 14868, 116, 3, 26, "Input",
  CellTags->"TransitionMatrixFA"],
Cell[354089, 14873, 118, 3, 26, "Input",
  CellTags->"TransitionMatrixFA"],
Cell[354210, 14878, 130, 5, 34, "ExampleText",
  CellTags->"TransitionMatrixFA"],
Cell[354343, 14885, 170, 4, 42, "Input",
  CellTags->"TransitionMatrixFA"],
Cell[354516, 14891, 76, 1, 32, "ObjectName",
  CellTags->"TransitionsToMatrix"],
Cell[354595, 14894, 64, 1, 50, "Highlight",
  CellTags->"TransitionsToMatrix"],
Cell[354662, 14897, 386, 14, 47, "Usage",
  CellTags->"TransitionsToMatrix"],
Cell[355051, 14913, 244, 9, 44, "Notes",
  CellTags->"TransitionsToMatrix"],
Cell[355298, 14924, 119, 5, 24, "Notes",
  CellTags->"TransitionsToMatrix"],
Cell[355420, 14931, 145, 5, 24, "Notes",
  CellTags->"TransitionsToMatrix"],
Cell[355568, 14938, 253, 5, 40, "Notes",
  CellTags->"TransitionsToMatrix"],
Cell[355824, 14945, 304, 12, 24, "Notes",
  CellTags->"TransitionsToMatrix"],
Cell[356131, 14959, 77, 1, 39, "ExampleSection",
  CellTags->"TransitionsToMatrix"],
Cell[356211, 14962, 153, 4, 34, "ExampleText",
  CellTags->"TransitionsToMatrix"],
Cell[356367, 14968, 179, 4, 26, "Input",
  CellTags->"TransitionsToMatrix"],
Cell[356549, 14974, 140, 3, 26, "Input",
  CellTags->"TransitionsToMatrix"],
Cell[356692, 14979, 185, 6, 34, "ExampleText",
  CellTags->"TransitionsToMatrix"],
Cell[356880, 14987, 298, 5, 58, "Input",
  CellTags->"TransitionsToMatrix"],
Cell[357181, 14994, 171, 4, 26, "Input",
  CellTags->"TransitionsToMatrix"],
Cell[357355, 15000, 60, 1, 32, "ObjectName",
  CellTags->"Transitions"],
Cell[357418, 15003, 56, 1, 50, "Highlight",
  CellTags->"Transitions"],
Cell[357477, 15006, 214, 5, 47, "Usage",
  CellTags->"Transitions"],
Cell[357694, 15013, 371, 12, 40, "Notes",
  CellTags->"Transitions"],
Cell[358068, 15027, 293, 11, 24, "Notes",
  CellTags->"Transitions"],
Cell[358364, 15040, 69, 1, 39, "ExampleSection",
  CellTags->"Transitions"],
Cell[358436, 15043, 124, 4, 34, "ExampleText",
  CellTags->"Transitions"],
Cell[358563, 15049, 121, 5, 45, "Input",
  CellTags->"Transitions"],
Cell[358687, 15056, 115, 5, 45, "Input",
  CellTags->"Transitions"],
Cell[358805, 15063, 80, 1, 32, "ObjectName",
  CellTags->"TransitiveLanguageQFA"],
Cell[358888, 15066, 66, 1, 50, "Highlight",
  CellTags->"TransitiveLanguageQFA"],
Cell[358957, 15069, 202, 6, 31, "Usage",
  CellTags->"TransitiveLanguageQFA"],
Cell[359162, 15077, 259, 9, 44, "Notes",
  CellTags->"TransitiveLanguageQFA"],
Cell[359424, 15088, 183, 6, 24, "Notes",
  CellTags->"TransitiveLanguageQFA"],
Cell[359610, 15096, 180, 7, 24, "Notes",
  CellTags->"TransitiveLanguageQFA"],
Cell[359793, 15105, 79, 1, 39, "ExampleSection",
  CellTags->"TransitiveLanguageQFA"],
Cell[359875, 15108, 133, 4, 34, "ExampleText",
  CellTags->"TransitiveLanguageQFA"],
Cell[360011, 15114, 98, 2, 28, "Input",
  CellTags->"TransitiveLanguageQFA"],
Cell[360112, 15118, 120, 3, 26, "Input",
  CellTags->"TransitiveLanguageQFA"],
Cell[360235, 15123, 64, 1, 32, "ObjectName",
  CellTags->"TransitiveQFA"],
Cell[360302, 15126, 58, 1, 50, "Highlight",
  CellTags->"TransitiveQFA"],
Cell[360363, 15129, 138, 4, 31, "Usage",
  CellTags->"TransitiveQFA"],
Cell[360504, 15135, 299, 10, 61, "Notes",
  CellTags->"TransitiveQFA"],
Cell[360806, 15147, 201, 6, 40, "Notes",
  CellTags->"TransitiveQFA"],
Cell[361010, 15155, 180, 7, 24, "Notes",
  CellTags->"TransitiveQFA"],
Cell[361193, 15164, 71, 1, 39, "ExampleSection",
  CellTags->"TransitiveQFA"],
Cell[361267, 15167, 151, 4, 34, "ExampleText",
  CellTags->"TransitiveQFA"],
Cell[361421, 15173, 90, 2, 28, "Input",
  CellTags->"TransitiveQFA"],
Cell[361514, 15177, 104, 3, 26, "Input",
  CellTags->"TransitiveQFA"],
Cell[361621, 15182, 126, 3, 26, "Input",
  CellTags->"TransitiveQFA"],
Cell[361750, 15187, 56, 1, 32, "ObjectName",
  CellTags->"TransOnly"],
Cell[361809, 15190, 52, 1, 50, "Highlight",
  CellTags->"TransOnly"],
Cell[361864, 15193, 145, 6, 31, "Usage",
  CellTags->"TransOnly"],
Cell[362012, 15201, 152, 5, 24, "Notes",
  CellTags->"TransOnly"],
Cell[362167, 15208, 156, 7, 24, "Notes",
  CellTags->"TransOnly"],
Cell[362326, 15217, 67, 1, 39, "ExampleSection",
  CellTags->"TransOnly"],
Cell[362396, 15220, 73, 1, 34, "ExampleText",
  CellTags->"TransOnly"],
Cell[362472, 15223, 215, 5, 42, "Input",
  CellTags->"TransOnly"],
Cell[362690, 15230, 94, 3, 26, "Input",
  CellTags->"TransOnly"],
Cell[362787, 15235, 118, 3, 26, "Input",
  CellTags->"TransOnly"],
Cell[362908, 15240, 62, 1, 32, "ObjectName",
  CellTags->"TrapStatesFA"],
Cell[362973, 15243, 57, 1, 50, "Highlight",
  CellTags->"TrapStatesFA"],
Cell[363033, 15246, 169, 6, 31, "Usage",
  CellTags->"TrapStatesFA"],
Cell[363205, 15254, 227, 9, 44, "Notes",
  CellTags->"TrapStatesFA"],
Cell[363435, 15265, 203, 6, 40, "Notes",
  CellTags->"TrapStatesFA"],
Cell[363641, 15273, 153, 4, 24, "Notes",
  CellTags->"TrapStatesFA"],
Cell[363797, 15279, 159, 7, 24, "Notes",
  CellTags->"TrapStatesFA"],
Cell[363959, 15288, 70, 1, 39, "ExampleSection",
  CellTags->"TrapStatesFA"],
Cell[364032, 15291, 130, 4, 34, "ExampleText",
  CellTags->"TrapStatesFA"],
Cell[364165, 15297, 142, 5, 28, "Input",
  CellTags->"TrapStatesFA"],
Cell[364310, 15304, 156, 4, 42, "Input",
  CellTags->"TrapStatesFA"],
Cell[364469, 15310, 50, 1, 32, "ObjectName",
  CellTags->"TrimFA"],
Cell[364522, 15313, 51, 1, 50, "Highlight",
  CellTags->"TrimFA"],
Cell[364576, 15316, 232, 5, 47, "Usage",
  CellTags->"TrimFA"],
Cell[364811, 15323, 246, 9, 44, "Notes",
  CellTags->"TrimFA"],
Cell[365060, 15334, 196, 5, 40, "Notes",
  CellTags->"TrimFA"],
Cell[365259, 15341, 270, 11, 24, "Notes",
  CellTags->"TrimFA"],
Cell[365532, 15354, 64, 1, 39, "ExampleSection",
  CellTags->"TrimFA"],
Cell[365599, 15357, 157, 4, 50, "ExampleText",
  CellTags->"TrimFA"],
Cell[365759, 15363, 181, 5, 26, "Input",
  CellTags->"TrimFA"],
Cell[365943, 15370, 138, 4, 42, "Input",
  CellTags->"TrimFA"],
Cell[366084, 15376, 82, 1, 32, "ObjectName",
  CellTags->"TropicalIdentityMatrix"],
Cell[366169, 15379, 67, 1, 50, "Highlight",
  CellTags->"TropicalIdentityMatrix"],
Cell[366239, 15382, 239, 8, 31, "Usage",
  CellTags->"TropicalIdentityMatrix"],
Cell[366481, 15392, 357, 15, 24, "Notes",
  CellTags->"TropicalIdentityMatrix"],
Cell[366841, 15409, 80, 1, 39, "ExampleSection",
  CellTags->"TropicalIdentityMatrix"],
Cell[366924, 15412, 172, 6, 34, "ExampleText",
  CellTags->"TropicalIdentityMatrix"],
Cell[367099, 15420, 120, 2, 28, "Input",
  CellTags->"TropicalIdentityMatrix"],
Cell[367222, 15424, 54, 1, 32, "ObjectName",
  CellTags->"Tropical"],
Cell[367279, 15427, 51, 1, 50, "Highlight",
  CellTags->"Tropical"],
Cell[367333, 15430, 154, 6, 31, "Usage",
  CellTags->"Tropical"],
Cell[367490, 15438, 112, 4, 24, "Notes",
  CellTags->"Tropical"],
Cell[367605, 15444, 340, 11, 24, "Notes",
  CellTags->"Tropical"],
Cell[367948, 15457, 42, 1, 32, "ObjectName",
  CellTags->"TS"],
Cell[367993, 15460, 50, 1, 50, "Highlight",
  CellTags->"TS"],
Cell[368046, 15463, 222, 8, 31, "Usage",
  CellTags->"TS"],
Cell[368271, 15473, 318, 13, 24, "Notes",
  CellTags->"TS"],
Cell[368592, 15488, 265, 10, 40, "Notes",
  CellTags->"TS"],
Cell[368860, 15500, 372, 15, 24, "Notes",
  CellTags->"TS"],
Cell[369235, 15517, 60, 1, 39, "ExampleSection",
  CellTags->"TS"],
Cell[369298, 15520, 79, 1, 34, "ExampleText",
  CellTags->"TS"],
Cell[369380, 15523, 73, 2, 28, "Input",
  CellTags->"TS"],
Cell[369456, 15527, 73, 2, 28, "Input",
  CellTags->"TS"],
Cell[369532, 15531, 67, 2, 28, "Input",
  CellTags->"TS"],
Cell[369602, 15535, 80, 2, 28, "Input",
  CellTags->"TS"],
Cell[369685, 15539, 80, 2, 28, "Input",
  CellTags->"TS"],
Cell[369768, 15543, 74, 2, 28, "Input",
  CellTags->"TS"],
Cell[369845, 15547, 52, 1, 32, "ObjectName",
  CellTags->"UnionFA"],
Cell[369900, 15550, 52, 1, 50, "Highlight",
  CellTags->"UnionFA"],
Cell[369955, 15553, 250, 9, 47, "Usage",
  CellTags->"UnionFA"],
Cell[370208, 15564, 292, 10, 61, "Notes",
  CellTags->"UnionFA"],
Cell[370503, 15576, 134, 5, 24, "Notes",
  CellTags->"UnionFA"],
Cell[370640, 15583, 195, 8, 24, "Notes",
  CellTags->"UnionFA"],
Cell[370838, 15593, 134, 4, 24, "Notes",
  CellTags->"UnionFA"],
Cell[370975, 15599, 52, 1, 24, "Notes",
  CellTags->"UnionFA"],
Cell[371030, 15602, 348, 15, 24, "Notes",
  CellTags->"UnionFA"],
Cell[371381, 15619, 65, 1, 39, "ExampleSection",
  CellTags->"UnionFA"],
Cell[371449, 15622, 198, 5, 50, "ExampleText",
  CellTags->"UnionFA"],
Cell[371650, 15629, 187, 4, 42, "Input",
  CellTags->"UnionFA"],
Cell[371840, 15635, 123, 4, 34, "ExampleText",
  CellTags->"UnionFA"],
Cell[371966, 15641, 169, 4, 42, "Input",
  CellTags->"UnionFA"],
Cell[372138, 15647, 56, 1, 32, "ObjectName",
  CellTags->"Unlabeled"],
Cell[372197, 15650, 52, 1, 50, "Highlight",
  CellTags->"Unlabeled"],
Cell[372252, 15653, 190, 6, 31, "Usage",
  CellTags->"Unlabeled"],
Cell[372445, 15661, 108, 5, 24, "Notes",
  CellTags->"Unlabeled"],
Cell[372556, 15668, 156, 7, 24, "Notes",
  CellTags->"Unlabeled"],
Cell[372715, 15677, 67, 1, 39, "ExampleSection",
  CellTags->"Unlabeled"],
Cell[372785, 15680, 82, 1, 34, "ExampleText",
  CellTags->"Unlabeled"],
Cell[372870, 15683, 214, 5, 42, "Input",
  CellTags->"Unlabeled"],
Cell[373087, 15690, 80, 2, 28, "Input",
  CellTags->"Unlabeled"],
Cell[373170, 15694, 121, 3, 26, "Input",
  CellTags->"Unlabeled"],
Cell[373294, 15699, 54, 1, 32, "ObjectName",
  CellTags->"VerifyFA"],
Cell[373351, 15702, 53, 1, 50, "Highlight",
  CellTags->"VerifyFA"],
Cell[373407, 15705, 210, 7, 47, "Usage",
  CellTags->"VerifyFA"],
Cell[373620, 15714, 143, 5, 24, "Notes",
  CellTags->"VerifyFA"],
Cell[373766, 15721, 66, 1, 39, "ExampleSection",
  CellTags->"VerifyFA"],
Cell[373835, 15724, 54, 1, 32, "ObjectName",
  CellTags->"WordToFA"],
Cell[373892, 15727, 53, 1, 50, "Highlight",
  CellTags->"WordToFA"],
Cell[373948, 15730, 178, 6, 31, "Usage",
  CellTags->"WordToFA"],
Cell[374129, 15738, 398, 13, 79, "Notes",
  CellTags->"WordToFA"],
Cell[374530, 15753, 178, 5, 24, "Notes",
  CellTags->"WordToFA"],
Cell[374711, 15760, 178, 5, 24, "Notes",
  CellTags->"WordToFA"],
Cell[374892, 15767, 238, 8, 40, "Notes",
  CellTags->"WordToFA"],
Cell[375133, 15777, 258, 11, 24, "Notes",
  CellTags->"WordToFA"],
Cell[375394, 15790, 66, 1, 39, "ExampleSection",
  CellTags->"WordToFA"],
Cell[375463, 15793, 219, 8, 34, "ExampleText",
  CellTags->"WordToFA"],
Cell[375685, 15803, 160, 4, 42, "Input",
  CellTags->"WordToFA"],
Cell[375848, 15809, 220, 8, 34, "ExampleText",
  CellTags->"WordToFA"],
Cell[376071, 15819, 185, 4, 42, "Input",
  CellTags->"WordToFA"],
Cell[376259, 15825, 224, 8, 34, "ExampleText",
  CellTags->"WordToFA"],
Cell[376486, 15835, 185, 4, 42, "Input",
  CellTags->"WordToFA"],
Cell[376674, 15841, 122, 4, 34, "ExampleText",
  CellTags->"WordToFA"],
Cell[376799, 15847, 171, 5, 26, "Input",
  CellTags->"WordToFA"],
Cell[376973, 15854, 137, 4, 34, "ExampleText",
  CellTags->"WordToFA"],
Cell[377113, 15860, 248, 5, 58, "Input",
  CellTags->"WordToFA"],
Cell[377364, 15867, 144, 4, 34, "ExampleText",
  CellTags->"WordToFA"],
Cell[377511, 15873, 173, 4, 42, "Input",
  CellTags->"WordToFA"],
Cell[377687, 15879, 179, 5, 50, "ExampleText",
  CellTags->"WordToFA"],
Cell[377869, 15886, 339, 8, 74, "Input",
  CellTags->"WordToFA"],
Cell[378211, 15896, 60, 1, 32, "ObjectName",
  CellTags->"$regexrules"],
Cell[378274, 15899, 59, 1, 50, "Highlight",
  CellTags->"$regexrules"],
Cell[378336, 15902, 154, 4, 31, "Usage",
  CellTags->"$regexrules"],
Cell[378493, 15908, 196, 6, 24, "Notes",
  CellTags->"$regexrules"],
Cell[378692, 15916, 141, 4, 24, "Notes",
  CellTags->"$regexrules"],
Cell[378836, 15922, 164, 4, 40, "Notes",
  CellTags->"$regexrules"],
Cell[379003, 15928, 202, 9, 24, "Notes",
  CellTags->"$regexrules"],
Cell[379208, 15939, 69, 1, 39, "ExampleSection",
  CellTags->"$regexrules"],
Cell[379280, 15942, 171, 5, 34, "ExampleText",
  CellTags->"$regexrules"],
Cell[379454, 15949, 99, 2, 28, "Input",
  CellTags->"$regexrules"],
Cell[379556, 15953, 100, 2, 28, "Input",
  CellTags->"$regexrules"],
Cell[379659, 15957, 58, 1, 32, "ObjectName",
  CellTags->"ArdenSolve"],
Cell[379720, 15960, 55, 1, 50, "Highlight",
  CellTags->"ArdenSolve"],
Cell[379778, 15963, 320, 8, 63, "Usage",
  CellTags->"ArdenSolve"],
Cell[380101, 15973, 124, 5, 24, "Notes",
  CellTags->"ArdenSolve"],
Cell[380228, 15980, 213, 5, 40, "Notes",
  CellTags->"ArdenSolve"],
Cell[380444, 15987, 205, 9, 24, "Notes",
  CellTags->"ArdenSolve"],
Cell[380652, 15998, 68, 1, 39, "ExampleSection",
  CellTags->"ArdenSolve"],
Cell[380723, 16001, 271, 9, 50, "ExampleText",
  CellTags->"ArdenSolve"],
Cell[380997, 16012, 324, 7, 90, "Input",
  CellTags->"ArdenSolve"],
Cell[381324, 16021, 278, 12, 34, "ExampleText",
  CellTags->"ArdenSolve"],
Cell[381605, 16035, 140, 4, 26, "Input",
  CellTags->"ArdenSolve"],
Cell[381748, 16041, 301, 12, 34, "ExampleText",
  CellTags->"ArdenSolve"],
Cell[382052, 16055, 140, 4, 26, "Input",
  CellTags->"ArdenSolve"],
Cell[382195, 16061, 130, 4, 26, "Input",
  CellTags->"ArdenSolve"],
Cell[382328, 16067, 121, 4, 34, "ExampleText",
  CellTags->"ArdenSolve"],
Cell[382452, 16073, 136, 4, 26, "Input",
  CellTags->"ArdenSolve"],
Cell[382591, 16079, 130, 4, 26, "Input",
  CellTags->"ArdenSolve"],
Cell[382724, 16085, 197, 5, 50, "ExampleText",
  CellTags->"ArdenSolve"],
Cell[382924, 16092, 188, 5, 42, "Input",
  CellTags->"ArdenSolve"],
Cell[383115, 16099, 127, 4, 34, "ExampleText",
  CellTags->"ArdenSolve"],
Cell[383245, 16105, 229, 5, 58, "Input",
  CellTags->"ArdenSolve"],
Cell[383477, 16112, 105, 1, 34, "ExampleText",
  CellTags->"ArdenSolve"],
Cell[383585, 16115, 104, 3, 26, "Input",
  CellTags->"ArdenSolve"],
Cell[383692, 16120, 48, 1, 32, "ObjectName",
  CellTags->"Empty"],
Cell[383743, 16123, 48, 1, 50, "Highlight",
  CellTags->"Empty"],
Cell[383794, 16126, 134, 4, 31, "Usage",
  CellTags->"Empty"],
Cell[383931, 16132, 180, 7, 24, "Notes",
  CellTags->"Empty"],
Cell[384114, 16141, 52, 1, 32, "ObjectName",
  CellTags->"FormatX"],
Cell[384169, 16144, 52, 1, 50, "Highlight",
  CellTags->"FormatX"],
Cell[384224, 16147, 196, 8, 31, "Usage",
  CellTags->"FormatX"],
Cell[384423, 16157, 132, 4, 24, "Notes",
  CellTags->"FormatX"],
Cell[384558, 16163, 140, 4, 24, "Notes",
  CellTags->"FormatX"],
Cell[384701, 16169, 169, 7, 24, "Notes",
  CellTags->"FormatX"],
Cell[384873, 16178, 65, 1, 39, "ExampleSection",
  CellTags->"FormatX"],
Cell[384941, 16181, 90, 1, 34, "ExampleText",
  CellTags->"FormatX"],
Cell[385034, 16184, 223, 6, 58, "Input",
  CellTags->"FormatX"],
Cell[385260, 16192, 75, 1, 34, "ExampleText",
  CellTags->"FormatX"],
Cell[385338, 16195, 169, 4, 26, "Input",
  CellTags->"FormatX"],
Cell[385510, 16201, 94, 1, 34, "ExampleText",
  CellTags->"FormatX"],
Cell[385607, 16204, 134, 4, 42, "Input",
  CellTags->"FormatX"],
Cell[385744, 16210, 50, 1, 32, "ObjectName",
  CellTags->"REEpsQ"],
Cell[385797, 16213, 51, 1, 50, "Highlight",
  CellTags->"REEpsQ"],
Cell[385851, 16216, 197, 6, 31, "Usage",
  CellTags->"REEpsQ"],
Cell[386051, 16224, 376, 14, 40, "Notes",
  CellTags->"REEpsQ"],
Cell[386430, 16240, 155, 7, 24, "Notes",
  CellTags->"REEpsQ"],
Cell[386588, 16249, 64, 1, 39, "ExampleSection",
  CellTags->"REEpsQ"],
Cell[386655, 16252, 78, 1, 34, "ExampleText",
  CellTags->"REEpsQ"],
Cell[386736, 16255, 85, 2, 28, "Input",
  CellTags->"REEpsQ"],
Cell[386824, 16259, 90, 2, 28, "Input",
  CellTags->"REEpsQ"],
Cell[386917, 16263, 97, 2, 28, "Input",
  CellTags->"REEpsQ"],
Cell[387017, 16267, 58, 1, 32, "ObjectName",
  CellTags->"RegExpSimp"],
Cell[387078, 16270, 53, 1, 50, "Highlight",
  CellTags->"RegExpSimp"],
Cell[387134, 16273, 161, 6, 31, "Usage",
  CellTags->"RegExpSimp"],
Cell[387298, 16281, 167, 4, 40, "Notes",
  CellTags->"RegExpSimp"],
Cell[387468, 16287, 203, 7, 24, "Notes",
  CellTags->"RegExpSimp"],
Cell[387674, 16296, 84, 1, 32, "ObjectName",
  CellTags->"RegularExpressionToBEFA"],
Cell[387761, 16299, 68, 1, 50, "Highlight",
  CellTags->"RegularExpressionToBEFA"],
Cell[387832, 16302, 197, 6, 31, "Usage",
  CellTags->"RegularExpressionToBEFA"],
Cell[388032, 16310, 243, 9, 44, "Notes",
  CellTags->"RegularExpressionToBEFA"],
Cell[388278, 16321, 176, 5, 24, "Notes",
  CellTags->"RegularExpressionToBEFA"],
Cell[388457, 16328, 103, 1, 24, "Notes",
  CellTags->"RegularExpressionToBEFA"],
Cell[388563, 16331, 134, 4, 24, "Notes",
  CellTags->"RegularExpressionToBEFA"],
Cell[388700, 16337, 282, 11, 24, "Notes",
  CellTags->"RegularExpressionToBEFA"],
Cell[388985, 16350, 81, 1, 39, "ExampleSection",
  CellTags->"RegularExpressionToBEFA"],
Cell[389069, 16353, 257, 7, 34, "ExampleText",
  CellTags->"RegularExpressionToBEFA"],
Cell[389329, 16362, 147, 3, 26, "Input",
  CellTags->"RegularExpressionToBEFA"],
Cell[389479, 16367, 125, 3, 26, "Input",
  CellTags->"RegularExpressionToBEFA"],
Cell[389607, 16372, 98, 1, 34, "ExampleText",
  CellTags->"RegularExpressionToBEFA"],
Cell[389708, 16375, 150, 3, 26, "Input",
  CellTags->"RegularExpressionToBEFA"],
Cell[389861, 16380, 80, 1, 32, "ObjectName",
  CellTags->"RegularExpressionToFA"],
Cell[389944, 16383, 66, 1, 50, "Highlight",
  CellTags->"RegularExpressionToFA"],
Cell[390013, 16386, 237, 8, 47, "Usage",
  CellTags->"RegularExpressionToFA"],
Cell[390253, 16396, 337, 11, 79, "Notes",
  CellTags->"RegularExpressionToFA"],
Cell[390593, 16409, 171, 5, 24, "Notes",
  CellTags->"RegularExpressionToFA"],
Cell[390767, 16416, 180, 5, 24, "Notes",
  CellTags->"RegularExpressionToFA"],
Cell[390950, 16423, 172, 5, 24, "Notes",
  CellTags->"RegularExpressionToFA"],
Cell[391125, 16430, 245, 6, 24, "Notes",
  CellTags->"RegularExpressionToFA"],
Cell[391373, 16438, 79, 1, 39, "ExampleSection",
  CellTags->"RegularExpressionToFA"],
Cell[391455, 16441, 254, 7, 34, "ExampleText",
  CellTags->"RegularExpressionToFA"],
Cell[391712, 16450, 144, 3, 26, "Input",
  CellTags->"RegularExpressionToFA"],
Cell[391859, 16455, 128, 3, 26, "Input",
  CellTags->"RegularExpressionToFA"],
Cell[391990, 16460, 116, 3, 26, "Input",
  CellTags->"RegularExpressionToFA"],
Cell[392109, 16465, 146, 4, 34, "ExampleText",
  CellTags->"RegularExpressionToFA"],
Cell[392258, 16471, 195, 5, 26, "Input",
  CellTags->"RegularExpressionToFA"],
Cell[392456, 16478, 137, 4, 34, "ExampleText",
  CellTags->"RegularExpressionToFA"],
Cell[392596, 16484, 163, 4, 26, "Input",
  CellTags->"RegularExpressionToFA"],
Cell[392762, 16490, 84, 1, 32, "ObjectName",
  CellTags->"RegularExpressionToNNFA"],
Cell[392849, 16493, 68, 1, 50, "Highlight",
  CellTags->"RegularExpressionToNNFA"],
Cell[392920, 16496, 197, 6, 31, "Usage",
  CellTags->"RegularExpressionToNNFA"],
Cell[393120, 16504, 138, 4, 24, "Notes",
  CellTags->"RegularExpressionToNNFA"],
Cell[393261, 16510, 282, 11, 24, "Notes",
  CellTags->"RegularExpressionToNNFA"],
Cell[393546, 16523, 81, 1, 39, "ExampleSection",
  CellTags->"RegularExpressionToNNFA"],
Cell[393630, 16526, 257, 7, 34, "ExampleText",
  CellTags->"RegularExpressionToNNFA"],
Cell[393890, 16535, 147, 3, 26, "Input",
  CellTags->"RegularExpressionToNNFA"],
Cell[394040, 16540, 125, 3, 26, "Input",
  CellTags->"RegularExpressionToNNFA"],
Cell[394168, 16545, 98, 1, 34, "ExampleText",
  CellTags->"RegularExpressionToNNFA"],
Cell[394269, 16548, 150, 3, 26, "Input",
  CellTags->"RegularExpressionToNNFA"],
Cell[394422, 16553, 44, 1, 32, "ObjectName",
  CellTags->"REP"],
Cell[394469, 16556, 51, 1, 50, "Highlight",
  CellTags->"REP"],
Cell[394523, 16559, 130, 4, 31, "Usage",
  CellTags->"REP"],
Cell[394656, 16565, 97, 5, 24, "Notes",
  CellTags->"REP"],
Cell[394756, 16572, 379, 15, 24, "Notes",
  CellTags->"REP"],
Cell[395138, 16589, 61, 1, 39, "ExampleSection",
  CellTags->"REP"],
Cell[395202, 16592, 279, 10, 50, "ExampleText",
  CellTags->"REP"],
Cell[395484, 16604, 137, 5, 45, "Input",
  CellTags->"REP"],
Cell[395624, 16611, 111, 3, 26, "Input",
  CellTags->"REP"],
Cell[395738, 16616, 98, 3, 26, "Input",
  CellTags->"REP"],
Cell[395839, 16621, 58, 1, 32, "ObjectName",
  CellTags->"REQuotient"],
Cell[395900, 16624, 55, 1, 50, "Highlight",
  CellTags->"REQuotient"],
Cell[395958, 16627, 234, 8, 47, "Usage",
  CellTags->"REQuotient"],
Cell[396195, 16637, 523, 17, 40, "Notes",
  CellTags->"REQuotient"],
Cell[396721, 16656, 320, 7, 40, "Notes",
  CellTags->"REQuotient"],
Cell[397044, 16665, 185, 7, 24, "Notes",
  CellTags->"REQuotient"],
Cell[397232, 16674, 68, 1, 39, "ExampleSection",
  CellTags->"REQuotient"],
Cell[397303, 16677, 256, 8, 34, "ExampleText",
  CellTags->"REQuotient"],
Cell[397562, 16687, 101, 2, 28, "Input",
  CellTags->"REQuotient"],
Cell[397666, 16691, 188, 6, 34, "ExampleText",
  CellTags->"REQuotient"],
Cell[397857, 16699, 98, 3, 26, "Input",
  CellTags->"REQuotient"],
Cell[397958, 16704, 44, 1, 32, "ObjectName",
  CellTags->"RES"],
Cell[398005, 16707, 51, 1, 50, "Highlight",
  CellTags->"RES"],
Cell[398059, 16710, 136, 4, 31, "Usage",
  CellTags->"RES"],
Cell[398198, 16716, 94, 5, 24, "Notes",
  CellTags->"RES"],
Cell[398295, 16723, 379, 15, 24, "Notes",
  CellTags->"RES"],
Cell[398677, 16740, 61, 1, 39, "ExampleSection",
  CellTags->"RES"],
Cell[398741, 16743, 279, 10, 50, "ExampleText",
  CellTags->"RES"],
Cell[399023, 16755, 138, 5, 45, "Input",
  CellTags->"RES"],
Cell[399164, 16762, 112, 3, 26, "Input",
  CellTags->"RES"],
Cell[399279, 16767, 99, 3, 26, "Input",
  CellTags->"RES"],
Cell[399381, 16772, 130, 4, 34, "ExampleText",
  CellTags->"RES"],
Cell[399514, 16778, 118, 3, 26, "Input",
  CellTags->"RES"],
Cell[399635, 16783, 58, 1, 32, "ObjectName",
  CellTags->"RESimplify"],
Cell[399696, 16786, 55, 1, 50, "Highlight",
  CellTags->"RESimplify"],
Cell[399754, 16789, 238, 8, 47, "Usage",
  CellTags->"RESimplify"],
Cell[399995, 16799, 129, 4, 24, "Notes",
  CellTags->"RESimplify"],
Cell[400127, 16805, 268, 12, 24, "Notes",
  CellTags->"RESimplify"],
Cell[400398, 16819, 68, 1, 39, "ExampleSection",
  CellTags->"RESimplify"],
Cell[400469, 16822, 195, 6, 34, "ExampleText",
  CellTags->"RESimplify"],
Cell[400667, 16830, 95, 2, 28, "Input",
  CellTags->"RESimplify"],
Cell[400765, 16834, 220, 7, 34, "ExampleText",
  CellTags->"RESimplify"],
Cell[400988, 16843, 110, 2, 28, "Input",
  CellTags->"RESimplify"],
Cell[401101, 16847, 44, 1, 32, "ObjectName",
  CellTags->"RET"],
Cell[401148, 16850, 51, 1, 50, "Highlight",
  CellTags->"RET"],
Cell[401202, 16853, 138, 4, 31, "Usage",
  CellTags->"RET"],
Cell[401343, 16859, 97, 5, 24, "Notes",
  CellTags->"RET"],
Cell[401443, 16866, 379, 15, 24, "Notes",
  CellTags->"RET"],
Cell[401825, 16883, 61, 1, 39, "ExampleSection",
  CellTags->"RET"],
Cell[401889, 16886, 279, 10, 50, "ExampleText",
  CellTags->"RET"],
Cell[402171, 16898, 137, 5, 45, "Input",
  CellTags->"RET"],
Cell[402311, 16905, 111, 3, 26, "Input",
  CellTags->"RET"],
Cell[402425, 16910, 98, 3, 26, "Input",
  CellTags->"RET"],
Cell[402526, 16915, 62, 1, 32, "ObjectName",
  CellTags->"SubstituteRE"],
Cell[402591, 16918, 57, 1, 50, "Highlight",
  CellTags->"SubstituteRE"],
Cell[402651, 16921, 233, 8, 47, "Usage",
  CellTags->"SubstituteRE"],
Cell[402887, 16931, 133, 5, 24, "Notes",
  CellTags->"SubstituteRE"],
Cell[403023, 16938, 205, 9, 24, "Notes",
  CellTags->"SubstituteRE"],
Cell[403231, 16949, 70, 1, 39, "ExampleSection",
  CellTags->"SubstituteRE"],
Cell[403304, 16952, 167, 4, 50, "ExampleText",
  CellTags->"SubstituteRE"],
Cell[403474, 16958, 281, 7, 90, "Input",
  CellTags->"SubstituteRE"],
Cell[403758, 16967, 104, 1, 34, "ExampleText",
  CellTags->"SubstituteRE"],
Cell[403865, 16970, 106, 3, 26, "Input",
  CellTags->"SubstituteRE"],
Cell[403974, 16975, 293, 12, 34, "ExampleText",
  CellTags->"SubstituteRE"],
Cell[404270, 16989, 134, 4, 42, "Input",
  CellTags->"SubstituteRE"],
Cell[404407, 16995, 113, 2, 34, "ExampleText",
  CellTags->"SubstituteRE"],
Cell[404523, 16999, 232, 6, 74, "Input",
  CellTags->"SubstituteRE"],
Cell[404758, 17007, 54, 1, 32, "ObjectName",
  CellTags->"ToRegExp"],
Cell[404815, 17010, 53, 1, 50, "Highlight",
  CellTags->"ToRegExp"],
Cell[404871, 17013, 417, 14, 63, "Usage",
  CellTags->"ToRegExp"],
Cell[405291, 17029, 138, 4, 24, "Notes",
  CellTags->"ToRegExp"],
Cell[405432, 17035, 498, 22, 24, "Notes",
  CellTags->"ToRegExp"],
Cell[405933, 17059, 66, 1, 39, "ExampleSection",
  CellTags->"ToRegExp"],
Cell[406002, 17062, 150, 4, 50, "ExampleText",
  CellTags->"ToRegExp"],
Cell[406155, 17068, 110, 2, 28, "Input",
  CellTags->"ToRegExp"],
Cell[406268, 17072, 93, 3, 26, "Input",
  CellTags->"ToRegExp"],
Cell[406364, 17077, 84, 2, 26, "Input",
  CellTags->"ToRegExp"],
Cell[406451, 17081, 64, 1, 32, "ObjectName",
  CellTags->"BFunctionToCA"],
Cell[406518, 17084, 58, 1, 50, "Highlight",
  CellTags->"BFunctionToCA"],
Cell[406579, 17087, 376, 14, 47, "Usage",
  CellTags->"BFunctionToCA"],
Cell[406958, 17103, 309, 12, 40, "Notes",
  CellTags->"BFunctionToCA"],
Cell[407270, 17117, 224, 9, 24, "Notes",
  CellTags->"BFunctionToCA"],
Cell[407497, 17128, 283, 10, 24, "Notes",
  CellTags->"BFunctionToCA"],
Cell[407783, 17140, 269, 11, 24, "Notes",
  CellTags->"BFunctionToCA"],
Cell[408055, 17153, 71, 1, 39, "ExampleSection",
  CellTags->"BFunctionToCA"],
Cell[408129, 17156, 136, 4, 34, "ExampleText",
  CellTags->"BFunctionToCA"],
Cell[408268, 17162, 126, 3, 26, "Input",
  CellTags->"BFunctionToCA"],
Cell[408397, 17167, 75, 1, 34, "ExampleText",
  CellTags->"BFunctionToCA"],
Cell[408475, 17170, 157, 4, 42, "Input",
  CellTags->"BFunctionToCA"],
Cell[408635, 17176, 189, 6, 34, "ExampleText",
  CellTags->"BFunctionToCA"],
Cell[408827, 17184, 122, 3, 26, "Input",
  CellTags->"BFunctionToCA"],
Cell[408952, 17189, 75, 1, 34, "ExampleText",
  CellTags->"BFunctionToCA"],
Cell[409030, 17192, 163, 4, 42, "Input",
  CellTags->"BFunctionToCA"],
Cell[409196, 17198, 42, 1, 32, "ObjectName",
  CellTags->"CA"],
Cell[409241, 17201, 50, 1, 50, "Highlight",
  CellTags->"CA"],
Cell[409294, 17204, 272, 10, 47, "Usage",
  CellTags->"CA"],
Cell[409569, 17216, 381, 12, 40, "Notes",
  CellTags->"CA"],
Cell[409953, 17230, 202, 9, 24, "Notes",
  CellTags->"CA"],
Cell[410158, 17241, 60, 1, 39, "ExampleSection",
  CellTags->"CA"],
Cell[410221, 17244, 130, 4, 34, "ExampleText",
  CellTags->"CA"],
Cell[410354, 17250, 178, 5, 42, "Input",
  CellTags->"CA"],
Cell[410535, 17257, 239, 6, 50, "ExampleText",
  CellTags->"CA"],
Cell[410777, 17265, 350, 8, 58, "Input",
  CellTags->"CA"],
Cell[411130, 17275, 130, 4, 34, "ExampleText",
  CellTags->"CA"],
Cell[411263, 17281, 69, 2, 28, "Input",
  CellTags->"CA"],
Cell[411335, 17285, 58, 1, 32, "ObjectName",
  CellTags->"ClassifyCA"],
Cell[411396, 17288, 55, 1, 50, "Highlight",
  CellTags->"ClassifyCA"],
Cell[411454, 17291, 282, 7, 47, "Usage",
  CellTags->"ClassifyCA"],
Cell[411739, 17300, 296, 10, 61, "Notes",
  CellTags->"ClassifyCA"],
Cell[412038, 17312, 417, 17, 97, "Notes",
  CellTags->"ClassifyCA"],
Cell[412458, 17331, 157, 5, 24, "Notes",
  CellTags->"ClassifyCA"],
Cell[412618, 17338, 186, 5, 40, "Notes",
  CellTags->"ClassifyCA"],
Cell[412807, 17345, 157, 4, 40, "Notes",
  CellTags->"ClassifyCA"],
Cell[412967, 17351, 200, 9, 24, "Notes",
  CellTags->"ClassifyCA"],
Cell[413170, 17362, 68, 1, 39, "ExampleSection",
  CellTags->"ClassifyCA"],
Cell[413241, 17365, 102, 1, 34, "ExampleText",
  CellTags->"ClassifyCA"],
Cell[413346, 17368, 124, 3, 26, "Input",
  CellTags->"ClassifyCA"],
Cell[413473, 17373, 121, 3, 26, "Input",
  CellTags->"ClassifyCA"],
Cell[413597, 17378, 121, 3, 26, "Input",
  CellTags->"ClassifyCA"],
Cell[413721, 17383, 121, 3, 26, "Input",
  CellTags->"ClassifyCA"],
Cell[413845, 17388, 164, 4, 50, "ExampleText",
  CellTags->"ClassifyCA"],
Cell[414012, 17394, 120, 3, 26, "Input",
  CellTags->"ClassifyCA"],
Cell[414135, 17399, 151, 4, 34, "ExampleText",
  CellTags->"ClassifyCA"],
Cell[414289, 17405, 176, 4, 26, "Input",
  CellTags->"ClassifyCA"],
Cell[414468, 17411, 56, 1, 32, "ObjectName",
  CellTags->"ComposeCA"],
Cell[414527, 17414, 54, 1, 50, "Highlight",
  CellTags->"ComposeCA"],
Cell[414584, 17417, 257, 8, 47, "Usage",
  CellTags->"ComposeCA"],
Cell[414844, 17427, 247, 9, 44, "Notes",
  CellTags->"ComposeCA"],
Cell[415094, 17438, 284, 9, 40, "Notes",
  CellTags->"ComposeCA"],
Cell[415381, 17449, 168, 6, 24, "Notes",
  CellTags->"ComposeCA"],
Cell[415552, 17457, 131, 6, 24, "Notes",
  CellTags->"ComposeCA"],
Cell[415686, 17465, 67, 1, 39, "ExampleSection",
  CellTags->"ComposeCA"],
Cell[415756, 17468, 99, 1, 34, "ExampleText",
  CellTags->"ComposeCA"],
Cell[415858, 17471, 113, 3, 26, "Input",
  CellTags->"ComposeCA"],
Cell[415974, 17476, 136, 3, 26, "Input",
  CellTags->"ComposeCA"],
Cell[416113, 17481, 50, 1, 32, "ObjectName",
  CellTags->"Cyclic"],
Cell[416166, 17484, 49, 1, 50, "Highlight",
  CellTags->"Cyclic"],
Cell[416218, 17487, 237, 8, 47, "Usage",
  CellTags->"Cyclic"],
Cell[416458, 17497, 338, 15, 24, "Notes",
  CellTags->"Cyclic"],
Cell[416799, 17514, 60, 1, 32, "ObjectName",
  CellTags->"EvolutionCA"],
Cell[416862, 17517, 56, 1, 50, "Highlight",
  CellTags->"EvolutionCA"],
Cell[416921, 17520, 199, 5, 47, "Usage",
  CellTags->"EvolutionCA"],
Cell[417123, 17527, 299, 10, 61, "Notes",
  CellTags->"EvolutionCA"],
Cell[417425, 17539, 142, 5, 24, "Notes",
  CellTags->"EvolutionCA"],
Cell[417570, 17546, 320, 12, 24, "Notes",
  CellTags->"EvolutionCA"],
Cell[417893, 17560, 112, 5, 24, "Notes",
  CellTags->"EvolutionCA"],
Cell[418008, 17567, 154, 6, 24, "Notes",
  CellTags->"EvolutionCA"],
Cell[418165, 17575, 328, 13, 24, "Notes",
  CellTags->"EvolutionCA"],
Cell[418496, 17590, 69, 1, 39, "ExampleSection",
  CellTags->"EvolutionCA"],
Cell[418568, 17593, 88, 1, 34, "ExampleText",
  CellTags->"EvolutionCA"],
Cell[418659, 17596, 90, 2, 28, "Input",
  CellTags->"EvolutionCA"],
Cell[418752, 17600, 197, 5, 50, "ExampleText",
  CellTags->"EvolutionCA"],
Cell[418952, 17607, 236, 5, 42, "Input",
  CellTags->"EvolutionCA"],
Cell[419191, 17614, 62, 1, 32, "ObjectName",
  CellTags->"DependencyCA"],
Cell[419256, 17617, 57, 1, 50, "Highlight",
  CellTags->"DependencyCA"],
Cell[419316, 17620, 235, 7, 47, "Usage",
  CellTags->"DependencyCA"],
Cell[419554, 17629, 165, 7, 24, "Notes",
  CellTags->"DependencyCA"],
Cell[419722, 17638, 70, 1, 39, "ExampleSection",
  CellTags->"DependencyCA"],
Cell[419795, 17641, 160, 4, 50, "ExampleText",
  CellTags->"DependencyCA"],
Cell[419958, 17647, 143, 4, 42, "Input",
  CellTags->"DependencyCA"],
Cell[420104, 17653, 95, 3, 26, "Input",
  CellTags->"DependencyCA"],
Cell[420202, 17658, 64, 1, 32, "ObjectName",
  CellTags->"FSurjectiveCA"],
Cell[420269, 17661, 58, 1, 50, "Highlight",
  CellTags->"FSurjectiveCA"],
Cell[420330, 17664, 213, 6, 47, "Usage",
  CellTags->"FSurjectiveCA"],
Cell[420546, 17672, 185, 5, 24, "Notes",
  CellTags->"FSurjectiveCA"],
Cell[420734, 17679, 334, 12, 40, "Notes",
  CellTags->"FSurjectiveCA"],
Cell[421071, 17693, 206, 9, 24, "Notes",
  CellTags->"FSurjectiveCA"],
Cell[421280, 17704, 71, 1, 39, "ExampleSection",
  CellTags->"FSurjectiveCA"],
Cell[421354, 17707, 177, 4, 50, "ExampleText",
  CellTags->"FSurjectiveCA"],
Cell[421534, 17713, 186, 4, 42, "Input",
  CellTags->"FSurjectiveCA"],
Cell[421723, 17719, 130, 3, 26, "Input",
  CellTags->"FSurjectiveCA"],
Cell[421856, 17724, 62, 1, 32, "ObjectName",
  CellTags->"FunctionToCA"],
Cell[421921, 17727, 57, 1, 50, "Highlight",
  CellTags->"FunctionToCA"],
Cell[421981, 17730, 295, 10, 47, "Usage",
  CellTags->"FunctionToCA"],
Cell[422279, 17742, 235, 8, 40, "Notes",
  CellTags->"FunctionToCA"],
Cell[422517, 17752, 374, 15, 24, "Notes",
  CellTags->"FunctionToCA"],
Cell[422894, 17769, 70, 1, 39, "ExampleSection",
  CellTags->"FunctionToCA"],
Cell[422967, 17772, 123, 4, 34, "ExampleText",
  CellTags->"FunctionToCA"],
Cell[423093, 17778, 134, 3, 26, "Input",
  CellTags->"FunctionToCA"],
Cell[423230, 17783, 89, 1, 34, "ExampleText",
  CellTags->"FunctionToCA"],
Cell[423322, 17786, 100, 3, 26, "Input",
  CellTags->"FunctionToCA"],
Cell[423425, 17791, 110, 3, 26, "Input",
  CellTags->"FunctionToCA"],
Cell[423538, 17796, 60, 1, 32, "ObjectName",
  CellTags->"GlobalMapCA"],
Cell[423601, 17799, 56, 1, 50, "Highlight",
  CellTags->"GlobalMapCA"],
Cell[423660, 17802, 310, 10, 47, "Usage",
  CellTags->"GlobalMapCA"],
Cell[423973, 17814, 274, 9, 61, "Notes",
  CellTags->"GlobalMapCA"],
Cell[424250, 17825, 112, 5, 24, "Notes",
  CellTags->"GlobalMapCA"],
Cell[424365, 17832, 142, 5, 24, "Notes",
  CellTags->"GlobalMapCA"],
Cell[424510, 17839, 189, 6, 40, "Notes",
  CellTags->"GlobalMapCA"],
Cell[424702, 17847, 164, 7, 24, "Notes",
  CellTags->"GlobalMapCA"],
Cell[424869, 17856, 69, 1, 39, "ExampleSection",
  CellTags->"GlobalMapCA"],
Cell[424941, 17859, 100, 1, 34, "ExampleText",
  CellTags->"GlobalMapCA"],
Cell[425044, 17862, 108, 3, 26, "Input",
  CellTags->"GlobalMapCA"],
Cell[425155, 17867, 148, 4, 26, "Input",
  CellTags->"GlobalMapCA"],
Cell[425306, 17873, 127, 3, 26, "Input",
  CellTags->"GlobalMapCA"],
Cell[425436, 17878, 89, 1, 34, "ExampleText",
  CellTags->"GlobalMapCA"],
Cell[425528, 17881, 264, 5, 58, "Input",
  CellTags->"GlobalMapCA"],
Cell[425795, 17888, 56, 1, 32, "ObjectName",
  CellTags->"InverseCA"],
Cell[425854, 17891, 54, 1, 50, "Highlight",
  CellTags->"InverseCA"],
Cell[425911, 17894, 180, 6, 31, "Usage",
  CellTags->"InverseCA"],
Cell[426094, 17902, 292, 10, 61, "Notes",
  CellTags->"InverseCA"],
Cell[426389, 17914, 147, 5, 24, "Notes",
  CellTags->"InverseCA"],
Cell[426539, 17921, 201, 8, 24, "Notes",
  CellTags->"InverseCA"],
Cell[426743, 17931, 196, 9, 24, "Notes",
  CellTags->"InverseCA"],
Cell[426942, 17942, 67, 1, 39, "ExampleSection",
  CellTags->"InverseCA"],
Cell[427012, 17945, 87, 1, 34, "ExampleText",
  CellTags->"InverseCA"],
Cell[427102, 17948, 170, 5, 42, "Input",
  CellTags->"InverseCA"],
Cell[427275, 17955, 141, 4, 34, "ExampleText",
  CellTags->"InverseCA"],
Cell[427419, 17961, 198, 6, 58, "Input",
  CellTags->"InverseCA"],
Cell[427620, 17969, 56, 1, 32, "ObjectName",
  CellTags->"IterateCA"],
Cell[427679, 17972, 54, 1, 50, "Highlight",
  CellTags->"IterateCA"],
Cell[427736, 17975, 222, 8, 31, "Usage",
  CellTags->"IterateCA"],
Cell[427961, 17985, 249, 9, 44, "Notes",
  CellTags->"IterateCA"],
Cell[428213, 17996, 383, 13, 40, "Notes",
  CellTags->"IterateCA"],
Cell[428599, 18011, 67, 1, 39, "ExampleSection",
  CellTags->"IterateCA"],
Cell[428669, 18014, 146, 4, 34, "ExampleText",
  CellTags->"IterateCA"],
Cell[428818, 18020, 94, 2, 28, "Input",
  CellTags->"IterateCA"],
Cell[428915, 18024, 144, 4, 42, "Input",
  CellTags->"IterateCA"],
Cell[429062, 18030, 58, 1, 32, "ObjectName",
  CellTags->"LocalMapCA"],
Cell[429123, 18033, 55, 1, 50, "Highlight",
  CellTags->"LocalMapCA"],
Cell[429181, 18036, 213, 8, 31, "Usage",
  CellTags->"LocalMapCA"],
Cell[429397, 18046, 229, 9, 44, "Notes",
  CellTags->"LocalMapCA"],
Cell[429629, 18057, 111, 5, 24, "Notes",
  CellTags->"LocalMapCA"],
Cell[429743, 18064, 141, 5, 24, "Notes",
  CellTags->"LocalMapCA"],
Cell[429887, 18071, 188, 6, 40, "Notes",
  CellTags->"LocalMapCA"],
Cell[430078, 18079, 165, 7, 24, "Notes",
  CellTags->"LocalMapCA"],
Cell[430246, 18088, 68, 1, 39, "ExampleSection",
  CellTags->"LocalMapCA"],
Cell[430317, 18091, 99, 1, 34, "ExampleText",
  CellTags->"LocalMapCA"],
Cell[430419, 18094, 106, 3, 26, "Input",
  CellTags->"LocalMapCA"],
Cell[430528, 18099, 123, 3, 26, "Input",
  CellTags->"LocalMapCA"],
Cell[430654, 18104, 176, 4, 50, "ExampleText",
  CellTags->"LocalMapCA"],
Cell[430833, 18110, 177, 4, 42, "Input",
  CellTags->"LocalMapCA"],
Cell[431013, 18116, 58, 1, 32, "ObjectName",
  CellTags->"NumberOfCA"],
Cell[431074, 18119, 55, 1, 50, "Highlight",
  CellTags->"NumberOfCA"],
Cell[431132, 18122, 234, 8, 47, "Usage",
  CellTags->"NumberOfCA"],
Cell[431369, 18132, 132, 6, 24, "Notes",
  CellTags->"NumberOfCA"],
Cell[431504, 18140, 183, 7, 24, "Notes",
  CellTags->"NumberOfCA"],
Cell[431690, 18149, 68, 1, 39, "ExampleSection",
  CellTags->"NumberOfCA"],
Cell[431761, 18152, 75, 1, 34, "ExampleText",
  CellTags->"NumberOfCA"],
Cell[431839, 18155, 116, 2, 28, "Input",
  CellTags->"NumberOfCA"],
Cell[431958, 18159, 78, 1, 32, "ObjectName",
  CellTags->"NumberOfTotalisticCA"],
Cell[432039, 18162, 65, 1, 50, "Highlight",
  CellTags->"NumberOfTotalisticCA"],
Cell[432107, 18165, 268, 8, 47, "Usage",
  CellTags->"NumberOfTotalisticCA"],
Cell[432378, 18175, 150, 6, 24, "Notes",
  CellTags->"NumberOfTotalisticCA"],
Cell[432531, 18183, 401, 15, 24, "Notes",
  CellTags->"NumberOfTotalisticCA"],
Cell[432935, 18200, 78, 1, 39, "ExampleSection",
  CellTags->"NumberOfTotalisticCA"],
Cell[433016, 18203, 79, 1, 34, "ExampleText",
  CellTags->"NumberOfTotalisticCA"],
Cell[433098, 18206, 136, 2, 28, "Input",
  CellTags->"NumberOfTotalisticCA"],
Cell[433237, 18210, 52, 1, 32, "ObjectName",
  CellTags->"OrbitCA"],
Cell[433292, 18213, 52, 1, 50, "Highlight",
  CellTags->"OrbitCA"],
Cell[433347, 18216, 294, 10, 47, "Usage",
  CellTags->"OrbitCA"],
Cell[433644, 18228, 340, 11, 79, "Notes",
  CellTags->"OrbitCA"],
Cell[433987, 18241, 138, 5, 24, "Notes",
  CellTags->"OrbitCA"],
Cell[434128, 18248, 196, 6, 24, "Notes",
  CellTags->"OrbitCA"],
Cell[434327, 18256, 235, 8, 40, "Notes",
  CellTags->"OrbitCA"],
Cell[434565, 18266, 351, 7, 56, "Notes",
  CellTags->"OrbitCA"],
Cell[434919, 18275, 263, 11, 24, "Notes",
  CellTags->"OrbitCA"],
Cell[435185, 18288, 65, 1, 39, "ExampleSection",
  CellTags->"OrbitCA"],
Cell[435253, 18291, 128, 4, 34, "ExampleText",
  CellTags->"OrbitCA"],
Cell[435384, 18297, 147, 5, 45, "Input",
  CellTags->"OrbitCA"],
Cell[435534, 18304, 103, 3, 26, "Input",
  CellTags->"OrbitCA"],
Cell[435640, 18309, 117, 4, 34, "ExampleText",
  CellTags->"OrbitCA"],
Cell[435760, 18315, 133, 3, 26, "Input",
  CellTags->"OrbitCA"],
Cell[435896, 18320, 52, 1, 32, "ObjectName",
  CellTags->"Padding"],
Cell[435951, 18323, 50, 1, 50, "Highlight",
  CellTags->"Padding"],
Cell[436004, 18326, 209, 6, 47, "Usage",
  CellTags->"Padding"],
Cell[436216, 18334, 149, 5, 24, "Notes",
  CellTags->"Padding"],
Cell[436368, 18341, 363, 15, 24, "Notes",
  CellTags->"Padding"],
Cell[436734, 18358, 65, 1, 39, "ExampleSection",
  CellTags->"Padding"],
Cell[436802, 18361, 125, 4, 34, "ExampleText",
  CellTags->"Padding"],
Cell[436930, 18367, 110, 3, 26, "Input",
  CellTags->"Padding"],
Cell[437043, 18372, 129, 3, 26, "Input",
  CellTags->"Padding"],
Cell[437175, 18377, 136, 3, 26, "Input",
  CellTags->"Padding"],
Cell[437314, 18382, 66, 1, 32, "ObjectName",
  CellTags->"PolynomialToCA"],
Cell[437383, 18385, 59, 1, 50, "Highlight",
  CellTags->"PolynomialToCA"],
Cell[437445, 18388, 303, 10, 47, "Usage",
  CellTags->"PolynomialToCA"],
Cell[437751, 18400, 271, 11, 24, "Notes",
  CellTags->"PolynomialToCA"],
Cell[438025, 18413, 372, 15, 24, "Notes",
  CellTags->"PolynomialToCA"],
Cell[438400, 18430, 72, 1, 39, "ExampleSection",
  CellTags->"PolynomialToCA"],
Cell[438475, 18433, 146, 4, 34, "ExampleText",
  CellTags->"PolynomialToCA"],
Cell[438624, 18439, 152, 4, 42, "Input",
  CellTags->"PolynomialToCA"],
Cell[438779, 18445, 52, 1, 32, "ObjectName",
  CellTags->"PrintCA"],
Cell[438834, 18448, 52, 1, 50, "Highlight",
  CellTags->"PrintCA"],
Cell[438889, 18451, 209, 6, 47, "Usage",
  CellTags->"PrintCA"],
Cell[439101, 18459, 129, 6, 24, "Notes",
  CellTags->"PrintCA"],
Cell[439233, 18467, 65, 1, 39, "ExampleSection",
  CellTags->"PrintCA"],
Cell[439301, 18470, 105, 2, 34, "ExampleText",
  CellTags->"PrintCA"],
Cell[439409, 18474, 84, 2, 28, "Input",
  CellTags->"PrintCA"],
Cell[439496, 18478, 90, 2, 28, "Input",
  CellTags->"PrintCA"],
Cell[439589, 18482, 50, 1, 32, "ObjectName",
  CellTags->"RuleCA"],
Cell[439642, 18485, 51, 1, 50, "Highlight",
  CellTags->"RuleCA"],
Cell[439696, 18488, 166, 6, 31, "Usage",
  CellTags->"RuleCA"],
Cell[439865, 18496, 488, 18, 40, "Notes",
  CellTags->"RuleCA"],
Cell[440356, 18516, 106, 5, 24, "Notes",
  CellTags->"RuleCA"],
Cell[440465, 18523, 235, 11, 24, "Notes",
  CellTags->"RuleCA"],
Cell[440703, 18536, 64, 1, 39, "ExampleSection",
  CellTags->"RuleCA"],
Cell[440770, 18539, 81, 1, 34, "ExampleText",
  CellTags->"RuleCA"],
Cell[440854, 18542, 91, 5, 45, "Input",
  CellTags->"RuleCA"],
Cell[440948, 18549, 56, 1, 32, "ObjectName",
  CellTags->"RulesToCA"],
Cell[441007, 18552, 54, 1, 50, "Highlight",
  CellTags->"RulesToCA"],
Cell[441064, 18555, 312, 10, 47, "Usage",
  CellTags->"RulesToCA"],
Cell[441379, 18567, 762, 29, 40, "Notes",
  CellTags->"RulesToCA"],
Cell[442144, 18598, 245, 7, 40, "Notes",
  CellTags->"RulesToCA"],
Cell[442392, 18607, 379, 15, 24, "Notes",
  CellTags->"RulesToCA"],
Cell[442774, 18624, 67, 1, 39, "ExampleSection",
  CellTags->"RulesToCA"],
Cell[442844, 18627, 150, 4, 34, "ExampleText",
  CellTags->"RulesToCA"],
Cell[442997, 18633, 527, 11, 170, "Input",
  CellTags->"RulesToCA"],
Cell[443527, 18646, 147, 4, 34, "ExampleText",
  CellTags->"RulesToCA"],
Cell[443677, 18652, 223, 5, 42, "Input",
  CellTags->"RulesToCA"],
Cell[443903, 18659, 275, 5, 42, "Input",
  CellTags->"RulesToCA"],
Cell[444181, 18666, 72, 1, 32, "ObjectName",
  CellTags->"SeedConfiguration"],
Cell[444256, 18669, 62, 1, 50, "Highlight",
  CellTags->"SeedConfiguration"],
Cell[444321, 18672, 234, 8, 31, "Usage",
  CellTags->"SeedConfiguration"],
Cell[444558, 18682, 377, 11, 97, "Notes",
  CellTags->"SeedConfiguration"],
Cell[444938, 18695, 690, 23, 106, "Notes",
  CellTags->"SeedConfiguration"],
Cell[445631, 18720, 238, 7, 40, "Notes",
  CellTags->"SeedConfiguration"],
Cell[445872, 18729, 109, 1, 24, "Notes",
  CellTags->"SeedConfiguration"],
Cell[445984, 18732, 262, 11, 24, "Notes",
  CellTags->"SeedConfiguration"],
Cell[446249, 18745, 75, 1, 39, "ExampleSection",
  CellTags->"SeedConfiguration"],
Cell[446327, 18748, 94, 1, 34, "ExampleText",
  CellTags->"SeedConfiguration"],
Cell[446424, 18751, 122, 3, 26, "Input",
  CellTags->"SeedConfiguration"],
Cell[446549, 18756, 143, 3, 26, "Input",
  CellTags->"SeedConfiguration"],
Cell[446695, 18761, 88, 1, 34, "ExampleText",
  CellTags->"SeedConfiguration"],
Cell[446786, 18764, 141, 3, 26, "Input",
  CellTags->"SeedConfiguration"],
Cell[446930, 18769, 146, 3, 26, "Input",
  CellTags->"SeedConfiguration"],
Cell[447079, 18774, 204, 6, 34, "ExampleText",
  CellTags->"SeedConfiguration"],
Cell[447286, 18782, 133, 3, 26, "Input",
  CellTags->"SeedConfiguration"],
Cell[447422, 18787, 361, 7, 66, "ExampleText",
  CellTags->"SeedConfiguration"],
Cell[447786, 18796, 151, 4, 26, "Input",
  CellTags->"SeedConfiguration"],
Cell[447940, 18802, 58, 1, 32, "ObjectName",
  CellTags->"SimplifyCA"],
Cell[448001, 18805, 55, 1, 50, "Highlight",
  CellTags->"SimplifyCA"],
Cell[448059, 18808, 253, 7, 47, "Usage",
  CellTags->"SimplifyCA"],
Cell[448315, 18817, 226, 9, 44, "Notes",
  CellTags->"SimplifyCA"],
Cell[448544, 18828, 150, 5, 24, "Notes",
  CellTags->"SimplifyCA"],
Cell[448697, 18835, 133, 6, 24, "Notes",
  CellTags->"SimplifyCA"],
Cell[448833, 18843, 68, 1, 39, "ExampleSection",
  CellTags->"SimplifyCA"],
Cell[448904, 18846, 237, 5, 50, "ExampleText",
  CellTags->"SimplifyCA"],
Cell[449144, 18853, 106, 3, 26, "Input",
  CellTags->"SimplifyCA"],
Cell[449253, 18858, 148, 4, 42, "Input",
  CellTags->"SimplifyCA"],
Cell[449404, 18864, 163, 4, 50, "ExampleText",
  CellTags->"SimplifyCA"],
Cell[449570, 18870, 247, 7, 74, "Input",
  CellTags->"SimplifyCA"],
Cell[449820, 18879, 93, 3, 26, "Input",
  CellTags->"SimplifyCA"],
Cell[449916, 18884, 56, 1, 32, "ObjectName",
  CellTags->"ToRulesCA"],
Cell[449975, 18887, 54, 1, 50, "Highlight",
  CellTags->"ToRulesCA"],
Cell[450032, 18890, 200, 5, 47, "Usage",
  CellTags->"ToRulesCA"],
Cell[450235, 18897, 227, 9, 44, "Notes",
  CellTags->"ToRulesCA"],
Cell[450465, 18908, 163, 5, 24, "Notes",
  CellTags->"ToRulesCA"],
Cell[450631, 18915, 67, 1, 39, "ExampleSection",
  CellTags->"ToRulesCA"],
Cell[450701, 18918, 165, 6, 34, "ExampleText",
  CellTags->"ToRulesCA"],
Cell[450869, 18926, 101, 3, 26, "Input",
  CellTags->"ToRulesCA"],
Cell[450973, 18931, 66, 1, 34, "ExampleText",
  CellTags->"ToRulesCA"],
Cell[451042, 18934, 123, 3, 26, "Input",
  CellTags->"ToRulesCA"],
Cell[451168, 18939, 74, 1, 32, "ObjectName",
  CellTags->"TotalisticRuleToCA"],
Cell[451245, 18942, 63, 1, 50, "Highlight",
  CellTags->"TotalisticRuleToCA"],
Cell[451311, 18945, 240, 8, 47, "Usage",
  CellTags->"TotalisticRuleToCA"],
Cell[451554, 18955, 176, 6, 24, "Notes",
  CellTags->"TotalisticRuleToCA"],
Cell[451733, 18963, 187, 7, 24, "Notes",
  CellTags->"TotalisticRuleToCA"],
Cell[451923, 18972, 76, 1, 39, "ExampleSection",
  CellTags->"TotalisticRuleToCA"],
Cell[452002, 18975, 133, 4, 34, "ExampleText",
  CellTags->"TotalisticRuleToCA"],
Cell[452138, 18981, 100, 2, 28, "Input",
  CellTags->"TotalisticRuleToCA"],
Cell[452241, 18985, 141, 3, 26, "Input",
  CellTags->"TotalisticRuleToCA"],
Cell[452385, 18990, 74, 1, 32, "ObjectName",
  CellTags->"ToTotalisticRuleCA"],
Cell[452462, 18993, 63, 1, 50, "Highlight",
  CellTags->"ToTotalisticRuleCA"],
Cell[452528, 18996, 225, 8, 31, "Usage",
  CellTags->"ToTotalisticRuleCA"],
Cell[452756, 19006, 147, 5, 24, "Notes",
  CellTags->"ToTotalisticRuleCA"],
Cell[452906, 19013, 187, 7, 24, "Notes",
  CellTags->"ToTotalisticRuleCA"],
Cell[453096, 19022, 76, 1, 39, "ExampleSection",
  CellTags->"ToTotalisticRuleCA"],
Cell[453175, 19025, 80, 1, 34, "ExampleText",
  CellTags->"ToTotalisticRuleCA"],
Cell[453258, 19028, 108, 2, 28, "Input",
  CellTags->"ToTotalisticRuleCA"],
Cell[453369, 19032, 52, 1, 32, "ObjectName",
  CellTags->"WelchCA"],
Cell[453424, 19035, 52, 1, 50, "Highlight",
  CellTags->"WelchCA"],
Cell[453479, 19038, 183, 6, 31, "Usage",
  CellTags->"WelchCA"],
Cell[453665, 19046, 290, 10, 61, "Notes",
  CellTags->"WelchCA"],
Cell[453958, 19058, 142, 5, 24, "Notes",
  CellTags->"WelchCA"],
Cell[454103, 19065, 167, 6, 24, "Notes",
  CellTags->"WelchCA"],
Cell[454273, 19073, 125, 6, 24, "Notes",
  CellTags->"WelchCA"],
Cell[454401, 19081, 65, 1, 39, "ExampleSection",
  CellTags->"WelchCA"],
Cell[454469, 19084, 129, 4, 34, "ExampleText",
  CellTags->"WelchCA"],
Cell[454601, 19090, 179, 4, 42, "Input",
  CellTags->"WelchCA"],
Cell[454783, 19096, 101, 2, 28, "Input",
  CellTags->"WelchCA"],
Cell[454887, 19100, 82, 1, 34, "ExampleText",
  CellTags->"WelchCA"],
Cell[454972, 19103, 116, 3, 26, "Input",
  CellTags->"WelchCA"],
Cell[455091, 19108, 102, 3, 26, "Input",
  CellTags->"WelchCA"],
Cell[455196, 19113, 48, 1, 32, "ObjectName",
  CellTags->"Where"],
Cell[455247, 19116, 48, 1, 50, "Highlight",
  CellTags->"Where"],
Cell[455298, 19119, 192, 6, 31, "Usage",
  CellTags->"Where"],
Cell[455493, 19127, 197, 9, 24, "Notes",
  CellTags->"Where"],
Cell[455693, 19138, 361, 15, 24, "Notes",
  CellTags->"Where"],
Cell[456057, 19155, 63, 1, 39, "ExampleSection",
  CellTags->"Where"],
Cell[456123, 19158, 123, 4, 34, "ExampleText",
  CellTags->"Where"],
Cell[456249, 19164, 90, 2, 28, "Input",
  CellTags->"Where"],
Cell[456342, 19168, 102, 2, 28, "Input",
  CellTags->"Where"],
Cell[456447, 19172, 103, 2, 28, "Input",
  CellTags->"Where"],
Cell[456553, 19176, 52, 1, 32, "ObjectName",
  CellTags->"WidthCA"],
Cell[456608, 19179, 52, 1, 50, "Highlight",
  CellTags->"WidthCA"],
Cell[456663, 19182, 162, 6, 31, "Usage",
  CellTags->"WidthCA"],
Cell[456828, 19190, 107, 5, 24, "Notes",
  CellTags->"WidthCA"],
Cell[456938, 19197, 234, 11, 24, "Notes",
  CellTags->"WidthCA"],
Cell[457175, 19210, 65, 1, 39, "ExampleSection",
  CellTags->"WidthCA"],
Cell[457243, 19213, 123, 4, 34, "ExampleText",
  CellTags->"WidthCA"],
Cell[457369, 19219, 91, 5, 45, "Input",
  CellTags->"WidthCA"],
Cell[457463, 19226, 52, 1, 32, "ObjectName",
  CellTags->"WordMap"],
Cell[457518, 19229, 50, 1, 50, "Highlight",
  CellTags->"WordMap"],
Cell[457571, 19232, 158, 4, 31, "Usage",
  CellTags->"WordMap"],
Cell[457732, 19238, 332, 14, 108, "Notes",
  CellTags->"WordMap"],
Cell[458067, 19254, 65, 1, 39, "ExampleSection",
  CellTags->"WordMap"],
Cell[458135, 19257, 161, 4, 50, "ExampleText",
  CellTags->"WordMap"],
Cell[458299, 19263, 124, 3, 26, "Input",
  CellTags->"WordMap"],
Cell[458426, 19268, 87, 3, 26, "Input",
  CellTags->"WordMap"],
Cell[458516, 19273, 52, 1, 32, "ObjectName",
  CellTags->"AddEdge"],
Cell[458571, 19276, 52, 1, 50, "Highlight",
  CellTags->"AddEdge"],
Cell[458626, 19279, 172, 8, 31, "Usage",
  CellTags->"AddEdge"],
Cell[458801, 19289, 232, 8, 40, "Notes",
  CellTags->"AddEdge"],
Cell[459036, 19299, 324, 15, 24, "Notes",
  CellTags->"AddEdge"],
Cell[459363, 19316, 65, 1, 39, "ExampleSection",
  CellTags->"AddEdge"],
Cell[459431, 19319, 92, 1, 34, "ExampleText",
  CellTags->"AddEdge"],
Cell[459526, 19322, 81, 2, 28, "Input",
  CellTags->"AddEdge"],
Cell[459610, 19326, 103, 3, 26, "Input",
  CellTags->"AddEdge"],
Cell[459716, 19331, 111, 3, 26, "Input",
  CellTags->"AddEdge"],
Cell[459830, 19336, 56, 1, 32, "ObjectName",
  CellTags->"AddVertex"],
Cell[459889, 19339, 54, 1, 50, "Highlight",
  CellTags->"AddVertex"],
Cell[459946, 19342, 152, 6, 31, "Usage",
  CellTags->"AddVertex"],
Cell[460101, 19350, 177, 7, 24, "Notes",
  CellTags->"AddVertex"],
Cell[460281, 19359, 286, 12, 24, "Notes",
  CellTags->"AddVertex"],
Cell[460570, 19373, 324, 15, 24, "Notes",
  CellTags->"AddVertex"],
Cell[460897, 19390, 67, 1, 39, "ExampleSection",
  CellTags->"AddVertex"],
Cell[460967, 19393, 98, 1, 34, "ExampleText",
  CellTags->"AddVertex"],
Cell[461068, 19396, 117, 3, 26, "Input",
  CellTags->"AddVertex"],
Cell[461188, 19401, 154, 4, 26, "Input",
  CellTags->"AddVertex"],
Cell[461345, 19407, 159, 4, 42, "Input",
  CellTags->"AddVertex"],
Cell[461507, 19413, 72, 1, 32, "ObjectName",
  CellTags->"BiAccessibleGraph"],
Cell[461582, 19416, 62, 1, 50, "Highlight",
  CellTags->"BiAccessibleGraph"],
Cell[461647, 19419, 291, 10, 47, "Usage",
  CellTags->"BiAccessibleGraph"],
Cell[461941, 19431, 127, 4, 24, "Notes",
  CellTags->"BiAccessibleGraph"],
Cell[462071, 19437, 185, 6, 24, "Notes",
  CellTags->"BiAccessibleGraph"],
Cell[462259, 19445, 480, 20, 24, "Notes",
  CellTags->"BiAccessibleGraph"],
Cell[462742, 19467, 75, 1, 39, "ExampleSection",
  CellTags->"BiAccessibleGraph"],
Cell[462820, 19470, 152, 4, 34, "ExampleText",
  CellTags->"BiAccessibleGraph"],
Cell[462975, 19476, 222, 6, 42, "Input",
  CellTags->"BiAccessibleGraph"],
Cell[463200, 19484, 104, 2, 28, "Input",
  CellTags->"BiAccessibleGraph"],
Cell[463307, 19488, 141, 4, 34, "ExampleText",
  CellTags->"BiAccessibleGraph"],
Cell[463451, 19494, 125, 5, 45, "Input",
  CellTags->"BiAccessibleGraph"],
Cell[463579, 19501, 68, 1, 32, "ObjectName",
  CellTags->"AccessibleGraph"],
Cell[463650, 19504, 60, 1, 50, "Highlight",
  CellTags->"AccessibleGraph"],
Cell[463713, 19507, 330, 11, 47, "Usage",
  CellTags->"AccessibleGraph"],
Cell[464046, 19520, 137, 4, 24, "Notes",
  CellTags->"AccessibleGraph"],
Cell[464186, 19526, 458, 19, 24, "Notes",
  CellTags->"AccessibleGraph"],
Cell[464647, 19547, 73, 1, 39, "ExampleSection",
  CellTags->"AccessibleGraph"],
Cell[464723, 19550, 73, 1, 34, "ExampleText",
  CellTags->"AccessibleGraph"],
Cell[464799, 19553, 103, 2, 28, "Input",
  CellTags->"AccessibleGraph"],
Cell[464905, 19557, 104, 1, 34, "ExampleText",
  CellTags->"AccessibleGraph"],
Cell[465012, 19560, 115, 5, 45, "Input",
  CellTags->"AccessibleGraph"],
Cell[465130, 19567, 92, 1, 34, "ExampleText",
  CellTags->"AccessibleGraph"],
Cell[465225, 19570, 111, 2, 28, "Input",
  CellTags->"AccessibleGraph"],
Cell[465339, 19574, 66, 1, 32, "ObjectName",
  CellTags->"CirculantGraph"],
Cell[465408, 19577, 59, 1, 50, "Highlight",
  CellTags->"CirculantGraph"],
Cell[465470, 19580, 246, 8, 47, "Usage",
  CellTags->"CirculantGraph"],
Cell[465719, 19590, 315, 10, 40, "Notes",
  CellTags->"CirculantGraph"],
Cell[466037, 19602, 156, 6, 24, "Notes",
  CellTags->"CirculantGraph"],
Cell[466196, 19610, 88, 1, 24, "Notes",
  CellTags->"CirculantGraph"],
Cell[466287, 19613, 251, 11, 24, "Notes",
  CellTags->"CirculantGraph"],
Cell[466541, 19626, 72, 1, 39, "ExampleSection",
  CellTags->"CirculantGraph"],
Cell[466616, 19629, 84, 1, 34, "ExampleText",
  CellTags->"CirculantGraph"],
Cell[466703, 19632, 101, 2, 28, "Input",
  CellTags->"CirculantGraph"],
Cell[466807, 19636, 119, 3, 26, "Input",
  CellTags->"CirculantGraph"],
Cell[466929, 19641, 95, 1, 34, "ExampleText",
  CellTags->"CirculantGraph"],
Cell[467027, 19644, 101, 2, 28, "Input",
  CellTags->"CirculantGraph"],
Cell[467131, 19648, 119, 3, 26, "Input",
  CellTags->"CirculantGraph"],
Cell[467253, 19653, 62, 1, 32, "ObjectName",
  CellTags->"CyclicQGraph"],
Cell[467318, 19656, 57, 1, 50, "Highlight",
  CellTags->"CyclicQGraph"],
Cell[467378, 19659, 160, 6, 31, "Usage",
  CellTags->"CyclicQGraph"],
Cell[467541, 19667, 60, 1, 24, "Notes",
  CellTags->"CyclicQGraph"],
Cell[467604, 19670, 155, 7, 24, "Notes",
  CellTags->"CyclicQGraph"],
Cell[467762, 19679, 70, 1, 39, "ExampleSection",
  CellTags->"CyclicQGraph"],
Cell[467835, 19682, 66, 1, 34, "ExampleText",
  CellTags->"CyclicQGraph"],
Cell[467904, 19685, 221, 6, 42, "Input",
  CellTags->"CyclicQGraph"],
Cell[468128, 19693, 102, 3, 26, "Input",
  CellTags->"CyclicQGraph"],
Cell[468233, 19698, 114, 3, 26, "Input",
  CellTags->"CyclicQGraph"],
Cell[468350, 19703, 103, 3, 26, "Input",
  CellTags->"CyclicQGraph"],
Cell[468456, 19708, 64, 1, 32, "ObjectName",
  CellTags->"DeBruijnEdges"],
Cell[468523, 19711, 58, 1, 50, "Highlight",
  CellTags->"DeBruijnEdges"],
Cell[468584, 19714, 180, 5, 31, "Usage",
  CellTags->"DeBruijnEdges"],
Cell[468767, 19721, 230, 9, 44, "Notes",
  CellTags->"DeBruijnEdges"],
Cell[469000, 19732, 155, 6, 24, "Notes",
  CellTags->"DeBruijnEdges"],
Cell[469158, 19740, 209, 7, 24, "Notes",
  CellTags->"DeBruijnEdges"],
Cell[469370, 19749, 172, 7, 24, "Notes",
  CellTags->"DeBruijnEdges"],
Cell[469545, 19758, 71, 1, 39, "ExampleSection",
  CellTags->"DeBruijnEdges"],
Cell[469619, 19761, 175, 6, 34, "ExampleText",
  CellTags->"DeBruijnEdges"],
Cell[469797, 19769, 112, 3, 26, "Input",
  CellTags->"DeBruijnEdges"],
Cell[469912, 19774, 84, 1, 34, "ExampleText",
  CellTags->"DeBruijnEdges"],
Cell[469999, 19777, 133, 3, 26, "Input",
  CellTags->"DeBruijnEdges"],
Cell[470135, 19782, 64, 1, 32, "ObjectName",
  CellTags->"DeBruijnGraph"],
Cell[470202, 19785, 58, 1, 50, "Highlight",
  CellTags->"DeBruijnGraph"],
Cell[470263, 19788, 161, 4, 31, "Usage",
  CellTags->"DeBruijnGraph"],
Cell[470427, 19794, 230, 9, 44, "Notes",
  CellTags->"DeBruijnGraph"],
Cell[470660, 19805, 155, 6, 24, "Notes",
  CellTags->"DeBruijnGraph"],
Cell[470818, 19813, 839, 30, 56, "Notes",
  CellTags->"DeBruijnGraph"],
Cell[471660, 19845, 172, 7, 24, "Notes",
  CellTags->"DeBruijnGraph"],
Cell[471835, 19854, 71, 1, 39, "ExampleSection",
  CellTags->"DeBruijnGraph"],
Cell[471909, 19857, 165, 6, 34, "ExampleText",
  CellTags->"DeBruijnGraph"],
Cell[472077, 19865, 141, 3, 26, "Input",
  CellTags->"DeBruijnGraph"],
Cell[472221, 19870, 100, 3, 26, "Input",
  CellTags->"DeBruijnGraph"],
Cell[472324, 19875, 150, 4, 34, "ExampleText",
  CellTags->"DeBruijnGraph"],
Cell[472477, 19881, 182, 4, 42, "Input",
  CellTags->"DeBruijnGraph"],
Cell[472662, 19887, 60, 1, 32, "ObjectName",
  CellTags->"DegreeGraph"],
Cell[472725, 19890, 56, 1, 50, "Highlight",
  CellTags->"DegreeGraph"],
Cell[472784, 19893, 199, 8, 31, "Usage",
  CellTags->"DegreeGraph"],
Cell[472986, 19903, 227, 9, 44, "Notes",
  CellTags->"DegreeGraph"],
Cell[473216, 19914, 182, 7, 24, "Notes",
  CellTags->"DegreeGraph"],
Cell[473401, 19923, 150, 5, 24, "Notes",
  CellTags->"DegreeGraph"],
Cell[473554, 19930, 154, 7, 24, "Notes",
  CellTags->"DegreeGraph"],
Cell[473711, 19939, 69, 1, 39, "ExampleSection",
  CellTags->"DegreeGraph"],
Cell[473783, 19942, 135, 4, 34, "ExampleText",
  CellTags->"DegreeGraph"],
Cell[473921, 19948, 97, 2, 28, "Input",
  CellTags->"DegreeGraph"],
Cell[474021, 19952, 103, 3, 26, "Input",
  CellTags->"DegreeGraph"],
Cell[474127, 19957, 124, 3, 26, "Input",
  CellTags->"DegreeGraph"],
Cell[474254, 19962, 121, 3, 26, "Input",
  CellTags->"DegreeGraph"],
Cell[474378, 19967, 58, 1, 32, "ObjectName",
  CellTags->"DeleteEdge"],
Cell[474439, 19970, 55, 1, 50, "Highlight",
  CellTags->"DeleteEdge"],
Cell[474497, 19973, 183, 8, 31, "Usage",
  CellTags->"DeleteEdge"],
Cell[474683, 19983, 153, 5, 24, "Notes",
  CellTags->"DeleteEdge"],
Cell[474839, 19990, 324, 15, 24, "Notes",
  CellTags->"DeleteEdge"],
Cell[475166, 20007, 68, 1, 39, "ExampleSection",
  CellTags->"DeleteEdge"],
Cell[475237, 20010, 110, 2, 34, "ExampleText",
  CellTags->"DeleteEdge"],
Cell[475350, 20014, 85, 2, 28, "Input",
  CellTags->"DeleteEdge"],
Cell[475438, 20018, 162, 4, 42, "Input",
  CellTags->"DeleteEdge"],
Cell[475603, 20024, 62, 1, 32, "ObjectName",
  CellTags->"DeleteVertex"],
Cell[475668, 20027, 57, 1, 50, "Highlight",
  CellTags->"DeleteVertex"],
Cell[475728, 20030, 211, 9, 31, "Usage",
  CellTags->"DeleteVertex"],
Cell[475942, 20041, 135, 5, 24, "Notes",
  CellTags->"DeleteVertex"],
Cell[476080, 20048, 324, 15, 24, "Notes",
  CellTags->"DeleteVertex"],
Cell[476407, 20065, 70, 1, 39, "ExampleSection",
  CellTags->"DeleteVertex"],
Cell[476480, 20068, 113, 2, 34, "ExampleText",
  CellTags->"DeleteVertex"],
Cell[476596, 20072, 87, 2, 28, "Input",
  CellTags->"DeleteVertex"],
Cell[476686, 20076, 114, 3, 26, "Input",
  CellTags->"DeleteVertex"],
Cell[476803, 20081, 168, 4, 42, "Input",
  CellTags->"DeleteVertex"],
Cell[476974, 20087, 48, 1, 32, "ObjectName",
  CellTags->"EdgeQ"],
Cell[477025, 20090, 50, 1, 50, "Highlight",
  CellTags->"EdgeQ"],
Cell[477078, 20093, 208, 8, 31, "Usage",
  CellTags->"EdgeQ"],
Cell[477289, 20103, 154, 7, 24, "Notes",
  CellTags->"EdgeQ"],
Cell[477446, 20112, 128, 5, 24, "Notes",
  CellTags->"EdgeQ"],
Cell[477577, 20119, 148, 7, 24, "Notes",
  CellTags->"EdgeQ"],
Cell[477728, 20128, 48, 1, 32, "ObjectName",
  CellTags->"Edges"],
Cell[477779, 20131, 50, 1, 50, "Highlight",
  CellTags->"Edges"],
Cell[477832, 20134, 116, 4, 31, "Usage",
  CellTags->"Edges"],
Cell[477951, 20140, 166, 4, 40, "Notes",
  CellTags->"Edges"],
Cell[478120, 20146, 255, 11, 24, "Notes",
  CellTags->"Edges"],
Cell[478378, 20159, 63, 1, 39, "ExampleSection",
  CellTags->"Edges"],
Cell[478444, 20162, 75, 1, 34, "ExampleText",
  CellTags->"Edges"],
Cell[478522, 20165, 91, 2, 28, "Input",
  CellTags->"Edges"],
Cell[478616, 20169, 88, 3, 26, "Input",
  CellTags->"Edges"],
Cell[478707, 20174, 48, 1, 32, "ObjectName",
  CellTags->"Graph"],
Cell[478758, 20177, 53, 1, 50, "Highlight",
  CellTags->"Graph"],
Cell[478814, 20180, 297, 10, 47, "Usage",
  CellTags->"Graph"],
Cell[479114, 20192, 188, 5, 40, "Notes",
  CellTags->"Graph"],
Cell[479305, 20199, 90, 1, 24, "Notes",
  CellTags->"Graph"],
Cell[479398, 20202, 370, 12, 40, "Notes",
  CellTags->"Graph"],
Cell[479771, 20216, 602, 23, 40, "Notes",
  CellTags->"Graph"],
Cell[480376, 20241, 63, 1, 39, "ExampleSection",
  CellTags->"Graph"],
Cell[480442, 20244, 96, 1, 34, "ExampleText",
  CellTags->"Graph"],
Cell[480541, 20247, 167, 5, 26, "Input",
  CellTags->"Graph"],
Cell[480711, 20254, 119, 4, 42, "Input",
  CellTags->"Graph"],
Cell[480833, 20260, 123, 4, 34, "ExampleText",
  CellTags->"Graph"],
Cell[480959, 20266, 100, 3, 26, "Input",
  CellTags->"Graph"],
Cell[481062, 20271, 98, 3, 26, "Input",
  CellTags->"Graph"],
Cell[481163, 20276, 70, 1, 34, "ExampleText",
  CellTags->"Graph"],
Cell[481236, 20279, 111, 3, 26, "Input",
  CellTags->"Graph"],
Cell[481350, 20284, 110, 3, 26, "Input",
  CellTags->"Graph"],
Cell[481463, 20289, 56, 1, 32, "ObjectName",
  CellTags->"GridGraph"],
Cell[481522, 20292, 57, 1, 50, "Highlight",
  CellTags->"GridGraph"],
Cell[481582, 20295, 206, 8, 31, "Usage",
  CellTags->"GridGraph"],
Cell[481791, 20305, 224, 9, 44, "Notes",
  CellTags->"GridGraph"],
Cell[482018, 20316, 156, 5, 24, "Notes",
  CellTags->"GridGraph"],
Cell[482177, 20323, 150, 5, 24, "Notes",
  CellTags->"GridGraph"],
Cell[482330, 20330, 188, 6, 24, "Notes",
  CellTags->"GridGraph"],
Cell[482521, 20338, 256, 11, 24, "Notes",
  CellTags->"GridGraph"],
Cell[482780, 20351, 67, 1, 39, "ExampleSection",
  CellTags->"GridGraph"],
Cell[482850, 20354, 104, 1, 34, "ExampleText",
  CellTags->"GridGraph"],
Cell[482957, 20357, 104, 3, 26, "Input",
  CellTags->"GridGraph"],
Cell[483064, 20362, 105, 3, 26, "Input",
  CellTags->"GridGraph"],
Cell[483172, 20367, 127, 3, 26, "Input",
  CellTags->"GridGraph"],
Cell[483302, 20372, 108, 2, 34, "ExampleText",
  CellTags->"GridGraph"],
Cell[483413, 20376, 169, 4, 26, "Input",
  CellTags->"GridGraph"],
Cell[483585, 20382, 64, 1, 32, "ObjectName",
  CellTags->"LabelEdgeList"],
Cell[483652, 20385, 58, 1, 50, "Highlight",
  CellTags->"LabelEdgeList"],
Cell[483713, 20388, 181, 6, 31, "Usage",
  CellTags->"LabelEdgeList"],
Cell[483897, 20396, 96, 1, 24, "Notes",
  CellTags->"LabelEdgeList"],
Cell[483996, 20399, 245, 11, 24, "Notes",
  CellTags->"LabelEdgeList"],
Cell[484244, 20412, 71, 1, 39, "ExampleSection",
  CellTags->"LabelEdgeList"],
Cell[484318, 20415, 89, 1, 34, "ExampleText",
  CellTags->"LabelEdgeList"],
Cell[484410, 20418, 88, 2, 28, "Input",
  CellTags->"LabelEdgeList"],
Cell[484501, 20422, 117, 3, 26, "Input",
  CellTags->"LabelEdgeList"],
Cell[484621, 20427, 60, 1, 32, "ObjectName",
  CellTags->"LongestPath"],
Cell[484684, 20430, 56, 1, 50, "Highlight",
  CellTags->"LongestPath"],
Cell[484743, 20433, 189, 6, 31, "Usage",
  CellTags->"LongestPath"],
Cell[484935, 20441, 227, 9, 44, "Notes",
  CellTags->"LongestPath"],
Cell[485165, 20452, 166, 5, 24, "Notes",
  CellTags->"LongestPath"],
Cell[485334, 20459, 151, 4, 24, "Notes",
  CellTags->"LongestPath"],
Cell[485488, 20465, 204, 5, 40, "Notes",
  CellTags->"LongestPath"],
Cell[485695, 20472, 320, 11, 24, "Notes",
  CellTags->"LongestPath"],
Cell[486018, 20485, 69, 1, 39, "ExampleSection",
  CellTags->"LongestPath"],
Cell[486090, 20488, 72, 1, 34, "ExampleText",
  CellTags->"LongestPath"],
Cell[486165, 20491, 208, 6, 42, "Input",
  CellTags->"LongestPath"],
Cell[486376, 20499, 104, 3, 26, "Input",
  CellTags->"LongestPath"],
Cell[486483, 20504, 86, 1, 34, "ExampleText",
  CellTags->"LongestPath"],
Cell[486572, 20507, 121, 3, 26, "Input",
  CellTags->"LongestPath"],
Cell[486696, 20512, 66, 1, 32, "ObjectName",
  CellTags->"NormalizeGraph"],
Cell[486765, 20515, 59, 1, 50, "Highlight",
  CellTags->"NormalizeGraph"],
Cell[486827, 20518, 174, 6, 31, "Usage",
  CellTags->"NormalizeGraph"],
Cell[487004, 20526, 201, 5, 40, "Notes",
  CellTags->"NormalizeGraph"],
Cell[487208, 20533, 130, 4, 24, "Notes",
  CellTags->"NormalizeGraph"],
Cell[487341, 20539, 355, 15, 24, "Notes",
  CellTags->"NormalizeGraph"],
Cell[487699, 20556, 72, 1, 39, "ExampleSection",
  CellTags->"NormalizeGraph"],
Cell[487774, 20559, 179, 5, 34, "ExampleText",
  CellTags->"NormalizeGraph"],
Cell[487956, 20566, 216, 6, 42, "Input",
  CellTags->"NormalizeGraph"],
Cell[488175, 20574, 147, 4, 42, "Input",
  CellTags->"NormalizeGraph"],
Cell[488325, 20580, 72, 1, 32, "ObjectName",
  CellTags->"NonTransientGraph"],
Cell[488400, 20583, 62, 1, 50, "Highlight",
  CellTags->"NonTransientGraph"],
Cell[488465, 20586, 190, 6, 31, "Usage",
  CellTags->"NonTransientGraph"],
Cell[488658, 20594, 234, 9, 44, "Notes",
  CellTags->"NonTransientGraph"],
Cell[488895, 20605, 261, 6, 40, "Notes",
  CellTags->"NonTransientGraph"],
Cell[489159, 20613, 143, 2, 24, "Notes",
  CellTags->"NonTransientGraph"],
Cell[489305, 20617, 416, 15, 24, "Notes",
  CellTags->"NonTransientGraph"],
Cell[489724, 20634, 75, 1, 39, "ExampleSection",
  CellTags->"NonTransientGraph"],
Cell[489802, 20637, 94, 1, 34, "ExampleText",
  CellTags->"NonTransientGraph"],
Cell[489899, 20640, 247, 6, 42, "Input",
  CellTags->"NonTransientGraph"],
Cell[490149, 20648, 143, 4, 42, "Input",
  CellTags->"NonTransientGraph"],
Cell[490295, 20654, 58, 1, 32, "ObjectName",
  CellTags->"NonTrivial"],
Cell[490356, 20657, 53, 1, 50, "Highlight",
  CellTags->"NonTrivial"],
Cell[490412, 20660, 175, 6, 31, "Usage",
  CellTags->"NonTrivial"],
Cell[490590, 20668, 170, 4, 40, "Notes",
  CellTags->"NonTrivial"],
Cell[490763, 20674, 192, 7, 24, "Notes",
  CellTags->"NonTrivial"],
Cell[490958, 20683, 54, 1, 32, "ObjectName",
  CellTags->"NumEdges"],
Cell[491015, 20686, 53, 1, 50, "Highlight",
  CellTags->"NumEdges"],
Cell[491071, 20689, 161, 6, 31, "Usage",
  CellTags->"NumEdges"],
Cell[491235, 20697, 338, 15, 24, "Notes",
  CellTags->"NumEdges"],
Cell[491576, 20714, 66, 1, 39, "ExampleSection",
  CellTags->"NumEdges"],
Cell[491645, 20717, 77, 1, 34, "ExampleText",
  CellTags->"NumEdges"],
Cell[491725, 20720, 128, 3, 26, "Input",
  CellTags->"NumEdges"],
Cell[491856, 20725, 112, 3, 26, "Input",
  CellTags->"NumEdges"],
Cell[491971, 20730, 60, 1, 32, "ObjectName",
  CellTags->"NumVertices"],
Cell[492034, 20733, 56, 1, 50, "Highlight",
  CellTags->"NumVertices"],
Cell[492093, 20736, 170, 6, 31, "Usage",
  CellTags->"NumVertices"],
Cell[492266, 20744, 335, 15, 24, "Notes",
  CellTags->"NumVertices"],
Cell[492604, 20761, 69, 1, 39, "ExampleSection",
  CellTags->"NumVertices"],
Cell[492676, 20764, 83, 1, 34, "ExampleText",
  CellTags->"NumVertices"],
Cell[492762, 20767, 134, 3, 26, "Input",
  CellTags->"NumVertices"],
Cell[492899, 20772, 118, 3, 26, "Input",
  CellTags->"NumVertices"],
Cell[493020, 20777, 56, 1, 32, "ObjectName",
  CellTags->"PathGraph"],
Cell[493079, 20780, 54, 1, 50, "Highlight",
  CellTags->"PathGraph"],
Cell[493136, 20783, 170, 6, 31, "Usage",
  CellTags->"PathGraph"],
Cell[493309, 20791, 225, 9, 44, "Notes",
  CellTags->"PathGraph"],
Cell[493537, 20802, 141, 5, 24, "Notes",
  CellTags->"PathGraph"],
Cell[493681, 20809, 256, 11, 24, "Notes",
  CellTags->"PathGraph"],
Cell[493940, 20822, 67, 1, 39, "ExampleSection",
  CellTags->"PathGraph"],
Cell[494010, 20825, 74, 1, 34, "ExampleText",
  CellTags->"PathGraph"],
Cell[494087, 20828, 101, 3, 26, "Input",
  CellTags->"PathGraph"],
Cell[494191, 20833, 115, 3, 26, "Input",
  CellTags->"PathGraph"],
Cell[494309, 20838, 84, 1, 34, "ExampleText",
  CellTags->"PathGraph"],
Cell[494396, 20841, 120, 3, 26, "Input",
  CellTags->"PathGraph"],
Cell[494519, 20846, 139, 3, 26, "Input",
  CellTags->"PathGraph"],
Cell[494661, 20851, 62, 1, 32, "ObjectName",
  CellTags->"PermuteGraph"],
Cell[494726, 20854, 57, 1, 50, "Highlight",
  CellTags->"PermuteGraph"],
Cell[494786, 20857, 186, 8, 31, "Usage",
  CellTags->"PermuteGraph"],
Cell[494975, 20867, 125, 5, 24, "Notes",
  CellTags->"PermuteGraph"],
Cell[495103, 20874, 155, 7, 24, "Notes",
  CellTags->"PermuteGraph"],
Cell[495261, 20883, 70, 1, 39, "ExampleSection",
  CellTags->"PermuteGraph"],
Cell[495334, 20886, 126, 4, 34, "ExampleText",
  CellTags->"PermuteGraph"],
Cell[495463, 20892, 87, 2, 28, "Input",
  CellTags->"PermuteGraph"],
Cell[495553, 20896, 131, 3, 26, "Input",
  CellTags->"PermuteGraph"],
Cell[495687, 20901, 62, 1, 32, "ObjectName",
  CellTags->"ProductGraph"],
Cell[495752, 20904, 57, 1, 50, "Highlight",
  CellTags->"ProductGraph"],
Cell[495812, 20907, 216, 8, 31, "Usage",
  CellTags->"ProductGraph"],
Cell[496031, 20917, 273, 10, 61, "Notes",
  CellTags->"ProductGraph"],
Cell[496307, 20929, 604, 22, 40, "Notes",
  CellTags->"ProductGraph"],
Cell[496914, 20953, 320, 11, 24, "Notes",
  CellTags->"ProductGraph"],
Cell[497237, 20966, 305, 11, 24, "Notes",
  CellTags->"ProductGraph"],
Cell[497545, 20979, 70, 1, 39, "ExampleSection",
  CellTags->"ProductGraph"],
Cell[497618, 20982, 98, 1, 34, "ExampleText",
  CellTags->"ProductGraph"],
Cell[497719, 20985, 161, 4, 42, "Input",
  CellTags->"ProductGraph"],
Cell[497883, 20991, 112, 3, 26, "Input",
  CellTags->"ProductGraph"],
Cell[497998, 20996, 107, 1, 34, "ExampleText",
  CellTags->"ProductGraph"],
Cell[498108, 20999, 100, 3, 26, "Input",
  CellTags->"ProductGraph"],
Cell[498211, 21004, 91, 1, 34, "ExampleText",
  CellTags->"ProductGraph"],
Cell[498305, 21007, 119, 3, 26, "Input",
  CellTags->"ProductGraph"],
Cell[498427, 21012, 119, 3, 26, "Input",
  CellTags->"ProductGraph"],
Cell[498549, 21017, 118, 3, 26, "Input",
  CellTags->"ProductGraph"],
Cell[498670, 21022, 76, 1, 32, "ObjectName",
  CellTags->"ProductLabeledGraph"],
Cell[498749, 21025, 64, 1, 50, "Highlight",
  CellTags->"ProductLabeledGraph"],
Cell[498816, 21028, 227, 8, 31, "Usage",
  CellTags->"ProductLabeledGraph"],
Cell[499046, 21038, 495, 16, 40, "Notes",
  CellTags->"ProductLabeledGraph"],
Cell[499544, 21056, 298, 11, 24, "Notes",
  CellTags->"ProductLabeledGraph"],
Cell[499845, 21069, 77, 1, 39, "ExampleSection",
  CellTags->"ProductLabeledGraph"],
Cell[499925, 21072, 210, 5, 50, "ExampleText",
  CellTags->"ProductLabeledGraph"],
Cell[500138, 21079, 235, 5, 42, "Input",
  CellTags->"ProductLabeledGraph"],
Cell[500376, 21086, 126, 3, 26, "Input",
  CellTags->"ProductLabeledGraph"],
Cell[500505, 21091, 124, 3, 26, "Input",
  CellTags->"ProductLabeledGraph"],
Cell[500632, 21096, 119, 3, 26, "Input",
  CellTags->"ProductLabeledGraph"],
Cell[500754, 21101, 62, 1, 32, "ObjectName",
  CellTags->"ReverseGraph"],
Cell[500819, 21104, 57, 1, 50, "Highlight",
  CellTags->"ReverseGraph"],
Cell[500879, 21107, 160, 6, 31, "Usage",
  CellTags->"ReverseGraph"],
Cell[501042, 21115, 126, 4, 24, "Notes",
  CellTags->"ReverseGraph"],
Cell[501171, 21121, 255, 11, 24, "Notes",
  CellTags->"ReverseGraph"],
Cell[501429, 21134, 70, 1, 39, "ExampleSection",
  CellTags->"ReverseGraph"],
Cell[501502, 21137, 81, 1, 34, "ExampleText",
  CellTags->"ReverseGraph"],
Cell[501586, 21140, 87, 2, 28, "Input",
  CellTags->"ReverseGraph"],
Cell[501676, 21144, 102, 3, 26, "Input",
  CellTags->"ReverseGraph"],
Cell[501781, 21149, 137, 3, 26, "Input",
  CellTags->"ReverseGraph"],
Cell[501921, 21154, 60, 1, 32, "ObjectName",
  CellTags->"SetVertices"],
Cell[501984, 21157, 56, 1, 50, "Highlight",
  CellTags->"SetVertices"],
Cell[502043, 21160, 215, 8, 31, "Usage",
  CellTags->"SetVertices"],
Cell[502261, 21170, 128, 4, 24, "Notes",
  CellTags->"SetVertices"],
Cell[502392, 21176, 135, 6, 24, "Notes",
  CellTags->"SetVertices"],
Cell[502530, 21184, 69, 1, 39, "ExampleSection",
  CellTags->"SetVertices"],
Cell[502602, 21187, 133, 4, 34, "ExampleText",
  CellTags->"SetVertices"],
Cell[502738, 21193, 133, 3, 26, "Input",
  CellTags->"SetVertices"],
Cell[502874, 21198, 97, 3, 26, "Input",
  CellTags->"SetVertices"],
Cell[502974, 21203, 146, 4, 34, "ExampleText",
  CellTags->"SetVertices"],
Cell[503123, 21209, 165, 4, 42, "Input",
  CellTags->"SetVertices"],
Cell[503291, 21215, 62, 1, 32, "ObjectName",
  CellTags->"ShortestPath"],
Cell[503356, 21218, 57, 1, 50, "Highlight",
  CellTags->"ShortestPath"],
Cell[503416, 21221, 278, 10, 47, "Usage",
  CellTags->"ShortestPath"],
Cell[503697, 21233, 228, 9, 44, "Notes",
  CellTags->"ShortestPath"],
Cell[503928, 21244, 137, 5, 24, "Notes",
  CellTags->"ShortestPath"],
Cell[504068, 21251, 250, 11, 24, "Notes",
  CellTags->"ShortestPath"],
Cell[504321, 21264, 379, 15, 24, "Notes",
  CellTags->"ShortestPath"],
Cell[504703, 21281, 70, 1, 39, "ExampleSection",
  CellTags->"ShortestPath"],
Cell[504776, 21284, 127, 4, 34, "ExampleText",
  CellTags->"ShortestPath"],
Cell[504906, 21290, 122, 3, 26, "Input",
  CellTags->"ShortestPath"],
Cell[505031, 21295, 105, 3, 26, "Input",
  CellTags->"ShortestPath"],
Cell[505139, 21300, 123, 3, 26, "Input",
  CellTags->"ShortestPath"],
Cell[505265, 21305, 92, 1, 32, "ObjectName",
  CellTags->"StronglyConnectedComponents"],
Cell[505360, 21308, 72, 1, 50, "Highlight",
  CellTags->"StronglyConnectedComponents"],
Cell[505435, 21311, 212, 5, 47, "Usage",
  CellTags->"StronglyConnectedComponents"],
Cell[505650, 21318, 316, 10, 61, "Notes",
  CellTags->"StronglyConnectedComponents"],
Cell[505969, 21330, 210, 6, 40, "Notes",
  CellTags->"StronglyConnectedComponents"],
Cell[506182, 21338, 96, 1, 24, "Notes",
  CellTags->"StronglyConnectedComponents"],
Cell[506281, 21341, 504, 19, 24, "Notes",
  CellTags->"StronglyConnectedComponents"],
Cell[506788, 21362, 85, 1, 39, "ExampleSection",
  CellTags->"StronglyConnectedComponents"],
Cell[506876, 21365, 117, 1, 34, "ExampleText",
  CellTags->"StronglyConnectedComponents"],
Cell[506996, 21368, 255, 6, 42, "Input",
  CellTags->"StronglyConnectedComponents"],
Cell[507254, 21376, 130, 3, 26, "Input",
  CellTags->"StronglyConnectedComponents"],
Cell[507387, 21381, 111, 1, 34, "ExampleText",
  CellTags->"StronglyConnectedComponents"],
Cell[507501, 21384, 156, 3, 26, "Input",
  CellTags->"StronglyConnectedComponents"],
Cell[507660, 21389, 54, 1, 32, "ObjectName",
  CellTags->"SubGraph"],
Cell[507717, 21392, 53, 1, 50, "Highlight",
  CellTags->"SubGraph"],
Cell[507773, 21395, 204, 8, 31, "Usage",
  CellTags->"SubGraph"],
Cell[507980, 21405, 225, 9, 44, "Notes",
  CellTags->"SubGraph"],
Cell[508208, 21416, 118, 4, 24, "Notes",
  CellTags->"SubGraph"],
Cell[508329, 21422, 251, 11, 24, "Notes",
  CellTags->"SubGraph"],
Cell[508583, 21435, 66, 1, 39, "ExampleSection",
  CellTags->"SubGraph"],
Cell[508652, 21438, 122, 4, 34, "ExampleText",
  CellTags->"SubGraph"],
Cell[508777, 21444, 83, 2, 28, "Input",
  CellTags->"SubGraph"],
Cell[508863, 21448, 123, 3, 26, "Input",
  CellTags->"SubGraph"],
Cell[508989, 21453, 96, 3, 26, "Input",
  CellTags->"SubGraph"],
Cell[509088, 21458, 166, 4, 42, "Input",
  CellTags->"SubGraph"],
Cell[509257, 21464, 68, 1, 32, "ObjectName",
  CellTags->"ToAdjacencyList"],
Cell[509328, 21467, 60, 1, 50, "Highlight",
  CellTags->"ToAdjacencyList"],
Cell[509391, 21470, 178, 6, 31, "Usage",
  CellTags->"ToAdjacencyList"],
Cell[509572, 21478, 129, 4, 24, "Notes",
  CellTags->"ToAdjacencyList"],
Cell[509704, 21484, 157, 4, 24, "Notes",
  CellTags->"ToAdjacencyList"],
Cell[509864, 21490, 364, 15, 24, "Notes",
  CellTags->"ToAdjacencyList"],
Cell[510231, 21507, 73, 1, 39, "ExampleSection",
  CellTags->"ToAdjacencyList"],
Cell[510307, 21510, 99, 1, 34, "ExampleText",
  CellTags->"ToAdjacencyList"],
Cell[510409, 21513, 96, 2, 28, "Input",
  CellTags->"ToAdjacencyList"],
Cell[510508, 21517, 143, 3, 26, "Input",
  CellTags->"ToAdjacencyList"],
Cell[510654, 21522, 108, 3, 26, "Input",
  CellTags->"ToAdjacencyList"],
Cell[510765, 21527, 72, 1, 32, "ObjectName",
  CellTags->"ToAdjacencyMatrix"],
Cell[510840, 21530, 62, 1, 50, "Highlight",
  CellTags->"ToAdjacencyMatrix"],
Cell[510905, 21533, 185, 6, 31, "Usage",
  CellTags->"ToAdjacencyMatrix"],
Cell[511093, 21541, 147, 4, 24, "Notes",
  CellTags->"ToAdjacencyMatrix"],
Cell[511243, 21547, 362, 15, 24, "Notes",
  CellTags->"ToAdjacencyMatrix"],
Cell[511608, 21564, 75, 1, 39, "ExampleSection",
  CellTags->"ToAdjacencyMatrix"],
Cell[511686, 21567, 102, 1, 34, "ExampleText",
  CellTags->"ToAdjacencyMatrix"],
Cell[511791, 21570, 98, 2, 28, "Input",
  CellTags->"ToAdjacencyMatrix"],
Cell[511892, 21574, 147, 3, 26, "Input",
  CellTags->"ToAdjacencyMatrix"],
Cell[512042, 21579, 82, 1, 32, "ObjectName",
  CellTags->"ToDirectedAcyclicGraph"],
Cell[512127, 21582, 67, 1, 50, "Highlight",
  CellTags->"ToDirectedAcyclicGraph"],
Cell[512197, 21585, 304, 7, 47, "Usage",
  CellTags->"ToDirectedAcyclicGraph"],
Cell[512504, 21594, 233, 8, 44, "Notes",
  CellTags->"ToDirectedAcyclicGraph"],
Cell[512740, 21604, 178, 4, 40, "Notes",
  CellTags->"ToDirectedAcyclicGraph"],
Cell[512921, 21610, 503, 19, 24, "Notes",
  CellTags->"ToDirectedAcyclicGraph"],
Cell[513427, 21631, 80, 1, 39, "ExampleSection",
  CellTags->"ToDirectedAcyclicGraph"],
Cell[513510, 21634, 112, 1, 34, "ExampleText",
  CellTags->"ToDirectedAcyclicGraph"],
Cell[513625, 21637, 252, 6, 42, "Input",
  CellTags->"ToDirectedAcyclicGraph"],
Cell[513880, 21645, 88, 1, 34, "ExampleText",
  CellTags->"ToDirectedAcyclicGraph"],
Cell[513971, 21648, 153, 4, 42, "Input",
  CellTags->"ToDirectedAcyclicGraph"],
Cell[514127, 21654, 58, 1, 32, "ObjectName",
  CellTags->"ToEdgeList"],
Cell[514188, 21657, 55, 1, 50, "Highlight",
  CellTags->"ToEdgeList"],
Cell[514246, 21660, 163, 6, 31, "Usage",
  CellTags->"ToEdgeList"],
Cell[514412, 21668, 369, 15, 24, "Notes",
  CellTags->"ToEdgeList"],
Cell[514784, 21685, 68, 1, 39, "ExampleSection",
  CellTags->"ToEdgeList"],
Cell[514855, 21688, 105, 1, 34, "ExampleText",
  CellTags->"ToEdgeList"],
Cell[514963, 21691, 384, 8, 74, "Input",
  CellTags->"ToEdgeList"],
Cell[515350, 21701, 111, 3, 26, "Input",
  CellTags->"ToEdgeList"],
Cell[515464, 21706, 111, 3, 26, "Input",
  CellTags->"ToEdgeList"],
Cell[515578, 21711, 56, 1, 32, "ObjectName",
  CellTags->"ToTDGraph"],
Cell[515637, 21714, 54, 1, 50, "Highlight",
  CellTags->"ToTDGraph"],
Cell[515694, 21717, 235, 8, 47, "Usage",
  CellTags->"ToTDGraph"],
Cell[515932, 21727, 226, 9, 44, "Notes",
  CellTags->"ToTDGraph"],
Cell[516161, 21738, 77, 1, 24, "Notes",
  CellTags->"ToTDGraph"],
Cell[516241, 21741, 177, 7, 24, "Notes",
  CellTags->"ToTDGraph"],
Cell[516421, 21750, 67, 1, 39, "ExampleSection",
  CellTags->"ToTDGraph"],
Cell[516491, 21753, 187, 6, 34, "ExampleText",
  CellTags->"ToTDGraph"],
Cell[516681, 21761, 349, 8, 26, "Input",
  CellTags->"ToTDGraph"],
Cell[517033, 21771, 115, 3, 26, "Input",
  CellTags->"ToTDGraph"],
Cell[517151, 21776, 89, 2, 28, "Input",
  CellTags->"ToTDGraph"],
Cell[517243, 21780, 106, 3, 26, "Input",
  CellTags->"ToTDGraph"],
Cell[517352, 21785, 56, 1, 32, "ObjectName",
  CellTags->"TypeGraph"],
Cell[517411, 21788, 54, 1, 50, "Highlight",
  CellTags->"TypeGraph"],
Cell[517468, 21791, 152, 6, 31, "Usage",
  CellTags->"TypeGraph"],
Cell[517623, 21799, 192, 5, 40, "Notes",
  CellTags->"TypeGraph"],
Cell[517818, 21806, 94, 1, 24, "Notes",
  CellTags->"TypeGraph"],
Cell[517915, 21809, 152, 7, 24, "Notes",
  CellTags->"TypeGraph"],
Cell[518070, 21818, 67, 1, 39, "ExampleSection",
  CellTags->"TypeGraph"],
Cell[518140, 21821, 100, 1, 34, "ExampleText",
  CellTags->"TypeGraph"],
Cell[518243, 21824, 170, 5, 26, "Input",
  CellTags->"TypeGraph"],
Cell[518416, 21831, 95, 3, 26, "Input",
  CellTags->"TypeGraph"],
Cell[518514, 21836, 129, 3, 26, "Input",
  CellTags->"TypeGraph"],
Cell[518646, 21841, 131, 3, 26, "Input",
  CellTags->"TypeGraph"],
Cell[518780, 21846, 56, 1, 32, "ObjectName",
  CellTags->"VertexSet"],
Cell[518839, 21849, 54, 1, 50, "Highlight",
  CellTags->"VertexSet"],
Cell[518896, 21852, 158, 6, 31, "Usage",
  CellTags->"VertexSet"],
Cell[519057, 21860, 185, 6, 24, "Notes",
  CellTags->"VertexSet"],
Cell[519245, 21868, 238, 11, 24, "Notes",
  CellTags->"VertexSet"],
Cell[519486, 21881, 67, 1, 39, "ExampleSection",
  CellTags->"VertexSet"],
Cell[519556, 21884, 101, 1, 34, "ExampleText",
  CellTags->"VertexSet"],
Cell[519660, 21887, 133, 3, 26, "Input",
  CellTags->"VertexSet"],
Cell[519796, 21892, 96, 3, 26, "Input",
  CellTags->"VertexSet"],
Cell[519895, 21897, 88, 1, 32, "ObjectName",
  CellTags->"WeaklyConnectedComponents"],
Cell[519986, 21900, 70, 1, 50, "Highlight",
  CellTags->"WeaklyConnectedComponents"],
Cell[520059, 21903, 252, 8, 47, "Usage",
  CellTags->"WeaklyConnectedComponents"],
Cell[520314, 21913, 161, 4, 24, "Notes",
  CellTags->"WeaklyConnectedComponents"],
Cell[520478, 21919, 298, 11, 24, "Notes",
  CellTags->"WeaklyConnectedComponents"],
Cell[520779, 21932, 83, 1, 39, "ExampleSection",
  CellTags->"WeaklyConnectedComponents"],
Cell[520865, 21935, 95, 1, 34, "ExampleText",
  CellTags->"WeaklyConnectedComponents"],
Cell[520963, 21938, 116, 2, 28, "Input",
  CellTags->"WeaklyConnectedComponents"],
Cell[521082, 21942, 234, 5, 58, "Input",
  CellTags->"WeaklyConnectedComponents"],
Cell[521319, 21949, 70, 1, 32, "ObjectName",
  CellTags->"$generateSGBound"],
Cell[521392, 21952, 64, 1, 50, "Highlight",
  CellTags->"$generateSGBound"],
Cell[521459, 21955, 206, 6, 31, "Usage",
  CellTags->"$generateSGBound"],
Cell[521668, 21963, 65, 1, 24, "Notes",
  CellTags->"$generateSGBound"],
Cell[521736, 21966, 164, 7, 24, "Notes",
  CellTags->"$generateSGBound"],
Cell[521903, 21975, 74, 1, 39, "ExampleSection",
  CellTags->"$generateSGBound"],
Cell[521980, 21978, 88, 1, 34, "ExampleText",
  CellTags->"$generateSGBound"],
Cell[522071, 21981, 107, 3, 26, "Input",
  CellTags->"$generateSGBound"],
Cell[522181, 21986, 68, 1, 32, "ObjectName",
  CellTags->"$identitystring"],
Cell[522252, 21989, 63, 1, 50, "Highlight",
  CellTags->"$identitystring"],
Cell[522318, 21992, 215, 7, 47, "Usage",
  CellTags->"$identitystring"],
Cell[522536, 22001, 122, 4, 24, "Notes",
  CellTags->"$identitystring"],
Cell[522661, 22007, 271, 11, 24, "Notes",
  CellTags->"$identitystring"],
Cell[522935, 22020, 73, 1, 39, "ExampleSection",
  CellTags->"$identitystring"],
Cell[523011, 22023, 88, 1, 34, "ExampleText",
  CellTags->"$identitystring"],
Cell[523102, 22026, 119, 3, 26, "Input",
  CellTags->"$identitystring"],
Cell[523224, 22031, 60, 1, 32, "ObjectName",
  CellTags->"$nullstring"],
Cell[523287, 22034, 59, 1, 50, "Highlight",
  CellTags->"$nullstring"],
Cell[523349, 22037, 205, 6, 47, "Usage",
  CellTags->"$nullstring"],
Cell[523557, 22045, 117, 4, 24, "Notes",
  CellTags->"$nullstring"],
Cell[523677, 22051, 269, 11, 24, "Notes",
  CellTags->"$nullstring"],
Cell[523949, 22064, 69, 1, 39, "ExampleSection",
  CellTags->"$nullstring"],
Cell[524021, 22067, 84, 1, 34, "ExampleText",
  CellTags->"$nullstring"],
Cell[524108, 22070, 111, 3, 26, "Input",
  CellTags->"$nullstring"],
Cell[524222, 22075, 54, 1, 32, "ObjectName",
  CellTags->"AllTimes"],
Cell[524279, 22078, 53, 1, 50, "Highlight",
  CellTags->"AllTimes"],
Cell[524335, 22081, 261, 10, 47, "Usage",
  CellTags->"AllTimes"],
Cell[524599, 22093, 244, 8, 40, "Notes",
  CellTags->"AllTimes"],
Cell[524846, 22103, 78, 1, 24, "Notes",
  CellTags->"AllTimes"],
Cell[524927, 22106, 319, 15, 24, "Notes",
  CellTags->"AllTimes"],
Cell[525249, 22123, 66, 1, 39, "ExampleSection",
  CellTags->"AllTimes"],
Cell[525318, 22126, 107, 2, 34, "ExampleText",
  CellTags->"AllTimes"],
Cell[525428, 22130, 118, 2, 28, "Input",
  CellTags->"AllTimes"],
Cell[525549, 22134, 130, 4, 34, "ExampleText",
  CellTags->"AllTimes"],
Cell[525682, 22140, 117, 5, 45, "Input",
  CellTags->"AllTimes"],
Cell[525802, 22147, 74, 1, 32, "ObjectName",
  CellTags->"AlternatingGroupSG"],
Cell[525879, 22150, 230, 6, 47, "Usage",
  CellTags->"AlternatingGroupSG"],
Cell[526112, 22158, 258, 9, 24, "Notes",
  CellTags->"AlternatingGroupSG"],
Cell[526373, 22169, 543, 23, 24, "Notes",
  CellTags->"AlternatingGroupSG"],
Cell[526919, 22194, 76, 1, 39, "ExampleSection",
  CellTags->"AlternatingGroupSG"],
Cell[526998, 22197, 94, 1, 34, "ExampleText",
  CellTags->"AlternatingGroupSG"],
Cell[527095, 22200, 98, 2, 28, "Input",
  CellTags->"AlternatingGroupSG"],
Cell[527196, 22204, 182, 6, 34, "ExampleText",
  CellTags->"AlternatingGroupSG"],
Cell[527381, 22212, 108, 5, 45, "Input",
  CellTags->"AlternatingGroupSG"],
Cell[527492, 22219, 62, 1, 32, "ObjectName",
  CellTags->"AperiodicQSG"],
Cell[527557, 22222, 57, 1, 50, "Highlight",
  CellTags->"AperiodicQSG"],
Cell[527617, 22225, 173, 6, 31, "Usage",
  CellTags->"AperiodicQSG"],
Cell[527793, 22233, 149, 5, 24, "Notes",
  CellTags->"AperiodicQSG"],
Cell[527945, 22240, 359, 12, 40, "Notes",
  CellTags->"AperiodicQSG"],
Cell[528307, 22254, 421, 19, 24, "Notes",
  CellTags->"AperiodicQSG"],
Cell[528731, 22275, 70, 1, 39, "ExampleSection",
  CellTags->"AperiodicQSG"],
Cell[528804, 22278, 122, 4, 34, "ExampleText",
  CellTags->"AperiodicQSG"],
Cell[528929, 22284, 557, 11, 122, "Input",
  CellTags->"AperiodicQSG"],
Cell[529489, 22297, 102, 3, 26, "Input",
  CellTags->"AperiodicQSG"],
Cell[529594, 22302, 97, 1, 34, "ExampleText",
  CellTags->"AperiodicQSG"],
Cell[529694, 22305, 108, 2, 28, "Input",
  CellTags->"AperiodicQSG"],
Cell[529805, 22309, 60, 1, 32, "ObjectName",
  CellTags->"AperiodicSG"],
Cell[529868, 22312, 56, 1, 50, "Highlight",
  CellTags->"AperiodicSG"],
Cell[529927, 22315, 172, 6, 31, "Usage",
  CellTags->"AperiodicSG"],
Cell[530102, 22323, 231, 9, 24, "Notes",
  CellTags->"AperiodicSG"],
Cell[530336, 22334, 426, 19, 24, "Notes",
  CellTags->"AperiodicSG"],
Cell[530765, 22355, 69, 1, 39, "ExampleSection",
  CellTags->"AperiodicSG"],
Cell[530837, 22358, 140, 4, 34, "ExampleText",
  CellTags->"AperiodicSG"],
Cell[530980, 22364, 118, 3, 26, "Input",
  CellTags->"AperiodicSG"],
Cell[531101, 22369, 104, 5, 45, "Input",
  CellTags->"AperiodicSG"],
Cell[531208, 22376, 66, 1, 32, "ObjectName",
  CellTags->"BlocksToDClass"],
Cell[531277, 22379, 59, 1, 50, "Highlight",
  CellTags->"BlocksToDClass"],
Cell[531339, 22382, 224, 6, 47, "Usage",
  CellTags->"BlocksToDClass"],
Cell[531566, 22390, 114, 5, 24, "Notes",
  CellTags->"BlocksToDClass"],
Cell[531683, 22397, 435, 13, 42, "Notes",
  CellTags->"BlocksToDClass"],
Cell[532121, 22412, 180, 6, 24, "Notes",
  CellTags->"BlocksToDClass"],
Cell[532304, 22420, 351, 15, 24, "Notes",
  CellTags->"BlocksToDClass"],
Cell[532658, 22437, 72, 1, 39, "ExampleSection",
  CellTags->"BlocksToDClass"],
Cell[532733, 22440, 171, 4, 50, "ExampleText",
  CellTags->"BlocksToDClass"],
Cell[532907, 22446, 295, 5, 58, "Input",
  CellTags->"BlocksToDClass"],
Cell[533205, 22453, 102, 3, 26, "Input",
  CellTags->"BlocksToDClass"],
Cell[533310, 22458, 106, 3, 26, "Input",
  CellTags->"BlocksToDClass"],
Cell[533419, 22463, 83, 1, 34, "ExampleText",
  CellTags->"BlocksToDClass"],
Cell[533505, 22466, 112, 3, 26, "Input",
  CellTags->"BlocksToDClass"],
Cell[533620, 22471, 54, 1, 32, "ObjectName",
  CellTags->"BoundCSM"],
Cell[533677, 22474, 51, 1, 50, "Highlight",
  CellTags->"BoundCSM"],
Cell[533731, 22477, 199, 6, 31, "Usage",
  CellTags->"BoundCSM"],
Cell[533933, 22485, 57, 1, 24, "Notes",
  CellTags->"BoundCSM"],
Cell[533993, 22488, 132, 6, 24, "Notes",
  CellTags->"BoundCSM"],
Cell[534128, 22496, 54, 1, 32, "ObjectName",
  CellTags->"BoundGSG"],
Cell[534185, 22499, 56, 1, 50, "Highlight",
  CellTags->"BoundGSG"],
Cell[534244, 22502, 193, 6, 31, "Usage",
  CellTags->"BoundGSG"],
Cell[534440, 22510, 120, 5, 24, "Notes",
  CellTags->"BoundGSG"],
Cell[534563, 22517, 173, 7, 24, "Notes",
  CellTags->"BoundGSG"],
Cell[534739, 22526, 56, 1, 32, "ObjectName",
  CellTags->"CollapseT"],
Cell[534798, 22529, 54, 1, 50, "Highlight",
  CellTags->"CollapseT"],
Cell[534855, 22532, 190, 6, 31, "Usage",
  CellTags->"CollapseT"],
Cell[535048, 22540, 172, 7, 24, "Notes",
  CellTags->"CollapseT"],
Cell[535223, 22549, 152, 5, 24, "Notes",
  CellTags->"CollapseT"],
Cell[535378, 22556, 576, 23, 24, "Notes",
  CellTags->"CollapseT"],
Cell[535957, 22581, 67, 1, 39, "ExampleSection",
  CellTags->"CollapseT"],
Cell[536027, 22584, 82, 1, 34, "ExampleText",
  CellTags->"CollapseT"],
Cell[536112, 22587, 84, 2, 28, "Input",
  CellTags->"CollapseT"],
Cell[536199, 22591, 122, 4, 42, "Input",
  CellTags->"CollapseT"],
Cell[536324, 22597, 96, 3, 26, "Input",
  CellTags->"CollapseT"],
Cell[536423, 22602, 56, 1, 32, "ObjectName",
  CellTags->"ConstantT"],
Cell[536482, 22605, 57, 1, 50, "Highlight",
  CellTags->"ConstantT"],
Cell[536542, 22608, 225, 9, 31, "Usage",
  CellTags->"ConstantT"],
Cell[536770, 22619, 320, 15, 24, "Notes",
  CellTags->"ConstantT"],
Cell[537093, 22636, 67, 1, 39, "ExampleSection",
  CellTags->"ConstantT"],
Cell[537163, 22639, 74, 1, 34, "ExampleText",
  CellTags->"ConstantT"],
Cell[537240, 22642, 127, 4, 42, "Input",
  CellTags->"ConstantT"],
Cell[537370, 22648, 76, 1, 32, "ObjectName",
  CellTags->"CycleDecompositionT"],
Cell[537449, 22651, 64, 1, 50, "Highlight",
  CellTags->"CycleDecompositionT"],
Cell[537516, 22654, 166, 4, 31, "Usage",
  CellTags->"CycleDecompositionT"],
Cell[537685, 22660, 235, 9, 44, "Notes",
  CellTags->"CycleDecompositionT"],
Cell[537923, 22671, 150, 5, 24, "Notes",
  CellTags->"CycleDecompositionT"],
Cell[538076, 22678, 203, 5, 40, "Notes",
  CellTags->"CycleDecompositionT"],
Cell[538282, 22685, 334, 15, 24, "Notes",
  CellTags->"CycleDecompositionT"],
Cell[538619, 22702, 77, 1, 39, "ExampleSection",
  CellTags->"CycleDecompositionT"],
Cell[538699, 22705, 100, 1, 34, "ExampleText",
  CellTags->"CycleDecompositionT"],
Cell[538802, 22708, 135, 5, 45, "Input",
  CellTags->"CycleDecompositionT"],
Cell[538940, 22715, 135, 3, 26, "Input",
  CellTags->"CycleDecompositionT"],
Cell[539078, 22720, 54, 1, 32, "ObjectName",
  CellTags->"CycleToT"],
Cell[539135, 22723, 53, 1, 50, "Highlight",
  CellTags->"CycleToT"],
Cell[539191, 22726, 176, 5, 47, "Usage",
  CellTags->"CycleToT"],
Cell[539370, 22733, 78, 1, 24, "Notes",
  CellTags->"CycleToT"],
Cell[539451, 22736, 345, 15, 24, "Notes",
  CellTags->"CycleToT"],
Cell[539799, 22753, 66, 1, 39, "ExampleSection",
  CellTags->"CycleToT"],
Cell[539868, 22756, 77, 1, 34, "ExampleText",
  CellTags->"CycleToT"],
Cell[539948, 22759, 119, 3, 26, "Input",
  CellTags->"CycleToT"],
Cell[540070, 22764, 92, 3, 26, "Input",
  CellTags->"CycleToT"],
Cell[540165, 22769, 144, 4, 26, "Input",
  CellTags->"CycleToT"],
Cell[540312, 22775, 104, 3, 26, "Input",
  CellTags->"CycleToT"],
Cell[540419, 22780, 42, 1, 32, "ObjectName",
  CellTags->"DC"],
Cell[540464, 22783, 50, 1, 50, "Highlight",
  CellTags->"DC"],
Cell[540517, 22786, 151, 4, 31, "Usage",
  CellTags->"DC"],
Cell[540671, 22792, 269, 11, 24, "Notes",
  CellTags->"DC"],
Cell[540943, 22805, 410, 19, 24, "Notes",
  CellTags->"DC"],
Cell[541356, 22826, 60, 1, 39, "ExampleSection",
  CellTags->"DC"],
Cell[541419, 22829, 81, 1, 34, "ExampleText",
  CellTags->"DC"],
Cell[541503, 22832, 371, 7, 74, "Input",
  CellTags->"DC"],
Cell[541877, 22841, 140, 4, 42, "Input",
  CellTags->"DC"],
Cell[542020, 22847, 54, 1, 32, "ObjectName",
  CellTags->"DClasses"],
Cell[542077, 22850, 51, 1, 50, "Highlight",
  CellTags->"DClasses"],
Cell[542131, 22853, 145, 6, 31, "Usage",
  CellTags->"DClasses"],
Cell[542279, 22861, 108, 5, 24, "Notes",
  CellTags->"DClasses"],
Cell[542390, 22868, 355, 15, 24, "Notes",
  CellTags->"DClasses"],
Cell[542748, 22885, 76, 1, 32, "ObjectName",
  CellTags->"DClassDecomposition"],
Cell[542827, 22888, 64, 1, 50, "Highlight",
  CellTags->"DClassDecomposition"],
Cell[542894, 22891, 200, 6, 31, "Usage",
  CellTags->"DClassDecomposition"],
Cell[543097, 22899, 241, 9, 44, "Notes",
  CellTags->"DClassDecomposition"],
Cell[543341, 22910, 160, 5, 24, "Notes",
  CellTags->"DClassDecomposition"],
Cell[543504, 22917, 259, 8, 40, "Notes",
  CellTags->"DClassDecomposition"],
Cell[543766, 22927, 128, 4, 24, "Notes",
  CellTags->"DClassDecomposition"],
Cell[543897, 22933, 259, 8, 40, "Notes",
  CellTags->"DClassDecomposition"],
Cell[544159, 22943, 344, 15, 24, "Notes",
  CellTags->"DClassDecomposition"],
Cell[544506, 22960, 77, 1, 39, "ExampleSection",
  CellTags->"DClassDecomposition"],
Cell[544586, 22963, 140, 4, 34, "ExampleText",
  CellTags->"DClassDecomposition"],
Cell[544729, 22969, 156, 4, 42, "Input",
  CellTags->"DClassDecomposition"],
Cell[544888, 22975, 131, 3, 26, "Input",
  CellTags->"DClassDecomposition"],
Cell[545022, 22980, 162, 4, 42, "Input",
  CellTags->"DClassDecomposition"],
Cell[545187, 22986, 113, 1, 34, "ExampleText",
  CellTags->"DClassDecomposition"],
Cell[545303, 22989, 152, 3, 26, "Input",
  CellTags->"DClassDecomposition"],
Cell[545458, 22994, 58, 1, 32, "ObjectName",
  CellTags->"DClassSize"],
Cell[545519, 22997, 55, 1, 50, "Highlight",
  CellTags->"DClassSize"],
Cell[545577, 23000, 157, 6, 31, "Usage",
  CellTags->"DClassSize"],
Cell[545737, 23008, 226, 9, 44, "Notes",
  CellTags->"DClassSize"],
Cell[545966, 23019, 435, 17, 40, "Notes",
  CellTags->"DClassSize"],
Cell[546404, 23038, 110, 5, 24, "Notes",
  CellTags->"DClassSize"],
Cell[546517, 23045, 261, 11, 24, "Notes",
  CellTags->"DClassSize"],
Cell[546781, 23058, 68, 1, 39, "ExampleSection",
  CellTags->"DClassSize"],
Cell[546852, 23061, 77, 1, 34, "ExampleText",
  CellTags->"DClassSize"],
Cell[546932, 23064, 460, 9, 106, "Input",
  CellTags->"DClassSize"],
Cell[547395, 23075, 95, 1, 34, "ExampleText",
  CellTags->"DClassSize"],
Cell[547493, 23078, 129, 3, 26, "Input",
  CellTags->"DClassSize"],
Cell[547625, 23083, 52, 1, 32, "ObjectName",
  CellTags->"DClassT"],
Cell[547680, 23086, 52, 1, 50, "Highlight",
  CellTags->"DClassT"],
Cell[547735, 23089, 212, 8, 31, "Usage",
  CellTags->"DClassT"],
Cell[547950, 23099, 92, 1, 24, "Notes",
  CellTags->"DClassT"],
Cell[548045, 23102, 241, 11, 24, "Notes",
  CellTags->"DClassT"],
Cell[548289, 23115, 65, 1, 39, "ExampleSection",
  CellTags->"DClassT"],
Cell[548357, 23118, 93, 1, 34, "ExampleText",
  CellTags->"DClassT"],
Cell[548453, 23121, 137, 4, 42, "Input",
  CellTags->"DClassT"],
Cell[548593, 23127, 167, 4, 42, "Input",
  CellTags->"DClassT"],
Cell[548763, 23133, 167, 4, 42, "Input",
  CellTags->"DClassT"],
Cell[548933, 23139, 167, 4, 42, "Input",
  CellTags->"DClassT"],
Cell[549103, 23145, 66, 1, 32, "ObjectName",
  CellTags->"DClassToBlocks"],
Cell[549172, 23148, 59, 1, 50, "Highlight",
  CellTags->"DClassToBlocks"],
Cell[549234, 23151, 191, 5, 47, "Usage",
  CellTags->"DClassToBlocks"],
Cell[549428, 23158, 114, 5, 24, "Notes",
  CellTags->"DClassToBlocks"],
Cell[549545, 23165, 323, 9, 42, "Notes",
  CellTags->"DClassToBlocks"],
Cell[549871, 23176, 180, 6, 24, "Notes",
  CellTags->"DClassToBlocks"],
Cell[550054, 23184, 351, 15, 24, "Notes",
  CellTags->"DClassToBlocks"],
Cell[550408, 23201, 72, 1, 39, "ExampleSection",
  CellTags->"DClassToBlocks"],
Cell[550483, 23204, 96, 1, 34, "ExampleText",
  CellTags->"DClassToBlocks"],
Cell[550582, 23207, 292, 6, 58, "Input",
  CellTags->"DClassToBlocks"],
Cell[550877, 23215, 124, 3, 26, "Input",
  CellTags->"DClassToBlocks"],
Cell[551004, 23220, 68, 1, 32, "ObjectName",
  CellTags->"DihedralGroupSG"],
Cell[551075, 23223, 60, 1, 50, "Highlight",
  CellTags->"DihedralGroupSG"],
Cell[551138, 23226, 174, 6, 31, "Usage",
  CellTags->"DihedralGroupSG"],
Cell[551315, 23234, 92, 1, 24, "Notes",
  CellTags->"DihedralGroupSG"],
Cell[551410, 23237, 338, 12, 40, "Notes",
  CellTags->"DihedralGroupSG"],
Cell[551751, 23251, 546, 23, 24, "Notes",
  CellTags->"DihedralGroupSG"],
Cell[552300, 23276, 73, 1, 39, "ExampleSection",
  CellTags->"DihedralGroupSG"],
Cell[552376, 23279, 82, 1, 34, "ExampleText",
  CellTags->"DihedralGroupSG"],
Cell[552461, 23282, 112, 3, 26, "Input",
  CellTags->"DihedralGroupSG"],
Cell[552576, 23287, 171, 5, 34, "ExampleText",
  CellTags->"DihedralGroupSG"],
Cell[552750, 23294, 240, 5, 74, "Input",
  CellTags->"DihedralGroupSG"],
Cell[552993, 23301, 44, 1, 32, "ObjectName",
  CellTags->"GEN"],
Cell[553040, 23304, 51, 1, 50, "Highlight",
  CellTags->"GEN"],
Cell[553094, 23307, 190, 6, 31, "Usage",
  CellTags->"GEN"],
Cell[553287, 23315, 251, 11, 24, "Notes",
  CellTags->"GEN"],
Cell[553541, 23328, 309, 15, 24, "Notes",
  CellTags->"GEN"],
Cell[553853, 23345, 60, 1, 32, "ObjectName",
  CellTags->"GenerateCSM"],
Cell[553916, 23348, 56, 1, 50, "Highlight",
  CellTags->"GenerateCSM"],
Cell[553975, 23351, 311, 10, 47, "Usage",
  CellTags->"GenerateCSM"],
Cell[554289, 23363, 330, 11, 79, "Notes",
  CellTags->"GenerateCSM"],
Cell[554622, 23376, 212, 6, 40, "Notes",
  CellTags->"GenerateCSM"],
Cell[554837, 23384, 230, 7, 40, "Notes",
  CellTags->"GenerateCSM"],
Cell[555070, 23393, 143, 5, 24, "Notes",
  CellTags->"GenerateCSM"],
Cell[555216, 23400, 285, 9, 40, "Notes",
  CellTags->"GenerateCSM"],
Cell[555504, 23411, 125, 4, 24, "Notes",
  CellTags->"GenerateCSM"],
Cell[555632, 23417, 278, 11, 24, "Notes",
  CellTags->"GenerateCSM"],
Cell[555913, 23430, 69, 1, 39, "ExampleSection",
  CellTags->"GenerateCSM"],
Cell[555985, 23433, 134, 4, 34, "ExampleText",
  CellTags->"GenerateCSM"],
Cell[556122, 23439, 189, 5, 42, "Input",
  CellTags->"GenerateCSM"],
Cell[556314, 23446, 249, 7, 58, "Input",
  CellTags->"GenerateCSM"],
Cell[556566, 23455, 194, 5, 50, "ExampleText",
  CellTags->"GenerateCSM"],
Cell[556763, 23462, 136, 4, 26, "Input",
  CellTags->"GenerateCSM"],
Cell[556902, 23468, 127, 4, 34, "ExampleText",
  CellTags->"GenerateCSM"],
Cell[557032, 23474, 284, 6, 58, "Input",
  CellTags->"GenerateCSM"],
Cell[557319, 23482, 306, 6, 66, "ExampleText",
  CellTags->"GenerateCSM"],
Cell[557628, 23490, 243, 7, 42, "Input",
  CellTags->"GenerateCSM"],
Cell[557874, 23499, 172, 4, 26, "Input",
  CellTags->"GenerateCSM"],
Cell[558049, 23505, 76, 1, 32, "ObjectName",
  CellTags->"GenerateCayleyGraph"],
Cell[558128, 23508, 64, 1, 50, "Highlight",
  CellTags->"GenerateCayleyGraph"],
Cell[558195, 23511, 341, 10, 47, "Usage",
  CellTags->"GenerateCayleyGraph"],
Cell[558539, 23523, 326, 10, 79, "Notes",
  CellTags->"GenerateCayleyGraph"],
Cell[558868, 23535, 220, 6, 40, "Notes",
  CellTags->"GenerateCayleyGraph"],
Cell[559091, 23543, 199, 7, 24, "Notes",
  CellTags->"GenerateCayleyGraph"],
Cell[559293, 23552, 150, 5, 24, "Notes",
  CellTags->"GenerateCayleyGraph"],
Cell[559446, 23559, 293, 9, 40, "Notes",
  CellTags->"GenerateCayleyGraph"],
Cell[559742, 23570, 179, 6, 24, "Notes",
  CellTags->"GenerateCayleyGraph"],
Cell[559924, 23578, 270, 11, 24, "Notes",
  CellTags->"GenerateCayleyGraph"],
Cell[560197, 23591, 77, 1, 39, "ExampleSection",
  CellTags->"GenerateCayleyGraph"],
Cell[560277, 23594, 147, 4, 34, "ExampleText",
  CellTags->"GenerateCayleyGraph"],
Cell[560427, 23600, 224, 6, 58, "Input",
  CellTags->"GenerateCayleyGraph"],
Cell[560654, 23608, 133, 3, 26, "Input",
  CellTags->"GenerateCayleyGraph"],
Cell[560790, 23613, 162, 4, 26, "Input",
  CellTags->"GenerateCayleyGraph"],
Cell[560955, 23619, 292, 7, 34, "ExampleText",
  CellTags->"GenerateCayleyGraph"],
Cell[561250, 23628, 284, 6, 58, "Input",
  CellTags->"GenerateCayleyGraph"],
Cell[561537, 23636, 188, 5, 26, "Input",
  CellTags->"GenerateCayleyGraph"],
Cell[561728, 23643, 144, 3, 26, "Input",
  CellTags->"GenerateCayleyGraph"],
Cell[561875, 23648, 58, 1, 32, "ObjectName",
  CellTags->"GenerateSG"],
Cell[561936, 23651, 55, 1, 50, "Highlight",
  CellTags->"GenerateSG"],
Cell[561994, 23654, 183, 6, 31, "Usage",
  CellTags->"GenerateSG"],
Cell[562180, 23662, 509, 14, 133, "Notes",
  CellTags->"GenerateSG"],
Cell[562692, 23678, 154, 5, 24, "Notes",
  CellTags->"GenerateSG"],
Cell[562849, 23685, 483, 16, 56, "Notes",
  CellTags->"GenerateSG"],
Cell[563335, 23703, 172, 5, 24, "Notes",
  CellTags->"GenerateSG"],
Cell[563510, 23710, 154, 5, 24, "Notes",
  CellTags->"GenerateSG"],
Cell[563667, 23717, 181, 6, 24, "Notes",
  CellTags->"GenerateSG"],
Cell[563851, 23725, 349, 10, 56, "Notes",
  CellTags->"GenerateSG"],
Cell[564203, 23737, 161, 4, 24, "Notes",
  CellTags->"GenerateSG"],
Cell[564367, 23743, 407, 19, 24, "Notes",
  CellTags->"GenerateSG"],
Cell[564777, 23764, 68, 1, 39, "ExampleSection",
  CellTags->"GenerateSG"],
Cell[564848, 23767, 131, 4, 34, "ExampleText",
  CellTags->"GenerateSG"],
Cell[564982, 23773, 136, 6, 62, "Input",
  CellTags->"GenerateSG"],
Cell[565121, 23781, 140, 4, 34, "ExampleText",
  CellTags->"GenerateSG"],
Cell[565264, 23787, 145, 4, 26, "Input",
  CellTags->"GenerateSG"],
Cell[565412, 23793, 104, 3, 26, "Input",
  CellTags->"GenerateSG"],
Cell[565519, 23798, 149, 4, 34, "ExampleText",
  CellTags->"GenerateSG"],
Cell[565671, 23804, 108, 3, 26, "Input",
  CellTags->"GenerateSG"],
Cell[565782, 23809, 170, 4, 50, "ExampleText",
  CellTags->"GenerateSG"],
Cell[565955, 23815, 102, 3, 26, "Input",
  CellTags->"GenerateSG"],
Cell[566060, 23820, 115, 3, 28, "Input",
  CellTags->"GenerateSG"],
Cell[566178, 23825, 138, 3, 26, "Input",
  CellTags->"GenerateSG"],
Cell[566319, 23830, 166, 4, 50, "ExampleText",
  CellTags->"GenerateSG"],
Cell[566488, 23836, 139, 3, 26, "Input",
  CellTags->"GenerateSG"],
Cell[566630, 23841, 157, 4, 26, "Input",
  CellTags->"GenerateSG"],
Cell[566790, 23847, 153, 4, 42, "Input",
  CellTags->"GenerateSG"],
Cell[566946, 23853, 249, 8, 34, "ExampleText",
  Evaluatable->False,
  CellTags->"GenerateSG"],
Cell[567198, 23863, 361, 9, 74, "Input",
  CellTags->"GenerateSG"],
Cell[567562, 23874, 227, 8, 34, "ExampleText",
  Evaluatable->False,
  CellTags->"GenerateSG"],
Cell[567792, 23884, 100, 3, 26, "Input",
  CellTags->"GenerateSG"],
Cell[567895, 23889, 54, 1, 32, "ObjectName",
  CellTags->"GreenHQT"],
Cell[567952, 23892, 53, 1, 50, "Highlight",
  CellTags->"GreenHQT"],
Cell[568008, 23895, 250, 10, 31, "Usage",
  CellTags->"GreenHQT"],
Cell[568261, 23907, 476, 19, 24, "Notes",
  CellTags->"GreenHQT"],
Cell[568740, 23928, 54, 1, 32, "ObjectName",
  CellTags->"GreenJQT"],
Cell[568797, 23931, 53, 1, 50, "Highlight",
  CellTags->"GreenJQT"],
Cell[568853, 23934, 250, 10, 31, "Usage",
  CellTags->"GreenJQT"],
Cell[569106, 23946, 316, 10, 40, "Notes",
  CellTags->"GreenJQT"],
Cell[569425, 23958, 427, 15, 24, "Notes",
  CellTags->"GreenJQT"],
Cell[569855, 23975, 362, 15, 24, "Notes",
  CellTags->"GreenJQT"],
Cell[570220, 23992, 54, 1, 32, "ObjectName",
  CellTags->"GreenLQT"],
Cell[570277, 23995, 53, 1, 50, "Highlight",
  CellTags->"GreenLQT"],
Cell[570333, 23998, 250, 10, 31, "Usage",
  CellTags->"GreenLQT"],
Cell[570586, 24010, 476, 19, 24, "Notes",
  CellTags->"GreenLQT"],
Cell[571065, 24031, 54, 1, 32, "ObjectName",
  CellTags->"GreenRQT"],
Cell[571122, 24034, 53, 1, 50, "Highlight",
  CellTags->"GreenRQT"],
Cell[571178, 24037, 250, 10, 31, "Usage",
  CellTags->"GreenRQT"],
Cell[571431, 24049, 476, 19, 24, "Notes",
  CellTags->"GreenRQT"],
Cell[571910, 24070, 58, 1, 32, "ObjectName",
  CellTags->"HClassSize"],
Cell[571971, 24073, 55, 1, 50, "Highlight",
  CellTags->"HClassSize"],
Cell[572029, 24076, 173, 6, 31, "Usage",
  CellTags->"HClassSize"],
Cell[572205, 24084, 110, 5, 24, "Notes",
  CellTags->"HClassSize"],
Cell[572318, 24091, 164, 7, 24, "Notes",
  CellTags->"HClassSize"],
Cell[572485, 24100, 52, 1, 32, "ObjectName",
  CellTags->"HClassT"],
Cell[572540, 24103, 52, 1, 50, "Highlight",
  CellTags->"HClassT"],
Cell[572595, 24106, 206, 8, 31, "Usage",
  CellTags->"HClassT"],
Cell[572804, 24116, 79, 1, 24, "Notes",
  CellTags->"HClassT"],
Cell[572886, 24119, 149, 7, 24, "Notes",
  CellTags->"HClassT"],
Cell[573038, 24128, 60, 1, 32, "ObjectName",
  CellTags->"HTrivialQSG"],
Cell[573101, 24131, 56, 1, 50, "Highlight",
  CellTags->"HTrivialQSG"],
Cell[573160, 24134, 165, 6, 31, "Usage",
  CellTags->"HTrivialQSG"],
Cell[573328, 24142, 88, 1, 24, "Notes",
  CellTags->"HTrivialQSG"],
Cell[573419, 24145, 692, 27, 40, "Notes",
  CellTags->"HTrivialQSG"],
Cell[574114, 24174, 69, 1, 39, "ExampleSection",
  CellTags->"HTrivialQSG"],
Cell[574186, 24177, 90, 1, 34, "ExampleText",
  CellTags->"HTrivialQSG"],
Cell[574279, 24180, 196, 5, 42, "Input",
  CellTags->"HTrivialQSG"],
Cell[574478, 24187, 100, 3, 26, "Input",
  CellTags->"HTrivialQSG"],
Cell[574581, 24192, 50, 1, 32, "ObjectName",
  CellTags->"IdealT"],
Cell[574634, 24195, 51, 1, 50, "Highlight",
  CellTags->"IdealT"],
Cell[574688, 24198, 220, 8, 31, "Usage",
  CellTags->"IdealT"],
Cell[574911, 24208, 73, 1, 24, "Notes",
  CellTags->"IdealT"],
Cell[574987, 24211, 257, 11, 24, "Notes",
  CellTags->"IdealT"],
Cell[575247, 24224, 64, 1, 39, "ExampleSection",
  CellTags->"IdealT"],
Cell[575314, 24227, 125, 4, 34, "ExampleText",
  CellTags->"IdealT"],
Cell[575442, 24233, 109, 3, 26, "Input",
  CellTags->"IdealT"],
Cell[575554, 24238, 108, 3, 26, "Input",
  CellTags->"IdealT"],
Cell[575665, 24243, 108, 3, 26, "Input",
  CellTags->"IdealT"],
Cell[575776, 24248, 50, 1, 32, "ObjectName",
  CellTags->"IdemSG"],
Cell[575829, 24251, 51, 1, 50, "Highlight",
  CellTags->"IdemSG"],
Cell[575883, 24254, 163, 6, 31, "Usage",
  CellTags->"IdemSG"],
Cell[576049, 24262, 73, 1, 24, "Notes",
  CellTags->"IdemSG"],
Cell[576125, 24265, 161, 6, 24, "Notes",
  CellTags->"IdemSG"],
Cell[576289, 24273, 106, 5, 24, "Notes",
  CellTags->"IdemSG"],
Cell[576398, 24280, 253, 11, 24, "Notes",
  CellTags->"IdemSG"],
Cell[576654, 24293, 64, 1, 39, "ExampleSection",
  CellTags->"IdemSG"],
Cell[576721, 24296, 128, 4, 34, "ExampleText",
  CellTags->"IdemSG"],
Cell[576852, 24302, 111, 3, 26, "Input",
  CellTags->"IdemSG"],
Cell[576966, 24307, 108, 3, 26, "Input",
  CellTags->"IdemSG"],
Cell[577077, 24312, 91, 1, 34, "ExampleText",
  CellTags->"IdemSG"],
Cell[577171, 24315, 90, 3, 26, "Input",
  CellTags->"IdemSG"],
Cell[577264, 24320, 50, 1, 32, "ObjectName",
  CellTags->"ImageT"],
Cell[577317, 24323, 51, 1, 50, "Highlight",
  CellTags->"ImageT"],
Cell[577371, 24326, 171, 6, 31, "Usage",
  CellTags->"ImageT"],
Cell[577545, 24334, 106, 5, 24, "Notes",
  CellTags->"ImageT"],
Cell[577654, 24341, 457, 19, 24, "Notes",
  CellTags->"ImageT"],
Cell[578114, 24362, 64, 1, 39, "ExampleSection",
  CellTags->"ImageT"],
Cell[578181, 24365, 91, 1, 34, "ExampleText",
  CellTags->"ImageT"],
Cell[578275, 24368, 197, 5, 42, "Input",
  CellTags->"ImageT"],
Cell[578475, 24375, 56, 1, 32, "ObjectName",
  CellTags->"InGroupSG"],
Cell[578534, 24378, 54, 1, 50, "Highlight",
  CellTags->"InGroupSG"],
Cell[578591, 24381, 218, 8, 31, "Usage",
  CellTags->"InGroupSG"],
Cell[578812, 24391, 76, 1, 24, "Notes",
  CellTags->"InGroupSG"],
Cell[578891, 24394, 169, 6, 24, "Notes",
  CellTags->"InGroupSG"],
Cell[579063, 24402, 109, 5, 24, "Notes",
  CellTags->"InGroupSG"],
Cell[579175, 24409, 250, 11, 24, "Notes",
  CellTags->"InGroupSG"],
Cell[579428, 24422, 67, 1, 39, "ExampleSection",
  CellTags->"InGroupSG"],
Cell[579498, 24425, 94, 1, 34, "ExampleText",
  CellTags->"InGroupSG"],
Cell[579595, 24428, 112, 3, 26, "Input",
  CellTags->"InGroupSG"],
Cell[579710, 24433, 124, 4, 42, "Input",
  CellTags->"InGroupSG"],
Cell[579837, 24439, 118, 3, 26, "Input",
  CellTags->"InGroupSG"],
Cell[579958, 24444, 54, 1, 32, "ObjectName",
  CellTags->"InverseT"],
Cell[580015, 24447, 53, 1, 50, "Highlight",
  CellTags->"InverseT"],
Cell[580071, 24450, 163, 6, 31, "Usage",
  CellTags->"InverseT"],
Cell[580237, 24458, 142, 5, 24, "Notes",
  CellTags->"InverseT"],
Cell[580382, 24465, 247, 11, 24, "Notes",
  CellTags->"InverseT"],
Cell[580632, 24478, 66, 1, 39, "ExampleSection",
  CellTags->"InverseT"],
Cell[580701, 24481, 95, 1, 34, "ExampleText",
  CellTags->"InverseT"],
Cell[580799, 24484, 142, 5, 45, "Input",
  CellTags->"InverseT"],
Cell[580944, 24491, 60, 1, 32, "ObjectName",
  CellTags->"JTrivialQSG"],
Cell[581007, 24494, 56, 1, 50, "Highlight",
  CellTags->"JTrivialQSG"],
Cell[581066, 24497, 164, 6, 31, "Usage",
  CellTags->"JTrivialQSG"],
Cell[581233, 24505, 92, 1, 24, "Notes",
  CellTags->"JTrivialQSG"],
Cell[581328, 24508, 142, 5, 24, "Notes",
  CellTags->"JTrivialQSG"],
Cell[581473, 24515, 692, 27, 40, "Notes",
  CellTags->"JTrivialQSG"],
Cell[582168, 24544, 69, 1, 39, "ExampleSection",
  CellTags->"JTrivialQSG"],
Cell[582240, 24547, 76, 1, 34, "ExampleText",
  CellTags->"JTrivialQSG"],
Cell[582319, 24550, 135, 3, 26, "Input",
  CellTags->"JTrivialQSG"],
Cell[582457, 24555, 100, 3, 26, "Input",
  CellTags->"JTrivialQSG"],
Cell[582560, 24560, 138, 3, 26, "Input",
  CellTags->"JTrivialQSG"],
Cell[582701, 24565, 100, 3, 26, "Input",
  CellTags->"JTrivialQSG"],
Cell[582804, 24570, 58, 1, 32, "ObjectName",
  CellTags->"JTrivialSG"],
Cell[582865, 24573, 55, 1, 50, "Highlight",
  CellTags->"JTrivialSG"],
Cell[582923, 24576, 186, 5, 47, "Usage",
  CellTags->"JTrivialSG"],
Cell[583112, 24583, 185, 6, 24, "Notes",
  CellTags->"JTrivialSG"],
Cell[583300, 24591, 484, 19, 24, "Notes",
  CellTags->"JTrivialSG"],
Cell[583787, 24612, 68, 1, 39, "ExampleSection",
  CellTags->"JTrivialSG"],
Cell[583858, 24615, 75, 1, 34, "ExampleText",
  CellTags->"JTrivialSG"],
Cell[583936, 24618, 98, 3, 26, "Input",
  CellTags->"JTrivialSG"],
Cell[584037, 24623, 101, 3, 26, "Input",
  CellTags->"JTrivialSG"],
Cell[584141, 24628, 83, 2, 28, "Input",
  CellTags->"JTrivialSG"],
Cell[584227, 24632, 66, 1, 32, "ObjectName",
  CellTags->"KernelImageToT"],
Cell[584296, 24635, 59, 1, 50, "Highlight",
  CellTags->"KernelImageToT"],
Cell[584358, 24638, 167, 4, 31, "Usage",
  CellTags->"KernelImageToT"],
Cell[584528, 24644, 145, 4, 24, "Notes",
  CellTags->"KernelImageToT"],
Cell[584676, 24650, 467, 19, 24, "Notes",
  CellTags->"KernelImageToT"],
Cell[585146, 24671, 72, 1, 39, "ExampleSection",
  CellTags->"KernelImageToT"],
Cell[585221, 24674, 128, 4, 34, "ExampleText",
  CellTags->"KernelImageToT"],
Cell[585352, 24680, 88, 2, 28, "Input",
  CellTags->"KernelImageToT"],
Cell[585443, 24684, 145, 4, 42, "Input",
  CellTags->"KernelImageToT"],
Cell[585591, 24690, 111, 3, 26, "Input",
  CellTags->"KernelImageToT"],
Cell[585705, 24695, 52, 1, 32, "ObjectName",
  CellTags->"KernelT"],
Cell[585760, 24698, 52, 1, 50, "Highlight",
  CellTags->"KernelT"],
Cell[585815, 24701, 155, 6, 31, "Usage",
  CellTags->"KernelT"],
Cell[585973, 24709, 107, 5, 24, "Notes",
  CellTags->"KernelT"],
Cell[586083, 24716, 233, 6, 40, "Notes",
  CellTags->"KernelT"],
Cell[586319, 24724, 456, 19, 24, "Notes",
  CellTags->"KernelT"],
Cell[586778, 24745, 65, 1, 39, "ExampleSection",
  CellTags->"KernelT"],
Cell[586846, 24748, 93, 1, 34, "ExampleText",
  CellTags->"KernelT"],
Cell[586942, 24751, 199, 5, 42, "Input",
  CellTags->"KernelT"],
Cell[587144, 24758, 58, 1, 32, "ObjectName",
  CellTags->"LeftIdealT"],
Cell[587205, 24761, 55, 1, 50, "Highlight",
  CellTags->"LeftIdealT"],
Cell[587263, 24764, 167, 4, 31, "Usage",
  CellTags->"LeftIdealT"],
Cell[587433, 24770, 77, 1, 24, "Notes",
  CellTags->"LeftIdealT"],
Cell[587513, 24773, 253, 11, 24, "Notes",
  CellTags->"LeftIdealT"],
Cell[587769, 24786, 68, 1, 39, "ExampleSection",
  CellTags->"LeftIdealT"],
Cell[587840, 24789, 134, 4, 34, "ExampleText",
  CellTags->"LeftIdealT"],
Cell[587977, 24795, 113, 3, 26, "Input",
  CellTags->"LeftIdealT"],
Cell[588093, 24800, 116, 3, 26, "Input",
  CellTags->"LeftIdealT"],
Cell[588212, 24805, 116, 3, 26, "Input",
  CellTags->"LeftIdealT"],
Cell[588331, 24810, 58, 1, 32, "ObjectName",
  CellTags->"LeftNullSG"],
Cell[588392, 24813, 55, 1, 50, "Highlight",
  CellTags->"LeftNullSG"],
Cell[588450, 24816, 162, 6, 31, "Usage",
  CellTags->"LeftNullSG"],
Cell[588615, 24824, 77, 1, 24, "Notes",
  CellTags->"LeftNullSG"],
Cell[588695, 24827, 239, 10, 24, "Notes",
  CellTags->"LeftNullSG"],
Cell[588937, 24839, 110, 5, 24, "Notes",
  CellTags->"LeftNullSG"],
Cell[589050, 24846, 340, 15, 24, "Notes",
  CellTags->"LeftNullSG"],
Cell[589393, 24863, 68, 1, 39, "ExampleSection",
  CellTags->"LeftNullSG"],
Cell[589464, 24866, 123, 4, 34, "ExampleText",
  CellTags->"LeftNullSG"],
Cell[589590, 24872, 93, 2, 28, "Input",
  CellTags->"LeftNullSG"],
Cell[589686, 24876, 102, 2, 26, "Input",
  CellTags->"LeftNullSG"],
Cell[589791, 24880, 98, 3, 26, "Input",
  CellTags->"LeftNullSG"],
Cell[589892, 24885, 56, 1, 32, "ObjectName",
  CellTags->"LeftOneSG"],
Cell[589951, 24888, 54, 1, 50, "Highlight",
  CellTags->"LeftOneSG"],
Cell[590008, 24891, 165, 6, 31, "Usage",
  CellTags->"LeftOneSG"],
Cell[590176, 24899, 76, 1, 24, "Notes",
  CellTags->"LeftOneSG"],
Cell[590255, 24902, 251, 10, 24, "Notes",
  CellTags->"LeftOneSG"],
Cell[590509, 24914, 109, 5, 24, "Notes",
  CellTags->"LeftOneSG"],
Cell[590621, 24921, 337, 15, 24, "Notes",
  CellTags->"LeftOneSG"],
Cell[590961, 24938, 67, 1, 39, "ExampleSection",
  CellTags->"LeftOneSG"],
Cell[591031, 24941, 138, 4, 34, "ExampleText",
  CellTags->"LeftOneSG"],
Cell[591172, 24947, 91, 2, 28, "Input",
  CellTags->"LeftOneSG"],
Cell[591266, 24951, 126, 3, 26, "Input",
  CellTags->"LeftOneSG"],
Cell[591395, 24956, 95, 3, 26, "Input",
  CellTags->"LeftOneSG"],
Cell[591493, 24961, 80, 1, 32, "ObjectName",
  CellTags->"LocallyCommutativeQSG"],
Cell[591576, 24964, 66, 1, 50, "Highlight",
  CellTags->"LocallyCommutativeQSG"],
Cell[591645, 24967, 189, 6, 31, "Usage",
  CellTags->"LocallyCommutativeQSG"],
Cell[591837, 24975, 256, 9, 24, "Notes",
  CellTags->"LocallyCommutativeQSG"],
Cell[592096, 24986, 254, 9, 24, "Notes",
  CellTags->"LocallyCommutativeQSG"],
Cell[592353, 24997, 609, 23, 40, "Notes",
  CellTags->"LocallyCommutativeQSG"],
Cell[592965, 25022, 79, 1, 39, "ExampleSection",
  CellTags->"LocallyCommutativeQSG"],
Cell[593047, 25025, 115, 1, 34, "ExampleText",
  CellTags->"LocallyCommutativeQSG"],
Cell[593165, 25028, 157, 5, 45, "Input",
  CellTags->"LocallyCommutativeQSG"],
Cell[593325, 25035, 120, 2, 34, "ExampleText",
  CellTags->"LocallyCommutativeQSG"],
Cell[593448, 25039, 197, 5, 58, "Input",
  CellTags->"LocallyCommutativeQSG"],
Cell[593648, 25046, 120, 3, 26, "Input",
  CellTags->"LocallyCommutativeQSG"],
Cell[593771, 25051, 78, 1, 32, "ObjectName",
  CellTags->"LocallyIdempotentQSG"],
Cell[593852, 25054, 65, 1, 50, "Highlight",
  CellTags->"LocallyIdempotentQSG"],
Cell[593920, 25057, 187, 6, 31, "Usage",
  CellTags->"LocallyIdempotentQSG"],
Cell[594110, 25065, 270, 9, 24, "Notes",
  CellTags->"LocallyIdempotentQSG"],
Cell[594383, 25076, 253, 9, 24, "Notes",
  CellTags->"LocallyIdempotentQSG"],
Cell[594639, 25087, 709, 27, 40, "Notes",
  CellTags->"LocallyIdempotentQSG"],
Cell[595351, 25116, 78, 1, 39, "ExampleSection",
  CellTags->"LocallyIdempotentQSG"],
Cell[595432, 25119, 113, 1, 34, "ExampleText",
  CellTags->"LocallyIdempotentQSG"],
Cell[595548, 25122, 156, 5, 45, "Input",
  CellTags->"LocallyIdempotentQSG"],
Cell[595707, 25129, 196, 5, 58, "Input",
  CellTags->"LocallyIdempotentQSG"],
Cell[595906, 25136, 118, 3, 26, "Input",
  CellTags->"LocallyIdempotentQSG"],
Cell[596027, 25141, 54, 1, 32, "ObjectName",
  CellTags->"Merge12T"],
Cell[596084, 25144, 53, 1, 50, "Highlight",
  CellTags->"Merge12T"],
Cell[596140, 25147, 161, 6, 31, "Usage",
  CellTags->"Merge12T"],
Cell[596304, 25155, 101, 1, 24, "Notes",
  CellTags->"Merge12T"],
Cell[596408, 25158, 350, 15, 24, "Notes",
  CellTags->"Merge12T"],
Cell[596761, 25175, 66, 1, 39, "ExampleSection",
  CellTags->"Merge12T"],
Cell[596830, 25178, 92, 1, 34, "ExampleText",
  CellTags->"Merge12T"],
Cell[596925, 25181, 156, 4, 26, "Input",
  CellTags->"Merge12T"],
Cell[597084, 25187, 110, 3, 26, "Input",
  CellTags->"Merge12T"],
Cell[597197, 25192, 108, 2, 34, "ExampleText",
  CellTags->"Merge12T"],
Cell[597308, 25196, 136, 3, 26, "Input",
  CellTags->"Merge12T"],
Cell[597447, 25201, 110, 3, 26, "Input",
  CellTags->"Merge12T"],
Cell[597560, 25206, 62, 1, 32, "ObjectName",
  CellTags->"MakeMonoidSG"],
Cell[597625, 25209, 57, 1, 50, "Highlight",
  CellTags->"MakeMonoidSG"],
Cell[597685, 25212, 194, 6, 31, "Usage",
  CellTags->"MakeMonoidSG"],
Cell[597882, 25220, 163, 7, 24, "Notes",
  CellTags->"MakeMonoidSG"],
Cell[598048, 25229, 70, 1, 39, "ExampleSection",
  CellTags->"MakeMonoidSG"],
Cell[598121, 25232, 78, 1, 34, "ExampleText",
  CellTags->"MakeMonoidSG"],
Cell[598202, 25235, 134, 3, 26, "Input",
  CellTags->"MakeMonoidSG"],
Cell[598339, 25240, 102, 3, 26, "Input",
  CellTags->"MakeMonoidSG"],
Cell[598444, 25245, 56, 1, 32, "ObjectName",
  CellTags->"MonoidQSG"],
Cell[598503, 25248, 54, 1, 50, "Highlight",
  CellTags->"MonoidQSG"],
Cell[598560, 25251, 160, 6, 31, "Usage",
  CellTags->"MonoidQSG"],
Cell[598723, 25259, 694, 27, 40, "Notes",
  CellTags->"MonoidQSG"],
Cell[599420, 25288, 67, 1, 39, "ExampleSection",
  CellTags->"MonoidQSG"],
Cell[599490, 25291, 75, 1, 34, "ExampleText",
  CellTags->"MonoidQSG"],
Cell[599568, 25294, 132, 3, 26, "Input",
  CellTags->"MonoidQSG"],
Cell[599703, 25299, 97, 3, 26, "Input",
  CellTags->"MonoidQSG"],
Cell[599803, 25304, 62, 1, 32, "ObjectName",
  CellTags->"NilpotentQSG"],
Cell[599868, 25307, 57, 1, 50, "Highlight",
  CellTags->"NilpotentQSG"],
Cell[599928, 25310, 218, 6, 47, "Usage",
  CellTags->"NilpotentQSG"],
Cell[600149, 25318, 691, 27, 40, "Notes",
  CellTags->"NilpotentQSG"],
Cell[600843, 25347, 70, 1, 39, "ExampleSection",
  CellTags->"NilpotentQSG"],
Cell[600916, 25350, 107, 1, 34, "ExampleText",
  CellTags->"NilpotentQSG"],
Cell[601026, 25353, 140, 3, 26, "Input",
  CellTags->"NilpotentQSG"],
Cell[601169, 25358, 110, 3, 26, "Input",
  CellTags->"NilpotentQSG"],
Cell[601282, 25363, 103, 3, 26, "Input",
  CellTags->"NilpotentQSG"],
Cell[601388, 25368, 50, 1, 32, "ObjectName",
  CellTags->"NullSG"],
Cell[601441, 25371, 51, 1, 50, "Highlight",
  CellTags->"NullSG"],
Cell[601495, 25374, 170, 6, 31, "Usage",
  CellTags->"NullSG"],
Cell[601668, 25382, 202, 8, 24, "Notes",
  CellTags->"NullSG"],
Cell[601873, 25392, 106, 5, 24, "Notes",
  CellTags->"NullSG"],
Cell[601982, 25399, 353, 15, 24, "Notes",
  CellTags->"NullSG"],
Cell[602338, 25416, 64, 1, 39, "ExampleSection",
  CellTags->"NullSG"],
Cell[602405, 25419, 69, 1, 34, "ExampleText",
  CellTags->"NullSG"],
Cell[602477, 25422, 546, 10, 106, "Input",
  CellTags->"NullSG"],
Cell[603026, 25434, 119, 4, 42, "Input",
  CellTags->"NullSG"],
Cell[603148, 25440, 112, 3, 26, "Input",
  CellTags->"NullSG"],
Cell[603263, 25445, 48, 1, 32, "ObjectName",
  CellTags->"OneSG"],
Cell[603314, 25448, 50, 1, 50, "Highlight",
  CellTags->"OneSG"],
Cell[603367, 25451, 164, 6, 31, "Usage",
  CellTags->"OneSG"],
Cell[603534, 25459, 206, 8, 24, "Notes",
  CellTags->"OneSG"],
Cell[603743, 25469, 105, 5, 24, "Notes",
  CellTags->"OneSG"],
Cell[603851, 25476, 235, 11, 24, "Notes",
  CellTags->"OneSG"],
Cell[604089, 25489, 63, 1, 39, "ExampleSection",
  CellTags->"OneSG"],
Cell[604155, 25492, 96, 1, 34, "ExampleText",
  CellTags->"OneSG"],
Cell[604254, 25495, 206, 5, 42, "Input",
  CellTags->"OneSG"],
Cell[604463, 25502, 129, 3, 26, "Input",
  CellTags->"OneSG"],
Cell[604595, 25507, 94, 1, 34, "ExampleText",
  CellTags->"OneSG"],
Cell[604692, 25510, 90, 2, 28, "Input",
  CellTags->"OneSG"],
Cell[604785, 25514, 89, 3, 26, "Input",
  CellTags->"OneSG"],
Cell[604877, 25519, 104, 2, 34, "ExampleText",
  CellTags->"OneSG"],
Cell[604984, 25523, 126, 3, 26, "Input",
  CellTags->"OneSG"],
Cell[605113, 25528, 88, 3, 26, "Input",
  CellTags->"OneSG"],
Cell[605204, 25533, 94, 3, 26, "Input",
  CellTags->"OneSG"],
Cell[605301, 25538, 46, 1, 32, "ObjectName",
  CellTags->"OneT"],
Cell[605350, 25541, 52, 1, 50, "Highlight",
  CellTags->"OneT"],
Cell[605405, 25544, 226, 8, 31, "Usage",
  CellTags->"OneT"],
Cell[605634, 25554, 199, 6, 24, "Notes",
  CellTags->"OneT"],
Cell[605836, 25562, 149, 7, 24, "Notes",
  CellTags->"OneT"],
Cell[605988, 25571, 62, 1, 39, "ExampleSection",
  CellTags->"OneT"],
Cell[606053, 25574, 83, 1, 34, "ExampleText",
  CellTags->"OneT"],
Cell[606139, 25577, 72, 2, 28, "Input",
  CellTags->"OneT"],
Cell[606214, 25581, 113, 3, 26, "Input",
  CellTags->"OneT"],
Cell[606330, 25586, 50, 1, 32, "ObjectName",
  CellTags->"OrderT"],
Cell[606383, 25589, 51, 1, 50, "Highlight",
  CellTags->"OrderT"],
Cell[606437, 25592, 125, 4, 31, "Usage",
  CellTags->"OrderT"],
Cell[606565, 25598, 247, 11, 24, "Notes",
  CellTags->"OrderT"],
Cell[606815, 25611, 64, 1, 39, "ExampleSection",
  CellTags->"OrderT"],
Cell[606882, 25614, 121, 4, 34, "ExampleText",
  CellTags->"OrderT"],
Cell[607006, 25620, 91, 2, 28, "Input",
  CellTags->"OrderT"],
Cell[607100, 25624, 148, 4, 42, "Input",
  CellTags->"OrderT"],
Cell[607251, 25630, 60, 1, 32, "ObjectName",
  CellTags->"PrintDClass"],
Cell[607314, 25633, 56, 1, 50, "Highlight",
  CellTags->"PrintDClass"],
Cell[607373, 25636, 154, 6, 31, "Usage",
  CellTags->"PrintDClass"],
Cell[607530, 25644, 111, 5, 24, "Notes",
  CellTags->"PrintDClass"],
Cell[607644, 25651, 194, 6, 24, "Notes",
  CellTags->"PrintDClass"],
Cell[607841, 25659, 358, 15, 24, "Notes",
  CellTags->"PrintDClass"],
Cell[608202, 25676, 69, 1, 39, "ExampleSection",
  CellTags->"PrintDClass"],
Cell[608274, 25679, 78, 1, 34, "ExampleText",
  CellTags->"PrintDClass"],
Cell[608355, 25682, 380, 7, 74, "Input",
  CellTags->"PrintDClass"],
Cell[608738, 25691, 100, 3, 26, "Input",
  CellTags->"PrintDClass"],
Cell[608841, 25696, 66, 1, 32, "ObjectName",
  CellTags->"PrintEquations"],
Cell[608910, 25699, 59, 1, 50, "Highlight",
  CellTags->"PrintEquations"],
Cell[608972, 25702, 153, 4, 31, "Usage",
  CellTags->"PrintEquations"],
Cell[609128, 25708, 279, 10, 61, "Notes",
  CellTags->"PrintEquations"],
Cell[609410, 25720, 149, 5, 24, "Notes",
  CellTags->"PrintEquations"],
Cell[609562, 25727, 458, 19, 24, "Notes",
  CellTags->"PrintEquations"],
Cell[610023, 25748, 72, 1, 39, "ExampleSection",
  CellTags->"PrintEquations"],
Cell[610098, 25751, 142, 4, 34, "ExampleText",
  CellTags->"PrintEquations"],
Cell[610243, 25757, 160, 4, 26, "Input",
  CellTags->"PrintEquations"],
Cell[610406, 25763, 100, 3, 26, "Input",
  CellTags->"PrintEquations"],
Cell[610509, 25768, 102, 1, 34, "ExampleText",
  CellTags->"PrintEquations"],
Cell[610614, 25771, 170, 4, 42, "Input",
  CellTags->"PrintEquations"],
Cell[610787, 25777, 66, 1, 32, "ObjectName",
  CellTags->"PrintWitnesses"],
Cell[610856, 25780, 59, 1, 50, "Highlight",
  CellTags->"PrintWitnesses"],
Cell[610918, 25783, 204, 6, 31, "Usage",
  CellTags->"PrintWitnesses"],
Cell[611125, 25791, 458, 19, 24, "Notes",
  CellTags->"PrintWitnesses"],
Cell[611586, 25812, 72, 1, 39, "ExampleSection",
  CellTags->"PrintWitnesses"],
Cell[611661, 25815, 142, 4, 34, "ExampleText",
  CellTags->"PrintWitnesses"],
Cell[611806, 25821, 160, 4, 26, "Input",
  CellTags->"PrintWitnesses"],
Cell[611969, 25827, 110, 3, 26, "Input",
  CellTags->"PrintWitnesses"],
Cell[612082, 25832, 56, 1, 32, "ObjectName",
  CellTags->"RegularSG"],
Cell[612141, 25835, 54, 1, 50, "Highlight",
  CellTags->"RegularSG"],
Cell[612198, 25838, 166, 6, 31, "Usage",
  CellTags->"RegularSG"],
Cell[612367, 25846, 81, 1, 24, "Notes",
  CellTags->"RegularSG"],
Cell[612451, 25849, 204, 8, 24, "Notes",
  CellTags->"RegularSG"],
Cell[612658, 25859, 162, 7, 24, "Notes",
  CellTags->"RegularSG"],
Cell[612823, 25868, 67, 1, 39, "ExampleSection",
  CellTags->"RegularSG"],
Cell[612893, 25871, 104, 1, 34, "ExampleText",
  CellTags->"RegularSG"],
Cell[613000, 25874, 147, 4, 26, "Input",
  CellTags->"RegularSG"],
Cell[613150, 25880, 121, 3, 26, "Input",
  CellTags->"RegularSG"],
Cell[613274, 25885, 97, 3, 26, "Input",
  CellTags->"RegularSG"],
Cell[613374, 25890, 94, 3, 26, "Input",
  CellTags->"RegularSG"],
Cell[613471, 25895, 52, 1, 32, "ObjectName",
  CellTags->"RandomT"],
Cell[613526, 25898, 52, 1, 50, "Highlight",
  CellTags->"RandomT"],
Cell[613581, 25901, 132, 4, 31, "Usage",
  CellTags->"RandomT"],
Cell[613716, 25907, 155, 6, 24, "Notes",
  CellTags->"RandomT"],
Cell[613874, 25915, 142, 7, 24, "Notes",
  CellTags->"RandomT"],
Cell[614019, 25924, 65, 1, 39, "ExampleSection",
  CellTags->"RandomT"],
Cell[614087, 25927, 74, 1, 34, "ExampleText",
  CellTags->"RandomT"],
Cell[614164, 25930, 89, 2, 28, "Input",
  CellTags->"RandomT"],
Cell[614256, 25934, 48, 1, 32, "ObjectName",
  CellTags->"RankT"],
Cell[614307, 25937, 50, 1, 50, "Highlight",
  CellTags->"RankT"],
Cell[614360, 25940, 105, 5, 24, "Notes",
  CellTags->"RankT"],
Cell[614468, 25947, 196, 6, 31, "Usage",
  CellTags->"RankT"],
Cell[614667, 25955, 105, 5, 24, "Notes",
  CellTags->"RankT"],
Cell[614775, 25962, 458, 19, 24, "Notes",
  CellTags->"RankT"],
Cell[615236, 25983, 63, 1, 39, "ExampleSection",
  CellTags->"RankT"],
Cell[615302, 25986, 90, 1, 34, "ExampleText",
  CellTags->"RankT"],
Cell[615395, 25989, 149, 4, 26, "Input",
  CellTags->"RankT"],
Cell[615547, 25995, 60, 1, 32, "ObjectName",
  CellTags->"RegularOnly"],
Cell[615610, 25998, 54, 1, 50, "Highlight",
  CellTags->"RegularOnly"],
Cell[615667, 26001, 169, 6, 31, "Usage",
  CellTags->"RegularOnly"],
Cell[615839, 26009, 189, 6, 40, "Notes",
  CellTags->"RegularOnly"],
Cell[616031, 26017, 186, 5, 40, "Notes",
  CellTags->"RegularOnly"],
Cell[616220, 26024, 256, 11, 24, "Notes",
  CellTags->"RegularOnly"],
Cell[616479, 26037, 60, 1, 32, "ObjectName",
  CellTags->"RightIdealT"],
Cell[616542, 26040, 56, 1, 50, "Highlight",
  CellTags->"RightIdealT"],
Cell[616601, 26043, 170, 4, 31, "Usage",
  CellTags->"RightIdealT"],
Cell[616774, 26049, 78, 1, 24, "Notes",
  CellTags->"RightIdealT"],
Cell[616855, 26052, 252, 11, 24, "Notes",
  CellTags->"RightIdealT"],
Cell[617110, 26065, 69, 1, 39, "ExampleSection",
  CellTags->"RightIdealT"],
Cell[617182, 26068, 136, 4, 34, "ExampleText",
  CellTags->"RightIdealT"],
Cell[617321, 26074, 115, 3, 26, "Input",
  CellTags->"RightIdealT"],
Cell[617439, 26079, 119, 3, 26, "Input",
  CellTags->"RightIdealT"],
Cell[617561, 26084, 119, 3, 26, "Input",
  CellTags->"RightIdealT"],
Cell[617683, 26089, 60, 1, 32, "ObjectName",
  CellTags->"RightNullSG"],
Cell[617746, 26092, 56, 1, 50, "Highlight",
  CellTags->"RightNullSG"],
Cell[617805, 26095, 164, 6, 31, "Usage",
  CellTags->"RightNullSG"],
Cell[617972, 26103, 78, 1, 24, "Notes",
  CellTags->"RightNullSG"],
Cell[618053, 26106, 218, 8, 24, "Notes",
  CellTags->"RightNullSG"],
Cell[618274, 26116, 111, 5, 24, "Notes",
  CellTags->"RightNullSG"],
Cell[618388, 26123, 357, 15, 24, "Notes",
  CellTags->"RightNullSG"],
Cell[618748, 26140, 69, 1, 39, "ExampleSection",
  CellTags->"RightNullSG"],
Cell[618820, 26143, 154, 4, 50, "ExampleText",
  CellTags->"RightNullSG"],
Cell[618977, 26149, 95, 2, 28, "Input",
  CellTags->"RightNullSG"],
Cell[619075, 26153, 131, 3, 26, "Input",
  CellTags->"RightNullSG"],
Cell[619209, 26158, 101, 3, 26, "Input",
  CellTags->"RightNullSG"],
Cell[619313, 26163, 58, 1, 32, "ObjectName",
  CellTags->"RightOneSG"],
Cell[619374, 26166, 55, 1, 50, "Highlight",
  CellTags->"RightOneSG"],
Cell[619432, 26169, 168, 6, 31, "Usage",
  CellTags->"RightOneSG"],
Cell[619603, 26177, 77, 1, 24, "Notes",
  CellTags->"RightOneSG"],
Cell[619683, 26180, 220, 8, 24, "Notes",
  CellTags->"RightOneSG"],
Cell[619906, 26190, 110, 5, 24, "Notes",
  CellTags->"RightOneSG"],
Cell[620019, 26197, 336, 15, 24, "Notes",
  CellTags->"RightOneSG"],
Cell[620358, 26214, 68, 1, 39, "ExampleSection",
  CellTags->"RightOneSG"],
Cell[620429, 26217, 140, 4, 34, "ExampleText",
  CellTags->"RightOneSG"],
Cell[620572, 26223, 92, 2, 28, "Input",
  CellTags->"RightOneSG"],
Cell[620667, 26227, 129, 3, 26, "Input",
  CellTags->"RightOneSG"],
Cell[620799, 26232, 97, 3, 26, "Input",
  CellTags->"RightOneSG"],
Cell[620899, 26237, 50, 1, 32, "ObjectName",
  CellTags->"ShiftT"],
Cell[620952, 26240, 54, 1, 50, "Highlight",
  CellTags->"ShiftT"],
Cell[621009, 26243, 171, 6, 31, "Usage",
  CellTags->"ShiftT"],
Cell[621183, 26251, 153, 6, 24, "Notes",
  CellTags->"ShiftT"],
Cell[621339, 26259, 132, 4, 24, "Notes",
  CellTags->"ShiftT"],
Cell[621474, 26265, 414, 16, 24, "Notes",
  CellTags->"ShiftT"],
Cell[621891, 26283, 64, 1, 39, "ExampleSection",
  CellTags->"ShiftT"],
Cell[621958, 26286, 101, 1, 34, "ExampleText",
  CellTags->"ShiftT"],
Cell[622062, 26289, 106, 3, 26, "Input",
  CellTags->"ShiftT"],
Cell[622171, 26294, 122, 4, 34, "ExampleText",
  CellTags->"ShiftT"],
Cell[622296, 26300, 143, 4, 26, "Input",
  CellTags->"ShiftT"],
Cell[622442, 26306, 42, 1, 32, "ObjectName",
  CellTags->"SG"],
Cell[622487, 26309, 50, 1, 50, "Highlight",
  CellTags->"SG"],
Cell[622540, 26312, 124, 4, 31, "Usage",
  CellTags->"SG"],
Cell[622667, 26318, 275, 9, 24, "Notes",
  CellTags->"SG"],
Cell[622945, 26329, 513, 20, 24, "Notes",
  CellTags->"SG"],
Cell[623461, 26351, 60, 1, 39, "ExampleSection",
  CellTags->"SG"],
Cell[623524, 26354, 112, 4, 34, "ExampleText",
  CellTags->"SG"],
Cell[623639, 26360, 95, 3, 26, "Input",
  CellTags->"SG"],
Cell[623737, 26365, 52, 1, 32, "ObjectName",
  CellTags->"SolveSG"],
Cell[623792, 26368, 52, 1, 50, "Highlight",
  CellTags->"SolveSG"],
Cell[623847, 26371, 237, 8, 47, "Usage",
  CellTags->"SolveSG"],
Cell[624087, 26381, 308, 10, 61, "Notes",
  CellTags->"SolveSG"],
Cell[624398, 26393, 193, 7, 24, "Notes",
  CellTags->"SolveSG"],
Cell[624594, 26402, 158, 7, 24, "Notes",
  CellTags->"SolveSG"],
Cell[624755, 26411, 240, 11, 24, "Notes",
  CellTags->"SolveSG"],
Cell[624998, 26424, 65, 1, 39, "ExampleSection",
  CellTags->"SolveSG"],
Cell[625066, 26427, 93, 1, 34, "ExampleText",
  CellTags->"SolveSG"],
Cell[625162, 26430, 90, 2, 28, "Input",
  CellTags->"SolveSG"],
Cell[625255, 26434, 198, 5, 42, "Input",
  CellTags->"SolveSG"],
Cell[625456, 26441, 108, 2, 34, "ExampleText",
  CellTags->"SolveSG"],
Cell[625567, 26445, 90, 2, 28, "Input",
  CellTags->"SolveSG"],
Cell[625660, 26449, 199, 5, 42, "Input",
  CellTags->"SolveSG"],
Cell[625862, 26456, 70, 1, 32, "ObjectName",
  CellTags->"SymmetricGroupSG"],
Cell[625935, 26459, 61, 1, 50, "Highlight",
  CellTags->"SymmetricGroupSG"],
Cell[625999, 26462, 148, 4, 31, "Usage",
  CellTags->"SymmetricGroupSG"],
Cell[626150, 26468, 254, 9, 24, "Notes",
  CellTags->"SymmetricGroupSG"],
Cell[626407, 26479, 74, 1, 39, "ExampleSection",
  CellTags->"SymmetricGroupSG"],
Cell[626484, 26482, 87, 1, 34, "ExampleText",
  CellTags->"SymmetricGroupSG"],
Cell[626574, 26485, 93, 2, 28, "Input",
  CellTags->"SymmetricGroupSG"],
Cell[626670, 26489, 76, 1, 32, "ObjectName",
  CellTags->"SyntacticCongruence"],
Cell[626749, 26492, 64, 1, 50, "Highlight",
  CellTags->"SyntacticCongruence"],
Cell[626816, 26495, 214, 6, 47, "Usage",
  CellTags->"SyntacticCongruence"],
Cell[627033, 26503, 212, 7, 24, "Notes",
  CellTags->"SyntacticCongruence"],
Cell[627248, 26512, 510, 19, 24, "Notes",
  CellTags->"SyntacticCongruence"],
Cell[627761, 26533, 77, 1, 39, "ExampleSection",
  CellTags->"SyntacticCongruence"],
Cell[627841, 26536, 146, 4, 34, "ExampleText",
  CellTags->"SyntacticCongruence"],
Cell[627990, 26542, 136, 3, 26, "Input",
  CellTags->"SyntacticCongruence"],
Cell[628129, 26547, 128, 3, 26, "Input",
  CellTags->"SyntacticCongruence"],
Cell[628260, 26552, 117, 2, 34, "ExampleText",
  CellTags->"SyntacticCongruence"],
Cell[628380, 26556, 163, 4, 42, "Input",
  CellTags->"SyntacticCongruence"],
Cell[628546, 26562, 80, 1, 32, "ObjectName",
  CellTags->"SyntacticHomomorphism"],
Cell[628629, 26565, 66, 1, 50, "Highlight",
  CellTags->"SyntacticHomomorphism"],
Cell[628698, 26568, 196, 6, 31, "Usage",
  CellTags->"SyntacticHomomorphism"],
Cell[628897, 26576, 122, 5, 24, "Notes",
  CellTags->"SyntacticHomomorphism"],
Cell[629022, 26583, 284, 9, 40, "Notes",
  CellTags->"SyntacticHomomorphism"],
Cell[629309, 26594, 412, 15, 24, "Notes",
  CellTags->"SyntacticHomomorphism"],
Cell[629724, 26611, 79, 1, 39, "ExampleSection",
  CellTags->"SyntacticHomomorphism"],
Cell[629806, 26614, 110, 1, 34, "ExampleText",
  CellTags->"SyntacticHomomorphism"],
Cell[629919, 26617, 113, 2, 28, "Input",
  CellTags->"SyntacticHomomorphism"],
Cell[630035, 26621, 123, 3, 26, "Input",
  CellTags->"SyntacticHomomorphism"],
Cell[630161, 26626, 100, 3, 26, "Input",
  CellTags->"SyntacticHomomorphism"],
Cell[630264, 26631, 113, 3, 26, "Input",
  CellTags->"SyntacticHomomorphism"],
Cell[630380, 26636, 60, 1, 32, "ObjectName",
  CellTags->"SyntacticSG"],
Cell[630443, 26639, 56, 1, 50, "Highlight",
  CellTags->"SyntacticSG"],
Cell[630502, 26642, 167, 6, 31, "Usage",
  CellTags->"SyntacticSG"],
Cell[630672, 26650, 206, 7, 24, "Notes",
  CellTags->"SyntacticSG"],
Cell[630881, 26659, 304, 11, 24, "Notes",
  CellTags->"SyntacticSG"],
Cell[631188, 26672, 69, 1, 39, "ExampleSection",
  CellTags->"SyntacticSG"],
Cell[631260, 26675, 135, 4, 34, "ExampleText",
  CellTags->"SyntacticSG"],
Cell[631398, 26681, 128, 3, 26, "Input",
  CellTags->"SyntacticSG"],
Cell[631529, 26686, 100, 3, 26, "Input",
  CellTags->"SyntacticSG"],
Cell[631632, 26691, 40, 1, 32, "ObjectName",
  CellTags->"T"],
Cell[631675, 26694, 49, 1, 50, "Highlight",
  CellTags->"T"],
Cell[631727, 26697, 134, 4, 31, "Usage",
  CellTags->"T"],
Cell[631864, 26703, 182, 7, 24, "Notes",
  CellTags->"T"],
Cell[632049, 26712, 282, 11, 40, "Notes",
  CellTags->"T"],
Cell[632334, 26725, 587, 27, 24, "Notes",
  CellTags->"T"],
Cell[632924, 26754, 59, 1, 39, "ExampleSection",
  CellTags->"T"],
Cell[632986, 26757, 135, 4, 34, "ExampleText",
  CellTags->"T"],
Cell[633124, 26763, 96, 5, 45, "Input",
  CellTags->"T"],
Cell[633223, 26770, 106, 4, 42, "Input",
  CellTags->"T"],
Cell[633332, 26776, 52, 1, 32, "ObjectName",
  CellTags->"TableSG"],
Cell[633387, 26779, 52, 1, 50, "Highlight",
  CellTags->"TableSG"],
Cell[633442, 26782, 213, 8, 31, "Usage",
  CellTags->"TableSG"],
Cell[633658, 26792, 194, 7, 24, "Notes",
  CellTags->"TableSG"],
Cell[633855, 26801, 92, 1, 24, "Notes",
  CellTags->"TableSG"],
Cell[633950, 26804, 155, 7, 24, "Notes",
  CellTags->"TableSG"],
Cell[634108, 26813, 65, 1, 39, "ExampleSection",
  CellTags->"TableSG"],
Cell[634176, 26816, 117, 4, 34, "ExampleText",
  CellTags->"TableSG"],
Cell[634296, 26822, 152, 4, 26, "Input",
  CellTags->"TableSG"],
Cell[634451, 26828, 97, 3, 26, "Input",
  CellTags->"TableSG"],
Cell[634551, 26833, 70, 1, 32, "ObjectName",
  CellTags->"ToIdentityString"],
Cell[634624, 26836, 61, 1, 50, "Highlight",
  CellTags->"ToIdentityString"],
Cell[634688, 26839, 233, 8, 47, "Usage",
  CellTags->"ToIdentityString"],
Cell[634924, 26849, 289, 11, 24, "Notes",
  CellTags->"ToIdentityString"],
Cell[635216, 26862, 74, 1, 39, "ExampleSection",
  CellTags->"ToIdentityString"],
Cell[635293, 26865, 83, 1, 34, "ExampleText",
  CellTags->"ToIdentityString"],
Cell[635379, 26868, 110, 2, 28, "Input",
  CellTags->"ToIdentityString"],
Cell[635492, 26872, 70, 1, 32, "ObjectName",
  CellTags->"TransformationSG"],
Cell[635565, 26875, 61, 1, 50, "Highlight",
  CellTags->"TransformationSG"],
Cell[635629, 26878, 186, 6, 31, "Usage",
  CellTags->"TransformationSG"],
Cell[635818, 26886, 254, 8, 24, "Notes",
  CellTags->"TransformationSG"],
Cell[636075, 26896, 384, 15, 24, "Notes",
  CellTags->"TransformationSG"],
Cell[636462, 26913, 74, 1, 39, "ExampleSection",
  CellTags->"TransformationSG"],
Cell[636539, 26916, 101, 1, 34, "ExampleText",
  CellTags->"TransformationSG"],
Cell[636643, 26919, 98, 2, 28, "Input",
  CellTags->"TransformationSG"],
Cell[636744, 26923, 106, 3, 26, "Input",
  CellTags->"TransformationSG"],
Cell[636853, 26928, 74, 1, 32, "ObjectName",
  CellTags->"TransientPeriodicT"],
Cell[636930, 26931, 63, 1, 50, "Highlight",
  CellTags->"TransientPeriodicT"],
Cell[636996, 26934, 244, 7, 47, "Usage",
  CellTags->"TransientPeriodicT"],
Cell[637243, 26943, 163, 4, 24, "Notes",
  CellTags->"TransientPeriodicT"],
Cell[637409, 26949, 163, 7, 24, "Notes",
  CellTags->"TransientPeriodicT"],
Cell[637575, 26958, 76, 1, 39, "ExampleSection",
  CellTags->"TransientPeriodicT"],
Cell[637654, 26961, 160, 4, 34, "ExampleText",
  CellTags->"TransientPeriodicT"],
Cell[637817, 26967, 122, 3, 26, "Input",
  CellTags->"TransientPeriodicT"],
Cell[637942, 26972, 129, 3, 26, "Input",
  CellTags->"TransientPeriodicT"],
Cell[638074, 26977, 167, 4, 26, "Input",
  CellTags->"TransientPeriodicT"],
Cell[638244, 26983, 66, 1, 32, "ObjectName",
  CellTags->"TranspositionT"],
Cell[638313, 26986, 59, 1, 50, "Highlight",
  CellTags->"TranspositionT"],
Cell[638375, 26989, 222, 8, 31, "Usage",
  CellTags->"TranspositionT"],
Cell[638600, 26999, 122, 5, 24, "Notes",
  CellTags->"TranspositionT"],
Cell[638725, 27006, 251, 11, 24, "Notes",
  CellTags->"TranspositionT"],
Cell[638979, 27019, 72, 1, 39, "ExampleSection",
  CellTags->"TranspositionT"],
Cell[639054, 27022, 114, 2, 34, "ExampleText",
  CellTags->"TranspositionT"],
Cell[639171, 27026, 93, 2, 28, "Input",
  CellTags->"TranspositionT"],
Cell[639267, 27030, 127, 2, 28, "Input",
  CellTags->"TranspositionT"],
Cell[639397, 27034, 105, 3, 26, "Input",
  CellTags->"TranspositionT"],
Cell[639505, 27039, 60, 1, 32, "ObjectName",
  CellTags->"UnitElement"],
Cell[639568, 27042, 56, 1, 50, "Highlight",
  CellTags->"UnitElement"],
Cell[639627, 27045, 195, 6, 31, "Usage",
  CellTags->"UnitElement"],
Cell[639825, 27053, 210, 6, 40, "Notes",
  CellTags->"UnitElement"],
Cell[640038, 27061, 118, 5, 24, "Notes",
  CellTags->"UnitElement"],
Cell[640159, 27068, 159, 7, 24, "Notes",
  CellTags->"UnitElement"],
Cell[640321, 27077, 69, 1, 39, "ExampleSection",
  CellTags->"UnitElement"],
Cell[640393, 27080, 69, 1, 34, "ExampleText",
  CellTags->"UnitElement"],
Cell[640465, 27083, 91, 2, 28, "Input",
  CellTags->"UnitElement"],
Cell[640559, 27087, 96, 2, 28, "Input",
  CellTags->"UnitElement"],
Cell[640658, 27091, 72, 1, 32, "ObjectName",
  CellTags->"$rewriteSimpBound"],
Cell[640733, 27094, 65, 1, 50, "Highlight",
  CellTags->"$rewriteSimpBound"],
Cell[640801, 27097, 225, 5, 47, "Usage",
  CellTags->"$rewriteSimpBound"],
Cell[641029, 27104, 71, 1, 24, "Notes",
  CellTags->"$rewriteSimpBound"],
Cell[641103, 27107, 187, 7, 24, "Notes",
  CellTags->"$rewriteSimpBound"],
Cell[641293, 27116, 75, 1, 39, "ExampleSection",
  CellTags->"$rewriteSimpBound"],
Cell[641371, 27119, 88, 1, 34, "ExampleText",
  CellTags->"$rewriteSimpBound"],
Cell[641462, 27122, 110, 3, 26, "Input",
  CellTags->"$rewriteSimpBound"],
Cell[641575, 27127, 70, 1, 32, "ObjectName",
  CellTags->"AllStringReplace"],
Cell[641648, 27130, 69, 1, 50, "Highlight",
  CellTags->"AllStringReplace"],
Cell[641720, 27133, 256, 8, 47, "Usage",
  CellTags->"AllStringReplace"],
Cell[641979, 27143, 247, 7, 40, "Notes",
  CellTags->"AllStringReplace"],
Cell[642229, 27152, 124, 5, 24, "Notes",
  CellTags->"AllStringReplace"],
Cell[642356, 27159, 159, 7, 24, "Notes",
  CellTags->"AllStringReplace"],
Cell[642518, 27168, 74, 1, 39, "ExampleSection",
  CellTags->"AllStringReplace"],
Cell[642595, 27171, 103, 1, 34, "ExampleText",
  CellTags->"AllStringReplace"],
Cell[642701, 27174, 114, 2, 28, "Input",
  CellTags->"AllStringReplace"],
Cell[642818, 27178, 147, 3, 26, "Input",
  CellTags->"AllStringReplace"],
Cell[642968, 27183, 141, 3, 26, "Input",
  CellTags->"AllStringReplace"],
Cell[643112, 27188, 54, 1, 32, "ObjectName",
  CellTags->"BoundETW"],
Cell[643169, 27191, 51, 1, 50, "Highlight",
  CellTags->"BoundETW"],
Cell[643223, 27194, 156, 6, 31, "Usage",
  CellTags->"BoundETW"],
Cell[643382, 27202, 56, 1, 24, "Notes",
  CellTags->"BoundETW"],
Cell[643441, 27205, 176, 7, 24, "Notes",
  CellTags->"BoundETW"],
Cell[643620, 27214, 52, 1, 32, "ObjectName",
  CellTags->"BoundWR"],
Cell[643675, 27217, 50, 1, 50, "Highlight",
  CellTags->"BoundWR"],
Cell[643728, 27220, 182, 6, 31, "Usage",
  CellTags->"BoundWR"],
Cell[643913, 27228, 55, 1, 24, "Notes",
  CellTags->"BoundWR"],
Cell[643971, 27231, 155, 7, 24, "Notes",
  CellTags->"BoundWR"],
Cell[644129, 27240, 46, 1, 32, "ObjectName",
  CellTags->"Cols"],
Cell[644178, 27243, 47, 1, 50, "Highlight",
  CellTags->"Cols"],
Cell[644228, 27246, 137, 6, 31, "Usage",
  CellTags->"Cols"],
Cell[644368, 27254, 160, 7, 24, "Notes",
  CellTags->"Cols"],
Cell[644531, 27263, 78, 1, 32, "ObjectName",
  CellTags->"EquationsToWitnesses"],
Cell[644612, 27266, 65, 1, 50, "Highlight",
  CellTags->"EquationsToWitnesses"],
Cell[644680, 27269, 285, 8, 47, "Usage",
  CellTags->"EquationsToWitnesses"],
Cell[644968, 27279, 283, 10, 61, "Notes",
  CellTags->"EquationsToWitnesses"],
Cell[645254, 27291, 217, 7, 24, "Notes",
  CellTags->"EquationsToWitnesses"],
Cell[645474, 27300, 185, 4, 40, "Notes",
  CellTags->"EquationsToWitnesses"],
Cell[645662, 27306, 173, 7, 24, "Notes",
  CellTags->"EquationsToWitnesses"],
Cell[645838, 27315, 78, 1, 39, "ExampleSection",
  CellTags->"EquationsToWitnesses"],
Cell[645919, 27318, 194, 6, 34, "ExampleText",
  CellTags->"EquationsToWitnesses"],
Cell[646116, 27326, 161, 4, 26, "Input",
  CellTags->"EquationsToWitnesses"],
Cell[646280, 27332, 180, 4, 26, "Input",
  CellTags->"EquationsToWitnesses"],
Cell[646463, 27338, 391, 12, 34, "ExampleText",
  CellTags->"EquationsToWitnesses"],
Cell[646857, 27352, 194, 4, 42, "Input",
  CellTags->"EquationsToWitnesses"],
Cell[647054, 27358, 137, 4, 34, "ExampleText",
  CellTags->"EquationsToWitnesses"],
Cell[647194, 27364, 203, 4, 42, "Input",
  CellTags->"EquationsToWitnesses"],
Cell[647400, 27370, 109, 3, 26, "Input",
  CellTags->"EquationsToWitnesses"],
Cell[647512, 27375, 56, 1, 32, "ObjectName",
  CellTags->"Equations"],
Cell[647571, 27378, 52, 1, 50, "Highlight",
  CellTags->"Equations"],
Cell[647626, 27381, 126, 4, 31, "Usage",
  CellTags->"Equations"],
Cell[647755, 27387, 255, 11, 24, "Notes",
  CellTags->"Equations"],
Cell[648013, 27400, 74, 1, 32, "ObjectName",
  CellTags->"FromIdentityString"],
Cell[648090, 27403, 63, 1, 50, "Highlight",
  CellTags->"FromIdentityString"],
Cell[648156, 27406, 236, 8, 47, "Usage",
  CellTags->"FromIdentityString"],
Cell[648395, 27416, 125, 4, 24, "Notes",
  CellTags->"FromIdentityString"],
Cell[648523, 27422, 147, 2, 24, "Notes",
  CellTags->"FromIdentityString"],
Cell[648673, 27426, 1040, 39, 56, "Notes",
  CellTags->"FromIdentityString"],
Cell[649716, 27467, 76, 1, 39, "ExampleSection",
  CellTags->"FromIdentityString"],
Cell[649795, 27470, 106, 1, 34, "ExampleText",
  CellTags->"FromIdentityString"],
Cell[649904, 27473, 117, 2, 28, "Input",
  CellTags->"FromIdentityString"],
Cell[650024, 27477, 97, 2, 28, "Input",
  CellTags->"FromIdentityString"],
Cell[650124, 27481, 117, 2, 28, "Input",
  CellTags->"FromIdentityString"],
Cell[650244, 27485, 62, 1, 32, "ObjectName",
  CellTags->"IrreducibleQ"],
Cell[650309, 27488, 57, 1, 50, "Highlight",
  CellTags->"IrreducibleQ"],
Cell[650369, 27491, 228, 8, 31, "Usage",
  CellTags->"IrreducibleQ"],
Cell[650600, 27501, 139, 5, 24, "Notes",
  CellTags->"IrreducibleQ"],
Cell[650742, 27508, 275, 11, 24, "Notes",
  CellTags->"IrreducibleQ"],
Cell[651020, 27521, 70, 1, 39, "ExampleSection",
  CellTags->"IrreducibleQ"],
Cell[651093, 27524, 74, 1, 34, "ExampleText",
  CellTags->"IrreducibleQ"],
Cell[651170, 27527, 127, 3, 26, "Input",
  CellTags->"IrreducibleQ"],
Cell[651300, 27532, 118, 3, 26, "Input",
  CellTags->"IrreducibleQ"],
Cell[651421, 27537, 91, 2, 26, "Input",
  CellTags->"IrreducibleQ"],
Cell[651515, 27541, 62, 1, 32, "ObjectName",
  CellTags->"PairsToRules"],
Cell[651580, 27544, 57, 1, 50, "Highlight",
  CellTags->"PairsToRules"],
Cell[651640, 27547, 130, 4, 31, "Usage",
  CellTags->"PairsToRules"],
Cell[651773, 27553, 565, 19, 24, "Notes",
  CellTags->"PairsToRules"],
Cell[652341, 27574, 70, 1, 39, "ExampleSection",
  CellTags->"PairsToRules"],
Cell[652414, 27577, 103, 1, 34, "ExampleText",
  CellTags->"PairsToRules"],
Cell[652520, 27580, 211, 4, 42, "Input",
  CellTags->"PairsToRules"],
Cell[652734, 27586, 118, 3, 26, "Input",
  CellTags->"PairsToRules"],
Cell[652855, 27591, 62, 1, 32, "ObjectName",
  CellTags->"RulesToPairs"],
Cell[652920, 27594, 57, 1, 50, "Highlight",
  CellTags->"RulesToPairs"],
Cell[652980, 27597, 138, 4, 31, "Usage",
  CellTags->"RulesToPairs"],
Cell[653121, 27603, 565, 19, 24, "Notes",
  CellTags->"RulesToPairs"],
Cell[653689, 27624, 70, 1, 39, "ExampleSection",
  CellTags->"RulesToPairs"],
Cell[653762, 27627, 100, 1, 34, "ExampleText",
  CellTags->"RulesToPairs"],
Cell[653865, 27630, 241, 5, 42, "Input",
  CellTags->"RulesToPairs"],
Cell[654109, 27637, 102, 3, 26, "Input",
  CellTags->"RulesToPairs"],
Cell[654214, 27642, 72, 1, 32, "ObjectName",
  CellTags->"SimplifyEquations"],
Cell[654289, 27645, 62, 1, 50, "Highlight",
  CellTags->"SimplifyEquations"],
Cell[654354, 27648, 226, 6, 47, "Usage",
  CellTags->"SimplifyEquations"],
Cell[654583, 27656, 232, 9, 44, "Notes",
  CellTags->"SimplifyEquations"],
Cell[654818, 27667, 275, 9, 40, "Notes",
  CellTags->"SimplifyEquations"],
Cell[655096, 27678, 288, 11, 24, "Notes",
  CellTags->"SimplifyEquations"],
Cell[655387, 27691, 75, 1, 39, "ExampleSection",
  CellTags->"SimplifyEquations"],
Cell[655465, 27694, 141, 4, 34, "ExampleText",
  CellTags->"SimplifyEquations"],
Cell[655609, 27700, 161, 4, 26, "Input",
  CellTags->"SimplifyEquations"],
Cell[655773, 27706, 107, 3, 26, "Input",
  CellTags->"SimplifyEquations"],
Cell[655883, 27711, 140, 3, 26, "Input",
  CellTags->"SimplifyEquations"],
Cell[656026, 27716, 80, 1, 32, "ObjectName",
  CellTags->"SimplifyRewriteSystem"],
Cell[656109, 27719, 66, 1, 50, "Highlight",
  CellTags->"SimplifyRewriteSystem"],
Cell[656178, 27722, 211, 6, 47, "Usage",
  CellTags->"SimplifyRewriteSystem"],
Cell[656392, 27730, 240, 9, 44, "Notes",
  CellTags->"SimplifyRewriteSystem"],
Cell[656635, 27741, 254, 6, 40, "Notes",
  CellTags->"SimplifyRewriteSystem"],
Cell[656892, 27749, 218, 6, 40, "Notes",
  CellTags->"SimplifyRewriteSystem"],
Cell[657113, 27757, 188, 7, 24, "Notes",
  CellTags->"SimplifyRewriteSystem"],
Cell[657304, 27766, 79, 1, 39, "ExampleSection",
  CellTags->"SimplifyRewriteSystem"],
Cell[657386, 27769, 199, 5, 50, "ExampleText",
  CellTags->"SimplifyRewriteSystem"],
Cell[657588, 27776, 165, 4, 26, "Input",
  CellTags->"SimplifyRewriteSystem"],
Cell[657756, 27782, 111, 3, 26, "Input",
  CellTags->"SimplifyRewriteSystem"],
Cell[657870, 27787, 135, 3, 26, "Input",
  CellTags->"SimplifyRewriteSystem"],
Cell[658008, 27792, 114, 3, 26, "Input",
  CellTags->"SimplifyRewriteSystem"],
Cell[658125, 27797, 58, 1, 32, "ObjectName",
  CellTags->"WordReduce"],
Cell[658186, 27800, 55, 1, 50, "Highlight",
  CellTags->"WordReduce"],
Cell[658244, 27803, 201, 8, 31, "Usage",
  CellTags->"WordReduce"],
Cell[658448, 27813, 275, 10, 61, "Notes",
  CellTags->"WordReduce"],
Cell[658726, 27825, 288, 8, 40, "Notes",
  CellTags->"WordReduce"],
Cell[659017, 27835, 147, 5, 24, "Notes",
  CellTags->"WordReduce"],
Cell[659167, 27842, 121, 4, 24, "Notes",
  CellTags->"WordReduce"],
Cell[659291, 27848, 295, 11, 24, "Notes",
  CellTags->"WordReduce"],
Cell[659589, 27861, 68, 1, 39, "ExampleSection",
  CellTags->"WordReduce"],
Cell[659660, 27864, 72, 1, 34, "ExampleText",
  CellTags->"WordReduce"],
Cell[659735, 27867, 142, 3, 26, "Input",
  CellTags->"WordReduce"],
Cell[659880, 27872, 123, 3, 26, "Input",
  CellTags->"WordReduce"],
Cell[660006, 27877, 152, 4, 26, "Input",
  CellTags->"WordReduce"],
Cell[660161, 27883, 99, 1, 34, "ExampleText",
  CellTags->"WordReduce"],
Cell[660263, 27886, 155, 4, 26, "Input",
  CellTags->"WordReduce"],
Cell[660421, 27892, 72, 1, 32, "ObjectName",
  CellTags->"BooleanComplement"],
Cell[660496, 27895, 62, 1, 50, "Highlight",
  CellTags->"BooleanComplement"],
Cell[660561, 27898, 324, 9, 47, "Usage",
  CellTags->"BooleanComplement"],
Cell[660888, 27909, 155, 4, 24, "Notes",
  CellTags->"BooleanComplement"],
Cell[661046, 27915, 818, 27, 40, "Notes",
  CellTags->"BooleanComplement"],
Cell[661867, 27944, 75, 1, 39, "ExampleSection",
  CellTags->"BooleanComplement"],
Cell[661945, 27947, 99, 1, 34, "ExampleText",
  CellTags->"BooleanComplement"],
Cell[662047, 27950, 146, 5, 45, "Input",
  CellTags->"BooleanComplement"],
Cell[662196, 27957, 99, 2, 28, "Input",
  CellTags->"BooleanComplement"],
Cell[662298, 27961, 96, 2, 28, "Input",
  CellTags->"BooleanComplement"],
Cell[662397, 27965, 74, 1, 32, "ObjectName",
  CellTags->"BooleanComposition"],
Cell[662474, 27968, 63, 1, 50, "Highlight",
  CellTags->"BooleanComposition"],
Cell[662540, 27971, 281, 8, 47, "Usage",
  CellTags->"BooleanComposition"],
Cell[662824, 27981, 156, 4, 24, "Notes",
  CellTags->"BooleanComposition"],
Cell[662983, 27987, 817, 27, 40, "Notes",
  CellTags->"BooleanComposition"],
Cell[663803, 28016, 76, 1, 39, "ExampleSection",
  CellTags->"BooleanComposition"],
Cell[663882, 28019, 199, 6, 34, "ExampleText",
  CellTags->"BooleanComposition"],
Cell[664084, 28027, 111, 2, 28, "Input",
  CellTags->"BooleanComposition"],
Cell[664198, 28031, 134, 4, 34, "ExampleText",
  CellTags->"BooleanComposition"],
Cell[664335, 28037, 165, 4, 42, "Input",
  CellTags->"BooleanComposition"],
Cell[664503, 28043, 174, 4, 42, "Input",
  CellTags->"BooleanComposition"],
Cell[664680, 28049, 68, 1, 32, "ObjectName",
  CellTags->"BooleanConverse"],
Cell[664751, 28052, 60, 1, 50, "Highlight",
  CellTags->"BooleanConverse"],
Cell[664814, 28055, 218, 6, 47, "Usage",
  CellTags->"BooleanConverse"],
Cell[665035, 28063, 231, 7, 24, "Notes",
  CellTags->"BooleanConverse"],
Cell[665269, 28072, 153, 4, 24, "Notes",
  CellTags->"BooleanConverse"],
Cell[665425, 28078, 611, 21, 40, "Notes",
  CellTags->"BooleanConverse"],
Cell[666039, 28101, 73, 1, 39, "ExampleSection",
  CellTags->"BooleanConverse"],
Cell[666115, 28104, 214, 6, 34, "ExampleText",
  CellTags->"BooleanConverse"],
Cell[666332, 28112, 134, 5, 45, "Input",
  CellTags->"BooleanConverse"],
Cell[666469, 28119, 106, 2, 28, "Input",
  CellTags->"BooleanConverse"],
Cell[666578, 28123, 76, 1, 32, "ObjectName",
  CellTags->"BooleanIntersection"],
Cell[666657, 28126, 64, 1, 50, "Highlight",
  CellTags->"BooleanIntersection"],
Cell[666724, 28129, 292, 8, 47, "Usage",
  CellTags->"BooleanIntersection"],
Cell[667019, 28139, 157, 4, 24, "Notes",
  CellTags->"BooleanIntersection"],
Cell[667179, 28145, 816, 27, 40, "Notes",
  CellTags->"BooleanIntersection"],
Cell[667998, 28174, 77, 1, 39, "ExampleSection",
  CellTags->"BooleanIntersection"],
Cell[668078, 28177, 143, 4, 34, "ExampleText",
  CellTags->"BooleanIntersection"],
Cell[668224, 28183, 144, 5, 45, "Input",
  CellTags->"BooleanIntersection"],
Cell[668371, 28190, 147, 5, 45, "Input",
  CellTags->"BooleanIntersection"],
Cell[668521, 28197, 117, 2, 28, "Input",
  CellTags->"BooleanIntersection"],
Cell[668641, 28201, 62, 1, 32, "ObjectName",
  CellTags->"BooleanPower"],
Cell[668706, 28204, 57, 1, 50, "Highlight",
  CellTags->"BooleanPower"],
Cell[668766, 28207, 246, 8, 47, "Usage",
  CellTags->"BooleanPower"],
Cell[669015, 28217, 150, 4, 24, "Notes",
  CellTags->"BooleanPower"],
Cell[669168, 28223, 823, 27, 40, "Notes",
  CellTags->"BooleanPower"],
Cell[669994, 28252, 70, 1, 39, "ExampleSection",
  CellTags->"BooleanPower"],
Cell[670067, 28255, 193, 6, 34, "ExampleText",
  CellTags->"BooleanPower"],
Cell[670263, 28263, 105, 2, 28, "Input",
  CellTags->"BooleanPower"],
Cell[670371, 28267, 126, 4, 34, "ExampleText",
  CellTags->"BooleanPower"],
Cell[670500, 28273, 153, 4, 42, "Input",
  CellTags->"BooleanPower"],
Cell[670656, 28279, 62, 1, 32, "ObjectName",
  CellTags->"BooleanUnion"],
Cell[670721, 28282, 57, 1, 50, "Highlight",
  CellTags->"BooleanUnion"],
Cell[670781, 28285, 266, 8, 47, "Usage",
  CellTags->"BooleanUnion"],
Cell[671050, 28295, 150, 4, 24, "Notes",
  CellTags->"BooleanUnion"],
Cell[671203, 28301, 823, 27, 40, "Notes",
  CellTags->"BooleanUnion"],
Cell[672029, 28330, 70, 1, 39, "ExampleSection",
  CellTags->"BooleanUnion"],
Cell[672102, 28333, 135, 4, 34, "ExampleText",
  CellTags->"BooleanUnion"],
Cell[672240, 28339, 131, 5, 45, "Input",
  CellTags->"BooleanUnion"],
Cell[672374, 28346, 134, 5, 45, "Input",
  CellTags->"BooleanUnion"],
Cell[672511, 28353, 104, 2, 28, "Input",
  CellTags->"BooleanUnion"],
Cell[672618, 28357, 52, 1, 32, "ObjectName",
  CellTags->"Boolean"],
Cell[672673, 28360, 50, 1, 50, "Highlight",
  CellTags->"Boolean"],
Cell[672726, 28363, 242, 7, 47, "Usage",
  CellTags->"Boolean"],
Cell[672971, 28372, 145, 4, 24, "Notes",
  CellTags->"Boolean"],
Cell[673119, 28378, 216, 9, 24, "Notes",
  CellTags->"Boolean"],
Cell[673338, 28389, 65, 1, 39, "ExampleSection",
  CellTags->"Boolean"],
Cell[673406, 28392, 72, 1, 34, "ExampleText",
  CellTags->"Boolean"],
Cell[673481, 28395, 107, 3, 26, "Input",
  CellTags->"Boolean"],
Cell[673591, 28400, 127, 3, 26, "Input",
  CellTags->"Boolean"],
Cell[673721, 28405, 83, 2, 28, "Input",
  CellTags->"Boolean"],
Cell[673807, 28409, 86, 2, 28, "Input",
  CellTags->"Boolean"],
Cell[673896, 28413, 70, 1, 32, "ObjectName",
  CellTags->"RelationToMatrix"],
Cell[673969, 28416, 61, 1, 50, "Highlight",
  CellTags->"RelationToMatrix"],
Cell[674033, 28419, 245, 8, 47, "Usage",
  CellTags->"RelationToMatrix"],
Cell[674281, 28429, 199, 7, 24, "Notes",
  CellTags->"RelationToMatrix"],
Cell[674483, 28438, 108, 1, 24, "Notes",
  CellTags->"RelationToMatrix"],
Cell[674594, 28441, 830, 31, 40, "Notes",
  CellTags->"RelationToMatrix"],
Cell[675427, 28474, 74, 1, 39, "ExampleSection",
  CellTags->"RelationToMatrix"],
Cell[675504, 28477, 86, 1, 34, "ExampleText",
  CellTags->"RelationToMatrix"],
Cell[675593, 28480, 122, 2, 28, "Input",
  CellTags->"RelationToMatrix"],
Cell[675718, 28484, 56, 1, 32, "ObjectName",
  CellTags->"ToClasses"],
Cell[675777, 28487, 54, 1, 50, "Highlight",
  CellTags->"ToClasses"],
Cell[675834, 28490, 236, 8, 47, "Usage",
  CellTags->"ToClasses"],
Cell[676073, 28500, 252, 9, 44, "Notes",
  CellTags->"ToClasses"],
Cell[676328, 28511, 337, 11, 40, "Notes",
  CellTags->"ToClasses"],
Cell[676668, 28524, 181, 7, 24, "Notes",
  CellTags->"ToClasses"],
Cell[676852, 28533, 263, 11, 24, "Notes",
  CellTags->"ToClasses"],
Cell[677118, 28546, 67, 1, 39, "ExampleSection",
  CellTags->"ToClasses"],
Cell[677188, 28549, 124, 4, 34, "ExampleText",
  CellTags->"ToClasses"],
Cell[677315, 28555, 139, 3, 26, "Input",
  CellTags->"ToClasses"],
Cell[677457, 28560, 118, 3, 26, "Input",
  CellTags->"ToClasses"],
Cell[677578, 28565, 90, 1, 34, "ExampleText",
  CellTags->"ToClasses"],
Cell[677671, 28568, 127, 3, 26, "Input",
  CellTags->"ToClasses"],
Cell[677801, 28573, 136, 3, 26, "Input",
  CellTags->"ToClasses"],
Cell[677940, 28578, 76, 1, 34, "ExampleText",
  CellTags->"ToClasses"],
Cell[678019, 28581, 111, 3, 26, "Input",
  CellTags->"ToClasses"],
Cell[678133, 28586, 138, 3, 26, "Input",
  CellTags->"ToClasses"],
Cell[678274, 28591, 66, 1, 34, "ExampleText",
  CellTags->"ToClasses"],
Cell[678343, 28594, 135, 3, 26, "Input",
  CellTags->"ToClasses"],
Cell[678481, 28599, 137, 3, 26, "Input",
  CellTags->"ToClasses"],
Cell[678621, 28604, 72, 1, 32, "ObjectName",
  CellTags->"TransitiveClosure"],
Cell[678696, 28607, 62, 1, 50, "Highlight",
  CellTags->"TransitiveClosure"],
Cell[678761, 28610, 193, 6, 31, "Usage",
  CellTags->"TransitiveClosure"],
Cell[678957, 28618, 238, 9, 44, "Notes",
  CellTags->"TransitiveClosure"],
Cell[679198, 28629, 211, 6, 40, "Notes",
  CellTags->"TransitiveClosure"],
Cell[679412, 28637, 424, 11, 40, "Notes",
  CellTags->"TransitiveClosure"],
Cell[679839, 28650, 560, 19, 40, "Notes",
  CellTags->"TransitiveClosure"],
Cell[680402, 28671, 104, 1, 24, "Notes",
  CellTags->"TransitiveClosure"],
Cell[680509, 28674, 402, 15, 24, "Notes",
  CellTags->"TransitiveClosure"],
Cell[680914, 28691, 75, 1, 39, "ExampleSection",
  CellTags->"TransitiveClosure"],
Cell[680992, 28694, 91, 1, 34, "ExampleText",
  CellTags->"TransitiveClosure"],
Cell[681086, 28697, 347, 6, 74, "Input",
  CellTags->"TransitiveClosure"],
Cell[681436, 28705, 110, 3, 26, "Input",
  CellTags->"TransitiveClosure"],
Cell[681549, 28710, 127, 3, 26, "Input",
  CellTags->"TransitiveClosure"],
Cell[681679, 28715, 54, 1, 32, "ObjectName",
  CellTags->"Warshall"],
Cell[681736, 28718, 53, 1, 50, "Highlight",
  CellTags->"Warshall"],
Cell[681792, 28721, 200, 6, 31, "Usage",
  CellTags->"Warshall"],
Cell[681995, 28729, 383, 14, 64, "Notes",
  CellTags->"Warshall"],
Cell[682381, 28745, 126, 4, 24, "Notes",
  CellTags->"Warshall"],
Cell[682510, 28751, 164, 5, 24, "Notes",
  CellTags->"Warshall"],
Cell[682677, 28758, 256, 7, 40, "Notes",
  CellTags->"Warshall"],
Cell[682936, 28767, 120, 4, 24, "Notes",
  CellTags->"Warshall"],
Cell[683059, 28773, 265, 11, 24, "Notes",
  CellTags->"Warshall"],
Cell[683327, 28786, 66, 1, 39, "ExampleSection",
  CellTags->"Warshall"],
Cell[683396, 28789, 178, 6, 34, "ExampleText",
  CellTags->"Warshall"],
Cell[683577, 28797, 135, 5, 45, "Input",
  CellTags->"Warshall"],
Cell[683715, 28804, 110, 3, 26, "Input",
  CellTags->"Warshall"],
Cell[683828, 28809, 136, 3, 26, "Input",
  CellTags->"Warshall"],
Cell[683967, 28814, 146, 4, 34, "ExampleText",
  CellTags->"Warshall"],
Cell[684116, 28820, 475, 11, 111, "Input",
  CellTags->"Warshall"],
Cell[684594, 28833, 127, 4, 34, "ExampleText",
  CellTags->"Warshall"],
Cell[684724, 28839, 98, 3, 26, "Input",
  CellTags->"Warshall"],
Cell[684825, 28844, 62, 1, 32, "ObjectName",
  CellTags->"AnalyzeOrbit"],
Cell[684890, 28847, 57, 1, 50, "Highlight",
  CellTags->"AnalyzeOrbit"],
Cell[684950, 28850, 472, 15, 63, "Usage",
  CellTags->"AnalyzeOrbit"],
Cell[685425, 28867, 339, 11, 79, "Notes",
  CellTags->"AnalyzeOrbit"],
Cell[685767, 28880, 250, 7, 40, "Notes",
  CellTags->"AnalyzeOrbit"],
Cell[686020, 28889, 256, 6, 40, "Notes",
  CellTags->"AnalyzeOrbit"],
Cell[686279, 28897, 236, 8, 24, "Notes",
  CellTags->"AnalyzeOrbit"],
Cell[686518, 28907, 138, 4, 24, "Notes",
  CellTags->"AnalyzeOrbit"],
Cell[686659, 28913, 261, 9, 40, "Notes",
  CellTags->"AnalyzeOrbit"],
Cell[686923, 28924, 239, 11, 24, "Notes",
  CellTags->"AnalyzeOrbit"],
Cell[687165, 28937, 70, 1, 39, "ExampleSection",
  CellTags->"AnalyzeOrbit"],
Cell[687238, 28940, 84, 1, 34, "ExampleText",
  CellTags->"AnalyzeOrbit"],
Cell[687325, 28943, 110, 5, 45, "Input",
  CellTags->"AnalyzeOrbit"],
Cell[687438, 28950, 104, 3, 26, "Input",
  CellTags->"AnalyzeOrbit"],
Cell[687545, 28955, 104, 3, 26, "Input",
  CellTags->"AnalyzeOrbit"],
Cell[687652, 28960, 124, 3, 26, "Input",
  CellTags->"AnalyzeOrbit"],
Cell[687779, 28965, 125, 3, 26, "Input",
  CellTags->"AnalyzeOrbit"],
Cell[687907, 28970, 159, 4, 50, "ExampleText",
  CellTags->"AnalyzeOrbit"],
Cell[688069, 28976, 207, 5, 58, "Input",
  CellTags->"AnalyzeOrbit"],
Cell[688279, 28983, 169, 4, 26, "Input",
  CellTags->"AnalyzeOrbit"],
Cell[688451, 28989, 215, 5, 50, "ExampleText",
  CellTags->"AnalyzeOrbit"],
Cell[688669, 28996, 110, 3, 26, "Input",
  CellTags->"AnalyzeOrbit"],
Cell[688782, 29001, 149, 5, 34, "ExampleText",
  CellTags->"AnalyzeOrbit"],
Cell[688934, 29008, 241, 6, 74, "Input",
  CellTags->"AnalyzeOrbit"],
Cell[689178, 29016, 52, 1, 32, "ObjectName",
  CellTags->"BoundAO"],
Cell[689233, 29019, 50, 1, 50, "Highlight",
  CellTags->"BoundAO"],
Cell[689286, 29022, 184, 6, 31, "Usage",
  CellTags->"BoundAO"],
Cell[689473, 29030, 62, 1, 24, "Notes",
  CellTags->"BoundAO"],
Cell[689538, 29033, 159, 7, 24, "Notes",
  CellTags->"BoundAO"],
Cell[689700, 29042, 70, 1, 32, "ObjectName",
  CellTags->"CartesianProduct"],
Cell[689773, 29045, 61, 1, 50, "Highlight",
  CellTags->"CartesianProduct"],
Cell[689837, 29048, 214, 8, 31, "Usage",
  CellTags->"CartesianProduct"],
Cell[690054, 29058, 116, 4, 24, "Notes",
  CellTags->"CartesianProduct"],
Cell[690173, 29064, 233, 8, 26, "Notes",
  CellTags->"CartesianProduct"],
Cell[690409, 29074, 162, 2, 24, "Notes",
  CellTags->"CartesianProduct"],
Cell[690574, 29078, 357, 13, 24, "Notes",
  CellTags->"CartesianProduct"],
Cell[690934, 29093, 74, 1, 39, "ExampleSection",
  CellTags->"CartesianProduct"],
Cell[691011, 29096, 83, 1, 34, "ExampleText",
  CellTags->"CartesianProduct"],
Cell[691097, 29099, 129, 5, 45, "Input",
  CellTags->"CartesianProduct"],
Cell[691229, 29106, 111, 1, 34, "ExampleText",
  CellTags->"CartesianProduct"],
Cell[691343, 29109, 112, 2, 28, "Input",
  CellTags->"CartesianProduct"],
Cell[691458, 29113, 186, 5, 50, "ExampleText",
  CellTags->"CartesianProduct"],
Cell[691647, 29120, 136, 5, 45, "Input",
  CellTags->"CartesianProduct"],
Cell[691786, 29127, 64, 1, 32, "ObjectName",
  CellTags->"$closureBound"],
Cell[691853, 29130, 61, 1, 50, "Highlight",
  CellTags->"$closureBound"],
Cell[691917, 29133, 181, 5, 31, "Usage",
  CellTags->"$closureBound"],
Cell[692101, 29140, 105, 1, 24, "Notes",
  CellTags->"$closureBound"],
Cell[692209, 29143, 155, 7, 24, "Notes",
  CellTags->"$closureBound"],
Cell[692367, 29152, 71, 1, 39, "ExampleSection",
  CellTags->"$closureBound"],
Cell[692441, 29155, 84, 1, 34, "ExampleText",
  CellTags->"$closureBound"],
Cell[692528, 29158, 100, 3, 26, "Input",
  CellTags->"$closureBound"],
Cell[692631, 29163, 62, 1, 32, "ObjectName",
  CellTags->"ClosureBound"],
Cell[692696, 29166, 55, 1, 50, "Highlight",
  CellTags->"ClosureBound"],
Cell[692754, 29169, 196, 8, 31, "Usage",
  CellTags->"ClosureBound"],
Cell[692953, 29179, 259, 11, 24, "Notes",
  CellTags->"ClosureBound"],
Cell[693215, 29192, 76, 1, 32, "ObjectName",
  CellTags->"$closureSearchBound"],
Cell[693294, 29195, 67, 1, 50, "Highlight",
  CellTags->"$closureSearchBound"],
Cell[693364, 29198, 178, 6, 31, "Usage",
  CellTags->"$closureSearchBound"],
Cell[693545, 29206, 126, 4, 24, "Notes",
  CellTags->"$closureSearchBound"],
Cell[693674, 29212, 173, 7, 24, "Notes",
  CellTags->"$closureSearchBound"],
Cell[693850, 29221, 77, 1, 39, "ExampleSection",
  CellTags->"$closureSearchBound"],
Cell[693930, 29224, 90, 1, 34, "ExampleText",
  CellTags->"$closureSearchBound"],
Cell[694023, 29227, 112, 3, 26, "Input",
  CellTags->"$closureSearchBound"],
Cell[694138, 29232, 64, 1, 32, "ObjectName",
  CellTags->"ClosureSearch"],
Cell[694205, 29235, 58, 1, 50, "Highlight",
  CellTags->"ClosureSearch"],
Cell[694266, 29238, 272, 10, 47, "Usage",
  CellTags->"ClosureSearch"],
Cell[694541, 29250, 309, 10, 61, "Notes",
  CellTags->"ClosureSearch"],
Cell[694853, 29262, 135, 5, 24, "Notes",
  CellTags->"ClosureSearch"],
Cell[694991, 29269, 328, 10, 40, "Notes",
  CellTags->"ClosureSearch"],
Cell[695322, 29281, 182, 4, 40, "Notes",
  CellTags->"ClosureSearch"],
Cell[695507, 29287, 406, 15, 24, "Notes",
  CellTags->"ClosureSearch"],
Cell[695916, 29304, 71, 1, 39, "ExampleSection",
  CellTags->"ClosureSearch"],
Cell[695990, 29307, 107, 1, 34, "ExampleText",
  CellTags->"ClosureSearch"],
Cell[696100, 29310, 179, 4, 42, "Input",
  CellTags->"ClosureSearch"],
Cell[696282, 29316, 135, 3, 26, "Input",
  CellTags->"ClosureSearch"],
Cell[696420, 29321, 85, 1, 34, "ExampleText",
  CellTags->"ClosureSearch"],
Cell[696508, 29324, 187, 4, 42, "Input",
  CellTags->"ClosureSearch"],
Cell[696698, 29330, 188, 4, 42, "Input",
  CellTags->"ClosureSearch"],
Cell[696889, 29336, 52, 1, 32, "ObjectName",
  CellTags->"Closure"],
Cell[696944, 29339, 52, 1, 50, "Highlight",
  CellTags->"Closure"],
Cell[696999, 29342, 147, 4, 31, "Usage",
  CellTags->"Closure"],
Cell[697149, 29348, 475, 14, 133, "Notes",
  CellTags->"Closure"],
Cell[697627, 29364, 129, 5, 24, "Notes",
  CellTags->"Closure"],
Cell[697759, 29371, 322, 10, 40, "Notes",
  CellTags->"Closure"],
Cell[698084, 29383, 258, 9, 40, "Notes",
  CellTags->"Closure"],
Cell[698345, 29394, 185, 6, 24, "Notes",
  CellTags->"Closure"],
Cell[698533, 29402, 174, 5, 24, "Notes",
  CellTags->"Closure"],
Cell[698710, 29409, 132, 5, 24, "Notes",
  CellTags->"Closure"],
Cell[698845, 29416, 412, 15, 24, "Notes",
  CellTags->"Closure"],
Cell[699260, 29433, 65, 1, 39, "ExampleSection",
  CellTags->"Closure"],
Cell[699328, 29436, 83, 1, 34, "ExampleText",
  CellTags->"Closure"],
Cell[699414, 29439, 101, 2, 28, "Input",
  CellTags->"Closure"],
Cell[699518, 29443, 100, 2, 28, "Input",
  CellTags->"Closure"],
Cell[699621, 29447, 101, 1, 34, "ExampleText",
  CellTags->"Closure"],
Cell[699725, 29450, 174, 4, 42, "Input",
  CellTags->"Closure"],
Cell[699902, 29456, 110, 2, 28, "Input",
  CellTags->"Closure"],
Cell[700015, 29460, 133, 4, 34, "ExampleText",
  CellTags->"Closure"],
Cell[700151, 29466, 199, 6, 42, "Input",
  CellTags->"Closure"],
Cell[700353, 29474, 251, 5, 66, "ExampleText",
  CellTags->"Closure"],
Cell[700607, 29481, 155, 4, 26, "Input",
  CellTags->"Closure"],
Cell[700765, 29487, 312, 6, 66, "ExampleText",
  CellTags->"Closure"],
Cell[701080, 29495, 198, 5, 42, "Input",
  CellTags->"Closure"],
Cell[701281, 29502, 83, 3, 26, "Input",
  CellTags->"Closure"],
Cell[701367, 29507, 85, 3, 26, "Input",
  CellTags->"Closure"],
Cell[701455, 29512, 48, 1, 32, "ObjectName",
  CellTags->"Cycle"],
Cell[701506, 29515, 48, 1, 50, "Highlight",
  CellTags->"Cycle"],
Cell[701557, 29518, 185, 6, 31, "Usage",
  CellTags->"Cycle"],
Cell[701745, 29526, 157, 7, 24, "Notes",
  CellTags->"Cycle"],
Cell[701905, 29535, 70, 1, 32, "ObjectName",
  CellTags->"DeleteDuplicates"],
Cell[701978, 29538, 61, 1, 50, "Highlight",
  CellTags->"DeleteDuplicates"],
Cell[702042, 29541, 190, 7, 31, "Usage",
  CellTags->"DeleteDuplicates"],
Cell[702235, 29550, 236, 9, 44, "Notes",
  CellTags->"DeleteDuplicates"],
Cell[702474, 29561, 186, 5, 24, "Notes",
  CellTags->"DeleteDuplicates"],
Cell[702663, 29568, 247, 8, 40, "Notes",
  CellTags->"DeleteDuplicates"],
Cell[702913, 29578, 323, 9, 40, "Notes",
  CellTags->"DeleteDuplicates"],
Cell[703239, 29589, 136, 6, 24, "Notes",
  CellTags->"DeleteDuplicates"],
Cell[703378, 29597, 74, 1, 39, "ExampleSection",
  CellTags->"DeleteDuplicates"],
Cell[703455, 29600, 100, 1, 34, "ExampleText",
  CellTags->"DeleteDuplicates"],
Cell[703558, 29603, 128, 5, 45, "Input",
  CellTags->"DeleteDuplicates"],
Cell[703689, 29610, 81, 1, 34, "ExampleText",
  CellTags->"DeleteDuplicates"],
Cell[703773, 29613, 128, 3, 26, "Input",
  CellTags->"DeleteDuplicates"],
Cell[703904, 29618, 160, 4, 26, "Input",
  CellTags->"DeleteDuplicates"],
Cell[704067, 29624, 56, 1, 32, "ObjectName",
  CellTags->"DisjointQ"],
Cell[704126, 29627, 54, 1, 50, "Highlight",
  CellTags->"DisjointQ"],
Cell[704183, 29630, 197, 8, 31, "Usage",
  CellTags->"DisjointQ"],
Cell[704383, 29640, 156, 7, 24, "Notes",
  CellTags->"DisjointQ"],
Cell[704542, 29649, 67, 1, 39, "ExampleSection",
  CellTags->"DisjointQ"],
Cell[704612, 29652, 86, 1, 34, "ExampleText",
  CellTags->"DisjointQ"],
Cell[704701, 29655, 96, 2, 28, "Input",
  CellTags->"DisjointQ"],
Cell[704800, 29659, 94, 2, 28, "Input",
  CellTags->"DisjointQ"],
Cell[704897, 29663, 54, 1, 32, "ObjectName",
  CellTags->"EnumForm"],
Cell[704954, 29666, 53, 1, 50, "Highlight",
  CellTags->"EnumForm"],
Cell[705010, 29669, 210, 8, 31, "Usage",
  CellTags->"EnumForm"],
Cell[705223, 29679, 221, 9, 44, "Notes",
  CellTags->"EnumForm"],
Cell[705447, 29690, 162, 7, 24, "Notes",
  CellTags->"EnumForm"],
Cell[705612, 29699, 130, 5, 24, "Notes",
  CellTags->"EnumForm"],
Cell[705745, 29706, 179, 8, 24, "Notes",
  CellTags->"EnumForm"],
Cell[705927, 29716, 66, 1, 39, "ExampleSection",
  CellTags->"EnumForm"],
Cell[705996, 29719, 95, 1, 34, "ExampleText",
  CellTags->"EnumForm"],
Cell[706094, 29722, 174, 4, 42, "Input",
  CellTags->"EnumForm"],
Cell[706271, 29728, 94, 3, 26, "Input",
  CellTags->"EnumForm"],
Cell[706368, 29733, 106, 2, 34, "ExampleText",
  CellTags->"EnumForm"],
Cell[706477, 29737, 95, 3, 26, "Input",
  CellTags->"EnumForm"],
Cell[706575, 29742, 54, 1, 32, "ObjectName",
  CellTags->"External"],
Cell[706632, 29745, 51, 1, 50, "Highlight",
  CellTags->"External"],
Cell[706686, 29748, 156, 4, 31, "Usage",
  CellTags->"External"],
Cell[706845, 29754, 255, 8, 40, "Notes",
  CellTags->"External"],
Cell[707103, 29764, 120, 5, 24, "Notes",
  CellTags->"External"],
Cell[707226, 29771, 124, 4, 24, "Notes",
  CellTags->"External"],
Cell[707353, 29777, 441, 19, 24, "Notes",
  CellTags->"External"],
Cell[707797, 29798, 62, 1, 32, "ObjectName",
  CellTags->"FastPowerSet"],
Cell[707862, 29801, 57, 1, 50, "Highlight",
  CellTags->"FastPowerSet"],
Cell[707922, 29804, 147, 4, 31, "Usage",
  CellTags->"FastPowerSet"],
Cell[708072, 29810, 152, 5, 24, "Notes",
  CellTags->"FastPowerSet"],
Cell[708227, 29817, 156, 7, 24, "Notes",
  CellTags->"FastPowerSet"],
Cell[708386, 29826, 70, 1, 39, "ExampleSection",
  CellTags->"FastPowerSet"],
Cell[708459, 29829, 122, 4, 34, "ExampleText",
  CellTags->"FastPowerSet"],
Cell[708584, 29835, 128, 3, 26, "Input",
  CellTags->"FastPowerSet"],
Cell[708715, 29840, 124, 3, 26, "Input",
  CellTags->"FastPowerSet"],
Cell[708842, 29845, 130, 4, 42, "Input",
  CellTags->"FastPowerSet"],
Cell[708975, 29851, 50, 1, 32, "ObjectName",
  CellTags->"Filter"],
Cell[709028, 29854, 49, 1, 50, "Highlight",
  CellTags->"Filter"],
Cell[709080, 29857, 201, 5, 47, "Usage",
  CellTags->"Filter"],
Cell[709284, 29864, 76, 1, 24, "Notes",
  CellTags->"Filter"],
Cell[709363, 29867, 116, 4, 24, "Notes",
  CellTags->"Filter"],
Cell[709482, 29873, 159, 7, 24, "Notes",
  CellTags->"Filter"],
Cell[709644, 29882, 64, 1, 39, "ExampleSection",
  CellTags->"Filter"],
Cell[709711, 29885, 106, 2, 34, "ExampleText",
  CellTags->"Filter"],
Cell[709820, 29889, 255, 5, 58, "Input",
  CellTags->"Filter"],
Cell[710078, 29896, 120, 3, 26, "Input",
  CellTags->"Filter"],
Cell[710201, 29901, 231, 6, 42, "Input",
  CellTags->"Filter"],
Cell[710435, 29909, 100, 3, 26, "Input",
  CellTags->"Filter"],
Cell[710538, 29914, 60, 1, 32, "ObjectName",
  CellTags->"Frequencies"],
Cell[710601, 29917, 56, 1, 50, "Highlight",
  CellTags->"Frequencies"],
Cell[710660, 29920, 179, 6, 31, "Usage",
  CellTags->"Frequencies"],
Cell[710842, 29928, 165, 6, 24, "Notes",
  CellTags->"Frequencies"],
Cell[711010, 29936, 69, 1, 39, "ExampleSection",
  CellTags->"Frequencies"],
Cell[711082, 29939, 94, 1, 34, "ExampleText",
  CellTags->"Frequencies"],
Cell[711179, 29942, 199, 5, 58, "Input",
  CellTags->"Frequencies"],
Cell[711381, 29949, 64, 1, 32, "ObjectName",
  CellTags->"FromBitVector"],
Cell[711448, 29952, 58, 1, 50, "Highlight",
  CellTags->"FromBitVector"],
Cell[711509, 29955, 212, 8, 31, "Usage",
  CellTags->"FromBitVector"],
Cell[711724, 29965, 97, 1, 24, "Notes",
  CellTags->"FromBitVector"],
Cell[711824, 29968, 168, 7, 24, "Notes",
  CellTags->"FromBitVector"],
Cell[711995, 29977, 71, 1, 39, "ExampleSection",
  CellTags->"FromBitVector"],
Cell[712069, 29980, 100, 1, 34, "ExampleText",
  CellTags->"FromBitVector"],
Cell[712172, 29983, 93, 2, 28, "Input",
  CellTags->"FromBitVector"],
Cell[712268, 29987, 132, 3, 26, "Input",
  CellTags->"FromBitVector"],
Cell[712403, 29992, 48, 1, 32, "ObjectName",
  CellTags->"Index"],
Cell[712454, 29995, 48, 1, 50, "Highlight",
  CellTags->"Index"],
Cell[712505, 29998, 197, 8, 31, "Usage",
  CellTags->"Index"],
Cell[712705, 30008, 235, 11, 24, "Notes",
  CellTags->"Index"],
Cell[712943, 30021, 46, 1, 32, "ObjectName",
  CellTags->"Kary"],
Cell[712992, 30024, 47, 1, 50, "Highlight",
  CellTags->"Kary"],
Cell[713042, 30027, 231, 8, 47, "Usage",
  CellTags->"Kary"],
Cell[713276, 30037, 160, 7, 24, "Notes",
  CellTags->"Kary"],
Cell[713439, 30046, 86, 1, 32, "ObjectName",
  CellTags->"LongestCommonSubsequence"],
Cell[713528, 30049, 69, 1, 50, "Highlight",
  CellTags->"LongestCommonSubsequence"],
Cell[713600, 30052, 245, 8, 47, "Usage",
  CellTags->"LongestCommonSubsequence"],
Cell[713848, 30062, 239, 9, 44, "Notes",
  CellTags->"LongestCommonSubsequence"],
Cell[714090, 30073, 169, 5, 24, "Notes",
  CellTags->"LongestCommonSubsequence"],
Cell[714262, 30080, 114, 1, 24, "Notes",
  CellTags->"LongestCommonSubsequence"],
Cell[714379, 30083, 181, 7, 24, "Notes",
  CellTags->"LongestCommonSubsequence"],
Cell[714563, 30092, 82, 1, 39, "ExampleSection",
  CellTags->"LongestCommonSubsequence"],
Cell[714648, 30095, 92, 1, 34, "ExampleText",
  CellTags->"LongestCommonSubsequence"],
Cell[714743, 30098, 294, 6, 58, "Input",
  CellTags->"LongestCommonSubsequence"],
Cell[715040, 30106, 149, 4, 34, "ExampleText",
  CellTags->"LongestCommonSubsequence"],
Cell[715192, 30112, 189, 4, 42, "Input",
  CellTags->"LongestCommonSubsequence"],
Cell[715384, 30118, 56, 1, 32, "ObjectName",
  CellTags->"MakeIndex"],
Cell[715443, 30121, 54, 1, 50, "Highlight",
  CellTags->"MakeIndex"],
Cell[715500, 30124, 220, 8, 31, "Usage",
  CellTags->"MakeIndex"],
Cell[715723, 30134, 129, 6, 24, "Notes",
  CellTags->"MakeIndex"],
Cell[715855, 30142, 116, 2, 24, "Notes",
  CellTags->"MakeIndex"],
Cell[715974, 30146, 401, 12, 40, "Notes",
  CellTags->"MakeIndex"],
Cell[716378, 30160, 160, 7, 24, "Notes",
  CellTags->"MakeIndex"],
Cell[716541, 30169, 67, 1, 39, "ExampleSection",
  CellTags->"MakeIndex"],
Cell[716611, 30172, 88, 1, 34, "ExampleText",
  CellTags->"MakeIndex"],
Cell[716702, 30175, 135, 3, 26, "Input",
  CellTags->"MakeIndex"],
Cell[716840, 30180, 50, 1, 32, "ObjectName",
  CellTags->"Matrix"],
Cell[716893, 30183, 49, 1, 50, "Highlight",
  CellTags->"Matrix"],
Cell[716945, 30186, 199, 8, 31, "Usage",
  CellTags->"Matrix"],
Cell[717147, 30196, 157, 7, 24, "Notes",
  CellTags->"Matrix"],
Cell[717307, 30205, 82, 1, 32, "ObjectName",
  CellTags->"NonDecreasingSequences"],
Cell[717392, 30208, 67, 1, 50, "Highlight",
  CellTags->"NonDecreasingSequences"],
Cell[717462, 30211, 238, 8, 31, "Usage",
  CellTags->"NonDecreasingSequences"],
Cell[717703, 30221, 218, 6, 24, "Notes",
  CellTags->"NonDecreasingSequences"],
Cell[717924, 30229, 183, 5, 24, "Notes",
  CellTags->"NonDecreasingSequences"],
Cell[718110, 30236, 270, 12, 24, "Notes",
  CellTags->"NonDecreasingSequences"],
Cell[718383, 30250, 80, 1, 39, "ExampleSection",
  CellTags->"NonDecreasingSequences"],
Cell[718466, 30253, 200, 6, 34, "ExampleText",
  CellTags->"NonDecreasingSequences"],
Cell[718669, 30261, 107, 2, 28, "Input",
  CellTags->"NonDecreasingSequences"],
Cell[718779, 30265, 85, 1, 34, "ExampleText",
  CellTags->"NonDecreasingSequences"],
Cell[718867, 30268, 150, 5, 45, "Input",
  CellTags->"NonDecreasingSequences"],
Cell[719020, 30275, 145, 4, 34, "ExampleText",
  CellTags->"NonDecreasingSequences"],
Cell[719168, 30281, 121, 3, 26, "Input",
  CellTags->"NonDecreasingSequences"],
Cell[719292, 30286, 117, 3, 26, "Input",
  CellTags->"NonDecreasingSequences"],
Cell[719412, 30291, 50, 1, 32, "ObjectName",
  CellTags->"NoRest"],
Cell[719465, 30294, 49, 1, 50, "Highlight",
  CellTags->"NoRest"],
Cell[719517, 30297, 141, 6, 31, "Usage",
  CellTags->"NoRest"],
Cell[719661, 30305, 154, 7, 24, "Notes",
  CellTags->"NoRest"],
Cell[719818, 30314, 54, 1, 32, "ObjectName",
  CellTags->"OutPairs"],
Cell[719875, 30317, 51, 1, 50, "Highlight",
  CellTags->"OutPairs"],
Cell[719929, 30320, 160, 6, 31, "Usage",
  CellTags->"OutPairs"],
Cell[720092, 30328, 93, 1, 24, "Notes",
  CellTags->"OutPairs"],
Cell[720188, 30331, 168, 7, 24, "Notes",
  CellTags->"OutPairs"],
Cell[720359, 30340, 48, 1, 32, "ObjectName",
  CellTags->"Pairs"],
Cell[720410, 30343, 50, 1, 50, "Highlight",
  CellTags->"Pairs"],
Cell[720463, 30346, 209, 8, 31, "Usage",
  CellTags->"Pairs"],
Cell[720675, 30356, 94, 4, 24, "Notes",
  CellTags->"Pairs"],
Cell[720772, 30362, 151, 5, 24, "Notes",
  CellTags->"Pairs"],
Cell[720926, 30369, 93, 1, 24, "Notes",
  CellTags->"Pairs"],
Cell[721022, 30372, 207, 7, 24, "Notes",
  CellTags->"Pairs"],
Cell[721232, 30381, 63, 1, 39, "ExampleSection",
  CellTags->"Pairs"],
Cell[721298, 30384, 62, 1, 34, "ExampleText",
  CellTags->"Pairs"],
Cell[721363, 30387, 86, 2, 28, "Input",
  CellTags->"Pairs"],
Cell[721452, 30391, 94, 2, 28, "Input",
  CellTags->"Pairs"],
Cell[721549, 30395, 76, 2, 28, "Input",
  CellTags->"Pairs"],
Cell[721628, 30399, 77, 2, 28, "Input",
  CellTags->"Pairs"],
Cell[721708, 30403, 58, 1, 32, "ObjectName",
  CellTags->"Partitions"],
Cell[721769, 30406, 55, 1, 50, "Highlight",
  CellTags->"Partitions"],
Cell[721827, 30409, 153, 6, 31, "Usage",
  CellTags->"Partitions"],
Cell[721983, 30417, 218, 8, 24, "Notes",
  CellTags->"Partitions"],
Cell[722204, 30427, 261, 11, 24, "Notes",
  CellTags->"Partitions"],
Cell[722468, 30440, 68, 1, 39, "ExampleSection",
  CellTags->"Partitions"],
Cell[722539, 30443, 144, 6, 34, "ExampleText",
  CellTags->"Partitions"],
Cell[722686, 30451, 82, 2, 28, "Input",
  CellTags->"Partitions"],
Cell[722771, 30455, 54, 1, 32, "ObjectName",
  CellTags->"Periodic"],
Cell[722828, 30458, 51, 1, 50, "Highlight",
  CellTags->"Periodic"],
Cell[722882, 30461, 215, 6, 47, "Usage",
  CellTags->"Periodic"],
Cell[723100, 30469, 112, 5, 24, "Notes",
  CellTags->"Periodic"],
Cell[723215, 30476, 160, 7, 24, "Notes",
  CellTags->"Periodic"],
Cell[723378, 30485, 54, 1, 32, "ObjectName",
  CellTags->"PowerSet"],
Cell[723435, 30488, 53, 1, 50, "Highlight",
  CellTags->"PowerSet"],
Cell[723491, 30491, 186, 8, 31, "Usage",
  CellTags->"PowerSet"],
Cell[723680, 30501, 136, 5, 24, "Notes",
  CellTags->"PowerSet"],
Cell[723819, 30508, 136, 6, 24, "Notes",
  CellTags->"PowerSet"],
Cell[723958, 30516, 201, 7, 24, "Notes",
  CellTags->"PowerSet"],
Cell[724162, 30525, 66, 1, 39, "ExampleSection",
  CellTags->"PowerSet"],
Cell[724231, 30528, 146, 6, 34, "ExampleText",
  CellTags->"PowerSet"],
Cell[724380, 30536, 86, 2, 28, "Input",
  CellTags->"PowerSet"],
Cell[724469, 30540, 88, 2, 28, "Input",
  CellTags->"PowerSet"],
Cell[724560, 30544, 72, 1, 32, "ObjectName",
  CellTags->"RandomPermutation"],
Cell[724635, 30547, 62, 1, 50, "Highlight",
  CellTags->"RandomPermutation"],
Cell[724700, 30550, 180, 6, 31, "Usage",
  CellTags->"RandomPermutation"],
Cell[724883, 30558, 122, 4, 24, "Notes",
  CellTags->"RandomPermutation"],
Cell[725008, 30564, 162, 7, 24, "Notes",
  CellTags->"RandomPermutation"],
Cell[725173, 30573, 75, 1, 39, "ExampleSection",
  CellTags->"RandomPermutation"],
Cell[725251, 30576, 159, 6, 34, "ExampleText",
  CellTags->"RandomPermutation"],
Cell[725413, 30584, 96, 2, 28, "Input",
  CellTags->"RandomPermutation"],
Cell[725512, 30588, 46, 1, 32, "ObjectName",
  CellTags->"Rank"],
Cell[725561, 30591, 47, 1, 50, "Highlight",
  CellTags->"Rank"],
Cell[725611, 30594, 134, 6, 31, "Usage",
  CellTags->"Rank"],
Cell[725748, 30602, 151, 7, 24, "Notes",
  CellTags->"Rank"],
Cell[725902, 30611, 62, 1, 39, "ExampleSection",
  CellTags->"Rank"],
Cell[725967, 30614, 62, 1, 32, "ObjectName",
  CellTags->"RankingRules"],
Cell[726032, 30617, 57, 1, 50, "Highlight",
  CellTags->"RankingRules"],
Cell[726092, 30620, 204, 6, 31, "Usage",
  CellTags->"RankingRules"],
Cell[726299, 30628, 277, 10, 61, "Notes",
  CellTags->"RankingRules"],
Cell[726579, 30640, 186, 7, 24, "Notes",
  CellTags->"RankingRules"],
Cell[726768, 30649, 202, 6, 40, "Notes",
  CellTags->"RankingRules"],
Cell[726973, 30657, 220, 6, 40, "Notes",
  CellTags->"RankingRules"],
Cell[727196, 30665, 229, 6, 40, "Notes",
  CellTags->"RankingRules"],
Cell[727428, 30673, 70, 1, 39, "ExampleSection",
  CellTags->"RankingRules"],
Cell[727501, 30676, 157, 4, 50, "ExampleText",
  CellTags->"RankingRules"],
Cell[727661, 30682, 97, 2, 28, "Input",
  CellTags->"RankingRules"],
Cell[727761, 30686, 83, 1, 34, "ExampleText",
  CellTags->"RankingRules"],
Cell[727847, 30689, 218, 5, 58, "Input",
  CellTags->"RankingRules"],
Cell[728068, 30696, 84, 1, 34, "ExampleText",
  CellTags->"RankingRules"],
Cell[728155, 30699, 139, 3, 26, "Input",
  CellTags->"RankingRules"],
Cell[728297, 30704, 95, 1, 34, "ExampleText",
  CellTags->"RankingRules"],
Cell[728395, 30707, 197, 4, 42, "Input",
  CellTags->"RankingRules"],
Cell[728595, 30713, 56, 1, 32, "ObjectName",
  CellTags->"Reflexive"],
Cell[728654, 30716, 52, 1, 50, "Highlight",
  CellTags->"Reflexive"],
Cell[728709, 30719, 156, 5, 31, "Usage",
  CellTags->"Reflexive"],
Cell[728868, 30726, 110, 5, 24, "Notes",
  CellTags->"Reflexive"],
Cell[728981, 30733, 176, 7, 24, "Notes",
  CellTags->"Reflexive"],
Cell[729160, 30742, 54, 1, 32, "ObjectName",
  CellTags->"Relation"],
Cell[729217, 30745, 51, 1, 50, "Highlight",
  CellTags->"Relation"],
Cell[729271, 30748, 203, 8, 31, "Usage",
  CellTags->"Relation"],
Cell[729477, 30758, 154, 7, 24, "Notes",
  CellTags->"Relation"],
Cell[729634, 30767, 54, 1, 32, "ObjectName",
  CellTags->"RootList"],
Cell[729691, 30770, 53, 1, 50, "Highlight",
  CellTags->"RootList"],
Cell[729747, 30773, 304, 12, 47, "Usage",
  CellTags->"RootList"],
Cell[730054, 30787, 131, 5, 24, "Notes",
  CellTags->"RootList"],
Cell[730188, 30794, 281, 11, 24, "Notes",
  CellTags->"RootList"],
Cell[730472, 30807, 66, 1, 39, "ExampleSection",
  CellTags->"RootList"],
Cell[730541, 30810, 79, 1, 34, "ExampleText",
  CellTags->"RootList"],
Cell[730623, 30813, 117, 3, 26, "Input",
  CellTags->"RootList"],
Cell[730743, 30818, 94, 3, 26, "Input",
  CellTags->"RootList"],
Cell[730840, 30823, 56, 1, 32, "ObjectName",
  CellTags->"SelectorQ"],
Cell[730899, 30826, 54, 1, 50, "Highlight",
  CellTags->"SelectorQ"],
Cell[730956, 30829, 229, 8, 31, "Usage",
  CellTags->"SelectorQ"],
Cell[731188, 30839, 231, 9, 24, "Notes",
  CellTags->"SelectorQ"],
Cell[731422, 30850, 162, 7, 24, "Notes",
  CellTags->"SelectorQ"],
Cell[731587, 30859, 67, 1, 39, "ExampleSection",
  CellTags->"SelectorQ"],
Cell[731657, 30862, 174, 6, 34, "ExampleText",
  CellTags->"SelectorQ"],
Cell[731834, 30870, 138, 3, 26, "Input",
  CellTags->"SelectorQ"],
Cell[731975, 30875, 116, 3, 26, "Input",
  CellTags->"SelectorQ"],
Cell[732094, 30880, 115, 3, 26, "Input",
  CellTags->"SelectorQ"],
Cell[732212, 30885, 58, 1, 32, "ObjectName",
  CellTags->"SetClosure"],
Cell[732273, 30888, 53, 1, 50, "Highlight",
  CellTags->"SetClosure"],
Cell[732329, 30891, 245, 8, 47, "Usage",
  CellTags->"SetClosure"],
Cell[732577, 30901, 254, 11, 24, "Notes",
  CellTags->"SetClosure"],
Cell[732834, 30914, 52, 1, 32, "ObjectName",
  CellTags->"Shuffle"],
Cell[732889, 30917, 52, 1, 50, "Highlight",
  CellTags->"Shuffle"],
Cell[732944, 30920, 175, 8, 31, "Usage",
  CellTags->"Shuffle"],
Cell[733122, 30930, 240, 11, 24, "Notes",
  CellTags->"Shuffle"],
Cell[733365, 30943, 270, 11, 24, "Notes",
  CellTags->"Shuffle"],
Cell[733638, 30956, 65, 1, 39, "ExampleSection",
  CellTags->"Shuffle"],
Cell[733706, 30959, 105, 2, 34, "ExampleText",
  CellTags->"Shuffle"],
Cell[733814, 30963, 91, 2, 28, "Input",
  CellTags->"Shuffle"],
Cell[733908, 30967, 119, 3, 26, "Input",
  CellTags->"Shuffle"],
Cell[734030, 30972, 52, 1, 32, "ObjectName",
  CellTags->"SubsetQ"],
Cell[734085, 30975, 52, 1, 50, "Highlight",
  CellTags->"SubsetQ"],
Cell[734140, 30978, 185, 8, 31, "Usage",
  CellTags->"SubsetQ"],
Cell[734328, 30988, 126, 5, 24, "Notes",
  CellTags->"SubsetQ"],
Cell[734457, 30995, 156, 7, 24, "Notes",
  CellTags->"SubsetQ"],
Cell[734616, 31004, 65, 1, 39, "ExampleSection",
  CellTags->"SubsetQ"],
Cell[734684, 31007, 78, 1, 34, "ExampleText",
  CellTags->"SubsetQ"],
Cell[734765, 31010, 91, 2, 28, "Input",
  CellTags->"SubsetQ"],
Cell[734859, 31014, 91, 2, 28, "Input",
  CellTags->"SubsetQ"],
Cell[734953, 31018, 60, 1, 32, "ObjectName",
  CellTags->"ToBitVector"],
Cell[735016, 31021, 56, 1, 50, "Highlight",
  CellTags->"ToBitVector"],
Cell[735075, 31024, 206, 8, 31, "Usage",
  CellTags->"ToBitVector"],
Cell[735284, 31034, 95, 1, 24, "Notes",
  CellTags->"ToBitVector"],
Cell[735382, 31037, 170, 7, 24, "Notes",
  CellTags->"ToBitVector"],
Cell[735555, 31046, 69, 1, 39, "ExampleSection",
  CellTags->"ToBitVector"],
Cell[735627, 31049, 99, 1, 34, "ExampleText",
  CellTags->"ToBitVector"],
Cell[735729, 31052, 92, 2, 28, "Input",
  CellTags->"ToBitVector"],
Cell[735824, 31056, 127, 3, 26, "Input",
  CellTags->"ToBitVector"],
Cell[735954, 31061, 50, 1, 32, "ObjectName",
  CellTags->"ToList"],
Cell[736007, 31064, 51, 1, 50, "Highlight",
  CellTags->"ToList"],
Cell[736061, 31067, 182, 8, 31, "Usage",
  CellTags->"ToList"],
Cell[736246, 31077, 52, 1, 32, "ObjectName",
  CellTags->"ToPairs"],
Cell[736301, 31080, 52, 1, 50, "Highlight",
  CellTags->"ToPairs"],
Cell[736356, 31083, 178, 6, 31, "Usage",
  CellTags->"ToPairs"],
Cell[736537, 31091, 223, 9, 44, "Notes",
  CellTags->"ToPairs"],
Cell[736763, 31102, 145, 4, 24, "Notes",
  CellTags->"ToPairs"],
Cell[736911, 31108, 217, 8, 24, "Notes",
  CellTags->"ToPairs"],
Cell[737131, 31118, 421, 15, 40, "Notes",
  CellTags->"ToPairs"],
Cell[737555, 31135, 331, 15, 24, "Notes",
  CellTags->"ToPairs"],
Cell[737889, 31152, 65, 1, 39, "ExampleSection",
  CellTags->"ToPairs"],
Cell[737957, 31155, 96, 1, 34, "ExampleText",
  CellTags->"ToPairs"],
Cell[738056, 31158, 107, 5, 45, "Input",
  CellTags->"ToPairs"],
Cell[738166, 31165, 122, 3, 26, "Input",
  CellTags->"ToPairs"],
Cell[738291, 31170, 178, 5, 50, "ExampleText",
  CellTags->"ToPairs"],
Cell[738472, 31177, 196, 4, 42, "Input",
  CellTags->"ToPairs"],
Cell[738671, 31183, 271, 6, 58, "Input",
  CellTags->"ToPairs"],
Cell[738945, 31191, 50, 1, 32, "ObjectName",
  CellTags->"Tuples"],
Cell[738998, 31194, 51, 1, 50, "Highlight",
  CellTags->"Tuples"],
Cell[739052, 31197, 221, 8, 31, "Usage",
  CellTags->"Tuples"],
Cell[739276, 31207, 142, 6, 24, "Notes",
  CellTags->"Tuples"],
Cell[739421, 31215, 311, 11, 24, "Notes",
  CellTags->"Tuples"],
Cell[739735, 31228, 177, 7, 24, "Notes",
  CellTags->"Tuples"],
Cell[739915, 31237, 165, 7, 24, "Notes",
  CellTags->"Tuples"],
Cell[740083, 31246, 64, 1, 39, "ExampleSection",
  CellTags->"Tuples"],
Cell[740150, 31249, 92, 1, 34, "ExampleText",
  CellTags->"Tuples"],
Cell[740245, 31252, 98, 3, 26, "Input",
  CellTags->"Tuples"],
Cell[740346, 31257, 119, 4, 34, "ExampleText",
  CellTags->"Tuples"],
Cell[740468, 31263, 103, 3, 26, "Input",
  CellTags->"Tuples"],
Cell[740574, 31268, 84, 1, 34, "ExampleText",
  CellTags->"Tuples"],
Cell[740661, 31271, 129, 4, 42, "Input",
  CellTags->"Tuples"],
Cell[740793, 31277, 46, 1, 32, "ObjectName",
  CellTags->"Type"],
Cell[740842, 31280, 47, 1, 50, "Highlight",
  CellTags->"Type"],
Cell[740892, 31283, 137, 6, 31, "Usage",
  CellTags->"Type"],
Cell[741032, 31291, 230, 11, 24, "Notes",
  CellTags->"Type"],
Cell[741265, 31304, 150, 7, 24, "Notes",
  CellTags->"Type"],
Cell[741418, 31313, 54, 1, 32, "ObjectName",
  CellTags->"$extlink"],
Cell[741475, 31316, 56, 1, 50, "Highlight",
  CellTags->"$extlink"],
Cell[741534, 31319, 143, 5, 31, "Usage",
  CellTags->"$extlink"],
Cell[741680, 31326, 260, 10, 25, "Notes",
  CellTags->"$extlink"],
Cell[741943, 31338, 140, 5, 24, "Notes",
  CellTags->"$extlink"],
Cell[742086, 31345, 201, 9, 24, "Notes",
  CellTags->"$extlink"],
Cell[742290, 31356, 66, 1, 39, "ExampleSection",
  CellTags->"$extlink"],
Cell[742359, 31359, 160, 7, 34, "ExampleText",
  CellTags->"$extlink"],
Cell[742522, 31368, 91, 3, 26, "Input",
  CellTags->"$extlink"],
Cell[742616, 31373, 70, 1, 32, "ObjectName",
  CellTags->"$defaultexternal"],
Cell[742689, 31376, 64, 1, 50, "Highlight",
  CellTags->"$defaultexternal"],
Cell[742756, 31379, 180, 5, 31, "Usage",
  CellTags->"$defaultexternal"],
Cell[742939, 31386, 268, 10, 25, "Notes",
  CellTags->"$defaultexternal"],
Cell[743210, 31398, 148, 5, 24, "Notes",
  CellTags->"$defaultexternal"],
Cell[743361, 31405, 397, 18, 24, "Notes",
  CellTags->"$defaultexternal"],
Cell[743761, 31425, 74, 1, 39, "ExampleSection",
  CellTags->"$defaultexternal"],
Cell[743838, 31428, 89, 1, 34, "ExampleText",
  CellTags->"$defaultexternal"],
Cell[743930, 31431, 107, 3, 26, "Input",
  CellTags->"$defaultexternal"],
Cell[744040, 31436, 46, 1, 32, "ObjectName",
  CellTags->"$LLL"],
Cell[744089, 31439, 52, 1, 50, "Highlight",
  CellTags->"$LLL"],
Cell[744144, 31442, 246, 8, 47, "Usage",
  CellTags->"$LLL"],
Cell[744393, 31452, 355, 14, 40, "Notes",
  CellTags->"$LLL"],
Cell[744751, 31468, 127, 4, 24, "Notes",
  CellTags->"$LLL"],
Cell[744881, 31474, 127, 6, 24, "Notes",
  CellTags->"$LLL"],
Cell[745011, 31482, 62, 1, 39, "ExampleSection",
  CellTags->"$LLL"],
Cell[745076, 31485, 172, 6, 34, "ExampleText",
  CellTags->"$LLL"],
Cell[745251, 31493, 192, 5, 42, "Input",
  CellTags->"$LLL"],
Cell[745446, 31500, 91, 1, 34, "ExampleText",
  CellTags->"$LLL"],
Cell[745540, 31503, 126, 3, 26, "Input",
  CellTags->"$LLL"],
Cell[745669, 31508, 163, 5, 34, "ExampleText",
  CellTags->"$LLL"],
Cell[745835, 31515, 133, 3, 26, "Input",
  CellTags->"$LLL"],
Cell[745971, 31520, 92, 1, 34, "ExampleText",
  CellTags->"$LLL"],
Cell[746066, 31523, 135, 3, 26, "Input",
  CellTags->"$LLL"],
Cell[746204, 31528, 136, 4, 34, "ExampleText",
  CellTags->"$LLL"],
Cell[746343, 31534, 124, 3, 26, "Input",
  CellTags->"$LLL"],
Cell[746470, 31539, 60, 1, 32, "ObjectName",
  CellTags->"$shortlines"],
Cell[746533, 31542, 59, 1, 50, "Highlight",
  CellTags->"$shortlines"],
Cell[746595, 31545, 186, 5, 47, "Usage",
  CellTags->"$shortlines"],
Cell[746784, 31552, 165, 5, 24, "Notes",
  CellTags->"$shortlines"],
Cell[746952, 31559, 64, 1, 24, "Notes",
  CellTags->"$shortlines"],
Cell[747019, 31562, 172, 7, 24, "Notes",
  CellTags->"$shortlines"],
Cell[747194, 31571, 69, 1, 39, "ExampleSection",
  CellTags->"$shortlines"],
Cell[747266, 31574, 80, 1, 34, "ExampleText",
  CellTags->"$shortlines"],
Cell[747349, 31577, 97, 3, 26, "Input",
  CellTags->"$shortlines"],
Cell[747449, 31582, 84, 1, 34, "ExampleText",
  CellTags->"$shortlines"],
Cell[747536, 31585, 131, 3, 26, "Input",
  CellTags->"$shortlines"],
Cell[747670, 31590, 126, 4, 42, "Input",
  CellTags->"$shortlines"],
Cell[747799, 31596, 80, 2, 26, "Input",
  CellTags->"$shortlines"],
Cell[747882, 31600, 68, 1, 32, "ObjectName",
  CellTags->"$transdispwidth"],
Cell[747953, 31603, 63, 1, 50, "Highlight",
  CellTags->"$transdispwidth"],
Cell[748019, 31606, 200, 6, 31, "Usage",
  CellTags->"$transdispwidth"],
Cell[748222, 31614, 62, 1, 24, "Notes",
  CellTags->"$transdispwidth"],
Cell[748287, 31617, 157, 7, 24, "Notes",
  CellTags->"$transdispwidth"],
Cell[748447, 31626, 73, 1, 39, "ExampleSection",
  CellTags->"$transdispwidth"],
Cell[748523, 31629, 88, 1, 34, "ExampleText",
  CellTags->"$transdispwidth"],
Cell[748614, 31632, 96, 2, 28, "Input",
  CellTags->"$transdispwidth"],
Cell[748713, 31636, 130, 3, 26, "Input",
  CellTags->"$transdispwidth"],
Cell[748846, 31641, 44, 1, 32, "ObjectName",
  CellTags->"All"],
Cell[748893, 31644, 46, 1, 50, "Highlight",
  CellTags->"All"],
Cell[748942, 31647, 120, 4, 31, "Usage",
  CellTags->"All"],
Cell[749065, 31653, 430, 15, 24, "Notes",
  CellTags->"All"],
Cell[749498, 31670, 61, 1, 39, "ExampleSection",
  CellTags->"All"],
Cell[749562, 31673, 207, 7, 50, "ExampleText",
  CellTags->"All"],
Cell[749772, 31682, 105, 2, 28, "Input",
  CellTags->"All"],
Cell[749880, 31686, 90, 1, 34, "ExampleText",
  CellTags->"All"],
Cell[749973, 31689, 118, 2, 28, "Input",
  CellTags->"All"],
Cell[750094, 31693, 66, 1, 32, "ObjectName",
  CellTags->"AssignFunction"],
Cell[750163, 31696, 59, 1, 50, "Highlight",
  CellTags->"AssignFunction"],
Cell[750225, 31699, 327, 12, 47, "Usage",
  CellTags->"AssignFunction"],
Cell[750555, 31713, 375, 13, 40, "Notes",
  CellTags->"AssignFunction"],
Cell[750933, 31728, 277, 10, 61, "Notes",
  CellTags->"AssignFunction"],
Cell[751213, 31740, 162, 5, 24, "Notes",
  CellTags->"AssignFunction"],
Cell[751378, 31747, 226, 8, 24, "Notes",
  CellTags->"AssignFunction"],
Cell[751607, 31757, 158, 7, 24, "Notes",
  CellTags->"AssignFunction"],
Cell[751768, 31766, 72, 1, 39, "ExampleSection",
  CellTags->"AssignFunction"],
Cell[751843, 31769, 93, 1, 34, "ExampleText",
  CellTags->"AssignFunction"],
Cell[751939, 31772, 123, 5, 45, "Input",
  CellTags->"AssignFunction"],
Cell[752065, 31779, 153, 4, 34, "ExampleText",
  CellTags->"AssignFunction"],
Cell[752221, 31785, 193, 5, 42, "Input",
  CellTags->"AssignFunction"],
Cell[752417, 31792, 254, 8, 50, "ExampleText",
  CellTags->"AssignFunction"],
Cell[752674, 31802, 231, 5, 58, "Input",
  CellTags->"AssignFunction"],
Cell[752908, 31809, 93, 3, 26, "Input",
  CellTags->"AssignFunction"],
Cell[753004, 31814, 48, 1, 32, "ObjectName",
  CellTags->"Curry"],
Cell[753055, 31817, 52, 1, 50, "Highlight",
  CellTags->"Curry"],
Cell[753110, 31820, 152, 6, 31, "Usage",
  CellTags->"Curry"],
Cell[753265, 31828, 112, 5, 24, "Notes",
  CellTags->"Curry"],
Cell[753380, 31835, 142, 4, 24, "Notes",
  CellTags->"Curry"],
Cell[753525, 31841, 63, 1, 39, "ExampleSection",
  CellTags->"Curry"],
Cell[753591, 31844, 83, 1, 34, "ExampleText",
  CellTags->"Curry"],
Cell[753677, 31847, 116, 3, 26, "Input",
  CellTags->"Curry"],
Cell[753796, 31852, 173, 4, 42, "Input",
  CellTags->"Curry"],
Cell[753972, 31858, 58, 1, 32, "ObjectName",
  CellTags->"Dependency"],
Cell[754033, 31861, 55, 1, 50, "Highlight",
  CellTags->"Dependency"],
Cell[754091, 31864, 261, 8, 47, "Usage",
  CellTags->"Dependency"],
Cell[754355, 31874, 367, 14, 24, "Notes",
  CellTags->"Dependency"],
Cell[754725, 31890, 129, 5, 24, "Notes",
  CellTags->"Dependency"],
Cell[754857, 31897, 167, 7, 24, "Notes",
  CellTags->"Dependency"],
Cell[755027, 31906, 68, 1, 39, "ExampleSection",
  CellTags->"Dependency"],
Cell[755098, 31909, 101, 1, 34, "ExampleText",
  CellTags->"Dependency"],
Cell[755202, 31912, 222, 6, 42, "Input",
  CellTags->"Dependency"],
Cell[755427, 31920, 98, 3, 26, "Input",
  CellTags->"Dependency"],
Cell[755528, 31925, 52, 1, 32, "ObjectName",
  CellTags->"DoTrace"],
Cell[755583, 31928, 50, 1, 50, "Highlight",
  CellTags->"DoTrace"],
Cell[755636, 31931, 166, 5, 31, "Usage",
  CellTags->"DoTrace"],
Cell[755805, 31938, 572, 23, 40, "Notes",
  CellTags->"DoTrace"],
Cell[756380, 31963, 46, 1, 32, "ObjectName",
  CellTags->"Drei"],
Cell[756429, 31966, 54, 1, 50, "Highlight",
  CellTags->"Drei"],
Cell[756486, 31969, 147, 6, 31, "Usage",
  CellTags->"Drei"],
Cell[756636, 31977, 103, 5, 24, "Notes",
  CellTags->"Drei"],
Cell[756742, 31984, 333, 15, 24, "Notes",
  CellTags->"Drei"],
Cell[757078, 32001, 60, 1, 32, "ObjectName",
  CellTags->"ExtAutomata"],
Cell[757141, 32004, 56, 1, 50, "Highlight",
  CellTags->"ExtAutomata"],
Cell[757200, 32007, 148, 4, 31, "Usage",
  CellTags->"ExtAutomata"],
Cell[757351, 32013, 181, 7, 24, "Notes",
  CellTags->"ExtAutomata"],
Cell[757535, 32022, 183, 7, 24, "Notes",
  CellTags->"ExtAutomata"],
Cell[757721, 32031, 204, 9, 24, "Notes",
  CellTags->"ExtAutomata"],
Cell[757928, 32042, 69, 1, 39, "ExampleSection",
  CellTags->"ExtAutomata"],
Cell[758000, 32045, 166, 4, 50, "ExampleText",
  CellTags->"ExtAutomata"],
Cell[758169, 32051, 118, 3, 26, "Input",
  CellTags->"ExtAutomata"],
Cell[758290, 32056, 151, 4, 42, "Input",
  CellTags->"ExtAutomata"],
Cell[758444, 32062, 156, 4, 42, "Input",
  CellTags->"ExtAutomata"],
Cell[758603, 32068, 58, 1, 32, "ObjectName",
  CellTags->"FlattenOne"],
Cell[758664, 32071, 55, 1, 50, "Highlight",
  CellTags->"FlattenOne"],
Cell[758722, 32074, 154, 6, 31, "Usage",
  CellTags->"FlattenOne"],
Cell[758879, 32082, 70, 1, 24, "Notes",
  CellTags->"FlattenOne"],
Cell[758952, 32085, 153, 7, 24, "Notes",
  CellTags->"FlattenOne"],
Cell[759108, 32094, 68, 1, 39, "ExampleSection",
  CellTags->"FlattenOne"],
Cell[759179, 32097, 139, 4, 34, "ExampleText",
  CellTags->"FlattenOne"],
Cell[759321, 32103, 97, 2, 28, "Input",
  CellTags->"FlattenOne"],
Cell[759421, 32107, 138, 3, 26, "Input",
  CellTags->"FlattenOne"],
Cell[759562, 32112, 66, 1, 32, "ObjectName",
  CellTags->"FormatAutomata"],
Cell[759631, 32115, 59, 1, 50, "Highlight",
  CellTags->"FormatAutomata"],
Cell[759693, 32118, 184, 5, 47, "Usage",
  CellTags->"FormatAutomata"],
Cell[759880, 32125, 190, 5, 40, "Notes",
  CellTags->"FormatAutomata"],
Cell[760073, 32132, 167, 5, 24, "Notes",
  CellTags->"FormatAutomata"],
Cell[760243, 32139, 199, 6, 24, "Notes",
  CellTags->"FormatAutomata"],
Cell[760445, 32147, 197, 5, 40, "Notes",
  CellTags->"FormatAutomata"],
Cell[760645, 32154, 169, 6, 24, "Notes",
  CellTags->"FormatAutomata"],
Cell[760817, 32162, 161, 4, 40, "Notes",
  CellTags->"FormatAutomata"],
Cell[760981, 32168, 196, 6, 24, "Notes",
  CellTags->"FormatAutomata"],
Cell[761180, 32176, 168, 7, 24, "Notes",
  CellTags->"FormatAutomata"],
Cell[761351, 32185, 72, 1, 39, "ExampleSection",
  CellTags->"FormatAutomata"],
Cell[761426, 32188, 101, 1, 34, "ExampleText",
  CellTags->"FormatAutomata"],
Cell[761530, 32191, 149, 4, 26, "Input",
  CellTags->"FormatAutomata"],
Cell[761682, 32197, 128, 4, 42, "Input",
  CellTags->"FormatAutomata"],
Cell[761813, 32203, 128, 4, 42, "Input",
  CellTags->"FormatAutomata"],
Cell[761944, 32209, 133, 4, 42, "Input",
  CellTags->"FormatAutomata"],
Cell[762080, 32215, 132, 4, 42, "Input",
  CellTags->"FormatAutomata"],
Cell[762215, 32221, 128, 4, 42, "Input",
  CellTags->"FormatAutomata"],
Cell[762346, 32227, 73, 1, 34, "ExampleText",
  CellTags->"FormatAutomata"],
Cell[762422, 32230, 113, 3, 26, "Input",
  CellTags->"FormatAutomata"],
Cell[762538, 32235, 46, 1, 32, "ObjectName",
  CellTags->"Full"],
Cell[762587, 32238, 47, 1, 50, "Highlight",
  CellTags->"Full"],
Cell[762637, 32241, 116, 4, 31, "Usage",
  CellTags->"Full"],
Cell[762756, 32247, 1980, 79, 72, "Notes",
  CellTags->"Full"],
Cell[764739, 32328, 56, 1, 32, "ObjectName",
  CellTags->"MinusEins"],
Cell[764798, 32331, 59, 1, 50, "Highlight",
  CellTags->"MinusEins"],
Cell[764860, 32334, 159, 7, 31, "Usage",
  CellTags->"MinusEins"],
Cell[765022, 32343, 114, 5, 24, "Notes",
  CellTags->"MinusEins"],
Cell[765139, 32350, 346, 15, 24, "Notes",
  CellTags->"MinusEins"],
Cell[765488, 32367, 56, 1, 32, "ObjectName",
  CellTags->"MinusZwei"],
Cell[765547, 32370, 59, 1, 50, "Highlight",
  CellTags->"MinusZwei"],
Cell[765609, 32373, 170, 6, 31, "Usage",
  CellTags->"MinusZwei"],
Cell[765782, 32381, 114, 5, 24, "Notes",
  CellTags->"MinusZwei"],
Cell[765899, 32388, 346, 15, 24, "Notes",
  CellTags->"MinusZwei"],
Cell[766248, 32405, 46, 1, 32, "ObjectName",
  CellTags->"Opts"],
Cell[766297, 32408, 47, 1, 50, "Highlight",
  CellTags->"Opts"],
Cell[766347, 32411, 142, 6, 31, "Usage",
  CellTags->"Opts"],
Cell[766492, 32419, 85, 1, 24, "Notes",
  CellTags->"Opts"],
Cell[766580, 32422, 160, 7, 24, "Notes",
  CellTags->"Opts"],
Cell[766743, 32431, 62, 1, 32, "ObjectName",
  CellTags->"PositionList"],
Cell[766808, 32434, 57, 1, 50, "Highlight",
  CellTags->"PositionList"],
Cell[766868, 32437, 245, 8, 47, "Usage",
  CellTags->"PositionList"],
Cell[767116, 32447, 227, 9, 44, "Notes",
  CellTags->"PositionList"],
Cell[767346, 32458, 215, 6, 40, "Notes",
  CellTags->"PositionList"],
Cell[767564, 32466, 169, 7, 24, "Notes",
  CellTags->"PositionList"],
Cell[767736, 32475, 70, 1, 39, "ExampleSection",
  CellTags->"PositionList"],
Cell[767809, 32478, 132, 4, 34, "ExampleText",
  CellTags->"PositionList"],
Cell[767944, 32484, 108, 2, 28, "Input",
  CellTags->"PositionList"],
Cell[768055, 32488, 143, 4, 42, "Input",
  CellTags->"PositionList"],
Cell[768201, 32494, 114, 3, 26, "Input",
  CellTags->"PositionList"],
Cell[768318, 32499, 216, 6, 50, "ExampleText",
  CellTags->"PositionList"],
Cell[768537, 32507, 99, 2, 28, "Input",
  CellTags->"PositionList"],
Cell[768639, 32511, 141, 3, 26, "Input",
  CellTags->"PositionList"],
Cell[768783, 32516, 189, 5, 26, "Input",
  CellTags->"PositionList"],
Cell[768975, 32523, 96, 3, 26, "Input",
  CellTags->"PositionList"],
Cell[769074, 32528, 60, 1, 32, "ObjectName",
  CellTags->"PositionOne"],
Cell[769137, 32531, 56, 1, 50, "Highlight",
  CellTags->"PositionOne"],
Cell[769196, 32534, 162, 6, 31, "Usage",
  CellTags->"PositionOne"],
Cell[769361, 32542, 60, 1, 24, "Notes",
  CellTags->"PositionOne"],
Cell[769424, 32545, 69, 1, 39, "ExampleSection",
  CellTags->"PositionOne"],
Cell[769496, 32548, 132, 4, 34, "ExampleText",
  CellTags->"PositionOne"],
Cell[769631, 32554, 107, 2, 28, "Input",
  CellTags->"PositionOne"],
Cell[769741, 32558, 103, 3, 26, "Input",
  CellTags->"PositionOne"],
Cell[769847, 32563, 62, 1, 32, "ObjectName",
  CellTags->"ReplaceParts"],
Cell[769912, 32566, 57, 1, 50, "Highlight",
  CellTags->"ReplaceParts"],
Cell[769972, 32569, 154, 4, 31, "Usage",
  CellTags->"ReplaceParts"],
Cell[770129, 32575, 75, 1, 24, "Notes",
  CellTags->"ReplaceParts"],
Cell[770207, 32578, 70, 1, 39, "ExampleSection",
  CellTags->"ReplaceParts"],
Cell[770280, 32581, 73, 1, 34, "ExampleText",
  CellTags->"ReplaceParts"],
Cell[770356, 32584, 123, 3, 26, "Input",
  CellTags->"ReplaceParts"],
Cell[770482, 32589, 64, 1, 32, "ObjectName",
  CellTags->"SelectOptions"],
Cell[770549, 32592, 58, 1, 50, "Highlight",
  CellTags->"SelectOptions"],
Cell[770610, 32595, 222, 8, 31, "Usage",
  CellTags->"SelectOptions"],
Cell[770835, 32605, 193, 6, 24, "Notes",
  CellTags->"SelectOptions"],
Cell[771031, 32613, 50, 1, 32, "ObjectName",
  CellTags->"Status"],
Cell[771084, 32616, 51, 1, 50, "Highlight",
  CellTags->"Status"],
Cell[771138, 32619, 139, 4, 31, "Usage",
  CellTags->"Status"],
Cell[771280, 32625, 64, 1, 39, "ExampleSection",
  CellTags->"Status"],
Cell[771347, 32628, 62, 1, 34, "ExampleText",
  CellTags->"Status"],
Cell[771412, 32631, 73, 2, 28, "Input",
  CellTags->"Status"],
Cell[771488, 32635, 58, 1, 32, "ObjectName",
  CellTags->"SymbolList"],
Cell[771549, 32638, 55, 1, 50, "Highlight",
  CellTags->"SymbolList"],
Cell[771607, 32641, 246, 10, 31, "Usage",
  CellTags->"SymbolList"],
Cell[771856, 32653, 114, 4, 24, "Notes",
  CellTags->"SymbolList"],
Cell[771973, 32659, 68, 1, 39, "ExampleSection",
  CellTags->"SymbolList"],
Cell[772044, 32662, 67, 1, 34, "ExampleText",
  CellTags->"SymbolList"],
Cell[772114, 32665, 116, 5, 45, "Input",
  CellTags->"SymbolList"],
Cell[772233, 32672, 46, 1, 32, "ObjectName",
  CellTags->"Zwei"],
Cell[772282, 32675, 54, 1, 50, "Highlight",
  CellTags->"Zwei"],
Cell[772339, 32678, 145, 6, 31, "Usage",
  CellTags->"Zwei"],
Cell[772487, 32686, 105, 5, 24, "Notes",
  CellTags->"Zwei"],
Cell[772595, 32693, 333, 15, 24, "Notes",
  CellTags->"Zwei"],
Cell[772931, 32710, 82, 1, 32, "ObjectName",
  CellTags->"$AutomataReleaseNumber"],
Cell[773016, 32713, 70, 1, 50, "Highlight",
  CellTags->"$AutomataReleaseNumber"],
Cell[773089, 32716, 172, 4, 31, "Usage",
  CellTags->"$AutomataReleaseNumber"],
Cell[773264, 32722, 80, 1, 39, "ExampleSection",
  CellTags->"$AutomataReleaseNumber"],
Cell[773347, 32725, 81, 1, 34, "ExampleText",
  CellTags->"$AutomataReleaseNumber"],
Cell[773431, 32728, 119, 3, 26, "Input",
  CellTags->"$AutomataReleaseNumber"],
Cell[773553, 32733, 54, 1, 32, "ObjectName",
  CellTags->"PostLoad"],
Cell[773610, 32736, 53, 1, 50, "Highlight",
  CellTags->"PostLoad"],
Cell[773666, 32739, 176, 5, 31, "Usage",
  CellTags->"PostLoad"],
Cell[773845, 32746, 52, 1, 32, "ObjectName",
  CellTags->"PreLoad"],
Cell[773900, 32749, 51, 1, 50, "Highlight",
  CellTags->"PreLoad"],
Cell[773954, 32752, 167, 5, 31, "Usage",
  CellTags->"PreLoad"],
Cell[774124, 32759, 60, 1, 32, "ObjectName",
  CellTags->"$deffatline"],
Cell[774187, 32762, 59, 1, 50, "Highlight",
  CellTags->"$deffatline"],
Cell[774249, 32765, 179, 6, 31, "Usage",
  CellTags->"$deffatline"],
Cell[774431, 32773, 149, 5, 24, "Notes",
  CellTags->"$deffatline"],
Cell[774583, 32780, 166, 7, 24, "Notes",
  CellTags->"$deffatline"],
Cell[774752, 32789, 69, 1, 39, "ExampleSection",
  CellTags->"$deffatline"],
Cell[774824, 32792, 84, 1, 34, "ExampleText",
  CellTags->"$deffatline"],
Cell[774911, 32795, 97, 3, 26, "Input",
  CellTags->"$deffatline"],
Cell[775011, 32800, 68, 1, 32, "ObjectName",
  CellTags->"$deflabeloffset"],
Cell[775082, 32803, 63, 1, 50, "Highlight",
  CellTags->"$deflabeloffset"],
Cell[775148, 32806, 179, 6, 31, "Usage",
  CellTags->"$deflabeloffset"],
Cell[775330, 32814, 69, 1, 24, "Notes",
  CellTags->"$deflabeloffset"],
Cell[775402, 32817, 266, 11, 24, "Notes",
  CellTags->"$deflabeloffset"],
Cell[775671, 32830, 73, 1, 39, "ExampleSection",
  CellTags->"$deflabeloffset"],
Cell[775747, 32833, 86, 1, 34, "ExampleText",
  CellTags->"$deflabeloffset"],
Cell[775836, 32836, 105, 3, 26, "Input",
  CellTags->"$deflabeloffset"],
Cell[775944, 32841, 64, 1, 32, "ObjectName",
  CellTags->"$defpointsize"],
Cell[776011, 32844, 61, 1, 50, "Highlight",
  CellTags->"$defpointsize"],
Cell[776075, 32847, 174, 6, 31, "Usage",
  CellTags->"$defpointsize"],
Cell[776252, 32855, 150, 5, 24, "Notes",
  CellTags->"$defpointsize"],
Cell[776405, 32862, 264, 11, 24, "Notes",
  CellTags->"$defpointsize"],
Cell[776672, 32875, 71, 1, 39, "ExampleSection",
  CellTags->"$defpointsize"],
Cell[776746, 32878, 133, 5, 34, "ExampleText",
  CellTags->"$defpointsize"],
Cell[776882, 32885, 96, 2, 28, "Input",
  CellTags->"$defpointsize"],
Cell[776981, 32889, 150, 4, 26, "Input",
  CellTags->"$defpointsize"],
Cell[777134, 32895, 62, 1, 32, "ObjectName",
  CellTags->"$defthinline"],
Cell[777199, 32898, 60, 1, 50, "Highlight",
  CellTags->"$defthinline"],
Cell[777262, 32901, 174, 6, 31, "Usage",
  CellTags->"$defthinline"],
Cell[777439, 32909, 150, 5, 24, "Notes",
  CellTags->"$defthinline"],
Cell[777592, 32916, 259, 11, 24, "Notes",
  CellTags->"$defthinline"],
Cell[777854, 32929, 70, 1, 39, "ExampleSection",
  CellTags->"$defthinline"],
Cell[777927, 32932, 134, 5, 34, "ExampleText",
  CellTags->"$defthinline"],
Cell[778064, 32939, 93, 2, 28, "Input",
  CellTags->"$defthinline"],
Cell[778160, 32943, 129, 5, 45, "Input",
  CellTags->"$defthinline"],
Cell[778292, 32950, 129, 5, 45, "Input",
  CellTags->"$defthinline"],
Cell[778424, 32957, 70, 1, 32, "ObjectName",
  CellTags->"$plotrangeoffset"],
Cell[778497, 32960, 64, 1, 50, "Highlight",
  CellTags->"$plotrangeoffset"],
Cell[778564, 32963, 173, 6, 31, "Usage",
  CellTags->"$plotrangeoffset"],
Cell[778740, 32971, 70, 1, 24, "Notes",
  CellTags->"$plotrangeoffset"],
Cell[778813, 32974, 170, 7, 24, "Notes",
  CellTags->"$plotrangeoffset"],
Cell[778986, 32983, 74, 1, 39, "ExampleSection",
  CellTags->"$plotrangeoffset"],
Cell[779063, 32986, 80, 1, 34, "ExampleText",
  CellTags->"$plotrangeoffset"],
Cell[779146, 32989, 107, 3, 26, "Input",
  CellTags->"$plotrangeoffset"],
Cell[779256, 32994, 64, 1, 32, "ObjectName",
  CellTags->"$rastercolors"],
Cell[779323, 32997, 61, 1, 50, "Highlight",
  CellTags->"$rastercolors"],
Cell[779387, 33000, 173, 5, 31, "Usage",
  CellTags->"$rastercolors"],
Cell[779563, 33007, 210, 6, 24, "Notes",
  CellTags->"$rastercolors"],
Cell[779776, 33015, 342, 15, 24, "Notes",
  CellTags->"$rastercolors"],
Cell[780121, 33032, 71, 1, 39, "ExampleSection",
  CellTags->"$rastercolors"],
Cell[780195, 33035, 211, 6, 50, "ExampleText",
  CellTags->"$rastercolors"],
Cell[780409, 33043, 100, 3, 26, "Input",
  CellTags->"$rastercolors"],
Cell[780512, 33048, 129, 3, 26, "Input",
  CellTags->"$rastercolors"],
Cell[780644, 33053, 151, 4, 34, "ExampleText",
  CellTags->"$rastercolors"],
Cell[780798, 33059, 182, 5, 26, "Input",
  CellTags->"$rastercolors"],
Cell[780983, 33066, 50, 1, 32, "ObjectName",
  CellTags->"ColorQ"],
Cell[781036, 33069, 51, 1, 50, "Highlight",
  CellTags->"ColorQ"],
Cell[781090, 33072, 157, 6, 31, "Usage",
  CellTags->"ColorQ"],
Cell[781250, 33080, 82, 1, 24, "Notes",
  CellTags->"ColorQ"],
Cell[781335, 33083, 272, 12, 24, "Notes",
  CellTags->"ColorQ"],
Cell[781610, 33097, 50, 1, 32, "ObjectName",
  CellTags->"DoShow"],
Cell[781663, 33100, 51, 1, 50, "Highlight",
  CellTags->"DoShow"],
Cell[781717, 33103, 253, 10, 47, "Usage",
  CellTags->"DoShow"],
Cell[781973, 33115, 134, 5, 24, "Notes",
  CellTags->"DoShow"],
Cell[782110, 33122, 157, 7, 24, "Notes",
  CellTags->"DoShow"],
Cell[782270, 33131, 64, 1, 39, "ExampleSection",
  CellTags->"DoShow"],
Cell[782337, 33134, 80, 1, 34, "ExampleText",
  CellTags->"DoShow"],
Cell[782420, 33137, 177, 5, 26, "Input",
  CellTags->"DoShow"],
Cell[782600, 33144, 96, 3, 26, "Input",
  CellTags->"DoShow"],
Cell[782699, 33149, 54, 1, 32, "ObjectName",
  CellTags->"GridLine"],
Cell[782756, 33152, 53, 1, 50, "Highlight",
  CellTags->"GridLine"],
Cell[782812, 33155, 218, 8, 31, "Usage",
  CellTags->"GridLine"],
Cell[783033, 33165, 353, 11, 79, "Notes",
  CellTags->"GridLine"],
Cell[783389, 33178, 90, 1, 24, "Notes",
  CellTags->"GridLine"],
Cell[783482, 33181, 259, 11, 24, "Notes",
  CellTags->"GridLine"],
Cell[783744, 33194, 66, 1, 39, "ExampleSection",
  CellTags->"GridLine"],
Cell[783813, 33197, 62, 1, 34, "ExampleText",
  CellTags->"GridLine"],
Cell[783878, 33200, 114, 3, 26, "Input",
  CellTags->"GridLine"],
Cell[783995, 33205, 80, 1, 34, "ExampleText",
  CellTags->"GridLine"],
Cell[784078, 33208, 325, 8, 58, "Input",
  CellTags->"GridLine"],
Cell[784406, 33218, 81, 2, 28, "Input",
  CellTags->"GridLine"],
Cell[784490, 33222, 58, 1, 32, "ObjectName",
  CellTags->"GridOffset"],
Cell[784551, 33225, 53, 1, 50, "Highlight",
  CellTags->"GridOffset"],
Cell[784607, 33228, 147, 6, 31, "Usage",
  CellTags->"GridOffset"],
Cell[784757, 33236, 158, 5, 24, "Notes",
  CellTags->"GridOffset"],
Cell[784918, 33243, 159, 7, 24, "Notes",
  CellTags->"GridOffset"],
Cell[785080, 33252, 58, 1, 32, "ObjectName",
  CellTags->"GridPoints"],
Cell[785141, 33255, 55, 1, 50, "Highlight",
  CellTags->"GridPoints"],
Cell[785199, 33258, 221, 8, 31, "Usage",
  CellTags->"GridPoints"],
Cell[785423, 33268, 458, 14, 97, "Notes",
  CellTags->"GridPoints"],
Cell[785884, 33284, 91, 1, 24, "Notes",
  CellTags->"GridPoints"],
Cell[785978, 33287, 257, 11, 24, "Notes",
  CellTags->"GridPoints"],
Cell[786238, 33300, 68, 1, 39, "ExampleSection",
  CellTags->"GridPoints"],
Cell[786309, 33303, 65, 1, 34, "ExampleText",
  CellTags->"GridPoints"],
Cell[786377, 33306, 144, 4, 26, "Input",
  CellTags->"GridPoints"],
Cell[786524, 33312, 72, 1, 34, "ExampleText",
  CellTags->"GridPoints"],
Cell[786599, 33315, 355, 7, 66, "Input",
  CellTags->"GridPoints"],
Cell[786957, 33324, 68, 1, 32, "ObjectName",
  CellTags->"HueRasterColors"],
Cell[787028, 33327, 60, 1, 50, "Highlight",
  CellTags->"HueRasterColors"],
Cell[787091, 33330, 179, 6, 31, "Usage",
  CellTags->"HueRasterColors"],
Cell[787273, 33338, 132, 5, 24, "Notes",
  CellTags->"HueRasterColors"],
Cell[787408, 33345, 400, 15, 24, "Notes",
  CellTags->"HueRasterColors"],
Cell[787811, 33362, 73, 1, 39, "ExampleSection",
  CellTags->"HueRasterColors"],
Cell[787887, 33365, 76, 1, 34, "ExampleText",
  CellTags->"HueRasterColors"],
Cell[787966, 33368, 142, 3, 26, "Input",
  CellTags->"HueRasterColors"],
Cell[788111, 33373, 151, 4, 26, "Input",
  CellTags->"HueRasterColors"],
Cell[788265, 33379, 56, 1, 32, "ObjectName",
  CellTags->"LabelGrid"],
Cell[788324, 33382, 52, 1, 50, "Highlight",
  CellTags->"LabelGrid"],
Cell[788379, 33385, 224, 7, 47, "Usage",
  CellTags->"LabelGrid"],
Cell[788606, 33394, 157, 7, 24, "Notes",
  CellTags->"LabelGrid"],
Cell[788766, 33403, 60, 1, 32, "ObjectName",
  CellTags->"LabelPoints"],
Cell[788829, 33406, 54, 1, 50, "Highlight",
  CellTags->"LabelPoints"],
Cell[788886, 33409, 201, 6, 31, "Usage",
  CellTags->"LabelPoints"],
Cell[789090, 33417, 182, 7, 24, "Notes",
  CellTags->"LabelPoints"],
Cell[789275, 33426, 159, 7, 24, "Notes",
  CellTags->"LabelPoints"],
Cell[789437, 33435, 56, 1, 32, "ObjectName",
  CellTags->"LineStyle"],
Cell[789496, 33438, 52, 1, 50, "Highlight",
  CellTags->"LineStyle"],
Cell[789551, 33441, 207, 6, 47, "Usage",
  CellTags->"LineStyle"],
Cell[789761, 33449, 134, 5, 24, "Notes",
  CellTags->"LineStyle"],
Cell[789898, 33456, 159, 7, 24, "Notes",
  CellTags->"LineStyle"],
Cell[790060, 33465, 64, 1, 32, "ObjectName",
  CellTags->"MakePlotRange"],
Cell[790127, 33468, 58, 1, 50, "Highlight",
  CellTags->"MakePlotRange"],
Cell[790188, 33471, 203, 6, 47, "Usage",
  CellTags->"MakePlotRange"],
Cell[790394, 33479, 91, 1, 24, "Notes",
  CellTags->"MakePlotRange"],
Cell[790488, 33482, 143, 6, 24, "Notes",
  CellTags->"MakePlotRange"],
Cell[790634, 33490, 54, 1, 32, "ObjectName",
  CellTags->"MultiCol"],
Cell[790691, 33493, 51, 1, 50, "Highlight",
  CellTags->"MultiCol"],
Cell[790745, 33496, 145, 6, 31, "Usage",
  CellTags->"MultiCol"],
Cell[790893, 33504, 156, 7, 24, "Notes",
  CellTags->"MultiCol"],
Cell[791052, 33513, 48, 1, 32, "ObjectName",
  CellTags->"NRows"],
Cell[791103, 33516, 48, 1, 50, "Highlight",
  CellTags->"NRows"],
Cell[791154, 33519, 139, 6, 31, "Usage",
  CellTags->"NRows"],
Cell[791296, 33527, 153, 7, 24, "Notes",
  CellTags->"NRows"],
Cell[791452, 33536, 72, 1, 32, "ObjectName",
  CellTags->"PlotComputationFA"],
Cell[791527, 33539, 62, 1, 50, "Highlight",
  CellTags->"PlotComputationFA"],
Cell[791592, 33542, 224, 8, 31, "Usage",
  CellTags->"PlotComputationFA"],
Cell[791819, 33552, 379, 12, 97, "Notes",
  CellTags->"PlotComputationFA"],
Cell[792201, 33566, 222, 6, 40, "Notes",
  CellTags->"PlotComputationFA"],
Cell[792426, 33574, 179, 7, 24, "Notes",
  CellTags->"PlotComputationFA"],
Cell[792608, 33583, 293, 11, 24, "Notes",
  CellTags->"PlotComputationFA"],
Cell[792904, 33596, 75, 1, 39, "ExampleSection",
  CellTags->"PlotComputationFA"],
Cell[792982, 33599, 206, 7, 34, "ExampleText",
  CellTags->"PlotComputationFA"],
Cell[793191, 33608, 118, 3, 26, "Input",
  CellTags->"PlotComputationFA"],
Cell[793312, 33613, 155, 4, 34, "ExampleText",
  CellTags->"PlotComputationFA"],
Cell[793470, 33619, 126, 3, 26, "Input",
  CellTags->"PlotComputationFA"],
Cell[793599, 33624, 129, 4, 34, "ExampleText",
  CellTags->"PlotComputationFA"],
Cell[793731, 33630, 183, 4, 26, "Input",
  CellTags->"PlotComputationFA"],
Cell[793917, 33636, 62, 1, 32, "ObjectName",
  CellTags->"PlotFunction"],
Cell[793982, 33639, 57, 1, 50, "Highlight",
  CellTags->"PlotFunction"],
Cell[794042, 33642, 256, 10, 31, "Usage",
  CellTags->"PlotFunction"],
Cell[794301, 33654, 442, 14, 97, "Notes",
  CellTags->"PlotFunction"],
Cell[794746, 33670, 168, 7, 24, "Notes",
  CellTags->"PlotFunction"],
Cell[794917, 33679, 70, 1, 39, "ExampleSection",
  CellTags->"PlotFunction"],
Cell[794990, 33682, 90, 1, 34, "ExampleText",
  CellTags->"PlotFunction"],
Cell[795083, 33685, 174, 4, 47, "Input",
  CellTags->"PlotFunction"],
Cell[795260, 33691, 58, 1, 32, "ObjectName",
  CellTags->"PlotMatrix"],
Cell[795321, 33694, 55, 1, 50, "Highlight",
  CellTags->"PlotMatrix"],
Cell[795379, 33697, 163, 6, 31, "Usage",
  CellTags->"PlotMatrix"],
Cell[795545, 33705, 528, 15, 151, "Notes",
  CellTags->"PlotMatrix"],
Cell[796076, 33722, 156, 5, 24, "Notes",
  CellTags->"PlotMatrix"],
Cell[796235, 33729, 139, 5, 24, "Notes",
  CellTags->"PlotMatrix"],
Cell[796377, 33736, 142, 5, 24, "Notes",
  CellTags->"PlotMatrix"],
Cell[796522, 33743, 140, 5, 24, "Notes",
  CellTags->"PlotMatrix"],
Cell[796665, 33750, 179, 5, 24, "Notes",
  CellTags->"PlotMatrix"],
Cell[796847, 33757, 199, 6, 40, "Notes",
  CellTags->"PlotMatrix"],
Cell[797049, 33765, 171, 5, 24, "Notes",
  CellTags->"PlotMatrix"],
Cell[797223, 33772, 185, 7, 24, "Notes",
  CellTags->"PlotMatrix"],
Cell[797411, 33781, 373, 15, 24, "Notes",
  CellTags->"PlotMatrix"],
Cell[797787, 33798, 68, 1, 39, "ExampleSection",
  CellTags->"PlotMatrix"],
Cell[797858, 33801, 96, 1, 34, "ExampleText",
  CellTags->"PlotMatrix"],
Cell[797957, 33804, 221, 8, 41, "Input",
  CellTags->"PlotMatrix"],
Cell[798181, 33814, 79, 1, 34, "ExampleText",
  CellTags->"PlotMatrix"],
Cell[798263, 33817, 127, 3, 26, "Input",
  CellTags->"PlotMatrix"],
Cell[798393, 33822, 88, 1, 34, "ExampleText",
  CellTags->"PlotMatrix"],
Cell[798484, 33825, 297, 7, 58, "Input",
  CellTags->"PlotMatrix"],
Cell[798784, 33834, 62, 1, 32, "ObjectName",
  CellTags->"PlotRelation"],
Cell[798849, 33837, 57, 1, 50, "Highlight",
  CellTags->"PlotRelation"],
Cell[798909, 33840, 263, 10, 47, "Usage",
  CellTags->"PlotRelation"],
Cell[799175, 33852, 531, 16, 97, "Notes",
  CellTags->"PlotRelation"],
Cell[799709, 33870, 254, 11, 24, "Notes",
  CellTags->"PlotRelation"],
Cell[799966, 33883, 70, 1, 39, "ExampleSection",
  CellTags->"PlotRelation"],
Cell[800039, 33886, 90, 1, 34, "ExampleText",
  CellTags->"PlotRelation"],
Cell[800132, 33889, 142, 3, 30, "Input",
  CellTags->"PlotRelation"],
Cell[800277, 33894, 132, 3, 26, "Input",
  CellTags->"PlotRelation"],
Cell[800412, 33899, 60, 1, 32, "ObjectName",
  CellTags->"PlotTDGraph"],
Cell[800475, 33902, 56, 1, 50, "Highlight",
  CellTags->"PlotTDGraph"],
Cell[800534, 33905, 213, 8, 31, "Usage",
  CellTags->"PlotTDGraph"],
Cell[800750, 33915, 530, 16, 97, "Notes",
  CellTags->"PlotTDGraph"],
Cell[801283, 33933, 155, 5, 24, "Notes",
  CellTags->"PlotTDGraph"],
Cell[801441, 33940, 221, 7, 24, "Notes",
  CellTags->"PlotTDGraph"],
Cell[801665, 33949, 328, 15, 24, "Notes",
  CellTags->"PlotTDGraph"],
Cell[801996, 33966, 69, 1, 39, "ExampleSection",
  CellTags->"PlotTDGraph"],
Cell[802068, 33969, 189, 6, 34, "ExampleText",
  CellTags->"PlotTDGraph"],
Cell[802260, 33977, 447, 10, 58, "Input",
  CellTags->"PlotTDGraph"],
Cell[802710, 33989, 108, 3, 26, "Input",
  CellTags->"PlotTDGraph"],
Cell[802821, 33994, 163, 4, 26, "Input",
  CellTags->"PlotTDGraph"],
Cell[802987, 34000, 48, 1, 32, "ObjectName",
  CellTags->"PlotT"],
Cell[803038, 34003, 50, 1, 50, "Highlight",
  CellTags->"PlotT"],
Cell[803091, 34006, 194, 8, 31, "Usage",
  CellTags->"PlotT"],
Cell[803288, 34016, 217, 9, 24, "Notes",
  CellTags->"PlotT"],
Cell[803508, 34027, 101, 5, 24, "Notes",
  CellTags->"PlotT"],
Cell[803612, 34034, 254, 12, 24, "Notes",
  CellTags->"PlotT"],
Cell[803869, 34048, 63, 1, 39, "ExampleSection",
  CellTags->"PlotT"],
Cell[803935, 34051, 98, 1, 34, "ExampleText",
  CellTags->"PlotT"],
Cell[804036, 34054, 79, 2, 28, "Input",
  CellTags->"PlotT"],
Cell[804118, 34058, 93, 3, 26, "Input",
  CellTags->"PlotT"],
Cell[804214, 34063, 67, 1, 34, "ExampleText",
  CellTags->"PlotT"],
Cell[804284, 34066, 163, 4, 26, "Input",
  CellTags->"PlotT"],
Cell[804450, 34072, 60, 1, 32, "ObjectName",
  CellTags->"RasterStyle"],
Cell[804513, 34075, 54, 1, 50, "Highlight",
  CellTags->"RasterStyle"],
Cell[804570, 34078, 203, 6, 31, "Usage",
  CellTags->"RasterStyle"],
Cell[804776, 34086, 214, 9, 24, "Notes",
  CellTags->"RasterStyle"],
Cell[804993, 34097, 270, 11, 24, "Notes",
  CellTags->"RasterStyle"],
Cell[805266, 34110, 48, 1, 32, "ObjectName",
  CellTags->"RView"],
Cell[805317, 34113, 48, 1, 50, "Highlight",
  CellTags->"RView"],
Cell[805368, 34116, 139, 6, 31, "Usage",
  CellTags->"RView"],
Cell[805510, 34124, 153, 7, 24, "Notes",
  CellTags->"RView"],
Cell[805666, 34133, 56, 1, 32, "ObjectName",
  CellTags->"ShowArray"],
Cell[805725, 34136, 54, 1, 50, "Highlight",
  CellTags->"ShowArray"],
Cell[805782, 34139, 231, 8, 31, "Usage",
  CellTags->"ShowArray"],
Cell[806016, 34149, 191, 7, 24, "Notes",
  CellTags->"ShowArray"],
Cell[806210, 34158, 162, 7, 24, "Notes",
  CellTags->"ShowArray"],
Cell[806375, 34167, 67, 1, 39, "ExampleSection",
  CellTags->"ShowArray"],
Cell[806445, 34170, 86, 1, 34, "ExampleText",
  CellTags->"ShowArray"],
Cell[806534, 34173, 239, 5, 42, "Input",
  CellTags->"ShowArray"],
Cell[806776, 34180, 102, 3, 26, "Input",
  CellTags->"ShowArray"],
Cell[806881, 34185, 64, 1, 32, "ObjectName",
  CellTags->"StaircasePlot"],
Cell[806948, 34188, 58, 1, 50, "Highlight",
  CellTags->"StaircasePlot"],
Cell[807009, 34191, 199, 6, 31, "Usage",
  CellTags->"StaircasePlot"],
Cell[807211, 34199, 278, 10, 61, "Notes",
  CellTags->"StaircasePlot"],
Cell[807492, 34211, 128, 5, 24, "Notes",
  CellTags->"StaircasePlot"],
Cell[807623, 34218, 440, 19, 24, "Notes",
  CellTags->"StaircasePlot"],
Cell[808066, 34239, 71, 1, 39, "ExampleSection",
  CellTags->"StaircasePlot"],
Cell[808140, 34242, 92, 1, 34, "ExampleText",
  CellTags->"StaircasePlot"],
Cell[808235, 34245, 223, 5, 42, "Input",
  CellTags->"StaircasePlot"]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

